/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.13 or later
*(C)*
*(C)* Copyright (c) 2002 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

/*
  970807 Macros for type and shape changed to enum constants
  970813 Added macro mvSeterror to ease use of new standard callback function
  020923 Updated enum mvTypeShapeCodes and parallel defines to reflect
         changes in keywords.h
*/

/*
  This file should be included in any MacAnova user function.

  If the user function makes call backs, the following should appear
  in user function source files:
    #define MVCALLBACKS
    #include "Userfun.h"
  
  For a multi-file user function that makes call backs, the following
  should also appear before #include "Userfun.h" in all except one
  source file.
    #define USERSUBFUNCTION

  For a Macintosh compilation, define MACINTOSH before #include "Userfun.h".
  If the compiler is Metrowerks CodeWarrior, also define MW_CW 

  For a Windows compilation define WIN32 before #include "Userfun.h"

  A user function should be declared as, say,
    void EXPORTED foo(arg1, ... )

  An arginfo function should be declared as
    long * EXPORTED arginfo_foo(void)

  See below for macros to help in declaring and using arguments 

  In a function for a 68K Macintosh, each function must be compiled separately
  into differently named resources (type 'MV6n' if no co-processor is needed,
  or 'MV6c' if a co-processor is needed) .  The entry should have name 'main';
  the name argument to User() is matched against the resource name.  Multiple
  resource can be combined in one file, either using resedit or the
  option Merge to File in the Metrowerks CodeWarrior compiler.

  In a function for a PPC Macintosh, several functions can be compiled
  in one named code resource of type 'MVPP', but one of them must be 'main'.
  The resource name should match the name of one of the user functions, and
  User("goo",resource:"foo", ....) will be required to access the
  function 'goo' in a resource named 'foo'
*/

#ifndef USERFUN_H
#define USERFUN_H
#if defined(USERSUBFUNCTION) && !defined(USERFUNCTION)
#define USERFUNCTION
#endif /*USERSUBFUNCTION && !USERFUNCTION*/

/*
  POINTERARGS should be defined only if you want other than the
  default behavior (handle arguments on Macintosh, pointer args otherwise).
  On a Macintosh use #define POINTERARGS 1 to expect arguments as pointers
  Otherwise use #define POINTERARGS 0 to expect arguments as handles
*/
#ifndef POINTERARGS
#ifdef MACINTOSH
#define POINTERARGS 0
#else /*MACINTOSH*/
#define POINTERARGS 1
#endif /*MACINTOSH*/
#endif /*POINTERARGS*/

#ifdef __cplusplus
extern "C"
{
#define mycplusplus __cplusplus
#undef __cplusplus
#endif /*__cplusplus*/

#ifdef OS_WIN32
#include <windows.h>

#define EXPORTED _export

#else /*OS_WIN32*/
#define EXPORTED
#endif /*OS_WIN32*/

#ifdef MACINTOSH
#ifdef powerc
/* nothing special*/
#else /*powerc*/

#ifdef MW_CW

#if __option(code68881)
#define NEEDS68881
#endif /*__option(code68881)*/

/*
    In writing a function for a 68K Macintosh, to ensure correct
    compilation, all declarations of call back functions and arginfo
    functions be bracketed by
    
	#pragma mpwc on
	...
	#pragma mpwc off
	This assumes that the 68K version of MacAnova is compiled by MPW C.  If
	it is compiled by MW CW, PRAGMAMPWC should not be defined
*/
#define PRAGMAMPWC
#elif defined(MPW) && defined(mc68881) /*MW_CW*/
#define NEEDS68881
#endif  /*MW_CW*/

#ifdef PRAGMAMPWC
#pragma mpwc on
#endif /*PRAGMAMPWC*/

#include <A4Stuff.h>

#ifdef PRAGMAMPWC
#pragma mpwc off
#endif /*PRAGMAMPWC*/

#endif /*powerc*/
#endif /*MACINTOSH*/


/*
  Duplicate of some of Symbol.h, globdefs.h and mainpars.h
  If anything is changed there, it must be changed here, too
  If you want the full capability of working with all types of
  symbols, you must define USESYMBOL_H and have the MacAnova source
  header files available.
*/

#ifdef PRAGMAMPWC
#pragma mpwc on
#endif /*PRAGMAMPWC*/

#ifdef USESYMBOL_H
#include "globdefs.h"
#include "Symbol.h" /*defines SYMBOLTYPESDEFINED*/
#endif /*USESYMBOL_H*/

/*
    For easier programming, types are provided for the standard call
	back functions returned	by findfun.
      mygethandletype      mygethandle;
      mydisphandletype     mydisphandle;
      mygrowhandletype     mygrowhandle;
      sprintftype          sprintf;
      sscanftype           sscanf.

    Example (uses macro setMvFuns; see below):
    
    #define MVCALLBACKS 
    #include "Userfun.h"

    void foo(double *x, MacAnovaCBSPtr funlist)
    {
        char                line[30];
        void               *(*findfun)(char *) = funlist->findfun;
        void               *(*print)(char *) = funlist->print;
        sprintftype         sprintf;

        sprintf = (sprintftype) findfun("sprintf");
        
        sprintf(line, "*x = %.17g", *x);
        print(line);
    }
    
    On most systems you can link the library versions of sprintf and
    sscanf directly
*/
typedef char ** (*mygethandletype) (const long);
typedef void (*mydisphandletype) (char **);
typedef void (*mygrowhandletype) (char **, const long);
typedef int  (*sprintftype)(char *, const char *,...);
typedef int  (*sscanftype)(const char *, const char *,...);

#ifndef SYMBOLTYPESDEFINED /* defined in Symbol.h if included above*/
#define SYMBOLTYPESDEFINED

/* not the real definition of whole_graph in plot.h */
typedef struct 
{
	long        dummy;
} whole_graph;

#ifndef MAXDIMS
#define MAXDIMS 31  /* also defined in globdefs.h */
#endif /*MAXDIMS*/

#ifndef NAMELENGTH
#define NAMELENGTH 12  /* also defined in globdefs.h */
#endif /*NAMELENGTH*/

typedef struct Symbol
{				/* symbol table entry */
	char            name[NAMELENGTH+1];
	long            type;
	union
	{
		double        **data;	/* handle for REAL data */
		char          **string;	/* handle for CHARACTER data or a macro */
		long          **longdata;  /* primarily for model parsing */
		whole_graph   **graph; /* handle for info on graph */
		struct Symbol **(*fptr) (struct Symbol **);	/* for a builtin function */
		struct Symbol ****Comp;
		/* handle to vector of Symbolhandles, for components of a structure
		    or elements of a list.  The difference
		    between a list and a structure is that
		    elements of a list are in the symbol
		    table while structure components are not
		*/
		char        ****garbage; /* for keeping track of temporary handles */
	}               value;
	struct Symbol **next;	/* next element in symbol table */
	struct Symbol **prev;	/* previous element , i.e. doubly linked list */
	long            nclass;	/* number of classes for factors */
	char          **labels; /* row, column etc. labels */
	long            dim[MAXDIMS + 1];/* dimensions, dim[0] is number of dims */
}               Symbol, **Symbolhandle;

#ifdef PRAGMAMPWC
#pragma mpwc off
#endif /*PRAGMAMPWC*/


/* The following assumes that the type of a symbol is an integer < 65536 */
#define TYPEMASK     0xffff
#define SPECIALMASK  0x10000

/* Argument A to the following macros is a Symbolhandle */
#define DIM(A)  ((*(A))->dim)

#define NDIMS(A) (DIM(A)[0])  /* number of dimensions of Symbol */
#define NCOMPS(A) (DIM(A)[1]) /* number of components of structure */

/* value of I-th dimension of A, I > 0 */
#define DIMVAL(A,I) DIM(A)[I]

#define COMPLETETYPE(A) ((*(A))->type)
/* type of A, such are CHAR, REAL or LOGIC*/
#define TYPE(A) (COMPLETETYPE(A) & TYPEMASK)

/* pointer to null-terminated Symbol name */
#define NAME(A) ((*(A))->name)

/* NCLASS(A) > 0 => A is factor with NCLASS(A) levels */
#define NCLASS(A) ((*(A))->nclass)

/*handle to vector of Symbolhandles making up components of a structure*/
#define COMP(A) ((*(A))->value.Comp)
/*pointer to vector of Symbolhandles making up components of a structure*/
#define COMPPTR(A) (*COMP(A))
/*the Symbolhandle which is component I of structure A, 0 <= I < NCOMPS(A)*/
#define COMPVALUE(A,I) (COMPPTR(A)[I])

/* handle to double data vector of REAL or LOGICAL symbol*/
#define DATA(A) ((*(A))->value.data)
/* pointer to double data vector of REAL or LOGICAL symbol*/
#define DATAPTR(A) (*DATA(A))
/* data element I of REAL or LOGICAL Symbol, I >= 0 */
#define DATAVALUE(A,I) (DATAPTR(A)[I])

/* handle to character data vector of CHARACTER or MACRO symbol */
#define STRING(A) ((*(A))->value.string)
/* pointer to character data vector of CHARACTER or MACRO symbol */
#define STRINGPTR(A) (*STRING(A))
/* character I of CHARACTER or MACRO symbol, I >= 0*/
#define STRINGVALUE(A,I) (STRINGPTR(A)[I])

/*
  LABELS(A) is handle to a char vector of '\0' separated labels for each
  level of each dimension, DIMVAL(A,1)*...*DIMVAL(A,NDIMS(A)) in all
  Find Kth label (K >= 0) by
  	for (I=0, tmp=LABELSPTR(A); I < K; I++){tmp += strlen(tmp) + 1;}
*/
#define LABELS(A) ((*(A))->labels)
/* tests whether a Symbol has labels*/
#define HASLABELS(A) (LABELS(A) != (char **) 0)
/* pointer to char vector containing labels */
#define LABELSPTR(A) (*LABELS(A))
/* character I of vector containing labels, I >= 0*/
#define LABELSVALUE(A,I) (LABELSPTR(A))[I]

/* macros to check types that is safe with NULL Symbolhandles */
#define isUndef(A) (((Symbolhandle) A) != (Symbolhandle) 0 &&\
					TYPE(A) == UNDEF)
#define isMacro(A) (((Symbolhandle) A) != (Symbolhandle) 0 &&\
					TYPE(A) == MACRO)
#define isGraph(A) (((Symbolhandle) A) != (Symbolhandle) 0 &&\
					TYPE(A) == PLOTINFO)
#define isStruc(A) (((Symbolhandle) A) != (Symbolhandle) 0 &&\
					TYPE(A) == STRUC)
#define isBltin(A) (((Symbolhandle) A) != (Symbolhandle) 0 &&\
					TYPE(A) == BLTIN)
#define isSpecial(A) (((Symbolhandle) A) != (Symbolhandle) 0 &&\
					  COMPLETETYPE(A) & SPECIALMASK)

/* macros to set components of a symbol; H is a handle*/
#define setDATA(A,H)       (DATA(A)      =   (H))
#define setCOMP(A,H)       (COMP(A)      =   (H))
#define setSTRING(A,H)     (STRING(A)    =   (H))
#define setLABELS(A,H)     (LABELS(A)    =   (H))

/* N >= 0 is an integer; I > 0 is an integer <= NDIMS(A)*/
#define setNDIMS(A,N)    (NDIMS(A)    = (N))
#define setNCOMPS(A,N)   (NCOMPS(A)   = (N))
#define setDIM(A,I,N)    (DIM(A)[I]   = (N))

#define setTYPE(A,T)     (COMPLETETYPE(A) = (T))
#define setNCLASS(A,N)   (NCLASS(A)   = (N))
/* setNAME is defined as Setname(A,P) in Symbol.h*/
#define setNAME(A,P)     (strncpy(NAME(A),P,NAMELENGTH), NAME(A)[NAMELENGTH] = '\0')

#endif /*SYMBOLTYPESDEFINED*/

/*
  The following are some of principal type codes taken from MacAnova header
  file globdefs.h (formerly in mainpars.h). They are unlikely to change there,
  but if they do, this file must be modified
*/
#ifndef CHAR
# define CHAR    257       /* type of CHARACTER variable */
# define REAL    258       /* type of REAL variable */
# define STRUC   261       /* type of STRUCTURE variable */
# define MACRO   262       /* type of macro */
# define UNDEF   263       /* type of undefined variable */
# define LOGIC   264       /* type of LOGICAL variable */
# define NULLSYM 270       /* type of NULL variable */
#endif /*CHAR*/

/*
  The following defines of value types are taken from MacAnova
  source file keywords.h.  These must be kept consistent with the defines
  in keywords.h.
*/
#ifndef VALUETYPESDEFINED  /* defined in keyword.h*/
#define VALUETYPESDEFINED

#if(!defined(WXWINMSW) && !defined(BCPP))
enum mvTypeShapeCodes
{
	REALVALUE       = 0x00001000UL,
	LOGICVALUE      = 0x00002000UL,
	CHARVALUE       = 0x00004000UL,
	SYMHVALUE       = 0x00008000UL,
	INTEGERTYPE     = 0x00010000UL,
	POSITIVETYPE    = 0x00020000UL,
	NONNEGATIVETYPE = 0x00040000UL,
	SCALARTYPE      = 0x00080000UL,
	VECTORTYPE      = 0x00100000UL,
	MATRIXTYPE      = 0x00200000UL,
	ARRAYTYPE       = 0x00400000UL,
	NONMISSINGTYPE  = 0x00800000UL,
	SQUARETYPE      = 0x01000000UL,
	STRUCTURETYPE   = 0x02000000UL,
	NULLSYMTYPE     = 0x04000000UL,
	LONGVALUE       = 0x08000000UL,
	GRAPHVALUE      = 0x10000000UL,
	REALSCALAR            = (REALVALUE|SCALARTYPE),
	POSITIVESCALAR        = (REALSCALAR|NONMISSINGTYPE|POSITIVETYPE),
	NONNEGATIVESCALAR     = (REALSCALAR|NONMISSINGTYPE|NONNEGATIVETYPE),
	NONMISSINGREAL        = (REALSCALAR|NONMISSINGTYPE),
	NONNEGATIVEREAL       = (NONMISSINGREAL|NONNEGATIVETYPE),
	POSITIVEREAL          = (NONMISSINGREAL|POSITIVETYPE),
	REALVECTOR            = (REALVALUE|VECTORTYPE),
	NONMISSINGREALVECTOR  = (REALVALUE|VECTORTYPE|NONMISSINGTYPE),
	NONNEGATIVEVECTOR     = (NONMISSINGREALVECTOR|NONNEGATIVETYPE),
	POSITIVEVECTOR        = (NONMISSINGREALVECTOR|POSITIVETYPE),
	REALVECTORORNULL      = (REALVECTOR|NULLSYMTYPE),
	REALMATRIX            = (REALVALUE|MATRIXTYPE),
	NONMISSINGREALMATRIX  = (REALVALUE|MATRIXTYPE|NONMISSINGTYPE),
	NONNEGATIVEMATRIX     = (NONMISSINGREALMATRIX|NONNEGATIVETYPE),
	POSITIVEMATRIX        = (NONMISSINGREALMATRIX|POSITIVETYPE),
	REALMATRIXORNULL      = (REALMATRIX|NULLSYMTYPE),
	REALARRAY             = (REALVALUE|ARRAYTYPE),
	NONMISSINGREALARRAY   = (REALVALUE|ARRAYTYPE|NONMISSINGTYPE),
	NONNEGATIVEARRAY      = (NONMISSINGREALARRAY|NONNEGATIVETYPE),
	POSITIVEARRAY         = (NONMISSINGREALARRAY|POSITIVETYPE),
	REALSTRUCTURE         = (REALVALUE|STRUCTURETYPE),
	INTSCALAR             = (NONMISSINGREAL|INTEGERTYPE),
	NONNEGATIVEINT        = (INTSCALAR|NONNEGATIVETYPE),
	POSITIVEINT           = (INTSCALAR|POSITIVETYPE),
	INTVECTOR             = (NONMISSINGREALVECTOR|INTEGERTYPE),
	NONNEGATIVEINTVECTOR  = (INTVECTOR|NONNEGATIVETYPE),
	POSITIVEINTVECTOR     = (INTVECTOR|POSITIVETYPE),
	INTMATRIX             = (NONMISSINGREALMATRIX|INTEGERTYPE),
	NONNEGATIVEINTMATRIX  = (INTMATRIX|NONNEGATIVETYPE),
	POSITIVEINTMATRIX     = (INTMATRIX|POSITIVETYPE),
	INTARRAY              = (NONMISSINGREALARRAY|INTEGERTYPE),
	NONNEGATIVEINTARRAY   = (INTARRAY|NONNEGATIVETYPE),
	POSITIVEINTARRAY      = (INTARRAY|POSITIVETYPE),
	LOGICSCALAR           = (LOGICVALUE|SCALARTYPE|NONMISSINGTYPE),
	LOGICVECTOR           = (LOGICVALUE|VECTORTYPE|NONMISSINGTYPE),
	LOGICMATRIX           = (LOGICVALUE|MATRIXTYPE|NONMISSINGTYPE),
	LOGICARRAY            = (LOGICVALUE|ARRAYTYPE|NONMISSINGTYPE),
	LOGICSTRUCTURE        = (LOGICVALUE|STRUCTURETYPE),
	CHARSCALAR            = (CHARVALUE|SCALARTYPE),
	CHARVECTOR            = (CHARVALUE|VECTORTYPE),
	CHARMATRIX            = (CHARVALUE|MATRIXTYPE),
	CHARARRAY             = (CHARVALUE|ARRAYTYPE),
	CHARSTRUCTURE         = (CHARVALUE|STRUCTURETYPE),
	LONGSCALAR            = (LONGVALUE|SCALARTYPE),
	NONNEGATIVELONG       = (LONGSCALAR|NONNEGATIVETYPE),
	POSITIVELONG          = (LONGSCALAR|POSITIVETYPE),
	LONGVECTOR            = (LONGVALUE|VECTORTYPE),
	NONNEGATIVELONGVECTOR = (LONGVECTOR|NONNEGATIVETYPE),
	POSITIVELONGVECTOR    = (LONGVECTOR|POSITIVETYPE),
	LONGMATRIX            = (LONGVALUE|MATRIXTYPE),
	NONNEGATIVELONGMATRIX = (LONGMATRIX|NONNEGATIVETYPE),
	POSITIVELONGMATRIX    = (LONGMATRIX|POSITIVETYPE),
	LONGARRAY             = (LONGVALUE|ARRAYTYPE),
	NONNEGATIVELONGARRAY  = (LONGARRAY|NONNEGATIVETYPE),
	POSITIVELONGARRAY     = (LONGARRAY|POSITIVETYPE),
	SQUAREMATRIX          = (MATRIXTYPE|SQUARETYPE),
	REALSQUAREMATRIX      = (REALVALUE|SQUAREMATRIX)
};
#else /*!defined(WXWINMSW) && !defined(BCPP)*/
#define REALVALUE       0x00001000UL
#define LOGICVALUE      0x00002000UL
#define CHARVALUE       0x00004000UL
#define SYMHVALUE       0x00008000UL
#define INTEGERTYPE     0x00010000UL
#define POSITIVETYPE    0x00020000UL
#define NONNEGATIVETYPE 0x00040000UL
#define SCALARTYPE      0x00080000UL
#define VECTORTYPE      0x00100000UL
#define MATRIXTYPE      0x00200000UL
#define ARRAYTYPE       0x00400000UL
#define NONMISSINGTYPE  0x00800000UL
#define SQUARETYPE      0x01000000UL
#define STRUCTURETYPE   0x02000000UL
#define NULLSYMTYPE     0x04000000UL
#define LONGVALUE       0x08000000UL
#define GRAPHVALUE      0x10000000UL

#define REALSCALAR            (REALVALUE|SCALARTYPE)
#define POSITIVESCALAR        (REALSCALAR|NONMISSINGTYPE|POSITIVETYPE)
#define NONNEGATIVESCALAR     (REALSCALAR|NONMISSINGTYPE|NONNEGATIVETYPE)
#define NONMISSINGREAL        (REALSCALAR|NONMISSINGTYPE)
#define NONNEGATIVEREAL       (NONMISSINGREAL|NONNEGATIVETYPE)
#define POSITIVEREAL          (NONMISSINGREAL|POSITIVETYPE)

#define REALVECTOR            (REALVALUE|VECTORTYPE)
#define NONMISSINGREALVECTOR  (REALVALUE|VECTORTYPE|NONMISSINGTYPE)
#define NONNEGATIVEVECTOR     (NONMISSINGREALVECTOR|NONNEGATIVETYPE)
#define POSITIVEVECTOR        (NONMISSINGREALVECTOR|POSITIVETYPE)
#define REALVECTORORNULL      (REALVECTOR|NULLSYMTYPE)

#define REALMATRIX            (REALVALUE|MATRIXTYPE)
#define NONMISSINGREALMATRIX  (REALVALUE|MATRIXTYPE|NONMISSINGTYPE)
#define NONNEGATIVEMATRIX     (NONMISSINGREALMATRIX|NONNEGATIVETYPE)
#define POSITIVEMATRIX        (NONMISSINGREALMATRIX|POSITIVETYPE)
#define REALMATRIXORNULL      (REALMATRIX|NULLSYMTYPE)

#define REALARRAY             (REALVALUE|ARRAYTYPE)
#define NONMISSINGREALARRAY   (REALVALUE|ARRAYTYPE|NONMISSINGTYPE)
#define NONNEGATIVEARRAY      (NONMISSINGREALARRAY|NONNEGATIVETYPE)
#define POSITIVEARRAY         (NONMISSINGREALARRAY|POSITIVETYPE)
#define REALSTRUCTURE         (REALVALUE|STRUCTURETYPE)

#define INTSCALAR             (NONMISSINGREAL|INTEGERTYPE)
#define NONNEGATIVEINT        (INTSCALAR|NONNEGATIVETYPE)
#define POSITIVEINT           (INTSCALAR|POSITIVETYPE)

#define INTVECTOR             (NONMISSINGREALVECTOR|INTEGERTYPE)
#define NONNEGATIVEINTVECTOR  (INTVECTOR|NONNEGATIVETYPE)
#define POSITIVEINTVECTOR     (INTVECTOR|POSITIVETYPE)

#define INTMATRIX             (NONMISSINGREALMATRIX|INTEGERTYPE)
#define NONNEGATIVEINTMATRIX  (INTMATRIX|NONNEGATIVETYPE)
#define POSITIVEINTMATRIX     (INTMATRIX|POSITIVETYPE)

#define INTARRAY              (NONMISSINGREALARRAY|INTEGERTYPE)
#define NONNEGATIVEINTARRAY   (INTARRAY|NONNEGATIVETYPE)
#define POSITIVEINTARRAY      (INTARRAY|POSITIVETYPE)

#define LOGICSCALAR           (LOGICVALUE|SCALARTYPE|NONMISSINGTYPE)
#define LOGICVECTOR           (LOGICVALUE|VECTORTYPE|NONMISSINGTYPE)
#define LOGICMATRIX           (LOGICVALUE|MATRIXTYPE|NONMISSINGTYPE)
#define LOGICARRAY            (LOGICVALUE|ARRAYTYPE|NONMISSINGTYPE)
#define LOGICSTRUCTURE        (LOGICVALUE|STRUCTURETYPE)

#define CHARSCALAR            (CHARVALUE|SCALARTYPE)
#define CHARVECTOR            (CHARVALUE|VECTORTYPE)
#define CHARMATRIX            (CHARVALUE|MATRIXTYPE)
#define CHARARRAY             (CHARVALUE|ARRAYTYPE)
#define CHARSTRUCTURE         (CHARVALUE|STRUCTURETYPE)

#define LONGSCALAR            (LONGVALUE|SCALARTYPE)
#define NONNEGATIVELONG       (LONGSCALAR|NONNEGATIVETYPE)
#define POSITIVELONG          (LONGSCALAR|POSITIVETYPE)

#define LONGVECTOR            (LONGVALUE|VECTORTYPE)
#define NONNEGATIVELONGVECTOR (LONGVECTOR|NONNEGATIVETYPE)
#define POSITIVELONGVECTOR    (LONGVECTOR|POSITIVETYPE)

#define LONGMATRIX            (LONGVALUE|MATRIXTYPE)
#define NONNEGATIVELONGMATRIX (LONGMATRIX|NONNEGATIVETYPE)
#define POSITIVELONGMATRIX    (LONGMATRIX|POSITIVETYPE)

#define LONGARRAY             (LONGVALUE|ARRAYTYPE)
#define NONNEGATIVELONGARRAY  (LONGARRAY|NONNEGATIVETYPE)
#define POSITIVELONGARRAY     (LONGARRAY|POSITIVETYPE)

#define SQUAREMATRIX          (MATRIXTYPE|SQUARETYPE)
#define REALSQUAREMATRIX      (REALVALUE|SQUAREMATRIX)
#endif /*!defined(WXWINMSW) && !defined(BCPP)*/
#endif /*VALUETYPESDEFINED*/

#ifdef PRAGMAMPWC
#pragma mpwc on
#endif /*PRAGMAMPWC*/

#include "dynload.h" /*typedefs for MacAnovaCBS, MacAnovaCBSPtr, MacAnovaCBSH*/

#ifdef PRAGMAMPWC
#pragma mpwc off
#endif /*PRAGMAMPWC*/

/*
  Macros useful in writing user functions

  Use DOUBLEARG(arg), CHARARG(arg), LONGARG(arg) and SYMBOLARG(arg) to
  declare arguments other than the list of call back functions.  They
  expand to handles (double ** arg, char ** arg, ...) on a Macintosh and
  to pointers (double * arg, char * arg, ...)  everywhere else.

  Use CALLBACKLIST(funlist) to declare the call back function list
  structure.  It expands to a handle on a Macintosh and a pointer
  everywhere else.

  Use arg = THEARG(argx) to obtain pointers to non-symbol arguments.
  THEARG dereferences a handle on a Macintosh and does nothing on
  everywhere else.

  Use commandH = THECOMMAND(argx) to obtain a handle (char **) to a
  CHARACTER argument that is to be an argument to the mvEval() call back
  function.

  Use symhArg = THESYMBOL(argx) to obtain a Symbolhandle (Symbol **) for a
  Symbol type argument.  Except on a Macintosh, unless pointers:F is used
  on User(), the handle will not be true MacAnova handle.

  Use CALLBACKFUN(funlist, name) to obtain a function pointer from a list
  of call back functions.

  If MACINTOSH is defined, the macros referencing arguments (THEARG,
  THECOMMAND, THESYMBOL, CALLBACKFUN) assume the arguments are handles;
  if MACINTOSH is not defined, they assume arguments are pointers.

  Define MVCALLBACKS in any user function making callbacks.  This results
  in the declaration of MvCallbackFuns, a global pointer to a call back
  function list.  You can assure this will be correctly initialized by
  using macro setMvFuns as one of the first executable lines in the
  user function.  If you do this, you can use macro mvPrint, mvAlert,
  mvEval, mvIsmissing, mvSeterror and mvFindfun to call the functions
  in the list.
  See example below.  
  
  If the user function makes callbacks and has more than one source file,
  define USERSUBFUNCTION in all but one source file.  This assures that
  MvCallbackFuns will not be multiply defined.

Typical declarations
  void EXPORTED foo(DOUBLEARG(argx), LONGARG(argn), CHARARG(labelarg))
  {
      double      *x = THEARG(argx);
      long        *n = THEARG(argn);
      char        *label = THEARG(labelarg);
      ....
  }

  static long fooarginfo[] =
    {3, NOCALLBACK|POINTERUSE, REALVECTOR, POSITIVEINT, CHARSCALAR};
  long * EXPORTED arginfo_foo(void)
  {
      return (fooarginfo);
  }

  If an argument is a command to be executed by calling back to mvEval(),
  it should be declared as follows
    
  void EXPORTED fooeval(CHARARG(commandarg), ..., CALLBACKLIST(funsarg))
  {
      char            **commandH = THECOMMAND(commandarg);
      Symbolhandle      result;
      . . . .
      setMvFuns(funsarg);
      result = mvEval(commandH);
      if (result == (Symbolhandle) 0)
      {
          mvSeterror(silentCallbackError);
          return;
      }
      . . . .
  }

  void EXPORTED foosymbol(SYMBOLARG(argx), SYMBOLARG(argy))
  {
     Symbolhandle       x = THESYMBOL(argx), y = THESYMBOL(argy);
     . . . .
  }
  
  static long foosymbolarginfo[] =
    {2, NOCALLBACK|POINTERUSE|SYMBOLARGS, SYMHVALUE, REALMATRIX};
  
  long * EXPORTED arginfo_foosymbol(void)
  {
      return (foosymbolarginfo);
  }
     
*/
#if POINTERARGS
/*all arguments are pointers */
#define LONGARG(A)       long * A
#define DOUBLEARG(A)     double * A
#define CHARARG(A)       char * A
#define SYMBOLARG(A)     Symbol * A
#define THEARG(A)        (A)
#define THECOMMAND(A)    ((char **) &A)
#define THESYMBOL(A)     ((Symbolhandle) &A)
#define CALLBACKLIST(A)  MacAnovaCBSPtr A
#define CALLBACKFUN(A,F) (*A).F
#else /*POINTERARGS*/
/*all arguments are handles */
#define LONGARG(A)       long ** A
#define DOUBLEARG(A)     double ** A
#define CHARARG(A)       char ** A
#define SYMBOLARG(A)     Symbolhandle A
#define THEARG(A)        (*(A))
#define THECOMMAND(A)    ((char **) A)
#define THESYMBOL(A)     ((Symbolhandle) A)
#define CALLBACKLIST(A)  MacAnovaCBSH A
#define CALLBACKFUN(A,F) (**A).F
#endif /*POINTERARGS*/

#if defined(MACINTOSH) && !defined(powerc) && defined(NEEDS68881)
/*
   In a 68K Macintosh user function that requires a 68881 coprocessor,
   CHECK68881(arginfo) sets the COPROCESSORERROR bit of arginfo[1]
   if no coprocessor is available.  In other situations it expands
   to nothing.
*/
#define CHECK68881(AI) \
	{\
		SysEnvRec        r;\
		\
		SysEnvirons(1, &r);\
		AI[1] |= (r.hasFPU) ? COPROCESSOROK : COPROCESSORERROR;\
	}
#else /*MACINTOSH && !powerc && NEEDS68881*/
#define CHECK68881(I)
#endif /*MACINTOSH && !powerc && NEEDS68881*/

/*
	Define MVCALLBACKS in any user function making call backs
    USERSUBFUNCTION should be defined before including Userfun.h
    in all but one source file of a multi-file user function
*/
#ifdef MVCALLBACKS

/* define global pointer or handle to call back list structure*/
#ifndef USERSUBFUNCTION
CALLBACKLIST(MvCallbackFuns);
#else /*USERSUBFUNCTION*/
extern CALLBACKLIST(MvCallbackFuns);
#endif /*USERSUBFUNCTION*/

#if (0)
	If user function is
		void foo(... CALLBACKLIST(funlist) ... )
	then
	    setMvFuns(funlist);
	should appear before any call back functions are used
    The standard call back functions can then be used similarly to
    the following
      mvPrint("message to be output");
      mvAlert("Message to be displayed in dialog box");
      wts[i] = (mvIsmissing(&x[i])) ? 0.0 : 1.0;
      symh = mvEval(command);      /*command a string handle (char **) */
      mvSetError(10);  /*sets callback error code to 10*/
      mygethandle = (mygethandletype) mvFindfun("mygethandle");
#endif /*0*/

#define setMvFuns(A)     MvCallbackFuns = (A)

#define mvCallback(F)    CALLBACKFUN(MvCallbackFuns,F)
#define mvPrint          mvCallback(print)
#define mvAlert          mvCallback(alert)
#define mvIsmissing      mvCallback(ismissing)
#define mvSeterror       mvCallback(seterror)
#define mvEval           mvCallback(eval)
#define mvFindfun        mvCallback(findfun)
#endif /*MVCALLBACKS*/

/*
	Entercode();
	must follow all other declarations and precede any executable statements
	ExitCode();
	should come just before return
    They both expand to nothing except on 68K Macintosh
*/
#if defined(MACINTOSH) && !defined(powerc)
#define EnterCode() EnterCodeResource()
#define ExitCode() ExitCodeResource()
#else /*MACINTOSH && !powerc*/
#define EnterCode()
#define ExitCode()
#endif /*MACINTOSH && !powerc*/

#ifdef mycplusplus
#define __cplusplus mycplusplus
} /*extern "C"*/
#endif /*mycplusplus*/
#endif /*USERFUN_H*/
