/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.05 or later
*(C)*
*(C)* Copyright (c) 1997 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

/*
	Example source for user function to compute the sum of two REAL
    scalars.

    File also contains user function source to compute the inner product of two
	REAL vectors.  Called by User("goo",x,y,asLong(length(x)),result:0)

    On a Macintosh 68K, to compile them all, you need to use 4 CodeWarrior
    projects with the same files, creating 68K coderesources 'foo',
    'arginfo_foo', 'goo' and 'arginfo_goo', each with differing ID's, say
    4000, 4001, 4002, 4003, and all targeted at the same resource file,
    say 'foo.rez'.  On all but the project for foo, check the Merge to File
    option on the 68K Project preferences

    On a Macintosh PPC, they can all be compiled in one project, creating one
    PPC code resource 'foo' in file 'fooppc.rez', say.  After
    loadUser("fooppc.rez"), to run 'goo', you need to use
    User("goo",resource:"foo",x,y,asLong(n),result:0)

    On other platforms except DJGPP (protected mode DOS), all can be
    compiled and accessed

    On DJGPP, only one user function in a function in a .dxe file can
    so you might as well not compile anything but 'foo' to 'foo.dxe' or
    'goo' to 'goo.dxe'
*/

#include "Userfun.h"

#ifndef MACINTOSH
#define MAINFUN1    /*user function 1 will be compiled*/
#ifndef DJGPP
#define INFOFUN1    /*arginfo entry for user function 1 will be compiled*/
#define MAINFUN2    /*user function 2 will be compiled*/
#define INFOFUN2    /*arginfo entry for user function 2 will be compiled*/
#endif /*DJGPP*/
#define USERFUN1 foo
#define ARGINFO1 arginfo_foo
#define USERFUN2 goo
#define ARGINFO2 arginfo_goo

#else /*!MACINTOSH*/

#ifdef powerc
#define MAINFUN1
#define INFOFUN1
#define MAINFUN2
#define INFOFUN2
#define main_foo main /*one entry must be called "main"*/
#else /*powerc*/

/* define which of the 4 functions this project is for*/
#define WHICHFUN 1

#if WHICHFUN == 1
#define MAINFUN1
#elif WHICHFUN == 2
#define INFOFUN1
#elif WHICHFUN == 3
#define MAINFUN2
#elif WHICHFUN == 4
#define INFOFUN2
#else
#error /*nothing to compile*/
#endif

#if defined(MAINFUN1)
#define main_foo    main
#elif defined(INFOFUN1)
#define info_foo main
#elif defined(MAINFUN2)
#define main_goo main
#else
#define info_goo main
#endif

#endif /*powerc*/

#define USERFUN1 main_foo
#define USERFUN1ENTRY foo
#define ARGINFO1 info_foo
#define ARGINFO1ENTRY arginfo_foo

#define USERFUN2 main_goo
#define USERFUN2ENTRY goo
#define ARGINFO2 info_goo
#define ARGINFO2ENTRY arginfo_goo

#endif /*!MACINTOSH*/

#ifdef MAINFUN1
void EXPORTED USERFUN1(DOUBLEARG(argx), DOUBLEARG(argy), DOUBLEARG(argz))
{
	double   *x = THEARG(argx);
	double   *y = THEARG(argy);
	double   *z = THEARG(argz);

	EnterCode();
	*z = *x + *y;
	ExitCode();
}
#endif /*MAINFUN1*/

#ifdef INFOFUN1
static long Fooarginfo[] =
{
	3, NOCALLBACK | POINTERUSE,
	REALSCALAR, REALSCALAR, REALSCALAR
};

#ifdef PRAGMAMPWC
#pragma mpwc on
#endif /*PRAGMAMPWC*/
long * EXPORTED ARGINFO1(void)
{
	long         *arginfo;
	
	EnterCode();
	
	arginfo = Fooarginfo;
	CHECK68881(arginfo);

	ExitCode();
	
	return(arginfo);
}

#ifdef PRAGMAMPWC
#pragma mpwc off
#endif /*PRAGMAMPWC*/
#endif /*INFOFUN1*/

#ifdef MAINFUN2
void EXPORTED USERFUN2(DOUBLEARG(argx), DOUBLEARG(argy),
					   LONGARG(argn),DOUBLEARG(argresult))
{
	double     *x = THEARG(argx);
	double     *y = THEARG(argy);
	long       *n = THEARG(argn);
	double     *result = THEARG(argresult);
	int         i;
	
	EnterCode();
	*result = 0.0;
	for (i = 0; i < *n; i++)
	{
		*result += x[i]*y[i];
	}

	ExitCode();
}
#endif /*MAINFUN2*/

#ifdef INFOFUN2
static long Gooarginfo[] =
{
	4, NOCALLBACK | POINTERUSE,
	NONMISSINGREALVECTOR, NONMISSINGREALVECTOR, LONGSCALAR, REALSCALAR
};

#ifdef PRAGMAMPWC
#pragma mpwc on
#endif /*PRAGMAMPWC*/
long * EXPORTED ARGINFO2(void)
{
	long         *arginfo;
	
	EnterCode();

	arginfo = Gooarginfo;
	CHECK68881(arginfo);

	ExitCode();
	
	return(arginfo);
}
#ifdef PRAGMAMPWC
#pragma mpwc off
#endif /*PRAGMAMPWC*/
#endif /*INFOFUN2*/

#ifdef powerc

RoutineDescriptor	USERFUN1ENTRY =
    BUILD_ROUTINE_DESCRIPTOR(uppMainEntryProcInfo03, USERFUN1);
RoutineDescriptor	ARGINFO1ENTRY =
    BUILD_ROUTINE_DESCRIPTOR(uppArgInfoEntryProcInfo, ARGINFO1);

RoutineDescriptor	USERFUN2ENTRY =
    BUILD_ROUTINE_DESCRIPTOR(uppMainEntryProcInfo04, USERFUN2);
RoutineDescriptor	ARGINFO2ENTRY =
    BUILD_ROUTINE_DESCRIPTOR(uppArgInfoEntryProcInfo, ARGINFO2);
#endif /*powerc*/
