/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.05 or later
*(C)*
*(C)* Copyright (c) 1997 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

/*
	Example user function source to test use of symbol arguments.
	Called by User("foosymh",symbol:T,result:x)
	If x is REAL, x[1] is set to -x[1]
	If x is LOGICAL, x[1] is set to !x[1]
	If x is CHARACTER, x[1] is set to NAME(x) if x[1] is long enough
	
*/
#include "Userfun.h"

#ifndef MACINTOSH
#define MAINFUN    /*main entry will be compiled*/
#ifndef DJGPP
#define INFOFUN    /*arginfo entry will be compiled*/
#endif /*DJGPP*/
#define USERFUN foosymh
#define ARGINFO arginfo_foosymh
#else /*!MACINTOSH*/

/*
	For PPC MAC, one function must be called main
	For 68K MAC, only one function is reachable per compilation
	project and it must be called main
*/
#ifdef powerc
#define MAINFUN    /*main entry will be compiled*/
#define INFOFUN    /*arginfo entry will be compiled*/
#define main_foosymh main
#else /*powerc*/

/* define which of the 2 functions will be compiled*/
#define WHICHFUN 1  /*must be 1 or 2 */
#if WHICHFUN == 1
#define MAINFUN
#else
#define INFOFUN
#endif

#if defined(MAINFUN)
#define main_foosymh    main
#else
#define info_foosymh    main
#endif

#endif /*powerc*/

#define USERFUN      main_foosymh
#define USERFUNENTRY foosymh
#define ARGINFO      info_foosymh
#define ARGINFOENTRY arginfo_foosymh

#endif /*!MACINTOSH*/

#ifdef MAINFUN
static int strlen(const char * s)
{
	int      n = 0;
	
	while (*s++)
	{
		n++;
	}
	return (n);
}

static void strcpy(char * s1, const char *s2)
{
	while (*s2)
	{
		*s1++ = *s2++;
	}
	*s1 = '\0';
}

/*
  EXPORTED is _export for Windows compiled by Borland C/C++ 4.5
  DOUBLEARG(argx) expands as double * argx or double ** argx, and
  similarly for LONGARG(argn)
*/

void EXPORTED USERFUN(SYMBOLARG(argx))
{
/* THEARG(argx) expands as argx or *argx */
	Symbolhandle    x = THESYMBOL(argx);
	long            type = TYPE(x);


	EnterCode();
	switch (type)
	{
		case  REAL:
			DATAVALUE(x,0) = -DATAVALUE(x,0);
			break;
		case  CHAR:
			if (strlen(STRINGPTR(x)) >= 12)
			{
				strcpy(STRINGPTR(x),NAME(x));
			}
			else if (STRINGVALUE(x, 0) != '\0')
			{
				STRINGVALUE(x,0) = STRINGVALUE(x, 0) + (char) 1;
			}
			break;
		case  LOGIC:
			DATAVALUE(x,0) = 1.0 - DATAVALUE(x,0);
			break;
	}	
	ExitCode();
}
#endif /*MAINFUN*/

#ifdef INFOFUN
static long Gooarginfo[] =
{
	1, NOCALLBACK | POINTERUSE | SYMBOLARGS,
	SYMHVALUE
};

#ifdef PRAGMAMPWC /*PRAGMAMPWC defined only for 68K Mac*/
#pragma mpwc on
#endif /*PRAGMAMPWC*/
long * EXPORTED ARGINFO(void)
{
	long         *arginfo;

	EnterCode();

	arginfo = Gooarginfo;
	CHECK68881(arginfo);

	ExitCode();

	return(arginfo);
}
#ifdef PRAGMAMPWC
#pragma mpwc off
#endif /*PRAGMAMPWC*/
#endif /*INFOFUN*/

#ifdef powerc /*powerc defined means compiling for Power PC */
RoutineDescriptor USERFUNENTRY =
	BUILD_ROUTINE_DESCRIPTOR(uppMainEntryProcInfo01, USERFUN);
RoutineDescriptor ARGINFOENTRY =
	BUILD_ROUTINE_DESCRIPTOR(uppArgInfoEntryProcInfo, ARGINFO);
#endif /*powerc*/
