/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.13 or later
*(C)*
*(C)* Copyright (c) 2003 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

#ifndef SYMBOLH__
#define SYMBOLH__

#include "plot.h"
/*
   950904 changed what identifies a special type from a number < 0 to
   having bit specified by SPECIALMASK set and changes various macros
   accordingly

   960717 changed types Long, Double, Char to long, double, char
   970619 added macros isInline and setInline
   971104 added badHandleCodes
   980312 added NOMISSING bit to type component.  If set, symbol is guaranteed
          to have no missing values.  This can make anyMissing() check more
          efficient.  It will be optionally set when a function knows
          a symbol has no missing values.  Care must be taken in subscript
          assignment to make sure the NOMISSING bit is unset if any assigned
          value is MISSING.  It should be preserved by subscript selection.
          Its use will be phased in gradually.
   980719 Added new component notes and macros to go with it.
   980720 Added new type ShortSymbol and macros to go with it.  Also
          define types LabelsType and NotesType (currently both (char **))
   990211 Added (long) cast in definition of TYPE and other macros using masks
   990415 Changed calling sequence of specialSymh.setSymbol() and modified
          macros SETSPECIAL and GETSPECIAL
   990423 Implemented linked bit in COMPLETETYPE(symh) to mark GRAPH symbols
          in special symbol GRAPHWINDOWS that are in the Symbol table.
   990705 Moved typdef of mvSymbolName (formerly mvMacroName) from globdefs.h
          and modified definitions of Symbol, ShortSymbol and SpecialSymbol to
          use mvSymbolName
   000503 Added macros QUOTEARGSMASK and QUOTEARGS
   000817 Moved enums and definitions of constants having to do with
          symbols from globdefs.h
   010109 Moved enums and definitions related to memory allocation to
          globdefs.h
          Added new defines SymbolError, SymbolZero and EmptyArgument
          all expanding to ((Symbolhandle) 0)
   010726 Added locked bit; when fully implemented, when set, a symbol
          can't be deleted or changed
   010727 Changed macros ISLINKED and ISSHORT to isLinked and isShort
   010815 Added macro isENDPREV to get around a problem with the MRC compiler
   011214 Modified macro getScratch() so error message will be printed when
          amount requested is too big
   030923 Modified HASNOTES and HASLABELS to check the Symbolhandle is
          non-NULL
*/
#ifndef DEFAULTLISTLENGTH
#define DEFAULTLISTLENGTH 20 /* default length for LIST symbols  */
#endif /*DEFAULTLISTLENGTH*/
/*
	Evertything from here to #define SYMBOLTYPESDEFINED moved from
	globdefs.h 000817
*/

#define MAXDIMS          31 /* maximum of dimensions in symbol */
#define NAMELENGTH       12 /* maximum length of symbol names */
#define MAXCOORD 2000000000	/* max size in one coord direction */

/*
	The following are masked by TYPE1MASK = 0x000ff000U
	defined in globdefs
*/
#define REALTYPE1      0x00001000UL
#define LOGICTYPE1     0x00002000UL
#define CHARTYPE1      0x00004000UL
#define STRUCTYPE1     0x00008000UL
#define PLOTINFOTYPE1  0x00010000UL
#define LISTTYPE1      0x00020000UL
#define NULLTYPE1      0x00040000UL
#define UNKNOWNTYPE1   0x00080000UL

/*
	The following are masked by TYPE2MASK = 0x0ff00000U
	defined in globdefs
*/
#define REALTYPE2      0x00100000UL
#define LOGICTYPE2     0x00200000UL
#define CHARTYPE2      0x00400000UL
#define STRUCTYPE2     0x00800000UL
#define PLOTINFOTYPE2  0x01000000UL
#define LISTTYPE2      0x02000000UL
#define NULLTYPE2      0x04000000UL
#define UNKNOWNTYPE2   0x08000000UL

#define SCALAR1        0x00001000UL /* total length 1 */
#define VECTOR1        0x00002000UL /* dim[1] == length */
#define MATRIX1        0x00004000UL /* no more than 2 dims > 1 */
#define ARRAY1         0x00008000UL /* no restrictions on dims */

#define SCALAR2        0x00100000UL /* total length 1 */
#define VECTOR2        0x00200000UL /* dim[1] == length */
#define MATRIX2        0x00400000UL /* no more than 2 dims > 1 */
#define ARRAY2         0x00800000UL /* no restrictions on dims */

/*
  Type defines moved out of mainpars.h (generated by yacc run on mainpars.c)
  970714.  Putting them here removes one inconvenience to be overcome
  if the version of yacc is changed since these should remain fixed.
*/
#ifdef OS_WIN32
/* 980729 change because CHAR and LONG are typedefs in winnt.h */
#define CHAR    CHAR_MACANOVA
#define LONG    LONG_MACANOVA
#endif /*OS_WIN32*/

enum symbolTypes
{
	CHAR     = 257,
	REAL     = 258,
	BLTIN    = 259,
	LIST     = 260,
	STRUC    = 261,
	MACRO    = 262,
	UNDEF    = 263,
	LOGIC    = 264,
	PLOTINFO = 267,
	GARB     = 268,
	LONG     = 269,
	NULLSYM  = 270,
	ASSIGNED = 283
};

/*
  Type codes from mainpars.h as of 930621 (originally defines)
  They are here in case different version of yacc generates different codes
  They are needed when restoring save files from version 3.1x
*/
enum v31symbolTypes
{
	CHAR_V31     = 257,
	REAL_V31     = 258,
	BLTIN_V31    = 259,
	LIST_V31     = 260,
	STRUC_V31    = 261,
	MACRO_V31    = 262,
	UNDEF_V31    = 263,
	LOGIC_V31    = 266,
	MACRO1_V31   = 267,
	PLOTINFO_V31 = 278,
	GARB_V31     = 280,
	LONG_V31     = 281
};

/* End of stuff moved from globdefs.h */
#define SYMBOLTYPESDEFINED /* prevents dynload.h from re-defining*/

struct           Symbol;
struct           ShortSymbol;
struct           SpecialSymbol;

typedef char     mvSymbolName[NAMELENGTH + 1];

typedef struct Symbol **(*setSpecialFunction)(struct Symbol **,
											  struct Symbol **,
											  struct Symbol **, int);
typedef struct Symbol **(*getSpecialFunction)(struct Symbol **, int);

/*
  If LABELSARECHAR is defined, labels are raw character handles
  directly attached to a Symbol.  Otherwise, labels are short CHARACTER
  symbols whose handles directly attached to Symbols.  The save() and
  restore() formats differ, too.

  Similarly, depending on whether NOTESARECHAR is or is not defined, notes
  are either raw character handles or short CHARACTER symbols, with differening
  save() and restore() formats.

  980721 Successful test runs both with and without LABELSARECHAR defined
*/  
	
#if (0)
#define LABELSARECHAR
#define NOTESARECHAR
#endif

#ifdef LABELSARECHAR
typedef char                LabelsType;
#else /*LABELSARECHAR*/
typedef struct ShortSymbol  LabelsType;
#endif /*LABELSARECHAR*/

#ifdef NOTESARECHAR
typedef char                NotesType;
#else /*NOTESARECHAR*/
typedef struct ShortSymbol  NotesType;
#endif /*NOTESARECHAR*/

typedef struct Symbol
{				/* symbol table entry */
	mvSymbolName    name;
	long            type;	/* CHAR, REAL, BLTIN, LIST, STRUC, MACRO, UNDEF, LOGIC */
	union
	{
		double        **data;	/* for real data */
		char          **string;	/* for a character string, or MACRO */
		long          **longdata;  /* primarily for model parsing */
		whole_graph   **graph; /* for info on graph */
		struct Symbol **(*fptr) (struct Symbol **);	/* for a builtin function */
		struct Symbol ****Comp;
		/* for components of a structure
		    or elements of a list.  The difference
		    between a list and a structure is that
		    elements of a list are in the symbol
		    table while structure components are not
		*/
		char        ****garbage; /* for keeping track of temporary handles */
	}               value;
	struct Symbol **next;	/* next element in symbol table */
	struct Symbol **prev;	/* previous element , ie doubly linked list */
	long            nclass;	/* number of classes for factors */
	LabelsType **   labels; /* row, column etc. labels */
	NotesType  **   notes;  /* descriptive information */ 
	long            dim[MAXDIMS + 1];/* dimensions, dim[0] is number of dims */
} Symbol, **Symbolhandle;

/*
  Symbol type for one dimensional objects.  It is identical to Symbol
  except component dim is of length 2.  It should never be put in the
  symbol table but should be reserved for things to attach to other
  symbols like labels or notes
*/
typedef struct ShortSymbol
{				/* symbol table entry */
	mvSymbolName    name;
	long            type;	/* CHAR, REAL, BLTIN, LIST, STRUC, MACRO, UNDEF, LOGIC, PLOTINFO */
	union
	{
		double        **data;	/* for real data */
		char          **string;	/* for a character string, or MACRO */
		long          **longdata;  /* primarily for model parsing */
		whole_graph   **graph; /* for info on graph */
		struct Symbol **(*fptr) (Symbol **);	/* for a builtin function */
		struct Symbol ****Comp;
		/* for components of a structure
		    or elements of a list.  The difference
		    between a list and a structure is that
		    elements of a list are in the symbol
		    table while structure components are not
		*/
		char        ****garbage; /* for keeping track of temporary handles */

	}               value;
	struct Symbol **next;	/* next element in symbol table */
	struct Symbol **prev;	/* previous element , ie doubly linked list */
	long            nclass;	/* number of classes for factors */
	LabelsType **   labels; /* row, column etc. labels */
	NotesType  **   notes;  /* descriptive information */ 
	long            dim[2];	/* dimensions, dim[0] should be 1 */
} ShortSymbol, **ShortSymbolhandle;

/*
   Type SpecialSymbol is identical to Symbol with the addition of two
   function pointers
*/

typedef struct SpecialSymbol
{				/* symbol table entry */
	mvSymbolName    name;
	long            type;	/* CHAR, REAL, BLTIN, LIST, STRUC, MACRO, UNDEF, LOGIC */
	union
	{
		double        **data;	/* for real data */
		char          **string;	/* for a character string, or MACRO */
		long          **longdata;  /* primarily for model parsing */
		whole_graph   **graph; /* for info on graph */
		struct Symbol **(*fptr) (struct Symbol **);	/* for a builtin function */
		struct Symbol ****Comp;
		/* for components of a structure
		    or elements of a list.  The difference
		    between a list and a structure is that
		    elements of a list are in the symbol
		    table while structure components are not
		*/
		char        ****garbage; /* for keeping track of temporary handles */
	}               value;
	struct Symbol **next;	/* next element in symbol table */
	struct Symbol **prev;	/* previous element , ie doubly linked list */
	long            nclass;	/* number of classes for factors */
	LabelsType **   labels; /* row, column etc. labels */
	NotesType  **   notes;  /* descriptive information */ 
	long            dim[MAXDIMS + 1];/* dimensions, dim[0] is number of dims */
	setSpecialFunction setSpecial;
	getSpecialFunction getSpecial;
}               SpecialSymbol, **SpecialSymbolhandle;

/*
  Type for function symbols (that do not change).  Same as ShortSymbol for
  a storage saving of MAXDIMS-2 longs
*/

typedef struct ShortSymbol FunctionSymbol, **FunctionSymbolHandle;
typedef LabelsType  ** LabelsHandle;
typedef NotesType   ** NotesHandle;

/* 010109 Three synonyms for the same thing */
#define EmptyArgument   ((Symbolhandle) 0)
#define SymbolError     ((Symbolhandle) 0) /* to be used as a return value*/
#define SymbolZero      ((Symbolhandle) 0)

#define ENDPREV         ((Symbolhandle) -1)

/*
  010815 defined macro to avoid problem with MPW MRC compiler
         previously the simple == was used
*/
#if (1)
#define isENDPREV(prev)  ((unsigned long) prev == (unsigned long) ENDPREV)
#else /*1*/
#define isENDPREV(prev)  (prev == ENDPREV)
#endif /*1*/

/* The following assumes that the type of a symbol is an integer < 65536 */
#define TYPEMASK        0x0000ffffUL /*masks actual symbol type */
#define NONTYPEMASK     (~TYPEMASK)  /*selects remaining bits in Symbol.type*/
#define SPECIALMASK     0x00010000UL /* defines Special bit in Symbol.type */
#define NOMISSINGMASK   0x00020000UL /* defines NOMISSING bit in Symbol.type*/
#define SHORTSYMBOL     0x00040000UL /* define ShortSymbol bit in Symbol.type*/
#define LINKEDMASK      0x00080000UL /* define linked bit in Symbol.type*/

#define SHORTCHAR       (CHAR | SHORTSYMBOL) /*These 4 moved from globdefs.h 000817*/
#define SHORTREAL       (REAL | SHORTSYMBOL)
#define SHORTLOGIC      (LOGIC | SHORTSYMBOL)
#define SHORTLONG       (LONG | SHORTSYMBOL)
/*
  The linked bit is set only in a Symbol that is in symbol table or a
  component of a Symbol in the symbol table and is also a Symbol or a
  component of a Symbol that is not in the symbol table.  It was introduced
  components of special variable GRAPHWINDOWS that were the same
  as a GRAPH symbol in the symbol table.  Removesymbol() should not
  delete a symbol with the bit set, but should unset the bit.
*/
#define QUOTEARGSMASK   0x00100000UL /* quote args to function*/
/*
  The quote args bit may be implemented on function symbols to tell the
  lexer and parser to transform arguments to character scalars just
  as arguments to macros are transformed.  This allows delayed evaluation
  of function arguments
*/
#define LOCKEDMASK      0x00200000UL /* locked symbol*/
/*
  The locked bit may set set on symbols to prevent them from being deleted
  or assigned to
*/

#define DIM(A)  ((*(A))->dim)
#define DIMVAL(A,I) DIM(A)[I]  /*length of I-th dimension*/
#define COMPLETETYPE(A) ((*(A))->type)
#define TYPE(A) ((long)(COMPLETETYPE(A) & TYPEMASK)) /*type of symbol*/
#define NONTYPE(A) ((long)(COMPLETETYPE(A) & NONTYPEMASK))
#define NOMISSING(A) ((long)(COMPLETETYPE(A) & NOMISSINGMASK))

#define QUOTEARGS(A)  ((long)(COMPLETETYPE(A) & QUOTEARGSMASK))

#define NAME(A) ((*(A))->name)
#define NCLASS(A) ((*(A))->nclass)
#define NEXT(A) ((*(A))->next)
#define PREV(A) ((*(A))->prev)
#define LABELS(A) ((*(A))->labels)
#define NOTES(A) ((*(A))->notes)

/*
  The following two macros are used only when A is a SpecialSymbolhandle
  990415 Changed their definition  The first argument is always to
         Special symbol.  The third argument to SETSPECIAL is not
         currently used but might be used for a list of subscripts
*/
#define SETSPECIAL(A, B, C, CMD)\
	((*((SpecialSymbolhandle) A))->setSpecial)(A, B, C, CMD)
#define GETSPECIAL(A, CMD) ((*((SpecialSymbolhandle) A))->getSpecial)(A, CMD)

#define FPTR(A) ((*(A))->value.fptr)

#define COMP(A) ((*(A))->value.Comp)
#define COMPPTR(A) (*COMP(A))
#define COMPVALUE(A,I) (COMPPTR(A)[I])

#define DATA(A) ((*(A))->value.data)
#define DATAPTR(A) (*DATA(A))
#define DATAVALUE(A,I) (DATAPTR(A)[I])

#define STRING(A) ((*(A))->value.string)
#define STRINGPTR(A) (*STRING(A))
#define STRINGVALUE(A,I) (STRINGPTR(A)[I])

#define LONGDATA(A) ((*(A))->value.longdata)
#define LONGDATAPTR(A) (*LONGDATA(A))
#define LONGDATAVALUE(A,I) (LONGDATAPTR(A)[I])

#define GARBAGE(A) ((*(A))->value.garbage)
#define GARBAGEPTR(A) (*GARBAGE(A))
#define GARBAGEVALUE(A,I) (GARBAGEPTR(A)[I]) /* handle for I-th scratch var*/

#define GRAPH(A) ((*(A))->value.graph)  /* handle of graph */
#define GRAPHPTR(A) (*GRAPH(A))       /* pointer to graph */

#define HASLABELS(A) ((A) != SymbolZero && LABELS(A) != (LabelsHandle) 0)
#define HASNOTES(A) ((A) != SymbolZero && NOTES(A) != (NotesHandle) 0)

#ifdef LABELSARECHAR
#define LABELSHANDLE(A) (LABELS(A))
#else /*LABELSARECHAR*/
#define LABELSHANDLE(A) (STRING(LABELS(A)))
#endif /*LABELSARECHAR*/

#ifdef NOTESARECHAR
#define NOTESHANDLE(A)  (NOTES(A))
#else /*NOTESARECHAR*/
#define NOTESHANDLE(A)  (STRING(NOTES(A)))
#endif /*NOTESARECHAR*/

#define LABELSPTR(A) (*LABELSHANDLE(A))
#define NOTESPTR(A) (*NOTESHANDLE(A))

#define NDIMS(A) (DIM(A)[0])
#define NARGS(A) (DIM(A)[1])
#define NCOMPS(A) (DIM(A)[1])

#define isReal(A)  (getSingleType((Symbolhandle) A)==REAL) /* checks entire structure */
#define isLogic(A) (getSingleType((Symbolhandle) A)==LOGIC) /* checks entire structure */
#define isChar(A)  (getSingleType((Symbolhandle) A)==CHAR) /* checks entire structure */
#define isUndef(A) (((Symbolhandle) A) != (Symbolhandle) 0 &&\
					TYPE(A) == UNDEF)
#define isMacro(A) (((Symbolhandle) A) != (Symbolhandle) 0 &&\
					TYPE(A) == MACRO)
#define isGraph(A) (((Symbolhandle) A) != (Symbolhandle) 0 &&\
					TYPE(A) == PLOTINFO)
#define isStruc(A) (((Symbolhandle) A) != (Symbolhandle) 0 &&\
					TYPE(A) == STRUC)
#define isBltin(A) (((Symbolhandle) A) != (Symbolhandle) 0 &&\
					TYPE(A) == BLTIN)
#define isSpecial(A) (((Symbolhandle) A) != (Symbolhandle) 0 &&\
					  COMPLETETYPE(A) & SPECIALMASK)

#define setDATA(A,H)       (DATA(A)      =   (H))
#define setLONGDATA(A,H)   (LONGDATA(A)  =   (H))
#define setCOMP(A,H)       (COMP(A)      =   (H))
#define setGRAPH(A,H)      (GRAPH(A)     =   (H))
#define setSTRING(A,H)     (STRING(A)    =   (H))
#define setFPTR(A,H)       (FPTR(A)      =   (H))
#define setGARBAGE(A,H)    (GARBAGE(A)   =   (H))
#define setPREV(A,H)       (PREV(A)      =   (H))
#define setNEXT(A,H)       (NEXT(A)      =   (H))

#define setLABELS(A,H)     (LABELS(A)    =   (H))
#define setLABELSHANDLE(A,H) (LABELSHANDLE(A) = (H))
#define clearLABELS(A)     clearLabels(A)

#define setNOTES(A,H)      (NOTES(A)     =   (H))
#define clearNOTES(A)      clearNotes(A)
#define setNOTESHANDLE(A,H) (NOTESHANDLE(A) = (H))

/*
  990415 New macros to access components setSpecial and getSpecial
         These formerly were SETSPECIAL and GETSPECIAL
         These are only used when A is a SpecialSymbolhandle
*/
#define SETSPECIALFUNCTION(A) ((*(A))->setSpecial)
#define GETSPECIALFUNCTION(A) ((*(A))->getSpecial)

/* The following two macros are used only when A is a SpecialSymbolhandle */
#define setSETSPECIAL(A,FNPTR) (SETSPECIALFUNCTION(A) = (FNPTR))
#define setGETSPECIAL(A,FNPTR) (GETSPECIALFUNCTION(A) = (FNPTR))

#define setNDIMS(A,N)    (NDIMS(A)    = (N))
#define setNCOMPS(A,N)   (NCOMPS(A)   = (N))
#define setDIM(A,I,N)    (DIM(A)[I]   = (N)) /*I = 1 means 1st subscript */

#define setTYPE(A,T)     (COMPLETETYPE(A) = (T))
#define setSPECIALTYPE(A,T) (COMPLETETYPE(A) = ((T) | SPECIALMASK))
#define setNONTYPE(A,T)  (COMPLETETYPE(A) = TYPE(A) | ((T) & NONTYPEMASK))
#define setNOMISSING(A)  (COMPLETETYPE(A) |= NOMISSINGMASK)
#define clearNOMISSING(A) (COMPLETETYPE(A) &= ~NOMISSINGMASK)

#define isShort(A)       ((long) (COMPLETETYPE(A) & SHORTSYMBOL))

#define isLinked(A)      ((long) (COMPLETETYPE(A) & LINKEDMASK))
#define setLinked(A)     (COMPLETETYPE(A) |= LINKEDMASK)
#define clearLinked(A)   (COMPLETETYPE(A) &= ~LINKEDMASK)

#define isLocked(A)      ((long) (COMPLETETYPE(A) & LOCKEDMASK))
#define setLocked(A)     (COMPLETETYPE(A) |= LOCKEDMASK)
#define clearLocked(A)   (COMPLETETYPE(A) &= ~LOCKEDMASK)

#define setNCLASS(A,N)   (NCLASS(A)   = (N))
#define setNAME(A,P)     Setname(A,P)
#define setInline(A, I)  (NCLASS(A) = (I) ? -1 : 0)
#define isInline(A)      (NCLASS(A) < 0)

/* op codes for SETSPECIAL */
enum setSpecialCodes
{
	CHECKLEFTSPECIAL  = 1, /* check whether it's o.k. to assign */
	CHECKRIGHTSPECIAL,     /* check suitability of right hand side */
	CHECKASSIGNFROMSPECIAL,/* check if x <- special is OK */
	GETSPECIALHANDLE,      /* get handle to special variable */
	CLEARSPECIAL,          /* clear contents */
	ASSIGNSPECIAL          /* assign new value to */
};

/*
   950215  A new class of special symbols has been implemented.

   990422  (c) and (d) below brought up to date.

   A special symbol specialSymh has the same properties as a regular symbol,
   except:

   (a)  specialSymh may not be in the Symbol table and its name must be
        recognizable by Findspecial().
   (b)  isSpecial(specialSymh) != 0
   (c)  Additional component getSpecial is a function pointer to a function
        of type getSpecialFunction which returns a Symbolhandle and is
        accessed by value = GETSPECIAL(specialSymh, GETSPECIALHANDLE).
        If the value of specialSymh depends on the environment
		(e.g., CLIPBOARD), it is refreshed. 
		GETSPECIAL(specialSymh, CHECKASSIGNFROMSPECIAL) returns
        (Symbolhandle) 0 if a <- specialSymbol is illegal for any LHS.
   (d)  Additional component setSpecial is function pointer to a function of
        type setSpecialFunction which returns a Symbolhandle and is accessed
        by  value = SETSPECIAL(specialSymh,rhsSymh,itemList, op), where
        itemList is non-null only if assignment is to a subscript of
        specialSymh in which case itemList is a LONG symbol containing the
        positions in specialSymh to be changed.  The value returned will
        be the value of the assignment generating the call.


   For most purposes a SymbolHandle will be used to refer to a SpecialSymbol.
   Although at present, the only SpecialSymbol is CLIPBOARD which is not
   in the Symbol table, probably eventually is should be in the symbol table.

   The argument to GETSPECIAL will normally be specialSymh and it
   will normally return specialSymh, updating its value if necessary.

   SETSPECIAL(specialSymh, (Symbolhandle ) 0, (Symbolhandle) 0,
              CHECKLEFTSPECIAL)
   should return a non-zero value if and only if assignment to the special
   symbol is legal.  On assignment to a subscript, the call should be
    SETSPECIAL(specialSymh, (Symbolhandle ) 0, list, CHECKLEFTSPECIAL)
   shere list is the LIST of subscripts

   SETSPECIAL(specialSymh, symhRhs, (Symbolhandle) 0, CHECKRIGHTSPECIAL)
   is invoked on an orderinary assignment and should return a non-zero
   value if and only if the value of arg is appropriate to be assigned to
   specialSymh.  On subscripted assignment, the call is
    SETSPECIAL(specialSymh, symhRhs, list, CHECKRIGHTSPECIAL)
   Generally, there should be no need to check the subscripts since that
   will be done in Subassign() using the dimensioning information in
   specialSymh.

   SETSPECIAL(specialSymh, (Symbolhandle) 0, (Symbolhandle) 0, CLEARSPECIAL)
   clears the contents of specialSymh.  They should be replaced by something
   innocuous such as a null string.

   SETSPECIAL(specialSymh, arg, (Symbolhandle) 0, ASSIGNSPECIAL) assigns
   the value of arg to specialSymh, possibly transforming it in the process.
   If it is unable to do so it should return (Symbolhandle) 0; otherwise
   it returns the value to the assignment statement, normally the
   special Symbolhandle itself.

   SETSPECIAL(specialSymh, arg, itemList, ASSIGNSPECIAL) assigns the
   value of arg to subscripts specified in LONG Symbolhandle itemList
   whose contents are the zero-based indices of of elements of specialSymh

   As of 990501 there are two special symbols, CLIPBOARD and GRAPHWINDOWS;
   the Motif version also has SELECTION.
*/

/*
   960307  Added handles for row and column labels to Symbol structures
*/

#define FAKESYMBOLCLASS   -32768
#define markFakeSymbol(S)    (setNCLASS(S,FAKESYMBOLCLASS),\
							  setTYPE(S, TYPE(S) | SHORTSYMBOL))
#define isFakeSymbol(S)      (NCLASS(S) == FAKESYMBOLCLASS)

/*
  Macros to make it easier to allocate scratch handles that will automatically
  be destroyed in case of an interrupt or aborted output.  The handles are
  saved in GARB Symbol with name 'TRASH__' which is assumed to be in the
  scratch symbol table and hence will be released at prompt time.  Typical
  usage of these macros is as follows:

#define GTMP   0
#define GY     1
#define NTRASH 2
. . . . .
		double    **y;
		long      **tmp;
		WHERE("name")
		TRASH(NTRASH,errorExit);  * allocates trashbasket for 2 handles *

		if(!getScratch(tmp,GTMP,n,long) ||
		   !getScratch(y,GY,n,double))
		{
			goto errorExit;
		}

  Before returning, use
		emptyTrash();

  If it turns out that y is not to be discarded, but, say, is to be returned
  as part of symbol result, then use
		unTrash(GY);
		DATA(result) = y;
		emptyTrash();
		return (result);
....
	  errorExit:
		emptyTrash();
		return (0);
*/

#define NAMEFORTRASH  TRASH__

/*
  Following macro MUST immediately follow declarations, including macro WHERE,
  before any executable statements.
  It declares scratch GARB Symbol TRASH__ with space for N handles,
  transferring to LABEL in case of error.
*/

#define TRASH(N,LABEL)	Symbolhandle  NAMEFORTRASH = GarbInstall(N);\
						if(NAMEFORTRASH == (Symbolhandle) 0)goto LABEL

/*
  allocate a handle of size LENGTH*sizeof(DATATYPE) to variable NAME and put
  it in slot INDEX of TRASH__.  getScratch returns a value of zero in case
  space cannot be allocated. The handle will automatically be freed when
  TRASH__ is deleted.
  011214 modified definition so that it prints an error message if the
  size of the allocation is too large
*/

#define getScratch(NAME,INDEX,LENGTH,DATATYPE) \
	(\
		GARBAGEVALUE(NAMEFORTRASH,INDEX) = 	mygethandle(\
			(MAXHANDLESIZE/sizeof(DATATYPE) >= (LENGTH)) ?\
				(LENGTH) * sizeof(DATATYPE) : MAXHANDLESIZE + 1), \
		(NAME = (DATATYPE **) GARBAGEVALUE(NAMEFORTRASH,INDEX)) != (DATATYPE **) 0\
	)

#if (0) /*previous definition*/
#define getScratch(NAME,INDEX,LENGTH,DATATYPE) \
	(\
		GARBAGEVALUE(NAMEFORTRASH,INDEX) = mygethandle((LENGTH) * sizeof(DATATYPE)),\
		(NAME = (DATATYPE **) GARBAGEVALUE(NAMEFORTRASH,INDEX)) != (DATATYPE **) 0\
	)
#endif /*0*/

/* add handle to items in trash */
#define toTrash(NAME,INDEX) GARBAGEVALUE(NAMEFORTRASH,INDEX) = (char **) (NAME)

/*
  Clear handle slot INDEX in TRASH__ so that deleting TRASH__ will not delete
  the handle in that slot
*/
#define unTrash(INDEX) 	GARBAGEVALUE(NAMEFORTRASH,INDEX) = (char **) 0

/*
  Immediately free space associated with handle NAME in slot INDEX of TRASH__
*/
#define discardScratch(NAME,INDEX,DATATYPE) (unTrash(INDEX),\
											 mydisphandle((char **) NAME),\
											 NAME = (DATATYPE **) 0)
/*
  Remove TRASH__ from the scratch symbol table and delete it, along with
  all the handles in its slots.  It has an argument solely so that it can
  be invoked as if it were a function.
*/
#define emptyTrash()	(Removesymbol(NAMEFORTRASH) , NAMEFORTRASH = (Symbolhandle) 0)

#endif /*SYMBOLH__*/
