/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 5.00 or later
*(C)*
*(C)* Copyright (c) 2004 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/


#ifdef SEGMENTED
#if defined(MPW) || defined(MW_CW)
#pragma segment Pvalsub1
#endif /*MPW||MW_CW*/
#endif /*SEGMENTED*/

/*
  This file contains the following functions


  Static functions
  double logbeta(double p, double q)
    Log beta function

  double xinbta(double p, double q, double beta, double alpha, long * ifault)
    Inverse beta distribution (AS109)

  Externally visible functions
  double ppbeta(double p, double a, double b, int upper, long * ifault)
    Inverse beta distribution; front end to xintbta

  double betai(double x, double pin, double quin)
    Cumulative beta distribution (CACM Alg 179)

  double betanc(double x, double a, double b, double lambda, long * ifault)
    Cumulative non-central beta distribution (AS226)
*/

#include <math.h>
void myprint(char * /*msg*/);
void putOutMsg(char * /*msg*/);
#include "dbug.h"

#include "platform.h"

#ifdef USEPOW
#define intpow pow
#else /*USEPOW*/
double intpow(double /*x*/, double /*p*/);
#endif /*USEPOW*/

#define lgamma mylgamma /*960220*/

/* external function */
#if (0)
extern double macheps(), lgamma();
#endif /*0*/

#ifndef NOPROTOTYPES
#if (0)
double macheps(void);
void betabase(double */*x*/, double */*a*/, double */*b*/, long */*gia*/,
			  long */*gib*/, double */*cdf*/);
#endif /*0*/
void   betai1(double /*x*/, double /*pin*/, double /*qin*/, double * /*lower*/,
			  double * /*upper*/);
double betai(double /*x*/, double /*pin*/, double /*qin*/);
double ppbeta(double /*p*/, double /*a*/, double /*b*/, 
			  int upper, long */*ifault*/);
double betanc(double /*x*/,double /*a*/,double /*b*/,double /*lambda*/,
			  double /*errmax*/, long /*itrmax*/, int /*upper*/,
			  long * /*ifault*/);
#else /*NOPROTOTYPES*/
#if (0)
double macheps(/*void*/);v
void betabase(/*double *x, double *a, double *b, long *gia, long *gib,
				double *cdf*/);
#endif /*0*/
void   betai1(/*double x, double pin, double qin, double * lower,
				double * upper*/);
double betai(/*double x, double pin, double qin*/);
double ppbeta(/*double p, double a, double b, int upper, long *ifault*/);
double betanc(/*double x,double a,double b,double lambda, 
				double errmax, long itrmax, int upper,
				long * ifault*/);
#endif /*NOPROTOTYPES*/

extern double   lgamma(double);

/*
	D1MACH( 1) = B**(EMIN-1), the smallest positive magnitude.
	D1MACH( 2) = B**EMAX*(1 - B**(-T)), the largest magnitude.
	D1MACH( 3) = B**(-T), the smallest relative spacing.
	D1MACH( 4) = B**(1-T), the largest relative spacing.
	D1MACH( 5) = LOG10(B)
 */
#ifndef DBL_MIN /*B**(EMIN-1)*/
#define DBL_MIN   2.2250738585072014e-308   /* Min decimal value of a double */
#endif /* DBL_MIN */

#ifndef DBL_MANT_DIG
#define DBL_MANT_DIG   53
#endif /*DBL_MANT_DIG*/

#define d1mach(I) ((I == 1) ? DBL_MIN : \
                ((I == 3) ? intpow(2.0,(double) -DBL_MANT_DIG) : 0.0))

#if (0)
double          macheps(void)
{
	return (1e-14);
}
#else /*0*/
#define macheps()  (1e-14)
#endif /*0*/

/* forward declarations */
/*
  Static routines
*/

static double   logbeta(double p, double q)
{
	return (lgamma(p) + lgamma(q) - lgamma(p + q));
}

/*
  xinbta.f -- translated by f2c and modified

  Algorithm AS 109 Appl. Statist. (1977), Vol.26, No.1
  (replacing Algorithm AS 64  Appl. Statist. (1973), Vol.22, No.3)

  Remark AS R83 has been incorporated in this version.

  Computes inverse of the incomplete beta function
  ratio for given positive values of the arguments
  p and q, alpha between zero and one.
  log of complete beta function, beta, is assumed to be known.

  Auxiliary function required: betai

  SAE below is the most negative decimal exponent which does not
  cause an underflow; a value of -308 or thereabouts will often be

  020805 Added argument upper; upper != 0 signifies upper tail
*/

static double   xinbta(double p, double q, double beta, double alpha,
					   int upper, long * ifault)
{
	/* Initialized data */
	double   sae = -30.0;	/* this should be sufficient */

	/* System generated locals */
	double          ret_val, d_1, d_2;

	/* Local variables */
	int      swapTails;
	double   prev, a, g, h, r, s, t, w, y, yprev, pp, qq;
	double   sq, tx, adj, acu;
	long     iex;
	double   fpu, xin;

	/* Define accuracy and initialise. */
	fpu = sae * 10.;
	ret_val = alpha;

	/* test for admissibility of parameters */
	if (p <= 0.0 || q <= 0.0)
	{
		*ifault = 1;
		return ret_val;
	}
	if (alpha < 0.0 || alpha > 1.0)
	{
		*ifault = 2;
		return ret_val;
	}
	*ifault = 0;
	if (alpha == 0.0 || alpha == 1.0)
	{
		return ((upper) ? 1.0 - alpha : alpha);
	}
	
	if (upper)
	{
		alpha = 1.0 - alpha;
	}
	
	/* change tail if necessary */
	if (alpha <= .5)
	{
		a = alpha;
		pp = p;
		qq = q;
		swapTails = 0;
	}
	else
	{
		a = 1.0 - alpha;
		pp = q;
		qq = p;
		swapTails = 1;
	}

	/* calculate the initial approximation */
	r = sqrt(-log(a * a));
	y = r - (r * .27061 + 2.30753) / (1.0 + (r * .04481 + .99229) * r);
	if (pp > 1.0 && qq > 1.0)
	{
		r = (y * y - 3.0) / 6.0;
		s = 1.0 / (pp + pp - 1.0);
		t = 1.0 / (qq + qq - 1.0);
		h = 2.0 / (s + t);
		d_1 = y * sqrt(h + r) / h;
		d_2 = (t - s) * (r + 5.0 / 6.0 - 2.0 / (3.0 * h));
		w = d_1 - d_2;
		ret_val = pp / (pp + qq * exp(w + w));
	} /*if (pp > 1.0 && qq > 1.0)*/
	else
	{
		r = qq + qq;
		t = 1.0 / (qq * 9.);
		/* Computing 3rd power */
		d_1 = 1.0 - t + y * sqrt(t);
		t = r * (d_1 * d_1 * d_1);
		if (t <= 0.0)
		{
			ret_val = 1.0 - exp((log((1.0 - a) * qq) + beta) / qq);
		}
		else
		{
			t = (4.0 * pp + r - 2.0) / t;
			if (t <= 1.0)
			{
				ret_val = exp((log(a * pp) + beta) / pp);
			}
			else
			{
				ret_val = 1.0 - 2.0 / (t + 1.0);
			}
		}
	} /*if (pp > 1.0 && qq > 1.0){}else{}*/

	/*
      Solve for x by a modified newton-raphson method, using betai()
    */
	r = 1.0 - pp;
	t = 1.0 - qq;
	yprev = 0.0;
	sq = 1.0;
	prev = 1.0;
	if (ret_val < 1e-4)
	{
		ret_val = 1e-4;
	}
	else if (ret_val > .9999)
	{
		ret_val = .9999;
	}
	/* Computing MAX, two 2nd powers */
	d_1 = -5.0 / (pp * pp) - 1.0 / (a * a) - 13.0;
	iex = (sae > d_1) ? sae : d_1;

	acu = intpow(10.0, (double) iex);

	do
	{
		y = betai(ret_val, pp, qq);
		xin = ret_val;
		y = (y - a) * exp(beta + r * log(xin) + t * log(1.0 - xin));
		if (y * yprev <= 0.0)
		{
			prev = (sq > fpu) ? sq : fpu;
		}
		g = 1.0;
		do
		{
			adj = g * y;
			sq = adj * adj;
			if (sq < prev)
			{
				tx = ret_val - adj;
				if (tx >= 0.0 && tx <= 1.0)
				{
					if (prev <= acu || y * y <= acu)
					{
						if (swapTails)
						{
							ret_val = 1.0 - ret_val;
						}
						return ret_val;
					}
					if (tx != 0.0 && tx != 1.0)
					{
						break;
					}
				}
			}
			g /= 3.0;
		} while (1);
		if (tx == ret_val)
		{
			if (swapTails)
			{
				ret_val = 1.0 - ret_val;
			}
			break;
		}
		ret_val = tx;
		yprev = y;
	} while (1);
	return (ret_val);
} /*xinbta()*/

#if (0) /*betabase calls changed to call betai() directly*/
void betabase(double * x, double * a, double * b, long * gia, long * gib,
			  double * cdf)
{
	*cdf = betai(*x, *a, *b);
}
#endif /*0*/

/*
  020522   ppbeta() no longer computes logbeta(a,b) if a and b haven't changed
  020805   added argument upper
*/
double          ppbeta(double p, double a, double b, int upper, long * ifault)
{
	static double       lasta = -1.0, lastb = -1.0;
	static double       lnbeta = 0.0;
	
	if (a != lasta || b != lastb)
	{
		lasta = a;
		lastb = b;
		lnbeta = logbeta(a, b);
	}
	
	return (xinbta(a, b, lnbeta , p, upper, ifault));
} /*ppbeta()*/

#define Min(x,y) (((x) < (y)) ? (x) : (y))
#define Max(x,y) (((x) > (y)) ? (x) : (y))

#define Cut   1e-3

/* Translated from FORTRAN
	   July 1977 Edition.  W. Fullerton, C3, Los Alamos Scientific Lab.
	   Based on Bosten and Battiste, Remark on Algorithm 179, Comm. ACM,
	   V 17, p 153, (1974).

	   Input arguments --
	   x      upper limit of integration.  x must be in (0,1) inclusive.
	   p      first beta distribution parameter.  p must be gt 0.0.
	   q      second beta distribution parameter.  q must be gt 0.0.
	   betai  the incomplete beta function ratio is the probability that a
			  random variable from a beta distribution having parameters
			  p and q will be less than or equal to x.

  020802 Renamed betai1 returns both lower and upper tails; betai() calls
         now calls betai1() and returns the appropriate tail depending on
         the value of its new argument 'upper'

         Modified the test determining which tail was computed, attempting
         to evaluate the tail that is very small, if either is.
*/
void   betai1(double x, double pin, double qin, double * lowerProb,
			  double * upperProb)
{
	double          p, q, xb, y, dbetai, lnbeta, mu, sigma;
	static double   eps = 0.0, alneps = 0.0, sml = 0.0, alnsml = 0.0;
	int             swap;
	
	/* I'm not sure these tolerances are optimal */
	if (eps == 0.0)
	{
#ifndef d1mach
		eps = macheps(); /*d1mach(3) in original*/
		sml = macheps(); /*d1mach(1) in original*/
#else /*d1mach*/
		eps = d1mach(3);
		sml = d1mach(1);
#endif /*d1mach*/
		alneps = log(eps);
		alnsml = log(sml);
	} /*if (eps == 0.0)*/

	y = x;
	p = pin;
	q = qin;
	
	lnbeta = logbeta(p, q);
	mu = p/(p + q);
	sigma = sqrt(mu*(1.0 - mu)/(p + q + 1));
	
	/*
	  020802 Previous code was equivalent to
	    swap = (q > p && x >= 0.2 || x >= 0.8);
	*/
	if ((p < 1 || q == 1) && p*log(x) - log(p) < Cut)
	{
		swap = 0;
	}
	else if ((q < 1 || p == 1) && q*log(1 - x) < Cut)
	{
		swap = 1;
	}
	else if (fabs(x - mu) > 3.0*sigma)
	{
		swap = x > mu;
	}
	else
	{
		swap = (q > p && x >= 0.2 || x >= 0.8);
	}
	if (swap)
	{
		y = 1.0 - y;
		p = qin;
		q = pin;
	} /*if (swap)*/
	
	if ((p + q) * y / (p + 1.0) < eps)
	{
		/* use x^p/(p*B(p,q) */
		dbetai = 0.0;
		xb = p * log(Max(y, sml)) - log(p) - lnbeta;
		if (xb > alnsml && y != 0.0)
		{
			dbetai = exp(xb);
		}
	} /*if ((p + q) * y / (p + 1.0) < eps)*/
	else
	{
		/*
	       Evaluate the infinite sum first.  Term will equal
	       y**p/beta(ps,p) * (1.-ps)-sub-i * y**i / fac(i) .
	    */
		double         finsum, infsum, term, ps, xi;
		long           i, n;
		
		ps = q - floor(q);
		if (ps == 0.0)
		{
			infsum = exp(p*log(y));
		} /*if (ps == 0.0)*/
		else
		{
			xb = p * log(y) - logbeta(ps, p) - log(p);

			if (xb >= alnsml)
			{
				infsum = exp(xb);
				term = infsum * p;

				n = (long) (alneps / log(y));
				if(n < 4)
				{
					n = 4;
				}
				for (i = 1; i <= n; i++)
				{
					xi = i;
					term *= (xi - ps) * y / xi;
					infsum += term / (p + xi);
				} /*for (i = 1; i <= n; i++)*/
			} /*if (xb >= alnsml)*/
			else
			{
				infsum = 0.0;
			}
		} /*if (ps == 0.0){}else{}*/
		
		/*
		  Now evaluate the finite sum, maybe.
         */
		if (q > 1.0)
		{
			double         c, p1;
			long           ib;
			
			xb = p * log(y) + q * log(1.0 - y) - lnbeta - log(q);
			ib = (long) (xb / alnsml);
			if(ib < 0)
			{
				ib = 0;
			}
			/* Scale term as (y^p*(1-y)^q/(q*B(p,q)))/sml^ib */
			term = exp(xb - ((double) ib) * alnsml);
			c = 1.0 / (1.0 - y);
			p1 = q * c / (p + q - 1.0);

			finsum = 0.0;
			n = (int) q;
			if (q == (double) n)
			{
				n--;
			}

			for (i = 1; i <= n; i++)
			{
				if (p1 <= 1.0 && term / eps <= finsum)
				{
					break;
				}
				xi = i;
				term = (q - xi + 1.0) * c * term / (p + q - xi);

				if (term > 1.0)
				{
					ib--;
					term *= sml;
				}
				
				if (ib == 0)
				{
					finsum += term;
				}
			} /*for (i = 1; i <= n; i++)*/
		} /*if (q > 1.0)*/
		else
		{
			finsum = 0.0;
		}
		
#if (0)
		PRINT("p = %.16g, y = %.16g, infsum = %.16g, finsum = %.16g",p,y,infsum,finsum);
#endif /*0*/
		dbetai = infsum + finsum;
	} /*if ((p + q) * y / (p + 1.0) < eps){}else{}*/
	dbetai = Max(Min(dbetai, 1.0), 0.0);
	if (swap)
	{
		*lowerProb = 1.0 - dbetai;
		*upperProb = dbetai;
	}
	else
	{
		*lowerProb = dbetai;
		*upperProb = 1.0 - dbetai;
	}
} /*betai1()*/

double   betai(double x, double pin, double qin)
{
	double           lowerProb, upperProb;

	betai1(x, pin, qin, &lowerProb, &upperProb);
	return (lowerProb);
} /*betai()*/ 

/*
	Algorithm AS226 Appl. Statist. (1987) Vol. 36, No. 2
	Incorporates modification as R84 from as Vol. 39, pp311-2, 1990
	Incorporates bug fix by C. Bingham.

	Returns the cumulative probability of x for the non-central beta
	distribution with parameters a, b and non-centrality lambda

	Auxiliary routines required: alogam - log-gamma function (acm
	291 or as 245), and betain - incomplete-beta function (AS 63)

	020727 modified to handle argument upper.  If upper != 0, P(u >= x)
	is computed instead of P(u <= x)

	020802 correctly handles upper when lambda > 0 and uses new betai1() to
	get better tail probabilities.  Decreased ERRMAX and increased ITRMAX

	041006 new arguments errmax and itrmax replace hard coded ERRMAX and ITRMAX
*/

double betanc(double x, double a, double b, double lambda,
			  double errmax, long itrmax, int upper, long * ifault)
{
	double       c, errbd, gx, q, sumq, temp, xj;
	double       a0, x0;
	double       top, bottom;
	double       ualpha = 5.0;
	double       retVal = 0.0;
	double       lowerProb, upperProb;
	long         iter;
	WHERE("betanc");

	*ifault = 0;
	if(lambda < 0.0 || a <= 0.0 || b <= 0.0)
	{
		*ifault = 2;
	}
	else if(x < 0.0 || x > 1.0)
	{
		*ifault = 3;
	}
	else if (x == 0.0 || x == 1.0)
	{
		retVal = x;
	}
	else if(lambda == 0.0)
	{
		
		betai1(x, a, b, &lowerProb, &upperProb);
		
		retVal = (upper) ? upperProb : lowerProb;
	}
	else
	{/* lambda != 0.0 */
		c = lambda*0.5;

		/*      initialize the series ... */

		x0 = floor(c - ualpha*sqrt(c));
		if(x0 < 0.0)
		{
			x0 = 0.0;
		}
			
		a0 = a + x0;
		betai1(x, a0, b, &lowerProb, &upperProb);
		temp = (upper) ? upperProb : lowerProb;
		
		/*gx = x^a0*(1-x)^b/(a0*B(a0,b))*/
		gx = exp(a0*log(x) + b*log(1.0 - x) -
				 logbeta(a0, b) - log(a0));
		if (upper)
		{
			gx = -gx;
		}
		
		q = (a0 <= a) ? exp(-c) : exp(-c + x0*log(c) - lgamma(x0 + 1.0));

		xj = x0 + 1.0;			/* corrected from xj = 1.0; by kb */

		sumq = 1.0 - q;
		retVal = q*temp;
		top = a0 + b;
		bottom = a0 + 1.0;
		for (iter = 0;iter < itrmax;iter++)
		{
			temp -= gx;
			gx = x*(top++)*gx/(bottom++);
			q *= c/xj++;
			sumq -= q;
			retVal += temp*q;

			/*      check for convergence and act accordingly... */

			errbd = (temp - gx)*sumq;
			if(errbd <= errmax)
			{
				break;
			}
		} /*for (iter = 0;iter < itrmax;iter++)*/

		if (errbd > errmax)
		{
			*ifault = iter;
		}
	}
	return(retVal);
} /*betanc()*/

