/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.12 or later
*(C)*
*(C)* Copyright (c) 2002 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

/*
   960716 Changed types Long and Double to long and double
   990619 Removed include "typedefs.h"
   020324 Added define for level 3 routine dgemm()
*/
/* 
  Define names for blas routines and provide prototypes.  The following
  macros define entry names
	DAXPY, DDOT, DASUM, DCOPY, DNRM2, DROT, DROTG, DROTM, DROTMG,
	DSCAL, DSWAP, and IDAMAX

  Level 3 routines
    DGEMM

  The default definitions are of the form daxpy_, ddot_, ... but may
  be overridden by Make defines (e.g., -DDAXPY=daxpy__ )
  A routine in blas.c will be compiled only if the corresponding entry name
  macro is defined.

  The prototypes in this file follow the Fortran calling conventions, with
  all arguments pointers so any library of blas routines following this
  convention can be substituted.  If it is optimized for the machine
  architecture it might provide substantial improvement.

  All integer arguments and most integer variables have type INT.  The
  definition here translates INT to long, but it can be overridden by
  make (e.g., -DINT=int)

*/


#ifndef BLASPROTH__
#define BLASPROTH__

#include <math.h>

#ifndef INT
#define INT long
#endif /*INT*/

#ifndef DOUBLE
#define DOUBLE double
#endif /*DOUBLE*/

#ifndef DAXPY
#define DAXPY daxpy_
#endif /*DAXPY*/

#ifndef DDOT
#define DDOT ddot_
#endif /*DDOT*/

#ifndef DASUM
#define DASUM dasum_
#endif /*DASUM*/

#ifndef DCOPY
#define DCOPY dcopy_
#endif /*DCOPY*/

#ifndef DNRM2
#define DNRM2 dnrm2_
#endif /*DNRM2*/

#ifndef DROT
#define DROT drot_
#endif /*DROT*/

#ifndef DROTG
#define DROTG drotg_
#endif /*DROTG*/

#ifndef DROTM
#define DROTM drotm_
#endif /*DROTM*/

#ifndef DROTMG
#define DROTMG drotmg_
#endif /*DROTMG*/

#ifndef DSCAL
#define DSCAL dscal_
#endif /*DSCAL*/

#ifndef DSWAP
#define DSWAP dswap_
#endif /*DSWAP*/

#ifndef IDAMAX
#define IDAMAX idamax_
#endif /*IDAMAX*/

/*
  Level 3 routines
*/
#ifndef DGEMM
#define DGEMM dgemm_
#endif /*DGEMM*/

/*
   Note that the calling sequences are Fortran compatible and should be
   interchangeable with the Fortran versions, at least when Fortran is
   callable from C.
*/

#ifndef NOPROTOTYPES

/* blas.c */
/* Level 1 BLAS routines */
void DAXPY(INT */*pn*/, DOUBLE */*pa*/, DOUBLE */*x*/, INT */*pincx*/,
		   DOUBLE */*y*/, INT */*pincy*/);
DOUBLE DDOT(INT */*pn*/, DOUBLE */*x*/, INT */*pincx*/, DOUBLE */*y*/,
			INT */*pincy*/);
DOUBLE DASUM(INT */*pn*/, DOUBLE /*dx*/[], INT */*pincx*/);
void DCOPY(INT */*pn*/, DOUBLE /*dx*/[], INT */*pincx*/, DOUBLE /*dy*/[],
		   INT */*pincy*/);
DOUBLE DNRM2(INT */*pn*/, DOUBLE /*dx*/[], INT */*pincx*/);
void DROT(INT */*pn*/, DOUBLE /*dx*/[], INT */*pincx*/, DOUBLE /*dy*/[],
		  INT */*pincy*/, DOUBLE */*pdc*/, DOUBLE */*pds*/);
void DROTG(DOUBLE */*da*/, DOUBLE */*db*/, DOUBLE */*dc*/, DOUBLE */*ds*/);
void DROTM(INT */*pn*/, DOUBLE /*dx*/[], INT */*pincx*/, DOUBLE /*dy*/[],
		   INT */*pincy*/, DOUBLE /*dparam*/[5]);
void DROTMG(DOUBLE /*dd1*/, DOUBLE /*dd2*/, DOUBLE /*dx1*/, DOUBLE /*dy1*/,
			DOUBLE /*dparam*/[5]);
void DSCAL(INT */*pn*/, DOUBLE */*pda*/, DOUBLE /*dx*/[], INT */*pincx*/);
void DSWAP(INT */*pn*/, DOUBLE /*dx*/[], INT */*pincx*/, DOUBLE /*dy*/[],
		   INT */*pincy*/);
INT IDAMAX(INT */*pn*/, DOUBLE /*dx*/[], INT */*pincx*/);

/* Level 3 BLAS routines*/
void DGEMM(char * /*transa*/, char * /*transb*/, INT * /*m1*/, INT * /*n1*/,
		   INT * /*k1*/, double * /*alpha1*/, double * /*a*/, INT * /*lda1*/,
		   double * /*b*/, INT * /*ldb1*/, double * /*beta1*/, double * /*c*/,
		   INT * /*ldc1*/);

#else /*NOPROTOTYPES*/

/* blas.c */
/* Level 1 routines*/
void DAXPY(/*INT *pn, DOUBLE *pa, DOUBLE *x, INT *pincx, DOUBLE *y,
			 INT *pincy*/);
DOUBLE DDOT(/*INT *pn, DOUBLE *x, INT *pincx, DOUBLE *y, INT *pincy*/);
DOUBLE DASUM(/*INT *pn, DOUBLE dx[], INT *pincx*/);
void DCOPY(/*INT *pn, DOUBLE dx[], INT *pincx, DOUBLE dy[], INT *pincy*/);
DOUBLE DNRM2(/*INT *pn, DOUBLE dx[], INT *pincx*/);
void DROT(/*INT *pn, DOUBLE dx[], INT *pincx, DOUBLE dy[], INT *pincy,
			DOUBLE *pdc, DOUBLE *pds*/);
void DROTG(/*DOUBLE *da, DOUBLE *db, DOUBLE *dc, DOUBLE *ds*/);
void DROTM(/*INT *pn, DOUBLE dx[], INT *pincx, DOUBLE dy[], INT *pincy,
			 DOUBLE dparam[5]*/);
void DROTMG(/*DOUBLE dd1, DOUBLE dd2, DOUBLE dx1, DOUBLE dy1,
			  DOUBLE dparam[5]*/);
void DSCAL(/*INT *pn, DOUBLE *pda, DOUBLE dx[], INT *pincx*/);
void DSWAP(/*INT *pn, DOUBLE dx[], INT *pincx, DOUBLE dy[], INT *pincy*/);
INT IDAMAX(/*INT *pn, DOUBLE dx[], INT *pincx*/);

/* Level 3 routines*/
void DGEMM(/*char * transa, char * transb, INT * m1, INT * n1, INT * k1,
		   double * alpha1, double * a, INT * lda1, double * b, INT * ldb1,
		   double * beta1, double * c, INT * ldc1*/);

#endif /*NOPROTOTYPES*/
#endif /*BLASPROTH__*/
