/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.12 or later
*(C)*
*(C)* Copyright (c) 2001 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/


#ifdef SEGMENTED
#if defined(MPW) || defined(MW_CW)
#pragma segment Symbol
#endif /*MPW||MW_CW*/
#endif /*SEGMENTED*/

/*
  970115 modifications to help connect CLIPBOARD with Windows Clipboard
     Moved Macintosh specific code to macUtils.c as mygetscrap() and
	 myputscrap().

  To connect CLIPBOARD on a new platform with a Clipboard, you need to
  (a)  Add #define HASCLIPBOARD to platform.h
  (b)  Add an appropriate #include "xxIface.h" here
  (c)  Write appropriate versions of mygetscrap() and myputscrap().  For
       Windows these are in wxtwind.cc and presumably versions for Motif will
	   be there as well.

  970124 modification to connect SELECTION with Motif XA_PRIMARY selection
  980215 bug fix in deleteClipboard()
  990419 added recognition of assignment to CLIPBOARD[1] or even
         CLIPBOARD[rep(1,k)]
  990422 Renamed copyFromClipboard() and copyToClipboard() to
         getFromClipboard() and putToClipboard()
         Added op as argument to getFromClipboard() since Assign() now
         checks to see if the RHS is special and can be assigned to
         a variable.
*/

#include "globals.h"

#include "mvIface.h"

/*
  This implements special symbols CLIPBOARD and (for Motif) SELECTION

  getFromClipboard() is the getSpecial function for special symbols
  CLIPBOARD and (Motif only) SELECTION
  putToClipboard() is the setSpecial function for CLIPBOARD and SELECTION

  getFromClipboard() is invoked using
      GETSPECIAL(CLIPBOARDSYMBOL, CHECKASSIGNFROMSPECIAL)
      GETSPECIAL(CLIPBOARDSYMBOL, GETSPECIALHANDLE)
  putToClipboard() is invoked as
      SETSPECIAL(CLIPBOARDSYMBOL, 0, 0, CLEARSPECIAL)
	  SETSPECIAL(CLIPBOARDSYMBOL, 0, 0, CHECKLEFTSPECIAL)
	  SETSPECIAL(CLIPBOARDSYMBOL, rhsSymh, 0, CHECKRIGHTSPECIAL)
	  SETSPECIAL(CLIPBOARDSYMBOL, rhsSymh, 0, ASSIGNSPECIAL)
	  SETSPECIAL(CLIPBOARDSYMBOL, rhsSymh, symhPlaces, ASSIGNSPECIAL)
	   where symhPlaces is a LONG vector vector(fromPosition, toPosition)

  All the 0's are (Symbolhandle) 0's
  At present, the only place they are called is from Assign(),
  Removesymbol() and Lookup() in Symbol.c and Subassign.c in Subassig.c
*/
/*
   Discard contents of CLIPBOARDSYMBOL (or SELECTIONSYMBOL) and
   replace with null string.  Does not delete clipSymh
   Does not interact with Clipboard or any Selection at all

   971104 added check for value of myhandlelength()
   980215 Fixed bug in check of handleLength
*/

static void deleteClipboard(Symbolhandle clipSymh)
{
	char       **contents;
	
	if (clipSymh != (Symbolhandle) 0)
	{
		long         handleLength;
		
		contents = STRING(clipSymh);
		handleLength = myhandlelength(contents);
		
		if (handleLength == NULLHANDLE || handleLength > 1)
		{
			setSTRING(clipSymh, mygethandle(1));
			if (STRING(clipSymh) != (char **) 0)
			{
				mydisphandle(contents);
				STRINGVALUE(clipSymh, 0) = '\0';
			}
			else
			{
				setSTRING(clipSymh, contents); /* restore to what it was */
			}
		}
		else if (handleLength != CORRUPTEDHANDLE)
		{
			(*contents)[0] = '\0';
		}
	} /*if (clipSymh != (Symbolhandle) 0)*/
} /*deleteClipboard()*/

#if !defined(HASCLIPBOARD) && !defined(HASSELECTION)

/*
   thisSymh should be CLIPBOARDSYMBOL or SELECTIONSYMBOL
*/
static Symbolhandle getFromClipboard(Symbolhandle thisSymh, int op)
{	
	Symbolhandle       result;
	WHERE("getFromClipboard");
	
	if (op == CHECKASSIGNFROMSPECIAL)
	{
		/* x <- CLIPBOARD always OK */
		result = thisSymh;
	}
	else
	{
		/* op must be GETSPECIALHANDLE */
		if (STRING(thisSymh) == (char **) 0)
		{ /* should never happen */
			if ((TMPHANDLE = mygethandle(1)) != (char **) 0)
			{
				setSTRING(thisSymh, TMPHANDLE);
				setNDIMS(thisSymh,1);
				setDIM(thisSymh,1,1);
				setNAME(thisSymh, CLIPBOARDNAME);
				(*TMPHANDLE)[0] = '\0'; /* null string */
			}
			else
			{
				thisSymh = (Symbolhandle) 0;
			}
		} /*if (STRING(thisSymh) == (char **) 0)*/
		result = thisSymh;
	}
	return (result);
	
} /*getFromClipboard()*/

#else /*!HASCLIPBOARD&&!HASSELECTION*/

/*
   Should always return a non-zero symbol, even if there is not enough
   memory to copy from the Clipboard (or selection buffer), in which case
   it returns an empty string

   thisSymh should be CLIPBOARDSYMBOL or SELECTIONSYMBOL
*/

static Symbolhandle getFromClipboard(Symbolhandle thisSymh, int op)
{
	Symbolhandle   result = (Symbolhandle) thisSymh;
	char         **contents;
	WHERE("getFromClipboard");

	if (op == CHECKASSIGNFROMSPECIAL)
	{
		/* x <- CLIPBOARD always OK */
		result = thisSymh;
	}
	else
	{
		/*
		  op must be GETSPECIALHANDLE
		  make sure contents is null string, discardinging current contents
		*/
		deleteClipboard(thisSymh);

		contents = STRING(thisSymh);
#ifndef HASSELECTION
		setSTRING(thisSymh, mygetscrap()); /* get whatever is on Clipboard */
#else /*HASSELECTION*/
		setSTRING(thisSymh,
				  (thisSymh == CLIPBOARDSYMBOL) ?
				  mygetscrap() : mygetselection());
#endif /*HASSELECTION*/
		if (STRING(thisSymh) != (char **) 0)
		{
			mydisphandle(contents);
		}
		else
		{
			setSTRING(thisSymh, contents); /*restore previous contents*/
		}
		result = thisSymh;
	}
	
	return (result);
} /*getFromClipboard()*/

#endif /*HASCLIPBOARD*/

#define CLIPFIELDSEP   '\t'

#ifdef MACINTOSH
#define CLIPLINESEP    13  /*\r*/
#else /*MACINTOSH*/
#define CLIPLINESEP    10  /*\n*/
#endif /*MACINTOSH*/

#define CLIPFORMAT     "%0.17g"

#define CLIPMISSING    "?"

/*
  When op is CHECKLEFTSPECIAL or CLEARSPECIAL, arg is the Special
  symbol in question.

  On CHECKLEFTSPECIAL, arg is saved as static thisSymh so that
  an immediately following call with op == ASSIGNSPECIAL knows
  which symbol is being assigned to.  A call with op ==
  ASSIGNSPECIAL is always preceded by a call with op ==
  CHECKLEFTSPECIAL.
  
  When op is CHECKRIGHTSPECIAL or ASSIGNSPECIAL, arg is the
  variable being assigned to the special symbol.

  990419 handles CLIPBOARD[1] <- x or even CLIPBOARD[rep(1,k)] <- x. This
  is signalled by itemList != (Symbolhandle) 0.  
  In this case, only the last element in arg is processed.
*/

static Symbolhandle putToClipboard(Symbolhandle thisSymh, Symbolhandle arg,
									Symbolhandle itemList, int op)
{
	Symbolhandle        result = arg;
	char               *stringForMissing = CLIPMISSING;
	long                type;
	WHERE("putToClipboard");
	
	switch (op)
	{
	  case CHECKLEFTSPECIAL:
		result = thisSymh;
		/* always O.K. to assign to CLIPBOARD or SELECTION */
		break;
		
	  case CHECKRIGHTSPECIAL:
		/* check type of what is to be assigned to CLIPBOARD or SELECTION */
		type = TYPE(arg);
		if (type != REAL && type != LOGIC && type != CHAR && type != MACRO)
		{
			sprintf(OUTSTR,
					"ERROR: cannot assign variable of type %s to %s",
					typeName(type), NAME(thisSymh));
			putErrorOUTSTR();
			result = (Symbolhandle) 0;
		}
		break;
		
	  case CLEARSPECIAL:
		/* replace contents of CLIPBOARD or SELECTION with null string */
		deleteClipboard(thisSymh);
		result = (STRING(thisSymh) != (char **) 0) ?
		  thisSymh : (Symbolhandle) 0;
		break;
		
	  case ASSIGNSPECIAL:
		/*
			itemList != (Symbolhandle) 0 means this is an assignment
			to CLIPBOARD[1].  It is treated the same as an assignment
			to CLIPBOARD of the last element in arg1
		*/

		if (itemList == (Symbolhandle) 0 || isScalar(arg))
		{
			result = multipaste(arg, CLIPFORMAT, &stringForMissing,
								CLIPFIELDSEP, CLIPLINESEP);
		} /*if (itemList == (Symbolhandle) 0 || isScalar(arg))*/
		else
		{
			Symbolhandle      symhTemp;
			long              nitems = DIMVAL(itemList, 1);
			
			if (TYPE(arg) == CHAR)
			{
				symhTemp =
					CInstall(SCRATCH,
							 strlen(skipStrings(STRINGPTR(arg), nitems-1)) + 1);
			}
			else
			{
				symhTemp = RInstall(SCRATCH, 1);
			}
			if (symhTemp != (Symbolhandle) 0)
			{
				/*
					do multipaste on last element of arg
				*/
				if (TYPE(arg) == CHAR)
				{
					strcpy(STRINGPTR(symhTemp),
						   skipStrings(STRINGPTR(arg), nitems-1));
				}
				else
				{
					setTYPE(symhTemp, TYPE(arg));
					DATAVALUE(symhTemp, 0) = DATAVALUE(arg, nitems - 1);
				}
				result = multipaste(symhTemp, CLIPFORMAT, &stringForMissing,
									CLIPFIELDSEP, CLIPLINESEP);
				Removesymbol(symhTemp);
			}
			else
			{
				result = (Symbolhandle) 0;
			}
		} /*if (itemList == (Symbolhandle) 0 || isScalar(arg)){}else{}*/

#ifdef SEGMENTED
		UNLOADSEG(multipaste);
#endif /*SEGMENTED*/
		/* This should work even if arg is CLIPBOARDSYMBOL */

		/* delete current contents of thisSymh in any case */
		deleteClipboard(thisSymh);
		if (result != (Symbolhandle) 0)
		{
			/* transfer contents to thisSymh*/
			TMPHANDLE = STRING(result);
			setSTRING(result, (char **) 0);
			setSTRING(thisSymh, TMPHANDLE);
			setDIM(thisSymh, 1, 1);
			Removesymbol(result); /* remove from the symbol table and delete*/
			result = thisSymh;
#ifdef HASCLIPBOARD
			if (thisSymh == CLIPBOARDSYMBOL && !myputscrap(TMPHANDLE))
			{
				putOutErrorMsg("WARNING: unable to copy to Clipboard");
			}
#endif /*HASCLIPBOARD*/
#ifdef HASSELECTION
			if (thisSymh == SELECTIONSYMBOL && !myputselection(TMPHANDLE))
			{
				putOutErrorMsg("WARNING: unable to copy as Selection");
			}
#endif /*HASSELECTION*/
		} /*if (result != (Symbolhandle) 0)*/
		else
		{
			result = thisSymh;
		}
		break;
	} /*switch (op)*/
	return (result);
	
} /*putToClipboard()*/

/* only public function */
long iniClipboard(char * name)
{
	Symbolhandle     clipSymh;
#ifdef HASSELECTION
	int              clipboard = strcmp(name, CLIPBOARDNAME) == 0;

	clipSymh = (clipboard) ? CLIPBOARDSYMBOL : SELECTIONSYMBOL;
#else /*HASSELECTION*/
	clipSymh = CLIPBOARDSYMBOL;
#endif /*HASSELECTION*/

	if (clipSymh == (Symbolhandle) 0)
	{ /* only do something if clipSymh not initialized */
		clipSymh = Makespecialsymbol(CHAR, name,
									 (setSpecialFunction) putToClipboard,
									 (getSpecialFunction) getFromClipboard);
#ifndef HASSELECTION
		CLIPBOARDSYMBOL = clipSymh;
#else /*HASSELECTION*/
		if (clipboard)
		{
			CLIPBOARDSYMBOL = clipSymh;
		}
		else
		{
			SELECTIONSYMBOL = clipSymh;
		}
#endif /*HASSELECTION*/
		if (clipSymh != (Symbolhandle) 0 &&
			SETSPECIAL(clipSymh, (Symbolhandle) 0, (Symbolhandle) 0,
					   CLEARSPECIAL) != (Symbolhandle) 0)
		{
			setNDIMS(clipSymh, 1);
			setDIM(clipSymh, 1, 1);
			Addsymbol(clipSymh); /* add it to symbol table */
		}
	} /*if (clipSymh == (Symbolhandle) 0)*/
	
	return (clipSymh != (Symbolhandle) 0);
} /*iniClipboard()*/
