/*
*(C)* The file is part of the source distribution of Carapace 
*(C)* version 0.5 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* This file is distributed under the GNU General Public License
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

#include "wx/xrc/xmlres.h"
#include "wx/xrc/xh_all.h"
#include "wx/xrc/xh_menu.h"
#include "wx/html/htmlwin.h"
#include "wx/socket.h"
#if defined(__WXMAC__) || defined(__WXMSW__)
#include "wx/generic/dcpsg.h"
#endif


#include "cpcdefs.h"
#include "cpcstrings.h"
extern "C" {
#include "cpcinterface.h"
}

#include <setjmp.h>
#include "wx/spinctrl.h"
#include "wx/printdlg.h"
#include "wx/list.h"
#include "wx/app.h"
#include "wx/stattext.h"
#include "wx/textctrl.h"
#include "wx/pen.h"
#include "wx/defs.h"
#include "wx/menu.h"
#include "wx/dynarray.h"
#include "wx/hashmap.h"
#include "wx/splash.h"
#include "wx/listctrl.h"
#include "wx/generic/dcpsg.h"
#include "wx/generic/printps.h"
#include "wx/print.h"
#include "wx/dcps.h"
#include "wx/imaglist.h"

#include "dataobject.h"
#include "viewers.h"


class CarapaceSplitTextFrame;
class CarapaceCommandCtrl;
class CarapaceOutputCtrl;
class CarapaceCommand;
class CarapaceGraphFrame;
class CarapaceStaticGraph;
class CarapaceGraphElement;
class CarapacePanelGraph;
class CarapacePanelGraphFrame;
class CarapaceDialogFrame;
class CarapaceModalDialog;
class CarapaceNonModalFrame;
class CarapaceTPrintout;
class CarapaceGPrintout;
class CarapaceHistory;
class CarapaceHelpFrame;
class CarapacePreferences;
class CarapaceMatches;
class CarapaceGraphState;
class CarapacePostScriptGraph;
class CarapacePostScriptDC;
class CarapaceXmlResource;
class cpcRHCHolder;
class wxDC;
class wxPaintDC;
class wxColour;
class wxPostScriptDC;
class cpcSplashScreen;
class cpcInOutCtrl;
class cpcInOutItem;
class DialogObjectInfo;

extern wxRealPoint CarapaceDefaultScales;

extern wxCriticalSection FileSelectionLock;
extern wxCriticalSection BufferedOutputLock;
extern wxCriticalSection CommandLock;
extern wxMutex FileSelectionMutex;

typedef void (*MDValueExtractor)(wxDialog*, wxString&);

WX_DECLARE_LIST(CarapaceSplitTextFrame, CarapaceSplitTextFrameList);
WX_DECLARE_LIST(CarapaceGraphFrame, CarapaceGraphFrameList);
WX_DECLARE_LIST(CarapacePanelGraphFrame, CarapacePanelGraphFrameList);
WX_DECLARE_LIST(CarapaceGraphElement, CarapaceGraphElementList);
WX_DECLARE_LIST(CarapaceDialogFrame, CarapaceDialogFrameList);
WX_DECLARE_LIST(CarapaceCommand, CarapaceCommandList);
WX_DECLARE_LIST(wxString, CarapaceHistoryList);
WX_DECLARE_LIST(wxKeyEvent, CarapaceTypeAheadList);
WX_DECLARE_LIST(CarapaceMatches, CarapaceMatchList);
WX_DECLARE_LIST(CarapaceGraphState, CarapaceGraphStateList);
WX_DECLARE_OBJARRAY(wxAcceleratorEntry, ArrayofAccelerators);
WX_DECLARE_OBJARRAY(DialogObjectInfo, ArrayOfDialogObjectInfo);
WX_DECLARE_OBJARRAY(cpcInOutItem, ArrayOfInOutItems);

WX_DECLARE_STRING_HASH_MAP(MDValueExtractor,HashofValueExtractors);



// Carapace application object holds information about the
// current status of the application (as well as being the
// closest thing to a "main" routine).

class CarapaceApp: public wxApp
{
  public:

// everybody needs someplace to start or finish
	bool                          OnInit(void);
	void                          DoQuit(int);
	bool                          Uninitialized;

// callback related members
	bool                          (*InputReady)(const char *,void *,int);
	int                           (*ProcessCommand)(const char *);
	void                          (*CleanUp)(void);
	void                          (*InterruptCleanUp)(void);
	int                           (*QuitCleanUp)(int);
	int                           (*Initializer)(int,char**);
	void                          (*WindowClosed)(int,int);

// user setup/preferences information
	CarapacePreferences          *Preferences;
	wxString                      PreferencesPath;

// apple event files
	wxArrayString                 AEOpenFileNames;
	void                          MacOpenFile(const wxString & );
	void                          MacPrintFile(const wxString & );


// text frame lists and information
	CarapaceSplitTextFrameList    STFrameList;
	int                           numTextFrames;
	int                           TFrameSequence;
	int                           RunningFrame;
	CarapaceOutputCtrl           *OutputWindow;
	bool                          SplitRunning;
	int                           InitialMenuCount;

// text frame actions
	int  NewSplitTextFrame(bool untitled);
	int  NewSplitTextFrame(bool untitled, wxString& menuresource, wxString& menuname);
	void CutTextFrame(int);
	void CutSplitTextFrame(int);
	void SetTextFromFile(wxString& filename, long windno);
	int  NVisible();
	void TextFrameFront(int);

// help frame stuff
	CarapaceHelpFrame            *HelpFrame;

// graph frame lists and information
	CarapaceGraphFrameList        GFrameList;
	CarapacePanelGraphFrameList   PGFrameList;
	int                           numGraphFrames;
	int                           GFrameSequence;
	void                          FixPanels(void);
	void                          RefreshPanel(long);
	bool                          InGraphicsPause;
	int                           PausedGraphFrame;

// graph frame actions
	int                           NewGraphFrame(wxSize& , long graphno = 0);
	int                           NewGraphFrame(wxSize& , wxString& menuresource,
					wxString& menuname, long graphno = 0);
	void                          CutGraphFrame(int);
	void                          GraphFrameFront(int);
	void                          PanelFrameFront(int);
	void                          AddGraphCommand(cpc_sg_ptr);
	void                          DoGraphRequest(cpc_sg_ptr);
	void                          DoGraphParms(cpc_sg_ptr);
	void                          PaintGraph(int);
	int                           MouseRequestState;
	int                           MouseSelectedFrame;
	wxString                      MouseHelpString;

// postscript graph
	CarapacePostScriptGraph      *PSGraph;

// dialog frame list
	CarapaceDialogFrameList       DFrameList;
	int                           numDialogFrames;
	int                           DFrameSequence;
	int                           NewDialogFrame(wxString& resource, wxString& name);
	void                          CutDialogFrame(int);

// modal dialog members
	wxArrayString                 LastModalDialogNames;
	wxArrayString                 LastModalDialogValues;
	long                          LastModalDialogCount;
	wxArrayString                 RHCreators;
	HashofValueExtractors         MDVExtractors;
	void                          InitializeDialogs(void);

// Dynamic data members
//      DynamicInteraction            DynamicData;
	DynamicInteraction            *DynamicDataP;     //modified on Feb 26,2005
	void                          InitializeDynamic(void);
	void                          FreeDynamic(void);  //Append on March 7,2005
	wxArrayString                 ViewerClassNames;
	wxImageList                   *ColorGlyphPatches;
	wxArrayString                 ColorArray;
	wxArrayString                 GlyphArray;
	long                          ColorArrayLength;
	long                          GlyphArrayLength;


// menu related members
	void                          UpdateMenus(bool initial = FALSE);

// i/o related members
	void                          BufferedPrint(char *msg);
	void                          BufferToWindow(wxString& msg);
	void                          ComFlushBuffer(wxCommandEvent&);
	void                          FlushBuffer(void);
	wxString                      OutLineBuffer;
	int                           SwallowOutput;
	int                           OldSwallowOutput;
	wxString                      ArgV0;
	wxSocketServer                *mainChannelServer;
	wxSocketServer                *backChannelServer;
	wxSocketServer                *intrChannelServer;
	wxSocketBase                  *mainChannelSocket;
	wxSocketBase                  *backChannelSocket;
	wxSocketBase                  *intrChannelSocket;
	void                          OnSocketEvent(wxSocketEvent& event);
	void                          OnServerEvent(wxSocketEvent& event);
	wxString                      MainInputBuffer;
	wxString                      MainInputCommandBuffer;
	wxString                      BackInputBuffer;
	wxArrayString                 BackInputCommandBuffer;
	void                          HandleMainInput(void);
	void                          HandleBackInput(void);
	void                          BackChannelCommand(wxString& str);


// command processing members
	CarapaceCommandList           CList;
	long                          Running;
	void                          Interrupt(void);
	int                           ReturnValue;
	wxString                      CommandLine;
	int                           CommandFrame;
	CarapaceHistory              *History;
	void                          OnIdle(wxIdleEvent& event);

// printing members
	wxPrintData                  *PrintData;
	wxPageSetupData              *PageSetupData;
	wxString                      AFMPath;

// Gui
	void             GuiProcessCommand(wxCommandEvent& event);
	bool                          SplashShown;
	cpcSplashScreen              *Splash;

private:
    // any class wishing to process wxWindows events must use this macro
    DECLARE_EVENT_TABLE()

}; /*class CarapaceApp: public wxApp*/

extern CarapaceApp *thisApp;





class CarapaceSplitTextFrame : public wxFrame
{
public:

// constructor/destructor
	CarapaceSplitTextFrame(wxWindow *frame, const wxString& title, 
		const wxPoint& pos, const wxSize& size, bool hidden=FALSE);
	CarapaceSplitTextFrame(wxWindow *frame, const wxString& title, 
		const wxPoint& pos, const wxSize& size, 
		wxString& menuresource, wxString& menutitle, bool hidden=FALSE);
	~CarapaceSplitTextFrame(void);
	bool                  OKToClose(int);
	void                  OnClose(wxCloseEvent&);

	void                  DoNothing(void);

// members related to identity
	int                   TextFrameIndex;
	bool                  Untitled;

// members related to text window, content, status
	CarapaceCommandCtrl  *command_ctrl;
	wxFont               *outputFont;
	wxFont               *commandFont;
	wxColor              *outputFGColor;
	wxColor              *outputBGColor;
	CarapaceOutputCtrl   *output_ctrl;
	wxSpinCtrl           *history_spinner;
	wxStaticText         *history_counter;
	wxButton             *execute_button;
	wxButton             *back_button;
	wxButton             *forward_button;
	wxButton             *clear_button;
	wxButton             *undo_button;
	wxButton             *interrupt_button;
	wxBoxSizer           *vert_sizer;
	wxBoxSizer           *vert_sizer2;
	wxBoxSizer           *hor_sizer;
	void                  DoSpinner(wxSpinEvent&);
	wxString              LastCommand;
	wxStaticText         *RunningMeter;
	wxStaticBitmap       *SyntaxAlert;
	int                   SErrorPlace;
	wxBitmap              NullBitmap;
	wxBitmap              WarningBitmap;

// members for menu maintenance
	wxXmlResource        *Resource;
	wxString              ResourceString;
	ArrayofAccelerators  *Accelerators;
	wxAcceleratorTable   *AcceleratorTable;
	wxString              MenuResourceInUse;
	void                  SetMenuBarFromString(wxString resource, wxString menuname);
	void                  UpdateMenuBar(void);
	void                  SetUndoItem(wxUpdateUIEvent& event);
	void                  EnableHide(void);
	void                  UpdateUserCommand(wxUpdateUIEvent& event);
	
// members for menu actions
	void            Open(void);
	bool            SaveWindow(void);
	bool            SaveWindowAs(void);
	void  		OnQuit(wxCommandEvent& event);
	void  		FastQuit(wxCommandEvent& event);
	void            ProcessUserCommand(wxCommandEvent& event);
	void            OnFont(wxCommandEvent& event);
	void            ForegroundColor(wxCommandEvent& event);
	void            BackgroundColor(wxCommandEvent& event);
	void            OnPageSetup(wxCommandEvent& event);
	void            OnPrint(wxCommandEvent& event);
	void            MenuInterrupt(wxCommandEvent& event);
	void            Undo(wxCommandEvent&);
	void            DoEdits(wxCommandEvent& event);
	void            Hide(void);
	void            DoClose(void);
	void            DoFastClose(void);
	void            NewWindow(void);
	void            ShowTextFrame(wxCommandEvent& event);
	void            ShowGraphFrame(wxCommandEvent& event);
	void            GoToTop(void);
	void            GoToEnd(void);
	void            GoToCommandPoint(void);
	void            Execute(wxCommandEvent&);
	void            DoMatch(void);
	void            PlaceError(void);
	void            Forward(wxCommandEvent&);
	void            Back(wxCommandEvent&);
	void            ClearCommand(wxCommandEvent&);

private:
    // any class wishing to process wxWindows events must use this macro
    DECLARE_EVENT_TABLE()
};



class CarapaceOutputCtrl: public wxTextCtrl
{
  public:

// constructor/destructor
    CarapaceOutputCtrl(wxWindow *parent, wxWindowID id, const wxString &value,
		const wxPoint &pos, const wxSize &size, long style = 0) ;
	~CarapaceOutputCtrl(void);

// members related to identity
	CarapaceSplitTextFrame        *Frame;

// members related to command point status
	long         CmdStrPos;
	long         OldCmdStrPos;
	void         PutPrompt();
	long         ScrollBackTo;
	void         SetScrollBack();

// members related to undo
	long         LastCmdStrPos;
	long         LastTypingPos;
	long         LastLength;
	long         PasteLength;
	int          UndoStatus;
	long         UndoPlace;
	wxString     UndoBuffer;
	long         UndoBufferAllocation;
	long         LastChar;
	void         clearUndoInfo(void);
	void         ToUndoBuffer(void);
	long         FromUndoBuffer(void);

// members related to processing typing
	void         OnChar(wxKeyEvent& event);
	void         OnIdle(wxIdleEvent& event);

	long         ControlActionStatus;
	long         GetInsertionLine(void);
	bool         LoadFile(const wxString& filename);
#if defined(__WXMAC__)
	bool         SaveFile(const wxString& filename);
#endif

// members related to selection/cut/paste
	void         GetSelection(long *from, long *to);
	void         GetSelectionContents(long from, long to);
	void         GetSelectionContents(void);
	wxString     Selection;
	void         Undo(void);
	void         Cut(void);
	void         Copy(void);
	void         Paste(void);
	void         CopyToEnd(void);
	void         Execute(void);
	void         Execute(wxCommandEvent&);
#if defined(__WXMAC__)
	bool         PositionToXY(long pos, long *x, long *y);
	long         XYToPosition(long x, long y);
	wxString     GetLineText(long lineno) const; 
	int          GetNumberOfLines(void) const;
#endif

	void         matchParen(char ch);

private:
    // any class wishing to process wxWindows events must use this macro
    DECLARE_EVENT_TABLE()
}; /*class CarapaceOutputCtrl: public wxTextCtrl*/


class CarapaceCommandCtrl: public wxTextCtrl
{
public:
	CarapaceCommandCtrl(wxWindow *parent, wxWindowID id, const wxString &value,
                const wxPoint &pos, const wxSize &size, long style = 0) ;
	~CarapaceCommandCtrl();

	CarapaceSplitTextFrame *Frame;

	void         UpHistory(void);
	void         DownHistory(void);
	void         SetHistoryIndex(int);
	void         SetHistoryIndexMess(wxCommandEvent&);
	void         ProcessUserCommand(wxCommandEvent&);
	long         HistoryPosition;
	void         IncHistPtr(void);
	wxString     PendingCommand;

	wxString     Selection;
	void         GetSelection(long *from, long *to);
	void         GetSelectionContents(long from, long to);
	void         GetSelectionContents(void);
	void         GetAll();
	void         OnIdle(wxIdleEvent& event);
	void         ComOnChange(wxCommandEvent&);
	void         OnChange(void);
	void         OnChar(wxKeyEvent& event);
	void         SetContent(wxString & stuff);
	void         Paste(void);

#if defined(__WXMAC__)
	bool         PositionToXY(long pos, long *x, long *y);
	long         XYToPosition(long x, long y);
	wxString     GetLineText(long lineno) const;
	int          GetNumberOfLines(void) const;
#endif

	CarapaceMatchList     Matches;
	bool                  MisMatch;
	void                  AddMatch(int position, int paren);
	void                  PendingMatch(void);
	void                  PendingMatch(int);
	bool                  ShowMatches;
	bool                  InMatch;
	bool                  InOnChange;
	unsigned long         LastHash;
	unsigned long         Hash(wxString &);
	wxString              typeAhead;
	wxString              LastValue;
	long                  MatchStart;
	long                  MatchEnd;

private:
    // any class wishing to process wxWindows events must use this macro
    DECLARE_EVENT_TABLE()
};

class CarapaceCommand
{
public:
	wxString           command;
	int                frame;
	bool               hidecommand;
	bool               hideoutput;
};

class CarapaceHelpFrame : public wxFrame
{
public:
	CarapaceHelpFrame(wxWindow *frame, wxString& file);
	~CarapaceHelpFrame(void);

	void 		SetSourceFile(wxString &file);
	void		Forward(wxCommandEvent &event);
	void		Back(wxCommandEvent &event);
	void		Open(wxCommandEvent &event);
	void            OnClose(wxCloseEvent&);

	wxString	LastDirectory;
	
	wxHtmlWindow   *html;

	DECLARE_EVENT_TABLE()
};

class CarapaceGraphState
{
public:
	wxBrush         theBackground;
	int             theBackgroundMode;
	wxBrush         theBrush;
	wxCoord         theClippingBox[4];
	wxFont          theFont;
	int             theLogicalFunction;
	wxPoint         theOrigin;
	wxPen           thePen;
	wxColour        theTextForeground;
	double          theUserScale[2];
};

class CarapaceGraphStructure
{
public:
	CarapaceGraphStructure(const wxSize &, CarapaceGraphStructure *parent = NULL);
	CarapaceGraphStructure(void);
	~CarapaceGraphStructure(void);

	CarapaceGraphStructure   *Parent;
	long                      Depth;

	wxSize          Size;
	wxPoint         Origin;
	wxRealPoint     Scales;

	CarapaceGraphElementList  CGEList;

	CarapaceGraphElement     *Working;
	bool                      AddGraphCommand(cpc_sg_ptr);
	bool                      DoGraphRequest(cpc_sg_ptr);
	bool                      OpenStructureElement(cpc_sg_ptr);
	bool                      CloseStructureElement(void);
	bool                      DeleteStructureElement(wxString&);

	void                      Draw(wxDC &, wxSize &, wxPoint &, double);
	void                      DrawFromString(wxDC &, wxString *, double);
	void                      Draw(wxDC &, wxSize &, wxPoint &, double, double);
	void                      DrawFromString(wxDC &, wxString *, double, double);
        CarapaceGraphState        OldState;
	void                      GetOldState(wxDC &);
	void                      RestoreOldState(wxDC &);
	CarapaceGraphStateList    GraphStateList;
	void                      SaveState(wxDC &);
	void                      RestoreState(wxDC &);
};


class CarapaceGraphElement
{
public:
	long            Type;
	void           *Parent;
	union  cge
	{
		wxString                 *cgs;
		CarapaceGraphStructure   *cgstructure;
	} Data;
	~CarapaceGraphElement(void);
};

class CarapaceStaticGraph: public wxScrolledWindow
{
public:
	CarapaceStaticGraph();
	CarapaceStaticGraph( CarapaceGraphFrame *parent, wxWindowID, const wxPoint &pos, const wxSize &size );
	~CarapaceStaticGraph();

	void OnPaint( wxPaintEvent &event );
	void OnMouse( wxMouseEvent &event );
	void OnScroll( wxScrollWinEvent &event );
	void OnChar(wxKeyEvent& event);

	CarapaceGraphFrame         *Parent;
	CarapaceGraphStructure     *GStructure;
	wxSize                      Size;
	wxSize                      OriginalSize;
	bool                        DoGraphRequest(cpc_sg_ptr);
	bool                        CrossHairMode;
	void                        Draw(wxDC &, wxSize &, wxPoint &, double, double);
 
	int                         XAxisType;
	int                         YAxisType;
	double                      XAxisParms[2];
	double                      YAxisParms[2];

	int                         MouseRequestState;
	int                         MouseCoords[4];
	double                      MouseDParms[4];
	wxString                    MouseHelpString;
 
	
 
	DECLARE_DYNAMIC_CLASS(CarapaceStaticGraph)
	DECLARE_EVENT_TABLE()
};
 
// CarapaceGraphFrame 
 
class CarapaceGraphFrame: public wxFrame
{
public:
// constructor/destructor
        CarapaceGraphFrame();
	CarapaceGraphFrame(const wxString& title, const wxPoint& pos, const wxSize& size) ;
	CarapaceGraphFrame(const wxString& title, const wxPoint& pos, const wxSize& size,
		wxString& menuresource, wxString& menuname) ;
	~CarapaceGraphFrame(void);

	void                  DoNothing(void);

	void                  OnClose(wxCloseEvent&);
// members related to content
	CarapaceStaticGraph  *Graph;

// members related to identity
	int                   GraphFrameIndex;

// members for menu maintenance
	wxMenuBar            *GetDefaultMenuBar(void);
	void                  DoMenuBar(wxString& resource, wxString& menuname);
	wxXmlResource        *Resource;
	wxString              ResourceString;
	wxString              MenuResourceInUse;
	void                  SetMenuBarFromString(wxString resource, wxString menuname);
	void                  UpdateMenuBar(void);
	void                  UpdateUserCommand(wxUpdateUIEvent& event);
	ArrayofAccelerators  *Accelerators;
	
	
// members for menu actions
	bool            SaveWindowAs(void);
	void            OnCopy(wxCommandEvent& event);
	void  		OnQuit(wxCommandEvent& event);
	void  		FastQuit(wxCommandEvent& event);
	void            OnPageSetup(wxCommandEvent& event);
	void            OnPrint(wxCommandEvent& event);
	void            MenuInterrupt(wxCommandEvent& event);
	void            Hide(void);
	void            DoClose(void);
	void            ShowTextFrame(wxCommandEvent& event);
	void            ShowGraphFrame(wxCommandEvent& event);
	void            ZoomIn(wxCommandEvent& event);
	void            ZoomOut(wxCommandEvent& event);
	void            Fit(wxCommandEvent& event);
	void            FitPreserve(wxCommandEvent& event);
	void            SetAspect(wxCommandEvent& event);
	void            Revert(wxCommandEvent& event);
	void            ProcessUserCommand(wxCommandEvent& event);

	void            OnChar(wxKeyEvent& event);
	
    DECLARE_DYNAMIC_CLASS(CarapaceGraphFrame)
    DECLARE_EVENT_TABLE()
};                                                                                    

class CarapacePanelGraphFrame : public wxFrame
{
public:
	CarapacePanelGraphFrame();
	CarapacePanelGraphFrame(const wxString& title, const wxPoint& pos, 
		const wxSize& size, long no) ;
	CarapacePanelGraphFrame(const wxString& title, const wxPoint& pos, 
		const wxSize& size, long no, wxString& res, wxString& name) ;
	~CarapacePanelGraphFrame(void);
	CarapacePanelGraph    *Graph;
	int                    PanelFrameIndex;
	void                   FitBest(wxCommandEvent& event);
	int                    FitType;
	void                   OnClose(wxCloseEvent&);
	wxMenuBar            *GetDefaultMenuBar(void);
	void                  DoMenuBar(wxString& resource, wxString& menuname);
	void                  DoNothing(void);
	wxXmlResource        *Resource;
	wxString              ResourceString;
	wxString              MenuResourceInUse;
	void                  SetMenuBarFromString(wxString resource, wxString menuname);
	void                  UpdateMenuBar(void);
	void                  UpdateUserCommand(wxUpdateUIEvent& event);
	ArrayofAccelerators  *Accelerators;

	
// members for menu actions
	bool            SaveWindowAs(void);
	void  		OnCopy(wxCommandEvent& event);
	void  		OnQuit(wxCommandEvent& event);
	void  		FastQuit(wxCommandEvent& event);
	void            OnPageSetup(wxCommandEvent& event);
	void            OnPrint(wxCommandEvent& event);
	void            MenuInterrupt(wxCommandEvent& event);
	void            Hide(void);
	void            ShowTextFrame(wxCommandEvent& event);
	void            ShowGraphFrame(wxCommandEvent& event);
	void            ZoomIn(wxCommandEvent& event);
	void            ZoomOut(wxCommandEvent& event);
	void            Fit(wxCommandEvent& event);
	void            FitPreserve(wxCommandEvent& event);
	void            SetAspect(wxCommandEvent& event);
	void            Revert(wxCommandEvent& event);
	void            ProcessUserCommand(wxCommandEvent& event);
	
    DECLARE_DYNAMIC_CLASS(CarapacePanelGraphFrame)
    DECLARE_EVENT_TABLE()
};                                                                                    
 

class CarapacePanelGraph: public wxScrolledWindow
{
public:
	CarapacePanelGraph();
	CarapacePanelGraph( CarapacePanelGraphFrame *parent, wxWindowID, const wxPoint &pos, const wxSize &size , long nrows = CPC_PANELROWS, long ncols = CPC_PANELCOLS);
	CarapacePanelGraph( CarapacePanelGraphFrame *parent, wxWindowID, const wxPoint &pos, const wxSize &size , long ngraphs, long *graphnos);
	~CarapacePanelGraph();
	void 				OnChar(wxKeyEvent& event);
	void                             OnPaint( wxPaintEvent &event );
	void                             OnMouse( wxMouseEvent &event );
	void                             OnScroll( wxScrollWinEvent &event );
	void                             Draw(wxDC& dc, double unitscale);
	void                             Draw(wxDC& dc, double xscale, double yscale);
	void                             Draw(wxDC& dc, wxSize& size, wxPoint& point, double xscale, double yscale);
	CarapacePanelGraphFrame         *Parent;
	int             NCols;
	int             NRows;
	int             NGraphs;
	int            *Frames;
	bool            RowWise;
	wxSize          Size;
	wxSize          OriginalSize;
	bool            DoGraphRequest(cpc_sg_ptr);
	int             Fitting;

	wxButton *m_button;
 
	DECLARE_DYNAMIC_CLASS(CarapacePanelGraph)
	DECLARE_EVENT_TABLE()
};
 

class CarapaceDialogFrame : public wxFrame
{
public:
	void                  DoMenuBar(bool);
	void                  UpdateMenuBar(void);
private:
	DECLARE_EVENT_TABLE()
};

class DialogObjectInfo
{
public:

// constructor/destructor
	DialogObjectInfo(void){};
	~DialogObjectInfo(void){};
	
	wxString objectname;
	wxString objectclass;

private:


};

class CarapaceXmlResource : public wxXmlResource
{
public:
	CarapaceXmlResource();
	~CarapaceXmlResource();
	bool LoadFromString(const wxString& xmlstring);
	wxXmlResourceDataRecord stringXRDR;
	wxDialog *LoadDialog(wxWindow *, const wxString& );
	bool LoadDialog(wxDialog *, wxWindow *, const wxString& );
	wxFrame *LoadFrame(wxWindow* , const wxString& );
	bool LoadFrame(wxFrame* , wxWindow *, const wxString& );
	wxObject *LoadObject(wxWindow *, const wxString& , const wxString& );
	bool LoadObject(wxObject *, wxWindow *, const wxString& , const wxString& );
	wxMenuBar *LoadMenuBar(wxWindow *, const wxString& );
	wxMenuBar *LoadMenuBar(const wxString& name) { return LoadMenuBar(NULL, name); }
	wxXmlNode *FindResource(const wxString& , const wxString& , bool recur = FALSE );

	DECLARE_DYNAMIC_CLASS(CarapaceXmlResource)
};

class CarapaceModalDialog : public wxDialog
{
public:

// constructor/destructor
	CarapaceModalDialog(void);
	~CarapaceModalDialog(void);

	int SetModalDLGFromString(wxWindow * parent, wxString resourcestr, wxString MDLGname);
	int ListObjectsFromString(wxString resourcestr);
	int MakeOutput(int);
	wxXmlResource		*Resource;
	ArrayOfDialogObjectInfo	namelist;
	void                    OnButton(wxCommandEvent& event);

private:
	DECLARE_EVENT_TABLE()

};



class CarapaceNonModalFrame : public wxFrame
{
public:

// constructor/destructor
	CarapaceNonModalFrame(void);
	~CarapaceNonModalFrame(void);

	int SetDLGFromString(wxWindow * parent, wxString resourcestr, wxString MDLGname);
	int ListObjectsFromString(wxString resourcestr);
	int                     Index;
	bool			OK;
	bool			isOK();
	wxXmlResource		*Resource;
	wxArrayString		namelist;
	wxArrayString		typelist;
	void                    OnClose(wxCloseEvent&);
	void                    DoClose(void);
	void                    ProcessUserCommand(wxCommandEvent& event);
	void                    UpdateMenuBar(void);
	void  		        OnQuit(wxCommandEvent& event);

private:
	DECLARE_EVENT_TABLE()

};

class cpcColourDialogXmlHandler : public wxXmlResourceHandler
{
public:
    cpcColourDialogXmlHandler();
    ~cpcColourDialogXmlHandler();
    wxObject *DoCreateResource();
    bool CanHandle(wxXmlNode *node);

private:
	DECLARE_DYNAMIC_CLASS(cpcColourDialogXmlHandler)
    
};

class cpcFontDialogXmlHandler : public wxXmlResourceHandler
{
public:
    cpcFontDialogXmlHandler();
    ~cpcFontDialogXmlHandler();
    wxObject *DoCreateResource();
    bool CanHandle(wxXmlNode *node);

private:
	DECLARE_DYNAMIC_CLASS(cpcFontDialogXmlHandler)
    
};

class cpcInOutItem 
{
public:
	cpcInOutItem(void) {};
	~cpcInOutItem(void) {};

	wxString     Label;
	int          Index;
        wxArrayInt   LogicAttr;
	int          Dimension;
};

class cpcInOutCtrlXmlHandler : public wxXmlResourceHandler
{
public:
    cpcInOutCtrlXmlHandler();
    ~cpcInOutCtrlXmlHandler();
    wxObject *DoCreateResource();
    bool CanHandle(wxXmlNode *node);

private:
	bool m_insideCtrl, m_insideItems, m_inattrlist;
	cpcInOutItem m_item;
	ArrayOfInOutItems m_itemlist;
	wxArrayString m_labels;
	wxArrayInt  m_checked;
	long  m_fontsize;
	long  m_border;
	long  m_boxheight;
	DECLARE_DYNAMIC_CLASS(cpcInOutCtrlXmlHandler)
};


class CarapaceTPrintout : public wxPrintout
{
public:
	                    CarapaceTPrintout(char *title, CarapaceOutputCtrl *window);
	                    CarapaceTPrintout(char *title):wxPrintout(title){};
	CarapaceOutputCtrl *OPrintingCtrl;
	wxFont              PrintingFont;
	void                GetPageInfo(int *minPage, int *maxPage, int *pageFrom, int *pageTo);
	bool                OnPrintPage(int page);
	bool                HasPage(int page);
	bool                OnBeginDocument(int startPage, int endPage);

	long                PageHeight;
	long                LineHeight;
	long                TopMargin;
	long                LeftMargin;

	long                TotalLines;
	long                LinesPerPage;
	long                TotalPages;
	float               OverallScale;
};

class CarapaceGPrintout : public wxPrintout
{
public:
	                    CarapaceGPrintout(char *title, long windno);
	                    CarapaceGPrintout(char *title, CarapaceGraphFrame *cgfp);
	                    CarapaceGPrintout(char *title, CarapacePanelGraphFrame *cpfp);
	                    CarapaceGPrintout(char *title):wxPrintout(title){};
	void                GetPageInfo(int *minPage, int *maxPage, int *pageFrom, int *pageTo);
	bool                OnPrintPage(int page);
	bool                HasPage(int page);
	bool                OnBeginDocument(int startPage, int endPage);

	long                Windno;
	CarapaceGraphFrame *GraphFramePtr;
	CarapacePanelGraphFrame *PanelFramePtr;

};

class CarapacePostScriptGraph
{
public:
	                    CarapacePostScriptGraph(wxString& filename, bool append);
  	                   ~CarapacePostScriptGraph();
	void                OpenPage(wxSize&);
	void                ClosePage(void);
	void                CloseFile(void);

	CarapaceGraphStructure *GStructure;
	wxPostScriptDC         *dc;
};

class CarapacePostScriptDC : public wxPostScriptDC
{
public:
	CarapacePostScriptDC(const wxPrintData& printData);
	~CarapacePostScriptDC();
	void DoDrawRotatedText( const wxString& text, wxCoord x, wxCoord y, double angle );
	void EndDoc(void);
};

class CarapacePostScriptPrinter : public wxPostScriptPrinter
{
public:
	CarapacePostScriptPrinter(wxPrintDialogData *data = (wxPrintDialogData *) NULL);
	~CarapacePostScriptPrinter();
	bool Print(wxWindow *parent, wxPrintout *printout, bool prompt = true);
	DECLARE_DYNAMIC_CLASS(CarapacePostScriptPrinter)
};

class CarapacePrintFactory : public wxNativePrintFactory
{
public:
	CarapacePrintFactory() {}
	~CarapacePrintFactory() {}
	wxPrinterBase *CreatePrinter( wxPrintDialogData* data );
	DECLARE_DYNAMIC_CLASS(CarapacePrintFactory)
};


class CarapaceHistory
{
public:
	                    CarapaceHistory(long);
	                    ~CarapaceHistory(void);
	long                HistoryLength;
	unsigned long       Capacity;
	CarapaceHistoryList commands;
	void                ClearCommands(void);
	void                AddCommand(wxString&);
	wxString*           GetCommand(long);
	void                ChangeHistoryLength(unsigned long);
	bool                HasLine(long);
};

class CarapacePreferences
{
public:

                            CarapacePreferences();
	void                SetPreferences(void);

// general
	int                 windowOffset;

// stuff about text windows
	wxString            mainPrompt;
	int                 textWindowSize[2];
	wxSize              tWindowSize;
	int                 historyLength;
	int                 textFontSize;
	int                 textFontFamily;
	wxString            textFontFaceName;
	int                 textFontStyle;
	int                 textFontWeight;
	wxString            textFontColorName;
	int                 textFontColorVals[3];
	wxString            textBackgroundColorName;
	int                 textBackgroundColorVals[3];
	int                 commandPaneLines;

// stuff about help window
	int                 helpWindowSize[2];
	wxSize              hWindowSize;

// stuff about printing
	bool                useA4Paper;
	int                 printerFontSize;
	int                 printerFontFamily;
	wxString            printerFontFaceName;
	int                 printerFontWeight;
	int                 printerFontStyle;

// stuff about graph window
	int                 graphWindowSize[2];
	wxSize              gWindowSize;
	int                 graphFontSize;
	int                 graphFontFamily;
	wxString            graphFontFaceName;
	int                 graphFontWeight;
	int                 graphFontStyle;
	wxString            graphFontColorName;
	int                 graphFontColorVals[3];
	wxString            graphBackgroundColorName;
	int                 graphBackgroundColorVals[3];
};

class CarapaceMatches
{
public:
	unsigned int        position;
	char                paren;
};

// utilities

// alert dialog box
extern void CarapaceAlert(const char *);
 
// beep
extern void CarapaceBeep(long);
 
// delay
extern void CarapaceWaitForTicks(long);
 
// printing
extern void CarapaceUseA4Paper(void);
extern void CarapaceSetLandscape(void);
 
// utilities
extern int  CarapaceGetOsVersion(void);
extern char * CarapaceGetTextFromUser(char *, char *);
extern char * CarapaceFindDir(const char *, const char *);
extern char * CarapaceFindFile(const char *, const char *, const char *,
	const char *, const char *, int);
 
void CarapaceShowCurrentCmdWindow(void);
void CarapaceSetTextFrameFromFile(char * , long );

extern void CarapaceGetTextColors(wxColor **fg, wxColor **bg);

extern void CarapacePushShort(wxString *, long);
extern void CarapacePushString(wxString *, wxString &);
extern short CarapaceGetShort(wxString *, long *);
extern wxString * CarapaceGetString(wxString *, long *);
extern wxColour *CarapaceColorFromName(long);
extern int Carapace2WxStyle(long);
extern void CarapaceDrawGlyph(wxDC &, long, long, long, long, long);
extern void CarapaceDrawGlyph2(wxDC &, double, double, long, double, long);

extern wxMenu *FixCommandSubmenu(wxMenuBar *);
extern wxMenu *FixGraphSubmenu(wxMenuBar *);
extern wxMenu *FixDialogSubmenu(wxMenuBar *);

// xml utils
extern void getNVPairs(wxString&, wxString&, wxString&);
extern int GetXmlSkeleton(wxString, wxString&, wxString, wxString);

extern "C" {
extern int sortgraphs(const CarapaceGraphFrame **f1, const CarapaceGraphFrame **f2);
extern int sortpanels(const CarapacePanelGraphFrame **f1, const CarapacePanelGraphFrame **f2);
}


class cpcMenuItem : public wxMenuItem
{
public:
	cpcMenuItem(wxMenu* parentMenu, int id, const wxString& text,
		 const wxString& help, wxItemKind kind) : wxMenuItem(parentMenu,
		 id, text, help,kind){};
	wxString actionname;
	wxString action;
	wxString selectionaction;
	wxString label;
	wxString selectionlabel;
	wxString accel;
	bool     hidecommand;
	bool     hideoutput;
	bool     leaveoncommandline;
};

class cpcMenuXmlHandler : public wxXmlResourceHandler
{
public:
    cpcMenuXmlHandler();
    wxObject *DoCreateResource();
    bool CanHandle(wxXmlNode *node);

private:
    bool m_insideMenu;
};

class cpcAccelXmlHandler : public wxXmlResourceHandler
{
public:
    cpcAccelXmlHandler();
    wxObject *DoCreateResource();
    bool CanHandle(wxXmlNode *node);
    ArrayofAccelerators  Accelerators;

private:
    bool m_insideMenu;
};


class cpcAccelArrayXmlHandler : public wxXmlResourceHandler
{
public:
    cpcAccelArrayXmlHandler();
    wxObject *DoCreateResource();
    bool CanHandle(wxXmlNode *node);
    ArrayofAccelerators  *Accelerators;

private:
    bool m_insideMenu;
};

extern void GetAccelArrayFromMenuBar(ArrayofAccelerators &, wxMenuBar *);
extern void GetAccelArrayFromMenu(ArrayofAccelerators &, wxMenu *);

class cpcRHCHolder
{
public:
	wxXmlResourceHandler *Create(void);
};

class cpcSplashScreen : public wxSplashScreen
{
public:
        cpcSplashScreen(const wxBitmap& bitmap, long splashStyle, int milliseconds,
                 wxWindow* parent, wxWindowID id, const wxPoint& pos = wxDefaultPosition,
                 const wxSize& size = wxDefaultSize,
                 long style = wxSIMPLE_BORDER|wxFRAME_NO_TASKBAR|wxSTAY_ON_TOP) :
        wxSplashScreen(bitmap, splashStyle, milliseconds, parent, id, pos , size , style ) {};
        void OnWindowClose(wxCloseEvent& event)
        {
               thisApp->SplashShown = FALSE;
		wxSplashScreen::OnCloseWindow(event);
        };
    	DECLARE_EVENT_TABLE()
};




class cpcInListCtrl : public wxListCtrl
{
public:
	cpcInListCtrl(wxWindow *);
	cpcInListCtrl(wxWindow *, long);
	~cpcInListCtrl();
	cpcInOutCtrl *Parent;
	void OnSelected(wxListEvent&);
private:
	DECLARE_EVENT_TABLE();
};

class cpcOutListCtrl : public wxListCtrl
{
public:
	cpcOutListCtrl(wxWindow *);
	cpcOutListCtrl(wxWindow *, long);
	~cpcOutListCtrl();
	cpcInOutCtrl *Parent;
	void OnSelected(wxListEvent&);
private:
	DECLARE_EVENT_TABLE();
};

class cpcLogicAttrCheckBox : public wxCheckBox
{
public:
	cpcLogicAttrCheckBox(wxWindow *,wxString);
	~cpcLogicAttrCheckBox();
	cpcInOutCtrl   *Parent;
	int          Index;
	void         OnClick(wxCommandEvent &);

	DECLARE_EVENT_TABLE();
};

class cpcInOutCtrl : public wxPanel
{
public:
	cpcInOutCtrl(void);
	cpcInOutCtrl(wxWindow *parent,
		wxArrayString logiclabels,
		wxArrayInt logicchecked,
                wxWindowID winid = wxID_ANY,
                const wxPoint &pos = wxDefaultPosition,
                const wxSize &size = wxDefaultSize);
	cpcInOutCtrl(wxWindow *parent,
		wxArrayString logiclabels,
		wxArrayInt logicchecked,
		long fontsize,
		long border,
		long boxheight,
                wxWindowID winid = wxID_ANY,
                const wxPoint &pos = wxDefaultPosition,
                const wxSize &size = wxDefaultSize);
	bool Create(wxWindow *parent,
                wxWindowID winid = wxID_ANY,
                const wxPoint &pos = wxDefaultPosition,
                const wxSize &size = wxDefaultSize);
        ~cpcInOutCtrl();

	ArrayOfInOutItems  Items;
	void               AddItem(cpcInOutItem&);

	cpcInListCtrl     *Ins;
	cpcOutListCtrl    *Outs;
	void               PutinIn(long);
	void               PutinOut(long);
	wxString           GetSelection(void);
	int                MaxSelected;
	wxFont             font;

	wxArrayInt     ActiveLogicAttr;
	wxArrayString  LogicAttrLabels;
	void           AddLogicAttr(wxString);
	wxArrayInt     ActiveDimensionAttr;

	void           ResetSubset(void);
	void           ResetSubset(int);

private:
	DECLARE_DYNAMIC_CLASS(cpcInOutCtrl);
	DECLARE_EVENT_TABLE();
};



extern FILE *LOGFILE;
#define LOGIT(A) \
fprintf(LOGFILE,"%s\n",A);\
fflush(LOGFILE);\

