#ifndef _WX_COLORCTRL_H
#define _WX_COLORCTRL_H

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma interface "colrctrl.h"
#endif

#include "wx/setup.h"
#include "wx/defs.h"
#include "wx/object.h"
#include "wx/gdicmn.h"
#include "wx/cmndata.h"
#include "wx/control.h"
#include "wx/generic/colrdlgg.h"

//#define wxID_ADD_CUSTOM     3000
//#define wxID_RED_SLIDER     3001
//#define wxID_GREEN_SLIDER   3002
//#define wxID_BLUE_SLIDER    3003

class WXDLLEXPORT wxSlider;
class WXDLLEXPORT cpcGenericColourControl : public wxControl
{
public:
    cpcGenericColourControl();
    cpcGenericColourControl(wxWindow *parent, wxWindowID id, const wxString& label,
           wxColourData *data = (wxColourData *) NULL,
           const wxPoint& pos = wxDefaultPosition,
           const wxSize& size = wxDefaultSize, long style = 0,
           const wxValidator& validator = wxDefaultValidator,
           const wxString& name = wxControlNameStr);
    virtual ~cpcGenericColourControl();

    bool Create(wxWindow *parent, wxWindowID id, const wxString& label,
           wxColourData *data = (wxColourData *) NULL,
           const wxPoint& pos = wxDefaultPosition,
           const wxSize& size = wxDefaultSize, long style = 0,
           const wxString& name = wxControlNameStr);

    wxColourData &GetColourData() { return colourData; }

    virtual bool Show(bool show = TRUE);

    // Internal functions
    void OnMouseEvent(wxMouseEvent& event);
    void OnPaint(wxPaintEvent& event);

    wxSizer * CreateButtonSizer( long flags );

    virtual void CalculateMeasurements();
    virtual void CreateWidgets();
    virtual void InitializeColours();

    virtual void PaintBasicColours(wxDC& dc);
    virtual void PaintCustomColours(wxDC& dc);
    virtual void PaintCustomColour(wxDC& dc);
    virtual void PaintHighlight(wxDC& dc, bool draw);

    virtual void OnBasicColourClick(int which);
    virtual void OnCustomColourClick(int which);

    void OnAddCustom(wxCommandEvent& event);

    void OnRedSlider(wxCommandEvent& event);
    void OnGreenSlider(wxCommandEvent& event);
    void OnBlueSlider(wxCommandEvent& event);

    void OnCloseWindow(wxCloseEvent& event);

protected:
    wxColourData colourData;
    wxWindow *controlParent;

    // Area reserved for grids of colours
    wxRect standardColoursRect;
    wxRect customColoursRect;
    wxRect singleCustomColourRect;

    // Size of each colour rectangle
    wxPoint smallRectangleSize;

    // For single customizable colour
    wxPoint customRectangleSize;

    // Grid spacing (between rectangles)
    int gridSpacing;

    // Section spacing (between left and right halves of dialog box)
    int sectionSpacing;

    // 48 'standard' colours
    wxColour standardColours[48];

    // 16 'custom' colours
    wxColour customColours[16];

    // One single custom colour (use sliders)
    wxColour singleCustomColour;

    // Which colour is selected? An index into one of the two areas.
    int colourSelection;
    int whichKind; // 1 for standard colours, 2 for custom colours,

    wxSlider *redSlider;
    wxSlider *greenSlider;
    wxSlider *blueSlider;

    int buttonY;

    int okButtonX;
    int customButtonX;

    //  static bool colourDialogCancelled;

private:
    DECLARE_EVENT_TABLE();
    DECLARE_DYNAMIC_CLASS(cpcGenericColourControl)
};

/* This shouldn't be necessary, we have a #define in wx/colordlg.h.
#ifdef __WXGTK__
typedef cpcGenericColourControl wxColourDialog;
#endif
*/

#endif // _WX_COLORCTRL_H
