#include "cpcplatform.h"

#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "cpccontrols.h"

IMPLEMENT_DYNAMIC_CLASS(cpcDisplayButton,wxButton)

BEGIN_EVENT_TABLE(cpcDisplayButton,wxButton)
    EVT_BUTTON( CPC_DLGCTRL_TEXTBUTTON, cpcDisplayButton::OnClicked )
END_EVENT_TABLE()

cpcDisplayButton::cpcDisplayButton()
{
}

cpcDisplayButton::cpcDisplayButton(wxWindow *parent, wxWindowID id, 
		const wxString& label, const wxPoint& pos, 
		const wxSize& size, long style )
{
      Create(parent, id, label, pos, size, style);
}

cpcDisplayButton::~cpcDisplayButton()
{     
}

void cpcDisplayButton::OnClicked( wxCommandEvent &event )
{
        wxSizer *topsizer;
        wxTextCtrl *text;
	wxHtmlWindow *html;
        wxDialog dlg(this, -1, title);

        topsizer = new wxBoxSizer(wxVERTICAL);

	if(textcontent != "" || textfile != "")
	{
        	text = new wxTextCtrl(&dlg, -1, "", pos, size, wxTE_READONLY | wxTE_MULTILINE);
		if(textcontent != "")
		{
			text->SetValue(textcontent);
		}
		else
		{
			text->LoadFile(textfile);
		}
		topsizer -> Add(text, 1, wxALL, 10);
	}
	else if(htmlcontent != "" || htmlfile != "")
	{
		html = new wxHtmlWindow(&dlg, -1, pos, size);
		if(htmlcontent != "")
		{
			html->LoadPage(htmlcontent);
		}
		else
		{
			html->LoadFile(htmlfile);
		}
		topsizer -> Add(html, 1, wxALL, 10);
	}


        wxButton *bu1 = new wxButton(&dlg, wxID_OK, _("OK"));
        bu1 -> SetDefault();

        topsizer -> Add(bu1, 0, wxALL | wxALIGN_RIGHT, 15);

        dlg.SetAutoLayout(TRUE);
        dlg.SetSizer(topsizer);
        topsizer -> Fit(&dlg);

        dlg.ShowModal();
}


IMPLEMENT_DYNAMIC_CLASS(cpcDisplayButtonXmlHandler,wxXmlResourceHandler)

cpcDisplayButtonXmlHandler::cpcDisplayButtonXmlHandler() 
: wxXmlResourceHandler() 
{
    XRC_ADD_STYLE(wxBU_LEFT);
    XRC_ADD_STYLE(wxBU_RIGHT);
    XRC_ADD_STYLE(wxBU_TOP);
    XRC_ADD_STYLE(wxBU_BOTTOM);
    XRC_ADD_STYLE(wxBU_EXACTFIT);
    AddWindowStyles();
}

wxObject *cpcDisplayButtonXmlHandler::DoCreateResource()
{ 
XRC_MAKE_INSTANCE(textbutton, cpcDisplayButton)

	textbutton->Create(m_parentAsWindow,
                    CPC_DLGCTRL_TEXTBUTTON,
                    GetText(wxT("label")),
                    GetPosition(), wxDefaultSize,
                    GetStyle());
	textbutton->title = GetText(wxT("title"));
	textbutton->textcontent = GetText(wxT("textcontent"));
	textbutton->textfile = GetText(wxT("textfile"));
	textbutton->htmlcontent = GetText(wxT("htmlcontent"));
	textbutton->htmlfile = GetText(wxT("htmlfile"));
	textbutton->size.SetWidth(600);
	textbutton->size.SetHeight(400);
	if(HasParam("size"))
	{
		textbutton->size = GetSize();
	}
	textbutton->pos = wxDefaultPosition;
	if(HasParam("position"))
	{
		textbutton->pos = GetPosition();
	}

	if (GetBool(wxT("default"), 0))
	{
		textbutton->SetDefault();
	}
	SetupWindow(textbutton);
    
	return textbutton;
}

bool cpcDisplayButtonXmlHandler::CanHandle(wxXmlNode *node)
{
    return IsOfClass(node, wxT("cpcDisplayButton"));
}

IMPLEMENT_DYNAMIC_CLASS(cpcHtmlWindow,wxHtmlWindow)

BEGIN_EVENT_TABLE(cpcHtmlWindow, wxHtmlWindow)
    EVT_LEFT_DOWN(cpcHtmlWindow::OnMouseEvent)
    EVT_RIGHT_DOWN(cpcHtmlWindow::OnMouseEvent)
END_EVENT_TABLE()


cpcHtmlWindow::cpcHtmlWindow() : wxHtmlWindow()
{
	EnableLinks = TRUE;
}

cpcHtmlWindow::cpcHtmlWindow(wxWindow *parent, wxWindowID id, const wxPoint& pos,
	 const wxSize& size, long style) : wxHtmlWindow(parent,id,pos,size,style)
{
	EnableLinks = TRUE;
}

cpcHtmlWindow::~cpcHtmlWindow()
{     
}

void cpcHtmlWindow::OnMouseEvent(wxMouseEvent& event)
{     
	if(EnableLinks)
	{
		event.Skip();
	}
}

IMPLEMENT_DYNAMIC_CLASS(cpcHtmlControlXmlHandler,wxXmlResourceHandler)

cpcHtmlControlXmlHandler::cpcHtmlControlXmlHandler() 
: wxXmlResourceHandler() 
{
    AddWindowStyles();
}

wxObject *cpcHtmlControlXmlHandler::DoCreateResource()
{ 
	//cpcHtmlWindow *html = new cpcHtmlWindow(m_parentAsWindow, -1, wxDefaultPosition, wxDefaultSize); 
	cpcHtmlWindow *html = new cpcHtmlWindow(m_parentAsWindow, -1, wxDefaultPosition, wxSize(800,500)); 
	if(HasParam("htmlcontent"))
	{
		html->SetPage(GetText("htmlcontent"));
	}
	else
	{
		html->LoadFile(GetText("htmlfile"));
	}
	if(HasParam("size"))
	{
		html->SetSize(GetSize());
	}
	if(HasParam("enablelinks"))
	{
		html->EnableLinks = GetBool("enablelinks");
	}
    
	return html;
}

bool cpcHtmlControlXmlHandler::CanHandle(wxXmlNode *node)
{
    return IsOfClass(node, wxT("cpcHtmlControl"));
}


IMPLEMENT_DYNAMIC_CLASS(cpcTextControlXmlHandler,wxXmlResourceHandler)

cpcTextControlXmlHandler::cpcTextControlXmlHandler() 
: wxXmlResourceHandler() 
{
    AddWindowStyles();
}

wxObject *cpcTextControlXmlHandler::DoCreateResource()
{ 
	//wxTextCtrl *text = new wxTextCtrl(m_parentAsWindow, -1, "",wxDefaultPosition, wxDefaultSize,
	wxTextCtrl *text = new wxTextCtrl(m_parentAsWindow, -1, "",wxDefaultPosition, wxSize(600,400),
		wxTE_RICH | wxTE_MULTILINE); 
	if(HasParam("textcontent"))
	{
		text->SetValue(GetText("textcontent"));
	}
	else
	{
		text->LoadFile(GetText("textfile"));
	}
	if(HasParam("size"))
	{
		text->SetSize(GetSize());
	}
	if(HasParam("enableedit"))
	{
		text->SetEditable(GetBool("enableedit"));
	}
    
	return text;
}

bool cpcTextControlXmlHandler::CanHandle(wxXmlNode *node)
{
    return IsOfClass(node, wxT("cpcTextControl"));
}

