/*
*(C)* The file is part of the source distribution of Carapace
*(C)* version 0.5 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* This file is distributed under the GNU General Public License
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

/* This file based the the wxWidgets file xh_menu.cpp by Vaclav Slavik */

#include "cpcplatform.h"
 
// modified by GWO for Carapace Menu items

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/gdicmn.h"
#include "wx/fontdlg.h"
#include "wx/colour.h"
#include "wx/caret.h"
#include "wx/print.h"
#include "wx/printdlg.h"
#include "wx/colordlg.h"
#include "wx/fontmap.h"
#include "wx/splitter.h"

#include "wx/xrc/xh_menu.h"
#include "wx/menu.h"
#include "wx/frame.h"
#include "wx/cmndata.h"
#include "wx/arrimpl.cpp"

#include "cpcdefs.h"
#include "carapace.h"


cpcMenuXmlHandler::cpcMenuXmlHandler() : 
        wxXmlResourceHandler(), m_insideMenu(FALSE)
{
    //XRC_ADD_STYLE(wxMENU_TEAROFF);
}

wxObject *cpcMenuXmlHandler::DoCreateResource()
{
    if (m_class == wxT("cpcMenu"))
    {
        wxMenu *menu = new wxMenu();
        wxString title = GetText(wxT("label"));
        wxString help = GetText(wxT("help"));
        
        bool oldins = m_insideMenu;
        m_insideMenu = TRUE;
        CreateChildren(menu, TRUE/*only this handler*/);
        m_insideMenu = oldins;

        wxMenuBar *p_bar = wxDynamicCast(m_parent, wxMenuBar);
        if (p_bar)
            p_bar->Append(menu, title);
        else
        {
            wxMenu *p_menu = wxDynamicCast(m_parent, wxMenu);
            if (p_menu)
                p_menu->Append(GetID(), title, menu, help);
        }

        return menu;
    }

    else
    {
        wxMenu *p_menu = wxDynamicCast(m_parent, wxMenu);
        
        if (m_class == wxT("separator"))
            p_menu->AppendSeparator();
        else if (m_class == wxT("break"))
            p_menu->Break();
        else /*cpcMenuItem*/
        {          
            int id = GetID();
            wxString label = GetText(wxT("label"));
            wxString accel = GetText(wxT("accel"), FALSE);
            wxString fullLabel = label;
            if (!accel.IsEmpty())
                fullLabel << wxT("\t") << accel;

            wxItemKind kind = wxITEM_NORMAL;
            if (GetBool(wxT("radio")))
                kind = wxITEM_RADIO;
            if (GetBool(wxT("checkable")))
            {
                wxASSERT_MSG( kind == wxITEM_NORMAL, _T("can't have both checkable and radion button at once") );
                kind = wxITEM_CHECK;
            }

            cpcMenuItem *mitem = new cpcMenuItem(p_menu, id, fullLabel,
                                               GetText(wxT("help")), kind);
                                               
#if wxCHECK_VERSION(2,3,0) || defined(__WXMSW__)
                if (HasParam(wxT("bitmap")))
                    mitem->SetBitmap(GetBitmap(wxT("bitmap"), wxART_MENU));
#endif
            p_menu->Append((wxMenuItem *) mitem);
            mitem->Enable(GetBool(wxT("enabled"), TRUE));
            if (kind == wxITEM_CHECK)
                mitem->Check(GetBool(wxT("checked")));

            wxString action = GetText(wxT("action"));
            wxString selectionaction = GetText(wxT("selectionaction"));
            wxString selectionlabel = GetText(wxT("selectionlabel"));
            bool leaveoncommandline = GetBool(wxT("leaveoncommandline"));
            bool hidecommand = GetBool(wxT("hidecommand"));
            bool hideoutput = GetBool(wxT("hideoutput"));
            mitem->action = action;
            mitem->selectionaction = selectionaction;
            mitem->label = label;
            mitem->selectionlabel = selectionlabel;
            mitem->accel = accel;
            mitem->leaveoncommandline = leaveoncommandline;
            mitem->hidecommand = hidecommand;
            mitem->hideoutput = hideoutput;


        }
        return NULL;
    }
}



bool cpcMenuXmlHandler::CanHandle(wxXmlNode *node)
{
    return IsOfClass(node, wxT("cpcMenu")) ||
           (m_insideMenu && 
               (IsOfClass(node, wxT("cpcMenuItem")) ||
                IsOfClass(node, wxT("break")) ||
                IsOfClass(node, wxT("separator")))
           );
}

WX_DEFINE_OBJARRAY(ArrayofAccelerators);

cpcAccelXmlHandler::cpcAccelXmlHandler() : 
        wxXmlResourceHandler()
{
	Accelerators.Empty();
}


wxObject *cpcAccelXmlHandler::DoCreateResource()
{
    int  i,count;
    if (m_class == wxT("cpcAccelTable"))
    {
        CreateChildren(NULL, TRUE/*only this handler*/);

	count = Accelerators.GetCount();
	if(count == 0)
	{
		return NULL;
	}
	// I got strange crashes when I tried to set up entries with a new, so
	// we do this
	wxAcceleratorEntry entries[1000];
	if(count > 1000) 
	{
		count = 1000;
	}
	for(i=0;i<count;i++)
	{
		entries[i] = Accelerators.Item(i);
	}
	wxAcceleratorTable *table = new wxAcceleratorTable(count,entries);

        return table;
    }

    else
    {
        
        int id = GetID();
        wxString modifier = GetText(wxT("modifier"));
	int intmod;
	if(modifier == "wxACCEL_ALT")
	{
		intmod = wxACCEL_ALT;
	}
	else if(modifier == "wxACCEL_SHIFT")
	{
		intmod = wxACCEL_SHIFT;
	}
	else if(modifier == "wxACCEL_CTRL")
	{
		intmod = wxACCEL_CTRL;
	}
	else
	{
		intmod = wxACCEL_NORMAL;
	}

        char key = GetText(wxT("key")).GetChar(0);

  	wxAcceleratorEntry *entry = new wxAcceleratorEntry(intmod,key,id);

	Accelerators.Add(*entry);

        return NULL;
    }
}



bool cpcAccelXmlHandler::CanHandle(wxXmlNode *node)
{
    return (IsOfClass(node, wxT("cpcAccelTable")) ||
               IsOfClass(node, wxT("cpcAccelerator")));
}

cpcAccelArrayXmlHandler::cpcAccelArrayXmlHandler() : 
        wxXmlResourceHandler()
{
	Accelerators = new ArrayofAccelerators;
}


wxObject *cpcAccelArrayXmlHandler::DoCreateResource()
{
    int count;
    if (m_class == wxT("cpcAccelArray"))
    {
        CreateChildren(NULL, TRUE/*only this handler*/);

	count = Accelerators->GetCount();
	if(count == 0)
	{
		return NULL;
	}
	return (wxObject *) Accelerators;
    }

    else
    {
        
        int id = GetID();
        wxString modifier = GetText(wxT("modifier"));
	int intmod;
	if(modifier == "wxACCEL_ALT")
	{
		intmod = wxACCEL_ALT;
	}
	else if(modifier == "wxACCEL_SHIFT")
	{
		intmod = wxACCEL_SHIFT;
	}
	else if(modifier == "wxACCEL_CTRL")
	{
		intmod = wxACCEL_CTRL;
	}
	else
	{
		intmod = wxACCEL_NORMAL;
	}

        char key = GetText(wxT("key")).GetChar(0);

  	wxAcceleratorEntry *entry = new wxAcceleratorEntry(intmod,key,id);

	Accelerators->Add(*entry);

        return NULL;
    }
}



bool cpcAccelArrayXmlHandler::CanHandle(wxXmlNode *node)
{
    return (IsOfClass(node, wxT("cpcAccelArray")) ||
               IsOfClass(node, wxT("cpcAccelerator")));
}

