

// For compilers that support precompilation, includes "wx.h".
#include "wx/wx.h"

#include <locale.h>

#include "wx/dialog.h"
#include "wx/panel.h"
#include "wx/frame.h"
#include "wx/wfstream.h"
#include "wx/filesys.h"
#include "wx/filename.h"
#include "wx/log.h"
#include "wx/intl.h"
#include "wx/tokenzr.h"
#include "wx/fontenum.h"
#include "wx/module.h"
#include "wx/bitmap.h"
#include "wx/image.h"
#include "wx/fontmap.h"
#include "wx/artprov.h"
#include "wx/settings.h"
#include "wx/sizer.h"
#include "wx/print.h"

#if wxCHECK_VERSION(2,6,0)
#include "wx/xml/xml.h"
#else
#include "wx/xrc/xml.h"
#endif
#include "wx/xrc/xmlres.h"

#include "wx/mstream.h" 

#include "carapace.h"


#if 0
wxMenu *CarapaceXmlResource::LoadMenu(const wxString& name)
{
    return (wxMenu*)CreateResFromNode(FindResource(name, wxT("wxMenu")), NULL, NULL);
}




#if wxUSE_TOOLBAR
wxToolBar *CarapaceXmlResource::LoadToolBar(wxWindow *parent, const wxString& name)
{
    return (wxToolBar*)CreateResFromNode(FindResource(name, wxT("wxToolBar")), parent, NULL);
}
#endif



wxPanel *CarapaceXmlResource::LoadPanel(wxWindow *parent, const wxString& name)
{
    return (wxPanel*)CreateResFromNode(FindResource(name, wxT("wxPanel")), parent, NULL);
}

bool CarapaceXmlResource::LoadPanel(wxPanel *panel, wxWindow *parent, const wxString& name)
{
    return CreateResFromNode(FindResource(name, wxT("wxPanel")), parent, panel) != NULL;
}

wxBitmap CarapaceXmlResource::LoadBitmap(const wxString& name)
{
    wxBitmap *bmp = (wxBitmap*)CreateResFromNode(
                               FindResource(name, wxT("wxBitmap")), NULL, NULL);
    wxBitmap rt;

    if (bmp) { rt = *bmp; delete bmp; }
    return rt;
}

wxIcon CarapaceXmlResource::LoadIcon(const wxString& name)
{
    wxIcon *icon = (wxIcon*)CreateResFromNode(
                            FindResource(name, wxT("wxIcon")), NULL, NULL);
    wxIcon rt;

    if (icon) { rt = *icon; delete icon; }
    return rt;
}

#endif // 0


wxDialog *CarapaceXmlResource::LoadDialog(wxWindow *parent, const wxString& name)
{
    return (wxDialog*)CreateResFromNode(FindResource(name, wxT("wxDialog")), parent, NULL);
}

bool CarapaceXmlResource::LoadDialog(wxDialog *dlg, wxWindow *parent, const wxString& name)
{
    return CreateResFromNode(FindResource(name, wxT("wxDialog")), parent, dlg) != NULL;
}

wxFrame *CarapaceXmlResource::LoadFrame(wxWindow* parent, const wxString& name)
{
    return (wxFrame*)CreateResFromNode(FindResource(name, wxT("wxFrame")), parent, NULL);
}

bool CarapaceXmlResource::LoadFrame(wxFrame* frame, wxWindow *parent, const wxString& name)
{
    return CreateResFromNode(FindResource(name, wxT("wxFrame")), parent, frame) != NULL;
}

wxObject *CarapaceXmlResource::LoadObject(wxWindow *parent, const wxString& name, const wxString& classname)
{
    return CreateResFromNode(FindResource(name, classname), parent, NULL);
}

bool CarapaceXmlResource::LoadObject(wxObject *instance, wxWindow *parent, const wxString& name, const wxString& classname)
{
    return CreateResFromNode(FindResource(name, classname), parent, instance) != NULL;
}

wxMenuBar *CarapaceXmlResource::LoadMenuBar(wxWindow *parent, const wxString& name)
{
    return (wxMenuBar*)CreateResFromNode(FindResource(name, wxT("wxMenuBar")), parent, NULL);
}




wxXmlNode *CarapaceXmlResource::FindResource(const wxString& name,
                                       const wxString& classname,
                                       bool recursive)
{

	if ( stringXRDR.Doc == NULL || stringXRDR.Doc->GetRoot() == NULL )
	{
		return NULL;
	}

	wxXmlNode* found = DoFindResource(stringXRDR.Doc->GetRoot(),
		name, classname, recursive);
	if ( found )
	{
		return found;
	}

	wxLogError(_("XRC resource '%s' (class '%s') not found!"),
		name.c_str(), classname.c_str());
	return NULL;
}


IMPLEMENT_DYNAMIC_CLASS(CarapaceXmlResource, wxXmlResource)

CarapaceXmlResource::CarapaceXmlResource() : wxXmlResource()
{
}

CarapaceXmlResource::~CarapaceXmlResource()
{
    ClearHandlers();
}


bool CarapaceXmlResource::LoadFromString(const wxString& xmlstring)
{

	wxString encoding(wxT("UTF-8"));
	wxMemoryInputStream *stream = new wxMemoryInputStream(
		xmlstring.c_str(),xmlstring.Len());

	if (stream)
	{
		stringXRDR.Doc = new wxXmlDocument;
	}
	if (!stream || !stringXRDR.Doc->Load(*stream,encoding))
	{
		wxLogError(_("Cannot load resources from string."));
		wxDELETE(stringXRDR.Doc);
		return FALSE;
	}
	else if (stringXRDR.Doc->GetRoot()->GetName() != wxT("resource"))
	{
		wxLogError(_("Invalid XML resource '%s': doesn't have root node 'resource'."), 
			xmlstring.c_str());
		wxDELETE(stringXRDR.Doc);
		return FALSE;
	}

	return TRUE;
}




