//#define #define NDEBUG
#ifndef DATAOBJECT_H
#define DATAOBJECT_H


#include<vector>
#include<iostream>
#include<map> 
#include "wx/hashmap.h"
#include <cassert>

#include<string>

class Viewer;
class CarapaceNonModalDialog;

using namespace std;

enum Type {
	UNDEFT, 
	INTT,
	DOULBET,
	STRINGT,
	LOGICT,
	COLORT
};  //may need more type later


class Element
{
public:
	Type type;

	Element(int v);	
	Element(double v); 
	Element(wxString v);
	Element(bool v); 
	void GetValue(int& RtValue);	
	void GetValue(double& RtValue); 	
	void GetValue(wxString& RtValue); 	
	void GetValue(bool& RtValue); 

	void SetValue(int Value);	
	void SetValue(double Value); 	
	void SetValue(wxString Value); 	
	void SetValue(bool Value); 

private:
	void* val;
};


//template< class T >
class DataObject
{
	
	public:	
		DataObject();
		DataObject(wxString&, int row, int col, vector<Element>&, Type);
		DataObject(wxString&, int row, int col, vector<int>&, Type);
		DataObject(wxString&, int row, int col, vector<double>&, Type);
		DataObject(wxString&, int row, int col, vector<wxString>&, Type);
		DataObject(wxString&, int row, int col, vector<bool>&, Type);
		
		DataObject(wxString&, int row, int col, int *contvec, Type);
		DataObject(wxString&, int row, int col, double *contvec, Type);		
		DataObject(wxString&, int row, int col, char *contvec[], Type);		
		DataObject(wxString&, int row, int col, bool *contvec, Type);						
		
		~DataObject();
		
		Type type;
		wxString  label;
		int rows;
		int cols;
				
		vector<Element> content;

		bool persistent;
		Viewer *modifiedBy;
				
		bool insertRow(vector<int>&, int where);
		bool insertRow(vector<double>&, int where);		
		bool insertRow(vector<wxString>&, int where);
		bool insertRow(vector<bool>&, int where);		
		bool insertRow(int *newrowct, int where);	
		bool insertRow(double *newrowct, int where);	
		bool insertRow(char *newrowct[], int where);		
		bool insertRow(bool *newrowct, int where);						
		
		
		bool appendRow(vector<int>&);
		bool appendRow(vector<double>&);
		bool appendRow(vector<wxString>&);	
	 	bool appendRow(vector<bool>&);	
		bool appendRow(int *newrowct);	 	
		bool appendRow(double *newrowct);	 	 		
		bool appendRow(char *newrowct[]);	
		bool appendRow(bool *newrowct);	  		
		
		bool changeRow(vector<int>&, int where);
		bool changeRow(vector<double>&, int where);  	
		bool changeRow(vector<wxString>&, int where);
		bool changeRow(vector<bool>&, int where); 
		bool changeRow(int *newcont, int where);
		bool changeRow(double *newcont, int where);  	
		bool changeRow(char *newcont[], int where);
		bool changeRow(bool *newcont, int where); 		 			
		
		
		bool deleteRow(int where);
		
		bool insertCol(vector<int>&, int where);
		bool insertCol(vector<double>&, int where);		
		bool insertCol(vector<wxString>&, int where);
		bool insertCol(vector<bool>&, int where);
		bool insertCol(int *newcolct, int where);
		bool insertCol(double *newcolct, int where);		
		bool insertCol(char *newcolct[], int where);
		bool insertCol(bool *newcolct, int where);		
		
		bool appendCol(vector<int>&);
		bool appendCol(vector<double>&);		
		bool appendCol(vector<wxString>&);		
		bool appendCol(vector<bool>&);	
		bool appendCol(int *newcolct);
		bool appendCol(double *newcolct);		
		bool appendCol(char *newcolct[]);		
		bool appendCol(bool *newcolct);				
		
				
		bool changeCol(vector<int>&, int where);
		bool changeCol(vector<double>&, int where);
		bool changeCol(vector<wxString>&, int where);
		bool changeCol(vector<bool>&, int where);
		bool changeCol(int *newcolct, int where);
		bool changeCol(double *newcolct, int where);		
		bool changeCol(char *newcolct[], int where);
		bool changeCol(bool *newcolct, int where);				
		
		bool deleteCol(int where);
	 		 	
		bool setValue(Element&, int row, int col);
		bool setValue(int newvalue, int row, int col);	
		bool setValue(double newvalue, int row, int col);
		bool setValue(char *newvalue, int row, int col);
		bool setValue(bool newvalue, int row, int col);
		
		Element& operator[](int id) {
			return content[id];
		}
	 	
	 	
	 	//test
	 	void print();
	 	
	 	void printCont();
	
	private:
		void IntVtoEleV(vector<int>& vec, vector<Element>& RstVector);
		void DoubleVtoEleV(vector<double>& vec, vector<Element>& RstVector);
		void StringVtoEleV(vector<wxString>& vec, vector<Element>& RstVector);
		void BoolVtoEleV(vector<bool>& vec, vector<Element>& RstVector);
};



typedef map< wxString, DataObject > wxHash;

class DataHash : public wxHash
{
	
	public:
		DataHash();
		~DataHash();
		
		vector<Element>& getContent(wxString);
		int getRows(wxString);
		int getCols(wxString);
		Type getType(wxString);
		DataObject& getObject(wxString);
		void getVal(wxString,int);
};




//02/11/2005
//set up the association between dataobjects and viewers

	
	
typedef map<wxString, Viewer *> ViewerHash;

typedef map< wxString,vector<wxString> > Association;

class DynamicInteraction
{
	public:
		DynamicInteraction();
		DynamicInteraction(DataHash&, ViewerHash&);
		~DynamicInteraction(){};
		 

		DataHash  *data;
		ViewerHash *viewers;
		Association byData;
		Association byViewer;
			
		bool addDataObject(wxString&, DataObject& );
		bool addDataObject(wxString&, int row, int col, vector<int>&);
		bool addDataObject(wxString&, int row, int col, vector<double>&);		
		bool addDataObject(wxString&, int row, int col, vector<wxString>&);	
		bool addDataObject(wxString&, int row, int col, vector<bool>&);		
		bool addDataObject(wxString&, int row, int col, int *contvec);
		bool addDataObject(wxString&, int row, int col, double *contvec);
		bool addDataObject(wxString&, int row, int col, char *contvec[]);
		bool addDataObject(wxString&, int row, int col, bool *contvec);	
		bool addDataObject(wxString&, int row, int col, unsigned int *contvec);	

		
		bool deleteDataObject(wxString&);
		bool deleteUnreferencedDataObjects(void);
		bool MakeDataObjectPersistent(wxString&);
		
		DataObject& getDataObjectValue(wxString&);
		void getDataObjectContent(wxString&, int& row, int& col, vector<int>&);
		void getDataObjectContent(wxString&, int& row, int& col, vector<double>&);
		void getDataObjectContent(wxString&, int& row, int& col, vector<wxString>&);
		void getDataObjectContent(wxString&, int& row, int& col, vector<bool>&);						
		void getDataObjectContent(wxString&, int& row, int& col, int *contvec);
		void getDataObjectContent(wxString&, int& row, int& col, double *contvec);
		void getDataObjectContent(wxString&, int& row, int& col, char *contvec[]);	
		void getDataObjectContent(wxString&, int& row, int& col, bool *contvec);
		void getDataObjectContent(wxString&, int& row, int& col, unsigned int *contvec);				
		
		bool setDataObjectValue(DataObject&,wxString&);
		bool setDataObjectValue(vector<int>&, int row, int col,wxString&);
		bool setDataObjectValue(vector<double>&, int row, int col,wxString&);
		bool setDataObjectValue(vector<wxString>&, int row, int col,wxString&);
		bool setDataObjectValue(vector<bool>&, int row, int col,wxString&);										
		bool setDataObjectValue(int *contvec, int row, int col,wxString&);	
		bool setDataObjectValue(double *contvec, int row, int col,wxString&);	
		bool setDataObjectValue(char *contvec[], int row, int col,wxString&);	
		bool setDataObjectValue(bool *contvec, int row, int col,wxString&);		
		bool setDataObjectValue(unsigned int *contvec, int row, int col,wxString&);				
	
				
		bool addViewer(wxString&,Viewer*);
		bool addViewer(wxString&, vector<int>&, int row, int col);
		bool addViewer(wxString&, vector<double>&, int row, int col);
		bool addViewer(wxString&, vector<wxString>&, int row, int col);
		bool addViewer(wxString&, vector<bool>&, int row, int col);						
		bool addViewer(wxString&, int *ViewerState, int row, int col);
		bool addViewer(wxString&, double *ViewerState, int row, int col);
		bool addViewer(wxString&, char *ViewerState[], int row, int col);		
		bool addViewer(wxString&, bool *ViewerState, int row, int col);	
		bool addViewer(wxString&, unsigned int *ViewerState, int row, int col);				
		
		bool setViewerData(vector<wxString>&,wxString&);
		bool setViewerData(char *dataos[], int dataslength, wxString&);
		
		bool deleteViewer(wxString&);
		bool delistViewer(wxString&);
		
		DataObject& getViewerState(wxString&);
		void getViewerStateContent(wxString&, int&, int&, vector<int>&);
		void getViewerStateContent(wxString&, int&, int&, vector<double>&);
		void getViewerStateContent(wxString&, int&, int&, vector<wxString>&);
		void getViewerStateContent(wxString&, int&, int&, vector<bool>&);
		void getViewerStateContent(wxString&, int&, int&, int *ViewerState);	
		void getViewerStateContent(wxString&, int&, int&, double *ViewerState);		
		void getViewerStateContent(wxString&, int&, int&, char *ViewerState[]);	
		void getViewerStateContent(wxString&, int&, int&, bool *ViewerState);	
		void getViewerStateContent(wxString&, int&, int&, unsigned int *ViewerState);			
		
		bool setViewerState(wxString&,DataObject&);
		bool setViewerState(wxString&,vector<int>&, int row, int col);
		bool setViewerState(wxString&,vector<double>&, int row, int col);
		bool setViewerState(wxString&,vector<wxString>&, int row, int col);
		bool setViewerState(wxString&,vector<bool>&, int row, int col);
		bool setViewerState(wxString&,int *DOContent, int row, int col);		
		bool setViewerState(wxString&,double *DOContent, int row, int col);
		bool setViewerState(wxString&,char *DOContent[], int row, int col);	
		bool setViewerState(wxString&,bool *DOContent, int row, int col);		
		bool setViewerState(wxString&,unsigned int *DOContent, int row, int col);				
		
		void doRedraw(void);
		void redrawByNamedData(wxString);

		bool redraw(wxString&);
		bool updateByData();	
		
		//The following is the operations to single DataObject
		bool InsertDataObjectRow(wxString& ,vector<int>&, int where);		
		bool InsertDataObjectRow(wxString& ,vector<double>&, int where);	
		bool InsertDataObjectRow(wxString& ,vector<wxString>&, int where);	
		bool InsertDataObjectRow(wxString& ,vector<bool>&, int where);
		bool InsertDataObjectRow(wxString& ,int *newrowct, int where);		
		bool InsertDataObjectRow(wxString& ,double *newrowct, int where);	
		bool InsertDataObjectRow(wxString& ,char *newrowct[], int where);	
		bool InsertDataObjectRow(wxString& ,bool *newrowct, int where);	
		bool InsertDataObjectRow(wxString& ,unsigned int *newrowct, int where);	

		bool AppendDataObjectRow(wxString&, vector<int>&);
		bool AppendDataObjectRow(wxString&, vector<double>&);
		bool AppendDataObjectRow(wxString&, vector<wxString>&);	
	 	bool AppendDataObjectRow(wxString&, vector<bool>&);	
		bool AppendDataObjectRow(wxString&, int *newrowct);	 	
		bool AppendDataObjectRow(wxString&, double *newrowct);	 	 		
		bool AppendDataObjectRow(wxString&, char *newrowct[]);	
		bool AppendDataObjectRow(wxString&, bool *newrowct);
		bool AppendDataObjectRow(wxString&, unsigned int *newrowct);			  		

		bool ChangeDataObjectRow(wxString&, vector<int>&, int where);					
		bool ChangeDataObjectRow(wxString&, vector<double>&, int where);	
		bool ChangeDataObjectRow(wxString&, vector<wxString>&, int where);	
		bool ChangeDataObjectRow(wxString&, vector<bool>&, int where);	
		bool ChangeDataObjectRow(wxString&, int *newrowct, int where);					
		bool ChangeDataObjectRow(wxString&, double *newrowct, int where);	
		bool ChangeDataObjectRow(wxString&, char *newrowct[], int where);	
		bool ChangeDataObjectRow(wxString&, bool *newrowct, int where);		
		bool ChangeDataObjectRow(wxString&, unsigned int *newrowct, int where);				
		
		bool DeleteDataObjectRow(wxString&, int where);		
		
		bool InsertDataObjectCol(wxString&, vector<int>&, int where);		
		bool InsertDataObjectCol(wxString&, vector<double>&, int where);		
		bool InsertDataObjectCol(wxString&, vector<wxString>&, int where);		
		bool InsertDataObjectCol(wxString&, vector<bool>&, int where);	
		bool InsertDataObjectCol(wxString&, int *newcolct, int where);		
		bool InsertDataObjectCol(wxString&, double *newcolct, int where);		
		bool InsertDataObjectCol(wxString&, char *newcolct[], int where);		
		bool InsertDataObjectCol(wxString&, bool *newcolct, int where);	
		bool InsertDataObjectCol(wxString&, unsigned int *newcolct, int where);				

		bool AppendDataObjectCol(wxString&, vector<int>&);		
		bool AppendDataObjectCol(wxString&, vector<double>&);	
		bool AppendDataObjectCol(wxString&, vector<wxString>&);					
		bool AppendDataObjectCol(wxString&, vector<bool>&);	
		bool AppendDataObjectCol(wxString&, int *newcolct);		
		bool AppendDataObjectCol(wxString&, double *newcolct);	
		bool AppendDataObjectCol(wxString&, char *newcolct[]);					
		bool AppendDataObjectCol(wxString&, bool *newcolct);	
		bool AppendDataObjectCol(wxString&, unsigned int *newcolct);			
		
		bool ChangeDataObjectCol(wxString&, vector<int>&, int where);					
		bool ChangeDataObjectCol(wxString&, vector<double>&, int where);	
		bool ChangeDataObjectCol(wxString&, vector<wxString>&, int where);	
		bool ChangeDataObjectCol(wxString&, vector<bool>&, int where);	
		bool ChangeDataObjectCol(wxString&, int *newcolct, int where);					
		bool ChangeDataObjectCol(wxString&, double *newcolct, int where);	
		bool ChangeDataObjectCol(wxString&, char *newcolct[], int where);	
		bool ChangeDataObjectCol(wxString&, bool *newcolct, int where);	
		bool ChangeDataObjectCol(wxString&, unsigned int *newcolct, int where);			
		
		bool DeleteDataObjectCol(wxString&, int where);	

		bool SetDataObjectElement(wxString&, int newvalue, int row, int col);	
		bool SetDataObjectElement(wxString&, double newvalue, int row, int col);
		bool SetDataObjectElement(wxString&, char *newvalue, int row, int col);
		bool SetDataObjectElement(wxString&, bool newvalue, int row, int col);	
		bool SetDataObjectElement(wxString&, unsigned int newvalue, int row, int col);	
		
		bool getDataObjectProperty(wxString&, int& row, int& col, int& datatype);					
																								
};
		

#endif		
