/*
*(C)* The file is part of the source distribution of Carapace
*(C)* version 0.5 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* This file is distributed under the GNU General Public License
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

#include "cpcplatform.h"


// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/print.h"
#include "wx/printdlg.h"
#include "wx/generic/dcpsg.h"
#include "wx/clipbrd.h"


#include <math.h>

#include "carapace.h"

#undef UNDEFINED


#include "wx/image.h"
#include "wx/listctrl.h"
#include "wx/sizer.h"
#include "wx/log.h"


#include "wx/xrc/xmlres.h"
#include "wx/xrc/xh_all.h"



// CarapaceGraphFrame


IMPLEMENT_DYNAMIC_CLASS( CarapaceGraphFrame, wxFrame )

// event table for graph frames

BEGIN_EVENT_TABLE(CarapaceGraphFrame, wxFrame)
        EVT_MENU_RANGE(1,2000000000, CarapaceGraphFrame::ProcessUserCommand)
#if 0
        EVT_UPDATE_UI_RANGE(1,2000000000, CarapaceGraphFrame::UpdateUserCommand)
#endif
	EVT_CLOSE(CarapaceGraphFrame::OnClose)

END_EVENT_TABLE()



#define ID_QUIT 67

CarapaceGraphFrame::CarapaceGraphFrame()
       : wxFrame( (wxFrame *)NULL, -1, "sample", wxPoint(20,20), wxSize(200,200))
{
	wxString defres = CPC_STRING_DEFAULTGRAPHRESOURCE;
	wxString defname = CPC_STRING_DEFAULTGRAPHMENUNAME;

        //Resource = new wxXmlResource();
        Resource = new CarapaceXmlResource();
        if(Resource)
        {
                Resource->AddHandler(new wxMenuXmlHandler);
                Resource->AddHandler(new cpcMenuXmlHandler);
                Resource->AddHandler(new wxMenuBarXmlHandler);
        }

	Accelerators = new ArrayofAccelerators;

	DoMenuBar( defres, defname);

	CreateStatusBar(1);

	Graph = new CarapaceStaticGraph( this, -1, wxPoint(0,0), wxSize(100,100) );
	Graph->SetScrollbars( 10, 10, 50, 100 );
  
	Show(TRUE);
}

CarapaceGraphFrame::CarapaceGraphFrame(const wxString& title, const wxPoint& offset, 
		const wxSize& size, wxString& resource, wxString& menuname)
       : wxFrame( (wxFrame *)NULL, -1, title, offset, size)
{

        //Resource = new wxXmlResource();
        Resource = new CarapaceXmlResource();
        if(Resource)
        {
                Resource->AddHandler(new wxMenuXmlHandler);
                Resource->AddHandler(new cpcMenuXmlHandler);
                Resource->AddHandler(new wxMenuBarXmlHandler);
        }

	Accelerators = new ArrayofAccelerators;

	DoMenuBar(resource,menuname);

	CreateStatusBar(1);

	Graph = new CarapaceStaticGraph( this, -1, wxPoint(0,0), size );
	Graph->SetScrollbars( 10, 10, (long) (size.GetWidth()/10), (long) (size.GetHeight()/10) );
// we appear to need to set the client size a bit bigger, so that
// we cover any space needed for scroll bars
	SetClientSize(wxSize(size.GetWidth()+50,size.GetHeight()+50));

	Show(TRUE);
}

void CarapaceGraphFrame::OnClose(wxCloseEvent& event)
{
	thisApp->CutGraphFrame(GraphFrameIndex);
	Show(FALSE);
	wxYield();
	event.Skip();
}


CarapaceGraphFrame::~CarapaceGraphFrame()
{
}



void CarapaceGraphFrame::DoMenuBar(wxString& resource, wxString& menuname)
{

	wxString defres = CPC_STRING_DEFAULTGRAPHRESOURCE;
	wxString defname = CPC_STRING_DEFAULTGRAPHMENUNAME;

	if(resource != "" && menuname != "")
	{
		SetMenuBarFromString( resource, menuname);
	}
	else
	{
		SetMenuBarFromString( defres, defname);
	}
		
	UpdateMenuBar();
#if 0
	// here we update windows, graphs, and user menus

	wxMenuBar *thisMenuBar = GetMenuBar();

	FixCommandSubmenu(thisMenuBar);
	FixGraphSubmenu(thisMenuBar);
	//FixDialogSubmenu(thisMenuBar);
#endif

} /* CarapaceGraphFrame::DoMenuBar() */

void CarapaceGraphFrame::UpdateMenuBar(void)
{
	// here we update windows, graphs, and user menus

	wxMenuBar *thisMenuBar = GetMenuBar();

	FixCommandSubmenu(thisMenuBar);
	FixGraphSubmenu(thisMenuBar);
	FixDialogSubmenu(thisMenuBar);

} /* CarapaceGraphFrame::UpdateMenuBar() */


void CarapaceGraphFrame::SetMenuBarFromString(wxString resourcestr, wxString menuname)
{
        wxMenuBar *bar;
	//wxXmlResource *newresource ;
	CarapaceXmlResource *newresource ;

	//newresource = new wxXmlResource();
	newresource = new CarapaceXmlResource();
	if(newresource)
        {
		newresource->AddHandler(new wxMenuXmlHandler);
		newresource->AddHandler(new cpcMenuXmlHandler);
		newresource->AddHandler(new wxMenuBarXmlHandler);
		
		newresource->LoadFromString(resourcestr);
		bar = newresource->LoadMenuBar(menuname.c_str());
		if(bar)
		{
			SetMenuBar(bar);
			ResourceString = resourcestr;
			MenuResourceInUse = menuname;
			if(Resource)
			{
				delete Resource;
			}
			Resource = newresource;

			if(Accelerators)
			{
				delete(Accelerators);
			}
			Accelerators = new ArrayofAccelerators;
			if(Accelerators)
			{
				GetAccelArrayFromMenuBar(*Accelerators,bar);
			}
		}
	}
}



wxMenuBar * CarapaceGraphFrame::GetDefaultMenuBar(void)
{
	wxMenu *fileMenu = new wxMenu();
	wxMenu *windMenu = new wxMenu();
	wxMenu *w1 = new wxMenu();
	wxMenu *w2 = new wxMenu();
	wxMenu *w3 = new wxMenu();
	wxMenuBar *bar = new wxMenuBar();

	if(!fileMenu || !windMenu || !bar)
	{
		return(NULL);
	}

	fileMenu->Append(CPC_FILE_SAVEWINDOWAS, "Save graph as ...",
         "Save the contents of the graph window");
	fileMenu->AppendSeparator();
	fileMenu->Append(CPC_FILE_PAGESETUP, "Page setup", "Printing setup");
	fileMenu->Append(CPC_FILE_PRINTSELECTION, "&Print window", "Print this graph");
	fileMenu->AppendSeparator();
	fileMenu->Append(CPC_FILE_INTERRUPT, "&Interrupt\tCtrl+I", "Interrupt processing");
	fileMenu->AppendSeparator();
	fileMenu->Append(CPC_FILE_QUIT, "&Quit\tCtrl+Q", "Quit");

	w1->Append(CPC_WINDOWS_HIDE, "&Hide", "Hide window");
	w2->Append(CPC_WINDOWS_HIDE, "&Hide", "Hide window");
	w3->Append(CPC_WINDOWS_HIDE, "&Hide", "Hide window");
	windMenu->Append(CPC_WINDOWS_HIDE, "&Hide", "Hide window");
	windMenu->Append(CPC_WINDOWS_CLOSE, "&Close\tCtrl+W", "Close window");
	windMenu->AppendSeparator();
	windMenu->Append(CPC_WINDOWS_ZOOMIN, "Zoom in", "Enlarge graphic");
	windMenu->Append(CPC_WINDOWS_ZOOMOUT, "Zoom out", "Shrink graphic");
	windMenu->Append(CPC_WINDOWS_FIT, "Fit", "Fit graphic to frame");
	windMenu->Append(CPC_WINDOWS_FITPRESERVE, "Fit keep aspect", "Fit graphic to frame but retain original aspect ratio");
	windMenu->Append(CPC_WINDOWS_SETASPECT, "Set aspect", "Set the aspect ratio of the graphic");
	windMenu->AppendSeparator();
	windMenu->Append(CPC_WINDOWS_TEXTWINDOW, "Output windows", w1, 
	 "Select output window");
	windMenu->AppendSeparator();
	windMenu->Append(CPC_WINDOWS_GRAPH, "Graph windows", w2, "Select graph window");
	windMenu->AppendSeparator();
   
	bar->Append(fileMenu,"&File");
	bar->Append(windMenu,"&Windows");

	return(bar);
}

bool CarapaceGraphFrame::SaveWindowAs(void)
{
	// return TRUE for a successful save, FALSE otherwise
	// prompt for a file name and save to that file
	wxInitAllImageHandlers();
	wxSize size = Graph->GStructure->Size;
	int decision = wxYES;

	wxBitmap bitmap(size.GetWidth(),size.GetHeight(),-1);
	wxMemoryDC dc;
	dc.SelectObject(bitmap);
	wxPoint point = wxPoint();
	Graph->GStructure->Draw(dc,size,point,.1,.1);
	dc.SelectObject(wxNullBitmap);
	//wxImage image(bitmap);

	wxFileDialog dialog(NULL,CPC_STRING_SAVEGRAPH,"","",
#ifdef SAVEGRAPHTOPOSTSCRIPT
		"JPG|*.jpg|PNG|*.png|TIFF|*.tif|PostScript|*.ps",wxSAVE);
#else
		"JPG|*.jpg|PNG|*.png|TIFF|*.tif",wxSAVE);
#endif
	if(dialog.ShowModal() == wxID_OK)
	{
		if (wxFileExists(dialog.GetPath())){
			decision = wxMessageBox(CPC_STRING_FILEEXISTS,CPC_STRING_ALERT,wxYES_NO);
		}
		if (decision != wxYES)
		{
			return FALSE;
		}

		switch(dialog.GetFilterIndex())
		{
			case 0:
				bitmap.SaveFile(dialog.GetPath(),wxBITMAP_TYPE_JPEG);
				//image.SaveFile(dialog.GetPath(),wxBITMAP_TYPE_JPEG);
				break;
			case 1:
				bitmap.SaveFile(dialog.GetPath(),wxBITMAP_TYPE_PNG);
				//image.SaveFile(dialog.GetPath(),wxBITMAP_TYPE_PNG);
				break;
			case 2:
				bitmap.SaveFile(dialog.GetPath(),wxBITMAP_TYPE_TIF);
				//image.SaveFile(dialog.GetPath(),wxBITMAP_TYPE_TIF);
				break;
			case 3:
			{
				wxPrintData pd;
        			pd.SetPaperId(wxPAPER_LETTER);
				pd.SetFilename(dialog.GetPath());
				pd.SetPrintMode(wxPRINT_MODE_FILE);
				// 2.8 change pd.SetFontMetricPath(thisApp->AFMPath);
				//wxPostScriptDC psdc(pd);
				CarapacePostScriptDC psdc(pd);

				int ppih,offset;
				double scalefactor;

				ppih = psdc.GetResolution();
				scalefactor = ppih/720.;
				offset = (int) (200*scalefactor);  // 200 decipoints

				psdc.StartDoc("this graph");
				psdc.StartPage();
				psdc.SetDeviceOrigin(offset,offset);
				Graph->GStructure->Draw(psdc,size,point,scalefactor,scalefactor);
				psdc.EndPage();
				psdc.EndDoc();
				break;
			}
		}
		return TRUE;
	}
	return FALSE;

} 


void CarapaceGraphFrame::OnCopy(wxCommandEvent & WXUNUSED(event))
{
	// return TRUE for a successful save, FALSE otherwise
	// prompt for a file name and save to that file
	wxInitAllImageHandlers();
	wxSize size = Graph->GStructure->Size;

	wxBitmap bitmap(size.GetWidth(),size.GetHeight(),-1);
	wxMemoryDC dc;
	dc.SelectObject(bitmap);
	wxPoint point = wxPoint();
	Graph->GStructure->Draw(dc,size,point,.1,.1);
	dc.SelectObject(wxNullBitmap);

	if(wxTheClipboard->Open())
	{
		wxTheClipboard->SetData(new wxBitmapDataObject(bitmap));
		wxTheClipboard->Close();
	}

} 

void CarapaceGraphFrame::OnQuit(wxCommandEvent & WXUNUSED(event))
{
	thisApp->DoQuit(CPC_EXIT_SAVECANCANCEL);
}

void CarapaceGraphFrame::FastQuit(wxCommandEvent & WXUNUSED(event))
{
	thisApp->DoQuit(CPC_EXIT_FASTFORCEQUIT);
}

void CarapaceGraphFrame::OnPageSetup(wxCommandEvent& WXUNUSED(event))
{
    (*thisApp->PageSetupData) = * thisApp->PrintData;

    wxPageSetupDialog pageSetupDialog(this, thisApp->PageSetupData);
    pageSetupDialog.ShowModal();

    (*thisApp->PrintData) = pageSetupDialog.GetPageSetupData().GetPrintData();
    (*thisApp->PageSetupData) = pageSetupDialog.GetPageSetupData();
}


void CarapaceGraphFrame::OnPrint(wxCommandEvent & WXUNUSED(event))
{
	wxPrintDialogData printDialogData(*(thisApp->PrintData));

	wxPrinter printer(& printDialogData);
	CarapaceGPrintout printout("My printout",GraphFrameIndex);
	if (!printer.Print(this,&printout, TRUE))
	{
		if (wxPrinter::GetLastError() == wxPRINTER_ERROR)
		{
			CarapaceAlert(CPC_STRING_PRINTINGERROR /*"Printing error"*/);
		}
	}
	else
	{
		*(thisApp->PrintData) = printer.GetPrintDialogData().GetPrintData();
	}
}

void CarapaceGraphFrame::MenuInterrupt(wxCommandEvent & WXUNUSED(event))
{
	thisApp->Interrupt();
}

void CarapaceGraphFrame::Hide(void)
{
	Show(FALSE);
}

void CarapaceGraphFrame::DoClose(void)
{
	wxCloseEvent ce = wxCloseEvent();
	ce.SetCanVeto(TRUE);
	OnClose(ce);
}


void CarapaceGraphFrame::ShowTextFrame(wxCommandEvent& event)
{
	int index = event.GetId() - CPC_TEXT_WINDOWS_BASE;
	thisApp->TextFrameFront(thisApp->STFrameList.Item(index)->GetData()->TextFrameIndex);
}

void CarapaceGraphFrame::ShowGraphFrame(wxCommandEvent& event)
{
	int index = event.GetId() - CPC_GRAPH_WINDOWS_BASE;
	if(index <= (int) thisApp->GFrameList.GetCount() - 1)
	{
		thisApp->GraphFrameFront(thisApp->GFrameList.Item(index)->GetData()->GraphFrameIndex);
	}
	else
	{
		index -= thisApp->GFrameList.GetCount();
		thisApp->PanelFrameFront(thisApp->PGFrameList.Item(index)->GetData()->PanelFrameIndex);
	}
}


void CarapaceGraphFrame::ZoomIn(wxCommandEvent& event)
{
	Graph->Size.SetWidth((int) (Graph->Size.GetWidth()*1.414));
	Graph->Size.SetHeight((int) (Graph->Size.GetHeight()*1.414));
	Graph->SetScrollbars( 10, 10, (long) (Graph->Size.GetWidth()/10), (long) (Graph->Size.GetHeight()/10) );
	Graph->Refresh();
}

void CarapaceGraphFrame::ZoomOut(wxCommandEvent& event)
{
	Graph->Size.SetWidth((int) (Graph->Size.GetWidth()/1.414));
	Graph->Size.SetHeight((int) (Graph->Size.GetHeight()/1.414));
	Graph->SetScrollbars( 10, 10, (long) (Graph->Size.GetWidth()/10), (long) (Graph->Size.GetHeight()/10) );
	Graph->Refresh();
}

void CarapaceGraphFrame::Fit(wxCommandEvent& event)
{
	int w,h;
	GetClientSize(&w,&h);
	Graph->Size.SetWidth(w);
	Graph->Size.SetHeight(h);
	Graph->SetScrollbars( 10, 10, (long) (Graph->Size.GetWidth()/10), (long) (Graph->Size.GetHeight()/10) );
	Graph->Refresh();
}

void CarapaceGraphFrame::FitPreserve(wxCommandEvent& event)
{
	int w,h;
	double r1, r2;
	GetClientSize(&w,&h);
	r1 = ((double) w)/Graph->Size.GetWidth();
	r2 = ((double) h)/Graph->Size.GetHeight();
	if(r2 < r1)
	{
		r1 = r2;
	}
	Graph->Size.SetWidth((int) (Graph->Size.GetWidth()*r1));
	Graph->Size.SetHeight((int) (Graph->Size.GetHeight()*r1));
	Graph->Refresh();
	Graph->SetScrollbars( 10, 10, (long) (Graph->Size.GetWidth()/10), (long) (Graph->Size.GetHeight()/10) );
}


void CarapaceGraphFrame::SetAspect(wxCommandEvent& event)
{

}

void CarapaceGraphFrame::Revert(wxCommandEvent& event)
{
	Graph->Size = Graph->OriginalSize;
	Graph->Refresh();
}

void CarapaceGraphFrame::DoNothing(void)
{
}


#define DO_EVT_MENU(id,callback,ev)   \
        if(which == wxXmlResource::GetXRCID(#id))  \
        { \
                callback(ev); \
                return; \
        }



void CarapaceGraphFrame::ProcessUserCommand(wxCommandEvent& event)
{
        CarapaceCommand *cptr;
        cpcMenuItem *menuItem;
        wxString base,full;
        int i;

        int which = event.GetId();

	DO_EVT_MENU(CPC_FILE_SAVEWINDOWAS, CarapaceGraphFrame::SaveWindowAs,)
	DO_EVT_MENU(CPC_EDIT_COPY, CarapaceGraphFrame::OnCopy,event)
	DO_EVT_MENU(CPC_FILE_PAGESETUP, CarapaceGraphFrame::OnPageSetup,event)
	DO_EVT_MENU(CPC_FILE_PRINTSELECTION, CarapaceGraphFrame::OnPrint,event)
	DO_EVT_MENU(CPC_FILE_INTERRUPT, CarapaceGraphFrame::MenuInterrupt,event)
	DO_EVT_MENU(CPC_FILE_QUIT, CarapaceGraphFrame::OnQuit,event)
	DO_EVT_MENU(CPC_FILE_FASTQUIT, CarapaceGraphFrame::FastQuit,event)


	DO_EVT_MENU(CPC_WINDOWS_HIDE, CarapaceGraphFrame::Hide,)
	DO_EVT_MENU(CPC_WINDOWS_CLOSE, CarapaceGraphFrame::DoClose,)
	DO_EVT_MENU(CPC_WINDOWS_ZOOMIN, CarapaceGraphFrame::ZoomIn,event)
	DO_EVT_MENU(CPC_WINDOWS_ZOOMOUT, CarapaceGraphFrame::ZoomOut,event)
	DO_EVT_MENU(CPC_WINDOWS_FIT, CarapaceGraphFrame::Fit,event)
	DO_EVT_MENU(CPC_WINDOWS_FITPRESERVE, CarapaceGraphFrame::FitPreserve,event)
	DO_EVT_MENU(CPC_WINDOWS_SETASPECT, CarapaceGraphFrame::SetAspect,event)
	DO_EVT_MENU(CPC_WINDOWS_REVERT, CarapaceGraphFrame::Revert,event)

        base = "CPC_TEXT_WINDOWS_BASE";
        for(i=0;i<thisApp->numTextFrames;i++)
        {
                full = base;
                full << i;
                if(which == wxXmlResource::GetXRCID(full.GetData()) )
                {
                        thisApp->TextFrameFront(thisApp->STFrameList.Item(i)->GetData()->TextFrameIndex);
                        return;
                }
        }


        base = "CPC_GRAPH_WINDOWS_BASE";
        for(i=0;i<thisApp->numGraphFrames;i++)
        {
                full = base;
                full << i;
                if(which == wxXmlResource::GetXRCID(full.GetData()) )
                {
                        thisApp->GraphFrameFront(thisApp->GFrameList.Item(i)->GetData()->GraphFrameIndex);
                        return;
                }
        }
        for(i=0;i<thisApp->numGraphFrames;i++)
        {
                full = base;
                full << (i+thisApp->numGraphFrames);
                if(which == wxXmlResource::GetXRCID(full.GetData()) )
                {
                        thisApp->PanelFrameFront(thisApp->PGFrameList.Item(i)->GetData()->PanelFrameIndex);
                        return;
                }
        }


	
	base = "CPC_DIALOG_WINDOWS_BASE";
	for(i=0;i<thisApp->DFrameList.GetCount();i++)
	{
       		full = base;
       		full << i;
		if(which == wxXmlResource::GetXRCID(full.GetData()) )
		{
			//thisApp->TextFrameFront(thisApp->STFrameList.Item(i)->GetData()->TextFrameIndex);
			thisApp->DFrameList.Item(i)->GetData()->Raise();
			return;
		}
	}




        menuItem = (cpcMenuItem *) GetMenuBar()->FindItem(which);
        if(!menuItem || menuItem->action == "")
        {
                return;
        }

        wxCommandEvent procevent(wxEVT_COMMAND_MENU_SELECTED,CPC_GUI_PROCESSCOMMAND);
        cptr = new CarapaceCommand;
        cptr->command = menuItem->action;
        cptr->command << "\n";

// commands should come from some command frame, so in a graphics frame,
// we use the first command frame.
	wxCarapaceSplitTextFrameListNode *stnode = thisApp->STFrameList.GetFirst();
	CarapaceSplitTextFrame *stfp = stnode->GetData();

        cptr->frame = stfp->TextFrameIndex;  
        cptr->hidecommand = menuItem->hidecommand;
        cptr->hideoutput = menuItem->hideoutput;
        procevent.SetClientData((void *) cptr);
        wxPostEvent(thisApp,procevent);
}


#define DO_UPDATE_UI(id,callback,ev)   \
        if(which == wxXmlResource::GetXRCID(#id))  \
        { \
                callback(ev); \
                return; \
        }

void CarapaceGraphFrame::UpdateUserCommand(wxUpdateUIEvent& event)
{
        int which = event.GetId();
        wxMenuBar *bar;
        cpcMenuItem *item;
        wxString newlabel;

        bar = GetMenuBar();
        item = (cpcMenuItem *) bar->FindItem(which);
        if(!item || item->GetKind() != wxITEM_NORMAL || item->GetSubMenu())
        {
                return;
        }
#if 0
        if(item->selectionlabel !="")
        {
                event.SetText(item->selectionlabel);
        }
        else
#endif /* 0 */
        {
                event.SetText(item->label);
        }
}

