/*
*(C)* The file is part of the source distribution of Carapace
*(C)* version 0.5 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* This file is distributed under the GNU General Public License
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/


#include "cpcplatform.h"



// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/fontdlg.h"
#include "wx/colour.h"
#include "wx/caret.h"
#include "wx/print.h"
#include "wx/printdlg.h"
#include "wx/cmndata.h"


#include <math.h>

#include "carapace.h"

#undef UNDEFINED

CarapaceGPrintout::CarapaceGPrintout(char *title, long windno) : 
	wxPrintout(title)
{
	Windno = windno;
	GraphFramePtr = NULL;
	PanelFramePtr = NULL;

} /*CarapaceGPrintout::CarapaceGPrintout()*/

CarapaceGPrintout::CarapaceGPrintout(char *title, CarapaceGraphFrame *gfp) : 
	wxPrintout(title)
{
	Windno = 0;
	GraphFramePtr = gfp;
	PanelFramePtr = NULL;

} /*CarapaceGPrintout::CarapaceGPrintout()*/

CarapaceGPrintout::CarapaceGPrintout(char *title, CarapacePanelGraphFrame *pgfp) : 
	wxPrintout(title)
{
	Windno = 0;
	GraphFramePtr = NULL;
	PanelFramePtr = pgfp;

} /*CarapaceGPrintout::CarapaceGPrintout()*/


bool CarapaceGPrintout::OnBeginDocument(int startPage, int endPage) 
{

	if (!wxPrintout::OnBeginDocument(startPage,endPage))
	{
		return(FALSE);
	}

	return(TRUE);
} /*CarapaceGPrintout::OnBeginDocument()*/


bool CarapaceGPrintout::OnPrintPage(int page)
{
	
	wxCarapaceGraphFrameListNode *gnode = NULL;
	CarapaceGraphFrame *gfp;
	wxCarapacePanelGraphFrameListNode *pgnode = NULL;
	CarapacePanelGraphFrame *pgfp;
	wxPoint point = wxPoint(00,00);
	wxSize size;
	//thisApp->PrintData->SetFontMetricPath("/home/gary/wxGTK-2.2.7/misc/gs_afm");

  	wxDC *dc = GetDC();
  	if (dc)
  	{
		dc->SetDeviceOrigin(200,200);
 		if(Windno > 0)
		{
			gnode = thisApp->GFrameList.Find(Windno);
		}
		else if(GraphFramePtr != NULL)
		{
			int i = thisApp->GFrameList.IndexOf(GraphFramePtr);
			if(i != wxNOT_FOUND)
			{
				gnode = thisApp->GFrameList.Item(i);
			}
		}
		else if(PanelFramePtr != NULL)
		{
			int i = thisApp->PGFrameList.IndexOf(PanelFramePtr);
			if(i != wxNOT_FOUND)
			{
				pgnode = thisApp->PGFrameList.Item(i);
			}
		}
		if(gnode != NULL)
		{
			gfp = gnode->GetData();
			size.SetWidth(10*gfp->Graph->OriginalSize.GetWidth());
			size.SetHeight(10*gfp->Graph->OriginalSize.GetHeight());
			gfp->Graph->Draw(*dc, size, point,.1,.1);
		}
		if(pgnode != NULL)
		{
			pgfp = pgnode->GetData();
			size.SetWidth(10*pgfp->Graph->OriginalSize.GetWidth());
			size.SetHeight(10*pgfp->Graph->OriginalSize.GetHeight());
			pgfp->Graph->Draw(*dc, size, point,.1,.1);
		}
		return(TRUE);
	} /*if (dc)*/
  	else
	{
		return FALSE;
	}
	return(TRUE);
} /*CarapaceGPrintout::OnPrintPage()*/


bool CarapaceGPrintout::HasPage(int pageNum)
{
	return (pageNum == 1);
} /*CarapaceGPrintout::HasPage()*/

void CarapaceGPrintout::GetPageInfo(int *minPage, int *maxPage,
		int *pageFrom, int *pageTo)
{
	*minPage = 1;
	*maxPage = 1;
	*pageFrom = 1;
	*pageTo = 1;
} /*CarapaceGPrintout::GetPageInfo()*/
