/*
*(C)* The file is part of the source distribution of Carapace
*(C)* version 0.5 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* This file is distributed under the GNU General Public License
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

#include "cpcplatform.h"



// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/print.h"


#include <math.h>

#include "carapace.h"

#undef UNDEFINED


#include "wx/image.h"
#include "wx/listctrl.h"
#include "wx/sizer.h"
#include "wx/log.h"


// CarapaceStaticGraph

IMPLEMENT_DYNAMIC_CLASS(CarapaceStaticGraph, wxScrolledWindow)

BEGIN_EVENT_TABLE(CarapaceStaticGraph, wxScrolledWindow)
  EVT_PAINT(                  CarapaceStaticGraph::OnPaint)
  EVT_MOUSE_EVENTS(                  CarapaceStaticGraph::OnMouse)
  EVT_SCROLLWIN(              CarapaceStaticGraph::OnScroll)
  EVT_CHAR(CarapaceStaticGraph::OnChar)
END_EVENT_TABLE()

CarapaceStaticGraph::CarapaceStaticGraph( void )
        : wxScrolledWindow( NULL, -1)
{

}


CarapaceStaticGraph::CarapaceStaticGraph( CarapaceGraphFrame *parent, wxWindowID id,
                    const wxPoint &pos, const wxSize &size )
        : wxScrolledWindow( parent, id, pos, size, wxSUNKEN_BORDER, "canvas" )
{
	Parent = parent;
	Size = size;
	OriginalSize = size;
	CrossHairMode = TRUE;
	MouseRequestState = CPC_SG_NOMOUSEREQUEST;
	MouseHelpString = "";

	GStructure = new CarapaceGraphStructure(size);

	//SetCursor(*wxCROSS_CURSOR);
}

CarapaceStaticGraph::~CarapaceStaticGraph()
{
	delete GStructure;
}

void CarapaceStaticGraph::OnMouse( wxMouseEvent &event )
{
	if(!CrossHairMode)
	{
		return;
	}
	int xd,yd,xl,yl;
	double x,y;
	event.GetPosition(&xd,&yd);
	CalcUnscrolledPosition(xd,yd,&xl,&yl);
	x = ((double) xl)*OriginalSize.GetWidth()/Size.GetWidth();
	x = XAxisParms[0] + XAxisParms[1]*x*10;
	if(XAxisType == CPC_SG_AXISLOG10)
	{
		x = pow(10.,x);
	}
	else if(XAxisType == CPC_SG_AXISLOGE)
	{
		x = exp(x);
	}
	else if(XAxisType == CPC_SG_AXISLOG2)
	{
		x = pow(2.,x);
	}
	else if(XAxisType == CPC_SG_AXISRECIPROCAL)
	{
		x = 1./x;
	}
	y = ((double) yl)*OriginalSize.GetHeight()/Size.GetHeight();
	y = OriginalSize.GetHeight() - y;
	y = YAxisParms[0] + YAxisParms[1]*y*10;
	if(YAxisType == CPC_SG_AXISLOG10)
	{
		y = pow(10.,y);
	}
	else if(YAxisType == CPC_SG_AXISLOGE)
	{
		y = exp(y);
	}
	else if(YAxisType == CPC_SG_AXISLOG2)
	{
		y = pow(2.,y);
	}
	else if(YAxisType == CPC_SG_AXISRECIPROCAL)
	{
		y = 1./y;
	}
	wxString text;
	text.Alloc(100);
	if(event.LeftDown())
	{
		if(thisApp->MouseRequestState == CPC_SG_CHOOSEFRAME)
		{
			thisApp->MouseSelectedFrame = Parent->GraphFrameIndex;
			thisApp->MouseRequestState = CPC_SG_MOUSEDONE;
			thisApp->MouseHelpString = "";
		}
		if(MouseRequestState == CPC_SG_MOUSEPOINT)
		{
			wxClientDC dc(this);
			MouseRequestState = CPC_SG_MOUSEDONE;
			MouseHelpString = "";
			dc.SetLogicalFunction(wxINVERT);
			if(MouseCoords[0] != MouseCoords[2] ||
				MouseCoords[1] != MouseCoords[3])
			{
				// erase old line
				dc.DrawLine(MouseCoords[2],0,MouseCoords[2],Size.GetHeight());
				dc.DrawLine(0,MouseCoords[3],Size.GetWidth(),MouseCoords[3]);
			}
			dc.SetLogicalFunction(wxCOPY);
			MouseDParms[0] = x;
			MouseDParms[1] = y;
			MouseCoords[0] = xd;
			MouseCoords[1] = yd;
			dc.DrawLine(xd-10,yd,xd+10,yd);
			dc.DrawLine(xd,yd-10,xd,yd+10);
		}
		if(MouseRequestState == CPC_SG_MOUSESEGMENT ||
			MouseRequestState == CPC_SG_MOUSEBOX)
		{
			MouseRequestState = CPC_SG_MOUSEDONE;
			MouseHelpString = "";
		}
		//printf("button down, getpos xy %d %d, clcunscr xy %d %d xx,yy %g %g\n",xd,yd,xl,yl,x,y);
	}
	if(event.Moving())
	{
		if(thisApp->MouseRequestState == CPC_SG_CHOOSEFRAME)
		{
			Parent->GetStatusBar()->SetStatusText(thisApp->MouseHelpString);
			return;
		}

		text.Printf("  %12.3g, %12.3g", x, y);
		text.Prepend(MouseHelpString);
		Parent->GetStatusBar()->SetStatusText(text);

		if(MouseRequestState == CPC_SG_MOUSEPOINT)
		{
			//wxPaintDC dc(this);
			wxClientDC dc(this);
			dc.SetLogicalFunction(wxINVERT);
			if(MouseCoords[0] != MouseCoords[2] ||
				MouseCoords[1] != MouseCoords[3])
			{
				// erase old line
				dc.DrawLine(MouseCoords[2],0,MouseCoords[2],Size.GetHeight());
				dc.DrawLine(0,MouseCoords[3],Size.GetWidth(),MouseCoords[3]);
				MouseCoords[2] = MouseCoords[0];
				MouseCoords[3] = MouseCoords[1];
				MouseDParms[2] = MouseDParms[0];
				MouseDParms[3] = MouseDParms[1];
			}
			if(MouseCoords[0] == MouseCoords[2] &&
				MouseCoords[1] == MouseCoords[3])
			{
				// draw cross hairs at new location
				MouseCoords[0] = -1; // make sure unequal to [2]
				MouseCoords[2] = xd;
				MouseCoords[3] = yd;
				MouseDParms[2] = x;
				MouseDParms[3] = y;
				dc.DrawLine(MouseCoords[2],0,MouseCoords[2],Size.GetHeight());
				dc.DrawLine(0,MouseCoords[3],Size.GetWidth(),MouseCoords[3]);
			}
			dc.SetLogicalFunction(wxCOPY);
		}

		if(MouseRequestState == CPC_SG_MOUSESEGMENT)
		{
			//wxPaintDC dc(this);
			wxClientDC dc(this);
			dc.SetLogicalFunction(wxINVERT);
			if(MouseCoords[0] != MouseCoords[2] ||
				MouseCoords[1] != MouseCoords[3])
			{
				// erase old line
				dc.DrawLine(MouseCoords[0],MouseCoords[1],
					MouseCoords[2],MouseCoords[3]);
				MouseCoords[2] = MouseCoords[0];
				MouseCoords[3] = MouseCoords[1];
				MouseDParms[2] = MouseDParms[0];
				MouseDParms[3] = MouseDParms[1];
			}
			if(MouseCoords[0] == MouseCoords[2] &&
				MouseCoords[1] == MouseCoords[3])
			{
				// add new line
				dc.DrawLine(MouseCoords[0],MouseCoords[1],
					xd,yd);
				MouseCoords[2] = xd;
				MouseCoords[3] = yd;
				MouseDParms[2] = x;
				MouseDParms[3] = y;
			}
			dc.SetLogicalFunction(wxCOPY);
		}

		if(MouseRequestState == CPC_SG_MOUSEBOX)
		{
			//wxPaintDC dc(this);
			wxClientDC dc(this);
			dc.SetLogicalFunction(wxINVERT);
			if(MouseCoords[0] != MouseCoords[2] || MouseCoords[1] != MouseCoords[3])
			{
				// erase old box
				dc.DrawLine(MouseCoords[0],MouseCoords[1],MouseCoords[0],MouseCoords[3]);
				dc.DrawLine(MouseCoords[0],MouseCoords[3],MouseCoords[2],MouseCoords[3]);
				dc.DrawLine(MouseCoords[2],MouseCoords[3],MouseCoords[2],MouseCoords[1]);
				dc.DrawLine(MouseCoords[2],MouseCoords[1],MouseCoords[0],MouseCoords[1]);
				MouseCoords[2] = MouseCoords[0];
				MouseCoords[3] = MouseCoords[1];
				MouseDParms[2] = MouseDParms[0];
				MouseDParms[3] = MouseDParms[1];
			}
			if(MouseCoords[0] == MouseCoords[2] && MouseCoords[1] == MouseCoords[3])
			{
				// add new box
				MouseCoords[2] = xd;
				MouseCoords[3] = yd;
				MouseDParms[2] = x;
				MouseDParms[3] = y;
				dc.DrawLine(MouseCoords[0],MouseCoords[1],MouseCoords[0],MouseCoords[3]);
				dc.DrawLine(MouseCoords[0],MouseCoords[3],MouseCoords[2],MouseCoords[3]);
				dc.DrawLine(MouseCoords[2],MouseCoords[3],MouseCoords[2],MouseCoords[1]);
				dc.DrawLine(MouseCoords[2],MouseCoords[1],MouseCoords[0],MouseCoords[1]);
			}
			dc.SetLogicalFunction(wxCOPY);
		}
		
	}
	if(event.Leaving())
	{
		if(MouseRequestState == CPC_SG_MOUSEPOINT 
			|| MouseRequestState == CPC_SG_MOUSESEGMENT)
		{
			MouseRequestState = CPC_SG_MOUSELEAVE;
			MouseHelpString = "";
		}
		Parent->GetStatusBar()->SetStatusText("");
	}
}

void CarapaceStaticGraph::Draw(wxDC &dc, wxSize& size, wxPoint& pos, double xscale, double yscale)
{
	GStructure->Draw(dc,size,pos,xscale,yscale);
}

void CarapaceStaticGraph::OnPaint( wxPaintEvent &WXUNUSED(event) )
{

	wxPaintDC dc( this );
	PrepareDC( dc );
	wxPoint point = wxPoint();

	thisApp->RefreshPanel( (Parent->GraphFrameIndex-1)/CPC_GRAPHSPERPANEL + 1);
	wxSize size = wxSize(Size.GetWidth(),Size.GetHeight());
	Draw( dc, size, point, .1, .1);

}


bool CarapaceStaticGraph::DoGraphRequest(cpc_sg_ptr sgp)
{
	//wxPaintDC dc( (wxWindow*) Parent );
	//((wxScrolledWindow *) Parent)->PrepareDC( dc );
	wxClientDC dc( this );
	switch(sgp->type)
	{
		case CPC_SG_GETCHARWIDTH:
			sgp->size = dc.GetCharWidth()*10;
			//sgp->size = 6;
			break;

		case CPC_SG_GETCHARHEIGHT:
			sgp->size = dc.GetCharHeight()*10;
			//sgp->size = 14;
			break;

		default:
			return(FALSE);
	}
	return(TRUE);
}


void CarapaceStaticGraph::OnScroll( wxScrollWinEvent &event )
{
    event.Skip();
}

CarapaceGraphElement::~CarapaceGraphElement(void)
{
	if(Type == CPC_SG_STRINGTYPE)
	{
		delete Data.cgs;
	}
	else
	{
		delete Data.cgstructure;
	}
}

CarapaceGraphStructure::CarapaceGraphStructure(void)
{
}

CarapaceGraphStructure::CarapaceGraphStructure(const wxSize & size, 
	CarapaceGraphStructure *parent)
{
	Parent = parent;

	Size = size;

	wxString *string = new wxString;
	CarapaceGraphElement *ge = new CarapaceGraphElement;
	ge->Type = CPC_SG_STRINGTYPE;
	ge->Parent = this;
	ge->Data.cgs = string;

	CGEList = CarapaceGraphElementList(wxKEY_INTEGER);
	CGEList.Append(-1,ge);
	Working = ge;
}

void CarapaceGraphStructure::Draw(wxDC &dc, wxSize& size, wxPoint& pos, double xscale, double yscale)
{
	GetOldState(dc);

	Scales.x = ((double) size.GetWidth())/Size.GetWidth();
	Scales.y = ((double) size.GetHeight())/Size.GetHeight();

	double newxscale = xscale*Scales.x;
	double newyscale = yscale*Scales.y;
	Origin = pos;

	dc.DestroyClippingRegion();
	dc.SetClippingRegion(Origin.x,Origin.y,size.x,size.y);

	wxCarapaceGraphElementListNode *cgeln = CGEList.GetFirst();
	CarapaceGraphElement *ge;
	while(cgeln)
	{
		ge = cgeln->GetData();
		if(ge->Type == CPC_SG_STRINGTYPE)
		{
			DrawFromString(dc,ge->Data.cgs,newxscale,newyscale);
		}
		cgeln = cgeln->GetNext();
	}

	RestoreOldState(dc);
}

CarapaceGraphStructure::~CarapaceGraphStructure(void)
{
	CGEList.DeleteContents(TRUE);
	CGEList.Clear();
}

void CarapaceGraphStructure::DrawFromString(wxDC &dc, wxString *sp, double xscale, double yscale)
{
	long pos = 0;
	long cmd,x1,x2,y1,y2,style,size,name,r,g,b,family,weight;
	long tw,th,td;
	wxString *strp;
	wxPen *pen;
	wxBrush *brush;
	wxFont *font;
	float minscale;
	double rx1,ry1,rsize;

	minscale = (xscale < yscale) ? xscale : yscale;

	while((unsigned) pos+1 <  sp->Length())
	{
		cmd = CarapaceGetShort(sp,&pos);
		switch(cmd)
		{
			case CPC_SG_DRAWLINE:
				x1 = (long) (Origin.x+CarapaceGetShort(sp,&pos)*xscale +.5);
				y1 = (long) (Origin.y+CarapaceGetShort(sp,&pos)*yscale +.5);
				x2 = (long) (Origin.x+CarapaceGetShort(sp,&pos)*xscale +.5);
				y2 = (long) (Origin.y+CarapaceGetShort(sp,&pos)*yscale +.5);
				dc.DrawLine(x1,y1,x2,y2);
				break;
	
			case CPC_SG_DRAWTEXT:
				dc.SetBackgroundMode(wxTRANSPARENT);
				x1 = (long) (Origin.x+CarapaceGetShort(sp,&pos)*xscale +.5);
				y1 = (long) (Origin.y+CarapaceGetShort(sp,&pos)*yscale +.5);
				style = Carapace2WxStyle(CarapaceGetShort(sp,&pos));
				strp = CarapaceGetString(sp,&pos);
				dc.GetTextExtent(*strp,&tw,&th,&td);
				if(style == CPC_SG_LEFTTOP)
				{
					dc.DrawText(*strp,x1,y1);
				}
				else if(style == CPC_SG_CENTERTOP)
				{
					dc.DrawText(*strp,x1-tw/2,y1);
				}
				else if(style == CPC_SG_RIGHTTOP)
				{
					dc.DrawText(*strp,x1-tw,y1);
				}
				else if(style == CPC_SG_LEFTBASELINE)
				{
					dc.DrawText(*strp,x1,y1-th+td);
				}
				else if(style == CPC_SG_CENTERBASELINE)
				{
					dc.DrawText(*strp,x1-tw/2,y1-th+td);
				}
				else if(style == CPC_SG_RIGHTBASELINE)
				{
					dc.DrawText(*strp,x1-tw,y1-th+td);
				}
				else if(style == CPC_SG_LEFTBOTTOM)
				{
					dc.DrawText(*strp,x1,y1-th);
				}
				else if(style == CPC_SG_CENTERBOTTOM)
				{
					dc.DrawText(*strp,x1-tw/2,y1-th);
				}
				else if(style == CPC_SG_RIGHTBOTTOM)
				{
					dc.DrawText(*strp,x1-tw,y1-th);
				}

				delete strp;
				break;
	
			case CPC_SG_DRAWTEXTUP:
				dc.SetBackgroundMode(wxTRANSPARENT);
				x1 = (long) (Origin.x+CarapaceGetShort(sp,&pos)*xscale +.5);
				y1 = (long) (Origin.y+CarapaceGetShort(sp,&pos)*yscale +.5);
				style = Carapace2WxStyle(CarapaceGetShort(sp,&pos));
				strp = CarapaceGetString(sp,&pos);
				dc.GetTextExtent(*strp,&tw,&th,&td);
				if(style == CPC_SG_LEFTTOP)
				{
					dc.DrawRotatedText(*strp,x1,y1,90);
				}
				else if(style == CPC_SG_CENTERTOP)
				{
					dc.DrawRotatedText(*strp,x1,y1+tw/2,90);
				}
				else if(style == CPC_SG_RIGHTTOP)
				{
					dc.DrawRotatedText(*strp,x1,y1+tw,90);
				}
				else if(style == CPC_SG_LEFTBASELINE)
				{
					dc.DrawRotatedText(*strp,x1-th+td,y1,90);
				}
				else if(style == CPC_SG_CENTERBASELINE)
				{
					dc.DrawRotatedText(*strp,x1-th+td,y1+tw/2,90);
				}
				else if(style == CPC_SG_RIGHTBASELINE)
				{
					dc.DrawRotatedText(*strp,x1-th+td,y1+tw,90);
				}
				else if(style == CPC_SG_LEFTBOTTOM)
				{
					dc.DrawRotatedText(*strp,x1-th,y1,90);
				}
				else if(style == CPC_SG_CENTERBOTTOM)
				{
					dc.DrawRotatedText(*strp,x1-th,y1+tw/2,90);
				}
				else if(style == CPC_SG_RIGHTBOTTOM)
				{
					dc.DrawRotatedText(*strp,x1-th,y1+tw,90);
				}

				delete strp;
				break;
	
			case CPC_SG_NEWPENNAME:
				name = CarapaceGetShort(sp,&pos);
				size = (long) floor(CarapaceGetShort(sp,&pos)*yscale + .5);
				style = Carapace2WxStyle(CarapaceGetShort(sp,&pos));
				pen = wxThePenList->FindOrCreatePen( 
					*CarapaceColorFromName(name),size,style);
				dc.SetPen(*pen);
				break;

			case CPC_SG_NEWPENRGB:
				r = CarapaceGetShort(sp,&pos);
				g = CarapaceGetShort(sp,&pos);
				b = CarapaceGetShort(sp,&pos);
				size = (long) floor(CarapaceGetShort(sp,&pos)*yscale + .5);
				style = Carapace2WxStyle(CarapaceGetShort(sp,&pos));
				pen = wxThePenList->FindOrCreatePen( 
					wxColour(r,g,b),size,style);
				dc.SetPen(*pen);
				break;
	
			case CPC_SG_NEWBRUSHNAME:
				name = CarapaceGetShort(sp,&pos);
				style = Carapace2WxStyle(CarapaceGetShort(sp,&pos));
				brush = wxTheBrushList->FindOrCreateBrush(
					*CarapaceColorFromName(name),style);
				dc.SetBrush(*brush);
				break;
	
			case CPC_SG_NEWBRUSHRGB:
				r = CarapaceGetShort(sp,&pos);
				g = CarapaceGetShort(sp,&pos);
				b = CarapaceGetShort(sp,&pos);
				style = Carapace2WxStyle(CarapaceGetShort(sp,&pos));
				brush = wxTheBrushList->FindOrCreateBrush(
					wxColour(r,g,b),style);
				dc.SetBrush(*brush);
				break;
	
			case CPC_SG_SETBACKGROUNDNAME:
				name = CarapaceGetShort(sp,&pos);
				brush = wxTheBrushList->FindOrCreateBrush(
					*CarapaceColorFromName(name),wxSOLID);
				brush = new wxBrush(*CarapaceColorFromName(name),wxSOLID);
				dc.SetBackground(*brush);
				dc.Clear();
				break;
	
			case CPC_SG_SETBACKGROUNDRGB:
				r = CarapaceGetShort(sp,&pos);
				g = CarapaceGetShort(sp,&pos);
				b = CarapaceGetShort(sp,&pos);
				brush = wxTheBrushList->FindOrCreateBrush(
					wxColour(r,g,b),wxSOLID);
				dc.SetBackground(*brush);
				break;
	
			case CPC_SG_SETFOREGROUNDNAME:
				name = CarapaceGetShort(sp,&pos);
				dc.SetTextForeground(*CarapaceColorFromName(name));
				break;
	
			case CPC_SG_SETFOREGROUNDRGB:
				r = CarapaceGetShort(sp,&pos);
				g = CarapaceGetShort(sp,&pos);
				b = CarapaceGetShort(sp,&pos);
				dc.SetTextForeground(wxColour(r,g,b));
				break;
	
			case CPC_SG_SETFONT:
				size = (long) floor(CarapaceGetShort(sp,&pos)*yscale+.5);
				style = Carapace2WxStyle(CarapaceGetShort(sp,&pos));
				family = Carapace2WxStyle(CarapaceGetShort(sp,&pos));
				weight = Carapace2WxStyle(CarapaceGetShort(sp,&pos));
				font = wxTheFontList->FindOrCreateFont(
					size,family,style,weight);
				dc.SetFont(*font);
				break;
	
			
			case CPC_SG_SAVESTATE:
				SaveState(dc);
				break;

			case CPC_SG_RESTORESTATE:
				RestoreState(dc);
				break;

        		case CPC_SG_CIRCLE:
        		case CPC_SG_DIAMOND:
        		case CPC_SG_SQUARE:
        		case CPC_SG_TRIANGLE:
        		case CPC_SG_EXE:
        		case CPC_SG_CROSS:
				x1 = (long) (Origin.x+CarapaceGetShort(sp,&pos)*xscale +.5);
				y1 = (long) (Origin.y+CarapaceGetShort(sp,&pos)*yscale +.5);
				size = (long) (CarapaceGetShort(sp,&pos)*minscale +.5);
				style = CarapaceGetShort(sp,&pos);
				CarapaceDrawGlyph(dc,x1,y1,cmd,size,style);
			break;

        		case CPC_SG_STAR:
				rx1 = Origin.x+CarapaceGetShort(sp,&pos)*xscale;
				ry1 = Origin.y+CarapaceGetShort(sp,&pos)*yscale;
				rsize = CarapaceGetShort(sp,&pos)*minscale;
				style = CarapaceGetShort(sp,&pos);
				CarapaceDrawGlyph2(dc,rx1,ry1,cmd,rsize,style);
				break;
		
			default:
			return;
		}
	}
}

void CarapaceGraphStructure::SaveState(wxDC& dc)
{
	CarapaceGraphState *state = new CarapaceGraphState;

	state->theBackground = dc.GetBackground();
        state->theBackgroundMode = dc.GetBackgroundMode();
        state->theBrush = dc.GetBrush();
	dc.GetClippingBox(state->theClippingBox, state->theClippingBox+1,
		state->theClippingBox+2,state->theClippingBox+3);
        state->theFont = dc.GetFont();
        state->theLogicalFunction = dc.GetLogicalFunction();
        state->thePen = dc.GetPen();
        state->theTextForeground = dc.GetTextForeground();
	dc.GetUserScale(state->theUserScale,state->theUserScale+1);
	GraphStateList.Append(state);
}

void CarapaceGraphStructure::RestoreState(wxDC& dc)
{
	wxCarapaceGraphStateListNode *node =
		GraphStateList.GetLast();
	if(!node)
	{
		return;
	}
	CarapaceGraphState *state = node->GetData();
	if(!state)
	{
		return;
	}
	
	dc.SetBackground(state->theBackground);
        dc.SetBackgroundMode(state->theBackgroundMode);
        dc.SetBrush(state->theBrush);
        dc.SetFont(state->theFont);
        dc.SetLogicalFunction(state->theLogicalFunction);
	dc.SetUserScale(state->theUserScale[0],state->theUserScale[1]);
	dc.DestroyClippingRegion();
	dc.SetClippingRegion(state->theClippingBox[0], state->theClippingBox[1],
		state->theClippingBox[2],state->theClippingBox[3]);
        dc.SetPen(state->thePen);
        dc.SetTextForeground(state->theTextForeground);

	if(GraphStateList.GetCount() > 1)
	{
		GraphStateList.DeleteContents(TRUE);
		GraphStateList.DeleteNode(node);
	}
}

void CarapaceGraphStructure::GetOldState(wxDC& dc)
{
	wxCarapaceGraphStateListNode *node;
	
	while(GraphStateList.GetCount() > 0)
	{
		node = GraphStateList.GetLast();
		GraphStateList.DeleteContents(TRUE);
		GraphStateList.DeleteNode(node);
	}
	SaveState(dc);
}

void CarapaceGraphStructure::RestoreOldState(wxDC& dc)
{
	wxCarapaceGraphStateListNode *node;
	
	while(GraphStateList.GetCount() > 1)
	{
		node = GraphStateList.GetLast();
		GraphStateList.DeleteContents(TRUE);
		GraphStateList.DeleteNode(node);
	}
	RestoreState(dc);
}

bool CarapaceGraphStructure::DoGraphRequest(cpc_sg_ptr sgp)
{
	//wxPaintDC dc( (wxWindow*) Parent );
	//((wxScrolledWindow *) Parent)->PrepareDC( dc );
	wxClientDC dc( (wxWindow*) Parent );
	//((wxScrolledWindow *) Parent)->PrepareDC( dc );
	switch(sgp->type)
	{
		case CPC_SG_GETCHARWIDTH:
			sgp->size = dc.GetCharWidth()*10;
			//sgp->size = 6;
			break;

		case CPC_SG_GETCHARHEIGHT:
			sgp->size = dc.GetCharHeight()*10;
			//sgp->size = 14;
			break;

		default:
			return(FALSE);
	}
	return(TRUE);
}

bool CarapaceGraphStructure::AddGraphCommand(cpc_sg_ptr sgp)
{
	wxString *theString,tmpstring;
	if(Working->Type == CPC_SG_STRUCTURETYPE)
	{
		return(Working->Data.cgstructure->AddGraphCommand(sgp));
	}

	theString = Working->Data.cgs;
	switch(sgp->type)
	{
		// here Working is string type and we just append command to list
		case CPC_SG_DRAWLINE:
			CarapacePushShort(theString,CPC_SG_DRAWLINE);
			CarapacePushShort(theString,sgp->coords[0]);
			CarapacePushShort(theString,sgp->coords[1]);
			CarapacePushShort(theString,sgp->coords[2]);
			CarapacePushShort(theString,sgp->coords[3]);
			break;

		case CPC_SG_DRAWTEXT:
			CarapacePushShort(theString,CPC_SG_DRAWTEXT);
			CarapacePushShort(theString,sgp->coords[0]);
			CarapacePushShort(theString,sgp->coords[1]);
			CarapacePushShort(theString,sgp->style);
			tmpstring = sgp->string;
			CarapacePushString(theString,tmpstring);
			break;

		case CPC_SG_DRAWTEXTUP:
			CarapacePushShort(theString,CPC_SG_DRAWTEXTUP);
			CarapacePushShort(theString,sgp->coords[0]);
			CarapacePushShort(theString,sgp->coords[1]);
			CarapacePushShort(theString,sgp->style);
			tmpstring = sgp->string;
			CarapacePushString(theString,tmpstring);
			break;

		case CPC_SG_NEWPENNAME:
			CarapacePushShort(theString,CPC_SG_NEWPENNAME);
			CarapacePushShort(theString,sgp->colorname);
			CarapacePushShort(theString,sgp->size);
			CarapacePushShort(theString,sgp->style);
			break;

		case CPC_SG_NEWPENRGB:
			CarapacePushShort(theString,CPC_SG_NEWPENRGB);
			CarapacePushShort(theString,sgp->colorRGB[0]);
			CarapacePushShort(theString,sgp->colorRGB[1]);
			CarapacePushShort(theString,sgp->colorRGB[2]);
			CarapacePushShort(theString,sgp->size);
			CarapacePushShort(theString,sgp->style);
			break;

		case CPC_SG_NEWBRUSHNAME:
			CarapacePushShort(theString,CPC_SG_NEWBRUSHNAME);
			CarapacePushShort(theString,sgp->colorname);
			CarapacePushShort(theString,sgp->style);
			break;

		case CPC_SG_NEWBRUSHRGB:
			CarapacePushShort(theString,CPC_SG_NEWBRUSHRGB);
			CarapacePushShort(theString,sgp->colorRGB[0]);
			CarapacePushShort(theString,sgp->colorRGB[1]);
			CarapacePushShort(theString,sgp->colorRGB[2]);
			CarapacePushShort(theString,sgp->style);
			break;

		case CPC_SG_SETBACKGROUNDNAME:
			CarapacePushShort(theString,CPC_SG_SETBACKGROUNDNAME);
			CarapacePushShort(theString,sgp->colorname);
			break;

		case CPC_SG_SETBACKGROUNDRGB:
			CarapacePushShort(theString,CPC_SG_SETBACKGROUNDRGB);
			CarapacePushShort(theString,sgp->colorRGB[0]);
			CarapacePushShort(theString,sgp->colorRGB[1]);
			CarapacePushShort(theString,sgp->colorRGB[2]);
			break;

		case CPC_SG_SETFOREGROUNDNAME:
			CarapacePushShort(theString,CPC_SG_SETFOREGROUNDNAME);
			CarapacePushShort(theString,sgp->colorname);
			break;

		case CPC_SG_SETFOREGROUNDRGB:
			CarapacePushShort(theString,CPC_SG_SETBACKGROUNDRGB);
			CarapacePushShort(theString,sgp->colorRGB[0]);
			CarapacePushShort(theString,sgp->colorRGB[1]);
			CarapacePushShort(theString,sgp->colorRGB[2]);
			break;

		case CPC_SG_SETFONT:
			CarapacePushShort(theString,CPC_SG_SETFONT);
			CarapacePushShort(theString,sgp->size);
			CarapacePushShort(theString,sgp->style);
			CarapacePushShort(theString,sgp->family);
			CarapacePushShort(theString,sgp->weight);
			break;

		case CPC_SG_SETFONTNAME:
			CarapacePushShort(theString,CPC_SG_SETFONT);
			tmpstring = sgp->string;
			CarapacePushString(theString,tmpstring);
			break;

		case CPC_SG_SAVESTATE:
			CarapacePushShort(theString,CPC_SG_SAVESTATE);
			break;

		case CPC_SG_RESTORESTATE:
			CarapacePushShort(theString,CPC_SG_RESTORESTATE);
			break;

        	case CPC_SG_CIRCLE:
        	case CPC_SG_DIAMOND:
        	case CPC_SG_SQUARE:
        	case CPC_SG_TRIANGLE:
        	case CPC_SG_EXE:
        	case CPC_SG_CROSS:
        	case CPC_SG_STAR:
        	case CPC_SG_DINGBAT:
			CarapacePushShort(theString,sgp->type);
			CarapacePushShort(theString,sgp->coords[0]);
			CarapacePushShort(theString,sgp->coords[1]);
			CarapacePushShort(theString,sgp->size);
			CarapacePushShort(theString,sgp->style);
			break;

		default:
		return(FALSE);
	}
	return(TRUE);
}

void CarapaceStaticGraph::OnChar(wxKeyEvent &event)
{
	long       ch,count,i;
	wxAcceleratorEntry entry;

	ch =  event.m_keyCode;

// sometimes you have to hit things with a really big brick.  Some text widgets (e.g., gtk)
// grab hold of key presses before the accelerator table can deal with them.  To get around
// this, we keep our own list of accelerators and handle the troublesome cases.
	if(ch < 26 && Parent->Accelerators)
	{
		count = Parent->Accelerators->GetCount();
		for(i=0;i<count;i++)
		{
			entry = Parent->Accelerators->Item(i);
			if( (ch == entry.GetKeyCode()-64 || ch == entry.GetKeyCode()-96) &&
				entry.GetFlags() == wxACCEL_CTRL )
			{
				wxCommandEvent command_event( wxEVT_COMMAND_MENU_SELECTED, entry.GetCommand() );
				Parent->GetEventHandler()->ProcessEvent( command_event );
				return;
			}
		}
	}

        if(event.GetKeyCode() == WXK_RETURN)
        {
		if(thisApp->InGraphicsPause && 
			thisApp->PausedGraphFrame == Parent->GraphFrameIndex)
		{
			thisApp->InGraphicsPause = FALSE;
			return;
		}
                thisApp->TextFrameFront(thisApp->CommandFrame);
        }
	if((event.GetKeyCode() == 'q' || event.GetKeyCode() == 'Q')
		&& MouseRequestState != CPC_SG_NOMOUSEREQUEST 
		&& MouseRequestState != CPC_SG_MOUSEINTERRUPT)
	{
		MouseRequestState = CPC_SG_MOUSEQUIT;
	}
	if((event.GetKeyCode() == 'q' || event.GetKeyCode() == 'Q')
		&& thisApp->MouseRequestState != CPC_SG_NOMOUSEREQUEST)
	{
		thisApp->MouseRequestState = CPC_SG_MOUSEQUIT;
	}
}

