/*
*(C)* The file is part of the source distribution of Carapace
*(C)* version 0.5 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* This file is distributed under the GNU General Public License
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/


#include "cpcplatform.h"



// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif
#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif


#include "wx/fontdlg.h"
#include "wx/colour.h"
#include "wx/caret.h"
#include "wx/print.h"
#include "wx/printdlg.h"



#include <math.h>

#include "carapace.h"

#undef UNDEFINED

#include "wx/listimpl.cpp"
 

CarapaceHistory::CarapaceHistory(long nlines)
{
	Capacity = nlines;
	commands = CarapaceHistoryList();
}

CarapaceHistory::~CarapaceHistory(void)
{
	commands.DeleteContents(TRUE);
	while(commands.GetCount() > 0)
	{
		commands.DeleteNode((wxCarapaceHistoryListNode *)commands.GetLast());
	}
}

void CarapaceHistory::ChangeHistoryLength(unsigned long newlength)
{
	commands.DeleteContents(TRUE);
	while(commands.GetCount() > newlength)
	{
		commands.DeleteNode((wxCarapaceHistoryListNode *)commands.GetLast());
	}
	Capacity = newlength;
}


void CarapaceHistory::ClearCommands(void)
{
	commands.DeleteContents(TRUE);
	while(commands.GetCount() > 0)
	{
		commands.DeleteNode((wxCarapaceHistoryListNode *)commands.GetLast());
	}
}

void CarapaceHistory::AddCommand(wxString& newCmd) 
{
	wxCarapaceSplitTextFrameListNode *stnode = thisApp->STFrameList.GetFirst();
	CarapaceSplitTextFrame *stfp;

	wxString *tmp = new wxString;
	*tmp = newCmd;
	if(tmp->Last() == '\n')
	{
		tmp->RemoveLast();
	}
	commands.Insert(tmp);
	if(commands.GetCount() > Capacity)
	{
		commands.DeleteContents(TRUE);
		commands.DeleteNode((wxCarapaceHistoryListNode *)commands.GetLast());
	}

	// increment history pointers in text windows
 
        stnode = thisApp->STFrameList.GetFirst();
        while(stnode)
        {
                stfp = stnode->GetData();
                stnode = stnode->GetNext();
		stfp->command_ctrl->IncHistPtr();
#if 0
		//stfp->history_spinner->SetRange(0,commands.GetCount());
		unsigned int val = stfp->history_spinner->GetValue();
		if(val > 0 && val < commands.GetCount())
		{
			val++;
			stfp->history_spinner->SetValue(val);
		}
#endif /* 0 */
		long val;
		wxString tmpstring;
		stfp->history_counter->GetLabel().ToLong(&val);
		if(val > 0 && val < (long) commands.GetCount())
		{
			val++;
			tmpstring << val;
			stfp->history_counter->SetLabel(tmpstring);
		}
        } 
}

bool CarapaceHistory::HasLine(long index)
{
	int len = commands.GetCount();
	if(index < 1 || index > len)
	{
		return(FALSE);
	}
	return(TRUE);
}

wxString* CarapaceHistory::GetCommand(long index)
{
	int len = commands.GetCount();
	if(index < 1 || index > len)
	{
		CarapaceBeep(1);
	}
	if(len == 0)
	{
		return(NULL);
	}
	index = index < 1 ? 1 : index;
	index = index > len ? len : index;

	return(commands.Item(index-1)->GetData());
}
