/*
*(C)* The file is part of the source distribution of Carapace 0.1.1
*(C)*
*(C)* Copyright (c) 2000 by Gary Oehlert  and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* This file is distributed under the GNU General Public License
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

#include "cpcplatform.h"

// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/gdicmn.h"
#include "wx/fontdlg.h"
#include "wx/colour.h"
#include "wx/caret.h"
#include "wx/print.h"
#include "wx/printdlg.h"
#include "wx/colordlg.h"
#include "wx/fontmap.h"
#include "wx/splitter.h"
#include "wx/generic/dirctrlg.h"
#include "wx/calctrl.h"
#include "wx/colordlg.h"
#include "wx/cmndata.h"
#include "wx/colour.h"
#include "wx/gdicmn.h"
#include "wx/list.h"
#include "wx/font.h"
#if defined(__WXGTK__)
#include "wx/generic/listctrl.h"
#elif defined(__WXMSW__)
#include "wx/listctrl.h"
#elif defined(__WXMAC__)
#include "wx/listctrl.h"
#endif


#include <math.h>

#include "cpcdefs.h"
#include "carapace.h"
#include "buttoncontrols.h"
#include "cpccontrols.h"

#include "wx/xrc/xmlres.h"
#include "wx/xrc/xh_all.h"
#include "wx/dynarray.h"
#include "wx/arrimpl.cpp"

WX_DEFINE_OBJARRAY(ArrayOfDialogObjectInfo);


#undef UNDEFINED

#define ALMDN thisApp->LastModalDialogNames
#define ALMDV thisApp->LastModalDialogValues

void cpcTextCtrlExtractor(wxDialog *dlg, wxString& name)
{
	wxTextCtrl *dptr;
	dptr = XRCCTRL(*dlg,name,wxTextCtrl);
	if(dptr == NULL)
	{
		return;
	}
	ALMDN.Add(name);
	ALMDV.Add(dptr->GetValue());
}
void cpcComboBoxExtractor(wxDialog *dlg, wxString& name)
{
	wxComboBox *dptr;
	dptr = XRCCTRL(*dlg,name,wxComboBox);
	if(dptr == NULL)
	{
		return;
	}
	ALMDN.Add(name);
	ALMDV.Add(dptr->GetValue());
}
void cpcCheckBoxExtractor(wxDialog *dlg, wxString& name)
{
	wxCheckBox *dptr;
	dptr = XRCCTRL(*dlg,name,wxCheckBox);
	if(dptr == NULL)
	{
		return;
	}
	ALMDN.Add(name);
	if(dptr->GetValue())
	{
		ALMDV.Add("checked");
	}
	else
	{
		ALMDV.Add("unchecked");
	}
}
void cpcCheckListBoxExtractor(wxDialog *dlg, wxString& name)
{
	int i;
	wxCheckListBox *dptr;
	dptr = XRCCTRL(*dlg,name,wxCheckListBox);
	if(dptr == NULL)
	{
		return;
	}
	for (i=0;i<dptr->GetCount();i++)
	{
		if (dptr->IsChecked(i))
		{
			ALMDN.Add(name);
			ALMDV.Add(dptr->GetString(i));
		}
	}
}
void cpcListBoxExtractor(wxDialog *dlg, wxString& name)
{
	int i;
	wxListBox *dptr;
	dptr = XRCCTRL(*dlg,name,wxListBox);
	if(dptr == NULL)
	{
		return;
	}
	for (i=0;i<dptr->GetCount();i++)
	{
		if (dptr->Selected(i))
		{
			ALMDN.Add(name);
			ALMDV.Add(dptr->GetString(i));
		}
	}
}
void cpcChoiceExtractor(wxDialog *dlg, wxString& name)
{
	wxChoice *dptr;
	dptr = XRCCTRL(*dlg,name,wxChoice);
	if(dptr == NULL)
	{
		return;
	}
	ALMDN.Add(name);
	ALMDV.Add(dptr->GetString(dptr->GetSelection()));
}
void cpcGenericDirCtrlExtractor(wxDialog *dlg, wxString& name)
{
	wxGenericDirCtrl *dptr;
	dptr = XRCCTRL(*dlg,name,wxGenericDirCtrl);
	if(dptr == NULL)
	{
		return;
	}
	ALMDN.Add(name);
	ALMDV.Add(dptr->GetFilePath());
}
void cpcRadioBoxExtractor(wxDialog *dlg, wxString& name)
{
	wxRadioBox *dptr;
	dptr = XRCCTRL(*dlg,name,wxRadioBox);
	if(dptr == NULL)
	{
		return;
	}
	ALMDN.Add(name);
	ALMDV.Add(dptr->GetStringSelection());
}
void cpcRadioButtonExtractor(wxDialog *dlg, wxString& name)
{
	wxRadioButton *dptr;
	dptr = XRCCTRL(*dlg,name,wxRadioButton);
	if(dptr == NULL)
	{
		return;
	}
	ALMDN.Add(name);
	if (dptr->GetValue() )
	{
		ALMDV.Add("Selected");
	}
	else
	{
		ALMDV.Add("Not Selected");
	}
}
void cpcCalendarCtrlExtractor(wxDialog *dlg, wxString& name)
{
	wxCalendarCtrl *dptr;
	dptr = XRCCTRL(*dlg,name,wxCalendarCtrl);
	if(dptr == NULL)
	{
		return;
	}
	ALMDN.Add(name);
	ALMDV.Add(dptr->GetDate().FormatDate());
}
void cpcScrollBarExtractor(wxDialog *dlg, wxString& name)
{
	wxScrollBar *dptr;
	dptr = XRCCTRL(*dlg,name,wxScrollBar);
	if(dptr == NULL)
	{
		return;
	}
	ALMDN.Add(name);
	ALMDV.Add("");
	ALMDV.Last() << dptr->GetThumbPosition() << " out of "
		<< dptr->GetRange();
}
void cpcSliderExtractor(wxDialog *dlg, wxString& name)
{
	wxSlider *dptr;
	dptr = XRCCTRL(*dlg,name,wxSlider);
	if(dptr == NULL)
	{
		return;
	}
	ALMDN.Add(name);
	ALMDV.Add("");
	ALMDV.Last() << dptr->GetValue() << " "
		<< dptr->GetMin() << " " << dptr->GetMax();
}
void cpcSpinButtonExtractor(wxDialog *dlg, wxString& name)
{
	wxSpinButton *dptr;
	dptr = XRCCTRL(*dlg,name,wxSpinButton);
	if(dptr == NULL)
	{
		return;
	}
	ALMDN.Add(name);
	ALMDV.Add("");
	ALMDV.Last() << dptr->GetValue() << " "
		<< dptr->GetMin() << " " << dptr->GetMax();
}
void cpcSpinCtrlExtractor(wxDialog *dlg, wxString& name)
{
	wxSpinCtrl *dptr;
	dptr = XRCCTRL(*dlg,name,wxSpinCtrl);
	if(dptr == NULL)
	{
		return;
	}
	ALMDN.Add(name);
	ALMDV.Add("");
	ALMDV.Last() << dptr->GetValue() << " "
		<< dptr->GetMin() << " " << dptr->GetMax();
}
void cpcListCtrlExtractor(wxDialog *dlg, wxString& name)
{
	long item = -1;
	wxListCtrl *dptr;
	dptr = XRCCTRL(*dlg,name,wxListCtrl);
	if(dptr == NULL)
	{
		return;
	}
	for ( ;; )
	{
		item = dptr->GetNextItem(item,wxLIST_NEXT_ALL,wxLIST_STATE_SELECTED);
		if ( item != -1 )
		{
			ALMDN.Add(name);
			ALMDV.Add(dptr->GetItemText(item));
		}
		else
		{
			break;
		}
	}
}
void cpcInOutCtrlExtractor(wxDialog *dlg, wxString& name)
{
	cpcInOutCtrl *dptr;
	dptr = XRCCTRL(*dlg,name,cpcInOutCtrl);
	if(dptr == NULL)
	{
		return;
	}
	ALMDN.Add(name);
	ALMDV.Add(dptr->GetSelection());
}
void cpcColourDialogExtractorBase(wxColourDialog *dptr, wxString& name)
{
	wxColour Colour;
	wxString ColourName="",StepResult="";
	
#if wxCHECK_VERSION(2,5,0)
	wxColourDatabase * NewColourDatabase=new wxColourDatabase();
#else
	wxColourDatabase * NewColourDatabase=new wxColourDatabase(wxKEY_STRING);
	NewColourDatabase->Initialize();
#endif

	Colour=dptr->GetColourData().GetColour();
	ColourName = NewColourDatabase->FindName(Colour);

	StepResult="";
	StepResult << " red " << Colour.Red();
	ALMDN.Add(name);
	ALMDV.Add(StepResult);
						
	StepResult="";
	StepResult << " green " << Colour.Green();
	ALMDN.Add(name);
	ALMDV.Add(StepResult);
						
	StepResult="";
	StepResult << " blue " << (int)Colour.Blue();
	ALMDN.Add(name);
	ALMDV.Add(StepResult);
						
	StepResult="";
	StepResult << " ColorName " << ColourName;
	ALMDN.Add(name);
	ALMDV.Add(StepResult);
}
void cpcColourDialogExtractor(wxDialog *dlg, wxString& name)
{
	wxColourDialog *dptr;
	dptr = XRCCTRL(*dlg,name,wxColourDialog);
	if(dptr == NULL)
	{
		return;
	}
	cpcColourDialogExtractorBase(dptr,name);
}
void cpcFontDialogExtractorBase(wxFontDialog *dptr, wxString& name)
{
	wxFont Font;
	wxColour Colour;
	wxString StepResult="";
	Font=dptr->GetFontData().GetChosenFont();

	StepResult="";
	StepResult << " PointSize " << Font.GetPointSize();
	ALMDN.Add(name);
	ALMDV.Add(StepResult);

	StepResult="";
	StepResult << " family " << Font.GetFamily();
	ALMDN.Add(name);
	ALMDV.Add(StepResult);

	StepResult="";
	StepResult << " style " << Font.GetStyle();
	ALMDN.Add(name);
	ALMDV.Add(StepResult);
						
	StepResult="";
	StepResult << " weight " << Font.GetWeight();
	ALMDN.Add(name);
	ALMDV.Add(StepResult);

	StepResult="";
	StepResult << " underlined " << Font.GetUnderlined();
	ALMDN.Add(name);
	ALMDV.Add(StepResult);
						
	StepResult="";
	StepResult << " facename " << Font.GetFaceName();
	ALMDN.Add(name);
	ALMDV.Add(StepResult);
						
	StepResult="";
	StepResult << " DefaultEncoding " << Font.GetDefaultEncoding();
	ALMDN.Add(name);
	ALMDV.Add(StepResult);

	StepResult="";
	StepResult << " size " << Font.GetPointSize();
	ALMDN.Add(name);
	ALMDV.Add(StepResult);
						
	StepResult="";
	StepResult << " FontInfoDesc " << Font.GetNativeFontInfoDesc();
	ALMDN.Add(name);
	ALMDV.Add(StepResult);

}
void cpcFontDialogExtractor(wxDialog *dlg, wxString& name)
{
	wxFontDialog *dptr;
	dptr = XRCCTRL(*dlg,name,wxFontDialog);
	if(dptr == NULL)
	{
		return;
	}
	cpcFontDialogExtractorBase(dptr,name);
}

#define PATCHSIZE 24
void CarapaceApp::InitializeDialogs(void)
{

	long i,j,type,size;
	//wxBitmap bitmap(16,16,-1);
	wxBitmap bitmap(PATCHSIZE,PATCHSIZE,-1);
	wxBitmap *bitmapp;
	wxMemoryDC dc;
	wxColour col;
	wxBrush brush;
	wxPen pen;

	RHCreators.Add("cpcColourDialogXmlHandler");
	RHCreators.Add("cpcFontDialogXmlHandler");
	RHCreators.Add("wxHtmlButtonXmlHandler");
	RHCreators.Add("wxTextButtonXmlHandler");
	RHCreators.Add("cpcDisplayButtonXmlHandler");
	RHCreators.Add("cpcHtmlControlXmlHandler");
	RHCreators.Add("cpcTextControlXmlHandler");
	RHCreators.Add("SliderViewerXmlHandler");
	RHCreators.Add("ListViewerXmlHandler");
	//RHCreators.Add("ListViewerBXmlHandler");
	RHCreators.Add("StaticTextViewerXmlHandler");
	RHCreators.Add("ScrolledTextViewerXmlHandler");
	RHCreators.Add("ShowFileViewerXmlHandler");
	RHCreators.Add("SmallColorPickerXmlHandler");
	RHCreators.Add("MediumColorPickerXmlHandler");
	RHCreators.Add("GlyphPickerXmlHandler");
	RHCreators.Add("cpcPopupMenuXmlHandler");
	RHCreators.Add("CheckboxViewerXmlHandler");
	RHCreators.Add("RadioboxViewerXmlHandler");	
	RHCreators.Add("SpinCtrlViewerXmlHandler");		
	RHCreators.Add("ButtonViewerXmlHandler");		
	RHCreators.Add("cpcInOutCtrlXmlHandler");
	RHCreators.Add("PlotViewerXmlHandler");

	ViewerClassNames.Add("SliderViewer");
	ViewerClassNames.Add("ListViewer");
	//ViewerClassNames.Add("ListViewerB");
	ViewerClassNames.Add("StaticTextViewer");
	ViewerClassNames.Add("ScrolledTextViewer");
	ViewerClassNames.Add("ShowFileViewer");
	ViewerClassNames.Add("SmallColorPicker");
	ViewerClassNames.Add("MediumColorPicker");
	ViewerClassNames.Add("GlyphPicker");
	ViewerClassNames.Add("CheckboxViewer");
	ViewerClassNames.Add("RadioboxViewer");	
	ViewerClassNames.Add("SpinCtrlViewer");	
	ViewerClassNames.Add("ButtonViewer");	
	ViewerClassNames.Add("PlotAreaRecViewer");	

	MDVExtractors["wxTextCtrl"] = cpcTextCtrlExtractor;
	MDVExtractors["wxComboBox"] = cpcComboBoxExtractor;
	MDVExtractors["wxCheckBox"] = cpcCheckBoxExtractor;
	MDVExtractors["wxCheckListBox"] = cpcCheckListBoxExtractor;
	MDVExtractors["wxListBox"] = cpcListBoxExtractor;
	MDVExtractors["wxChoice"] = cpcChoiceExtractor;
	MDVExtractors["wxGenericDirCtrl"] = cpcGenericDirCtrlExtractor;
	MDVExtractors["wxRadioBox"] = cpcRadioBoxExtractor;
	MDVExtractors["wxRadioButton"] = cpcRadioButtonExtractor;
	MDVExtractors["wxCalendarCtrl"] = cpcCalendarCtrlExtractor;
	MDVExtractors["wxScrollBar"] = cpcScrollBarExtractor;
	MDVExtractors["wxSlider"] = cpcSliderExtractor;
	MDVExtractors["wxSpinButton"] = cpcSpinButtonExtractor;
	MDVExtractors["wxSpinCtrl"] = cpcSpinCtrlExtractor;
	MDVExtractors["wxListCtrl"] = cpcListCtrlExtractor;
	MDVExtractors["cpcInOutCtrl"] = cpcInOutCtrlExtractor;

	thisApp->ColorArray.Add("AQUAMARINE");
	thisApp->ColorArray.Add("BLACK");
	thisApp->ColorArray.Add("BLUE");
	thisApp->ColorArray.Add("BLUE VIOLET");
	thisApp->ColorArray.Add("BROWN");
	thisApp->ColorArray.Add("CADET BLUE");
	thisApp->ColorArray.Add("CORAL");
	thisApp->ColorArray.Add("CORNFLOWER BLUE");
	thisApp->ColorArray.Add("CYAN");
	thisApp->ColorArray.Add("DARK GREY");
	thisApp->ColorArray.Add("DARK GREEN");
	thisApp->ColorArray.Add("DARK OLIVE GREEN");
	thisApp->ColorArray.Add("DARK ORCHID");
	thisApp->ColorArray.Add("DARK SLATE BLUE");
	thisApp->ColorArray.Add("DARK SLATE GREY");
	thisApp->ColorArray.Add("DARK TURQUOISE");
	thisApp->ColorArray.Add("DIM GREY");
	thisApp->ColorArray.Add("FIREBRICK");
	thisApp->ColorArray.Add("FOREST GREEN");
	thisApp->ColorArray.Add("GOLD");
	thisApp->ColorArray.Add("GOLDENROD");
	thisApp->ColorArray.Add("GREY");
	thisApp->ColorArray.Add("GREEN");
	thisApp->ColorArray.Add("GREEN YELLOW");
	thisApp->ColorArray.Add("INDIAN RED");
	thisApp->ColorArray.Add("KHAKI");
	thisApp->ColorArray.Add("LIGHT BLUE");
	thisApp->ColorArray.Add("LIGHT GREY");
	thisApp->ColorArray.Add("LIGHT STEEL BLUE");
	thisApp->ColorArray.Add("LIME GREEN");
	thisApp->ColorArray.Add("LIGHT MAGENTA");
	thisApp->ColorArray.Add("MAGENTA");
	thisApp->ColorArray.Add("MAROON");
	thisApp->ColorArray.Add("MEDIUM AQUAMARINE");
	thisApp->ColorArray.Add("MEDIUM GREY");
	thisApp->ColorArray.Add("MEDIUM BLUE");
	thisApp->ColorArray.Add("MEDIUM FOREST GREEN");
	thisApp->ColorArray.Add("MEDIUM GOLDENROD");
	thisApp->ColorArray.Add("MEDIUM ORCHID");
	thisApp->ColorArray.Add("MEDIUM SEA GREEN");
	thisApp->ColorArray.Add("MEDIUM SLATE BLUE");
	thisApp->ColorArray.Add("MEDIUM SPRING GREEN");
	thisApp->ColorArray.Add("MEDIUM TURQUOISE");
	thisApp->ColorArray.Add("MEDIUM VIOLET RED");
	thisApp->ColorArray.Add("MIDNIGHT BLUE");
	thisApp->ColorArray.Add("NAVY");
	thisApp->ColorArray.Add("ORANGE");
	thisApp->ColorArray.Add("ORANGE RED");
	thisApp->ColorArray.Add("ORCHID");
	thisApp->ColorArray.Add("PALE GREEN");
	thisApp->ColorArray.Add("PINK");
	thisApp->ColorArray.Add("PLUM");
	thisApp->ColorArray.Add("PURPLE");
	thisApp->ColorArray.Add("RED");
	thisApp->ColorArray.Add("SALMON");
	thisApp->ColorArray.Add("SEA GREEN");
	thisApp->ColorArray.Add("SIENNA");
	thisApp->ColorArray.Add("SKY BLUE");
	thisApp->ColorArray.Add("SLATE BLUE");
	thisApp->ColorArray.Add("SPRING GREEN");
	thisApp->ColorArray.Add("STEEL BLUE");
	thisApp->ColorArray.Add("TAN");
	thisApp->ColorArray.Add("THISTLE");
	thisApp->ColorArray.Add("TURQUOISE");
	thisApp->ColorArray.Add("VIOLET");
	thisApp->ColorArray.Add("VIOLET RED");
	thisApp->ColorArray.Add("WHEAT");
	thisApp->ColorArray.Add("WHITE");
	thisApp->ColorArray.Add("YELLOW");
	thisApp->ColorArray.Add("YELLOW GREEN");

	thisApp->GlyphArray.Add("NONE");
	thisApp->GlyphArray.Add("DOT");
	thisApp->GlyphArray.Add("EXE");
	thisApp->GlyphArray.Add("CROSS");
	thisApp->GlyphArray.Add("SPIDER");
	thisApp->GlyphArray.Add("CIRCLE");
	thisApp->GlyphArray.Add("DIAMOND");
	thisApp->GlyphArray.Add("SQUARE");
	thisApp->GlyphArray.Add("TRIANGLE");

	thisApp->ColorArrayLength = thisApp->ColorArray.GetCount();
	thisApp->GlyphArrayLength = thisApp->GlyphArray.GetCount();
	
	thisApp->ColorGlyphPatches = new wxImageList(PATCHSIZE,PATCHSIZE,TRUE,
		thisApp->ColorArrayLength*thisApp->GlyphArrayLength+1);
	size = (long) (PATCHSIZE/2.5);
	for(i=0;i<thisApp->GlyphArrayLength;i++)
	{
		for(j=0;j<thisApp->ColorArrayLength;j++)
		{
			bitmapp = new wxBitmap(PATCHSIZE,PATCHSIZE,-1);
			dc.SelectObject(*bitmapp);
			brush.SetColour(thisApp->ColorArray[j]);
			pen.SetColour(thisApp->ColorArray[j]);
			dc.SetBrush(brush);
			dc.SetPen(pen);
			dc.DrawRectangle(0,0,PATCHSIZE,PATCHSIZE);
			if(i==0) // no glyph
			{
				dc.SelectObject(wxNullBitmap);
				thisApp->ColorGlyphPatches->Add(*bitmapp);
				continue;
			}
			pen.SetColour("BLACK");
			if(thisApp->ColorArray[j] == "BLACK")
			{
				pen.SetColour("LIGHT GREY");
			}
			dc.SetPen(pen);
			size = (long) (PATCHSIZE/2.5);
			switch(i)
			{	
			case 1: // DOT
				type = CPC_SG_CIRCLE;
				size = 1;
				break;
			case 2: // EXE
				type = CPC_SG_EXE;
				break;
			case 3: // CROSS
				type = CPC_SG_CROSS;
				break;
			case 4: // SPIDER
				type = CPC_SG_STAR;
				CarapaceDrawGlyph2(dc,PATCHSIZE/2.,PATCHSIZE/2.
					,type,PATCHSIZE/2.5,0);
				dc.SelectObject(wxNullBitmap);
				thisApp->ColorGlyphPatches->Add(*bitmapp);
				continue;
			case 5: // CIRCLE
				type = CPC_SG_CIRCLE;
				break;
			case 6: // DIAMOND
				type = CPC_SG_DIAMOND;
				break;
			case 7: // SQUARE
				type = CPC_SG_SQUARE;
				break;
			case 8: // TRIANGLE
				type = CPC_SG_TRIANGLE;
				break;
			default:
				type = CPC_SG_CIRCLE;
			}
			CarapaceDrawGlyph(dc,PATCHSIZE/2,PATCHSIZE/2,type,size,0);
			dc.SelectObject(wxNullBitmap);
			thisApp->ColorGlyphPatches->Add(*bitmapp);
		}
	}
	bitmapp = new wxBitmap(PATCHSIZE,PATCHSIZE,-1);
	dc.SelectObject(*bitmapp);
	brush.SetColour("WHITE");
	pen.SetColour("WHITE");
	dc.SetBrush(brush);
	dc.SetPen(pen);
	dc.DrawRectangle(0,0,PATCHSIZE,PATCHSIZE);
	pen.SetColour("RED");
	pen.SetWidth(3);
	dc.SetPen(pen);
	dc.DrawCircle(PATCHSIZE/2,PATCHSIZE/2,size);
	i = (long) (PATCHSIZE/2-.7*size+1);
	j = (long) (PATCHSIZE/2+.7*size);
	dc.DrawLine(i,i,j,j);
	dc.SelectObject(wxNullBitmap);
	thisApp->ColorGlyphPatches->Add(*bitmapp);
	// need to add the deleted glyph
}

#undef ALMDN
#undef ALMDV

