#include "cpcplatform.h"

#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif
#include "carapace.h"
#include "wx/listctrl.h"
#include "wx/font.h"
#include "wx/arrimpl.cpp"


WX_DEFINE_OBJARRAY(ArrayOfInOutItems);

BEGIN_EVENT_TABLE(cpcInListCtrl,wxListCtrl)
EVT_LIST_ITEM_SELECTED(CPC_DLGCTRL_LISTCLICK, cpcInListCtrl::OnSelected)
END_EVENT_TABLE()

cpcInListCtrl::cpcInListCtrl(wxWindow * parent) : wxListCtrl(parent,CPC_DLGCTRL_LISTCLICK,
	wxDefaultPosition, wxSize(300,200), wxLC_REPORT | wxLC_NO_HEADER)
{
	Parent = (cpcInOutCtrl *) parent;
	InsertColumn(0,"Selected");
}

cpcInListCtrl::cpcInListCtrl(wxWindow * parent, long boxheight) :
	wxListCtrl(parent,CPC_DLGCTRL_LISTCLICK, wxDefaultPosition, 
		wxSize(300,boxheight), wxLC_REPORT | wxLC_NO_HEADER)
{
	Parent = (cpcInOutCtrl *) parent;
	InsertColumn(0,"Selected");
}

cpcInListCtrl::~cpcInListCtrl(void)
{
}

void cpcInListCtrl::OnSelected(wxListEvent& event)
{
	long i = event.GetIndex();
	long index = GetItemData(i);
	DeleteItem(i);
	Parent->PutinOut(index);
	if(GetItemCount() == 0) 
	{
		Parent->ResetSubset();
	}
}


BEGIN_EVENT_TABLE(cpcOutListCtrl,wxListCtrl)
EVT_LIST_ITEM_SELECTED(CPC_DLGCTRL_LISTCLICK, cpcOutListCtrl::OnSelected)
END_EVENT_TABLE()

cpcOutListCtrl::cpcOutListCtrl(wxWindow * parent) : wxListCtrl(parent,CPC_DLGCTRL_LISTCLICK,
	wxDefaultPosition, wxSize(300,200), wxLC_REPORT | wxLC_NO_HEADER)
{
	Parent = (cpcInOutCtrl *) parent;
	InsertColumn(0,"Available");
}

cpcOutListCtrl::cpcOutListCtrl(wxWindow * parent, long boxheight) : 
	wxListCtrl(parent,CPC_DLGCTRL_LISTCLICK,
	wxDefaultPosition, wxSize(300,boxheight), wxLC_REPORT | wxLC_NO_HEADER)
{
	Parent = (cpcInOutCtrl *) parent;
	InsertColumn(0,"Available");
}

cpcOutListCtrl::~cpcOutListCtrl(void)
{
}

void cpcOutListCtrl::OnSelected(wxListEvent& event)
{
	long i = event.GetIndex();
	long index = GetItemData(i);
	DeleteItem(i);
	Parent->PutinIn(index);
	Parent->ResetSubset(index);
}

BEGIN_EVENT_TABLE(cpcLogicAttrCheckBox,wxCheckBox)
EVT_CHECKBOX(-1,cpcLogicAttrCheckBox::OnClick)
END_EVENT_TABLE()

cpcLogicAttrCheckBox::cpcLogicAttrCheckBox(wxWindow *parent,
	wxString label) : wxCheckBox(parent,-1,label)
{
	Parent = (cpcInOutCtrl *) parent;
}

cpcLogicAttrCheckBox::~cpcLogicAttrCheckBox()
{
}

void cpcLogicAttrCheckBox::OnClick(wxCommandEvent& ev)
{
	Parent->ActiveLogicAttr[Index] = GetValue();
	Parent->ResetSubset();
}

IMPLEMENT_DYNAMIC_CLASS(cpcInOutCtrl,wxPanel)

BEGIN_EVENT_TABLE(cpcInOutCtrl,wxPanel)
END_EVENT_TABLE()

cpcInOutCtrl::cpcInOutCtrl(void)
{
}

cpcInOutCtrl::cpcInOutCtrl(wxWindow *parent, wxArrayString labels, 
		wxArrayInt checked, wxWindowID winid,
                const wxPoint &pos, const wxSize &size) 
{
	unsigned int i;
	cpcLogicAttrCheckBox *box;
	LogicAttrLabels = labels;
	wxPanel::Create(parent, winid, pos,size);
	wxBoxSizer *vsizer = new wxBoxSizer(wxVERTICAL);
	wxBoxSizer *hsizer = new wxBoxSizer(wxHORIZONTAL);
	Outs = new cpcOutListCtrl(this);
	hsizer->Add(Outs,1,wxEXPAND|wxALL,5);
	Ins = new cpcInListCtrl(this);
	hsizer->Add(Ins,1,wxEXPAND|wxALL,5);
	vsizer->Add(hsizer,1,wxEXPAND|wxALL,1);
	for(i=0;i<LogicAttrLabels.Count();i++)
	{
		box = new cpcLogicAttrCheckBox(this,LogicAttrLabels[i]);
		if(checked.Count() > i) 
		{
			ActiveLogicAttr.Add(checked[i]);
		}
		else
		{
			ActiveLogicAttr.Add(1);
		}
		box->SetValue(ActiveLogicAttr[i]);
		box->Index = i;
		vsizer->Add(box,0,wxALL,4);
	}
		
	SetSizer(vsizer);
	vsizer->SetSizeHints(this);
	MaxSelected = 0;
}

cpcInOutCtrl::cpcInOutCtrl(wxWindow *parent, wxArrayString labels, 
		wxArrayInt checked, 
		long fontsize, long border, long boxheight,
		wxWindowID winid,
                const wxPoint &pos, const wxSize &size) 
{
	unsigned int i;
	cpcLogicAttrCheckBox *box;
	LogicAttrLabels = labels;
	wxPanel::Create(parent, winid, pos,size);
	wxBoxSizer *vsizer = new wxBoxSizer(wxVERTICAL);
	wxBoxSizer *hsizer = new wxBoxSizer(wxHORIZONTAL);
	font = wxFont((int) fontsize,wxFONTFAMILY_SWISS,wxFONTSTYLE_NORMAL,wxFONTWEIGHT_NORMAL);
	Outs = new cpcOutListCtrl(this,boxheight);
	hsizer->Add(Outs,1,wxEXPAND|wxALL,border);
	Ins = new cpcInListCtrl(this,boxheight);
	hsizer->Add(Ins,1,wxEXPAND|wxALL,border);
	vsizer->Add(hsizer,1,wxEXPAND|wxALL,1);
	for(i=0;i<LogicAttrLabels.Count();i++)
	{
		box = new cpcLogicAttrCheckBox(this,LogicAttrLabels[i]);
		if(checked.Count() > i) 
		{
			ActiveLogicAttr.Add(checked[i]);
		}
		else
		{
			ActiveLogicAttr.Add(1);
		}
		box->SetValue(ActiveLogicAttr[i]);
		box->Index = i;
		vsizer->Add(box,0,wxALL,border);
	}
		
	SetSizer(vsizer);
	vsizer->SetSizeHints(this);
	MaxSelected = 0;
}

cpcInOutCtrl::~cpcInOutCtrl()
{
}

void cpcInOutCtrl::AddItem(cpcInOutItem& item)
{
	int index = Items.Count();
	item.Index = index;
	Items.Add(item);
}

void cpcInOutCtrl::ResetSubset(void)
{
	unsigned int i,j,count,subcount;
	bool use;
	wxListItem item;

	count = Items.Count();
	Ins->DeleteAllItems();
	Outs->DeleteAllItems();
	item.SetMask(wxLIST_MASK_STATE | wxLIST_MASK_TEXT);
	item.SetStateMask(wxLIST_STATE_SELECTED);
	for(i=0;i<count;i++)
	{
		use = TRUE;
		if(ActiveLogicAttr.Count() > 0)
		{
			use = FALSE;
			// one or more active logic attrs must be true
			// if attributes are missing, they are assumed to
			// be true
			for(j=0;j<ActiveLogicAttr.Count();j++)
			{
				if(!ActiveLogicAttr[j])
				{
					continue;
				}
				if(j >= Items[i].LogicAttr.Count())
				{
					use = TRUE;
					break;
				}
				if(Items[i].LogicAttr[j])
				{
					use = TRUE;
					break;
				}
			}
		}
		if(use)
		{
			subcount = Outs->GetItemCount();
			item.SetId(subcount);
			item.SetText(Items[i].Label);
			item.SetData(i);
			item.SetFont(font);
			item.SetBackgroundColour(*wxWHITE);
			Outs->InsertItem(item);
		}
	}
	Outs->SetColumnWidth(0,wxLIST_AUTOSIZE);
	Ins->SetColumnWidth(0,Outs->GetColumnWidth(0));
}


void cpcInOutCtrl::ResetSubset(int index)
{
// put into the Outs list all items in the current Outs list
// that match the dimension of Item[index] except Item[index] itself
	int i,count,matchdimen;
	wxListItem item;

	matchdimen = Items[index].Dimension;

	count = Outs->GetItemCount();
	for(i=count-1;i>=0;i--)
	{
		if(Items[Outs->GetItemData(i)].Dimension != matchdimen)
		{
			Outs->DeleteItem(i);
		}
	}
}

void cpcInOutCtrl::PutinOut(long index)
{
	int i, subcount;
	wxListItem item;

	subcount = Outs->GetItemCount();
	item.SetText(Items[index].Label);
	item.SetData(index);
	item.SetFont(font);
	item.SetBackgroundColour(*wxWHITE);
	for(i=0;i<subcount;i++)
	{
		if((unsigned) index < Outs->GetItemData(i))
		{
			item.SetId(i);
			Outs->InsertItem(item);
			Outs->SetColumnWidth(0,wxLIST_AUTOSIZE);
			return;
		}
	}
	item.SetId(index);
	Outs->InsertItem(item);
	Outs->SetColumnWidth(0,wxLIST_AUTOSIZE);
}

void cpcInOutCtrl::PutinIn(long index)
{
	long subcount,ind;
	wxListItem item;

	subcount = Ins->GetItemCount();

	item.SetId(subcount);
	item.SetText(Items[index].Label);
	item.SetData(index);
	item.SetFont(font);
	item.SetBackgroundColour(*wxWHITE);
	Ins->InsertItem(item);

	if(MaxSelected > 0 && Ins->GetItemCount() > MaxSelected) 
	{
		ind = Ins->GetItemData(0);
		Ins->DeleteItem(0);
		PutinOut(ind);
	}
	Ins->SetColumnWidth(0,wxLIST_AUTOSIZE);
}

static wxString selectionString;

wxString cpcInOutCtrl::GetSelection(void)
{
	int i;
	selectionString.Empty();
	for(i=0;i<Ins->GetItemCount();i++)
	{
		selectionString << Ins->GetItemData(i) << " ";
	}
	return(selectionString);
}
