/*
*(C)* The file is part of the source distribution of Carapace
*(C)* version 0.5 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)* Some parts Copyright (c) 2004 Huixin Wang
*(C)*
*(C)* This file is distributed under the GNU General Public License
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

#include "cpcplatform.h"



// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include "wx/wx.h"
#include "wx/print.h"
#include "wx/splash.h"
#if wxCHECK_VERSION(2,4,0)
#include "wx/imagpng.h"
#include "wx/imaggif.h"
#include "wx/imagjpeg.h"
#include "wx/imagxpm.h"
#endif


#include <math.h>

#include "wx/clipbrd.h"
#include "wx/dataobj.h"

#include "carapace.h"

#undef UNDEFINED

extern CarapaceApp     *thisApp;
wxString CarapaceClipboard;
wxString CarapaceSelection;
wxString CarapaceHistory;


BEGIN_EVENT_TABLE(cpcSplashScreen, wxSplashScreen)
    EVT_CLOSE(cpcSplashScreen::OnWindowClose)
END_EVENT_TABLE()


extern "C" 
{

int cpc_doio(cpc_io_ptr iop)
{

	wxTextDataObject clipData;
	if(thisApp->Uninitialized)
	{
		return(CPC_ERROR_UNINITIALIZED);
	}
	if(iop == (void *) 0) // null pointer ?
	{
		return(CPC_ERROR_NULLPOINTER);
	}


	switch(iop->type)
	{

		case CPC_IO_NEWWINDOW:
			if(iop->string != (char *) 0 && iop->string2 != (char *) 0)
			{
				wxString res = iop->string;
				wxString name = iop->string2;
				iop->parm = thisApp->NewSplitTextFrame(TRUE,res,name);
			}
			else
			{
				iop->parm = thisApp->NewSplitTextFrame(TRUE);
			}
			return(CPC_NOERROR);
	
		case CPC_IO_SWALLOWOUTPUT:
			thisApp->SwallowOutput = iop->parm;
			thisApp->OldSwallowOutput = iop->parm;
			return(CPC_NOERROR);
			

		case CPC_IO_BUFFEREDPRINT:
			if(iop->string == (char *) 0) 
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			if(iop->parm == CPC_SWALLOW_FORCEIT ||
				!(thisApp->SwallowOutput & CPC_SWALLOW_PRINTOUT))
			{
				thisApp->BufferedPrint(iop->string);
			}
			return(CPC_NOERROR);

		case CPC_IO_FLUSHBUFFER:
			thisApp->FlushBuffer();
			return(CPC_NOERROR);

		case CPC_IO_PRIMARYSELECTION:
			if(iop->length)
			{
				wxTheClipboard->UsePrimarySelection(TRUE);
			}
			else
			{
				wxTheClipboard->UsePrimarySelection(FALSE);
			}
			return(CPC_NOERROR);

		case CPC_IO_GETCLIPTEXTLENGTH:
			if (wxTheClipboard->Open())
			{
				if (wxTheClipboard->IsSupported( wxDF_TEXT ))
				{
					wxTheClipboard->GetData( clipData );
				}
				wxTheClipboard->Close();
			}
 
			CarapaceClipboard = clipData.GetText();
			iop->length = CarapaceClipboard.Length() + 1; 
			return(CPC_NOERROR);

		case CPC_IO_GETCLIPTEXT:
			if (wxTheClipboard->Open())
			{
				if (wxTheClipboard->IsSupported( wxDF_TEXT ))
				{
					wxTheClipboard->GetData( clipData );
				}
				wxTheClipboard->Close();
			}
 
			CarapaceClipboard = clipData.GetText();
			if(iop->string == (char *) 0) 
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			if( (unsigned long) iop->length < CarapaceClipboard.Length() + 1) 
			{
				iop->length = CarapaceClipboard.Length() + 1;
				return(CPC_ERROR_STRINGTOOSHORT);
			}
			strcpy( iop->string, CarapaceClipboard.c_str());
			return(CPC_NOERROR);

		case CPC_IO_SETCLIPTEXT:
			if(iop->string == (char *) 0) 
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			CarapaceClipboard = iop->string;
			if (wxTheClipboard->Open())
			{
				// Data objects are held by the clipboard,
				// so do not delete them in the app.
				wxTheClipboard->SetData( new wxTextDataObject(CarapaceClipboard) );
				wxTheClipboard->Close();
				return(CPC_NOERROR);
			}
			return(CPC_ERROR_DIDNOTSETCLIP);

		case CPC_IO_ALERT:
			if(iop->string == (char *) 0) 
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			CarapaceAlert(iop->string);
			return(CPC_NOERROR);

		case CPC_IO_SPLASH:
		{
			if(iop->string == (char *) 0) 
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			wxBitmap bitmap;
			long bittype;
			if(iop->parm == CPC_IO_GIFTYPE)
			{
				bittype = wxBITMAP_TYPE_GIF;
				wxImage::AddHandler(new wxGIFHandler);
			}
			else if(iop->parm == CPC_IO_JPEGTYPE)
			{
				bittype = wxBITMAP_TYPE_JPEG;
				wxImage::AddHandler(new wxJPEGHandler);
			}
			else if(iop->parm == CPC_IO_XPMTYPE)
			{
				bittype = wxBITMAP_TYPE_XPM;
				wxImage::AddHandler(new wxXPMHandler);
			}
			else if(iop->parm == CPC_IO_PNGTYPE)
			{
				bittype = wxBITMAP_TYPE_PNG;
				wxImage::AddHandler(new wxPNGHandler);
			}
			else 
			{
				return(CPC_ERROR_UNSUPPORTEDBITMAPTYPE);
			}
			if (bitmap.LoadFile(iop->string, (wxBitmapType) bittype))
			{
				thisApp->Splash = new cpcSplashScreen(bitmap,
				wxSPLASH_CENTRE_ON_SCREEN|wxSPLASH_TIMEOUT,
				6000, NULL, -1, wxDefaultPosition, wxDefaultSize,
				wxSIMPLE_BORDER|wxSTAY_ON_TOP);
				thisApp->SplashShown = TRUE;
				//while(thisApp->SplashShown)
				//{
				//	wxYield();
				//}
			}

			return(CPC_NOERROR);
		}

		case CPC_IO_PUSHCOMMAND:
			if(iop->string == (char *) 0) 
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			if(iop->string[0] != '\0')
			{
				wxCommandEvent procevent(wxEVT_COMMAND_MENU_SELECTED,CPC_GUI_PROCESSCOMMAND);
				CarapaceCommand *cptr = new CarapaceCommand;
				cptr->command = iop->string;
				if(cptr->command.Last() != '\n')
				{
					cptr->command << "\n";
				}
				if(thisApp->Running == CPC_RUNNING)
				{
					cptr->frame = thisApp->RunningFrame;
				}
				else
				{
					// this associates the command with the first text frame
					cptr->frame = thisApp->STFrameList.GetFirst()->GetData()->TextFrameIndex;
				}
				cptr->hidecommand = iop->length;
				cptr->hideoutput = iop->parm;
				procevent.SetClientData((void *) cptr);
				wxPostEvent(thisApp,procevent);
			}
			return(CPC_NOERROR);


		case CPC_IO_SETPROMPT:
			if(iop->string == (char *) 0) 
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			thisApp->Preferences->mainPrompt = iop->string;
			return(CPC_NOERROR);

		case CPC_IO_BEEP:
			CarapaceBeep(iop->length);
			return(CPC_NOERROR);

		case CPC_IO_SETCMDWINDFROMFILE:
		{
			if(iop->string == (char *) 0) 
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			wxString fname(iop->string);
			thisApp->SetTextFromFile(fname, iop->parm);
			return(CPC_NOERROR);
		}

		case CPC_IO_SHOWCMDWINDOW:
		{
			thisApp->TextFrameFront(thisApp->RunningFrame);
			return(CPC_NOERROR);
		}

		case CPC_IO_USEA4PAPER:
		{
			CarapaceUseA4Paper();
			return(CPC_NOERROR);
		}

		case CPC_IO_PARENMATCH:
		{
			CarapaceSplitTextFrame * frame = (CarapaceSplitTextFrame *) iop->string;
			if(!thisApp->STFrameList.Find(frame))
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			frame->command_ctrl->AddMatch(iop->length,iop->parm);
			return(CPC_NOERROR);
		}

		case CPC_IO_SYNTAXERROR:
		{
			CarapaceSplitTextFrame * frame = (CarapaceSplitTextFrame *) iop->string;
			if(!thisApp->STFrameList.Find(frame))
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			// FIXME frame->SyntaxAlert->SetBitmap(frame->WarningBitmap);
			frame->SErrorPlace = iop->parm;
			return(CPC_NOERROR);
			
		}

		case CPC_IO_FILENAMEFROMPATH:
		{
			if(iop->string == (char *) 0) 
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			strcpy(iop->string,wxFileNameFromPath(iop->string));
			return(CPC_NOERROR);
		}

		case CPC_IO_PATHONLY:
		{
			if(iop->string == (char *) 0) 
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			strcpy(iop->string,wxPathOnly(iop->string).c_str());
			return(CPC_NOERROR);
		}

		case CPC_IO_INTERRUPTED:
		{
			wxYield();
			iop->parm = CPC_INTERRUPT;
			return(CPC_NOERROR);
		}


		default:
			;
	}
	return(CPC_ERROR_BADCODE);  // only get here if incorrect code
}

int cpc_TestForInterrupt(void)
{
	thisApp->Yield();
	return(CPC_INTERRUPT);
}
 
 
int cpc_setCallback(cpc_callback_ptr cbp)
{
	if(cbp == (void *) 0) // null pointer ?
	{
		return(CPC_ERROR_NULLPOINTER);
	}
	if(cbp->callback.inputReady == (void *) 0) // null pointer?
	{
		return(CPC_ERROR_INTERNALNULLPOINTER);
	}
	switch(cbp->type)
	{

		case CPC_CB_INPUTREADY:
			thisApp->InputReady = (bool (*)(const char *, void *, int))
				cbp->callback.inputReady;
			return(CPC_NOERROR);

		case CPC_CB_PROCESSCOMMAND:
			thisApp->ProcessCommand = cbp->callback.processCommand;
			return(CPC_NOERROR);

		case CPC_CB_CLEANUP:
			thisApp->CleanUp = cbp->callback.cleanUp;
			return(CPC_NOERROR);

		case CPC_CB_INTERRUPTCLEANUP:
			thisApp->InterruptCleanUp = cbp->callback.interruptCleanUp;
			return(CPC_NOERROR);

		case CPC_CB_QUITCLEANUP:
			thisApp->QuitCleanUp = cbp->callback.quitCleanUp;
			return(CPC_NOERROR);

		case CPC_CB_INITIALIZER:
			thisApp->Initializer = cbp->callback.initializer;
			return(CPC_NOERROR);

		case CPC_CB_WINDOWCLOSED:
			thisApp->WindowClosed = cbp->callback.windowClosed;
			return(CPC_NOERROR);

		case CPC_CB_SETPREFSFILEPATH:
			thisApp->PreferencesPath = cbp->callback.string;
			if(thisApp->Preferences)
			{
				thisApp->Preferences->SetPreferences();
			}
			return(CPC_NOERROR);

		default:
			;
	}
	return(CPC_ERROR_BADCODE);  // only get here if incorrect code
}
 

int cpc_usermenus(cpc_io_ptr iop)
{
	wxString name;
	CarapaceSplitTextFrame *stfp;
	wxCarapaceSplitTextFrameListNode *stnode;

	if(thisApp->Uninitialized)
	{
		return(CPC_ERROR_UNINITIALIZED);
	}
	if(iop == (void *) 0) // null pointer ?
	{
		return(CPC_ERROR_NULLPOINTER);
	}


	switch(iop->type)
	{
		case CPC_MENU_SETUPUSERMENUS:
			if(iop->string == (char *) 0 || iop->string2 == (char *) 0)
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			if(iop->parm < 0)
			{
				iop->parm = thisApp->RunningFrame;
			}
			stnode = thisApp->STFrameList.Find(iop->parm);
			if(!stnode) 
			{
				return(CPC_ERROR_BADWINDOW);
			}
			stfp = stnode->GetData();
			stfp->SetMenuBarFromString(iop->string,iop->string2);

			return(CPC_NOERROR);
	
		case CPC_MENU_GETMENURESOURCELENGTH:

			if(iop->parm < 0)
			{
				iop->parm = thisApp->RunningFrame;
			}
			stnode = thisApp->STFrameList.Find(iop->parm);
			if(!stnode) 
			{
				return(CPC_ERROR_BADWINDOW);
			}
			stfp = stnode->GetData();
			iop->length = stfp->ResourceString.Length();

			return(CPC_NOERROR);
			
		case CPC_MENU_GETMENURESOURCE:

			if(iop->string == (char *) 0)
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			if(iop->parm < 0)
			{
				iop->parm = thisApp->RunningFrame;
			}
			stnode = thisApp->STFrameList.Find(iop->parm);
			if(!stnode) 
			{
				return(CPC_ERROR_BADWINDOW);
			}
			stfp = stnode->GetData();
			if(iop->length < (long) stfp->ResourceString.Length() + 1)
			{
				return(CPC_ERROR_STRINGTOOSHORT);
			}
			sprintf(iop->string,"%s",stfp->ResourceString.c_str());

			return(CPC_NOERROR);
			
		default:
			;
	}
	return(CPC_ERROR_BADCODE);  // only get here if incorrect code
}	


/* this code originally done by Huixin Wang */
// this code modified by GWO
int cpc_doDialogs(cpc_io_ptr iop)
{
	wxString name;
	CarapaceModalDialog *cmdp;
	CarapaceNonModalFrame *cnmfp;
	wxCommandEvent time;
	long i,length;
	char *thisstring;


	if(thisApp->Uninitialized)
	{
		return(CPC_ERROR_UNINITIALIZED);
	}
	if(iop == (void *) 0) // null pointer ?
	{
		return(CPC_ERROR_NULLPOINTER);
	}

	switch(iop->type)
	{
		case CPC_DYN_SETUPDIALOG:
		{
			if(iop->string == (char *) 0 || iop->string2 == (char *) 0)
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			if(iop->parm < 0)
			{
				iop->parm = thisApp->RunningFrame;
			}
			wxString res = iop->string2;
			wxString dname = iop->string;
			return(thisApp->NewDialogFrame(res,dname));
			return(CPC_NOERROR);
		}

		case CPC_MDLG_SETUPUSERMDLGS:
			if(iop->string == (char *) 0 || iop->string2 == (char *) 0)
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			if(iop->parm < 0)
			{
				iop->parm = thisApp->RunningFrame;
			}
			cmdp=new CarapaceModalDialog();
			//cmdp->SetModalDLGFromString(NULL,iop->string /*DialogName*/,
			cmdp->SetModalDLGFromString(thisApp->OutputWindow,iop->string /*DialogName*/,
				iop->string2 /*ResourceString*/);
			return(thisApp->LastModalDialogCount);

		case CPC_MDLG_GETLASTMDLGOUTPUTLENGTH:
			iop->type = thisApp->LastModalDialogCount;
			length=0;
			for (i=0;i<thisApp->LastModalDialogCount;i++)
			{
				length+=thisApp->LastModalDialogNames[i].Length();
			}
			iop->length = length;
			length=0;
			for (i=0;i<thisApp->LastModalDialogCount;i++)
			{
				length+=thisApp->LastModalDialogValues[i].Length();
			}
			iop->parm = length;

			return(CPC_NOERROR);
			
		case CPC_MDLG_GETLASTMDLGOUTPUT:
			if(iop->string == (char *) 0)
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			thisstring=iop->string;
			for (i=0;i<thisApp->LastModalDialogCount;i++)
			{
				sprintf(thisstring,"%s\0",thisApp->LastModalDialogNames[i].c_str());
				thisstring+=(thisApp->LastModalDialogNames[i].Length()+1);
			}
			
			for (i=0;i<thisApp->LastModalDialogCount;i++)
			{	sprintf(thisstring,"%s\0",thisApp->LastModalDialogValues[i].c_str());
				thisstring+=(thisApp->LastModalDialogValues[i].Length()+1);
			}
			

			return(CPC_NOERROR);
			
		default:
			;
	}
	return(CPC_ERROR_BADCODE);  // only get here if incorrect code
}
/* end Huixin Wang */


//The following was written on Feb 26,2005
int cpc_doDyc(cpc_dyn_ptr dynp)
{
	//vector<int> dtobjcontentInt;
	int *dtobjcontentInt;
	bool dodycsucc = FALSE;
	//double *dtobjcontentdouble;
	//bool   *dtobjcontentInt;
	//vector<wxString> ViewValueVector;
	int DtObjRowN, DtObjColN,Dtlength,i;
	wxString keystr;
	//char* *strp;
	
	if(thisApp->Uninitialized)
	{
		return(CPC_ERROR_UNINITIALIZED);
	}
	
	if(!dynp) // null pointer ?
	{
		return(CPC_ERROR_NULLPOINTER);
	}
	wxString keystring(dynp->name);
	keystr = wxString(dynp->DbjParmValue_ptr->name);
	switch(dynp->type)
	{
		
		case CPC_DYN_ADDDATAOBJECT:
// OK, there is something very peculiar here.  If the following print
// is included, then an int data object is not persistent.  However, if
// it is not included, then the object is persistent
//cout << "in adddataobject interface\n";
			if(dynp->name == (char *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			DtObjRowN = dynp->DbjParmValue_ptr->RowsNumber;				
			DtObjColN = dynp->DbjParmValue_ptr->ColumnNumber;
			Dtlength = DtObjRowN*DtObjColN;
			if (dynp->DbjParmValue_ptr->DataType == INTT) 
				dodycsucc = thisApp->DynamicDataP->addDataObject(
					keystr, DtObjRowN, DtObjColN, 
					dynp->DbjParmValue_ptr->IntContentVector);
			if (dynp->DbjParmValue_ptr->DataType == DOULBET) 
				dodycsucc = thisApp->DynamicDataP->addDataObject(
					keystr, DtObjRowN, DtObjColN, 
					dynp->DbjParmValue_ptr->DoubleContentVector);
			if (dynp->DbjParmValue_ptr->DataType == STRINGT) 
				dodycsucc = thisApp->DynamicDataP->addDataObject(
					keystr, DtObjRowN, DtObjColN, 
					dynp->DbjParmValue_ptr->StrContentVector);
			if (dynp->DbjParmValue_ptr->DataType == LOGICT) 
				dodycsucc = thisApp->DynamicDataP->addDataObject(
					keystr, DtObjRowN, DtObjColN, 
					dynp->DbjParmValue_ptr->BoolContentVector);
			if (dodycsucc)	
				return(CPC_NOERROR);
			else
				return(CPC_ERROR_ADDDATAOBJCTFAILURE);
			
		case CPC_DYN_MAKEPERSISTENTDATAOBJECT:
			if(dynp->name == (char *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}

			if (thisApp->DynamicDataP->MakeDataObjectPersistent(keystr))		
				return(CPC_NOERROR);
			else
				return(CPC_ERROR_MAKEPERSISTENTFAILURE);	
				
		case CPC_DYN_DELETEDATAOBJECT:
			if(dynp->name == (char *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}

			if (thisApp->DynamicDataP->deleteDataObject(keystr))		
				return(CPC_NOERROR);
			else
				return(CPC_ERROR_DELETEDATAOBJECTFAILURE);
			
		case CPC_DYN_GETDATAOBJECTVALUE:
			if(dynp->name == (char *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}

			if (dynp->DbjParmValue_ptr->DataType == INTT) 
				thisApp->DynamicDataP->getDataObjectContent(keystr, DtObjRowN, DtObjColN, dynp->DbjParmValue_ptr->IntContentVector);
			if (dynp->DbjParmValue_ptr->DataType == DOULBET) 
				thisApp->DynamicDataP->getDataObjectContent(keystr, DtObjRowN, DtObjColN, dynp->DbjParmValue_ptr->DoubleContentVector);			
			if (dynp->DbjParmValue_ptr->DataType == STRINGT) 
				thisApp->DynamicDataP->getDataObjectContent(keystr, DtObjRowN, DtObjColN, dynp->DbjParmValue_ptr->StrContentVector);			
			if (dynp->DbjParmValue_ptr->DataType == LOGICT) 
				//thisApp->DynamicDataP->getDataObjectContent(keystr, DtObjRowN, DtObjColN, dynp->DbjParmValue_ptr->IntContentVector);	
				thisApp->DynamicDataP->getDataObjectContent(keystr, DtObjRowN, DtObjColN, dynp->DbjParmValue_ptr->BoolContentVector);
			dynp->DbjParmValue_ptr->RowsNumber = DtObjRowN;
			dynp->DbjParmValue_ptr->ColumnNumber = DtObjColN;
			return(CPC_NOERROR);												

		case CPC_DYN_SETDATAOBJECTVALUE:
			if(dynp->name == (char *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			DtObjRowN = dynp->DbjParmValue_ptr->RowsNumber;				
			DtObjColN = dynp->DbjParmValue_ptr->ColumnNumber;
			Dtlength = DtObjRowN*DtObjColN;
			if (dynp->DbjParmValue_ptr->DataType == INTT) 
				dodycsucc = thisApp->DynamicDataP->setDataObjectValue(dynp->DbjParmValue_ptr->IntContentVector, DtObjRowN, DtObjColN, keystr);
			if (dynp->DbjParmValue_ptr->DataType == DOULBET) 
				dodycsucc = thisApp->DynamicDataP->setDataObjectValue(dynp->DbjParmValue_ptr->DoubleContentVector, DtObjRowN, DtObjColN, keystr);		
			if (dynp->DbjParmValue_ptr->DataType == STRINGT) 
				dodycsucc = thisApp->DynamicDataP->setDataObjectValue(dynp->DbjParmValue_ptr->StrContentVector, DtObjRowN, DtObjColN, keystr);			
			if (dynp->DbjParmValue_ptr->DataType == LOGICT) 
				//dodycsucc = thisApp->DynamicDataP->setDataObjectValue(dynp->DbjParmValue_ptr->IntContentVector, DtObjRowN, DtObjColN, keystr);	
				dodycsucc = thisApp->DynamicDataP->setDataObjectValue(dynp->DbjParmValue_ptr->BoolContentVector, DtObjRowN, DtObjColN, keystr);								
			if (dodycsucc)	
				return(CPC_NOERROR);
			else
				return(CPC_ERROR_SETDATAOBJECTFAILURE);			
			/*
			if (dynp->DbjParmValue_ptr->DataType == INTT) {
				DtObjRowN = dynp->DbjParmValue_ptr->RowsNumber;
				DtObjColN = dynp->DbjParmValue_ptr->ColumnNumber;
				Dtlength = DtObjRowN*DtObjColN;
				dtobjcontentInt = dynp->DbjParmValue_ptr->IntContentVector;
	
				if (thisApp->DynamicDataP->setDataObjectValue(dtobjcontentInt, DtObjRowN, DtObjColN, keystr))	
					return(CPC_NOERROR);
				else
				  return(CPC_ERROR_SETDATAOBJECTFAILURE);
			}
		  */
		case CPC_DYN_ADDVIEWER:
			return(CPC_NOERROR);
#if 0
			if(dynp->name == (char *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			
			DtObjRowN = dynp->DbjParmValue_ptr->RowsNumber;				
			DtObjColN = dynp->DbjParmValue_ptr->ColumnNumber;
			Dtlength = DtObjRowN*DtObjColN;
			if (dynp->DbjParmValue_ptr->DataType == INTT) 
				dodycsucc = thisApp->DynamicDataP->addViewer(keystr, dynp->DbjParmValue_ptr->IntContentVector, DtObjRowN, DtObjColN);
			if (dynp->DbjParmValue_ptr->DataType == DOULBET) 
				dodycsucc = thisApp->DynamicDataP->addViewer(keystr, dynp->DbjParmValue_ptr->DoubleContentVector, DtObjRowN, DtObjColN);			
			if (dynp->DbjParmValue_ptr->DataType == STRINGT) 
				dodycsucc = thisApp->DynamicDataP->addViewer(keystr, dynp->DbjParmValue_ptr->StrContentVector, DtObjRowN, DtObjColN);			
			if (dynp->DbjParmValue_ptr->DataType == LOGICT) 
				//dodycsucc = thisApp->DynamicDataP->addViewer(keystr, dynp->DbjParmValue_ptr->IntContentVector, DtObjRowN, DtObjColN);	
				dodycsucc = thisApp->DynamicDataP->addViewer(keystr, dynp->DbjParmValue_ptr->BoolContentVector, DtObjRowN, DtObjColN);									
			if (dodycsucc)	
				return(CPC_NOERROR);
			else
				return(CPC_ERROR_ADDVIEWERFAILURE);		
				/*	
			if (dynp->DbjParmValue_ptr->DataType == INTT) {
				DtObjRowN = dynp->DbjParmValue_ptr->RowsNumber;
				DtObjColN = dynp->DbjParmValue_ptr->ColumnNumber;
				Dtlength = DtObjRowN*DtObjColN;
				dtobjcontentInt = dynp->DbjParmValue_ptr->IntContentVector;				
				if (thisApp->DynamicDataP->addViewer(keystr, dtobjcontentInt, DtObjRowN, DtObjColN))	
					return(CPC_NOERROR);
				else
					return(CPC_ERROR_ADDVIEWERFAILURE);
			}	*/	
			
		case CPC_DYN_SETVIEWERDATA:
			if(dynp->name == (char *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}

			Dtlength = dynp->DbjParmValue_ptr->StrArraySize;
			if (thisApp->DynamicDataP->setViewerData(dynp->DbjParmValue_ptr->StrArray, Dtlength, keystring))
				return(CPC_NOERROR);
			else
				return(CPC_ERROR_SETVIEWERDATAFAILURE);				
#endif //0
		
		case CPC_DYN_GETVIEWERSTATE:
			if(dynp->name == (char *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			if (dynp->DbjParmValue_ptr->DataType == INTT) 
				thisApp->DynamicDataP->getViewerStateContent(keystr, DtObjRowN, DtObjColN, dynp->DbjParmValue_ptr->IntContentVector);
			if (dynp->DbjParmValue_ptr->DataType == DOULBET) 
				thisApp->DynamicDataP->getViewerStateContent(keystr, DtObjRowN, DtObjColN, dynp->DbjParmValue_ptr->DoubleContentVector);			
			if (dynp->DbjParmValue_ptr->DataType == STRINGT) 
				thisApp->DynamicDataP->getViewerStateContent(keystr, DtObjRowN, DtObjColN, dynp->DbjParmValue_ptr->StrContentVector);			
			if (dynp->DbjParmValue_ptr->DataType == LOGICT) 			
				//thisApp->DynamicDataP->getViewerStateContent(keystr, DtObjRowN, DtObjColN, dynp->DbjParmValue_ptr->IntContentVector);	
				thisApp->DynamicDataP->getViewerStateContent(keystr, DtObjRowN, DtObjColN, dynp->DbjParmValue_ptr->BoolContentVector);	
			dynp->DbjParmValue_ptr->RowsNumber = DtObjRowN;
			dynp->DbjParmValue_ptr->ColumnNumber = DtObjColN;
			return(CPC_NOERROR);			
			/*
			if (dynp->DbjParmValue_ptr->DataType == INTT) {
				dtobjcontentInt = dynp->DbjParmValue_ptr->IntContentVector;				
				thisApp->DynamicDataP->getViewerStateContent(keystr, DtObjRowN, DtObjColN, dtobjcontentInt);
				dynp->DbjParmValue_ptr->RowsNumber = DtObjRowN;
				dynp->DbjParmValue_ptr->ColumnNumber = DtObjColN;
				return(CPC_NOERROR);			
			}	*/													
		
		case CPC_DYN_SETVIEWERSTATE:
			if(dynp->name == (char *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			DtObjRowN = dynp->DbjParmValue_ptr->RowsNumber;				
			DtObjColN = dynp->DbjParmValue_ptr->ColumnNumber;
			Dtlength = DtObjRowN*DtObjColN;
			if (dynp->DbjParmValue_ptr->DataType == INTT) 
				dodycsucc = thisApp->DynamicDataP->setViewerState(keystr, dynp->DbjParmValue_ptr->IntContentVector, DtObjRowN, DtObjColN);
			if (dynp->DbjParmValue_ptr->DataType == DOULBET) 
				dodycsucc = thisApp->DynamicDataP->setViewerState(keystr, dynp->DbjParmValue_ptr->DoubleContentVector, DtObjRowN, DtObjColN);		
			if (dynp->DbjParmValue_ptr->DataType == STRINGT) 
				dodycsucc = thisApp->DynamicDataP->setViewerState(keystr, dynp->DbjParmValue_ptr->StrContentVector, DtObjRowN, DtObjColN);			
			if (dynp->DbjParmValue_ptr->DataType == LOGICT) 
				//dodycsucc = thisApp->DynamicDataP->setViewerState(keystr, dynp->DbjParmValue_ptr->IntContentVector, DtObjRowN, DtObjColN);	
				dodycsucc = thisApp->DynamicDataP->setViewerState(keystr, dynp->DbjParmValue_ptr->BoolContentVector, DtObjRowN, DtObjColN);								
			if (dodycsucc)	
				return(CPC_NOERROR);
			else
				return(CPC_ERROR_SETVIEWERSTATEFAILURE);		
			
			/*
			if (dynp->DbjParmValue_ptr->DataType == INTT) {
				DtObjRowN = dynp->DbjParmValue_ptr->RowsNumber;
				DtObjColN = dynp->DbjParmValue_ptr->ColumnNumber;
				Dtlength = DtObjRowN*DtObjColN;
				dtobjcontentInt = dynp->DbjParmValue_ptr->IntContentVector;
				if (thisApp->DynamicDataP->setViewerState(keystr, dtobjcontentInt, DtObjRowN, DtObjColN))	
					return(CPC_NOERROR);
				else
				  return(CPC_ERROR_SETVIEWERSTATEFAILURE);
			}*/					
					
		case CPC_DYN_DELETEVIEWER:
			if(dynp->name == (char *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			if (thisApp->DynamicDataP->deleteViewer(keystring))		
				return(CPC_NOERROR);
			else
			  return(CPC_ERROR_DELETEVIEWERFAILURE);
		
#if 0
		case CPC_DYN_REDRAW:
			if(dynp->name == (char *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			if (thisApp->DynamicDataP->redraw(keystr))		
				return(CPC_NOERROR);		
			else
				return(CPC_ERROR_REDRAWFAILURE);	
#endif
				
		case CPC_DYN_INSERTDATAOBJECTROW:
			if(dynp->DbjParmValue_ptr->name == (char *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}	
			DtObjRowN = dynp->DbjParmValue_ptr->RowsNumber;		
			if (dynp->DbjParmValue_ptr->DataType == INTT) 
				dodycsucc = thisApp->DynamicDataP->InsertDataObjectRow(keystr, dynp->DbjParmValue_ptr->IntContentVector, DtObjRowN);
			if (dynp->DbjParmValue_ptr->DataType == DOULBET) 
				dodycsucc = thisApp->DynamicDataP->InsertDataObjectRow(keystr, dynp->DbjParmValue_ptr->DoubleContentVector, DtObjRowN);			
			if (dynp->DbjParmValue_ptr->DataType == STRINGT) 
				dodycsucc = thisApp->DynamicDataP->InsertDataObjectRow(keystr, dynp->DbjParmValue_ptr->StrContentVector, DtObjRowN);			
			if (dynp->DbjParmValue_ptr->DataType == LOGICT) 
				//dodycsucc = thisApp->DynamicDataP->InsertDataObjectRow(keystr, dynp->DbjParmValue_ptr->IntContentVector, DtObjRowN);		
				dodycsucc = thisApp->DynamicDataP->InsertDataObjectRow(keystr, dynp->DbjParmValue_ptr->BoolContentVector, DtObjRowN);								
			if (dodycsucc)	
				return(CPC_NOERROR);
			else
				return(CPC_ERROR_INSERTROWFAILURE);			

		case CPC_DYN_APPENDDATAOBJECTROW:
			if(dynp->DbjParmValue_ptr->name == (char *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}	
			if (dynp->DbjParmValue_ptr->DataType == INTT) 
				dodycsucc = thisApp->DynamicDataP->AppendDataObjectRow(keystr, dynp->DbjParmValue_ptr->IntContentVector);
			if (dynp->DbjParmValue_ptr->DataType == DOULBET) 
				dodycsucc = thisApp->DynamicDataP->AppendDataObjectRow(keystr, dynp->DbjParmValue_ptr->DoubleContentVector);			
			if (dynp->DbjParmValue_ptr->DataType == STRINGT) 
				dodycsucc = thisApp->DynamicDataP->AppendDataObjectRow(keystr, dynp->DbjParmValue_ptr->StrContentVector);			
			if (dynp->DbjParmValue_ptr->DataType == LOGICT) 
				//dodycsucc = thisApp->DynamicDataP->AppendDataObjectRow(keystr, dynp->DbjParmValue_ptr->IntContentVector);	
				dodycsucc = thisApp->DynamicDataP->AppendDataObjectRow(keystr, dynp->DbjParmValue_ptr->BoolContentVector);									
			if (dodycsucc)	
				return(CPC_NOERROR);
			else
				return(CPC_ERROR_APPENDROWFAILURE);
				
		case CPC_DYN_CHANGEDATAOBJECTROW:
			if(dynp->DbjParmValue_ptr->name == (char *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}	
			DtObjRowN = dynp->DbjParmValue_ptr->RowsNumber;		
			if (dynp->DbjParmValue_ptr->DataType == INTT) 
				dodycsucc = thisApp->DynamicDataP->ChangeDataObjectRow(keystr, dynp->DbjParmValue_ptr->IntContentVector, DtObjRowN);
			if (dynp->DbjParmValue_ptr->DataType == DOULBET) 
				dodycsucc = thisApp->DynamicDataP->ChangeDataObjectRow(keystr, dynp->DbjParmValue_ptr->DoubleContentVector, DtObjRowN);			
			if (dynp->DbjParmValue_ptr->DataType == STRINGT) 
				dodycsucc = thisApp->DynamicDataP->ChangeDataObjectRow(keystr, dynp->DbjParmValue_ptr->StrContentVector, DtObjRowN);			
			if (dynp->DbjParmValue_ptr->DataType == LOGICT) 
			//	dodycsucc = thisApp->DynamicDataP->ChangeDataObjectRow(keystr, dynp->DbjParmValue_ptr->IntContentVector, DtObjRowN);	
				dodycsucc = thisApp->DynamicDataP->ChangeDataObjectRow(keystr, dynp->DbjParmValue_ptr->BoolContentVector, DtObjRowN);									
			if (dodycsucc)	
				return(CPC_NOERROR);
			else
				return(CPC_ERROR_CHANGEROWFAILURE);		
				
		case CPC_DYN_DELETEDATAOBJECTROW:
			if(dynp->DbjParmValue_ptr->name == (char *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}	
			DtObjRowN = dynp->DbjParmValue_ptr->RowsNumber;		
			if (dynp->DbjParmValue_ptr->DataType == INTT) 
				dodycsucc = thisApp->DynamicDataP->DeleteDataObjectRow(keystr, DtObjRowN);
			if (dynp->DbjParmValue_ptr->DataType == DOULBET) 
				dodycsucc = thisApp->DynamicDataP->DeleteDataObjectRow(keystr, DtObjRowN);			
			if (dynp->DbjParmValue_ptr->DataType == STRINGT) 
				dodycsucc = thisApp->DynamicDataP->DeleteDataObjectRow(keystr, DtObjRowN);			
			if (dynp->DbjParmValue_ptr->DataType == LOGICT) 
				dodycsucc = thisApp->DynamicDataP->DeleteDataObjectRow(keystr, DtObjRowN);										
			if (dodycsucc)	
				return(CPC_NOERROR);
			else
				return(CPC_ERROR_DELETEROWFAILURE);	
				
		case CPC_DYN_INSERTDATAOBJECTCOL:
			if(dynp->DbjParmValue_ptr->name == (char *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}	
			DtObjColN = dynp->DbjParmValue_ptr->ColumnNumber;		
			if (dynp->DbjParmValue_ptr->DataType == INTT) 
				dodycsucc = thisApp->DynamicDataP->InsertDataObjectCol(keystr, dynp->DbjParmValue_ptr->IntContentVector, DtObjColN);
			if (dynp->DbjParmValue_ptr->DataType == DOULBET) 
				dodycsucc = thisApp->DynamicDataP->InsertDataObjectCol(keystr, dynp->DbjParmValue_ptr->DoubleContentVector, DtObjColN);			
			if (dynp->DbjParmValue_ptr->DataType == STRINGT) 
				dodycsucc = thisApp->DynamicDataP->InsertDataObjectCol(keystr, dynp->DbjParmValue_ptr->StrContentVector, DtObjColN);			
			if (dynp->DbjParmValue_ptr->DataType == LOGICT) 
				//dodycsucc = thisApp->DynamicDataP->InsertDataObjectCol(keystr, dynp->DbjParmValue_ptr->IntContentVector, DtObjColN);	
				dodycsucc = thisApp->DynamicDataP->InsertDataObjectCol(keystr, dynp->DbjParmValue_ptr->BoolContentVector, DtObjColN);									
			if (dodycsucc)	
				return(CPC_NOERROR);
			else
				return(CPC_ERROR_INSERTCOLFAILURE);

		case CPC_DYN_APPENDDATAOBJECTCOL:
			if(dynp->DbjParmValue_ptr->name == (char *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}	
			if (dynp->DbjParmValue_ptr->DataType == INTT) 
				dodycsucc = thisApp->DynamicDataP->AppendDataObjectCol(keystr, dynp->DbjParmValue_ptr->IntContentVector);
			if (dynp->DbjParmValue_ptr->DataType == DOULBET) 
				dodycsucc = thisApp->DynamicDataP->AppendDataObjectCol(keystr, dynp->DbjParmValue_ptr->DoubleContentVector);			
			if (dynp->DbjParmValue_ptr->DataType == STRINGT) 
				dodycsucc = thisApp->DynamicDataP->AppendDataObjectCol(keystr, dynp->DbjParmValue_ptr->StrContentVector);			
			if (dynp->DbjParmValue_ptr->DataType == LOGICT) 
				//dodycsucc = thisApp->DynamicDataP->AppendDataObjectCol(keystr, dynp->DbjParmValue_ptr->IntContentVector);	
				dodycsucc = thisApp->DynamicDataP->AppendDataObjectCol(keystr, dynp->DbjParmValue_ptr->BoolContentVector);										
			if (dodycsucc)	
				return(CPC_NOERROR);
			else
				return(CPC_ERROR_APPENDCOLFAILURE);
				
		case CPC_DYN_CHANGEDATAOBJECTCOL:
			if(dynp->DbjParmValue_ptr->name == (char *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}	
			DtObjColN = dynp->DbjParmValue_ptr->ColumnNumber;		
			if (dynp->DbjParmValue_ptr->DataType == INTT) 
				dodycsucc = thisApp->DynamicDataP->ChangeDataObjectCol(keystr, dynp->DbjParmValue_ptr->IntContentVector, DtObjColN);
			if (dynp->DbjParmValue_ptr->DataType == DOULBET) 
				dodycsucc = thisApp->DynamicDataP->ChangeDataObjectCol(keystr, dynp->DbjParmValue_ptr->DoubleContentVector, DtObjColN);			
			if (dynp->DbjParmValue_ptr->DataType == STRINGT) 
				dodycsucc = thisApp->DynamicDataP->ChangeDataObjectCol(keystr, dynp->DbjParmValue_ptr->StrContentVector, DtObjColN);			
			if (dynp->DbjParmValue_ptr->DataType == LOGICT) 
				//dodycsucc = thisApp->DynamicDataP->ChangeDataObjectCol(keystr, dynp->DbjParmValue_ptr->IntContentVector, DtObjColN);	
				dodycsucc = thisApp->DynamicDataP->ChangeDataObjectCol(keystr, dynp->DbjParmValue_ptr->BoolContentVector, DtObjColN);									
			if (dodycsucc)	
				return(CPC_NOERROR);
			else
				return(CPC_ERROR_CHANGECOLFAILURE);	
				
		case CPC_DYN_DELETEDATAOBJECTCOL:
			if(dynp->DbjParmValue_ptr->name == (char *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}	
			DtObjColN = dynp->DbjParmValue_ptr->ColumnNumber;		
			if (dynp->DbjParmValue_ptr->DataType == INTT) 
				dodycsucc = thisApp->DynamicDataP->DeleteDataObjectCol(keystr, DtObjColN);
			if (dynp->DbjParmValue_ptr->DataType == DOULBET) 
				dodycsucc = thisApp->DynamicDataP->DeleteDataObjectCol(keystr, DtObjColN);			
			if (dynp->DbjParmValue_ptr->DataType == STRINGT) 
				dodycsucc = thisApp->DynamicDataP->DeleteDataObjectCol(keystr, DtObjColN);			
			if (dynp->DbjParmValue_ptr->DataType == LOGICT) 
				dodycsucc = thisApp->DynamicDataP->DeleteDataObjectCol(keystr, DtObjColN);										
			if (dodycsucc)	
				return(CPC_NOERROR);
			else
				return(CPC_ERROR_DELETECOLFAILURE);		

		case CPC_DYN_SETDATAOBJECTELEMENT:
			if(dynp->DbjParmValue_ptr->name == (char *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			DtObjRowN = dynp->DbjParmValue_ptr->RowsNumber;				
			DtObjColN = dynp->DbjParmValue_ptr->ColumnNumber;
			if (dynp->DbjParmValue_ptr->DataType == INTT) 
				dodycsucc = thisApp->DynamicDataP->SetDataObjectElement(keystr, *(dynp->DbjParmValue_ptr->IntContentVector), DtObjRowN, DtObjColN);
			if (dynp->DbjParmValue_ptr->DataType == DOULBET) 
				dodycsucc = thisApp->DynamicDataP->SetDataObjectElement(keystr, *(dynp->DbjParmValue_ptr->DoubleContentVector),DtObjRowN, DtObjColN);			
			if (dynp->DbjParmValue_ptr->DataType == STRINGT) 
				dodycsucc = thisApp->DynamicDataP->SetDataObjectElement(keystr, *(dynp->DbjParmValue_ptr->StrContentVector),DtObjRowN, DtObjColN);			
			if (dynp->DbjParmValue_ptr->DataType == LOGICT) 
				//dodycsucc = thisApp->DynamicDataP->SetDataObjectElement(keystr, *(dynp->DbjParmValue_ptr->IntContentVector),DtObjRowN, DtObjColN);	
				dodycsucc = thisApp->DynamicDataP->SetDataObjectElement(keystr, *(dynp->DbjParmValue_ptr->BoolContentVector),DtObjRowN, DtObjColN);									
			if (dodycsucc)	
				return(CPC_NOERROR);
			else
				return(CPC_ERROR_SETELEMENTFAILURE);	
				
		case CPC_DYN_GETDATAOBJECTPROPERTY:		
			if(dynp->DbjParmValue_ptr->name == (char *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			dodycsucc = thisApp->DynamicDataP->getDataObjectProperty(keystr,DtObjRowN, DtObjColN, i);
			//DtObjRowN = 2;
			//DtObjColN = 1;
			//dodycsucc = true;
			if (dodycsucc) {
				dynp->DbjParmValue_ptr->RowsNumber = DtObjRowN;
				dynp->DbjParmValue_ptr->ColumnNumber = DtObjColN;
				dynp->DbjParmValue_ptr->DataType= i;
				return(CPC_NOERROR);
			}
			else
			  return(CPC_ERROR_GETPROPERTYFAILURE);
																				
		default:
			;
	}
	return(CPC_ERROR_BADCODE);  // only get here if incorrect code
}
//The above was written on Feb 26,2005

int cpc_hist(cpc_io_ptr iop)
{

	if(thisApp->Uninitialized)
	{
		return(CPC_ERROR_UNINITIALIZED);
	}
	if(iop == (void *) 0) // null pointer ?
	{
		return(CPC_ERROR_NULLPOINTER);
	}


	switch(iop->type)
	{

		case CPC_HS_ADDTOHISTORY:
			{
				if(iop->string == (void *) 0)
				{
					return(CPC_ERROR_INTERNALNULLPOINTER);
				}
				wxString hline(iop->string);
				thisApp->History->AddCommand(hline);
				return(CPC_NOERROR);
			}

		case CPC_HS_RESETHISTORY:
			thisApp->History->ClearCommands();
			return(CPC_NOERROR);

		case CPC_HS_GETHISTORYLENGTH:
			iop->length = thisApp->History->commands.GetCount();
			return(CPC_NOERROR);

		case CPC_HS_SETHISTORYLENGTH:
			thisApp->History->ChangeHistoryLength(iop->length);
			return(CPC_NOERROR);

		case CPC_HS_GETHISTORYLINE:
			{
			wxString *item;
			if(thisApp->History->commands.GetCount() == 0)
			{
				return(CPC_ERROR_NOHISTORY);
			}
			if(iop->string == (void *) 0)
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			item = thisApp->History->commands.Item(iop->parm-1)->GetData();
			if((unsigned long) iop->length < item->Length()+1)
			{
				iop->length = item->Length()+1;
				return(CPC_ERROR_STRINGTOOSHORT);
			}
			iop->length = item->Length()+1;
			strcpy( iop->string, item->c_str());
			return(CPC_NOERROR);
			}

		default:
			;
	}
	return(CPC_ERROR_BADCODE);  // only get here if incorrect code
}	

int cpc_dostddlg(cpc_stddlg *dlg)
{
	int answer;
	int flags;

	if(thisApp->Uninitialized)
	{
		return(CPC_ERROR_UNINITIALIZED);
	}
	if(dlg == (void *) 0) // null pointer ?
	{
		return(CPC_ERROR_NULLPOINTER);
	}


	switch(dlg->type)
	{
	
		case CPC_DLG_GETTEXTFROMUSER:
			if(dlg->message == (void *) 0 ||
				dlg->title == (void *) 0)
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			dlg->string = CarapaceGetTextFromUser(dlg->message,
				dlg->title);
			if(dlg->string[0] == 0)
			{
				return(CPC_ERROR_CANCELLED);
			}
			dlg->length = strlen(dlg->string)+1;
			return(CPC_NOERROR);

		case CPC_DLG_GETFILENAME:
		case CPC_DLG_GETFILENAMEREAD:
		case CPC_DLG_GETFILENAMESAVE:
			if(dlg->message == (void *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			flags = 0;
			if(dlg->type ==CPC_DLG_GETFILENAMEREAD)
			{
				flags = wxOPEN;
			}
			if(dlg->type ==CPC_DLG_GETFILENAMESAVE)
			{
				flags = wxSAVE;
			}
			dlg->string = CarapaceFindFile(dlg->message, 
				dlg->param1, dlg->param2, // path and name
				dlg->param3, dlg->param4, // extension and wildcard
				flags); // flags
			//thisApp->TextFrameFront(thisApp->CommandFrame);
			if(dlg->string == NULL)
			{
				return(CPC_ERROR_CANCELLED);
			}
			dlg->length = strlen(dlg->string)+1;
			return(CPC_NOERROR);

		case CPC_DLG_GETDIRNAME:
			if(dlg->message == (void *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			flags = 0;
			dlg->string = CarapaceFindDir(dlg->message, dlg->param1);
			//thisApp->TextFrameFront(thisApp->CommandFrame);
			if(dlg->string == NULL)
			{
				return(CPC_ERROR_CANCELLED);
			}
			dlg->length = strlen(dlg->string)+1;
			return(CPC_NOERROR);

		case CPC_DLG_YESNOCANCELMESSAGE:
			if(dlg->message == (void *) 0 || dlg->param1 == (void *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			answer = wxMessageBox(dlg->message,dlg->param1,
				wxYES_NO | wxCANCEL | wxCENTRE);
			if(answer == wxYES)
			{
				return(CPC_IO_YES);
			}
			if(answer == wxNO)
			{
				return(CPC_IO_NO);
			}
			return(CPC_IO_CANCEL);

		case CPC_DLG_YESNOMESSAGE:
			if(dlg->message == (void *) 0 || dlg->param1 == (void *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			answer = wxMessageBox(dlg->message,dlg->param1,
				wxYES_NO | wxCENTRE);
			if(answer == wxYES)
			{
				return(CPC_IO_YES);
			}
			return(CPC_IO_NO);

		case CPC_DLG_OKMESSAGE:
			if(dlg->message == (void *) 0 || dlg->param1 == (void *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			wxMessageBox(dlg->message,dlg->param1,
				wxOK | wxCENTRE);
			return(CPC_NOERROR);


		case CPC_DLG_HELPFRAME:
			{
			wxString file;
			if(dlg->string == (void *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			file = dlg->string;
			if(!thisApp->HelpFrame)
			{
				thisApp->HelpFrame = new CarapaceHelpFrame((wxWindow *) 0,file);
				if(!thisApp->HelpFrame)
				{
					return(CPC_ERROR_CANTOPENWINDOW);
				}
			}
			else
			{
				thisApp->HelpFrame->SetSourceFile(file);
				thisApp->HelpFrame->Show(TRUE);
				thisApp->HelpFrame->Raise();
			}
			return(CPC_NOERROR);
			}

		default:
			;
	}
	return(CPC_ERROR_BADCODE);
}

int cpc_dosg(cpc_sg_ptr sgp)
{
	int windno;
	wxSize size;
	wxString tmpstring;
	wxCarapaceGraphFrameListNode *node;

	if(sgp == (void *) 0) // null pointer ?
	{
		return(CPC_ERROR_NULLPOINTER);
	}


	switch(sgp->type)
	{
		case CPC_SG_DRAWLINE:
		case CPC_SG_DRAWTEXT:
		case CPC_SG_DRAWTEXTUP:
		case CPC_SG_DRAWTEXTDOWN:
		case CPC_SG_NEWPENNAME:
		case CPC_SG_NEWPENRGB:
		case CPC_SG_NEWBRUSHNAME:
		case CPC_SG_NEWBRUSHRGB:
		case CPC_SG_SETBACKGROUNDNAME:
		case CPC_SG_SETBACKGROUNDRGB:
		case CPC_SG_SETFOREGROUNDNAME:
		case CPC_SG_SETFOREGROUNDRGB:
		case CPC_SG_SETFONT:
		case CPC_SG_SETFONTNAME:
		case CPC_SG_SAVESTATE:
		case CPC_SG_RESTORESTATE:
		case CPC_SG_CIRCLE:
        	case CPC_SG_DIAMOND:
        	case CPC_SG_SQUARE:
        	case CPC_SG_TRIANGLE:
        	case CPC_SG_EXE:
        	case CPC_SG_CROSS:
        	case CPC_SG_STAR:
			thisApp->AddGraphCommand(sgp);
			return(CPC_NOERROR);

		case CPC_SG_DOPAINT:
#if defined(__WXMSW__)
// this is a bit of a kludge.  The content of some graphics windows won't
// show up until the window is hidden and then revealed
		{
			wxCarapaceGraphFrameListNode *gnode;
			CarapaceGraphFrame *gfp;
			gnode = thisApp->GFrameList.Find(sgp->graphno);
			if(gnode != NULL)
			{
				gfp = gnode->GetData();
				gfp->Show(FALSE);
				gfp->Show(TRUE);
			}
		}
#endif
			thisApp->PaintGraph(sgp->graphno);
			return(CPC_NOERROR);

		case CPC_SG_NEWGRAPH:
			size = wxSize(sgp->coords[0],sgp->coords[1]);
			windno = sgp->graphno;
			windno = thisApp->NewGraphFrame(size,windno);
			if(windno == 0)
			{
				return(CPC_ERROR_CANTOPENWINDOW);
			}
			sgp->graphno = windno;
			return(CPC_NOERROR);

		case CPC_SG_GRAPHICSPAUSE:
			// first check that windno exists
			if((node = thisApp->GFrameList.Find(sgp->graphno)) == NULL)
			{
				return(CPC_ERROR_BADWINDOW);
			}
			thisApp->InGraphicsPause = TRUE;
			thisApp->PausedGraphFrame = sgp->graphno;
			while(thisApp->InGraphicsPause)
			{
				wxYield();
			}
			return(CPC_NOERROR);

		case CPC_SG_DODRAW:
		{
			// first check that windno exists
			wxCarapaceGraphFrameListNode *gnode;
			wxPaintEvent pe;
			if((gnode = thisApp->GFrameList.Find(sgp->graphno)) == NULL)
			{
				return(CPC_ERROR_BADWINDOW);
			}
			gnode->GetData()->Graph->OnPaint(pe);
			return(CPC_NOERROR);
		}

		case CPC_SG_OPENPOSTSCRIPTFILE:
			if(sgp->string == (void *) 0 )
			{
				return(CPC_ERROR_INTERNALNULLPOINTER);
			}
			if(thisApp->PSGraph != NULL)
			{
				delete thisApp->PSGraph;
			}
			tmpstring = sgp->string;
			thisApp->PSGraph = new CarapacePostScriptGraph(tmpstring,sgp->style);
			if(thisApp->PSGraph == NULL)
			{
				return(CPC_ERROR_CANTOPENWINDOW);
			}
			return(CPC_NOERROR);

		case CPC_SG_OPENPOSTSCRIPTPAGE:
			size = wxSize(sgp->coords[0]/10,sgp->coords[1]/10);
			if(thisApp->PSGraph)
			{
				thisApp->PSGraph->OpenPage(size);
			}
			return(CPC_NOERROR);

		case CPC_SG_CLOSEPOSTSCRIPTPAGE:
			if(thisApp->PSGraph)
			{
				thisApp->PSGraph->ClosePage();
			}
			return(CPC_NOERROR);

		case CPC_SG_CLOSEPOSTSCRIPTFILE:
			if(thisApp->PSGraph != NULL)
			{
				delete thisApp->PSGraph;
			}
			return(CPC_NOERROR);

		case CPC_SG_GETCHARWIDTH:
		case CPC_SG_GETCHARHEIGHT:
			thisApp->DoGraphRequest(sgp);
			return(CPC_NOERROR);

		case CPC_SG_XPARMS:
		case CPC_SG_YPARMS:
			thisApp->DoGraphParms(sgp);
			return(CPC_NOERROR);

		case CPC_SG_SETAFMPATH:
			thisApp->AFMPath = sgp->string;
			return(CPC_NOERROR);

		case CPC_SG_CHOOSEFRAME:
			if(thisApp->numGraphFrames == 0)
			{
				sgp->type = CPC_ERROR_BADWINDOW;
				return(CPC_ERROR_MOUSE);
			}
			thisApp->MouseRequestState = CPC_SG_CHOOSEFRAME;
			thisApp->MouseHelpString = sgp->string;
			while(thisApp->MouseRequestState == CPC_SG_CHOOSEFRAME)
			{
				wxYield();
			}
			if(thisApp->MouseRequestState == CPC_SG_MOUSEDONE)
			{
				sgp->graphno = thisApp->MouseSelectedFrame;
				thisApp->MouseRequestState = CPC_SG_NOMOUSEREQUEST;
				return(CPC_NOERROR);
			}
			if(thisApp->MouseRequestState == CPC_SG_MOUSEQUIT)
			{
				sgp->type = CPC_SG_MOUSEQUIT;
				thisApp->MouseRequestState = CPC_SG_NOMOUSEREQUEST;
				thisApp->MouseHelpString = "";
				return(CPC_NOERROR);
			}
			if(thisApp->MouseRequestState == CPC_SG_MOUSEINTERRUPT)
			{
				sgp->type = CPC_SG_MOUSEINTERRUPT;
				thisApp->MouseRequestState = CPC_SG_NOMOUSEREQUEST;
				thisApp->MouseHelpString = "";
				return(CPC_NOERROR);
			}
			// shouldn't get to here
			thisApp->MouseRequestState = CPC_SG_NOMOUSEREQUEST;
			return(CPC_ERROR_MOUSE);

		case CPC_SG_CLEARMOUSETRACKS:
		{
			wxCarapaceGraphFrameListNode *gnode;
			CarapaceStaticGraph *csgp;

			gnode = thisApp->GFrameList.Find(sgp->graphno);
			if(gnode == NULL)
			{
				return(CPC_ERROR_BADWINDOW);
			}
			csgp = gnode->GetData()->Graph;
			csgp->Refresh();

			return(CPC_NOERROR);
		}

		case CPC_SG_MOUSEPOINT:
		{
			wxCarapaceGraphFrameListNode *gnode;
			CarapaceStaticGraph *csgp;

			gnode = thisApp->GFrameList.Find(sgp->graphno);
			if(gnode == NULL)
			{
				return(CPC_ERROR_BADWINDOW);
			}
			csgp = gnode->GetData()->Graph;
			csgp->MouseRequestState = CPC_SG_MOUSEPOINT;
			csgp->MouseHelpString = sgp->string;
			csgp->MouseCoords[0] = sgp->coords[0];
			csgp->MouseCoords[1] = sgp->coords[1];
			csgp->MouseCoords[2] = sgp->coords[0];
			csgp->MouseCoords[3] = sgp->coords[1];
			csgp->MouseDParms[0] = sgp->dparms[0];
			csgp->MouseDParms[1] = sgp->dparms[1];
			csgp->MouseDParms[2] = sgp->dparms[0];
			csgp->MouseDParms[3] = sgp->dparms[1];

			while(csgp->MouseRequestState == CPC_SG_MOUSEPOINT)
			{
				wxYield();
			}

			if(csgp->MouseRequestState == CPC_SG_MOUSEDONE)
			{
				sgp->coords[0] = csgp->MouseCoords[0];
				sgp->coords[1] = csgp->MouseCoords[1];
				sgp->dparms[0] = csgp->MouseDParms[0];
				sgp->dparms[1] = csgp->MouseDParms[1];
				csgp->MouseRequestState = CPC_SG_NOMOUSEREQUEST;
				return(CPC_NOERROR);
			}
			if(csgp->MouseRequestState == CPC_SG_MOUSEQUIT)
			{
				sgp->type = CPC_SG_MOUSEQUIT;
				csgp->MouseRequestState = CPC_SG_NOMOUSEREQUEST;
				csgp->MouseHelpString = "";
				return(CPC_NOERROR);
			}
			if(csgp->MouseRequestState == CPC_SG_MOUSELEAVE)
			{
				sgp->type = CPC_SG_MOUSELEAVE;
				csgp->MouseRequestState = CPC_SG_NOMOUSEREQUEST;
				return(CPC_NOERROR);
			}
			if(csgp->MouseRequestState == CPC_SG_MOUSEINTERRUPT)
			{
				sgp->type = CPC_SG_MOUSEINTERRUPT;
				csgp->MouseRequestState = CPC_SG_NOMOUSEREQUEST;
				csgp->MouseHelpString = "";
				return(CPC_NOERROR);
			}
			// shouldn't get to here
			csgp->MouseRequestState = CPC_SG_NOMOUSEREQUEST;
			return(CPC_ERROR_MOUSE);
		}

		case CPC_SG_MOUSESEGMENT:
		case CPC_SG_MOUSEBOX:
		{
			wxCarapaceGraphFrameListNode *gnode;
			CarapaceStaticGraph *csgp;

			gnode = thisApp->GFrameList.Find(sgp->graphno);
			if(gnode == NULL)
			{
				return(CPC_ERROR_BADWINDOW);
			}
			csgp = gnode->GetData()->Graph;
			csgp->MouseRequestState = sgp->type; //CPC_SG_MOUSESEGMENT or CPC_SG_MOUSEBOX
			csgp->MouseHelpString = sgp->string;
			csgp->MouseCoords[0] = sgp->coords[0];
			csgp->MouseCoords[1] = sgp->coords[1];
			csgp->MouseCoords[2] = sgp->coords[0];
			csgp->MouseCoords[3] = sgp->coords[1];
			csgp->MouseDParms[0] = sgp->dparms[0];
			csgp->MouseDParms[1] = sgp->dparms[1];
			csgp->MouseDParms[2] = sgp->dparms[0];
			csgp->MouseDParms[3] = sgp->dparms[1];

			while(csgp->MouseRequestState == sgp->type)
			{
				wxYield();
			}

			if(csgp->MouseRequestState == CPC_SG_MOUSEDONE)
			{
				sgp->coords[0] = csgp->MouseCoords[0];
				sgp->coords[1] = csgp->MouseCoords[1];
				sgp->coords[2] = csgp->MouseCoords[2];
				sgp->coords[3] = csgp->MouseCoords[3];
				sgp->dparms[0] = csgp->MouseDParms[0];
				sgp->dparms[1] = csgp->MouseDParms[1];
				sgp->dparms[2] = csgp->MouseDParms[2];
				sgp->dparms[3] = csgp->MouseDParms[3];
				csgp->MouseRequestState = CPC_SG_NOMOUSEREQUEST;
				return(CPC_NOERROR);
			}
			if(csgp->MouseRequestState == CPC_SG_MOUSEQUIT)
			{
				sgp->type = CPC_SG_MOUSEQUIT;
				csgp->MouseRequestState = CPC_SG_NOMOUSEREQUEST;
				csgp->MouseHelpString = "";
				return(CPC_NOERROR);
			}
			if(csgp->MouseRequestState == CPC_SG_MOUSELEAVE)
			{
				sgp->type = CPC_SG_MOUSELEAVE;
				csgp->MouseRequestState = CPC_SG_NOMOUSEREQUEST;
				return(CPC_NOERROR);
			}
			if(csgp->MouseRequestState == CPC_SG_MOUSEINTERRUPT)
			{
				sgp->type = CPC_SG_MOUSEINTERRUPT;
				csgp->MouseRequestState = CPC_SG_NOMOUSEREQUEST;
				csgp->MouseHelpString = "";
				return(CPC_NOERROR);
			}
			// shouldn't get to here
			csgp->MouseRequestState = CPC_SG_NOMOUSEREQUEST;
			return(CPC_ERROR_MOUSE);
		}

		default:
			;
	}
	return(CPC_ERROR_BADCODE);
}

} /* extern "C" */
