/*
*(C)* The file is part of the source distribution of Carapace
*(C)* version 0.5 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* This file is distributed under the GNU General Public License
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

#include "cpcplatform.h"



// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/ffile.h"
#include "wx/print.h"
#include "wx/printdlg.h"
#include "wx/clipbrd.h"
#include "wx/paper.h"
#include "wx/cmdline.h"


#include <new>

#include <math.h>
#include <stdio.h>
#include <signal.h>

#define MAIN__
#include "carapace.h"

#include "forcelnk.h"
//FORCE_LINK(carapace_gtk)
//FORCE_LINK(dcpsg)
//FORCE_LINK(psdc)

#undef UNDEFINED

#ifndef STDOUT
#define STDOUT (FILE *) 0
#endif

CarapaceApp     *thisApp;

BEGIN_EVENT_TABLE(CarapaceApp, wxApp)
        EVT_MENU(CPC_GUI_PROCESSCOMMAND, CarapaceApp::GuiProcessCommand)
        EVT_MENU(CPC_GUI_FLUSHBUFFER, CarapaceApp::ComFlushBuffer)
	EVT_SOCKET(CPC_SERVER_MAIN,  CarapaceApp::OnServerEvent)
	EVT_SOCKET(CPC_SERVER_BACK,  CarapaceApp::OnServerEvent)
	EVT_SOCKET(CPC_SERVER_INTR,  CarapaceApp::OnServerEvent)
	EVT_SOCKET(CPC_SOCKET_MAIN,  CarapaceApp::OnSocketEvent)
	EVT_SOCKET(CPC_SOCKET_BACK,  CarapaceApp::OnSocketEvent)
	EVT_SOCKET(CPC_SOCKET_INTR,  CarapaceApp::OnSocketEvent)
	EVT_IDLE(CarapaceApp::OnIdle)
END_EVENT_TABLE()

wxCriticalSection FileSelectionLock;
wxCriticalSection BufferedOutputLock;
wxCriticalSection CommandLock;
wxMutex FileSelectionMutex;
long CPC_INTERRUPT;

wxRealPoint CarapaceDefaultScales = wxRealPoint(1.,1.);
 
#include "wx/listimpl.cpp"
 
WX_DEFINE_LIST(CarapaceSplitTextFrameList); 
WX_DEFINE_LIST(CarapaceGraphFrameList); 
WX_DEFINE_LIST(CarapacePanelGraphFrameList); 
WX_DEFINE_LIST(CarapaceGraphElementList); 
WX_DEFINE_LIST(CarapaceDialogFrameList); 
WX_DEFINE_LIST(CarapaceCommandList); 
WX_DEFINE_LIST(CarapaceHistoryList); 
WX_DEFINE_LIST(CarapaceTypeAheadList); 
WX_DEFINE_LIST(CarapaceMatchList); 
WX_DEFINE_LIST(CarapaceGraphStateList); 


FILE *LOGFILE;

void InitializeResourceIDs(void);
extern int   SendCommands(const char *);
 
static const wxCmdLineEntryDesc cmdLineDesc[] =
{
    { wxCMD_LINE_PARAM, "port", "port", "socket port number",
	wxCMD_LINE_VAL_NUMBER, wxCMD_LINE_OPTION_MANDATORY },
    { wxCMD_LINE_OPTION, "splash", "splash", "path to splash screen",
	wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL },
    { wxCMD_LINE_OPTION, "menu", "menu", "path to menu file",
	wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL },
    { wxCMD_LINE_OPTION, "afmdir", "afmdir", "directory where Adobe font metrics are found",
	wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL },
    { wxCMD_LINE_OPTION, "prefs", "preferences", "preferences file",
	wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL },

    { wxCMD_LINE_NONE }
};


bool CarapaceApp::OnInit(void)
{

	long              i,port;
	char            *filename,*afmPath;
	wxString         afmdir,splash,prefs,portparam;
	wxString         menufile,menuXML;
	char *menubuf = (char *) 0;

	cpc_callback     cb;
	cpc_io           io;
	cpc_sg           sg;

	std::set_new_handler(0); // return NULL on "new" failure
	
	//LOGFILE = fopen("LOGFILE","w");

	// initialize pointer for external access
	thisApp = this;

	Uninitialized = TRUE;

	// initialize callback pointers to NULL
	// the userinitialization should change these
	InputReady = NULL;
	ProcessCommand = NULL;
	CleanUp = NULL;
	Initializer = NULL;
	HelpFrame = NULL;
	PSGraph = NULL;
	WindowClosed = NULL;
	Splash = NULL;

	// initialize output files
	SwallowOutput = CPC_SWALLOW_NOTHING;

	ArgV0 = argv[0];  // so we can get to root file name of application

// this should initialize all the callback functions 
// If we are not running stand alone, it should also set the
// preferences file path

	if(cpc_initcallbacks(argc,argv) != CPC_NOERROR)
	{
		return(FALSE);
	}

	InitializeResourceIDs();
	InitializeDialogs();
	InitializeDynamic();
#ifdef __WXMAC__
	s_macExitMenuItemId = wxXmlResource::GetXRCID("CPC_FILE_QUIT");
	s_macAboutMenuItemId = wxXmlResource::GetXRCID("CPC_HELP_ABOUT");
	s_macPreferencesMenuItemId = wxXmlResource::GetXRCID("CPC_FILE_PREFERENCES");
	s_macHelpMenuTitleName = "Help";
#endif //__WXMAC__

// install default preferences
	Preferences = new CarapacePreferences();
	if(Preferences == (CarapacePreferences *) 0) 
	{
		(void) wxMessageBox(CPC_STRING_PREFERENCESERROR,
		   CPC_STRING_ERROR , wxOK|wxCENTRE);
		return(FALSE);
	}
	Preferences->SetPreferences();


	OutLineBuffer.Alloc(OUTLINESIZE);
	OutLineBuffer.Empty();
	MainInputCommandBuffer.Alloc(OUTLINESIZE);
	MainInputCommandBuffer.Empty();
	MainInputBuffer.Alloc(OUTLINESIZE);
	MainInputBuffer.Empty();


	STFrameList = CarapaceSplitTextFrameList(wxKEY_INTEGER);
	GFrameList = CarapaceGraphFrameList(wxKEY_INTEGER);
	PGFrameList = CarapacePanelGraphFrameList(wxKEY_INTEGER);
	DFrameList = CarapaceDialogFrameList(wxKEY_INTEGER);
	CList = CarapaceCommandList(wxKEY_INTEGER);

	numTextFrames = 0;
	numGraphFrames = 0;
	TFrameSequence = 1;
	GFrameSequence = 1;
	DFrameSequence = 1;

	InGraphicsPause = FALSE;

	MouseRequestState = CPC_SG_NOMOUSEREQUEST;
	MouseHelpString = "";

	RunningFrame = 1;
	Running = CPC_IDLE;
	SplitRunning = TRUE;


	History = new CarapaceHistory(Preferences->historyLength);
	if(History == (void *) 0)
	{
		(void) wxMessageBox(CPC_STRING_HISTORYSETUPERROR
			/*"Cannot set up history"*/,
		   CPC_STRING_ERROR /*"Error"*/, wxOK|wxCENTRE);
		return(FALSE);
	}

	Uninitialized = FALSE;
#ifdef CPC_STANDALONE

	ProcessCommand = SendCommands;

	wxCmdLineParser lineParser(cmdLineDesc,argc,argv);
	if(lineParser.Parse(TRUE))
	{
		return(CPC_ERROR_INITFAILURE);
	}

	portparam = lineParser.GetParam(0);
	if(!portparam.ToLong(&port))
	{
		wxMessageBox("Port parameter must be a positive integer");
		return(CPC_ERROR_INITFAILURE);
	}
// set up sockets based on port, port+1, and port+2
	wxIPV4address addr;
  	addr.Service(port);
	mainChannelServer = new wxSocketServer(addr,wxSOCKET_NOWAIT);
  	addr.Service(port+1);
	backChannelServer = new wxSocketServer(addr,wxSOCKET_NOWAIT);
  	addr.Service(port+2);
	intrChannelServer = new wxSocketServer(addr,wxSOCKET_NOWAIT);
	if(!mainChannelServer || !mainChannelServer->Ok() || 
		!backChannelServer || !backChannelServer->Ok() ||
		!intrChannelServer || !intrChannelServer->Ok())
	{
		return(CPC_ERROR_INITFAILURE);
	}
	mainChannelServer->SetEventHandler(*this, CPC_SERVER_MAIN);
	mainChannelServer->SetNotify(wxSOCKET_CONNECTION_FLAG);
	mainChannelServer->Notify(true);
	backChannelServer->SetEventHandler(*this, CPC_SERVER_BACK);
	backChannelServer->SetNotify(wxSOCKET_CONNECTION_FLAG);
	backChannelServer->Notify(true);
	intrChannelServer->SetEventHandler(*this, CPC_SERVER_INTR);
	intrChannelServer->SetNotify(wxSOCKET_CONNECTION_FLAG);
	intrChannelServer->Notify(true);



	if(lineParser.Found("splash",&splash))
	{
		wxFileName fn(splash);
		if(!fn.IsOk())
		{
			wxMessageBox("Illegal file name for splash screen");
			return(CPC_ERROR_INITFAILURE);
		}
		if(!fn.FileExists())
		{
			wxMessageBox("Splash screen file does not exist");
			return(CPC_ERROR_INITFAILURE);
		}
		fn.MakeAbsolute();
		splash = fn.GetFullPath();
		wxString ext = fn.GetExt();
		ext = ext.MakeLower();

		io.type = CPC_IO_SPLASH;
		io.string = (char *) splash.c_str();
		if(ext == "gif")
		{
			io.parm = CPC_IO_GIFTYPE;
		}
		else if(ext == "jpg" || ext == "jpeg")
		{
			io.parm = CPC_IO_JPEGTYPE;
		}
		else if(ext == "png")
		{
			io.parm = CPC_IO_PNGTYPE;
		}
		else if(ext == "xpm")
		{
			io.parm = CPC_IO_XPMTYPE;
		}
		else
		{
			wxMessageBox("Unsupported splash screen type");
			return(CPC_ERROR_INITFAILURE);
		}
		cpc_doio(&io);
	}

	if(lineParser.Found("prefs",&prefs))
	{
		wxFileName fn(prefs);
		if(!fn.IsOk())
		{
			wxMessageBox("Illegal file name for preferences file");
			return(CPC_ERROR_INITFAILURE);
		}
		if(!fn.FileExists())
		{
			wxMessageBox("Preferences file does not exist");
			return(CPC_ERROR_INITFAILURE);
		}
		fn.MakeAbsolute();
		prefs = fn.GetFullPath();

		cb.type = CPC_CB_SETPREFSFILEPATH;
		cb.callback.string = (char *) prefs.c_str();
		cpc_setCallback(&cb);
	}


	if(lineParser.Found("menus",&menufile))
	{
		wxFileName fn(menufile);
		if(!fn.IsOk())
		{
			wxMessageBox("Illegal file name for menu file");
			return(CPC_ERROR_INITFAILURE);
		}
		if(!fn.FileExists())
		{
			wxMessageBox("Menu file does not exist");
			return(CPC_ERROR_INITFAILURE);
		}
		fn.MakeAbsolute();
		menufile = fn.GetFullPath();

		wxFFile menuFFile(menufile,"r");
		if(menuFFile.IsOpened()) 
		{
			menuFFile.ReadAll(&menuXML);
			menubuf = (char *) menuXML.c_str();
                	io.string2 = "defaultmenu";
		}
	}
	else
	{
		menubuf = CPC_STRING_DEFAULTCOMMANDRESOURCE;
               	io.string2 = "defaultmenu";
	}
	io.type = CPC_IO_NEWWINDOW;
	io.string = menubuf;
	cpc_doio(&io);

	if(lineParser.Found("afmdir",&afmdir))
	{
		wxFileName fn(afmdir);
		if(!fn.IsOk())
		{
			wxMessageBox("Illegal file name for afm directory");
			return(CPC_ERROR_INITFAILURE);
		}
		if(!fn.IsDir())
		{
			wxMessageBox("afm directory is not a directory!");
			return(CPC_ERROR_INITFAILURE);
		}
		if(!fn.DirExists())
		{
			wxMessageBox("afm directory does not exist");
			return(CPC_ERROR_INITFAILURE);
		}
		fn.MakeAbsolute();
		afmdir = fn.GetFullPath();

		sg.type = CPC_SG_SETAFMPATH;
		sg.string = (char *) afmdir.c_str();
		cpc_dosg(&sg);
	}

#endif /* CPC_STANDALONE */

	int argc2;
	char **argv2;
#ifdef __WXMAC__
	argc2 = argc + AEOpenFileNames.GetCount();
	argv2 = (char **) new char(argc2*sizeof(char *));
	for(i=0;i<argc;i++)
	{
		argv2[i] = argv[i];
	}
	for(i=argc;i<argc2;i++)
	{
		argv2[i] = (char *) AEOpenFileNames[i-argc].c_str();
	}
#else
	argc2 = argc;
	argv2 = argv;
#endif
	if(Initializer(argc2,argv2) != CPC_NOERROR)
	{
		DoQuit(TRUE);
	}
	// printing stuff
	PageSetupData = new wxPageSetupDialogData;
	PrintData = new wxPrintData;
	if(AFMPath != "")
	{
		// 2.8 change PrintData->SetFontMetricPath(AFMPath);
	}
	if(PageSetupData == (void *) 0 || PrintData == (void *) 0)
	{
		(void) wxMessageBox(CPC_STRING_PRINTSETUPERROR
			/*"Cannot set up printing"*/,
	   	CPC_STRING_ERROR /*"Error"*/, wxOK|wxCENTRE);
		DoQuit(FALSE);
	}
			
	PrintData->SetPaperId(wxPAPER_LETTER);
	if(Preferences->useA4Paper)
	{
		PrintData->SetPaperId(wxPAPER_A4);
	}
	
	UpdateMenus();
	

	return(TRUE);
	
} /*CarapaceApp::OnInit()*/

// This statement initialises the whole application
IMPLEMENT_APP(CarapaceApp)

void CarapaceApp::DoQuit(int quitStatus)
{
	int           decision;
	CarapaceSplitTextFrame *stfp;
	wxCarapaceSplitTextFrameListNode *stnode = STFrameList.GetFirst();

	if (quitStatus == CPC_EXIT_NOQUIT)
	{
		return;
	}

	if (quitStatus)
	{
		if(QuitCleanUp)
		{
			if(quitStatus == CPC_EXIT_SAVECANCANCEL)
			{
				decision = QuitCleanUp(TRUE);
			}
			else
			{
				decision = QuitCleanUp(FALSE);
			}
			if ( decision == CPC_IO_CANCEL )
			{
				return;
			}
		}


		stnode = STFrameList.GetFirst();
		while(stnode)
		{
			stfp = stnode->GetData();
			if (!stfp->OKToClose(quitStatus))
			{
				return;
			}
			stnode = stnode->GetNext();
		}
	} /*quitStatus*/

	if(PSGraph != NULL)
	{
		delete PSGraph;
	}

	//ExitMainLoop();
	stnode = STFrameList.GetFirst();
	if(stnode)
	{
		SetTopWindow(stnode->GetData());
	}
	SetExitOnFrameDelete(TRUE);


// for reasons I don't understand, just killing off the windows results
// in a segmentation violation.  This seems to work though.
		exit(0);

	stnode = STFrameList.GetFirst();
	while(stnode)
	{
		stfp = stnode->GetData();
		stnode = stnode->GetNext();
		STFrameList.DeleteObject(stfp);
		delete stfp;
	}

	FreeDynamic(); // Free the DynamicInteraction, append on March 7,2005

	//delete this;
} /*CarapaceApp::DoQuit()*/


int CarapaceApp::NewSplitTextFrame(bool untitled)
{
	wxString res,name;
	res = "";
	name = "";
	return(NewSplitTextFrame(untitled,res,name));
}

#define OFFSETBASE 50
int CarapaceApp::NewSplitTextFrame(bool untitled, wxString& resource, wxString& menuname)
{
	wxString     buffer;
	int          offset = OFFSETBASE + ((TFrameSequence-1)%10) * Preferences->windowOffset;
	int          windno = TFrameSequence;
	CarapaceSplitTextFrame *stfp;
	wxSize thisSize;
	int  swidth,sheight;
	
	// create a new text frame if there are any slots left
	

	// if an untitled frame, give it the name untitled

	wxDisplaySize(&swidth,&sheight);
	thisSize = Preferences->tWindowSize;
	if(offset + thisSize.GetWidth() > swidth)
	{
		thisSize.SetWidth(swidth-offset);
	}
	if(offset + thisSize.GetHeight() > sheight)
	{
		thisSize.SetHeight(sheight-offset);
	}

	if(untitled)
	{
		buffer << CPC_STRING_UNTITLED /*"Untitled "*/ << TFrameSequence;
		stfp = new CarapaceSplitTextFrame(NULL, buffer, 
			wxPoint(offset, offset), thisSize,
			resource, menuname);
		if (stfp == NULL)
		{
			(void) wxMessageBox(CPC_STRING_CANTOPENCOMMANDWIND
				/*"Unable to open new text window"*/,
				  CPC_STRING_ERROR /*"Error"*/, wxOK|wxCENTRE);
			return(-1);
		}
		stfp->Untitled = TRUE;
	}
	else
	{
		if(FileSelectionMutex.TryLock() != wxMUTEX_NO_ERROR)
		{
			return(-1);
		}
		//SendIdleEvents();
		const wxString& s = wxFileSelector(CPC_STRING_SELECTTEXTFILE
			/*"Load text file"*/,"","","","*",wxOPEN);
		FileSelectionMutex.Unlock();
		if(s.Cmp(""))
		{
			wxFFile *loadFile = new wxFFile(s);
			if(!loadFile->IsOpened())
			{
				// can't open file
				(void) wxMessageBox(CPC_STRING_CANTOPENFILE
					/*"Cannot open file"*/,
					CPC_STRING_ERROR /*"Error"*/,wxOK|wxCENTRE);
				return(-1);
			}
			stfp = new CarapaceSplitTextFrame(NULL, s, 
				wxPoint(offset,offset), thisSize,
				resource, menuname);

			if (stfp == NULL)
			{
				(void) wxMessageBox(CPC_STRING_CANTOPENCOMMANDWIND
					/*"Unable to open new text window"*/,
				   	CPC_STRING_ERROR /*"Error"*/, wxOK|wxCENTRE);
				return(-1);
			}

			loadFile->Close();
			stfp->output_ctrl->LoadFile(s);
			delete loadFile;
			stfp->Untitled = FALSE;
			stfp->output_ctrl->AppendText("\n");
		}
		else
		{
			//cancelled
			return(-1);
		}
	}

	STFrameList.Append(windno, stfp);
	TFrameSequence++;

	TextFrameFront(windno);
	stfp->TextFrameIndex = windno;
	stfp->SetFocus();
	stfp->command_ctrl->SetFocus();
	stfp->command_ctrl->SetValue("");
	numTextFrames++;
	UpdateMenus();

	if(Splash && SplashShown)
	{
		Splash->Raise();
	}

	OutputWindow = stfp->output_ctrl;
	

	return(windno);
} /*CarapaceApp::NewSplitTextFrame()*/

int CarapaceApp::NewGraphFrame(wxSize& size, long windno)
{
	wxString resource = "";
	wxString name = "";
	return(NewGraphFrame(size,resource,name,windno));
}

int CarapaceApp::NewGraphFrame(wxSize& size, wxString& resource, wxString& menuname, long windno)
{
	wxString     buffer;
	int          offset;
	//int          windno = GFrameSequence;
	CarapaceGraphFrame *gfp;
	wxCarapaceGraphFrameListNode *node;
	wxSize size10;
 
	size10.SetWidth(size.GetWidth()/10);
	size10.SetHeight(size.GetHeight()/10);

	if((node = GFrameList.Find(windno)) != NULL)
	{ // existing window, so clear and return
		gfp = node->GetData();
		delete gfp->Graph;
		gfp->Graph = new CarapaceStaticGraph(gfp,-1,wxPoint(0,0),size10);
		// kludge, we really should find out the actual size of
		// the scroll bars
		gfp->Graph->SetScrollbars( 10, 10, (long) (size10.GetWidth()/10),\
			 (long) (size10.GetHeight()/10) );
		gfp->SetClientSize(size10.GetWidth()+50,size10.GetHeight()+50);
		((wxWindow *)gfp)->Fit();
		GraphFrameFront(windno);
		gfp->Graph->SetFocus();
		return(windno);
	}

	if(windno == 0)
	{ // find first empty slot
		windno = 1;
		while( (node = GFrameList.Find(windno)) != NULL)
		{
		        windno++;
		}
	}

		

	buffer << CPC_STRING_GRAPH /*"Untitled "*/ << windno;
	offset = OFFSETBASE + ((windno-1)%10) * Preferences->windowOffset;
	gfp = new CarapaceGraphFrame(buffer, wxPoint(offset, offset), size10, 
			resource, menuname);
	if (gfp == NULL)
	{
		(void) wxMessageBox(CPC_STRING_CANTOPENGRAPHWIND,
			  CPC_STRING_ERROR /*"Error"*/, wxOK|wxCENTRE);
		return(0);
	}


	GFrameList.Append(windno, gfp);
	GFrameSequence++;

	((wxWindow *)gfp)->Fit();
	gfp->GraphFrameIndex = windno;
	gfp->SetFocus();
	numGraphFrames++;
	UpdateMenus();
#ifdef __WXGTK__
	if(numGraphFrames == 1)
	{
// on GTK, you seem to need to hide the first graph once, then show it
// in order for OnChar to work
		gfp->Show(FALSE);
	}
#endif //__WXGTK__
	GraphFrameFront(windno);
	
	if(Splash && SplashShown)
	{
		Splash->Raise();
	}

	return(windno);
} /*CarapaceApp::NewGraphFrame()*/

void CarapaceApp::RefreshPanel(long panelno)
{
	CarapacePanelGraphFrame *pgfp;
	wxCarapacePanelGraphFrameListNode *pgnode;
	wxPaintEvent ev;

	pgnode = PGFrameList.Find(panelno);
	if(pgnode && (pgfp = pgnode->GetData()) && pgfp->Graph)
	{
		pgfp->Graph->OnPaint(ev);
	}
}

void CarapaceApp::FixPanels(void)
{
	CarapaceGraphFrame *gfp;
	wxCarapaceGraphFrameListNode *gnode;
	CarapacePanelGraphFrame *pgfp;
	wxCarapacePanelGraphFrameListNode *pgnode, *pgnodeold;
	int i,frameno,panelno;
	wxString buffer;
	wxPoint pos = wxPoint(OFFSETBASE,OFFSETBASE);
// FIX ME  these sizes should be set by an option or a preference somewhere
	wxSize size = wxSize(600,400);
	bool  nograph;

	// Make sure there is a panel for every graph and each panel has a graph

	gnode = GFrameList.GetFirst();
	while(gnode)
	{
		gfp = gnode->GetData();
		frameno = gfp->GraphFrameIndex;
		panelno = (frameno-1)/CPC_GRAPHSPERPANEL + 1;

		if(PGFrameList.Find(panelno) == NULL)
		{ // need a panel for this graph
			buffer << CPC_STRING_PANEL << panelno;
			pgfp = new CarapacePanelGraphFrame(buffer,pos,size,panelno);
			PGFrameList.Append(panelno,pgfp);
		}
		gnode = gnode->GetNext();
	}

	pgnode = PGFrameList.GetFirst();
	while(pgnode)
	{
		pgfp = pgnode->GetData();
		panelno = pgfp->PanelFrameIndex;
		pgfp->Graph->Refresh();

		nograph = TRUE;
		for(i=0;i<CPC_GRAPHSPERPANEL;i++)
		{
			if(GFrameList.Find((panelno-1)*CPC_GRAPHSPERPANEL+i+1) != NULL)
			{
				nograph = FALSE;
			}
		}
		pgnodeold = pgnode;
		pgnode = pgnode->GetNext();
		if(nograph)
		{
			PGFrameList.DeleteContents(TRUE);
			PGFrameList.DeleteNode(pgnodeold);
		}
	}

}

void CarapaceApp::AddGraphCommand(cpc_sg_ptr sgp)
{
	wxCarapaceGraphFrameListNode *cgfln;
	CarapaceGraphFrame *cgfp;

	if(sgp->graphno == -1 && PSGraph != NULL)
	{
		PSGraph->GStructure->AddGraphCommand(sgp);
	}
	
	cgfln = GFrameList.Find(sgp->graphno);
	if(cgfln)
	{
		cgfp = cgfln->GetData();
		if(cgfp)
		{
			cgfp->Graph->GStructure->AddGraphCommand(sgp);
		}
	}
}

void CarapaceApp::DoGraphRequest(cpc_sg_ptr sgp)
{
	wxCarapaceGraphFrameListNode *cgfln;
	CarapaceGraphFrame *cgfp;
	
	cgfln = GFrameList.Find(sgp->graphno);
	if(cgfln)
	{
		cgfp = cgfln->GetData();
		if(cgfp && cgfp->Graph)
		{
			cgfp->Graph->DoGraphRequest(sgp);
			//cgfp->Graph->GStructure->DoGraphRequest(sgp);
		}
	}
}

void CarapaceApp::DoGraphParms(cpc_sg_ptr sgp)
{
	wxCarapaceGraphFrameListNode *cgfln;
	CarapaceGraphFrame *cgfp;
	
	cgfln = GFrameList.Find(sgp->graphno);
	if(cgfln)
	{
		cgfp = cgfln->GetData();
		if(cgfp && cgfp->Graph)
		{
			if(sgp->type == CPC_SG_XPARMS)
			{
				cgfp->Graph->XAxisType = sgp->style;
				cgfp->Graph->XAxisParms[0] = sgp->dparms[0];
				cgfp->Graph->XAxisParms[1] = sgp->dparms[1];
			}
			else
			{
				cgfp->Graph->YAxisType = sgp->style;
				cgfp->Graph->YAxisParms[0] = sgp->dparms[0];
				cgfp->Graph->YAxisParms[1] = sgp->dparms[1];
			}
		}
	}
}

void CarapaceApp::PaintGraph(int graphno)
{
	wxCarapaceGraphFrameListNode *cgfln;
	CarapaceGraphFrame *cgfp;
	
	cgfln = GFrameList.Find(graphno);
	if(cgfln)
	{
		cgfp = cgfln->GetData();
		if(cgfp)
		{
			wxPaintEvent ev;
			cgfp->Graph->OnPaint(ev);
		}
	}
}


int CarapaceApp::NewDialogFrame(wxString& resource, wxString& name)
{

	int index = DFrameSequence++;
	CarapaceNonModalFrame *cnmfp = new CarapaceNonModalFrame();
	if(cnmfp == NULL)
	{
		return(0);
	}
	cnmfp->SetDLGFromString(NULL,name,resource);
	if(!cnmfp->isOK())
	{
		delete cnmfp;
		return(0);
	}
	cnmfp->Index = index;
	DFrameList.Append(index, cnmfp);

	numDialogFrames++;
	UpdateMenus();
	
	return(index);
} /*CarapaceApp::NewDialogFrame()*/


void CarapaceApp::SetTextFromFile(wxString& filename, long windno)
{
	wxString     buffer;
	CarapaceSplitTextFrame  *stfp;
	wxCarapaceSplitTextFrameListNode *stnode;

	stnode = STFrameList.Find(windno);
	if(stnode == NULL)
	{
		CarapaceAlert(CPC_STRING_NONEXISTANTCMDWIND
		/*"Attempt to set nonexisting text frame"*/);
		return;
	}
	stfp = stnode->GetData();
	

	wxFFile *loadFile = new wxFFile(filename);
	if(!loadFile->IsOpened())
	{
		// can't open file
		CarapaceAlert(CPC_STRING_CANTOPENFILE);
		return;
	}

	loadFile->Close();
	stfp->output_ctrl->LoadFile(filename);
	delete loadFile;
	stfp->Untitled = FALSE;
	stfp->SetTitle(filename);
	UpdateMenus();

} /*CarapaceApp::SetTextFromFile()*/

void CarapaceApp::CutTextFrame(int frame_index)
{
	
} 

void CarapaceApp::CutSplitTextFrame(int frame_index)
{
	
	CarapaceSplitTextFrame *stfp;
	wxCarapaceSplitTextFrameListNode *stnode = STFrameList.Find(frame_index);

	if(stnode != NULL)
	{
		stfp = stnode->GetData();
		STFrameList.DeleteNode(stnode);
		UpdateMenus();
		numTextFrames--;
		if(WindowClosed)
		{
			WindowClosed(stfp->TextFrameIndex,CPC_WINDOWTYPE_COMMAND);
		}
	}

} 

void CarapaceApp::CutGraphFrame(int frame_index)
{
	
	CarapaceGraphFrame *gfp;
	wxCarapaceGraphFrameListNode *gnode = GFrameList.Find(frame_index);

	if(InGraphicsPause && PausedGraphFrame == frame_index)
	{
		InGraphicsPause = FALSE;
	}

	if(gnode != NULL)
	{
		gfp = gnode->GetData();
		GFrameList.DeleteNode(gnode);
		UpdateMenus();
		numGraphFrames--;
		if(WindowClosed)
		{
			WindowClosed(gfp->GraphFrameIndex,CPC_WINDOWTYPE_GRAPH);
		}
	}

} 


void CarapaceApp::CutDialogFrame(int frame_index)
{
	
	CarapaceDialogFrame *dfp;
	wxCarapaceDialogFrameListNode *dnode = DFrameList.Find(frame_index);

	if(dnode != NULL)
	{
		dfp = dnode->GetData();
		DFrameList.DeleteNode(dnode);
		UpdateMenus();
	}

} 


void CarapaceApp::UpdateMenus(bool initial)
{

	CarapaceSplitTextFrame  *stfp;
	wxCarapaceSplitTextFrameListNode *stnode;
	CarapaceGraphFrame  *gfp;
	wxCarapaceGraphFrameListNode *gnode;
	CarapacePanelGraphFrame  *pgfp;
	wxCarapacePanelGraphFrameListNode *pgnode;
	CarapaceDialogFrame  *dfp;
	wxCarapaceDialogFrameListNode *dnode;

	FixPanels();

// FIX ME   We should add a SortByKey member to list that actually keeps the key/content
// linkage.  The current sort breaks the linkage.  
//	GFrameList.SortByKey();
//  	PGFrameList.SortByKey();

	stnode = STFrameList.GetFirst();
	while(stnode)
	{
		stfp = stnode->GetData();
		stnode = stnode->GetNext();
		stfp->UpdateMenuBar();
	}
	Yield();

	gnode = GFrameList.GetFirst();
	while(gnode)
	{
		gfp = gnode->GetData();
		gnode = gnode->GetNext();
		gfp->UpdateMenuBar();
	}
	Yield();

	pgnode = PGFrameList.GetFirst();
	while(pgnode)
	{
		pgfp = pgnode->GetData();
		pgnode = pgnode->GetNext();
		if(pgfp)
		{
			pgfp->UpdateMenuBar();
		}
	}
	Yield();

	dnode = DFrameList.GetFirst();
	while(dnode)
	{
		dfp = dnode->GetData();
		dnode = dnode->GetNext();
		if(dfp)
		{
			dfp->UpdateMenuBar();
		}
	}

} /*CarapaceApp::UpdateMenus(void)*/



//	Add msg to OutLineBuf and flush *OutLineBuf if last character is NL

void CarapaceApp::BufferedPrint(char *msg)
{

	//wxCriticalSectionLocker locker(BufferedOutputLock);

	BufferedOutputLock.Enter();
	/* collect line in OutLineBuf */
	OutLineBuffer << msg;
	OutLineBuffer.Replace("\r\n","\n");
	OutLineBuffer.Replace("\r","\n");
	BufferedOutputLock.Leave();

        if (OutLineBuffer.Length() > OUTLINESIZE ||
                (OutLineBuffer.Length() > 0 && OutLineBuffer.Last() == '\n'))
        {
		FlushBuffer();
        } 


} /*BufferedPrint()*/


void CarapaceApp::ComFlushBuffer(wxCommandEvent& event)
{
	FlushBuffer();
} /*FlushBuffer()*/

void CarapaceApp::FlushBuffer(void)
{
	wxCriticalSectionLocker locker(BufferedOutputLock);

	BufferToWindow(OutLineBuffer);
} /*FlushBuffer()*/


/*
  Write buffer to window.
  interpret leading '\007's as BEL's
*/

#define BEL '\007'

void CarapaceApp::BufferToWindow(wxString& msg)
{
	bool            isOutLine = (msg == OutLineBuffer);
	wxString        outstr;
	long            windLength,msgLength,i=0;
	const char     *cmsg;
	CarapaceSplitTextFrame *stframe;

	if(!OutputWindow)
	{
		return;
	}
	stframe = OutputWindow->Frame;


	msgLength = msg.Length();
	if(msgLength == 0) 
	{
		return;
	}

	cmsg = msg.c_str();

	i = 0;
	while(cmsg[i] == BEL)
	{			/* turn leading BEL's into beeps */
		wxBell();
		i++;
	}
	if(i > 0)
	{
		msg.Remove(0,i);
	}
	msgLength -= i;

	if (msgLength == 0)
	{
		return;
	}

	if (OutputWindow->UndoStatus != CANTUNDO &&
		OutputWindow->UndoStatus != OUTPUTUNDO)
	{
		OutputWindow->clearUndoInfo();
	}
	windLength = OutputWindow->GetLastPosition();
#ifdef __WXGTK__
	OutputWindow->ShowPosition(windLength);
#endif
#ifdef USEAPPENDTEXT
	OutputWindow->SetSelection(windLength,windLength);
	OutputWindow->AppendText(msg.c_str());
#else
	OutputWindow->Replace(windLength,windLength,msg);
#endif /*USEAPPENDTEXT*/
#ifdef __WXMAC__
        // wxMac seems to need this
      //  OutputWindow->SetStyle(0,OutputWindow->GetLastPosition(), 
       //        wxTextAttr(*(stframe->outputFGColor),
        //       *(stframe->outputBGColor),*(stframe->outputFont)));         
#endif /*WXMAC*/
	

	if (isOutLine)
	{
		OutLineBuffer.Empty();
	}
} /*CarapaceApp::BufferToWindow()*/


void CarapaceApp::Interrupt(void)
{
	wxCarapaceGraphFrameListNode *gnode;
	CarapaceStaticGraph *csgp;
	wxString msg = CPC_STRING_INTERRUPT; /*"\n*****  Interrupt  *****\n"*/
	long windLength;

	SwallowOutput = CPC_SWALLOW_NOTHING;
	if(Running != CPC_RUNNING)
	{
		return;
	}
	if(!OutputWindow)
	{
		return;
	}
	windLength = OutputWindow->GetLastPosition();
#ifdef USEAPPENDTEXT
	OutputWindow->SetSelection(windLength,windLength);
	OutputWindow->AppendText(msg.c_str());
#else
	OutputWindow->Replace(windLength,windLength,msg);
#endif /*USEAPPENDTEXT*/
	SwallowOutput = OldSwallowOutput;

	CPC_INTERRUPT = CPC_INTSET; /* set interrupt flag */
	if(MouseRequestState != CPC_SG_NOMOUSEREQUEST)
	{
		MouseRequestState = CPC_SG_MOUSEINTERRUPT;
	}
	gnode = GFrameList.GetFirst();
	while(gnode)
	{
		csgp = gnode->GetData()->Graph;
		if(csgp->MouseRequestState != CPC_SG_NOMOUSEREQUEST)
		{
			csgp->MouseRequestState = CPC_SG_MOUSEINTERRUPT;
		}
		gnode = gnode->GetNext();
	}

	InGraphicsPause = FALSE;
}

int CarapaceApp::NVisible(void)
{
	int visi = 0;
	wxCarapaceSplitTextFrameListNode *stnode;

	stnode = STFrameList.GetFirst();
	while(stnode)
	{
		visi += stnode->GetData()->IsShown();
		stnode = stnode->GetNext();
	}
	return(visi);
}

void  CarapaceApp::TextFrameFront(int windno)
{
	wxCarapaceSplitTextFrameListNode *stnode;
	CarapaceSplitTextFrame *stfp;
	
	stnode = STFrameList.Find(windno);
	if(stnode != NULL)
	{
		stfp = stnode->GetData();
#ifdef __WXGTK__
		// seem to need to hide, then show for focus to work properly
		stfp->Show(FALSE);
#endif //__WXGTK__
		stfp->Show(TRUE);
		UpdateMenus();
		stfp->Raise();

		stfp->SetFocus();
		stfp->command_ctrl->SetFocus();
	} 
} 


void  CarapaceApp::GraphFrameFront(int windno)
{
	wxCarapaceGraphFrameListNode *gnode;
	CarapaceGraphFrame *gfp;
	
	gnode = GFrameList.Find(windno);
	if(gnode != NULL)
	{
		gfp = gnode->GetData();
		gfp->Show(TRUE);
		UpdateMenus();
		gfp->Raise();

		gfp->Graph->SetFocus();
	} 
} 


void  CarapaceApp::PanelFrameFront(int windno)
{
	wxCarapacePanelGraphFrameListNode *pgnode;
	CarapacePanelGraphFrame *pgfp;
	
	pgnode = PGFrameList.Find(windno);
	if(pgnode != NULL)
	{
		pgfp = pgnode->GetData();
		// seem to need to hide, then show for focus to work properly
		if(pgfp)
		{
			pgfp->Show(FALSE);
			pgfp->Show(TRUE);
			UpdateMenus();
			pgfp->Raise();

			pgfp->SetFocus();
		}
	} 
} 


void CarapaceApp::OnIdle(wxIdleEvent& event)
{
	CarapaceCommand *cptr;
	wxCarapaceSplitTextFrameListNode *stnode = NULL;
	CarapaceSplitTextFrame *stframe;


	DynamicDataP->doRedraw();

	stnode = STFrameList.GetFirst();
#ifdef __WXMAC__
	while(stnode)
	{
		// wxMac seems to need this
		stnode->GetData()->output_ctrl->Refresh();
		stnode = stnode->GetNext();
	}
#endif
	if(CList.GetCount() > 0 && Running == CPC_IDLE)
	{
		cptr = (CarapaceCommand *) CList.GetFirst()->GetData();
		CommandLock.Enter();
		CList.DeleteObject(cptr);
		CommandLock.Leave();
		OldSwallowOutput = SwallowOutput;
		SwallowOutput = CPC_SWALLOW_NOTHING;
		if(cptr->hidecommand)
		{
			SwallowOutput = SwallowOutput | CPC_SWALLOW_COMMANDS;
		}
		if(cptr->hideoutput)
		{
			SwallowOutput = SwallowOutput | CPC_SWALLOW_PRINTOUT;
		}
		if( !(SwallowOutput & CPC_SWALLOW_COMMANDS) )
		{
			History->AddCommand(cptr->command);
		}
		
		RunningFrame = cptr->frame;
		stnode = STFrameList.Find(RunningFrame);
		if(stnode)
		{
			SplitRunning = TRUE;
			OutputWindow = stnode->GetData()->output_ctrl;
			stframe = stnode->GetData();
			OutputWindow->UndoStatus = OUTPUTUNDO;
			OutputWindow->LastLength = OutputWindow->GetLastPosition();
			if( !(SwallowOutput & CPC_SWALLOW_COMMANDS) )
			{
				stnode->GetData()->LastCommand = cptr->command;
#ifdef __WXGTK__
				OutputWindow->ShowPosition(OutputWindow->GetLastPosition());
#endif
				OutputWindow->SetInsertionPointEnd();
				OutputWindow->SetSelection(OutputWindow->GetLastPosition(),OutputWindow->GetLastPosition());
				FlushBuffer();
#ifdef USEPUTPROMPT
				OutputWindow->AppendText("\n");
				OutputWindow->AppendText(Preferences->mainPrompt);
#endif /*USEPUTPROMPT*/
				OutputWindow->AppendText(cptr->command);
#ifdef __WXMAC__
				// wxMac seems to need this
        			OutputWindow->SetStyle(0,OutputWindow->GetLastPosition(), 
					wxTextAttr(*(stframe->outputFGColor),
                			*(stframe->outputBGColor),*(stframe->outputFont))); 
#endif /*WXMAC*/


#define COMMANDSTYLE	wxTextAttr(*wxBLACK,*wxWHITE,*stnode->GetData()->commandFont)
#ifdef SETCOMMANDSTYLE
				OutputWindow->SetStyle(OutputWindow->LastLength,
					OutputWindow->GetLastPosition()-1,COMMANDSTYLE);
#endif /*SETCOMMANDSTYLE*/
				if(cptr->command.Last() == '\n')
				{
					stnode->GetData()->LastCommand.RemoveLast();
				}
			}
		}
		Running = CPC_RUNNING;
#ifndef CPC_STANDALONE
		stnode = STFrameList.GetFirst();
		while(stnode)
		{
			stnode->GetData()->RunningMeter->SetLabel(CPC_STRING_RUNNING);
			stnode->GetData()->SetStatusText(cptr->command);
			wxYield();
			stnode = stnode->GetNext();
		}
#endif /*CPC_STANDALONE*/

		CommandFrame = cptr->frame;
		CommandLine = cptr->command;
		ReturnValue =  CPC_EXIT_NOQUIT;

		if(ProcessCommand)
		{
			ReturnValue = ProcessCommand(cptr->command.c_str());
		}
		DynamicDataP->doRedraw();
		delete cptr;
		Running = CPC_DONE;
	}

	if(Running == CPC_DONE)
	{
		Running = CPC_IDLE;
		SwallowOutput = OldSwallowOutput;
		DoQuit(thisApp->ReturnValue);
		if(CPC_INTERRUPT != CPC_INTNOTSET)
		{
			CPC_INTERRUPT = CPC_INTNOTSET;
			if(InterruptCleanUp)
			{
				InterruptCleanUp();
			}
		}
			
		if(CleanUp)
		{
			CleanUp();
		}
#ifndef CPC_STANDALONE
		stnode = STFrameList.GetFirst();
		while(stnode)
		{
			stnode->GetData()->RunningMeter->SetLabel("");
			stnode->GetData()->SetStatusText("");
			stnode = stnode->GetNext();
		}
#endif /*CPC_STANDALONE*/

		wxYield();
		//wxWakeUpIdle();
	}
	event.Skip();
}

void CarapaceApp::GuiProcessCommand(wxCommandEvent& event)
{
	CarapaceCommand *cptr;

	cptr = (CarapaceCommand *) event.GetClientData();
	CommandLock.Enter();
	CList.Append(cptr->frame,cptr);
	CommandLock.Leave();

} /*CarapaceApp::GuiProcessCommand*/

void CarapaceApp::MacOpenFile(const wxString& filename)
{
	AEOpenFileNames.Add(filename);
}

void CarapaceApp::MacPrintFile(const wxString& filename)
{
}

CarapacePreferences::CarapacePreferences(void)
{

// set defaults first

// general
	windowOffset = 10;

// stuff about text windows
	mainPrompt = "Cmd> ";
	textWindowSize[0] = 920;
	textWindowSize[1] = 650;
	tWindowSize.SetWidth(textWindowSize[0]);
	tWindowSize.SetHeight(textWindowSize[1]);
	historyLength = 100;
	textFontSize = 14;
	textFontFamily = wxMODERN;
	textFontFaceName = "";
	textFontWeight = wxNORMAL;
	textFontStyle = wxNORMAL;
	textFontColorName = "BLACK";
	textFontColorVals[0] = textFontColorVals[1] = textFontColorVals[2] = 0;
	textBackgroundColorName = "WHITE";
	textBackgroundColorVals[0] = textBackgroundColorVals[1] = 
		textBackgroundColorVals[2] = 254;
	// pure white sometimes gets ignored
	commandPaneLines = 5;

// stuff about printing
	useA4Paper = FALSE;

// currently none of the remaining preferences is used.  They
// are mostly fossils of earlier coding, but could conceivable
// be resurrected.

// The printerFont stuff is not used because we use whatever
// font the user has chosen for their window.  It might be
// useful to allow users to specify a different sized font
// for printing
	printerFontSize = 11;
	printerFontFamily = wxMODERN;
	printerFontFaceName = "";
	printerFontWeight = wxNORMAL;
	printerFontStyle = wxNORMAL;

// stuff about graph window
// again, not used because everything is set through cpc_sg calls
	graphWindowSize[0] = 800;
	graphWindowSize[1] = 700;
	gWindowSize.SetWidth(graphWindowSize[0]);
	gWindowSize.SetHeight(graphWindowSize[1]);
	graphFontSize = 12;
	graphFontFamily = wxSWISS;
	graphFontFaceName = "";
	graphFontWeight = wxNORMAL;
	graphFontStyle = wxNORMAL;
	graphFontColorName = "BLACK";
	graphFontColorVals[0] = graphFontColorVals[1] = graphFontColorVals[2] = 0;
	graphBackgroundColorName = "WHITE";
	graphBackgroundColorVals[0] = graphBackgroundColorVals[1] = 
		graphBackgroundColorVals[2] = 255;

// stuff about help window
// we have pulled help windows at the moment
	helpWindowSize[0] = 740;
	helpWindowSize[1] = 480;
	hWindowSize.SetWidth(helpWindowSize[0]);
	hWindowSize.SetHeight(helpWindowSize[1]);

}

# define BUFFERLEN 1024
 
static long lookfor( FILE *file, char * keyword)
{
	char buffer[BUFFERLEN];
	char format[30];
	sprintf(format,"%%%ds",BUFFERLEN-1);
 
	fseek(file,0,SEEK_SET);
	while( fscanf(file,format,buffer) != EOF)
	{
		buffer[BUFFERLEN-1] = 0;
		if(strcmp(keyword,buffer) == 0)
		{
			return(1);
		}
	}
	return(0);
}

static bool lookforint(FILE *fp, char *keyword, int *parm)
{
	int dummy;
	if(!lookfor(fp,keyword) || fscanf(fp,"%d",&dummy) < 1)
	{
		return(FALSE);
	}
	*parm = dummy;
	return(TRUE);
}
	
static bool lookforint2(FILE *fp, char *keyword, int *parm1, int *parm2)
{
	int dummy1, dummy2;
	if(!lookfor(fp,keyword) || fscanf(fp,"%d %d",&dummy1,&dummy2) < 2)
	{
		return(FALSE);
	}
	*parm1 = dummy1;
	*parm2 = dummy2;
	return(TRUE);
}
	
static bool lookforint3(FILE *fp, char *keyword, int *parm1, int *parm2, int *parm3)
{
	int dummy1, dummy2, dummy3;
	if(!lookfor(fp,keyword) || fscanf(fp,"%d %d %d",&dummy1,&dummy2,&dummy3) < 3)
	{
		return(FALSE);
	}
	*parm1 = dummy1;
	*parm2 = dummy2;
	*parm3 = dummy3;
	return(TRUE);
}
	
static bool lookforstring(FILE *fp, char *keyword, wxString *parm)
{
	char buffer[BUFFERLEN];
	char format[30];
	sprintf(format,"%%%ds",BUFFERLEN-1);

	if(!lookfor(fp,keyword) || fscanf(fp,format,buffer) < 1)
	{
		return(FALSE);
	}
	buffer[BUFFERLEN-1] = 0;
	*parm = buffer;
	return(TRUE);
}
	
static bool lookforstringeol(FILE *fp, char *keyword, wxString *parm)
{
	int pos;
	char buffer[BUFFERLEN],c;
	char format[30];
	sprintf(format,"%%%ds",BUFFERLEN-1);

	//if(!lookfor(fp,keyword) || fscanf(fp,format,buffer) < 1)
	if(!lookfor(fp,keyword))
	{
		return(FALSE);
	}
	while(isspace(c=fgetc(fp)) && c != '\n' && c != EOF) 
	{
		;
	}
	ungetc(c,fp);

	pos = 0;
	while(pos < BUFFERLEN - 1 && (c = fgetc(fp)) != EOF && c != '\n')
	{
		buffer[pos++] = c;
	}
	buffer[pos] = 0;
	*parm = buffer;
	return(TRUE);
}
	
static bool lookforbool(FILE *fp, char *keyword, bool *parm)
{
	wxString buf;
	if(!lookforstring(fp,keyword,&buf))
	{
		return(FALSE);
	}
	if(buf.Upper() == "TRUE")
	{
		*parm = TRUE;
	}
	else
	{
		*parm = FALSE;
	}
	return(TRUE);
}
	

void CarapacePreferences::SetPreferences(void)
{
	FILE *fp;
	wxString buf;
	int swidth,sheight;

	if(thisApp->PreferencesPath == "")
	{
		return;
	}
	if( (fp = fopen(thisApp->PreferencesPath.GetData(),"r")) == (FILE *) 0)
	{
		CarapaceAlert(CPC_STRING_PREFSFILEERROR);
		return;
	}
	
// general
	lookforint(fp,"windowOffset:",&windowOffset);

// stuff about text windows
	lookforstringeol(fp,"mainPrompt:",&mainPrompt);
	if(lookforint2(fp,"textWindowSize:",textWindowSize,textWindowSize+1))
	{
		tWindowSize.SetWidth(textWindowSize[0]);
		tWindowSize.SetHeight(textWindowSize[1]);
	}
	lookforint(fp,"historyLength:",&historyLength); 
	lookforint(fp,"textFontSize:",&textFontSize); 
	if(lookforstring(fp,"textFontFamily:",&buf))
	{
		if(buf == "MODERN")
			textFontFamily = wxMODERN;
		else if(buf == "ROMAN")
			textFontFamily = wxROMAN;
		else if(buf == "SWISS")
			textFontFamily = wxSWISS;
		else if(buf == "DECORATIVE")
			textFontFamily = wxDECORATIVE;
		else if(buf == "SCRIPT")
			textFontFamily = wxSCRIPT;
	}
	lookforstring(fp,"textFontFaceName:",&textFontFaceName);

	if(lookforstring(fp,"textFontWeight:",&buf))
	{
		if(buf == "NORMAL")
			textFontWeight = wxNORMAL;
		else if(buf == "LIGHT")
			textFontWeight = wxLIGHT;
		else if(buf == "BOLD")
			textFontWeight = wxBOLD;
	}

	if(lookforstring(fp,"textFontStyle:",&buf))
	{
		if(buf == "NORMAL")
			textFontStyle = wxNORMAL;
		else if(buf == "SLANT")
			textFontStyle = wxSLANT;
		else if(buf == "ITALIC")
			textFontStyle = wxITALIC;
	}
	lookforstring(fp,"textFontColorName:",&textFontColorName);
	lookforint3(fp,"textFontColorVals:",textFontColorVals,
		textFontColorVals+1, textFontColorVals+2);
	lookforstring(fp,"textBackgroundColorName:",&textBackgroundColorName);
	lookforint3(fp,"textBackgroundColorVals:",textBackgroundColorVals,
		textBackgroundColorVals+1, textBackgroundColorVals+2);
	lookforint(fp,"commandPaneLines:",&commandPaneLines);

// stuff about help window
	if(lookforint2(fp,"helpWindowSize:",helpWindowSize,helpWindowSize+1))
	{
		hWindowSize.SetWidth(helpWindowSize[0]);
		hWindowSize.SetHeight(helpWindowSize[1]);
	}

// stuff about printing
	lookforbool(fp,"useA4Paper:",&useA4Paper);
	lookforint(fp,"printerFontSize:",&printerFontSize); 
	if(lookforstring(fp,"printerFontFamily:",&buf))
	{
		if(buf == "MODERN")
			printerFontFamily = wxMODERN;
		else if(buf == "ROMAN")
			printerFontFamily = wxROMAN;
		else if(buf == "SWISS")
			printerFontFamily = wxSWISS;
		else if(buf == "DECORATIVE")
			printerFontFamily = wxDECORATIVE;
		else if(buf == "SCRIPT")
			printerFontFamily = wxSCRIPT;
	}
	lookforstring(fp,"printerFontFaceName:",&printerFontFaceName);

	if(lookforstring(fp,"printerFontWeight:",&buf))
	{
		if(buf == "NORMAL")
			printerFontWeight = wxNORMAL;
		else if(buf == "LIGHT")
			printerFontWeight = wxLIGHT;
		else if(buf == "SWISS")
			printerFontWeight = wxBOLD;
	}

	if(lookforstring(fp,"printerFontStyle:",&buf))
	{
		if(buf == "NORMAL")
			printerFontStyle = wxNORMAL;
		else if(buf == "SLANT")
			printerFontStyle = wxSLANT;
		else if(buf == "SWISS")
			printerFontStyle = wxITALIC;
	}

// stuff about graph window
	if(lookforint2(fp,"graphWindowSize:",graphWindowSize,graphWindowSize+1))
	{
		gWindowSize.SetWidth(graphWindowSize[0]);
		gWindowSize.SetHeight(graphWindowSize[1]);
	}
	lookforint(fp,"graphFontSize:",&graphFontSize); 
	if(lookforstring(fp,"graphFontFamily:",&buf))
	{
		if(buf == "MODERN")
			graphFontFamily = wxMODERN;
		else if(buf == "ROMAN")
			graphFontFamily = wxROMAN;
		else if(buf == "SWISS")
			graphFontFamily = wxSWISS;
		else if(buf == "DECORATIVE")
			graphFontFamily = wxDECORATIVE;
		else if(buf == "SCRIPT")
			graphFontFamily = wxSCRIPT;
	}
	lookforstring(fp,"graphFontFaceName:",&graphFontFaceName);

	if(lookforstring(fp,"graphFontWeight:",&buf))
	{
		if(buf == "NORMAL")
			graphFontWeight = wxNORMAL;
		else if(buf == "LIGHT")
			graphFontWeight = wxLIGHT;
		else if(buf == "SWISS")
			graphFontWeight = wxBOLD;
	}

	if(lookforstring(fp,"graphFontStyle:",&buf))
	{
		if(buf == "NORMAL")
			graphFontStyle = wxNORMAL;
		else if(buf == "SLANT")
			graphFontStyle = wxSLANT;
		else if(buf == "SWISS")
			graphFontStyle = wxITALIC;
	}
	lookforstring(fp,"graphFontColorName:",&graphFontColorName);
	lookforint3(fp,"graphFontColorVals:",graphFontColorVals,
		graphFontColorVals+1, graphFontColorVals+2);
	lookforstring(fp,"graphBackgroundColorName:",&graphBackgroundColorName);
	lookforint3(fp,"graphBackgroundColorVals:",graphBackgroundColorVals,
		graphBackgroundColorVals+1, graphBackgroundColorVals+2);

	fclose(fp);

}


void InitializeResourceIDs(void)
{
	int i;
	wxString base,id;

	wxXmlResource::GetXRCID("CPC_MENU_SEPARATOR");
	wxXmlResource::GetXRCID("CPC_MENU_BREAK");

	wxXmlResource::GetXRCID("CPC_FILE_OPEN");
	wxXmlResource::GetXRCID("CPC_FILE_SAVEWINDOW");
	wxXmlResource::GetXRCID("CPC_FILE_SAVEWINDOWAS");
	wxXmlResource::GetXRCID("CPC_FILE_PAGESETUP");
	wxXmlResource::GetXRCID("CPC_FILE_PRINTSELECTION");
	wxXmlResource::GetXRCID("CPC_FILE_INTERRUPT");
	wxXmlResource::GetXRCID("CPC_FILE_GOON");
	wxXmlResource::GetXRCID("CPC_FILE_QUIT");
	wxXmlResource::GetXRCID("CPC_FILE_FASTQUIT");

	wxXmlResource::GetXRCID("CPC_EDIT_UNDO");
	wxXmlResource::GetXRCID("CPC_EDIT_CUT");
	wxXmlResource::GetXRCID("CPC_EDIT_COPY");
	wxXmlResource::GetXRCID("CPC_EDIT_PASTE");
	wxXmlResource::GetXRCID("CPC_EDIT_COPYTOEND");
	wxXmlResource::GetXRCID("CPC_EDIT_DUMPTEXT");
	wxXmlResource::GetXRCID("CPC_EDIT_EXECUTE");
	wxXmlResource::GetXRCID("CPC_EDIT_UPHISTORY");
	wxXmlResource::GetXRCID("CPC_EDIT_DOWNHISTORY");

	wxXmlResource::GetXRCID("CPC_WINDOWS_HIDE");
	wxXmlResource::GetXRCID("CPC_WINDOWS_CLOSE");
	wxXmlResource::GetXRCID("CPC_WINDOWS_NEWWINDOW");
	wxXmlResource::GetXRCID("CPC_WINDOWS_GRAPH");
	wxXmlResource::GetXRCID("CPC_WINDOWS_GOTOTOP");
	wxXmlResource::GetXRCID("CPC_WINDOWS_GOTOEND");
	wxXmlResource::GetXRCID("CPC_WINDOWS_PAGEUP");
	wxXmlResource::GetXRCID("CPC_WINDOWS_PAGEDOWN");
	wxXmlResource::GetXRCID("CPC_WINDOWS_TEXTWINDOW");
	wxXmlResource::GetXRCID("CPC_WINDOWS_TEXTWFONT");
	wxXmlResource::GetXRCID("CPC_WINDOWS_TEXTWFCOLOR");
	wxXmlResource::GetXRCID("CPC_WINDOWS_TEXTWBCOLOR");
	wxXmlResource::GetXRCID("CPC_WINDOWS_GOTOCOMMANDPOINT");
	wxXmlResource::GetXRCID("CPC_WINDOWS_FASTCLOSE");
	wxXmlResource::GetXRCID("CPC_WINDOWS_DIALOGS");
	wxXmlResource::GetXRCID("CPC_WINDOWS_SHOWMATCH");
	wxXmlResource::GetXRCID("CPC_WINDOWS_SHOWERROR");
	wxXmlResource::GetXRCID("CPC_WINDOWS_ZOOMIN");
	wxXmlResource::GetXRCID("CPC_WINDOWS_ZOOMOUT");
	wxXmlResource::GetXRCID("CPC_WINDOWS_FIT");
	wxXmlResource::GetXRCID("CPC_WINDOWS_FITPRESERVE");
	wxXmlResource::GetXRCID("CPC_WINDOWS_SETASPECT");
	wxXmlResource::GetXRCID("CPC_WINDOWS_REVERT");

	wxXmlResource::GetXRCID("CPC_HELP_ABOUT");
	wxXmlResource::GetXRCID("CPC_HELP_HELP");
	wxXmlResource::GetXRCID("CPC_HELP_FORWARD");
	wxXmlResource::GetXRCID("CPC_HELP_BACK");
	wxXmlResource::GetXRCID("CPC_HELP_OPEN");
	wxXmlResource::GetXRCID("CPC_HELP_TOOLBAR");

	wxXmlResource::GetXRCID("CPC_DIALOGS_HISTORYSLIDER");
	wxXmlResource::GetXRCID("CPC_DIALOGS_HISTORYSPINNER");
	wxXmlResource::GetXRCID("CPC_DIALOGS_EXECUTEBUTTON");
	wxXmlResource::GetXRCID("CPC_DIALOGS_COMMANDWINDOW");
	wxXmlResource::GetXRCID("CPC_DIALOGS_OPENMATCHES");
	wxXmlResource::GetXRCID("CPC_DIALOGS_RUNNING");
	wxXmlResource::GetXRCID("CPC_DIALOGS_SYNTAXERROR");

	wxXmlResource::GetXRCID("CPC_GRAPHICS_PAGESETUP");
	wxXmlResource::GetXRCID("CPC_GRAPHICS_PRINT");
	wxXmlResource::GetXRCID("CPC_GRAPHICS_HIDE");
	wxXmlResource::GetXRCID("CPC_GRAPHICS_CLOSE");
	wxXmlResource::GetXRCID("CPC_GRAPHICS_EPSPRINT");

	wxXmlResource::GetXRCID("CPC_GUI_SETHISTORYINDEX");

	base = "CPC_TEXT_WINDOWS_BASE";
	wxXmlResource::GetXRCID("CPC_TEXT_WINDOWS_BASE");
	for(i=0;i<500;i++)
	{
		id = base;
		id << i;
		wxXmlResource::GetXRCID(id.GetData());
	}
	base = "CPC_GRAPH_WINDOWS_BASE";
	wxXmlResource::GetXRCID("CPC_GRAPH_WINDOWS_BASE");
	for(i=0;i<500;i++)
	{
		id = base;
		id << i;
		wxXmlResource::GetXRCID(id.GetData());
	}
	base = "CPC_DIALOG_WINDOWS_BASE";
	wxXmlResource::GetXRCID("CPC_DIALOG_WINDOWS_BASE");
	for(i=0;i<500;i++)
	{
		id = base;
		id << i;
		wxXmlResource::GetXRCID(id.GetData());
	}

}


// The following was written on Feb 26,2005
void CarapaceApp::InitializeDynamic(void)
{
	//DataHash  *InitDataHashP;
	//ViewerHash *InitViewerHashP;

	//InitDataHashP = new DataHash;
	//InitViewerHashP = new ViewerHash;
	//Initialization the DynamicData with a empty DatHash and a ViewerHash
	//this->DynamicDataP = new DynamicInteraction(*InitDataHashP, *InitViewerHashP);

	DynamicDataP = new DynamicInteraction;
	DynamicDataP->data = new DataHash;
	DynamicDataP->viewers = new ViewerHash;
}
// The above was written on Feb 26,2005

// The following was written on March 7,2005
void CarapaceApp::FreeDynamic(void)
{
	delete this->DynamicDataP->data;
	delete this->DynamicDataP->viewers;
	delete this->DynamicDataP;
}
// The above was written on March 7,2005


#define BUFSIZE 4095
char SocketBuffer[BUFSIZE+1];


void CarapaceApp::HandleMainInput(void)
{
	int count,offset;
	wxString bit;
	mainChannelSocket->Read(SocketBuffer,BUFSIZE);
	while((count = mainChannelSocket->LastCount()) > 0)
	{
		SocketBuffer[count] = '\0';
		MainInputBuffer << SocketBuffer;
		mainChannelSocket->Read(SocketBuffer,BUFSIZE);
	}
	
	// deal with inline commands
	// if no command stub and no '\001' in stream, print
	if(MainInputBuffer.Find('\001') < 0 && MainInputCommandBuffer.Length() == 0)
	{
		BufferedPrint((char *) MainInputBuffer.c_str());
		MainInputBuffer.Empty();
		return;
	}
	// if a command stub, try to complete it from buffer
	if(MainInputCommandBuffer.Length() > 0)
	{
		// read to the next '\002', then execute command
		offset = MainInputBuffer.Find('\002');
		if(offset >= 0)
		{
			MainInputCommandBuffer << MainInputBuffer.Left(offset);
			MainInputBuffer = MainInputBuffer.Mid(offset+1);
			MainInputCommandBuffer = MainInputCommandBuffer.Mid(1); // get rid of \001
			// MIB buffer gets emptied in here
			BackChannelCommand(MainInputCommandBuffer);
		}
		else
		{
			MainInputCommandBuffer << MainInputBuffer;
			MainInputBuffer.Empty();
			return;
		}
	}
	// no stubs, now handle the rest
	while((offset = MainInputBuffer.Find('\001')) >= 0)
	{
		// print stuff ahead of command
		bit = MainInputBuffer.Left(offset);
		BufferedPrint((char *) bit.c_str());
		MainInputBuffer = MainInputBuffer.Mid(offset);

		// read to the next '\002', then execute command
		offset = MainInputBuffer.Find('\002');
		if(offset >= 0)
		{
			MainInputCommandBuffer << MainInputBuffer.Left(offset);
			MainInputBuffer = MainInputBuffer.Mid(offset+1);
			MainInputCommandBuffer = MainInputCommandBuffer.Mid(1); // get rid of \1
			// MIB buffer gets emptied in here
			BackChannelCommand(MainInputCommandBuffer);
			MainInputCommandBuffer.Empty();
		}
		else
		{
			MainInputCommandBuffer << MainInputBuffer;
			MainInputBuffer.Empty();
			return;
		}
	}
	if(MainInputBuffer.Length() > 0)
	{
	// anything left can just be printed
		BufferedPrint((char *) MainInputBuffer.c_str());
		MainInputBuffer.Empty();
	}
}

void CarapaceApp::OnSocketEvent(wxSocketEvent& event)
{
	unsigned long count;
	if(event.GetSocketEvent() == wxSOCKET_LOST)
	{
		wxMessageBox("Lost Connection");
		DoQuit(TRUE);
	}
	switch(event.GetId())
	{
		case CPC_SOCKET_MAIN:
			HandleMainInput();
			break;
		case CPC_SOCKET_BACK:
			HandleBackInput();
			break;
		default:
			break;
	}
}

void CarapaceApp::OnServerEvent(wxSocketEvent& event)
{
	switch(event.GetId())
	{
		case CPC_SERVER_MAIN:
			mainChannelSocket = mainChannelServer->Accept(false);
			if(!mainChannelSocket)
			{
				wxMessageBox("Cannot establish socket");
				DoQuit(TRUE);
			}
			mainChannelSocket->SetEventHandler(*this, CPC_SOCKET_MAIN);
			mainChannelSocket->SetNotify(wxSOCKET_LOST_FLAG | wxSOCKET_INPUT_FLAG);
			mainChannelSocket->Notify(true);
			break;
		case CPC_SERVER_BACK:
			backChannelSocket = backChannelServer->Accept(false);
			if(!backChannelSocket)
			{
				wxMessageBox("Cannot establish socket");
				DoQuit(TRUE);
			}
			backChannelSocket->SetEventHandler(*this, CPC_SOCKET_BACK);
			backChannelSocket->SetNotify(wxSOCKET_LOST_FLAG | wxSOCKET_INPUT_FLAG);
			backChannelSocket->Notify(true);
			break;
		case CPC_SERVER_INTR:
			intrChannelSocket = intrChannelServer->Accept(false);
			if(!intrChannelSocket)
			{
				wxMessageBox("Cannot establish socket");
				DoQuit(TRUE);
			}
			intrChannelSocket->SetEventHandler(*this, CPC_SOCKET_INTR);
			intrChannelSocket->SetNotify(wxSOCKET_LOST_FLAG);
			intrChannelSocket->Notify(true);
			break;
		default:
			wxMessageBox("Socket error");
			DoQuit(TRUE);
	}
}

int SendCommands(const char *text)
{
	unsigned int len = strlen(text),sent;
	
	while(len > 0)
	{
		thisApp->mainChannelSocket->Write(text,len);
		sent = thisApp->mainChannelSocket->LastCount();
		len -= sent;
		text += sent;
	}
	return(CPC_EXIT_NOQUIT);
}
