/*
*(C)* The file is part of the source distribution of Carapace 0.1.1
*(C)*
*(C)* Copyright (c) 2000 by Gary Oehlert  and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* This file is distributed under the GNU General Public License
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

#include "cpcplatform.h"

// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/gdicmn.h"
#include "wx/fontdlg.h"
#include "wx/colour.h"
#include "wx/caret.h"
#include "wx/print.h"
#include "wx/printdlg.h"
#include "wx/colordlg.h"
#include "wx/fontmap.h"
#include "wx/splitter.h"
#include "wx/generic/dirctrlg.h"
#include "wx/calctrl.h"
#include "wx/colordlg.h"
#include "wx/cmndata.h"
#include "wx/colour.h"
#include "wx/gdicmn.h"
#include "wx/list.h"
#include "wx/font.h"
#if defined(__WXGTK__)
#include "wx/generic/listctrl.h"
#elif defined(__WXMSW__)
#include "wx/listctrl.h"
#elif defined(__WXMAC__)
#include "wx/listctrl.h"
#endif


#include <math.h>

#include "cpcdefs.h"
#include "carapace.h"
#include "buttoncontrols.h"
#include "cpccontrols.h"

#include "wx/xrc/xmlres.h"
#include "wx/xrc/xh_all.h"
#include "wx/dynarray.h"
#include "wx/arrimpl.cpp"

BEGIN_EVENT_TABLE(CarapaceModalDialog, wxDialog)
EVT_COMMAND_RANGE(6000,7000,wxEVT_COMMAND_BUTTON_CLICKED,CarapaceModalDialog::OnButton)
END_EVENT_TABLE()

extern void cpcColourDialogExtractorBase(wxColourDialog *, wxString& );
extern void cpcFontDialogExtractorBase(wxFontDialog *, wxString& );


CarapaceModalDialog::CarapaceModalDialog()
{
} /*CarapaceModalDialog::CarapaceModalDialog()*/

CarapaceModalDialog::~CarapaceModalDialog()
{
	
} /*CarapaceModalDialog::~CarapaceModalDialog()*/

int CarapaceModalDialog::SetModalDLGFromString(wxWindow * parent, 
		wxString resourcestr, wxString MDLGname)
{
	wxColourDialog *Cdlg;
	wxFontDialog *Fdlg;
	//wxXmlResource *newresource ;
	CarapaceXmlResource *newresource ;
	unsigned long i;
	wxArrayString *LMDN,*LMDV;
	int value;

	// we're going to make a new dialog, so clear any
	// values from the previous dialog stored in the App object
	thisApp->LastModalDialogNames.Clear();
	thisApp->LastModalDialogValues.Clear();
	thisApp->LastModalDialogCount=0;
	LMDN = &(thisApp->LastModalDialogNames);
	LMDV = &(thisApp->LastModalDialogValues);

	//newresource = new wxXmlResource();
	newresource = new CarapaceXmlResource();

	// FIX ME.  ListObjectsFromString makes a list of ALL objects in the
	// resource string.  What we really want is all objects that are children
	// of the object named in MDLGname.  What this means is that we can
	// only specify one modal dialog in a resource string.  Not the end
	// of the world, but inelegant.
	if((i=ListObjectsFromString(resourcestr)))
	{
		return(i);
	}

	if(newresource)
	{
		newresource->InitAllHandlers();
// non-standard XmlResourceHandlers are named in thisApp->RHCreators
// these must be implemented as DYNAMIC_CLASSes
// the ones that Carapace knows about get initialized in
// thisApp->InitializeModalDialogs
		for(i=0;i<thisApp->RHCreators.GetCount();i++)
		{
			wxXmlResourceHandler *rhp;
			rhp = (wxXmlResourceHandler *) wxCreateDynamicObject(thisApp->RHCreators[i]);
			newresource->AddHandler(rhp);
		}
		newresource->LoadFromString(resourcestr);
		for (i=0;i<namelist.GetCount();i++)
		{
			if ((namelist[i].objectclass=="wxDialog")&&(namelist[i].objectname==MDLGname))
			{
				newresource->LoadDialog(this, parent, MDLGname);
				value = ShowModal();
				if(value != wxID_CANCEL)
				{
					MakeOutput(value);
				}
				continue;
			}
			if ((namelist[i].objectclass=="wxColourDialog")&&(namelist[i].objectname==MDLGname))
			{
// the color dialog is done differently right now, because it is
// a dialog and not a control.  That is, it is generated in a different
// way that we cannot recover from the "name".  In particular, a
// dialog is a top level window, and a control is not.
// What we need to do, and what Huixin began, is to implement a
// cpcColourControl as a control version of wxGenericColourDialog
				Cdlg=(wxColourDialog*)newresource->LoadObject(parent, 
					namelist[i].objectname.c_str(), wxT("wxColourDialog"));
				if (Cdlg->ShowModal()==wxID_OK)
				{
					cpcColourDialogExtractorBase(Cdlg,namelist[i].objectname);

					thisApp->LastModalDialogCount = LMDV->GetCount();
						
				}						
				Cdlg->EndModal( wxID_OK );
				continue;
			}
			if ((namelist[i].objectclass=="wxFontDialog")&&(namelist[i].objectname==MDLGname))
			{
// the font dialog is done differently right now, because it is
// a dialog and not a control.  That is, it is generated in a different
// way that we cannot recover from the "name".  In particular, a
// dialog is a top level window, and a control is not.
// What we need to do is to implement a  cpcFontControl as a control 
// version of wxGenericFontDialog
				Fdlg=(wxFontDialog*)newresource->LoadObject(parent, 
					namelist[i].objectname.c_str(), wxT("wxFontDialog"));
				if (Fdlg->ShowModal()==wxID_OK)
				{
					cpcFontDialogExtractorBase(Fdlg,namelist[i].objectname);

					thisApp->LastModalDialogCount = LMDV->GetCount();
				}
				Fdlg->EndModal( wxID_OK );
				continue;
			}
		}
	}
	return(CPC_NOERROR);
}

#if 0
static void getNVPairs(wxString& source, wxString& name, wxString& value)
{
	unsigned int firstsp,firsteq,firstqu;

	name = "";
	value = "";
	source.Trim();
	source.Trim(FALSE);
	if(source.Length() == 0)
	{
		return;
	}
	firstsp = source.Find(" ");
	firsteq = source.Find("=");
	if(firstsp < 0 && firsteq < 0)
	{ // all "name"
		name = source;
		source = "";
		return;
	}
	if(firsteq < 0)
	{ // no =, so everything to first space is name and no value
		name = source.Left(firstsp);
		source = source.Mid(firstsp+1);
		source.Trim(FALSE);
		return;
	}
	if(firstsp < 0 || firsteq < firstsp)
	{ // no space, or = before space, so everything to = is name
		name = source.Left(firsteq);
		source = source.Mid(firsteq); // leave in =
		source.Trim(FALSE);
	}
	else
	{ // here we have space and =, but space comes before =
		name = source.Left(firstsp);
		source = source.Mid(firstsp); // leave in space
		source.Trim(FALSE);
	}
	// at this point, if there is a leading =, we have a value,
	// if the leading char is not =, then we do not have a value
	if(source.Left(1) != "=")
	{
		return;
	}
	// finally, look for value.  It should be enclosed in quotes,
	// but just in case, we'll go to first space if not quoted
	source = source.Mid(1);
	source.Trim(FALSE);
	if(source.Left(1) != "\"")
	{
		// not quoted
		firstsp = source.Find(" ");
		if(firstsp < 0)
		{
			value = source;
			source = "";
			return;
		}
		value = source.Left(firstsp);
		source = source.Mid(firstsp);
		source.Trim(FALSE);
		return;
	}
	// now we look for a quoted value.  There should be a trailing quote,
	// but we'll accept to the end of the source string if no trailing quote
	source = source.Mid(1);
	firstqu = source.Find('"');
	if(firstqu < 0)
	{
		value = source;
		source = "";
		return;
	}
	// CHECK ME Do we need to look for protected quotes here?
	value = source.Left(firstqu);
	source = source.Mid(firstqu+1);
	source.Trim(FALSE);
	return;
}
#endif //0

// This is GWO's reimplementation of Huixin's ListObjectsFromString
// Huixin handwrote what looked like output from yacc.  It seemed
// to work, but didn't have any error checking and was pretty difficult
// to follow though its various states.  I think that this is a little
// more straightforward
#if 0
int CarapaceModalDialog::ListObjectsFromString(wxString resourcestr)
{
	long i,j;
	wxString name,value,leftstr,rightstr,mycopy;

	mycopy = resourcestr; // just in case

	// first strip all comments
	while((i=mycopy.Find("<!--")) >= 0)
	{
		leftstr = mycopy.Left(i);
		rightstr = mycopy.Mid(i);
		j = rightstr.Find("-->");
		if(j > 0)
		{
			mycopy = leftstr;
			mycopy << rightstr.Mid(j+3);
		}
		else
		{
			return(CPC_ERROR_XMLERROR);
		}
	}
	// change all "white space" to space.  We can do this
	// because we are only interested in names and classes inside
	// of <object name="..." class="..."> constructs
	mycopy.Replace("\t"," ");
	mycopy.Replace("\n"," ");
	mycopy.Replace("\r"," ");

	while(mycopy.Length() > 0)
	{
		i=mycopy.Find("<");  
		if(i < 0)
		{
			// it's possible there is an xml error here, but we've missed it
			return(CPC_NOERROR);
		}
		mycopy = mycopy.Mid(i+1);
		j = mycopy.Find(">");
		if(j < 0)
		{
			return(CPC_ERROR_XMLERROR);
		}
		leftstr = mycopy.Left(j);
		mycopy = mycopy.Mid(j+1);

		// leftstr has what is inside the angle brackets, we only care
		// if it starts with object, but we'll strip leading space just in case
		leftstr.Trim(FALSE);
		if(leftstr.Find("object") != 0)
		{
			continue;
		}
		leftstr = leftstr.Mid(6);
		// OK, now we have content of an object member, and we're looking for
		// class and possibly name elements
		namelist.Add(DialogObjectInfo());
		while(leftstr.Length() > 0)
		{
			getNVPairs(leftstr,name,value);
			if(name == "class")
			{
				namelist.Last().objectclass = value;
			}
			if(name == "name")
			{
				for(i=0;i<namelist.GetCount();i++)
				{
					if(namelist.Item(i).objectname == value)
					{
						wxString msg;
						msg << "Xml object name " << value << " is used more than once";
						wxMessageBox(msg);
					}
				}
				namelist.Last().objectname = value;
			}
		
		}
	}
	return(CPC_NOERROR);
}
#endif


int CarapaceModalDialog::ListObjectsFromString(wxString resourcestr)
{
	long i,j;
	wxString name,value,leftstr,rightstr,mycopy;

	mycopy = resourcestr; // just in case

	// first strip all comments
	while((i=mycopy.Find("<!--")) >= 0)
	{
		leftstr = mycopy.Left(i);
		rightstr = mycopy.Mid(i);
		j = rightstr.Find("-->");
		if(j > 0)
		{
			mycopy = leftstr;
			mycopy << rightstr.Mid(j+3);
		}
		else
		{
			return(CPC_ERROR_XMLERROR);
		}
	}
	// change all "white space" to space.  We can do this
	// because we are only interested in names and classes inside
	// of <object name="..." class="..."> constructs
	//mycopy.Replace("\t"," ");
	//mycopy.Replace("\n"," ");
	//mycopy.Replace("\r"," ");

	while((i=mycopy.Find("<object")) > 0)
	{
		mycopy = mycopy.Mid(i+6);
		j = mycopy.Find(">");
		if(j < 0)
		{
			return(CPC_ERROR_XMLERROR);
		}
		leftstr = mycopy.Left(j);
		mycopy = mycopy.Mid(j+1);

		// OK, now we have content of an object member, and we're looking for
		// class and possibly name elements
		namelist.Add(DialogObjectInfo());
		while(leftstr.Length() > 0)
		{
			getNVPairs(leftstr,name,value);
			if(name == "class")
			{
				namelist.Last().objectclass = value;
			}
			if(name == "name")
			{
				for(i=0;i<(long)namelist.GetCount();i++)
				{
					if(namelist.Item(i).objectname == value)
					{
						wxString msg;
						msg << "Xml object name " << value << " is used more than once";
						wxMessageBox(msg);
					}
				}
				namelist.Last().objectname = value;
			}
		
		}
	}
	return(CPC_NOERROR);
}

#define MDOBJ(type) (XRCCTRL(* this, namelist.Item(j).objectname,type))

int CarapaceModalDialog::MakeOutput(int value)
{
	long j;
	long listlength=namelist.GetCount();
	wxString StepResult;
	

// what we have is a hash map at the App level keyed
// on the object class.  The content of the map is a pointer to a
// function that extracts the desired content from the control and
// puts it into the App level output string.  We can easily "register"
// new kinds of controls
	for (j=0;j<listlength;j++)
	{
		MDValueExtractor mdvp;
		mdvp = thisApp->MDVExtractors[namelist.Item(j).objectclass];
		if(mdvp != NULL)
		{
			mdvp(this,namelist.Item(j).objectname);
		}
	}
	thisApp->LastModalDialogNames.Add("ReturnValue");
	wxString tmp;
	tmp << value;
	thisApp->LastModalDialogValues.Add(tmp);
	thisApp->LastModalDialogCount = thisApp->LastModalDialogNames.GetCount();
	return(thisApp->LastModalDialogCount);
}

void CarapaceModalDialog::OnButton(wxCommandEvent& ev)
{
	if (Validate() && TransferDataFromWindow())
	{
		EndModal(ev.GetId());
	}
}

IMPLEMENT_DYNAMIC_CLASS(cpcColourDialogXmlHandler, wxXmlResourceHandler)
cpcColourDialogXmlHandler::cpcColourDialogXmlHandler() :
	wxXmlResourceHandler()
{

}

cpcColourDialogXmlHandler::~cpcColourDialogXmlHandler()
{

}

wxObject * cpcColourDialogXmlHandler::DoCreateResource()
{
	wxColour col;//myCanvas->GetBackgroundColour();

	wxColourData data;
	data.SetColour(col);
	data.SetChooseFull(TRUE);
	for (int i = 0; i < 16; i++)
	{
		wxColour colour(i*16, i*16, i*16);
		data.SetCustomColour(i, colour);
	}
	
	wxColourDialog *dialog=new wxColourDialog(m_parentAsWindow, &data);
	dialog->SetTitle(_T("Choose the background colour"));
	return(dialog);
};

bool cpcColourDialogXmlHandler::CanHandle(wxXmlNode *node)
{
	return (IsOfClass(node, wxT("wxColourDialog")));

};

IMPLEMENT_DYNAMIC_CLASS(cpcFontDialogXmlHandler, wxXmlResourceHandler)
cpcFontDialogXmlHandler::cpcFontDialogXmlHandler() :
	wxXmlResourceHandler()
{

}

cpcFontDialogXmlHandler::~cpcFontDialogXmlHandler()
{

}

wxObject * cpcFontDialogXmlHandler::DoCreateResource()
{
	wxFontData data;
	wxFontDialog *dialog=new wxFontDialog((wxWindow *)m_parentAsWindow, data);
	dialog->SetTitle(_T("Choose the font"));
	return(dialog);
};

bool cpcFontDialogXmlHandler::CanHandle(wxXmlNode *node)
{
	return (IsOfClass(node, wxT("wxFontDialog")));

};

IMPLEMENT_DYNAMIC_CLASS(cpcInOutCtrlXmlHandler, wxXmlResourceHandler)
cpcInOutCtrlXmlHandler::cpcInOutCtrlXmlHandler() :
	wxXmlResourceHandler()
{
	m_insideCtrl = FALSE;
	m_insideItems = FALSE;
	m_inattrlist = FALSE;
	m_item.Label = "";
	m_item.LogicAttr.Empty();
	m_itemlist.Empty();
}

cpcInOutCtrlXmlHandler::~cpcInOutCtrlXmlHandler()
{

}

wxObject * cpcInOutCtrlXmlHandler::DoCreateResource()
{
	unsigned int i;
	wxString resname; 
	if(m_class == "cpcInOutCtrl")
	{
		m_insideCtrl = TRUE;
		resname = GetName();
		CreateChildrenPrivately(NULL, GetParamNode("attrnames"));	
		m_fontsize = GetLong("size",10);
		m_boxheight = GetLong("boxheight",100);
		m_border = GetLong("border",5);
		cpcInOutCtrl *ctrl = new cpcInOutCtrl(m_parentAsWindow,m_labels,
			m_checked,m_fontsize,m_border,m_boxheight,
			wxXmlResource::GetXRCID(resname));
		m_insideItems = TRUE;
		CreateChildrenPrivately(NULL, GetParamNode("content"));	
		m_insideItems = FALSE;
		m_insideCtrl = FALSE;
		for(i=0;i<m_itemlist.Count();i++)
		{
			ctrl->AddItem(m_itemlist[i]);
		}
		m_itemlist.Empty();
		ctrl->MaxSelected = GetLong("maxselected",0);
		ctrl->ResetSubset();
		return(ctrl);
	}
	else if(m_insideCtrl && !m_insideItems)
	{
		// here we are getting attribute names
		//wxString str = GetNodeContent(m_node);
		wxString str = GetText("name");
		m_labels.Add(str);
		m_checked.Add(GetBool("checked",FALSE));
		return NULL;
	}
	else if(m_insideCtrl && m_insideItems && !m_inattrlist)
	{
		// here we are staring an item
		m_item.Label = GetText("label");
		m_inattrlist = TRUE;
		CreateChildrenPrivately(NULL, GetParamNode("attributes"));
		m_inattrlist = FALSE;
		m_item.Dimension = GetLong("dimension",0);
		m_itemlist.Add(m_item);
		m_item.LogicAttr.Empty();
		return NULL;
	}
	else
	{
		// here we are getting an attribute
		wxString strval = GetNodeContent(m_node);
		strval.Trim(FALSE);strval.Trim(TRUE);
		if(strval == "0")
		{
			m_item.LogicAttr.Add(0);
		}
		else
		{
			m_item.LogicAttr.Add(1);
		}
		return NULL;
	}
};

bool cpcInOutCtrlXmlHandler::CanHandle(wxXmlNode *node)
{
	return (IsOfClass(node, wxT("cpcInOutCtrl")) ||
		m_insideCtrl || m_insideItems || m_inattrlist);

};

