/*
*(C)* The file is part of the source distribution of Carapace 0.1.1
*(C)*
*(C)* Copyright (c) 2000 by Gary Oehlert  and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* This file is distributed under the GNU General Public License
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

#include "cpcplatform.h"

// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/gdicmn.h"
#include "wx/fontdlg.h"
#include "wx/colour.h"
#include "wx/caret.h"
#include "wx/print.h"
#include "wx/printdlg.h"
#include "wx/colordlg.h"
#include "wx/fontmap.h"
#include "wx/splitter.h"
#include "wx/generic/dirctrlg.h"
#include "wx/calctrl.h"
#include "wx/colordlg.h"
#include "wx/cmndata.h"
#include "wx/colour.h"
#include "wx/gdicmn.h"
#include "wx/list.h"
#include "wx/font.h"
#if defined(__WXGTK__)
#include "wx/generic/listctrl.h"
#elif defined(__WXMSW__)
#include "wx/listctrl.h"
#elif defined(__WXMAC__)
#include "wx/listctrl.h"
#endif


#include <math.h>

#include "cpcdefs.h"
#include "carapace.h"
#include "buttoncontrols.h"
#include "cpccontrols.h"

#include "wx/xrc/xmlres.h"
#include "wx/xrc/xh_all.h"
#include "wx/dynarray.h"
#include "wx/arrimpl.cpp"




BEGIN_EVENT_TABLE(CarapaceNonModalFrame, wxFrame)
EVT_MENU_RANGE(1,2000000000, CarapaceNonModalFrame::ProcessUserCommand)
EVT_CLOSE(CarapaceNonModalFrame::OnClose)
END_EVENT_TABLE()


CarapaceNonModalFrame::CarapaceNonModalFrame()
{
	OK = FALSE;
} /*CarapaceNonModalFrame::CarapaceNonModalFrame()*/

CarapaceNonModalFrame::~CarapaceNonModalFrame()
{
	
} /*CarapaceNonModalFrame::~CarapaceNonModalFrame()*/

bool CarapaceNonModalFrame::isOK(void)
{
	return OK;
}

int CarapaceNonModalFrame::SetDLGFromString(wxWindow * parent, 
		wxString resourcestr, wxString DLGname)
{
	CarapaceXmlResource *newresource, *mresource ;
	wxMenuBar *bar;
	unsigned long i;
	wxString reducedresource;

	//newresource = new wxXmlResource();
	newresource = new CarapaceXmlResource();
	mresource = new CarapaceXmlResource();
	GetXmlSkeleton(resourcestr,reducedresource,"wxFrame",DLGname);
	if(reducedresource.Length() == 0)
	{
		return(CPC_ERROR_NODIALOGINRESOURCE);
	}

	if((i=ListObjectsFromString(reducedresource)))
	{
		return(CPC_ERROR_XMLERROR);
	}

	if(newresource)
	{
		newresource->InitAllHandlers();
// non-standard XmlResourceHandlers are named in thisApp->RHCreators
// these must be implemented as DYNAMIC_CLASSes
// the ones that Carapace knows about get initialized in
// thisApp->InitializeModalDialogs
		for(i=0;i<thisApp->RHCreators.GetCount();i++)
		{
			wxXmlResourceHandler *rhp;
			rhp = (wxXmlResourceHandler *) wxCreateDynamicObject(thisApp->RHCreators[i]);
			newresource->AddHandler(rhp);
		}
		newresource->LoadFromString(resourcestr);
		if(!(newresource->LoadFrame(this, parent, DLGname)))
		{
			return(CPC_ERROR_XMLERROR);
		}
		thisApp->DynamicDataP->updateByData();
		{
			// we don't know if the user has a menubar under the default
			// name in the resource.  we try it, but temporarily kill
			// error logging
			wxLogNull logNo;
			newresource->AddHandler(new wxMenuXmlHandler);
			newresource->AddHandler(new cpcMenuXmlHandler);
			newresource->AddHandler(new wxMenuBarXmlHandler);
			bar = newresource->LoadMenuBar(DLGname);
		}
		if(!bar && mresource)
		{
			wxString defres = CPC_STRING_DEFAULTDIALOGRESOURCE;
			wxString defname = CPC_STRING_DEFAULTDIALOGMENUNAME;
			mresource->LoadFromString(defres);
			mresource->AddHandler(new wxMenuXmlHandler);
			mresource->AddHandler(new cpcMenuXmlHandler);
			mresource->AddHandler(new wxMenuBarXmlHandler);
			bar = mresource->LoadMenuBar(defname);
		}
		if(bar)
		{
			SetMenuBar(bar);
		}
		OK = TRUE;
		// now we need to set the container of each viewer to this
		for(i = 0; i < namelist.GetCount(); i++)
		{
			Viewer *view;
			view  = (*(thisApp->DynamicDataP->viewers))[namelist[i]];
			if(view)
			{
				view->Container = this;
			}
		}
		Fit();
		Show();
		return(CPC_NOERROR);
	}
	return(CPC_ERROR_XMLERROR);
}


// This is GWO's reimplementation of Huixin's ListObjectsFromString
// Huixin handwrote what looked like output from yacc.  It seemed
// to work, but didn't have any error checking and was pretty difficult
// to follow though its various states.  I think that this is a little
// more straightforward

int CarapaceNonModalFrame::ListObjectsFromString(wxString resourcestr)
{
	long i,j;
	unsigned long k;
	wxString name,value,leftstr,rightstr,mycopy,thisclass,thisname;

	mycopy = resourcestr; // just in case


	while(mycopy.Length() > 0)
	{
		i=mycopy.Find("<");  
		if(i < 0)
		{
			// it's possible there is an xml error here, but we've missed it
			return(CPC_NOERROR);
		}
		mycopy = mycopy.Mid(i+1);
		j = mycopy.Find(">");
		if(j < 0)
		{
			return(CPC_ERROR_XMLERROR);
		}
		leftstr = mycopy.Left(j);
		mycopy = mycopy.Mid(j+1);

		// leftstr has what is inside the angle brackets, we only care
		// if it starts with object, but we'll strip leading space just in case
		leftstr.Trim(FALSE);
		if(leftstr.Find("object") != 0)
		{
			continue;
		}
		leftstr = leftstr.Mid(6);
		// OK, now we have content of an object member, and we're looking for
		// class and possibly name elements
		thisname = "";
		thisclass = "";
		while(leftstr.Length() > 0)
		{
			getNVPairs(leftstr,name,value);
			if(name == "class")
			{
				thisclass = value;
			}
			if(name == "name")
			{
				thisname = value;
			}
		}
		for(k=0;k<thisApp->ViewerClassNames.Count();k++)
		{
			if(thisclass == thisApp->ViewerClassNames[k])
			{
				namelist.Add(thisname);
				typelist.Add(thisclass);
			}
		}
	}
	return(CPC_NOERROR);
}

void CarapaceNonModalFrame::DoClose(void)
{
	wxCloseEvent ev;
	OnClose(ev);
}

void CarapaceNonModalFrame::OnClose(wxCloseEvent & event)
{
	unsigned int i;
	wxWindow *view;
	ShowFileViewer *sfview;
	for(i=0;i<namelist.GetCount();i++)
	{
		if(typelist[i] == "ShowFileViewer")
		{
			sfview  = (ShowFileViewer *) 
			    (*(thisApp->DynamicDataP->viewers))[namelist[i]];
			sfview->DoClose();
		}
		thisApp->DynamicDataP->delistViewer(namelist[i]);
	}
	thisApp->CutDialogFrame(Index);
	event.Skip();
}


void CarapaceDialogFrame::UpdateMenuBar(void)
{

        // here we update windows, graphs, and user menus

        wxMenuBar *thisMenuBar = GetMenuBar();

        FixCommandSubmenu(thisMenuBar);
        FixGraphSubmenu(thisMenuBar);
        FixDialogSubmenu(thisMenuBar);


} /* CarapaceDialogFrame::UpdateMenuBar() */


void CarapaceNonModalFrame::OnQuit(wxCommandEvent & WXUNUSED(event))
{
	thisApp->DoQuit(CPC_EXIT_SAVECANCANCEL);
}

#define DO_EVT_MENU(id,callback,ev)   \
        if(which == wxXmlResource::GetXRCID(#id))  \
        { \
                callback(ev); \
                return; \
        }



void CarapaceNonModalFrame::ProcessUserCommand(wxCommandEvent& event)
{
        CarapaceCommand *cptr;
        cpcMenuItem *menuItem;
        wxString base,full;
        int i;

        int which = event.GetId();

        //DO_EVT_MENU(CPC_FILE_SAVEWINDOWAS, CarapacePanelGraphFrame::SaveWindowAs,)
        //DO_EVT_MENU(CPC_EDIT_COPY, CarapacePanelGraphFrame::OnCopy,event)
        //DO_EVT_MENU(CPC_FILE_PAGESETUP, CarapacePanelGraphFrame::OnPageSetup,event)
        //DO_EVT_MENU(CPC_FILE_PRINTSELECTION, CarapacePanelGraphFrame::OnPrint,event)
        //DO_EVT_MENU(CPC_FILE_INTERRUPT, CarapacePanelGraphFrame::MenuInterrupt,event)
        DO_EVT_MENU(CPC_FILE_QUIT, CarapaceNonModalFrame::OnQuit,event)


        base = "CPC_TEXT_WINDOWS_BASE";
        for(i=0;i<thisApp->numTextFrames;i++)
        {
                full = base;
                full << i;
                if(which == wxXmlResource::GetXRCID(full.GetData()) )
                {
                        thisApp->TextFrameFront(thisApp->STFrameList.Item(i)->GetData()->TextFrameIndex);
                        return;
                }
        }


        base = "CPC_GRAPH_WINDOWS_BASE";
        for(i=0;i<thisApp->numGraphFrames;i++)
        {
                full = base;
                full << i;
                if(which == wxXmlResource::GetXRCID(full.GetData()) )
                {
                        thisApp->GraphFrameFront(thisApp->GFrameList.Item(i)->GetData()->GraphFrameIndex);
                        return;
                }
        }
        for(i=0;i<thisApp->numGraphFrames;i++)
        {
                full = base;
                full << (i+thisApp->numGraphFrames);
                if(which == wxXmlResource::GetXRCID(full.GetData()) )
                {
                        thisApp->PanelFrameFront(thisApp->PGFrameList.Item(i)->GetData()->PanelFrameIndex);
                        return;
                }
        }

	
	base = "CPC_DIALOG_WINDOWS_BASE";
	for(i=0;i<thisApp->DFrameList.GetCount();i++)
	{
       		full = base;
       		full << i;
		if(which == wxXmlResource::GetXRCID(full.GetData()) )
		{
			//thisApp->TextFrameFront(thisApp->STFrameList.Item(i)->GetData()->TextFrameIndex);
			thisApp->DFrameList.Item(i)->GetData()->Raise();
			return;
		}
	}



        menuItem = (cpcMenuItem *) GetMenuBar()->FindItem(which);
        if(!menuItem || menuItem->action == "")
        {
                return;
        }

        wxCommandEvent procevent(wxEVT_COMMAND_MENU_SELECTED,CPC_GUI_PROCESSCOMMAND);
        cptr = new CarapaceCommand;
        cptr->command = menuItem->action;
        cptr->command << "\n";

// commands should come from some command frame, so in a dialog frame,
// we use the first command frame.
        wxCarapaceSplitTextFrameListNode *stnode = thisApp->STFrameList.GetFirst();
        CarapaceSplitTextFrame *stfp = stnode->GetData();

        cptr->frame = stfp->TextFrameIndex;

        cptr->hidecommand = menuItem->hidecommand;
        cptr->hideoutput = menuItem->hideoutput;
        procevent.SetClientData((void *) cptr);
        wxPostEvent(thisApp,procevent);
}

