/*
*(C)* The file is part of the source distribution of Carapace
*(C)* version 0.5 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* This file is distributed under the GNU General Public License
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

#include "cpcplatform.h"



// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/print.h"
#include "wx/printdlg.h"
#include "wx/generic/dcpsg.h"
#include "wx/clipbrd.h"


#include <math.h>

#include "carapace.h"

#undef UNDEFINED


#include "wx/image.h"
#include "wx/listctrl.h"
#include "wx/sizer.h"
#include "wx/log.h"


#include "wx/xrc/xmlres.h"
#include "wx/xrc/xh_all.h"



// CarapacePanelGraphFrame


IMPLEMENT_DYNAMIC_CLASS(CarapacePanelGraphFrame, wxFrame)

// event table for graph frames

BEGIN_EVENT_TABLE(CarapacePanelGraphFrame, wxFrame)
        EVT_MENU_RANGE(1,2000000000, CarapacePanelGraphFrame::ProcessUserCommand)
#if 0
        EVT_UPDATE_UI_RANGE(1,2000000000, CarapacePanelGraphFrame::UpdateUserCommand)
#endif

	EVT_CLOSE(CarapacePanelGraphFrame::OnClose)

	//EVT_CHAR(wxLayoutWindow::OnChar)
END_EVENT_TABLE()




CarapacePanelGraphFrame::CarapacePanelGraphFrame()
       //: wxFrame( (wxFrame *)NULL, -1, "sample", wxPoint(20,20), wxSize(200,200), wxFRAME_NO_TASKBAR )
       : wxFrame( (wxFrame *)NULL, -1, "sample", wxPoint(20,20), wxSize(200,200))
{
        //Resource = new wxXmlResource();
        Resource = new CarapaceXmlResource();
        if(Resource)
        {
                Resource->AddHandler(new wxMenuXmlHandler);
                Resource->AddHandler(new cpcMenuXmlHandler);
                Resource->AddHandler(new wxMenuBarXmlHandler);
        }

	Accelerators = new ArrayofAccelerators;


    Show(FALSE);
}

CarapacePanelGraphFrame::CarapacePanelGraphFrame(const wxString& title, 
		const wxPoint& offset, const wxSize& size, long panelno)
       : wxFrame( (wxFrame *)NULL, -1, title, offset, size )
{
	wxString defres = CPC_STRING_DEFAULTGRAPHRESOURCE;
	wxString defname = CPC_STRING_DEFAULTGRAPHMENUNAME;

	PanelFrameIndex = panelno;

        //Resource = new wxXmlResource();
        Resource = new CarapaceXmlResource();
        if(Resource)
        {
                Resource->AddHandler(new wxMenuXmlHandler);
                Resource->AddHandler(new cpcMenuXmlHandler);
                Resource->AddHandler(new wxMenuBarXmlHandler);
        }

	Accelerators = new ArrayofAccelerators;


	DoMenuBar(defres,defname);
		

	CreateStatusBar(1);

	SetClientSize(size);
	wxPoint pos = wxPoint();
	Graph = new CarapacePanelGraph( this, -1, pos, size );
	Graph->SetScrollbars( 10, 10, (long) (size.GetWidth()/10), (long) (size.GetHeight()/10) );
  
	Show(TRUE);
	Lower();
}

CarapacePanelGraphFrame::CarapacePanelGraphFrame(const wxString& title, 
		const wxPoint& offset, const wxSize& size, long panelno,
		wxString& resource, wxString& menuname)
       : wxFrame( (wxFrame *)NULL, -1, title, offset, size )
{
	wxString defres = CPC_STRING_DEFAULTGRAPHRESOURCE;
	wxString defname = CPC_STRING_DEFAULTGRAPHMENUNAME;

	PanelFrameIndex = panelno;

        //Resource = new wxXmlResource();
        Resource = new CarapaceXmlResource();
        if(Resource)
        {
                Resource->AddHandler(new wxMenuXmlHandler);
                Resource->AddHandler(new cpcMenuXmlHandler);
                Resource->AddHandler(new wxMenuBarXmlHandler);
        }

	Accelerators = new ArrayofAccelerators;


	if(resource != "" && menuname != "")
	{
		DoMenuBar(resource,menuname);
	}
	else
	{
		DoMenuBar(defres,defname);
	}
		

	CreateStatusBar(1);

	SetClientSize(size);
	wxPoint pos = wxPoint();
	Graph = new CarapacePanelGraph( this, -1, pos, size );
	Graph->SetScrollbars( 10, 10, (long) (size.GetWidth()/10), (long) (size.GetHeight()/10) );
  
	Show(TRUE);
	Lower();
}

void CarapacePanelGraphFrame::OnClose(wxCloseEvent& event)
{
	Show(FALSE);
}


CarapacePanelGraphFrame::~CarapacePanelGraphFrame()
{
}



void CarapacePanelGraphFrame::DoMenuBar(wxString& resource, wxString& menuname)
{

	wxString menuName;
	wxString defres = CPC_STRING_DEFAULTGRAPHRESOURCE;
	wxString defname = CPC_STRING_DEFAULTGRAPHMENUNAME;

	if(resource != "" && menuname != "")
	{
		SetMenuBarFromString(resource,menuname);
	}
	else
	{
		SetMenuBarFromString(defres,defname);
	}

	UpdateMenuBar();
}

void CarapacePanelGraphFrame::UpdateMenuBar(void)
{
		
	// here we update windows, graphs, and user menus

	wxMenuBar *thisMenuBar = GetMenuBar();

	FixCommandSubmenu(thisMenuBar);
	FixGraphSubmenu(thisMenuBar);
	FixDialogSubmenu(thisMenuBar);

} /* CarapacePanelGraphFrame::UpdateMenuBar() */


void CarapacePanelGraphFrame::SetMenuBarFromString(wxString resourcestr, wxString menuname)
{
        wxMenuBar *bar;
	//wxXmlResource *newresource ;
	CarapaceXmlResource *newresource ;

	//newresource = new wxXmlResource();
	newresource = new CarapaceXmlResource();
	if(newresource)
        {
		newresource->AddHandler(new wxMenuXmlHandler);
		newresource->AddHandler(new cpcMenuXmlHandler);
		newresource->AddHandler(new wxMenuBarXmlHandler);
		
		newresource->LoadFromString(resourcestr);
		bar = newresource->LoadMenuBar(menuname.c_str());
		if(bar)
		{
			SetMenuBar(bar);
			ResourceString = resourcestr;
			MenuResourceInUse = menuname;
			if(Resource)
			{
				delete Resource;
			}
			Resource = newresource;

			if(Accelerators)
			{
				delete(Accelerators);
			}
			Accelerators = new ArrayofAccelerators;
			if(Accelerators)
			{
				GetAccelArrayFromMenuBar(*Accelerators,bar);
			}
		}
	}
}



wxMenuBar * CarapacePanelGraphFrame::GetDefaultMenuBar(void)
{
	wxMenu *fileMenu = new wxMenu();
	wxMenu *windMenu = new wxMenu();
	wxMenu *w1 = new wxMenu();
	wxMenu *w2 = new wxMenu();
	wxMenu *w3 = new wxMenu();
	wxMenuBar *bar = new wxMenuBar();

	if(!fileMenu || !windMenu || !bar)
	{
		return(NULL);
	}

	fileMenu->Append(CPC_FILE_SAVEWINDOWAS, "Save graph as ...",
         "Save the contents of the graph window");
	fileMenu->AppendSeparator();
	fileMenu->Append(CPC_FILE_PAGESETUP, "Page setup", "Printing setup");
	fileMenu->Append(CPC_FILE_PRINTSELECTION, "&Print window", "Print this graph");
	fileMenu->AppendSeparator();
	fileMenu->Append(CPC_FILE_INTERRUPT, "&Interrupt\tCtrl+I", "Interrupt processing");
	fileMenu->AppendSeparator();
	fileMenu->Append(CPC_FILE_QUIT, "&Quit\tCtrl+Q", "Quit");

	w1->Append(CPC_WINDOWS_HIDE, "&Hide", "Hide window");
	w2->Append(CPC_WINDOWS_HIDE, "&Hide", "Hide window");
	w3->Append(CPC_WINDOWS_HIDE, "&Hide", "Hide window");
	windMenu->Append(CPC_WINDOWS_HIDE, "&Hide", "Hide window");
	windMenu->AppendSeparator();
	windMenu->Append(CPC_WINDOWS_ZOOMIN, "Zoom in", "Enlarge graphic");
	windMenu->Append(CPC_WINDOWS_ZOOMOUT, "Zoom out", "Shrink graphic");
	windMenu->Append(CPC_WINDOWS_FIT, "Fit", "Fit graphics to panels");
	windMenu->Append(CPC_WINDOWS_FITPRESERVE, "Fit keep aspect", "Fit graphics to panels but retain original aspect ratios");
	windMenu->Append(CPC_WINDOWS_SETASPECT, "Best aspect fit", "Adjust panel sizes for best fit preserving aspect ratios");
	windMenu->AppendSeparator();
	windMenu->Append(CPC_WINDOWS_TEXTWINDOW, "Output windows", w1, 
	 "Select output window");
	windMenu->AppendSeparator();
	windMenu->Append(CPC_WINDOWS_GRAPH, "Graph windows", w2, "Select graph window");
	windMenu->AppendSeparator();
   
	bar->Append(fileMenu,"&File");
	bar->Append(windMenu,"&Windows");

	return(bar);
}


bool CarapacePanelGraphFrame::SaveWindowAs(void)
{
	// return TRUE for a successful save, FALSE otherwise
	// prompt for a file name and save to that file
	wxInitAllImageHandlers();
	wxSize size = Graph->Size;
	wxPoint point;
	int decision = wxYES;

	wxBitmap bitmap(size.GetWidth(),size.GetHeight(),-1);
	wxMemoryDC dc;
	dc.SelectObject(bitmap);
	Graph->Draw(dc,.1,.1);
	dc.SelectObject(wxNullBitmap);
	//wxImage image(bitmap);

	wxFileDialog dialog(NULL,CPC_STRING_SAVEGRAPH,"","",
#ifdef SAVEGRAPHTOPOSTSCRIPT
		"JPG|*.jpg|PNG|*.png|TIF|*.tif|PostScript|*.ps",wxSAVE);
#else
		"JPG|*.jpg|PNG|*.png|TIF|*.tif",wxSAVE);
#endif /*SAVEGRAPHTOPOSTSCRIPT*/
	if(dialog.ShowModal() == wxID_OK)
	{
		if (wxFileExists(dialog.GetPath())){
			decision = wxMessageBox(CPC_STRING_FILEEXISTS,CPC_STRING_ALERT,wxYES_NO);
		}
		if (decision != wxYES)
		{
			return FALSE;
		}

		switch(dialog.GetFilterIndex())
		{
			case 0:
				bitmap.SaveFile(dialog.GetPath(),wxBITMAP_TYPE_JPEG);
				//image.SaveFile(dialog.GetPath(),wxBITMAP_TYPE_JPEG);
				break;
			case 1:
				bitmap.SaveFile(dialog.GetPath(),wxBITMAP_TYPE_PNG);
				//image.SaveFile(dialog.GetPath(),wxBITMAP_TYPE_PNG);
				break;
			case 2:
				bitmap.SaveFile(dialog.GetPath(),wxBITMAP_TYPE_TIF);
				//image.SaveFile(dialog.GetPath(),wxBITMAP_TYPE_TIF);
				break;
			case 3:
			{
                                wxPrintData pd;
                                pd.SetPaperId(wxPAPER_LETTER);
                                pd.SetFilename(dialog.GetPath());
				//wxPostScriptDC psdc(pd);
				CarapacePostScriptDC psdc(pd);
				psdc.StartDoc("this graph");
				psdc.StartPage();
				//psdc.SetDeviceOrigin(200,200);
				size.SetWidth(10*size.GetWidth());
				size.SetHeight(10*size.GetHeight());
				//Graph->Draw(psdc,.1,.1);
				Graph->Draw(psdc,size,point,.1,.1);
				psdc.EndPage();
				psdc.EndDoc();
				break;
			}
		}
		return TRUE;
	}
	return FALSE;

} 

void CarapacePanelGraphFrame::OnCopy(wxCommandEvent & WXUNUSED(event))
{
	// return TRUE for a successful save, FALSE otherwise
	// prompt for a file name and save to that file
	wxInitAllImageHandlers();
	wxSize size = Graph->Size;

	wxBitmap bitmap(size.GetWidth(),size.GetHeight(),-1);
	wxMemoryDC dc;
	dc.SelectObject(bitmap);
	wxPoint point = wxPoint();
	Graph->Draw(dc,.1,.1);
	dc.SelectObject(wxNullBitmap);


	if(wxTheClipboard->Open())
	{
		wxTheClipboard->SetData(new wxBitmapDataObject(bitmap));
		wxTheClipboard->Close();
	}

} 


void CarapacePanelGraphFrame::OnQuit(wxCommandEvent & WXUNUSED(event))
{
	thisApp->DoQuit(CPC_EXIT_SAVECANCANCEL);
}

void CarapacePanelGraphFrame::FastQuit(wxCommandEvent & WXUNUSED(event))
{
	thisApp->DoQuit(CPC_EXIT_FASTFORCEQUIT);
}

void CarapacePanelGraphFrame::OnPageSetup(wxCommandEvent& WXUNUSED(event))
{
    (*thisApp->PageSetupData) = * thisApp->PrintData;

    wxPageSetupDialog pageSetupDialog(this, thisApp->PageSetupData);
    pageSetupDialog.ShowModal();

    (*thisApp->PrintData) = pageSetupDialog.GetPageSetupData().GetPrintData();
    (*thisApp->PageSetupData) = pageSetupDialog.GetPageSetupData();
}


void CarapacePanelGraphFrame::OnPrint(wxCommandEvent & WXUNUSED(event))
{
	wxPrintDialogData printDialogData(*(thisApp->PrintData));

	wxPrinter printer(& printDialogData);
	//CarapaceGPrintout printout("My printout",PanelFrameIndex);
	CarapaceGPrintout printout("My printout",this);
	if (!printer.Print(this,&printout, TRUE))
	{
		if (wxPrinter::GetLastError() == wxPRINTER_ERROR)
		{
			CarapaceAlert(CPC_STRING_PRINTINGERROR /*"Printing error"*/);
		}
	}
	else
	{
		*(thisApp->PrintData) = printer.GetPrintDialogData().GetPrintData();
	}
}

void CarapacePanelGraphFrame::MenuInterrupt(wxCommandEvent & WXUNUSED(event))
{
	thisApp->Interrupt();
}

void CarapacePanelGraphFrame::Hide(void)
{
	Show(FALSE);
}


void CarapacePanelGraphFrame::ShowTextFrame(wxCommandEvent& event)
{
	int index = event.GetId() - CPC_TEXT_WINDOWS_BASE;
	thisApp->TextFrameFront(thisApp->STFrameList.Item(index)->GetData()->TextFrameIndex);
}

void CarapacePanelGraphFrame::ShowGraphFrame(wxCommandEvent& event)
{
	int index = event.GetId() - CPC_GRAPH_WINDOWS_BASE;
	thisApp->GraphFrameFront(thisApp->GFrameList.Item(index)->GetData()->GraphFrameIndex);
}

void CarapacePanelGraphFrame::ZoomIn(wxCommandEvent& event)
{
	Graph->Size.SetWidth((int) (Graph->Size.GetWidth()*1.414) );
	Graph->Size.SetHeight((int) (Graph->Size.GetHeight()*1.414) );
	Graph->SetScrollbars( 10, 10, (long) (Graph->Size.GetWidth()/10), (long) (Graph->Size.GetHeight()/10) );
	Graph->Refresh();
}

void CarapacePanelGraphFrame::ZoomOut(wxCommandEvent& event)
{
	Graph->Size.SetWidth((int) (Graph->Size.GetWidth()/1.414) );
	Graph->Size.SetHeight((int) (Graph->Size.GetHeight()/1.414) );
	Graph->SetScrollbars( 10, 10, (long) (Graph->Size.GetWidth()/10), (long) (Graph->Size.GetHeight()/10) );
	Graph->Refresh();
}

void CarapacePanelGraphFrame::Fit(wxCommandEvent& event)
{
	int w,h;
	GetClientSize(&w,&h);
	Graph->Size.SetWidth(w);
	Graph->Size.SetHeight(h);
	Graph->SetScrollbars( 10, 10, (long) (Graph->Size.GetWidth()/10), (long) (Graph->Size.GetHeight()/10) );
	Graph->Refresh();
}

void CarapacePanelGraphFrame::FitPreserve(wxCommandEvent& event)
{
	int w,h;
	double r1, r2;
	GetClientSize(&w,&h);
	r1 = ((double) w)/Graph->Size.GetWidth();
	r2 = ((double) h)/Graph->Size.GetHeight();
	if(r2 < r1)
	{
		r1 = r2;
	}
	Graph->Size.SetWidth((int) (Graph->Size.GetWidth()*r1) );
	Graph->Size.SetHeight((int) (Graph->Size.GetHeight()*r1) );
	Graph->SetScrollbars( 10, 10, (long) (Graph->Size.GetWidth()/10), (long) (Graph->Size.GetHeight()/10) );
	Graph->Refresh();
}


void CarapacePanelGraphFrame::SetAspect(wxCommandEvent& event)
{

}

void CarapacePanelGraphFrame::Revert(wxCommandEvent& event)
{
	Graph->Size = Graph->OriginalSize;
	Graph->Refresh();
}

void CarapacePanelGraphFrame::DoNothing(void)
{
}


#define DO_EVT_MENU(id,callback,ev)   \
        if(which == wxXmlResource::GetXRCID(#id))  \
        { \
                callback(ev); \
                return; \
        }



void CarapacePanelGraphFrame::ProcessUserCommand(wxCommandEvent& event)
{
        CarapaceCommand *cptr;
        cpcMenuItem *menuItem;
        wxString base,full;
        int i;

        int which = event.GetId();

        DO_EVT_MENU(CPC_FILE_SAVEWINDOWAS, CarapacePanelGraphFrame::SaveWindowAs,)
        DO_EVT_MENU(CPC_EDIT_COPY, CarapacePanelGraphFrame::OnCopy,event)
        DO_EVT_MENU(CPC_FILE_PAGESETUP, CarapacePanelGraphFrame::OnPageSetup,event)
        DO_EVT_MENU(CPC_FILE_PRINTSELECTION, CarapacePanelGraphFrame::OnPrint,event)
        DO_EVT_MENU(CPC_FILE_INTERRUPT, CarapacePanelGraphFrame::MenuInterrupt,event)
        DO_EVT_MENU(CPC_FILE_QUIT, CarapacePanelGraphFrame::OnQuit,event)
        DO_EVT_MENU(CPC_FILE_FASTQUIT, CarapacePanelGraphFrame::FastQuit,event)


        DO_EVT_MENU(CPC_WINDOWS_HIDE, CarapacePanelGraphFrame::Hide,)
        DO_EVT_MENU(CPC_WINDOWS_ZOOMIN, CarapacePanelGraphFrame::ZoomIn,event)
        DO_EVT_MENU(CPC_WINDOWS_ZOOMOUT, CarapacePanelGraphFrame::ZoomOut,event)
        DO_EVT_MENU(CPC_WINDOWS_FIT, CarapacePanelGraphFrame::Fit,event)
        DO_EVT_MENU(CPC_WINDOWS_FITPRESERVE, CarapacePanelGraphFrame::FitPreserve,event)
        DO_EVT_MENU(CPC_WINDOWS_SETASPECT, CarapacePanelGraphFrame::SetAspect,event)
        DO_EVT_MENU(CPC_WINDOWS_REVERT, CarapacePanelGraphFrame::Revert,event)

        base = "CPC_TEXT_WINDOWS_BASE";
        for(i=0;i<thisApp->numTextFrames;i++)
        {
                full = base;
                full << i;
                if(which == wxXmlResource::GetXRCID(full.GetData()) )
                {
                        thisApp->TextFrameFront(thisApp->STFrameList.Item(i)->GetData()->TextFrameIndex);
                        return;
                }
        }


        base = "CPC_GRAPH_WINDOWS_BASE";
        for(i=0;i<thisApp->numGraphFrames;i++)
        {
                full = base;
                full << i;
                if(which == wxXmlResource::GetXRCID(full.GetData()) )
                {
                        thisApp->GraphFrameFront(thisApp->GFrameList.Item(i)->GetData()->GraphFrameIndex);
                        return;
                }
        }
        for(i=0;i<thisApp->numGraphFrames;i++)
        {
                full = base;
                full << (i+thisApp->numGraphFrames);
                if(which == wxXmlResource::GetXRCID(full.GetData()) )
                {
                        thisApp->PanelFrameFront(thisApp->PGFrameList.Item(i)->GetData()->PanelFrameIndex);
                        return;
                }
        }

	
	base = "CPC_DIALOG_WINDOWS_BASE";
	for(i=0;i<thisApp->DFrameList.GetCount();i++)
	{
       		full = base;
       		full << i;
		if(which == wxXmlResource::GetXRCID(full.GetData()) )
		{
			//thisApp->TextFrameFront(thisApp->STFrameList.Item(i)->GetData()->TextFrameIndex);
			thisApp->DFrameList.Item(i)->GetData()->Raise();
			return;
		}
	}



        menuItem = (cpcMenuItem *) GetMenuBar()->FindItem(which);
        if(!menuItem || menuItem->action == "")
        {
                return;
        }

        wxCommandEvent procevent(wxEVT_COMMAND_MENU_SELECTED,CPC_GUI_PROCESSCOMMAND);
        cptr = new CarapaceCommand;
        cptr->command = menuItem->action;
        cptr->command << "\n";

// commands should come from some command frame, so in a graphics frame,
// we use the first command frame.
        wxCarapaceSplitTextFrameListNode *stnode = thisApp->STFrameList.GetFirst();
        CarapaceSplitTextFrame *stfp = stnode->GetData();

        cptr->frame = stfp->TextFrameIndex;

        cptr->hidecommand = menuItem->hidecommand;
        cptr->hideoutput = menuItem->hideoutput;
        procevent.SetClientData((void *) cptr);
        wxPostEvent(thisApp,procevent);
}

