/*
*(C)* The file is part of the source distribution of Carapace
*(C)* version 0.5 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* This file is distributed under the GNU General Public License
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

#include "cpcplatform.h"



// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/print.h"


#include <math.h>

#include "carapace.h"

#undef UNDEFINED


#include "wx/image.h"
#include "wx/listctrl.h"
#include "wx/sizer.h"
#include "wx/log.h"


// CarapaceStaticGraph

IMPLEMENT_DYNAMIC_CLASS(CarapacePanelGraph, wxScrolledWindow)

BEGIN_EVENT_TABLE(CarapacePanelGraph, wxScrolledWindow)
  EVT_PAINT(                  CarapacePanelGraph::OnPaint)
  EVT_MOUSE_EVENTS(                  CarapacePanelGraph::OnMouse)
  EVT_SCROLLWIN(              CarapacePanelGraph::OnScroll)
  EVT_CHAR(CarapacePanelGraph::OnChar)
END_EVENT_TABLE()

CarapacePanelGraph::CarapacePanelGraph( void )
        : wxScrolledWindow( NULL, -1)
{

}


CarapacePanelGraph::CarapacePanelGraph( CarapacePanelGraphFrame *parent, wxWindowID id,
                    const wxPoint &pos, const wxSize &size, long nrows, long ncols)
        : wxScrolledWindow( parent, id, pos, size, wxSUNKEN_BORDER, "canvas" )
{
	Parent = parent;
	Size = size;
	OriginalSize = size;
	NRows = nrows;
	NCols = ncols;
	NGraphs = NRows*NCols;
	Frames = new int[NGraphs];
	
	int i;
	for(i = 0; i< NGraphs; i++)
	{
		Frames[i] = (Parent->PanelFrameIndex -1)*NGraphs + i + 1;
	}

	Fitting = CPC_WINDOWS_FIT;
	RowWise = TRUE;
}

CarapacePanelGraph::~CarapacePanelGraph()
{
	delete[] Frames;
}

void CarapacePanelGraph::OnMouse( wxMouseEvent &event )
{
	int xd,yd,xl,yl,xinc,yinc,base,count;
	CarapaceGraphFrame  *gfp;
	wxCarapaceGraphFrameListNode *gnode;

	event.GetPosition(&xd,&yd);
	CalcUnscrolledPosition(xd,yd,&xl,&yl);
	xinc = Size.GetWidth()/CPC_PANELCOLS;
	yinc = Size.GetHeight()/CPC_PANELROWS;

	base = (Parent->PanelFrameIndex-1)*NGraphs;
	if(RowWise)
	{
		count = (yl/yinc)*CPC_PANELROWS + (xl/xinc) + base + 1;
	}
	else
	{
		count = (yl/yinc) + (xl/xinc)*CPC_PANELCOLS + base + 1;
	}

	gnode = thisApp->GFrameList.Find(count);
	if(event.Moving())
	{
		if(gnode)
		{
			gfp = gnode->GetData();
			Parent->GetStatusBar()->SetStatusText(gfp->GetTitle());
		}
		else
		{
			Parent->GetStatusBar()->SetStatusText("");
		}
	}
	if(event.Leaving())
	{
		Parent->GetStatusBar()->SetStatusText("");
	}
	if(event.LeftDown())
	{
		if(gnode)
		{
			gfp = gnode->GetData();
			gfp->Show(FALSE);
			gfp->Show(TRUE);
			thisApp->UpdateMenus();
			gfp->Raise();
			gfp->Graph->SetFocus();
		}
	}

	

}

void CarapacePanelGraph::OnPaint( wxPaintEvent &WXUNUSED(event) )
{
	wxPaintDC dc( this );
	PrepareDC( dc );

	Draw(dc,.1,.1);
}

void CarapacePanelGraph::Draw( wxDC& dc, double xscale, double yscale )
{

	int i,j,k,xinc,yinc;
	wxPoint point;
	CarapaceGraphFrame  *gfp;
	wxCarapaceGraphFrameListNode *gnode;
	wxSize size;

	switch(Fitting)
	{
		case CPC_WINDOWS_FIT:
		{
			xinc = Size.GetWidth()/CPC_PANELCOLS;
			yinc = Size.GetHeight()/CPC_PANELROWS;
			size = wxSize(xinc,yinc);
			if(RowWise)
			{
				k = 0;
				for(i=0;i<NRows;i++)
				{
					for(j=0;j<NCols;j++)
					{
						point.x = j*xinc;
						point.y = i*yinc;
						gnode = thisApp->GFrameList.Find(Frames[k]);
						if(gnode)
						{
							gfp = gnode->GetData();
							if(gfp && gfp->Graph && gfp->Graph->GStructure)
							{
								gfp->Graph->GStructure->Draw(dc,size,point,xscale,yscale);
							}
						}
						k++;
					}
				}
			}
			else
			{
				k = 0;
				for(j=0;j<NCols;j++)
				{
					for(i=0;i<NRows;i++)
					{
						point.x = j*xinc;
						point.y = i*yinc;
						gnode = thisApp->GFrameList.Find(Frames[k]);
						if(gnode)
						{
							gfp = gnode->GetData();
							if(gfp && gfp->Graph && gfp->Graph->GStructure)
							{
								gfp->Graph->GStructure->Draw(dc,size,point,xscale,yscale);
							}
						}
						k++;
					}
				}
			}
		}
		case CPC_WINDOWS_FITPRESERVE:
		case CPC_WINDOWS_SETASPECT:
		default:
			;
	}
}


void CarapacePanelGraph::Draw( wxDC& dc, wxSize& drawSize, wxPoint& unused, double xscale, double yscale )
{

	int i,j,k,xinc,yinc;
	wxPoint point;
	CarapaceGraphFrame  *gfp;
	wxCarapaceGraphFrameListNode *gnode;
	wxSize size;

	switch(Fitting)
	{
		case CPC_WINDOWS_FIT:
		{
			xinc = drawSize.GetWidth()/CPC_PANELCOLS;
			yinc = drawSize.GetHeight()/CPC_PANELROWS;
			size = wxSize(xinc,yinc);
			if(RowWise)
			{
				k = 0;
				for(i=0;i<NRows;i++)
				{
					for(j=0;j<NCols;j++)
					{
						point.x = j*xinc;
						point.y = i*yinc;
						gnode = thisApp->GFrameList.Find(Frames[k]);
						if(gnode)
						{
							gfp = gnode->GetData();
							if(gfp && gfp->Graph && gfp->Graph->GStructure)
							{
								gfp->Graph->GStructure->Draw(dc,size,point,xscale,yscale);
							}
						}
						k++;
					}
				}
			}
			else
			{
				k = 0;
				for(j=0;j<NCols;j++)
				{
					for(i=0;i<NRows;i++)
					{
						point.x = j*xinc;
						point.y = i*yinc;
						gnode = thisApp->GFrameList.Find(Frames[k]);
						if(gnode)
						{
							gfp = gnode->GetData();
							if(gfp && gfp->Graph && gfp->Graph->GStructure)
							{
								gfp->Graph->GStructure->Draw(dc,size,point,xscale,yscale);
							}
						}
						k++;
					}
				}
			}
		}
		case CPC_WINDOWS_FITPRESERVE:
		case CPC_WINDOWS_SETASPECT:
		default:
			;
	}
}


bool CarapacePanelGraph::DoGraphRequest(cpc_sg_ptr sgp)
{
	wxClientDC dc( this );
	switch(sgp->type)
	{
		case CPC_SG_GETCHARWIDTH:
			sgp->size = dc.GetCharWidth()*10;
			break;

		case CPC_SG_GETCHARHEIGHT:
			sgp->size = dc.GetCharHeight()*10;
			break;

		default:
			return(FALSE);
	}
	return(TRUE);
}


void CarapacePanelGraph::OnScroll( wxScrollWinEvent &event )
{
    event.Skip();
}

void CarapacePanelGraph::OnChar( wxKeyEvent &event)
{
	long       ch,count,i;
	wxAcceleratorEntry entry;

	ch =  event.m_keyCode;

// sometimes you have to hit things with a really big brick.  Some text widgets (e.g., gtk)
// grab hold of key presses before the accelerator table can deal with them.  To get around
// this, we keep our own list of accelerators and handle the troublesome cases.
	if(ch < 26 && Parent->Accelerators)
	{
		count = Parent->Accelerators->GetCount();
		for(i=0;i<count;i++)
		{
			entry = Parent->Accelerators->Item(i);
			if( (ch == entry.GetKeyCode()-64 || ch == entry.GetKeyCode()-96) &&
				entry.GetFlags() == wxACCEL_CTRL )
			{
				wxCommandEvent command_event( wxEVT_COMMAND_MENU_SELECTED, entry.GetCommand() );
				Parent->GetEventHandler()->ProcessEvent( command_event );
				return;
			}
	
		}
	}
	event.Skip();
}
