/*
*(C)* The file is part of the source distribution of Carapace
*(C)* version 0.5 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* This file is distributed under the GNU General Public License
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

#include "cpcplatform.h"



// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/print.h"
#include "wx/prntbase.h"
#include "wx/printdlg.h"
#include "wx/progdlg.h"
#include "wx/cmndata.h"


#include <math.h>

#include "carapace.h"

#undef UNDEFINED


#include "wx/image.h"
#include "wx/listctrl.h"
#include "wx/sizer.h"
#include "wx/log.h"


#include "wx/xrc/xmlres.h"
#include "wx/xrc/xh_all.h"

#include "wx/generic/dcpsg.h"


// CarapacePostScriptDC

CarapacePostScriptDC::CarapacePostScriptDC(const wxPrintData& printData) :
	wxPostScriptDC(printData)
{
}

CarapacePostScriptDC::~CarapacePostScriptDC() 
{
}

void CarapacePostScriptDC::DoDrawRotatedText( const wxString& text, wxCoord x, wxCoord y, double angle )
{
	if (angle == 0.0)
	{
		DoDrawText(text, x, y);
		return;
	}

	wxCHECK_RET( m_ok && m_pstream, wxT("invalid postscript dc") );

	SetFont( m_font );

	int size = m_font.GetPointSize();
// begin GWO
	wxCoord text_w, text_h, text_descent;

	GetTextExtent(text, &text_w, &text_h, &text_descent);

	long by = y ; // approximate baseline

	// FIXME only correct for 90 degrees
	PsPrintf("%d %d moveto\n",
	LogicalToDeviceX((wxCoord)(x +size - text_descent)), LogicalToDeviceY((wxCoord)by) );
// end GWO

#if 0
	long by = y + (long)floor( double(size) * 2.0 / 3.0 ); // approximate baseline

	// FIXME only correct for 90 degrees
	PsPrintf("%d %d moveto\n",
	LogicalToDeviceX((wxCoord)(x + size)), LogicalToDeviceY((wxCoord)by) );
#endif

	wxPostScriptDC::DoDrawRotatedText(text,x,y,angle);
}

void CarapacePostScriptDC::EndDoc ()
{
	wxCHECK_RET( m_ok && m_pstream, wxT("invalid postscript dc") );

	if (m_clipping)
	{
		m_clipping = FALSE;
		PsPrint("grestore\n");
	}

#if 0  
// this doesn't work in 2.8
	// THE FOLLOWING HAS BEEN CONTRIBUTED BY Andy Fyfe <andy@hyperparallel.com>
	wxCoord wx_printer_translate_x, wx_printer_translate_y;
	double wx_printer_scale_x, wx_printer_scale_y;

	wx_printer_translate_x = (wxCoord)m_printData.GetPrinterTranslateX();
	wx_printer_translate_y = (wxCoord)m_printData.GetPrinterTranslateY();

	wx_printer_scale_x = m_printData.GetPrinterScaleX();
	wx_printer_scale_y = m_printData.GetPrinterScaleY();

	// Compute the bounding box.  Note that it is in the default user
	// coordinate system, thus we have to convert the values.
	wxCoord minX = (wxCoord) LogicalToDeviceX(m_minX);
	wxCoord minY = (wxCoord) LogicalToDeviceY(m_minY);
	wxCoord maxX = (wxCoord) LogicalToDeviceX(m_maxX);
	wxCoord maxY = (wxCoord) LogicalToDeviceY(m_maxY);

	// LOG2DEV may have changed the minimum to maximum vice versa
	if ( minX > maxX ) { wxCoord tmp = minX; minX = maxX; maxX = tmp; }
	if ( minY > maxY ) { wxCoord tmp = minY; minY = maxY; maxY = tmp; }

	// account for used scaling (boundingbox is before scaling in ps-file)
	//double scale_x = m_printData.GetPrinterScaleX() / ms_PSScaleFactor;
	//double scale_y = m_printData.GetPrinterScaleY() / ms_PSScaleFactor;
	double scale_x = m_printData.GetPrinterScaleX() / (GetResolution()/72.);
	double scale_y = m_printData.GetPrinterScaleY() / (GetResolution()/72.);

	wxCoord llx, lly, urx, ury;
	llx = (wxCoord) ((minX+wx_printer_translate_x)*scale_x);
	lly = (wxCoord) ((minY+wx_printer_translate_y)*scale_y);
	urx = (wxCoord) ((maxX+wx_printer_translate_x)*scale_x);
	ury = (wxCoord) ((maxY+wx_printer_translate_y)*scale_y);
	// (end of bounding box computation)


// If we're landscape, our sense of "x" and "y" is reversed.
	if (m_printData.GetOrientation() == wxLANDSCAPE)
	{
		wxCoord tmp;
		tmp = llx; llx = lly; lly = tmp;
		tmp = urx; urx = ury; ury = tmp;
	
		// We need either the two lines that follow, or we need to subtract
		// min_x from real_translate_y, which is commented out below.
		llx = llx - (wxCoord)(m_minX*wx_printer_scale_y);
		urx = urx - (wxCoord)(m_minX*wx_printer_scale_y);
	}

	// The Adobe specifications call for integers; we round as to make
	// the bounding larger.
	PsPrintf( "%%%%BoundingBox: %d %d %d %d\n",
	(wxCoord)floor((double)llx), (wxCoord)floor((double)lly),
	(wxCoord)ceil((double)urx), (wxCoord)ceil((double)ury) );

	// To check the correctness of the bounding box, postscript commands
	// to draw a box corresponding to the bounding box are generated below.
	// But since we typically don't want to print such a box, the postscript
	// commands are generated within comments.  These lines appear before any
	// adjustment of scale, rotation, or translation, and hence are in the
	// default user coordinates.
	// fprintf( m_pstream, "%% newpath\n" );
	// fprintf( m_pstream, "%% %d %d moveto\n", llx, lly );
	// fprintf( m_pstream, "%% %d %d lineto\n", urx, lly );
	// fprintf( m_pstream, "%% %d %d lineto\n", urx, ury );
	// fprintf( m_pstream, "%% %d %d lineto closepath stroke\n", llx, ury );
#endif

	wxPostScriptDC::EndDoc();
}

IMPLEMENT_DYNAMIC_CLASS(CarapacePostScriptPrinter,wxPostScriptPrinter)

CarapacePostScriptPrinter::CarapacePostScriptPrinter(wxPrintDialogData *data) :
	wxPostScriptPrinter(data)
{
}

CarapacePostScriptPrinter::~CarapacePostScriptPrinter()
{
}

bool CarapacePostScriptPrinter::Print(wxWindow *parent, wxPrintout *printout, bool prompt)
{
	sm_abortIt = false;
	sm_abortWindow = (wxWindow *) NULL;

	if (!printout)
	{
		sm_lastError = wxPRINTER_ERROR;
		return false;
	}

	printout->SetIsPreview(false);

	if (m_printDialogData.GetMinPage() < 1)
		m_printDialogData.SetMinPage(1);
	if (m_printDialogData.GetMaxPage() < 1)
		m_printDialogData.SetMaxPage(9999);

	// Create a suitable device context
	wxDC *dc;
	if (prompt)
	{
		dc = PrintDialog(parent);
		if (!dc)
			return false;
	}
	else
	{
		dc = new CarapacePostScriptDC(GetPrintDialogData().GetPrintData());
	}

	// May have pressed cancel.
	if (!dc || !dc->Ok())
	{
		if (dc) delete dc;
		sm_lastError = wxPRINTER_ERROR;
		return false;
	}

	wxSize ScreenPixels = wxGetDisplaySize();
	wxSize ScreenMM = wxGetDisplaySizeMM();

	printout->SetPPIScreen( (int) ((ScreenPixels.GetWidth() * 25.4) / ScreenMM.GetWidth()),
		    (int) ((ScreenPixels.GetHeight() * 25.4) / ScreenMM.GetHeight()) );
	printout->SetPPIPrinter( wxPostScriptDC::GetResolution(),
		     wxPostScriptDC::GetResolution() );

	// Set printout parameters
	printout->SetDC(dc);

	int w, h;
	dc->GetSize(&w, &h);
	printout->SetPageSizePixels((int)w, (int)h);
	dc->GetSizeMM(&w, &h);
	printout->SetPageSizeMM((int)w, (int)h);

	// Create an abort window
	wxBeginBusyCursor();

	printout->OnPreparePrinting();

	// Get some parameters from the printout, if defined
	int fromPage, toPage;
	int minPage, maxPage;
	printout->GetPageInfo(&minPage, &maxPage, &fromPage, &toPage);

	if (maxPage == 0)
	{
		sm_lastError = wxPRINTER_ERROR;
		wxEndBusyCursor();
		return false;
	}

	// Only set min and max, because from and to have been
	// set by the user
	m_printDialogData.SetMinPage(minPage);
	m_printDialogData.SetMaxPage(maxPage);
	
	if (m_printDialogData.GetFromPage() < minPage)
		m_printDialogData.SetFromPage( minPage );
	if (m_printDialogData.GetToPage() > maxPage)
		m_printDialogData.SetToPage( maxPage );
	
	int
	   pagesPerCopy = m_printDialogData.GetToPage()-m_printDialogData.GetFromPage()+1,
	   totalPages = pagesPerCopy * m_printDialogData.GetNoCopies(),
	   printedPages = 0;
	// Open the progress bar dialog
	wxProgressDialog *progressDialog = new wxProgressDialog (
	   printout->GetTitle(),
	   _("Printing..."),
	   totalPages,
	   parent,
	   wxPD_CAN_ABORT|wxPD_AUTO_HIDE|wxPD_APP_MODAL);

	printout->OnBeginPrinting();

	sm_lastError = wxPRINTER_NO_ERROR;

	bool keepGoing = true;

	int copyCount;
	for (copyCount = 1; copyCount <= m_printDialogData.GetNoCopies(); copyCount ++)
	{
		if (!printout->OnBeginDocument(m_printDialogData.GetFromPage(), m_printDialogData.GetToPage()))
		{
			wxEndBusyCursor();
			wxLogError(_("Could not start printing."));
			sm_lastError = wxPRINTER_ERROR;
			break;
		}
		if (sm_abortIt)
		{
			sm_lastError = wxPRINTER_CANCELLED;
			break;
		}

		int pn;
		for (pn = m_printDialogData.GetFromPage(); keepGoing && (pn <= m_printDialogData.GetToPage()) && printout->HasPage(pn);
		pn++)
		{
			if (sm_abortIt)
			{
				keepGoing = false;
				sm_lastError = wxPRINTER_CANCELLED;
				break;
			}
			else
			{
			   wxString msg;
			   msg.Printf(_("Printing page %d..."), printedPages+1);
			   if(progressDialog->Update(printedPages++, msg))
			   {
				  dc->StartPage();
				  printout->OnPrintPage(pn);
				  dc->EndPage();
			   }
			   else
			   {
				  sm_abortIt = true;
				  sm_lastError = wxPRINTER_CANCELLED;
				  keepGoing = false;
			   }
			}
			wxYield();
		}
		printout->OnEndDocument();
	}

	printout->OnEndPrinting();
	delete progressDialog;

	wxEndBusyCursor();

	delete dc;

	return (sm_lastError == wxPRINTER_NO_ERROR);
}


// CarapacePostScriptGraph


CarapacePostScriptGraph::CarapacePostScriptGraph(wxString& filename, bool append)
{
	GStructure = NULL;
	wxPrintData pd;
	pd.SetPaperId(wxPAPER_LETTER);
	pd.SetFilename(filename);
	pd.SetPrintMode(wxPRINT_MODE_FILE);
	// 2.8 change pd.SetFontMetricPath(thisApp->AFMPath);
	//dc = new wxPostScriptDC(pd);
	dc = new CarapacePostScriptDC(pd);
	dc->StartDoc(filename);
}

CarapacePostScriptGraph::~CarapacePostScriptGraph()
{
	CloseFile();
}

void CarapacePostScriptGraph::OpenPage(wxSize& size)
{
	int ppih,offset;
	double scalefactor;
	ppih = dc->GetResolution();
	scalefactor = ppih/720.;
	offset = (int) (200*scalefactor);  // 200 decipoints

	ClosePage();
	GStructure = new CarapaceGraphStructure(size);
	dc->StartPage();
	dc->SetDeviceOrigin(offset,offset);
}

void CarapacePostScriptGraph::ClosePage()
{
	if(GStructure == NULL)
	{
		return;
	}
	wxPoint point;
	int ppih;
	double scalefactor;
	ppih = dc->GetResolution();
	scalefactor = ppih/720.;
	wxSize size = wxSize(GStructure->Size.GetWidth(),GStructure->Size.GetHeight());
	GStructure->Draw(*dc,size,point,scalefactor,scalefactor);
	dc->EndPage();
	delete GStructure;
	GStructure = NULL;
}

void CarapacePostScriptGraph::CloseFile()
{
	ClosePage();
	dc->EndDoc();
	delete dc;
	thisApp->PSGraph = NULL;
}

