/*
*(C)* The file is part of the source distribution of Carapace
*(C)* version 0.5 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* This file is distributed under the GNU General Public License
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

#include "cpcplatform.h"



// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/gdicmn.h"
#include "wx/fontdlg.h"
#include "wx/colour.h"
#include "wx/caret.h"
#include "wx/print.h"
#include "wx/printdlg.h"
#include "wx/colordlg.h"
#include "wx/fontmap.h"
#include "wx/splitter.h"


#include <math.h>

#include "carapace.h"

#include "wx/xrc/xmlres.h"
#include "wx/xrc/xh_all.h"


#if 0
#include "empty.xpm"
#include "warning.xpm"
#endif

#undef UNDEFINED

// event table for text frames


BEGIN_EVENT_TABLE(CarapaceSplitTextFrame, wxFrame)

	EVT_SPINCTRL(CPC_DIALOGS_HISTORYSPINNER, CarapaceSplitTextFrame::DoSpinner)
	EVT_BUTTON(CPC_DIALOGS_EXECUTEBUTTON,CarapaceSplitTextFrame::Execute)
	EVT_BUTTON(CPC_DIALOGS_FORWARDBUTTON,CarapaceSplitTextFrame::Forward)
	EVT_BUTTON(CPC_DIALOGS_BACKBUTTON,CarapaceSplitTextFrame::Back)
	EVT_BUTTON(CPC_DIALOGS_CLEARBUTTON,CarapaceSplitTextFrame::ClearCommand)
	EVT_BUTTON(CPC_DIALOGS_UNDOBUTTON,CarapaceSplitTextFrame::Undo)
	EVT_BUTTON(CPC_DIALOGS_INTERRUPTBUTTON,CarapaceSplitTextFrame::MenuInterrupt)


	EVT_MENU_RANGE(1,2000000000, CarapaceSplitTextFrame::ProcessUserCommand)
	EVT_UPDATE_UI_RANGE(1,2000000000, CarapaceSplitTextFrame::UpdateUserCommand)
	
	EVT_CLOSE(CarapaceSplitTextFrame::OnClose)

END_EVENT_TABLE()


CarapaceSplitTextFrame::CarapaceSplitTextFrame(wxWindow *frame, const wxString & title,
			 const wxPoint & pos, const wxSize & size, bool hidden):
				 wxFrame(frame, -1, title, pos,size )
{
	
	wxColor *fg,*bg;


	CarapaceGetTextColors(&fg,&bg);
	outputFGColor = fg;
	outputBGColor = bg;

	outputFont = new wxFont(thisApp->Preferences->textFontSize,
		thisApp->Preferences->textFontFamily,
		thisApp->Preferences->textFontStyle,
		thisApp->Preferences->textFontWeight,
		FALSE,
		thisApp->Preferences->textFontFaceName);
	commandFont = new wxFont(thisApp->Preferences->textFontSize+2,
		thisApp->Preferences->textFontFamily,
		thisApp->Preferences->textFontStyle,
		//thisApp->Preferences->textFontWeight,
		wxBOLD,
		FALSE,
		thisApp->Preferences->textFontFaceName);

	CreateStatusBar();

	Accelerators = new ArrayofAccelerators;

	//Resource = new wxXmlResource();
	Resource = new CarapaceXmlResource();
	if(Resource)
        {
		Resource->AddHandler(new wxMenuXmlHandler);
		Resource->AddHandler(new cpcMenuXmlHandler);
		Resource->AddHandler(new wxMenuBarXmlHandler);
	}

	SetMenuBarFromString(CPC_STRING_DEFAULTCOMMANDRESOURCE,CPC_STRING_DEFAULTMENUNAME);


	wxSplitterWindow *splitter = new wxSplitterWindow(this,-1,wxDefaultPosition,size);

	// Make a text window
	output_ctrl = new CarapaceOutputCtrl((wxWindow *) splitter, (wxWindowID) -1,
		//" ", wxPoint(0, 250), size,
		" ", wxPoint(0, 250), wxSize(10,30),
#ifdef __WXMAC__
		wxTE_PROCESS_TAB | wxTE_RICH | wxTE_MULTILINE );
#else
		wxTE_PROCESS_TAB | wxTE_RICH | wxTE_MULTILINE | wxHSCROLL );
#endif
	output_ctrl->Frame = this;



	output_ctrl->SetFont(*outputFont);
	output_ctrl->SetDefaultStyle(wxTextAttr(*outputFGColor,*outputBGColor,*outputFont));
	output_ctrl->SetStyle(0,output_ctrl->GetLastPosition(),
		wxTextAttr(*outputFGColor,*outputBGColor,*outputFont));
	output_ctrl->Refresh();

	GetCaret()->SetBlinkTime(500);


	command_ctrl = new CarapaceCommandCtrl((wxWindow *) splitter, CPC_DIALOGS_COMMANDWINDOW,
		//" ", wxPoint(0, 250), wxSize(700, 100),
		" ", wxPoint(0, 250), wxSize(10, 10),
#ifdef __WXMAC__
		wxTE_PROCESS_TAB | wxTE_RICH | wxTE_MULTILINE );
#else
		wxTE_PROCESS_TAB | wxTE_RICH | wxTE_MULTILINE | wxHSCROLL );
#endif
	command_ctrl->Frame = this;
	command_ctrl->SetFont(*outputFont);
	command_ctrl->SetDefaultStyle(wxTextAttr(*outputFGColor,*outputBGColor,*outputFont));
	wxString tmpstring = "";
	command_ctrl->SetContent(tmpstring);
	command_ctrl->Refresh();
	command_ctrl->SetFocus();


#if 0
	history_spinner = new wxSpinCtrl( (wxWindow *) this, CPC_DIALOGS_HISTORYSPINNER,
		"",wxDefaultPosition, wxSize(50,30),wxSP_ARROW_KEYS,
		0,10000,0,"Command History");
#endif

	history_counter = new wxStaticText( (wxWindow *) this, CPC_DIALOGS_HISTORYCOUNTER,
		"", wxDefaultPosition, wxSize(40,20), wxALIGN_RIGHT, "");

	undo_button = new wxButton( (wxWindow *) this, CPC_DIALOGS_UNDOBUTTON,
		CPC_STRING_UNDOCOMMAND);

	clear_button = new wxButton( (wxWindow *) this, CPC_DIALOGS_CLEARBUTTON,
		CPC_STRING_CLEARCOMMAND);

	execute_button = new wxButton( (wxWindow *) this, CPC_DIALOGS_EXECUTEBUTTON,
		CPC_STRING_EXECUTE);

	back_button = new wxButton( (wxWindow *) this, CPC_DIALOGS_BACKBUTTON,
		CPC_STRING_BACK);

	forward_button = new wxButton( (wxWindow *) this, CPC_DIALOGS_FORWARDBUTTON,
		CPC_STRING_FORWARD);

	interrupt_button = new wxButton( (wxWindow *) this, CPC_DIALOGS_INTERRUPTBUTTON,
		CPC_STRING_INTERRUPTBUT);


	RunningMeter = new wxStaticText( (wxWindow *) this, CPC_DIALOGS_RUNNING,
		"", wxDefaultPosition, wxSize(100,20), wxALIGN_RIGHT,
		"");

#if 0
	// We've decided not to use the syntax alert warning bitmap.  It's an OK idea, but
	// a beep probably works just as well
	WarningBitmap = wxBitmap(warning_xpm);
	NullBitmap = wxBitmap(empty_xpm);
	SyntaxAlert = new wxStaticBitmap ( (wxWindow *) this, CPC_DIALOGS_SYNTAXERROR,
		NullBitmap,  wxDefaultPosition, wxSize(80,20),0, "");
#endif
	SetAutoLayout(TRUE);

	hor_sizer = new wxBoxSizer(wxHORIZONTAL);
	vert_sizer = new wxBoxSizer(wxVERTICAL);

	hor_sizer->Add(RunningMeter,0,wxALIGN_TOP | wxALL,3);
	hor_sizer->Add(30,10,0);
	//hor_sizer->Add(history_spinner,0,wxALIGN_TOP | wxALL,3);
	hor_sizer->Add(history_counter,0,wxALIGN_TOP | wxALL,3);
	hor_sizer->Add(undo_button,0,wxALIGN_TOP | wxALL,3);
	hor_sizer->Add(execute_button,0,wxALIGN_TOP | wxALL,3);
	hor_sizer->Add(back_button,0,wxALIGN_TOP | wxALL,3);
	hor_sizer->Add(forward_button,0,wxALIGN_TOP | wxALL,3);
	hor_sizer->Add(clear_button,0,wxALIGN_TOP | wxALL,3);
	hor_sizer->Add(30,30,0);
	hor_sizer->Add(interrupt_button,0,wxALIGN_TOP | wxALL,3);
#if 0
	hor_sizer->Add(SyntaxAlert,0,wxALIGN_BOTTOM | wxALL,3);
#endif
	splitter->SplitHorizontally(output_ctrl,command_ctrl,-175);

	vert_sizer->Add(splitter,1,wxEXPAND | wxALL,3);
	vert_sizer->Add(hor_sizer,0,wxEXPAND | wxALL,3);
	SetSizer(vert_sizer);
	Show(TRUE);
	

} /*CarapaceSplitTextFrame::CarapaceSplitTextFrame()*/


CarapaceSplitTextFrame::CarapaceSplitTextFrame(wxWindow *frame, const wxString & title,
			 const wxPoint & pos, const wxSize & size, 
			wxString& menuresource, wxString& menutitle, bool hidden):
				 wxFrame(frame, -1, title, pos,size )
{
	
	wxColor *fg,*bg;


	CarapaceGetTextColors(&fg,&bg);
	outputFGColor = fg;
	outputBGColor = bg;

	outputFont = new wxFont(thisApp->Preferences->textFontSize,
		thisApp->Preferences->textFontFamily,
		thisApp->Preferences->textFontStyle,
		thisApp->Preferences->textFontWeight,
		FALSE,
		thisApp->Preferences->textFontFaceName);
	commandFont = new wxFont(thisApp->Preferences->textFontSize+2,
		thisApp->Preferences->textFontFamily,
		thisApp->Preferences->textFontStyle,
		//thisApp->Preferences->textFontWeight,
		wxBOLD,
		FALSE,
		thisApp->Preferences->textFontFaceName);

	CreateStatusBar();

	Accelerators = new ArrayofAccelerators;

	//Resource = new wxXmlResource();
	Resource = new CarapaceXmlResource();
	if(Resource)
        {
		Resource->AddHandler(new wxMenuXmlHandler);
		Resource->AddHandler(new cpcMenuXmlHandler);
		Resource->AddHandler(new wxMenuBarXmlHandler);
	}

	if(menuresource != "" && menutitle != "")
	{
		SetMenuBarFromString(menuresource,menutitle);
	}
	else
	{
		SetMenuBarFromString(CPC_STRING_DEFAULTCOMMANDRESOURCE,CPC_STRING_DEFAULTMENUNAME);
	}


	wxSplitterWindow *splitter = new wxSplitterWindow(this,-1,wxDefaultPosition,size);

	// Make a text window
	output_ctrl = new CarapaceOutputCtrl((wxWindow *) splitter, (wxWindowID) -1,
		//" ", wxPoint(0, 250), size,
		" ", wxPoint(0, 250), wxSize(10,30),
#ifdef __WXMAC__
		wxTE_PROCESS_TAB | wxTE_RICH | wxTE_MULTILINE );
#else
		wxTE_PROCESS_TAB | wxTE_RICH | wxTE_MULTILINE | wxHSCROLL );
#endif
	output_ctrl->Frame = this;



	output_ctrl->SetFont(*outputFont);
	output_ctrl->SetDefaultStyle(wxTextAttr(*outputFGColor,*outputBGColor,*outputFont));
	output_ctrl->SetStyle(0,output_ctrl->GetLastPosition(),
		wxTextAttr(*outputFGColor,*outputBGColor,*outputFont));
	output_ctrl->Refresh();

	GetCaret()->SetBlinkTime(500);


	command_ctrl = new CarapaceCommandCtrl((wxWindow *) splitter, CPC_DIALOGS_COMMANDWINDOW,
		//" ", wxPoint(0, 250), wxSize(700, 100),
		" ", wxPoint(0, 250), wxSize(10, 10),
#ifdef __WXMAC__
		wxTE_PROCESS_TAB | wxTE_RICH | wxTE_MULTILINE );
#else
		wxTE_PROCESS_TAB | wxTE_RICH | wxTE_MULTILINE | wxHSCROLL );
#endif
	command_ctrl->Frame = this;
	command_ctrl->SetFont(*outputFont);
	command_ctrl->SetDefaultStyle(wxTextAttr(*outputFGColor,*outputBGColor,*outputFont));
	wxString tmpstring = "";
	command_ctrl->SetContent(tmpstring);
	command_ctrl->Refresh();
	command_ctrl->SetFocus();


#if 0
	history_spinner = new wxSpinCtrl( (wxWindow *) this, CPC_DIALOGS_HISTORYSPINNER,
		"",wxDefaultPosition, wxSize(50,30),wxSP_ARROW_KEYS,
		0,10000,0,"Command History");
#endif

	history_counter = new wxStaticText( (wxWindow *) this, CPC_DIALOGS_HISTORYCOUNTER,
		"", wxDefaultPosition, wxSize(40,20), wxALIGN_RIGHT, "");

	undo_button = new wxButton( (wxWindow *) this, CPC_DIALOGS_UNDOBUTTON,
		CPC_STRING_UNDOCOMMAND);

	clear_button = new wxButton( (wxWindow *) this, CPC_DIALOGS_CLEARBUTTON,
		CPC_STRING_CLEARCOMMAND);

	execute_button = new wxButton( (wxWindow *) this, CPC_DIALOGS_EXECUTEBUTTON,
		CPC_STRING_EXECUTE);

	back_button = new wxButton( (wxWindow *) this, CPC_DIALOGS_BACKBUTTON,
		CPC_STRING_BACK);

	forward_button = new wxButton( (wxWindow *) this, CPC_DIALOGS_FORWARDBUTTON,
		CPC_STRING_FORWARD);

	interrupt_button = new wxButton( (wxWindow *) this, CPC_DIALOGS_INTERRUPTBUTTON,
		CPC_STRING_INTERRUPTBUT);


	RunningMeter = new wxStaticText( (wxWindow *) this, CPC_DIALOGS_RUNNING,
		"", wxDefaultPosition, wxSize(100,20), wxALIGN_RIGHT,
		"");

#if 0
	// We've decided not to use the syntax alert warning bitmap.  It's an OK idea, but
	// a beep probably works just as well
	WarningBitmap = wxBitmap(warning_xpm);
	NullBitmap = wxBitmap(empty_xpm);
	SyntaxAlert = new wxStaticBitmap ( (wxWindow *) this, CPC_DIALOGS_SYNTAXERROR,
		NullBitmap,  wxDefaultPosition, wxSize(80,20),0, "");
#endif
	SetAutoLayout(TRUE);

	hor_sizer = new wxBoxSizer(wxHORIZONTAL);
	vert_sizer = new wxBoxSizer(wxVERTICAL);

	hor_sizer->Add(RunningMeter,0,wxALIGN_TOP | wxALL,3);
	hor_sizer->Add(30,10,0);
	//hor_sizer->Add(history_spinner,0,wxALIGN_TOP | wxALL,3);
	hor_sizer->Add(history_counter,0,wxALIGN_TOP | wxALL,3);
	hor_sizer->Add(undo_button,0,wxALIGN_TOP | wxALL,3);
	hor_sizer->Add(execute_button,0,wxALIGN_TOP | wxALL,3);
	hor_sizer->Add(back_button,0,wxALIGN_TOP | wxALL,3);
	hor_sizer->Add(forward_button,0,wxALIGN_TOP | wxALL,3);
	hor_sizer->Add(clear_button,0,wxALIGN_TOP | wxALL,3);
	hor_sizer->Add(30,30,0);
	hor_sizer->Add(interrupt_button,0,wxALIGN_TOP | wxALL,3);
#if 0
	hor_sizer->Add(SyntaxAlert,0,wxALIGN_BOTTOM | wxALL,3);
#endif
	splitter->SplitHorizontally(output_ctrl,command_ctrl,-175);

	vert_sizer->Add(splitter,1,wxEXPAND | wxALL,3);
	vert_sizer->Add(hor_sizer,0,wxEXPAND | wxALL,3);
	SetSizer(vert_sizer);
	Show(TRUE);
	

} /*CarapaceSplitTextFrame::CarapaceSplitTextFrame()*/

CarapaceSplitTextFrame::~CarapaceSplitTextFrame(void)
{
//	if(output_ctrl) delete output_ctrl;
//	if(command_ctrl) delete command_ctrl;
}



void CarapaceSplitTextFrame::UpdateMenuBar(void)
{
		

	// here we update windows, graphs, and user menus

	wxMenuBar *thisMenuBar = GetMenuBar();

	FixCommandSubmenu(thisMenuBar);
	FixGraphSubmenu(thisMenuBar);
	FixDialogSubmenu(thisMenuBar);


} /* CarapaceSplitTextFrame::UpdateMenuBar() */

void CarapaceSplitTextFrame::Open(void)
{
	thisApp->NewSplitTextFrame(FALSE);
}

bool CarapaceSplitTextFrame::SaveWindow(void)
{
	// return TRUE for a successful save, FALSE otherwise
	// do a save-as if untitled, otherwise a straight write
	// to the file in the frame title
	if(Untitled)
	{
		return(SaveWindowAs());
	}
	{
	wxLogNull logno;
	//begin original codes
	if( !(output_ctrl->SaveFile(GetTitle())) )
	{
		(void) wxMessageBox(CPC_STRING_FILESAVEFAIL
			/*"File Save Unsuccessful"*/, 
			CPC_STRING_ERROR /*"ERROR!"*/, wxOK|wxCENTRE);
		return FALSE;
	}
	//end of original codes
	}// ~wxLogNull called, old log sink restored
	output_ctrl->DiscardEdits();
	return TRUE;
} 

bool CarapaceSplitTextFrame::SaveWindowAs(void)
{
	// return TRUE for a successful save, FALSE otherwise
	// prompt for a file name and save to that file
	wxString       s = wxFileSelector(CPC_STRING_SAVEFILENAME
		/*"File for output window"*/, "", "", "", "*",wxSAVE);
	int decision = wxYES;

	if (!s)						// pressed cancel
	{
		return FALSE;
	}

	if (wxFileExists(s))
	{
		decision = wxMessageBox(CPC_STRING_FILEEXISTS,CPC_STRING_ALERT,wxYES_NO);
	}

	if (decision != wxYES)
	{
		return FALSE;
	}

	{
	wxLogNull logno;
	//begin original codes

	if( !(output_ctrl->SaveFile(s)) )
	{
		(void) wxMessageBox(CPC_STRING_FILESAVEFAIL
			/*"File Save Unsuccessful"*/, 
			CPC_STRING_ERROR /*"ERROR!"*/, wxOK|wxCENTRE);
		return FALSE;
	}
	//end of original codes
	}// ~wxLogNull called, old log sink restored

 	// housekeeping after a frame gets a new name
	Untitled = FALSE;
	SetTitle(s);
	thisApp->UpdateMenus();
	output_ctrl->DiscardEdits();
	return TRUE;
} 

void CarapaceSplitTextFrame::OnQuit(wxCommandEvent & WXUNUSED(event))
{
	thisApp->DoQuit(CPC_EXIT_SAVECANCANCEL);
}

void CarapaceSplitTextFrame::FastQuit(wxCommandEvent & WXUNUSED(event))
{
	thisApp->DoQuit(CPC_EXIT_FASTFORCEQUIT);
}

/*
  Returns TRUE if it is OK to close this frame.  It's OK if the file is
  titled and unmodified or if the user doesn't wish to save the file or if
  the file is successfully saved.  Cancels and unsuccessful saves return
  false.
*/
bool CarapaceSplitTextFrame::OKToClose(int closeStatus)
{
	int          decision;
	wxString message;

	if (Untitled || output_ctrl->IsModified())
	{
		message << CPC_STRING_ASKSAVEWINDOW
			/*"Do you wish to save the window\n"*/ << GetTitle() << 
			CPC_STRING_QUESTION /*"?"*/;
		if(closeStatus == CPC_EXIT_SAVECANCANCEL)
		{
			decision = wxMessageBox(message,
				CPC_STRING_ALERT,
				wxYES_NO | wxCANCEL | wxCENTRE);
		}
		else
		{
			decision = wxMessageBox(message,
				CPC_STRING_ALERT,
				wxYES_NO | wxCENTRE);
		}
		if( decision == wxCANCEL )
		{
			return FALSE;
		}
		if( decision == wxYES )
		{
			if(!SaveWindow() && (closeStatus == CPC_EXIT_SAVECANCANCEL))
			{
				return FALSE;
			}
		}
	} /*if (Untitled || text_ctrl->Modified())*/
	return (TRUE);
} 

/*
  If we hit the close button, check to see if OKtoclose
  (that saves if necessary).  If OK, cut the frame from
  the main list (frame will be deleted by other mechanisms)
  If this is the last text frame, do a application quit as well.
*/
void CarapaceSplitTextFrame::OnClose(wxCloseEvent& event)
{

	bool SaveOnClose = TRUE;
	if(!event.CanVeto())
	{
		SaveOnClose = FALSE;
	}

	if (thisApp->numTextFrames == 1)
	{
		if (!SaveOnClose)
		{
			thisApp->DoQuit(CPC_EXIT_FASTFORCEQUIT);
		}
		else if (event.CanVeto())
		{
			thisApp->DoQuit(CPC_EXIT_SAVECANCANCEL);
		}
		else
		{
			thisApp->DoQuit(CPC_EXIT_SAVEFORCEQUIT);
		}
		return ; // if we get to here, we cancelled
	} /*if (thisApp->numTextFrames == 1)*/


	if(SaveOnClose && (event.CanVeto() ? !OKToClose(CPC_EXIT_SAVECANCANCEL) : !OKToClose(CPC_EXIT_SAVEFORCEQUIT)))
	{
		return ;
	}

	thisApp->CutSplitTextFrame(TextFrameIndex);
	Show(FALSE);
	
	if (thisApp->NVisible() <= 0)
	{
		/* everything invisible; display first window */
		if(thisApp->STFrameList.GetCount() > 0)
		{
			thisApp->TextFrameFront(thisApp->STFrameList.GetLast()->GetData()->TextFrameIndex);
		} 
	}
	wxYield();
	event.Skip();
	
} 

void CarapaceSplitTextFrame::OnPageSetup(wxCommandEvent& WXUNUSED(event))
{
    (*thisApp->PageSetupData) = * thisApp->PrintData;

    wxPageSetupDialog pageSetupDialog(this, thisApp->PageSetupData);
    pageSetupDialog.ShowModal();

    (*thisApp->PrintData) = pageSetupDialog.GetPageSetupData().GetPrintData();
    (*thisApp->PageSetupData) = pageSetupDialog.GetPageSetupData();
}


void CarapaceSplitTextFrame::OnPrint(wxCommandEvent & WXUNUSED(event))
{
	wxPrintDialogData printDialogData(*(thisApp->PrintData));

	wxPrinter printer(& printDialogData);
	CarapaceTPrintout printout("My printout",output_ctrl);
	if (!printer.Print(this,&printout, TRUE))
	{
		if (wxPrinter::GetLastError() == wxPRINTER_ERROR)
		{
			CarapaceAlert(CPC_STRING_PRINTINGERROR /*"Printing error"*/);
		}
	}
	else
	{
		*(thisApp->PrintData) = printer.GetPrintDialogData().GetPrintData();
	}
}

void CarapaceSplitTextFrame::Undo(wxCommandEvent& event)
{
	output_ctrl->Undo();
}

void CarapaceSplitTextFrame::SetUndoItem(wxUpdateUIEvent & event)
{
	switch(output_ctrl->UndoStatus)
	{
		case CANTUNDO:
			event.SetText(CPC_STRING_CANTUNDO /*"Can't &Undo"*/);
			event.Enable(FALSE);
			break;

		case OUTPUTUNDO:
			event.SetText(CPC_STRING_OUTPUTUNDO /*"&Undo Output"*/);
			event.Enable(TRUE);
			break;

		case CUTUNDO:
			event.SetText(CPC_STRING_CUTUNDO /*"&Undo Cut"*/);
			event.Enable(TRUE);
			break;
			
		case PASTEUNDO:
			event.SetText(CPC_STRING_PASTEUNDO /*"&Undo Paste"*/);
			event.Enable(TRUE);
			break;
			
		case TYPINGUNDO:
		case DELETEUNDO:
			event.SetText(CPC_STRING_TYPINGUNDO /*"&Undo Typing"*/);
			event.Enable(TRUE);
			break;
			
		case TYPINGREDO:
		case DELETEREDO:
			event.SetText(CPC_STRING_TYPINGREDO /*"Redo Typing"*/);
			event.Enable(TRUE);
			break;
			
		case OUTPUTREDO:
			event.SetText(CPC_STRING_OUTPUTREDO /*"Redo Output"*/);
			event.Enable(TRUE);
			break;
	}
}
			
void CarapaceSplitTextFrame::Execute(wxCommandEvent& event)
{
	output_ctrl->Execute();
}

void CarapaceSplitTextFrame::Forward(wxCommandEvent& event)
{
	command_ctrl->DownHistory();
}

void CarapaceSplitTextFrame::Back(wxCommandEvent& event)
{
	command_ctrl->UpHistory();
}

void CarapaceSplitTextFrame::ClearCommand(wxCommandEvent& event)
{
	wxString nothing = "";
	command_ctrl->SetContent(nothing);
	command_ctrl->SetFocus();
}

#define DO_EDIT_MENU(id,callback)   \
	if(which == wxXmlResource::GetXRCID(#id))  \
	{ \
		callback(); \
		return; \
	}

void CarapaceSplitTextFrame::DoEdits(wxCommandEvent &event)
{
	int which = event.GetId();

	DO_EDIT_MENU(CPC_EDIT_CUT,output_ctrl->Cut)
	DO_EDIT_MENU(CPC_EDIT_COPY,output_ctrl->Copy)
	DO_EDIT_MENU(CPC_EDIT_COPYTOEND,output_ctrl->CopyToEnd)
	DO_EDIT_MENU(CPC_EDIT_EXECUTE,output_ctrl->Execute)
	DO_EDIT_MENU(CPC_EDIT_UPHISTORY,command_ctrl->UpHistory)
	DO_EDIT_MENU(CPC_EDIT_DOWNHISTORY,command_ctrl->DownHistory)
	if(which == wxXmlResource::GetXRCID("CPC_EDIT_PASTE"))
	{
		if( (wxWindow *) output_ctrl == FindFocus())
		{
			output_ctrl->Paste();
		}
		else
		{
			command_ctrl->Paste();
		}
	}
}

void CarapaceSplitTextFrame::MenuInterrupt(wxCommandEvent & WXUNUSED(event))
{
        wxString msg = CPC_STRING_INTERRUPT; /*"\n*****  Interrupt  *****\n"*/
        long windLength,wasrunning = 0;
	if(thisApp->Running == CPC_RUNNING)  
	{
		wasrunning = 1;
		thisApp->Interrupt();
	}
	if(command_ctrl->GetLastPosition() > 0)
	{
		msg << "\nIncomplete command: " << 
			command_ctrl->GetValue() << "\n";
		command_ctrl->SetValue("");
		windLength = output_ctrl->GetLastPosition();
#ifdef USEAPPENDTEXT
		output_ctrl->SetSelection(windLength,windLength);
		output_ctrl->AppendText(msg.c_str());
#else
		output_ctrl->Replace(windLength,windLength,msg);
#endif /*USEAPPENDTEXT*/
	}
	else if(!wasrunning)
	{
		wxMessageBox("Nothing to interrupt");
	}
}

void CarapaceSplitTextFrame::Hide(void)
{
	Show(FALSE);
}

void CarapaceSplitTextFrame::EnableHide(void)
{
	wxMenuBar *thisMenuBar = GetMenuBar();

	// only enable hide if more than one window shown
	if(thisApp->NVisible() > 1)
	{
		thisMenuBar->Enable(wxXmlResource::GetXRCID("CPC_WINDOWS_HIDE"),TRUE);
	}
	else
	{
		thisMenuBar->Enable(wxXmlResource::GetXRCID("CPC_WINDOWS_HIDE"),FALSE);
	}
}

void CarapaceSplitTextFrame::DoFastClose(void)
{
	wxCloseEvent ce = wxCloseEvent();
	ce.SetCanVeto(FALSE);
	OnClose(ce);
}

void CarapaceSplitTextFrame::DoClose(void)
{
	wxCloseEvent ce = wxCloseEvent();
	ce.SetCanVeto(TRUE);
	OnClose(ce);
}

void CarapaceSplitTextFrame::NewWindow(void)
{
	int windno;
	if(ResourceString != "" && MenuResourceInUse != "")
	{
		windno = thisApp->NewSplitTextFrame(TRUE,ResourceString,MenuResourceInUse);
	}
	else
	{
		windno = thisApp->NewSplitTextFrame(TRUE);
	}
	if(windno < 0)
	{
		return;
	}
}

void CarapaceSplitTextFrame::ShowTextFrame(wxCommandEvent& event)
{
	int index = event.GetId() - CPC_TEXT_WINDOWS_BASE;
	thisApp->TextFrameFront(thisApp->STFrameList.Item(index)->GetData()->TextFrameIndex);
}

void CarapaceSplitTextFrame::ShowGraphFrame(wxCommandEvent& event)
{
	int index = event.GetId() - CPC_GRAPH_WINDOWS_BASE;
        if(index <= (int) thisApp->GFrameList.GetCount() - 1)
        {
                thisApp->GraphFrameFront(thisApp->GFrameList.Item(index)->GetData()->GraphFrameIndex);
        }
        else
        {
                index -= thisApp->GFrameList.GetCount();
                thisApp->PanelFrameFront(thisApp->PGFrameList.Item(index)->GetData()->PanelFrameIndex);
        }
}

void CarapaceSplitTextFrame::OnFont(wxCommandEvent& WXUNUSED(event))
{
// adapted from the font.cpp sample in the wx distribution
	wxFontData data;
	data.SetInitialFont(output_ctrl->GetFont());

	wxFontDialog dialog((wxWindow *)this, data);
	if ( dialog.ShowModal() == wxID_OK )
	{
		wxFontData retData = dialog.GetFontData();
		wxFont font = retData.GetChosenFont();

		if ( font.Ok() )
		{
			outputFont = new wxFont(font);
			commandFont = new wxFont(font);
			command_ctrl->SetDefaultStyle(wxTextAttr(*outputFGColor,
				*outputBGColor,*outputFont));
			command_ctrl->SetStyle(0,command_ctrl->GetLastPosition(),
				wxTextAttr(*outputFGColor,*outputBGColor,*outputFont));
			command_ctrl->SetFont(*outputFont);
			output_ctrl->SetDefaultStyle(wxTextAttr(*outputFGColor,
				*outputBGColor,*outputFont));
			output_ctrl->SetStyle(0,output_ctrl->GetLastPosition(),
				wxTextAttr(*outputFGColor,*outputBGColor,*outputFont));
			output_ctrl->SetFont(*outputFont);
			Refresh();
		}
	}
}

void CarapaceSplitTextFrame::BackgroundColor(wxCommandEvent& WXUNUSED(event))
{
	wxColourDialog *dialog = new wxColourDialog(this);
	dialog->SetTitle(CPC_STRING_BACKGROUNDCOLOR /*"Choose the background color"*/);
	if(dialog->ShowModal() == wxID_OK)
	{
		wxColourData retData = dialog->GetColourData();
		wxColour *col = new wxColor(retData.GetColour());
		outputBGColor = col;
		output_ctrl->SetBackgroundColour(*col);
		command_ctrl->SetBackgroundColour(*col);
		//output_ctrl->SetFont(*outputFont);
		//command_ctrl->SetFont(*outputFont);
		Refresh();

	}
	dialog->Destroy();
}

void CarapaceSplitTextFrame::ForegroundColor(wxCommandEvent& WXUNUSED(event))
{
	wxColourDialog *dialog = new wxColourDialog(this);
	dialog->SetTitle(CPC_STRING_FOREGROUNDCOLOR /*"Choose the foreground color"*/);
	if(dialog->ShowModal() == wxID_OK)
	{
		wxColourData retData = dialog->GetColourData();
		wxColour col = retData.GetColour();
#if 0
		output_ctrl->SetForegroundColour(col);
		output_ctrl->Refresh();
		command_ctrl->SetForegroundColour(col);
		command_ctrl->Refresh();
#endif /* 0 */
		outputFGColor = &col;
		command_ctrl->SetStyle(0,output_ctrl->GetLastPosition(),
			wxTextAttr(*outputFGColor,*outputBGColor));
		output_ctrl->SetStyle(0,output_ctrl->GetLastPosition(),
			wxTextAttr(*outputFGColor,*outputBGColor));
		output_ctrl->SetFont(*outputFont);
		output_ctrl->Refresh();
		command_ctrl->SetFont(*outputFont);
		command_ctrl->Refresh();
	}
	dialog->Destroy();
}

void CarapaceSplitTextFrame::GoToTop(void)
{
	output_ctrl->ShowPosition(0);
	output_ctrl->SetInsertionPoint(0);
}

void CarapaceSplitTextFrame::GoToEnd(void)
{
	output_ctrl->ShowPosition(output_ctrl->GetLastPosition());
	output_ctrl->SetInsertionPoint(output_ctrl->GetLastPosition());
}

void CarapaceSplitTextFrame::GoToCommandPoint(void)
{
	command_ctrl->SetFocus();
	//wxFocusEvent ev;
	//command_ctrl->OnSetFocus(ev);
	//command_ctrl->Refresh(FALSE);
}


#define DO_UPDATE_UI(id,callback,ev)   \
	if(which == wxXmlResource::GetXRCID(#id))  \
	{ \
		callback(ev); \
		return; \
	}

void CarapaceSplitTextFrame::UpdateUserCommand(wxUpdateUIEvent& event)
{
	int which = event.GetId();
	wxMenuBar *bar;
	cpcMenuItem *item;
	wxString newlabel;

	DO_UPDATE_UI(CPC_EDIT_UNDO, CarapaceSplitTextFrame::SetUndoItem,event)
	DO_UPDATE_UI(CPC_WINDOWS_HIDE, CarapaceSplitTextFrame::EnableHide,)

	bar = GetMenuBar();
	item = (cpcMenuItem *) bar->FindItem(which);
	if(!item || item->GetKind() != wxITEM_NORMAL || item->GetSubMenu())
	{
		return;
	}
	if(item->selectionlabel !="")
	{
		event.SetText(item->label);

		long selfrom,selto;
		output_ctrl->GetSelection(&selfrom,&selto);
		if(selfrom != selto)
		{
			event.SetText(item->selectionlabel);
		}
		command_ctrl->GetSelection(&selfrom,&selto);
		if(selfrom != selto)
		{
			event.SetText(item->selectionlabel);
		}
	}
	else
	{
		event.SetText(item->label);
	}
}

#define DO_EVT_MENU(id,callback,ev)   \
	if(which == wxXmlResource::GetXRCID(#id))  \
	{ \
		callback(ev); \
		return; \
	}

void CarapaceSplitTextFrame::ProcessUserCommand(wxCommandEvent& event)
{
	CarapaceCommand *cptr;
	cpcMenuItem *menuItem;
	wxString base,full;
	int i;

	int which = event.GetId();

	DO_EVT_MENU(CPC_FILE_OPEN, CarapaceSplitTextFrame::Open,)
	DO_EVT_MENU(CPC_FILE_SAVEWINDOW, CarapaceSplitTextFrame::SaveWindow,)
	DO_EVT_MENU(CPC_FILE_SAVEWINDOWAS, CarapaceSplitTextFrame::SaveWindowAs,)
	DO_EVT_MENU(CPC_FILE_PAGESETUP, CarapaceSplitTextFrame::OnPageSetup,event)
	DO_EVT_MENU(CPC_FILE_PRINTSELECTION, CarapaceSplitTextFrame::OnPrint,event)
	DO_EVT_MENU(CPC_FILE_INTERRUPT, CarapaceSplitTextFrame::MenuInterrupt,event)
	DO_EVT_MENU(CPC_FILE_QUIT, CarapaceSplitTextFrame::OnQuit,event)
	DO_EVT_MENU(CPC_FILE_FASTQUIT, CarapaceSplitTextFrame::FastQuit,event)


	DO_EVT_MENU(CPC_EDIT_UNDO, CarapaceSplitTextFrame::Undo,event)
	DO_EVT_MENU(CPC_EDIT_CUT, CarapaceSplitTextFrame::DoEdits,event)
	DO_EVT_MENU(CPC_EDIT_COPY, CarapaceSplitTextFrame::DoEdits,event)
	DO_EVT_MENU(CPC_EDIT_PASTE, CarapaceSplitTextFrame::DoEdits,event)
	DO_EVT_MENU(CPC_EDIT_COPYTOEND, CarapaceSplitTextFrame::DoEdits,event)
	DO_EVT_MENU(CPC_EDIT_EXECUTE, CarapaceSplitTextFrame::DoEdits,event)
	DO_EVT_MENU(CPC_EDIT_UPHISTORY, CarapaceSplitTextFrame::DoEdits,event)
	DO_EVT_MENU(CPC_EDIT_DOWNHISTORY, CarapaceSplitTextFrame::DoEdits,event)

	DO_EVT_MENU(CPC_WINDOWS_HIDE, CarapaceSplitTextFrame::Hide,)
	DO_EVT_MENU(CPC_WINDOWS_CLOSE, CarapaceSplitTextFrame::DoClose,)
	DO_EVT_MENU(CPC_WINDOWS_FASTCLOSE, CarapaceSplitTextFrame::DoFastClose,)
	DO_EVT_MENU(CPC_WINDOWS_NEWWINDOW, CarapaceSplitTextFrame::NewWindow,)
	DO_EVT_MENU(CPC_WINDOWS_GRAPH, CarapaceSplitTextFrame::DoNothing,)
	DO_EVT_MENU(CPC_WINDOWS_TEXTWFONT, CarapaceSplitTextFrame::OnFont,event)
	//DO_EVT_MENU(CPC_WINDOWS_TEXTWFCOLOR, CarapaceSplitTextFrame::ForegroundColor,event)
	//DO_EVT_MENU(CPC_WINDOWS_TEXTWBCOLOR, CarapaceSplitTextFrame::BackgroundColor,event)
	DO_EVT_MENU(CPC_WINDOWS_GOTOTOP, CarapaceSplitTextFrame::GoToTop,)
	DO_EVT_MENU(CPC_WINDOWS_GOTOEND, CarapaceSplitTextFrame::GoToEnd,)
	DO_EVT_MENU(CPC_WINDOWS_GOTOCOMMANDPOINT, CarapaceSplitTextFrame::GoToCommandPoint,)
	//DO_EVT_MENU(CPC_WINDOWS_PAGEUP, CarapaceSplitTextFrame::DoNothing,)
	//DO_EVT_MENU(CPC_WINDOWS_PAGEDOWN, CarapaceSplitTextFrame::DoNothing,)

	
	
	base = "CPC_TEXT_WINDOWS_BASE";
	for(i=0;i<thisApp->numTextFrames;i++)
	{
       		full = base;
       		full << i;
		if(which == wxXmlResource::GetXRCID(full.GetData()) )
		{
			thisApp->TextFrameFront(thisApp->STFrameList.Item(i)->GetData()->TextFrameIndex);
			return;
		}
	}

	
	base = "CPC_GRAPH_WINDOWS_BASE";
	for(i=0;i<thisApp->numGraphFrames;i++)
	{
       		full = base;
       		full << i;
		if(which == wxXmlResource::GetXRCID(full.GetData()) )
		{
			thisApp->GraphFrameFront(thisApp->GFrameList.Item(i)->GetData()->GraphFrameIndex);
			return;
		}
	}

	for(i=0;i<thisApp->numGraphFrames;i++)
	{
       		full = base;
       		full << (i+thisApp->numGraphFrames);
		if(which == wxXmlResource::GetXRCID(full.GetData()) )
		{
			thisApp->PanelFrameFront(thisApp->PGFrameList.Item(i)->GetData()->PanelFrameIndex);
			return;
		}
	}

	
	base = "CPC_DIALOG_WINDOWS_BASE";
	for(i=0;i<thisApp->DFrameList.GetCount();i++)
	{
       		full = base;
       		full << i;
		if(which == wxXmlResource::GetXRCID(full.GetData()) )
		{
			//thisApp->TextFrameFront(thisApp->STFrameList.Item(i)->GetData()->TextFrameIndex);
			thisApp->DFrameList.Item(i)->GetData()->Raise();
			return;
		}
	}



	menuItem = (cpcMenuItem *) GetMenuBar()->FindItem(which);
	if(!menuItem || menuItem->action == "")
	{
		return;
	}

	if(menuItem->leaveoncommandline)
	{
		command_ctrl->AppendText(menuItem->action);
		return;
	}

	wxCommandEvent procevent(wxEVT_COMMAND_MENU_SELECTED,CPC_GUI_PROCESSCOMMAND);
	cptr = new CarapaceCommand;
	cptr->command = menuItem->action;
	cptr->command << "\n";
	if(menuItem->selectionaction != "")
	{
		long selfrom,selto;
		command_ctrl->GetSelection(&selfrom,&selto);
		if(selfrom != selto)
		{
			command_ctrl->GetSelectionContents();
			cptr->command.Printf(menuItem->selectionaction.c_str(),
				command_ctrl->Selection.c_str());
			cptr->command << "\n";
		}
		else
		{
			output_ctrl->GetSelection(&selfrom,&selto);
			if(selfrom != selto)
			{
				output_ctrl->GetSelectionContents();
				cptr->command.Printf(menuItem->selectionaction.c_str(),
					output_ctrl->Selection.c_str());
				cptr->command << "\n";
			}
		}
	}
			
	cptr->frame = TextFrameIndex;
	cptr->hidecommand = menuItem->hidecommand;
	cptr->hideoutput = menuItem->hideoutput;
	procevent.SetClientData((void *) cptr);
	wxPostEvent(thisApp,procevent);
}

void CarapaceSplitTextFrame::DoNothing(void)
{
}

void CarapaceSplitTextFrame::DoSpinner(wxSpinEvent& event)
{
	int val;
	
	val = history_spinner->GetValue();
	if((unsigned int) val > thisApp->History->commands.GetCount())
	{
		val = thisApp->History->commands.GetCount();
		history_spinner->SetValue(val);
		return;
	}
	//command_ctrl->SetHistoryIndex(val);

        wxCommandEvent procevent(wxEVT_COMMAND_MENU_SELECTED,
		wxXmlResource::GetXRCID("CPC_GUI_SETHISTORYINDEX"));
        procevent.SetExtraLong(val);
        wxPostEvent(command_ctrl,procevent);
}


void CarapaceSplitTextFrame::SetMenuBarFromString(wxString resourcestr, wxString menuname)
{
	wxMenuBar *bar;
	//wxXmlResource *newresource ;
	CarapaceXmlResource *newresource ;

	//newresource = new wxXmlResource();
	newresource = new CarapaceXmlResource();
	if(newresource)
        {
		newresource->AddHandler(new wxMenuXmlHandler);
		newresource->AddHandler(new cpcMenuXmlHandler);
		newresource->AddHandler(new wxMenuBarXmlHandler);
		
		newresource->LoadFromString(resourcestr);
		bar = newresource->LoadMenuBar(menuname.c_str());
		if(bar)
		{
			SetMenuBar(bar);
			ResourceString = resourcestr;
			MenuResourceInUse = menuname;
			if(Resource)
			{
				delete Resource;
			}
			Resource = newresource;

			if(Accelerators)
			{
				delete(Accelerators);
			}
			Accelerators = new ArrayofAccelerators;
			if(Accelerators)
			{
				GetAccelArrayFromMenuBar(*Accelerators,bar);
			}
		}
	}
}

