/*
*(C)* The file is part of the source distribution of Carapace
*(C)* version 0.5 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* This file is distributed under the GNU General Public License
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

#include "cpcplatform.h"



// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/fontdlg.h"
#include "wx/colour.h"
#include "wx/caret.h"
#include "wx/print.h"
#include "wx/clipbrd.h"
#include "wx/dataobj.h"
#include "wx/ffile.h"


#include <math.h>

#include "carapace.h"

extern CarapaceApp  *thisApp;


#undef UNDEFINED

BEGIN_EVENT_TABLE(CarapaceOutputCtrl, wxTextCtrl)
	EVT_CHAR(CarapaceOutputCtrl::OnChar)
	//EVT_IDLE(CarapaceOutputCtrl::OnIdle)
END_EVENT_TABLE()



CarapaceOutputCtrl::CarapaceOutputCtrl(wxWindow *frame, wxWindowID x, 
	const wxString &title, const wxPoint &pos, const wxSize &size,  long style):
	wxTextCtrl(frame, x, title, pos, size, style)
{
	UndoBuffer.Alloc(UNDOBUFFERINITIALALLOCATION);
	Frame = (CarapaceSplitTextFrame *) frame;
	UndoStatus = CANTUNDO;

} /*CarapaceOutputCtrl::CarapaceOutputCtrl()*/

CarapaceOutputCtrl::~CarapaceOutputCtrl(void)
{
}


void CarapaceOutputCtrl::clearUndoInfo(void)
{
	UndoStatus = CANTUNDO;
	PasteLength = -1;
	UndoPlace = -1;
	LastTypingPos = -1;
	LastCmdStrPos = -1;
	LastLength = -1;
	UndoBuffer.Empty();
} /*CarapaceOutputCtrl::clearUndoInfo*/


void CarapaceOutputCtrl::ToUndoBuffer(void)
{
	long          selStart,selEnd;
	
	GetSelection(&selStart,&selEnd);
	if (selStart != selEnd) 
 	{
		GetSelectionContents(); 
		if (Selection.Length() != 0)
		{
			UndoBuffer = Selection;
		}
		else
		{
			UndoBuffer.Empty();
		}
	}
	UndoPlace = selStart;
} /*CarapaceOutputCtrl::ToUndoBuffer()*/

long CarapaceOutputCtrl::FromUndoBuffer(void)
{
	long        scrapLength;
	
	if (UndoPlace != -1)
	{
		scrapLength = UndoBuffer.Length();
		Replace(UndoPlace,UndoPlace,UndoBuffer);
		SetSelection(UndoPlace,UndoPlace+scrapLength);
	}
	else
	{
		scrapLength = 0;
	}
	clearUndoInfo(); /* can't redo */
	
	return (scrapLength);
	
} /*CarapaceOutputCtrl::FromUndoBuffer()*/

void CarapaceOutputCtrl::GetSelection(long *from, long *to)
{
	long tmp;
	wxTextCtrl::GetSelection(from,to);
	if(*from > *to)
	{
		tmp = *to;
		*to = *from;
		*from = tmp;
	}
}

void CarapaceOutputCtrl::GetSelectionContents(void)
{
	GetSelectionContents(-1,-1);
}

void CarapaceOutputCtrl::GetSelectionContents(long from = -1, long to = -1)
{
	
	long i;

	if (from < 0 || to < 0)
	{
		// defaults to get the selection on screen
		GetSelection ( &from,  &to);
	}
	else
	{// can specify what to get
		i = GetLastPosition();
		from = (from < 0 ? 0 : from);
		from = (from > i ? i : from);
		to = (to < 0 ? 0 : to);
		to = (to > i ? i : to);
	}

	Selection.Clear();

	if (from == to)
	{
		return;
	}

	// God help us.  We read it in line by line
	long            fromLine, toLine, fromLineChar, toLineChar;
	long            fillingLineLength;

	PositionToXY(from,&fromLineChar, &fromLine);
	PositionToXY(to,&toLineChar, &toLine);

	if( fromLine == toLine)
	{
		Selection << GetLineText(fromLine).Mid(fromLineChar,toLineChar-fromLineChar);
	}
	else
	{
		fillingLineLength = GetLineLength(fromLine);
		Selection << GetLineText(fromLine).Right(fillingLineLength-fromLineChar);
		Selection << '\n';
		fromLine++;
		while(fromLine < toLine)
		{
			Selection << GetLineText(fromLine++) << '\n';
		}
		Selection << GetLineText(fromLine).Left(toLineChar);
	}
} /*CarapaceOutputCtrl::GetSelectionContents()*/

long CarapaceOutputCtrl::GetInsertionLine(void)
{
	long lineno,charno;

	PositionToXY(GetInsertionPoint(),&charno,&lineno);
	return(lineno);
}


void CarapaceOutputCtrl::OnChar(wxKeyEvent& event)
{

	long       ch,count,i;
	bool       moveBack;
	long       selStart, selEnd;
	wxAcceleratorEntry entry;

	ch =  event.m_keyCode;

	if (ch == WXK_TAB && event.ControlDown())
	{
		Frame->command_ctrl->SetFocus();
		return;
	}

	if(ch == WXK_TAB)
	{
	// send genuine tabs to output control
		event.Skip();
		return;
	}
// sometimes you have to hit things with a really big brick.  Some text widgets (e.g., gtk)
// grab hold of key presses before the accelerator table can deal with them.  To get around
// this, we keep our own list of accelerators and handle the troublesome cases.
	if(ch < 26 && Frame->Accelerators)
	{
#if defined(__WXMSW__)
// windows seems to do a ctrl-V before control ever gets here, so
// we must ignore ctrl-V
		if(ch == 22) // v
		{
			return;
		}
#endif // defined(__WXMSW__)
		count = Frame->Accelerators->GetCount();
		for(i=0;i<count;i++)
		{
			entry = Frame->Accelerators->Item(i);
			if( (ch == entry.GetKeyCode()-64 || ch == entry.GetKeyCode()-96) &&
				entry.GetFlags() == wxACCEL_CTRL )
			{
				wxCommandEvent command_event( wxEVT_COMMAND_MENU_SELECTED, entry.GetCommand() );
				Frame->GetEventHandler()->ProcessEvent( command_event );
				return;
			}
		}
	}
	if (ControlActionStatus)
	{
/*
   if ControlActionStatus, then the OnChar came from some roundabout
   call from elsewhere.  Send the keystroke, but don't update anything else,
   because it is done elsewhere
*/
		event.Skip();
		return;
	} /*if (ControlActionStatus)*/


	GetSelection(&selStart, &selEnd);

	moveBack = (ch == WXK_BACK || ch == WXK_DELETE);

	if (ch == WXK_BACK && selStart == selEnd && selStart == 0)
	{
		/*
		  Ignore BS at start of window or at CmdStrPos with nothing selected
		*/
		return;
	}

	if (ch == WXK_DELETE && selStart == selEnd &&
		selStart == GetLastPosition())
	{
		/*
		  Ignore DEL at end of window 
		*/
		return;
	}


	/*
	   If we get here, then key will change command window
	*/

	bool      nullSelection = selStart == selEnd;

	if (!nullSelection)
	{/* whatever the key, it will clear or replace something */
		clearUndoInfo();
	} /*if (!nullSelection)*/

	if (UndoStatus != TYPINGUNDO && UndoStatus != DELETEUNDO )
	{
		clearUndoInfo(); /* forget undo information */
		UndoPlace = selStart;
		if (selStart == selEnd)
		{
			UndoStatus = TYPINGUNDO;
		}
	} /*if (UndoStatus != TYPINGUNDO && UndoStatus != DELETEUNDO )*/

	if (UndoStatus == TYPINGUNDO || UndoStatus == DELETEUNDO)
	{
		if (selStart != LastTypingPos)
		{/* we've moved! */
			clearUndoInfo();
			UndoStatus = TYPINGUNDO;
			UndoPlace = selStart;
		}
		if (moveBack && selStart <= UndoPlace)
		{
			clearUndoInfo();
		}
	} /*if (UndoStatus == TYPINGUNDO || UndoStatus == DELETEUNDO)*/


	if (!nullSelection)
	{
		ToUndoBuffer();

		UndoStatus = DELETEUNDO;	/* deletion or replacement can be undone */
	} /*if (!nullSelection)*/
	
	event.Skip();

	// at least in GTK, the event.Skip doesn't actually commit the
	// change right then, so we need to allow for it
	//LastTypingPos = GetInsertionPoint() - (selEnd - selStart) + 1;
	LastTypingPos = selStart + 1;

	if (moveBack && selStart != UndoPlace)
	{
		ch = '0'; //all we really need is something that is not a backspace
	}
	LastChar = ch;

} /*CarapaceOutputCtrl::OnChar()*/

void CarapaceOutputCtrl::Cut(void)
{
	long           selStart,selEnd;
	
	GetSelection(&selStart,&selEnd);
	if (selStart != selEnd)
 	{
		GetSelectionContents(selStart,selEnd);
		Copy();
		ControlActionStatus = 1;
		clearUndoInfo();
		UndoPlace = selStart;
		ToUndoBuffer();
		Replace(selStart,selEnd,"");
		UndoStatus = CUTUNDO;
		LastChar = WXK_BACK;
		ControlActionStatus = 0;

	} /*if (selStart != selEnd)*/
} /*CarapaceOutputCtrl::Cut()*/

void CarapaceOutputCtrl::Copy(void)
{
	long            selStart,selEnd;

	GetSelection(&selStart,&selEnd);
	if (UndoStatus == CUTUNDO)
	{
		clearUndoInfo();
	}
	GetSelectionContents(selStart,selEnd);
	if (wxTheClipboard->Open())
	{
		// Data objects are held by the clipboard, 
		// so do not delete them in the app.
		wxTheClipboard->SetData( new wxTextDataObject(Selection) );
		wxTheClipboard->Close();
	}

} /*CarapaceOutputCtrl::Copy()*/


void CarapaceOutputCtrl::Paste(void)
{
	long         selStart, selEnd, clipLength;

	if(FindFocus() != this)
	{
		Frame->command_ctrl->Paste();
		return;
	}

	GetSelection(&selStart, &selEnd);
	if (selStart == selEnd)
	{
		selStart = selEnd = GetInsertionPoint();
	}

	wxTextDataObject clipData;
	if (wxTheClipboard->Open())
	{
		if (wxTheClipboard->IsSupported( wxDF_TEXT ))
		{
			wxTheClipboard->GetData( clipData );
		}  
		wxTheClipboard->Close();
	}

	
	clipLength = clipData.GetTextLength() - 1;


	if (clipLength > 0)
	{
		long         displacedLength = selEnd - selStart;

		ControlActionStatus = 1;

		// need to worry about overfilling a window here
		clearUndoInfo();
		PasteLength = clipLength;
		UndoStatus = PASTEUNDO;
		if (displacedLength > 0)
		{
			ToUndoBuffer();				// save what we're pasting over
		}

		Replace(selStart,selEnd,clipData.GetText());

		UndoPlace = selStart;
		ControlActionStatus = 0;
	} /*if (clipLength > 0)*/
} /*CarapaceOutputCtrl::Paste()*/

void CarapaceOutputCtrl::Undo(void)
{
	long       scrapLength,selStart,selEnd,location,length,change;
	wxString   tmpstring;
	
	ControlActionStatus = 1;
	switch (UndoStatus)
	{
	  case PASTEUNDO:
		/* undo result of previous paste*/
		Replace(UndoPlace, UndoPlace+PasteLength, "");
		/* restore what it replaced */
		scrapLength = FromUndoBuffer();
		SetFocus();
		break;
		
	  case DELETEUNDO:
	  case DELETEREDO:
		/* Must have been deletion or replacement that enabled undo */
		location = UndoPlace;
		length = LastTypingPos - UndoPlace;
		scrapLength = FromUndoBuffer(); /* restore what was deleted*/

		GetSelection(&selStart,&selEnd);
		if (LastChar != WXK_BACK)
		{/* was replacement of selection by typing, so remove it */
			SetSelection(selEnd, selEnd+length);
			ToUndoBuffer();
			UndoPlace = location;
			LastTypingPos = location + scrapLength;
			Replace(selEnd,selEnd+length,"");
			SetSelection(selStart, selEnd);
			UndoStatus = (UndoStatus == DELETEREDO) ? DELETEUNDO : DELETEREDO;
		} /*if (LastChar != WXK_BACK)*/
		SetFocus();
		break;
		
	  case CUTUNDO:
		/* undo previous cut */
		location = UndoPlace;
		change = strlen(UndoBuffer);
		scrapLength = FromUndoBuffer();
		clearUndoInfo();			/* can't undo*/
		SetSelection(location, location + change);
		SetFocus();
		break;
		
	  case OUTPUTUNDO:
		/*
		   Undo output from most recent command.  LastLength is length,
		   including final newline, just before DoStats() and LastCmdStrPos
		   is the value of CmdStrPos at that time.  We cut all the output,
		   plus the final newline.
		   */
		SetSelection(LastLength, GetLastPosition());
		ToUndoBuffer(); /* save stuff on local scrap */
		Remove(LastLength, GetLastPosition());
		LastChar = WXK_BACK;
		UndoStatus = OUTPUTREDO;	/* can redo */
		Frame->command_ctrl->SetContent(Frame->LastCommand);
		Frame->command_ctrl->SetFocus();
		break;
		
	  case OUTPUTREDO:
		/* redo output just undone */
		length = GetLastPosition();
		location = CmdStrPos;
		scrapLength = FromUndoBuffer(); /*get previous output from local scrap*/
		SetSelection(GetLastPosition(), GetLastPosition());
		CmdStrPos = GetLastPosition();
		clearUndoInfo();
		LastCmdStrPos = location;
		LastLength = length;
		UndoStatus = OUTPUTUNDO;
		tmpstring = "";
		Frame->command_ctrl->SetContent(tmpstring);
		Frame->command_ctrl->SetFocus();
		break;
		
	  case TYPINGUNDO:
		if (LastTypingPos > UndoPlace)
		{
			location = UndoPlace;
			length = LastTypingPos - location;
			UndoStatus = TYPINGREDO;
			OldCmdStrPos = CmdStrPos;
			SetSelection(UndoPlace, LastTypingPos);
			ToUndoBuffer();
			if (location < CmdStrPos)
			{
				CmdStrPos -= length;
			}
			//Cut();
			Replace(UndoPlace, LastTypingPos, "");
			SetSelection(location, location);
		}
		SetFocus();
		break;
		
	  case TYPINGREDO:
		selStart = UndoPlace;
		SetSelection(selStart,selStart);
		length = FromUndoBuffer();
		UndoPlace = selStart;
		LastTypingPos = UndoPlace + length;
		UndoStatus = TYPINGUNDO;
		SetFocus();
	} /*switch (UndoStatus)*/
	ControlActionStatus = 0;
} /*CarapaceOutputCtrl::Undo()*/

void CarapaceOutputCtrl::CopyToEnd(void)
{
	long       selStart, selEnd;

	GetSelection(&selStart,&selEnd);
	if (selStart < selEnd)
	{/* copy selection to end */
		GetSelectionContents(selStart,selEnd);
		SetSelection(GetLastPosition(),GetLastPosition());
		wxYield();
		if (Selection.Length() != 0)
		{
			Frame->command_ctrl->AppendText(Selection);
		}
	}
} /* CarapaceOutputCtrl::CopyToEnd()*/

void CarapaceOutputCtrl::Execute(void)
{
	CopyToEnd();
	Frame->command_ctrl->AppendText("\n");
	Frame->command_ctrl->OnChange();
	Frame->command_ctrl->SetFocus();
	
} /* CarapaceOutputCtrl::Execute()*/

void CarapaceOutputCtrl::Execute(wxCommandEvent& event)
{
	CopyToEnd();
	Frame->command_ctrl->AppendText("\n");
	Frame->command_ctrl->OnChange();
	Frame->command_ctrl->SetFocus();
}

bool CarapaceOutputCtrl::LoadFile(const wxString& filename)
{
	wxFFile file(filename);
	if ( file.IsOpened() )
	{
		wxString text;
		if ( file.ReadAll(&text) )
		{
			text.Replace("\r\n","\n",TRUE);
			text.Replace("\r","\n",TRUE);
			SetValue(text);

			DiscardEdits();

			m_filename = filename;

			return true;
		}
	}

	wxLogError(_("File couldn't be loaded."));

	return false;
}

#ifdef __WXMAC__
extern void wxMacConvertNewlines10To13( wxString * ) ;

bool CarapaceOutputCtrl::SaveFile(const wxString & filename)
{
	wxString filenameToUse = filename.empty() ? m_filename : filename;
	if ( filenameToUse.empty() )
	{
	// what kind of message to give? is it an error or a program bug?
	wxLogDebug(wxT("Can't save textctrl to file without filename."));

	return false;
	}

	wxFFile file(filenameToUse, _T("w"));
	if ( file.IsOpened() )
	{
		wxString value = GetValue();
		wxMacConvertNewlines10To13(&value);
		if(file.Write(value))
		{
			// it's not modified any longer
			DiscardEdits();

			// if it worked, save for future calls
			m_filename = filenameToUse;

			return true;
		}
	}

	wxLogError(_("The text couldn't be saved."));

	return false;
}
#endif //WXMAC




BEGIN_EVENT_TABLE(CarapaceCommandCtrl, wxTextCtrl)
	EVT_CHAR(CarapaceCommandCtrl::OnChar)
	//EVT_IDLE(CarapaceCommandCtrl::OnIdle)
        EVT_MENU_RANGE(1,2000000000, CarapaceCommandCtrl::ProcessUserCommand)
	//EVT_MENU(CPC_GUI_SETHISTORYINDEX,CarapaceCommandCtrl::SetHistoryIndexMess)
	EVT_TEXT(CPC_DIALOGS_COMMANDWINDOW,CarapaceCommandCtrl::ComOnChange)
END_EVENT_TABLE()


CarapaceCommandCtrl::CarapaceCommandCtrl(wxWindow *frame, wxWindowID x, 
	const wxString &title, const wxPoint &pos, const wxSize &size,  long style):
	wxTextCtrl(frame, x, title, pos, size, style)
{

	Frame = (CarapaceSplitTextFrame *) frame;

	HistoryPosition = 0;
	PendingCommand = "";
	ShowMatches = FALSE;
	InMatch = FALSE;
	InOnChange = FALSE;
	LastHash = 0;
	typeAhead = "";
	LastValue = "";

} /*CarapaceCommandCtrl::CarapaceCommandCtrl()*/

CarapaceCommandCtrl::~CarapaceCommandCtrl(void)
{
}

#define DO_EVT_MENU(id,callback,ev)   \
        if(which == wxXmlResource::GetXRCID(#id))  \
        { \
                callback(ev); \
                return; \
        }

void CarapaceCommandCtrl::ProcessUserCommand(wxCommandEvent& event)
{
        int which = event.GetId();


        DO_EVT_MENU(CPC_GUI_SETHISTORYINDEX, CarapaceCommandCtrl::SetHistoryIndexMess,event)
}


void CarapaceCommandCtrl::GetAll(void)
{
	Selection = GetValue();
}

void CarapaceCommandCtrl::SetContent(wxString &newcontent)
{
	LastHash = 0;
	SetValue(newcontent);
	SetInsertionPointEnd();
#ifdef __WXMAC__
// wxMac seems to need this
	ShowPosition(0);
	SetStyle(0,GetLastPosition(), wxTextAttr(*(Frame->outputFGColor),
		*(Frame->outputBGColor),*(Frame->outputFont))); 
#endif /*WXMAC*/
}


void CarapaceCommandCtrl::GetSelection(long *from, long *to)
{
	long tmp;
	wxTextCtrl::GetSelection(from,to);
	if(*from > *to)
	{
		tmp = *to;
		*to = *from;
		*from = tmp;
	}
}

void CarapaceCommandCtrl::GetSelectionContents(void)
{
	GetSelectionContents(-1,-1);
}

void CarapaceCommandCtrl::GetSelectionContents(long from = -1, long to = -1)
{
	
	long i;

	if (from < 0 || to < 0)
	{
		// defaults to get the selection on screen
		GetSelection ( &from,  &to);
	}
	else
	{// can specify what to get
		i = GetLastPosition();
		from = (from < 0 ? 0 : from);
		from = (from > i ? i : from);
		to = (to < 0 ? 0 : to);
		to = (to > i ? i : to);
	}

	Selection.Clear();

	if (from == to)
	{
		return;
	}

	// God help us.  We read it in line by line
	long            fromLine, toLine, fromLineChar, toLineChar;
	long            fillingLineLength;

	PositionToXY(from,&fromLineChar, &fromLine);
	PositionToXY(to,&toLineChar, &toLine);

	if( fromLine == toLine)
	{
		Selection << GetLineText(fromLine).Mid(fromLineChar,toLineChar-fromLineChar);
	}
	else
	{
		fillingLineLength = GetLineLength(fromLine);
		Selection << GetLineText(fromLine).Right(fillingLineLength-fromLineChar);
		Selection << '\n';
		fromLine++;
		while(fromLine < toLine)
		{
			Selection << GetLineText(fromLine++) << '\n';
		}
		Selection << GetLineText(fromLine).Left(toLineChar);
	}
} /*CarapaceCommandCtrl::GetSelectionContents()*/


void CarapaceCommandCtrl::IncHistPtr(void)
{
        if(HistoryPosition > 0 && thisApp->History->HasLine(HistoryPosition+1))
        {
                HistoryPosition++;
        }
} 
	
void CarapaceCommandCtrl::SetHistoryIndexMess(wxCommandEvent & event)
{
	SetHistoryIndex(event.GetExtraLong());
}

void CarapaceCommandCtrl::SetHistoryIndex(int where)
{

	wxString acommand;
	// check for beginning
	if(where == 0)
	{
		acommand = PendingCommand;
	}
	else if(!thisApp->History->HasLine(where))
	{
		return;
	}
	else if(HistoryPosition == 0)
	{
		GetAll();
		PendingCommand = Selection;
		acommand = *thisApp->History->GetCommand(where);
	}
	else
	{
		acommand = *thisApp->History->GetCommand(where);
	}

	HistoryPosition = where;

	SetContent(acommand);
	ShowPosition(0);
}

void CarapaceCommandCtrl::UpHistory(void)
{

	wxString acommand;

	// check for beginning
	if(!thisApp->History->HasLine(HistoryPosition+1))
	{
		return;
	}
	if(HistoryPosition == 0)
	{
		GetAll();
		PendingCommand = Selection;
	}

	HistoryPosition++;
	wxString tmpstring; tmpstring << HistoryPosition;
	//Frame->history_spinner->SetValue(HistoryPosition);
	Frame->history_counter->SetLabel(tmpstring);
	acommand = *thisApp->History->GetCommand(HistoryPosition);


	SetContent(acommand);
	ShowPosition(0);
	SetFocus();
}

void CarapaceCommandCtrl::DownHistory(void)
{

	wxString acommand;

	// check for beginning
	if(HistoryPosition == 0)
	{
		return;
	}
	HistoryPosition--;
	if(HistoryPosition == 0)
	{
		acommand = PendingCommand;
	}
	else
	{
		acommand = *thisApp->History->GetCommand(HistoryPosition);
	}
	
	wxString tmpstring; tmpstring << HistoryPosition;
	//Frame->history_spinner->SetValue(HistoryPosition);
	Frame->history_counter->SetLabel(tmpstring);
	SetContent(acommand);
	ShowPosition(0);
	SetFocus();
}

void CarapaceCommandCtrl::AddMatch(int position, int paren)
{
	CarapaceMatches *match;

	switch(paren)
	{
		case '(':
		case '[':
		case '{':
		case '"':
		case '\'':
			match = new CarapaceMatches;
			match->position = position;
			match->paren = paren;
			Matches.Append(match);
			return;

		case '}':
		case ']':
			if(Matches.GetCount() > 0 &&
				Matches.GetLast()->GetData()->paren == paren-2)
			{ // a match
				position = Matches.GetLast()->GetData()->position;
				Matches.DeleteContents(TRUE);
				Matches.DeleteNode(Matches.GetLast());
				DiscardEdits();
				return;
			}
			// here is a mismatch
			MisMatch = TRUE;
			match = new CarapaceMatches;
			match->position = position;
			match->paren = paren;
			Matches.Append(match);
			return;
		
		case ')':
			if(Matches.GetCount() > 0 &&
				Matches.GetLast()->GetData()->paren == paren-1)
			{ // a match
				position = Matches.GetLast()->GetData()->position;
				Matches.DeleteContents(TRUE);
				Matches.DeleteNode(Matches.GetLast());
				DiscardEdits();
				return;
			}
			// here is a mismatch
			MisMatch = TRUE;
			match = new CarapaceMatches;
			match->position = position;
			match->paren = paren;
			Matches.Append(match);
			return;
		
		case -34: // -'"'
		case -44: // -'''
			// a negative double quote closes an opening double quote
			// similarly for single quotes
			if(Matches.GetCount() > 0 &&
				Matches.GetLast()->GetData()->paren == -paren)
			{ // a match
				position = Matches.GetLast()->GetData()->position;
				Matches.DeleteContents(TRUE);
				Matches.DeleteNode(Matches.GetLast());
				DiscardEdits();
				return;
			}
			// here is a mismatch
			// this should never happen with a decent lexer
			MisMatch = TRUE;
			match = new CarapaceMatches;
			match->position = position;
			match->paren = -paren;
			Matches.Append(match);
			return;
		default:
			;
	}
}


void CarapaceCommandCtrl::OnIdle(wxIdleEvent& event)
{
	event.Skip();
}

void CarapaceCommandCtrl::OnChar(wxKeyEvent& event)
{

	int count,ch,i;
	wxAcceleratorEntry entry;
	ch =  event.m_keyCode;

// in win32, tabs work fine.  Shift tabs shift panes but they
// don't come through here.  Alt tabs switch frames, but they don't
// come through here.  Control tabs seem to be entered as tabs, but
// they don't show up here.

	if (event.GetKeyCode() == WXK_TAB && event.m_controlDown)
	{
		Frame->output_ctrl->SetFocus();
		return;
	}

	if(ch == WXK_TAB)
	{
	// send genuine tabs to command control
		event.Skip();
		return;
	}
// sometimes you have to hit things with a really big brick.  Some text widgets (e.g., gtk)
// grab hold of key presses before the accelerator table can deal with them.  To get around
// this, we keep our own list of accelerators and handle the troublesome cases.
	if(ch < 26 && Frame->Accelerators)
	{
#if defined(__WXMSW__)
// windows seems to do a ctrl-V before control ever gets here, so
// we must ignore ctrl-V
		if(ch == 22) // v
		{
			return;
		}
#endif // defined(__WXMSW__)
		count = Frame->Accelerators->GetCount();
		for(i=0;i<count;i++)
		{
			entry = Frame->Accelerators->Item(i);
			if( (ch == entry.GetKeyCode()-64 || ch == entry.GetKeyCode()-96) &&
				entry.GetFlags() == wxACCEL_CTRL )
			{
				wxCommandEvent command_event( wxEVT_COMMAND_MENU_SELECTED, entry.GetCommand() );
				Frame->GetEventHandler()->ProcessEvent( command_event );
				return;
			}
		}
	}

	ShowMatches = TRUE;
	if(event.GetKeyCode() == WXK_UP)
	{
		UpHistory();
		return;
	}
	if(event.GetKeyCode() == WXK_DOWN)
	{
		DownHistory();
		return;
	}

	if(event.GetKeyCode() == WXK_PRIOR)
	{
		Frame->output_ctrl->ScrollPages(-1);
		return;
	}

	if(event.GetKeyCode() == WXK_NEXT)
	{
		Frame->output_ctrl->ScrollPages(1);
		return;
	}

	if(event.GetKeyCode() == WXK_RETURN && 
		(event.ControlDown() || event.ShiftDown()))
	{
        	//AppendText("\n");
		SetInsertionPointEnd();
        	//WriteText("\n");
        	//OnChange();
		//return;
	}
#if defined(__WXGTK__)
	if(event.GetRawKeyCode() == 65421)
	{
		SetInsertionPointEnd();
	}
#endif
#if defined(__WXMAC__)
	if(event.GetKeyCode() == WXK_NUMPAD_ENTER)
	{
	// please note, we will only see WXK_NUMPAD_ENTER if the
	// wxMac/src/mac/carbon/app.cpp file is modified to let
	// use it
		SetInsertionPointEnd();
		WriteText("\n");
		OnChange();
		return;
	}
#endif

	if(InMatch)
	{
		typeAhead.Append(event.m_keyCode);
		return;
	}

	event.Skip();
}

void CarapaceCommandCtrl::ComOnChange(wxCommandEvent& event)
{
	OnChange();
}

void CarapaceCommandCtrl::Paste(void)
{
	long         selStart, selEnd, clipLength;

	GetSelection(&selStart, &selEnd);
	if (selStart == selEnd)
	{
		selStart = selEnd = GetInsertionPoint();
	}

	wxTextDataObject clipData;
	if (wxTheClipboard->Open())
	{
		if (wxTheClipboard->IsSupported( wxDF_TEXT ))
		{
			wxTheClipboard->GetData( clipData );
		}  
		wxTheClipboard->Close();
	}

	
	clipLength = clipData.GetTextLength() - 1;


	if (clipLength > 0)
	{
		Replace(selStart,selEnd,clipData.GetText());
	} /*if (clipLength > 0)*/
} /*CarapaceCommandCtrl::Paste()*/


void CarapaceCommandCtrl::OnChange(void)
{
	CarapaceCommand *cptr;
	wxString  opens = "";
	wxCarapaceMatchListNode *mnode = NULL;
	bool   appended1char;
	wxString tmp;
	int  insertedaspace,lastMatchPos,lastMatchCount;


	if(InOnChange)
	{
		return;
	}
	InOnChange = TRUE;
	Refresh(); // wxMAC needs this
	GetAll();
	if(Selection.Length() == 0 || !thisApp->InputReady ||
		LastHash == Hash(Selection))
	{
		InOnChange = FALSE;
		return;
	}

	
	LastValue = Selection;

	opens = Selection;
	opens.RemoveLast();
	appended1char = (LastHash == Hash(opens));
	mnode = Matches.GetLast();
	lastMatchPos = (mnode ?  (int) mnode->GetData()->position : -1);
	lastMatchCount = Matches.GetCount();
	if(!appended1char)
	{
		Matches.Clear();
	}

	LastHash = Hash(Selection);
	opens.Clear();
	MisMatch = FALSE;
	InMatch = TRUE;
#if 0
	if(Frame->SyntaxAlert)
	{
		Frame->SyntaxAlert->SetBitmap(Frame->NullBitmap);
	}
#endif
	insertedaspace = 0;
	if(!thisApp->InputReady(Selection,Frame,appended1char))
	{
		if(MisMatch && appended1char)
		{
			CarapaceBeep(3);
		}
		if(appended1char && 
			Matches.GetCount() == (unsigned int) lastMatchCount && 
			GetLastPosition() == (lastMatchPos + 1))
		{
			SetStyle(lastMatchPos,GetLastPosition(),
			  wxTextAttr(*wxBLACK,wxColour(254,254,254)));
#ifdef __WXGTK__
// GTK 2 seems to be a little lazy and appreciates the push
			SetStyle(lastMatchPos,GetLastPosition(),
			  wxTextAttr(*wxBLACK,wxColour(254,254,254)));
#endif
			//wxTextAttr(*(Frame->outputFGColor),*(Frame->outputBGColor)));
		}
		if(appended1char && Matches.GetCount() == (unsigned int) lastMatchCount )
		{
			;; // do nothing
		}
		else if (appended1char && 
			(Matches.GetCount()+1) == (unsigned int) lastMatchCount)
		{
			SetStyle(lastMatchPos-1,GetLastPosition(),
			  wxTextAttr(*wxBLACK,wxColour(254,254,254)));
#ifdef __WXGTK__
// GTK 2 seems to be a little lazy and appreciates the push
			SetStyle(lastMatchPos-1,GetLastPosition(),
			  wxTextAttr(*wxBLACK,wxColour(254,254,254)));
#endif
			Refresh();
		}
		else if (appended1char && 
			Matches.GetCount() == (unsigned int) (lastMatchCount+1) &&
			GetLastPosition() > 1)
		{
			mnode = Matches.GetLast();
			int position = mnode->GetData()->position;
#ifdef USEBACKGROUNDPARENMATCH
			SetStyle(position-1,position,
				wxTextAttr(*wxBLACK,wxColour(255,200,0)));
#ifdef __WXGTK__
// GTK 2 seems to be a little lazy and appreciates the push
			SetStyle(position-1,position,
				wxTextAttr(*wxBLACK,wxColour(255,200,0)));
#endif
#else
			SetStyle(position-1,position,
				wxTextAttr(*wxRED));
#endif /*USEBACKGROUNDPARENMATCH*/
			Refresh();
		}
		else
		{
			SetStyle(0,GetLastPosition(),
			  wxTextAttr(*wxBLACK, wxColour(254,254,254)));
			  //wxTextAttr(*(Frame->outputFGColor), *(Frame->outputBGColor)));
#ifdef __WXGTK__
			// just setting the attrib doesn't seem to be enough for GTK 2 
			int tmp = GetInsertionPoint();
			GetAll();
			Clear();
			SetValue(Selection);
			SetInsertionPoint(tmp);
#endif
			mnode = Matches.GetFirst();
			while(mnode)
			{
				int position = mnode->GetData()->position;
#ifdef USEBACKGROUNDPARENMATCH
				SetStyle(position-1+insertedaspace,position+insertedaspace,
					wxTextAttr(*wxBLACK,wxColour(254,200,0)));
#ifdef __WXGTK__
// GTK 2 seems to be a little lazy and appreciates the push
				SetStyle(position-1+insertedaspace,position+insertedaspace,
					wxTextAttr(*wxBLACK,wxColour(254,200,0)));
#endif
#else
				SetStyle(position-1+insertedaspace,position+insertedaspace,
					wxTextAttr(*wxRED));
#endif /*USEBACKGROUNDPARENMATCH*/
				mnode = mnode->GetNext();
			}
			Refresh();
		}
		tmp = typeAhead;
		typeAhead.Empty();
		InMatch = FALSE;
		if(tmp.Length() > 0)
		{
			WriteText(tmp);
		}
		InOnChange = FALSE;
		return;
	}

	if(Selection.Last() != '\n')
	{
		InOnChange = FALSE;
		InMatch = FALSE;
		return;
	}

	InMatch = FALSE;
	wxCommandEvent procevent(wxEVT_COMMAND_MENU_SELECTED,CPC_GUI_PROCESSCOMMAND);
	cptr = new CarapaceCommand;
	cptr->command = Selection;
	cptr->frame = Frame->TextFrameIndex;
	cptr->hidecommand = FALSE;
	cptr->hideoutput = FALSE;
	procevent.SetClientData((void *) cptr);

	Selection = "";
	SetContent(Selection);
	LastHash = 0;
	LastValue = "";
	HistoryPosition = 0;
	//Frame->history_spinner->SetValue(0);
	Frame->history_counter->SetLabel("0");
	ShowMatches = FALSE;
	Matches.Clear();
	SetStyle(0,GetLastPosition(),wxTextAttr(*(Frame->outputFGColor),
		*(Frame->outputBGColor)));

	wxPostEvent(thisApp,procevent);
	InOnChange = FALSE;
} /*CarapaceCommandCtrl::OnChange*/

unsigned long CarapaceCommandCtrl::Hash(wxString & data)
{
	unsigned long val = 0,i,len = data.Length();
	const char *str = data.c_str();

	for(i=0;i<len;i++)
	{
		val = val * 69069 + str[i];
	}
	return(val);
}

#if defined(__WXMAC__)
long CarapaceCommandCtrl::XYToPosition(long x, long y)
{
	wxString content = GetValue() ;
	long count = 0, row = 0, maxcount;
	maxcount = content.Length();
	
	while(count < maxcount && row < y)
	{
		if(content.GetChar(count) == '\n')
		{
			row++;
		}
		count++;
	}
	if((count + x) >= maxcount || row < y)
	{
		// got to end without finding row y
		// what should we return?
		return(-1);
	}
	return(count+x);
}
		

bool CarapaceCommandCtrl::PositionToXY(long pos, long *x, long *y)
{
	wxString content = GetValue() ;
	long count = 0;

	if(pos < 0 || pos >= content.Length())
	{  // too long
		return FALSE;
	}

	*x = 0;
	*y = 0;
	
	while(count < pos)
	{
		*x = *x + 1;
		if(content.GetChar(count) == '\n')
		{
			*y =*y + 1;
			*x = 0;
		}
		count++;
	}
	return TRUE;
}

wxString CarapaceCommandCtrl::GetLineText(long lineNo) const
{
  // TODO change this if possible to reflect real lines
  wxString content = GetValue() ;

    // Find line first
    int count = 0; 
    for (size_t i = 0; i < content.Length() ; i++)
    {
        if (count == lineNo)
        {
            // Add chars in line then
            wxString tmp("");

            for (size_t j = i; j < content.Length(); j++)
            {
                if (content[j] == '\n')
                    return tmp;

                tmp += content[j];
            }

            return tmp;
        }
        if (content[i] == '\n') count++;
    }
    return "" ;
}

int CarapaceCommandCtrl::GetNumberOfLines() const
{
  // TODO change this if possible to reflect real lines
  wxString content = GetValue() ;

    int count = 1;
    for (size_t i = 0; i < content.Length() ; i++)
    { 
        if (content[i] == '\n') count++;
    }

    return count;
}



	
long CarapaceOutputCtrl::XYToPosition(long x, long y)
{
	wxString content = GetValue() ;
	long count = 0, row = 0, maxcount;
	maxcount = content.Length();
	
	while(count < maxcount && row < y)
	{
		if(content.GetChar(count) == '\n')
		{
			row++;
		}
		count++;
	}
	if((count + x) >= maxcount || row < y)
	{
		// got to end without finding row y
		// what should we return?
		return(-1);
	}
	return(count+x);
}
		

bool CarapaceOutputCtrl::PositionToXY(long pos, long *x, long *y)
{
	wxString content = GetValue() ;
	long count = 0;

	if(pos < 0 || pos > content.Length())
	{  // too long
		return FALSE;
	}

	*x = 0;
	*y = 0;
	
	while(count < pos)
	{
		*x = *x + 1;
		if(content.GetChar(count) == '\n')
		{
			*y =*y + 1;
			*x = 0;
		}
		count++;
	}
	return TRUE;
}

wxString CarapaceOutputCtrl::GetLineText(long lineNo) const
{
  // TODO change this if possible to reflect real lines
  wxString content = GetValue() ;

    // Find line first
    int count = 0; 
    for (size_t i = 0; i < content.Length() ; i++)
    {
        if (count == lineNo)
        {
            // Add chars in line then
            wxString tmp("");

            for (size_t j = i; j < content.Length(); j++)
            {
                if (content[j] == '\n')
                    return tmp;

                tmp += content[j];
            }

            return tmp;
        }
        if (content[i] == '\n') count++;
    }
    return "" ;
}

int CarapaceOutputCtrl::GetNumberOfLines() const
{
  // TODO change this if possible to reflect real lines
  wxString content = GetValue() ;

    int count = 1;
    for (size_t i = 0; i < content.Length() ; i++)
    { 
        if (content[i] == '\n') count++;
    }

    return count;
}

	
#endif
