/*
*(C)* The file is part of the source distribution of Carapace
*(C)* version 0.5 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* This file is distributed under the GNU General Public License
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

#include "cpcplatform.h"



// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/fontdlg.h"
#include "wx/colour.h"
#include "wx/caret.h"
#include "wx/print.h"
#include "wx/printdlg.h"


#include <math.h>

#include "carapace.h"

#undef UNDEFINED


CarapaceTPrintout::CarapaceTPrintout(char *title, CarapaceOutputCtrl *window) : 
	wxPrintout(title)
{
	OPrintingCtrl = window;
	PrintingFont = (OPrintingCtrl->Frame->outputFont);

	TotalLines = OPrintingCtrl->GetNumberOfLines();
} /*CarapaceTPrintout::CarapaceTPrintout()*/


bool CarapaceTPrintout::OnBeginDocument(int startPage, int endPage) 
{

	if (!wxPrintout::OnBeginDocument(startPage,endPage))
	{
		return(FALSE);
	}
	wxDC *printerDC = GetDC();

	int             pageMaxX,pageMaxY;		// pixels per page
	GetPageSizePixels(&pageMaxX,&pageMaxY);	// page dimensions in pixels,
	// which appear to be decipoints (1/720 inch) at least in Linux/PostScript

	PrintingFont = &(OPrintingCtrl->GetFont());
	printerDC->SetFont(*PrintingFont);
	LineHeight = (long) (printerDC->GetCharHeight() + CPC_LEADING);

// these lines are needed for PostScript under Linux, because we keep switching between
// points for some things and decipoints for others
#if !defined(WIN32)
	PrintingFont->SetPointSize(10*PrintingFont->GetPointSize());
#endif
	printerDC->SetFont(*PrintingFont);
	PageHeight =  pageMaxY/10;

	// we put a 1 in margin on top, .75 in on bottom
	// 4/8 in on left, the right will have to fend for itself
	TopMargin = 72;			// 1" 
	LeftMargin = (72*4)/8;		// 4/8" assuming points

	LinesPerPage = (long) ceil((PageHeight - 1.75*TopMargin)/LineHeight);
	TotalPages = TotalLines/LinesPerPage;
	if(TotalLines > TotalPages*LinesPerPage)
	{
		TotalPages++;
	}
	return(TRUE);
} /*CarapaceTPrintout::OnBeginDocument()*/


bool CarapaceTPrintout::OnPrintPage(int page)
{
	long       lineno, linebase, linelength;
	wxString   lineText;
	
  	wxDC *dc = GetDC();
  	if (dc)
  	{
		linebase = (page-1)*LinesPerPage;
		float       headerLine = TopMargin/2;
		float       line0 = TopMargin ;

		dc->SetUserScale(10.,10.);
#if !defined(WIN32)
		dc->SetDeviceOrigin(0,-1080);  // needed in Linux for PostScript
#endif
		for (lineno = 0; lineno < LinesPerPage;lineno++)
		{
			linelength = OPrintingCtrl->GetLineLength(linebase+lineno);
			lineText = OPrintingCtrl->GetLineText(linebase+lineno);
#if !defined(WIN32)
			dc->SetUserScale(10.,10.);
#endif
			dc->DrawText(lineText, LeftMargin, line0 + lineno*LineHeight);
		} /*for (lineno = 0; lineno < LinesPerPage;lineno++)*/

		lineText.Clear();
		lineText << wxFileNameFromPath(OPrintingCtrl->Frame->GetTitle()) 
			<< CPC_STRING_PAGEHEADER /*" --- Page "*/ << page;
#if !defined(WIN32)
		dc->SetUserScale(10.,10.);
#endif
		dc->DrawText(lineText, LeftMargin, line0 - TopMargin/2);/* char *, float, float*/
		return TRUE;
	} /*if (dc)*/
  	else
	{
		return FALSE;
	}
} /*CarapaceTPrintout::OnPrintPage()*/


bool CarapaceTPrintout::HasPage(int pageNum)
{
	return (pageNum >= 1 && pageNum <= TotalPages);
} /*CarapaceTPrintout::HasPage()*/

void CarapaceTPrintout::GetPageInfo(int *minPage, int *maxPage,
									int *pageFrom, int *pageTo)
{
	*minPage = 1;
	*maxPage = 32000;
	*pageFrom = 1;
	*pageTo = 32000;
} /*CarapaceTPrintout::GetPageInfo()*/
