/*
*(C)* The file is part of the source distribution of Carapace
*(C)* version 0.5 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* This file is distributed under the GNU General Public License
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

#include "cpcplatform.h"



// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/fontdlg.h"
#include "wx/colour.h"
#include "wx/caret.h"
#include "wx/print.h"
#include "wx/printdlg.h"


#include <math.h>

#include "carapace.h"

#undef UNDEFINED


CarapaceTPrintout::CarapaceTPrintout(char *title, CarapaceOutputCtrl *window) : 
	wxPrintout(title)
{
	OPrintingCtrl = window;
	PrintingFont = *(OPrintingCtrl->Frame->outputFont);

	TotalLines = OPrintingCtrl->GetNumberOfLines();
} /*CarapaceTPrintout::CarapaceTPrintout()*/


bool CarapaceTPrintout::OnBeginDocument(int startPage, int endPage) 
{
	if (!wxPrintout::OnBeginDocument(startPage,endPage))
	{
		return(FALSE);
	}
	wxDC *printerDC = GetDC();

	int   pageMaxX,pageMaxY;  // pixels per page
	int   ppiv,ppih,mmv,mmh;  // pixels per inch, page size mm

	GetPageSizePixels(&pageMaxX,&pageMaxY);	// page dimensions in pixels,
	// which appear to be points (1/72 inch) at least in 2.4.1

	GetPPIPrinter(&ppih,&ppiv);
	GetPageSizeMM(&mmh,&mmv);

	int windowfontsize, scaledfontsize;
	double fontrescalefactor;

	PrintingFont = *(OPrintingCtrl->Frame->outputFont);
	windowfontsize = PrintingFont.GetPointSize();
	scaledfontsize =  (int) ( ((double) windowfontsize)/72.*ppiv + .5) ;
	
	printerDC->SetFont(PrintingFont);
	LineHeight = (long) (printerDC->GetCharHeight() + CPC_LEADING);

	fontrescalefactor = ((double) scaledfontsize)/LineHeight;

	windowfontsize = (int) (windowfontsize*fontrescalefactor + .5);
	PrintingFont.SetPointSize(windowfontsize);
	printerDC->SetFont(PrintingFont);
	LineHeight = (long) (printerDC->GetCharHeight() + CPC_LEADING);

	printerDC->SetFont(PrintingFont);
	PageHeight =  pageMaxY;

	// we put a 1 in margin on top, .75 in on bottom
	// 4/8 in on left, the right will have to fend for itself
	TopMargin = ppiv;		// 1" 
	LeftMargin = ppih/2;		// 1/2" 

	LinesPerPage = (long) ceil((PageHeight - 1.75*TopMargin)/LineHeight);
	TotalPages = TotalLines/LinesPerPage;
	if(TotalLines > TotalPages*LinesPerPage)
	{
		TotalPages++;
	}
	return(TRUE);
} /*CarapaceTPrintout::OnBeginDocument()*/


bool CarapaceTPrintout::OnPrintPage(int page)
{
	long       lineno, linebase, linelength;
	wxString   lineText;
	
  	wxDC *dc = GetDC();
  	if (dc)
  	{
		linebase = (page-1)*LinesPerPage;
		float       line0 = TopMargin ;  

		dc->SetFont(PrintingFont); // wxMAC seems to need this
		for (lineno = 0; lineno < LinesPerPage;lineno++)
		{
			linelength = OPrintingCtrl->GetLineLength(linebase+lineno);
			lineText = OPrintingCtrl->GetLineText(linebase+lineno);
			dc->DrawText(lineText, LeftMargin, (int) (line0 + lineno*LineHeight) );
		} /*for (lineno = 0; lineno < LinesPerPage;lineno++)*/

		lineText.Clear();
		lineText << wxFileNameFromPath(OPrintingCtrl->Frame->GetTitle()) 
			<< CPC_STRING_PAGEHEADER /*" --- Page "*/ << page;
		dc->DrawText(lineText, LeftMargin, (int) (line0 - TopMargin/2) );
		return TRUE;
	} /*if (dc)*/
  	else
	{
		return FALSE;
	}
} /*CarapaceTPrintout::OnPrintPage()*/


bool CarapaceTPrintout::HasPage(int pageNum)
{
	return (pageNum >= 1 && pageNum <= TotalPages);
} /*CarapaceTPrintout::HasPage()*/

void CarapaceTPrintout::GetPageInfo(int *minPage, int *maxPage, int *pageFrom, int *pageTo)
{
	int   ppiv;  // pixels per inch, page size mm



	int windowfontsize;

	PrintingFont = *(OPrintingCtrl->Frame->outputFont);
	windowfontsize = PrintingFont.GetPointSize();
	
	// approximate, since we can't access device DC yet
	LineHeight = (long) (1.1 * windowfontsize);

	// assume letter paper
	PageHeight =  11*72;
	ppiv = 72;

	// we put a 1 in margin on top, .75 in on bottom
	// 4/8 in on left, the right will have to fend for itself
	TopMargin = ppiv;		// 1" 

	LinesPerPage = (long) ceil((PageHeight - 1.75*TopMargin)/LineHeight);
	TotalPages = TotalLines/LinesPerPage;
	if(TotalLines > TotalPages*LinesPerPage)
	{
		TotalPages++;
	}
	*minPage = 1;
	*maxPage = TotalPages;
	*pageFrom = 1;
	*pageTo = TotalPages;
} /*CarapaceTPrintout::GetPageInfo()*/
