/*
*(C)* The file is part of the source distribution of Carapace
*(C)* version 0.5 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* This file is distributed under the GNU General Public License
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

#include "cpcplatform.h"



// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/print.h"
#include "wx/gdicmn.h"
#include "wx/utils.h"


#include <math.h>

#include "carapace.h"

#undef UNDEFINED



// used in various places
void CarapaceAlert(const char *msgs)
{
	(void) wxMessageBox(msgs,CPC_STRING_ALERT,wxOK);
}

int CarapaceGetOsVersion(void)
{
	return (::wxGetOsVersion());
} /*mygetosversion*/



void CarapaceBeep(long n)
{
	while(n-- > 0)
	{
		wxBell();
	}
} /*CarapaceBeep()*/

#ifndef TICKMULTIPLIER
#ifdef __WIN32__
#define TICKMULTIPLIER 10
#else /*__WIN32__*/
#define TICKMULTIPLIER 100
#endif /*__WIN32__*/
#endif /*TICKMULTIPLIER*/

void CarapaceWaitForTicks(long wait_ticks)
{
        long         maxcount = wait_ticks*TICKMULTIPLIER;
/*
        maxcount is for safety to prevent essentially infinite loop
        which appeared on windows wx version
*/
        (void) wxGetElapsedTime(TRUE); // Reset timer
        while (wxGetElapsedTime(FALSE) < wait_ticks && maxcount-- > 0)
        {
                ;
        }
} /*waitForTicks()*/
 
void CarapaceUseA4Paper(void)
{
	thisApp->PrintData->SetPaperId(wxPAPER_A4);
}

void CarapaceSetLandscape(void) 
{
	thisApp->PrintData->SetOrientation(wxLANDSCAPE);
}


static wxString TextFromUserString;
static wxString  FileNameFromSelector;
static wxString  DirNameFromSelector;

char * CarapaceGetTextFromUser(char * message, char * title)
{
	TextFromUserString = wxGetTextFromUser(message,title,
		"",NULL,-1,-1,FALSE);
	return((char *) TextFromUserString.c_str());
}


char * CarapaceFindFile(const char *message, const char *defaultpath, const char *defaultname,
	const char * extension, const char * wildcard, int flags)
{
	wxString     argPath = "", argName = "";

	if (defaultname && defaultname[0] != '\0')
	{
		argName = wxFileNameFromPath(defaultname);

	} /*if (defaultname && defaultname[0] != '\0')*/
	if (defaultpath && defaultpath[0] != '\0')
	{
		argPath = wxPathOnly(defaultpath);

	} 
	else if(defaultname && defaultname[0] != '\0')
	{
		argPath = wxPathOnly(defaultname);
	}
	if(FileSelectionMutex.Lock() != wxMUTEX_NO_ERROR)
	{
		return(NULL);
	}
	const wxString& s  = wxFileSelector(message, argPath, argName, 
		extension, wildcard ,flags);
	FileSelectionMutex.Unlock();
	
	if(s.Length())
	{
		FileNameFromSelector = s;
		return((char *) FileNameFromSelector.c_str());
	}
	else
	{
		return(0);
	}
} /*CarapaceFindFile()*/	


char * CarapaceFindDir(const char *message, const char *defaultpath)
{
	wxString     argPath = "";

	if (defaultpath && defaultpath[0] != '\0')
	{
		argPath = wxPathOnly(defaultpath);

	} 
	const wxString& s  = wxDirSelector(message, argPath); 
	
	if(s.Length())
	{
		DirNameFromSelector = s;
		return((char *) DirNameFromSelector.c_str());
	}
	else
	{
		return(0);
	}
} /*CarapaceFindDir()*/	


void CarapaceGetTextColors(wxColor **fg, wxColor **bg)
{
	int *prf;

	prf = thisApp->Preferences->textFontColorVals;
	if(prf[0] != 0 && prf[1] != 0 && prf[2] != 0)
	{
		*fg = new wxColor(prf[0],prf[1],prf[2]);
	}
	else
	{
		*fg = wxTheColourDatabase->FindColour(
			thisApp->Preferences->textFontColorName);
	}
	prf = thisApp->Preferences->textBackgroundColorVals;
	if(prf[0] != 255 && prf[1] != 255 && prf[2] != 255)
	{
		*bg = new wxColor(prf[0],prf[1],prf[2]);
	}
	else
	{
		*bg = wxTheColourDatabase->FindColour(
			thisApp->Preferences->textBackgroundColorName);
	}

	if(!*fg) 
	{
		*fg = (wxColour *)wxBLACK;
	}
	if(!*bg) 
	{
		*bg = (wxColour *)wxWHITE;
	}
}



void CarapacePushShort(wxString * string, long y)
{
	short py[2];
	char *schars;
	unsigned int i;

	schars = (char *) py;
	py[0] = (short) y;
	py[1] = 0;
	
	//string->Append(schars);

	for(i=0;i<sizeof(short);i++)
	{
		string->Append(schars[i]);
	}
}

void CarapacePushString(wxString * string, wxString & toadd)
{
	int i,len=toadd.Length();
	CarapacePushShort(string,toadd.Length());
	for(i=0;i<len;i++)
	{
		string->Append(toadd.GetChar(i));
	}
}

short CarapaceGetShort(wxString * string, long *place)
{
	short py[2];
	unsigned long i;
	char *lchars;

	lchars = (char *) py;

	for(i=0;i<sizeof(short);i++)
	{
		lchars[i] = string->GetChar(*place+i);
	}
	*place += sizeof(short);
	return(py[0]);
}

wxString * CarapaceGetString(wxString * string, long *place)
{
	const char *cp;
	short i,len = CarapaceGetShort(string,place);

	cp = string->c_str();

	wxString *sp = new wxString;

	for(i=0;i<len;i++)
	{
		sp->Append(cp[*place]);
		*place = *place + 1;
	}
	return(sp);
}

int Carapace2WxStyle(long cpcstyle)
{
	switch(cpcstyle)
	{
        	case CPC_SG_LEFTTOP:
        	case CPC_SG_CENTERTOP:
        	case CPC_SG_RIGHTTOP:
        	case CPC_SG_LEFTBASELINE:
        	case CPC_SG_CENTERBASELINE:
        	case CPC_SG_RIGHTBASELINE:
        	case CPC_SG_LEFTBOTTOM:
        	case CPC_SG_CENTERBOTTOM:
        	case CPC_SG_RIGHTBOTTOM:
			return(cpcstyle);

        	case CPC_SG_SOLID:
			return(wxSOLID);

        	case CPC_SG_TRANSPARENT:
			return(wxTRANSPARENT);

        	case CPC_SG_DOT:
			return(wxDOT);

        	case CPC_SG_LONG_DASH:
			return(wxLONG_DASH);

        	case CPC_SG_SHORT_DASH:
			return(wxSHORT_DASH);

        	case CPC_SG_DOT_DASH:
			return(wxDOT_DASH);

        	case CPC_SG_DEFAULT:
			return(wxDEFAULT);

        	case CPC_SG_DECORATIVE:
			return(wxDECORATIVE);

        	case CPC_SG_ROMAN:
			return(wxROMAN);

        	case CPC_SG_SCRIPT:
			return(wxSCRIPT);

        	case CPC_SG_SWISS:
			return(wxSWISS);

        	case CPC_SG_MODERN:
			return(wxMODERN);

        	case CPC_SG_TELETYPE:
			return(wxTELETYPE);

        	//case CPC_SG_SYMBOL:
		//	return(wxSYMBOL);
 
        	case CPC_SG_NORMAL:
			return(wxNORMAL);

        	case CPC_SG_LIGHT:
			return(wxLIGHT);

        	case CPC_SG_BOLD:
			return(wxBOLD);

        	case CPC_SG_ITALIC:
			return(wxITALIC);

        	case CPC_SG_SLANT:
			return(wxSLANT);

        	case CPC_SG_BDIAGONAL_HATCH:
			return(wxBDIAGONAL_HATCH);

        	case CPC_SG_CROSSDIAG_HATCH:
			return(wxCROSSDIAG_HATCH);

        	case CPC_SG_FDIAGONAL_HATCH:
			return(wxFDIAGONAL_HATCH);

        	case CPC_SG_CROSS_HATCH:
			return(wxCROSS_HATCH);

        	case CPC_SG_HORIZONTAL_HATCH:
			return(wxHORIZONTAL_HATCH);

        	case CPC_SG_VERTICAL_HATCH:
			return(wxVERTICAL_HATCH);


		default:
			;
	}
	return(0);
}

wxColour * CarapaceColorFromName(long name)
{
	switch(name)
	{
		case CPC_AQUAMARINE:
			return(wxTheColourDatabase->FindColour("AQUAMARINE"));
		case CPC_BLACK:
			return(wxTheColourDatabase->FindColour("BLACK"));
		case CPC_BLUE:
			return(wxTheColourDatabase->FindColour("BLUE"));
		case CPC_BLUE_VIOLET:
			return(wxTheColourDatabase->FindColour("BLUE VIOLET"));
		case CPC_BROWN:
			return(wxTheColourDatabase->FindColour("BROWN"));
		case CPC_CADET_BLUE:
			return(wxTheColourDatabase->FindColour("CADET BLUE"));
		case CPC_CORAL:
			return(wxTheColourDatabase->FindColour("CORAL"));
		case CPC_CORNFLOWER_BLUE:
			return(wxTheColourDatabase->FindColour("CORNFLOWER BLUE"));
		case CPC_CYAN:
			return(wxTheColourDatabase->FindColour("CYAN"));
		case CPC_DARK_GREY:
			return(wxTheColourDatabase->FindColour("DARK GREY"));
		case CPC_DARK_GREEN:
			return(wxTheColourDatabase->FindColour("DARK GREEN"));
		case CPC_DARK_OLIVE_GREEN:
			return(wxTheColourDatabase->FindColour("DARK OLIVE GREEN"));
		case CPC_DARK_ORCHID:
			return(wxTheColourDatabase->FindColour("DARK ORCHID"));
		case CPC_DARK_SLATE_BLUE:
			return(wxTheColourDatabase->FindColour("DARK SLATE BLUE"));
		case CPC_DARK_SLATE_GREY:
			return(wxTheColourDatabase->FindColour("DARK SLATE GREY"));
		case CPC_DARK_TURQUOISE:
			return(wxTheColourDatabase->FindColour("DARK TURQUOISE"));
		case CPC_DIM_GREY:
			return(wxTheColourDatabase->FindColour("DIM GREY"));
		case CPC_FIREBRICK:
			return(wxTheColourDatabase->FindColour("FIREBRICK"));
		case CPC_FOREST_GREEN:
			return(wxTheColourDatabase->FindColour("FOREST GREEN"));
		case CPC_GOLD:
			return(wxTheColourDatabase->FindColour("GOLD"));
		case CPC_GOLDENROD:
			return(wxTheColourDatabase->FindColour("GOLDENROD"));
		case CPC_GREY:
			return(wxTheColourDatabase->FindColour("GREY"));
		case CPC_GREEN:
			return(wxTheColourDatabase->FindColour("GREEN"));
		case CPC_GREEN_YELLOW:
			return(wxTheColourDatabase->FindColour("GREEN YELLOW"));
		case CPC_INDIAN_RED:
			return(wxTheColourDatabase->FindColour("INDIAN RED"));
		case CPC_KHAKI:
			return(wxTheColourDatabase->FindColour("KHAKI"));
		case CPC_LIGHT_BLUE:
			return(wxTheColourDatabase->FindColour("LIGHT BLUE"));
		case CPC_LIGHT_GREY:
			return(wxTheColourDatabase->FindColour("LIGHT GREY"));
		case CPC_LIGHT_STEEL_BLUE:
			return(wxTheColourDatabase->FindColour("LIGHT STEEL BLUE"));
		case CPC_LIME_GREEN:
			return(wxTheColourDatabase->FindColour("LIME GREEN"));
		case CPC_MAGENTA:
			return(wxTheColourDatabase->FindColour("MAGENTA"));
		case CPC_MAROON:
			return(wxTheColourDatabase->FindColour("MAROON"));
		case CPC_MEDIUM_AQUAMARINE:
			return(wxTheColourDatabase->FindColour("MEDIUM AQUAMARINE"));
		case CPC_MEDIUM_BLUE:
			return(wxTheColourDatabase->FindColour("MEDIUM BLUE"));
		case CPC_MEDIUM_FOREST_GREEN:
			return(wxTheColourDatabase->FindColour("MEDIUM FOREST GREEN"));
		case CPC_MEDIUM_GOLDENROD:
			return(wxTheColourDatabase->FindColour("MEDIUM GOLDENROD"));
		case CPC_MEDIUM_ORCHID:
			return(wxTheColourDatabase->FindColour("MEDIUM ORCHID"));
		case CPC_MEDIUM_SEA_GREEN:
			return(wxTheColourDatabase->FindColour("MEDIUM SEA GREEN"));
		case CPC_MEDIUM_SLATE_BLUE:
			return(wxTheColourDatabase->FindColour("MEDIUM SLATE BLUE"));
		case CPC_MEDIUM_SPRING_GREEN:
			return(wxTheColourDatabase->FindColour("MEDIUM SPRING GREEN"));
		case CPC_MEDIUM_TURQUOISE:
			return(wxTheColourDatabase->FindColour("MEDIUM TURQUOISE"));
		case CPC_MEDIUM_VIOLET_RED:
			return(wxTheColourDatabase->FindColour("MEDIUM VIOLET RED"));
		case CPC_MIDNIGHT_BLUE:
			return(wxTheColourDatabase->FindColour("MIDNIGHT BLUE"));
		case CPC_NAVY:
			return(wxTheColourDatabase->FindColour("NAVY"));
		case CPC_ORANGE:
			return(wxTheColourDatabase->FindColour("ORANGE"));
		case CPC_ORANGE_RED:
			return(wxTheColourDatabase->FindColour("ORANGE RED"));
		case CPC_ORCHID:
			return(wxTheColourDatabase->FindColour("ORCHID"));
		case CPC_PALE_GREEN:
			return(wxTheColourDatabase->FindColour("PALE GREEN"));
		case CPC_PINK:
			return(wxTheColourDatabase->FindColour("PINK"));
		case CPC_PLUM:
			return(wxTheColourDatabase->FindColour("PLUM"));
		case CPC_PURPLE:
			return(wxTheColourDatabase->FindColour("PURPLE"));
		case CPC_RED:
			return(wxTheColourDatabase->FindColour("RED"));
		case CPC_SALMON:
			return(wxTheColourDatabase->FindColour("SALMON"));
		case CPC_SEA_GREEN:
			return(wxTheColourDatabase->FindColour("SEA GREEN"));
		case CPC_SIENNA:
			return(wxTheColourDatabase->FindColour("SIENNA"));
		case CPC_SKY_BLUE:
			return(wxTheColourDatabase->FindColour("SKY BLUE"));
		case CPC_SLATE_BLUE:
			return(wxTheColourDatabase->FindColour("SLATE BLUE"));
		case CPC_SPRING_GREEN:
			return(wxTheColourDatabase->FindColour("SPRING GREEN"));
		case CPC_STEEL_BLUE:
			return(wxTheColourDatabase->FindColour("STEEL BLUE"));
		case CPC_TAN:
			return(wxTheColourDatabase->FindColour("TAN"));
		case CPC_THISTLE:
			return(wxTheColourDatabase->FindColour("THISTLE"));
		case CPC_TURQUOISE:
			return(wxTheColourDatabase->FindColour("TURQUOISE"));
		case CPC_VIOLET:
			return(wxTheColourDatabase->FindColour("VIOLET"));
		case CPC_VIOLET_RED:
			return(wxTheColourDatabase->FindColour("VIOLET RED"));
		case CPC_WHEAT:
			return(wxTheColourDatabase->FindColour("WHEAT"));
		case CPC_WHITE:
			return(wxTheColourDatabase->FindColour("WHITE"));
		case CPC_YELLOW:
			return(wxTheColourDatabase->FindColour("YELLOW"));
		case CPC_YELLOW_GREEN:
			return(wxTheColourDatabase->FindColour("YELLOW GREEN"));
		default:
			return(NULL);
	}
	return(NULL);
}



wxMenu *FixCommandSubmenu(wxMenuBar *thisMenuBar)
{
	cpcMenuItem *tmpitem;
	int i;
	wxString base,full,itemLabel;

	if(!thisMenuBar)
	{
		return(NULL);
	}

	wxMenu *textWM = NULL;
	wxMenuItem *textMI = thisMenuBar->FindItem(wxXmlResource::GetXRCID("CPC_WINDOWS_TEXTWINDOW"));
	if(textMI && (textWM = textMI->GetSubMenu()))
	{
		while(textWM->GetMenuItemCount())
		{
			tmpitem = (cpcMenuItem *) textWM->Remove(textWM->GetMenuItems().GetFirst()->GetData());
			delete tmpitem;
		}
        	CarapaceSplitTextFrame *stfp;
        	wxCarapaceSplitTextFrameListNode *stnode = thisApp->STFrameList.GetFirst();
	
		i=0;
	
		base = "CPC_TEXT_WINDOWS_BASE";
		while(stnode)
		{
			if(i > 0 && i % 20 == 0)
			{
				textWM->Break();
			}
			stfp = stnode->GetData();
			full = base;
			full << i;
			itemLabel = stfp->GetTitle();
			itemLabel << "\tCtrl+" << i+1;
			tmpitem = new cpcMenuItem(textWM,wxXmlResource::GetXRCID(full.GetData()),
				//stfp->GetTitle(), "", wxITEM_NORMAL);
			 	itemLabel, wxString(""), wxITEM_NORMAL);
			tmpitem->label = itemLabel;
			textWM->Append((wxMenuItem *) tmpitem);
			stnode = stnode->GetNext();
			i++;
		}
	}
	return(textWM);
}

extern "C" {
int sortgraphs(const CarapaceGraphFrame **f1, const CarapaceGraphFrame **f2)
{
	if((*f1)->GraphFrameIndex < 
		(*f2)->GraphFrameIndex)
	{
		return(-1);
	}
	if((*f1)->GraphFrameIndex > 
		(*f2)->GraphFrameIndex)
	{
		return(1);
	}
	return(0);
}

int sortpanels(const CarapacePanelGraphFrame **f1, const CarapacePanelGraphFrame **f2)
{
	if((*f1)->PanelFrameIndex < 
		(*f2)->PanelFrameIndex)
	{
		return(-1);
	}
	if((*f1)->PanelFrameIndex > 
		(*f2)->PanelFrameIndex)
	{
		return(1);
	}
	return(0);
}
}

wxMenu *FixGraphSubmenu(wxMenuBar *thisMenuBar)
{
	cpcMenuItem *tmpitem;
	int i;
	wxString base,full;
	wxString itemLabel;

	if(!thisMenuBar)
	{
		return(NULL);
	}

	wxMenu *graphWM = NULL;
	wxMenuItem *graphMI = thisMenuBar->FindItem(wxXmlResource::GetXRCID("CPC_WINDOWS_GRAPH"));
	if(graphMI && (graphWM = graphMI->GetSubMenu()))
	{
		while(graphWM->GetMenuItemCount())
		{
			tmpitem = (cpcMenuItem *)graphWM->Remove(graphWM->GetMenuItems().GetFirst()->GetData());
			delete tmpitem;
		}
	
		i=0;
		base = "CPC_GRAPH_WINDOWS_BASE";
        	CarapaceGraphFrame  *gfp;
        	wxCarapaceGraphFrameListNode *gnode = thisApp->GFrameList.GetFirst();
		while(gnode)
		{
			if(i > 0 && i % 20 == 0)
			{
				graphWM->Break();
			}
			gfp = gnode->GetData();
			full = base;
			full << i;
			itemLabel = gfp->GetTitle();
			itemLabel << "\tF" << gfp->GraphFrameIndex;
			tmpitem = new cpcMenuItem(graphWM,wxXmlResource::GetXRCID(full.GetData()),
				//gfp->GetTitle(), wxString(""), wxITEM_NORMAL);
				itemLabel, wxString(""), wxITEM_NORMAL);
			tmpitem->label = itemLabel;
			graphWM->Append((wxMenuItem *) tmpitem);
			//graphWM->Append( wxXmlResource::GetXRCID(full.GetData()),
                       	//	gfp->GetTitle(), "", FALSE);
			gnode = gnode->GetNext();
			i++;
		}
        	CarapacePanelGraphFrame  *pgfp;
        	wxCarapacePanelGraphFrameListNode *pgnode = thisApp->PGFrameList.GetFirst();
		while(pgnode)
		{
			if(i > 0 && i % 20 == 0)
			{
				graphWM->Break();
			}
			pgfp = pgnode->GetData();
			full = base;
			full << i;
			tmpitem = new cpcMenuItem(graphWM,wxXmlResource::GetXRCID(full.GetData()),
				pgfp->GetTitle(), wxString(""), wxITEM_NORMAL);
			tmpitem->label = pgfp->GetTitle();
			graphWM->Append((wxMenuItem *) tmpitem);
			//graphWM->Append( wxXmlResource::GetXRCID(full.GetData()),
                       	//	gfp->GetTitle(), "", FALSE);
			pgnode = pgnode->GetNext();
			i++;
		}
	}
	return(graphWM);
}


wxMenu *FixDialogSubmenu(wxMenuBar *thisMenuBar)
{
	cpcMenuItem *tmpitem;
	int i;
	wxString base,full;

	if(!thisMenuBar)
	{
		return(NULL);
	}

	wxMenu *dialogWM = NULL;
	wxMenuItem *dialogMI = thisMenuBar->FindItem(wxXmlResource::GetXRCID("CPC_WINDOWS_DIALOGS"));
	if(dialogMI && (dialogWM = dialogMI->GetSubMenu()))
	{
		while(dialogWM->GetMenuItemCount())
		{
			tmpitem = (cpcMenuItem *) dialogWM->Remove(dialogWM->GetMenuItems().GetFirst()->GetData());
			delete tmpitem;
		}
        	CarapaceDialogFrame *dfp;
        	wxCarapaceDialogFrameListNode *dnode = thisApp->DFrameList.GetFirst();
	
		i=0;
	
		base = "CPC_DIALOG_WINDOWS_BASE";
		while(dnode)
		{
			if(i > 0 && i % 20 == 0)
			{
				dialogWM->Break();
			}
			dfp = dnode->GetData();
			full = base;
			full << i;
			tmpitem = new cpcMenuItem(dialogWM,wxXmlResource::GetXRCID(full.GetData()),
				dfp->GetTitle(), "", wxITEM_NORMAL);
			tmpitem->label = dfp->GetTitle();
			dialogWM->Append((wxMenuItem *) tmpitem);
			dnode = dnode->GetNext();
			i++;
		}
	}
	return(dialogWM);
}


void CarapaceDrawGlyph(wxDC &dc, long x, long y, long type, long size, long style)
{
	wxPoint pts[5];
	wxBrush brush;

	switch(type)
	{
		case CPC_SG_CIRCLE:
			if(! style & CPC_SG_FILLED)
			{
				brush = dc.GetBrush();
				dc.SetBrush(*wxTRANSPARENT_BRUSH);
				dc.DrawEllipse(x-size,y-size,2*size,2*size);
				dc.SetBrush(brush);
			}
			else
			{
				dc.DrawEllipse(x-size,y-size,2*size,2*size);
			}
			break;

		case CPC_SG_DIAMOND:
			pts[0].x = x + size;
			pts[1].x = x;
			pts[2].x = x - size;
			pts[3].x = x;
			pts[4].x = x + size;
		
			pts[0].y = y;
			pts[1].y = y + size;
			pts[2].y = y;
			pts[3].y = y - size;
			pts[4].y = y;
		
			if(style & CPC_SG_FILLED)
			{
				dc.DrawPolygon(5,pts);
			}
			else
			{
				dc.DrawLines(5,pts);
			}
			break;

		case CPC_SG_SQUARE:
			pts[0].x = x + size;
			pts[1].x = x - size;
			pts[2].x = x - size;
			pts[3].x = x + size;
			pts[4].x = x + size;
		
			pts[0].y = y + size;
			pts[1].y = y + size;
			pts[2].y = y - size;
			pts[3].y = y - size;
			pts[4].y = y + size;
		
			if(style & CPC_SG_FILLED)
			{
				dc.DrawPolygon(5,pts);
			}
			else
			{
				dc.DrawLines(5,pts);
			}
			break;

		case CPC_SG_TRIANGLE:
			if(style & CPC_SG_LEFT)
			{
				pts[0].x = x - size;
				pts[1].x = x + size;
				pts[2].x = x + size;
				pts[3].x = x - size;
			
				pts[0].y = y;
				pts[1].y = y - size/2;
				pts[2].y = y + size/2;
				pts[3].y = y;
			}
			else if(style & CPC_SG_DOWN)
			{
				pts[0].x = x;
				pts[1].x = x - size/2;
				pts[2].x = x + size/2;
				pts[3].x = x;
			
				pts[0].y = y - size;
				pts[1].y = y + size;
				pts[2].y = y + size;
				pts[3].y = y - size;
			}
			else if(style & CPC_SG_RIGHT)
			{
				pts[0].x = x + size;
				pts[1].x = x - size;
				pts[2].x = x - size;
				pts[3].x = x + size;
			
				pts[0].y = y;
				pts[1].y = y - size/2;
				pts[2].y = y + size/2;
				pts[3].y = y;
			}
			else
			{
				pts[0].x = x;
				pts[1].x = x - size/2;
				pts[2].x = x + size/2;
				pts[3].x = x;
			
				pts[0].y = y + size;
				pts[1].y = y - size;
				pts[2].y = y - size;
				pts[3].y = y + size;
			}
		
			if(style & CPC_SG_FILLED)
			{
				dc.DrawPolygon(4,pts);
			}
			else
			{
				dc.DrawLines(4,pts);
			}
			break;

		case CPC_SG_CROSS:
			dc.DrawLine(x-size,y,x+size,y);
			dc.DrawLine(x,y-size,x,y+size);
			break;

		case CPC_SG_EXE:
			dc.DrawLine(x-size,y-size,x+size,y+size);
			dc.DrawLine(x+size,y-size,x-size,y+size);
			break;

		case CPC_SG_STAR:
			dc.DrawLine(x-size,y,x+size,y);
			dc.DrawLine(x,y-size,x,y+size);
			dc.DrawLine((wxCoord) (x-size*.71),(wxCoord) (y-size*.71),(wxCoord) (x+size*.71),(wxCoord) (y+size*.71));
			dc.DrawLine((wxCoord) (x+size*.71),(wxCoord) (y-size*.71),(wxCoord) (x-size*.71),(wxCoord) (y+size*.71));
			break;

		case CPC_SG_SYMBOL:
		case CPC_SG_DINGBAT:
		;
	}
}

void CarapaceDrawGlyph2(wxDC &dc, double x, double y, long type, double size, long style)
{
	wxBrush brush;
	long off;

	off = (long) (size * .707 + 1.5);

	switch(type)
	{

		case CPC_SG_STAR:
			dc.DrawLine((wxCoord) (x-size+.5),(wxCoord) (y + .5), 
				(wxCoord) (x+size+.5), (wxCoord) (y + .5));
			dc.DrawLine((wxCoord) (x+.5),(wxCoord) (y-size+.5),
				(wxCoord) (x+.5),(wxCoord) (y+size+.5));
			dc.DrawLine((wxCoord) (x+.5),(wxCoord) (y+.5),
				(wxCoord) (x+off+.5),(wxCoord) (y+off+.5));
			dc.DrawLine((wxCoord) (x+.5),(wxCoord) (y+.5),
				(wxCoord) (x-off+.5),(wxCoord) (y+off+.5));
			dc.DrawLine((wxCoord) (x+.5),(wxCoord) (y+.5),
				(wxCoord) (x-off+.5),(wxCoord) (y-off+.5));
			dc.DrawLine((wxCoord) (x+.5),(wxCoord) (y+.5),
				(wxCoord) (x+off+.5),(wxCoord) (y-off+.5));
			break;

	}
}

void GetAccelArrayFromMenu(ArrayofAccelerators &aarray, wxMenu *menu)
{
	long count,i;
	wxMenuItem *item;
	wxMenu     *submenu;
	wxAcceleratorEntry *entry;
	
	count = menu->GetMenuItemCount();
	for(i = 0; i< count; i++)
	{
		item = menu->FindItemByPosition(i);
		if(!item)
		{
			continue;
		}
		if( (submenu = item->GetSubMenu()) )
		{
			GetAccelArrayFromMenu(aarray,submenu);
		}
		else
		{
			entry = item->GetAccel();
			if(entry)
			{
				entry->Set(entry->GetFlags(),entry->GetKeyCode(),item->GetId());
				aarray.Add(*entry);
			}
		}
	}
}

void GetAccelArrayFromMenuBar(ArrayofAccelerators &aarray, wxMenuBar *bar)
{
	long count,i;
	wxMenu *menu;
	
	count = bar->GetMenuCount();
	for(i=0;i<count;i++)
	{
		menu = bar->GetMenu(i);
		GetAccelArrayFromMenu(aarray,menu);
	}
}



void getNVPairs(wxString& source, wxString& name, wxString& value)
{
	unsigned int firstsp,firsteq,firstqu;

	name = "";
	value = "";
	source.Trim();
	source.Trim(FALSE);
	if(source.Length() == 0)
	{
		return;
	}
	firstsp = source.Find(" ");
	firsteq = source.Find("=");
	if(firstsp < 0 && firsteq < 0)
	{ // all "name"
		name = source;
		source = "";
		return;
	}
	if(firsteq < 0)
	{ // no =, so everything to first space is name and no value
		name = source.Left(firstsp);
		source = source.Mid(firstsp+1);
		source.Trim(FALSE);
		return;
	}
	if(firstsp < 0 || firsteq < firstsp)
	{ // no space, or = before space, so everything to = is name
		name = source.Left(firsteq);
		source = source.Mid(firsteq); // leave in =
		source.Trim(FALSE);
	}
	else
	{ // here we have space and =, but space comes before =
		name = source.Left(firstsp);
		source = source.Mid(firstsp); // leave in space
		source.Trim(FALSE);
	}
	// at this point, if there is a leading =, we have a value,
	// if the leading char is not =, then we do not have a value
	if(source.Left(1) != "=")
	{
		return;
	}
	// finally, look for value.  It should be enclosed in quotes,
	// but just in case, we'll go to first space if not quoted
	source = source.Mid(1);
	source.Trim(FALSE);
	if(source.Left(1) != "\"")
	{
		// not quoted
		firstsp = source.Find(" ");
		if(firstsp < 0)
		{
			value = source;
			source = "";
			return;
		}
		value = source.Left(firstsp);
		source = source.Mid(firstsp);
		source.Trim(FALSE);
		return;
	}
	// now we look for a quoted value.  There should be a trailing quote,
	// but we'll accept to the end of the source string if no trailing quote
	source = source.Mid(1);
	firstqu = source.Find('"');
	if(firstqu < 0)
	{
		value = source;
		source = "";
		return;
	}
	// CHECK ME Do we need to look for protected quotes here?
	value = source.Left(firstqu);
	source = source.Mid(firstqu+1);
	source.Trim(FALSE);
	return;
}

int GetXmlSkeleton(wxString input, wxString& output, 
	wxString targetclass, wxString targetname)
{
// find the Xml skeleton of objects for the object class/name
	long i,j,level;
	wxString name,value,leftstr,rightstr,mycopy,chunk,thisname,thisclass;

	mycopy = input; // just in case

	// first strip all comments
	while((i=mycopy.Find("<!--")) >= 0)
	{
		leftstr = mycopy.Left(i);
		rightstr = mycopy.Mid(i);
		j = rightstr.Find("-->");
		if(j > 0)
		{
			mycopy = leftstr;
			mycopy << rightstr.Mid(j+3);
		}
		else
		{
			return(CPC_ERROR_XMLERROR);
		}
	}
	// change all "white space" to space.  We can do this
	// because we are only interested in names and classes inside
	// of <object name="..." class="..."> constructs
	mycopy.Replace("\t"," ");
	mycopy.Replace("\n"," ");
	mycopy.Replace("\r"," ");

	output = "";

	// strip everything except what is in <...> constructs
	// only keep the stuff that is within an object tree with
	// the targetclass and targetname

	level = -32000;
	while(mycopy.Length() > 0)
	{
		i=mycopy.Find("<");  
		if(i < 0)
		{
			// it's possible there is an xml error here, but we've missed it
			return(CPC_NOERROR);
		}
		mycopy = mycopy.Mid(i+1);
		j = mycopy.Find(">");
		if(j < 0)
		{
			return(CPC_ERROR_XMLERROR);
		}
		leftstr = mycopy.Left(j);
		mycopy = mycopy.Mid(j+1);
		// leftstr has what is inside the angle brackets
		chunk = "<";
		chunk << leftstr << ">";

		// if it starts with object, but we'll strip leading space just in case
		leftstr.Trim(FALSE);
		if(leftstr.Find("/object") == 0)
		{
			level--;
			continue;
		}
		if(leftstr.Find("object") != 0)
		{
			continue;
		}
		leftstr = leftstr.Mid(6);
		// OK, now we have content of an object member, and we're looking for
		// class and possibly name elements
		level++;
		while(leftstr.Length() > 0)
		{
			getNVPairs(leftstr,name,value);
			if(name == "class")
			{
				thisclass = value;
			}
			if(name == "name")
			{
				thisname = value;
			}
		}
		if(thisclass == targetclass && thisname == targetname)
		{
			// OK now we are in the object of interest
			level = 1;
		}
		if(level >= 1)
		{
			output << chunk;
		}
	}
	return(CPC_NOERROR);
}



