#include "cpcplatform.h"

#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif
#include "wx/print.h"
#include "wx/printdlg.h"
#include "wx/generic/dcpsg.h"
#include "wx/clipbrd.h"


#include "carapace.h"


// internal classes

class CarapacePatch : public wxControl
{
public:
	CarapacePatch(wxWindow *parent);
	CarapacePatch(wxWindow *parent, wxSize size);
	~CarapacePatch();
	PatchViewer              *Viewer;
	long                      Index;
	wxString                  Label;
	void                      OnMouse(wxMouseEvent& event);

	DECLARE_EVENT_TABLE();
};

BEGIN_EVENT_TABLE(CarapacePatch,wxControl)
EVT_LEFT_DOWN(CarapacePatch::OnMouse)
END_EVENT_TABLE()


CarapacePatch::CarapacePatch(wxWindow* parent, wxSize size) : 
	wxControl(parent,-1,wxDefaultPosition, size)
{
	Viewer = (PatchViewer *) parent;
}

CarapacePatch::~CarapacePatch()
{
}

void CarapacePatch::OnMouse(wxMouseEvent& event)
{
	Viewer->ProcessClick(Label,Index);
}

class CarapaceColorPatch : public CarapacePatch
{
public:
	CarapaceColorPatch(wxWindow *parent, wxSize size = wxSize(16,16));
	CarapaceColorPatch(wxWindow *parent, wxString label, long index,
		wxSize size = wxSize(16,16));
	~CarapaceColorPatch();
	void         OnPaint(wxPaintEvent& event);
	void         CatchRightClick(wxMouseEvent& event) {Viewer->DoPopup(event);}
	void         CatchMenu(wxCommandEvent& event) {Viewer->ProcessMenuCommand(event);}
	DECLARE_EVENT_TABLE();
};

BEGIN_EVENT_TABLE(CarapaceColorPatch,CarapacePatch)
EVT_PAINT(CarapaceColorPatch::OnPaint)
EVT_RIGHT_DOWN(CarapaceColorPatch::CatchRightClick)
END_EVENT_TABLE()


CarapaceColorPatch::CarapaceColorPatch(wxWindow *parent, wxSize size) : 
		CarapacePatch(parent,size)
{
}

CarapaceColorPatch::~CarapaceColorPatch() 
{
}

CarapaceColorPatch::CarapaceColorPatch(wxWindow *parent, wxString label,
	long index, wxSize size) : CarapacePatch(parent,size)
{
	Label = label;
	Index = index;
}

void CarapaceColorPatch::OnPaint(wxPaintEvent& event)
{
	wxPaintDC dc( this );

	wxColour col(Label);
	wxBrush brush(col);
	dc.SetBrush(brush);
	wxPen pen(col);
	dc.SetPen(pen);
	wxCoord h,w;
	dc.GetSize(&w,&h);
	dc.DrawRectangle(0,0,w,h);
}

class CarapaceGlyphPatch : public CarapacePatch
{
public:
	CarapaceGlyphPatch(wxWindow *parent, wxSize size = wxSize(16,16));
	CarapaceGlyphPatch(wxWindow *parent, wxString label, long index,
		wxSize size = wxSize(16,16));
	~CarapaceGlyphPatch();
	void         OnPaint(wxPaintEvent& event);
	void         CatchRightClick(wxMouseEvent& event) {Viewer->DoPopup(event);}
	void         CatchMenu(wxCommandEvent& event) {Viewer->ProcessMenuCommand(event);}

	DECLARE_EVENT_TABLE();
};

BEGIN_EVENT_TABLE(CarapaceGlyphPatch,CarapacePatch)
EVT_PAINT(CarapaceGlyphPatch::OnPaint)
EVT_RIGHT_DOWN(CarapaceGlyphPatch::CatchRightClick)
END_EVENT_TABLE()


CarapaceGlyphPatch::CarapaceGlyphPatch(wxWindow *parent, wxSize size) : 
		CarapacePatch(parent,size)
{
}

CarapaceGlyphPatch::~CarapaceGlyphPatch() 
{
}

CarapaceGlyphPatch::CarapaceGlyphPatch(wxWindow *parent, wxString label,
	long index, wxSize size) : CarapacePatch(parent,size)
{
	Label = label;
	Index = index;
}

void CarapaceGlyphPatch::OnPaint(wxPaintEvent& event)
{
	wxCoord h,w;
	long x,y,size,style,type;
	double xx,yy,zz;


	wxPaintDC dc( this );

	dc.SetBrush(*wxWHITE_BRUSH);
	dc.SetPen(*wxWHITE_PEN);
	dc.GetSize(&w,&h);
	dc.DrawRectangle(0,0,w,h);
	wxPen pen;
	pen.SetColour("BLACK");
	pen.SetWidth((long)(w/10));
	dc.SetPen(pen);
	zz = w/2.5;
	size = (long) zz;
	xx = yy = x = y = w/2;
	style = 0;
	switch(Index)
	{	
	case 1: // DOT
		type = CPC_SG_CIRCLE;
		size = 1;
		break;
	case 2: // EXE
		type = CPC_SG_EXE;
		break;
	case 3: // CROSS
		type = CPC_SG_CROSS;
		break;
	case 4: // SPIDER
		type = CPC_SG_STAR;
		CarapaceDrawGlyph2(dc,xx,yy,type,zz,style);
		return;
	case 5: // CIRCLE
		type = CPC_SG_CIRCLE;
		break;
	case 6: // DIAMOND
		type = CPC_SG_DIAMOND;
		break;
	case 7: // SQUARE
		type = CPC_SG_SQUARE;
		break;
	case 8: // TRIANGLE
		type = CPC_SG_TRIANGLE;
		break;
	default:
		type = CPC_SG_CIRCLE;
	}
	CarapaceDrawGlyph(dc,x,y,type,size,style);
}

BEGIN_EVENT_TABLE(PatchViewer,wxPanel)
EVT_RIGHT_DOWN(PatchViewer::DoPopup)
EVT_MENU_RANGE(1,200000000,PatchViewer::CatchMenu)
END_EVENT_TABLE()

PatchViewer::PatchViewer(long type)
{
	Type = type;
	actionstring = "";
	action = stringIndex = longIndex = NULL;
	hidecommand = FALSE;
	hideoutput = FALSE;
}

bool  PatchViewer::Create(wxWindow *parent, wxWindowID id)
{
	CarapacePatch *item;
	wxPanel::Create(parent,id);
	SetClientSize(100,50);
	switch(Type)
	{
	case CPC_DYN_COLORSMALL:
		Sizer = new wxGridSizer(2,4,1,1);
		item = new CarapaceColorPatch(this,"RED",53);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		item = new CarapaceColorPatch(this,"BLUE",2);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		item = new CarapaceColorPatch(this,"YELLOW",68);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		item = new CarapaceColorPatch(this,"GREEN",22);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);

		item = new CarapaceColorPatch(this,"CORAL",6);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		item = new CarapaceColorPatch(this,"BLACK",1);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		item = new CarapaceColorPatch(this,"GREY",21);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		item = new CarapaceColorPatch(this,"WHITE",67);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		SetSizer(Sizer);
		Sizer->Fit(this);
		break;

	case CPC_DYN_COLORMEDIUM:
		Sizer = new wxGridSizer(3,5,1,1);
		item = new CarapaceColorPatch(this,"RED",53);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		item = new CarapaceColorPatch(this,"BLUE",2);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		item = new CarapaceColorPatch(this,"YELLOW",68);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		item = new CarapaceColorPatch(this,"GREEN",22);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		item = new CarapaceColorPatch(this,"CORAL",6);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);

		item = new CarapaceColorPatch(this,"PURPLE",52);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		item = new CarapaceColorPatch(this,"CYAN",8);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		item = new CarapaceColorPatch(this,"FOREST GREEN",18);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		item = new CarapaceColorPatch(this,"ORANGE RED",47);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		item = new CarapaceColorPatch(this,"THISTLE",62);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);

		item = new CarapaceColorPatch(this,"BLACK",1);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		item = new CarapaceColorPatch(this,"DARK GREY",9);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		item = new CarapaceColorPatch(this,"GREY",21);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		item = new CarapaceColorPatch(this,"LIGHT GREY",27);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		item = new CarapaceColorPatch(this,"WHITE",67);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		SetSizer(Sizer);
		Sizer->Fit(this);

		break;

	case CPC_DYN_COLORLARGE:

		break;

	case CPC_DYN_GLYPH:
		Sizer = new wxGridSizer(2,4,1,1);
		item = new CarapaceGlyphPatch(this,"DOT",1);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		item = new CarapaceGlyphPatch(this,"EXE",2);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		item = new CarapaceGlyphPatch(this,"CROSS",3);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		item = new CarapaceGlyphPatch(this,"SPIDER",4);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);

		item = new CarapaceGlyphPatch(this,"CIRCLE",5);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		item = new CarapaceGlyphPatch(this,"DIAMOND",6);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		item = new CarapaceGlyphPatch(this,"SQUARE",7);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		item = new CarapaceGlyphPatch(this,"TRIANGLE",8);
		Sizer->Add(item,0,wxSHAPED | wxALL,3)->SetRatio(1);
		SetSizer(Sizer);
		Sizer->Fit(this);
		break;


		break;

	default:
		;
	}
	return TRUE;
}
	

PatchViewer::~PatchViewer()
{     
	delete popupmenu;
}

void PatchViewer::ProcessClick(wxString& label, long& index)
{
	if(stringIndex)
	{
		(*stringIndex)[0].SetValue(label);
	}
	if(longIndex)
	{
		int tmp = index;
		(*longIndex)[0].SetValue(tmp);
	}
	ProcessAction();
}

bool PatchViewer::draw(DataObject& dobj)
{
	ProcessAction();
	return TRUE;
}

IMPLEMENT_DYNAMIC_CLASS(SmallColorPickerXmlHandler,wxXmlResourceHandler)

SmallColorPickerXmlHandler::SmallColorPickerXmlHandler() 
: wxXmlResourceHandler() 
{
    AddWindowStyles();
}

wxObject *SmallColorPickerXmlHandler::DoCreateResource()
{ 
	wxString colorname = GetText(wxT("colorname"));
	wxString indexname = GetText(wxT("indexname"));
	wxString viewername = GetName();
	wxString actionstring = GetText("action");
	wxString actionname = GetText("actionname");
	wxString error;
	vector<wxString> viewerdata;

	DataObject *dobj,*dobj2;

	dobj = GetDynFromName(colorname,"colorname",STRINGT,-1,-1,"SmallColorPicker",viewername,FALSE);
	if(dobj != NULL)
	{
		viewerdata.push_back(colorname);
	}
	dobj2 = GetDynFromName(indexname,"indexname",INTT,-1,-1,"SmallColorPicker",viewername,FALSE);
	if(dobj2 != NULL)
	{
		viewerdata.push_back(indexname);
	}

	if(dobj == NULL && dobj2 == NULL)
	{
		error.Printf(CPC_STRING_VIEWER0013,"colorname","indexname","SmallColorPicker",
			viewername.c_str());
		wxMessageBox(error);
		return(NULL);
	}

	PatchViewer *view = new PatchViewer(CPC_DYN_COLORSMALL);
	view->Create(m_parentAsWindow,wxXmlResource::GetXRCID(viewername));
	view->stringIndex = dobj;
	view->longIndex = dobj2;
	KludgeView = view;
	CreateChildren(view);


	if(actionstring != "")
	{
		view->actionstring = actionstring;
	}
	dobj = GetDynFromName(actionname,"actionname",STRINGT,-1,-1,
		"SmallColorViewer",viewername,FALSE);
	if(dobj != NULL)
	{
		view->action = dobj;
		viewerdata.push_back(actionname);
	}

	view->hidecommand = GetBool("hidecommand",0);
	view->hideoutput = GetBool("hideoutput",0);
	thisApp->DynamicDataP->addViewer(viewername,(Viewer*) view);
	thisApp->DynamicDataP->setViewerData(viewerdata,viewername);
    
	return view;
}

bool SmallColorPickerXmlHandler::CanHandle(wxXmlNode *node)
{
    return IsOfClass(node, wxT("SmallColorPicker"));
}


IMPLEMENT_DYNAMIC_CLASS(MediumColorPickerXmlHandler,wxXmlResourceHandler)

MediumColorPickerXmlHandler::MediumColorPickerXmlHandler() 
: wxXmlResourceHandler() 
{
    AddWindowStyles();
}


wxObject *MediumColorPickerXmlHandler::DoCreateResource()
{ 
	wxString colorname = GetText(wxT("colorname"));
	wxString indexname = GetText(wxT("indexname"));
	wxString viewername = GetName();
	wxString actionstring = GetText("action");
	wxString actionname = GetText("actionname");
	wxString error;
	vector<wxString> viewerdata;

	DataObject *dobj,*dobj2;

	dobj = GetDynFromName(colorname,"colorname",STRINGT,-1,-1,"MediumColorPicker",viewername,FALSE);
	if(dobj != NULL)
	{
		viewerdata.push_back(colorname);
	}
	dobj2 = GetDynFromName(indexname,"indexname",INTT,-1,-1,"MediumColorPicker",viewername,FALSE);
	if(dobj2 != NULL)
	{
		viewerdata.push_back(indexname);
	}

	if(dobj == NULL && dobj2 == NULL)
	{
		error.Printf(CPC_STRING_VIEWER0013,"colorname","indexname","MediumColorPicker",
			viewername.c_str());
		wxMessageBox(error);
		return(NULL);
	}

	PatchViewer *view = new PatchViewer(CPC_DYN_COLORMEDIUM);
	view->Create(m_parentAsWindow,wxXmlResource::GetXRCID(viewername));
	view->stringIndex = dobj;
	view->longIndex = dobj2;
	KludgeView = view;
	CreateChildren(view);


	if(actionstring != "")
	{
		view->actionstring = actionstring;
	}
	dobj = GetDynFromName(actionname,"actionname",STRINGT,-1,-1,
		"MediumColorViewer",viewername,FALSE);
	if(dobj != NULL)
	{
		view->action = dobj;
		viewerdata.push_back(actionname);
	}
	view->hidecommand = GetBool("hidecommand",0);
	view->hideoutput = GetBool("hideoutput",0);

	thisApp->DynamicDataP->addViewer(viewername,(Viewer*) view);
	thisApp->DynamicDataP->setViewerData(viewerdata,viewername);
    
	return view;
}

bool MediumColorPickerXmlHandler::CanHandle(wxXmlNode *node)
{
    return IsOfClass(node, wxT("MediumColorPicker"));
}





IMPLEMENT_DYNAMIC_CLASS(GlyphPickerXmlHandler,wxXmlResourceHandler)

GlyphPickerXmlHandler::GlyphPickerXmlHandler() 
: wxXmlResourceHandler() 
{
    AddWindowStyles();
}

Viewer *KludgeView;

wxObject *GlyphPickerXmlHandler::DoCreateResource()
{ 
	wxString glyphname = GetText(wxT("glyphname"));
	wxString indexname = GetText(wxT("indexname"));
	wxString viewername = GetName();
	wxString actionstring = GetText("action");
	wxString actionname = GetText("actionname");
	wxString error;
	vector<wxString> viewerdata;

	DataObject *dobj,*dobj2;

	dobj = GetDynFromName(glyphname,"glyphname",STRINGT,-1,-1,"GlyphPicker",viewername,FALSE);
	if(dobj != NULL)
	{
		viewerdata.push_back(glyphname);
	}
	dobj2 = GetDynFromName(indexname,"indexname",INTT,-1,-1,"GlyphPicker",viewername,FALSE);
	if(dobj2 != NULL)
	{
		viewerdata.push_back(indexname);
	}

	if(dobj == NULL && dobj2 == NULL)
	{
		error.Printf(CPC_STRING_VIEWER0013,"glyphname","indexname","GlyphPicker",
			viewername.c_str());
		wxMessageBox(error);
		return(NULL);
	}
	PatchViewer *view = new PatchViewer(CPC_DYN_GLYPH);
	view->Create(m_parentAsWindow,wxXmlResource::GetXRCID(viewername));
	view->stringIndex = dobj;
	view->longIndex = dobj2;
	KludgeView = view;
	CreateChildren(view);


	if(actionstring != "")
	{
		view->actionstring = actionstring;
	}
	dobj = GetDynFromName(actionname,"actionname",STRINGT,-1,-1,
		"MediumColorViewer",viewername,FALSE);
	if(dobj != NULL)
	{
		view->action = dobj;
		viewerdata.push_back(actionname);
	}
	view->hidecommand = GetBool("hidecommand",0);
	view->hideoutput = GetBool("hideoutput",0);
	thisApp->DynamicDataP->addViewer(viewername,(Viewer*) view);
	thisApp->DynamicDataP->setViewerData(viewerdata,viewername);
    
	return view;
}

bool GlyphPickerXmlHandler::CanHandle(wxXmlNode *node)
{
    return IsOfClass(node, wxT("GlyphPicker"));
}



