#include "cpcplatform.h"

#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif
#include "wx/print.h"
#include "wx/printdlg.h"
#include "wx/generic/dcpsg.h"
#include "wx/clipbrd.h"


#include "carapace.h"
//#include "math.h"

#define DBL_MIN             2.2250738585072014E-308
#define DBL_EPSILON         2.2204460492503131E-16
#define DBL_MAX             2.2250738585072014E+308

//define error message
#define CPC_STRING_VIEWER0100		"the column selected should be within the column range of %s or %s or equal to 0 in %s %s"
#define CPC_STRING_VIEWER0101		"the Upper value of  %s is expected  in %s %s,So default value will be used"
#define CPC_STRING_VIEWER0102		"the  %s or %s mismatch in %s %s, colorname will be used "
#define CPC_STRING_VIEWER0103		"the  %s is not defined in %s %s, default plot parameters will be used "
#define CPC_STRING_VIEWER0104		"the  %s is not defined in %s %s, the plot name is blank "
#define CPC_STRING_VIEWER0105		"the  %s and %s are not defined in %s %s, default color will be used "
#define CPC_STRING_VIEWER0106		"the  %s is not defined in %s %s, the default points size will be used "
#define CPC_STRING_VIEWER0107		"the  %s is not defined in %s %s, the default points type will be used "
#define CPC_STRING_VIEWER0108		"the  %s is not defined in %s %s, the axis-x label is blank "
#define CPC_STRING_VIEWER0109		"the  %s is not defined in %s %s, the axis-y label is blank "
#define CPC_STRING_VIEWER0110		"the  %s is not defined in %s %s, the default tick number will be used "
#define CPC_STRING_VIEWER0111		"the  %s is not defined in %s %s, the default tick length will be used "
#define CPC_STRING_VIEWER0112		"the  %s is not defined in %s %s, the default x and y tick location will be used "
#define CPC_STRING_VIEWER0112		"the  %s is not defined in %s %s, the default x and y tick location will be used "
#define CPC_STRING_VIEWER0113		"the  min value is more than max value of %s  in %s %s, the default min and max value will be calculated again "
#define CPC_STRING_VIEWER0114		"the  %s is not defined in %s %s, set it to be false "

// internal function
void PlotDrawGlyph (wxDC & dc, long x, long y, long type, long size,
		    long style);
double R_pretty0 (double *lo, double *up, int *ndiv, int min_n,
		  double shrink_sml, double high_u_fact[], int eps_correction,
		  int return_bounds);
void R_pretty (double *lo, double *up, int *ndiv, int *min_n,
	       double *shrink_sml, double *high_u_fact, int *eps_correction);
double fmax2 (double db1, double db2);
int imax2 (int int1, int int2);
int getlabeloffset (wxString label, int curfontsize);
int getlabelYoffset (int curfontsize);
int GetColorIndexfromName (wxString colorname);
wxString GetColorNamefromIndex (int colorindex);


BEGIN_EVENT_TABLE (PlotAreaRecViewer, wxWindow) EVT_PAINT (PlotAreaRecViewer::OnPaint) EVT_MOUSE_EVENTS (PlotAreaRecViewer::OnMouse) END_EVENT_TABLE ()PlotAreaRecViewer::PlotAreaRecViewer (wxWindow * parent, wxWindowID id, wxPoint pos, wxSize size):wxWindow (parent, id, pos, size, wxSIMPLE_BORDER,
	  "plotarea")
{

  actionstring = "";
  //action = stringIndex = longIndex = NULL;
  action = datax = datay = dataxaxislabel = datayaxislabel = datarangex =
    datarangey = datacolorindex = datacolorname = NULL;
  datalinescolor = datalinescolorindex = pointsdel = curvedel = NULL;
  hidecommand = FALSE;
  hideoutput = FALSE;
  xscalenum = 4;
  yscalenum = 4;
  CurPointSelState = Select_Raady;
  Clickblankareaflag = true;
  SetBackgroundColour (*wxWHITE);
}

bool PlotAreaRecViewer::Create (wxWindow * parent, wxWindowID id)
{

  return TRUE;
}


PlotAreaRecViewer::~PlotAreaRecViewer ()
{
  delete popupmenu;
}

void
PlotAreaRecViewer::OnMouse (wxMouseEvent & event)
{
  wxPaintDC dc (this);
  wxPoint curp;
  long curpx, curpy;
  curp = event.GetLogicalPosition (dc);
  curpx = curp.x;
  curpy = curp.y;

  //ProcessClick(Label,Index);
  switch (CurPointSelState)
  {
  case Select_Raady:
    if (event.LeftDown ())
    {
      //      if (!event.ShiftDown()) {
      CurPointSelState = Draw_Rectangle_Ready;
      xStateMousePosition[Draw_Rectangle_Ready] = curpx;
      yStateMousePosition[Draw_Rectangle_Ready] = curpy;
      //      }
    }
    if (event.ShiftDown ())
      ShiftDownFlag = true;
    else
      ShiftDownFlag = false;

    break;
  case Point_Select:
    if (event.LeftUp ())
    {
      CurPointSelState = Select_Raady;
      xStateMousePosition[Select_Raady] = 0;
      yStateMousePosition[Select_Raady] = 0;
    }
    break;
  case Draw_Rectangle_Ready:
    if (event.LeftUp ())
    {
      CurPointSelState = Select_Raady;
      xStateMousePosition[Select_Raady] = 0;
      yStateMousePosition[Select_Raady] = 0;
    }
    if (event.Dragging ())
    {
      CurPointSelState = Draw_Rectangle;
      xStateMousePosition[Draw_Rectangle] = curpx;
      yStateMousePosition[Draw_Rectangle] = curpy;
    }
    if (event.ShiftDown ())
      ShiftDownFlag = true;
    else
      ShiftDownFlag = false;
    break;
  case Draw_Rectangle:
    if (event.LeftUp ())
    {
      CurPointSelState = Rectangle_Stable;
      xStateMousePosition[Rectangle_Stable] = curpx;
      yStateMousePosition[Rectangle_Stable] = curpy;
    }
    else
    {
      xStateMousePosition[Draw_Rectangle] = curpx;
      yStateMousePosition[Draw_Rectangle] = curpy;
    }
    if (event.ShiftDown ())
      ShiftDownFlag = true;
    else
      ShiftDownFlag = false;
    break;
  case Rectangle_Stable:
    if (event.LeftDown ())
    {
      if (((abs (curpx - xStateMousePosition[Draw_Rectangle_Ready]) +
	    abs (curpx - xStateMousePosition[Rectangle_Stable])) ==
	   abs (xStateMousePosition[Draw_Rectangle_Ready] -
		xStateMousePosition[Rectangle_Stable]))
	  &&
	  ((abs (curpy - yStateMousePosition[Draw_Rectangle_Ready]) +
	    abs (curpy - yStateMousePosition[Rectangle_Stable])) ==
	   abs (yStateMousePosition[Draw_Rectangle_Ready] -
		yStateMousePosition[Rectangle_Stable])))
      {
	CurPointSelState = Move_Rectangle_Ready;
	xStateMousePosition[Move_Rectangle_Ready] = curpx;
	yStateMousePosition[Move_Rectangle_Ready] = curpy;
      }
      else
      {
	CurPointSelState = Select_Raady;
	xStateMousePosition[Select_Raady] = curpx;
	yStateMousePosition[Select_Raady] = curpy;
      }
    }

    if (event.ShiftDown ())
      ShiftDownFlag = true;
    else
      ShiftDownFlag = false;
    break;
  case Move_Rectangle_Ready:
    if (event.Dragging ())
    {
      CurPointSelState = Move_Rectangle;
      xStateMousePosition[Move_Rectangle] = curpx;
      yStateMousePosition[Move_Rectangle] = curpy;
    }
    //append08/21
    if (event.LeftDown ())
    {
      if (((abs (curpx - xStateMousePosition[Draw_Rectangle_Ready]) +
	    abs (curpx - xStateMousePosition[Rectangle_Stable])) ==
	   abs (xStateMousePosition[Draw_Rectangle_Ready] -
		xStateMousePosition[Rectangle_Stable]))
	  &&
	  ((abs (curpy - yStateMousePosition[Draw_Rectangle_Ready]) +
	    abs (curpy - yStateMousePosition[Rectangle_Stable])) ==
	   abs (yStateMousePosition[Draw_Rectangle_Ready] -
		yStateMousePosition[Rectangle_Stable])))
      {
	CurPointSelState = Move_Rectangle_Ready;
	xStateMousePosition[Move_Rectangle_Ready] = curpx;
	yStateMousePosition[Move_Rectangle_Ready] = curpy;
      }
      else
      {
	CurPointSelState = Select_Raady;
	xStateMousePosition[Select_Raady] = curpx;
	yStateMousePosition[Select_Raady] = curpy;
      }
    }
    //append 08/21
    if (event.ShiftDown ())
      ShiftDownFlag = true;
    else
      ShiftDownFlag = false;
    break;
  case Move_Rectangle:
    if (event.LeftUp ())
    {
      CurPointSelState = Rectangle_Stable;
      xStateMousePosition[Draw_Rectangle_Ready] =
	curpx - xStateMousePosition[Move_Rectangle_Ready] +
	xStateMousePosition[Draw_Rectangle_Ready];
      yStateMousePosition[Draw_Rectangle_Ready] =
	curpy - yStateMousePosition[Move_Rectangle_Ready] +
	yStateMousePosition[Draw_Rectangle_Ready];
      xStateMousePosition[Rectangle_Stable] =
	curpx - xStateMousePosition[Move_Rectangle_Ready] +
	xStateMousePosition[Rectangle_Stable];
      yStateMousePosition[Rectangle_Stable] =
	curpy - yStateMousePosition[Move_Rectangle_Ready] +
	yStateMousePosition[Rectangle_Stable];
    }
    else
    {
      xStateMousePosition[Move_Rectangle] = curpx;
      yStateMousePosition[Move_Rectangle] = curpy;
    }
    if (event.ShiftDown ())
      ShiftDownFlag = true;
    else
      ShiftDownFlag = false;
    break;
  }
  ProcessClick (curpx, curpy);
}

void
PlotAreaRecViewer::ProcessClick (long &curpx, long &curpy)
{
  int pointsnum, i, datahighlightnum;
  long curdist;
  long curradius;
  long px, py;
  bool isblankarea;
  isblankarea = true;
  //pointsnum = datax->rows * datax->cols;
  pointsnum = xPointValue.size ();
  if (datahighlight != NULL)
  {
    datahighlightnum = datahighlight->rows * datahighlight->cols;
  }
  else
    return;

  if (CurPointSelState == Draw_Rectangle_Ready)
  {
    for (i = 0; i < pointsnum; i++)
    {
      //(*datax)[i].GetValue(px);
      //(*datay)[i].GetValue(py);
      px = xoffset[i];
      py = yoffset[i];
      curradius = pointsize[i];
      curdist = (curpx - px) * (curpx - px) + (curpy - py) * (curpy - py);
      if (curdist <= (curradius * curradius))
      {
	if (selpoints[i] == false)
	{
	  selpoints[i] = true;
	  if (datahighlight != NULL)
	  {
	    if (i < datahighlightnum)
	      (*datahighlight)[i].SetValue (true);
	  }
	  //(*datahighlight).modifiedBy = this;
	  pstyle[i] = pstyle[i] | CPC_SG_FILLED;
	}
	else
	{
	  selpoints[i] = false;
	  if (datahighlight != NULL)
	  {
	    if (i < datahighlightnum)
	      (*datahighlight)[i].SetValue (false);
	  }
	  //(*datahighlight).modifiedBy = this;                                   
	  pstyle[i] = pstyle[i] & (~CPC_SG_FILLED);
	}
	isblankarea = false;
      }
      else
      {
	if (!ShiftDownFlag)
	{
	  selpoints[i] = false;
	  if (datahighlight != NULL)
	  {
	    if (i < datahighlightnum)
	      (*datahighlight)[i].SetValue (false);
	  }
	  //(*datahighlight).modifiedBy = this;                                   
	  pstyle[i] = pstyle[i] & (~CPC_SG_FILLED);
	  //pstyle[i] =pstyle[i] | CPC_SG_FILLED; 
	}
      }
    }
    if ((!isblankarea) || ((isblankarea) && (!ShiftDownFlag)))
      datahighlight->modifiedBy = this;
  }

  if (CurPointSelState == Point_Select)
  {
    for (i = 0; i < pointsnum; i++)
    {
      //(*datax)[i].GetValue(px);
      //(*datay)[i].GetValue(py);
      px = xoffset[i];
      py = yoffset[i];
      curradius = pointsize[i];
      curdist = (curpx - px) * (curpx - px) + (curpy - py) * (curpy - py);
      if (curdist <= (curradius * curradius))
      {
	if (selpoints[i] == false)
	{
	  selpoints[i] = true;
	  if (datahighlight != NULL)
	  {
	    if (i < datahighlightnum)
	      (*datahighlight)[i].SetValue (true);
	  }
	  //(*datahighlight).modifiedBy = this;
	  pstyle[i] = pstyle[i] | CPC_SG_FILLED;
	}
	else
	{
	  selpoints[i] = false;
	  if (i < datahighlightnum)
	    if (datahighlight != NULL)
	    {
	      (*datahighlight)[i].SetValue (false);
	    }
	  //(*datahighlight).modifiedBy = this;                                   
	  pstyle[i] = pstyle[i] & (~CPC_SG_FILLED);
	}
	isblankarea = false;
      }
    }
    if (!isblankarea)
      datahighlight->modifiedBy = this;
  }

  if (CurPointSelState == Draw_Rectangle)
  {
    for (i = 0; i < pointsnum; i++)
    {

      px = xoffset[i];
      py = yoffset[i];
      if (((abs (px - xStateMousePosition[Draw_Rectangle_Ready]) +
	    abs (px - xStateMousePosition[Draw_Rectangle])) ==
	   abs (xStateMousePosition[Draw_Rectangle_Ready] -
		xStateMousePosition[Draw_Rectangle]))
	  &&
	  ((abs (py - yStateMousePosition[Draw_Rectangle_Ready]) +
	    abs (py - yStateMousePosition[Draw_Rectangle])) ==
	   abs (yStateMousePosition[Draw_Rectangle_Ready] -
		yStateMousePosition[Draw_Rectangle])))
      {

//              if ((px > xStateMousePosition[Draw_Rectangle_Ready]) && (py > yStateMousePosition[Draw_Rectangle_Ready]) && (px < xStateMousePosition[Draw_Rectangle]) && (py < yStateMousePosition[Draw_Rectangle])) {
	selpoints[i] = true;
	if (datahighlight != NULL)
	{
	  if (i < datahighlightnum)
	    (*datahighlight)[i].SetValue (true);
	}
	//      (*datahighlight).modifiedBy = this;
	pstyle[i] = pstyle[i] | CPC_SG_FILLED;
	isblankarea = false;
      }
      else
      {
	if (!ShiftDownFlag)
	{
	  selpoints[i] = false;
	  if (datahighlight != NULL)
	  {
	    if (i < datahighlightnum)
	      (*datahighlight)[i].SetValue (false);
	  }
	  //(*datahighlight).modifiedBy = this;                                   
	  pstyle[i] = pstyle[i] & (~CPC_SG_FILLED);
	  //pstyle[i] =pstyle[i] | CPC_SG_FILLED;
	}
      }
    }
    if ((!isblankarea) || ((isblankarea) && (!ShiftDownFlag)))
      datahighlight->modifiedBy = this;
  }



  if (CurPointSelState == Rectangle_Stable)
  {
    for (i = 0; i < pointsnum; i++)
    {

      px = xoffset[i];
      py = yoffset[i];
      if (((abs (px - xStateMousePosition[Draw_Rectangle_Ready]) +
	    abs (px - xStateMousePosition[Rectangle_Stable])) ==
	   abs (xStateMousePosition[Draw_Rectangle_Ready] -
		xStateMousePosition[Rectangle_Stable]))
	  &&
	  ((abs (py - yStateMousePosition[Draw_Rectangle_Ready]) +
	    abs (py - yStateMousePosition[Rectangle_Stable])) ==
	   abs (yStateMousePosition[Draw_Rectangle_Ready] -
		yStateMousePosition[Rectangle_Stable])))
      {

//              if ((px > xStateMousePosition[Draw_Rectangle_Ready]) && (py > yStateMousePosition[Draw_Rectangle_Ready]) && (px < xStateMousePosition[Rectangle_Stable]) && (py < yStateMousePosition[Rectangle_Stable])) {
	selpoints[i] = true;
	if (datahighlight != NULL)
	{
	  if (i < datahighlightnum)
	    (*datahighlight)[i].SetValue (true);
	}
	//      (*datahighlight).modifiedBy = this;
	pstyle[i] = pstyle[i] | CPC_SG_FILLED;
	isblankarea = false;
      }
      /*
         else {
         if (!ShiftDownFlag) {                        
         selpoints[i] = false;
         (*datahighlight)[i].SetValue(false);
         //   (*datahighlight).modifiedBy = this;                                     
         pstyle[i] =pstyle[i] & (~CPC_SG_FILLED); 
         //pstyle[i] =pstyle[i] | CPC_SG_FILLED; 
         }                    
         }
       */
    }
    if ((!isblankarea) || ((isblankarea) && (!ShiftDownFlag)))
      datahighlight->modifiedBy = this;
  }

  int newstartx, newstarty, newendx, newendy;
  if (CurPointSelState == Move_Rectangle)
  {
    for (i = 0; i < pointsnum; i++)
    {

      px = xoffset[i];
      py = yoffset[i];
      newstartx =
	(xStateMousePosition[Draw_Rectangle_Ready] +
	 xStateMousePosition[Move_Rectangle] -
	 xStateMousePosition[Move_Rectangle_Ready]);
      newstarty =
	(yStateMousePosition[Draw_Rectangle_Ready] +
	 yStateMousePosition[Move_Rectangle] -
	 yStateMousePosition[Move_Rectangle_Ready]);
      newendx =
	(xStateMousePosition[Rectangle_Stable] +
	 xStateMousePosition[Move_Rectangle] -
	 xStateMousePosition[Move_Rectangle_Ready]);
      newendy =
	(yStateMousePosition[Rectangle_Stable] +
	 yStateMousePosition[Move_Rectangle] -
	 yStateMousePosition[Move_Rectangle_Ready]);
      if (((abs (px - newstartx) + abs (px - newendx)) ==
	   abs (newstartx - newendx))
	  && ((abs (py - newstarty) + abs (py - newendy)) ==
	      abs (newstarty - newendy)))
      {
//              if ((px > (xStateMousePosition[Draw_Rectangle_Ready]+ xStateMousePosition[Move_Rectangle]-xStateMousePosition[Move_Rectangle_Ready])) && (py > (yStateMousePosition[Draw_Rectangle_Ready]+yStateMousePosition[Move_Rectangle]-yStateMousePosition[Move_Rectangle_Ready])) && (px < (xStateMousePosition[Rectangle_Stable]+xStateMousePosition[Move_Rectangle]-xStateMousePosition[Move_Rectangle_Ready])) && (py < (yStateMousePosition[Rectangle_Stable]+yStateMousePosition[Move_Rectangle]-yStateMousePosition[Move_Rectangle_Ready]))) {
	selpoints[i] = true;
	if (datahighlight != NULL)
	{
	  if (i < datahighlightnum)
	    (*datahighlight)[i].SetValue (true);
	}
	//(*datahighlight).modifiedBy = this;
	pstyle[i] = pstyle[i] | CPC_SG_FILLED;
	isblankarea = false;
      }

      else
      {
	if (!ShiftDownFlag)
	{
	  //      selpoints[i] = false;
	  if (datahighlight != NULL)
	  {
	    if (i < datahighlightnum)
	      (*datahighlight)[i].SetValue (false);
	  }
	  //(*datahighlight).modifiedBy = this;                                   
	  pstyle[i] = pstyle[i] & (~CPC_SG_FILLED);
	  //pstyle[i] =pstyle[i] | CPC_SG_FILLED;         
	}
      }

    }
    if ((!isblankarea) || ((isblankarea) && (!ShiftDownFlag)))
      datahighlight->modifiedBy = this;
  }

  //this->Refresh(true);
  /*
     if (isblankarea == false)
     ProcessAction();
   */
  if (isblankarea)
    Clickblankareaflag = true;
  else
    Clickblankareaflag = false;
  return;
}

bool PlotAreaRecViewer::draw (DataObject & dobj)
{
  //UpdateColIndex();
  this->Refresh (true);		//Append On July 21,2005
  if (!Clickblankareaflag)
    ProcessAction ();
  Clickblankareaflag = true;
//      wxPaintEvent ev;
//      OnPaint(ev);
  DoActualDraw (FALSE);
  return TRUE;
}

void
PlotAreaRecViewer::OnPaint (wxPaintEvent & event)
{
  DoActualDraw (TRUE);
}

void
PlotAreaRecViewer::DoActualDraw (bool usePaintDC)
{
  double startx, starty, endx, endy;
  int xaxisspacenum, yaxisspacenum;
  int i, j;
  int currx, stepx, curry, stepy;
  double valuestepx, valuestepy, currvaluex, currvaluey;
  int pointsnum;
  int curfontsize, curlabeloffset;
  int xaxisbeginblank, yaxisbeginblank, xaxisendblank, yaxisendblank;
  int h, w;
  int leftmargin, rightmargin, topmargin, bottommargin;

  GetClientSize (&w, &h);
  wxBitmap tempBitmap = wxBitmap (w, h, -1);
  wxMemoryDC dc;
  dc.SelectObject (tempBitmap);

  wxPaintDC curpdc (this);
  wxClientDC curcdc (this);
  wxString label;
  wxColour col ("BLUE");
  wxBrush brush (*wxWHITE);
  dc.SetBrush (brush);
  wxPen pen1 (*wxWHITE);
  dc.SetPen (pen1);


  dc.DrawRectangle (0, 0, w, h);

  wxPen pen (col);
  dc.SetPen (pen);

  UpdatePlotParameters ();
  leftmargin = plotleftmargin;
  rightmargin = plotrightmargin;
  topmargin = plottopmargin;
  bottommargin = plotbottommargin;

  int plotwidth = w - leftmargin - rightmargin;
  int plotheight = h - topmargin - bottommargin;

  curfontsize = (dc.GetFont ()).GetPointSize ();
  curlabeloffset = getlabeloffset (plotnamelabel, curfontsize);
  dc.DrawText (plotnamelabel, w / 2 - curlabeloffset, plotnamemargin);
  dc.DrawRectangle (leftmargin, topmargin, plotwidth, plotheight);
  //label.Printf( wxT("the width=%d   height=%d"), w,h );
  //dc.DrawText( label, 80, (h-40)/2 );
  // Update Tick parameters
  UpdateTickParameters ();

  // Update axis-Y or axis-Y Range       
  UpdateRange ();
  startx = xaxismin;
  starty = yaxismin;
  endx = xaxismax;
  endy = yaxismax;
  xaxisspacenum = xscalenum;
  yaxisspacenum = yscalenum;

  xaxisbeginblank = 5;
  yaxisbeginblank = 5;
  xaxisendblank = 5;
  yaxisendblank = 5;

  //             UpdateTickParameters();                 
  // draw x axis
  if (!isxtickblank)
  {
    // currx = 65;
    // currx = leftmargin;
    currx = leftmargin + xaxisbeginblank;
    //  stepx =(w-80)/xaxisspacenum;
    //      stepx =plotwidth/xaxisspacenum; 
    //stepx =(plotwidth-xaxisbeginblank)/xaxisspacenum;
    stepx = (plotwidth - xaxisbeginblank - xaxisendblank) / xaxisspacenum;
    valuestepx = ((double) (endx - startx)) / xaxisspacenum;
    currvaluex = startx;

    j = 0;
    for (i = 0; i < xaxisspacenum + 1; i++)
    {
      if ((currvaluex >= xstart) && (currvaluex <= xend))
      {
	if (xtickloc == 0)
	  dc.DrawLine (currx, h - bottommargin, currx, h - bottommargin - xticklen);	// x tick up
	else
	  dc.DrawLine (currx, h - bottommargin, currx, h - bottommargin + xticklen);	// x tick down
	wxString label1;

	if (!isplotxticklabel)
	  label1.Printf (_T ("%f"), currvaluex);
	else
	{
	  if (xticklabel != NULL)
	  {
	    if (j == 0)
	    {
	      if (i < plotxticklabels.size ())
		label1.Printf (_T ("%s"), plotxticklabels[i - j].c_str ());
	      else
		label1.Printf (_T (" "));

	    }
	    if (j == 1)
	    {
	      if (i < plotxticklabels.size () + 1)
		label1.Printf (_T ("%s"), plotxticklabels[i - j].c_str ());
	      else
		label1.Printf (_T (" "));
	    }
	  }
	  else
	    label1.Printf (_T (" "));
	}
	while (label1.Last () == _T ('0'))
	  label1.RemoveLast ();
	if ((label1.Last () == _T ('.')) || (label1.Last () == _T (',')))
	  label1.Append (_T ('0'));
	//dc.DrawText( label1, currx-4, h-55 );
	//      dc.DrawText( label1, currx-8, h-55 );
	curlabeloffset = getlabeloffset (label1, curfontsize);
	// dc.DrawText( label1, currx-5, h-bottommargin+5 );
	dc.DrawText (label1, currx - curlabeloffset,
		     h - bottommargin + xticklen + 2);

      }
      else
	j = 1;
      currx = currx + stepx;
      currvaluex = currvaluex + valuestepx;
    }

  }

  //draw X label
  if (!isaxisxlabelblank)
  {
    wxString labelx;
    if (dataxaxislabel != NULL)
    {
      (*dataxaxislabel)[0].GetValue (labelx);
      curlabeloffset = getlabeloffset (labelx, curfontsize);
      dc.DrawText (labelx, (leftmargin + plotwidth / 2 - curlabeloffset),
		   h - bottommargin + 25);
      //      dc.DrawText( labelx, (leftmargin+plotwidth/2), h-bottommargin+25 );
    }
  }

  // draw Y axis
  if (!isytickblank)
  {
    curry = h - bottommargin - yaxisbeginblank;
    //  stepy =(h-80)/yaxisspacenum;
    //stepy =plotheight/yaxisspacenum;
    stepy = (plotheight - yaxisbeginblank - yaxisendblank) / yaxisspacenum;
    valuestepy = ((double) (endy - starty)) / yaxisspacenum;
    currvaluey = starty;
    j = 0;
    for (i = 0; i < yaxisspacenum + 1; i++)
    {
      if ((currvaluey >= ystart) && (currvaluey <= yend))
      {
	if (ytickloc == 0)
	  //dc.DrawLine(65, curry, 65+yticklen, curry ); // Y tick left
	  dc.DrawLine (leftmargin, curry, leftmargin + yticklen, curry);	// Y tick right
	else
	  //dc.DrawLine(65, curry, 65-yticklen, curry ); // Y tick left
	  dc.DrawLine (leftmargin, curry, leftmargin - yticklen, curry);	// Y tick left
	wxString label2;

	if (!isplotyticklabel)
	  label2.Printf (_T ("%f"), currvaluey);
	else
	{
	  if (yticklabel != NULL)
	  {
	    if (j == 0)
	    {
	      if (i < plotyticklabels.size ())
		label2.Printf (_T ("%s"), plotyticklabels[i - j].c_str ());
	      else
		label2.Printf (_T (" "));
	    }
	    if (j == 1)
	    {
	      if (i < plotyticklabels.size () + 1)
		label2.Printf (_T ("%s"), plotyticklabels[i - j].c_str ());
	      else
		label2.Printf (_T (" "));
	    }

	  }
	  else
	    label2.Printf (_T (" "));
	}
	//label2.Printf( _T("%f"), currvaluey ); 
	while (label2.Last () == _T ('0'))
	  label2.RemoveLast ();
	if ((label2.Last () == _T ('.')) || (label2.Last () == _T (',')))
	  label2.Append (_T ('0'));
	//dc.DrawText(label2, 25, curry-10);
//                      dc.DrawRotatedText(label2, leftmargin-30, curry-5,90);
	curlabeloffset = getlabeloffset (label2, curfontsize);
	dc.DrawRotatedText (label2, leftmargin - 30,
			    curry + curlabeloffset, 90);

      }
      else
	j = 1;
      curry = curry - stepy;
      currvaluey = currvaluey + valuestepy;
    }
  }

  //draw Y label
  if (!isaxisylabelblank)
  {
    wxString labely;
    if (datayaxislabel != NULL)
    {
      (*datayaxislabel)[0].GetValue (labely);
      curlabeloffset = getlabeloffset (labely, curfontsize);
      //dc.DrawRotatedText(labely,15,(h/2-40),-90);           
      //      dc.DrawRotatedText(labely,leftmargin-50,(h/2-40),-90);  
      dc.DrawRotatedText (labely, leftmargin - 50,
			  (topmargin + plotheight / 2 + curlabeloffset), 90);
    }
    plotheight = h - topmargin - bottommargin;
  }

  UpdateColIndex ();		// Update points value;
  //draw points

  //pointsnum = datax->rows * datax->cols;
  //pointsnum = datax->rows * datax->cols;
  pointsnum = xPointValue.size ();
  if (xoffset.empty () && yoffset.empty ())
  {
    for (i = 0; i < pointsnum; i++)
    {
      currvaluex = xPointValue[i];
      currvaluey = yPointValue[i];
      currx =
	leftmargin + xaxisbeginblank +
	((currvaluex - startx) / (endx - startx)) * (plotwidth -
						     xaxisbeginblank -
						     xaxisendblank);
      curry =
	h - bottommargin - yaxisbeginblank -
	((currvaluey - starty) / (endy - starty)) * (plotheight -
						     yaxisbeginblank -
						     yaxisendblank);
      xoffset.push_back (currx);
      yoffset.push_back (curry);
    }
  }
  else
  {
    //      xoffset.clear();
    //      yoffset.clear();
    for (i = 0; i < pointsnum; i++)
    {
      currvaluex = xPointValue[i];
      currvaluey = yPointValue[i];
      currx =
	leftmargin + xaxisbeginblank +
	((currvaluex - startx) / (endx - startx)) * (plotwidth -
						     xaxisbeginblank -
						     xaxisendblank);
      curry =
	h - bottommargin - yaxisbeginblank -
	((currvaluey - starty) / (endy - starty)) * (plotheight -
						     yaxisbeginblank -
						     yaxisendblank);

      xoffset[i] = currx;
      yoffset[i] = curry;
    }
  }

  int curcolornum, curpcolorindex;
  wxString curpcolor;
  if ((datacolorindex == NULL) && (datacolorname != NULL))
  {
    curcolornum = datacolorname->rows * datacolorname->cols;
    for (i = 0; i < pointsnum; i++)
    {
      if (i < curcolornum)
      {
	(*datacolorname)[i].GetValue (curpcolor);
	if (curpcolor != pcolor[i])
	  pcolor[i] = curpcolor;
      }
    }
  }
  if ((datacolorindex != NULL) && (datacolorname == NULL))
  {
    curcolornum = datacolorindex->rows * datacolorindex->cols;
    for (i = 0; i < pointsnum; i++)
    {
      if (i < curcolornum)
      {
	(*datacolorindex)[i].GetValue (curpcolorindex);
	curpcolor = GetColorNamefromIndex (curpcolorindex);
	if (curpcolor != pcolor[i])
	  pcolor[i] = curpcolor;
      }
    }
  }
  if ((datacolorindex != NULL) && (datacolorname != NULL))
  {
    if (datacolorname->rows * datacolorname->cols <
	datacolorindex->rows * datacolorindex->cols)
      curcolornum = datacolorname->rows * datacolorname->cols;
    else
      curcolornum = datacolorindex->rows * datacolorindex->cols;
    for (i = 0; i < pointsnum; i++)
    {
      if (i < curcolornum)
      {
	(*datacolorname)[i].GetValue (curpcolor);
	(*datacolorindex)[i].GetValue (curpcolorindex);
	if (curpcolor != pcolor[i])
	{
	  pcolor[i] = curpcolor;
	  curpcolorindex = GetColorIndexfromName (curpcolor);
	  (*datacolorindex)[i].SetValue (curpcolorindex);
	}
	else
	{
	  if (GetColorNamefromIndex (curpcolorindex) != pcolor[i])
	  {
	    pcolor[i] = GetColorNamefromIndex (curpcolorindex);
	    (*datacolorname)[i].SetValue (pcolor[i]);
	  }
	}
      }
    }
  }
  int curptypeindex, curptype, curptypenum;
  if (datatype != NULL)
  {
    curptypenum = datatype->rows * datatype->cols;
    for (i = 0; i < pointsnum; i++)
    {
      if (i < curptypenum)
      {
	(*datatype)[i].GetValue (curptypeindex);
	switch (curptypeindex)
	{
	case 1:
	  curptype = CPC_SG_CIRCLE;
	  pointsize[i] = 1;
	  pstyle[i] = pstyle[i] | CPC_SG_FILLED;
	  if ((datasize != NULL) && (i <= datasize->rows * datasize->cols))
	    (*datasize)[i].SetValue (1);
	  break;
	case 2:
	  curptype = CPC_SG_EXE;
	  break;
	case 3:
	  curptype = CPC_SG_CROSS;
	  break;
	case 4:
	  curptype = CPC_SG_STAR;
	  break;
	case 5:
	  curptype = CPC_SG_CIRCLE;
	  break;
	case 6:
	  curptype = CPC_SG_DIAMOND;
	  break;
	case 7:
	  curptype = CPC_SG_SQUARE;
	  break;
	case 8:
	  curptype = CPC_SG_TRIANGLE;
	  break;
	default:
	  curptype = CPC_SG_CIRCLE;

	}
	if (curptype != ptype[i])
	  ptype[i] = curptype;
      }
    }
  }
  int cursizenum, curpsize;
  if (datasize != NULL)
  {
    cursizenum = datasize->rows * datasize->cols;
    for (i = 0; i < pointsnum; i++)
    {
      if (i < cursizenum)
      {
	(*datasize)[i].GetValue (curpsize);
	if (curpsize != pointsize[i])
	  pointsize[i] = curpsize;
      }
    }
  }

  // Check whether points are deleted 
  bool curpointisdel;
  int pointsdeldtnum;
  int phighlightnum;
  if (pointsdel != NULL)
    pointsdeldtnum = pointsdel->rows * pointsdel->cols;
  else
    pointsdeldtnum = 0;
  if (datahighlight != NULL)
    phighlightnum = datahighlight->rows * datahighlight->cols;
  if (pointsdel != NULL)
  {
    for (i = 0; i < pointsnum; i++)
    {
      curpointisdel = ispointsdel[i];
      if (i < pointsdeldtnum)	//Modified on August 22,2005
	(*pointsdel)[i].GetValue (curpointisdel);
      if (curpointisdel)
      {
	ispointsdel[i] = true;
      }
      else
      {
	ispointsdel[i] = false;
      }
      if (ispointsdel[i])
      {
	if ((datahighlight != NULL) && (i < phighlightnum))
	{			//Modified on August 22,2005
	  (*datahighlight)[i].SetValue (false);
	  selpoints[i] = false;
	}
	//delpoint(dc,i);       
      }

    }
  }

  bool curphighlight;
  int datahighlightnum = 0;
  if (datahighlight != NULL)
  {
    datahighlightnum = datahighlight->rows * datahighlight->cols;
    for (i = 0; i < pointsnum; i++)
    {
      if ((xoffset[i] >= leftmargin + xaxisbeginblank)
	  && (xoffset[i] <= w - rightmargin - xaxisendblank)
	  && (yoffset[i] >= topmargin + yaxisendblank)
	  && (yoffset[i] <= h - bottommargin - yaxisbeginblank))
      {
	if ((xPointValue[i] >= xstart) && (xPointValue[i] <= xend)
	    && (yPointValue[i] >= ystart) && (yPointValue[i] <= yend))
	{
	  //(*datahighlight)[i].GetValue(selpoints[i]);               // Append on July 20,2005
	  if (datahighlight != NULL)
	  {
	    if (i < datahighlightnum)
	      (*datahighlight)[i].GetValue (curphighlight);
	    else
	      curphighlight = selpoints[i];
	  }
	  else
	    curphighlight = selpoints[i];
	  if (curphighlight)
	  {
	    selpoints[i] = true;
	    pstyle[i] = pstyle[i] | CPC_SG_FILLED;
	  }
	  else
	  {
	    selpoints[i] = false;
	    pstyle[i] = pstyle[i] & (~CPC_SG_FILLED);
	  }

	  if (selpoints[i])
	  {
	    //pen.SetColour(pcolor[i]);
	    //dc.SetPen(pen);
	    dc.SetPen (wxColour (pcolor[i]));
	    brush.SetColour (pcolor[i]);
	    dc.SetBrush (brush);
	    //brush.SetColour(pcolor[i]);
	    //CarapaceDrawGlyph(dc,xoffset[i],yoffset[i],CPC_SG_CIRCLE,pointsize[i],pstyle[i]);     
	    if (!ispointsdel[i])
	      PlotDrawGlyph (dc, xoffset[i], yoffset[i], ptype[i],
			     pointsize[i], pstyle[i]);
	  }
	  else
	  {
	    dc.SetPen (wxColour (pcolor[i]));
	    //pen.SetColour(pcolor[i]);
	    brush.SetColour ("WHITE");
	    dc.SetBrush (brush);
	    if (!ispointsdel[i])
	      CarapaceDrawGlyph (dc, xoffset[i], yoffset[i],
				 ptype[i], pointsize[i], pstyle[i]);

//PlotDrawGlyph(dc,xoffset[i],yoffset[i],ptype[i],pointsize[i],pstyle[i]);              
	  }
	}
      }
    }
  }


  // draw rectangle according current select state
  wxPen pen2 (*wxBLACK, 1, wxSHORT_DASH);
  dc.SetPen (pen2);
  int currecwidth, currechight, newstartx, newstarty;
  wxBrush brush2;
  switch (CurPointSelState)
  {
  case Select_Raady:
    break;
  case Point_Select:
    break;
  case Draw_Rectangle_Ready:
    break;
  case Draw_Rectangle:
    currecwidth =
      xStateMousePosition[Draw_Rectangle] -
      xStateMousePosition[Draw_Rectangle_Ready];
    currechight =
      yStateMousePosition[Draw_Rectangle] -
      yStateMousePosition[Draw_Rectangle_Ready];
    brush2 = dc.GetBrush ();
    dc.SetBrush (*wxTRANSPARENT_BRUSH);
    dc.DrawRectangle (xStateMousePosition[Draw_Rectangle_Ready],
		      yStateMousePosition[Draw_Rectangle_Ready],
		      currecwidth, currechight);
    dc.SetBrush (brush2);
    break;
  case Rectangle_Stable:
    currecwidth =
      xStateMousePosition[Rectangle_Stable] -
      xStateMousePosition[Draw_Rectangle_Ready];
    currechight =
      yStateMousePosition[Rectangle_Stable] -
      yStateMousePosition[Draw_Rectangle_Ready];
    brush2 = dc.GetBrush ();
    dc.SetBrush (*wxTRANSPARENT_BRUSH);
    dc.DrawRectangle (xStateMousePosition[Draw_Rectangle_Ready],
		      yStateMousePosition[Draw_Rectangle_Ready],
		      currecwidth, currechight);
    dc.SetBrush (brush2);
    break;
  case Move_Rectangle_Ready:
    currecwidth =
      xStateMousePosition[Rectangle_Stable] -
      xStateMousePosition[Draw_Rectangle_Ready];
    currechight =
      yStateMousePosition[Rectangle_Stable] -
      yStateMousePosition[Draw_Rectangle_Ready];
    //newstartx = xStateMousePosition[Draw_Rectangle_Ready] + xStateMousePosition[Move_Rectangle]   - xStateMousePosition[Move_Rectangle_Ready];
    //newstarty = yStateMousePosition[Draw_Rectangle_Ready] + yStateMousePosition[Move_Rectangle]   - yStateMousePosition[Move_Rectangle_Ready];
    brush2 = dc.GetBrush ();
    dc.SetBrush (*wxTRANSPARENT_BRUSH);
    dc.DrawRectangle (xStateMousePosition[Draw_Rectangle_Ready],
		      yStateMousePosition[Draw_Rectangle_Ready],
		      currecwidth, currechight);
    dc.SetBrush (brush2);
    break;
  case Move_Rectangle:
    currecwidth =
      xStateMousePosition[Rectangle_Stable] -
      xStateMousePosition[Draw_Rectangle_Ready];
    currechight =
      yStateMousePosition[Rectangle_Stable] -
      yStateMousePosition[Draw_Rectangle_Ready];
    newstartx =
      xStateMousePosition[Draw_Rectangle_Ready] +
      xStateMousePosition[Move_Rectangle] -
      xStateMousePosition[Move_Rectangle_Ready];
    newstarty =
      yStateMousePosition[Draw_Rectangle_Ready] +
      yStateMousePosition[Move_Rectangle] -
      yStateMousePosition[Move_Rectangle_Ready];
    brush2 = dc.GetBrush ();
    dc.SetBrush (*wxTRANSPARENT_BRUSH);
    dc.DrawRectangle (newstartx, newstarty, currecwidth, currechight);
    dc.SetBrush (brush2);
    break;
  }
  int curpenwidth, curpenstyle;
  dc.SetPen (pen);
  curpenwidth = pen.GetWidth ();
  curpenstyle = pen.GetStyle ();

  //decide whether curve will be deleted        

  bool curcurveisdel;
  if (curvedel != NULL)
  {
    (*curvedel)[0].GetValue (curcurveisdel);
    if (curcurveisdel)
    {
      iscurvedeleted = true;
    }
    else
    {
      iscurvedeleted = false;
    }
  }
  else
    iscurvedeleted = false;


  // draw curve
  if (!iscurvedeleted)
  {
    int linespointsnum;
    int curlinepointx, curlinepointy, curlinewidth, curlinestyle;
    UpdateLinesSplashFlag ();
    UpdateLinesPoints ();
//updated 08/18
    wxString curlncolor;
    int curlncolorindex;

    if ((datalinescolor != NULL) && (datalinescolorindex == NULL))
    {
      (*datalinescolor)[0].GetValue (curlncolor);
      if (curlncolor != linescolor)
	linescolor = curlncolor;
    }

    if ((datalinescolor == NULL) && (datalinescolorindex != NULL))
    {
      (*datalinescolorindex)[0].GetValue (curlncolorindex);
      curlncolor = GetColorNamefromIndex (curlncolorindex);
      if (curlncolor != linescolor)
	linescolor = curlncolor;
    }

    if ((datalinescolor != NULL) && (datalinescolorindex != NULL))
    {
      (*datalinescolor)[0].GetValue (curlncolor);
      (*datalinescolorindex)[0].GetValue (curlncolorindex);
      if (curlncolor != linescolor)
      {
	linescolor = curlncolor;
	curlncolorindex = GetColorIndexfromName (curlncolor);
	(*datalinescolorindex)[0].SetValue (curlncolorindex);
      }
      else
      {
	if (GetColorNamefromIndex (curlncolorindex) != linescolor)
	{
	  linescolor = GetColorNamefromIndex (curlncolorindex);
	  (*datalinescolor)[0].SetValue (linescolor);
	}
      }
    }

    int curlnwidth;
    if (datalineswidth != NULL)
    {
      (*datalineswidth)[0].GetValue (curlnwidth);
      if (curlnwidth != lineswidth)
	lineswidth = curlnwidth;
    }

    int curlnstyle, curlnstyleindex;
    if (datalinesstyle != NULL)
    {
      (*datalinesstyle)[0].GetValue (curlnstyleindex);
      switch (curlnstyleindex)
      {
      case 1:
	curlnstyle = wxSOLID;	// 1 - Solid
	break;
      case 2:
	curlnstyle = wxTRANSPARENT;	// 2 - TRANSPARENT
	break;
      case 3:
	curlnstyle = wxDOT;	// 3 - DOT        
	break;
      case 4:
	curlnstyle = wxLONG_DASH;	// 4 - LONG_DASH    
	break;
      case 5:
	curlnstyle = wxSHORT_DASH;	// 5 - SHORT_DASH  
	break;
      case 6:
	curlnstyle = wxDOT_DASH;	// 6 - DOT_DASH      
	break;
      case 7:
	curlnstyle = wxSTIPPLE;	// 7 - Use the stipple bitmap 
	break;
      case 8:
	curlnstyle = wxUSER_DASH;	// 8 - Use the user dashes  
	break;
      case 9:
	curlnstyle = wxBDIAGONAL_HATCH;	// 9 - Backward diagonal hatch        
	break;
      case 10:
	curlnstyle = wxCROSSDIAG_HATCH;	// 10 - Cross-diagonal hatch  
      case 11:
	curlnstyle = wxFDIAGONAL_HATCH;	// 11 - Forward diagonal hatch                                                                                                       
	break;
      case 12:
	curlnstyle = wxCROSS_HATCH;	// 12 - Cross hatch                                                                                                      
	break;
      case 13:
	curlnstyle = wxHORIZONTAL_HATCH;	// 12 - Horizontal hatch                                                                                                     
	break;
      case 14:
	curlnstyle = wxVERTICAL_HATCH;	// 12 - Vertical hatch                                                                                                        
	break;
      default:
	curlnstyle = 0;		// 0 - DEFAULT STYLE  
      }

      if (curlnstyle != linesstyle)
	linesstyle = curlnstyle;

    }



    if ((datalinesx != NULL) && (datalinesy == NULL))
    {
      if (lineswidth < 0)
	curlinewidth = curpenwidth;
      else
	curlinewidth = lineswidth;
      if (curlinestyle == 0)
	curlinestyle = curpenstyle;
      else
	curlinestyle = linesstyle;
      wxPen pen3 (linescolor, curlinewidth, curlinestyle);
      dc.SetPen (pen3);
      currx =
	leftmargin + xaxisbeginblank +
	((xlinesPointValue[0] - startx) / (endx - startx)) * (plotwidth -
							      xaxisbeginblank
							      -
							      xaxisendblank);
      if ((currx >= leftmargin + xaxisbeginblank)
	  && (currx <= plotwidth + leftmargin - xaxisendblank))
	dc.DrawLine (currx, h - bottommargin, currx, topmargin + yaxisendblank);	//                 
      dc.SetPen (pen);
    }

    if ((datalinesx == NULL) && (datalinesy != NULL))
    {
      if (lineswidth < 0)
	curlinewidth = curpenwidth;
      else
	curlinewidth = lineswidth;
      if (curlinestyle == 0)
	curlinestyle = curpenstyle;
      else
	curlinestyle = linesstyle;
      wxPen pen3 (linescolor, curlinewidth, curlinestyle);
      dc.SetPen (pen3);
      curry =
	h - bottommargin - yaxisbeginblank -
	((ylinesPointValue[0] - starty) / (endy - starty)) * (plotheight -
							      yaxisbeginblank
							      -
							      yaxisendblank);
      if ((curry >= topmargin + yaxisendblank)
	  && (curry <= h - bottommargin - yaxisbeginblank))
	dc.DrawLine (leftmargin, curry,
		     plotwidth + leftmargin - xaxisendblank, curry);
      dc.SetPen (pen);
    }

    int reallinepointsnum = 0;
    if ((datalinesx != NULL) && (datalinesy != NULL))
    {
      linespointsnum = xlinesPointValue.size ();

      for (i = 0; i < linespointsnum; i++)
      {

	currvaluex = xlinesPointValue[i];
	currvaluey = ylinesPointValue[i];

	currx =
	  leftmargin + xaxisbeginblank +
	  ((currvaluex - startx) / (endx - startx)) * (plotwidth -
						       xaxisbeginblank -
						       xaxisendblank);
	curry =
	  h - bottommargin - yaxisbeginblank -
	  ((currvaluey - starty) / (endy - starty)) * (plotheight -
						       yaxisbeginblank -
						       yaxisendblank);
	//linespointpsseq->Append(new wxPoint(currx,curry));
	if ((currx >= leftmargin + xaxisbeginblank)
	    && (currx <= plotwidth + leftmargin - xaxisendblank)
	    && (curry >= topmargin + yaxisendblank)
	    && (curry <= h - bottommargin - yaxisbeginblank))
	{
	  linespointpsseq[reallinepointsnum] = wxPoint (currx, curry);
	  reallinepointsnum = reallinepointsnum + 1;
	}
      }
      linespointpsseqnum = reallinepointsnum;

      if (lineswidth < 0)
	curlinewidth = curpenwidth;
      else
	curlinewidth = lineswidth;
      if (curlinestyle == 0)
	curlinestyle = curpenstyle;
      else
	curlinestyle = linesstyle;
      wxPen pen3 (linescolor, curlinewidth, curlinestyle);
      dc.SetPen (pen3);
      if (reallinepointsnum > 1)
      {
	if (!curveissplashline)
	  dc.DrawSpline (reallinepointsnum, linespointpsseq);
	else
	  DrawSplashLine (dc, reallinepointsnum, linespointpsseq);
      }
      dc.SetPen (pen);
    }
  }
//decide whether curve will be deleted  
  if (usePaintDC)
  {
    curpdc.Blit (0, 0, w, h, &dc, 0, 0);
  }
  else
  {
    curcdc.Blit (0, 0, w, h, &dc, 0, 0);
  }

}

IMPLEMENT_DYNAMIC_CLASS (PlotViewerXmlHandler, wxXmlResourceHandler) PlotViewerXmlHandler::PlotViewerXmlHandler ():wxXmlResourceHandler
  ()
{
  AddWindowStyles ();
}


wxObject *
PlotViewerXmlHandler::DoCreateResource ()
{
  wxString plotname = GetText (wxT ("title"));
  wxString plotparams = GetText (wxT ("dimensions"));

  wxString dataselcol = GetText (wxT ("columnselector"));
  wxString dataxname = GetText (wxT ("x"));
  wxString datayname = GetText (wxT ("y"));
  wxString dataxaxislabel = GetText (wxT ("xaxislabel"));
  wxString datayaxislabel = GetText (wxT ("yaxislabel"));
  wxString datarangex = GetText (wxT ("xrange"));
  wxString datarangey = GetText (wxT ("yrange"));
  wxString datapcolor = GetText (wxT ("pointcolorsbyindex"));
  wxString datapcolorname = GetText (wxT ("pointcolorsbyname"));
  wxString datapsize = GetText (wxT ("pointsizes"));
  wxString dataptype = GetText (wxT ("pointtypes"));
  wxString dataphighlight = GetText (wxT ("highlighted"));
  wxString curpointsdel = GetText (wxT ("deleted"));

  wxString ticknum = GetText (wxT ("ticknum"));
  wxString xticklabel = GetText (wxT ("xticklabel"));
  wxString yticklabel = GetText (wxT ("yticklabel"));
  wxString ticklocation = GetText (wxT ("ticklocation"));
  wxString ticklen = GetText (wxT ("ticklen"));
  wxString usecustomticklabels = GetText (wxT ("usecustomticklabels"));
  wxString isaxislabelblank = GetText (wxT ("isaxislabelblank"));

  wxString curlinescolorindex = GetText (wxT ("datalinescolorindex"));
  wxString curlinescolor = GetText (wxT ("datalinescolor"));
  wxString curlineswidth = GetText (wxT ("datalineswidth"));
  wxString curlinesstyle = GetText (wxT ("datalinesstyle"));
  wxString curlinesx = GetText (wxT ("linesx"));
  wxString curlinesy = GetText (wxT ("linesy"));

  wxString curcurvedel = GetText (wxT ("curvedel"));
  wxString currescale = GetText (wxT ("rescale"));
  wxString curdrawsplashline = GetText (wxT ("splashline"));

  wxString viewername = GetName ();
  wxString actionstring = GetText ("action");
  wxString actionname = GetText ("actionname");
  wxString error;

  bool verbose = GetBool ("verbose", FALSE);
  bool doxyupdate = GetBool("xyupdate", FALSE);

  vector < wxString > viewerdata;
  int rows, cols, i;
  vector < double >dvalsx;
  double tmpstartx, tmpendx;
  vector < double >dvalsy;
  double tmpstarty, tmpendy;
  DataObject *dobj, *dobj2, *dobj3;
  int pointsnum;
  int xdatatype, ydatatype, currows, curcols;
  int xdataselcolindex, ydataselcolindex;
  int curplotwidth, curplotheight, curleftmargin, currightmargin,
    curtopmargin, curbottommargin, curplotnamemargin;
  wxString curplotname;

  dobj =
    GetDynFromName (plotparams, "plotparameters", INTT, -1, -1,
		    "PlotAreaRecViewer", viewername, FALSE);
  if ((dobj != NULL) && (dobj->rows * dobj->cols > 6))
  {
    (*dobj)[0].GetValue (curplotwidth);
    (*dobj)[1].GetValue (curplotheight);
    (*dobj)[2].GetValue (curleftmargin);
    (*dobj)[3].GetValue (currightmargin);
    (*dobj)[4].GetValue (curtopmargin);
    (*dobj)[5].GetValue (curbottommargin);
    (*dobj)[6].GetValue (curplotnamemargin);
  }
  else
  {
    // default plot parameters
    curplotwidth = 480;
    curplotheight = 400;
    curleftmargin = 65;
    currightmargin = 25;
    curtopmargin = 40;
    curbottommargin = 60;
    curplotnamemargin = 15;
    error.Printf (CPC_STRING_VIEWER0103, "plotparameters",
		  "PlotAreaRecViewer", viewername.c_str ());
    if (verbose)
    {
      wxMessageBox (error);
    }
  }

  dobj =
    GetDynFromName (dataselcol, "dataselcol", INTT, -1, -1,
		    "PlotAreaRecViewer", viewername, FALSE);
  if (dobj != NULL)
  {
    if (dobj->rows * dobj->cols > 1)
    {
      (*dobj)[0].GetValue (xdataselcolindex);
      (*dobj)[1].GetValue (ydataselcolindex);
    }
    else
    {
      xdataselcolindex = 0;
      ydataselcolindex = 0;
    }
  }
  else
  {
    error.Printf (CPC_STRING_VIEWER0013, "dataselcol", "dataselcol",
		  "PlotAreaRecViewer", viewername.c_str ());
    wxMessageBox (error);
    return (NULL);
  }

  if ((xdataselcolindex < 0) || (ydataselcolindex < 0)
      || ((xdataselcolindex * ydataselcolindex == 0)
	  && ((xdataselcolindex + ydataselcolindex) != 0)))
  {
    error.Printf (CPC_STRING_VIEWER0100, "dataxname", "datayname",
		  "PlotAreaRecViewer", viewername.c_str ());
    wxMessageBox (error);
    return (NULL);
  }
  viewerdata.push_back (dataselcol);
  dobj3 = dobj;

  if (thisApp->DynamicDataP->
      getDataObjectProperty (dataxname, currows, curcols, xdatatype))
  {
    if ((xdatatype != INTT) && (xdatatype != DOULBET))
    {				// Append on August 22,2005
      error.Printf (CPC_STRING_VIEWER0013, "dataxname", "dataxname",
		    "PlotAreaRecViewer", viewername.c_str ());
      wxMessageBox (error);
      return (NULL);
    }
    if (xdatatype == INTT)
      dobj =
	GetDynFromName (dataxname, "dataxname", INTT, -1, -1,
			"PlotAreaRecViewer", viewername, FALSE);
    else
      dobj =
	GetDynFromName (dataxname, "dataxname", DOULBET, -1, -1,
			"PlotAreaRecViewer", viewername, FALSE);
  }
  else
  {
    error.Printf (CPC_STRING_VIEWER0013, "dataxname", "dataxname",
		  "PlotAreaRecViewer", viewername.c_str ());
    wxMessageBox (error);
    return (NULL);
  }


  if (thisApp->DynamicDataP->
      getDataObjectProperty (datayname, currows, curcols, ydatatype))
  {
    if ((ydatatype != INTT) && (ydatatype != DOULBET))
    {				// Append on August 22,2005
      error.Printf (CPC_STRING_VIEWER0013, "datayname", "datayname",
		    "PlotAreaRecViewer", viewername.c_str ());
      wxMessageBox (error);
      return (NULL);
    }
    if (ydatatype == INTT)
      dobj2 =
	GetDynFromName (datayname, "datayname", INTT, -1, -1,
			"PlotAreaRecViewer", viewername, FALSE);
    else
      dobj2 =
	GetDynFromName (datayname, "datayname", DOULBET, -1, -1,
			"PlotAreaRecViewer", viewername, FALSE);
  }
  else
  {
    error.Printf (CPC_STRING_VIEWER0013, "datayname", "datayname",
		  "PlotAreaRecViewer", viewername.c_str ());
    wxMessageBox (error);
    return (NULL);
  }


  if ((xdataselcolindex == 0) && (xdataselcolindex == 0))
  {
    if (((dobj->rows * dobj->cols) != (dobj2->rows * dobj2->cols)))
    {
      error.Printf (CPC_STRING_VIEWER0013, "dataxname", "datayname",
		    "PlotAreaRecViewer", viewername.c_str ());
      wxMessageBox (error);
      return (NULL);
    }
    else
    {
      viewerdata.push_back (dataxname);
      viewerdata.push_back (datayname);
      pointsnum = (dobj->rows) * (dobj->cols);
    }

  }
  else
  {
    if (dobj->rows != dobj2->rows)
    {
      error.Printf (CPC_STRING_VIEWER0018, "dataxname", "datayname",
		    "PlotAreaRecViewer", viewername.c_str ());
      wxMessageBox (error);
      return (NULL);
    }
    else
    {
      viewerdata.push_back (dataxname);
      viewerdata.push_back (datayname);
      pointsnum = dobj->rows;
    }
  }

//      PlotAreaRecViewer *view = new PlotAreaRecViewer(m_parentAsWindow,wxXmlResource::GetXRCID(viewername),wxDefaultPosition,wxSize(480,400));
  PlotAreaRecViewer *view = new PlotAreaRecViewer (m_parentAsWindow,
						   wxXmlResource::
						   GetXRCID (viewername),
						   wxDefaultPosition,
						   wxSize (curplotwidth,
							   curplotheight));

//      view->Create(m_parentAsWindow,wxXmlResource::GetXRCID(viewername));
  view->datax = dobj;
  view->datay = dobj2;
  view->dataColumnSel = dobj3;
  view->doxyupdate = doxyupdate;
  view->dataxname = dataxname;
  view->datayname = datayname;
  KludgeView = view;
  CreateChildren (view);
  int xcurvalint, ycurvalint;
  double xcurvaldbl, ycurvaldbl;
  //pointsnum = (view->datax->rows) * (view->datax->cols);
  if ((xdataselcolindex == 0) && (ydataselcolindex == 0))
  {
    for (i = 0; i < pointsnum; i++)
    {
      if (dobj->type == INTT)
      {
	(*dobj)[i].GetValue (xcurvalint);
	xcurvaldbl = xcurvalint;
	view->xPointValue.push_back (xcurvaldbl);
      }
      else
      {
	(*dobj)[i].GetValue (xcurvaldbl);
	view->xPointValue.push_back (xcurvaldbl);
      }
      if (dobj2->type == INTT)
      {
	(*dobj2)[i].GetValue (ycurvalint);
	ycurvaldbl = ycurvalint;
	view->yPointValue.push_back (ycurvaldbl);
      }
      else
      {
	(*dobj2)[i].GetValue (ycurvaldbl);
	view->yPointValue.push_back (ycurvaldbl);
      }
    }
    view->xColIndex = 0;
    view->yColIndex = 0;
  }
  else
  {
    for (i = 0; i < pointsnum; i++)
    {
      if (dobj->type == INTT)
      {
	(*dobj)[(xdataselcolindex - 1) * pointsnum + i].GetValue (xcurvalint);
	xcurvaldbl = xcurvalint;
	view->xPointValue.push_back (xcurvaldbl);
      }
      else
      {
	(*dobj)[(xdataselcolindex - 1) * pointsnum + i].GetValue (xcurvaldbl);
	view->xPointValue.push_back (xcurvaldbl);
      }
      if (dobj2->type == INTT)
      {
	(*dobj2)[(ydataselcolindex - 1) * pointsnum +
		 i].GetValue (ycurvalint);
	ycurvaldbl = ycurvalint;
	view->yPointValue.push_back (ycurvaldbl);
      }
      else
      {
	(*dobj2)[(ydataselcolindex - 1) * pointsnum +
		 i].GetValue (ycurvaldbl);
	view->yPointValue.push_back (ycurvaldbl);
      }
    }
    view->xColIndex = xdataselcolindex;
    view->yColIndex = ydataselcolindex;
  }

  dobj =
    GetDynFromName (plotparams, "plotparameters", INTT, -1, -1,
		    "PlotAreaRecViewer", viewername, FALSE);
  if (dobj != NULL)
  {
    view->plotparameters = dobj;
    viewerdata.push_back (plotparams);
  }
  else
  {
    view->plotparameters = NULL;
  }

  view->recwidth = curplotwidth;
  view->recheight = curplotheight;
  view->plotleftmargin = curleftmargin;
  view->plotrightmargin = currightmargin;
  view->plottopmargin = curtopmargin;
  view->plotbottommargin = curbottommargin;
  view->plotnamemargin = curplotnamemargin;

  dobj2 =
    GetDynFromName (plotname, "plotname", STRINGT, -1, -1,
		    "PlotAreaRecViewer", viewername, FALSE);
  if (dobj2 != NULL)
  {
    view->plotname = dobj2;
    viewerdata.push_back (plotname);
    (*dobj2)[0].GetValue (curplotname);
    view->plotnamelabel = curplotname;
  }
  else
  {
    view->plotname = NULL;
    curplotname = "";
    view->plotnamelabel = "";
    error.Printf (CPC_STRING_VIEWER0104, "plotname", "PlotAreaRecViewer",
		  viewername.c_str ());
    if (verbose)
    {
      wxMessageBox (error);
    }
  }


  dobj =
    GetDynFromName (datapcolor, "datacolorindex", INTT, -1, -1,
		    "PlotAreaRecViewer", viewername, FALSE);
  dobj2 =
    GetDynFromName (datapcolorname, "datacolorname", STRINGT, -1, -1,
		    "PlotAreaRecViewer", viewername, FALSE);
  int curcolornum, curcolorindex;
  wxString curcolorname;
  bool colormatchflag = true;

  if ((dobj != NULL) && (dobj2 != NULL))
  {
    if ((dobj->rows * dobj->cols) == (dobj2->rows * dobj2->cols))
    {
      curcolornum = dobj->rows * dobj->cols;
      for (i = 0; i < pointsnum; i++)
      {
	if (i < curcolornum)
	{
	  (*dobj)[i].GetValue (curcolorindex);
	  (*dobj2)[i].GetValue (curcolorname);
	  switch (curcolorindex)
	  {
	  case 53:
	    if (curcolorname != wxString ("RED"))
	      colormatchflag = false;
	    else
	      view->pcolor.push_back (wxString ("RED"));
	    break;
	  case 2:
	    if (curcolorname != wxString ("BLUE"))
	      colormatchflag = false;
	    else
	      view->pcolor.push_back (wxString ("BLUE"));
	    break;
	  case 68:
	    if (curcolorname != wxString ("YELLOW"))
	      colormatchflag = false;
	    else
	      view->pcolor.push_back (wxString ("YELLOW"));
	    break;
	  case 22:
	    if (curcolorname != wxString ("GREEN"))
	      colormatchflag = false;
	    else
	      view->pcolor.push_back (wxString ("GREEN"));
	    break;
	  case 6:
	    if (curcolorname != wxString ("CORAL"))
	      colormatchflag = false;
	    else
	      view->pcolor.push_back (wxString ("CORAL"));
	    break;
	  case 1:
	    if (curcolorname != wxString ("BLACK"))
	      colormatchflag = false;
	    else
	      view->pcolor.push_back (wxString ("BLACK"));
	    break;
	  case 21:
	    if (curcolorname != wxString ("GREY"))
	      colormatchflag = false;
	    else
	      view->pcolor.push_back (wxString ("GREY"));
	    break;
	  case 67:
	    if (curcolorname != wxString ("WHITE"))
	      colormatchflag = false;
	    else
	      view->pcolor.push_back (wxString ("WHITE"));
	    break;
	  default:
	    //if (curcolorname != wxString("BLACK"))
	    if (curcolorname != wxString ("BLUE"))
	      colormatchflag = false;
	    else
	      view->pcolor.push_back (wxString ("BLUE"));
	  }
	}
	else
	  view->pcolor.push_back (wxString ("BLACK"));

	if (colormatchflag == false)
	  view->pcolor.push_back (wxString ("BLUE"));
	//break;
      }

      if (colormatchflag == false)
      {
	error.Printf (CPC_STRING_VIEWER0102, "datacolorindex",
		      "datacolorname", "PlotAreaRecViewer",
		      viewername.c_str ());
	if (verbose)
	{
	  wxMessageBox (error);
	}

      }
      viewerdata.push_back (datapcolor);
      viewerdata.push_back (datapcolorname);
      view->datacolorindex = dobj;
      view->datacolorname = dobj2;
    }
    else
    {
      if ((dobj->rows * dobj->cols) < (dobj2->rows * dobj2->cols))
	curcolornum = dobj->rows * dobj->cols;
      else
	curcolornum = dobj2->rows * dobj2->cols;
      for (i = 0; i < pointsnum; i++)
      {
	if (i < curcolornum)
	{
	  (*dobj)[i].GetValue (curcolorindex);
	  (*dobj2)[i].GetValue (curcolorname);
	  switch (curcolorindex)
	  {
	  case 53:
	    if (curcolorname != wxString ("RED"))
	      colormatchflag = false;
	    else
	      view->pcolor.push_back (wxString ("RED"));
	    break;
	  case 2:
	    if (curcolorname != wxString ("BLUE"))
	      colormatchflag = false;
	    else
	      view->pcolor.push_back (wxString ("BLUE"));
	    break;
	  case 68:
	    if (curcolorname != wxString ("YELLOW"))
	      colormatchflag = false;
	    else
	      view->pcolor.push_back (wxString ("YELLOW"));
	    break;
	  case 22:
	    if (curcolorname != wxString ("GREEN"))
	      colormatchflag = false;
	    else
	      view->pcolor.push_back (wxString ("GREEN"));
	    break;
	  case 6:
	    if (curcolorname != wxString ("CORAL"))
	      colormatchflag = false;
	    else
	      view->pcolor.push_back (wxString ("CORAL"));
	    break;
	  case 1:
	    if (curcolorname != wxString ("BLACK"))
	      colormatchflag = false;
	    else
	      view->pcolor.push_back (wxString ("BLACK"));
	    break;
	  case 21:
	    if (curcolorname != wxString ("GREY"))
	      colormatchflag = false;
	    else
	      view->pcolor.push_back (wxString ("GREY"));
	    break;
	  case 67:
	    if (curcolorname != wxString ("WHITE"))
	      colormatchflag = false;
	    else
	      view->pcolor.push_back (wxString ("WHITE"));
	    break;
	  default:
	    //if (curcolorname != wxString("BLACK"))
	    if (curcolorname != wxString ("BLUE"))
	      colormatchflag = false;
	    else
	      view->pcolor.push_back (wxString ("BLUE"));
	  }
	}
	else
	  view->pcolor.push_back (wxString ("BLACK"));

	if (colormatchflag == false)
	  view->pcolor.push_back (wxString ("BLUE"));
	//break;
      }

      if (colormatchflag == false)
      {
	error.Printf (CPC_STRING_VIEWER0102, "datacolorindex",
		      "datacolorname", "PlotAreaRecViewer",
		      viewername.c_str ());
	if (verbose)
	{
	  wxMessageBox (error);
	}

      }
      viewerdata.push_back (datapcolor);
      viewerdata.push_back (datapcolorname);
      view->datacolorindex = dobj;
      view->datacolorname = dobj2;
    }
  }
  if ((dobj != NULL) && (dobj2 == NULL))
  {
    viewerdata.push_back (datapcolor);
    curcolornum = dobj->rows * dobj->cols;
    for (i = 0; i < pointsnum; i++)
    {
      if (i < curcolornum)
      {
	(*dobj)[i].GetValue (curcolorindex);
	switch (curcolorindex)
	{
	case 53:
	  view->pcolor.push_back (wxString ("RED"));
	  break;
	case 2:
	  view->pcolor.push_back (wxString ("BLUE"));
	  break;
	case 68:
	  view->pcolor.push_back (wxString ("YELLOW"));
	  break;
	case 22:
	  view->pcolor.push_back (wxString ("GREEN"));
	  break;
	case 6:
	  view->pcolor.push_back (wxString ("CORAL"));
	  break;
	case 1:
	  view->pcolor.push_back (wxString ("BLACK"));
	  break;
	case 21:
	  view->pcolor.push_back (wxString ("GREY"));
	  break;
	case 67:
	  view->pcolor.push_back (wxString ("WHITE"));
	  break;
	default:
//                                              view->pcolor.push_back(wxString("BLACK"));      
	  view->pcolor.push_back (wxString ("BLUE"));
	}
      }
      else
	view->pcolor.push_back (wxString ("BLUE"));
    }
    view->datacolorindex = dobj;
    view->datacolorname = NULL;
  }

  if ((dobj == NULL) && (dobj2 != NULL))
  {
    viewerdata.push_back (datapcolorname);
    curcolornum = dobj2->rows * dobj2->cols;
    colormatchflag = false;
    for (i = 0; i < pointsnum; i++)
    {
      if (i < curcolornum)
      {
	(*dobj2)[i].GetValue (curcolorname);
	if (curcolorname == wxString ("RED"))
	  colormatchflag = true;
	if (curcolorname == wxString ("BLUE"))
	  colormatchflag = true;
	if (curcolorname == wxString ("YELLOW"))
	  colormatchflag = true;
	if (curcolorname == wxString ("GREEN"))
	  colormatchflag = true;
	if (curcolorname == wxString ("CORAL"))
	  colormatchflag = true;
	if (curcolorname == wxString ("BLACK"))
	  colormatchflag = true;
	if (curcolorname == wxString ("GREY"))
	  colormatchflag = true;
	if (curcolorname == wxString ("WHITE"))
	  colormatchflag = true;
	if (colormatchflag)
	{
	  view->pcolor.push_back (curcolorname);
	  colormatchflag = false;
	}
	else
	  break;
      }
      else
	//       view->pcolor.push_back(wxString("BLACK"));     
	view->pcolor.push_back (wxString ("BLUE"));
    }
    if (i >= pointsnum)
    {
      view->datacolorindex = NULL;
      view->datacolorname = dobj2;
    }
    else
    {
      error.Printf (CPC_STRING_VIEWER0013, "datacolorname",
		    "datacolorname", "PlotAreaRecViewer",
		    viewername.c_str ());
      wxMessageBox (error);
      return (NULL);
    }
  }

  if ((dobj == NULL) && (dobj2 == NULL))
  {
    for (i = 0; i < pointsnum; i++)
      //view->pcolor.push_back(wxString("BLACK"));    
      view->pcolor.push_back (wxString ("BLUE"));
    view->datacolorindex = NULL;
    view->datacolorname = NULL;
    error.Printf (CPC_STRING_VIEWER0105, "datacolorname", "datacolorindex",
		  "PlotAreaRecViewer", viewername.c_str ());
    if (verbose)
    {
      wxMessageBox (error);
    }

  }

  dobj2 =
    GetDynFromName (datapsize, "datasize", INTT, -1, -1, "PlotAreaRecViewer",
		    viewername, FALSE);
  int curpsizenum, curpsize;
  if (dobj2 != NULL)
  {
    viewerdata.push_back (datapsize);
    curpsizenum = dobj2->rows * dobj2->cols;
    for (i = 0; i < pointsnum; i++)
    {
      if (i < curpsizenum)
      {
	(*dobj2)[i].GetValue (curpsize);
	view->pointsize.push_back (curpsize);
      }
      else
	view->pointsize.push_back (3);
    }
    view->datasize = dobj2;
  }
  else
  {
    for (i = 0; i < pointsnum; i++)
      view->pointsize.push_back (3);
    view->datasize = NULL;
    error.Printf (CPC_STRING_VIEWER0106, "datasize", "PlotAreaRecViewer",
		  viewername.c_str ());
    if (verbose)
    {
      wxMessageBox (error);
    }
  }

  dobj =
    GetDynFromName (dataptype, "datatype", INTT, -1, -1, "PlotAreaRecViewer",
		    viewername, FALSE);
  int curtypenum, curtypeindex;
  if (dobj != NULL)
  {
    viewerdata.push_back (dataptype);
    curtypenum = dobj->rows * dobj->cols;
    for (i = 0; i < pointsnum; i++)
    {
      if (i < curtypenum)
      {
	(*dobj)[i].GetValue (curtypeindex);
	switch (curtypeindex)
	{
	case 1:
	  view->ptype.push_back (CPC_SG_CIRCLE);
	  view->pointsize[i] = 1;
	  if ((view->datasize != NULL)
	      && (i <= view->datasize->rows * view->datasize->cols))
	  {
	    (*view->datasize)[i].SetValue (1);
	  }
	  break;
	case 2:
	  view->ptype.push_back (CPC_SG_EXE);
	  break;
	case 3:
	  view->ptype.push_back (CPC_SG_CROSS);
	  break;
	case 4:
	  view->ptype.push_back (CPC_SG_STAR);
	  break;
	case 5:
	  view->ptype.push_back (CPC_SG_CIRCLE);
	  break;
	case 6:
	  view->ptype.push_back (CPC_SG_DIAMOND);
	  break;
	case 7:
	  view->ptype.push_back (CPC_SG_SQUARE);
	  break;
	case 8:
	  view->ptype.push_back (CPC_SG_TRIANGLE);
	  break;
	default:
	  view->ptype.push_back (CPC_SG_CIRCLE);

	}
      }
      else
	view->ptype.push_back (CPC_SG_CIRCLE);
    }
    view->datatype = dobj;
  }
  else
  {
    for (i = 0; i < pointsnum; i++)
      view->ptype.push_back (CPC_SG_CIRCLE);
    view->datatype = NULL;
    error.Printf (CPC_STRING_VIEWER0107, "datatype", "PlotAreaRecViewer",
		  viewername.c_str ());
    if (verbose)
    {
      wxMessageBox (error);
    }
  }

  dobj2 =
    GetDynFromName (dataphighlight, "datahighlight", LOGICT, -1, -1,
		    "PlotAreaRecViewer", viewername, FALSE);
  int curphighlightnum;
  bool curphighlight;
  if (dobj2 != NULL)
  {
    viewerdata.push_back (dataphighlight);
    curphighlightnum = dobj2->rows * dobj2->cols;
    for (i = 0; i < pointsnum; i++)
    {
      if (i < curphighlightnum)
      {
	(*dobj2)[i].GetValue (curphighlight);
	if (curphighlight)
	  view->selpoints.push_back (true);
	else
	  view->selpoints.push_back (false);
      }
      else
	view->selpoints.push_back (false);
    }
    view->datahighlight = dobj2;
  }
  else
  {
    error.Printf (CPC_STRING_VIEWER0013, "dataphighlight", "datahighlight",
		  "PlotAreaRecViewer", viewername.c_str ());
    for (i = 0; i < pointsnum; i++)
      view->selpoints.push_back (false);
    view->datahighlight = NULL;
    if (verbose)
    {
      wxMessageBox (error);
    }
    //return(NULL); 
  }
  int curpstyle = 0;
  for (i = 0; i < pointsnum; i++)
  {
    //      view->selpoints.push_back(false);
    if ((view->selpoints[i])
	|| ((view->ptype[i] == CPC_SG_CIRCLE) && (view->pointsize[i] == 1)))
      curpstyle = curpstyle | CPC_SG_FILLED;
    else
      curpstyle = 0;
    view->pstyle.push_back (curpstyle);
    curpstyle = 0;
  }

  if (actionstring != "")
  {
    view->actionstring = actionstring;
  }
  dobj = GetDynFromName (actionname, "actionname", STRINGT, -1, -1,
			 "PlotViewer", viewername, FALSE);
  if (dobj != NULL)
  {
    view->action = dobj;
    viewerdata.push_back (actionname);
  }

  dobj =
    GetDynFromName (dataxaxislabel, "dataxaxislabel", STRINGT, -1, -1,
		    "PlotAreaRecViewer", viewername, FALSE);
  if (dobj != NULL)
  {
    view->dataxaxislabel = dobj;
    viewerdata.push_back (dataxaxislabel);
  }
  else
  {
    view->dataxaxislabel = NULL;
    error.Printf (CPC_STRING_VIEWER0108, "dataxaxislabel",
		  "PlotAreaRecViewer", viewername.c_str ());
    if (verbose)
    {
      wxMessageBox (error);
    }
  }

  dobj2 =
    GetDynFromName (datayaxislabel, "datayaxislabel", STRINGT, -1, -1,
		    "PlotAreaRecViewer", viewername, FALSE);
  if (dobj2 != NULL)
  {
    view->datayaxislabel = dobj2;
    viewerdata.push_back (datayaxislabel);
  }
  else
  {
    view->datayaxislabel = NULL;
    error.Printf (CPC_STRING_VIEWER0109, "datayaxislabel",
		  "PlotAreaRecViewer", viewername.c_str ());
    if (verbose)
    {
      wxMessageBox (error);
    }
  }

  int curxticknum, curyticknum;
  dobj =
    GetDynFromName (ticknum, "ticknum", INTT, -1, -1, "PlotAreaRecViewer",
		    viewername, FALSE);
  if (dobj != NULL)
  {
    view->ticknum = dobj;
    viewerdata.push_back (ticknum);
    if (dobj->rows * dobj->cols > 1)
    {
      (*dobj)[0].GetValue (curxticknum);
      (*dobj)[1].GetValue (curyticknum);
    }
    else
    {
      (*dobj)[0].GetValue (curxticknum);
      curyticknum = 5;
    }
    view->plotxticknum = curxticknum;
    view->plotyticknum = curyticknum;
    view->isxtickblank = false;
    view->isytickblank = false;
    if (curxticknum < 1)
    {
      view->isxtickblank = true;
      view->plotxticknum = 5;
    }
    if (curyticknum < 1)
    {
      view->isytickblank = true;
      view->plotyticknum = 5;
    }
  }
  else
  {
    view->ticknum = NULL;
    view->plotxticknum = 5;
    view->plotyticknum = 5;
    view->isxtickblank = true;
    view->isytickblank = true;
    error.Printf (CPC_STRING_VIEWER0110, "ticknum", "PlotAreaRecViewer",
		  viewername.c_str ());
    if (verbose)
    {
      wxMessageBox (error);
    }
  }

  dobj =
    GetDynFromName (ticklen, "ticklen", INTT, -1, -1, "PlotAreaRecViewer",
		    viewername, FALSE);
  if (dobj != NULL)
  {
    view->ticklen = dobj;
    viewerdata.push_back (ticklen);
    if (dobj->rows * dobj->cols > 1)
    {
      (*dobj)[0].GetValue (view->xticklen);
      (*dobj)[1].GetValue (view->yticklen);
    }
    else
    {
      (*dobj)[0].GetValue (view->xticklen);
      view->yticklen = 5;
    }

  }
  else
  {
    view->ticklen = NULL;
    view->xticklen = 5;
    view->yticklen = 5;
    error.Printf (CPC_STRING_VIEWER0111, "ticklen", "PlotAreaRecViewer",
		  viewername.c_str ());
    if (verbose)
    {
      wxMessageBox (error);
    }
  }

  dobj =
    GetDynFromName (ticklocation, "ticklocation", INTT, -1, -1,
		    "PlotAreaRecViewer", viewername, FALSE);
  if (dobj != NULL)
  {
    view->ticklocation = dobj;
    viewerdata.push_back (ticklocation);
    if (dobj->rows * dobj->cols > 1)
    {
      (*dobj)[0].GetValue (view->xtickloc);
      (*dobj)[1].GetValue (view->ytickloc);
    }
    else
    {
      (*dobj)[0].GetValue (view->xtickloc);
      view->ytickloc = 1;
    }
  }
  else
  {
    view->ticklocation = NULL;
    view->xtickloc = 0;
    view->ytickloc = 0;
    error.Printf (CPC_STRING_VIEWER0112, "ticklocation",
		  "PlotAreaRecViewer", viewername.c_str ());
    if (verbose)
    {
      wxMessageBox (error);
    }
  }

  dobj =
    GetDynFromName (usecustomticklabels, "usecustomticklabels", LOGICT, -1,
		    -1, "PlotAreaRecViewer", viewername, FALSE);
  if (dobj != NULL)
  {
    view->usecustomticklabels = dobj;
    viewerdata.push_back (usecustomticklabels);
    if (dobj->rows * dobj->cols > 1)
    {
      (*dobj)[0].GetValue (view->isplotxticklabel);
      (*dobj)[1].GetValue (view->isplotyticklabel);
    }
    else
    {
      (*dobj)[0].GetValue (view->isplotxticklabel);
      view->isplotyticklabel = false;
    }
  }
  else
  {
    view->usecustomticklabels = NULL;
    view->isplotxticklabel = false;
    view->isplotyticklabel = false;
    error.Printf (CPC_STRING_VIEWER0114, "usecustomticklabels",
		  "PlotAreaRecViewer", viewername.c_str ());
    if (verbose)
    {
      wxMessageBox (error);
    }
  }

  int labelnum;
  wxString curlabel;
  //if (view->isplotxticklabel) {
  dobj =
    GetDynFromName (xticklabel, "xticklabel", STRINGT, -1, -1,
		    "PlotAreaRecViewer", viewername, FALSE);
  if (dobj != NULL)
  {
    labelnum = dobj->rows * dobj->cols;
    for (i = 0; i < labelnum; i++)
    {
      (*dobj)[i].GetValue (curlabel);
      view->plotxticklabels.push_back (curlabel);
    }
    view->xticklabel = dobj;
    viewerdata.push_back (xticklabel);

  }
  else
  {
    error.Printf (CPC_STRING_VIEWER0013, "xticklabel", "xticklabel",
		  "PlotAreaRecViewer", viewername.c_str ());
    if (verbose)
    {
      wxMessageBox (error);
    }
    view->xticklabel = NULL;
    //return(NULL);
  }
  //}
  //if (view->isplotyticklabel) {
  dobj2 =
    GetDynFromName (yticklabel, "yticklabel", STRINGT, -1, -1,
		    "PlotAreaRecViewer", viewername, FALSE);
  if (dobj2 != NULL)
  {
    labelnum = dobj2->rows * dobj2->cols;
    for (i = 0; i < labelnum; i++)
    {
      (*dobj2)[i].GetValue (curlabel);
      view->plotyticklabels.push_back (curlabel);
    }
    view->yticklabel = dobj2;
    viewerdata.push_back (yticklabel);

  }
  else
  {
    error.Printf (CPC_STRING_VIEWER0013, "yticklabel", "yticklabel",
		  "PlotAreaRecViewer", viewername.c_str ());
    if (verbose)
    {
      wxMessageBox (error);
    }
    view->yticklabel = NULL;
    //return(NULL);
  }
  //}             

  dobj =
    GetDynFromName (isaxislabelblank, "isaxislabelblank", LOGICT, -1, -1,
		    "PlotAreaRecViewer", viewername, FALSE);
  if (dobj != NULL)
  {
    view->isaxislabelblank = dobj;
    viewerdata.push_back (isaxislabelblank);
    if (dobj->rows * dobj->cols > 1)
    {
      (*dobj)[0].GetValue (view->isaxisxlabelblank);
      (*dobj)[1].GetValue (view->isaxisylabelblank);
    }
    else
    {
      (*dobj)[0].GetValue (view->isaxisxlabelblank);
      view->isaxisylabelblank = false;
    }

  }
  else
  {
    view->isaxislabelblank = NULL;
    view->isaxisxlabelblank = false;
    view->isaxisylabelblank = false;
  }

  dobj = NULL;
  if (thisApp->DynamicDataP->
      getDataObjectProperty (datarangex, currows, curcols, xdatatype))
  {
    if (xdatatype == INTT)
      dobj =
	GetDynFromName (datarangex, "datarangex", INTT, -1, -1,
			"PlotAreaRecViewer", viewername, FALSE);
    else
      dobj =
	GetDynFromName (datarangex, "datarangex", DOULBET, -1, -1,
			"PlotAreaRecViewer", viewername, FALSE);
  }

  int xstartint, xendint;
  if ((dobj != NULL) && (dobj->rows * dobj->cols > 1))	//Modified on August 22,2005
  {
    //view->datarangex = dobj;
    //viewerdata.push_back(datarangex);
    if (dobj->type == DOULBET)
    {
      (*dobj)[0].GetValue (view->xstart);	// low range
      (*dobj)[1].GetValue (view->xend);	// high range
    }
    else
    {
      (*dobj)[0].GetValue (xstartint);	// low range
      (*dobj)[1].GetValue (xendint);	// high range
      view->xstart = xstartint;
      view->xend = xendint;
    }
    if (view->xstart < view->xend)
    {
      view->datarangex = dobj;
      viewerdata.push_back (datarangex);
    }
    else
    {
      dvalsx = view->xPointValue;
      tmpstartx = dvalsx[0];
      tmpendx = dvalsx[1];
      for (i = 0; i < pointsnum; i++)
      {
	if (tmpstartx > dvalsx[i])
	  tmpstartx = dvalsx[i];
	if (tmpendx < dvalsx[i])
	  tmpendx = dvalsx[i];
      }
      view->xstart = tmpstartx;
      view->xend = tmpendx;
      view->datarangex = NULL;
      error.Printf (CPC_STRING_VIEWER0113, "datarangex",
		    "PlotAreaRecViewer", viewername.c_str ());
      if (verbose)
      {
	wxMessageBox (error);
      }
    }
  }
  else
  {
    dvalsx = view->xPointValue;
    tmpstartx = dvalsx[0];
    tmpendx = dvalsx[1];
    for (i = 0; i < pointsnum; i++)
    {
      if (tmpstartx > dvalsx[i])
	tmpstartx = dvalsx[i];
      if (tmpendx < dvalsx[i])
	tmpendx = dvalsx[i];
    }
    view->xstart = tmpstartx;
    view->xend = tmpendx;
    view->datarangex = NULL;
    if (dobj != NULL)
    {				//Modified on August 22,2005
      error.Printf (CPC_STRING_VIEWER0101, "datarangex",
		    "PlotAreaRecViewer", viewername.c_str ());
      if (verbose)
      {
	wxMessageBox (error);
      }
    }
  }

  view->xaxismin = view->xstart;
  view->xaxismax = view->xend;
  int n = view->plotxticknum;
  int minnum = 1;
  double shrinknum = 0.75;
  double high_u_bias[2];
  high_u_bias[0] = 1.5;
  high_u_bias[1] = 0.5 + 1.5 * high_u_bias[0];
  int epscor = 0;
  R_pretty (&view->xaxismin, &view->xaxismax, &n, &minnum, &shrinknum,
	    high_u_bias, &epscor);
  view->xscalenum = n;

  dobj2 = NULL;
  if (thisApp->DynamicDataP->
      getDataObjectProperty (datarangey, currows, curcols, ydatatype))
  {
    if (ydatatype == INTT)
      dobj2 =
	GetDynFromName (datarangey, "datarangey", INTT, -1, -1,
			"PlotAreaRecViewer", viewername, FALSE);
    else
      dobj2 =
	GetDynFromName (datarangey, "datarangey", DOULBET, -1, -1,
			"PlotAreaRecViewer", viewername, FALSE);
  }

  int ystartint, yendint;

  if ((dobj2 != NULL) && (dobj2->rows * dobj2->cols > 1))	//Modified on August 22,2005
  {
//              view->datarangey = dobj2;
//              viewerdata.push_back(datarangey);
    if (dobj2->type == DOULBET)
    {
      (*dobj2)[0].GetValue (view->ystart);	// low range
      (*dobj2)[1].GetValue (view->yend);	// high range
    }
    else
    {
      (*dobj2)[0].GetValue (ystartint);	// low range
      (*dobj2)[1].GetValue (yendint);	// high range
      view->ystart = ystartint;
      view->yend = yendint;
    }
    if (view->ystart < view->yend)
    {
      view->datarangey = dobj2;
      viewerdata.push_back (datarangey);
    }
    else
    {
      dvalsy = view->yPointValue;
      tmpstarty = dvalsy[0];
      tmpendy = dvalsy[1];
      for (i = 0; i < pointsnum; i++)
      {
	if (tmpstarty > dvalsy[i])
	  tmpstarty = dvalsy[i];
	if (tmpendy < dvalsy[i])
	  tmpendy = dvalsy[i];
      }
      view->ystart = tmpstarty;
      view->yend = tmpendy;
      view->datarangey = NULL;
      error.Printf (CPC_STRING_VIEWER0113, "datarangey",
		    "PlotAreaRecViewer", viewername.c_str ());
      if (verbose)
      {
	wxMessageBox (error);
      }
    }
  }
  else
  {
    dvalsy = view->yPointValue;
    tmpstarty = dvalsy[0];
    tmpendy = dvalsy[1];
    for (i = 0; i < pointsnum; i++)
    {
      if (tmpstarty > dvalsy[i])
	tmpstarty = dvalsy[i];
      if (tmpendy < dvalsy[i])
	tmpendy = dvalsy[i];
    }
    view->ystart = tmpstarty;
    view->yend = tmpendy;
    view->datarangey = NULL;
    if (dobj != NULL)
    {				//Modified on August 22,2005
      error.Printf (CPC_STRING_VIEWER0101, "datarangey",
		    "PlotAreaRecViewer", viewername.c_str ());
      if (verbose)
      {
	wxMessageBox (error);
      }
    }
  }
  view->yaxismin = view->ystart;
  view->yaxismax = view->yend;
  n = view->plotyticknum;
  minnum = 1;
  shrinknum = 0.75;
  //double high_u_bias[2];
  high_u_bias[0] = 1.5;
  high_u_bias[1] = 0.5 + 1.5 * high_u_bias[0];
  epscor = 0;
  R_pretty (&view->yaxismin, &view->yaxismax, &n, &minnum, &shrinknum,
	    high_u_bias, &epscor);
  view->yscalenum = n;

  dobj =
    GetDynFromName (curlinescolorindex, "datalinescolorindex", INTT, -1, -1,
		    "PlotAreaRecViewer", viewername, FALSE);
  dobj2 =
    GetDynFromName (curlinescolor, "datalinescolor", STRINGT, -1, -1,
		    "PlotAreaRecViewer", viewername, FALSE);
  wxString curlinescolorname;
  int curlinescolorid;
  bool linescolormatchflag = true;

  if ((dobj != NULL) && (dobj2 != NULL))
  {
    (*dobj)[0].GetValue (curlinescolorid);
    (*dobj2)[0].GetValue (curlinescolorname);
    switch (curlinescolorid)
    {
    case 53:
      if (curlinescolorname != wxString ("RED"))
	linescolormatchflag = false;
      else
	view->linescolor = wxString ("RED");
      break;
    case 2:
      if (curlinescolorname != wxString ("BLUE"))
	linescolormatchflag = false;
      else
	view->linescolor = wxString ("BLUE");
      break;
    case 68:
      if (curlinescolorname != wxString ("YELLOW"))
	linescolormatchflag = false;
      else
	view->linescolor = wxString ("YELLOW");
      break;
    case 22:
      if (curlinescolorname != wxString ("GREEN"))
	linescolormatchflag = false;
      else
	view->linescolor = wxString ("GREEN");
      break;
    case 6:
      if (curlinescolorname != wxString ("CORAL"))
	linescolormatchflag = false;
      else
	view->linescolor = wxString ("CORAL");
      break;
    case 1:
      if (curlinescolorname != wxString ("BLACK"))
	linescolormatchflag = false;
      else
	view->linescolor = wxString ("BLACK");
      break;
    case 21:
      if (curlinescolorname != wxString ("GREY"))
	linescolormatchflag = false;
      else
	view->linescolor = wxString ("GREY");
      break;
    case 67:
      if (curlinescolorname != wxString ("WHITE"))
	linescolormatchflag = false;
      else
	view->linescolor = wxString ("WHITE");
      break;
    default:
      //if (curlinescolorname != wxString("BLACK"))
      if (curlinescolorname != wxString ("BLUE"))
	linescolormatchflag = false;
      else
	//view->linescolor = wxString("BLACK");         
	view->linescolor = wxString ("BLUE");
    }

    if (linescolormatchflag == false)
    {
      view->linescolor = wxString ("BLUE");
      error.Printf (CPC_STRING_VIEWER0020, "datalinescolorindex",
		    "datalinescolorname", "PlotAreaRecViewer",
		    viewername.c_str ());
      if (verbose)
      {
	wxMessageBox (error);
      }
    }
    viewerdata.push_back (curlinescolorindex);
    viewerdata.push_back (curlinescolor);
    view->datalinescolorindex = dobj;
    view->datalinescolor = dobj2;
  }

  if ((dobj != NULL) && (dobj2 == NULL))
  {
    viewerdata.push_back (curlinescolorindex);
    (*dobj)[0].GetValue (curlinescolorid);
    switch (curlinescolorid)
    {
    case 53:
      view->linescolor = (wxString ("RED"));
      break;
    case 2:
      view->linescolor = (wxString ("BLUE"));
      break;
    case 68:
      view->linescolor = (wxString ("YELLOW"));
      break;
    case 22:
      view->linescolor = (wxString ("GREEN"));
      break;
    case 6:
      view->linescolor = (wxString ("CORAL"));
      break;
    case 1:
      view->linescolor = (wxString ("BLACK"));
      break;
    case 21:
      view->linescolor = (wxString ("GREY"));
      break;
    case 67:
      view->linescolor = (wxString ("WHITE"));
      break;
    default:
      //view->linescolor =(wxString("BLACK"));                
      view->linescolor = (wxString ("BLUE"));
    }
    view->datalinescolorindex = dobj;
    view->datalinescolor = NULL;
  }
  if ((dobj == NULL) && (dobj2 != NULL))
  {
    view->datalinescolorindex = NULL;
    viewerdata.push_back (curlinescolor);
    colormatchflag = false;
    (*dobj2)[0].GetValue (curlinescolorname);
    if (curlinescolorname == wxString ("RED"))
      linescolormatchflag = true;
    if (curlinescolorname == wxString ("BLUE"))
      linescolormatchflag = true;
    if (curlinescolorname == wxString ("YELLOW"))
      linescolormatchflag = true;
    if (curlinescolorname == wxString ("GREEN"))
      linescolormatchflag = true;
    if (curlinescolorname == wxString ("CORAL"))
      linescolormatchflag = true;
    if (curlinescolorname == wxString ("BLACK"))
      linescolormatchflag = true;
    if (curlinescolorname == wxString ("GREY"))
      linescolormatchflag = true;
    if (curlinescolorname == wxString ("WHITE"))
      linescolormatchflag = true;
    if (linescolormatchflag)
    {
      view->linescolor = curlinescolorname;
      view->datalinescolor = dobj2;
      linescolormatchflag = false;
    }
    else
    {
      view->datalinescolor = NULL;
      //view->linescolor = wxString("BLACK"); 
      view->linescolor = wxString ("BLUE");
    }

  }

  if ((dobj == NULL) && (dobj2 == NULL))
  {
    //view->linescolor = wxString("BLACK"); 
    view->linescolor = wxString ("BLUE");
    view->datalinescolorindex = NULL;
    view->datalinescolor = NULL;
  }
  dobj =
    GetDynFromName (curlinesstyle, "datalinesstyle", INTT, -1, -1,
		    "PlotAreaRecViewer", viewername, FALSE);
  int curlinetypeindex;
  if (dobj != NULL)
  {
    viewerdata.push_back (curlinesstyle);
    (*dobj)[0].GetValue (curlinetypeindex);
    switch (curlinetypeindex)
    {
    case 1:
      view->linesstyle = wxSOLID;	// 1 - Solid
      break;
    case 2:
      view->linesstyle = wxTRANSPARENT;	// 2 - TRANSPARENT
      break;
    case 3:
      view->linesstyle = wxDOT;	// 3 - DOT  
      break;
    case 4:
      view->linesstyle = wxLONG_DASH;	// 4 - LONG_DASH      
      break;
    case 5:
      view->linesstyle = wxSHORT_DASH;	// 5 - SHORT_DASH    
      break;
    case 6:
      view->linesstyle = wxDOT_DASH;	// 6 - DOT_DASH        
      break;
    case 7:
      view->linesstyle = wxSTIPPLE;	// 7 - Use the stipple bitmap   
      break;
    case 8:
      view->linesstyle = wxUSER_DASH;	// 8 - Use the user dashes    
      break;
    case 9:
      view->linesstyle = wxBDIAGONAL_HATCH;	// 9 - Backward diagonal hatch  
      break;
    case 10:
      view->linesstyle = wxCROSSDIAG_HATCH;	// 10 - Cross-diagonal hatch    
    case 11:
      view->linesstyle = wxFDIAGONAL_HATCH;	// 11 - Forward diagonal hatch                                                                                                 
      break;
    case 12:
      view->linesstyle = wxCROSS_HATCH;	// 12 - Cross hatch                                                                                                        
      break;
    case 13:
      view->linesstyle = wxHORIZONTAL_HATCH;	// 12 - Horizontal hatch                                                                                                       
      break;
    case 14:
      view->linesstyle = wxVERTICAL_HATCH;	// 12 - Vertical hatch                                                                                                  
      break;
    default:
      view->linesstyle = 0;	// 0 - DEFAULT STYLE                                                                                                                                                                                                                                                                                                                    
    }
    if ((curlinetypeindex > 0) && (curlinetypeindex < 15))
    {
      view->datalinesstyle = dobj;
    }
    else
      view->datalinesstyle = NULL;

  }
  else
  {
    view->linesstyle = wxSOLID;	// 1 - Solid                              
    view->datalinesstyle = NULL;
    view->linesstyle = 0;	// 0 - DEFAULT STYLE                    
  }

  dobj =
    GetDynFromName (curlineswidth, "datalineswidth", INTT, -1, -1,
		    "PlotAreaRecViewer", viewername, FALSE);
  if (dobj != NULL)
  {
    view->datalineswidth = dobj;
    viewerdata.push_back (curlineswidth);
    (*dobj)[0].GetValue (view->lineswidth);
  }
  else
  {
    view->datalineswidth = NULL;
    view->lineswidth = -1;	//DEFAULT WIDTH
  }

  dobj = NULL;
  if (thisApp->DynamicDataP->
      getDataObjectProperty (curlinesx, currows, curcols, xdatatype))
  {
    if ((xdatatype != INTT) && (xdatatype != DOULBET))
    {				// Append on August 22,2005
      error.Printf (CPC_STRING_VIEWER0013, "datalinesx", "datalinesx",
		    "PlotAreaRecViewer", viewername.c_str ());
      if (verbose)
      {
	wxMessageBox (error);
      }
      dobj = NULL;
    }
    if (xdatatype == INTT)
      dobj =
	GetDynFromName (curlinesx, "datalinesx", INTT, -1, -1,
			"PlotAreaRecViewer", viewername, FALSE);
    if (xdatatype == DOULBET)
      dobj =
	GetDynFromName (curlinesx, "datalinesx", DOULBET, -1, -1,
			"PlotAreaRecViewer", viewername, FALSE);
  }
  dobj2 = NULL;
  if (thisApp->DynamicDataP->
      getDataObjectProperty (curlinesy, currows, curcols, ydatatype))
  {
    if ((ydatatype != INTT) && (ydatatype != DOULBET))
    {				// Append on August 22,2005
      error.Printf (CPC_STRING_VIEWER0013, "datalinesy", "datalinesy",
		    "PlotAreaRecViewer", viewername.c_str ());
      if (verbose)
      {
	wxMessageBox (error);
      }
      dobj2 = NULL;
    }
    if (ydatatype == INTT)
      dobj2 =
	GetDynFromName (curlinesy, "datalinesy", INTT, -1, -1,
			"PlotAreaRecViewer", viewername, FALSE);
    else
      dobj2 =
	GetDynFromName (curlinesy, "datalinesy", DOULBET, -1, -1,
			"PlotAreaRecViewer", viewername, FALSE);
  }
  if ((dobj == NULL) && (dobj2 == NULL))
  {
    view->datalinesx = NULL;
    view->datalinesy = NULL;
  }
  if ((dobj != NULL) && (dobj2 == NULL))
  {
    view->datalinesx = dobj;
    viewerdata.push_back (curlinesx);
    if (dobj->type == INTT)
    {
      (*dobj)[0].GetValue (xcurvalint);
      xcurvaldbl = xcurvalint;
    }
    else
    {
      (*dobj)[0].GetValue (xcurvaldbl);
    }
    view->datalinesy = NULL;
    view->xlinesPointValue.push_back (xcurvaldbl);
  }
  if ((dobj == NULL) && (dobj2 != NULL))
  {
    view->datalinesy = dobj2;
    viewerdata.push_back (curlinesy);
    view->datalinesx = NULL;
    if (dobj2->type == INTT)
    {
      (*dobj2)[0].GetValue (ycurvalint);
      ycurvaldbl = ycurvalint;
    }
    else
    {
      (*dobj2)[0].GetValue (ycurvaldbl);
    }
    view->ylinesPointValue.push_back (ycurvaldbl);
  }

  if ((dobj != NULL) && (dobj2 != NULL))
  {
    view->datalinesx = dobj;
    view->datalinesy = dobj2;
    viewerdata.push_back (curlinesx);
    viewerdata.push_back (curlinesy);
    int linespointsnum;

    //pointsnum = (view->datax->rows) * (view->datax->cols);
    linespointsnum = (view->datalinesx->rows) * (view->datalinesx->cols);

    for (i = 0; i < linespointsnum; i++)
    {
      if (dobj->type == INTT)
      {
	(*dobj)[i].GetValue (xcurvalint);
	xcurvaldbl = xcurvalint;
      }
      else
      {
	(*dobj)[i].GetValue (xcurvaldbl);
      }
      if (dobj2->type == INTT)
      {
	(*dobj2)[i].GetValue (ycurvalint);
	ycurvaldbl = ycurvalint;
      }
      else
      {
	(*dobj2)[i].GetValue (ycurvaldbl);
      }
      view->xlinesPointValue.push_back (xcurvaldbl);
      view->ylinesPointValue.push_back (ycurvaldbl);
    }
    //view->linespointpsseq = new PointList();
    view->curlinespointsvaluenum = linespointsnum;
    view->linespointpsseq = new wxPoint[linespointsnum];
  }

  dobj2 =
    GetDynFromName (curpointsdel, "pointsdel", LOGICT, -1, -1,
		    "PlotAreaRecViewer", viewername, FALSE);
  int curpointsnum;
  bool curpdel;
  if (dobj2 != NULL)
  {
    viewerdata.push_back (curpointsdel);
    curpointsnum = dobj2->rows * dobj2->cols;
    for (i = 0; i < pointsnum; i++)
    {
      if (i < curpointsnum)
      {
	(*dobj2)[i].GetValue (curpdel);
	if (curpdel)
	  view->ispointsdel.push_back (true);
	else
	  view->ispointsdel.push_back (false);
      }
      else
	view->ispointsdel.push_back (false);
    }
    view->pointsdel = dobj2;
  }

  else
  {
    for (i = 0; i < pointsnum; i++)
      view->ispointsdel.push_back (false);
    view->pointsdel = NULL;
    error.Printf (CPC_STRING_VIEWER0114, "pointsdel", "PlotAreaRecViewer",
		  viewername.c_str ());
    if (verbose)
    {
      wxMessageBox (error);
    }

  }

  dobj =
    GetDynFromName (curcurvedel, "curvedel", LOGICT, -1, -1,
		    "PlotAreaRecViewer", viewername, FALSE);
  bool curveisdel;
  if (dobj != NULL)
  {
    viewerdata.push_back (curcurvedel);
    (*dobj)[0].GetValue (curveisdel);
    if (curveisdel)
      view->iscurvedeleted = true;
    else
      view->iscurvedeleted = false;
    view->curvedel = dobj;
  }

  else
  {
    view->iscurvedeleted = false;
    view->curvedel = NULL;
    error.Printf (CPC_STRING_VIEWER0114, "curvedel", "PlotAreaRecViewer",
		  viewername.c_str ());
    if (verbose)
    {
      wxMessageBox (error);
    }
  }

  dobj2 =
    GetDynFromName (currescale, "rescale", LOGICT, -1, -1,
		    "PlotAreaRecViewer", viewername, FALSE);
  bool rescalevalue;
  if (dobj2 != NULL)
  {
    viewerdata.push_back (currescale);
    (*dobj2)[0].GetValue (rescalevalue);
    if (rescalevalue)
      view->isrescale = true;
    else
      view->isrescale = false;
    view->rescale = dobj2;
  }
  else
  {
    view->isrescale = false;
    view->rescale = NULL;
    error.Printf (CPC_STRING_VIEWER0114, "rescale", "PlotAreaRecViewer",
		  viewername.c_str ());
    if (verbose)
    {
      wxMessageBox (error);
    }
  }

  dobj =
    GetDynFromName (curdrawsplashline, "splashline", LOGICT, -1, -1,
		    "PlotAreaRecViewer", viewername, FALSE);
  bool cursplashlineflag;
  if (dobj != NULL)
  {
    viewerdata.push_back (curdrawsplashline);
    (*dobj)[0].GetValue (cursplashlineflag);
    if (cursplashlineflag)
      view->curveissplashline = true;
    else
      view->curveissplashline = false;
    view->drawSplashline = dobj;
  }
  else
  {
    view->curveissplashline = false;
    view->drawSplashline = NULL;
    error.Printf (CPC_STRING_VIEWER0114, "splashline", "PlotAreaRecViewer",
		  viewername.c_str ());
    if (verbose)
    {
      wxMessageBox (error);
    }
  }

  view->hidecommand = GetBool ("hidecommand", 0);
  view->hideoutput = GetBool ("hideoutput", 0);

  thisApp->DynamicDataP->addViewer (viewername, (Viewer *) view);
  thisApp->DynamicDataP->setViewerData (viewerdata, viewername);

  return view;
}

bool PlotViewerXmlHandler::CanHandle (wxXmlNode * node)
{
  return IsOfClass (node, wxT ("PlotViewer"));
}

void
PlotDrawGlyph (wxDC & dc, long x, long y, long type, long size, long style)
{
  wxPoint
    pts[5];
  wxBrush
    brush;
  wxPen
    curpen,
    newpen;

  switch (type)
  {
  case CPC_SG_CIRCLE:
    if (size == 1)
      CarapaceDrawGlyph (dc, x, y, type, size * 2, style);
    else
      CarapaceDrawGlyph (dc, x, y, type, size, style);
    break;

  case CPC_SG_DIAMOND:
    CarapaceDrawGlyph (dc, x, y, type, size, style);
    break;

  case CPC_SG_SQUARE:
    CarapaceDrawGlyph (dc, x, y, type, size, style);
    break;

  case CPC_SG_TRIANGLE:
    CarapaceDrawGlyph (dc, x, y, type, size, style);
    break;

  case CPC_SG_CROSS:
    curpen = dc.GetPen ();
    newpen = dc.GetPen ();
    newpen.SetWidth (2);
    dc.SetPen (newpen);
    dc.DrawLine (x - size, y, x + size, y);
    dc.DrawLine (x, y - size, x, y + size);
    dc.SetPen (curpen);
    break;

  case CPC_SG_EXE:
    curpen = dc.GetPen ();
    newpen = dc.GetPen ();
    newpen.SetWidth (2);
    dc.SetPen (newpen);
    dc.DrawLine (x - size, y - size, x + size, y + size);
    dc.DrawLine (x + size, y - size, x - size, y + size);
    dc.SetPen (curpen);
    break;

  case CPC_SG_STAR:
    curpen = dc.GetPen ();
    newpen = dc.GetPen ();
    newpen.SetWidth (2);
    dc.SetPen (newpen);
    dc.DrawLine (x - size, y, x + size, y);
    dc.DrawLine (x, y - size, x, y + size);
    dc.DrawLine ((wxCoord) (x - size * .71), (wxCoord) (y - size * .71),
		 (wxCoord) (x + size * .71), (wxCoord) (y + size * .71));
    dc.DrawLine ((wxCoord) (x + size * .71), (wxCoord) (y - size * .71),
		 (wxCoord) (x - size * .71), (wxCoord) (y + size * .71));
    dc.SetPen (curpen);
    break;

  case CPC_SG_SYMBOL:
  case CPC_SG_DINGBAT:
    ;
  }
}

// calculate pretty value for the plot
double
R_pretty0 (double *lo, double *up, int *ndiv, int min_n,
	   double shrink_sml, double high_u_fact[],
	   int eps_correction, int return_bounds)
{

#define rounding_eps 1e-7

#define h  high_u_fact[0]
#define h5 high_u_fact[1]

  double
    dx,
    cell,
    unit,
    base,
    U;
  double
    ns,
    nu;
  int
    k;
  bool
    i_small;

  dx = *up - *lo;
  if (dx == 0 && *up == 0)
  {				/*  up == lo == 0  */
    cell = 1;
    i_small = true;
  }
  else
  {
    cell = fmax2 (fabs (*lo), fabs (*up));
    /* U = upper bound on cell/unit */
    U = 1 + (h5 >= 1.5 * h + .5) ? 1 / (1 + h) : 1.5 / (1 + h5);
    /* added times 3, as several calculations here */
    i_small = dx < cell * U * imax2 (1, *ndiv) * DBL_EPSILON * 3;

  }

  /*OLD: cell = FLT_EPSILON+ dx / *ndiv; FLT_EPSILON = 1.192e-07 */
  if (i_small)
  {
    if (cell > 10)
      cell = 9 + cell / 10;
    cell *= shrink_sml;
    if (min_n > 1)
      cell /= min_n;
  }
  else
  {
    cell = dx;
    if (*ndiv > 1)
      cell /= *ndiv;
  }

  if (cell < 20 * DBL_MIN)
  {
    // cout<<"Internal(pretty()): very small range.. corrected";
    cell = 20 * DBL_MIN;
  }
  else if (cell * 10 > DBL_MAX)
  {
    // cout << "Internal(pretty()): very large range.. corrected";
    cell = .1 * DBL_MAX;
  }
  base = pow (10., floor (log10 (cell)));	/* base <= cell < 10*base */

  /* unit : from { 1,2,5,10 } * base
   *   such that |u - cell| is small,
   * favoring larger (if h > 1, else smaller)  u  values;
   * favor '5' more than '2'  if h5 > h  (default h5 = .5 + 1.5 h) */
  unit = base;
  if ((U = 2 * base) - cell < h * (cell - unit))
  {
    unit = U;
    if ((U = 5 * base) - cell < h5 * (cell - unit))
    {
      unit = U;
      if ((U = 10 * base) - cell < h * (cell - unit))
	unit = U;
    }
  }
  /* Result: c := cell,  u := unit,  b := base
   *  c in [  1,            (2+ h) /(1+h) ] b ==> u=  b
   *  c in ( (2+ h)/(1+h),  (5+2h5)/(1+h5)] b ==> u= 2b
   *  c in ( (5+2h)/(1+h), (10+5h) /(1+h) ] b ==> u= 5b
   *  c in ((10+5h)/(1+h),             10 ) b ==> u=10b
   *
   *  ===>    2/5 *(2+h)/(1+h)  <=  c/u  <=  (2+h)/(1+h)      */

  ns = floor (*lo / unit + rounding_eps);
  nu = ceil (*up / unit - rounding_eps);

  // cout<<"pretty(lo="<<*lo<<",up="<<*up<<",ndiv="<<*ndiv<<",min_n="<<min_n<<",shrink="<<shrink_sml<<",high_u=(" << h<<"," << h5<<"),"<<"eps="<<eps_correction<<")\n\t dx="<<dx<<"; is.small:"<<(int)i_small<<"==> cell="<<cell<<"; unit="<<unit<<"\n";

  if (eps_correction && (eps_correction > 1 || !i_small))
  {
    if (*lo)
      *lo *= (1 - DBL_EPSILON);
    else
      *lo = -DBL_MIN;
    if (*up)
      *up *= (1 + DBL_EPSILON);
    else
      *up = +DBL_MIN;
  }


  while (ns * unit > *lo + rounding_eps * unit)
    ns--;


  while (nu * unit < *up - rounding_eps * unit)
    nu++;

  k = .5 + nu - ns;
  if (k < min_n)
  {
    /* ensure that  nu - ns  == min_n */

    k = min_n - k;
    if (ns >= 0.)
    {
      nu += k / 2;
      ns -= k / 2 + k % 2;	/* ==> nu-ns = old(nu-ns) + min_n -k = min_n */
    }
    else
    {
      ns -= k / 2;
      nu += k / 2 + k % 2;
    }
    *ndiv = min_n;
  }
  else
  {
    *ndiv = k;
  }
  if (return_bounds)
  {				/* if()'s to ensure that result covers original range */
    if (ns * unit < *lo)
      *lo = ns * unit;
    if (nu * unit > *up)
      *up = nu * unit;
  }
  else
  {
    *lo = ns;
    *up = nu;
  }

  // cout<<"\t ns="<<ns<<"==> lo="<<*lo<<"\n";
  //  cout<<"\t nu="<<nu<<" ==> up="<<*up<<"==> ndiv ="<<*ndiv<<"\n";

  return unit;
#undef h
#undef h5
}

void
R_pretty (double *lo, double *up, int *ndiv, int *min_n,
	  double *shrink_sml, double *high_u_fact, int *eps_correction)
{
  R_pretty0 (lo, up, ndiv,
	     *min_n, *shrink_sml, high_u_fact, *eps_correction, 1);
}

double
fmax2 (double db1, double db2)
{
  if (db1 >= db2)
    return db1;
  else
    return db2;
}

int
imax2 (int int1, int int2)
{
  if (int1 >= int2)
    return int1;
  else
    return int2;
}

int
getlabeloffset (wxString label, int curfontsize)
{
  int
    strlen;
  strlen = label.Length ();
  return ((strlen * curfontsize) / 3);
}

int
getlabelYoffset (int curfontsize)
{
  return ((curfontsize * 2) / 3);
}

bool PlotAreaRecViewer::delpoint (wxDC & dc, int pointindex)
{
  int
    totalpointsnum;
  totalpointsnum = xoffset.size ();
  if ((pointindex < 0) || (pointindex > totalpointsnum))
    return false;
  wxPen
    curpen,
    newpen;
  wxBrush
    curbrush,
    newbrush;
  int
    curstyle = 0;
  curpen = dc.GetPen ();
  newpen = dc.GetPen ();
  newpen.SetColour (wxString ("WHITE"));
  dc.SetPen (newpen);
  curbrush = dc.GetBrush ();
  newbrush = curbrush;
  newbrush.SetColour (wxString ("WHITE"));
  dc.SetBrush (newbrush);
  curstyle = curstyle | CPC_SG_FILLED;
  PlotDrawGlyph (dc, xoffset[pointindex], yoffset[pointindex],
		 ptype[pointindex], pointsize[pointindex], curstyle);
  dc.SetPen (curpen);
  dc.SetBrush (curbrush);
  return true;
}

void
PlotAreaRecViewer::delcurve (wxDC & dc)
{
  int currx, curry;
  int w, h;
  int leftmargin, rightmargin, topmargin, bottommargin;
  double startx, starty, endx, endy;
  int xaxisbeginblank, yaxisbeginblank;
  wxPen curpen, newpen;
  int linespointsnum;
  int curlinewidth, curlinestyle;
  int curpenwidth, curpenstyle;

  curpen = dc.GetPen ();
  curpenwidth = curpen.GetWidth ();
  curpenstyle = curpen.GetStyle ();
  GetClientSize (&w, &h);

  startx = xaxismin;
  starty = yaxismin;
  endx = xaxismax;
  endy = yaxismax;
  xaxisbeginblank = 5;
  yaxisbeginblank = 5;
  leftmargin = plotleftmargin;
  rightmargin = plotrightmargin;
  topmargin = plottopmargin;
  bottommargin = plotbottommargin;
  int plotwidth = w - leftmargin - rightmargin;
  int plotheight = h - topmargin - bottommargin;

  if ((datalinesx != NULL) && (datalinesy == NULL))
  {
    if (lineswidth < 0)
      curlinewidth = curpenwidth;
    else
      curlinewidth = lineswidth;
    if (curlinestyle == 0)
      curlinestyle = curpenstyle;
    else
      curlinestyle = linesstyle;
    wxPen newpen (linescolor, curlinewidth, curlinestyle);
    newpen.SetColour (wxString ("WHITE"));
    dc.SetPen (newpen);
    currx =
      leftmargin + xaxisbeginblank +
      ((xlinesPointValue[0] - startx) / (endx - startx)) * (plotwidth -
							    xaxisbeginblank);
    if ((currx >= leftmargin + xaxisbeginblank)
	&& (currx <= plotwidth + leftmargin))
      dc.DrawLine (currx, h - bottommargin, currx, topmargin);	//               
    dc.SetPen (curpen);
  }

  if ((datalinesx == NULL) && (datalinesy != NULL))
  {
    if (lineswidth < 0)
      curlinewidth = curpenwidth;
    else
      curlinewidth = lineswidth;
    if (curlinestyle == 0)
      curlinestyle = curpenstyle;
    else
      curlinestyle = linesstyle;
    wxPen newpen (linescolor, curlinewidth, curlinestyle);
    newpen.SetColour (wxString ("WHITE"));
    dc.SetPen (newpen);
    curry =
      h - bottommargin - yaxisbeginblank -
      ((ylinesPointValue[0] - starty) / (endy - starty)) * (plotheight -
							    yaxisbeginblank);
    if ((curry >= topmargin) && (curry <= h - bottommargin - yaxisbeginblank))
      dc.DrawLine (leftmargin, curry, plotwidth + leftmargin, curry);
    dc.SetPen (curpen);
  }

  if ((datalinesx != NULL) && (datalinesy != NULL))
  {
    //linespointsnum = xlinesPointValue.size();     
    linespointsnum = linespointpsseqnum;
    if (lineswidth < 0)
      curlinewidth = curpenwidth;
    else
      curlinewidth = lineswidth;
    if (curlinestyle == 0)
      curlinestyle = curpenstyle;
    else
      curlinestyle = linesstyle;
    wxPen newpen (linescolor, curlinewidth, curlinestyle);
    newpen.SetColour (wxString ("WHITE"));
    dc.SetPen (newpen);
    dc.DrawSpline (linespointsnum, linespointpsseq);
    dc.SetPen (curpen);
  }
}

bool PlotAreaRecViewer::UpdateColIndex ()
{
  int
    i,
    xcurColIndex,
    ycurColIndex;
  int
    pointsnum,
    xcurvalint,
    ycurvalint;
  double
    xcurvaldbl,
    ycurvaldbl;
  if (dataColumnSel->rows * dataColumnSel->cols > 1)
  {
    (*dataColumnSel)[0].GetValue (xcurColIndex);
    (*dataColumnSel)[1].GetValue (ycurColIndex);
  }
  else
  {
    xcurColIndex = xColIndex;
    ycurColIndex = yColIndex;
  }
  if(doxyupdate)
  {
    int xcurvalint, ycurvalint;
    double xcurvaldbl, ycurvaldbl;
    xPointValue.clear ();
    yPointValue.clear ();
    if (xcurColIndex * ycurColIndex != 0)
    {
      pointsnum = datax->rows;
    }
    else
    {
      pointsnum = datax->rows * datax->cols;
    }
    if ((xcurColIndex == 0) && (ycurColIndex == 0))
    {
      for (i = 0; i < pointsnum; i++)
      {
        if (datax->type == INTT)
        {
  	(*datax)[i].GetValue (xcurvalint);
  	xcurvaldbl = xcurvalint;
  	xPointValue.push_back (xcurvaldbl);
        }
        else
        {
  	(*datax)[i].GetValue (xcurvaldbl);
  	xPointValue.push_back (xcurvaldbl);
        }
        if (datay->type == INTT)
        {
  	(*datay)[i].GetValue (ycurvalint);
  	ycurvaldbl = ycurvalint;
  	yPointValue.push_back (ycurvaldbl);
        }
        else
        {
  	(*datay)[i].GetValue (ycurvaldbl);
  	yPointValue.push_back (ycurvaldbl);
        }
      }
    }
    else
    {
      for (i = 0; i < pointsnum; i++)
      {
        if (datax->type == INTT)
        {
  	(*datax)[(xcurColIndex - 1) * pointsnum + i].GetValue (xcurvalint);
  	xcurvaldbl = xcurvalint;
  	xPointValue.push_back (xcurvaldbl);
        }
        else
        {
  	(*datax)[(xcurColIndex - 1) * pointsnum + i].GetValue (xcurvaldbl);
  	xPointValue.push_back (xcurvaldbl);
        }
        if (datay->type == INTT)
        {
  	(*datay)[(ycurColIndex - 1) * pointsnum +
  		 i].GetValue (ycurvalint);
  	ycurvaldbl = ycurvalint;
  	yPointValue.push_back (ycurvaldbl);
        }
        else
        {
  	(*datay)[(ycurColIndex - 1) * pointsnum +
  		 i].GetValue (ycurvaldbl);
  	yPointValue.push_back (ycurvaldbl);
        }
      }
    }
  }
  if ((xcurColIndex == xColIndex) && (ycurColIndex == yColIndex))
    return false;
  if ((xcurColIndex * ycurColIndex == 0)
      && (xcurColIndex + ycurColIndex != 0))
  {
    (*dataColumnSel)[0].SetValue (xColIndex);
    (*dataColumnSel)[1].SetValue (yColIndex);
    return false;
  }
  if ((xcurColIndex < 0) || (xcurColIndex > datax->cols))
  {
    (*dataColumnSel)[0].SetValue (xColIndex);
    return false;
  }
  if ((ycurColIndex < 0) || (ycurColIndex > datay->cols))
  {
    (*dataColumnSel)[1].SetValue (yColIndex);
    return false;
  }
  if (xcurColIndex * ycurColIndex != 0)
  {
    pointsnum = datax->rows;
    if (xcurColIndex != xColIndex)
    {
      for (i = 0; i < pointsnum; i++)
      {
	if (datax->type == INTT)
	{
	  (*datax)[(xcurColIndex - 1) * pointsnum + i].GetValue (xcurvalint);
	  xcurvaldbl = xcurvalint;
	  xPointValue[i] = xcurvaldbl;
	}
	else
	{
	  (*datax)[(xcurColIndex - 1) * pointsnum + i].GetValue (xcurvaldbl);
	  xPointValue[i] = xcurvaldbl;
	}
      }
      xColIndex = xcurColIndex;
    }
    if (ycurColIndex != yColIndex)
    {
      for (i = 0; i < pointsnum; i++)
      {
	if (datay->type == INTT)
	{
	  (*datay)[(ycurColIndex - 1) * pointsnum + i].GetValue (ycurvalint);
	  ycurvaldbl = ycurvalint;
	  yPointValue[i] = ycurvaldbl;
	}
	else
	{
	  (*datay)[(ycurColIndex - 1) * pointsnum + i].GetValue (ycurvaldbl);
	  yPointValue[i] = ycurvaldbl;
	}
      }
      yColIndex = ycurColIndex;
    }
    return true;
  }
  else
  {
    pointsnum = datax->rows * datax->cols;
    if (pointsnum != datay->rows * datay->cols)
      return false;
    xPointValue.clear ();
    yPointValue.clear ();
    xoffset.clear ();
    yoffset.clear ();
    for (i = 0; i < pointsnum; i++)
    {
      if (datax->type == INTT)
      {
	(*datax)[i].GetValue (xcurvalint);
	xcurvaldbl = xcurvalint;
	xPointValue.push_back (xcurvaldbl);
      }
      else
      {
	(*datax)[i].GetValue (xcurvaldbl);
	xPointValue.push_back (xcurvaldbl);
      }
      if (datay->type == INTT)
      {
	(*datay)[i].GetValue (ycurvalint);
	ycurvaldbl = ycurvalint;
	yPointValue.push_back (ycurvaldbl);
      }
      else
      {
	(*datay)[i].GetValue (ycurvaldbl);
	yPointValue.push_back (ycurvaldbl);
      }
    }
    xColIndex = 0;
    yColIndex = 0;
    return true;
  }

}

bool PlotAreaRecViewer::UpdateLinesSplashFlag ()
{
  bool
    curlinesplashflag;
  if (drawSplashline != NULL)
  {
    (*drawSplashline)[0].GetValue (curlinesplashflag);
    if (curlinesplashflag != curveissplashline)
    {
      curveissplashline = curlinesplashflag;
      return true;
    }
    else
      return false;
  }
  return false;
}

bool PlotAreaRecViewer::UpdateLinesPoints ()
{
  int
    i,
    curlinespointsnum,
    xcurvalint,
    ycurvalint;
  double
    xcurvaldbl,
    ycurvaldbl;
  bool
    linesupdateflag;

  linesupdateflag = false;
  if ((datalinesx != NULL) && (datalinesy == NULL))
  {
    if (datalinesx->type == INTT)
    {
      (*datalinesx)[0].GetValue (xcurvalint);
      xcurvaldbl = xcurvalint;
      if (xcurvaldbl != xlinesPointValue[0])
      {
	xlinesPointValue[0] = xcurvaldbl;
	linesupdateflag = true;
      }
    }
    else
    {
      (*datalinesx)[0].GetValue (xcurvaldbl);
      if (xcurvaldbl != xlinesPointValue[0])
      {
	xlinesPointValue[0] = xcurvaldbl;
	linesupdateflag = true;
      }
    }
    if (linesupdateflag)
      return true;
    else
      return false;
  }
  if ((datalinesx == NULL) && (datalinesy != NULL))
  {
    if (datalinesy->type == INTT)
    {
      (*datalinesy)[0].GetValue (ycurvalint);
      ycurvaldbl = ycurvalint;
      if (ycurvaldbl != ylinesPointValue[0])
      {
	ylinesPointValue[0] = ycurvaldbl;
	linesupdateflag = true;
      }
    }
    else
    {
      (*datalinesy)[0].GetValue (ycurvaldbl);
      if (ycurvaldbl != ylinesPointValue[0])
      {
	ylinesPointValue[0] = ycurvaldbl;
	linesupdateflag = true;
      }
    }
    if (linesupdateflag)
      return true;
    else
      return false;
  }
  if ((datalinesx != NULL) && (datalinesy != NULL))
  {
    curlinespointsnum = datalinesx->rows * datalinesx->cols;
    if (curlinespointsnum != datalinesy->rows * datalinesy->cols)
      return false;
    if (curlinespointsnum != curlinespointsvaluenum)
    {
      xlinesPointValue.clear ();
      ylinesPointValue.clear ();
      delete[]linespointpsseq;
      for (i = 0; i < curlinespointsnum; i++)
      {
	if (datalinesx->type == INTT)
	{
	  (*datalinesx)[i].GetValue (xcurvalint);
	  xcurvaldbl = xcurvalint;
	  xlinesPointValue.push_back (xcurvaldbl);
	}
	else
	{
	  (*datalinesx)[i].GetValue (xcurvaldbl);
	  xlinesPointValue.push_back (xcurvaldbl);
	}
	if (datalinesy->type == INTT)
	{
	  (*datalinesy)[i].GetValue (ycurvalint);
	  ycurvaldbl = ycurvalint;
	  ylinesPointValue.push_back (ycurvaldbl);
	}
	else
	{
	  (*datalinesy)[i].GetValue (ycurvaldbl);
	  ylinesPointValue.push_back (ycurvaldbl);
	}
      }
      curlinespointsvaluenum = curlinespointsnum;
      linespointpsseq = new wxPoint[curlinespointsnum];
      return true;
    }
    else
    {
      for (i = 0; i < curlinespointsnum; i++)
      {
	if (datalinesx->type == INTT)
	{
	  (*datalinesx)[i].GetValue (xcurvalint);
	  xcurvaldbl = xcurvalint;
	  if (xcurvaldbl != xlinesPointValue[i])
	  {
	    xlinesPointValue[i] = xcurvaldbl;
	    linesupdateflag = true;
	  }
	}
	else
	{
	  (*datalinesx)[i].GetValue (xcurvaldbl);
	  if (xcurvaldbl != xlinesPointValue[i])
	  {
	    xlinesPointValue[i] = xcurvaldbl;
	    linesupdateflag = true;
	  }
	}
	if (datalinesy->type == INTT)
	{
	  (*datalinesy)[i].GetValue (ycurvalint);
	  ycurvaldbl = ycurvalint;
	  if (ycurvaldbl != ylinesPointValue[i])
	  {
	    ylinesPointValue[i] = ycurvaldbl;
	    linesupdateflag = true;
	  }
	}
	else
	{
	  (*datalinesy)[i].GetValue (ycurvaldbl);
	  if (ycurvaldbl != ylinesPointValue[i])
	  {
	    ylinesPointValue[i] = ycurvaldbl;
	    linesupdateflag = true;
	  }
	}
      }
      if (linesupdateflag)
	return true;
      else
	return false;

    }
  }
}

bool PlotAreaRecViewer::UpdateRange ()
{


  int
    xstartint,
    xendint,
    ystartint,
    yendint,
    pointsnum,
    i;
  double
    xtartdbl,
    xenddbl,
    tmpstartx,
    tmpendx,
    ytartdbl,
    yenddbl,
    tmpstarty,
    tmpendy;
  bool
    updaterangexflag,
    updaterangeyflag;
  updaterangexflag = false;
  updaterangeyflag = false;
  vector < double >
    dvalsx,
    dvalsy;
  int
    n = plotxticknum;
  int
    minnum = 1;
  double
    shrinknum = 0.75;
  double
    high_u_bias[2];
  int
    epscor = 0;
  bool
    currescalevalue;
  bool
    isallpointsdel;

  int
    curxticknum,
    curyticknum;
  if (ticknum != NULL)
  {
    if (ticknum->rows * ticknum->cols > 1)
    {
      (*ticknum)[0].GetValue (curxticknum);
      (*ticknum)[1].GetValue (curyticknum);
    }
    else
    {
      (*ticknum)[0].GetValue (curxticknum);
      curyticknum = 5;
    }
    if ((curxticknum > 0) && (plotxticknum != curxticknum))
    {
      plotxticknum = curxticknum;
      updaterangexflag = true;
    }
    if ((curyticknum > 0) && (plotyticknum != curyticknum))
    {
      plotyticknum = curyticknum;
      updaterangeyflag = true;
    }
  }

  if (rescale != NULL)
    (*rescale)[0].GetValue (currescalevalue);
  else
    currescalevalue = false;
  if (currescalevalue != isrescale)
  {
    isrescale = currescalevalue;
  }

  //Check  whether datarangex was updated
  if ((datarangex != NULL) && (!isrescale)
      && (datarangex->rows * datarangex->cols > 1))
  {
    if (datarangex->type == DOULBET)
    {
      (*datarangex)[0].GetValue (xtartdbl);	// low range
      (*datarangex)[1].GetValue (xenddbl);	// high range
    }
    else
    {
      (*datarangex)[0].GetValue (xstartint);	// low range
      (*datarangex)[1].GetValue (xendint);	// high range
      xtartdbl = xstartint;
      xenddbl = xendint;
    }
    if (((xtartdbl != xstart) || (xenddbl != xend)) && (xtartdbl < xenddbl))
    {
      updaterangexflag = true;
      xstart = xtartdbl;
      xend = xenddbl;
    }
  }
  else
  {

    isallpointsdel = false;
    pointsnum = xPointValue.size ();
    dvalsx = xPointValue;
    i = 0;
    while ((ispointsdel[i]) && (i < pointsnum))
      i++;
    if (i < pointsnum)
    {
      tmpstartx = dvalsx[i];
      tmpendx = dvalsx[i];
    }
    else
    {
      tmpstartx = dvalsx[0];
      tmpendx = dvalsx[0];
      isallpointsdel = true;
    }
//        tmpstartx = dvalsx[0];
//        tmpendx = dvalsx[1];
    for (i = 0; i < pointsnum; i++)
    {
      if (!isallpointsdel)
      {
	if (!ispointsdel[i])
	{
	  if (tmpstartx > dvalsx[i])
	    tmpstartx = dvalsx[i];
	  if (tmpendx < dvalsx[i])
	    tmpendx = dvalsx[i];
	}
      }
      else
      {
	if (tmpstartx > dvalsx[i])
	  tmpstartx = dvalsx[i];
	if (tmpendx < dvalsx[i])
	  tmpendx = dvalsx[i];
      }
    }

    if ((tmpstartx != xstart) || (tmpendx != xend))
    {
      updaterangexflag = true;
      xstart = tmpstartx;
      xend = tmpendx;
    }

  }
  // Modify the tick parameters if datarangex is updated
  if (updaterangexflag)
  {
    xaxismin = xstart;
    xaxismax = xend;
    n = plotxticknum;
    minnum = 1;
    shrinknum = 0.75;
    high_u_bias[0] = 1.5;
    high_u_bias[1] = 0.5 + 1.5 * high_u_bias[0];
    epscor = 0;
    R_pretty (&xaxismin, &xaxismax, &n, &minnum, &shrinknum, high_u_bias,
	      &epscor);
    xscalenum = n;
  }

  //Check  whether datarangey was updated
  if ((datarangey != NULL) && (!isrescale)
      && (datarangey->rows * datarangey->cols > 1))
  {
    if (datarangey->type == DOULBET)
    {
      (*datarangey)[0].GetValue (ytartdbl);	// low range
      (*datarangey)[1].GetValue (yenddbl);	// high range
    }
    else
    {
      (*datarangey)[0].GetValue (ystartint);	// low range
      (*datarangey)[1].GetValue (yendint);	// high range
      ytartdbl = ystartint;
      yenddbl = yendint;
    }
    if (((ytartdbl != ystart) || (yenddbl != yend)) && (ytartdbl < yenddbl))
    {
      updaterangeyflag = true;
      ystart = ytartdbl;
      yend = yenddbl;
    }
  }
  else
  {
    isallpointsdel = false;
    dvalsy = yPointValue;
    pointsnum = yPointValue.size ();
    i = 0;
    while ((ispointsdel[i]) && (i < pointsnum))
      i++;
    if (i < pointsnum)
    {
      tmpstarty = dvalsy[i];
      tmpendy = dvalsy[i];
    }
    else
    {
      isallpointsdel = true;
      tmpstarty = dvalsy[0];
      tmpendy = dvalsy[0];
    }
    for (i = 0; i < pointsnum; i++)
    {
      if (!isallpointsdel)
      {
	if (!ispointsdel[i])
	{
	  if (tmpstarty > dvalsy[i])
	    tmpstarty = dvalsy[i];
	  if (tmpendy < dvalsy[i])
	    tmpendy = dvalsy[i];
	}
      }
      else
      {
	if (tmpstarty > dvalsy[i])
	  tmpstarty = dvalsy[i];
	if (tmpendy < dvalsy[i])
	  tmpendy = dvalsy[i];
      }

    }
    if ((tmpstarty != ystart) || (tmpendy != yend))
    {
      updaterangeyflag = true;
      ystart = tmpstarty;
      yend = tmpendy;
    }
  }
  // Modify the tick parameters if datarangex is updated
  if (updaterangeyflag)
  {
    yaxismin = ystart;
    yaxismax = yend;
    n = plotyticknum;
    minnum = 1;
    shrinknum = 0.75;
    high_u_bias[0] = 1.5;
    high_u_bias[1] = 0.5 + 1.5 * high_u_bias[0];
    epscor = 0;
    R_pretty (&yaxismin, &yaxismax, &n, &minnum, &shrinknum, high_u_bias,
	      &epscor);
    yscalenum = n;
  }

  if (updaterangexflag || updaterangeyflag)
    return true;
  else
    return false;
}

bool PlotAreaRecViewer::UpdatePlotParameters ()
{
  int
    curleftmargin,
    currightmargin,
    curtopmargin,
    curbottommargin,
    curplotnamemargin;
  wxString
    curplotname;
  bool
    updateplotflag;

  updateplotflag = false;
  curleftmargin = plotleftmargin;
  currightmargin = plotrightmargin;
  curtopmargin = plottopmargin;
  curbottommargin = plotbottommargin;
  curplotnamemargin = plotnamemargin;
  curplotname = plotnamelabel;
  if ((plotparameters != NULL)
      && (plotparameters->rows * plotparameters->cols > 6))
  {
    (*plotparameters)[2].GetValue (curleftmargin);
    (*plotparameters)[3].GetValue (currightmargin);
    (*plotparameters)[4].GetValue (curtopmargin);
    (*plotparameters)[5].GetValue (curbottommargin);
    (*plotparameters)[6].GetValue (curplotnamemargin);
  }
  if (plotname != NULL)
    (*plotname)[0].GetValue (curplotname);
  if (plotleftmargin != curleftmargin)
  {
    plotleftmargin = curleftmargin;
    updateplotflag = true;
  }
  if (plotrightmargin != currightmargin)
  {
    plotrightmargin = currightmargin;
    updateplotflag = true;
  }
  if (plottopmargin != curtopmargin)
  {
    plottopmargin = curtopmargin;
    updateplotflag = true;
  }
  if (plotbottommargin != curbottommargin)
  {
    plotbottommargin = curbottommargin;
    updateplotflag = true;
  }

  if (plotnamemargin != curplotnamemargin)
  {
    plotnamemargin = curplotnamemargin;
    updateplotflag = true;
  }
  if (plotnamelabel != curplotname)
  {
    plotnamelabel = curplotname;
    updateplotflag = true;
  }
  if (updateplotflag)
    return true;
  else
    return false;
}

bool PlotAreaRecViewer::UpdateTickParameters ()
{
  int
    xcurticklen,
    ycurticklen,
    xcurtickloc,
    ycurtickloc;
  bool
    updatetickflag,
    isplotxcurticklabel,
    isplotycurticklabel;
  updatetickflag = false;
  int
    i;
  if (ticklen != NULL)
  {
    if (ticklen->rows * ticklen->cols > 1)
    {
      (*ticklen)[0].GetValue (xcurticklen);
      (*ticklen)[1].GetValue (ycurticklen);
    }
    else
    {
      (*ticklen)[0].GetValue (xcurticklen);
      ycurticklen = 5;
    }
    if ((xcurticklen > -1) && (ycurticklen > -1)
	&& ((xticklen != xcurticklen) || (yticklen != ycurticklen)))
    {
      xticklen = xcurticklen;
      yticklen = ycurticklen;
      updatetickflag = true;
    }
  }



  if (ticklocation != NULL)
  {
    if (ticklocation->rows * ticklocation->cols > 1)
    {
      (*ticklocation)[0].GetValue (xcurtickloc);
      (*ticklocation)[1].GetValue (ycurtickloc);
    }
    else
    {
      (*ticklocation)[0].GetValue (xcurtickloc);
      ycurtickloc = 0;
    }
    if ((xtickloc != xcurtickloc) || (ytickloc != ycurtickloc))
    {
      xtickloc = xcurtickloc;
      ytickloc = ycurtickloc;
      updatetickflag = true;
    }
  }

  if (usecustomticklabels != NULL)
  {
    if (usecustomticklabels->rows * usecustomticklabels->cols > 1)
    {
      (*usecustomticklabels)[0].GetValue (isplotxcurticklabel);
      (*usecustomticklabels)[1].GetValue (isplotycurticklabel);
    }
    else
    {
      (*usecustomticklabels)[0].GetValue (isplotxcurticklabel);
      isplotycurticklabel = false;
    }
    if ((isplotxticklabel != isplotxcurticklabel)
	|| (isplotyticklabel != isplotycurticklabel))
    {
      isplotxticklabel = isplotxcurticklabel;
      isplotyticklabel = isplotycurticklabel;
      updatetickflag = true;
    }
  }
  int
    curxlabelnum,
    curylabelnum;
  wxString
    curxlabel,
    curylabel;
  if (isplotxticklabel)
  {
    if (xticklabel != NULL)
    {
      curxlabelnum = xticklabel->rows * xticklabel->cols;
      if (curxlabelnum == plotxticklabels.size ())
      {
	for (i = 0; i < curxlabelnum; i++)
	{
	  (*xticklabel)[i].GetValue (curxlabel);
	  if (curxlabel != plotxticklabels[i])
	  {
	    plotxticklabels[i] = curxlabel;
	    updatetickflag = true;
	  }
	}
      }
      else
      {
	plotxticklabels.clear ();
	for (i = 0; i < curxlabelnum; i++)
	{
	  (*xticklabel)[i].GetValue (curxlabel);
	  plotxticklabels.push_back (curxlabel);
	}
	updatetickflag = true;
      }
    }
  }

  if (isplotyticklabel)
  {
    if (yticklabel != NULL)
    {
      curylabelnum = yticklabel->rows * yticklabel->cols;
      if (curylabelnum == plotyticklabels.size ())
      {
	for (i = 0; i < curylabelnum; i++)
	{
	  (*yticklabel)[i].GetValue (curylabel);
	  if (curylabel != plotyticklabels[i])
	  {
	    plotyticklabels[i] = curylabel;
	    updatetickflag = true;
	  }
	}
      }
      else
      {
	plotyticklabels.clear ();
	for (i = 0; i < curylabelnum; i++)
	{
	  (*yticklabel)[i].GetValue (curylabel);
	  plotyticklabels.push_back (curylabel);
	}
	updatetickflag = true;
      }
    }
  }

  bool
    isaxisxcurlabelblank,
    isaxisycurlabelblank;
  if (isaxislabelblank != NULL)
  {
    if (isaxislabelblank->rows * isaxislabelblank->cols > 1)
    {
      (*isaxislabelblank)[0].GetValue (isaxisxcurlabelblank);
      (*isaxislabelblank)[1].GetValue (isaxisycurlabelblank);
    }
    else
    {
      (*isaxislabelblank)[0].GetValue (isaxisxcurlabelblank);
      isaxisycurlabelblank = false;
    }
    if ((isaxisxlabelblank != isaxisxcurlabelblank)
	|| (isaxisylabelblank != isaxisycurlabelblank))
    {
      isaxisxlabelblank = isaxisxcurlabelblank;
      isaxisylabelblank = isaxisycurlabelblank;
      updatetickflag = true;
    }
  }
  if (updatetickflag)
    return true;
  else
    return false;

}

int
GetColorIndexfromName (wxString colorname)
{

  if (colorname == wxString ("RED"))
    return (53);
  if (colorname == wxString ("BLUE"))
    return (2);
  if (colorname == wxString ("YELLOW"))
    return (68);
  if (colorname == wxString ("GREEN"))
    return (22);
  if (colorname == wxString ("CORAL"))
    return (6);
  if (colorname == wxString ("BLACK"))
    return (1);
  if (colorname == wxString ("GREY"))
    return (21);
  if (colorname == wxString ("WHITE"))
    return (67);
  return (-1);
}

wxString
GetColorNamefromIndex (int colorindex)
{
  wxString
    curColorName;
  switch (colorindex)
  {
  case 53:
    curColorName = wxString ("RED");
    break;
  case 2:
    curColorName = wxString ("BLUE");
    break;
  case 68:
    curColorName = wxString ("YELLOW");
    break;
  case 22:
    curColorName = wxString ("GREEN");
    break;
  case 6:
    curColorName = wxString ("CORAL");
    break;
  case 1:
    curColorName = wxString ("BLACK");
    break;
  case 21:
    curColorName = wxString ("GREY");
    break;
  case 67:
    curColorName = wxString ("WHITE");
    break;
  default:
    curColorName = wxString ("");
  }
  return (curColorName);
}

void
PlotAreaRecViewer::DrawSplashLine (wxDC & dc, int splashlinepointsnum,
				   wxPoint * splashlinespointseq)
{
  dc.DrawLines (splashlinepointsnum, splashlinespointseq, 0, 0);
}
