#include "cpcplatform.h"

#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif
#include "wx/print.h"
#include "wx/printdlg.h"
#include "wx/generic/dcpsg.h"
#include "wx/clipbrd.h"


#include "carapace.h"
#include "viewers.h"

#undef wxIMPLEMENT_CLASS_COMMON2
#define wxIMPLEMENT_CLASS_COMMON2(name, basename1, basename2, func)           \
    wxIMPLEMENT_CLASS_COMMON(name, basename1, &basename2::ms_classInfo,func)



BEGIN_EVENT_TABLE(ShowFileViewer,wxTextCtrl)
EVT_RIGHT_DOWN(ShowFileViewer::DoPopup)
EVT_MENU_RANGE(1,200000000,ShowFileViewer::CatchMenu)
END_EVENT_TABLE()

ShowFileViewer::ShowFileViewer()
{
	actionstring = "";
	action = NULL;
	hidecommand = FALSE;
	hideoutput = FALSE;
	fontSize = NULL;
	fontFamily = NULL;
	doCut = NULL;
	doCopy = NULL;
	doPaste = NULL;
	doSave = NULL;
	doClose = NULL;
}

ShowFileViewer::~ShowFileViewer()
{     
	delete popupmenu;
}

void ShowFileViewer::DoClose(void)
{
	if(IsEditable() && IsModified())
	{
		int answer = wxMessageBox("Save editor window?","Save?",
			wxYES_NO | wxCENTRE);
		if(answer == wxYES)
        	{
			SaveFile(filename);
		}
	}

}

void ShowFileViewer::DoSave(void)
{
	if(IsEditable() && IsModified())
	{
		SaveFile(filename);
	}
}

void ShowFileViewer::DoSaveAs(void)
{
	wxString fn = wxFileSelector("Select new file name",
		wxPathOnly(filename),"","","*.*",wxOPEN | wxSAVE);
	if(fn.Length())
	{
		// filename = fn;
		// SaveFile(fn);
		wxMessageBox(fn);
	}
}

bool ShowFileViewer::draw(DataObject& dobj)
{
	int newfontsize;
	wxString newfontfamily;
	ProcessAction();

	wxTextAttr  tAttr = GetDefaultStyle();
	if(fontSize == &dobj)
	{
		(*fontSize)[0].GetValue (newfontsize);
		if(newfontsize < 4 || newfontsize > 72)
		{
			// bad initialization?
			newfontsize = 12;
			(*fontSize)[0].SetValue (newfontsize);
		}
		wxFont font = tAttr.GetFont();
		font.SetPointSize(newfontsize);
		tAttr.SetFont(font);
		SetDefaultStyle(tAttr);
		SetStyle(0,GetLastPosition(),tAttr);
		return TRUE;
	}

	if(fontFamily == &dobj)
	{
		(*fontFamily)[0].GetValue (newfontfamily);
		wxFont curfont = tAttr.GetFont();
		if(newfontfamily == "CPC_FONT_ROMAN")
		{
			wxFont font(curfont.GetPointSize(),wxFONTFAMILY_ROMAN,
			    wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
			tAttr.SetFont(font);
		}
		else if (newfontfamily == "CPC_FONT_SANS")
		{
			wxFont font(curfont.GetPointSize(),wxFONTFAMILY_SWISS,
			    wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
			tAttr.SetFont(font);
		}
		else
		{
			wxFont font(curfont.GetPointSize(),wxFONTFAMILY_MODERN,
			    wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
			(*fontFamily)[0].SetValue ("CPC_FONT_FIXED");
			tAttr.SetFont(font);
		}
		SetDefaultStyle(tAttr);
		SetStyle(0,GetLastPosition(),tAttr);
		return TRUE;
	}

	if(doCut == &dobj && IsEditable())
	{
		Cut();
	}

	if(doPaste == &dobj && IsEditable())
	{
		Paste();
	}

	if(doCopy == &dobj)
	{
		Copy();
	}

	if(doClose == &dobj)
	{
	}

	if(doSave == &dobj)
	{
	}


	return TRUE;
}


void ShowFileViewer::CatchMenu(wxCommandEvent& ev)
{
	wxTextAttr  tAttr = GetDefaultStyle();
	int which = ev.GetId();
	if(which == wxXmlResource::GetXRCID("CPC_FONT_LARGER"))
	{
		wxFont font = tAttr.GetFont();
		font.SetPointSize(font.GetPointSize()+1);
		tAttr.SetFont(font);
		SetDefaultStyle(tAttr);
		SetStyle(0,GetLastPosition(),tAttr);
		return;
	}
	if(which == wxXmlResource::GetXRCID("CPC_FONT_SMALLER"))
	{
		wxFont font = tAttr.GetFont();
		font.SetPointSize(font.GetPointSize()-1);
		tAttr.SetFont(font);
		SetDefaultStyle(tAttr);
		SetStyle(0,GetLastPosition(),tAttr);
		return;
	}
	if(which == wxXmlResource::GetXRCID("CPC_FONT_FIXED"))
	{
		wxFont curfont = tAttr.GetFont();
		wxFont font(curfont.GetPointSize(),wxFONTFAMILY_MODERN,
			wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
		tAttr.SetFont(font);
		SetDefaultStyle(tAttr);
		SetStyle(0,GetLastPosition(),tAttr);
		return;
	}
	if(which == wxXmlResource::GetXRCID("CPC_FONT_ROMAN"))
	{
		wxFont curfont = tAttr.GetFont();
		wxFont font(curfont.GetPointSize(),wxFONTFAMILY_ROMAN,
			wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
		tAttr.SetFont(font);
		SetDefaultStyle(tAttr);
		SetStyle(0,GetLastPosition(),tAttr);
		return;
	}
	if(which == wxXmlResource::GetXRCID("CPC_FONT_SANS"))
	{
		wxFont curfont = tAttr.GetFont();
		wxFont font(curfont.GetPointSize(),wxFONTFAMILY_SWISS,
			wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
		tAttr.SetFont(font);
		SetDefaultStyle(tAttr);
		SetStyle(0,GetLastPosition(),tAttr);
		return;
	}
}

IMPLEMENT_DYNAMIC_CLASS(ShowFileViewerXmlHandler,wxXmlResourceHandler)

ShowFileViewerXmlHandler::ShowFileViewerXmlHandler() 
: wxXmlResourceHandler() 
{
    XRC_ADD_STYLE(wxTE_NO_VSCROLL);
    XRC_ADD_STYLE(wxTE_AUTO_SCROLL);
    XRC_ADD_STYLE(wxTE_PROCESS_ENTER);
    XRC_ADD_STYLE(wxTE_PROCESS_TAB);
    XRC_ADD_STYLE(wxTE_MULTILINE);
    XRC_ADD_STYLE(wxTE_PASSWORD);
    XRC_ADD_STYLE(wxTE_READONLY);
    XRC_ADD_STYLE(wxHSCROLL);
    XRC_ADD_STYLE(wxTE_RICH);
    XRC_ADD_STYLE(wxTE_RICH2);
    XRC_ADD_STYLE(wxTE_AUTO_URL);
    XRC_ADD_STYLE(wxTE_NOHIDESEL);
    XRC_ADD_STYLE(wxTE_LEFT);
    XRC_ADD_STYLE(wxTE_CENTRE);
    XRC_ADD_STYLE(wxTE_RIGHT);
    XRC_ADD_STYLE(wxTE_DONTWRAP);
    XRC_ADD_STYLE(wxTE_LINEWRAP);
    XRC_ADD_STYLE(wxTE_WORDWRAP);

    AddWindowStyles();
}

wxObject *ShowFileViewerXmlHandler::DoCreateResource()
{ 
	int rows,cols; 
	wxString filename = GetText(wxT("filename"));
	wxString viewername = GetName();
	wxString fontsizename = GetText("fontsizename");
	wxString fontfamilyname = GetText("fontfamilyname");
	wxString docutname = GetText("docutname");
	wxString docopyname = GetText("docopyname");
	wxString dopastename = GetText("dopastename");
	wxString doclosename = GetText("doclosename");
	wxString dosavename = GetText("dosavename");


	int fontsize = 12;
	wxString fontfamilystr = "CPC_FONT_FIXED";

	DataObject *dobj;

	ShowFileViewer *view = new ShowFileViewer;
	view->Create(m_parentAsWindow,wxXmlResource::GetXRCID(viewername),
		"", GetPosition(), GetSize(), GetStyle());

	view->filename = filename;
	view->LoadFile(filename);
	KludgeView = view;
	CreateChildren(view);
	vector<wxString> viewerdata;

	if(fontsizename != "")
	{
		dobj = GetDynFromName (fontsizename, "font size", INTT, -1, -1,
                    "ShowFileViewer", viewername, FALSE);
		if (dobj != NULL)
		{
			(*dobj)[0].GetValue (fontsize);
			if(fontsize < 4 || fontsize > 72)
			{
				// bad initialization?
				fontsize = 12;
				(*dobj)[0].SetValue (fontsize);
			}
			viewerdata.push_back (fontsizename);
			view->fontSize = dobj;
		}
	}

	if(fontfamilyname != "")
	{
		dobj = GetDynFromName (fontfamilyname, "font family", STRINGT, -1, -1,
                    "ShowFileViewer", viewername, FALSE);
		if (dobj != NULL)
		{
			(*dobj)[0].GetValue(fontfamilystr);
			if(fontfamilystr != "CPC_FONT_ROMAN" &&
			   fontfamilystr != "CPC_FONT_SANS")
			{
				fontfamilystr = "CPC_FONT_FIXED";
				(*dobj)[0].SetValue (fontfamilystr);
			}
			viewerdata.push_back (fontfamilyname);
			view->fontFamily = dobj;
		}
	}

	if(HasParam("font"))
	{
		wxFont font = GetFont();
		view->SetFont(font);
	}
	else
	{
	//	wxFont curfont = tAttr.GetFont();
	//	wxFont font(fontsize,wxFONTFAMILY_MODERN,
	//		wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
	//	tAttr.SetFont(font);
		if(fontfamilystr == "CPC_FONT_ROMAN")
		{
			wxFont font(fontsize,wxFONTFAMILY_ROMAN,
			    wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
			view->SetFont(font);
		}
		else if (fontfamilystr == "CPC_FONT_SANS")
		{
			wxFont font(fontsize,wxFONTFAMILY_SWISS,
			    wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
			view->SetFont(font);
		}
		else
		{
			wxFont font(fontsize,wxFONTFAMILY_MODERN,
			    wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
			view->SetFont(font);
			if(view->fontFamily)
			{
				(*(view->fontFamily))[0].SetValue ("CPC_FONT_FIXED");
			}
		}
	}

	if(doclosename != "")
	{
		dobj = GetDynFromName (doclosename, "Close flag", INTT, -1, -1,
                    "ShowFileViewer", viewername, FALSE);
		if (dobj != NULL)
		{
			(*dobj)[0].SetValue (0);  // nothing happens at first
			viewerdata.push_back (doclosename);
			view->doClose = dobj;
		}
	}


	if(dosavename != "")
	{
		dobj = GetDynFromName (dosavename, "Save flag", INTT, -1, -1,
                    "ShowFileViewer", viewername, FALSE);
		if (dobj != NULL)
		{
			(*dobj)[0].SetValue (0);  // nothing happens at first
			viewerdata.push_back (dosavename);
			view->doSave = dobj;
		}
	}

	if(docutname != "")
	{
		dobj = GetDynFromName (docutname, "Cut flag", INTT, -1, -1,
                    "ShowFileViewer", viewername, FALSE);
		if (dobj != NULL)
		{
			(*dobj)[0].SetValue (0);  // nothing happens at first
			viewerdata.push_back (docutname);
			view->doCut = dobj;
		}
	}

	if(docopyname != "")
	{
		dobj = GetDynFromName (docopyname, "Copy flag", INTT, -1, -1,
                    "ShowFileViewer", viewername, FALSE);
		if (dobj != NULL)
		{
			(*dobj)[0].SetValue (0);  // nothing happens at first
			viewerdata.push_back (docopyname);
			view->doCopy = dobj;
		}
	}

	if(dopastename != "")
	{
		dobj = GetDynFromName (dopastename, "Paste flag", INTT, -1, -1,
                    "ShowFileViewer", viewername, FALSE);
		if (dobj != NULL)
		{
			(*dobj)[0].SetValue (0);  // nothing happens at first
			viewerdata.push_back (dopastename);
			view->doPaste = dobj;
		}
	}


	thisApp->DynamicDataP->addViewer(viewername,(Viewer*) view);
	thisApp->DynamicDataP->setViewerData(viewerdata,viewername);
    
	return view;
}

bool ShowFileViewerXmlHandler::CanHandle(wxXmlNode *node)
{
    return IsOfClass(node, wxT("ShowFileViewer"));
}




