#include "cpcplatform.h"

#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif
#include "wx/print.h"
#include "wx/printdlg.h"
#include "wx/generic/dcpsg.h"
#include "wx/clipbrd.h"


#include "carapace.h"
#include "viewers.h"

#undef wxIMPLEMENT_CLASS_COMMON2
#define wxIMPLEMENT_CLASS_COMMON2(name, basename1, basename2, func)           \
    wxIMPLEMENT_CLASS_COMMON(name, basename1, &basename2::ms_classInfo,func)


BEGIN_EVENT_TABLE(SliderViewer,wxSlider)
EVT_RIGHT_DOWN(SliderViewer::DoPopup)
EVT_MENU_RANGE(1,200000000,SliderViewer::CatchMenu)
EVT_SCROLL(SliderViewer::OnSlider)
END_EVENT_TABLE()

SliderViewer::SliderViewer()
{
	wxSlider();
	intparms = doubleparms = action = label = vectorselect = NULL;
	actionstring = "";

}

SliderViewer::~SliderViewer()
{     
	delete popupmenu;
}

bool SliderViewer::draw(DataObject& dobj)
{
	int sval,slow,shigh,i;
	double dval,dlow,dhigh;
	wxString tmpstr;
	(*intparms)[0].GetValue(sval);
	if(GetValue() == sval)
	{
		return TRUE;
	}
	SetValue(sval);

	// only update something if no one else is trying to update it
	if(doubleparms && doubleparms->modifiedBy == NULL)
	{
		(*intparms)[1].GetValue(slow);
		(*intparms)[2].GetValue(shigh);
		(*doubleparms)[1].GetValue(dlow);
		(*doubleparms)[2].GetValue(dhigh);
		dval = ((double)(sval-slow))/(shigh-slow)*(dhigh-dlow)+dlow;
		(*doubleparms)[0].SetValue(dval);
		doubleparms->modifiedBy = this;
	}
	if(vectorselect && vectorselect->modifiedBy == NULL)
	{
		(*intparms)[1].GetValue(slow);
		(*intparms)[2].GetValue(shigh);
		for(i=slow;i<=shigh;i++)
		{
			(*vectorselect)[i-slow].SetValue(FALSE);
		}
		(*vectorselect)[sval-slow] = TRUE;
		vectorselect->modifiedBy = this;
	}
	if(label && label->modifiedBy == NULL)
	{
		if(doubleparms)
		{
			(*doubleparms)[0].GetValue(dval);
			tmpstr.Printf("%.3g",dval);
		}
		else
		{
			tmpstr.Printf("%d",sval);
		}
		(*label)[0].SetValue(tmpstr);
		label->modifiedBy = this;
	}
	ProcessAction();
	return TRUE;
}

void SliderViewer::OnSlider(wxScrollEvent& event)
{
	int dynvalue, sval,slow,shigh,i;
	double dval=0.,dlow,dhigh;
	wxString tmpstr;
	(*intparms)[0].GetValue(dynvalue);
	sval = GetValue();
	if(sval == dynvalue)
	{
		return;
	}
	(*intparms)[0].SetValue(sval);
	(*intparms).modifiedBy = this;
	if(doubleparms)
	{
		(*intparms)[1].GetValue(slow);
		(*intparms)[2].GetValue(shigh);
		(*doubleparms)[1].GetValue(dlow);
		(*doubleparms)[2].GetValue(dhigh);
		dval = ((double)(sval-slow))/(shigh-slow)*(dhigh-dlow)+dlow;
		(*doubleparms)[0].SetValue(dval);
		doubleparms->modifiedBy = this;
	}
	if(vectorselect)
	{
		(*intparms)[1].GetValue(slow);
		(*intparms)[2].GetValue(shigh);
		for(i=slow;i<=shigh;i++)
		{
			(*vectorselect)[i-slow].SetValue(FALSE);
		}
		(*vectorselect)[sval-slow] = TRUE;
		vectorselect->modifiedBy = this;
	}
	if(label)
	{
		if(doubleparms)
		{
			tmpstr.Printf("%.3g",dval);
		}
		else
		{
			tmpstr.Printf("%d",sval);
		}
		(*label)[0].SetValue(tmpstr);
		label->modifiedBy = this;
	}
	ProcessAction();
}


IMPLEMENT_DYNAMIC_CLASS(SliderViewerXmlHandler,wxXmlResourceHandler)

SliderViewerXmlHandler::SliderViewerXmlHandler() 
: wxXmlResourceHandler() 
{
    AddWindowStyles();
}

wxObject *SliderViewerXmlHandler::DoCreateResource()
{ 
	int rows,cols,i; 
	vector<int> svals;
	vector<double> dvals;
	vector<bool> bvals;

	wxString viewername = GetName();
	wxString intname = GetText(wxT("parmsname"));
	wxString doublename = GetText(wxT("doubleparmsname"));
	wxString actionstring = GetText("action");
	wxString actionname = GetText("actionname");
	wxString labelname = GetText("labelname");
	wxString vectorname = GetText("vectorname");
        bool isvertical = GetBool("vertical",0);
	wxString error;

	long style = wxSL_HORIZONTAL;
	if(isvertical) 
	{
		style = wxSL_VERTICAL;
	}

	DataObject *dobj;

	dobj = GetDynFromName(intname,"parmsname",INTT,3,1,"SliderViewer",viewername,TRUE);
	if(dobj == NULL)
	{
		return NULL;
	}
	thisApp->DynamicDataP->getDataObjectContent(intname,rows,cols,svals);
	// svals should contain value, min, and max
	if(svals[0] < svals[1] || svals[0] > svals[2] || svals[2] <= svals[1])
	{
		// screwy parameters
		error.Printf(CPC_STRING_VIEWER0012,intname.c_str(),"SliderViewer",viewername.c_str());
		wxMessageBox(error);
		return NULL;
	}
	// now everything should be ducky
	SliderViewer *view = new SliderViewer;
	view->Create(m_parentAsWindow,wxXmlResource::GetXRCID(viewername),svals[0],svals[1],
		svals[2],wxDefaultPosition,wxSize(100,-1),style);
	view->intparms = dobj;
	thisApp->DynamicDataP->addViewer(viewername,(Viewer*) view);
	KludgeView = view;
	CreateChildren(view);

	vector<wxString> viewerdata;
	viewerdata.push_back(intname);

	dobj = GetDynFromName(doublename,"doubleparmsname",DOULBET,3,1,"SliderViewer",viewername,FALSE);
	if(dobj != NULL)
	{
		thisApp->DynamicDataP->getDataObjectContent(doublename,rows,cols,dvals);
		// dvals should contain value, min, and max
		if(dvals[2] <= dvals[1])
		{
			// screwy parameters
			error.Printf(CPC_STRING_VIEWER0012,doublename.c_str(),
				"SliderViewer",viewername.c_str());
			wxMessageBox(error);
			dobj = NULL;
		}
		else
		{
			dvals[0] = dvals[1]+(dvals[2]-dvals[1])*
				(svals[0]-svals[1])/(svals[2]-svals[1]);
			view->doubleparms = dobj;
			(*(view->doubleparms))[0].SetValue(dvals[0]);
			viewerdata.push_back(doublename);
		}
	}

	dobj = GetDynFromName(vectorname,"vectorname",LOGICT,svals[2]-svals[1]+1,1,
		"SliderViewer",viewername,FALSE);
	if(dobj != NULL)
	{
		view->vectorselect = dobj;
		for(i=0;i<(svals[2]-svals[1]+1);i++)
		{
			(*(view->vectorselect))[0].SetValue(FALSE);
		}
		(*(view->vectorselect))[svals[0]-svals[1]].SetValue(TRUE);
		viewerdata.push_back(vectorname);
	}

	if(actionstring != "")
	{
		view->actionstring = actionstring;
	}

	dobj = GetDynFromName(actionname,"actionname",STRINGT,-1,-1,
		"SliderViewer",viewername,FALSE);
	if(dobj != NULL)
	{
		view->action = &(thisApp->DynamicDataP->getDataObjectValue(actionname));
		viewerdata.push_back(actionname);
	}
	view->hidecommand = GetBool("hidecommand",0);
	view->hideoutput = GetBool("hideoutput",0);

	dobj = GetDynFromName(labelname,"labelname",STRINGT,-1,-1,
		"SliderViewer",viewername,FALSE);
	if(dobj != NULL)
	{
		view->label = &(thisApp->DynamicDataP->getDataObjectValue(labelname));
		viewerdata.push_back(labelname);
	}

	thisApp->DynamicDataP->setViewerData(viewerdata,viewername);
    

	return view;
}

bool SliderViewerXmlHandler::CanHandle(wxXmlNode *node)
{
    return IsOfClass(node, wxT("SliderViewer"));
}


