#include "cpcplatform.h"

#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif
#include "wx/print.h"
#include "wx/printdlg.h"
#include "wx/generic/dcpsg.h"
#include "wx/clipbrd.h"


#include "carapace.h"
#include "viewers.h"

#undef wxIMPLEMENT_CLASS_COMMON2
#define wxIMPLEMENT_CLASS_COMMON2(name, basename1, basename2, func)           \
    wxIMPLEMENT_CLASS_COMMON(name, basename1, &basename2::ms_classInfo,func)


void Viewer::ProcessMenuCommand(wxCommandEvent& event)
{
	cpcMenuItem *item;
	DataObject *dobj = NULL;

	int which = event.GetId();

	item = (cpcMenuItem *) popupmenu->FindItem(which);
	if(!item)
	{
		return;
	}
	if(item->actionname.Length() > 0)
	{
		dobj = &(thisApp->DynamicDataP->getDataObjectValue(item->actionname));
		if(dobj == NULL)
		{
			wxString error;
			error << CPC_STRING_VIEWER0001 << item->actionname << CPC_STRING_VIEWER0002
				<< "PopupMenu";
			return;
		}
	}
	ProcessAction(item->action,dobj);
}

void Viewer::ProcessAction(void)
{
	ProcessAction(actionstring,action);
}

void Viewer::ProcessAction(wxString& actionstr, DataObject* dobj)
{
	wxString tmpstr = actionstr;
	if(dobj)
	{
		(*dobj)[0].GetValue(tmpstr);
	}
	if(tmpstr != "")
	{
		wxCommandEvent procevent(wxEVT_COMMAND_MENU_SELECTED,CPC_GUI_PROCESSCOMMAND);
		CarapaceCommand *cptr = new CarapaceCommand;
		cptr->command = tmpstr;
		if(tmpstr.Last() != '\n')
		{
			cptr->command << "\n";
		}
		// this associates the command with the first text frame
		cptr->frame = thisApp->STFrameList.GetFirst()->GetData()->TextFrameIndex;
		cptr->hidecommand = hidecommand;
		cptr->hideoutput = hideoutput;
		procevent.SetClientData((void *) cptr);
		wxPostEvent(thisApp,procevent);
	}
}


IMPLEMENT_DYNAMIC_CLASS(cpcPopupMenuXmlHandler,wxXmlResourceHandler)

cpcPopupMenuXmlHandler::cpcPopupMenuXmlHandler() 
: wxXmlResourceHandler() 
{
}

wxObject *cpcPopupMenuXmlHandler::DoCreateResource()
{ 

    if (m_class == wxT("cpcPopupMenu"))
    {
        wxMenu *menu = new wxMenu();
        wxString title = GetText(wxT("label"));
        wxString help = GetText(wxT("help"));
        
        bool oldins = m_insideMenu;
        m_insideMenu = TRUE;
        CreateChildren(menu, TRUE/*only this handler*/);
        m_insideMenu = oldins;

        wxMenu *p_menu = wxDynamicCast(m_parent, wxMenu);
        if (p_menu)
        {
            p_menu->Append(GetID(), title, menu, help);
            return p_menu;
	}
	// here the parent must be a Viewer
	// for some reason, the parent is not getting set correctly
	// so we use the kludge of a global pointer
	// FIX ME
	// m_parent seems to be off by 60 bytes (consistently) so
	// it must be some other structure that I haven't yet figured out
	//((Viewer *) m_parent)->popupmenu = menu;
	KludgeView->popupmenu = menu;
        return NULL;
    }

    else
    {
        wxMenu *p_menu = wxDynamicCast(m_parent, wxMenu);
        
        if (m_class == wxT("separator"))
            p_menu->AppendSeparator();
        else if (m_class == wxT("break"))
            p_menu->Break();
        else /*cpcMenuItem*/
        {          
            int id = GetID();
            wxString label = GetText(wxT("label"));
            wxString accel = GetText(wxT("accel"), FALSE);
            wxString fullLabel = label;
            if (!accel.IsEmpty())
                fullLabel << wxT("\t") << accel;

            wxItemKind kind = wxITEM_NORMAL;

            cpcMenuItem *mitem = new cpcMenuItem(p_menu, id, fullLabel,
                                               GetText(wxT("help")), kind);
                                               
            p_menu->Append((wxMenuItem *) mitem);
            mitem->Enable(GetBool(wxT("enabled"), TRUE));
            if (kind == wxITEM_CHECK)
                mitem->Check(GetBool(wxT("checked")));

            wxString action = GetText(wxT("action"));
            wxString actionname = GetText(wxT("actionname"));
            bool leaveoncommandline = GetBool(wxT("leaveoncommandline"));
            bool hidecommand = GetBool(wxT("hidecommand"));
            bool hideoutput = GetBool(wxT("hideoutput"));
            mitem->actionname = actionname;
            mitem->action = action;
            mitem->label = label;
            mitem->accel = accel;
            mitem->leaveoncommandline = leaveoncommandline;
            mitem->hidecommand = hidecommand;
            mitem->hideoutput = hideoutput;


        }
        return NULL;
    }
}



bool cpcPopupMenuXmlHandler::CanHandle(wxXmlNode *node)
{
    return IsOfClass(node, wxT("cpcPopupMenu")) ||
           (m_insideMenu && 
               (IsOfClass(node, wxT("cpcMenuItem")) ||
                IsOfClass(node, wxT("break")) ||
                IsOfClass(node, wxT("separator")))
           );
}

DataObject *GetDynFromName(wxString& oname, char * olabel, long type, long rows, 
	long cols, char * vtype, wxString& vname, bool warn)
{
	wxString error;
	int r,c,t;

	if(oname == "")
	{
		if(warn)
		{
			error.Printf(CPC_STRING_VIEWER0010,olabel,vtype,vname.c_str());
			wxMessageBox(error);
		}
		return NULL;
	}
	if(!thisApp->DynamicDataP->getDataObjectProperty(oname,r,c,t))
	{
		error.Printf(CPC_STRING_VIEWER0011,oname.c_str(),vtype,vname.c_str());
		wxMessageBox(error);
		return NULL;
	}
	if(type != t ||
		(rows > 0 && r != rows) ||
		(cols > 0 && c != cols))
	{
		// wrong type or shape
		error.Printf(CPC_STRING_VIEWER0012,oname.c_str(),vtype,vname.c_str());
		wxMessageBox(error);
		return NULL;
	}
	return  &(thisApp->DynamicDataP->getDataObjectValue(oname));
}

