#ifndef _VIEWERS_H_
#define _VIEWERS_H_

#include "cpcdefs.h"
#include "wx/wxhtml.h"
#include "wx/button.h"
#include "wx/defs.h"
#include "wx/object.h"
#include "wx/list.h"
#include "wx/control.h"
#include "wx/sizer.h"
#include "wx/statline.h"

class DataObject;
class CarapaceNonModalFrame;

#include "wx/xrc/xmlres.h"

extern DataObject *GetDynFromName(wxString& oname, char * olabel, long type, long rows, 
	long cols, char * vtype, wxString& vname, bool warnonemptyname=FALSE);

class Viewer
{
public:
	Viewer(){action = NULL; popupmenu = NULL;};
	virtual ~Viewer(){};

	DataObject *action;
	wxString actionstring;
	bool hidecommand;
	bool hideoutput;
	wxMenu *popupmenu;
	void ProcessMenuCommand(wxCommandEvent& event);
	void ProcessAction();
	void ProcessAction(wxString& actionstring, DataObject* dojb);
	void OnClose(wxCloseEvent& ev) {wxMessageBox("in onclose");};

	bool setState() {return true;};
	virtual bool draw(DataObject& dobj) {cout << "in viewer::close" << endl; return true;};
	DataObject state;
	CarapaceNonModalFrame  *Container;
};


class cpcPopupMenuXmlHandler : public wxXmlResourceHandler
{
public:
    cpcPopupMenuXmlHandler();
    wxObject *DoCreateResource();
    bool CanHandle(wxXmlNode *node);

private:
    bool m_insideMenu;
    DECLARE_DYNAMIC_CLASS(cpcPopupMenuXmlHandler)
};


class SliderViewer: public Viewer, public wxSlider
{
public:
	SliderViewer();
	~SliderViewer();

        DataObject *intparms;
        DataObject *doubleparms;
	DataObject *label;
	DataObject *vectorselect;

        bool draw(DataObject& dobj);
	void OnSlider(wxScrollEvent &);

	void                CatchMenu(wxCommandEvent& event) {ProcessMenuCommand(event);}
	void                DoPopup(wxMouseEvent& ev) {if(popupmenu) PopupMenu(popupmenu);}

protected:
    
private:
    DECLARE_EVENT_TABLE();
};

class SliderViewerXmlHandler : public wxXmlResourceHandler
{
public:
    SliderViewerXmlHandler();
    virtual wxObject *DoCreateResource();
    virtual bool CanHandle(wxXmlNode *node);
private:
    DECLARE_DYNAMIC_CLASS(SliderViewerXmlHandler)
};

class StaticTextViewer: public Viewer, public wxStaticText
{
public:
	StaticTextViewer();
	~StaticTextViewer();

        DataObject *content;
        bool draw(DataObject& dobj);

	void                CatchMenu(wxCommandEvent& event) {ProcessMenuCommand(event);}
	void                DoPopup(wxMouseEvent& ev) {if(popupmenu) PopupMenu(popupmenu);}

protected:
    
private:
    DECLARE_EVENT_TABLE();
};

class StaticTextViewerXmlHandler : public wxXmlResourceHandler
{
public:
    StaticTextViewerXmlHandler();
    virtual wxObject *DoCreateResource();
    virtual bool CanHandle(wxXmlNode *node);
private:
    DECLARE_DYNAMIC_CLASS(StaticTextViewerXmlHandler)
};


class ScrolledTextViewer: public Viewer, public wxTextCtrl
{
public:
	ScrolledTextViewer();
	~ScrolledTextViewer();

        DataObject *content;
        bool draw(DataObject& dobj);

	void                CatchMenu(wxCommandEvent& event) {ProcessMenuCommand(event);}
	void                DoPopup(wxMouseEvent& ev) {if(popupmenu) PopupMenu(popupmenu);}

protected:
    
private:
    DECLARE_EVENT_TABLE();
};

class ScrolledTextViewerXmlHandler : public wxXmlResourceHandler
{
public:
    ScrolledTextViewerXmlHandler();
    virtual wxObject *DoCreateResource();
    virtual bool CanHandle(wxXmlNode *node);
private:
    DECLARE_DYNAMIC_CLASS(ScrolledTextViewerXmlHandler)
};



class ShowFileViewer: public Viewer, public wxTextCtrl
{
public:
	ShowFileViewer();
	~ShowFileViewer();

	wxString filename;
	int currentFontSize;
	int currentFontFamily;
        DataObject *fontSize;
        DataObject *fontFamily;
        DataObject *doCut;
        DataObject *doCopy;
        DataObject *doPaste;
        DataObject *doSave;
        DataObject *doClose;
        bool draw(DataObject& dobj);

	void     CatchMenu(wxCommandEvent& event);
	void     DoPopup(wxMouseEvent& ev) {if(popupmenu) PopupMenu(popupmenu);}
	void     DoClose(void);
	void     DoSave(void);
	void     DoSaveAs(void);

protected:
    
private:
    DECLARE_EVENT_TABLE();
};

class ShowFileViewerXmlHandler : public wxXmlResourceHandler
{
public:
    ShowFileViewerXmlHandler();
    virtual wxObject *DoCreateResource();
    virtual bool CanHandle(wxXmlNode *node);
private:
    DECLARE_DYNAMIC_CLASS(ShowFileViewerXmlHandler)
};


#if 0
class ListViewer: public Viewer, public wxListBox
{
public:
	ListViewer();
	~ListViewer();

        DataObject *labels;
        DataObject *selected;
        bool draw(DataObject& dobj);
	void OnListbox(wxCommandEvent &);

	void                CatchMenu(wxCommandEvent& event) {ProcessMenuCommand(event);}
	void                DoPopup(wxMouseEvent& ev) {if(popupmenu) PopupMenu(popupmenu);}

protected:
    
private:
    DECLARE_EVENT_TABLE();
};

class ListViewerXmlHandler : public wxXmlResourceHandler
{
public:
    ListViewerXmlHandler();
    virtual wxObject *DoCreateResource();
    virtual bool CanHandle(wxXmlNode *node);
private:
    DECLARE_DYNAMIC_CLASS(ListViewerXmlHandler)
};
#endif


// this class was originally so that I could change the
// colors in the labels. I can do that, but the highlighted
// colors are outside by control.  
class ListViewer: public Viewer, public wxListCtrl
{
public:
	ListViewer();
	~ListViewer();

        DataObject *labels;
        DataObject *selected;
        DataObject *highlighted;
        DataObject *deleted;
	DataObject *colors;
	DataObject *glyphs;
        bool draw(DataObject& dobj);
	void OnSelected(wxListEvent &);
	void OnDeselected(wxListEvent &);

	void                CatchMenu(wxCommandEvent& event) {ProcessMenuCommand(event);}
	void                DoPopup(wxMouseEvent& ev) {if(popupmenu) PopupMenu(popupmenu);}

	vector<wxString> lvals;
	vector<bool> hvals;
	vector<bool> svals;
	vector<bool> dvals;
	vector<int> cvals;
	vector<int> gvals;

	int rows;
protected:
    
private:
    DECLARE_EVENT_TABLE();
};

class ListViewerXmlHandler : public wxXmlResourceHandler
{
public:
    ListViewerXmlHandler();
    virtual wxObject *DoCreateResource();
    virtual bool CanHandle(wxXmlNode *node);
private:
    DECLARE_DYNAMIC_CLASS(ListViewerXmlHandler)
};

class PatchViewer : public Viewer, public wxPanel
{
public:
	PatchViewer(long type);
	~PatchViewer();

	bool Create(wxWindow *parent, wxWindowID id);

	long                Type;
	long                SelectedIndex;
        wxString            SelectedLabel;
	wxGridSizer        *Sizer;

        DataObject         *stringIndex;            
        DataObject         *longIndex;            

	void                ProcessClick(wxString& string, long& index);
	void                OnClose(wxCloseEvent&);
        bool                draw(DataObject& dobj);

	void                CatchMenu(wxCommandEvent& event) {ProcessMenuCommand(event);}
	void                DoPopup(wxMouseEvent& ev) {if(popupmenu) PopupMenu(popupmenu);}

	DECLARE_EVENT_TABLE();
};

extern Viewer *KludgeView;

class SmallColorPickerXmlHandler : public wxXmlResourceHandler
{
public:
    SmallColorPickerXmlHandler();
    virtual wxObject *DoCreateResource();
    virtual bool CanHandle(wxXmlNode *node);
private:
    DECLARE_DYNAMIC_CLASS(SmallColorPickerXmlHandler)
};


class MediumColorPickerXmlHandler : public wxXmlResourceHandler
{
public:
    MediumColorPickerXmlHandler();
    virtual wxObject *DoCreateResource();
    virtual bool CanHandle(wxXmlNode *node);
private:
    DECLARE_DYNAMIC_CLASS(MediumColorPickerXmlHandler)
};


class LargeColorPickerXmlHandler : public wxXmlResourceHandler
{
public:
    LargeColorPickerXmlHandler();
    virtual wxObject *DoCreateResource();
    virtual bool CanHandle(wxXmlNode *node);
private:
    DECLARE_DYNAMIC_CLASS(LargeColorPickerXmlHandler)
};


class GlyphPickerXmlHandler : public wxXmlResourceHandler
{
public:
    GlyphPickerXmlHandler();
    virtual wxObject *DoCreateResource();
    virtual bool CanHandle(wxXmlNode *node);
private:
    DECLARE_DYNAMIC_CLASS(GlyphPickerXmlHandler)
};

// The following appended on April 15,2005
class CheckboxViewer: public Viewer, public wxCheckBox
{
public:
	CheckboxViewer();
	~CheckboxViewer();

        DataObject *parameters;            // check or unchecked value(true or false)
        DataObject *label;
       
        bool draw(DataObject& dobj);
        bool close(void);
	void OnCheckbox(wxCommandEvent &);
	void                CatchMenu(wxCommandEvent& event) {ProcessMenuCommand(event);}
	void                DoPopup(wxMouseEvent& ev) {if(popupmenu) PopupMenu(popupmenu);}


protected:
    
private:
    DECLARE_EVENT_TABLE();
    //DECLARE_DYNAMIC_CLASS(CheckboxViewer)
};

class CheckboxViewerXmlHandler : public wxXmlResourceHandler
{
public:
    CheckboxViewerXmlHandler();
    virtual wxObject *DoCreateResource();
    virtual bool CanHandle(wxXmlNode *node);
private:
    DECLARE_DYNAMIC_CLASS(CheckboxViewerXmlHandler)
};

// The following appended on May 20,2005
class RadioboxViewer: public Viewer, public wxRadioBox
{
public:
	RadioboxViewer();
	~RadioboxViewer();

        DataObject *parameters;            // 0 - the selected number of the RadioBox,1- the number of choices of RadioBox
 //       DataObject *parameterResultdt;     // the result dataobject of each choices
        DataObject *parameterLabeldt;     //  the label of each radio button in the checkbox
        DataObject *label;     //  the label of each radio button in the checkbox
       
        bool draw(DataObject& dobj);
        //bool close(void);
	void OnRadiobox(wxCommandEvent &);
	void                CatchMenu(wxCommandEvent& event) {ProcessMenuCommand(event);}
	void                DoPopup(wxMouseEvent& ev) {if(popupmenu) PopupMenu(popupmenu);}
	
	
 // wxString strdoname;

protected:
    
private:
    DECLARE_EVENT_TABLE();
};

class RadioboxViewerXmlHandler : public wxXmlResourceHandler
{
public:
    RadioboxViewerXmlHandler();
    virtual wxObject *DoCreateResource();
    virtual bool CanHandle(wxXmlNode *node);
private:
    DECLARE_DYNAMIC_CLASS(RadioboxViewerXmlHandler)
};


class SpinCtrlViewer: public Viewer, public wxSpinCtrl
{
public:
	SpinCtrlViewer();
	~SpinCtrlViewer();

  DataObject *intparms;      // 0 - current value;1 - min value; 2 - max value
	DataObject *label;
	
  bool draw(DataObject& dobj);
	void OnSpinCtrl(wxSpinEvent &);
	void OnSpinCtrlText(wxCommandEvent &);

	void                CatchMenu(wxCommandEvent& event) {ProcessMenuCommand(event);}
	void                DoPopup(wxMouseEvent& ev) {if(popupmenu) PopupMenu(popupmenu);}

protected:
    
private:
    DECLARE_EVENT_TABLE();
};

class SpinCtrlViewerXmlHandler : public wxXmlResourceHandler
{
public:
    SpinCtrlViewerXmlHandler();
    virtual wxObject *DoCreateResource();
    virtual bool CanHandle(wxXmlNode *node);
private:
    DECLARE_DYNAMIC_CLASS(SpinCtrlViewerXmlHandler)
};

class ButtonViewer: public Viewer, public wxButton
{
public:
	ButtonViewer();
	~ButtonViewer();

  DataObject *intparms;
	DataObject *label;


        bool draw(DataObject& dobj);
	void OnButton(wxCommandEvent &);

	void                CatchMenu(wxCommandEvent& event) {ProcessMenuCommand(event);}
	void                DoPopup(wxMouseEvent& ev) {if(popupmenu) PopupMenu(popupmenu);}

protected:
    
private:
    DECLARE_EVENT_TABLE();
};

class ButtonViewerXmlHandler : public wxXmlResourceHandler
{
public:
    ButtonViewerXmlHandler();
    virtual wxObject *DoCreateResource();
    virtual bool CanHandle(wxXmlNode *node);
private:
    DECLARE_DYNAMIC_CLASS(ButtonViewerXmlHandler)
};


class PlotAreaRecViewer : public Viewer, public wxWindow
{
public:
	PlotAreaRecViewer(wxWindow *parent,wxWindowID id,wxPoint pos ,wxSize size);
	~PlotAreaRecViewer();
	
	DataObject *datax;
	DataObject *datay;
	DataObject *dataxaxislabel;
	DataObject *datayaxislabel;
	DataObject *datarangex;
	DataObject *datarangey;
	//DataObject *datacolor;
	DataObject *datacolorindex;
	DataObject *datacolorname;	
	DataObject *datasize;
	DataObject *datatype;
	DataObject *datahighlight;	
	DataObject *dataColumnSel;
	DataObject *ticknum;
	DataObject *xticklabel;
	DataObject *yticklabel;
	DataObject *ticklocation;
	DataObject *ticklen;
	DataObject *usecustomticklabels;
	DataObject *plotname;
	DataObject *plotparameters;
	DataObject *isaxislabelblank;	
	DataObject *datalinesx;
	DataObject *datalinesy;
	DataObject *datalinescolor;	
	DataObject *datalinescolorindex;		
	DataObject *datalineswidth;	
	DataObject *datalinesstyle;		
	DataObject *pointsdel;	
	DataObject *curvedel;		
	DataObject *rescale;	
	DataObject *drawSplashline;								
	double     xstart,xend,ystart,yend;
	double     xaxismin,xaxismax,yaxismin,yaxismax;
	int        xColIndex,yColIndex;
	int        xscalenum,yscalenum;
	int        CurPointSelState;
	int        xStateMousePosition[10], yStateMousePosition[10];	
	bool       ShiftDownFlag;
	int        plotxticknum,plotyticknum;
	bool       isplotxticklabel;
	bool       isplotyticklabel;	
	bool       isxtickblank;
	bool       isytickblank;
	bool       isaxisxlabelblank;
	bool       isaxisylabelblank;   
	bool       iscurvedeleted;
	bool       isrescale;
	int        xtickloc,ytickloc;
	int        xticklen,yticklen;
	int        plotleftmargin,plotrightmargin,plottopmargin,plotbottommargin,plotnamemargin,recwidth,recheight;
	wxString   linescolor;
	int        lineswidth,linesstyle;
	wxString   plotnamelabel;
	vector<int> xoffset,yoffset,pointsize,pstyle,ptype;
	vector<double> xPointValue,yPointValue;
	vector<wxString> pcolor;
	vector<bool> selpoints;
	vector<bool> ispointsdel;
	vector<wxString> plotxticklabels;
	vector<wxString> plotyticklabels;
	vector<double> xlinesPointValue,ylinesPointValue;
	wxPoint  *linespointpsseq;
	int       linespointpsseqnum;
	int       curlinespointsvaluenum;
	bool      Clickblankareaflag;
	bool      curveissplashline;

	bool      doxyupdate;
	wxString  dataxname;
	wxString  datayname;

	bool Create(wxWindow *parent, wxWindowID id);



	void                ProcessClick(long& curpx, long& curpy);
	void                OnClose(wxCloseEvent&);
        bool                draw(DataObject& dobj);
	void OnPaint(wxPaintEvent& event);
	void OnMouse(wxMouseEvent& event);
	
	bool delpoint(wxDC &dc,int pointindex);
	void delcurve(wxDC &dc);
	void DrawSplashLine(wxDC &dc,int splashlinepointsnum, wxPoint *splashlinespointseq);
	bool UpdateColIndex();
	bool UpdateLinesSplashFlag();
	bool UpdateLinesPoints();
	bool UpdateRange();
	bool UpdatePlotParameters();
	bool UpdateTickParameters();
	void DoActualDraw(bool usePaintDC);
	
        

	void                CatchMenu(wxCommandEvent& event) {ProcessMenuCommand(event);}
	void                DoPopup(wxMouseEvent& ev) {if(popupmenu) PopupMenu(popupmenu);}

	DECLARE_EVENT_TABLE();
};

class PlotViewerXmlHandler : public wxXmlResourceHandler
{
public:
    PlotViewerXmlHandler();
    virtual wxObject *DoCreateResource();
    virtual bool CanHandle(wxXmlNode *node);
private:
    DECLARE_DYNAMIC_CLASS(PlotViewerXmlHandler)
};

enum point_select_state
{
	Select_Raady = 0,
	Point_Select,
	Draw_Rectangle_Ready,
	Draw_Rectangle,
	Rectangle_Stable,
	Move_Rectangle_Ready,
	Move_Rectangle	
};

#endif
