/*
*(C)* The file is part of the source distribution of Carapace
*(C)* version 0.5 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)* Some parts Copyright (c) 2004 Huixin Wang
*(C)*
*(C)* This file is distributed under the GNU General Public License
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

#include "cpcplatform.h"



// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include "wx/wx.h"
#include "wx/print.h"
#include "wx/splash.h"
#if wxCHECK_VERSION(2,4,0)
#include "wx/imagpng.h"
#include "wx/imaggif.h"
#include "wx/imagjpeg.h"
#include "wx/imagxpm.h"
#endif


#include <math.h>

#include "wx/clipbrd.h"
#include "wx/dataobj.h"

#include "carapace.h"

#undef UNDEFINED

extern CarapaceApp     *thisApp;
extern wxString CarapaceClipboard;
extern wxString CarapaceSelection;
extern wxString CarapaceHistory;

#define BUFSIZE 4095
static char SocketBuffer[BUFSIZE+1];

void CarapaceApp::HandleBackInput(void)
{
	int count,offset;
	wxString tmpbuffer,line;

	// read in anything waiting in the socket and cache it
	backChannelSocket->Read(SocketBuffer,BUFSIZE);
	while((count = backChannelSocket->LastCount()) > 0)
	{
		SocketBuffer[count] = '\0';
		tmpbuffer << SocketBuffer;
		backChannelSocket->Read(SocketBuffer,BUFSIZE);
	}

	// tokenize splitting at newlines and put in
	// BackInputBuffer

	while((offset = tmpbuffer.Find('\n')) >= 0)
	{
		line = tmpbuffer.Left(offset+1);
		tmpbuffer = tmpbuffer.Mid(offset+1);
		if(BackInputCommandBuffer.Last().Last() != '\n')
		{
			BackInputCommandBuffer.Last() << line;
		}
		else
		{
			BackInputCommandBuffer.Add(line);
		}
	}

}

void CarapaceApp::BackChannelCommand(wxString& command)
{
	int offset;
	wxArrayString tokenBuffer;
	wxString line; 
	wxCarapaceSplitTextFrameListNode *stnode = NULL;
	// the command should hold a single, complete command,
	// including terminating newline

	// first check simple ones
	if(command == "<FlushMain/>\n")
	{
		FlushBuffer();
		command.Empty();
		return;
	}
	
	if(command == "<Busy/>\n")
	{
		stnode = STFrameList.GetFirst();
		while(stnode)
		{
			stnode->GetData()->RunningMeter->SetLabel(CPC_STRING_RUNNING);
			wxYield();
			stnode = stnode->GetNext();
		}  
		return;
	}
	
	if(command == "<Idle/>\n")
	{
		stnode = STFrameList.GetFirst();
		while(stnode)
		{
			stnode->GetData()->RunningMeter->SetLabel("");
			wxYield();
			stnode = stnode->GetNext();
		}  
		return;
	}
	
	if(command == "<ResetConsole/>\n")
	{
		command.Empty();
		MainInputBuffer.Empty();
		BackInputBuffer.Empty();
		BackInputCommandBuffer.Empty();
		return;
	}

	// tokenize command splitting at newlines and put in
	// tokenBuffer

	while((offset = command.Find('\n')) >= 0)
	{
		line = command.Left(offset+1);
		command = command.Mid(offset+1);
		tokenBuffer.Add(line);
	}
	if(command.Length() > 0)
	{
		wxMessageBox("Warning: additional characters at end of in-stream command");
	}
	if(tokenBuffer[0].Mid(1) != tokenBuffer.Last().Mid(2))
	{
		// first line should be <foo> last should be </foo>
		// error if they don't match 
		wxMessageBox("Illegal Command");
	}
	if(tokenBuffer[0] == "<ShowMessage>\n")
	{
		wxString message = "";
		int i;
		for(i=1;i<tokenBuffer.GetCount()-1;i++)
		{
			message << tokenBuffer[i];
		}
		wxMessageBox(message,"Message from R", wxOK | wxCENTRE);
		return;
	}

	wxMessageBox("Illegal Command");
}
