/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 5.01 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*                                                                 
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/


/*
  980119 added OverflowTextFrame()
  980328 added #include <math.h> before extern "C" {... #include "globals.h"}
         and removed undef and def of __cplusplus
  980401 moved #include <Xm/Text.h> to after #include "wx.h"
  980729 fixed wx_msw bug in fmyeol() which put two newlines on spool file
         for every one that was wanted.
  981002 fixed bug in get_dataPath that did not get the complete path
         when it containd a blank.  This affected only Windows version
  981028 Moved #include <pwd.h> to after #include "../globals.h"
  981110 Corrected Code for reading from console so it doesn't crash when
         cancel is clicked.
  990215 Replaced myerrorout() by putOutErrorMsg() and putOUTSTR() by
         putErrorOUTSTR()
  000308 If argument to myAlert is OUTSTR, OUTSTR is cleared
  000705 Added code to check for 'q' and 'r' during mouse input
*/

#include "../globals.h"
#include "../mvIo.h"

#include "cpcIface.h"

#include "cpcinterface.h"
#include <math.h>


#ifdef HASPWDH
#include <pwd.h>
#endif /*HASPWDH*/

#define FALSE 0
#define TRUE 1


/*
  Functions in this file
  Various input/output related utilities
	void fmyeol(FILE *fp)
	void putprompt(char *prompt)
	void myAlert(char *msgs)
	int mygetosversion(void)
	void fmyprint(char *msg,FILE *fp)

  Functions having to do with obtaining and checking file names
	char *expandFilename(char *fname)
	static int filenameFormOK(char *fname, int path)
	static int filenameCharsOK(char * fname)
	long okfilename(char *fname)
	long isfilename(char *fname)

  More output utilities
	void fmyflush(FILE *fp)
	void mybeep(long n)

  More input/output utilities
	char *get_dataPath(void)
	void incrementNLINES(char *msg)
	FILE * fmyopen(char * fileName, char * mode)
	char * findFileByName(char * fileName, char * mode) 
	char * cpcFindFile(char *fileName, char *message, char *defaultname)
*/

void fmyeol(FILE *fp)
{
	fmyprint("\n",fp);
	if (interrupted(DEFAULTTICKS))
	{
		INTERRUPT = PRINTABORT;
	}

} /*fmyeol()*/

void putprompt(char *prompt)
{
	char *prmpt;
	prmpt = (prompt == (char *) 0) ? PROMPT : prompt;

	fmyprint(prmpt,(FILE *) 0);
}

void myAlert(char *msgs)
{
	cpc_io   io;
	io.type = CPC_IO_ALERT;
	io.string = msgs;
	cpc_doio(&io);

	if (msgs == OUTSTR)
	{
		*msgs = '\0';
	}
}

#define BEL '\007'

void fmyprint(char *msg,FILE *fp)
{
	cpc_io  io;
        long            msgLength;
        const char     *outline;

	if(fp != STDOUT)
	{
		fputs(msg, fp);
		return;
	}

	io.type = CPC_IO_BUFFEREDPRINT;
	io.string = msg;
	io.parm = CPC_SWALLOW_NOTHING;
	cpc_doio(&io);

	if(SPOOLFILE != (FILE *) 0)
	{
		msgLength = strlen(msg);
		outline = msg;
		while(*outline == BEL)
		{	     /* strip leading BEL's */
		outline++;
		msgLength--;
	}
		if(msgLength > 0)
		{
			fputs(outline, SPOOLFILE);
		}
	} 
}


#ifdef BASE_MICROSOFT
/*  '/' equivalent to '\' on OS_WIN32*/
#define isSlash(C) ((C) == DIRSEPARATOR[0] || (C) == '/')
#else /*OS_WIN32*/
#define isSlash(C) ((C) == DIRSEPARATOR[0])
#endif /*OS_WIN32*/

/*
  Expand leading '~' in a filename.  A name of the form ~/path (or ~\path
  under WIN32) is expanded to home/path or home\path, where home is the value
  of MacAnova CHARACTER variable HOME.  Under Unix, HOME is predefined to
  have value taken from environmental variable $HOME.  Under WIN32, HOME is
  predefined to contain the name of the directory containing MACANOVA.EXE.
  Under Unix, a name of form ~username/path is expanded to homedir/path where
  getpwname() is used to get the home directory homedir for user username.
  Such a file name is not expanded under WIN32 and results in an error.
  On Macintosh, HOME is not predefined, but may be set in MacAnova.ini.  The
  usage ~username/path is not legal on a Mac

  950829 Fixed bug in DJGPP version
         Under WIN32, any embedded '/' are changed to DIRSEPARATOR[0]
         (backslash))
*/

static char    FullPathName[PATHSIZE];

char *expandFilename(char *fname)
{
	Symbolhandle    home;
	char           *pc;
	char           *userdir;
	long            dirNameLength;
#ifdef HASPWDH
	struct passwd  *passwd;
#endif /*HASPWDH*/
		
	if (fname[0] == EXPANSIONPREFIX)
	{
		if (isSlash(fname[1]))
		{						/* treat ~/... as $HOME/... */
			home = Lookup("HOME"); /* should be initialized at startup */
			if(home == (Symbolhandle) 0 || TYPE(home) != CHAR ||
			   !isScalar(home))
			{
				userdir = getenv("HOME");
				if (userdir == (char *) 0)
				{
					sprintf(OUTSTR,
							"ERROR: cannot expand %c%c;  no CHARACTER var. HOME or environment var. $HOME",
							fname[0], fname[1]);
					goto errorExit;
				}
			}
			else
			{
				userdir = STRINGPTR(home);
			}
			
			pc = fname+2;
		} /*if (isSlash(fname[1]))*/
		else
		{
#ifndef HASPWDH
			/* can't expand ~name on except on Unix */
			userdir = (char *) 0;
#else  /*HASPWDH*/
			char        c;

			for(pc = fname+1;*pc && !isSlash(*pc);pc++)
			{
				;
			}
			c = *pc;
			*pc = '\0';
			passwd = getpwnam(fname+1);
			*pc = c;
			if(c != '\0')
			{
				pc++;
			}
				
			userdir = (passwd != (struct passwd *) 0) ?
				passwd->pw_dir :  (char *) 0;
#endif /*HASPWDH*/
		} /*if (isSlash(fname[1])){}else{}*/
		
		if(userdir == (char *) 0)
		{
			putPieces("ERROR: unable to expand ", fname,
					  (char *) 0, (char *) 0);
			goto errorExit;
		}
			
		dirNameLength = strlen(userdir);
		/* allow for the possiblity that HOME ends in '/' */
		if (isSlash(userdir[dirNameLength-1]))
		{
			dirNameLength--;
		}
		
		if(dirNameLength + strlen(pc) + 1 >= PATHSIZE)
		{
			putPieces("ERROR: expanded file name ",userdir,pc," is too long");
			goto errorExit;
		}
		
		strcpy(FullPathName,userdir);
		FullPathName[dirNameLength++] = DIRSEPARATOR[0];
		FullPathName[dirNameLength] = '\0';
		strcat(FullPathName,pc);
	} /*if (fname[0] == EXPANSIONPREFIX)*/
	else
	{
		if(strlen(fname) >= PATHSIZE)
		{
			putPieces("ERROR: file name ", fname, " is too long", (char *) 0);
			goto errorExit;
		}
		
		strcpy(FullPathName, fname);
	} /*if (fname[0] == EXPANSIONPREFIX){}else{}*/
	
#ifdef OS_WIN32
	for (pc = FullPathName; *pc; pc++)
	{
		if (isSlash(*pc))
		{
			*pc = DIRSEPARATOR[0];
		}
	} /*for (pc = FullPathName; *pc; pc++)*/
#endif /*OS_WIN32*/

	return ((char *) FullPathName);
		
  errorExit:
	putErrorOUTSTR();
	
	return ((char *) 0);

} /*expandFilename()*/
	
/*
  Check file names for legality on platform
*/

#define isDot(C) ((C) == '.')

#ifdef OS_WIN32

/*
  For MSDOS and Windows acceptable names are of the form
    pathElement/.../pathElement, or /pathElement/.../pathElement,
  possible prefixed by a drive designator of the form [a-zA-Z]:
  '/' may be replaced by '\'.

  For MSDOS pathElement is of form '.', '..', Name or Name.Ext, and
  Name has from 1 to 8 each characters and Ext has from 0 to three
  characters

  For Windows, when long file names are not permitted (Windows 3.1),
  acceptable names are the same as for MSDOS.
  When long file names are permitted (Windows 95 and Windows NT),
  a path element may contain up to 255 characters including any
  number of dots and the elements of a name (sequences of non-dots,
  non-slashes) preceding and/or following a dot can be of any length
  including 0.  Thus ..abc..d is a legal name.

  If path != 0, names ending in / or \ are acceptable.  Otherwise, they
  are not.

  970805 In Windows 95 and Windows NT long file names with multiple
  dots (a.b.c) are allowed
*/
#define MAXPATHCOMPONENT  255 /*limit on component of Windows 95 path*/

static int filenameFormOK(char *fname, int path)
{
	int        maxComponent, maxNamePart, maxNDots;
	char      *start = fname, *pc;
	int        namePartLength = 0, componentLength = 0, nDots = 0;
	int        longFileNames = 1;

	if (strlen(start) >= PATHSIZE)
	{
		/* length too long for buffers */
		return (0);
	}
	if (isalpha(start[0]) && start[1] == ':')
	{ /*skip volume id and ':'*/
		start += 2;
	}

	maxComponent = (!longFileNames) ? 12 : MAXPATHCOMPONENT;
	maxNamePart = (!longFileNames) ? 8 : MAXPATHCOMPONENT;
	maxNDots = (!longFileNames) ? 1 : MAXPATHCOMPONENT/2;

	for(pc = start;*pc != '\0';pc++)
	{
		if(*pc == ':')
		{	/* have already skipped 'x:', ':' now illegal */
			return (0);
		}
		if(isDot(*pc))
		{
			if (nDots == 0 && namePartLength == 0)
			{
				if(isSlash(pc[1]))
				{/* pathElement is '.' */
					pc++;
				}
				else if(isDot(pc[1]) && isSlash(pc[2]))
				{/* pathElement is '..' */
					pc += 2;
				}
				else
				{
					if (!longFileNames)
					{ 	/* no Name; error */
						return (0);
					}
					nDots++;
				}
			} /*if (nDots == 0 && namePartLength == 0)*/
			else
			{/* Name completed */
				if (++nDots > maxNDots)
				{
					return (0);
				}
				namePartLength = 0;
				componentLength++;
				maxNamePart = (!longFileNames) ? 3 : maxNamePart;
			}
		} /*if(isDot(*pc))*/
 		else if(isSlash(*pc))
		{/* end of pathElement */
			componentLength = namePartLength = nDots = 0;
			maxNamePart = (!longFileNames) ? 8 : maxNamePart;
		}
		else
		{ /* name character */
			namePartLength++;
			componentLength++;
			if (namePartLength > maxNamePart ||
				componentLength > maxComponent)
			{
				return (0);
			}
		}
	} /*for(pc = start;*pc;pc++)*/

	return ( path || (--pc , !isSlash(*pc)));
} /* filenameFormOK()*/

#else /*OS_WIN32*/

/*
  On Unix accept non-empty names of the form
       pathElement or pathElement/pathElement or pathElement/.../pathElement
  where a pathElement does not contain '/' but can be empty/  Note that
  this includes paths with repeated '/' (e.g., "foo//bar") or
  of the form "../bar"
*/
static int filenameFormOK(char * fname, int path)
{
	/*
	   only current restriction is that it can't end in '/';
	   or start with '-'
	   repeated '/'s allowed
	   */
	return (fname[0] != '-' && (path || !isSlash(fname[strlen(fname)-1])));

} /* filenameFormOK()*/

#endif /*OS_WIN32*/

/*
   Check for illegal characters in fname
   All alphanumerics are legal plus the characters in okNonAlnum.
   Since fname has been checked for form before being checked
   here, special characters like '/' are included in okNonAlnum
*/
static int filenameCharsOK(char * fname)
{
#if defined(OS_WIN32)
	/*
	  Only allow standard DOS filename characters under Windows
	  under Windows 95 and NT allow also '[', /]' and ' '
	*/
	char      *okNonAlnum = ".\\/:_-`!@#$%^&(){}[]' ";
#elif defined(VMS) /*OS_WIN32*/
	/* For VMS allow only : */
	char      *okNonAlnum = ".:[]<>_-+$";
#else  /*OS_WIN32*/
	/*don't allow (){}[]`'";|\~ even though that are technically legal */
	char      *okNonAlnum = " ./_,-+!@#$%^&";
#endif /*OS_WIN32*/
	char       c;
	
	while((c = *fname++) != '\0')
	{
		if (!isalnum(c) && !strchr(okNonAlnum, c))
		{
			return (0);
		}
	} /*while((c = *fname++) != '\0')*/
	return (1);
} /*filenameCharsOK()*/

/*
   silent check of appropriateness of file name
*/
long okfilename(char *fname)
{
	return (*fname != '\0' && filenameFormOK(fname, 0) &&
			filenameCharsOK(fname));
} /*okfilename()*/

/*
   silent check of appropriateness of path name
*/
long okpathname(char *fname)
{
	return (*fname != '\0' && filenameFormOK(fname, 1) &&
			filenameCharsOK(fname));
} /*okpathname()*/

/* Check filename with error messages */
long isfilename(char *fname)
{
	if (okfilename(fname))
	{
		return (1);
	}

	if(*fname == '\0')
	{
		putOutErrorMsg("ERROR: zero length file name");
	}
	else
	{
		putPieces("ERROR: improper file name ", fname, (char *) 0, (char *) 0);
	}
	return (0);
} /*isfilename()*/

void fmyflush(FILE *fp)
{
	cpc_io  io;
	if(fp != STDOUT)
	{
		fflush(fp);
	}
	else
	{
		io.type = CPC_IO_FLUSHBUFFER;
		cpc_doio(&io);

		if(SPOOLFILE != (FILE *) 0)
		{
			fflush(SPOOLFILE);
		}
	}
} /*fmyflush()*/

void mybeep(long n)
{
	cpc_io   io;
	if(BDEPTH == 0 && ISATTY)
	{
		io.type = CPC_IO_BEEP;
		io.length = n;
		cpc_doio(&io);
	}
} /*mybeep()*/



/*
  Fill line from dialog window (CONSOLE).  This code was formerly part of
  fillLINE() which was moved to commonio.c along with supporting routines
  datagetc and fileToLine()

  981110 Fixed code so that it recognizes when the uses clicks cancel
         This now signals the end of the read.  We never see anything
         the user had already put in the dialog box so we can't use it;
         this contrasts with the Done button in the Macintosh version
*/

#define MAXLINE 2000
static unsigned int   HitDone = 0;
static unsigned char  LineBuffer[MAXLINE+1];
static unsigned char *Place = (unsigned char *) LineBuffer;

	
int fillFromDialog(void)
{
	long               length;
	unsigned char     *pc;
	long               Length;
	char              *reply;
	cpc_stddlg         dlg;
	WHERE("fillFromDialog");

	/*
	  Use dialog window or what was read previously
	  Always terminate (*LINE) with '\n'
	  Return only through first '\n'.
	  */
	if((*LINE)[0] == '\0')
	{					/* first time in this trip */
		Place = LineBuffer;
		HitDone = 0;
	}

	(*LINE)[0] = '\0';
	if(Place == LineBuffer)
	{
		if(HitDone)
		{
			HitDone = 0;
			return (EOF);
		}
		/* need to read from Dialog box */
		dlg.type = CPC_DLG_GETTEXTFROMUSER;
		dlg.message = "Enter Line";
		dlg.title = "Console Input";
		HitDone = (cpc_dostddlg(&dlg) == CPC_ERROR_CANCELLED);
		reply = dlg.string;

        if (HitDone)
        {
        	Length = 0;
        }
        else
        {
			Length = strlen(reply);
			if(Length > MAXLINE)
			{
				sprintf(OUTSTR,
						"WARNING: more than %ld characters entered; last %ld ignored",
						(long) MAXLINE, Length - MAXLINE);
				myAlert(OUTSTR);
				Length = MAXLINE;
			}
			strncpy((char *) LineBuffer,reply,Length);
        }
        LineBuffer[Length] = '\0';
		strcpy(*LINE, (char *) LineBuffer);

		if(LineBuffer[0] == '\0' && HitDone)
		{
			HitDone = 0;
			return (EOF);
		}
	} /*if(Place == LineBuffer)*/
	else
	{
		strcpy(*LINE, (char *) Place);
	}

	pc = (unsigned char *) strchr((char *) Place, '\n');
	if(pc != (unsigned char *) 0 && pc[1] != '\0')
	{/* still something not seen */
		pc++;
		length = pc - Place;
		Place = pc;
	} /*if(pc != (unsigned char *) 0)*/
	else
	{					/* this is the last of it */
		length = strlen((char *) Place);
		if(pc == (unsigned char *) 0)
		{
			(*LINE)[length++] = '\n';
		}
		Place = LineBuffer;
		HitDone = 0;
	}						/*if(Place == LineBuffer){}else{}*/
	(*LINE)[length] = '\0';
	return (HitDone ? EOF : 1);
} /*fillFromDialog()*/


#ifdef OS_WIN32
extern char      **ArgV; /*defined and set equal to argv in main()*/
static char        Path[PATHSIZE+1];
/* instead of string.h */
/*
  981002 Fixed bug so that it correctly handles path names containing
         spaces
*/
char *get_dataPath(void)
{
	char    *pathp;
	int      quoted = 0, i;
	char     c;

	//pathp = GetCommandLine();
	pathp = ArgV[0];

	if (pathp[0] == '"')
	{
		pathp++;
		quoted = 1;
	}
	for (i = 0; (c = pathp[i]) != '\0' && i < PATHSIZE ; i++)
	{
		if (quoted && c == '"' || !quoted && c == ' ')
		{
			break;
		}
		Path[i] = c;
	} /*for (i = 0; (c = pathp[i]) != '\0' && i < PATHSIZE ; i++)*/
	Path[i] = '\0';
	if (i == PATHSIZE)
	{
		putOutErrorMsg("ERROR: Path to MacAnova.exe too long");
		Path[0] = '\0';
		return(Path);
	}

	if( (pathp = strrchr(Path,'\\')) == (char *) 0 )
	{
		pathp = strrchr(Path,'/');
	}
	if(pathp)
	{
		*(++pathp) = 0x00;
	}
	else
	{
		Path[0] = '\0';
	}

	return(Path);
} /*get_dataPath()*/

#endif /* OS_WIN32 */


#ifdef OS_WIN32
/*  '/' equivalent to '\' on MSDOS*/
#define isSlash(C) ((C) == DIRSEPARATOR[0] || (C) == '/')
#define isPath(N) (strchr(N, '/') || strchr(N, '\\') || strchr(N, ':'))
#else							/*MSDOS*/
#define isSlash(C) ((C) == DIRSEPARATOR[0])
#define isPath(N) strchr(N, DIRSEPARATOR[0])
#endif							/*MSDOS*/

static FILE * tryFile(char lastPath [], char * fileName, char * mode)
{
    int             pathLength = strlen(lastPath);
    int             nameLength = strlen(fileName);
    FILE           *trialFile = (FILE *) 0;

    if (pathLength > 0 && !isSlash(lastPath[pathLength-1]))
    {
        lastPath[pathLength++] = DIRSEPARATOR[0];
        lastPath[pathLength] = '\0';
    }
    if (pathLength + nameLength <= PATHSIZE)
    {
        strcpy(lastPath + pathLength, fileName);
        if (okfilename(lastPath))
        {
            trialFile = fopen(lastPath,mode);
        }
    } /*if (okpathname(lastPath) && pathLength+nameLength <= PATHSIZE)*/
    return (trialFile);
} /*tryFile()*/

/*
   Open file in appropriate mode; may do things differently on different
   platforms;

   990713 Path names starting with '~' from DATAPATHS and DATAPATH
          are now expanded

   990723 Fixed bug arising when file name had already been expanded so
          it really pointed to the static buffer used by expandFilename()
          which was overwritten by expandFilename() here
*/

FILE * fmyopen(char * name, char * mode)
{
	Symbolhandle   dataPaths = (Symbolhandle) 0;
	FILE           *trialFile;
	long            ipath, npaths, pathLength, nameLength;
	char           *pathName;
	char            path[PATHSIZE + 1];
	char            lastPath[PATHSIZE + 1];
	char            fileName[PATHSIZE + 1];

	strcpy(fileName, name);
	trialFile = fopen(fileName, mode);
	if (trialFile != (FILE *) 0 || strchr(mode, 'w') || isPath(fileName))
	{
		strcpy(LastPathOpened, fileName);
		return (trialFile);
	}

	if (((dataPaths = Lookup("DATAPATHS")) != (Symbolhandle) 0 ||
		 (dataPaths = Lookup("DATAPATH")) != (Symbolhandle) 0) &&
		TYPE(dataPaths) == CHAR)
	{
		char        *pathName1;

		pathName = STRINGPTR(dataPaths);
		npaths = symbolSize(dataPaths);
		nameLength = strlen(fileName);
		for (ipath = 0; ipath < npaths;
			 ipath++, pathName = skipStrings(pathName, 1))
		{
			pathName1 = expandFilename(pathName);
			
			if (pathName1 != (char *) 0)
			{
				pathLength = strlen(pathName1);
				if (okpathname(pathName1) &&
					pathLength + nameLength <= PATHSIZE)
				{
					strcpy(path, pathName1);
					if (pathLength > 0 && !isSlash(pathName1[pathLength-1]))
					{
						path[pathLength++] = DIRSEPARATOR[0];
					}
					if (pathLength + nameLength <= PATHSIZE)
					{
						strcpy(path + pathLength, fileName);
						if (okfilename(path) &&
							(trialFile = fopen(path,mode)) != (FILE *) 0)
						{
                					strcpy(LastPathOpened, path);
							break;
						}
					} /*if (pathLength + nameLength <= PATHSIZE)*/
				} /*if (okpathname(pathName1) && pathLength+nameLength <= PATHSIZE)*/
			} /*if (pathName1 != (char *) 0)*/
		} /*for (ipath = 0; ipath < npaths; ...)*/
		/* Kludge: Fixup expandFilename buffer if needed */
		if (pathName1 == name)
		{
			strcpy(name, fileName);
		}
		if (trialFile == (FILE *) 0 && isMvDirInfoSet(lastVol))
		{
			char         *lastDirectory =
  				getMvDirDirectory(getMvDirInfo(lastVol));

			pathLength = strlen(lastDirectory);
			if (pathLength + nameLength <= PATHSIZE)
			{
				strcpy(lastPath, lastDirectory);
				trialFile = tryFile(lastPath, fileName, mode);
        			if (trialFile != (FILE *) 0)
        			{
                			strcpy(LastPathOpened, lastPath);
        			}
			}
		} /*if (trialFile == (FILE *) 0 && isMvDirInfoSet(lastVol))*/

	}
	return (trialFile);
} /*fmyopen()*/


char * findFileByName(char * name, char * mode)
{
	Symbolhandle   dataPaths = (Symbolhandle) 0;
	FILE           *trialFile;
	long            ipath, npaths, pathLength, nameLength;
	char           *pathName;
	char            path[PATHSIZE + 1];
	char            expandedpath[PATHSIZE + 1];
	char            fileName[PATHSIZE + 1];

	strcpy(FullPathName, name);
	trialFile = fopen(FullPathName, mode);
	if (trialFile != (FILE *) 0)
	{
		fclose(trialFile);
		return (FullPathName);
	}

	if (((dataPaths = Lookup("DATAPATHS")) != (Symbolhandle) 0 ||
		 (dataPaths = Lookup("DATAPATH")) != (Symbolhandle) 0) &&
		TYPE(dataPaths) == CHAR)
	{
		char        *pathName1;

		pathName = STRINGPTR(dataPaths);
		npaths = symbolSize(dataPaths);
		nameLength = strlen(name);
		for (ipath = 0; ipath < npaths;
			 ipath++, pathName = skipStrings(pathName, 1))
		{
			pathName1 = expandFilename(pathName);
			
			if (pathName1 != (char *) 0)
			{
				strcpy(expandedpath,pathName1);
				pathLength = strlen(expandedpath);
				if (okpathname(expandedpath) &&
					pathLength + nameLength <= PATHSIZE)
				{
					strcpy(FullPathName, expandedpath);
					if (pathLength > 0 && !isSlash(expandedpath[pathLength-1]))
					{
						FullPathName[pathLength++] = DIRSEPARATOR[0];
					}
					if (pathLength + nameLength <= PATHSIZE)
					{
						strcpy(FullPathName + pathLength, name);
						if (okfilename(FullPathName) &&
							//(trialFile = fopen(path,mode)) != (FILE *) 0)
							(trialFile = fopen(FullPathName,mode)) != (FILE *) 0)
						{
							fclose(trialFile);
							return(FullPathName);
						}
					} /*if (pathLength + nameLength <= PATHSIZE)*/
				} /*if (okpathname(pathName1) && pathLength+nameLength <= PATHSIZE)*/
			} /*if (pathName1 != (char *) 0)*/
		} /*for (ipath = 0; ipath < npaths; ...)*/
	}
	return ((char *) 0);
} /*findFileByName()*/


static char FileNameFromFileSelector[CPCMAXPATHLENGTH+1];

char * cpcFindFile(char *fileName, char * message, char * defaultname,
				  unsigned long control, mvDirInfoPtr dirInfo)
{
	cpc_stddlg dlg;
	cpc_io  io;

	if (fileName[0] == '\0')
	{
		char         tempPath[CPCMAXPATHLENGTH+1], tempName[CPCMAXPATHLENGTH+1];
		char        *argPath = (char *) 0, *argName = (char *) 0;
		char        *dirName = NullString;

		if (defaultname && defaultname[0] != '\0')
		{
			io.type = CPC_IO_FILENAMEFROMPATH;
			strncpy(tempName, defaultname, CPCMAXPATHLENGTH);
			io.string = tempName;
			cpc_doio(&io);
			if (tempName[0] != '\0')
			{
				argName = tempName;
			}
			strncpy(tempPath, defaultname, CPCMAXPATHLENGTH);
			io.string = tempPath;
			io.type = CPC_IO_PATHONLY;
			cpc_doio(&io);
			dirName = tempPath;
		} /*if (defaultname && defaultname[0] != '\0')*/

		if (dirName[0] == '\0')
		{
			dirName = getMvDirDirectory(dirInfo);
		}
		if (dirName[0] != '\0')
		{
			strncpy(tempPath, dirName, CPCMAXPATHLENGTH);
			argPath = tempPath;
		}
		else
		{
			argPath = (char *) 0;
		}
		
		dlg.type = CPC_DLG_GETFILENAMEREAD;
		if(control & mvWriteit)
		{
			dlg.type = CPC_DLG_GETFILENAMESAVE;
		}
		dlg.message = message;
		dlg.param1 = argPath; /* default path */
		dlg.param2 = argName;
		dlg.param3 = NULL; /* extension */
		dlg.param4 = "*"; /* wildcard */
		if(cpc_dostddlg(&dlg) == CPC_ERROR_CANCELLED)
		{
			return((char *) 0);
		}
		argPath = dlg.string;

 		if(argPath == (char *) 0)
		{
			return(0);
		}
		strncpy(FileNameFromFileSelector, argPath, CPCMAXPATHLENGTH);
		fileName = FileNameFromFileSelector;
		if (dirInfo != (mvDirInfoPtr) 0)
		{
			char         thisDir[CPCMAXPATHLENGTH + 1];
		
			splitPathName(0, FileNameFromFileSelector,
						  thisDir, CPCMAXPATHLENGTH + 1);
			if (thisDir[0] != '\0')
			{
				setMvDirDirectory(dirInfo, thisDir);
				copyMvDirInfo(lastVol, dirInfo);
			}
		} /*if (dirInfo != (mvDirInfoPtr) 0)*/
	} /*if (fileName[0] == '\0')*/
	return(fileName);
} /*cpcFindFile()*/

static char DirNameFromDirSelector[CPCMAXPATHLENGTH+1];

char * cpcFindDir(char * message, char * defaultdir)
{
	cpc_stddlg dlg;
	cpc_io  io;

	char         tempPath[CPCMAXPATHLENGTH+1], tempName[CPCMAXPATHLENGTH+1];
	char        *argPath = (char *) 0, *argName = (char *) 0;
	char        *dirName = NullString;

	dlg.type = CPC_DLG_GETDIRNAME;
	dlg.message = message;
	dlg.param1 = argPath; /* default path */
	if(cpc_dostddlg(&dlg) == CPC_ERROR_CANCELLED)
	{
		return((char *) 0);
	}
	argPath = dlg.string;

 	if(argPath == (char *) 0)
	{
		return(0);
	}
	strncpy(DirNameFromDirSelector, argPath, CPCMAXPATHLENGTH);
	return(DirNameFromDirSelector);
} /*cpcFindDir()*/


void incrementNLINES(char *msg)
{
	while (*msg != '\0')
	{
		if (*msg++ == '\n')
		{
			NLINES++;
		}
	}
} /*incrementNLINES()*/

int readFromMouse(double x[], double y[],
             int op, long * length)
{
	cpc_sg  sg;
	int i,graphno;

	sg.type = CPC_SG_CHOOSEFRAME;
	sg.string = "Select frame for mouse input";
	if(cpc_dosg(&sg) == CPC_ERROR_MOUSE)
	{
		return(MouseNoGraphs);  // only error is for no graph windows
	}
	if(sg.type == CPC_SG_MOUSEQUIT)
	{
		return(MouseAbort);  // user pressed q
	}
	if(sg.type == CPC_SG_MOUSEINTERRUPT)
	{
		return(MouseInterrupt);  // user pressed q
	}
	graphno = sg.graphno;
	

	if(op == getPoint)
	{
		i = 0;
		while(i < *length)
		{
			sg.type = CPC_SG_MOUSEPOINT;
			sg.graphno = graphno;
			sg.string = "click the mouse to select a point";
			cpc_dosg(&sg);
			
			if(sg.type == CPC_SG_MOUSEQUIT)
			{
				sg.type = CPC_SG_CLEARMOUSETRACKS;
				cpc_dosg(&sg);
				return(MouseAbort);  // user pressed q
			}
			if(sg.type == CPC_SG_MOUSEINTERRUPT)
			{
				sg.type = CPC_SG_CLEARMOUSETRACKS;
				cpc_dosg(&sg);
				return(MouseInterrupt);  // user pressed q
			}
			if(sg.type == CPC_SG_MOUSELEAVE)
			{
				i = 0;
				sg.type = CPC_SG_CLEARMOUSETRACKS;
				cpc_dosg(&sg);
			}
			if(sg.type == CPC_SG_MOUSEPOINT)
			{
				x[i] = sg.dparms[0];
				y[i] = sg.dparms[1];
				i++;
			}
		}
		sg.type = CPC_SG_CLEARMOUSETRACKS;
		cpc_dosg(&sg);
		return(MouseNormalReturn);
	}


	if(op == getLine)
	{
		i = 0;
		while(i < *length+1)
		{
			if(i == 0)
			{
				sg.type = CPC_SG_MOUSEPOINT;
				sg.graphno = graphno;
				sg.string = "click the mouse to select a point";
				cpc_dosg(&sg);
				
				if(sg.type == CPC_SG_MOUSEQUIT)
				{
					sg.type = CPC_SG_CLEARMOUSETRACKS;
					cpc_dosg(&sg);
					return(MouseAbort);  // user pressed q
				}
				if(sg.type == CPC_SG_MOUSEINTERRUPT)
				{
					sg.type = CPC_SG_CLEARMOUSETRACKS;
					cpc_dosg(&sg);
					return(MouseInterrupt);  // user pressed q
				}
				if(sg.type == CPC_SG_MOUSELEAVE)
				{
					i = 0;
					sg.type = CPC_SG_CLEARMOUSETRACKS;
					cpc_dosg(&sg);
					continue;
				}
				if(sg.type == CPC_SG_MOUSEPOINT)
				{
					x[i] = sg.dparms[0];
					y[i] = sg.dparms[1];
					i++;
					sg.dparms[2] = sg.dparms[0];
					sg.dparms[3] = sg.dparms[1];
					sg.coords[2] = sg.coords[0];
					sg.coords[3] = sg.coords[1];
				}
			}

			sg.type = CPC_SG_MOUSESEGMENT;
			sg.string = "click the mouse to select the segment";
			cpc_dosg(&sg);
			
			if(sg.type == CPC_SG_MOUSEQUIT)
			{
				sg.type = CPC_SG_CLEARMOUSETRACKS;
				cpc_dosg(&sg);
				return(MouseAbort);  // user pressed q
			}
			if(sg.type == CPC_SG_MOUSEINTERRUPT)
			{
				sg.type = CPC_SG_CLEARMOUSETRACKS;
				cpc_dosg(&sg);
				return(MouseInterrupt);  // user pressed q
			}
			if(sg.type == CPC_SG_MOUSELEAVE)
			{
				sg.type = CPC_SG_CLEARMOUSETRACKS;
				cpc_dosg(&sg);
				i = 0;
			}
			if(sg.type == CPC_SG_MOUSESEGMENT)
			{
				x[i] = sg.dparms[2];
				y[i] = sg.dparms[3];
				i++;
				sg.dparms[0] = sg.dparms[2];
				sg.dparms[1] = sg.dparms[3];
				sg.coords[0] = sg.coords[2];
				sg.coords[1] = sg.coords[3];
			}
		}
		*length = *length + 1;
		sg.type = CPC_SG_CLEARMOUSETRACKS;
		cpc_dosg(&sg);
		return(MouseNormalReturn);
	}

	if(op == getBox)
	{
		while(1)
		{
			sg.type = CPC_SG_MOUSEPOINT;
			sg.graphno = graphno;
			sg.string = "click the mouse to select a corner";
			cpc_dosg(&sg);
			
			if(sg.type == CPC_SG_MOUSEQUIT)
			{
				sg.type = CPC_SG_CLEARMOUSETRACKS;
				cpc_dosg(&sg);
				return(MouseAbort);  // user pressed q
			}
			if(sg.type == CPC_SG_MOUSEINTERRUPT)
			{
				sg.type = CPC_SG_CLEARMOUSETRACKS;
				cpc_dosg(&sg);
				return(MouseInterrupt);  // user pressed q
			}
			if(sg.type == CPC_SG_MOUSELEAVE)
			{
				sg.type = CPC_SG_CLEARMOUSETRACKS;
				cpc_dosg(&sg);
				continue;
			}
			if(sg.type == CPC_SG_MOUSEPOINT)
			{
				x[0] = sg.dparms[0];
				y[0] = sg.dparms[1];
				sg.dparms[2] = sg.dparms[0];
				sg.dparms[3] = sg.dparms[1];
				sg.coords[2] = sg.coords[0];
				sg.coords[3] = sg.coords[1];
			}

			sg.type = CPC_SG_MOUSEBOX;
			sg.string = "click the mouse to select the other corner";
			cpc_dosg(&sg);
			
			if(sg.type == CPC_SG_MOUSEQUIT)
			{
				sg.type = CPC_SG_CLEARMOUSETRACKS;
				cpc_dosg(&sg);
				return(MouseAbort);  // user pressed q
			}
			if(sg.type == CPC_SG_MOUSEINTERRUPT)
			{
				sg.type = CPC_SG_CLEARMOUSETRACKS;
				cpc_dosg(&sg);
				return(MouseInterrupt); 
			}
			if(sg.type == CPC_SG_MOUSELEAVE)
			{
				sg.type = CPC_SG_CLEARMOUSETRACKS;
				cpc_dosg(&sg);
				continue;
			}
			if(sg.type == CPC_SG_MOUSEBOX)
			{
				x[2] = sg.dparms[2];
				y[2] = sg.dparms[3];
				break;
			}
		}
		x[1] = x[0];
		x[3] = x[2];
		x[4] = x[0];
		y[1] = y[2];
		y[3] = y[0];
		y[4] = y[0];
		*length = 5;
		sg.type = CPC_SG_CLEARMOUSETRACKS;
		cpc_dosg(&sg);
		return(MouseNormalReturn);
	}

	return(MouseAbort);
}

char **mygetselection(void)
{
	char  **hand,*ptr;
	long    i;
	cpc_io  io;

	io.type = CPC_IO_PRIMARYSELECTION;
	io.length = 1; // use selection
	cpc_doio(&io);

	io.type = CPC_IO_GETCLIPTEXTLENGTH;
	cpc_doio(&io);
	hand = mygethandle(io.length + 1);
	if(hand == (char **) 0)
	{
		io.type = CPC_IO_PRIMARYSELECTION;
		io.length = 1; // put back to clipboard
		cpc_doio(&io);
		return(hand);
	}
	for(i=0;i<=io.length;i++)
	{
		(*hand)[i] = '\0';
	}
	io.type = CPC_IO_GETCLIPTEXT;
	io.length = io.length+1;
	io.string = *hand;
	cpc_doio(&io);

	io.type = CPC_IO_PRIMARYSELECTION;
	io.length = 1; // put back to clipboard
	cpc_doio(&io);

	return(hand);
}

char **mygetscrap(void)
{
	char  **hand,*ptr;
	long    i;
	cpc_io  io;

	io.type = CPC_IO_GETCLIPTEXTLENGTH;
	cpc_doio(&io);
	hand = mygethandle(io.length + 1);
	if(hand == (char **) 0)
	{
		return(hand);
	}
	for(i=0;i<=io.length;i++)
	{
		(*hand)[i] = '\0';
	}
	io.type = CPC_IO_GETCLIPTEXT;
	io.length = io.length+1;
	io.string = *hand;
	cpc_doio(&io);
	return(hand);
}

int myputselection(char **hand)
{
	cpc_io  io;

	io.type = CPC_IO_PRIMARYSELECTION;
	io.length = 1; // use selection
	cpc_doio(&io);

	io.type = CPC_IO_SETCLIPTEXT;
	io.string = *hand;
	cpc_doio(&io);

	io.type = CPC_IO_PRIMARYSELECTION;
	io.length = 0; // put back to clipboard
	cpc_doio(&io);
	return(1);
}

int myputscrap(char **hand)
{
	cpc_io  io;

	io.type = CPC_IO_SETCLIPTEXT;
	io.string = *hand;
	cpc_doio(&io);
	return(1);
}

extern long HISTORY;

enum historyScratch
{
	GLINES = 0,
	NTRASH
};

long getSomeHistory(long nlines, char *** historyHandle)
{
	char         **lines;
	long           i, needed,SavedLines;
	char          *place;
	cpc_io         io;
	TRASH(NTRASH, errorExit);

	io.type = CPC_HS_GETHISTORYLENGTH;
	cpc_hist(&io);
	SavedLines = io.length;
	
	if (SavedLines <= 0)
	{
		nlines = 0;
	}
	else if (nlines == 0 || nlines >= SavedLines)
	{
		nlines = SavedLines;
	}
	
	if (nlines == 0)
	{
		needed = 1;
	}
	else
	{
		needed = 0;
		for (i = 1; i <= nlines; i++)
		{
			io.type = CPC_HS_GETHISTORYLINE;
			io.parm = i;
			io.length = 0;
			io.string = "";
			cpc_hist(&io); /* returns error, but also required length */
			needed += io.length + 1;
		}
	}
	
	if (!getScratch(lines, GLINES, needed, char))
	{
		goto errorExit;
	}
	
	place = *lines;

	if (nlines == 0)
	{
		nlines = 1;
		place[0] = '\0';
	}
	else
	{
		for (i = nlines; i > 0; i--)
		{
			io.type = CPC_HS_GETHISTORYLINE;
			io.parm = i;
			io.length = 0;
			io.string = "";
			cpc_hist(&io); /* returns error, but also required length */
			io.type = CPC_HS_GETHISTORYLINE;
			io.length = needed;
			io.string = place;
			io.parm = i;
			cpc_hist(&io);
			place = copyStrings(io.string, place, 1);
			needed = needed - strlen(io.string) - 1;
		}
	}
	unTrash(GLINES);
	*historyHandle = lines;
	emptyTrash();
	return (nlines);
		
  errorExit:
	return (0);
} /*getSomeHistory()*/

void splitPathName(int nameOnly, char fullPathName [], char pathPart [],
                                   int partLength)
{
        int        n = strlen(fullPathName);
        char       separator = DIRSEPARATOR[0];
        char      *tail, *pieceWanted;
 
        for (tail = fullPathName + n; *tail != separator && tail >= fullPathName;
                 n--, tail--)
        {
                ;
        }
        tail++;
        n++;
 
        if (nameOnly)
        {
                pieceWanted = tail;
                n = strlen(tail);
        }
        else
        {
                pieceWanted = fullPathName;
        }
        if (n >= partLength)
        {
                n = partLength - 1;
        }
 
        strncpy(pathPart, pieceWanted, n);
        pathPart[n] = '\0';
} /*splitPathName()*/

void setSomeHistory(long nlines, char ** commands)
{
	long         i, history = HISTORY;
 	int          skippedlines = (nlines > HISTORY) ? nlines - HISTORY : 0;
	cpc_io       io;

	io.type = CPC_HS_RESETHISTORY;
	cpc_hist(&io);

	for (i = 0; i < nlines; i++, skippedlines++)
	{
		io.type = CPC_HS_ADDTOHISTORY;
		io.string = skipStrings(*commands, skippedlines);
		cpc_hist(&io);
	}
} /*setSomeHistory()*/


char *findPlotFile(plotKeyValuesPtr keyValues, char * kind)
{
return(keyValues->file);
#if 0
	char        prompt[40], defaultName[20];
	char       *wxFileName;
	int         which;
	int         newFile = keyValues->newFile;
	char       *fileName = keyValues->file;

	if (keyValues->epsf)
	{
		which = epsfFile;
	}
	else if (keyValues->ps)
	{
		which = psFile;
	}
	else if (keyValues->dumb)
	{
		which = dumbFile;
	}
	else
	{ /*should not happen on WX*/
		which = pictFile;
	}
	
	if (fileName[0] == '\0')
	{
		if (PlotFileCount[which] == 0 || newFile)
		{
			PlotFileCount[which]++;
		}
		sprintf(prompt, "%s File...", Description[which][fileDescription]);
		sprintf(defaultName, "%s%02d.%s", kind, PlotFileCount[which],
				Description[which][fileExtension]);
	}
	else
	{
		prompt[0] = defaultName[0] = '\0';
	}

	wxFileName = wxFindFile(fileName, prompt, defaultName, 0L,
							getMvDirInfo(plotVol));
	if (wxFileName == (char *) 0)
	{ /* Canceled */
		if (PlotFileCount[which] == 1 || newFile)
		{
			PlotFileCount[which]--;
		}
	}
	return (wxFileName);
#endif
} /*findPlotFile()*/
	
