/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 5.01 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*                                                                 ^F
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

struct plotKeyValues;
typedef struct plotKeyValues plotKeyValues, *plotKeyValuesPtr;


#include <new>
#include "mvcpcdefs.h"
#include "wx/wx.h"
#include "wx/cmdline.h"
#include "wx/ffile.h"

extern "C"{
#include "cpcIface.h"
#include <stdio.h> 
#include <string.h>
#include <unistd.h>
#include "cpcinterface.h"
#include "../globals.h"
}



#define RECOGNIZEBANG
#define BANG   '!'
#define INTNOTSET       0x00000  /* no interrupt in progress */
#define INTSET          0x10000  /* interrupt occurred */
#define FPESET          0x20000  /* FP error occurred */
#define PRINTABORT      0x40000  /* printing has been aborted */

extern wxString CpcHomePath, CpcUserPath, CpcAppPath, CpcAppPathOption;
extern "C" {
        char * findHome(void);
        char * findUserFiles(void);
        char * findAppFiles(void);
}


#ifndef FALSE
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE 1
#endif


#define MOTDMARKER       "!!!!" /* Marker for Message of the day in Help file*/ 
#define MOTDMARKERLENGTH  4  


//int   UseWindows;

extern long        INTERRUPT;
static wxArrayString CommandArgBuffers; 

 
char ** ArgV;

extern "C" {
extern int ProcessInput(const char *);
}

static int doBatch(char * fileName, int echo, char * prompt)
{
	wxString        command;


 /* dummy up MacAnova 'batch(filename,echo)' command */

	command = "batch(\"";
	command.Append(fileName);
	if(prompt)
	{
		command.Append("\",prompt:\"");
		command.Append(prompt);
		command.Append("\")");
	}
	else
	{
		if(echo) 
		{
			command.Append("\",echo:T)");
		}
		else
		{
			command.Append("\",echo:F)");
		}
	}
	ProcessInput(command.c_str());
	return (1);
} /*doBatch()*/

/*
	Dummy up restore(fileName) command and execute it.
*/

static int     doRestore(char * fileName)
{
	Symbolhandle list;
	long         typeList[1];
	char        *valueList[1];
	char        *keyList[1];
	FILE        *restoreFile;
	int          errorFlag = 0;

	restoreFile = fmyopen(fileName,"r");
	if (restoreFile == (FILE *) 0)
	{
		if (PRINTWARNINGS)
		{
			myprint("WARNING: unable to open file ");
			putOutErrorMsg(fileName);
		}
		errorFlag = 1;
	}
	else
	{
		fclose(restoreFile);

		/* dummy up restore(filename) */
		typeList[0] = CHAR;
		valueList[0] = fileName;
		keyList[0] = "";
		list = Buildlist(1,typeList,valueList,keyList);
		if (list == (Symbolhandle) 0)
		{
			errorFlag = 1;
		}
		else
		{
			if (restore(list) == (Symbolhandle) 0)
			{
				errorFlag = 1;
			}
			Removelist(list);
		}
		if (errorFlag && PRINTWARNINGS)
		{
			myprint("WARNING: unable to restore workspace from file ");
			putOutErrorMsg(fileName);
		}
	}
	return (errorFlag);
} /*doRestore()*/


#undef LF
#undef CR
#undef NL
#define LF            10
#define NL            10
#define CR            13      

extern "C" {
extern unsigned int inputReady(const char *, void *, int);
extern int DoStats(char *);
extern int ProcessInput(const char *);
extern void cleanitup(void);
extern void interruptCleanUp(void);
extern void saveWorkspace(unsigned int);
extern void restoreWorkspace(void);
extern void aboutFunction(void);
extern void helpFunction(const char *);
extern int doQuitCleanUp(int);
extern int doInitialize(int, char **);

void interruptCleanUp(void)
{
	NERRORS++;
        if (BDEPTH > 0)
        {
            putOutErrorMsg("ERROR: batch file(s) terminated");
            closeBatch(1); /* shut down all batch files*/
        }
}

int cpc_initcallbacks(int argc, char ** argv)
{

	cpc_callback     cb;

	cb.type = CPC_CB_INPUTREADY;
	cb.callback.inputReady = inputReady;
	cpc_setCallback(&cb);

	cb.type = CPC_CB_PROCESSCOMMAND;
	cb.callback.processCommand = ProcessInput;
	cpc_setCallback(&cb);

	cb.type = CPC_CB_CLEANUP;
	cb.callback.cleanUp = cleanitup;
	cpc_setCallback(&cb);

	cb.type = CPC_CB_INTERRUPTCLEANUP;
	cb.callback.interruptCleanUp = interruptCleanUp;
	cpc_setCallback(&cb);

	cb.type = CPC_CB_QUITCLEANUP;
	cb.callback.quitCleanUp = doQuitCleanUp;
	cpc_setCallback(&cb);

	cb.type = CPC_CB_INITIALIZER;
	cb.callback.initializer = doInitialize;
	cpc_setCallback(&cb);

	UseWindows = 1;
	ArgV = argv;

	return(CPC_NOERROR);
}

extern void cpcinitialize(long maxModelStackDepth, char ** searchPaths,  char ** helpFiles,    
        char ** macroFiles,   char ** dataFiles,    char *  defaultHelpFile,  
        char * homePathName, char * prompt);

extern MacAnovaStartupArgs *ProcessArgs(int, char **);


int doInitialize(int argc, char ** argv)
{

	int              i;
	FILE            *batchFile;
	char            *filename,*afmPath;
	wxString         afmdir;
	wxString         commandstr;
	long		 screenwidth;
	wxString menuXML;
	char *menufile;
	char *menubuf = (char *) 0;

	cpc_callback     cb;
	cpc_io           io;
	cpc_sg           sg;

	MacAnovaStartupArgs *startupArgs;

	FatalError = 0;

	std::set_new_handler(0); // 960826 return NULL on "new" failure

	myInitHandles(); // initializes memory management

	startupArgs = ProcessArgs(argc,argv);
	if (startupArgs == NULL)
	{
		return(CPC_ERROR_INITFAILURE);
	}

	/* allocate buffer for building messages and other strings*/
	OUTSTR = mygetpointer(BUFFERLENGTH * sizeof(char));
	if (OUTSTR == (char *) 0)
	{
		goto notAllocated;
	}

	screenwidth = startupArgs->ncols;
	if(startupArgs->appDir != (char *) 0)
	{
		CpcAppPathOption = startupArgs->appDir;
	}
	if(startupArgs->homePathName == (char *) 0)
	{
		startupArgs->homePathName = findHome();
	}
	if(startupArgs->initFiles[0] == (char *) 0)
	{
		startupArgs->initFiles[0] = MACANOVAINI;
	}
	// add standard search paths
	i=0;
	while(startupArgs->searchPaths[i] && i < MAXPATHS - 2)
	{
		i++;
	}
	startupArgs->searchPaths[i++] = findUserFiles();
	startupArgs->searchPaths[i] = findAppFiles();
	
	

	if(startupArgs->historyLength >= 0)
	{
		io.type = CPC_HS_SETHISTORYLENGTH;
		io.length = startupArgs->historyLength;
		cpc_hist(&io);
	}

	ISATTY = ITTYIN | ITTYOUT; /* Both input and output just for safety */

#if 0
	if (!isatty(0) || !isatty(1))
	{
		io.type = CPC_IO_ALERT;
		io.string = "This version of MacAnova does not process redirected input or output;\nUse a non-windowed version";
		cpc_doio(&io);
		return(CPC_ERROR_INITFAILURE);
	}
#endif

	INTERRUPT = INTNOTSET;
	
	UseWindows = 1;
	
	/*
	   Initialize symbol table
	   Note: UseWindows == 0 at this point suppressing any attempt to
	   use a window that has not yet been created
	   */

	MvDirectories =
	  (mvDirInfoPtr) mygetpointer(nVolumeCodes * sizeof(mvDirInfo));
	if (MvDirectories == (mvDirInfoPtr) 0)
	{
		goto notAllocated;
	}
	allocateLINE(LINECHUNK+1);

	cpcinitialize(3, startupArgs->searchPaths,  startupArgs->helpFiles,    
        startupArgs->macroFiles,   startupArgs->dataFiles,    startupArgs->defaultHelpFile,  
        startupArgs->homePathName, startupArgs->commandPrompt);

	if (FatalError)
	{
		goto notAllocated;
	}

	// MacAnova search paths are now set up, so we can go back and
	// sort out the types of the unknownFiles
	i = 0;
	while(startupArgs->unknownFiles[i])
	{
		char readbuffer[25];
		char **dest;
		int loc;
		wxString uFile;
		filename = findFileByName(startupArgs->unknownFiles[i++],"r");

		if(!filename)
		{
			wxString out;
			out << "Could not find file " << startupArgs->unknownFiles[i-1];
			myAlert((char *)out.c_str());
		}
		uFile = filename;
		wxFFile ffile(filename,"r");
		readbuffer[0] = 0;
		if(ffile.IsOpened())
		{
			ffile.Read(readbuffer,20);
			readbuffer[20] = 0;
		}

		// if the name ends .mvsave or if the file has the MacAnova
		// savefile signature first line, treat it as the restore file
		if( ((loc=uFile.Index(".mvsave")) >= 0 && 
			loc == (int) uFile.Length()-7) ||
		   (!strncmp(readbuffer,"////MacAnovaSaveFile",20)) )
		{
#ifdef BASE_MICROSOFT
			// filename gets put in a MacAnova quoted string
			// so backslashes must be doubled
			uFile.Replace("\\","\\\\");
#endif /*BASE_MICROSOFT*/
			if(startupArgs->restoreFileName != (char *) 0)
			{
				myAlert("Restore file specified more than once;\nonly the last one used");
			}
			CommandArgBuffers.Add(uFile);
                        startupArgs->restoreFileName  = (char *) CommandArgBuffers.Last().c_str();
		}
		// batch file if name ends .mvbat, .batch, .bat, .batch.txt, or .bat.txt
		// or if the file starts #!mvBatch
		else if( ((loc=uFile.Index(".batch")) >=0 && loc == (int) uFile.Length()-6) ||
		         ((loc=uFile.Index(".batch.txt")) >= 0 && loc == (int) uFile.Length()-10) ||
		         ((loc=uFile.Index(".bat")) >= 0 && loc == (int) uFile.Length()-4) ||
		         ((loc=uFile.Index(".bat.txt")) >= 0 && loc == (int) uFile.Length()-8) ||
		         ((loc=uFile.Index(".mvbat")) >= 0 && loc == (int) uFile.Length()-6) ||
		         (!strncmp(readbuffer,"#!mvBatch",9)) )
		{
			int j;
#ifdef BASE_MICROSOFT
			// filename gets put in a MacAnova quoted string
			// so backslashes must be doubled
			uFile.Replace("\\","\\\\");
#endif /*BASE_MICROSOFT*/
			
			dest = startupArgs->batchFiles;
			for(j=0;dest[j] && j < MAXPATHS; j++)
			{
				;;
			}
			if(j == MAXPATHS)
			{
				myAlert("Too many batch files specified; extras ignored");
			}
			else
			{
				CommandArgBuffers.Add(uFile);
				dest[j] = (char *) CommandArgBuffers.Last().c_str();
			}
		}
		else
		{
			// otherwise treat as a command file to open in a window
			int j;
			dest = startupArgs->openFiles;
			for(j=0;dest[j] && j < MAXPATHS; j++)
			{
				;;
			}
			if(j == MAXPATHS)
			{
				myAlert("Too many command files specified; extras ignored");
			}
			else
			{
				CommandArgBuffers.Add(filename);
				dest[j] = (char *) CommandArgBuffers.Last().c_str();
				startupArgs->quiet = 1;
			}
		}
	}

	if(!startupArgs->quiet && (filename = findFileByName("Welcome.gif","r")) != (char *) 0)
	{
		cpc_io io;
		io.type = CPC_IO_SPLASH;
		io.parm = CPC_IO_GIFTYPE;
		io.string = filename;
		cpc_doio(&io);
	}

	if(startupArgs->cpcConfigFile != (char *) 0)
	{
		filename = findFileByName(startupArgs->cpcConfigFile,"r");
	}
	else
	{
		filename = findFileByName("macanova.config","r");
	}
	if(filename != (char *) 0)
	{
		cb.type = CPC_CB_SETPREFSFILEPATH;
		cb.callback.string = filename;
		cpc_setCallback(&cb);
	}


	i = 0;
	menufile = findFileByName("MacAnova.menus.txt","r");
	if(menufile != (char *) 0) 
	{
		wxFFile menuFFile(menufile,"r");
		if(menuFFile.IsOpened()) 
		{
			menuFFile.ReadAll(&menuXML);
			menubuf = (char *) menuXML.c_str();
		}
	}
	while(startupArgs->openFiles[i])
	{
		io.type = CPC_IO_NEWWINDOW;
		//io.string = MACANOVA_DEFAULT_MENU_RESOURCE;
		io.string = menubuf;
		io.string2 = "mainmenu_split";
		cpc_doio(&io);
		io.type = CPC_IO_SETCMDWINDFROMFILE;
		io.string = startupArgs->openFiles[i];
		cpc_doio(&io);
		i++;
	}
	if(i == 0)
	{ // no open files
		io.type = CPC_IO_NEWWINDOW;
		//io.string = MACANOVA_DEFAULT_MENU_RESOURCE;
		io.string = menubuf;
		io.string2 = "mainmenu_split";
		cpc_doio(&io);
	}

#if defined(USEA4PAPER)
	io.type = CPC_IO_USEA4PAPER;
	cpc_doio(&io);
#endif /*defined(USEA4PAPER)*/

	io.type = CPC_IO_PRIMARYSELECTION;
	//io.length = 1;
	io.length = 0;
	cpc_doio(&io);

	
#ifdef BASE_MICROSOFT
	afmPath = findFileByName("afm\\Cour.afm","r");
#else
	afmPath = findFileByName("afm/Cour.afm","r");
#endif
	if(afmPath != (char *) 0)
	{
		//terminate before Cour.afm
		afmPath[strlen(afmPath)-8] = '\0';
		sg.type = CPC_SG_SETAFMPATH;
		sg.string = (char *) afmPath;
		cpc_dosg(&sg);
	}

	startupArgs->nlines = (startupArgs->nlines < 0) ? SCREENHEIGHT : startupArgs->nlines;
	startupArgs->ncols = (startupArgs->ncols < 0) ? SCREENWIDTH : startupArgs->ncols;
	installScreenDims(startupArgs->ncols, startupArgs->nlines);
	saveFormat();


	mvStartupMsg(startupArgs->quiet);

	/* try to open initialization and help files */

	/* make sure help file is found if there */
#ifndef BINARYHELPFILE
	HELPFILE = fopen(*HELPFILENAME,"r");
#else /*BINARYHELPFILE*/
	HELPFILE = fopen(*HELPFILENAME,"rb"); /* open in binary mode */
#endif /*BINARYHELPFILE*/
	
	if (!startupArgs->quiet && HELPFILE != (FILE *) 0 &&
	   (LINE = mygethandle(MAXLINE)) != (char **) 0)
	{
		/* look for message of the day preceded and followed by "!!!!"*/
		int       printit = 0;
		char     *msgEnd, *msgStart;
		char     *marker = MOTDMARKER;
		
		screenwidth = (SCREENWIDTH >= BUFFERLENGTH) ?
			BUFFERLENGTH - 1 : SCREENWIDTH;
		
		while (fillLINE(HELPFILE) != EOF)
		{
			msgStart = *LINE;
			if (strncmp(msgStart, marker, MOTDMARKERLENGTH) == 0)
			{
				if (printit)
				{
					break;
				}
				else
				{
					printit = 1;
					continue;
				}
			} /*if (strncmp(msgStart, marker, MOTDMARKERLENGTH) == 0)*/
			
			if (printit)
			{
				while (*msgStart == ' ')
				{
					msgStart++;
				}
				msgEnd = msgStart + strlen(msgStart) - 1;
				while (*msgEnd == ' ' && msgEnd > msgStart)
				{
					msgEnd--;
				}
				if (msgEnd[0] != '\n')
				{
					msgEnd++;
				}
				*msgEnd = '\0';
				
				(void) centerBuffer(OUTSTR, msgStart, screenwidth);
				if (printit == 1)
				{
					myeol();
					printit = 2;
				}
				putOUTSTR();
			} /*if (printit)*/
		} /*while (fillLINE(HELPFILE) != EOF)*/
		rewind(HELPFILE);
		mydisphandle(LINE);
		LINE = (char **) 0;
	}

	if(startupArgs->commandPrompt != (char *) 0)
	{
		startupArgs->commandPrompt = "Cmd> ";
	}
	io.type = CPC_IO_SETPROMPT;
	io.string = startupArgs->commandPrompt;
	cpc_doio(&io);
	
	if (startupArgs->restoreFileName != (char *) 0)
	{
		if (doRestore(startupArgs->restoreFileName))
		{
			if (PRINTWARNINGS)
			{
				putPieces("WARNING: cannot restore workspace from file ",
						  startupArgs->restoreFileName, (char *) 0, (char *) 0);
			}				
			startupArgs->restoreFileName = (char *) 0;
		} /*if (doRestore(startupArgs->restoreFileName))*/
	} /*if (startupArgs->restoreFileName != (char *) 0)*/
	
	
	/* init files execute silently */
	i = 0;
	io.type = CPC_IO_SWALLOWOUTPUT;
	io.parm = CPC_SWALLOW_BOTH;
	cpc_doio(&io);
	while(startupArgs->initFiles[i])
	{ 
		commandstr = "batch(\"";
		commandstr << startupArgs->initFiles[i] << "\")\n";
		io.type = CPC_IO_PUSHCOMMAND;
		io.string = (char *) commandstr.c_str();
		io.length = 1; // hide command
		io.parm = 1; // hide output
		cpc_doio(&io);
		//(void) doBatch(startupArgs->initFiles[i],0, NullString);
		i++;
	}
	io.type = CPC_IO_SWALLOWOUTPUT;
	io.parm = CPC_SWALLOW_NOTHING;
	cpc_doio(&io);

	i = 0;
	if(startupArgs->batchPrompt != (char *) 0)
	{
		io.type = CPC_IO_SETPROMPT;
		io.string = startupArgs->batchPrompt;
		cpc_doio(&io);
	}
	while(startupArgs->batchFiles[i])
	{
		commandstr = "batch(\"";
		commandstr << startupArgs->batchFiles[i] << "\")\n";
		io.type = CPC_IO_PUSHCOMMAND;
		io.string = (char *) commandstr.c_str();
		io.length = 0; // hide command
		io.parm = 0; // hide output
		cpc_doio(&io);
		//doBatch(startupArgs->batchFiles[i],1, startupArgs->batchPrompt);
		i++;
	}
	io.type = CPC_IO_SETPROMPT;
	io.string = startupArgs->commandPrompt;
	cpc_doio(&io);
		
	if (startupArgs->expression != (char *) 0)
	{
		myeol();
		sprintf(OUTSTR, "Command Line> %s", startupArgs->expression);
		putOUTSTR();

		ProcessInput(startupArgs->expression);
	} /*if (expression != 0*/

	return(CPC_NOERROR);
	
  notAllocated:
	strcpy(OUTSTR,
		   "FATAL ERROR: unable to allocate memory during inialization sequence");
	/* fall through*/

	if (*OUTSTR)
	{
		io.type = CPC_IO_ALERT;
		io.string = OUTSTR;
		cpc_doio(&io);
	}
	return(CPC_ERROR_INITFAILURE);
	
} /*doInitialize*/


#ifndef ENVNAME
#define ENVNAME "MACANOVA"
#endif /*ENVNAME*/    


extern "C"
{
#include <stdio.h>
#include "cpcinterface.h"
struct plotKeyValues;
typedef struct plotKeyValues plotKeyValues, *plotKeyValuesPtr;
#include "cpcIface.h"
#include "../mvIo.h"
#include "../platform.h"
#include "../globals.h"
#include "../mainpars.h"
#include "../mvsave.h"
#include "../mvGraphs.h"
}


unsigned int inputReady(const char * Text, void * frame, int lastCharOnly)
{
	int            ichar, inquotes, bracklvl, foundComment, sendit;
	long           textLength ;
	char           c;
#ifdef RECOGNIZEBANG
	int            foundbang;
#endif /*RECOGNIZEBANG*/
	cpc_io         io;

	textLength = strlen(Text);

	io.string = (char *) frame;
	io.type = CPC_IO_PARENMATCH;


#ifdef RECOGNIZEBANG
	foundbang = (Text[0] == BANG);
#endif /*RECOGNIZEBANG*/

#ifdef RECOGNIZEBANG
	ichar = textLength-1;
	if (foundbang && Text[ichar] == '\n')
	{
		if(ichar > 0 && Text[ichar-1] == '\\')
		{
			return(0);
		}
		return(1);
	}
#endif /*RECOGNIZEBANG*/

	inquotes = 0;
	bracklvl = 0;
	foundComment = 0;


	ichar = 0;
	while (ichar < textLength && ichar >= 0)
	{/* now check balance of '{' and '}', keeping track of quotes */

		sendit = !lastCharOnly || (ichar == (textLength -1));
		c = Text[ichar++];
		
		if (c == '\\')
		{
#ifdef __WIN32__
			if (Text[ichar] == '\r' && Text[ichar+1] == '\n')
			{
				ichar++;
			}
#endif //__WIN32__
			ichar++;
		} /*if (c == '\\')*/
		else if (!inquotes || c == '"')
		{
			switch((int) c)
			{
			  case '"':
				if (!foundComment)
				{
					if(inquotes)
					{
						io.string = (char *) frame;
						io.type = CPC_IO_PARENMATCH;
						io.length = ichar;
						io.parm = -'"';
						if(sendit)
						{
							cpc_doio(&io);
						}
					}
					else
					{
						io.string = (char *) frame;
						io.type = CPC_IO_PARENMATCH;
						io.length = ichar;
						io.parm = '"';
						if(sendit)
						{
							cpc_doio(&io);
						}
					}
					inquotes = !inquotes;
				}
				break;
			  case '{':
				if (!foundComment)
				{
					io.string = (char *) frame;
					io.type = CPC_IO_PARENMATCH;
					io.length = ichar;
					io.parm = '{';
					if(sendit)
					{
						cpc_doio(&io);
					}
					bracklvl++;
				}
				break;
			  case '}':
				if (!foundComment)
				{
					io.string = (char *) frame;
					io.type = CPC_IO_PARENMATCH;
					io.length = ichar;
					io.parm = '}';
					if(sendit)
					{
						cpc_doio(&io);
					}
					bracklvl--;
				}
				break;
			  case '(':
				if (!foundComment)
				{
					io.string = (char *) frame;
					io.type = CPC_IO_PARENMATCH;
					io.length = ichar;
					io.parm = '(';
					if(sendit)
					{
						cpc_doio(&io);
					}
				}
				if(ichar < textLength && Text[ichar] == ';')
				{
					io.string = (char *) frame;
					io.type = CPC_IO_SYNTAXERROR;
					io.length = ichar;
					if(sendit)
					{
						cpc_doio(&io);
					}
				}
				break;
			  case ')':
				if (!foundComment)
				{
					io.string = (char *) frame;
					io.type = CPC_IO_PARENMATCH;
					io.length = ichar;
					io.parm = ')';
					if(sendit)
					{
						cpc_doio(&io);
					}
				}
				break;
			  case '[':
				if (!foundComment)
				{
					io.string = (char *) frame;
					io.type = CPC_IO_PARENMATCH;
					io.length = ichar;
					io.parm = '[';
					if(sendit)
					{
						cpc_doio(&io);
					}
				}
				break;
			  case ']':
				if (!foundComment)
				{
					io.string = (char *) frame;
					io.type = CPC_IO_PARENMATCH;
					io.length = ichar;
					io.parm = ']';
					if(sendit)
					{
						cpc_doio(&io);
					}
				}
				break;
			  case '#':
				foundComment = 1;
				break;
			  case LF:
			  case CR:
				if ( bracklvl <= 0 )
				{
					return(1);
				}
				foundComment = 0;
			  default:
				break;
			} /*switch((int) c)*/
		} /*if (c == '\\'){...}else if (!inquotes || c == '"'){...}*/
	} /*while (ichar < textLength && ichar >= CmdStrPos)*/
	/* if we get to here, not ready yet */
	/* Fall through */

	return(0);
} /*inputReady()*/

int doQuitCleanUp(int canCancel)
{
	int response;
	cpc_stddlg  dlg;
	dlg.message = "Do you wish to save your workspace?";
	dlg.param1 = "Save workspace?";
	if(canCancel)
	{
		dlg.type = CPC_DLG_YESNOCANCELMESSAGE;
	}
	else
	{
		dlg.type = CPC_DLG_YESNOMESSAGE;
	}
	response = cpc_dostddlg(&dlg);

	if(response == CPC_IO_YES)
	{
		ProcessInput("save(\"\")\n");
	}
	return(response);

}

void saveWorkspace(unsigned int doSaveAs)
{
	if(doSaveAs)
	{
		ProcessInput("save(\"\")\n");
	}
	else
	{
		ProcessInput("save()\n");
	}
	
}

#if 0
#include "keylist.h"
//static char *keylistdirectory = "/HOME/faculty/gary/macanova/docs/help.to.latex.to.html/reference/";
static char *keylistdirectory = "/home/gary/macanova/reference.html/";

void helpFunction(const char * selection)
{
	char buffer[400];
	cpc_stddlg dlg;
	int i = 0;

	dlg.type = CPC_DLG_HELPFRAME;
	sprintf(buffer,"%s%s",keylistdirectory,"reference.htm");
	if(selection && selection[0] != 0)
	{
		i = 0;
		while(key_node_list[i] && strcmp(selection,key_node_list[i]))
		{
			i += 2;
		}
		if(key_node_list[i])
		{
			sprintf(buffer,"%s%s",keylistdirectory,key_node_list[i+1]);
		}
	}
	dlg.string = buffer;
//	cpc_dostddlg(&dlg);
}
#endif /* 0 */

} /*extern "c"*/
