/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 5.01 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*                                                                 ^F
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/




#include <math.h>
extern "C" {
#include "../globals.h"
#include "../mvGraphs.h"
#include "cpcinterface.h"
}


#define FALSE 0
#define TRUE 1



static unsigned long cpc_currentx,cpc_currenty,cpc_ymax,cpc_textv,cpc_texth,cpc_graphno;




extern "C" {
extern struct        termentry term_tbl[];
/*
   Entries cpc_init(), cpc_graphics(), cpc_text(), cpc_reset(), cpc_linetype(),
           cpc_move(), cpc_vector(), cpc_str_text(), cpc_chr_text(),
           for gnuplot use that would otherwise be in term.c
*/
int            cpc_init(void);
int            cpc_graphics(void);
void           cpc_text(void);
void           cpc_reset(void);
void           cpc_linetype(long);
void           cpc_move(unsigned long, unsigned long);
void           cpc_vector(unsigned long, unsigned long);
void           cpc_str_text(char *);
void           cpc_chr_text(char);
void           cpc_setpen(long /* linetype */, long /* color */, long /* width */);
void           cpc_setbrush(long /* color */, long /* style */);
void           cpc_setfont(long /* size */, long /* family */,
                        long /* style */, long /* weight */);
void           cpc_setforegroundcolor(long /*color*/);
void           cpc_setbackgroundcolor(long /*color*/);
void           cpc_drawline(unsigned long /* x1 */, unsigned long /* y1 */,
                        unsigned long /* x2 */, unsigned long /* y2 */);
void           cpc_drawtext(unsigned long /* x1 */, unsigned long /* y1 */,
                        char * /* string */, long /* style */, long /* angle */);
void           cpc_drawglyph(unsigned long /* x1 */, unsigned long /* y1 */,
                        long /*type*/, long /* style */, long /* size */);
int            cpc_textextent(char * /* string */);

int            cpc_psinit(void);
int            cpc_psgraphics(void);
void           cpc_pstext(void);

#define cpc_NLINETYPES 5 /* added 000811; previous implicitly was 5*/

#define cpc_SIZEX 5000
#define cpc_SIZEY 5000

//extern struct termentry  term_tbl[];

int            cpc_init(void)
{
	cpc_sg sg;
	struct termentry       *t = term_tbl + term;

	sg.type = CPC_SG_NEWGRAPH;
// FIXME  kludge until the gnuplot drawing is fixed
	sg.coords[0] = t->xmax+100; //cpc_SIZEX;
	sg.coords[1] = t->ymax;  //cpc_SIZEY;
	sg.graphno = ThisWindow+1;
	cpc_dosg(&sg);
	if (sg.graphno <= 0)
	{
		return(1);
	}
	cpc_graphno = sg.graphno;
	ThisWindow = sg.graphno - 1;

	sg.type = CPC_SG_SETFONT;
	sg.size = 120;
	sg.family = CPC_SG_SWISS;
	sg.style = CPC_SG_NORMAL;
	sg.weight = CPC_SG_NORMAL;
	cpc_dosg(&sg);

	sg.type = CPC_SG_SETBACKGROUNDNAME;
	sg.colorname = CPC_WHITE;
	cpc_dosg(&sg);

	sg.type = CPC_SG_SETFOREGROUNDNAME;
	sg.colorname = CPC_BLACK;
	cpc_dosg(&sg);

	sg.type = CPC_SG_NEWPENNAME;
	sg.colorname = CPC_FIREBRICK;
	sg.size = 10;
	sg.style = CPC_SG_SOLID;
	cpc_dosg(&sg);


// FIXME  This may not work, because the setfont command
// may not yet have actually been run (it gets run in the
// Draw routine (in Paint, for example).  We need to add
// some kind of forced Draw/Paint.
	sg.type = CPC_SG_DODRAW;
	cpc_dosg(&sg);
	sg.type = CPC_SG_GETCHARHEIGHT;
	cpc_dosg(&sg);
	t->v_char = (unsigned long) sg.size;
	sg.type = CPC_SG_GETCHARWIDTH;
	cpc_dosg(&sg);
	t->h_char = (unsigned long) sg.size;
	t->v_tic = t->h_char;
	t->h_tic = t->h_char;
	//t->xmax = cpc_SIZEX - t->h_tic;
	//t->ymax = cpc_SIZEY - 1;

	cpc_ymax = t->ymax;
	cpc_textv = t->v_char;
	cpc_texth = t->h_char;

	sg.type = CPC_SG_XPARMS;
	sg.style = GraphMap->logx ? CPC_SG_AXISLOG10 : CPC_SG_AXISLINEAR;
	sg.dparms[0] = GraphMap->xmin - ((double) GraphMap->xoffset - .5)/GraphMap->xscale;
	sg.dparms[1] = 1./GraphMap->xscale;
	cpc_dosg(&sg);

	sg.type = CPC_SG_YPARMS;
	sg.style = GraphMap->logy ? CPC_SG_AXISLOG10 : CPC_SG_AXISLINEAR;
	sg.dparms[0] = GraphMap->ymin - ((double) GraphMap->yoffset - .5)/GraphMap->yscale;
	sg.dparms[1] = 1./GraphMap->yscale;
	cpc_dosg(&sg);

	return(0);
} /*cpc_init()*/
  
int            cpc_graphics(void)
{
	return(0);
} /*cpc_graphics()*/

void           cpc_text(void)
{
	cpc_sg sg;
	sg.graphno = cpc_graphno;
	sg.type = CPC_SG_DOPAINT;
	cpc_dosg(&sg);

	if(PauseAfterPlot)
	{
		sg.type = CPC_SG_GRAPHICSPAUSE;
		cpc_dosg(&sg);
	}

} /*cpc_text()*/


int            cpc_psinit(void)
{
	cpc_sg sg;
	struct termentry       *t = term_tbl + term;

	sg.type = CPC_SG_OPENPOSTSCRIPTFILE;
	sg.string = *PLOTFILENAME;
	sg.style = (PLOTFILEMODE[0] == 'w') ? 0 : 1; // the append flag
	cpc_dosg(&sg);

	sg.type = CPC_SG_OPENPOSTSCRIPTPAGE;
	cpc_graphno = -1;
	sg.graphno = -1;
	sg.coords[0] = (t->xmax + 100); //cpc_SIZEX;
	sg.coords[1] = t->ymax; //cpc_SIZEY;
	cpc_dosg(&sg);

	sg.type = CPC_SG_SETFONT;
	sg.size = 120;
	sg.family = CPC_SG_SWISS;
	sg.style = CPC_SG_NORMAL;
	sg.weight = CPC_SG_NORMAL;
	cpc_dosg(&sg);

	sg.type = CPC_SG_SETBACKGROUNDNAME;
	sg.colorname = CPC_WHITE;
	cpc_dosg(&sg);

	sg.type = CPC_SG_SETFOREGROUNDNAME;
	sg.colorname = CPC_BLACK;
	cpc_dosg(&sg);

	sg.type = CPC_SG_NEWPENNAME;
	sg.colorname = CPC_FIREBRICK;
	sg.size = 10;
	sg.style = CPC_SG_SOLID;
	cpc_dosg(&sg);


	//sg.type = CPC_SG_GETCHARHEIGHT;
	//cpc_dosg(&sg);
	//t->v_char = (unsigned long) sg.size;
	t->v_char = 120;
	//sg.type = CPC_SG_GETCHARWIDTH;
	//cpc_dosg(&sg);
	//t->h_char = (unsigned long) sg.size;
	t->h_char = 80;
	t->v_tic = t->h_char;
	t->h_tic = t->h_char;
	//t->xmax = cpc_SIZEX - t->h_tic;
	//t->ymax = cpc_SIZEY - 1;

	cpc_ymax = t->ymax;
	cpc_textv = t->v_char;
	cpc_texth = t->h_char;

	return(0);
} /*cpc_init()*/
  
int            cpc_psgraphics(void)
{
	return(0);
} /*cpc_graphics()*/

void           cpc_pstext(void)
{
	cpc_sg sg;
	sg.graphno = -1;
	sg.type = CPC_SG_CLOSEPOSTSCRIPTPAGE;
	cpc_dosg(&sg);
	sg.type = CPC_SG_CLOSEPOSTSCRIPTFILE;
	cpc_dosg(&sg);

} /*cpc_text()*/

void           cpc_reset(void)
{
}

/*
  modified to use cpc_NLINETYPES, redid switch using actual line type
  values instead of lintype - XAXLISLINETYPE
*/

void           cpc_linetype(long linetype)
{
	int        heavy = (10*linetype >= UNITWIDTH);
	double     thickness = (heavy) ?
		(double) (MAXLINETYPE * (linetype/MAXLINETYPE)) /
			(double) UNITWIDTH : 1.0;
	int        Thickness =  (int) ceil(thickness*10);

	cpc_sg sg;

	Thickness = (Thickness > 100 ? 100 : Thickness); // just in case

	sg.type = CPC_SG_NEWPENNAME;
	sg.graphno = cpc_graphno;
	sg.colorname = CPC_BLUE;
	sg.size = Thickness;

	if (heavy)
	{
		linetype %= MAXLINETYPE;
	}
	if (linetype < MINLINETYPE)
	{
		linetype = BORDERLINETYPE;
	}
	linetype = ((linetype < 0) ? linetype: linetype % cpc_NLINETYPES);

	switch(linetype)
	{
	  case XAXISLINETYPE: // x-axis
	  case YAXISLINETYPE: // y-axis
	  case 3: // regular short dash
	  case 5: // regular short dash
	  case 6: // regular short dash
		sg.style = CPC_SG_SHORT_DASH;
		break;
	  case 1: // dotted
		sg.style = CPC_SG_LONG_DASH;
		break;
	  case 2: // dotted
		sg.style = CPC_SG_DOT_DASH;
		break;
	  case 4: // dotted
		sg.style = CPC_SG_DOT;
		break;
	  case BORDERLINETYPE: // border
	  case SOLIDLINETYPE: // regular solid
	  default:
		sg.style = CPC_SG_SOLID;
		break;
	} /*switch(linetype)*/

	cpc_dosg(&sg);

} /*cpc_linetype()*/

void           cpc_move(unsigned long x, unsigned long y)
{
	cpc_currentx = x;
	cpc_currenty = cpc_ymax-y;
} /*cpc_move()*/

void           cpc_vector(unsigned long x, unsigned long y)
{
	cpc_sg sg;
	sg.type = CPC_SG_DRAWLINE;
	sg.graphno = cpc_graphno;
	sg.coords[0] = cpc_currentx;
	sg.coords[1] = cpc_currenty;
	sg.coords[2] = x;
	sg.coords[3] = cpc_ymax - y;
	cpc_dosg(&sg);
	cpc_currentx = x;
	cpc_currenty = cpc_ymax-y;
} /*cpc_vector()*/

void           cpc_str_text(char * str)
{
	int       stringlen;
	cpc_sg sg;
	sg.type = CPC_SG_DRAWTEXT;
	sg.graphno = cpc_graphno;
	sg.coords[0] = cpc_currentx;
	sg.coords[1] = cpc_currenty;
	sg.string = str;
	sg.style = CPC_SG_LEFTBASELINE;
	stringlen = strlen(str);
	cpc_currentx += stringlen * cpc_texth;
	cpc_dosg(&sg);
} /*cpc_str_text()*/

void           cpc_chr_text(char c)
{
	char        str[2];

	str[0] = c;
	str[1] = 0;
	cpc_sg sg;
	sg.type = CPC_SG_DRAWTEXT;
	sg.graphno = cpc_graphno;
	sg.coords[0] = cpc_currentx;
	sg.coords[1] = cpc_currenty;
	sg.string = str;
	sg.style = CPC_SG_LEFTBASELINE;
	cpc_currentx +=  cpc_texth;
	cpc_dosg(&sg);
} /*cpc_chr_text()*/

void   cpc_setpen(long  linetype, long color, long width)
{
	cpc_sg sg;
	sg.type = CPC_SG_NEWPENNAME;
	sg.graphno = cpc_graphno;
	sg.size = width;
	sg.style = linetype;
	sg.colorname = color;
	cpc_dosg(&sg);
}

void cpc_setbrush(long color, long style)
{
	cpc_sg sg;
	sg.type = CPC_SG_NEWBRUSHNAME;
	sg.graphno = cpc_graphno;
	sg.style = style;
	sg.colorname = color;
	cpc_dosg(&sg);
}
void cpc_setfont(long size, long family, long style, long weight)
{
	cpc_sg sg;
	sg.graphno = cpc_graphno;
	sg.type = CPC_SG_SETFONT;
	sg.size = size;
	sg.family = family;
	sg.style = style;
	sg.weight = weight;
	cpc_dosg(&sg);
}

void cpc_setforegroundcolor(long color)
{
	cpc_sg sg;
	sg.graphno = cpc_graphno;
	sg.type = CPC_SG_SETFOREGROUNDNAME;
	sg.colorname = color;
	cpc_dosg(&sg);
}

void cpc_setbackgroundcolor(long color)
{
	cpc_sg sg;
	sg.graphno = cpc_graphno;
	sg.type = CPC_SG_SETBACKGROUNDNAME;
	sg.colorname = color;
	cpc_dosg(&sg);
}

void cpc_drawline(unsigned long x1, unsigned long y1,
                        unsigned long x2, unsigned long y2)
{
	cpc_sg sg;
	sg.type = CPC_SG_DRAWLINE;
	sg.graphno = cpc_graphno;
	sg.coords[0] = x1;
	sg.coords[1] = cpc_ymax - y1;
	sg.coords[2] = x2;
	sg.coords[3] = cpc_ymax - y2;
	cpc_dosg(&sg);
	cpc_currentx = x2;
	cpc_currenty = cpc_ymax-y2;
}

void cpc_drawtext(unsigned long x1, unsigned long y1,
                        char * string, long style, long angle)
{
	cpc_sg sg;
	if(angle == 90)
	{
		sg.type = CPC_SG_DRAWTEXTUP;
	}
	else if(angle == -90 || angle == 270)
	{
		sg.type = CPC_SG_DRAWTEXTDOWN;
	}
	else
	{
		sg.type = CPC_SG_DRAWTEXT;
	}
	sg.graphno = cpc_graphno;
	sg.coords[0] = x1;
	sg.coords[1] = cpc_ymax - y1;
	sg.string = string;
	sg.style = style;
	cpc_dosg(&sg);
}

int            cpc_textextent(char * string)
{
	cpc_sg sg;
	sg.graphno = cpc_graphno;
	sg.string = string;
	cpc_dosg(&sg);
	return(sg.size);
}

void           cpc_drawglyph(unsigned long x1, unsigned long y1,
                        long type, long style, long size)
{
}

void   cpc_do_point(long x, long y, long number)
{
	cpc_sg sg;
	sg.graphno = cpc_graphno;
	sg.coords[0] = x;
	sg.coords[1] = cpc_ymax - y;
	//sg.style = CPC_SG_FILLED;
	sg.style = 0;
	sg.size = 50;
	cpc_setbrush(CPC_FIREBRICK,CPC_SG_SOLID);
	switch(number)
	{
        	case SMALLDIAMONDPOINT:
        	case SMALLPLUSPOINT:
        	case SMALLBOXPOINT:
        	case SMALLCROSSPOINT:
        	case SMALLTRIANGLEPOINT:
        	case SMALLSTARPOINT:
        	case SMALLCIRCLEPOINT:
			sg.size = sg.size*.5;
			break;
			
        	case SMALLERDIAMONDPOINT:
        	case SMALLERPLUSPOINT:
        	case SMALLERBOXPOINT:
        	case SMALLERCROSSPOINT:
        	case SMALLERTRIANGLEPOINT:
        	case SMALLERSTARPOINT:
        	case SMALLERCIRCLEPOINT:
			sg.size = sg.size*.25;
			break;
			
		default:
			;
	}
	switch(number)
	{
        	case DIAMONDPOINT:
        	case SMALLDIAMONDPOINT:
        	case SMALLERDIAMONDPOINT:
			sg.type = CPC_SG_DIAMOND;
			break;
			
        	case PLUSPOINT:
        	case SMALLPLUSPOINT:
        	case SMALLERPLUSPOINT:
			sg.type = CPC_SG_CROSS;
			break;
			
        	case BOXPOINT:
        	case SMALLBOXPOINT:
        	case SMALLERBOXPOINT:
			sg.type = CPC_SG_SQUARE;
			break;
			
        	case CROSSPOINT:
        	case SMALLCROSSPOINT:
        	case SMALLERCROSSPOINT:
			sg.type = CPC_SG_EXE;
			break;
			
        	case TRIANGLEPOINT:
        	case SMALLTRIANGLEPOINT:
        	case SMALLERTRIANGLEPOINT:
			sg.type = CPC_SG_TRIANGLE;
			break;
			
        	case STARPOINT:
        	case SMALLSTARPOINT:
        	case SMALLERSTARPOINT:
			sg.type = CPC_SG_STAR;
			break;
			
        	case CIRCLEPOINT:
        	case SMALLCIRCLEPOINT:
        	case SMALLERCIRCLEPOINT:
			sg.type = CPC_SG_CIRCLE;
			break;
			
        	case DOTPOINT:
        	case SMALLDOTPOINT:
        	case SMALLERDOTPOINT:
			sg.type = CPC_SG_CIRCLE;
			sg.size = 10;
			break;
			
		default:
			return;
	}
	cpc_dosg(&sg);
			
}

void		cpc_savestate(void)
{
	cpc_sg sg;
	sg.graphno = cpc_graphno;
	sg.type = CPC_SG_SAVESTATE;
	cpc_dosg(&sg);
}

void		cpc_restorestate(void)
{
	cpc_sg sg;
	sg.graphno = cpc_graphno;
	sg.type = CPC_SG_RESTORESTATE;
	cpc_dosg(&sg);
}

} /*extern "C" */
