/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 5.01 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*                                                                 ^F
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

struct plotKeyValues;
typedef struct plotKeyValues plotKeyValues, *plotKeyValuesPtr;


#include <new>
#include "mvcpcdefs.h"
#include "wx/wx.h"
#include "wx/cmdline.h"
#include "wx/ffile.h"

extern "C"{
#include "cpcIface.h"
#include <stdio.h> 
#include <string.h>
#include <unistd.h>
#include "cpcinterface.h"
#include "../globals.h"
}



#define RECOGNIZEBANG
#define BANG   '!'
#define INTNOTSET       0x00000  /* no interrupt in progress */

#ifndef FALSE
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE 1
#endif

wxString CpcHomePath, CpcUserPath, CpcAppPath, CpcAppPathOption;
extern "C" {
        char * findHome(void);
        char * findUserFiles(void);
        char * findAppFiles(void);
}


static wxArrayString CommandArgBuffers; 

 
#ifndef ENVNAME
#define ENVNAME "MACANOVA"
#endif /*ENVNAME*/    


extern "C"
{
#include <stdio.h>
#include "cpcinterface.h"
struct plotKeyValues;
typedef struct plotKeyValues plotKeyValues, *plotKeyValuesPtr;
#include "cpcIface.h"
#include "../mvIo.h"
#include "../platform.h"
#include "../globals.h"
#include "../mainpars.h"
#include "../mvsave.h"
#include "../mvGraphs.h"
}


MacAnovaStartupArgs::MacAnovaStartupArgs(void)
{
	int i;

        dataFileName = (char *) 0;
        macroFileName = (char *) 0;
        openFileName = (char *) 0;
        dataPathName = (char *) 0;
        macroPathName = (char *) 0;
        helpFileName = (char *) 0;
        initFileName = (char *) 0;
        batchFileName = (char *) 0;
        restoreFileName = (char *) 0;
        commandPrompt = (char *) 0;        
        batchPrompt = (char *) 0;
        buffer[0] = '\0';
        homePathName = (char *) 0;;
        defaultHelpFile = (char *) 0;;
        motdFile = (char *) 0;;
        cpcConfigFile = (char *) 0;;
        initFileName = (char *) 0;;
        batchFileName = (char *) 0;;
        expression = (char *) 0;;
        appDir = (char *) 0;;
/* end of old style and beginning of new style */

	for(i=0;i<MAXPATHS;i++)
	{
        	searchPaths[i] = (char *) 0;
        	helpFiles[i] = (char *) 0;
        	macroFiles[i] = (char *) 0;
        	dataFiles[i] = (char *) 0;
        	openFiles[i] = (char *) 0;
        	initFiles[i] = (char *) 0;
        	batchFiles[i] = (char *) 0;
        	unknownFiles[i] = (char *) 0;
	}

#ifdef BASE_UNIX
        homePathName = getenv("HOME");;
#else /*BASE_UNIX*/
        homePathName = (char *) 0;
#endif /*BASE_UNIX*/
        nlines = -1;
        ncols = -1;
        expressionQ = (char *) 0;
        historyLength = -1;
        quiet = 0;
} /*MacAnovaStartupArgs::MacAnovaStartupArgs()*/     


static char     UsageMsg[] =
"Usage: macanova [options] [files]\n\n\
Options are:\n\
  -q                   Suppress printing banner\n\
  -batch batchFile    #Simulate batch(\"batchFile\")\n\
  -restore restoreFile Simulate restore(\"restoreFile\")\n\
  -f startupFile      #Use startupFile instead of Macanova.ini.txt\n\
  -prompt Prompt       Use Prompt as input prompt\n\
  -bprompt Prompt      Use Prompt as batch prompt (with -batch)\n\
  -e Expression        Evaluate Expression before doing anything else\n\
  -help helpFile       Use helpFile as default file for help information\n\
  -macro macroFile    #Add \"macroFile\" to variable MACROFILES\n\
  -open windowFile    #Load windowFile into window at launch\n\
  -data dataFile      #Add \"dataFile\" to variable DATAFILES\n\
  -path dataPath      #Add \"dataPath\" to variable DATAPATHS\n\
  -addhelp hFile      #Add \"hFile\" to variable HELPFILES\n\
  -home homePath       Set variable HOME to \"homePath\"\n\
  -appdir appPath      Set the MacAnova root directory to \"appPath\"\n\
  -config confFile     Set the MacAnova preferences file to \"confFile\"\n\
  -l nlines            Set height for dumb plots to nlines (0 or >= 6)\n\
  -w ncols             Set width for dumb plots to ncols (>= 20)\n\
  -hist count          Keep count old commands in history\n\
\n\
  options marked with # may be used multiple times\n\
\n\
The files list can contain a restore file and/or one or more\n\
batch files and/or one or more window files.  Restore files are\n\
recognized by the extension .mvsave or by the internal save file\n\
signature.  Batch files are recognized by the extentions .mvbat,\n\
.bat, .batch, .bat.txt, and .batch.txt or by beginning with the\n\
characters '#!mvBatch'.  All other files are window files.";

static void usage(void)
{
        cpc_io  io;

        io.type = CPC_IO_ALERT;
        io.string = UsageMsg;
        cpc_doio(&io);
} /*usage()*/


#define ASFILENAME 1
#define ASPATHNAME 2
#define ARBITRARY  3

static int processOptionSingleString(wxArrayString& Args,
	char **destination, wxArrayString& errormsg,
	char *duplicatemessage, int type)
{
	wxString tmp;
	if(Args.GetCount() < 2)
	{
		tmp << "No value for option " << Args[0];
		errormsg.Add(tmp); 
		return(0);
	}
	if(type == ASFILENAME && !okfilename((char *)Args[1].c_str()))
	{
		tmp << Args[1] << " is not a legal file name";
		return(0);
	}
	if(type == ASPATHNAME && !okpathname((char *)Args[1].c_str()))
	{
		tmp << Args[1] << " is not a legal path name";
		return(0);
	}
	if(*destination != (char *) 0)
	{
		tmp << "Multiple " << duplicatemessage <<
			" specified; all but last ignored";
		errormsg.Add(tmp); 
	}
	CommandArgBuffers.Add(Args[1]);
	*destination = (char *) CommandArgBuffers.Last().c_str();
	Args.RemoveAt(0,2);

	return(1);
}


static int processOptionSingleLong(wxArrayString& Args,
	long *destination, wxArrayString& errormsg,
	char *duplicatemessage)
{
	wxString tmp;
	if(Args.GetCount() < 2)
	{
		tmp << "No value for option " << Args[0];
		errormsg.Add(tmp); 
		return(0);
	}
	if(*destination >= 0)
	{
		tmp << "Multiple " << duplicatemessage <<
			" specified; all but last ignored";
		errormsg.Add(tmp); 
		tmp.Empty();
	}
	if(!Args[1].ToLong(destination))
	{
		tmp << "Value for option " << Args[0] << " not an integer";
		errormsg.Add(tmp); 
		return(0);
	}
	Args.RemoveAt(0,2);

	return(1);
}

static int processOptionMultiString(wxArrayString& Args,
	char **destination, wxArrayString& errormsg,
	char *duplicatemessage, int type)
{
	int i;
	wxString tmp;
	if(Args.GetCount() < 2)
	{
		tmp << "No value for option " << Args[0];
		errormsg.Add(tmp); 
		return(0);
	}
	for(i=0;destination[i] && i < MAXPATHS; i++)
	{
		;;
	}
	if(i == MAXPATHS)
	{
		wxString tmp;
		tmp << "Too many " << duplicatemessage << " specified; extras ignored";
		errormsg.Add(tmp); 
		return(1);
	}
	if(type == ASFILENAME && !okfilename((char *)Args[1].c_str()))
	{
		tmp << Args[1] << " is not a legal file name";
		return(0);
	}
	if(type == ASPATHNAME && !okpathname((char *)Args[1].c_str()))
	{
		tmp << Args[1] << " is not a legal path name";
		return(0);
	}
	CommandArgBuffers.Add(Args[1]);
	destination[i] = (char *) CommandArgBuffers.Last().c_str();
	Args.RemoveAt(0,2);

	return(1);
}


static int processFileMultiString(wxArrayString& Args,
	char **destination, wxArrayString& errormsg,
	char *duplicatemessage, int type)
{
	int i;
	wxString tmp;
	for(i=0;destination[i] && i < MAXPATHS; i++)
	{
		;;
	}
	if(i == MAXPATHS)
	{
		wxString tmp;
		tmp << "Too many " << duplicatemessage << " specified; extras ignored";
		errormsg.Add(tmp); 
		return(1);
	}
	if(type == ASFILENAME && !okfilename((char *)Args[0].c_str()))
	{
		tmp << Args[0] << " is not a legal file name";
		return(0);
	}
	CommandArgBuffers.Add(Args[0]);
	destination[i] = (char *) CommandArgBuffers.Last().c_str();
	Args.RemoveAt(0,1);

	return(1);
}


static int ArgsParser(wxArrayString &Args,
	MacAnovaStartupArgs *startupArgs, wxArrayString &errormsg)
{
	wxString option, filepath;
	int i;

	while( Args.GetCount() > 0)
	{
		option = Args[0];
		if(option == "-q")
		{
			startupArgs->quiet = 1;
			Args.RemoveAt(0,1);
		}
		else if( option == "-appdir")
		{
			if(!processOptionSingleString(Args,&(startupArgs->appDir),
				errormsg, "application directory", ASPATHNAME))
			{
				return(0);
			}
		}
		else if( (option == "-r") || (option == "-restore"))
		{
#ifdef BASE_MICROSOFT
			// filename gets put in a MacAnova quoted string
			// so backslashes must be doubled
			if(Args.GetCount() >= 2)
			{
				Args[1].Replace("\\","\\\\");
			}
#endif /*BASE_MICROSOFT*/
			if(!processOptionSingleString(Args,&(startupArgs->restoreFileName),
				errormsg, "restore files", ASFILENAME))
			{
				return(0);
			}
		}
		else if( (option == "-h") || (option == "-help"))
		{
			if(!processOptionSingleString(Args,&(startupArgs->defaultHelpFile),
				errormsg, "default help files", ASFILENAME))
			{
				return(0);
			}
		}
		else if( (option == "-p") || (option == "-prompt"))
		{
			if(!processOptionSingleString(Args,&(startupArgs->commandPrompt),
				errormsg, "command prompts", ARBITRARY))
			{
				return(0);
			}
		}
		else if( (option == "-bp") || (option == "-bprompt"))
		{
			if(!processOptionSingleString(Args,&(startupArgs->batchPrompt),
				errormsg, "batch prompts", ARBITRARY))
			{
				return(0);
			}
		}
		else if(option == "-home")
		{
			if(Args.GetCount() >1 && Args[1].Last() != DIRSEPARATOR[0])
			{
				Args[1] << DIRSEPARATOR[0];
			}
			if(!processOptionSingleString(Args,&(startupArgs->homePathName),
				errormsg, "home paths", ASPATHNAME))
			{
				return(0);
			}
		}
		else if(option == "-config")
		{
			if(!processOptionSingleString(Args,&(startupArgs->cpcConfigFile),
				errormsg, "configuration files", ASFILENAME))
			{
				return(0);
			}
		}
		else if(option == "-motd")
		{
			if(!processOptionSingleString(Args,&(startupArgs->motdFile),
				errormsg, "motd files", ASFILENAME))
			{
				return(0);
			}
		}
#if 0
		else if(option == "-readlinerc")
		{
			if(!processOptionSingleString(Args,&(startupArgs->readlineRcFile),
				errormsg, "readline rc files", ASFILENAME))
			{
				return(0);
			}
		}
		else if(option == "-eq")
		{
			if(!processOptionSingleString(Args,&(startupArgs->expressionQ),
				errormsg, "single expressions", ARBITRARY))
			{
				return(0);
			}
		}
#endif
		else if(option == "-e")
		{
			if(!processOptionSingleString(Args,&(startupArgs->expression),
				errormsg, "expressions", ARBITRARY))
			{
				return(0);
			}
		}
		else if(option == "-l")
		{
			if(!processOptionSingleLong(Args,&(startupArgs->nlines),
				errormsg, "line counts"))
			{
				return(0);
			}
			if(startupArgs->nlines != 0  && startupArgs->nlines != -1 &&
				startupArgs->nlines < MINSCREENHEIGHT)
			{
				errormsg.Add("-l value not 0 and less than ");
				errormsg.Last() << MINSCREENHEIGHT;
				return(0);
			}
		}
		else if(option == "-w")
		{
			if(!processOptionSingleLong(Args,&(startupArgs->ncols),
				errormsg, "line widths"))
			{
				return(0);
			}
			if(startupArgs->ncols != 0  && startupArgs->ncols != -1 &&
				startupArgs->ncols < MINSCREENWIDTH)
			{
				errormsg.Add("-w value less than ");
				errormsg.Last() << MINSCREENWIDTH;
				return(0);
			}
		}
		else if(option == "-hist")
		{
			if(!processOptionSingleLong(Args,&(startupArgs->historyLength),
				errormsg, "history lengths"))
			{
				return(0);
			}
			if(startupArgs->historyLength < 0)
			{
				errormsg.Add("hist value is negative");
				return(0);
			}
		}
		else if((option == "-path") || (option == "-dpath") || (option == "-dp")
			|| (option == "-mpath") || (option == "-mp"))
		{
			if(Args.GetCount() >1 && Args[1].Last() != DIRSEPARATOR[0])
			{
				Args[1] << DIRSEPARATOR[0];
			}
			if(!processOptionMultiString(Args,startupArgs->searchPaths,
				errormsg, "search paths", ASPATHNAME))
			{
				return(0);
			}
		}
		else if( (option == "-b") || (option == "-batch"))
		{
#ifdef BASE_MICROSOFT
			// filename gets put in a MacAnova quoted string
			// so backslashes must be doubled
			if(Args.GetCount() >= 2)
			{
				Args[1].Replace("\\","\\\\");
			}
#endif /*BASE_MICROSOFT*/
			if(!processOptionMultiString(Args,startupArgs->batchFiles,
				errormsg, "batch files", ASFILENAME))
			{
				return(0);
			}
		}
		else if((option == "-f") || (option == "-initfile"))
		{
#ifdef BASE_MICROSOFT
			// filename gets put in a MacAnova quoted string
			// so backslashes must be doubled
			if(Args.GetCount() >= 2)
			{
				Args[1].Replace("\\","\\\\");
			}
#endif /*BASE_MICROSOFT*/
			if(!processOptionMultiString(Args,startupArgs->initFiles,
				errormsg, "init files", ASFILENAME))
			{
				return(0);
			}
		}
		else if(option == "-addhelp")
		{
			if(!processOptionMultiString(Args,startupArgs->helpFiles,
				errormsg, "help files", ASFILENAME))
			{
				return(0);
			}
		}
		else if((option == "-d") || (option == "-data"))
		{
			if(!processOptionMultiString(Args,startupArgs->dataFiles,
				errormsg, "data files", ASFILENAME))
			{
				return(0);
			}
		}
		else if((option == "-m") || (option == "-macro"))
		{
			if(!processOptionMultiString(Args,startupArgs->macroFiles,
				errormsg, "macro files", ASFILENAME))
			{
				return(0);
			}
		}
		else if(option == "-open")
		{
			if(!processOptionMultiString(Args,startupArgs->openFiles,
				errormsg, "output files", ASFILENAME))
			{
				return(0);
			}
		}
		else
		{
			if(option.Left(1) == "-")
			{
				wxString tmp;
				tmp << "Unrecognized option " << option;
				errormsg.Add(tmp);
				return(0);
			}
			// not a recognized switch, so we must be at nonoption file names
			break;
		}
	}
	// all remaining args must be file names
	while( Args.GetCount() > 0)
	{
		// right now just stash them in unknownFiles
		// we'll sort them out later after we have the search paths
		// completely set up
		if(Args[0].Left(1) == "-")
		{
			wxString tmp;
			tmp << "Option " << Args[0] << " follows file argument";
			errormsg.Add(tmp);
			return(0);
		}

		if(!processFileMultiString(Args,startupArgs->unknownFiles,
			errormsg, "unknown filetype", ASFILENAME))
		{
			return(0);
		}
	}

	return(1);
}

extern "C" 
{
MacAnovaStartupArgs * ProcessArgs(int argc, char **argv)
{
	MacAnovaStartupArgs *startupArgs  = new MacAnovaStartupArgs;
	char         *envName = ENVNAME;
	char         *macEnv = getenv(envName);
	int           i;
	WHERE("ProcessArgs");

	wxArrayString Envargs;
	wxArrayString Clargs;
	wxArrayString errormsg;

 	if(macEnv != (char *) 0)
	{
		Envargs = wxCmdLineParser::ConvertStringToArgs(macEnv);
	}
	for(i=1;i<argc;i++)
	{
		Clargs.Add(argv[i]);
	}

	if(!ArgsParser(Envargs,startupArgs,errormsg))
	{
		myAlert((char *)errormsg.Last().c_str());
		usage();
		return ((MacAnovaStartupArgs *) 0);
	}
	while(errormsg.GetCount() > 0)
	{
		errormsg[0] << " from environment variable " << envName;
		myAlert((char *)errormsg[0].c_str());
		errormsg.RemoveAt(0);
	}
	
	if(!ArgsParser(Clargs,startupArgs,errormsg))
	{
		myAlert((char *)errormsg.Last().c_str());
		usage();
		return ((MacAnovaStartupArgs *) 0);
	}
	while(errormsg.GetCount() > 0)
	{
		myAlert((char *)errormsg[0].c_str());
		errormsg.RemoveAt(0);
	}
	
	
	
	if (startupArgs->expressionQ != (char *) 0)
	{
		if (startupArgs->batchFileName != (char *) 0    ||
			startupArgs->expression != (char *) 0)
		{
			myAlert("ERROR: -eq is incompatible with -e and -batch");
			return ((MacAnovaStartupArgs *) 0);
		}
		startupArgs->quiet = 1;
		if (startupArgs->nlines < 0)
		{
			startupArgs->nlines = 0;
		}
	} /*if (expressionQ != (char *) 0)*/

	if (startupArgs->openFiles[0] != (char *) 0)
	{
		startupArgs->quiet = 1;
	}
	
	return (startupArgs);

} /*MacAnovaApp::ProcessArgs()*/

} /*extern C*/




extern "C" {
#if defined(OS_MACOSX)
#undef CR
#if 0
#include <Errors.h>
#endif
#include <CFBundle.h>
#endif

#define CPCHOMEPATHLENGTH 2000

char * findHome(void)
{
#ifdef BASE_MICROSOFT
	CpcHomePath << getenv("HOMEDRIVE");
	CpcHomePath << getenv("HOMEPATH");
#else
	wxGetHomeDir(&CpcHomePath);
#endif /* BASE_MICROSOFT*/
	return((char *) CpcHomePath.c_str());
}

char * findUserFiles(void)
{
#ifdef BASE_MICROSOFT
	CpcUserPath << getenv("HOMEDRIVE");
	CpcUserPath << getenv("HOMEPATH");
#else
	wxGetHomeDir(&CpcUserPath);
#endif /* BASE_MICROSOFT*/

	char sep;
#if defined(BASE_MICROSOFT)
	sep = '\\';
#else
	sep = '/';
#endif
	if(CpcUserPath.Last() != sep)
	{
		CpcUserPath << sep;
	}
#if defined(OS_MACOSX)
	CpcUserPath << "Library/Application Support/MacAnova";
#else 
	CpcUserPath << "MyMacAnovaFiles";
#endif
	CpcUserPath << sep;
	return((char *) CpcUserPath.c_str());
}

extern const char *cpcGetMswModuleDir(void);


char * findAppFiles(void)
{
#if defined(OS_MACOSX)
	OSErr err;
	CFBundleRef myAppsBundle = CFBundleGetMainBundle();
	if(myAppsBundle == NULL)
	{
		return(NULL);
	}
	CFURLRef myBundleURL = CFBundleCopyBundleURL(myAppsBundle);
	if(myBundleURL == NULL)
	{
		return(NULL);
	}
	
	FSRef myBundleRef;
	Boolean ok = CFURLGetFSRef(myBundleURL, &myBundleRef);
	CFRelease(myBundleURL);
	if(!ok)
	{
		return(NULL);
	}
	FSSpec theFSSpec;
	FSRef  theRef;
	err = FSGetCatalogInfo(&myBundleRef, kFSCatInfoNone,NULL,NULL,&theFSSpec, NULL);
	if(err != noErr)
	{
		return(NULL);
	}
	err = FSpMakeFSRef(&theFSSpec, &theRef);
	if(err != noErr)
	{
		return(NULL);
	}
	err = FSRefMakePath(&theRef,(UInt8 *) CpcAppPath.GetWriteBuf(CPCHOMEPATHLENGTH), 
		CPCHOMEPATHLENGTH-1);
	CpcAppPath.UngetWriteBuf();
	if(err != noErr)
	{
		return(NULL);
	}
	// this should end with /Macanova[...].app
	CpcAppPath  << "/Contents/SharedSupport/";
	return((char *) CpcAppPath.c_str());
#elif defined(BASE_MICROSOFT) 
#if 0
	wxString myfile;
	::GetModuleFileName(::GetModuleHandle(),myfile.GetWriteBuf(CPCHOMEPATHLENGTH), 
		CPCHOMEPATHLENGTH);
	myfile.UngetWriteBuf();
	wxSplitPath(myfile,&CpcAppPath,NULL,NULL);
#endif
	CpcAppPath << cpcGetMswModuleDir() << "\\SharedSupport\\";
	return((char *) CpcAppPath.c_str());
#elif defined(BASE_UNIX)
	CpcAppPath = DATAPATHNAME;
	if(CpcAppPathOption.Length() > 0)
	{
		CpcAppPath = CpcAppPathOption;
	}
	if(CpcAppPath.Last() != '/')
	{
		CpcAppPath << '/';
	}
	CpcAppPath << "SharedSupport/";
	return((char *) CpcAppPath.c_str());
#endif
}

} /*extern "c"*/
