/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 5.01 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*                                                                 ^F
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

#include <new.h> // for prototype for set_new_handler()

#include <math.h>
#include "wx/wx.h"
#include "wx/filename.h"
#include "wx/cmdline.h"
extern "C" {
	char * findHome(void);
	char * findUserFiles(void);
	char * findAppFiles(void);
}

#ifndef ENVNAME
#define ENVNAME "MACANOVA"
#endif /*ENVNAME*/    
#ifdef OS_WIN32
#define SEP '\\'
#else /*OS_WIN32*/
#define SEP '/'
#endif /*OS_WIN32*/

#ifndef FALSE
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE 1
#endif


#if 0
static char     UsageMsg[] =
"Usage: macanova [options] [files]\n\n\
Options are:\n\
  -q                   Suppress printing banner\n\
  -batch batchFile    #Simulate batch(\"batchFile\")\n\
  -restore restoreFile Simulate restore(\"restoreFile\")\n\
  -f startupFile      #Use startupFile instead of Macanova.ini.txt\n\
  -prompt Prompt       Use Prompt as input prompt\n\
  -bprompt Prompt      Use Prompt as batch prompt (with -batch)\n\
  -e Expression        Evaluate Expression before doing anything else\n\
  -help helpFile       Use helpFile as default file for help information\n\
  -macro macroFile    #Add \"macroFile\" to variable MACROFILES\n\
  -open windowFile    #Load windowFile into window at launch\n\
  -data dataFile      #Add \"dataFile\" to variable DATAFILES\n\
  -path dataPath      #Add \"dataPath\" to variable DATAPATHS\n\
  -addhelp hFile      #Add \"hFile\" to variable HELPFILES\n\
  -home homePath       Set variable HOME to \"homePath\"\n\
  -appdir appPath      Set the MacAnova root directory to \"appPath\"\n\
  -config confFile     Set the MacAnova preferences file to \"confFile\"\n\
  -l nlines            Set height for dumb plots to nlines (0 or >= 6)\n\
  -w ncols             Set width for dumb plots to ncols (>= 20)\n\
  -hist count          Keep count old commands in history\n\
\n\
  options marked with # may be used multiple times\n\
\n\
The files list can contain a restore file and/or one or more\n\
batch files and/or one or more window files.  Restore files are\n\
recognized by the extension .mvsave or by the internal save file\n\
signature.  Batch files are recognized by the extentions .mvbat,\n\
.bat, .batch, .bat.txt, and .batch.txt or by beginning with the\n\
characters '#!mvBatch'.  All other files are window files.";
                                                    
static void usage(void)
{
	cpc_io  io;
	
	io.type = CPC_IO_ALERT;
	io.string = UsageMsg;
	cpc_doio(&io);
} /*usage()*/    
#endif /*0*/

//char         *TooManyErrors = "ERROR: too many errors; execution terminated";
//int SaveOnQuit = 1;

extern "C"
{
#include <stdio.h>
#include "cpcinterface.h"
#define MAIN__
struct plotKeyValues;
typedef struct plotKeyValues plotKeyValues, *plotKeyValuesPtr;
#include "cpcIface.h"
#include "../mvIo.h"
#include "../platform.h"
#include "../globals.h"
#include "../mainpars.h"
#include "../mvsave.h"
#include "../mvGraphs.h"
}

#include "mvcpcdefs.h"

extern "C" {

/* C++ replacements for malloc() and free() using new and delete */
void * cpcMalloc(size_t size)
{
        char     *ptr = new char[size];

        return ((void *) ptr);
} /*cpcMalloc()*/

void cpcFree(void * ptr)
{
        delete [] (char *) ptr;
} /*cpcFree()*/

void closeBatch(long all)
{
	while (BDEPTH > 0)
	{
		mydisphandle(INPUTFILENAMES[BDEPTH]);
		INPUTFILENAMES[BDEPTH] = (char **) 0;
		fclose(INPUTFILE[BDEPTH]);
		INPUTFILE[BDEPTH--] = (FILE *) 0;
		if (!all)
		{
			break;
		}
	} /*while (BDEPTH > 0)*/
}

/* static char *InputLine; */
static wxString InputLine;

int GetC(long iPlace, int *done)
{
	if(iPlace < 0)
	{
		/* *done = strlen(InputLine); */
		*done = InputLine.Length();
		return(0);
	}
	/*((iPlace + 1 >= strlen(InputLine)) ? *done = 1 : *done = 0);*/
	((iPlace + 1 >= InputLine.Length()) ? *done = 1 : *done = 0);
	return(InputLine[(unsigned int) iPlace]);
} /*GetC()*/ 

#ifdef OS_WIN32
/*  '/' equivalent to '\' on MSDOS*/
#define isSlash(C) ((C) == DIRSEPARATOR[0] || (C) == '/')
#define isPath(N) (strchr(N, '/') || strchr(N, '\\') || strchr(N, ':'))
#else /*OS_WIN32*/
#define isSlash(C) ((C) == DIRSEPARATOR[0])
#define isPath(N) strchr(N, DIRSEPARATOR[0])
#endif /*OS_WIN32*/

static char * pathTail(char *filename)
{
        char     *tail;

        for (tail = filename + strlen(filename) - 1;tail >= filename;tail--)
        {                                                                
#ifndef OS_WIN32
                if (isSlash(*tail))
                {
                        break;
                }
#else /*OS_WIN32*/
                if (isSlash(*tail) || *tail == ':')
                {
                        break;
                }
#endif /*OS_WIN32*/
        } /*for (tail = filename + strlen(filename) - 1;tail >= filename;tail--)*/
        tail++;
        return (tail);
} /*pathTail()*/      

int ProcessInput(char * inputLine)
{
	long         done = 0, inputStatus;


	InputLine = inputLine;
//printf(inputLine);
	
	BDEPTH = 0;
	NERRORS = 0;
	do
	{
		if(BDEPTH > 0 && BATCHECHO[BDEPTH])
		{
			myprint("\n");
			myprint(pathTail(*INPUTFILENAMES[BDEPTH]));
			myprint("> ");
		}
		inputStatus = getinput();

		switch(inputStatus)
		{
		case -2:
		case -1:
		case  0:
			return(CPC_EXIT_SAVEFORCEQUIT);
	
		case  3:
			closeBatch(1);
			BDEPTH = 0;
			return(CPC_EXIT_NOQUIT);
	
		case 2:
			putOutErrorMsg("WARNING: extra characters after end of line");
		case 1:
	#ifdef RECOGNIZEBANG
			if ((*INPUTSTRING)[0] == BANG)
			{
				done = processBang();
			} /*if ((*INPUTSTRING)[0] == BANG)*/
			else
#endif /*RECOGNIZEBANG*/
			{
				done = yyparse(); /* go process input line */
			}
		
			FatalError = (done == FATALERROR) ? 1 : FatalError;
	
			if(FatalError)
			{
				return(CPC_EXIT_FASTFORCEQUIT);
			}
			
			done = (done == END);
					
			if (INTERRUPT != INTNOTSET && BDEPTH > 0)
			{
				closeBatch(1); /* close all batch files on interrupt */
			}
	
			if (done)
			{
				if(SaveOnQuit)
				{
					return(CPC_EXIT_SAVECANCANCEL); 
				}
				else
				{
					return(CPC_EXIT_FASTFORCEQUIT);
				}
			}

			if (MAXERRORS1 && NERRORS >= MAXERRORS1)
			{
				if (BDEPTH > 0)
				{
					myprint("WARNING: too many errors on batch file ");
					putOutErrorMsg(*INPUTFILENAMES[BDEPTH]);
					closeBatch(1); /* shut down all batch files*/
				}
				if (!ISATTY && MAXERRORS && NERRORS >= MAXERRORS)
				{
					putOutErrorMsg(TooManyErrors);
					done = FATALERROR;
					FatalError = 1;
				}
			} /*if (MAXERRORS1 && NERRORS >= MAXERRORS1)*/
			
			if (INTERRUPT != INTNOTSET && BDEPTH > 0)
			{
				putOutErrorMsg("ERROR: batch file(s) terminated");
				closeBatch(1); /* shut down all batch files*/
				INTERRUPT = INTNOTSET;
			} /*if (INTERRUPT != INTNOTSET && BDEPTH > 0)*/
		

 /* conditionally set flag for dump of yacc parsing steps */
			//yydebug = (GUBED & 2048) ? 1 : 0;
		}
	} while (BDEPTH > 0);
	return(CPC_EXIT_NOQUIT);
} /*ProcessInput()*/

void showCurrentCmdWindow(void)
{
	cpc_io  io;
	io.type = CPC_IO_SHOWCMDWINDOW;

	cpc_doio(&io);
}

int cpcTestForInterrupt(void)
{
	return(cpc_TestForInterrupt());
}

} /*extern "C"*/


