/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 5.01 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*                                                                 ^F
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/


#include "../globals.h"

#include "../mvIface.h"

#include "../mvIo.h"

#ifdef SKEL_CARAPACE
#include "cpcinterface.h"
#endif

/*
  Routine to set/get dynamic data in CARAPACE versions
*/

//The following was appended on March 20,2005
Symbolhandle    setdyn(Symbolhandle list)
{
	Symbolhandle    symhString;
	Symbolhandle    value;
	long            nargs = NARGS(list);
	char           *DataobjectName;
	long           valuerows;
	long           valuecols;
	int            i,j;
	//long           length;
	DbjValue        dyn_DbjValue;
	double         *doublevector;
	

	
	WHERE("setdyn");
	
	cpc_dyn          dyn;
	
	*OUTSTR = '\0';

	if (nargs != 2)
	{
		badNargs(FUNCNAME, 2);
		goto errorExit;
	}
	
	symhString = COMPVALUE(list,0);

	DataobjectName = (char *) 0;
	if (symhString == EmptyArgument || !isCharOrString(symhString) ||
		*(DataobjectName = STRINGPTR(symhString)) == 0)
	{
		notCharOrString("Dataobject name");
		goto errorExit;
	}
	
	if(!DataobjectName)
	{
		sprintf(OUTSTR,"ERROR: dataobject names must be CHAR in %s()",
				FUNCNAME);
		goto errorExit;
	}
	
	symhString = COMPVALUE(list,1);
	valuerows = DIMVAL(symhString,1);
	if (NDIMS(symhString) == 1)
		valuecols =	1;
	else	
		valuecols = DIMVAL(symhString,2);
	dyn.name = DataobjectName;	
	dyn.DbjParmValue_ptr = &dyn_DbjValue;
	
	if (isReal(symhString)) {
		TMPHANDLE = mygethandle((valuerows *  valuecols + 15)* sizeof(double) + 10); // 10 just for safety		
		doublevector = (double *)TMPHANDLE;
		for (i = 0;i< valuerows *  valuecols;i++) {
			doublevector[i] = DATAVALUE(symhString,i);
		}	
		dyn.DbjParmValue_ptr->DoubleContentVector = doublevector;
		dyn.DbjParmValue_ptr->DataType = 2;	
	}
  dyn.DbjParmValue_ptr->name = DataobjectName;
  dyn.DbjParmValue_ptr->RowsNumber = valuerows;
  dyn.DbjParmValue_ptr->ColumnNumber = valuecols;	
  dyn.type = CPC_DYN_ADDDATAOBJECT;

  cpc_doDyc(&dyn);
  
	for (i = 0;i< valuerows *  valuecols;i++) {       // clear the doublevector
		doublevector[i] = 0;
			}	 
			
  dyn.DbjParmValue_ptr->name = DataobjectName;
  dyn.DbjParmValue_ptr->DataType = 2;
	dyn.DbjParmValue_ptr->DoubleContentVector = doublevector;
  dyn.type = CPC_DYN_GETDATAOBJECTVALUE;	
  cpc_doDyc(&dyn);                                  // get the content of Dataobject
  
  valuerows = dyn.DbjParmValue_ptr->RowsNumber;
  valuecols = dyn.DbjParmValue_ptr->ColumnNumber;
	value = Makereal(valuerows * valuecols);	        // construct a symbolhander with the value that we got from the dataobject 
	if (NDIMS(symhString) == 2) 		
		setNDIMS(value,2);
	if (NDIMS(symhString) == 1) 		
		setNDIMS(value,1);		
	setDIM(value,1,valuerows);
	setDIM(value,2,valuecols);
	setNAME(value,"values");	
	for (i = 0; i< valuerows*valuecols; i++)
		DATAVALUE(value,i) = doublevector[i];
	  
  return (value);  		 

	//return (NULLSYMBOL);

  errorExit:
	putErrorOUTSTR();
	return (0);
	
}

Symbolhandle    getdyn(Symbolhandle list)
{

	/* get the content  from a Dataobject */

	Symbolhandle    result = (Symbolhandle) 0;
	Symbolhandle    symhString;
	Symbolhandle    name, value;
	long            nargs = NARGS(list);
	

//ouble         **dovalue;
	double         *dovalue;
	double         **doublmatrix;
	double         *doublvector;
	long           valuerows,valuecols;
	long           i, j, namelen, valuelen;
	char           *DataobjectName, *charptr;
	long           MaxValuelen = 100;
  
	
	WHERE("getdyn");

	cpc_dyn         dyn;
	DbjValue        dyn_DbjValue;

	*OUTSTR = '\0';

	if (nargs != 1)
	{
		badNargs(FUNCNAME, 1);
		goto errorExit;
	}
	
	symhString = COMPVALUE(list,0);

	DataobjectName = (char *) 0;
	if (symhString == EmptyArgument || !isCharOrString(symhString) ||
		*(DataobjectName = STRINGPTR(symhString)) == 0)
	{
		notCharOrString("Dataobject name");
		goto errorExit;
	}
	
	if(!DataobjectName)
	{
		sprintf(OUTSTR,"ERROR: dataobject names must be CHAR in %s()",
				FUNCNAME);
		goto errorExit;
	}
	/*
	result = StrucInstall(SCRATCH, 2);
	
	namelen = strlen(DataobjectName);
	name = Makechar(namelen);
	
	setNDIMS(name,1);
	setDIM(name,1,1);
	setNAME(name,"names");	
	
	charptr=STRINGPTR(name);
	strcpy(charptr,DataobjectName);

	COMPVALUE(result,0) = name;
	*/
	dyn.DbjParmValue_ptr = &dyn_DbjValue;
	dyn.type = CPC_DYN_GETDATAOBJECTPROPERTY;
	dyn.name = DataobjectName;
	dyn.DbjParmValue_ptr->name = DataobjectName;	
	if (cpc_doDyc(&dyn))
		goto errorExit;
 
 
	TMPHANDLE = mygethandle((dyn.DbjParmValue_ptr->RowsNumber * dyn.DbjParmValue_ptr->ColumnNumber + 1) * sizeof(double) ); // + 1 just for safety	
	
	dyn.type = CPC_DYN_GETDATAOBJECTVALUE;
	dyn.name = DataobjectName;
	dyn.DbjParmValue_ptr->name = DataobjectName;
	//dyn.DbjParmValue_ptr->DataType = 2;	      // DataType value has been got
	dyn.DbjParmValue_ptr->DoubleContentVector = (double *)TMPHANDLE;
	cpc_doDyc(&dyn);
 
 
 	result = StrucInstall(SCRATCH, 2);
	
	namelen = strlen(DataobjectName);
	name = Makechar(namelen);
	
	setNDIMS(name,1);
	setDIM(name,1,1);
	setNAME(name,"names");	
	
	charptr=STRINGPTR(name);
	strcpy(charptr,DataobjectName);

	COMPVALUE(result,0) = name;
	
  valuerows = dyn.DbjParmValue_ptr->RowsNumber;
  valuecols = dyn.DbjParmValue_ptr->ColumnNumber;
	dovalue =  	dyn.DbjParmValue_ptr->DoubleContentVector;
	value = Makereal(valuerows * valuecols);	

	setNDIMS(value,1);
	setDIM(value,1,valuerows * valuecols);
	setNAME(value,"values");	
	
	for (i = 0; i< valuerows*valuecols; i++)
		DATAVALUE(value,i) = dovalue[i];
	
	COMPVALUE(result,1) = value;				

	return (result);
	//return (value);

  errorExit:
	putErrorOUTSTR();
	Removesymbol(result);
	return (0);
}  
//The above was appended on March 20,2005
