/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 5.01 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*                                                                 ^F
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/


#include "../globals.h"

#include "../mvIface.h"

#include "../mvIo.h"

#ifdef SKEL_CARAPACE
#include "cpcinterface.h"
#endif

// check the whether the value is the int type(-32768-32767) 
long isMyIntValue(Symbolhandle value)
{
	long valuerows,valuecols,i;
	valuerows = DIMVAL(value,1);
	if (NDIMS(value) == 1)
		valuecols =	1;
	else	
		valuecols = DIMVAL(value,2);
	for (i = 0;i < valuerows*valuecols; i++) {
		if ((DATAVALUE(value,i) < -32768) || (DATAVALUE(value,i) > 32767) || (DATAVALUE(value,i) != floor(DATAVALUE(value,i)))) 
			return 1;	
	}
	return 0;	
}

/*
  Routine to set dataobject in CARAPACE versions
*/


//The following was appended on March 27,2005
Symbolhandle    setdyn(Symbolhandle list)
{
	Symbolhandle    symhString;
	Symbolhandle    value;
	long            nargs = NARGS(list);
	char           *DataobjectName;
	long           valuerows;
	long           valuecols;
	int            i,j;
	DbjValue        dyn_DbjValue;
	double         *doublevector;
	int            *intvector;
	unsigned int   *logicvector;
	char           *strvector;
	long           strcount;
	char           **StrContentVector;
	long           valuesize;
	

	
	WHERE("setdyn");
	
	cpc_dyn          dyn;
	
	*OUTSTR = '\0';

	if (nargs != 2)
	{
		badNargs(FUNCNAME, 2);
		goto errorExit;
	}
	
	symhString = COMPVALUE(list,0);

	DataobjectName = (char *) 0;
	if (symhString == EmptyArgument || !isCharOrString(symhString) ||
		*(DataobjectName = STRINGPTR(symhString)) == 0)
	{
		notCharOrString("Dataobject name");
		goto errorExit;
	}
	
	if(!DataobjectName)
	{
		sprintf(OUTSTR,"ERROR: dataobject names must be CHAR in %s()",
				FUNCNAME);
		goto errorExit;
	}
	
	symhString = COMPVALUE(list,1);
	valuerows = DIMVAL(symhString,1);
	if (NDIMS(symhString) == 1)
		valuecols =	1;
	else	
		valuecols = DIMVAL(symhString,2);
	dyn.name = DataobjectName;	
	dyn.DbjParmValue_ptr = &dyn_DbjValue;
	if (isReal(symhString)) 	
		valuesize = (valuerows *  valuecols + 15)* sizeof(double) + 10;
	if (isLogic(symhString))
		valuesize = (valuerows *  valuecols + 15)* sizeof(unsigned int) + 10;
	if (isChar(symhString)) {
		strcount = symbolSize(symhString);
		strvector = STRINGPTR(symhString);
		valuesize = (strcount)* sizeof(char *) + 10;
	}
	TMPHANDLE = mygethandle(valuesize);
 	if (TMPHANDLE == (char **) 0)	{
		goto errorExit;
	}	
	if (isReal(symhString)) {		
		if (isMyIntValue(symhString)) {		
			doublevector = (double *)TMPHANDLE;		
			for (i = 0;i< valuerows *  valuecols;i++) 
				doublevector[i] = DATAVALUE(symhString,i);	
			dyn.DbjParmValue_ptr->DoubleContentVector = doublevector;
			dyn.DbjParmValue_ptr->DataType = 2;			    // double Data type	
		}	
		else {
			intvector	= (int *)TMPHANDLE;		
			for (i = 0;i< valuerows *  valuecols;i++) 
				intvector[i] = DATAVALUE(symhString,i);	
			dyn.DbjParmValue_ptr->IntContentVector = intvector;
			dyn.DbjParmValue_ptr->DataType = 1;			    // Integer Data type	
		}					
	}
	if (isLogic(symhString)) {
		logicvector = (unsigned int *)TMPHANDLE;		
		for (i = 0;i< valuerows *  valuecols;i++) 
				logicvector[i] = (unsigned int )DATAVALUE(symhString,i);	
		dyn.DbjParmValue_ptr->BoolContentVector = logicvector;
		dyn.DbjParmValue_ptr->DataType = 4;								
	}		
	if (isChar(symhString)) {
		StrContentVector = (char **)TMPHANDLE;		
		for (i = 0; i < strcount;	i++, strvector = skipStrings(strvector, 1)) 
				StrContentVector[i] = strvector;	
		dyn.DbjParmValue_ptr->StrContentVector =  StrContentVector;		
		dyn.DbjParmValue_ptr->DataType = 3;								
	}			
		
  dyn.DbjParmValue_ptr->name = DataobjectName;
  dyn.DbjParmValue_ptr->RowsNumber = valuerows;
  dyn.DbjParmValue_ptr->ColumnNumber = valuecols;	
  dyn.type = CPC_DYN_ADDDATAOBJECT;

  if (cpc_doDyc(&dyn))  {
		sprintf(OUTSTR,"ERROR: dataobject name already exist in %s()",
				FUNCNAME);
		goto errorExit;
	}
	/*  The following is used to read content from the new dataobject
  if (isReal(symhString)) {
	for (i = 0;i< valuerows *  valuecols;i++) {       // clear the doublevector
		doublevector[i] = 0;
			}	 
			
  dyn.DbjParmValue_ptr->name = DataobjectName;
  dyn.DbjParmValue_ptr->DataType = 2;
	dyn.DbjParmValue_ptr->DoubleContentVector = doublevector;
  dyn.type = CPC_DYN_GETDATAOBJECTVALUE;	
  cpc_doDyc(&dyn);                                  // get the content of Dataobject
  
  valuerows = dyn.DbjParmValue_ptr->RowsNumber;
  valuecols = dyn.DbjParmValue_ptr->ColumnNumber;
	value = Makereal(valuerows * valuecols);	        // construct a symbolhander with the value that we got from the dataobject 
	if (NDIMS(symhString) == 2) 		
		setNDIMS(value,2);
	if (NDIMS(symhString) == 1) 		
		setNDIMS(value,1);		
	setDIM(value,1,valuerows);
	setDIM(value,2,valuecols);
	setNAME(value,"values");	
	if (NDIMS(symhString) == 2) {	
	for (i = 0; i< valuerows;i++)
		for (j = 0; j < valuecols;j++) {
				DATAVALUE(value,i*valuecols+j) = doublevector[i*valuecols+j];			 
			}
		setNDIMS(value,2);
	}	
	if (NDIMS(symhString) == 1) {	
		for (i = 0; i< valuerows*valuecols; i++)
			DATAVALUE(value,i) = doublevector[i];
	}	
	return (value);
	}
	*/ 		 
	return (NULLSYMBOL);

  errorExit:
	putErrorOUTSTR();
	return (0);
	
}

Symbolhandle    getdyn(Symbolhandle list)
{

	/* get the content  from a Dataobject */

	Symbolhandle    result = (Symbolhandle) 0;
	Symbolhandle    symhString;
	Symbolhandle    name, value;
	long            nargs = NARGS(list);
	
	double         *dovalue;
	int            *dointvalue;
	long           valuerows,valuecols;
	long           i, j, namelen, valuelen;
	char           *DataobjectName, *charptr;
	long           strContOffset;
	unsigned int   *boolvector;
	
  
	
	WHERE("getdyn");

	cpc_dyn         dyn;
	DbjValue        dyn_DbjValue;

	*OUTSTR = '\0';

	if (nargs != 1)
	{
		badNargs(FUNCNAME, 1);
		goto errorExit;
	}
	
	symhString = COMPVALUE(list,0);

	DataobjectName = (char *) 0;
	if (symhString == EmptyArgument || !isCharOrString(symhString) ||
		*(DataobjectName = STRINGPTR(symhString)) == 0)
	{
		notCharOrString("Dataobject name");
		goto errorExit;
	}
	
	if(!DataobjectName)
	{
		sprintf(OUTSTR,"ERROR: dataobject names must be CHAR in %s()",
				FUNCNAME);
		goto errorExit;
	}
	dyn.DbjParmValue_ptr = &dyn_DbjValue;
	dyn.type = CPC_DYN_GETDATAOBJECTPROPERTY;
	dyn.name = DataobjectName;
	dyn.DbjParmValue_ptr->name = DataobjectName;	
	if (cpc_doDyc(&dyn)) {
		sprintf(OUTSTR,"ERROR: dataobject name doesn't exist in %s()", FUNCNAME);
		goto errorExit;
		}
 
 if ((dyn.DbjParmValue_ptr->DataType != 3) && (dyn.DbjParmValue_ptr->DataType != 4))      //  DOUBLE OR INTERGER DATA TYPE
		valuelen = ((dyn.DbjParmValue_ptr->RowsNumber * dyn.DbjParmValue_ptr->ColumnNumber + 1) * sizeof(double) ); 
 if (dyn.DbjParmValue_ptr->DataType == 3) {                                                // STRING DATA TYPE
 		valuelen = dyn.DbjParmValue_ptr->ColumnNumber + dyn.DbjParmValue_ptr->RowsNumber * sizeof(char *) + 20;   //total memory for string operation
 		strContOffset = dyn.DbjParmValue_ptr->RowsNumber * sizeof(char *) + 10;
 }
if (dyn.DbjParmValue_ptr->DataType == 4)      // LOGIC DATA TYPE
		valuelen = ((dyn.DbjParmValue_ptr->RowsNumber * dyn.DbjParmValue_ptr->ColumnNumber + 1) * sizeof(unsigned int) );  
 TMPHANDLE = mygethandle(valuelen);	
 if (TMPHANDLE == (char **) 0)
	{
		goto errorExit;
	}
 	
 
 if (dyn.DbjParmValue_ptr->DataType == 1) {    // Integer DATA TYPE
		dyn.type = CPC_DYN_GETDATAOBJECTVALUE;
		dyn.name = DataobjectName;
		dyn.DbjParmValue_ptr->name = DataobjectName;
		//dyn.DbjParmValue_ptr->DataType = 2;	      // DataType value has been got
		dyn.DbjParmValue_ptr->IntContentVector = (int *)TMPHANDLE;
		cpc_doDyc(&dyn); 
  	valuerows = dyn.DbjParmValue_ptr->RowsNumber;
  	valuecols = dyn.DbjParmValue_ptr->ColumnNumber;
		dointvalue =  	dyn.DbjParmValue_ptr->IntContentVector;
		value = Makereal(valuerows * valuecols);	
		if (valuecols > 1) 
			setNDIMS(value,2);	
		else 
			setNDIMS(value,1);	
		setDIM(value,1,valuerows);	
		setDIM(value,2,valuecols);
		setNAME(value,"values");		
		setTYPE(value,REAL);	
		for (i = 0; i< valuerows*valuecols; i++)
			DATAVALUE(value,i) = dointvalue[i];
  	}
 if (dyn.DbjParmValue_ptr->DataType == 2) {    // DOUBLE DATA TYPE
		dyn.type = CPC_DYN_GETDATAOBJECTVALUE;
		dyn.name = DataobjectName;
		dyn.DbjParmValue_ptr->name = DataobjectName;
		//dyn.DbjParmValue_ptr->DataType = 2;	      // DataType value has been got
		dyn.DbjParmValue_ptr->DoubleContentVector = (double *)TMPHANDLE;
		cpc_doDyc(&dyn); 
  	valuerows = dyn.DbjParmValue_ptr->RowsNumber;
  	valuecols = dyn.DbjParmValue_ptr->ColumnNumber;
		dovalue =  	dyn.DbjParmValue_ptr->DoubleContentVector;
		value = Makereal(valuerows * valuecols);	
		if (valuecols > 1) 
			setNDIMS(value,2);	
		else 
			setNDIMS(value,1);	
		setDIM(value,1,valuerows);	
		setDIM(value,2,valuecols);
		setNAME(value,"values");		
		setTYPE(value,REAL);	
		for (i = 0; i< valuerows*valuecols; i++)
			DATAVALUE(value,i) = dovalue[i];

  	}
  if (dyn.DbjParmValue_ptr->DataType == 4)  {    // LOGIC DATA TYPE
		dyn.type = CPC_DYN_GETDATAOBJECTVALUE;
		dyn.name = DataobjectName;
		dyn.DbjParmValue_ptr->name = DataobjectName;
		//dyn.DbjParmValue_ptr->DataType = 2;	      // DataType value has been got
		dyn.DbjParmValue_ptr->BoolContentVector = (unsigned int *)TMPHANDLE;
		cpc_doDyc(&dyn); 
  	valuerows = dyn.DbjParmValue_ptr->RowsNumber;
  	valuecols = dyn.DbjParmValue_ptr->ColumnNumber;
		boolvector =  	dyn.DbjParmValue_ptr->BoolContentVector;
		value = Makereal(valuerows * valuecols);	
		if (valuecols > 1) 
			setNDIMS(value,2);	
		else 
			setNDIMS(value,1);	
		setDIM(value,1,valuerows);	
		setDIM(value,2,valuecols);
		setNAME(value,"values");	
		setTYPE(value,LOGIC);	
		for (i = 0; i< valuerows*valuecols; i++)
			DATAVALUE(value,i) = boolvector[i];
  	}
  
 if (dyn.DbjParmValue_ptr->DataType == 3)  {    //  string	
		dyn.type = CPC_DYN_GETDATAOBJECTVALUE;
		dyn.name = DataobjectName;
		dyn.DbjParmValue_ptr->name = DataobjectName;
		dyn.DbjParmValue_ptr->StrContentVector = (char **)TMPHANDLE;
		dyn.DbjParmValue_ptr->StrContentVector[0] = (char *)TMPHANDLE + strContOffset;
		cpc_doDyc(&dyn); 
  	valuerows = dyn.DbjParmValue_ptr->RowsNumber;
  	valuecols = dyn.DbjParmValue_ptr->ColumnNumber;
		value = Makechar(valuelen);		
		if (valuecols > 1) 
			setNDIMS(value,2);	
		else 
			setNDIMS(value,1);	
		setDIM(value,1,valuerows);	
		setDIM(value,2,valuecols);
		setNAME(value,"values");	
		charptr = STRINGPTR(value);	
		for (i = 0; i< valuerows*valuecols; i++) {
			strcpy(charptr,	dyn.DbjParmValue_ptr->StrContentVector[i]);
			charptr = skipStrings(charptr, 1);		
		}
  }  
  result = StrucInstall(SCRATCH, 2);  
	namelen = strlen(DataobjectName) + 1;
	name = Makechar(namelen);	
	setNDIMS(name,1);
	setDIM(name,1,1);
	setNAME(name,"names");		
	charptr=STRINGPTR(name);
	strcpy(charptr,DataobjectName);  
  COMPVALUE(result,0) = name;
  COMPVALUE(result,1) = value;
  return (result);
 errorExit:
	putErrorOUTSTR();
	Removesymbol(result);
	return (0);
}  
//The above was appended on March 27,2005


//The following was append on March 31,2005     
Symbolhandle    rmvdyn(Symbolhandle list)   
{

	/* erase a Dataobject */

	Symbolhandle    symhString;
	long            nargs = NARGS(list);
	char           *DataobjectName;

	
  
	
	WHERE("rmvdyn");

	cpc_dyn         dyn;
	DbjValue        dyn_DbjValue;

	*OUTSTR = '\0';

	if (nargs != 1)
	{
		badNargs(FUNCNAME, 1);
		goto errorExit;
	}
	
	symhString = COMPVALUE(list,0);

	DataobjectName = (char *) 0;
	if (symhString == EmptyArgument || !isCharOrString(symhString) ||
		*(DataobjectName = STRINGPTR(symhString)) == 0)
	{
		notCharOrString("Dataobject name");
		goto errorExit;
	}
	
	if(!DataobjectName)
	{
		sprintf(OUTSTR,"ERROR: dataobject names must be CHAR in %s()",
				FUNCNAME);
		goto errorExit;
	}
	dyn.DbjParmValue_ptr = &dyn_DbjValue;
	dyn.type = CPC_DYN_DELETEDATAOBJECT;
	dyn.name = DataobjectName;
	dyn.DbjParmValue_ptr->name = DataobjectName;	
	if (cpc_doDyc(&dyn)) {
		sprintf(OUTSTR,"ERROR: dataobject name doesn't exist in %s()", FUNCNAME);
		goto errorExit;
		}		
	return (NULLSYMBOL);

  errorExit:
	putErrorOUTSTR();
	return (0);	
}

Symbolhandle    resetdyn(Symbolhandle list)   
{

	/* reset a Dataobject */

	Symbolhandle    symhString;
	Symbolhandle    symhString2;
	long            nargs = NARGS(list);
	char           *DataobjectName;
	long           valuerows;
	long           valuecols;
	int            i,j;
	DbjValue        dyn_DbjValue;
	double         *doublevector;
	int            *intvector;
	unsigned int   *logicvector;
	char           *strvector;
	long           strcount;
	char           **StrContentVector;
	long           valuesize;	
	long           dttotalrows,dttotalcols;
	long           rowposition,colposition;
	long           dttype;
	
	
	WHERE("resetdyn");

	cpc_dyn         dyn;



	*OUTSTR = '\0';

	if (nargs != 3)
	{
		badNargs(FUNCNAME, 3);
		goto errorExit;
	}
	
	symhString = COMPVALUE(list,0);

	DataobjectName = (char *) 0;
	if (symhString == EmptyArgument || !isCharOrString(symhString) ||
		*(DataobjectName = STRINGPTR(symhString)) == 0)
	{
		notCharOrString("Dataobject name");
		goto errorExit;
	}
	
	if(!DataobjectName)
	{
		sprintf(OUTSTR,"ERROR: dataobject names must be CHAR in %s()",
				FUNCNAME);
		goto errorExit;
	}
	

	symhString = COMPVALUE(list,1);
	valuerows = DIMVAL(symhString,1);
	if (NDIMS(symhString) == 1)
		valuecols =	1;
	else	
		valuecols = DIMVAL(symhString,2);
	if (valuerows * valuecols < 2) {
	  		sprintf(OUTSTR,"ERROR: row or column number of the dataobject is expected in %s()", FUNCNAME);
		goto errorExit;
	}
	
	rowposition = DATAVALUE(symhString,0);
	colposition = DATAVALUE(symhString,1);
	
	dyn.DbjParmValue_ptr = &dyn_DbjValue;
	dyn.type = CPC_DYN_GETDATAOBJECTPROPERTY;
	dyn.name = DataobjectName;
	dyn.DbjParmValue_ptr->name = DataobjectName;	
	if (cpc_doDyc(&dyn)) {
		sprintf(OUTSTR,"ERROR: dataobject name doesn't exist in %s()", FUNCNAME);
		goto errorExit;
	}	
  dttype = dyn.DbjParmValue_ptr->DataType;
  dttotalrows = dyn.DbjParmValue_ptr->RowsNumber;
  dttotalcols = dyn.DbjParmValue_ptr->ColumnNumber;
  
  if ((rowposition > dttotalrows) || (colposition > dttotalcols) || (rowposition < 1) || (colposition < 1)) {
  		sprintf(OUTSTR,"ERROR: the element doesn't exist in %s()", FUNCNAME);
			goto errorExit;
  }
	symhString2 = COMPVALUE(list,2);
	
	unsigned int typematchflag = 0;
	dyn.name = DataobjectName;	
	dyn.DbjParmValue_ptr = &dyn_DbjValue;
	if (isReal(symhString2) && ((dttype == 1) || (dttype == 2))) 	{
		valuesize = (valuerows *  valuecols + 15)* sizeof(double) + 10;	
	  typematchflag = 1;
	}
	if (isLogic(symhString2) && (dttype == 4) ) {
		valuesize = (valuerows *  valuecols + 15)* sizeof(unsigned int) + 10;
	  typematchflag = 1;	
	 }  
	if (isChar(symhString2) && (dttype == 3))  {
		strcount = symbolSize(symhString2);
		strvector = STRINGPTR(symhString2);
		valuesize = (strcount)* sizeof(char *) + 10;
	  typematchflag = 1;			
	}

	if (typematchflag == 0) {
		sprintf(OUTSTR,"ERROR: data type mismatch in %s()", FUNCNAME);
		goto errorExit;		
	}
	TMPHANDLE = mygethandle(valuesize);
 	if (TMPHANDLE == (char **) 0)	{
		goto errorExit;
	}	
	if (isReal(symhString2)) {		
		if (dttype == 1) {
			if (isMyIntValue(symhString2)) {		
				intvector	= (int *)TMPHANDLE;		
			//for (i = 0;i< 2;i++) 
				intvector[0] = floor(DATAVALUE(symhString2,0));	
				dyn.DbjParmValue_ptr->IntContentVector = intvector;
				dyn.DbjParmValue_ptr->DataType = 1;			    // Integer Data type   	
			}	
			else {
				intvector	= (int *)TMPHANDLE;		
			//for (i = 0;i< 2;i++) 
				intvector[0] = DATAVALUE(symhString2,0);	
				dyn.DbjParmValue_ptr->IntContentVector = intvector;
				dyn.DbjParmValue_ptr->DataType = 1;			    // Integer Data type	
			}								
		}
		if (dttype == 2) {
				doublevector = (double *)TMPHANDLE;		
				doublevector[0] = DATAVALUE(symhString2,0);	
				dyn.DbjParmValue_ptr->DoubleContentVector = doublevector;
				dyn.DbjParmValue_ptr->DataType = 2;			    // double Data type	
			}	
	}
	if (isLogic(symhString2)) {
		logicvector = (unsigned int *)TMPHANDLE;		
		//for (i = 0;i< 2;i++) 
				logicvector[0] = (unsigned int )DATAVALUE(symhString2,0);	
		dyn.DbjParmValue_ptr->BoolContentVector = logicvector;
		dyn.DbjParmValue_ptr->DataType = 4;								
	}		
	if (isChar(symhString2)) {
		StrContentVector = (char **)TMPHANDLE;		
		for (i = 0; i < strcount;	i++, strvector = skipStrings(strvector, 1)) 
				StrContentVector[i] = strvector;	
		dyn.DbjParmValue_ptr->StrContentVector =  StrContentVector;		
		dyn.DbjParmValue_ptr->DataType = 3;								
	}			
		
  dyn.DbjParmValue_ptr->name = DataobjectName;
  //dyn.DbjParmValue_ptr->RowsNumber = rowposition;
  //dyn.DbjParmValue_ptr->ColumnNumber = colposition;	
  dyn.DbjParmValue_ptr->RowsNumber = rowposition - 1;
  dyn.DbjParmValue_ptr->ColumnNumber = colposition - 1;	
  dyn.type = CPC_DYN_SETDATAOBJECTELEMENT;

  if (cpc_doDyc(&dyn))  {
		sprintf(OUTSTR,"ERROR: dataobject name already exist in %s()",
				FUNCNAME);
		goto errorExit;
	}			
	return (NULLSYMBOL);

  errorExit:
	putErrorOUTSTR();
	return (0);	
}

Symbolhandle    reset2dyn(Symbolhandle list)
{
	Symbolhandle    symhString;
	Symbolhandle    value;
	long            nargs = NARGS(list);
	char           *DataobjectName;
	long           valuerows;
	long           valuecols;
	int            i,j;
	DbjValue        dyn_DbjValue;
	double         *doublevector;
	int            *intvector;
	unsigned int   *logicvector;
	char           *strvector;
	long           strcount;
	char           **StrContentVector;
	long           valuesize;
	long           dttype;
	
/* reset all elements of the dataobject */
	
	WHERE("reset2dyn");
	
	cpc_dyn          dyn;
	
	*OUTSTR = '\0';

	if (nargs != 2)
	{
		badNargs(FUNCNAME, 2);
		goto errorExit;
	}
	
	symhString = COMPVALUE(list,0);

	DataobjectName = (char *) 0;
	if (symhString == EmptyArgument || !isCharOrString(symhString) ||
		*(DataobjectName = STRINGPTR(symhString)) == 0)
	{
		notCharOrString("Dataobject name");
		goto errorExit;
	}
	
	if(!DataobjectName)
	{
		sprintf(OUTSTR,"ERROR: dataobject names must be CHAR in %s()",
				FUNCNAME);
		goto errorExit;
	}
	
	dyn.DbjParmValue_ptr = &dyn_DbjValue;
	dyn.type = CPC_DYN_GETDATAOBJECTPROPERTY;
	dyn.name = DataobjectName;
	dyn.DbjParmValue_ptr->name = DataobjectName;	
	if (cpc_doDyc(&dyn)) {
		sprintf(OUTSTR,"ERROR: dataobject name doesn't exist in %s()", FUNCNAME);
		goto errorExit;
	}		
	dttype = dyn.DbjParmValue_ptr->DataType;
	
	symhString = COMPVALUE(list,1);
	valuerows = DIMVAL(symhString,1);
	if (NDIMS(symhString) == 1)
		valuecols =	1;
	else	
		valuecols = DIMVAL(symhString,2);
	dyn.name = DataobjectName;	
	dyn.DbjParmValue_ptr = &dyn_DbjValue;
	unsigned int typematchflag = 0;
	if (isReal(symhString) && ((dttype==1)||(dttype==2))) 	{
		valuesize = (valuerows *  valuecols + 15)* sizeof(double) + 10;
		typematchflag = 1;
	}
	if (isLogic(symhString) && (dttype==4)) {
		valuesize = (valuerows *  valuecols + 15)* sizeof(unsigned int) + 10;
		typematchflag = 1;
	}		
	if (isChar(symhString) && (dttype==3)) {
		strcount = symbolSize(symhString);
		strvector = STRINGPTR(symhString);
		valuesize = (strcount)* sizeof(char *) + 10;
		typematchflag = 1;
	}
	if (typematchflag == 0) {
		sprintf(OUTSTR,"ERROR: data type mismatch in %s()", FUNCNAME);
		goto errorExit;		
	}	
	
	TMPHANDLE = mygethandle(valuesize);
 	if (TMPHANDLE == (char **) 0)	{
		goto errorExit;
	}	
	if (isReal(symhString)) {		
		if (isMyIntValue(symhString)) {		
			doublevector = (double *)TMPHANDLE;		
			for (i = 0;i< valuerows *  valuecols;i++) 
				doublevector[i] = DATAVALUE(symhString,i);	
			dyn.DbjParmValue_ptr->DoubleContentVector = doublevector;
			dyn.DbjParmValue_ptr->DataType = 2;			    // double Data type	
		}	
		else {
			intvector	= (int *)TMPHANDLE;		
			for (i = 0;i< valuerows *  valuecols;i++) 
				intvector[i] = DATAVALUE(symhString,i);	
			dyn.DbjParmValue_ptr->IntContentVector = intvector;
			dyn.DbjParmValue_ptr->DataType = 1;			    // Integer Data type	
		}					
	}
	if (isLogic(symhString)) {
		logicvector = (unsigned int *)TMPHANDLE;		
		for (i = 0;i< valuerows *  valuecols;i++) 
				logicvector[i] = (unsigned int )DATAVALUE(symhString,i);	
		dyn.DbjParmValue_ptr->BoolContentVector = logicvector;
		dyn.DbjParmValue_ptr->DataType = 4;								
	}		
	if (isChar(symhString)) {
		StrContentVector = (char **)TMPHANDLE;		
		for (i = 0; i < strcount;	i++, strvector = skipStrings(strvector, 1)) 
				StrContentVector[i] = strvector;	
		dyn.DbjParmValue_ptr->StrContentVector =  StrContentVector;		
		dyn.DbjParmValue_ptr->DataType = 3;								
	}			
		
  dyn.DbjParmValue_ptr->name = DataobjectName;
  dyn.DbjParmValue_ptr->RowsNumber = valuerows;
  dyn.DbjParmValue_ptr->ColumnNumber = valuecols;	
  dyn.type = CPC_DYN_SETDATAOBJECTVALUE;

  if (cpc_doDyc(&dyn))  {
		sprintf(OUTSTR,"ERROR: dataobject name doesn't exist in %s()",
				FUNCNAME);
		goto errorExit;
	}

	return (NULLSYMBOL);

  errorExit:
	putErrorOUTSTR();
	return (0);
	
}
