/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 5.01 or later
*(C)*
*(C)* Copyright (c) 2000-2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*                                                                 ^F
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/


#include "../globals.h"

#include "../mvIface.h"

#include "../mvIo.h"


/*
  string manipulation
*/

Symbolhandle    replacestr(Symbolhandle list)
{
	Symbolhandle    symhString,result;
	long            nargs = NARGS(list), newlength, inlen, outlen, srclen, offset;
	long            instances;
	char           *source, *out, *in, *new, *where;
	WHERE("replacestr");
	
	*OUTSTR = '\0';

	if (nargs < 3)
	{
		badNargs(FUNCNAME, atLeast3Args);
		goto errorExit;
	}

	symhString = COMPVALUE(list,0);

	source = (char *) 0;
	if (symhString == EmptyArgument || !isCharOrString(symhString) ||
		*(source = STRINGPTR(symhString)) == 0 ||
		NDIMS(symhString) > 1 || DIMVAL(symhString,1) > 1)
	{
		sprintf(OUTSTR,"source string must be a character scalar");
		goto errorExit;
	}
	
	symhString = COMPVALUE(list,1);

	out = (char *) 0;
	if (symhString == EmptyArgument || !isCharOrString(symhString) ||
		*(out = STRINGPTR(symhString)) == 0 ||
		NDIMS(symhString) > 1 || DIMVAL(symhString,1) > 1)
	{
		sprintf(OUTSTR,"target string must be a character scalar");
		goto errorExit;
	}
	
	symhString = COMPVALUE(list,2);

	in = (char *) 0;
	if (symhString == EmptyArgument || !isCharOrString(symhString) ||
		*(in = STRINGPTR(symhString)) == 0 ||
		NDIMS(symhString) > 1 || DIMVAL(symhString,1) > 1)
	{
		sprintf(OUTSTR,"replacement string must be a character scalar");
		goto errorExit;
	}
	
	result = Install(SCRATCH,CHAR);

	if (result == (Symbolhandle) 0)
	{
		goto errorExit;
	}
	setNAME(result, SCRATCH);
	setTYPE(result, CHAR);
	setNDIMS(result,1);
	setDIM(result,1,1);

	instances = 0;
	inlen = strlen(in);
	outlen = strlen(out);
	srclen = strlen(source);

	where = source;
	while(where = strstr(where,out))
	{
		instances++;
		where += outlen;
	}
	if(instances == 0)
	{
		TMPHANDLE = mygethandle(strlen(source) + 1);  
		setSTRING(result, TMPHANDLE);
		if (TMPHANDLE == (char **) 0)
		{
			goto errorExit;
		}
		strcpy(*TMPHANDLE,source);
		return (result);
	}

	if(nargs == 3) 
	{ /* no fourth argument means first instance only */
		instances = 1;
	}

	newlength = srclen+instances*(inlen-outlen);
	TMPHANDLE = mygethandle(newlength + 1);  
	setSTRING(result, TMPHANDLE);
	if (TMPHANDLE == (char **) 0)
	{
		goto errorExit;
	}
	(*TMPHANDLE)[0] = '\0';
	where = source;
	while(instances > 0)
	{
		instances--;
		where = strstr(source,out);
		offset = where - source;
		strncat(*TMPHANDLE,source,offset);
		strcat (*TMPHANDLE,in);
		source = source+offset+outlen;
	}
	strcat(*TMPHANDLE, source);

	return (result);

  errorExit:
	putErrorOUTSTR();
	return (0);
	
} 


