/*
   This file consists of various macros and declarations from readline.c
   together with the modified source for those functions that were changed
   by kb@stat.umn.edu for use with the DJGPP version of MacAnova
*/
/* readline.c -- a general facility for reading lines of input
   with emacs style editing and completion.  */

/* Copyright 1987, 1989, 1991, 1992 Free Software Foundation, Inc.

   This file contains the Readline Library (the Library), a set of
   routines for providing Emacs style line input to programs that ask
   for it.

   The Library is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   The Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* Remove these declarations when we have a complete libgnu.a. */
/* #define STATIC_MALLOC */

/*
   This file contains only rl_read_init_file() and rl_getc(), the only
   functions needing modification for use with the DGJPP version of
   MacAnova for DOS.
*/

#if !defined (STATIC_MALLOC)
extern char *xmalloc (), *xrealloc ();
#else
static char *xmalloc (), *xrealloc ();
#endif /* STATIC_MALLOC */

#include "sysdep.h"
#include <stdio.h>
#include <fcntl.h>
#ifndef	NO_SYS_FILE
#include <sys/file.h>
#endif
#include <signal.h>

#if defined (HAVE_UNISTD_H)
#  include <unistd.h>
#endif

#define NEW_TTY_DRIVER
#define HAVE_BSD_SIGNALS
/* #define USE_XON_XOFF */

#ifdef __MSDOS__
#undef NEW_TTY_DRIVER
#undef HAVE_BSD_SIGNALS
#endif

/* Some USG machines have BSD signal handling (sigblock, sigsetmask, etc.) */
#if defined (USG) && !defined (hpux)
#undef HAVE_BSD_SIGNALS
#endif

/* System V machines use termio. */
#if !defined (_POSIX_VERSION)
#  if defined (USG) || defined (hpux) || defined (Xenix) || defined (sgi) || defined (DGUX) || defined (__H3050R) || defined (__H3050RX)
#    undef NEW_TTY_DRIVER
#    define TERMIO_TTY_DRIVER
#    include <termio.h>
#    if !defined (TCOON)
#      define TCOON 1
#    endif
#  endif /* USG || hpux || Xenix || sgi || DUGX */
#endif /* !_POSIX_VERSION */

/* Posix systems use termios and the Posix signal functions. */
#if defined (_POSIX_VERSION)
#  if !defined (TERMIOS_MISSING)
#    undef NEW_TTY_DRIVER
#    define TERMIOS_TTY_DRIVER
#    include <termios.h>
#  endif /* !TERMIOS_MISSING */
#  define HAVE_POSIX_SIGNALS
#  if !defined (O_NDELAY)
#    define O_NDELAY O_NONBLOCK	/* Posix-style non-blocking i/o */
#  endif /* O_NDELAY */
#endif /* _POSIX_VERSION */

/* Other (BSD) machines use sgtty. */
#if defined (NEW_TTY_DRIVER)
#include <sgtty.h>
#endif

/* Define _POSIX_VDISABLE if we are not using the `new' tty driver and
   it is not already defined.  It is used both to determine if a
   special character is disabled and to disable certain special
   characters.  Posix systems should set to 0, USG systems to -1. */
#if !defined (NEW_TTY_DRIVER) && !defined (_POSIX_VDISABLE)
#  if defined (_POSIX_VERSION)
#    define _POSIX_VDISABLE 0
#  else /* !_POSIX_VERSION */
#    define _POSIX_VDISABLE -1
#  endif /* !_POSIX_VERSION */
#endif /* !NEW_TTY_DRIVER && !_POSIX_VDISABLE */

/* Define some macros for dealing with assorted signalling disciplines.

   These macros provide a way to use signal blocking and disabling
   without smothering your code in a pile of #ifdef's.

   SIGNALS_UNBLOCK;			Stop blocking all signals.

   {
     SIGNALS_DECLARE_SAVED (name);	Declare a variable to save the 
					signal blocking state.
	...
     SIGNALS_BLOCK (SIGSTOP, name);	Block a signal, and save the previous
					state for restoration later.
	...
     SIGNALS_RESTORE (name);		Restore previous signals.
   }

*/

#ifdef HAVE_POSIX_SIGNALS
							/* POSIX signals */

#define	SIGNALS_UNBLOCK \
      do { sigset_t set;	\
	sigemptyset (&set);	\
	sigprocmask (SIG_SETMASK, &set, (sigset_t *)NULL);	\
      } while (0)

#define	SIGNALS_DECLARE_SAVED(name)	sigset_t name

#define	SIGNALS_BLOCK(SIG, saved)	\
	do { sigset_t set;		\
	  sigemptyset (&set);		\
	  sigaddset (&set, SIG);	\
	  sigprocmask (SIG_BLOCK, &set, &saved);	\
	} while (0)

#define	SIGNALS_RESTORE(saved)		\
  sigprocmask (SIG_SETMASK, &saved, (sigset_t *)NULL)


#else	/* HAVE_POSIX_SIGNALS */
#ifdef HAVE_BSD_SIGNALS
							/* BSD signals */

#define	SIGNALS_UNBLOCK			sigsetmask (0)
#define	SIGNALS_DECLARE_SAVED(name)	int name
#define	SIGNALS_BLOCK(SIG, saved)	saved = sigblock (sigmask (SIG))
#define	SIGNALS_RESTORE(saved)		sigsetmask (saved)


#else  /* HAVE_BSD_SIGNALS */
							/* None of the Above */

#define	SIGNALS_UNBLOCK			/* nothing */
#define	SIGNALS_DECLARE_SAVED(name)	/* nothing */
#define	SIGNALS_BLOCK(SIG, saved)	/* nothing */
#define	SIGNALS_RESTORE(saved)		/* nothing */


#endif /* HAVE_BSD_SIGNALS */
#endif /* HAVE_POSIX_SIGNALS */

/*  End of signal handling definitions.  */


#include <errno.h>
extern int errno;

#include <setjmp.h>
#include <sys/stat.h>

/* Posix macro to check file in statbuf for directory-ness. */
#if defined (S_IFDIR) && !defined (S_ISDIR)
#define S_ISDIR(m) (((m)&S_IFMT) == S_IFDIR)
#endif

#ifndef __MSDOS__
/* These next are for filename completion.  Perhaps this belongs
   in a different place. */
#include <pwd.h>
#endif /* __MSDOS__ */

#if defined (USG) && !defined (isc386) && !defined (sgi)
struct passwd *getpwuid (), *getpwent ();
#endif

/* #define HACK_TERMCAP_MOTION */

/* Some standard library routines. */
#include "readline.h"
#include "history.h"

#ifndef digit
#define digit(c)  ((c) >= '0' && (c) <= '9')
#endif

#ifndef isletter
#define isletter(c) (((c) >= 'A' && (c) <= 'Z') || ((c) >= 'a' && (c) <= 'z'))
#endif

#ifndef digit_value
#define digit_value(c) ((c) - '0')
#endif

#ifndef member
#define member(c, s) ((c) ? index ((s), (c)) : 0)
#endif

#ifndef isident
#define isident(c) ((isletter(c) || digit(c) || c == '_'))
#endif

#ifndef exchange
#define exchange(x, y) {int temp = x; x = y; y = temp;}
#endif

#if !defined (rindex)
extern char *rindex ();
#endif /* rindex */

#if !defined (index)
extern char *index ();
#endif /* index */

extern char *getenv ();
extern char *tilde_expand ();

static update_line ();
static void output_character_function ();
static delete_chars ();
static void insert_some_chars ();

#if defined (VOID_SIGHANDLER)
#  define sighandler void
#else
#  define sighandler int
#endif /* VOID_SIGHANDLER */

/* This typedef is equivalant to the one for Function; it allows us
   to say SigHandler *foo = signal (SIGKILL, SIG_IGN); */
typedef sighandler SigHandler ();

/* If on, then readline handles signals in a way that doesn't screw. */
#define HANDLE_SIGNALS

#ifdef __GO32__
#include <pc.h>
#undef HANDLE_SIGNALS
/*
	Changes by kb@stat.umn.edu for use of readline with MacAnova are
	enabled by defining INPUTRCFIX and ARROWKEYFIX.

	When INPUTRCFIX is defined the inputrc file is opened in binary mode
	and extraneous CR's filtered out.  If not done, then the file size
	info from stat does not match what is read.

	Defining ARROWKEYFIX enables code that let PC arrow keys work the way
	you would expect them to

	These changes would appear desirable for any MSDOS use of readline, not
	just MacAnova
*/
#define INPUTRCFIX  /*makes changes needed to use INPUTRC*/
#define ARROWKEYFIX /*enables proper use of arrow keys*/
#endif


/* **************************************************************** */
/*								    */
/*			Line editing input utility		    */
/*								    */
/* **************************************************************** */

/* A pointer to the keymap that is currently in use.
   By default, it is the standard emacs keymap. */
Keymap keymap = emacs_standard_keymap;

#ifdef __MSDOS__
/* Don't know what to do, but this is a guess */
static char *last_readline_init_file = "/INPUTRC";
#else
static char *last_readline_init_file = "~/.inputrc";
#endif
/* Do key bindings from a file.  If FILENAME is NULL it defaults
   to `~/.inputrc'.  If the file existed and could be opened and
   read, 0 is returned, otherwise errno is returned. */
int
rl_read_init_file (filename)
     char *filename;
{
  register int i;
  char *buffer, *openname, *line, *end;
  struct stat finfo;
  int file;

  /* Default the filename. */
  if (!filename)
    filename = last_readline_init_file;

  openname = tilde_expand (filename);
  if (!openname || *openname == '\000')
    return ENOENT;

#ifdef INPUTRCFIX
  /*
	 When not opened in binary mode, the stat file length does not
	 match the number of bytes read in
  */
  if ((stat (openname, &finfo) < 0) ||
      (file = open (openname, O_RDONLY|O_BINARY, 0666)) < 0) /*KB 950919*/
#else /*INPUTRCFIX*/
  if ((stat (openname, &finfo) < 0) ||
      (file = open (openname, O_RDONLY, 0666)) < 0)
#endif /*INPUTRCFIX*/
    {
      free (openname);
      return (errno);
    }
  else
    free (openname);

  last_readline_init_file = filename;

  /* Read the file into BUFFER. */
  buffer = (char *)xmalloc (finfo.st_size + 1);
  i = read (file, buffer, finfo.st_size);
  close (file);

  if (i != finfo.st_size)
    return (errno);

  /* Loop over the lines in the file.  Lines that start with `#' are
     comments; all other lines are commands for readline initialization. */
  line = buffer;
  end = buffer + finfo.st_size;
  while (line < end)
    {
      /* Find the end of this line. */
      for (i = 0; line + i != end && line[i] != '\n'; i++);

      /* Mark end of line. */
      line[i] = '\0';
#ifdef INPUTRCFIX /*KB 950919 */
	  if (line[i-1] == '\r')
	  { /* trim off CR if any */
		  line[i-1] = '\0';
	  }
#endif /*INPUTRCFIX*/
      /* If the line is not a comment, then parse it. */
      if (*line != '#')
	rl_parse_and_bind (line);
      /* Move to the next line. */
      line += i + 1;
    }
  return (0);
}

#ifdef ARROWKEYFIX
/*
   rl_getc() has been hacked to translate MSDOS arrow keys to corresponding
   emacs movement key codes
*/

#define UP_ARROW     328
#define DOWN_ARROW   336
#define LEFT_ARROW   331
#define RIGHT_ARROW  333

#define CONTROL_B    0x02
#define CONTROL_F    0x06
#define CONTROL_N    0x0e
#define CONTROL_P    0x10
#endif /*ARROWKEYFIX*/

int
rl_getc (stream)
     FILE *stream;
{
	int result;
	unsigned char c;

#ifdef __GO32__
#ifdef ARROWKEYFIX
/* fix to make arrow keys equivalent to emacs motion keys */
	if (isatty(0))
	{
		int c = getkey();

		if(c >= 128) 
		{
			switch(c)
			{
			  case UP_ARROW:
				c = CONTROL_P;
				break;
			  case LEFT_ARROW:
				c = CONTROL_B;
				break;
			  case DOWN_ARROW:
				c = CONTROL_N;
				break;
			  case RIGHT_ARROW:
				c = CONTROL_F;
				break;
			  default:
				c &= 0x7f;
			} /*switch(c)*/
		} /*if(c >= 128) */

		return (c);
	} /*if (isatty(0))*/
#else /*ARROWKEYFIX*/
  if (isatty(0))
    return (getkey() & 0x7f);
#endif /*ARROWKEYFIX*/
#endif /* __GO32__ */

	while (1)
    {
		result = read (fileno (stream), &c, sizeof (char));

		if (result == sizeof (char))
			return (c);

		/* If zero characters are returned, then the file that we are
		   reading from is empty!  Return EOF in that case. */
		if (result == 0)
			return (EOF);

#ifndef __GO32__
		/* If the error that we received was SIGINT, then try again,
		   this is simply an interrupted system call to read ().
		   Otherwise, some error ocurred, also signifying EOF. */
		if (errno != EINTR)
			return (EOF);
#endif /* !__GO32__ */
    }
}

