/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.13 or later
*(C)*
*(C)* Copyright (c) 2003 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/


#ifdef SEGMENTED
#if defined(MPW) || defined(MW_CW)
#pragma segment Equal
#endif /*MPW||MW_CW*/
#endif /*SEGMENTED*/

#include "globals.h"

/*
  030520 workhorse of equal()
  030522 modified to allow for approximate equality
*/
static int doEqual(Symbolhandle arg1, Symbolhandle arg2,
				   char why [], int level, long compnumbers[],
				   int chknames, int chklabels, int chknotes,
				   double fuzz, int relative)
{
	long        type = TYPE(arg1);
	char       *part;

	if (arg1 == arg2)
	{
		/* arguments are the same */
		goto argsEqual;
	}
	
	if (type != TYPE(arg2))
	{
		goto typesDiffer;
	}

	if (type == NULLSYM)
	{ /* 2 NULL arguments are equal */
		goto argsEqual;
	}
	
	if (type == PLOTINFO)
	{
		sprintf(why, "ERROR: can't compare two graph %s for equality",
				(level) ? "component" : "arguments");
		goto errorExit;
	}

	if (type == CHAR || type == LOGIC || type == REAL || type == LONG)
	{
		long      i, length;
		
		if (NDIMS(arg1) != NDIMS(arg2))
		{
			goto ndimsDiffer;
		}
		
		for (i = 1; i <= NDIMS(arg1); i++)
		{
			if (DIMVAL(arg1,i) != DIMVAL(arg2, i))
			{
				goto dimensionsDiffer;
			}
		} /*for (i = 1; i <= NDIMS(arg1); i++)*/

		length = symbolSize(arg1);
		
		if (type == CHAR)
		{
			char     *string1 = STRINGPTR(arg1);
			char     *string2 = STRINGPTR(arg2);

			for (i = 0; i < length; i++)
			{
				if (strcmp(string1, string2) != 0)
				{
					goto valuesDiffer;
				}
				string1 = skipStrings(string1, 1);
				string2 = skipStrings(string2, 1);
			} /*for (i = 0; i < length; i++)*/
		}
		else if (type == LONG)
		{
			long           *longvalue1 = LONGDATAPTR(arg1);
			long           *longvalue2 = LONGDATAPTR(arg2);

			for (i = 0; i < length; i++)
			{
				if (longvalue1[i] != longvalue2[i])
				{
					goto valuesDiffer;
				}
			}
		}
		else
		{
			/* REAL and LOGICAL */
			double         *datavalue1 = DATAPTR(arg1);
			double         *datavalue2 = DATAPTR(arg2);
			
			for (i = 0; i < length; i++)
			{
				if (isMissing(datavalue1[i]) || isMissing(datavalue2[i]))
				{
					if (!isMissing(datavalue1[i]) || !isMissing(datavalue2[i]))
					{
						goto valuesDiffer;
					}
				}
				else if (type == REAL)
				{
					if (!fuzzyEqual(datavalue1[i], datavalue2[i],
									fuzz, relative))
					{
						goto valuesDiffer;
					}
				}
				else if ((datavalue1[i] != 0.0 || datavalue2[i] != 0.0) &&
						 (datavalue1[i] == 0.0 || datavalue2[i] == 0.0))
				{
					goto valuesDiffer;
				}
			} /*for (i = 0; i < length; i++)*/
		}
	} /*if (type == CHAR || type == LOGIC || type == REAL || type == LONG)*/
	else if (type == MACRO)
	{
		if (strcmp(STRINGPTR(arg1), STRINGPTR(arg2)) != 0)
		{
			goto macrosDiffer;
		}
	}
	else if (type == STRUC)
	{
		long       icomp, ncomps = NCOMPS(arg1);
		
		if (ncomps != NCOMPS(arg2))
		{
			goto ncompsDiffer;
		}
		for (icomp = 0; icomp < ncomps; icomp++)
		{
			int   status = doEqual(COMPVALUE(arg1, icomp),
								   COMPVALUE(arg2, icomp),
								   why, level + 1, compnumbers,
								   chknames, chklabels, chknotes,
								   fuzz, relative);
			if (status)
			{
				if (level <= MAXDIMS)
				{
					compnumbers[level] = icomp + 1;
				}
				return (status);
			}
		} /*for (icomp = 0; icomp < ncomps; icomp++)*/

		if (chknames)
		{
			for (icomp = 0; icomp < ncomps; icomp++)
			{
				if (strcmp(NAME(COMPVALUE(arg1, icomp)),
						   NAME(COMPVALUE(arg2, icomp))) != 0)
				{
					if (level <= MAXDIMS)
					{
						compnumbers[level] = icomp + 1;
					}
					goto namesDiffer;
				}
			} /*for (icomp = 0; icomp < ncomps; icomp++)*/
		} /*if (chknames)*/
	}
	else if (type == BLTIN)
	{
		goto functionsDiffer;
	}

	if (chklabels && (HASLABELS(arg1) || HASLABELS(arg2)))
	{
		long        i, nlabels = 0;
		char       *labels1, *labels2;

		if (!HASLABELS(arg1) || !HASLABELS(arg2))
		{
			goto labelsDiffer;
		}

		for (i = 1; i <= NDIMS(arg1); i++)
		{
			nlabels += DIMVAL(arg1, i);
		}

		labels1 = LABELSPTR(arg1);
		labels2 = LABELSPTR(arg2);
		for (i = 0; i < nlabels; i++)
		{
			if (strcmp(labels1, labels2) != 0)
			{
				goto labelsDiffer;
			}
			labels1 = skipStrings(labels1, 1);
			labels2 = skipStrings(labels2, 1);
		} /*for (i = 0; i < nlabels; i++)*/		
	}
		
	if (chknotes && (HASNOTES(arg1) || HASNOTES(arg2)))
	{
		long        i;
		char       *notes1, *notes2;
		
		if (!HASNOTES(arg1) || !HASNOTES(arg2) ||
			DIMVAL(NOTES(arg1), 1) != DIMVAL(NOTES(arg2), 1))
		{
			goto notesDiffer;
		}
		notes1 = NOTESPTR(arg1);
		notes2 = NOTESPTR(arg2);

		for (i = 0; i < DIMVAL(NOTES(arg1), 1); i++)
		{
			if (strcmp(notes1, notes2) != 0)
			{
				goto notesDiffer;
			}
			notes1 = skipStrings(notes1, 1);
			notes2 = skipStrings(notes2, 1);
		}
	} /*if (chknotes && (HASNOTES(arg1) || HASNOTES(arg2)))*/
	
	/* arguments are equal*/
  argsEqual:
	return (0);

  typesDiffer:
	part = "types";
	goto differExit;

  ndimsDiffer:
	part = "number of dimensions";
	goto differExit;

  dimensionsDiffer:
	part = "dimensions";
	goto differExit;
	
  valuesDiffer:
	part = "values";
	goto differExit;
	
  macrosDiffer:
	part = "macros";
	goto differExit;
	
  functionsDiffer:
	part = "functions";
	goto differExit;

  ncompsDiffer:
	part = "number of structure components";
	goto differExit;
	
  labelsDiffer:
	part = "labels";
	goto differExit;
	
  notesDiffer:
	part = "notes";
	goto differExit;
	
  namesDiffer:
	part = "names";
	goto differExit;
	
  differExit:
	sprintf(why, "%s differ", part);
	return (1);

  errorExit:
	return (-1);
	
} /*doEqual()*/

/*
  function to find how much room is needed for explanation of difference
  (when reason == (char *) 0) and to construct explanation (when
  reason != (char *) 0

  argsDiffer != 0 only when some difference has been found

  When argsDiffer != 0 and compnumbers[0] == 0, the arguments to equal()
  were either not both structures or differed in top level number of
  components, component names, labels, or notes

  When argsDiffer != 0 and compnumbers[0] != 0, compnumbers[0],
  compnumbers[1], ..., compnumbers[k-1] identify the component that
  differed, where compnumbers[k-1] != 0 and compnumbers[k] == 0.
  k is at most the greatest depth of nested structures.
*/
static long     makeReason(int argsDiffer, long compnumbers [], char * why,
						   char * reason)
{
	long              needed;
	
	if (!argsDiffer)
	{
		needed = strlen(why) + 1;
		
		if (reason)
		{
			strcpy(reason, why);
		}
	} /*if (!argsDiffer)*/
	else
	{
		int          level;
		char        *what; 
		/* 
		   message of one of forms
		   "Argument xxxx differ"
		   "Structure component [i][j]...[k] xxxx differ"
		*/

		if (compnumbers[0] == 0)
		{
			what = "Argument";
		}
		else
		{
			what = "Structure component ";
		}
		
		needed = strlen(what) + 1; /*+1 for terminating '\0'*/
		if (reason)
		{
			strcpy(reason, what);
			reason += strlen(what);
		}
		
		for (level = 0; level < MAXDIMS; level++)
		{
			char        number[20];
			
			if (compnumbers[level] == 0)
			{
				break;
			}
			sprintf(number, "[%ld]",  compnumbers[level]);
			needed += strlen(number);
			if (reason)
			{
				strcpy(reason, number);
				reason += strlen(number);
			}
		}
		needed += strlen(why) + 1; /* +1 for prepended space*/
		if (reason)
		{
			*reason++ = ' ';

			strcpy(reason, why);
		} /*if (reason)*/
	} /*if (!argsDiffer){}else{}*/

	return (needed);
} /*makeReason()*/
/*
  Function to test two symbols for equality

  equal(arg1, arg2) returns T if and only if the data in arg1 and
  arg2 is identical.  If arg1 and arg2 are structures, all component
  names must be identical.  Coordinate labels and attached notes
  are not compared.

  equal(arg1, arg2, explain:T) returns the result of the comparison
  and an explanation for it as structure(equal:T or F,why:explanation)
  where explanation is a CHARACTER scalar such as "Variables are equal"
  or "Structure component [1][2] values differ".

  equal(arg1, arg2, chknames:F [,explain:T]) does the same, but component
  names are not checked.

  equal(arg1, arg2, chklabels:T [,explain:T]) also checks coordinate labels
  for identity

  equal(arg1, arg2, chknotes:T [,explain:T]) also checks attach notes for
  identity

  Any combination of 'chknames', 'chklabels' and 'chknotes' may be used.

  030516 New function based on a macro with the same name
  030520 modified keywords and added identification of differing structure
         component with explain:T
  030522 added keyword phrases fuzz:epsilon, relative:T or F
*/

Symbolhandle isEqual(Symbolhandle list)
{
	Symbolhandle          arg1, arg2;
	Symbolhandle          result = SymbolZero;
	double                fuzz = 0.0;
	long                  iarg, nargs = NARGS(list);
	long                  compnumbers[MAXDIMS + 1];
	int                   level;
	int                   argsDiffer;
	int                   explain = 0, relative = 0;
	int                   chknames = 1, chklabels = 0, chknotes = 0;
	char                  why[100];

	for (level = 0; level <= MAXDIMS; level++)
	{
		compnumbers[level] = 0;
	}
	level = 0;

	if (nargs < 2)
	{
		badNargs(FUNCNAME, atLeast2Args);
		goto errorExit;
	}
	for (iarg = 2; iarg < nargs; iarg++)
	{
		Symbolhandle     symhKey = COMPVALUE(list, iarg);
		char            *keyword = isKeyword(symhKey);
		
		if (keyword)
		{
			int         logValue = (isTorF(symhKey)) ? 
			  DATAVALUE(symhKey, 0) != 0.0 : -1;

			if (strcmp(keyword, "explain") == 0)
			{
				explain = logValue;
			}
			else if (strncmp(keyword, "chknam", 6) == 0)
			{
				chknames = logValue;
			}
			else if (strncmp(keyword, "chklabel", 8) == 0)
			{
				chklabels = logValue;
			}
			else if (strncmp(keyword, "chknote", 7) == 0)
			{
				chknotes = logValue;
			}
			else if (strncmp(keyword, "relative", 7) == 0)
			{
				relative = logValue;
			}
			else if (strcmp(keyword, "fuzz") == 0)
			{
				if (!isNumber(symhKey, POSITIVEVALUE))
				{
					notPositiveReal(keyword);
					goto errorExit;
				}
				fuzz = DATAVALUE(symhKey, 0);
				logValue = 1;
			}
			else
			{
				badKeyword(FUNCNAME, keyword);
				goto errorExit;
			}
			if (logValue < 0)
			{
				notTorF(keyword);
				goto errorExit;
			}
		} /*if (keyword)*/
		else
		{
			sprintf(OUTSTR,
					"ERROR: %s() argument %ld > 2 not keyword phrase",
					FUNCNAME, iarg + 1);
			goto errorExit;
		}
	} /*for (iarg = 2; iarg < nargs; iarg++)*/
	
	if (relative && fuzz == 0.0)
	{
		sprintf(OUTSTR,
				"WARNING: relative:T in %s() ignored without fuzz:epsilon",
				FUNCNAME);
		putErrorOUTSTR();
	} /*if (relative && fuzz == 0.0)*/
	
	arg1 = COMPVALUE(list, 0);
	arg2 = COMPVALUE(list, 1);
	
	if (arg1 == EmptyArgument || arg2 == EmptyArgument)
	{
		sprintf(OUTSTR,
				"ERROR: %s() argument %d is missing",
				FUNCNAME, (arg1 == EmptyArgument) ? 1 : 2);
		goto errorExit;
	}
	if (!isDefined(arg1) || !isDefined(arg2))
	{
		sprintf(OUTSTR,
				"ERROR: %s() argument %d is not defined",
				FUNCNAME, (!isDefined(arg1)) ? 1 : 2);
		goto errorExit;
	}
	argsDiffer = doEqual(arg1, arg2, why, level, compnumbers,
						 chknames, chklabels, chknotes, fuzz, relative);
	if (argsDiffer < 0)
	{
		strcpy(OUTSTR, why);
		goto errorExit;
	}
	
	if (!explain)
	{
		result = RInstall(SCRATCH, 1);
		if (result == SymbolError)
		{
			goto errorExit;
		}
		setTYPE(result, LOGIC);
		DATAVALUE(result, 0) = (!argsDiffer) ? 1.0 : 0.0;
	} /*if (!explain)*/
	else
	{
		Symbolhandle     symhComp;
		long             needed;
		
		if (!argsDiffer)
		{
			strcpy(why, "Arguments are equal");
		}
		
		result = StrucInstall(SCRATCH, 2);
		if (result == SymbolError)
		{
			goto errorExit;
		}
		COMPVALUE(result, 0) = symhComp = Makereal(1);
		if (symhComp == SymbolError)
		{
			goto errorExit;
		}
		setTYPE(symhComp, LOGIC);
		Setname(symhComp, "equal");
		DATAVALUE(symhComp, 0) = (!argsDiffer) ? 1.0 : 0.0;

		
		COMPVALUE(result, 1) = symhComp = Makechar(0);
		if (symhComp == SymbolError)
		{
			goto errorExit;
		}
		Setname(symhComp, "why");
		setNDIMS(symhComp, 1);
		setDIM(symhComp, 1, 1);
		needed = makeReason(argsDiffer, compnumbers, why, (char *) 0);

		TMPHANDLE = mygethandle(needed);
		setSTRING(symhComp, TMPHANDLE);
		if (TMPHANDLE == (char **) 0)
		{
			Removesymbol(result);
			goto errorExit;
		}
		(void) makeReason(argsDiffer, compnumbers, why, *TMPHANDLE);
		if (compnumbers[MAXDIMS])
		{
			sprintf(OUTSTR,
					"WARNING: only first %d levels identified", MAXDIMS);
			putErrorOUTSTR();
		}
	} /*if (!explain){}else{}*/
	return (result);
	
  errorExit:
	putErrorOUTSTR();
	return (SymbolError);
	
} /*isEqual()*/
