/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.11 or later
*(C)*
*(C)* Copyright (c) 2001 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/


/*
   This file contains only macro defines, no declarations except for
   some enums replacing defines.
   Version of 951201
   Version of 960620  SAVEBINARY7 and SAVEASCII7 defined

   970613 added MAXMDEPTH, removed MAXSTRING
   971016 added error codes for fsolve()
   980521 added codes used by scanPat() and matchName()
   980717 added additional save() and restore() defines
   980718 moved save/restore stuff to new header vmsave.h
   980801 Modified enums used by scanPat() and matchName()
   990903 Definition of MAXLINE increased from 500 to 2000 except for BCPP
          Put definition of OUTLINESIZE here to override definitions
          in macIo.c, macMain.c, wxIo.cc
   000817 moved definitions of NAMELENGTH, MAXCOORD and MAXDIMS to Symbol.h
          moved enums and other constants having to do with Symbol type
          and shape to Symbol.h
   001025 replaced MAXLINE related stuff by new stuff in mvIo.h
   010109 Moved define and constants related to storage allocation here
          from Symbol.h
   010516 Added enum badNargsCodes; moved enum argCountCodes from globkb.h
   030708 Moved several definitions and enums to mvutils.h (new file) and
          labels.h
*/
#ifndef GLOBDEFSH__
#define GLOBDEFSH__

/* miscellaneous globally defined macros */
#ifndef NAMEFORWHERE  /* incase dbug.h not included */
#define NAMEFORWHERE where__ 
#endif /*NAMEFORWHERE*/

#ifndef WHERE /* in case dbug.h not included */
/* NB: macro WHERE should be followed by ';' among declarations */
#define WHERE(name) char *NAMEFORWHERE = name
#endif /*WHERE*/

#ifndef HUGEDBL
#define HUGEDBL 1.79769313486e+308
#endif /*HUGEDBL*/

#ifndef HUGEINT
#define HUGEINT     2146727027   /*2^31 - 756621*/
#endif /*HUGEINT*/

/*
  Codes and defines used by scanPat(), matchName() in utils.c
  help(), list(), match(), keyvalue()
  030708 moved to mvutils.h
*/

/*
  opcodes for isInteger
  030708 moved to mvutils.h from globdefs.h
*/

/*
  010323 Codes for use with countArgs() in utils.c
  010516 moved to globdefs.h from globkb.h
  030708 moved to mvutils.h from globdefs.h
*/

 /* These are referenced in glm.c, iterglm.c, ipf.c, funbalan.c*/
enum convergenceErrors
{
	ITERERROR = 0,
	ITERCONVERGED,
	ITERUNCONVERGED
};


/* errorcodes for fsolve() in mathutil.c */
enum fsolveErrors
{
	FSOLVENOTCONVERGED = 300,
	FSOLVEBADARGUMENTS,
	FSOLVEBADGUESSES,
	FSOLVENONMONOTONIC
};

#define NONCONVERGED (-16000)

/* various globally defined limits */
/* NOTE: As of 930518, MAXVARS & MAXERRTRMS are defined in glm.h */
#define MAXIDEPTH       100 /* maximum number of nested if statements*/
#define MAXWDEPTH        20 /* maximum number of nested while or for loops*/
#define MAXBDEPTH        10 /* maximum number of nested batch() commands*/
#define MAXMDEPTH        49 /* maximum number of nested macro calls */
#define MAXLEVEL         99 /* maximum permissible value for '$$' in macros*/
/* 001025 replaced MAXLINE definition by new stuff in mvIo.h */
#define OUTLINESIZE    1000 /* maximum length of output line */
#define BUFFERLENGTH    500 /* length of certain string buffers */

/* following is largest permissible value to bit operations */
#define MAXBITVALUE    4294967295.0 /*2^32-1 = ffffffff */

#ifndef PATHSIZE       /* length of buffers to hold complete path */
#ifdef MACINTOSH
#define PATHSIZE  255
#else /*MACINTOSH*/
#define PATHSIZE 256
#endif /*MACINTOSH*/
#endif /*PATHSIZE*/

/* Variously globally defined constants */
/* for use in getKeyword() and perhaps elsewhere */
#define OPMASK         0x00000fffUL
#define TYPE1MASK      0x000ff000UL
#define TYPE2MASK      0x0ff00000UL
#define CONTROLMASK    0xf0000000UL

/* 000817 moved masks having to do with type and shape to Symbols.h*/

/* miscellaneous control codes for recursion and elsewhere*/
#define LEFTRECUR      0x10000000UL /*1st argument or left operand is STRUC*/
#define RIGHTRECUR     0x20000000UL /*2nd argument or right operand is STRUC*/
#define REUSELEFT      0x40000000UL /*reuse 1st argument or left operand */
#define REUSERIGHT     0x80000000UL /*reuse 2nd argument or right operand */

/* miscellaneous status codes */
#define FOUNDMISSING    0x00100000UL
#define FOUNDNONPOS     0x00200000UL
#define FOUNDNEG        0x00400000UL
#define FOUNDDOMERR     0x00800000UL
#define FOUNDOVERFLOW   0x01000000UL
#define FOUNDZERODIV    0x02000000UL

/* Various op codes and return values */
/* op codes for dimcmp() */
#define STRICT          0x00000001UL
#define NEARSTRICT      0x00000002UL
#define HCONCAT         0x00000004UL
#define VCONCAT         0x00000008UL

/*
  opcode constants for badNargs()
  Typical calls
    badNargs(FUNCNAME, 1);
	badNargs(FUNCNAME, atLeast2Args);
	badNargs(FUNCNAME, atMost3Args);
	badNargs(FUNCNAME, atMost0Args);
  Added 010516
*/

enum badNargsCodes
{
	atMost1Arg     =      -1,
	atMost2Args    =      -2,
	atMost3Args    =      -3,
	atMost4Args    =      -4,
	atMost5Args    =      -5,
	atMost6Args    =      -6,
	atMost7Args    =      -7,
	atMost8Args    =      -8,
	atMost9Args    =      -9,
	atMost0Args    =   -1000,
	atLeastNArgs   = atMost0Args, /*add to -nargs*/
	atLeast1Args   = atLeastNArgs - 1,
	atLeast2Args   = atLeastNArgs - 2,
	atLeast3Args   = atLeastNArgs - 3,
	atLeast4Args   = atLeastNArgs - 4,
	atLeast5Args   = atLeastNArgs - 5,
	atLeast6Args   = atLeastNArgs - 6,
	atLeast7Args   = atLeastNArgs - 7,
	atLeast8Args   = atLeastNArgs - 8,
	atLeast9Args   = atLeastNArgs - 9,
	limitArgs      =     5000, /*larger than any required number of args*/
	nonKeywordArgs =    10000, /*add to coded number*/
	keywordArgs    =    20000  /*add to coded number*/
};

/*return values from dimcmp() and treecmp()*/
#define HASMISSING      0x00000001UL
#define BADDIMS         0x00000002UL
#define BADSTRUC        0x00000004UL

/*
  defines related to labels
  030708 moved to labels.h
*/

/*
  defines and codes related to memory allocation
  010109 moved from Symbol.h
*/
#define BADHANDLE   ((void **) -1)
enum badHandleCodes
{
	NULLHANDLE = -1,
	CORRUPTEDHANDLE = -2
};

/*
  980718 moved stuff related to save and restore to new header mvsave.h
  000817 moved stuff defining type codes to Symbol.h
*/
#endif /*GLOBDEFSH__*/
