/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.13 or later
*(C)*
*(C)* Copyright (c) 2002 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

/*
   Version of 951201
   960716 Changed types Long, Char, Double, uChar to long, char, double,
          unsigned char
   961127 Added Global NPRODUCTS
   961209 Moved global and related macros to handlers.h
   970124 Added globals SELECTIONSYMBOL and SELECTIONNAME when
          HASSELECTION is defined (Motif)
   970312 Added DEFAULTLEFTBRACKETS and LEFTBRACKETS, Lparens and Rparens
          for use in Labeling; Also DEFAULTUSECOLLABS and USECOLLABS
          Thes may someday be settable by setoption()
   970409 Added LASTINPUTWASCR[], flags to indicate whether last character
          from input file was CR
   970506 Removed initialization of INPUTFILE (it's done by initialize)
          Also added casts to 0 initializations of pointers
   970530 Added globals with various mathematical constants MV_PI, MV_E, etc.
   970613 Added INPUTLEVEL, INPUTSTRINGS and ISTRCHARS and made INPUTSTRING
          and ISTRCHAR macros
   970616 Undid some of previous change, replacing them with INPUTSTACK and
          various macros for accessing components of INPUTSTACK[INPUTLEVEL]
          INPUTSTRING and ISTRCHAR remain actual variables for efficiency
   970714 Added global EVALDEPTH, to be incremented and then decremented
          by each call to evaluate() (or to mvEval() from user function
   970805 Added global ThisMachine and enum osCodes.  ThisMachine
		  is set to the appropriate code in initialize()
   970818 Added global SCRATCHHANDLE for ephemeral scratch use to avoid
          memory leaks.
   970820 Added scratchhandle to struct mvMacro and additional macros
          Added global EVALSCRATCH and macro isevaluated
   970823 Added funcname to struct mvMacro and additional macros, including
          replacement for global FUNCNAME
   980329 Reorganized handling of prompts to make possible a 'prompt' keyword
          for batch() and a command line argument.  Also changed globals
          BATCHECHO, INPUTFILE, INPUTFILENAMES, LASTINPUTWASCR
          to pointers with initialize() allocating space for them.
          New type promptType defined.
   980420 Further changes with prompts.  New global array DEFAULTPROMPTS
          (allocated in initialize()).  If a -p flag is used at startup
          DEFAULTPROMPTS[0] is set to its value; if prompt:newprompt is an
          argument to batch(), DEFAULTPROMPTS[BDEPTH] is set to newprompt;
          setoptions(default:T) resets array PROMPTS from DEFAULTPROMPTS
          Macro DEFAULTPROMPT redefined.  New macro STANDARDPROMPT is
          defined in initiali.c
   980624 Added globals HISTRY and DEFAULTSAVEHISTORY in connection with
          new keyword phrase history on save().  DEFAULTSAVEHISTORY
          will allow new option 'savehistory' for setoptions.
   980718 Moved stuff related to save() and restore() to mvsave.h
   980722 Moved stuff related to help to new header mvhelp.h
   980723 Added globals LABELSSCRATCH and NOTESSCRATCH providing names
          for symbols for labels and notes.
   980902 Corrected initialization for MLEVEL
   981118 Added enum and globals to control automatic searches for
          undefined macros
   990423 Replaced globals CLIPBOARDSYMBOL and SELECTIONSYMBOL with Global
          vector SpecialSymbols and CLIPBOARDNAME and SELECTION NAME with
          global vector SpecialSymbolNames, using defines to define
          the previous variables.  Added a define for GRAPHWINDOWSSYMBOL and
          GRAPHWINDOWSNAME.  These should be initialized by calls from
          initialize()
   990523 Moved stuff related to graphs to new header mvGraphs.h
   990619 Removed include "typedefs.h"
   990705 Changed typedef mvMacroName to mvSymbolName and moved to Symbol.h
          Use mvSymbolName in defining struct mvMacro
          Replaced MACRONAMES by MACROINFO to allow for saving the opening
          bracket count of macros preparatory to implementing a method
          of exiting from a macro
          Replaced macros PushMacroName and PopMacroName by PushMacroInfo and
          PopMacroInfo
   990820 Added globals ToTek and FromTek, initialized in setOptio.c
   000115 Added global VecreadOldStyle, may be initialized in setOptio.c
   000120 rename VecreadOldStyle to VecreadByfields
   000216 Added globals MaxInteger and MaxLong
   000502 Added global MAXINPUTLEVEL
   000503 Added global MVWEB in anticipation of access over the WWW
   000708 Changed WHILESTARTS, WHILECONDITIONS, WHILELIMITS, WHILEBRACKETLEV,
          FORVECTORS, IFCONDITIONS, IFBRACKETLEV to pointers, initialized in
          initialize().
   000817 Changed declaration of EVALNAME to use mvSymbolName
   000818 Removed unnecessary macros specifying length of names of different
          types of scratch symbols
   000821 Added globals ErrorStart and WarningStart
   001026 Moved globals and constants related to input buffer to mvIo.h
   010323 Added enum argCountCodes for use with countArgs
   010516 moved enum argCountCodes to globdefs.h
   010730 added global LOCKEDSRATCH
   010802 added global QUIET and modified struct mvMacroInfo and macro
          PushMacroInfo
   010809 added global SQBRACKETLEV and added sqbracketlev component to
          struct mvMacro; add macro ThisSqbracketlev
   010810 added macros PastMacroName, PastMacroBracketlev, PastMacroQuiet
   010811 Add TRACEBACK, the value for new option 'traceback:T or F'
   020807 Added new code DOPVALUE to enum formatCodes and added global
          MINPVALUE
   020819 Added global ENABLEDITEMS and enum enabledItemsCodes to implement
          new option enableitem
   050117 Moved definition of MACANOVAINI from here to version.h
*/
#ifndef GLOBKBH__
#define GLOBKBH__

#undef UNDEFINED__

#undef EXTERN
#undef INIT
#undef INITDIM
#undef INITARRAY
#undef COMMA

#ifdef MAIN__
#define EXTERN
#define INIT(VAL) = VAL
#define INITDIM(N) N
#define INITARRAY(VALS) = { VALS }
#else
#define EXTERN extern
#define INIT(VAL)
#define INITDIM(N)
#define INITARRAY(VALS)
#endif /*MAIN__*/

#define COMMA ,

#define INVISIBLESYMBOLS

#ifndef MISSINGVALUE
#define MISSINGVALUE -99999.9999
#endif

#define OLDMISSINGVALUE -99999.9999 /* MISSING through 3.36 (at least) */

#ifndef FILE
#include <stdio.h>
#endif /*FILE*/

/* globals maintained by Addsymbol & Cutsymbol*/
EXTERN long Nscratch INIT(0);
EXTERN long Maxscratch INIT(0);
EXTERN long Nsymbols INIT(0);
EXTERN long Maxsymbols INIT(0);
EXTERN long Nfunctions INIT(0);

/*globals for keeping track of memory usage */
EXTERN long Nhandles INIT(0);
EXTERN long Maxhandles INIT(0);
EXTERN long CurrentMemory INIT(0);
EXTERN long MaxMemory INIT(0);

#ifdef ACTIVEUPDATE
EXTERN short ActiveUpdate INIT(ACTIVEUPDATE);
#else /*ACTIVEUPDATE*/
EXTERN short ActiveUpdate INIT(0);
#endif /*ACTIVEUPDATE*/

typedef union Bdouble
{
	unsigned long     bits[LONGSPERDOUBLE];
	double            value;
} Bdouble, *BdoublePtr, **BdoubleHandle;

/* in the following, X must be of type Bdouble */
#define BdoublePart(X, I) ((X).bits[I])
#define BdoubleHi(X) BdoublePart(X, BDOUBLEHI)
#define BdoubleLow(X) BdoublePart(X, BDOUBLELOW)
#ifdef BDOUBLEMID
#define BdoubleMid(X) BdoublePart(X, BDOUBLEMID)
#endif /*BDOUBLEMID*/

/* In the following, X must be an lvalue */
#define DoublePart(X, I) (((BdoublePtr) &(X))->bits[I])
#define DoubleHi(X) DoublePart(X, BDOUBLEHI)
#define DoubleLow(X) DoublePart(X, BDOUBLELOW)
#ifdef BDOUBLEMID
#define DoubleMid(X) DoublePart(X, BDOUBLEMID)
#endif /*BDOUBLEMID*/

/* These should be initialized by initialize() */
/* internal representation of missing value */
EXTERN Bdouble Missing;    /* current missing value */
EXTERN Bdouble OldMissing; /* <= version 3.36 missing value */

#define MISSING (Missing.value)
#define OLDMISSING (OldMissing.value)

#ifdef HASINFINITY
/* internal representations for +infinity and -infinity */
EXTERN Bdouble PlusInfinity;
EXTERN Bdouble MinusInfinity;
#define PLUSINFINITY (PlusInfinity.value)
#define MINUSINFINITY (MinusInfinity.value)
#endif /*HASINFINITY*/

EXTERN Bdouble TooBigValue;/*set to  mystrtod(TOOBIGVALUESTRING,(char *) 0)*/
#define TOOBIGVALUE TooBigValue.value

/* Numerical Constants initialized in initiali.c */
EXTERN double MV_E;           /*2.7182818284590452354*/
EXTERN double MV_LOG2E;       /*1.4426950408889634074*/
EXTERN double MV_LOG10E;     /*0.43429448190325182765*/
EXTERN double MV_LN2;        /*0.69314718055994530942*/
EXTERN double MV_LN10;       /*2.30258509299404568402*/
EXTERN double MV_PI;         /*3.14159265358979323846*/
EXTERN double MV_PI_2;       /*1.57079632679489661923*/
EXTERN double MV_PI_4;       /*0.78539816339744830962*/
EXTERN double MV_1_PI;       /*0.31830988618379067154*/
EXTERN double MV_2_PI;       /*0.63661977236758134308*/
EXTERN double MV_2_SQRTPI;   /*1.12837916709551257390*/
EXTERN double MV_SQRT2;      /*1.41421356237309504880*/
EXTERN double MV_SQRT1_2;    /*0.70710678118654752440*/

EXTERN char      NullString [] INITARRAY('\0');

#ifndef MAXINTEGER
#ifdef DSIGNIF
#define MAXINTEGER -1.0
#else /*DSIGNIF*/
#define MAXINTEGER 4503599627370496.0 /*2^52, may not be appropriate*/
#endif /*DSIGNIF*/
#endif /*MAXINTEGER*/

#ifndef MAXLONG
#define MAXLONG   2147483647L /* maximum value of long > 0 */
#endif /*MAXLONG*/

EXTERN double MaxInteger INIT(MAXINTEGER); /*maximum integer double */
EXTERN double MaxLong    INIT(MAXLONG);    /*maximum long > 0 */

/* name for MISSING in output */
#define LENGTHMISSING   20
EXTERN char   DEFAULTNAMEFORMISSING[]  INIT("MISSING");
EXTERN char   NAMEFORMISSING[LENGTHMISSING+1];

#ifndef isMissing
#ifdef MISSINGISNAN
#define isMissing(X)  doubleEqualBdouble(X, Missing)
#else /*MISSINGISNAN*/
#define isMissing(X)  ((X) == MISSING)
#endif /*MISSINGISNAN*/
#endif /*isMissing*/

#ifndef setMissing
#define setMissing(X) (X = MISSING)
#endif /*setMissing*/

#ifdef HASINFINITY
/* name for Infinity in output if desired*/
#define LENGTHINFINITY  20
EXTERN char   DEFAULTNAMEFORINFINITY[]  INIT("Infinity");
EXTERN char   NAMEFORINFINITY[LENGTHINFINITY+1];

#ifndef isInfinite
#define isInfinite(X) (doubleEqualBdouble(X, PlusInfinity) ||\
					   doubleEqualBdouble(X, MinusInfinity))
#endif /*isInfinite*/

#ifndef setPlusInfinity
#define setPlusInfinity(X) (X = PLUSINFINITY)
#endif /*setPlusInfinity*/

#ifndef setMinusInfinity
#define setMinusInfinity(X) (X = MINUSINFINITY)
#endif /*setMinusInfinity*/

#endif /*HASINFINITY*/

/* global variables related to input processing */

/*
  structure for holding information about where yylex() gets characters
  Component    Meaning
  inputstring  handle to current character vector containing commands being
               parsed, either from expanded out-of-line macro or evaluate()
               argument
  scratchhandle handle for use as transitory scratch space, e.g. for
               a command to be evaluated.
  start        position in inputstring of start of macro whose expanded
               text is in the next inputstring on the stack
               (0 if next is evaluate() string)
  end          next position in inputstring; set from ISTRCHAR
               on push and set to ISTRCHAR on pop
  bracketlev   BRACKETLEV before push; used to reset BRACKETLEV on pop
  parenlev     PARENLEV before push; used to reset PARENLEV on pop
  sqbracketlev SQBRACKETLEV before push; used to reset SQBRACKETLEV on pop  
  wdepth       WDEPTH before push; used to reset WDEPTH on pop
  funcname     name of current function
  inputname    name of expanded out-of-line macro or EVALNAME (see below)

  These are generally only updated on a call to pushInputLevel()
*/

typedef struct 
{
	unsigned char ** inputstring;
    char          ** scratchhandle; /*handle for transitory storage*/
	long             start;  /*1st character of macro call or inline expansion*/
	long             end;    /*1st character after macro call or inline expansion*/
	long             bracketlev; /*BRACKETLEV at start */
	long             parenlev; /*PARENLEV at start */
	long             sqbracketlev; /*SQBRACKETLEV at start*/
	long             wdepth; /*WDEPTH at start*/
	mvSymbolName     funcname; /*name of current function*/
	mvSymbolName     inputname;
} mvMacro, *mvMacroPtr, **mvMacroHandle;

/* INPUTSTACK of length MAXMDEPTH+1 allocated in initialize() */
EXTERN mvMacroPtr      INPUTSTACK;

#define NINPUTSTRINGS  (MAXMDEPTH+1)

EXTERN mvSymbolName    EVALNAME     INIT("@$EVAL$$$$$$");

EXTERN int             INPUTLEVEL   INIT(0);
/*
  Maximum level of possibly non-null elements of INPUTSTACK
*/
EXTERN int             MAXINPUTLEVEL INIT(0);

EXTERN long            MACROSTART; /*start of macro in current line*/

/* will be same as INPUTSTACK[INPUTLEVEL].inputstring */
EXTERN unsigned char **INPUTSTRING INIT((unsigned char **) 0);

/* will be same as INPUTSTACK[INPUTLEVEL].end */
EXTERN long            ISTRCHAR INIT(0);

/* bracket {} level at current parse level; must be restored on pop*/
EXTERN long            BRACKETLEV INIT(0); 
/* parenthesis level () at current parse level; must be restored on pop */
EXTERN long            PARENLEV INIT(0); 
/* square bracket [] level at current parse level; must be restored on pop */
EXTERN long            SQBRACKETLEV INIT(0); 

#define ThisInputstring     (INPUTSTACK[INPUTLEVEL].inputstring)
#define ThisInputstart      (INPUTSTACK[INPUTLEVEL].start)
#define ThisInputend        (INPUTSTACK[INPUTLEVEL].end)
#define ThisBracketlev      (INPUTSTACK[INPUTLEVEL].bracketlev)
#define ThisParenlev        (INPUTSTACK[INPUTLEVEL].parenlev)
#define ThisSqbracketlev    (INPUTSTACK[INPUTLEVEL].sqbracketlev)
#define ThisWdepth          (INPUTSTACK[INPUTLEVEL].wdepth)
#define ThisFuncname        (INPUTSTACK[INPUTLEVEL].funcname)
#define ThisInputname       (INPUTSTACK[INPUTLEVEL].inputname)
#define ThisIstrchar        ThisInputend
#define ThisScratchHandle   (INPUTSTACK[INPUTLEVEL].scratchhandle)

#define NextInputstring     (INPUTSTACK[INPUTLEVEL + 1].inputstring)
#define LastInputstring     (INPUTSTACK[INPUTLEVEL - 1].inputstring)
#define LastIstrchar        (INPUTSTACK[INPUTLEVEL - 1].end)

/*
  970823 Following replaced global FUNCNAME
*/
#define FUNCNAME            ThisFuncname

/*
  SCRATCHHANDLE is to be used for temporary scratch; it should be disposed
  of and set to (char **) as soon possible.  If not zero, cleanitup
  will dispose of it
*/
#define SCRATCHHANDLE        ThisScratchHandle


#define clearSCRATCHHANDLE() {\
								mydisphandle(SCRATCHHANDLE);\
								SCRATCHHANDLE = (char **) 0;\
							 }

#define DEFAULTEXPANDINLINE  1
/* 
   If EXPANDINLINE != 0, the default for macro expansion is in line
   settable by setoptions()
*/
EXTERN int EXPANDINLINE INIT(DEFAULTEXPANDINLINE);
/*
  981118 codes and global related to automatic search for macro when
         name(... is encounted, where name is undefined
*/
enum macroSearchCodes
{
	noMacroSearch = 0,
	verboseMacroSearch,
	quietMacroSearch
};

EXTERN long    DEFAULTMACROSEARCH INIT(verboseMacroSearch);
EXTERN long    MACROSEARCH INIT(verboseMacroSearch);

/* set by yyparse() to value of evaluated string */
EXTERN Symbolhandle   PARSERVALUE INIT((Symbolhandle) 0);

/* 
   Global variables related to syntax and loops

   MDEPTH is the current depth to which macros are nested
   MLEVEL is the number of macros using '$$' since last prompt, used
   as value for $$ in a macro
*/

#ifndef DEFAULTMAXWHILE
#define DEFAULTMAXWHILE   1001
#endif /*DEFAULTMAXWHILE*/

#ifndef MINMAXWHILE
#define MINMAXWHILE      10 /*minimum legal value for option maxwhile*/
#endif /*MINMAXWHILE*/

 /* maximum number of repetitions of while loop */

EXTERN long            MAXWHILE INIT(DEFAULTMAXWHILE);

EXTERN long            MDEPTH INIT(0);
EXTERN long            MLEVEL INIT(MAXMDEPTH + 1);
EXTERN long            ACTIVEMACROS INIT(0); 
typedef struct
{
	long          bracketlev; /*Bracket level at entry to macro*/
	int           quiet;      /*Current status of quiet option*/
	mvSymbolName  name;       /*Macroname*/
} mvMacroInfo;


EXTERN mvMacroInfo         *MACROINFO; /* allocated in initialize ()*/

#define ThisMacroName       MACROINFO[ACTIVEMACROS-1].name
#define ThisMacroBracketlev MACROINFO[ACTIVEMACROS-1].bracketlev
#define ThisMacroQuiet      MACROINFO[ACTIVEMACROS-1].quiet

#define PastMacroName(UP)   MACROINFO[ACTIVEMACROS-1-(UP)].name
#define PastMacroBracketlev MACROINFO[ACTIVEMACROS-1-(UP)].bracketlev
#define PastMacroQuiet      MACROINFO[ACTIVEMACROS-1-(UP)].quiet

#define PushMacroInfo(NAME,LEVEL)\
    (ACTIVEMACROS++, strcpy(ThisMacroName, NAME), \
	 ThisMacroBracketlev = LEVEL, \
	 ThisMacroQuiet = (ACTIVEMACROS > 1) ? \
        MACROINFO[ACTIVEMACROS-2].quiet : QUIET)
#define PopMacroInfo() ((ACTIVEMACROS == 0) ? 0 :\
						(ThisMacroName[0] = '\0',--ACTIVEMACROS))

enum quietCodes
{
	notQuiet      = 0x00,
	windowQuiet   = 0x01,
	spoolQuiet    = 0x02,
	allQuiet      = windowQuiet | spoolQuiet
};

#define SWALLOWOUTPUT \
		(((ACTIVEMACROS > 0) ? ThisMacroQuiet : QUIET) & windowQuiet)

#define SWALLOWSPOOL \
		(((ACTIVEMACROS > 0) ? ThisMacroQuiet : QUIET) == allQuiet)

/*
  depth of nested calls to evaluate(), including calls to mvEval()
  from user functions
*/
EXTERN long            EVALDEPTH INIT(0);
/*
  depth of nested while and for loops
*/
EXTERN long            WDEPTH INIT(0);
/*
  depth of nested if statements
*/
EXTERN long            IDEPTH INIT(0);

#if (0) /* previous declarations */
EXTERN long            WHILESTARTS[INITDIM(MAXWDEPTH)];
EXTERN long            WHILECONDITIONS[INITDIM(MAXWDEPTH)];
EXTERN long            WHILELIMITS[INITDIM(MAXWDEPTH)];
EXTERN long            WHILEBRACKETLEV[INITDIM(MAXWDEPTH)];
EXTERN double        **FORVECTORS[INITDIM(MAXWDEPTH)];
EXTERN long            IFCONDITIONS[INITDIM(MAXIDEPTH)];
EXTERN long            IFBRACKETLEV[INITDIM(MAXIDEPTH)];
#else /*now allocated in initialize() */
EXTERN long           *WHILESTARTS;
EXTERN long           *WHILECONDITIONS;
EXTERN long           *WHILELIMITS;
EXTERN long           *WHILEBRACKETLEV;
EXTERN double       ***FORVECTORS;
EXTERN long           *IFCONDITIONS;
EXTERN long           *IFBRACKETLEV;
#endif /*0*/

/* 970823 replaced following by define; see above */
#if (0)
EXTERN char            FUNCNAME[INITDIM(NAMELENGTH+1)];
#endif /*0*/
EXTERN Symbolhandle    NULLSYMBOL;
EXTERN char           *NULLNAME INIT("NULL");

enum specialSymbolIndices
{
	CLIPBOARDINDEX,
#if defined(wx_motif) || defined(__WXGTK__)
	SELECTIONINDEX,
#endif /*wx_motif*/
	GRAPHWINDOWSINDEX,
	NSPECIALSYMBOLS
};

/* The following two variables should be allocated in initialize */
EXTERN Symbolhandle   *SpecialSymbols;
EXTERN mvSymbolName   *SpecialSymbolNames;

#define CLIPBOARDSYMBOL    SpecialSymbols[CLIPBOARDINDEX]
#define CLIPBOARDNAME      SpecialSymbolNames[CLIPBOARDINDEX]
#define NAMEOFCLIPBOARD    "CLIPBOARD"


#if defined(wx_motif) || defined(__WXGTK__)
/*
  SELECTIONSYMBOL is like CLIPBOARDSYMBOL except it is connected
  with the XA_PRIMARY X-selection instead of the Clipboard
*/
#define SELECTIONSYMBOL    SpecialSymbols[SELECTIONINDEX]
#define SELECTIONNAME      SpecialSymbolNames[SELECTIONINDEX]
#define NAMEOFSELECTION    "SELECTION"
#endif /*wx_motif || WXGTK*/

#define GRAPHWINDOWSSYMBOL SpecialSymbols[GRAPHWINDOWSINDEX]
#define GRAPHWINDOWSNAME   SpecialSymbolNames[GRAPHWINDOWSINDEX]
#define NAMEOFGRAPHWINDOWS "GRAPHWINDOWS"

/* 010516 moved enum argCountCodes to globdefs.h */

#define TEMPPREFIX     '@'        /* 1st character of temporary names */

#define KEYPREFIX1     TEMPPREFIX /* 1st character of keyword names */
#define KEYPREFIX2     TEMPPREFIX /* 2nd character of keyword names */

#define SCRATCHPREFIX1 TEMPPREFIX /* 1st character of scratch names */
#define SCRATCHPREFIX2 '$'        /* 2nd character of scratch names */

#define INVISPREFIX    '_'   /* _.*  @_.*  and @$_.* are invisible names*/

/* standard name for used up keyword, must start with KEYPREFIXs */
EXTERN char             USEDKEYWORD[] INIT("@@@USED");

/* standard names for scratch variables.  Must start with SCRATCHPREFIXs */
EXTERN char             SCRATCH[]       INIT("@$SCRATCH");

EXTERN char             LOGICSCRATCH[]  INIT("@$LOGICAL");

EXTERN char             NUMSCRATCH[]    INIT("@$NUMBER");

EXTERN char             NULLSCRATCH[]   INIT("@$NULL");

EXTERN char             STRINGSCRATCH[] INIT("@$STRING");

EXTERN char             SCALSCRATCH[]   INIT("@$SCALAR");

EXTERN char             VECSCRATCH[]    INIT("@$VECTOR");

EXTERN char             MATSCRATCH[]    INIT("@$MATRIX");

EXTERN char             ARRAYSCRATCH[]  INIT("@$ARRAY");

EXTERN char             STRUCSCRATCH[]  INIT("@$STRUCTURE");

EXTERN char             GARBSCRATCH[]   INIT("@$GARBAGE");

EXTERN char             MACROSCRATCH[]  INIT("@$MACRO");

EXTERN char             LISTSCRATCH[]   INIT("@$LIST");

EXTERN char             GRAPHSCRATCH[]  INIT("@$GRAPH");

#ifdef INVISIBLESYMBOLS
EXTERN char             INVISSCRATCH[]  INIT("@$_SCRATCH");
#endif /*INVISIBLESYMBOLS*/

EXTERN char             LOCKEDSCRATCH[] INIT("@$LOCKED");

EXTERN char             EVALSCRATCH[]   INIT("@$EVALUATE");

EXTERN char             LABELSSCRATCH[] INIT("@$LABELS");
EXTERN char             NOTESSCRATCH[]  INIT("@$NOTES");

/*
  980718
  Moved globals such as as OPTIONS related to save() and restore() to mvsave.h
*/

/*
   960611 made '_' a legal character to start a name but any symbol
          with a name starting with '_' is "invisible" to list() and
          listbrief().  In addition, an unassigned invisible value is not
          printed.
   960612 names starting @_ and @$_ are also invisible.  
*/

#ifndef INVISIBLESYMBOLS  /* previous definition */
#define isnamestart(C) isalpha(C) /* check for legal start to name */
#define invisname(A) (0)
#else /*INVISIBLESYMBOLS*/
#define isnamestart(C)  (isalpha(C) || (C) == INVISPREFIX)
						/* check for legal characters to start name */
#define invisname(A) ((A)[0] == '_' ||\
					  istempname(A) && (A)[1] == INVISPREFIX ||\
					  isscratch(A) && (A)[2] == INVISPREFIX)
#endif /*INVISIBLESYMBOLS*/

#define isnamechar(C)  (isalnum(C) || (C) == '_')
						/* check for legal characters in name */
#define iskeystart(C)  isalpha(C) /* check for legal start to keyword name */
#define iskeyname(A)  ((A)[0] == KEYPREFIX1 && (A)[1] == KEYPREFIX2)
#define istempname(A) ((A)[0] == TEMPPREFIX)
#define isscratch(A)  ((A)[0] == SCRATCHPREFIX1 && (A)[1] == SCRATCHPREFIX2)
#define isevaluated(A) (strcmp((A), EVALSCRATCH) == 0)

/* prefix strings used for error and warning messages */
EXTERN char           ErrorStart[]    INIT("ERROR: ");
EXTERN char           WarningStart[]  INIT("WARNING: ");

#ifndef STDOUT
#define STDOUT (FILE *) 0
#endif /*STDOUT*/

#ifndef STDIN
#if defined(MACINTOSH) || defined(WXWIN) || defined(SKEL_CARAPACE)
#define STDIN (FILE *) 0
#else /*defined(MACINTOSH) || defined(WXWIN)*/
#define STDIN stdin
#endif /*defined(MACINTOSH) || defined(WXWIN)*/
#endif /*STDIN*/

#ifndef EXPANSIONPREFIX
#define EXPANSIONPREFIX   '~'
#endif /*EXPANSIONPREFIX*/

/* standard separators used in paths*/
#if defined(MACINTOSH)
EXTERN char * DIRSEPARATOR INIT(":");
#elif defined(BASE_MICROSOFT) || defined(MSDOS)
EXTERN char * DIRSEPARATOR INIT("\\");
#else
EXTERN char * DIRSEPARATOR INIT("/");
#endif /*MACINTOSH*/

#ifndef UNLOADSEG
#if defined(MACINTOSH) && !defined(powerc)
#include <SegLoad.h>
#define UNLOADSEG(F)    myUnloadSeg((Ptr) (F))
EXTERN         int UNLOADSEGMENTS INIT(0);
#else /*MACINTOSH && !powerc*/
#define UNLOADSEG(F)
#endif /*MACINTOSH && !powerc*/
#endif /*UNLOADSEG*/


/* global SPOOL file */
EXTERN FILE           *SPOOLFILE INIT((FILE *) 0);
EXTERN char          **SPOOLFILENAME INIT((char **) 0); /* handle */

/* global HELP file (kb) */
EXTERN FILE           *HELPFILE  INIT((FILE *) 0);
EXTERN char          **HELPFILENAME INIT((char **) 0); /* handle */

/* Global restore file info */
EXTERN FILE           *RESTOREFILE INIT((FILE *) 0);

/* Global save file info */
EXTERN FILE           *KEEPFILE INIT((FILE *) 0);/*file pointer for save file*/
EXTERN char          **KEEPFILENAME  INIT((char **) 0); /* handle */

/* Default	values for options affecting plotting */
EXTERN long            DEFAULTDUMBPLOT  INIT(0);
/* Following not implemented as of 990523 */
EXTERN long            DEFAULTUSEGRAPHW INIT(1);

/* default values for options affecting save() or restore()*/
EXTERN long            DEFAULTSAVEHISTORY INIT(0);
EXTERN long            DEFAULTRESTOREDEL INIT(1); /*delete:T or F restore() default*/

/* value for option 'quiet' */
#ifndef DEFAULTQUIET
#define DEFAULTQUIET 0
#endif /*DEFAULTQUIET*/
EXTERN long            QUIET INIT(DEFAULTQUIET);

/* value for option 'traceback' affecting error messages in macros */

#ifndef DEFAULTTRACEBACK
#define DEFAULTTRACEBACK 0
#endif /*DEFAULTTRACEBACK*/
EXTERN long            TRACEBACK INIT(DEFAULTTRACEBACK);

/* 
   Default values for keyword excludeM having to do with computing
   quartiles.  When EXCLUDEMEDIAN != 0, the median is excluded
   from the upper and lower halves of odd sized samples when computing
   quartiles
*/

EXTERN long            EXCLUDEMEDIAN INIT(0);

/*
  980718 Moved other stuff having to do with save() and restore() to mvsave.h
*/

/* Stack of input files, updated by batch() (kb) */
EXTERN long            BDEPTH INIT(0);
EXTERN long           *BATCHECHO; /*BATCHECHO[MAXBDEPTH]*/
EXTERN FILE          **INPUTFILE;
EXTERN char         ***INPUTFILENAMES;

/* Flag for whether last input character read was CR*/
EXTERN char           *LASTINPUTWASCR;

/* Descriptor of version, e.g., "MacAnova 3.32 of 11/01/93" */
EXTERN char          **VERSION_ID INIT((char **) 0);

/*
	Encoding of information about the machine/system
	ThisMachine is to be set in main() for each version
*/

enum osCodes
{
	mvMsDos         = 0x0100,
	mvMacintosh     = 0x0200,
	mvUnix          = 0x0400,
	mvVMS           = 0x0800,
	mvWin32s        = mvMsDos        | 0x0001,
	mvWindows95     = mvWin32s       | 0x0002,
	mvWindowsNT     = mvWin32s       | 0x0004,
	mvLinux         = mvMsDos        | 0x0008,
	mvMacintosh68K  = mvMacintosh    | 0x0001,
	mvMacintosh68Kc = mvMacintosh68K | 0x0010,
	mvMacintosh68Kn = mvMacintosh68K | 0x0020,
	mvMacintoshPPC  = mvMacintosh    | 0x0002,
	mvMotif         = mvUnix         | 0x0001,
	mvMotifVMS      = mvVMS          | 0x0001
};

EXTERN unsigned long   ThisMachine INIT(0);

/* resettable by option batchecho */
EXTERN long            DEFAULTBATCHECHO INIT(1);

/* resettable by option warning */
EXTERN long            PRINTWARNINGS INIT(1);

/* 
  020819 following added to implement option 'enableitem'
*/
enum enabledItemsCodes
{
	disablePrintWindow  = 0x0000,
	disablePrintGraph   = 0x0000,
	enablePrintWindow   = 0x0001,
	enablePrintGraph    = 0x0002,
	maxEnabledItems     = 2
};

#ifndef DEFAULTENABLEDITEMS /*could be set in Makefile*/
#if defined(wx_msw)
#define DEFAULTENABLEDITEMS  disablePrintWindow | disablePrintGraph
#elif defined(wx_motif)
#define DEFAULTENABLEDITEMS  disablePrintWindow | disablePrintGraph
#else
#define DEFAULTENABLEDITEMS  enablePrintWindow | enablePrintGraph
#endif
#endif /*DEFAULTENABLEDITEMS*/

EXTERN unsigned long   ENABLEDITEMS INIT(DEFAULTENABLEDITEMS);

/* global labelling defaults; may sometime be settable by setoptions()*/
EXTERN long            DEFAULTUSECOLLABS INIT(0);
EXTERN long            USECOLLABS INIT(0);
EXTERN char            DEFAULTLEFTBRACKET[] INITARRAY("(");
EXTERN char            LEFTBRACKET[] INITARRAY("(");
EXTERN char            RIGHTBRACKET[] INITARRAY(")");
EXTERN char            Lparens[] INITARRAY("#[({</\\");
EXTERN char            Rparens[] INITARRAY("#])}>/\\");

enum BracketIndices
{
	PoundIndex = 0,
	BracketIndex,
	ParenIndex,
	BraceIndex,
	LTIndex,
	SlashIndex,
	BslashIndex
};
/* prefix for labels expandable a print time */
#define NUMERICLABEL ((char) 0x40) /* '@' */

#ifdef CONSOLENAME
EXTERN char           *CONSOLE INIT(CONSOLENAME);
#endif /*CONSOLENAME*/

#ifndef MILLISECONDSPERTICK
#ifdef WXWIN
#ifdef USETICKTIMER
#define MILLISECONDSPERTICK 500
#else /*USETICKTIMER*/
#define MILLISECONDSPERTICK 1000
#endif /*USETICKTIMER*/
#elif defined(MACINTOSH)
#define MILLISECONDSPERTICK  17
#else
#define MILLISECONDSPERTICK 500
#endif
#endif /*MILLISECONDSPERTICK*/

EXTERN long            Ticks INIT(0);
EXTERN long            MilliSecondsPerTick INIT(MILLISECONDSPERTICK);

/* global variables related to error handling */
/*
  MAXERRORS is settable by setoptions() and is a global error limit.
  MAXERRORS1 is the operative limit.  At the stdin level is should be the
  same as MAXERRORS.  For batch files, MAXERRORS1 will be 1, if MAXERRORS == 0,
  or MAXERRORS, otherwise.

  if MAXERRORS1 != 0 NERRORS gets incremented by myprint() for every line
  starting with "ERROR:"  If MAXERRORS == 0, it is reset to zero before
  the prompt when BDEPTH == 0
*/
EXTERN long            NERRORS INIT(0); /* cumulative number of errors */
EXTERN long            MAXERRORS INIT(0);
	/* settable error limit, 0 means none */
EXTERN long            MAXERRORS1 INIT(0);/* operative error limit */
EXTERN long            WARNINGLEVEL INIT(0);/* 1 or 2 suppress WARNINGs or ERRORs*/
EXTERN long            FatalError INIT(0); /* != 0 if Fatal Error occurred */

/* global buffer of length BUFFERLENGTH for building output strings */
EXTERN char           *OUTSTR INIT((char *) 0);

/* 001026 moved declaration of LINE to mvIo.h */

EXTERN char          **TMPHANDLE INIT((char **) 0);/*generic temporary handle*/

/*
  001026 moved globals and definitions concerning input lines to mvIo.h
*/

#ifdef SAVEHISTORY
/* global variable for history lines to keep */

#ifndef DEFAULTHISTORYLENGTH
#define DEFAULTHISTORYLENGTH  100
#endif /*DEFAULTHISTORYLENGTH*/

EXTERN long            HISTORY INIT(DEFAULTHISTORYLENGTH);
#endif /*SAVEHISTORY*/

/* global variables related to algorithms */
EXTERN double          ANGLES2RADIANS INIT(1.0);
	/* either 1 (radians), 2*pi (cycles), or 2*pi/360 (degrees)*/

/* variables related to random number generation */
EXTERN long         RANDS1 INIT(0) ; /* random number generator seed */
EXTERN long         RANDS2 INIT(0) ; /* random number generator seed */

/* origin for subscripts (for possible future use) */
EXTERN long         ORIGIN INIT(1);

/* hidden option for debugging */
EXTERN long         GUBED  INIT(0);

/* global variables related to output formatting */
EXTERN long         NLINES INIT(0);
	/* number of output lines since start of command*/
#ifndef DEFAULTPRINTDEC
#define DEFAULTPRINTDEC  5
#endif /*DEFAULTPRINTDEC*/
#ifndef DEFAULTWRITEDEC
#define DEFAULTWRITEDEC  9
#endif /*DEFAULTWRITEDEC*/

#ifndef MINSCREENHEIGHT
#define MINSCREENHEIGHT  12
#endif /*MINSCREENHEIGHT*/

#ifndef MINSCREENWIDTH
#define MINSCREENWIDTH   30
#endif /*MINSCREENWIDTH*/

#ifndef DEFAULTSCREENHEIGHT
#ifdef MSDOS
#define DEFAULTSCREENHEIGHT 25
#else /*MSDOS*/
#define DEFAULTSCREENHEIGHT 24
#endif /*MSDOS*/
#endif /*DEFAULTSCREENHEIGHT*/

#ifndef DEFAULTSCREENWIDTH
#define DEFAULTSCREENWIDTH 80
#endif /*DEFAULTSCREENWIDTH*/

EXTERN long         SCREENHEIGHT INIT(0); /* number of lines on screen */
EXTERN long    		SCREENWIDTH INIT(DEFAULTSCREENWIDTH);   /* columns on screen */

/*
	globals related to automatic scrolling back after help()
	and possibly other commands.
	SCROLLBACK will have the value option 'scrollback' if and when implemented
	CURRENTSCROLLBACK determines behavior after current command line, which
	may reset it. 
*/
#ifdef SCROLLABLEWINDOW
#ifndef DEFAULTSCROLLBACK
#define DEFAULTSCROLLBACK   0
#endif /*DEFAULTSCROLLBACK*/

EXTERN long          SCROLLBACK INIT(DEFAULTSCROLLBACK);
EXTERN long          CURRENTSCROLLBACK INIT(DEFAULTSCROLLBACK);
#endif /*SCROLLABLEWINDOW*/

/* Format related items */
enum formatCodes
{
	DODEFAULT    = 0x00,  /* use default format */
	DOFIXED      = 0x01,  /* use fixed point format */
	DOFLOAT      = 0x02,  /* use floating point format */
	DOPVALUE     = 0x04,  /* special treatment of P-values */
	NOTRIM       = 0x00,  /* no trimming */
	TRIMRIGHT    = 0x08,  /* trim whitespace on right end */
	TRIMLEFT     = 0x10,  /* trim whitespace on left end */
	CHARASIS     = 0x20   /* put string as is, regardless of MISSINGFMT*/
};


EXTERN double       MAXFIXED; /* maximum size for fixed format number */
EXTERN double       MINFIXED; /* minimum size for fixed format */
EXTERN long         FIELDWIDTH INIT(DEFAULTPRINTDEC+7+1);  /* width of output field */
EXTERN char         FIXEDFMT[20]; /* e.g., " %12.5f" */
EXTERN char         FLOATFMT[20]; /* e.g., " %12.5g" */
EXTERN char         DATAFMT[20]; /* one of FIXEDFMT or FLOATFMT */
EXTERN char         MISSINGFMT[10]; /* for MISSING, e.g., " %12s" */
EXTERN long         BEFOREDEC INIT(DEFAULTPRINTDEC+7);   /* e.g. 12 in 12.5g */
EXTERN long         AFTERDEC INIT(DEFAULTPRINTDEC);
	/* # after dp in format, e.g., 5 in 12.5g */
EXTERN char         FMTTYPE INIT('g');
	/* output format type, e.g., 'g' in 12.5g */
EXTERN long         PRINTFORMAT[]
	INITARRAY(DEFAULTPRINTDEC+7 COMMA DEFAULTPRINTDEC COMMA (long) 'g');
EXTERN long         WRITEFORMAT[]
	INITARRAY(DEFAULTWRITEDEC+7 COMMA DEFAULTWRITEDEC COMMA (long) 'g');

#ifndef DEFAULTMINPVALUE
#define DEFAULTMINPVALUE  1e-8
#endif /*DEFAULTMINPVALUE*/
EXTERN double       MINPVALUE INIT(DEFAULTMINPVALUE);

/*
  VecreadByfields is associated with option 'vecread' withvalues
  "oldstyle" and "newstyle"
*/
#ifndef DEFAULTVECREADBYFIELDS
#define DEFAULTVECREADBYFIELDS 0 /*initial default is 0*/
#endif /*DEFAULTVECREADBYFIELDS*/

EXTERN long         VecreadByfields INIT(DEFAULTVECREADBYFIELDS);

/* Global prompts */
#define MAXPROMPT 21
typedef char   promptType[MAXPROMPT];

EXTERN promptType    *PROMPTS;
EXTERN promptType    *DEFAULTPROMPTS;
#define PROMPT        (char *) PROMPTS[BDEPTH]
#define DEFAULTPROMPT (char *) DEFAULTPROMPTS[BDEPTH]
EXTERN promptType     MOREPROMPT;

/* globals related to environment */
EXTERN long         ISATTY INIT(0); /* is input from keyboard */
EXTERN char        *TERMINAL INIT((char *) 0) ; /* on Unix set to $TERM */
/*
  The following will be initialized by a command line flag as indicating
  MacAnova is being run remotely over the web
*/
EXTERN long         MVWEB INIT(0); /*when non-zero, running on web*/

#ifdef TEK
/*
  The following should depend on the terminal emulator and are
  setable by setoptions()
  They are initialized by setDefaultOptions() in setOptio.c
*/

#define TEKLENGTH 20
/* code to switch to Tektronix mode */
EXTERN char        ToTek[INITDIM(TEKLENGTH)];
/* code to switch back */
EXTERN char        FromTek[INITDIM(TEKLENGTH)];

#endif /*TEK*/

/* constants for masking ISATTY */
#define ITTYIN      0x01
#define ITTYOUT     0x02


/* macro for emergency error messages */
#ifdef MACINTOSH
#define PUTALERT(MSGS) (myAlert(MSGS), myerrorout(MSGS))
#else /*MACINTOSH*/
#define PUTALERT(MSGS) (fprintf(stderr,"%s\n",(MSGS)), myerrorout(MSGS))
#endif /*MACINTOSH*/

#undef	EXTERN
#undef	INIT
#undef	INITDIM
#undef	INITARRAY
#undef  COMMA

#ifdef MAIN__
/* copyright notice */
static char     copyright1[] =
	"MacAnova, (C) Copyright 1994 Gary W. Oehlert and Christopher Bingham";
#ifdef MACINTOSH
#ifndef MPW
/* copyright notice */
static char     copyright2[] = "(C) Copyright 1986 THINK Technologies, Inc.  \n\
Certain portions of the Macintosh version of this software are copyrighted\n\
by THINK Technologies, Inc.";
#endif /*MPW*/
#endif /*MACINTOSH*/
#endif /* MAIN__ */

#endif /*GLOBKBH__*/
