/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.13 or later
*(C)*
*(C)* Copyright (c) 2003 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/


#ifdef SEGMENTED
#if defined(MPW) || defined(MW_CW)
#pragma segment Graphics
#endif /*MPW||MW_CW*/
#endif /*SEGMENTED*/


/*
 *
 *    G N U P L O T  --  graphics.c
 *
 *  Copyright (C) 1986, 1987  Thomas Williams, Colin Kelley
 *
 *  You may use this code as you wish if credit is given and this message
 *  is retained.
 *
 *  Please e-mail any useful additions to vu-vlsi!plot so they may be
 *  included in later releases.
 *
 *  This file should be edited with 4-column tabs!  (:set ts=4 sw=4 in vi)
 */

#include <math.h>
#undef  GRAPHICSC__
#define GRAPHICSC__ /* this should be defined only here */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "plot.h"
#include "dbug.h"
#if defined(SKEL_CARAPACE)
#include "cpcinterface.h"
#endif /*CARAPACE*/

/*
  990804 Removed globals log_x and log_y and references to them
         Modified code for computing equally spaced arithmetic
         tick locations
  030216 changed code for computing tick marks with logy:T
         defined macros moveTo, drawto, setLineType and drawPoint
  030615 Added ifdefs plus code to replace horstring() code for Carapace
*/
#if (0)
long            log_x = 0, log_y = 0;
#endif /*0*/

void myprint(char * /*msg*/); /*used by debugging macros*/

extern long GUBED;

#if (0) /* following is now defined in plot.h */
extern long     term;
#endif /*0*/

/*extern BOOLEAN term_init;*/
extern long     term_init;

extern struct   termentry term_tbl[];

extern long     ThisWindow;

#if (0) /*original definitions*/
#define map_x(x) (long)((x-xmin)*xscale+xoffset)	/* maps floating point x to screen */
#define map_y(y) (long)((y-ymin)*yscale+yoffset)	/* same for y */
#endif /*0*/

/*
  990805  mapXToScreen() and mapYToScreen() are functions in term.c
          Formerly they were macros in plot.h
*/
#define map_x(X) mapXToScreen(X) /*floating point X to screen*/
#define map_y(Y) mapYToScreen(Y) /*floating point Y to screen*/

#define raise   mvRaise /*990705 to avoid declaration conflict with signal.h*/

#define moveTo      (*t->move)
#define drawTo      (*t->vector)
#define setLineType (*t->linetype)
#define drawPoint   (*t->point)

#if defined(SKEL_CARAPACE)
#define CenterBottom      CPC_SG_CENTERBOTTOM
#define CenterTop         CPC_SG_CENTERTOP
#define CenterBaseline    CPC_SG_CENTERBASELINE
#define GraphicTitleFontSize   180
#define GraphicFontSize   120
#define GraphicFontName   CPC_SG_SWISS
#define GraphicFontStyle  CPC_SG_NORMAL
#define GraphicFontWeight CPC_SG_NORMAL
#else /*CARAPACE*/
#define CenterBottom      0
#define CenterTop         0
#define CenterBaseline    0
#define GraphicTitleFontSize   18
#define GraphicFontSize   12
#define GraphicFontName   0
#define GraphicFontStyle  0
#define GraphicFontWeight 0
#endif /*CARAPACE*/

static double   raise(double x, long y)
{
	long            i;
	double          val;

	val = 1.0;
	for (i = 0; i < abs((int) y); i++)
	{
		val *= x;
	}
	if (y < 0)
	{
		return (1.0 / val);
	}
	return (val);
} /*raise()*/

#define MAXNEAT 30

/*
  For arithmetic scale compute "neat" tick marks, x0, x0 + step,
  x0 + 2*step, ...
  If range = R*10^m, where 1 <= R < 10, then step is
     .2*10^m    1 <= R <= 2
	 .5*10^m    2 < R <= 5
	    10^m    5 < R < 10
  and x0 = step * floor(tmin / step)
  The value returned is min(i, x0 + i*step > tmax

  For log scale, the value is max(10^floor(log10(range)), 1)
  Currently, log scale is not used and it is not clear that this is
  appropriate

  990805 Changed argument list
         Deleted fossil code
  030216 split up make_ticks() into linearScaleTicks() and logScaleTicks()
        modified logScaleTicks() to work better with tmax/tmin > 1e10
*/
#define LARGERATIO   1e10

static int linearScaleTicks(double tmin, double tmax, double ticks [])
{
	double          xnorm, l10, step;
	double          basetick, lasttick, thistick;
	int             nticks;

	l10 = log10(fabs(tmax - tmin));
	xnorm = pow(10.0, l10 -
				(double) ((l10 >= 0.0) ? (long) l10 : (long) floor(l10)));

	if (xnorm <= 2)
	{
		step = 0.2;
	}
	else if (xnorm <= 5)
	{
		step = 0.5;
	}
	else
	{
		step = 1.0;
	}
	step *= raise(10.0, (long) floor(l10));

	basetick = step * floor(tmin / step);
	thistick = basetick;
	lasttick = basetick - step;

	/*
	  Compute ticks
	  The loop termination test is supposed to catch case when step
	  is too small relative to the thistick, so that thistick + step
	  == thistick
	*/

	for (nticks = 0;
		 nticks < MAXNEAT && thistick <= tmax && thistick != lasttick;
		 thistick += step)
	{
		/*
		  Make sure a zero tick really is zero
		*/
		if (fabs(thistick/step) < 1e-10)
		{
			thistick = 0.0;
		}

		if (thistick >= tmin)
		{
			ticks[nticks++] = thistick;
		}

		lasttick = thistick;
	} /*for(...)*/
	return (nticks);
} /*linearScaleTicks()*/

static int logScaleTicks(double tmin, double tmax, double ticks [])
{
	double         baseticks[10], power10;
	double         xr = tmax/tmin, factor;
	double         jpow;
	int            nbase = 0, i, k;

	baseticks[nbase++] = 1;

	if (xr <= 15.0)
	{
		baseticks[nbase++] = 1.5;
		baseticks[nbase++] = 2;
		baseticks[nbase++] = 3;
		baseticks[nbase++] = 4;
		baseticks[nbase++] = 5;
		baseticks[nbase++] = 6;
		baseticks[nbase++] = 8;
	}
	else if (xr <= 20.0)
	{
		baseticks[nbase++] = 2;
		baseticks[nbase++] = 3;
		baseticks[nbase++] = 4;
		baseticks[nbase++] = 5;
		baseticks[nbase++] = 7;
	}
	else if (xr <= 50.0)
	{
		baseticks[nbase++] = 2;
		baseticks[nbase++] = 3;
		baseticks[nbase++] = 5;
		baseticks[nbase++] = 7;
	}
	else if (xr <= 1000.0)
	{
		baseticks[nbase++] = 2;
		baseticks[nbase++] = 5;
	}
	else if (xr <= 30000.0)
	{
		baseticks[nbase++] = 3;
	}

	jpow = (xr <= LARGERATIO) ? 1.0 : ceil(log10(xr)/10.0);
	factor = pow(10.0,jpow);
	baseticks[nbase++] = factor;
	
	power10 = pow(10.0, jpow*floor(log10(tmin)/jpow));
	for (i = nbase-1; i >= 0; i--)
	{
		if (baseticks[i] < tmin/power10)
		{
			i++;
			break;
		}
	} /*for (i = nbase-1; i >= 0; i--)*/
	
	for (k = 0; k < MAXNEAT; k++)
	{
		double       place = baseticks[i++]*power10;

		if (place > tmax)
		{
			break;
		}
		ticks[k] = place;
		if (i >= nbase)
		{
			i = 1;
			power10 *= factor;
		}
#if (0)
		PRINT("ticks[%d] = %g, tmax = %g",k,ticks[k],tmax,0);
#endif
	} /*for (k = 0; k < MAXNEAT; k++)*/
	return (k);
} /*logScaleTicks()*/

static int make_tics(double tmin, double tmax, short logscale,
					  double ticks [])
{
	WHERE("make_tics");

	return ((!logscale || tmax/tmin <= 10.0) ?
			linearScaleTicks(tmin, tmax, ticks) :
			logScaleTicks(tmin,tmax,ticks));
} /*make_tics()*/

static char           *idx(char *a, char b)
{
	do
	{
		if (*a == b)
		{
			return (a);
		}
	} while (*a++);
	return (0);
} /*idx()*/

/* 990503 no longer uses  f format when abs(num) <= 1e-10 */
static void num2str(double num, char str[])
{
	static char     temp[80];
	double          d = fabs(num);
	char           *a, *b;

	if (d > 9999.0 || d < 0.001 && d != 0.0)
	{
		(void) sprintf(temp, "%-.3e", num);
	}
	else
	{
		(void) sprintf(temp, "%-.3f", num);
	}
	if ((b = idx(temp, 'e')))
	{
		a = b;
		while (*(--a) == '0')	/* trailing zeros */
		{
			;
		}
		if (*a == '.')
		{
			a--;
		}
		(void) strncpy(str, temp, (size_t) (a - temp) + 1);
		str[(long) (a - temp) + 1] = '\0';
		a = b + 1;	/* point to 1 after 'e' */
		(void) strcat(str, "e");
		if (*a == '-')
		{
			(void) strcat(str, "-");
		}
		a++;		/* advance a past '+' or '-' */
		while (*a == '0' && *(a + 1) != '\0')	/* leading zeroes */
		{
			a++;
		}
		(void) strcat(str, a);	/* copy rest of string */
	} /*if ((b = idx(temp, 'e')))*/
	else
	{
		(void) strcpy(str, temp);
		a = str + strlen(str) - 1;
		while (*a == '0')
		{
			*a = 0;
			a--;
		}
		if (*a == '.')
		{
			*a = 0;
		}
	} /*if ((b = idx(temp, 'e'))){}else{}*/
	if (strcmp(str, "-0") == 0)
	{
		str[0] = '0';
		str[1] = '\0';
	}
} /*num2str()*/

#define trimx(X) ((Tmp__ = ((X) >= Ixmin) ? (X) : Ixmin) , \
				  ((Tmp__ <= Ixmax) ? Tmp__ : Ixmax))
#define trimy(Y) ((Tmp__ = ((Y) >= Iymin) ? (Y) : Iymin) , \
				  ((Tmp__ <= Iymax) ? Tmp__ : Iymax))

static long Ixmin, Ixmax, Iymin, Iymax, Tmp__;

static void findEdge(long xin,long yin,long *xout,long *yout)
{
	long      xEdge, yEdge, dx, dy;
	double    slope;

	dx = *xout - xin;
	dy = *yout - yin;
	xEdge = trimx(*xout);
	yEdge = trimy(*yout);
	if(dx != 0 && dy != 0)
	{
		slope = (double) dy / (double) dx;
		xEdge = (dx > 0) ? Ixmax : Ixmin;
		yEdge = yin + slope * (xEdge - xin);
		if(trimy(yEdge) != yEdge)
		{
			yEdge = (dy > 0) ? Iymax : Iymin;
			xEdge = xin + (yEdge - yin)/slope;
		}
	}
	*xout = xEdge;
	*yout = yEdge;
} /*findEdge()*/

/* 
  findEdges() is called to find the visible portion if any of the line
  connecting (*x1,*y1) and (*x2,*y2).  If the line does not intersect the
  frame, findEdges() returns 0 without modifying its arguments.  Otherwise it
  returns 1 and the points are modified so as to lie on the boundary of the
  frame.
 */

static long findEdges(long *x1,long *y1,long *x2,long *y2)
{
	long      xEdge1, xEdge2, yEdge1, yEdge2, dx, dy;
	long      trialx, trialy;
	double    slope;

	xEdge1 = trimx(*x1);
	yEdge1 = trimy(*y1);
	xEdge2 = trimx(*x2);
	yEdge2 = trimy(*y2);

	if((dx = *x2 - *x1) == 0)
	{  /* vertical line */
		if(*x1 != xEdge1 || yEdge1 == yEdge2)
		{
			return (0);
		}
		*y1 = yEdge1;
		*y2 = yEdge2;
	}
	else if((dy = *y2 - *y1) == 0)
	{ /* horizontal line */
		if(*y1 != yEdge1 || xEdge1 == xEdge2)
		{
			return (0);
		}
		*x1 = xEdge1;
		*x2 = xEdge2;
	}
	else
	{ /* line not horizontal or vertical */
		if(xEdge1 == xEdge2 || yEdge1 == yEdge2)
		{ /* both points on same side of frame */
			return (0);
		}
		slope = (double) dy / (double) dx;
		trialx = xEdge1;
		trialy = *y1 + slope * (trialx - *x1);
		if(trialy != trimy(trialy))
		{
			trialy = yEdge1;
			trialx = *x1 + (trialy - *y1) / slope;
			if(trialx != trimx(trialx))
			{
				return (0);
			}
		}
		
		*x1 = trialx;
		*y1 = trialy;
		findEdge(trialx,trialy,x2,y2);
	}
	return (1);
} /*findEdges()*/

enum tickCodes
{
	XTICK,
	YTICK,
	TICKLABELSIZE  = 80
};

/*
  Draw tick marks
*/
static int puttick(int which, double ticplace, int ticklength,
				   char * ticklabel, short frame)
{
	struct termentry    *t = &term_tbl[term];
	int                  offset = 1;
	char                 label[TICKLABELSIZE+1];
	
	if (ticklabel == (char *) 0)
	{
		num2str(ticplace, label);
	}
	else
	{
		strncpy(label, ticklabel, TICKLABELSIZE);
		label[TICKLABELSIZE] = '\0';
	}
	
	if (which == YTICK)
	{
		if (ticklength != 0)
		{
			if (ticklength != GRIDLINES)
			{				
				ticklength = (ticklength * (long) t->h_tic)/TICKUNIT;
				if (frame & TICKSLEFTSIDE)
				{
					moveTo(Ixmin, map_y(ticplace));
					drawTo(Ixmin + ticklength, map_y(ticplace));
				}
				if (frame & TICKSRIGHTSIDE)
				{
					moveTo(Ixmax, map_y(ticplace));
					drawTo(Ixmax - ticklength, map_y(ticplace));
				}
			} /*if (ticklength != GRIDLINES)*/
			else
			{
				ticklength = 0;
				if (frame & (TICKSLEFTSIDE | TICKSRIGHTSIDE))
				{
					moveTo(Ixmin, map_y(ticplace));
					drawTo(Ixmax, map_y(ticplace));
				}
			}
		} /*if (ticklength != 0)*/
		offset = (ticklength >= 0) ? 1 : -ticklength + 1;
		if(t->h_char > 1)
		{
			if(t->drawtext)
			{
				t->drawtext(Ixmin - offset - 10, map_y(ticplace),label,
					CenterBottom,90);
			}
			else
			{
				horstring(Ixmin - (strlen(label)*(long) t->h_char) / 2 - offset,
						  map_y(ticplace), label, CENTRE);
			}
		}
		else
		{ /*dumb plot*/
			horstring(Ixmin - (strlen(label)*(long) t->h_char - 1) / 2 - offset,
					  map_y(ticplace), label, CENTRE);
		}
		offset--;
	} /*if (which == YTICK)*/
	else
	{
		if (ticklength != 0)
		{
			if (ticklength != GRIDLINES)
			{
				ticklength = (ticklength * (long) t->v_tic)/TICKUNIT;
				if (frame & TICKSBOTTOM)
				{
					moveTo(map_x(ticplace), Iymin);
					drawTo(map_x(ticplace), Iymin + ticklength);
				}
				if (frame & TICKSTOP)
				{
					moveTo(map_x(ticplace), Iymax);
					drawTo(map_x(ticplace), Iymax - ticklength);
				}
			} /*if (ticklength != GRIDLINES)*/
			else
			{
				if (frame & (TICKSBOTTOM | TICKSTOP))
				{
					moveTo(map_x(ticplace), Iymin);
					drawTo(map_x(ticplace), Iymax);
				}
				ticklength = 0;
			}
		} /*if (ticklength != 0)*/
		offset = (ticklength >= 0) ? 0 : -ticklength;
		
		if(t->v_char > 1)
		{
			if(t->drawtext)
			{
				t->drawtext(map_x(ticplace), (Iymin - offset) - 20, label,
					CenterTop,0);
			}
			else
			{
				horstring(map_x(ticplace),
						  Iymin - t->v_char/2 - offset, label, CENTRE);
			}
		}
		else
		{
			horstring(map_x(ticplace), (long) 1, label, CENTRE);
		}
	} /*if (which == YTICK){}else{}*/
	return (offset);
} /*puttick()*/

/*
	plots        linked list of plots
	pcount       count of plots in linked list
	strings      linked list of strings
	scount       count of strings in linked list
	xmin, xmax, ymin, ymax
	             actual plotting limits to use, may exclude data
    xticks, yticks
                 if non-null, values for ticks. 
	nxticks, nyticks
	             n{x,y}ticks == 0 means no {x,y} ticks
                 n{x,y}ticks > 0 means labelled ticks
				 When {x,y}ticks non null, n{x,y}ticks is number of
                 ticks
	xticklength, yticklength
                 Lengths of drawn ticks will be
				 ([xy]ticklength/TICKUNIT)*[hv]_tic with positive lengths
				 extending inside frame and negative lengths outside.
                 [xy]ticklength == GRIDLINES means draw entire grid line.
	logx, logy   determine whether variable should be plotted using log
                 scale (still ignored as of 960620)
    xaxis, yaxis determine whether axes are drawn if in the plot
    xlab, ylab   pointers to axis labels (used to be in struct curve_points)
    title        pointer to plot title

	960617 Added arguments {x,y}ticks, n{x,y}ticks {x,y}ticklength
	to allow greater control over tick drawing

    990503 Added arguments frame, {x,y}ticklabs
    990506 Changed reference to GraphMap to reflect its new status as
           a pointer
	990805 Changed code for drawing ticks
*/
long do_plot(curve_points STAR_ plots, long pcount,
			 plot_string STAR_ strings, long scount,
			 double xmin, double xmax, double ymin, double ymax,
			 double * xticks, double * yticks, short nxticks, short nyticks,
			 char * xticklabs, char * yticklabs,
			 short xticklength, short yticklength,
			 short logx, short logy, short xaxis, short yaxis, short frame,
			 char * xlab, char * ylab, char * title)
{
	long                 i, n, x, y, xt, yt;
	long                 xlast, ylast;
	struct termentry *t = &term_tbl[term];
	/*BOOLEAN prev_undef;*/
	long                 prev_undef;
	long                 curve, string, xaxis_y, yaxis_x;
	int                  style;
#ifdef USEHANDLES
	curve_points        **this_plot;
	plot_string         **this_string;
#else /*USEHANDLES*/
	curve_points *this_plot;
	plot_string         *this_string;
#endif /*USEHANDLES*/
	double               lxmin, lxmax, lymin, lymax;
	double               xscale, yscale, xoffset, yoffset;
	double               xcenter, ycenter;
	double               ticks[MAXNEAT];
	int                  xlaboffset = 0, ylaboffset = 0;
	unsigned long        c;
	WHERE("do_plot");

/*
	for MAC & conceivably for other terminals this initializes t->xmax,
	t->xmin, etc.  Hence it must precede their use.
*/
#if !defined(SKEL_CARAPACE)
	if((*t->init) () > 0)
	{
		return (1);
	}
	
	if ((*t->graphics) () > 0)
	{
		return (1);
	}
#endif /*CARAPACE*/
	lxmin = (logx) ? log10(xmin) : xmin;
	lxmax = (logx) ? log10(xmax) : xmax;
	lymin = (logy) ? log10(ymin) : ymin;
	lymax = (logy) ? log10(ymax) : ymax;

	GraphMap->logx = logx;
	GraphMap->logy = logy;
	GraphMap->xmin = lxmin;
	GraphMap->ymin = lymin;
	GraphMap->screenxmax = t->xmax;
	GraphMap->screenymax = t->ymax;

	if(t->h_char > 1)
	{
#ifdef SKEL_CARAPACE
		GraphMap->xoffset = xoffset = 3*t->v_char;
		GraphMap->xscale = xscale = (t->xmax - 2 - xoffset) / (lxmax - lxmin);
#else /*SKEL_CARAPACE*/
		GraphMap->xoffset = xoffset = (t->xmax - 2) * 0.15;		
		GraphMap->xscale = xscale = (t->xmax - 2) / (lxmax - lxmin) * 0.85;
#endif /*SKEL_CARAPACE*/
		Ixmin = map_x(xmin);
		Ixmax = map_x(xmax);
	}
	else
	{
		GraphMap->xoffset = xoffset = 11;
		GraphMap->xscale = xscale = (t->xmax - 1 - xoffset)/(lxmax - lxmin);
		Ixmin = xoffset;
		Ixmax = t->xmax - 1;
	}
	if(t->v_char > 1)
	{
#ifdef SKEL_CARAPACE
		GraphMap->yoffset = yoffset = 3*t->v_char;
		  (t->ymax - 2*t->v_char - 2) * 0.1;
		GraphMap->yscale = yscale = (t->ymax -2 -5*t->v_char)/(lymax-lymin);
#else /*SKEL_CARAPACE*/
		GraphMap->yoffset = yoffset =
		  (t->ymax - 2*t->v_char - 2) * 0.1;
		GraphMap->yscale = yscale =
		  (t->ymax - 2*t->v_char - 2) / (lymax - lymin) * 0.9;
#endif /*SKEL_CARAPACE*/
		Iymin = map_y(ymin);
		Iymax = map_y(ymax);
	}
	else
	{
		GraphMap->yoffset = yoffset = 2;
		GraphMap->yscale = yscale = (t->ymax - 2 - yoffset)/(lymax - lymin);
		Iymin = yoffset;
		Iymax = t->ymax - 2;
	}

#if defined(SKEL_CARAPACE)
	if((*t->init) () > 0)
	{
		return (1);
	}
	
	if ((*t->graphics) () > 0)
	{
		return (1);
	}
#endif /*CARAPACE*/
	setLineType((long) BORDERLINETYPE);	/* border linetype */

	if (frame)
	{
		void     (*moveOrDraw)(unsigned long, unsigned long);
		
		/* draw plot border */
		moveTo(Ixmax, Iymin);
		moveOrDraw = (frame & FRAMEBOTTOM) ? drawTo : moveTo;
		moveOrDraw(Ixmin, Iymin);
		moveOrDraw = (frame & FRAMELEFTSIDE) ? drawTo : moveTo;
		moveOrDraw(Ixmin, Iymax);
		moveOrDraw = (frame & FRAMETOP) ? drawTo : moveTo;
		moveOrDraw(Ixmax, Iymax);
		moveOrDraw = (frame & FRAMERIGHTSIDE) ? drawTo : moveTo;
		moveOrDraw(Ixmax, Iymin);
	}
	
	if (nyticks != 0)
	{
		nyticks = labs(nyticks);
		if (yticks == (double *) 0)
		{
			nyticks = make_tics(ymin, ymax, logy, ticks);
			yticks = ticks;
			yticklabs = (char *) 0;
		} /*if (yticks == (double *) 0)*/

		/* Draw and label y-axis ticks*/
		for(i = 0; i < nyticks;i++)
		{
			if (yticks[i] >= ymin && yticks[i] <= ymax)
			{
				ylaboffset = puttick(YTICK, yticks[i], yticklength,
									 yticklabs, frame);
			}
			if (yticklabs != (char *) 0)
			{
				yticklabs += strlen(yticklabs) + 1;
			}
		} /*for(i = 0; i < nyticks;i++)*/
	} /*if (nyticks != 0)*/

	if (nxticks != 0)
	{
		if (xticks == (double *) 0)
		{
			nxticks = make_tics(xmin, xmax, logx, ticks);
			xticks = ticks;
			xticklabs = (char *) 0;
		} /*if (xticks == (double *) 0)*/
		/* Draw and label x-axis ticks*/
		for (i = 0; i < nxticks; i++)
		{
			if (xticks[i] >= xmin && xticks[i] <= xmax)
			{
				xlaboffset = puttick(XTICK, xticks[i], xticklength,
									 xticklabs, frame);
			}
			if (xticklabs != (char *) 0)
			{
				xticklabs += strlen(xticklabs) + 1;
			}
		} /*for (i = 0; i < nxticks; i++)*/
	} /*if (nxticks != 0)*/

	/* DRAW AXES */
	xaxis_y = (logy) ? Iymin : map_y(0.0);
	yaxis_x = (logx) ? Ixmin : map_x(0.0);
	if (xaxis_y <= Iymin)
	{
		xaxis_y = Iymin;	/* save for impulse plotting */
	}
	else if (xaxis_y >= Iymax)
	{
		xaxis_y = Iymax;
	}
	else if (xaxis)
	{
		setLineType((long) XAXISLINETYPE);	/* x-axis line type */
		moveTo(Ixmin, xaxis_y);
		drawTo(Ixmax, xaxis_y);
	}

	if (yaxis && !logx && yaxis_x > Ixmin && yaxis_x < Ixmax)
	{
		setLineType((long) YAXISLINETYPE);	/* y-axis line type */
		moveTo(yaxis_x, Iymin);
		drawTo(yaxis_x, Iymax);
	}

	this_plot = plots;
	/* DRAW TITLE (kb) */

	if (*title || *xlab)
	{
		xcenter = (!logx) ?
		  (xmax + xmin)/2.0 : pow(10.0, (lxmax + lxmin)/2.0);
	}
	
	if (*ylab)
	{
		double       adjust;
		
		ycenter = (!logy) ? ymax + ymin : lymax + lymin;

		/* adjustment to try to ensure consistant rounding */
		adjust = (t->h_char > 1 || ycenter != floor(ycenter) ||
				  ycenter/2.0 == floor(ycenter/2.0)) ? 0.0 : 1e-6*yscale;

		ycenter = ycenter/2.0 + adjust;

		if (logy)
		{
			ycenter = pow(10.0, ycenter);
		}
	} /*if (*ylab)*/
		
	if (*title != '\0')
	{
		int    laboffset = (!(frame & FRAMETOP)) ? 0 : xlaboffset;
		
		if(t->drawtext)
		{
			t->savestate();
			t->setfont(GraphicTitleFontSize,GraphicFontName,
					   GraphicFontStyle, GraphicFontWeight);
			t->drawtext(map_x(xcenter),(Iymax+laboffset),title,CenterBottom,0);
			t->restorestate();
			t->setfont(GraphicFontSize,GraphicFontName,
					   GraphicFontStyle, GraphicFontWeight);
		}
		else
		{
			horstring(map_x(xcenter),Iymax + t->v_char + laboffset,
					  title, CENTRE);
		}
	}

	if (*xlab)
	{
		if(t->v_char > 1)
		{
			int    laboffset = (!(frame & FRAMEBOTTOM)) ? 0 : xlaboffset;

			if(t->drawtext)
			{
				t->drawtext(map_x(xcenter),(Iymin-1.5*t->v_char),xlab,CenterTop,0);
			}
			else
			{
				horstring(map_x(xcenter),
					  Iymin - 3*t->v_char/2 - xlaboffset, xlab, CENTRE);
			}
		}
		else
		{
			horstring(map_x(xcenter), (long) 0, xlab, CENTRE);	
		}
	} /*if (*xlab)*/
	

	if (*ylab)
	{
		if(t->h_char > 1)
		{
			int    laboffset = (!(frame & FRAMELEFTSIDE)) ? 0 : ylaboffset;

			if(t->drawtext)
			{
				t->drawtext((long)(1.5*t->v_char),
							map_y(ycenter),ylab,CenterBaseline,90);
			}
			else
			{
				vertstring((long) (0.2 * xoffset) - laboffset,
						   map_y(ycenter), ylab);
			}
		}
		else
		{		
			vertstring(1, map_y(ycenter), ylab);
		}
	} /*if (*ylab)*/
	

	/* DRAW CURVES */
	for (curve = 0; curve < pcount; this_plot = NEXTGRAPH(this_plot), curve++)
	{
		n = PCOUNT(this_plot);
		style = PLOTSTYLE(this_plot);
		/* draw lines first */
		if(style & LINES)
		{
			setLineType(LINETYPE(this_plot));

			prev_undef = 1;
			for (i = 0; i < n; i++)
			{
				double        dx = XCOORD(this_plot,i);
				double        dy = YCOORD(this_plot,i);

				if (!POINTUNDEF(this_plot,i) &&
					(!logx || dx > 0.0) && (!logy || dy > 0.0))
				{
					x = map_x(dx);
					y = map_y(dy);

					xt = trimx(x);
					yt = trimy(y);
					if(x == xt && y == yt)
					{ /* new point inside rectangle */
						if (prev_undef == 1)
						{
							moveTo(x,y);
						}
						else
						{
							if(prev_undef == 2)
							{	/* last point outside rectangle */
								findEdge(x,y,&xlast, &ylast);
							}
							moveTo(xlast,ylast);
							drawTo(x,y);
						}
						prev_undef = 0;
					} /* if(x == xt && y == yt) */
					else
					{ /* current point outside rectangle */
						xt = x;
						yt = y;
						if(prev_undef == 0)
						{ /* last point in rectangle */
							findEdge(xlast,ylast,&xt,&yt);
							drawTo(xt,yt);
						}
						else if(prev_undef == 2 &&
								findEdges(&xlast,&ylast,&xt, &yt))
						{ /* last point outside && edge/line intersection */
							moveTo(xlast,ylast);
							drawTo(xt,yt);
						}
						prev_undef = 2;
					}  /* if(x == xt && y == yt){}else{} */
					xlast = x;
					ylast = y;
				} /* if (!POINTUNDEF(this_plot,i)) */
				else
				{
					prev_undef = 1;
				}
			} /*for (i = 0; i < n; i++)*/
		} /*if(style & LINES)*/
		
		if(style & (POINTS | IMPULSES | CHARS))
		{
			for(i = 0; i < n; i++)
			{
				double        dx = XCOORD(this_plot,i);
				double        dy = YCOORD(this_plot,i);

				if (!POINTUNDEF(this_plot,i) &&
					(!logx || dx > 0.0) && (!logy || dy > 0.0))
				{
					x = map_x(dx);
					y = map_y(dy);
					xt = trimx(x);
					yt = trimy(y);

					if(style & IMPULSES && x == xt)
					{
						/* x in frame */
						setLineType(LINETYPE(this_plot));
						moveTo(x, xaxis_y);
						drawTo(x, yt);
					}

					if((style & (POINTS | CHARS)) && x == xt && y == yt)
					{ /* new point inside rectangle */
						moveTo(x,y);
						if(style & CHARS)
						{
							if((c = POINTSTRING(this_plot,i)[0]) != '\0' &&
							   c < ' ')
							{	/* drawn character */
								setLineType((long) POINTLINETYPE);
								drawPoint(x, y, c-1);
							}
							else
							{	/* printable character(s) */
								horstring(x, y, POINTSTRING(this_plot,i),
										  CENTRE);
							}
						} /*if(style & CHARS)*/

						if(style & POINTS)
						{
							setLineType((long) POINTLINETYPE);
							drawPoint(x, y, POINTTYPE(this_plot));
						}
					} /*if((style & (POINTS | CHARS)) && x == xt && y == yt)*/
				} /*if (!POINTUNDEF(this_plot,i))*/
			} /*for(i = 0; i < n; i++)*/
		} /*if(style & (POINTS | IMPULSES | CHARS))*/
	}/*for(curve=0;curve<pcount;this_plot=NEXTGRAPH(this_plot),curve++)*/

	/* DRAW STRINGS */
	this_string = strings;
	for (string = 0; string < scount; string++)
	{ /* Note: (x,y) may be outside frame */
		x = map_x(STRINGX(this_string));
		y = map_y(STRINGY(this_string));
#ifdef USEHANDLES
		horstring(x, y, *PLOTSTRING(this_string), (enum JUSTIFY) JUSTIFYSTR(this_string));
#else /*USEHANDLES*/
		horstring(x, y, PLOTSTRING(this_string), (enum JUSTIFY) JUSTIFYSTR(this_string));
#endif /*USEHANDLES*/
		this_string = NEXTSTRING(this_string);
	} /*for (string = 0; string < scount; string++)*/
	
	setLineType((long) BORDERLINETYPE);
	(*t->text) ();

	return (0);
} /*do_plot()*/

