/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 5.05 or later
*(C)*
*(C)* Copyright (c) 2005 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/



#ifdef SEGMENTED
#if defined(MPW) || defined(MW_CW)
#pragma segment Help
#endif /*MPW||MW_CW*/
#endif /*SEGMENTED*/


/*
  Functions related to help()
*/
#define HELP__  /*tells mvhelp.h to declare certain statics */

#include "globals.h"
#include "mvhelp.h" /*briefly named helpkb.h*/

#include "mvIface.h"

#ifdef MPW
#include <StdLib.h>
#endif /*MPW*/

#include "mvIo.h"
/*
  980722 moved a lot of stuff into header helpkb.h
*/

#if (0) /* the following now in helpkb.h*/
#define MARKER    '='
#define KEYMARKER '?'
#define NMARKERS 4   /* help topics start with NMARKERS MARKER's */
/* Both of the following strings should be NMARKERS long */
#define MARKERSTRING "===="
#define USAGESTRING  "%%%%"
#define KEYSTRING    "????"
#define STARTKEYS    '#'
#define SEPKEYS      ','
#define PUTUSAGE      1
#define PUTHELP       0
#define EOFSTRINGLENGTH 7
#define EOFSTRING    "_E_O_F_"
#define EOFSTRINGSTART '_'

#define KEYQUERY      '?'
#define SUBQUERY      '?'

#define HELPROOM      2000 /* initial space for list of topics */
#define KEYROOM        400  /* initial space for list of keys */
#define NTOPICS        250 /* initial number of topics */
#define KEYLENGTH       50
#define TOPICNAMELENGTH 20
#define TOPICCHUNK      10
#define HELPCHUNK (7*TOPICCHUNK)
#define MAXKEYS         32 /* maximum number of keys allowed per file */

#define CREATION    870000  /*MacAnova started in 1987*/
#define CENTURY    1000000  /*100 years*/
#define DATEDIGITS       8  /*strlen("19971231")*/


static short           NHelpTopics = -1; /* < 0 means initHelp not called */
static char          **Topics = (char **) 0;
static short         **TopicOffsets = (short **) 0; /* offsets in Topics */
static unsigned long **KeyTable = (unsigned long **) 0; /*Help key info */
static char          **KeyList = (char **) 0; /* list of help keys */
static short           KeyOffsets[MAXKEYS], NKeys;
#ifdef USEFSEEK
static long          **FileOffsets = (long **) 0; /* positions in file */
static long            LineOffset = 0;
#endif /*USEFSEEK*/

static char          **Buffer = (char **) 0; /* line buffer */
static short           CurrentTopic = 0;
static short           BufferFull = 0; /* != 0 means a line is in buffer */
static char          **HelpFileName = (char **) 0;
static FILE           *HelpFile = (FILE *) 0;
static char          **AltHelpFileName = (char **) 0;
static FILE           *AltHelpFile = (FILE *) 0;
static short           HelpEofHit = 0;
static char            LastHelpWasCr = 0; /*Flag for last Character == CR*/
static int             Damaged;

static long            LatestNews = -1; /* yyyymmdd date of latest news item*/
static long            OldestNews = -1; /* yyyymmdd date of oldest news item*/
static char            NEWS[] = "news";
#endif /*0*/

#define OrigHelpFile HELPFILE
#define OrigHelpFileName HELPFILENAME

enum helpFileCodes
{
	INEWALTFILE = 1,
	IORIGFILE,
	IALTFILE
};

/*
   950919 added check that HelpFile is non-null

   950928 Added new keyword alt.  alt:T switches to most recent file
          set by file:"filename".  Added globals AltHelpFile and
		  AltHelpFileName

   970919 Added checks for too long key names to avoid crash.  A count of
          bad lines is kept in global Damaged and printed if non-zero

   971118 Help file name is part of warning message when file can't be opened

   971222 News dates should now be o.k. until 2050.  All dates in file
          < 500000 are interpreted as 20000000 + date; all dates >=
          500000 and < 1000000 are interpreted as 19000000 + date

   980521 Added arguments to scanPat() to indicate it should check for
          the pattern being a possible name and for the length of pattern
		  Moved definition of EXACTMATCH, etc. to globdefs.h

   980827 Added 'next' to ControlNames (list of syntax elements)

   981214 Modified usage output for trig functions to reflect new keyword
          phrases degrees:T, radians:T and cycles:T
   981215 usage() now uses usage info from file for transformations rather
          than generating it.  help() still prints topic transformations
          for most transformations
   990215 Changed most putOUTSTR() to putErrorOUTSTR()
          Changed myerrorout() to putOutMsg() and putOutErrorMsg()
   990325 On Macintosh, help file can have type 'ttro'
   990528 Case is ignored on search for help topics.  This is done by
          adding new bit IGNORECASE to matchType.
   990712 'return' recognized as syntax element
   000824 Fixed bug that could result in a help file not being closed
   000929 Fixed minor bug that showed up when an topic had no keys assigned
   001012 Fixed bug; NKeys was not set to zero on reading a new help
          file
   001107 Added keyword phrase 'usage:T';  help() with 'usage:T' is like
          usage()
   001207 Did some reorganizing, moving code from help() to new functions
          putKeys() and getDates(); added capability of asking for
          help on subtopics by, say, help(options:"tekset")
   010113 Replaced instances of '(Symbolhandle) 0' with appropriate new
          macros SymbolZero, EmptyArgument or SymbolError
   010115 Request for list of subtopics must be "?", not something else
          starting with '?'
          Using new macros KEYQUERY and SUBQUERY, both with values '?'
   010521 Changed call to macFindFile()
   010525 Changed macFindFile() to mvFindFile() and dropped wxFindFile()
   010601 Now include mvIface.h instead of wxIface.h and wxIface.h
          MSDOS end of file (0x1a = 26) now treated as EOF in all versions
   010613 help() and usage() now return an invisible logical value which
          is True if any sought for topic was found and False otherwise
   010618 Added check of variable RUNNINGHELP to avoid any problems
          coming from usage like help(file:"a.hlp",help(file:"b.hlp"),topic)
   030710 Subtopic lines of the form @@@@suptopic_name#Subtopic title
          result in Subtopic title being printed as a centered title
   050908 Fixed failure to initialize bug in putHelp()
*/

static void rewindHelp(void)
{
	if (HelpFile != (FILE *) 0)
	{
		rewind(HelpFile);
	}

#ifdef USEFSEEK
	if (LineOffset > 0)
	{
		LineOffset = 0;
	}
#endif /*USEFSEEK*/
	BufferFull = 0;
	CurrentTopic = 0;
	HelpEofHit = 0;
	LastHelpWasCr = 0;
} /*rewindHelp()*/


static int helpgetc(FILE * fp)
{
	int           c;

	c = getc(fp);
	c = (c == NL && LastHelpWasCr) ? getc(fp) : c;

	LastHelpWasCr = (c == CR);

#ifndef MACINTOSH
	c = (LastHelpWasCr) ? '\n' : c;
#else
	c = (c == NL) ? '\n' : c;
#endif /*MACINTOSH*/

	/*
	  Following line no longer protected by ifdef MSDOS
	*/
	c = (c == MSDOSEOF) ? EOF : c;

	return (c);
} /*helpgetc()*/

/*
  Read upto n characters from fp into s[].  return 0 is EOF hit before
  any characters read
  A line starting with EOFSTRING is considered to be an internal EOF marker
  Line is put in *Buffer, without trailing newline
*/
static int getHelpLine(void)
{
	int         c, i, n = BUFFERLENGTH;
	char       *buffer = *Buffer;
	WHERE("getHelpLine");

#ifdef USEFSEEK
	if (LineOffset >= 0)
	{ /* need file position only during the first scann */
		LineOffset = ftell(HelpFile);
		LastHelpWasCr = 0;
	}
#endif /*USEFSEEK*/
	for (i = 0;i < n;i++)
	{
		c = helpgetc(HelpFile);
		if (isNewline(c) || c == EOF)
		{
			break;
		}
		else
		{
			buffer[i] = c;
		}
	} /*for (i = 0;i < n;i++)*/

	if (buffer[0] == EOFSTRINGSTART &&
		strncmp(buffer, EOFSTRING, EOFSTRINGLENGTH) == 0)
	{
		i = 0;
		c = EOF;
	}
	buffer[i] = '\0';

	HelpEofHit = (c == EOF);

	if (i == 0 && HelpEofHit)
	{
		return (0);
	}
	else
	{
		return (1);
	}
} /*getHelpLine()*/

/*
  read help file until line is found starting with ====topicName
  return pointer to topicName in buffer
  return NULL on EOF
*/

static int nextTopic(char *topic)
{
	int         i;
	char        *buffer;
	WHERE("nextTopic");

	if (HelpEofHit)
	{
		rewindHelp();
	}

	topic[0] = '\0';

	while (1)
	{
		if (!BufferFull)
		{
			if (getHelpLine() == 0)
			{
				return (0);
			}
		}
		else
		{
			BufferFull = 0;
		}
		buffer = *Buffer;
		if (strncmp(buffer, MARKERSTRING, NMARKERS) == 0)
		{
			BufferFull = 1;
			buffer += NMARKERS;
			for (i = 0;i < TOPICNAMELENGTH;i++)
			{
				if (!isnamechar(buffer[i]))
				{
					break;
				}
				topic[i] = buffer[i];
			}
			topic[i] = '\0';
			BufferFull = 1;
			return (1);
		} /*if (strncmp(buffer, MARKERSTRING, NMARKERS) == 0)*/
	} /* while (1) */
} /*nextTopic()*/

static void damagedFile(void)
{
	sprintf(OUTSTR,
			"WARNING: help file \"%s\" appears to have been damaged; bad line is",
			*HelpFileName);
	putErrorOUTSTR();
	putOutMsg(*Buffer);
} /*damagedFile()*/


/*
   Function to read list of help keys from file, allocate storage, and
   save them in *KeyList.  Offsets to the keys go in array KeyOffsets
   When called, the Key markers should be in the buffer.  Comma- and/or
   newline-separated keys start on the next line, terminated by a new
   set of key markers
*/

static void initKeys(void)
{
	int      c, nleft = KEYROOM, length, place = 0, ibuf;
	int      damaged;
	char     key[KEYLENGTH + 1], *pc;
	char    *buffer;
	WHERE("initKeys");

	NKeys = 0;
	mydisphandle(KeyList);
	KeyList = mygethandle(nleft*sizeof(char));
	if (KeyList == (char **) 0)
	{
		goto errorExit;
	}

	while (getHelpLine() != 0)
	{ /* loop over lines in help until end of keys or first topic is reached*/
		if (strncmp(*Buffer, KEYSTRING, NMARKERS) == 0 ||
			strncmp(*Buffer, MARKERSTRING, NMARKERS) == 0)
		{
			/* found ^???? or ^==== */
			if ((*Buffer)[0] == MARKER)
			{
				/* mark topic line as unprocessed*/
				BufferFull = 1;
			}
			break;
		}

		ibuf = 0;

		do /* while (c != '\0') */
		{ /* collect keys on current line */
			key[0] = '\0';
			pc = key;
			buffer = *Buffer;
			damaged = 0;
			while ((c = buffer[ibuf++]) != SEPKEYS && c != '\0')
			{
				if (pc - key > KEYLENGTH)
				{
					damaged = 1;
				} /*if (pc - key > KEYLENGTH)*/
				else if (c != ' ' || key[0] != '\0')
				{ /* skip leading spaces*/
					*pc++ = c;
				}
			} /*while ((c = buffer[ibuf++]) != SEPKEYS && c != '\0')*/
			if (damaged)
			{
				if (!Damaged)
				{
					damagedFile();
				}
				Damaged--;
			} /*if (damaged)*/

			pc--;
			while (*pc == ' ' && pc >= key)
			{ /* trim off trailing spaces */
				pc--;
			}
			pc[1] = '\0';
			if (key[0] != '\0')
			{ /* we got a key */
				length = strlen(key);
				if (length + 1 > nleft)
				{ /* we need more room */
					nleft = KEYROOM;
					KeyList = mygrowhandle(KeyList, place + nleft);
					if (KeyList == (char **) 0)
					{
						goto errorExit;
					}
				}
				strcpy(*KeyList + place, key);
				KeyOffsets[NKeys++] = place;
				place += length + 1;
				nleft -= length+1;
				if (NKeys == MAXKEYS)
				{
					break;
				}
			} /*if (key[0] != '\0')*/
		} while (c != '\0');
	} /*while (getHelpLine() != 0)*/

	if (NKeys == 0)
	{
		mydisphandle(KeyList);
		KeyList = (char **) 0;
	}
	else if ((KeyList = mygrowhandle(KeyList, place)) == (char **) 0)
	{
		goto errorExit;
	}

	return;

  errorExit:
	mydisphandle(KeyList);
	KeyList = (char **) 0;
	NKeys = 0;
} /*initKeys()*/

/* comparison of strings ignoring case */
static long keycmp(long iKey, char * string)
{
	char     *key = *KeyList + KeyOffsets[iKey];
	int       c1, c2;
	int       i;
	int       n = strlen(key);
	WHERE("keycmp");

	for (i = 0;i < n;i++)
	{
		c1 = key[i];
		c2 = string[i];
		c1 = (isupper(c1)) ? tolower(c1) : c1;
		c2 = (isupper(c2)) ? tolower(c2) : c2;
		if (c1 != c2)
		{ /* match if run out of string before we run out of key */
			return ((c2 == '\0') ? 0 : 1);
		}
		else if (c1 == '\0')
		{
			return 0;
		}
	}
	return (0);
} /*keycmp()*/

static void listKeys(void)
{
	short       iKey;
	short       maxlength = 0, length;
	short       nperline, nlines, line;
	char       *outstr;

	for (iKey = 0;iKey < NKeys;iKey++)
	{
		length = strlen(*KeyList + KeyOffsets[iKey]);
		maxlength = (length > maxlength) ? length : maxlength;
	}
	maxlength++;
	nperline = SCREENWIDTH/maxlength;
	nperline = (nperline > 0) ? nperline : 1;
	nlines = (NKeys - 1)/nperline + 1;
	sprintf(OUTSTR,
		"Type 'help(key:\"heading\")', where heading is in following list:");
	putOUTSTR();
	for (line = 0;line < nlines;line++)
	{
		outstr = OUTSTR;
		for (iKey = line; iKey < NKeys; iKey += nlines)
		{
			strcpy(outstr, *KeyList + KeyOffsets[iKey]);
			length = strlen(outstr);
			outstr += length;

			if (iKey + nlines <= NKeys)
			{
				while (length++ <= maxlength)
				{
					*outstr++ = ' ';
				}
			}
			*outstr = '\0';
		} /*for (iKey = line; iKey < NKeys; iKey += nlines)*/
		putOUTSTR();
	} /*for (iKey = 0;iKey < NKeys;)*/
} /*listKeys()*/

/*
  000929 modified so that it zeros keytable entry when buffer == (char *) 0
*/
static void setKeyTable(char *buffer, long n)
{
	unsigned long   *keyTable = *KeyTable + n - 1;
	short            i, length;
	char             key[KEYLENGTH + 1];
	char             c;
	WHERE("setKeyTable");

	*keyTable = 0;
	if (buffer != (char *) 0)
	{
		do /* while (c != '\0')*/
		{ /* loop over keys on line after '#'*/
			while (*buffer == ' ')
			{ /* skip spaces */
				buffer++;
			}
			length = 0;
			while ((c = *buffer++) != SEPKEYS && c != '\0')
			{
				if (length == KEYLENGTH)
				{
					if (Damaged <= 0)
					{
						damagedFile();
						Damaged = abs(Damaged);
					}
					Damaged++;
					return;
				}
				key[length++] = c;
			}

			while (length > 0 && key[length-1] == ' ')
			{ /* trim off trailing blanks */
				length--;
			}
			key[length] = '\0';

			if (key[0] != '\0')
			{
				for (i = 0;i < NKeys;i++)
				{
					if (keycmp(i, key) == 0)
					{
						*keyTable |= (1L << i);
					}
				}
			} /*if (key[0] != '\0')*/
		} while (c != '\0');
	}
} /*setKeyTable()*/

static long adjustDate(long date)
{
	if (date < CREATION)
	{
		date += 20*CENTURY;
	}
	else if (date < CENTURY)
	{
		date += 19*CENTURY;
	}
	return (date);
} /*adjustDate()*/

/* Called by help every time, normally does anything only the first time */
enum initHelpCodes
{
	JUSTOPEN    = 1,
	OPENANDSCAN
};

/*
   960502 Changed fopen() to fmyopen()
*/
static void initHelp(int justOpen)
{
	char       topic[TOPICNAMELENGTH+1], *keystart;
	long       room = 0, spaceleft = 0, topicsleft = 0;
	long       length, needed;
	WHERE("initHelp");

	if (NHelpTopics < 0)
	{
		/* no current help topics */
		Damaged = 0;
		if (HelpFileName == (char **) 0)
		{
			HelpFileName = OrigHelpFileName;
			if (HelpFileName == (char **) 0)
			{
				return;
			}
			HelpFile = OrigHelpFile;
		} /*if (HelpFileName == (char **) 0)*/

		if (HelpFile == (FILE *) 0)
		{
			HelpFile = fmyopen(*HelpFileName, TEXTREADMODE);
			LastHelpWasCr = 0;
		}

		if (HelpFile == (FILE *) 0)
		{
			sprintf(OUTSTR,
					"ERROR: unable to open help file '%s'",*HelpFileName);
			putErrorOUTSTR();
			goto errorExit;
		}
		else
		{
			if (HelpFileName == AltHelpFileName)
			{
				AltHelpFile = HelpFile;
			}

#ifdef USEFSEEK
			LineOffset = 0;
#endif /*USEFSEEK*/
			rewindHelp();
		}

		if (justOpen == JUSTOPEN)
		{
			return;
		}

		/* make sure static globals are clear */
		mydisphandle5((char **) TopicOffsets, Topics, Buffer,
					  (char **) KeyTable, (char **) 0);

		KeyTable = (unsigned long **) 0;
		NKeys = 0;

		/* allocate initial space for static globals */

		/* positions of topic names in list of topics */
		TopicOffsets = (short **) mygethandle(NTOPICS * sizeof(short));

		/* will be a CHARACTER vector of topic names */
		Topics = mygethandle(HELPROOM * sizeof(char));

		Buffer = mygethandle(BUFFERLENGTH * sizeof(char));
		if (TopicOffsets == (short **) 0 ||
			Topics == (char **) 0 || Buffer == (char **) 0)
		{
			goto noroom;
		}
#ifdef USEFSEEK
		mydisphandle((char **) FileOffsets);

		/* positions in file of the start of each topic */
		FileOffsets = (long **) mygethandle(NTOPICS * sizeof(long));
		if (FileOffsets == (long **) 0)
		{
			goto noroom;
		}
#endif /*USEFSEEK*/

		/* Now read file */
		while (getHelpLine() != 0)
		{
			if (strncmp(*Buffer, KEYSTRING, NMARKERS) == 0)
			{
				/* find all the keys and initialize table */
				initKeys();

				if (NKeys > 0)
				{ /* create table of keys associated with topics*/
					KeyTable = (unsigned long **)
						mygethandle(NTOPICS*sizeof(unsigned long));
					if (KeyTable == (unsigned long **) 0)
					{
						goto noroom;
					}
				}
				break;
			} /*if (strncmp(*Buffer, KEYSTRING, NMARKERS) == 0)*/
			else if (strncmp(*Buffer, MARKERSTRING, NMARKERS) == 0)
			{
				BufferFull = 1;
				break;
			}
		} /*while (getHelpLine() != 0)*/

		topicsleft = NTOPICS;
		spaceleft = HELPROOM;
		if (!BufferFull)
		{
			(*Buffer)[0] = '\0';
		}

	    /* find all help topics available */
		NHelpTopics = 0;
		LatestNews = OldestNews = -1;

		/* find all the topics; next topic name copied to topic */
		while (nextTopic(topic) != 0)
		{
			length = strlen(topic);
			if (length + 1 > spaceleft)
			{
				spaceleft = HELPCHUNK;
				Topics = mygrowhandle(Topics,(room + spaceleft)*sizeof(char));
				if (Topics == (char **) 0)
				{
					goto noroom;
				}
			} /*if (length + 1 > spaceleft)*/

			if (topicsleft <= 0)
			{
				/* run out of space for topics; allocated more */
				topicsleft = TOPICCHUNK;
				needed = NHelpTopics+topicsleft;
				TopicOffsets = (short **)
					mygrowhandle((char **) TopicOffsets, needed*sizeof(short));
				if (TopicOffsets == (short **) 0)
				{
					goto noroom;
				}
#ifdef USEFSEEK
				FileOffsets = (long **)
					mygrowhandle((char **) FileOffsets, needed*sizeof(long));
				if (FileOffsets == (long **) 0)
				{
					goto noroom;
				}
#endif /*USEFSEEK*/
				if (NKeys > 0)
				{
					KeyTable = (unsigned long **)
						mygrowhandle((char **) KeyTable,
									 needed*sizeof(unsigned long));
					if (KeyTable == (unsigned long **) 0)
					{
						goto noroom;
					}
				}

			} /*if (topicsleft <= 0)*/

			/* save topic name and offset to name */
			strcpy((*Topics) + room, topic);
			(*TopicOffsets)[NHelpTopics] = room;

#ifdef USEFSEEK
			/* save location in file of start of topic */
			(*FileOffsets)[NHelpTopics] = LineOffset;
#endif /*USEFSEEK*/

			NHelpTopics++;
			topicsleft--;
			room += length+1;
			spaceleft -= length+1;

			/*
			  000929 fixed bug so that KeyTable entry is always set
			         even when STARTKEYS is not found
			*/
			if (NKeys > 0)
			{
				keystart = strchr(*Buffer, STARTKEYS);
				if (keystart != (char *) 0)
				{ /* retrieve keys for this entry */
					keystart++;
				}
				setKeyTable(keystart, NHelpTopics);
			}

			BufferFull = 0;
			if (strcmp(topic, NEWS) == 0)
			{
				int        ic;

				/* find earliest and latest new item date */
				while (1)
				{
					while (getHelpLine() != 0 && !isdigit((*Buffer)[0]) &&
						   (*Buffer)[0] != MARKERSTRING[0])
					{
						;
					}
					if ((*Buffer)[0] == '\0' ||
						strncmp(*Buffer, MARKERSTRING, NMARKERS) == 0)
					{
						BufferFull = 1;
						break;
					}

					for (ic = 0; ic < DATEDIGITS &&
						   isdigit((*Buffer)[ic]); ic++)
					{
						;
					}
					if (ic == DATEDIGITS || ic == DATEDIGITS - 2)
					{
						long         itemDate;

						(*Buffer)[ic] = '\0';
						sscanf(*Buffer, "%ld", &itemDate);
						itemDate = adjustDate(itemDate);

						if (OldestNews < 0)
						{
							LatestNews = OldestNews = itemDate;
						}
						else if (itemDate > LatestNews)
						{
							LatestNews = itemDate;
						}
						else if (itemDate < OldestNews)
						{
							OldestNews = itemDate;
						}
					} /*if (ic == DATEDIGITS || ic == DATEDIGITS - 2)*/
					BufferFull = 0;
				} /*while (1)*/
				if (OldestNews < 0)
				{
					/* no dates found; 0 values means topic 'news' found */
					OldestNews = LatestNews = 0;
				}				
			} /*if (strcmp(topic, NEWS) == 0)*/
		} /*while (nextTopic(topic) != 0)*/

		if (Damaged)
		{
			sprintf(OUTSTR,
					"WARNING: %d bad lines were found in file \"%s\"",
					abs(Damaged), *HelpFileName);
			putErrorOUTSTR();
		} /*if (Damaged)*/

		rewindHelp();
#ifdef USEFSEEK
		LineOffset = -1; /* signals no need to use ftell()*/
#endif /*USEFSEEK*/
	} /*if (NHelpTopics < 0)*/
	return;

  noroom:
	putOutErrorMsg("ERROR: unable to allocate memory needed for help");

  errorExit:
	mydisphandle5((char **) TopicOffsets, Topics, Buffer,
				  (char **) 0, (char **) 0);
	TopicOffsets = (short **) 0;
	Topics = Buffer = (char **) 0;
#ifdef USEFSEEK
	mydisphandle((char **) FileOffsets);
	FileOffsets = (long **) 0;
#endif /*USEFSEEK*/
	NHelpTopics = -1;

} /*initHelp() */

/*
	990325 allow file type 'ttro' in addition to 'TEXT' on Macintosh
    001012 now sets NKeys to 0 and disposes of KeyTable when a file
           is changed
*/
static long       changeHelpFile(Symbolhandle symh, long action)
{
	char           *tempName;
	long            nameLength;
	WHERE("changeHelpFile");

	if (action == INEWALTFILE)
	{
		tempName = STRINGPTR(symh);
		if (strlen(tempName) >= PATHSIZE)
		{
			sprintf(OUTSTR, "ERROR: alternate help file name too long");
			goto errorExit;
		}
#ifdef HASFINDFILE
		tempName = mvFindFile(tempName, "Specify the Help File", NullString,
							  mvReadit | mvTextFile | mvROTextFile,
							  getMvDirInfo(helpVol));
		if (tempName == (char *) 0)
		{ /* cancelled */
			goto errorExit;
		}
#endif /*HASFINDFILE*/
		tempName = expandFilename(tempName);
		if (tempName == (char *) 0)
		{
			goto errorExit;
		}

		if (!isfilename(tempName))
		{
			goto errorExit;
		}

		if (HelpFileName == (char **) 0)
		{
			HelpFileName = OrigHelpFileName;
		}
		if (HelpFileName != (char **) 0 &&
			strcmp(tempName, *HelpFileName) == 0)
		{ /* file name is the same as current file; do nothing*/
			return (1);
		}

		/*
		  Note: OrigHelpFile is never closed, even when we are using an alternative
		  Similarly, AltHelpFile is not closed when we switch back to
		  OrigHelpFile
		*/
		if (HelpFile == OrigHelpFile)
		{
			rewindHelp();
		}

	    /* new alternative help file */
	    if (AltHelpFile != (FILE *) 0)
		{
			fclose(AltHelpFile);
		}
		AltHelpFile = (FILE *) 0;

		HelpFile = (FILE *) 0;
		NHelpTopics = -1;

		mydisphandle(AltHelpFileName);
		HelpFileName = AltHelpFileName = (char **) 0;

		nameLength = strlen(tempName);
		AltHelpFileName = mygethandle(nameLength + 1);
		if (AltHelpFileName == (char **) 0)
		{
			goto errorExit;
		}
		strcpy(*AltHelpFileName, tempName);
		HelpFileName = AltHelpFileName;
	} /*if (action == INEWALTFILE)*/
	else
	{
		if (action == IORIGFILE)
		{
			if (HelpFile != OrigHelpFile)
			{
				/* do not close AltHelpFile */
				HelpFile = OrigHelpFile;
				HelpFileName = OrigHelpFileName;
				rewindHelp();
				NHelpTopics = -1;
			} /*if (HelpFile != OrigHelpFile)*/
		} /*if (symh == symhOrig)*/
		else
		{
			if (AltHelpFile == (FILE *) 0)
			{
				sprintf(OUTSTR,
						"ERROR: no alternative help file has been previously specified");
				goto errorExit;
			}
			if (HelpFile != AltHelpFile)
			{
				HelpFile = AltHelpFile;
				HelpFileName = AltHelpFileName;
				rewindHelp();
				NHelpTopics = -1;
			} /*if (HelpFile != AltHelpFile)*/
		} /*if (symh == symhOrig){}else{}*/
	} /*if (action == INEWALTFILE){}else{}*/
	if (NHelpTopics < 0)
	{
		NKeys = 0;
		mydisphandle((char **) KeyTable);
		KeyTable = (unsigned long **) 0;
	}

	LastHelpWasCr = 0;

	return (1);

  errorExit:
	putErrorOUTSTR();
	return (0);

} /*changeHelpFile()*/

static long       fillTopiclist(char *topic, short topiclist[], long matchType,
								unsigned long keyMask, short *maxlength)
{
	short           itopic, ntopics = 0, length;

	*maxlength = 0;
	for (itopic = 0;itopic < NHelpTopics;itopic++)
	{ /* count topics to be listed */
		if (NKeys > 0 && (*KeyTable)[itopic] & keyMask ||
			matchName(*Topics + (*TopicOffsets)[itopic], matchType,topic))
		{
			length = strlen(*Topics + (*TopicOffsets)[itopic]);
			*maxlength = (length > *maxlength) ? length : *maxlength;
			topiclist[ntopics++] = itopic;
		}
	}
	return (ntopics);
} /*fillTopiclist()*/

static void       listTopics(char *topic, long matchType)
{
	short          nperline, itopic, nlines, ntopics = 0, line;
	short          length, topiclist[500], maxlength = 0;
	unsigned long  keyMask = 0;
	char           *name, *outstr;
	WHERE("listTopics");

	SETUPINT(errorExit);

	if (matchType < 0)
	{
		keyMask = (-matchType <= MAXKEYS) ? 1L << -(matchType+1) : 0xffffffff;
	}

	initHelp(OPENANDSCAN);

	if (NHelpTopics > 0)
	{
		if (matchType == -(MAXKEYS + 1) ||
			matchType > 0 && (matchType & ANYMATCH))
		{
			putOutMsg("Help is available on the following topics: ");
		}
		else if (matchType < 0)
		{
			sprintf(OUTSTR,
					"The following help topics concern %s",
					*KeyList + KeyOffsets[-(matchType+1)]);
			putOUTSTR();
		}

		ntopics = fillTopiclist(topic, topiclist, matchType,
								keyMask, &maxlength);
		maxlength++;
		nperline = SCREENWIDTH/maxlength;
		if (nperline < 1)
		{
			nperline = 1;
		}
		nlines = (ntopics - 1)/nperline + 1;

		for (line = 0;line < nlines;line++)
		{
			outstr = OUTSTR;
			for (itopic = line;itopic < ntopics;itopic += nlines)
			{
				name = *Topics + (*TopicOffsets)[topiclist[itopic]];
				strcpy(outstr, name);
				length = strlen(name);
				outstr += length;
				if (itopic + nlines < ntopics)
				{
					while (length++ < maxlength)
					{
						*outstr++ = ' ';
					}
				}
				*outstr = '\0';
			} /*for (itopic = line;itopic < NHelpTopics;itopic += nlines)*/
			putOUTSTR();
			checkInterrupt(errorExit);
		} /*for (line = 0;line < nlines;line++)*/

		if (matchType == ANYMATCH && HelpFile == OrigHelpFile)
		{
			putOutMsg("For information on help, enter help(help)");
		}
	} /*if (NHelpTopics > 0)*/

	/* fall through */

  errorExit:
	return;

} /*listTopics()*/

static char      *ControlNames[] =
{
	"break", "breakall", "else", "elseif", "for",
	"help", "Help", "if", "while", "next", "return",
	(char *) 0
};

#define isControl(NAME) matchKey(NAME, ControlNames, (long *) 0)

#undef SPECIALTRANS
#ifdef SPECIALTRANS
/*
  Note: atan() and round() are omitted from this list because they have
  their own entries
*/

#define IANGLEVALUE  10 /*index of "acos"*/
#define IANGLEARG    12 /*index of "cos" */

static char           *TransformationNames[] =
{
	"abs"    ,"exp"    ,"lgamma" ,"log"    ,"log10"  ,"sqrt"   ,"cosh"   ,
	"sinh"   ,"tanh"   , "atanh" , "acos"  ,"asin"   ,"cos"    ,"sin"    ,
	"tan"    , (char *) 0
};

#define isTransformation(NAME) matchKey(NAME, TransformationNames, (long *) 0)

#endif /*SPECIALTRANS*/

/*
  981215 Removed code for printing usage for transformations.  Transformation
         Usage is now taken from the help file as it is for all other topics
         Also removed 3rd argument char *topic

  990723 Disabled code for printing 'transformations' help for individual
         transformations such as cos() and exp(); help has been
         added to MacAnova.hlp for all transformations
  001207 Added arguments topic and subtopic to implement new usage
         help(topicname:"subtopic")
  001211 Changed subtopic argument to symhSub to allow for several subtopics
         of a single topic
  001214 Added argument whichSubtopic to return which of the requested
         subtopics was found
*/
static int  matchSubtopic(char  * subtopic, long nSubtopics,
						  char * thisSubtopic, long * whichSubtopic)
{
	long         i;

	for (i = 0; i < nSubtopics; i++, subtopic = skipStrings(subtopic, 1))
	{
		if (isalnum(subtopic[0]) &&
			mystrncmp(thisSubtopic, subtopic, strlen(subtopic)) == 0)
		{
			*whichSubtopic = i;
			return (1);
		}
	}
	return (0);
} /*matchSubtopic()*/


static void printSubtopicTitle(char *buffer, char * topic)
{
	int         place = 4, place1 = 0;
	int         center = (topic == (char *) 0);
	char        c = buffer[place];
	char        subtopicName[SUBTOPICNAMELENGTH + 1];

	while (isSubtopicChar(c) || c == ' ' || c == '\t')
	{
		if (c != ' ' && c != '\t' && place1 < SUBTOPICNAMELENGTH)
		{
			subtopicName[place1++] = c;
		}
		c = buffer[++place];
	}
	subtopicName[place1] = '\0';
	
	if (c == STARTCENTERTITLE || c == STARTLEFTTITLE)
	{
		char       *outstr = OUTSTR;
		int         titleLength, j, indent;

		center = center && c == STARTCENTERTITLE;

		c = buffer[++place];
		while (c == ' ' || c == '\t')
		{
			c = buffer[++place];
		}
		titleLength = strlen(buffer + place);
		if (titleLength > SUBTOPICTITLELENGTH)
		{
			titleLength = SUBTOPICTITLELENGTH;
		}
		indent = (center) ? (SUBTOPICTITLELENGTH - titleLength)/2 : 0;
		for (j = 0; j < indent; j++)
		{
			*outstr++ = ' ';
		}

		strncpy(outstr, buffer + place, SUBTOPICTITLELENGTH);
		OUTSTR[SUBTOPICTITLELENGTH] = '\0';
		if (topic != (char *) 0)
		{
			outstr += strlen(outstr);
			sprintf(outstr," (subtopic of '%s')", topic);
		}
	}
	else if (topic != (char *) 0)
	{
		sprintf(OUTSTR, "Subtopic '%s' of '%s'", subtopicName, topic);
	}
	putOUTSTR();
} /*printSubtopicTitle()*/


/*
  050908 fixed bug; whichSubtopic was not initialized
*/
static int putHelp(long dates[2], char *topic, Symbolhandle symhSub,
				   long usage)
{
	long         i, line = 0;
	long         dateSought = dates[1];
	int          needSubtopic = (symhSub != SymbolZero);
	int          foundDate = (dates[0] < 0 || needSubtopic) ? -1 : 0;
	int          printLine = (foundDate < 0);
	int          listSubtopics;
	int          foundUsage = 0, foundSubtopic = !needSubtopic;
	int          anySubtopics = 0;
	char         subtopicsFound[SUBTOPICLIMIT];
	long         nSubtopics, nSubtopicsFound = 0;
	char        *buffer = *Buffer;
	char        *subtopics = (needSubtopic) ?
	  STRINGPTR(symhSub) : (char *) 0;
	WHERE("putHelp");

	listSubtopics = needSubtopic && isSubQuery(subtopics);

	if (needSubtopic)
	{
		nSubtopics = symbolSize(symhSub);
		if (nSubtopics > 1)
		{
			if (listSubtopics)
			{
				sprintf(OUTSTR,
						"WARNING: subtopic names after \"%c\" ignored",
						SUBQUERY);
				putErrorOUTSTR();
				nSubtopics = 1;
			} /*if (listSubtopics)*/
			else
			{
				char        *place = subtopics;

				for (i = 0; i < nSubtopics; i++)
				{
					if (isSubQuery(place))
					{
						break;
					}
					place = skipStrings(place, 1);
				}
				if (i < nSubtopics)
				{
					sprintf(OUTSTR,
							"WARNING: \"%c\" after first subtopic ignored by %s()",
							SUBQUERY, FUNCNAME);
					putErrorOUTSTR();
				}
			} /*if (listSubtopics){}else{}*/
		} /*if (nSubtopics > 1)*/

		for (i = 0; i < SUBTOPICLIMIT; i++)
		{
			subtopicsFound[i] = 0;
		}
	} /*if (needSubtopic)*/
	else
	{
		nSubtopics = 0;
	}

	BufferFull = 0;

	/* now print it out */

	while (1)
	{
		int      isSubtopicMarker;

		checkInterrupt(errorExit);
		buffer[0] = '\0';
		if (getHelpLine() == 0 ||
			(buffer[0] == MARKER && buffer[1] == MARKER
			 && buffer[2] == MARKER && buffer[3] == MARKER))
		{					/* end of topic found */
			break;
		}

		isSubtopicMarker = strncmp(buffer, SUBTOPICMARKERSTRING, 4) == 0;
		if (isSubtopicMarker)
		{
			long          whichSubtopic = 0;

			if (listSubtopics ||
				needSubtopic &&
				matchSubtopic(subtopics, nSubtopics, buffer + 4,
							  &whichSubtopic))
			{
				int         place;
				char        thisSubtopic[SUBTOPICNAMELENGTH + 1];

				for (place = 0; place < SUBTOPICNAMELENGTH; place++)
				{
					char        c = buffer[place + 4];

					if (!isSubtopicChar(c))
					{
						break;
					}
					thisSubtopic[place] = c;
				}
				thisSubtopic[place] = '\0';
				nSubtopicsFound++;
				foundSubtopic = 1;

				if (whichSubtopic < SUBTOPICLIMIT)
				{
					subtopicsFound[whichSubtopic] = 1;
				}

				if (thisSubtopic[0] != '_')
				{
					/* not an end-of-subtopic line*/
					if (listSubtopics)
					{
						if (!anySubtopics)
						{
							sprintf(OUTSTR,
									"Available subtopics for topic '%s' are:", topic);
							putOUTSTR();
						}
						sprintf(OUTSTR, "  %s", thisSubtopic);
						putOUTSTR();
					} /*if (listSubtopics)*/
					else
					{
						printSubtopicTitle(buffer, topic);
					}
				} /*if (thisSubtopic[0] != '_')*/
			}
			else 
			{
				if (!needSubtopic)
				{
					printSubtopicTitle(buffer, (char *) 0);
				}
				foundSubtopic = (foundSubtopic) ? -1 : 0;
			}
			anySubtopics = 1;
		} /*if (isSubtopicMarker)*/
		else if (strncmp(buffer, USAGESTRING, 4) == 0)
		{
			if (foundUsage)
			{
				if (usage == PUTUSAGE)
				{
					break;
				}
				printLine = (foundDate < 0);
			}
			else
			{
				foundUsage = 1;
				printLine = (usage == PUTUSAGE) ? 1 : 0;
			}
		} /*if (strncmp(buffer, USAGESTRING, 4) == 0)*/
		else
		{
			if (foundDate >= 0)
			{/* topic is 'news' and there are dated news items in file */
				for (i = 0;i < DATEDIGITS;i++)
				{
					if (!isdigit(buffer[i]))
					{
						break;
					}
				} /*for (i = 0;i < DATEDIGITS;i++)*/

				if (i == DATEDIGITS || i == DATEDIGITS - 2)
				{/* line starts with date */
					char      datebuf[DATEDIGITS + 1];
					long      thisDate;

					strncpy(datebuf, buffer, i);
					datebuf[i] = '\0';

					sscanf(datebuf, "%ld", &thisDate);
					thisDate = adjustDate(thisDate);

					if (foundDate && thisDate < dateSought)
					{		/* earlier than date[0] */
						break;
					}
					if (!foundDate && thisDate <= dateSought)
					{
						foundDate = 1;
						printLine = thisDate <= LatestNews &&
						  thisDate >= OldestNews;
						dateSought = dates[0];
					}
				} /*if (i == DATEDIGITS)*/
			} /*if (foundDate >= 0)*/
			else if (!foundUsage && !printLine)
			{
				printLine = (line > 0);
			}

			if (printLine &&
				(!needSubtopic || !listSubtopics && foundSubtopic > 0))
			{
				putOutMsg(buffer);
			}
		} /*if (strncmp(buffer, USAGESTRING, 4) == 0){}else{}*/
		line++;
	} /*while (1)*/

	if (needSubtopic && nSubtopicsFound < nSubtopics)
	{
		if (!anySubtopics)
		{
			sprintf(OUTSTR,
					"WARNING: topic '%s' has no subtopics", topic);
			putErrorOUTSTR();
		}
		else
		{
			char       *aSubtopic = subtopics;

			for (i = 0; i < nSubtopics && i < SUBTOPICLIMIT; i++)
			{
				if (!subtopicsFound[i] && !isSubQuery(aSubtopic))
				{
					sprintf(OUTSTR,
							"WARNING: subtopic '%s' of topic '%s' not found",
							aSubtopic, topic);
					putErrorOUTSTR();
				}
				aSubtopic = skipStrings(aSubtopic, 1);
			}
		}
	} /*if (needSubtopic && foundSubtopic < nSubtopics)*/
	else if (listSubtopics)
	{
		char     *outstr = OUTSTR;
		char     *msgstart = "Type help(%s";
		char     *msgend = ":vector(\"subtopicA\",\"subtopicB\",...))";
		
		sprintf(outstr, msgstart, topic);
		outstr += strlen(outstr);
		
		if (strlen(topic) > 10)
		{
			strcat(outstr, ",subtopic");
			outstr += strlen(outstr);
		}
		strcat(outstr, msgend);
		putOUTSTR();
	}

	if (*buffer)
	{
		BufferFull = 1;
	}

	return ((foundUsage) ? 1 : -1);

  errorExit:
	*OUTSTR = '\0';
	return (0);
} /*putHelp()*/

/*
  001207 moved code from help() to new function putKeys() which processes
  help() keyword 'key'
*/
static int putKeys(Symbolhandle symhKey)
{
	int         i;
	long        matchType = 0;

	if (NKeys <= 0)
	{
		sprintf(OUTSTR,
				"ERROR: no keys used in help file");
		goto errorExit;
	}

	if (strcmp("all", STRINGPTR(symhKey)) == 0)
	{						/* list all topics */
		matchType = -(MAXKEYS + 1);
		i = 0;
	}
	else if (STRINGVALUE(symhKey, 0) == KEYQUERY)
	{
		i = NKeys + 1;
	}
	else
	{
		for (i = 0;i < NKeys;i++)
		{
			if (keycmp(i, STRINGPTR(symhKey)) == 0)
			{
				matchType = -(i + 1);
				break;
			}
		} /*for (i = 0;i < NKeys;i++)*/
	}

	if (i >= NKeys)
	{/* list keys */
		if (i == NKeys)
		{
			putPieces("ERROR: Key ", STRINGPTR(symhKey),
					  " does not match any key in the help file",
					  (char *) 0);
		}
		listKeys();
		if (i == NKeys)
		{
			goto errorExit;
		}
	} /*if (i >= NKeys)*/
	else
	{
		listTopics(NullString, matchType);
		putOutMsg("For help on topic foo, enter help(foo) or help(\"foo\")");
	}

	return (1);

  errorExit:
	return (0);
} /*putKeys()*/

/*
  001207 new function constructed from code moved from help()
  010621 substantially rewritten; new argument silent to suppres
         warning message
*/
static int getDates(Symbolhandle symhNews, long dates[], int silent)
{
	double         val;
	long           ndates = symbolSize(symhNews), type = TYPE(symhNews);
	/*
	  news:yymmdd
	  news:vector(yymmdd1, yymmdd2)
	  news:yyyymmdd
	  news:vector(yyyymmdd1, yyyymmdd2)
	  Immediately translate a date of the form yymmdd to yyyymmdd
	*/

	if (type != REAL || !isVector(symhNews) || ndates > 2)
	{
		goto errorExit;
	}

	if (OldestNews < 0)
	{
		/* no news in this file */
		return (-1);
	}
	
	val = DATAVALUE(symhNews, 0);
	if (val != floor(val) || val < 0)
	{
		goto errorExit;
	}
	dates[0] = adjustDate((long) val);
	
	if (ndates == 2)
	{
		val = DATAVALUE(symhNews, 1);
		if (val != floor(val) || val < 0)
		{
			goto errorExit;
		}
		dates[1] = adjustDate((long) val);
	}
	else
	{
		dates[1] = (dates[0] <= LatestNews) ? LatestNews : dates[0];
	}
	
	if (dates[1] < dates[0])
	{
		long     tmp = dates[0];

		dates[0] = dates[1];
		dates[1] = tmp;
	}
	return (ndates);

  errorExit:
	if (!silent)
	{
		sprintf(OUTSTR,
				"WARNING: value of '%s' not 1 or 2 dates of form yymmdd or yyyymmdd; ignored",
				isKeyword(symhNews));
		putErrorOUTSTR();
	}
	
	dates[0] = dates[1] = -1;
	return (-1);
} /*getDates()*/

/*
  returns 1 if first subtopic is "?" or if any subtopic starts
  with an alphanumeric character
*/
static int checkSubtopics(Symbolhandle symhSubtopic)
{
	long       nSubtopics = symbolSize(symhSubtopic), i;
	char      *subtopics = STRINGPTR(symhSubtopic);
	char       c;

	for (i = 0; i < nSubtopics; i++, subtopics = skipStrings(subtopics, 1))
	{
		c = subtopics[0];

		if (i == 0 && isSubQuery(subtopics) || isalnum(c))
		{
			break;
		}
	}
	return ((int) c);
} /*checkSubtopics()*/

/*
  001213 New function to check whether its argument is a topic
         name, possibly with leading or trailing spaces
*/
static int isHelpTopic(Symbolhandle symh)
{
	char         *argument = STRINGPTR(symh);

	while (isspace(*argument))
	{
		argument++;
	}

	while (isnamechar(*argument))
	{
		argument++;
	}
	while (isspace(*argument))
	{
		argument++;
	}
	return (*argument == '\0');
} /*isHelpTopic()*/


/*
   950907 added argument to initHelp() so that file is not scanned on
   calls of form help(file:filename) and help(orig:T).  This change means
   a startup file can set the default help file without the cost of
   scanning it before it's needed

   951212 added entry usage() and modified help file accordingly.  If
   the first line of a help entry starts with %%%%, everything up to a closing
   %%%% is considered information to be printed by usage() but not by help()
   If there is no %%%% marker, usage prints the same as help.

   For transformations in the list TransformationNames, usage() prints
   a generic message, while help() prints help for topic "transformations"

   960426 Made one of file:fileName, orig:T and alt:T legal with key:"Key"
   Also changeHelpFile() so there shouldn't be any unnecessary scans of
   the file.

   960723 Changed code so that 'help()' gives a short message but does not
   list all the topics.  'help("*")' gives all topics.  Also changed the
   "teaser" message to include 'help("*")' usage.

   970223 Fixed bug that could cause crash with too long help topics

   970808 Fixed bug so that now help(file:filename, "*") just lists topic
   names instead of all help

   990723 Removed special treatment of transformations

   001207 Added capability to give help on subtopics

   001213 Modified for having all arguments quoted and the quoted
          argument bit set in the symbol for help().  This eliminates
          the need to quote long topics and syntax elements
          (help(transformations) instead of help(transformations) and
          help(break) instead of help("break").

   010115 Modified short help information to include info on subtopics;
          also, removed check for help(__teaser__) just plain help()
          gets the short help information

   010613 help() and usage() now return an invisible LOGICAL scalar, True if
          at least one topic was found and False otherwise
          Also, new keyword phrase silent:T suppresses warning messages after
          the first scanning of the arguments.
   010619 Function names are now gethelp() and getusage(), with help and usage
          now macros
*/

Symbolhandle    help(Symbolhandle list)
{
	Symbolhandle    symh;
	Symbolhandle    symhFile = SymbolZero, symhOrig = SymbolZero;
	Symbolhandle    symhAlt = SymbolZero;
	Symbolhandle    symhKey = SymbolZero;
	Symbolhandle    symhUsage = SymbolZero;
	Symbolhandle    symhSubtopic = SymbolZero;
	Symbolhandle    symhSilent = SymbolZero;
	Symbolhandle    symhNews = SymbolZero;
	Symbolhandle    symhPrintname = SymbolZero;
	long            doWhat =
	  (strcmp(FUNCNAME, "getusage") != 0) ? PUTHELP : PUTUSAGE;
	long            iTopic, nTopics = 0, i;
	long            nFileActions = 0, action = 0, actionArg = -1;
	long            type;
	char           *name;
	long            nargs = NARGS(list), firstUsage = -1;
	long            karg;
	long            nUnused = nargs, firstUnused = -1;
	long            nargsOrig = nargs;
	long            matchType = 0;
	char            *what, *keyword = (char *) 0;
	char            target[TOPICNAMELENGTH+1];
	char          **subtopicH = (char **) 0, *subtopic = (char *) 0;
#ifdef SPECIALTRANS
	int             doneTransformation = 0;
	char           *tranName = "transformations";
#endif /*SPECIALTRANS*/
	int             reply;
	int             doneNull = 0, doneLogic = 0, doneNumber = 0;
	int             silent = 0, printname = 0;
	int             foundSomeHelp = 0, exactMatch = 1;
	char           *duplicateKeyMsg =
	  "WARNING: duplicate use of keyword '%s'; first one used";
	char           *logicName = "logic", *numberName = "number";
#ifdef SCROLLABLEWINDOW
	Symbolhandle    symhScrollback = SymbolZero;
	int             scrollback = CURRENTSCROLLBACK;
#endif /*SCROLLABLEWINDOW*/
	WHERE("help");

	if (RUNNINGHELP)
	{
		sprintf(OUTSTR,
				"ERROR: recursive use of %s() from %s()", FUNCNAME,
				RUNNINGHELP);
		goto errorExit;
	} /*if (RUNNINGHELP)*/
	RUNNINGHELP = FUNCNAME;

	/* NHelpTopics and TopicOffsets are initialized by initHelp() */
	symh = COMPVALUE(list, 0);

	if (nargs == 1 && symh == EmptyArgument)
	{
		/* just plain 'help()' */
		for (i = 0; HelpMsg[i] != (char *) 0; i++)
		{
			putOutMsg(HelpMsg[i]);
		}
		goto normalExit;
	} /*if (nargs == 1 && symh == EmptyArgument) */

	for (karg = 0; karg < nargs; karg++)
	{
		if (strcmp(STRINGPTR(COMPVALUE(list, karg)),"silent:T") == 0)
		{
			silent = 1;
			break;
		}
	} /*for (karg = 0; karg < nargs; karg++)*/
	
	for (karg = 0; karg < nargs; karg++)
	{
		symh = COMPVALUE(list, karg);

		if (symh == EmptyArgument)
		{
			if (!silent)
			{
				sprintf(OUTSTR,
						"WARNING: argument %ld to %s() is empty",
						karg+1, FUNCNAME);
			}			
		} /*if (symh == EmptyArgument)*/
		else
		{
			/* recall no arguments are yet evaluated */
			if (!isHelpTopic(symh))
			{
				/* must be keyword, quoted argument or an expression*/
				symh = evalQuotedArg(list, karg);
				if (symh == SymbolError)
				{
					/*presumably an error message was issued elsewhere */
					goto errorExit;
				}
			}

			keyword = isKeyword(symh);
			if (keyword != (char *) 0)
			{
				Symbolhandle    symhTF = SymbolZero;
				int             usedKey = 0;

				if (strcmp(keyword, "news") == 0)
				{
					if (symhNews != SymbolZero)
					{
						if (!silent)
						{
							sprintf(OUTSTR, duplicateKeyMsg, keyword);
						}
					}
					else
					{
						symhNews = symh;
					}
				}
				else if (strncmp(keyword,"key",3) == 0)
				{
					if (symhKey != SymbolZero)
					{
						if (!silent)
						{
							sprintf(OUTSTR, duplicateKeyMsg, keyword);
						}
					}
					else
					{
						symhKey = symh;
					}
					usedKey = 1;
				}
				else if (strcmp(keyword,"usage") == 0)
				{
					if (symhUsage != SymbolZero)
					{
						if (!silent)
						{
							sprintf(OUTSTR, duplicateKeyMsg, keyword);
						}
					}
					else
					{
						symhTF = symhUsage = symh;
					}
					usedKey = 1;
				}
				else if (strcmp(keyword,"silent") == 0)
				{
					if (symhSilent != SymbolZero)
					{
						if (!silent)
						{
							sprintf(OUTSTR, duplicateKeyMsg, keyword);
						}
					}
					else
					{
						symhTF = symhSilent = symh;
					}
					usedKey = 1;
				}
				else if (strcmp(keyword,"printname") == 0)
				{
					if (symhPrintname != SymbolZero)
					{
						if (!silent)
						{
							sprintf(OUTSTR, duplicateKeyMsg, keyword);
						}
					}
					else
					{
						symhTF = symhPrintname = symh;
					}
					usedKey = 1;
				}
				else if (strncmp(keyword,"subtopic", 8) == 0)
				{
					if (symhSubtopic != SymbolZero)
					{
						if (!silent)
						{
							sprintf(OUTSTR, duplicateKeyMsg, keyword);
						}
					}
					else
					{
						symhSubtopic = symh;
					}
					usedKey = 1;
				}
				else if (strcmp(keyword,"scrollback") == 0)
				{
#ifdef SCROLLABLEWINDOW
					if (symhScrollback != SymbolZero)
					{
						if (!silent)
						{
							sprintf(OUTSTR, duplicateKeyMsg, keyword);
						}
					}
					else
					{
						symhTF = symhScrollback = symh;
					}
#else /*SCROLLABLEWINDOW*/
					if (!silent)
					{
						sprintf(OUTSTR,
								"WARNING: %s() keyword '%s' ignored in this version",
								FUNCNAME, keyword);
						putErrorOUTSTR();
					}
#endif /*SCROLLABLEWINDOW*/
					usedKey = 1;
				}
				else if (strcmp(keyword,"file") == 0)
				{
					if (symhFile != SymbolZero)
					{
						if (!silent)
						{
							sprintf(OUTSTR, duplicateKeyMsg, keyword);
						}
					}
					else
					{
						symhFile = symh;
						actionArg = karg;
						nFileActions++;
					}
					usedKey = 1;
				}
				else if (strncmp(keyword,"orig", 4) == 0)
				{
					if (symhOrig != SymbolZero)
					{
						if (!silent)
						{
							sprintf(OUTSTR, duplicateKeyMsg, keyword);
						}
					}
					else
					{
						symhTF = symhOrig = symh;
						actionArg = karg;
						nFileActions++;
					}
					usedKey = 1;
				}
				else if (strncmp(keyword,"alt", 3) == 0)
				{
					if (symhAlt != SymbolZero)
					{
						if (!silent)
						{
							sprintf(OUTSTR, duplicateKeyMsg, keyword);
						}
					}
					else
					{
						symhTF = symhAlt = symh;
						actionArg = karg;
						nFileActions++;
					}
					usedKey = 1;
				}
				else if (strncmp(keyword, "allf", 4) == 0)
				{
					/* ignore keyword allfiles used by macro help */
					usedKey = 1;
				}
				
				if (usedKey)
				{
					setNAME(symh, USEDKEYWORD);
					nUnused--;
				}
				else if (firstUnused < 0)
				{
					firstUnused = karg;
				}
				if (symhTF != SymbolZero && !isTorF(symhTF))
				{
					notTorF(keyword);
					goto errorExit;
				}
			} /*if (keyword != (char *) 0)*/
			else if (firstUnused < 0)
			{
				firstUnused = karg;
			}
		}/*if (symh == EmptyArgument){}else{}*/
		if (*OUTSTR)
		{
			putErrorOUTSTR();
		}
	} /*for (karg = 0; karg < nargs; karg++)*/

	if (nFileActions > 1)
	{
		sprintf(OUTSTR,
				"ERROR: you can't use more than 1 of 'file', 'orig', or 'alt' with %s()",
				FUNCNAME);
		goto errorExit;
	} /*if (nFileActions > 1)*/


	if (symhUsage != SymbolZero)
	{
		doWhat = (DATAVALUE(symhUsage, 0) != 0.0) ? PUTUSAGE : PUTHELP;
		setNAME(symhUsage, USEDKEYWORD);
	} /*if (symhUsage != SymbolZero)*/

	if (symhSilent != SymbolZero)
	{
		silent = (DATAVALUE(symhSilent, 0) != 0.0) ? 1 : 0;
		setNAME(symhSilent, USEDKEYWORD);
	} /*if (symhSilent != SymbolZero)*/

	if (symhPrintname != SymbolZero)
	{
		printname = (DATAVALUE(symhPrintname, 0) != 0.0) ? 1 : 0;
		setNAME(symhPrintname, USEDKEYWORD);
	} /*if (symhPrintname != SymbolZero)*/

	if (symhKey != SymbolZero)
	{
		/* key:"keyword" was an argument */
		if (!isCharOrString(symhKey))
		{
			notCharOrString("key");
			goto errorExit;
		}
		if (symhUsage != SymbolZero ||
			symhSubtopic != SymbolZero || symhNews != SymbolZero)
		{
			sprintf(OUTSTR,
					"ERROR: you can't use keywords 'usage', 'subtopic' or 'news' with 'key'");
			goto errorExit;
		}
	} /*if (symhKey != SymbolZero)*/

	/*
	  All the LOGICAL scalar keywords have been checked
	*/
#ifdef SCROLLABLEWINDOW
	if (symhScrollback != SymbolZero)
	{
		/* scrollback:T or F */
		scrollback = DATAVALUE(symhScrollback, 0) != 0.0;
	} /*if (symhScrollback != SymbolZero)*/
#endif /*SCROLLABLEWINDOW*/

	if (symhSubtopic != SymbolZero)
	{
		char        *keyword = "subtopic";

		/* subtopic:"subtopicName" */
		if (symhNews != SymbolZero)
		{
			sprintf(OUTSTR,
					"ERROR: you can't use topic 'news' with keyword '%s'",
					keyword);
		}
		else if (nUnused == 0)
		{
			sprintf(OUTSTR,
					"ERROR: no topic specified when keyword '%s' used",
					keyword);
		}
		else if (nUnused > 1)
		{
			sprintf(OUTSTR,
					"ERROR: more than one topic specified with keyword '%s'",
					keyword);
		}
		else if (!isVector(symhSubtopic) || TYPE(symhSubtopic) != CHAR)
		{
			sprintf(OUTSTR,
				   "ERROR: value of '%s' not CHARACTER vector",
				   keyword);
			goto errorExit;
		}
		if (OUTSTR[0] != '\0')
		{
			goto errorExit;
		}

		reply = checkSubtopics(symhSubtopic);
		if (!isalnum(reply) && reply != SUBQUERY)
		{
			sprintf(OUTSTR,
				   "ERROR: value of '%s' does not specify a legal subtopic name",
				   keyword);
			goto errorExit;
		}
	} /*if (symhSubtopic != SymbolZero)*/

	if (nFileActions > 0)
	{ /* one of file:"filename", alt:T, or orig:T */
		if (symhFile != SymbolZero)
		{ /* file:"filename", sets new help file as alternate*/
			symh = symhFile;
			keyword = "file";
			action = INEWALTFILE;
			if (!isCharOrString(symh))
			{
#ifdef HASFINDFILE
				sprintf(OUTSTR,
						"ERROR: usage is %s(%s:\"fileName\") or %s(%s:\"\")",
						FUNCNAME, keyword, FUNCNAME, keyword);
#else /*HASFINDFILE*/
				sprintf(OUTSTR,
						"ERROR: usage is %s(%s:\"fileName\")",
						FUNCNAME, keyword);
#endif /*HASFINDFILE*/
				goto errorExit;
			} /*if (!isCharOrString(symh))*/
		} /*if (symhFile != SymbolZero)*/
		else
		{
			if (symhAlt != SymbolZero)
			{ /* alt:T means use file most recent file set by file:fileName	*/
				action = IALTFILE;
				symh = symhAlt;
				keyword = "alt";
			}
			else
			{ /* orig:T means use help file set at startup */
				action = IORIGFILE;
				symh = symhOrig;
				keyword = "orig";
			}
			if (DATAVALUE(symh, 0) == 0.0)
			{
				sprintf(OUTSTR,
						"ERROR: value for '%s' must be T", keyword);
				goto errorExit;
			}
		} /*if (symhFile != SymbolZero){}else{}*/

		if (!changeHelpFile(symh, action))
		{
			goto errorExit;
		}

		if (nUnused == 0 && symhKey == SymbolZero &&
			symhNews == SymbolZero)
		{ /* open Help file but don't read it */
			initHelp(JUSTOPEN);
			if (HelpFile == (FILE *) 0)
			{
				goto errorExit;
			}
			goto normalExit;
		}
	} /*if (nFileActions > 0)*/

	/*
	  Up to here, nothing has been read from the help file; if
	  it hasn't previously been read, no topics are known and initHelp()
	  sets up the topic and key tables
	*/
	initHelp(OPENANDSCAN);

	if (HelpFileName == (char **) 0 || HelpFile == (FILE *) 0)
	{
		goto errorExit;
	}
	if (NHelpTopics <= 0)
	{
		goto notAvailable;
	}

	if (symhKey != SymbolZero)
	{
		/*
		  001207 moved code from here to new function putKeys()
		*/
		if (!putKeys(symhKey))
		{
			goto errorExit;
		}
		goto normalExit;
	} /*if (symhKey != SymbolZero)*/

 
	symh = COMPVALUE(list, firstUnused);
	if (nUnused == 1 && !isKeyword(symh) &&
		isCharOrString(symh) && isscratch(NAME(symh)))
	{ /* if single quoted character string as argument*/
		name = STRINGPTR(symh);
		if (!scanPat(name,&matchType,target, topicNameCheck, TOPICNAMELENGTH))
		{
			/*
			  name contains non-wildcard character not in [a-zA-Z0-9_]
			  or was longer than TOPICNAMELENGTH
			*/
			sprintf(OUTSTR,
					"ERROR: %s() topic '%s' is %s",
					FUNCNAME, name, (matchType & PATTERNTOOLONG) ?
					"too long" : "improper");
			goto errorExit;
		}
		exactMatch = (matchType == EXACTMATCH) ? 1 : 0;
		
		if (!exactMatch && symhSubtopic != SymbolZero)
		{
			sprintf(OUTSTR,
					"ERROR: wild card characters illegal with keyword 'subtopics'");
			goto errorExit;
		}
		
		matchType |= MATCHIGNORECASE;

		for (i = 0; i < NHelpTopics;i++)
		{/* count the number of matches */
			name = *Topics + (*TopicOffsets)[i];
			if (matchName(name,matchType,target))
			{
				nTopics++;
			}
		}
		foundSomeHelp = nTopics != 0;
	} /* if (single quoted character string as argument)*/

	if (!exactMatch)
	{
		if (nTopics > 0)
		{ /* 1 argument with wild card ,  */
			if (printname)
			{
				sprintf(OUTSTR, "Topics matched on file %s", *HelpFileName);
				putOUTSTR();
			}
			listTopics(target, matchType);
			if (!printname)
			{
				putOutMsg("For help on topic foo, enter help(foo) or help(\"foo\")");
			}
			checkInterrupt(errorExit);
		}
	} /*if (nTopics > 1)*/
	else
	{ /* otherwise, print help for each argument*/
		long                karg;
		Symbolhandle        symhSub;

		for (karg = 0; karg < nargs; karg++)
		{
			long            dates[2], ndates = 0;
			char           *wantedTopic = (char *) 0;
			char           *argTopic = (char *) 0;
			char            topic[2*TOPICNAMELENGTH+1];

			symh = COMPVALUE(list, karg);
			if (symh == EmptyArgument ||
				strcmp(NAME(symh), USEDKEYWORD) == 0)
			{
				continue;
			}
			keyword = isKeyword(symh);

			symhSub = SymbolZero;
			dates[0] = dates[1] = -1;

			name = NAME(symh);
			type = TYPE(symh);
			matchType = 0;

			if (symh == symhNews)
			{
				/*
				  news:yymmdd
				  news:vector(yymmdd1, yymmdd2)
				  news:yyyymmdd
				  news:vector(yyyymmdd1, yyyymmdd2)
				  Immediately translate a date of the form yymmdd to yyyymmdd
				*/
				/* 001207 moved code to new function getDates() */
				ndates = getDates(symh, dates, silent);
				if (ndates < 0)
				{
					continue;
				}
				wantedTopic = argTopic = NEWS;
			} /*if (symh == symhNews)*/
			else if (keyword == (char *) 0)
			{
				if (isCharOrString(symh) && isscratch(name))
				{ /* quoted string */
					wantedTopic = argTopic = STRINGPTR(symh);
					if (strlen(argTopic) > TOPICNAMELENGTH)
					{
						if (!silent)
						{
							sprintf(OUTSTR,
									"WARNING: \"%s\" is too long a %s() topic name",
									argTopic, FUNCNAME);
							putErrorOUTSTR();
						}						
						continue;
					} /*if (strlen(argTopic) > TOPICNAMELENGTH)*/

					if (!isControl(argTopic) &&
						scanPat(argTopic, &matchType, target,
								topicNameCheck, TOPICNAMELENGTH))
					{ /* nTopics < 0 means 1st arg is syntax element */
						argTopic = target;
					}
					else
					{
						matchType = EXACTMATCH;
					}
					matchType |= MATCHIGNORECASE;
				}
				else if (isscratch(name))
				{
					if (strcmp(name, NULLSCRATCH) == 0)
					{ /* Topic NULL */
						if (doneNull)
						{
							continue;
						}
						doneNull = 1;
						wantedTopic = argTopic = NULLNAME;
					}
					else if (strcmp(name, LOGICSCRATCH) == 0)
					{ /*help(T) or help(F)*/
						if (doneLogic)
						{
							continue;
						}
						doneLogic = 1;
						wantedTopic = (DATAVALUE(symh, 0)) ? "T" : "F";
						argTopic = logicName;
					}
					else if (strcmp(name, NUMSCRATCH) == 0)
					{ /*e.g. help(3.12) or help(?)*/
						if (doneNumber)
						{
							continue;
						}
						doneNumber = 1;
						wantedTopic = (!isMissing(DATAVALUE(symh,0))) ?
						  NUMSCRATCH + 2 : "?";
						argTopic = numberName;
					}
					else
					{
						if (!silent)
						{
							sprintf(OUTSTR,
									"WARNING: improper argument to %s()", FUNCNAME);
							putErrorOUTSTR();
						}						
						continue;
					}
				}
				else
				{
					wantedTopic = argTopic = NAME(symh);
				}
				if (symhSubtopic != SymbolZero)
				{
					/* this must be the topic */
					symhSub = symhSubtopic;
				}
			} /*else if (keyword == (char *) 0)*/
			else
			{
				/* keyword that is not 'news'; must be topic with subtopics*/
				char       *msg = "WARNING: subtopic names for topic \"%s\" %s";
				if (doWhat == PUTUSAGE)
				{
					if (!silent)
					{
						sprintf(OUTSTR,
								"WARNING: you can't specify subtopics with when requesting usage info");
						putErrorOUTSTR();
					}					
					continue;
				} /*if (doWhat == PUTUSAGE)*/

				if (!isVector(symh) || TYPE(symh) != CHAR)
				{
					if (!silent)
					{
						sprintf(OUTSTR, msg, keyword, "not CHARACTER vector");
						putErrorOUTSTR();
					}					
					continue;
				}

				if (symhSubtopic != SymbolZero)
				{
					sprintf(OUTSTR,
							"ERROR: you can't specify more subtopics with 'subtopic:names'");
					goto errorExit;
				}

				reply = checkSubtopics(symh);
				if (!isalnum(reply) && reply != SUBQUERY)
				{
					if (!silent)
					{
						sprintf(OUTSTR, msg, keyword,
								"does not specify any legal subtopic name");
						putErrorOUTSTR();
					}					
					continue;
				}
				wantedTopic = argTopic = keyword;
				symhSub = symh;
			}

			if (strcmp(argTopic, NEWS) == 0)
			{
				if (ndates == 0 && OldestNews > 0)
				{
					long      threeMonthsEarlier;

					/*news with no dates supplied*/

					threeMonthsEarlier = (LatestNews % 10000 >= 400) ?
					  LatestNews - 300 : LatestNews - 9100;

					if (threeMonthsEarlier < OldestNews)
					{
						threeMonthsEarlier = OldestNews;
					}
					
					dates[0] = threeMonthsEarlier;
					dates[1] = LatestNews;
				} /*if (ndates == 0 && OldestNews > 0)*/
			} /*if (strcmp(argTopic, NEWS) == 0)*/
			if (firstUsage < 0)
			{
				firstUsage = karg;
			}

			if (matchType >= MATCHTYPEOFFSET)
			{
				int     nPieces = (matchType >= MATCHTYPEOFFSET) ?
				  matchType/MATCHTYPEOFFSET : 1;

				copyStrings(argTopic, topic, nPieces);
			}
#ifdef SPECIALTRANS
			else if (doWhat != PUTUSAGE && isTransformation(argTopic))
			{
				strcpy(topic, tranName);
			}
#endif /*SPECIALTRANS*/
			else
			{
				strcpy(topic, argTopic);
			}
			matchType = (matchType == 0) ?
			  (EXACTMATCH | MATCHIGNORECASE) : matchType;
			if (!(matchType & EXACTMATCH))
			{
				rewindHelp();
			}

			for (iTopic = 0; iTopic < NHelpTopics; iTopic++)
			{ /* look for first matching topic name */
				if (matchName(*Topics + (*TopicOffsets)[iTopic], matchType,
							  topic))
				{
					strcpy(target, *Topics + (*TopicOffsets)[iTopic]);
					break;
				}
			} /*for (iTopic = 0; iTopic < NHelpTopics; iTopic++)*/

#ifdef SPECIALTRANS
			if (strcmp(target, tranName) != 0 || !doneTransformation++)
#endif /*SPECIALTRANS*/
			{
				if (iTopic >= NHelpTopics)
				{
					if (!silent)
					{
						if (type == BLTIN)
						{
							what = "function";
						}
						else if (type == MACRO)
						{
							what = "macro";
						}
						else
						{
							what = "topic";
						}

						sprintf(OUTSTR, "WARNING: no help on %s '%s'",
								what, wantedTopic);
						putErrorOUTSTR();
					} /*if (!silent)*/
				} /*if (iTopic >= NHelpTopics)*/
				else
				{
#ifdef USEFSEEK
					fseek(HelpFile, (*FileOffsets)[iTopic], 0);
					BufferFull = 0;
					HelpEofHit = 0;
#else  /*USEFSEEK*/
					if (iTopic <= CurrentTopic)
					{
						rewindHelp();
					}
#endif /*USEFSEEK*/

					while (nextTopic(topic) != 0)
					{
						if (mystrncmp(topic, target, 0) == 0)
						{ /* found wanted topic */
							char       *what = (char *) 0;
							int         isNews = strcmp(topic, NEWS) == 0;
							
							if (doWhat == PUTUSAGE)
							{
								what = "Usage for ";
							}
							else if (!isNews || OldestNews == 0)
							{
								what = "Help on ";
							}
							else if ((dates[0] >= OldestNews &&
									  dates[0] <= LatestNews) ||
									 (dates[1] >= OldestNews &&
									  dates[1] <= LatestNews))
							{
								what = "News items";
							}

							if (what != (char *) 0 &&
								(printname || nUnused > 1))
							{
								char         *place = OUTSTR;
								
								sprintf(place, "%s", what);
								place += strlen(place);
								if (!isNews)
								{
									strcpy(place, topic);
									place += strlen(place);
								}
								if (printname)
								{
									sprintf(place,
											" from file \"%s\"",
											*HelpFileName);
								}
								putOUTSTR();
							}
							reply = putHelp(dates, argTopic, symhSub, doWhat);
							if (!reply)
							{
								goto errorExit;
							}
							foundSomeHelp = 1;
							break;
						} /*if (strcmp(topic,target) == 0)*/
						BufferFull = 0;
					} /*while (nextTopic(topic) != 0)*/

					if (topic[0] == '\0')
					{
						rewindHelp();
					}
					else
					{
						CurrentTopic = iTopic;
					}
				}/*if (iTopic >= NHelpTopics){}else{}*/
			}
		} /*for (karg = 0; karg < nargs; karg++)*/
	} /*if (nTopics > 1){}else{}*/

  notAvailable:
	if (NHelpTopics <= 0 && !silent)
	{
		sprintf(OUTSTR,"WARNING: Help is not available");
		if (printname)
		{
			sprintf(OUTSTR + strlen(OUTSTR)," on file \"%s\"",
					*HelpFileName);
		}
		putErrorOUTSTR();
	}
  normalExit:
	
#ifdef SCROLLABLEWINDOW
  	CURRENTSCROLLBACK = scrollback;
#endif /*SCROLLABLEWINDOW*/
	{
		Symbolhandle     result = RInstall(INVISSCRATCH, 1);
		
		if (result != SymbolError)
		{
			setTYPE(result, LOGIC);
			DATAVALUE(result, 0) = (double) foundSomeHelp;
		}
		RUNNINGHELP = (char *) 0;
		return (result);
	}
	
  duplicateKey:
	sprintf(OUTSTR, "ERROR: duplicate use of %s in %s()", keyword, FUNCNAME);
	goto errorExit;

  errorExit:
	putErrorOUTSTR();
	RUNNINGHELP = (char *) 0;

	return (SymbolError);
} /*help()*/


static void putComments(Symbolhandle macroSymh, int printName)
{
	int        foundComment;
	char      *outstr = OUTSTR, c;
	char      *name = NAME(macroSymh), *text = STRINGPTR(macroSymh);

	if (isscratch(name))
	{
		name += 2;
	}

	if (printName)
	{
		sprintf(OUTSTR, "%s:", name);
		putOUTSTR();
	}
	foundComment = 0;
	while (*text != '\0')
	{
		/* print to output all lines in macro starting with '#' */
		if (*text != '#')
		{/* skip lines that do not start with '#' */
			while (*text != '\n' && *text != '\0')
			{
				text++;
			}
		} /*if (*text != '#')*/
		else
		{
			foundComment = 1;
			outstr = OUTSTR;
			*outstr++ = ' ';
			while (*text != '\n' && *text != '\0')
			{
				if (outstr - OUTSTR >= BUFFERLENGTH - NAMELENGTH - 1)
				{
					*outstr = '\0';
					myprint(OUTSTR);
					*OUTSTR = '\0';
					outstr = OUTSTR;
				}
				c = *text++;
				if (c == '$' && text[0] == 'S' && !isnamechar(text[1]))
				{ /* expand $S */
					text++;
					strcpy(outstr, name);
					outstr += strlen(outstr);
				}
				else
				{
					*outstr++ = c;
				}
			} /*while (*text != '\n' && *text != '\0')*/
			*outstr = '\0';
			putOUTSTR();
		} /*if (*text != '#'){}else{}*/
		if (*text == '\n')
		{
			text++;
		}
	} /*while (*text != '\0')*/

	if (!foundComment)
	{
		putOutMsg(" (no comment lines found)");
	}
	*OUTSTR = '\0';
} /*putComments()*/

/*
   Function to list comments in macros

   usage:  macrousage(macro1 [, macro2 ...])
           macrousage(charVec)

   981203 Fix of minor bug in argument checking
*/

Symbolhandle macrousage(Symbolhandle list)
{
	Symbolhandle       arg;
	long               nargs = NARGS(list), n;
	long               i;
	int                foundSomeMacro = 0, silent = 0;
	WHERE("macrousage");

	*OUTSTR = '\0';

	for (i = 0; i < nargs; i++)
	{
		char         *keyword;
		
		arg = COMPVALUE(list, i);
		
		if (arg == SymbolZero && !argOK(arg, 0, (nargs > 1) ? i + 1 : 0))
		{
			goto errorExit;
		}
		keyword = isKeyword(arg);
		if (keyword != (char *) 0)
		{
			if (strcmp(keyword, "silent") == 0)
			{
				if (!isTorF(arg))
				{
					notTorF(keyword);
					goto errorExit;
				}
				silent = (DATAVALUE(arg, 0) != 0.0);
				Setname(arg, USEDKEYWORD);
			}
			else
			{
				badKeyword(FUNCNAME, keyword);
				goto errorExit;
			}
		} /*if (keyword != (char *) 0)*/
	} /*for (i = 0; i < nargs; i++)*/

	for (i = 0; i < nargs; i++)
	{
		long        iName;
		char       *isNot = "WARNING: %s is not %s";
		char       *name;

		arg = COMPVALUE(list, i);
		if (strcmp(NAME(arg), USEDKEYWORD) != 0)
		{
			if (TYPE(arg) == CHAR)
			{
				n = symbolSize(arg);
				name = STRINGPTR(arg);
				for (iName = 0; iName < n; iName++)
				{
					if (strlen(name) > NAMELENGTH)
					{
						sprintf(OUTSTR,
								"WARNING: '%s' has more than %ld characters; ignored",
								name, (long) NAMELENGTH);
					}
					else if (*name == '\0')
					{
							sprintf(OUTSTR,
								"WARNING: empty string in argument to %s()",FUNCNAME);
					}
					else
					{
						arg = Lookup(name);
						if (arg == SymbolZero)
						{
							sprintf(OUTSTR, isNot, name, "defined");
						}
						else if (TYPE(arg) != MACRO)
						{
							sprintf(OUTSTR, isNot, name, "a macro");
						}
						else
						{
							putComments(arg, n > 1 || nargs > 1);
							foundSomeMacro = 1;
						}
					}
					putOUTSTR();
					if (iName < n - 1)
					{
						name = skipStrings(name, 1);
					}
				} /*for (iName = 0; iName < n; iName++)*/
			} /*if (TYPE(arg) == CHAR)*/
			else
			{
				name = NAME(arg);
				if (TYPE(arg) == UNDEF)
				{
					sprintf(OUTSTR, isNot, name, "defined");
				}
				else if (TYPE(arg) != MACRO)
				{
					sprintf(OUTSTR, isNot, name, "a macro");
				}
				else
				{
					putComments(arg, nargs > 1);
					foundSomeMacro = 1;
				}
				if (silent)
				{
					*OUTSTR = '\0';
				}
				else
				{
					putOUTSTR();
				}
			} /*if (TYPE(arg) == CHAR){}else{}*/
		}
	} /*for (i = 0; i < nargs; i++)*/
	
  normalExit:
	{
		Symbolhandle        result = RInstall(INVISSCRATCH, 1);
		
		if (result != SymbolError)
		{
			setTYPE(result, LOGIC);
			DATAVALUE(result, 0) = (double) foundSomeMacro;
		}
		return (result);
	}
	
  errorExit:
	putErrorOUTSTR();
	return (0);
} /* macrousage()*/
