/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 5.03 or later
*(C)*
*(C)* Copyright (c) 2007 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

/*
	Contains the text of MacAnova pre-defined macros.  If this file
	is iniMacro.mac, they are in the standard form readable
	by macroread(), interspersed with C comments and pre-processor
	directives.  All temporary variables that should end in '$$' should
	have the '$$' attached as they will not be added automatically
	
	If this file is iniMacro.h, macros are as quoted strings in the format
	needed for inclusion in iniMacro.c and for processing by script
	iniMacro.awk

	The macros are used by initialize() to initialize pre-defined macros

	iniMacro.h is produced from iniMacro.mac by
	  sh iniMacro.sh iniMacro.mac iniMacro.h

	990305 File created with comment lines and preprocessor directives
	       from iniMacro.c.
    Further history comments follow line containing INIMACROHISTORY below
*/

#if 0
   In the following quoted comments are deliminated by /@..@/.

   This file must be in a particular format to allow creation of a MPW rez
   file using script iniMacro.awk .  If additional macros are added they
   should follow this format.

   The script copies the copyright information and exctracts the values of
   macros MACROTYPE, ENDNAME, IDBASE and NAMEPREFIX (see above) and then
   ignores everything up through the line '/@STARTMACROS@/'.  It ignores all
   other  preprocessor commands.

   The quoted name of each macro followed by a comma with no intervening
   whitespace must be the second field of any line line whose first field
   is '/@MNAME@/'.

   The quoted text of a macro must start with a line whose first field is
   '/@MTEXT@/'. All lines except the last should end in \

   The final line of a macro must terminate with '", /@MEND@/', where '"'
   is the terminating quote for the macro and there must be no intervening
   whitespace before the comma.  For a one line macro this would be on
   the same line with '/@MTEXT@/'

   The script skips any lines bracketed by lines whose first fields are
   '/@NOTFORMACSTART@/' and '/@NOTFORMACEND@/'.

   ('@' is used here instead of '#' because the MPW C preprocessor got
   confused otherwise.)

   Preprocessor lines are not copied

   Scanning terminates when the line containing '/@ENDMACROS@/' is found

   961104 Slightly modified many macros, replacing $1, $2, ... by $01, $02
   which will mean less memory needed for expansion and less scanning by
   yylex when expanded.

   980730 Changed type of component 'text' to char * from unsigned char *
#endif /*if 0*/

/*
   INIMACROHISTORY  (recognized by iniMacro.awk in creating macInimacro.r)
   971108 added new versions of resid() and yhat() which label rows and
          columns
   971111 added new pre-defined macro regcoefs()
   980102 Modified makecols() to allow final argument nomissing:T
   980710 Replaced anytrue() and alltrue() with versions that use evaluate
   980727 Added hasnotes()
   981120 Added nofileok:T to macroread() calls in getmacros()
   981121 Deleted all use of "ERROR: " in macro calls to error()
   990309 Minor modification of several macros and addition of comments
		  Deleted fcolplot() and frowplot()
   990324 Fixed bug in rowplot()
   990413 Updated twotailt to use argvalue()
   990823 Updated tek and vt to use new option 'tekset'
   990929 Added macros designhelp, tserhelp, arimahelp and _gethelp
   991005 New version of getmacros() that works with quoted macro names
		  as well as unquoted
   991012 New version of regs() allowing GLM keywords
   991108 New version of getdata() allowing quoted data set name
   991208 Modified makefactor() to allow makefactor(x,sort:F)
   000210 Add keyword 'stripdol' to edit() and more()
   000217 Modified regs()
   000228 cleaned up some uses of argvalue() and keyvalue()
   000229 Modified _gethelp() so _gethelp() is equivalent to
          _gethelp(index:T)
   000504 removed alltrue() and anytrue() to make way for functions
          with these names
   000505 Certain macros are now specified as out-of-line.  Supporting scripts
		  and iniMacro.c have been changed to ensure that they are
		  installed as out-of-line macros.  The 'trick' is that
		  iniMacro.sh prepends '%' to start of out-of-line macro names
   000508 Updated toclip() to have better keyword handling
   000510 Updated several macros including readcols() and makecols()
   000520 Replaced hist() and readcols(), updated other macros
   000524 More enhancements of hist()
   000601 Modified readcols() to handle vecread() keyword 'bypass'
   000614 Modified readcols() to process keyword phrase 'realorchar:T'
   000616 Modified makefactor() to label output and process
		  'labels:F'
   000824 Modified hist() to recognize 'save:T' and 'draw:T'
   000901 Added macro readdata()
   000904 Added macros graphicshelp(), mathhelp(), mulvarhelp(),
          regresshelp()
   000927 Modified _gethelp() to recognize keyword 'key'
   001002 Added macro userfunhelp()
   001110 Added macro clipreaddata(); did a little reformatting
   001130 Fixed bug in readdata() affecting reading of data files with
          empty lines
   001213 Modified _gethelp() to cope with subtopics in alternative help
		  files
   001226 Keyword 'n' made illegal in readdata(); previously it was ignored
   010104 Updated _gethelp() to allow argument news:dates like help()
   010207 Fixed bug in readdata()
   010316 Fixed bug in readdata()
   010324 Updated getmacros() to use new keyword 'printname' and to
          use getfilename(last:T)
   010327 Minor bugfix in getmacros()
   010618 Added addhelpfile() and gethelp()
   010619 Rename gethelp() to help() and added usage(); changed gethelp()
          calls to help() in arimahelp(), designhelp(), ... 
   010620 Added 'allfiles:T' to help() and usage() to allow searching all
          files and not stopping at the first one where a topic is found
   010703 addhelpfile() now updates predefined vector HELPINDICES as
          well as HELPFILES
          help(index:helpfilename) prints an annotated index of topics in
          help file helpfilename
   010724 resvsrankits(), resvsindex(), resvsyhat() have new keyword
          usehii:T or usehii:F, default plotting symbol "\6", and
          revised default axis labels and title.  resvsrankits() and
          resvsindex() work after arima()
		  regs(), yhat() and resid() refreshed from edited originals in
		  regress.mac
   010725 resvsrankits(), resvsindex(), resvsyhat() now differ only
          in their names and are front ends to new invisible macro
		  _resvsxxxx
   010806 Fixed bug in readcols() so it prints file name at most once
   011029 Modified makefactor() and added writedata() and clipwritedat()
   020111 Modified readdata() so it prints the number of missing values
          in a variable created
   021022 Modified colplot() and rowplot() so that x-axis tick marks are
          always equally spaced integers
          Replaced makecols() by newversion which recognizes keyword
          'factors'
          Replaced twotailt(), boxcox() with functionally equivalent
          versions
   021111 Value of makecols() keyword 'factors' can be integer vector
   021128 Fixed bug in makecols()
   030113 added keyword 'sort to readdata() (inherited by clipreaddata())
          addmacrofile() can be used as addmacrofile("") windowed
          systems
   031008 keyword 'xticks' now recognized by colplot() and rowplot()
   031028 Fixed bug in _resvsxxxx() that had been fixed in regress.mac
          on 011212 but never fixed here.
   041213 Fixed minor bugs in readcols() and readdata()
   060809 Labels retained in twotailt()
          Removed macro model()
   071109 Updated twotailt() (bug fix)
*/

#if defined(UNIX) && !defined(BASE_UNIX)
# define BASE_UNIX
#endif

#ifdef MACINTOSH
#include "macIface.h"

#define MACROTYPE   'wstr'
#define ENDNAME     "_END_"
#define IDBASE      10000
#define NAMEPREFIX  "_MV_"
#endif /*MACINTOSH*/

/* Start_Of_Macros */

/*STARTMACROS*/

/*LOCKED*/

/*MNAME*/ "&redo",
/*MTEXT*/ "REDO <- macro({if($N==0){LASTLINE}else{$01}})\n\
REDO()", /*MEND*/

#ifndef BCPP /* redo is only pre--defined macro in BC version */

/*MNAME*/ "twotailt",
/*MTEXT*/ "# $S(tval,df)\n\
@tval$$ <- argvalue($1,\"argument 1\",\"real nonmissing\")\n\
@df$$ <- argvalue($2,\"degrees of freedom\",\"positive\")\n\
if(!isscalar(@tval$$) && !isscalar(@df$$)){\n\
\t@ok$$ <- if(ndims(@tval$$) != ndims(@df$$)){\n\
\t\tF\n\
\t}else{\n\
\t\tsum(dim(@tval$$) != dim(@df$$)) == 0\n\
\t}\n\
\tif(!delete(@ok$$,return:T)){\n\
\t\terror(\"t-value and df not scalars and not identical shapes\")\n\
\t}\n\
}\n\
@labs$$ <- if(haslabels(@tval$$)){\n\
\tgetlabels(@tval$$)\n\
} else {\n\
\tNULL\n\
}\n\
array(2*cumstu(abs(delete(@tval$$,return:T)),delete(@df$$,return:T),upper:T),\\\n\
\t   labels:delete(@labs$$,return:T))", /*MEND*/


/*MNAME*/ "regcoefs",
/*MTEXT*/ "# $S([pvals:T][,byvar:T]) or $S(model[,pvals:T][,byvar:T])\n\
if ($v > 1 || $k > 2){\n\
\terror(\"usage: $S([pvals:T][,byvar:T]) or $S(model[,pvals:T][,byvar:T])\",\\\n\
\t\tmacroname:F)\n\
}\n\
@pvals$$ <- keyvalue($K,\"pval*\",\"TF\",default:getoptions(pvals:T))\n\
@byvar$$ <- keyvalue($K,\"byvar*\",\"TF\")\n\
if ($v == 1){\n\
\t@model$$ <- argvalue($1,\"model\",\"string\")\n\
\tmanova(delete(@model$$, return:T),silent:T)\n\
}\n\
@p$$ <- nrows(SS[1,,])\n\
@mterms$$ <- dim(SS)[1]\n\
if (isnull(@byvar$$)){\n\
\t@byvar$$ <- (@p$$ != 1)\n\
}elseif(@p$$ == 1 && @byvar$$){\n\
\tprint(\"WARNING: byvar:T ignored by $S with univariate response variable\")\n\
\t@byvar$$ <- F\n\
}\n\
@stuff$$ <- secoefs(byterm:F)\n\
if (max(vector(length(@stuff$$[1]))) > @p$$){\n\
\terror(\"$S does not allow factors in the model\", macroname:F)\n\
}\n\
if (isstruc(@stuff$$[1])){\n\
\t@rowlabels$$ <- compnames(@stuff$$[1])\n\
}else{\n\
\t@rowlabels$$ <- TERMNAMES[1]\n\
}\n\
@collabels$$ <- if (@p$$ == 1 || @byvar$$){\n\
\tvector(\"Coef\",\"StdErr\",\"t\")\n\
}else{\n\
\t@makelabs$$ <- macro(\"getlabels(vector(run(\\\\$1),labels:\\\\$2))\",inline:F)\n\
\tvector(@makelabs$$(@p$$,\"Coef \"),\\\n\
\t@makelabs$$(@p$$,\"StdErr \"),@makelabs$$(@p$$,\"t \"))\n\
}\n\
if (DF[@mterms$$] == 0){\n\
\t@pvals$$ <- F\n\
}\n\
if (@pvals$$){\n\
\t@collabels$$ <- if (@p$$ == 1 || @byvar$$){\n\
\t\tvector(@collabels$$,\"P-Value\")\n\
\t}else{\n\
\t\tvector(@collabels$$,@makelabs$$(@p$$,\"P-Value \"))\n\
\t}\n\
}\n\
delete(@makelabs$$,silent:T)\n\
@coefs$$ <- vector(@stuff$$[1])\n\
@stderrs$$ <-  vector(@stuff$$[2])\n\
if (@p$$ > 1){\n\
\t@coefs$$ <- matrix(@coefs$$,@p$$)'\n\
\t@stderrs$$ <- matrix(@stderrs$$,@p$$)'\n\
}\n\
@tstats$$ <- if (DF[@mterms$$] > 0){\n\
\t@coefs$$/@stderrs$$\n\
}else{\n\
\t@pvals$$ <- F\n\
\tarray(rep(?,(@mterms$$-1)*@p$$),dim(@coefs$$))\n\
}\n\
if (!@byvar$$ || @p$$ == 1){\n\
\t@o$$ <- hconcat(@coefs$$,@stderrs$$,@tstats$$)\n\
\tif (@pvals$$){\n\
\t\t@o$$ <- hconcat(@o$$,2*cumstu(abs(@tstats$$),DF[@mterms$$],upper:T))\n\
\t}\n\
\t@o$$ <- matrix(@o$$,labels:structure(@rowlabels$$,@collabels$$))\n\
}else{\n\
\t@o$$ <- split(rep(0,@p$$)', compnames:\"Variable_\") #create structure\n\
\tfor (@i$$,run(@p$$)){\n\
\t\t@oi$$ <- hconcat(@coefs$$[,@i$$],@stderrs$$[,@i$$],@tstats$$[,@i$$])\n\
\t\tif (@pvals$$){\n\
\t\t\t@oi$$ <- hconcat(@oi$$,\\\n\
\t\t\t\t2*cumstu(abs(@tstats$$[,@i$$]),DF[@mterms$$],upper:T))\n\
\t\t}\n\
\t\t@o$$[@i$$] <- matrix(@oi$$,labels:structure(@rowlabels$$,@collabels$$))\n\
\t}\n\
\tdelete(@oi$$,@i$$)\n\
}\n\
delete(@stuff$$,@p$$,@mterms$$,@pvals$$,@byvar$$,@coefs$$,@stderrs$$,@tstats$$)\n\
delete(@o$$,return:T)", /*MEND*/


/*MNAME*/ "_resvsxxxx",
/*MTEXT*/ "#$S(which, usehii, standres ,var ,C [,graphics keywords])\n\
@name$$ <- \"$1\"\n\
@usehii$$ <- $2\n\
@standres$$ <- $3\n\
\n\
if (match(\"resvsindex*\",@name$$,0,exact:F) != 0){\n\
\t@which$$ <- 1\n\
\t@xname$$ <- \"Case Numbers\"\n\
} elseif (match(\"resvsrankit*\",@name$$,0,exact:F) != 0) {\n\
\t@which$$ <- 2\n\
\t@xname$$ <- \"Normal Scores\"\n\
} elseif (match(\"resvsyhat*\",@name$$,0,exact:F) != 0) {\n\
\t@which$$ <- 3\n\
\t@xname$$ <- \"Fitted Values (Yhat)\"\n\
} else {\n\
\terror(paste(\"\\\"\",@name$$,\"\\\" improper first argument\",sep:\"\"))\n\
}\n\
@name$$ <- paste(@name$$,\"()\",sep:\"\")\n\
\n\
@var$$ <- $4\n\
\n\
if (!isscalar(@var$$,positive:T,integer:T)){\n\
\treturn(\"Argument 1 not positive integer\")\n\
}\n\
\n\
@chars$$ <- $5 # already checked\n\
\n\
if (!isdefined(RESIDUALS)){\n\
\treturn(\"Apparently no linear, GLM or ARIMA model has been fitted\")\n\
}\n\
\n\
@n$$ <- nrows(RESIDUALS)\n\
if (alltrue(isdefined(ALLRESIDUALS),nrows(ALLRESIDUALS) >= @n$$)){\n\
\t@J$$ <- run(nrows(ALLRESIDUALS) - @n$$ + 1, nrows(ALLRESIDUALS))\n\
\t@arma$$ <- sum(RESIDUALS != ALLRESIDUALS[@J$$]) == 0\n\
\tdelete(@J$$)\n\
} else {\n\
\t@arma$$ <- F\n\
}\n\
\n\
if (@var$$ > ncols(RESIDUALS)) {\n\
\treturn(\"argument 1 > ncols(RESIDUALS)\")\n\
}\n\
\n\
if (isnull(@usehii$$)) {\n\
\t@usehii$$ <- alltrue(!@arma$$, @standres$$, isvector(HII,real:T,nonneg:T))\n\
} elseif (@usehii$$ && !@standres$$) {\n\
\tprint(paste(\"WARNING: usehii:T ignored with stand:F in\",@name$$))\n\
\t@usehii$$ <- F\n\
}\n\
\n\
if (@usehii$$ && anytrue(!isvector(HII,real:T,nonneg:T),length(HII) != @n$$)){\n\
\treturn(\"no HII or HII is not a REAL vector => 0 of right length\")\n\
}\n\
\n\
if (@arma$$){\n\
\tif (@which$$ == 3){\n\
\t\treturn(paste(@name$$, \"can't be used after arima()\"))\n\
\t}\n\
\tif (@standres$$) {\n\
\t\tif (!isdefined(NPAR)){\n\
\t\t\treturn(\"Need NPAR to be defined after arima() with standres:T\")\n\
\t\t}\n\
\t\t@sd$$ <- sqrt(sum(RESIDUALS^2)/(@n$$ - NPAR))\n\
\t}\n\
} elseif (@standres$$) {\n\
\tif(!isdefined(DF) || !isdefined(SS)) {\n\
\t\treturn(\"SS and/or DF not defined.  No linear model fitted?\")\n\
\t}\n\
\t@m$$ <- length(DF)\n\
\tif (DF[@m$$] == 0){\n\
\t\treturn(\"no degrees of freedom for error; can't standardize\")\n\
\t}\n\
\t@sd$$ <- sqrt(SS[@m$$,@var$$,@var$$]/DF[@m$$])\n\
\tdelete(@m$$)\n\
}\n\
\n\
if (isnull(@chars$$)) {\n\
\t@chars$$ <- \"\\6\"\n\
} elseif (alltrue(isscalar(@chars$$, real:T), @chars$$ == 0)){\n\
\t@chars$$ <- \"###\"\n\
}\n\
\n\
@r$$ <- if(!@arma$$ && isdefined(WTDRESIDUALS)){\n\
\tWTDRESIDUALS[,@var$$]\n\
}else{\n\
\tRESIDUALS[,@var$$]\n\
}\n\
\n\
@w$$ <- getoptions(warnings:T)\n\
setoptions(warnings:F)\n\
\n\
@kind$$ <- if (@standres$$) {\n\
\tif (@usehii$$) {\n\
\t\t@sd$$ <-* sqrt(1 - HII)\n\
\t}\n\
\t@r$$ <-/ @sd$$\n\
\tdelete(@sd$$,@usehii$$)\n\
\t\"Standardized\"\n\
} else {\n\
\t\"\"\n\
}\n\
\n\
@x$$ <- if (@which$$ == 1) { #resvsindex\n\
\t1\n\
} elseif (@which$$ == 2) { # resvsrankits\n\
\trankits(@r$$)\n\
} else { # resvsyhat\n\
\tmodelvars(y:T)[,@var$$] - RESIDUALS[,@var$$]\n\
}\n\
setoptions(warnings:@w$$)\n\
\n\
chplot(delete(@x$$,return:T),@r$$,symbols:@chars$$,$K,yaxis:F,\\\n\
\ttitle:paste(@kind$$,\"Residuals vs\",@xname$$),\\\n\
\txlab:@xname$$,ylab:paste(@kind$$,\"Resids\"))\n\
delete(@w$$,@var$$,@r$$,@chars$$,@kind$$,@standres$$)", /*MEND*/


/*MNAME*/ "resvsindex",
/*MTEXT*/ "#$S([usehii:T or F] [,standres:T][,graphics keywords]) or $S(var [,keywords])\n\
\n\
if (!ismacro(_resvsxxxx)){\n\
\tgetmacros(_resvsxxxx,silent:T)\n\
}\n\
\n\
@standres$$ <- keyvalue($K,\"stand*\",\"TF\",default:T)\n\
@usehii$$ <- keyvalue($K,\"usehii\",\"TF\")\n\
\n\
@chars$$ <- keyvalue($K,\"symbol*\",\"vector\")\n\
\n\
@var$$ <- if($v == 0){\n\
\t1\n\
}else{\n\
\targvalue($1, \"argument 1\", \"positive integer scalar\")\n\
}\n\
\n\
if ($v > 2) {\n\
\terror(\"at most 2 non-keyword arguments allowed\")\n\
} elseif ($v == 2){ #resvsindex(i,c) allowed for backward compatibility\n\
\tif (!isnull(@chars$$)){\n\
\t\terror(\"at most 1 non-keyword argument with 'symbols'\")\n\
\t}\n\
\t@chars$$ <- argvalue($2, \"argument 2\", \"vector\")\n\
\tif (!ischar(@chars$$) && !isreal(@chars$$)){\n\
\t\terror(\"argument 2 not REAL or CHARACTER\")\n\
\t}\n\
}elseif (!isnull(@chars$$) && !isreal(@chars$$)  && !ischar(@chars$$)){\n\
\terror(\"value of 'symbols' not REAL or CHARACTER\")\n\
}\n\
\n\
@msg$$ <- _resvsxxxx($S, @usehii$$, @standres$$, @var$$, @chars$$, $K)\n\
if (!isnull(@msg$$)){\n\
\terror(@msg$$, macroname:match(\"resvs*\",@msg$$,0,exact:F) == 0)\n\
}\n\
delete(@usehii$$,@standres$$,@var$$,@chars$$,@msg$$)", /*MEND*/


/*MNAME*/ "resvsrankits",
/*MTEXT*/ "#$S([usehii:T or F] [,standres:T][,graphics keywords]) or $S(var [,keywords])\n\
\n\
if (!ismacro(_resvsxxxx)){\n\
\tgetmacros(_resvsxxxx,silent:T)\n\
}\n\
\n\
@standres$$ <- keyvalue($K,\"stand*\",\"TF\",default:T)\n\
@usehii$$ <- keyvalue($K,\"usehii\",\"TF\")\n\
\n\
@chars$$ <- keyvalue($K,\"symbol*\",\"vector\")\n\
\n\
@var$$ <- if($v == 0){\n\
\t1\n\
}else{\n\
\targvalue($1, \"argument 1\", \"positive integer scalar\")\n\
}\n\
\n\
if ($v > 2) {\n\
\terror(\"at most 2 non-keyword arguments allowed\")\n\
} elseif ($v == 2){ #resvsindex(i,c) allowed for backward compatibility\n\
\tif (!isnull(@chars$$)){\n\
\t\terror(\"at most 1 non-keyword argument with 'symbols'\")\n\
\t}\n\
\t@chars$$ <- argvalue($2, \"argument 2\", \"vector\")\n\
\tif (!ischar(@chars$$) && !isreal(@chars$$)){\n\
\t\terror(\"argument 2 not REAL or CHARACTER\")\n\
\t}\n\
}elseif (!isnull(@chars$$) && !isreal(@chars$$)  && !ischar(@chars$$)){\n\
\terror(\"value of 'symbols' not REAL or CHARACTER\")\n\
}\n\
\n\
@msg$$ <- _resvsxxxx($S, @usehii$$, @standres$$, @var$$, @chars$$, $K)\n\
if (!isnull(@msg$$)){\n\
\terror(@msg$$, macroname:match(\"resvs*\",@msg$$,0,exact:F) == 0)\n\
}\n\
delete(@usehii$$,@standres$$,@var$$,@chars$$,@msg$$)", /*MEND*/


/*MNAME*/ "resvsyhat",
/*MTEXT*/ "#$S([usehii:T or F] [,standres:T][,graphics keywords]) or $S(var [,keywords])\n\
\n\
if (!ismacro(_resvsxxxx)){\n\
\tgetmacros(_resvsxxxx,silent:T)\n\
}\n\
\n\
@standres$$ <- keyvalue($K,\"stand*\",\"TF\",default:T)\n\
@usehii$$ <- keyvalue($K,\"usehii\",\"TF\")\n\
\n\
@chars$$ <- keyvalue($K,\"symbol*\",\"vector\")\n\
\n\
@var$$ <- if($v == 0){\n\
\t1\n\
}else{\n\
\targvalue($1, \"argument 1\", \"positive integer scalar\")\n\
}\n\
\n\
if ($v > 2) {\n\
\terror(\"at most 2 non-keyword arguments allowed\")\n\
} elseif ($v == 2){ #resvsindex(i,c) allowed for backward compatibility\n\
\tif (!isnull(@chars$$)){\n\
\t\terror(\"at most 1 non-keyword argument with 'symbols'\")\n\
\t}\n\
\t@chars$$ <- argvalue($2, \"argument 2\", \"vector\")\n\
\tif (!ischar(@chars$$) && !isreal(@chars$$)){\n\
\t\terror(\"argument 2 not REAL or CHARACTER\")\n\
\t}\n\
}elseif (!isnull(@chars$$) && !isreal(@chars$$)  && !ischar(@chars$$)){\n\
\terror(\"value of 'symbols' not REAL or CHARACTER\")\n\
}\n\
\n\
@msg$$ <- _resvsxxxx($S, @usehii$$, @standres$$, @var$$, @chars$$, $K)\n\
if (!isnull(@msg$$)){\n\
\terror(@msg$$, macroname:match(\"resvs*\",@msg$$,0,exact:F) == 0)\n\
}\n\
delete(@usehii$$,@standres$$,@var$$,@chars$$,@msg$$)", /*MEND*/


/*MNAME*/ "hist",
/*MTEXT*/ "# usage: $S(x,vector(anchor, width)[,keywords]), $S(x,nbins[,keywords])\n\
#        $S(x,edges [,outsideok:T] [,keywords]), or $S(x[,keywords])\n\
if($v < 1 || $v > 2){\n\
\terror(\"usage: $S(x), $S(x,nbins), $S(x,edges) or $S(x,vector(anchor,width)\",\\\n\
\t\t macroname:F)\n\
}\n\
@x$$ <- argvalue($1, \"$1\", \"real vector\")\n\
if(anymissing(@x$$)){\n\
\t@x$$ <- @x$$[!ismissing(@x$$)]\n\
\tif (isnull(@x$$)){\n\
\t\terror(\"all values of $1 are MISSING\")\n\
\t}\n\
}\n\
if (alltrue(ndims(@x$$) == 2, !isnull(getlabels(@x$$,silent:T)))) {\n\
\t@xname$$ <- getlabels(@x$$,2)\n\
\tif (match(\"@*\",@xname$$,0,exact:F) != 0){\n\
\t\t@xname$$ <- \"$1\"\n\
\t}\n\
} else {\n\
\t@xname$$ <- \"$1\"\n\
}\n\
\n\
@keys$$ <- if ($k > 0) {\n\
\tstructure($K,notakey:NULL,silent:T)\n\
} else {\n\
\tstructure(notakey:NULL)\n\
}\n\
@keys1$$ <- keyvalue(@keys$$,\"key*\",\"structure\")\n\
if (!isnull(@keys1$$)){\n\
\tif ($k > 1) {\n\
\t\terror(\"you can't use other keywords with 'keys'\")\n\
\t}\n\
\t@keys$$ <- @keys1$$\n\
}\n\
delete(@keys1$$)\n\
\n\
@leftin$$ <- keyvalue(@keys$$,\"leftendin\",\"TF\",default:F)\n\
@freq$$ <- keyvalue(@keys$$, \"freq*\", \"TF\", default:F)\n\
@relfreq$$ <- keyvalue(@keys$$, \"relfreq*\", \"TF\", default:F)\n\
if (@freq$$ && @relfreq$$){\n\
\terror(\"both 'freq:T' and 'relfreq:T' illegal\")\n\
}\n\
@outsideok$$ <- keyvalue(@keys$$,\"outsideok\",\"TF\",default:F)\n\
@save$$ <- keyvalue(@keys$$,\"save\",\"TF\",default:F)\n\
@draw$$ <- keyvalue(@keys$$,\"draw\",\"TF\",default:!@save$$)\n\
if (!@save$$ && !@draw$$) {\n\
\terror(\"you can't use 'draw:F' with 'save:F'\")\n\
}\n\
\n\
@keynames$$ <- compnames(@keys$$)\n\
@exclkeys$$ <- vector(\"leftendin\",\"freq*\",\"relfreq*\",\"outsideok\",\"draw\",\"save\")\n\
@Jexcl$$ <- match(@exclkeys$$,@keynames$$,0,exact:F)\n\
@Jexcl$$ <- @Jexcl$$[@Jexcl$$ > 0]\n\
if (!isnull(@Jexcl$$)){\n\
\t@keys$$ <- if (length(@Jexcl$$) < ncomps(@keys$$)){\n\
\t\tstrconcat(@keys$$[-@Jexcl$$],compnames:@keynames$$[-@Jexcl$$])\n\
\t} else {\n\
\t\tstructure(notakey:T)\n\
\t}\n\
}\n\
delete(@Jexcl$$,@exclkeys$$,@keynames$$)\n\
\n\
@xmin$$ <- min(@x$$)\n\
@xmax$$ <- max(@x$$)\n\
\n\
if($v == 2){\n\
\t@nbins$$ <- argvalue($2, \"$2\", \"real nonmissing\")\n\
\tif(!isvector(@nbins$$)){\n\
\t\terror(\"$2 is not a REAL scalar or vector\")\n\
\t}\n\
\t@nbins$$ <- if (isscalar(@nbins$$)){\n\
\t\t@bins$$ <- NULL\n\
\t\targvalue(@nbins$$, \"number of bars\", \"positive integer scalar\")\n\
\t}elseif(length(@nbins$$) > 2){\n\
\t\t@bins$$ <- @nbins$$\n\
\t\tlength(@bins$$) - 1\n\
\t}else{\n\
\t\t# arg 2 of form vector(anchor, width)\n\
\t\t@anchor$$ <- @nbins$$[1]\n\
\t\t@w$$ <- @nbins$$[2]\n\
\t\tif (@w$$ <= 0){\n\
\t\t\terror(\"bar width <= 0\")\n\
\t\t}else{\n\
\t\t\t@anchor$$ <-- ceiling((@anchor$$ - @xmin$$)/@w$$)*@w$$\n\
\t\t\tif (@anchor$$ > @xmin$$ || @anchor$$ == @xmin$$ && !@leftin$$){\n\
\t\t\t\t@anchor$$ <-- @w$$\n\
\t\t\t}\n\
\t\t\t@bins$$ <- run(@anchor$$, @xmax$$, @w$$)\n\
\t\t\t@maxbins$$ <- max(@bins$$)\n\
\t\t\tif (@maxbins$$ < @xmax$$ || @maxbins$$ == @xmax$$ && @leftin$$){\n\
\t\t\t\t@bins$$ <- vector(@bins$$,@maxbins$$ + @w$$)\n\
\t\t\t}\n\
\t\t\tdelete(@w$$, @anchor$$, @maxbins$$)\n\
\t\t\t@nbins$$ <- length(@bins$$) - 1\n\
\t\t}\n\
\t}\n\
} else{\n\
\t@nbins$$ <- floor(log2(length(@x$$))) + 1\n\
\t@bins$$ <- NULL\n\
}\n\
if(isnull(@bins$$)){\n\
\t@min$$ <- 1.05*@xmin$$ - .05*@xmax$$\n\
\t@max$$ <- 1.05*@xmax$$ - .05*@xmin$$\n\
\t@widths$$ <- (@max$$ - @min$$)/@nbins$$\n\
\t@bins$$ <- run(@min$$,@max$$+.1*@widths$$,@widths$$)\n\
\tdelete(@min$$, @max$$)\n\
} else {\n\
\t@minbins$$ <- min(@bins$$)\n\
\t@maxbins$$ <- max(@bins$$)\n\
\t@nleft$$ <- sum(@x$$ < @minbins$$)\n\
\t@nright$$ <- sum(@x$$ > @maxbins$$)\n\
\tif (@leftin$$){\n\
\t\t@nright$$ <-+ sum(@x$$ == @maxbins$$)\n\
\t}else{\n\
\t\t@nleft$$ <-+ sum(@x$$ == @minbins$$)\n\
\t}\n\
\n\
\tif (@nleft$$ + @nright$$ > 0){\n\
\t\tif (!@outsideok$$){\n\
\t\t\terror(paste(\"data min =\",@xmin$$, \"or max =\",@xmax$$,\\\n\
\t\t\t\t\"outside of bars\"))\n\
\t\t}\n\
\t\t@msg$$ <- \"WARNING:\"\n\
\t\tif (@nleft$$ > 0){\n\
\t\t\t@msg$$ <- paste(@msg$$,@nleft$$,\"value(s) omitted on left\")\n\
\t\t\tif (@nright$$ > 0){\n\
\t\t\t\t@msg$$ <- paste(@msg$$,\"and\")\n\
\t\t\t}\n\
\t\t}\n\
\t\tif (@nright$$ > 0){\n\
\t\t\t@msg$$ <- paste(@msg$$,@nright$$,\"value(s) ommitted on right\")\n\
\t\t}\n\
\t\tprint(delete(@msg$$,return:T))\n\
\t}\n\
\tdelete(@minbins$$,@maxbins$$,@nleft$$,@nright$$)\n\
}\n\
delete(@xmax$$,@xmin$$)\n\
\n\
@widths$$ <- movavg(1,@bins$$)[-1]\n\
if(min(@widths$$) < 0){\n\
\terror(\"bin boundaries are not monotonic\")\n\
}\n\
\n\
@n$$ <- length(@x$$)\n\
@denom$$ <- if (@freq$$ || @relfreq$$){\n\
\tif (describe(@widths$$, stddev:T) > 1e-5 * sum(@widths$$)){\n\
\t\tprint(\"WARNING: 'freq:T' or 'relfreq:T' used with unequal class widths\",\\\n\
\t\t\tmacroname:T)\n\
\t}\n\
\tif (@freq$$){\n\
\t\t1\n\
\t}else{\n\
\t\t@n$$\n\
\t}\n\
}else{\n\
\t@widths$$*@n$$\n\
}\n\
\n\
@warning$$ <- getoptions(warnings:T)\n\
setoptions(warnings:F)\n\
@hts$$ <- bin(delete(@x$$, return:T), @bins$$,\\\n\
\tleftendin:delete(@leftin$$,return:T))$counts/@denom$$\n\
setoptions(warnings:delete(@warning$$,return:T))\n\
\n\
@xpts$$ <- rep(@bins$$,rep(3, @nbins$$+1))[-1]\n\
@ypts$$ <- vector(0,hconcat(@hts$$,@hts$$,0*@hts$$)',0)\n\
if (delete(@draw$$,return:T)){\n\
\t@ylab$$ <- if(@freq$$) {\n\
\t\t\"Frequency\"\n\
\t} elseif(@relfreq$$) {\n\
\t\t\"Relative frequency\"\n\
\t} else {\n\
\t\t\"Density\"\n\
\t}\n\
\n\
\t@title$$ <- if(@freq$$) {\n\
\t\tpaste(\"Frequency histogram of\",@xname$$)\n\
\t} elseif(@relfreq$$) {\n\
\t\tpaste(\"Relative frequency histogram of\",@xname$$)\n\
\t} else {\n\
\t\tpaste(\"Histogram of\", @xname$$, \"with total area 1\")\n\
\t}\n\
\tlineplot(@xpts$$,@ypts$$,\\\n\
\t\tkeys:strconcat(@keys$$,xlab:@xname$$,ylab:@ylab$$,title:@title$$))\n\
\tdelete(@ylab$$,@title$$)\n\
}\n\
delete(@bins$$,@nbins$$,@hts$$,@widths$$,@denom$$,@xname$$)\n\
\n\
if (delete(@save$$,return:T)) {\n\
\t@result$$ <- structure(x:@xpts$$,y:@ypts$$,lines:T)\n\
\tif (ncomps(@keys$$) > 1) {\n\
\t\t@keynames$$ <- compnames(@keys$$)\n\
\t\t@J$$ <- match(\"notakey\",@keynames$$,0)\n\
\t\t@keys$$ <- strconcat(@keys$$[-@J$$],compnames:@keynames$$[-@J$$])\n\
\t\t@result$$ <- strconcat(@result$$,@keys$$)\n\
\t\tdelete(@J$$,@keynames$$)\n\
\t}\n\
} else {\n\
\t@result$$ <- NULL\n\
}\n\
delete(@xpts$$,@ypts$$,@keys$$)\n\
delete(@result$$,return:T)", /*MEND*/


/*MNAME*/ "vboxplot",
/*MTEXT*/ "#usage: $S(x1,x2, ... [,keywords]  or $S(struct [,keywords])\n\
if ($v == 0){\n\
\terror(\"no data provided to $S\")\n\
}\n\
if($k>0){\n\
\tboxplot($V,$K,vertical:T)\n\
}else{\n\
\tboxplot($V,vertical:T)\n\
}", /*MEND*/


/*MNAME*/ "boxcox",
/*MTEXT*/ "# usage: $S(x,power), x a vector or matrix, power a scalar\n\
@x$$ <- argvalue($1,\"argument 1\", \"real\")\n\
@power$$ <- argvalue($2,\"power argument\",\"number\")\n\
@dims$$ <- dim(@x$$)\n\
\n\
if (min(vector(sum(!ismissing(@x$$)))) == 0){\n\
\terror(\"argument 1 is all MISSING or has an all MISSING column\")\n\
}\n\
if (min(@x$$[vector(!ismissing(@x$$))]) <= 0){\n\
\terror(\"argument 1 has zero or negative values\")\n\
}\n\
if (ndims(@x$$) > 2){\n\
\t@x$$ <- matrix(vector(@x$$),@dims$$[1])\n\
}\n\
@w$$ <- getoptions(warnings:T)\n\
setoptions(warnings:F)\n\
@gm$$ <- exp(describe(log(@x$$),mean:T))'\n\
@x$$ <- if ( @power$$ == 0 ) {\n\
\t@gm$$ * log(@x$$)\n\
} else {\n\
\t(@x$$^@power$$ - 1)/(@power$$*@gm$$^(@power$$-1))\n\
}\n\
setoptions(warnings:delete(@w$$,return:T))\n\
delete(@gm$$, @power$$)\n\
array(delete(@x$$,return:T),delete(@dims$$,return:T))", /*MEND*/


/*MNAME*/ "makefactor",
/*MTEXT*/ "# $S(values [,sort:F] [,labels:F or F]) where values is a vector\n\
if($v < 1 || $v > 2 || $k > 2){\n\
\terror(\"usage: $S(vec [, sort:T] [, labels:F])\",macroname:F)\n\
}\n\
@values$$ <- argvalue($1, \"argument 1\", \"vector\")\n\
@sort$$ <- if($v == 1){\n\
\tT\n\
}else{\n\
\targvalue($02,\"$2\",\"TF\")\n\
}\n\
@sort$$ <- keyvalue($K,\"sort\",\"TF\",default:@sort$$)\n\
@labels$$ <- getlabels(@values$$,1,silent:T)\n\
@addlabels$$ <- keyvalue($K,\"label*\",\"TF\",\\\n\
\tdefault:!isnull(@labels$$) || ischar(@values$$))\n\
\n\
if (@addlabels$$ && isnull(@labels$$)){\n\
\t@labels$$ <- paste(@values$$,multiline:T,missing:\"?\")\n\
}\n\
if (alltrue(!ischar(@values$$), sum(ismissing(@values$$)) == nrows(@values$$))){\n\
\terror(\"all elements in argument are MISSING\")\n\
}\n\
@warnings$$ <- getoptions(warnings:T)\n\
setoptions(warnings:F) # suppress warning messages with MISSING values\n\
if (islogic(@values$$)){\n\
\t@values$$ <- @values$$ + 0 #convert to REAL\n\
}\n\
\n\
@values$$ <- if(!delete(@sort$$, return:T)){\n\
\tmatch(@values$$,unique(@values$$))\n\
}elseif(ischar(@values$$) || !anymissing(@values$$)){\n\
\tmatch(@values$$,unique(sort(@values$$)))\n\
}else{\n\
\t@J$$ <- ismissing(@values$$)\n\
\tmatch(@values$$,unique(sort(@values$$[!ismissing(@values$$)])))\n\
}\n\
setoptions(warnings:delete(@warnings$$,return:T))\n\
\n\
if (delete(@addlabels$$, return:T)){\n\
\tfactor(vector(delete(@values$$,return:T),labels:delete(@labels$$,return:T)))\n\
} else {\n\
\tfactor(delete(@values$$, return:T))\n\
}", /*MEND*/


/*MNAME*/ "resid",
/*MTEXT*/ "# $S() or $S(model) or $S(model,T), where model is a linear model\n\
if($v > 2){\n\
\terror(\"usage: $S() or $S(model) or $S(model,T)\", macroname:F)\n\
}\n\
\n\
@reg$$ <- if($v <= 1){\n\
\tF\n\
}else{\n\
\targvalue($02,\"argument 2\",\"TF\")\n\
}#T forces regress()\n\
if($v >= 1){\n\
\t@model$$ <- argvalue($01,\"model\",\"string\")\n\
\tif(@reg$$){\n\
\t\tregress(@model$$,silent:T)\n\
\t}else{\n\
\t\tmanova(@model$$,silent:T)\n\
\t}\n\
\tdelete(@model$$)\n\
}\n\
if(!isdefined(RESIDUALS)||!isdefined(SS)||!isdefined(HII)||!isdefined(DF)){\n\
\terror(\"no active model; try $S(model)\", macroname:F)\n\
}\n\
@m$$ <- dim(SS)[1]\n\
if (DF[@m$$] == 0){\n\
\terror(\"no degrees of freedom for error; can't studentize\")\n\
}\n\
@r$$ <- if(isdefined(WTDRESIDUALS)){\n\
\tWTDRESIDUALS\n\
}else{\n\
\tRESIDUALS\n\
}\n\
@w$$ <- getoptions(warnings:T)\n\
setoptions(warnings:F)\n\
@r$$ <-/ sqrt((1-HII)*(diag(SS[@m$$,,])'/DF[@m$$]))\n\
@p$$ <- nrows(SS[1,,])\n\
if (@p$$ == 1){\n\
\t@labels$$ <- vector(\"Depvar\",\"StdResids\",\"HII\",\"Cook's D\",\\\n\
\t\t\"t-stats\")\n\
}else{\n\
\t@J$$ <- run(@p$$)\n\
\t@dlabels$$ <- getlabels(vector(@J$$,labels:\"Depvar \"))\n\
\t@rslabels$$ <- getlabels(vector(@J$$,labels:\"StdResids \"))\n\
\t@rclabels$$ <- getlabels(vector(@J$$,labels:\"Cook's D \"))\n\
\t@rtlabels$$ <- getlabels(vector(@J$$,labels:\"t-stats \"))\n\
\t@labels$$ <- vector(@dlabels$$,@rslabels$$,\"HII\",@rclabels$$,@rtlabels$$)\n\
\tdelete(@dlabels$$,@rslabels$$,@rtlabels$$,@rclabels$$,@J$$)\n\
}\n\
@o$$ <- matrix(vector(\\\n\
\tmodelvars(0),\\\n\
\t@r$$,\\\n\
\tHII,\\\n\
\t@r$$^2*HII/((1-HII)*sum(DF[-@m$$])),\\\n\
\t@r$$*sqrt((DF[@m$$]-1)/(DF[@m$$]-@r$$^2))),nrows(@r$$),\\\n\
\tlabels:structure(\"(\",@labels$$))\n\
setoptions(warnings:@w$$)\n\
delete(@w$$,@r$$,@m$$,@p$$,@labels$$)\n\
delete(@o$$,return:T)", /*MEND*/


/*MNAME*/ "yhat",
/*MTEXT*/ "# $S() or $S(model) or $S(model,T), where model is a linear model\n\
if($v > 2){\n\
\terror(\"usage: $S() or $S(model) or $S(model,T)\", macroname:F)\n\
}\n\
@reg$$ <- if($v <= 1){\n\
\tF\n\
}else{\n\
\targvalue($02,\"argument 2\",\"TF\")\n\
}\n\
if($v >= 1){\n\
\t@model$$ <- argvalue($01, \"model\", \"string\")\n\
\tif(@reg$$){\n\
\t\tregress(@model$$,silent:T)\n\
\t}else{\n\
\t\tmanova(@model$$,silent:T)\n\
\t}\n\
\tdelete(@model$$)\n\
}\n\
if(!isdefined(RESIDUALS) || !isdefined(SS) ||\\\n\
\t!isdefined(HII) || !isdefined(DF)){\n\
\terror(\"apparently no active linear model; try $S(model)\", macroname:F)\n\
}\n\
@m$$ <- dim(SS)[1]\n\
if (DF[@m$$] == 0){\n\
\terror(\"no degrees of freedom for error; can't studentize\")\n\
}\n\
\n\
@p$$ <- nrows(SS[1,,])\n\
if (@p$$ == 1){\n\
\t@labels$$ <- vector(\"Depvar\",\"Pred\",\"Pred Resid\",\"SE Est\",\\\n\
\t\t\"SE Pred\")\n\
}else{\n\
\t@ylabels$$ <- @plabels$$ <- @prlabels$$ <- @seelabels$$ <- @seplabels$$ <- NULL\n\
\t@J$$ <- run(@p$$)\n\
\t@ylabels$$ <- getlabels(vector(@J$$,labels:\"Depvar \"))\n\
\t@plabels$$ <- getlabels(vector(@J$$,labels:\"Pred \"))\n\
\t@prlabels$$ <- getlabels(vector(@J$$,labels:\"Pred Resid \"))\n\
\t@seelabels$$ <- getlabels(vector(@J$$,labels:\"SE Est \"))\n\
\t@seplabels$$ <- getlabels(vector(@J$$,labels:\"SE Pred \"))\n\
\t@labels$$ <- \\\n\
\t\tvector(@ylabels$$,@plabels$$,@prlabels$$,@seelabels$$,@seplabels$$)\n\
\tdelete(@ylabels$$,@plabels$$,@prlabels$$,@seelabels$$,@seplabels$$,@J$$)\n\
}\n\
@wts$$ <- if(isdefined(WTDRESIDUALS)){\n\
\tmodelinfo(weights:T)\n\
}else{\n\
\trep(1,nrows(RESIDUALS))\n\
}\n\
@mse$$ <- diag(SS[@m$$,,])'/DF[@m$$]\n\
@depvar$$ <- modelvars(0)\n\
delete(@reg$$, @m$$)\n\
\n\
@w$$ <- getoptions(warnings:T)\n\
setoptions(warnings:F)\n\
@o$$ <- matrix(vector(\\\n\
\t@depvar$$ ,\\\n\
\t@depvar$$-RESIDUALS,\\\n\
\tRESIDUALS/(1-HII),\\\n\
\tsqrt(HII*@mse$$/@wts$$),\\\n\
\tsqrt((1+HII/@wts$$)*@mse$$)),nrows(@depvar$$),\\\n\
\tlabels:structure(\"(\",@labels$$))\n\
setoptions(warnings:@w$$)\n\
delete(@w$$,@wts$$,@depvar$$,@mse$$)\n\
delete(@o$$, return:T)", /*MEND*/


/*MNAME*/ "readcols",
/*MTEXT*/ "# $S(filename,name1,...,namek [,echo:T or F]), only filename quoted\n\
if(isscalar(VERSION,char:T)){\n\
\tif(vecread(string:VERSION,silent:T)[2] < 4.11){\n\
\t\terror(\"$S requires MacAnova 4.11 or later\",macroname:F)\n\
\t}\n\
}\n\
@nv$$ <- $v - 1\n\
@string$$ <- keyvalue($K,\"string\",\"character vector\")\n\
@filename$$ <- keyvalue($K,\"file\",\"string\")\n\
if (!isnull(@string$$) || !isnull(@filename$$)){\n\
\tif (!isnull(@string$$) && !isnull(@filename$$)){\n\
\t\terror(\"illegal to use both 'file' and 'string'\")\n\
\t}\n\
\t@nv$$ <-+ 1\n\
}else{\n\
\t#just check argument (file name)\n\
\t@filename$$ <- argvalue($01,\"$1\",\"string\")\n\
}\n\
\n\
if (!isnull(keyvalue($K, \"startline\", \"positive integer scalar\"))){\n\
\terror(\"'startline:M' illegal on $S\",macroname:F)\n\
}\n\
\n\
if (keyvalue($K, \"byline*\",\"TF\",default:F)){\n\
\terror(\"'bylines:T illegal on $S\",macroname:F)\n\
}\n\
@bypass$$ <- keyvalue($K, \"bypass\",\"count\",default:0)\n\
@badvalue$$ <- keyvalue($K, \"badval*\",\"real scalar\", default:?)\n\
@quiet$$ <- keyvalue($K, \"quiet\",\"TF\", default:F)\n\
@echo$$ <- keyvalue($K, \"echo\",\"TF\", default:F)\n\
@silent$$ <- keyvalue($K, \"silent\",\"TF\", default:F)\n\
if (@silent$$ && (@echo$$ || !@quiet$$)){\n\
\terror(\"silent:T is incompatible with echo:T and quiet:F\")\n\
}\n\
@silent$$ <- !@echo$$ && @quiet$$\n\
\n\
@startline$$ <- 1\n\
\n\
@realorch$$ <- keyvalue($K, \"realorch*\", \"TF\", default:F)\n\
if (@realorch$$){\n\
\t@illegal$$ <- vector(\"byword\",\"bychar\",\"byline\",\"byfield\")\n\
\tfor (@i$$,1,4){\n\
\t\t@what$$ <- paste(@illegal$$[@i$$],\"*\",sep:\"\")\n\
\t\tif (!isnull(keyvalue($K, @what$$, \"TF\"))){\n\
\t\t@what$$ <- paste(@illegal$$[@i$$],\"s\",sep:\"\")\n\
\t\t\terror(paste(\"'\",@what$$,\"' can't be used with 'realorchar:T'\",\\\n\
\t\t\t\tsep:\"\"))\n\
\t\t}\n\
\t}\n\
\tdelete(@illegal$$,@what$$)\n\
}\n\
\n\
  #determine names of variables to be created\n\
@names$$ <- if (@nv$$ == 0){\n\
\t# look for first non blank line (very inefficient, esp if bypass>1)\n\
\tfor(@startline$$,1,200){\n\
\t\t@nameline$$ <- if(!isnull(@filename$$)){\n\
\t\t\tvecread(@filename$$,n:1,bylines:T,startline:@startline$$,\\\n\
\t\t\t\tbypass:@bypass$$,silent:T)\n\
\t\t}else{\n\
\t\t\tvecread(string:@string$$, n:1, bylines:T,startline:@startline$$,\\\n\
\t\t\t\tbypass:@bypass$$,silent:T)\n\
\t\t}\n\
\t\tif (isnull(@nameline$$)){\n\
\t\t\tbreak\n\
\t\t}\n\
\t\tif (@nameline$$ != \"\"){\n\
\t\t\t@tmp$$ <- vecread(string:delete(@nameline$$, return:T), bywords:T)\n\
\t\t\tif (anytrue(isnull(@tmp$$), length(@tmp$$) > 1, @tmp$$[1] == \"\")){\n\
\t\t\t\tbreak;\n\
\t\t\t}\n\
\t\t}\n\
\t}\n\
\tif (anytrue(!isdefined(@tmp$$), isnull(@tmp$$), @startline$$ == 200)){\n\
\t\terror(\"no data found\")\n\
\t}\n\
\t@startline$$ <-+ 1\n\
\tdelete(@tmp$$, return:T)\n\
}elseif(@nv$$ == 1){\n\
\tif(ischar($02)){\n\
\t\t$2\n\
\t}else{\n\
\t\t\"$2\"\n\
\t}\n\
}else{\n\
\t$A[1+run(@nv$$)]\n\
}\n\
delete(@filename$$,@string$$,@bypass$$)\n\
\n\
  #strip off any quotes and check that names are legal\n\
@n$$ <- length(@names$$)\n\
\n\
for(@i$$,1,@n$$){\n\
\t@NAME$$ <- @names$$[@i$$]\n\
\tif (match(\"\\\"*\\\"\", @NAME$$, 0, exact:F) != 0){\n\
\t\t@names$$[@i$$] <- @NAME$$ <- <<@NAME$$>>\n\
\t}\n\
\tif (!isname(@NAME$$)){\n\
\t\terror(paste(@NAME$$,\"is not a legal variable name\"))\n\
\t}\n\
\tif (isfunction(<<@NAME$$>>)){\n\
\t\terror(paste(@NAME$$, \"is a function name and can't be used for data\"))\n\
\t}\n\
\tif (ismacro(<<@NAME$$>>)){\n\
\t\tprint(paste(\"WARNING: macro\",@NAME$$,\\\n\
\t\t\t\"has been replaced by data vector\"), macroname:T)\n\
\t}\n\
}\n\
delete(@NAME$$)\n\
\n\
  # read data into vector, always CHARACTER with realorchar:T\n\
@data$$ <- if (@realorch$$){\n\
\tif(@nv$$ == $v){\n\
\t\tvecread($K,bywords:T,startline:@startline$$,silent:@silent$$,\\\n\
\t\t\tbadkeyok:T)\n\
\t}elseif($k > 0){\n\
\t\tvecread($01,$K,bywords:T,startline:@startline$$,silent:@silent$$,\\\n\
\t\t\tbadkeyok:T)\n\
\t}else{\n\
\t\tvecread($01,bywords:T,startline:@startline$$,silent:@silent$$,\\\n\
\t\t\tbadkeyok:T)\n\
\t}\n\
}elseif(@nv$$==$v){\n\
\tvecread($K,startline:@startline$$,silent:@silent$$,badkeyok:T)\n\
}elseif($k>0){\n\
\tvecread($01,$K,startline:@startline$$,silent:@silent$$,badkeyok:T)\n\
}else{\n\
\tvecread($01,startline:@startline$$,silent:@silent$$,badkeyok:T)\n\
}\n\
if (!@quiet$$ && @nv$$ == 0){\n\
\tprint(paste(\"Creating vectors\",@names$$))\n\
}\n\
delete(@silent$$,@echo$$,@quiet$$,@startline$$,@nv$$)\n\
\n\
  #matrix will complain if n does not divide length(data)\n\
@data$$ <- matrix(@data$$,@n$$)\n\
@bad$$ <- 8.3109081054232958e-203 #arbitrary\n\
for(@i$$,1,@n$$){\n\
\t<<@names$$[@i$$]>> <- if (@realorch$$){\n\
\t\tif (vecread(string:@data$$[@i$$,1],byfields:T,badvalue:@bad$$) == @bad$$){\n\
\t\t\t# CHARACTER variable\n\
\t\t\tvector(@data$$[@i$$,])\n\
\t\t} else {\n\
\t\t\t# REAL variable\n\
\t\t\tvecread(string:@data$$[@i$$,]',byfields:T,badvalue:@badvalue$$, silent:T)\n\
\t\t}\n\
\t}else{\n\
\t\tvector(@data$$[@i$$,])\n\
\t}\n\
}\n\
delete(@data$$,@names$$,@i$$,@n$$,@realorch$$,@badvalue$$,@bad$$)", /*MEND*/


/*MNAME*/ "readdata",
/*MTEXT*/ "# $S(filename,name1,...,namek [,echo:T or F]), only filename quoted\n\
@nv$$ <- $v - 1\n\
@string$$ <- keyvalue($K,\"string\",\"character vector\")\n\
@filename$$ <- keyvalue($K,\"file\",\"string\")\n\
if (!isnull(@string$$) || !isnull(@filename$$)){\n\
\tif (!isnull(@string$$) && !isnull(@filename$$)){\n\
\t\terror(\"illegal to use both 'file' and 'string'\")\n\
\t}\n\
\t@nv$$ <-+ 1\n\
}else{\n\
\t#just check argument (file name)\n\
\t@filename$$ <- argvalue($01,\"$1\",\"string\")\n\
}\n\
\n\
@factors$$ <- keyvalue($K,\"factor*\",\"TF\",default:T)\n\
@sort$$ <- keyvalue($K, \"sort\",\"TF\",default:T)\n\
@badvalue$$ <- keyvalue($K, \"badval*\",\"real scalar\", default:?)\n\
@quiet$$ <- keyvalue($K, \"quiet\",\"TF\", default:F)\n\
@echo$$ <- keyvalue($K, \"echo\",\"TF\", default:F)\n\
@silent$$ <- keyvalue($K, \"silent\",\"TF\", default:F)\n\
@stop$$ <- keyvalue($K, \"stop\",\"string\", default:\"!\")\n\
@skip$$ <- keyvalue($K, \"skip\",\"string\", default:\"#\")\n\
\n\
if (@silent$$ && (@echo$$ || !@quiet$$)){\n\
\terror(\"silent:T is incompatible with echo:T and quiet:F\")\n\
}\n\
@silent$$ <- !@echo$$ && @quiet$$\n\
\n\
@illegal$$ <- vector(\"byword\",\"char\",\"bychar\",\"byline\",\"byfield\",\"bypass\",\\\n\
\t\"n\", \"skipthru\")\n\
for (@i$$,1,length(@illegal$$)){\n\
\t@what$$ <- paste(@illegal$$[@i$$],\"*\",sep:\"\")\n\
\tif (!isnull(keyvalue($K, @what$$))){\n\
\t\t@what$$ <- paste(@illegal$$[@i$$],vector(\"\",\"s\")[1+(@i$$<=6)],sep:\"\")\n\
\t\terror(paste(\"'\",@what$$,\"' is not a legal keyword\", sep:\"\"))\n\
\t}\n\
}\n\
delete(@illegal$$)\n\
@what$$ <- vector(if (@factors$$){\n\
\tif (@sort$$){\n\
\t\t\"factor\"\n\
\t} else {\n\
\t\t\"unsorted factor\"\n\
\t}\n\
}else{\n\
\t\"CHARACTER vector\"\n\
}, \"REAL vector\")\n\
\n\
@contents$$ <- if (!isnull(@filename$$)){\n\
\tvecread(@filename$$,bylines:T,stop:@stop$$,skip:@skip$$,echo:@echo$$,\\\n\
\t\tsilent:@silent$$,quiet:@quiet$$)\n\
} else {\n\
\tvecread(string:@string$$,bylines:T,stop:@stop$$,skip:@skip$$,echo:@echo$$,\\\n\
\t\tsilent:@silent$$,quiet:@quiet$$)\n\
}\n\
\n\
if (isnull(@contents$$)){\n\
\terror(\"no data read\")\n\
}\n\
@nlines$$ <- length(@contents$$)\n\
if (@contents$$[@nlines$$] == \"\\032\"){ #remove trailing ^Z if any\n\
\t@contents$$ <- @contents$$[-@nlines$$]\n\
\t@nlines$$ <-- 1\n\
}\n\
\n\
if (@nlines$$ == 0){\n\
\terror(\"no data read\")\n\
}\n\
  #determine names of variables to be created\n\
@names$$ <- if (@nv$$ == 0){\n\
\tif (@nlines$$ < 2){\n\
\t\terror(\"no data read\")\n\
\t}\n\
\n\
\t@names$$ <- vecread(string:@contents$$[1],bywords:T)\n\
\tif (paste(vector(@names$$,\"\")) == \"\"){\n\
\t\terror(\"variable names not in line 1\")\n\
\t}\n\
\t@contents$$ <- @contents$$[-1]\n\
\t@nlines$$ <-- 1\n\
\t@names$$\n\
}elseif(@nv$$ == 1){\n\
\tif(ischar($02)){\n\
\t\t$2\n\
\t}else{\n\
\t\t\"$2\"\n\
\t}\n\
}else{\n\
\t$A[1+run(@nv$$)]\n\
}\n\
delete(@filename$$,@string$$)\n\
\n\
  #strip off any quotes and check that names are legal\n\
@n$$ <- length(@names$$)\n\
\n\
for(@i$$,1,@n$$){\n\
\t@NAME$$ <- @names$$[@i$$]\n\
\tif (match(\"\\\"*\\\"\", @NAME$$, 0, exact:F) != 0){\n\
\t\t@names$$[@i$$] <- @NAME$$ <- <<@NAME$$>>\n\
\t}\n\
\tif (!isname(@NAME$$)){\n\
\t\terror(paste(@NAME$$,\"is not a legal variable name\"))\n\
\t}\n\
\tif (isfunction(<<@NAME$$>>)){\n\
\t\terror(paste(@NAME$$, \"is a function name and can't be used for data\"))\n\
\t}\n\
\tif (ismacro(<<@NAME$$>>)){\n\
\t\tprint(paste(\"WARNING: data vector replacing macro\",@NAME$$),\\\n\
\t\t\t macroname:T)\n\
\t}\n\
}\n\
delete(@NAME$$)\n\
\n\
  # read data into CHARACTER vector\n\
\n\
@data$$ <- vecread(string:delete(@contents$$,return:T),bywords:T)\n\
\n\
if (anymissing(@data$$)){\n\
\t# at least one \"\"\n\
\t@data$$ <- @data$$[!ismissing(@data$$)]\n\
}\t\n\
if (isnull(@data$$)){\n\
\terror(\"no data read\")\n\
}\n\
\t\n\
if (length(@data$$) %% @n$$ != 0){\n\
\terror(paste(\"number of variables\",@n$$,\\\n\
\t\t\"does not divide number of data items\",length(@data$$)))\n\
}\n\
@data$$ <- matrix(@data$$,@n$$)\n\
\n\
@bad$$ <- 8.3109081054232958e-203 #arbitrary\n\
\n\
@type$$ <- 1 + (vecread(string:vector(@data$$[,1]),\\\n\
\t\t\tbyfields:T,badvalue:@bad$$) != @bad$$)\n\
\n\
delete(@bad$$,@silent$$,@echo$$)\n\
\n\
for(@i$$,1,@n$$){\n\
\t@var_i$$ <- vector(@data$$[@i$$,])\n\
\t<<@names$$[@i$$]>> <- if (@type$$[@i$$] == 1){\n\
\t\tif (!@factors$$){\n\
\t\t\t# CHARACTER variable\n\
\t\t\t@var_i$$\n\
\t\t} elseif (@sort$$) {\n\
\t\t\t# make a factor labelled with values\n\
\t\t\tfactor(vector(match(@var_i$$, unique(sort(@var_i$$))),\\\n\
\t\t\t\tlabels:paste(@var_i$$,multiline:T)))\n\
\t\t} else {\n\
\t\t\t# make a factor labelled with values\n\
\t\t\tfactor(vector(match(@var_i$$, unique(@var_i$$)),\\\n\
\t\t\t\tlabels:paste(@var_i$$,multiline:T)))\n\
\t\t}\n\
\t} else {\n\
\t\t# REAL variable\n\
\t\tvecread(string:@var_i$$, byfields:T,badvalue:@badvalue$$,silent:T)\n\
\t}\n\
\tif (!@quiet$$){\n\
\t\t@nmiss$$ <- if (isreal(<<@names$$[@i$$]>>)) {\n\
\t\t\tsum(ismissing(<<@names$$[@i$$]>>))\n\
\t\t} else {\n\
\t\t\t0\n\
\t\t}\n\
\t\t@missmsg$$ <- if (@nmiss$$ > 0) {\n\
\t\t\tpaste(\"with\",@nmiss$$,\"missing value(s)\")\n\
\t\t} else {\n\
\t\t\t\"\"\n\
\t\t}\n\
\t\tprint(paste(\"Column\",@i$$,\"saved as\", @what$$[@type$$[@i$$]],@names$$[@i$$],\\\n\
\t\t\t\t\t@missmsg$$))\n\
\t\tdelete(@nmiss$$,@missmsg$$)\n\
\t}\n\
}\n\
delete(@data$$,@var_i$$,@names$$,@i$$,@n$$,@badvalue$$,@quiet$$,@factors$$,@sort$$,@nv$$,@what$$)", /*MEND*/


/*MNAME*/ "writedata",
/*MTEXT*/ "# $S(filename,x1,x2,a,b,... [,missing:M] [, putnames:F] \\\n\
#        [,fieldwidth:w  or  format:fmt])\n\
if ($N < 2) {\n\
\terror(\"$S must have at least two arguments\")\n\
}\n\
\n\
@args$$ <- structure($0)\n\
@argnames$$ <- compnames(@args$$)\n\
@keep$$ <- if ($k > 0) {\n\
\tkeyvalue($K,\"keep\",\"TF\")\n\
} else {\n\
\tNULL\n\
}\n\
@j$$ <- if (!isnull(@keep$$)) {\n\
\tif (!@keep$$) {\n\
\t\terror(\"value for 'keep' must be T\")\n\
\t}\n\
\tmatch(\"keep\",@argnames$$)\n\
} else {\n\
\t@keep$$ <- F\n\
\t1\n\
}\n\
\n\
@args$$ <- @args$$[-@j$$]\n\
@argnames$$ <- @argnames$$[-@j$$]\n\
@arglist$$ <- $A[-@j$$]\n\
@nvars$$ <- $N-1\n\
\n\
@new$$ <- keyvalue($K, \"new\", \"TF\")\n\
if (isnull(@new$$) ) {\n\
\t@new$$ <- T\n\
} else {\n\
\tif (@keep$$){\n\
\t\terror(\"illegal to use 'keep' and 'new' together\")\n\
\t}\n\
\t@j$$ <- match(\"new\",@argnames$$)\n\
\t@args$$ <- @args$$[-@j$$]\n\
\t@argnames$$ <- @argnames$$[-@j$$]\n\
\t@arglist$$ <- @arglist$$[-@j$$]\n\
\t@nvars$$ <-- 1\n\
}\n\
\n\
@putnames$$ <- keyvalue($K, \"putnames\", \"TF\")\n\
if (isnull(@putnames$$) ) {\n\
\t@putnames$$ <- T\n\
} else {\n\
\t@j$$ <- match(\"putnames\",@argnames$$)\n\
\t@args$$ <- @args$$[-@j$$]\n\
\t@argnames$$ <- @argnames$$[-@j$$]\n\
\t@arglist$$ <- @arglist$$[-@j$$]\n\
\t@nvars$$ <-- 1\n\
}\n\
@missing$$ <- keyvalue($K, \"missing\", \"string\")\n\
if (isnull(@missing$$) ) {\n\
\t@missing$$ <- \"?\"\n\
} else {\n\
\t@j$$ <- match(\"missing\",@argnames$$)\n\
\t@args$$ <- @args$$[-@j$$]\n\
\t@argnames$$ <- @argnames$$[-@j$$]\n\
\t@arglist$$ <- @arglist$$[-@j$$]\n\
\t@nvars$$ <-- 1\n\
}\n\
\n\
@format$$ <- keyvalue($K, \"format\", \"string\")\n\
@fieldwid$$ <- keyvalue($K, \"fieldwidth\", \"positive count\")\n\
if (!isnull(@format$$) && !isnull(@fieldwid$$)) {\n\
\terror(\"'format' and 'fieldwidth' can't be used together\")\n\
}\n\
if (isnull(@format$$) && isnull(@fieldwid$$)) {\n\
\t@format$$ <- getoptions(format:T)\n\
} elseif (!isnull(@format$$) ) {\n\
\t@j$$ <- match(\"format\",@argnames$$)\n\
\t@args$$ <- @args$$[-@j$$]\n\
\t@argnames$$ <- @argnames$$[-@j$$]\n\
\t@arglist$$ <- @arglist$$[-@j$$]\n\
\t@nvars$$ <-- 1\n\
}\n\
if (isnull(@fieldwid$$) ) {\n\
\t@fieldwid$$ <- vecread(string:@format$$,silent:T)\n\
\t@fieldwid$$ <- if (@fieldwid$$ < 1) {\n\
\t\t@fieldwid$$ <- round(10*@fieldwid$$ + 7)\n\
\t} else {\n\
\t\tfloor(@fieldwid$$)\n\
\t}\n\
} else {\n\
\tif (isnull(@format$$)) {\n\
\t\t@format$$ <- paste(@fieldwid$$,\".\",max(@fieldwid$$ - 7,0),\"g\",sep:\"\")\n\
\t}\n\
\t@j$$ <- match(\"fieldwidth\",@argnames$$)\n\
\t@args$$ <- @args$$[-@j$$]\n\
\t@argnames$$ <- @argnames$$[-@j$$]\n\
\t@arglist$$ <- @arglist$$[-@j$$]\n\
\t@nvars$$ <-- 1\n\
}\n\
\n\
if (@nvars$$ < 1) {\n\
\terror(\"no data provided to write\")\n\
}\n\
\n\
if (!@keep$$) {\n\
\t@filename$$ <- \"$1\"\n\
\tif (match(\"\\\"*\\\"\", @filename$$, 0, exact:F) != 0) {\n\
\t\t@filename$$ <- <<@filename$$>>\n\
\t} elseif (alltrue(@filename$$ != \"CONSOLE\", isname(@filename$$),\\\n\
\t\t\t\t\t  isscalar(<<@filename$$>>, char:T))) {\n\
\t\t@filename$$ <- <<@filename$$>>\n\
\t}\n\
\tif (!isscalar(@filename$$, char:T)) {\n\
\t\terror(\"'$1' not a CHARACTER scalar\")\n\
\t}\n\
}\n\
\n\
@n$$ <- 0\n\
\n\
for (@i$$, 1, @nvars$$) {\n\
\tif (!isvector(@args$$[@i$$])) {\n\
\t\terror(paste(@arglist$$[@i$$], \"is not a vector\"))\n\
\t}\n\
\t@argi$$ <- @args$$[@i$$]\n\
\tif (@n$$ == 0) {\n\
\t\t@n$$ <- length(@argi$$)\n\
\t} elseif (length(@argi$$) != @n$$) {\n\
\t\terror(\"not all data vectors have the same length\")\n\
\t}\n\
}\n\
delete(@argi$$)\n\
\n\
@chars$$ <- rep(F,@nvars$$)\n\
for(@i$$,1,@nvars$$) {\n\
\tif (isfactor(@args$$[@i$$]) && haslabels(@args$$[@i$$])) {\n\
\t\t@labels$$ <- getlabels(@args$$[@i$$], 1)\n\
\t\t@labels1$$ <- unique(@labels$$)\n\
\t\tif (length(@labels1$$) == length(unique(@args$$[@i$$]))) {\n\
\t\t\tfor(@j$$,1,length(@labels1$$)) {\n\
\t\t\t\tif (describe(@args$$[@i$$][@labels$$ == @labels1$$[@j$$]],var:T) > 0) {\n\
\t\t\t\t\t@j$$ <- -1\n\
\t\t\t\t\tbreak\n\
\t\t\t\t}\n\
\t\t\t}\n\
\t\t\tif (@j$$ > 0) {\n\
\t\t\t\t@args$$[@i$$] <- @labels$$\n\
\t\t\t\t@chars$$[@i$$] <- T\n\
\t\t\t}\n\
\t\t}\n\
\t\tdelete(@labels$$)\n\
\t} else {\n\
\t\t@chars$$[@i$$] <- ischar(@args$$[@i$$])\n\
\t}\n\
}\n\
\n\
for (@j$$, 1, @nvars$$) {\n\
\t@x$$ <- @args$$[@j$$]\n\
\t@cx$$ <- rep(\"\", @n$$)\n\
\tif (!@chars$$[@j$$]) {\n\
\t\tfor (@i$$, 1, @n$$) {\n\
\t\t\t@cx$$[@i$$] <- if (isscalar(@x$$[@i$$],integer:T)) {\n\
\t\t\t\tpaste(@x$$[@i$$],intwidth:@fieldwid$$)\n\
\t\t\t} elseif (ismissing(@x$$[@i$$])) {\n\
\t\t\t\tpaste(@missing$$,charwidth:@fieldwid$$,justify:\"R\")\n\
\t\t\t} else {\n\
\t\t\t\tpaste(@x$$[@i$$],format:@format$$)\n\
\t\t\t}\n\
\t\t}\n\
\t} else {\n\
\t\tfor (@i$$, 1, @n$$) {\n\
\t\t\t@cx$$[@i$$] <- paste(@x$$[@i$$], charwidth:@fieldwid$$,justify:\"R\")\n\
\t\t}\n\
\t}\n\
\t@args$$[@j$$] <- @cx$$\n\
}\n\
delete(@x$$,@cx$$,@i$$,@j$$,@format$$)\n\
@output$$ <- paste(matrix(vector(@args$$),delete(@n$$,return:T)), multiline:T)\n\
if (delete(@putnames$$,return:T)) {\n\
\t@output$$ <- vector(paste(@argnames$$, charwidth:@fieldwid$$,justify:\"R\"),@output$$)\n\
}\n\
delete(@args$$,@argnames$$,@fieldwid$$)\n\
if (@keep$$) {\n\
\tdelete(@output$$,return:T)\n\
} else {\n\
\tprint(file:delete(@filename$$,return:T),paste(delete(@output$$,return:T),\\\n\
\t   multiline:T,linesep:\"\\n\"), new:delete(@new$$,return:T))\n\
}", /*MEND*/


/*MNAME*/ "makecols",
/*MTEXT*/ "# $S(y,name1,name2,...,namek [,nomissing:T] [,factors:TorFvec)\n\
#  y a matrix, name1,... quoted or unquoted names, TorFvec LOGICAL vec\n\
@data$$ <- matrix(argvalue($1, \"$1\", \"matrix\"))\n\
if ($v == 2){\n\
\t@names$$ <- if(ischar($2)){\n\
\t\t$2\n\
\t}else{\n\
\t\t\"$2\"\n\
\t}\n\
}elseif($v > 2){\n\
\t@names$$ <- $A[run(2,$v)]\n\
}\n\
@strip$$ <- keyvalue($K,\"nomissing\",\"TF\", default:F)\n\
@factors$$ <- keyvalue($K,\"factor*\",\"nonmissing vector\")\n\
@silent$$ <- keyvalue($K,\"silent\",\"TF\",default:F)\n\
@quiet$$ <- keyvalue($K,\"quiet\",\"TF\",default:F) || @silent$$\n\
\n\
if (ischar(@data$$)){\n\
\tif (!isvector(@data$$)){\n\
\t\terror(\"CHARACTER argument 1 must be a scalar or vector\")\n\
\t}\n\
\t@line1$$ <- vecread(string:@data$$,bylines:T,n:1) # first line\n\
\t@line1$$ <- vecread(string:@line1$$, bywords:T)#split it up\n\
\t@startline$$ <- 1\n\
\tif ($v == 1){\n\
\t\t@names$$ <- @line1$$\n\
\t\t@line1$$ <- vecread(string:@data$$,bylines:T,n:1,startline:2)\n\
\t\t@havedata$$ <- !isnull(@line1$$)\n\
\t\tif (@havedata$$){\n\
\t\t\t@line1$$ <- vecread(string:@line1$$, bywords:T)\n\
\t\t\t@havedata$$ <- @line1$$[1] != \"\"\n\
\t\t}\n\
\t\tif (!@havedata$$){\n\
\t\t\terror(\"No data in CHARACTER argument 1\")\n\
\t\t}\n\
\t\tdelete(@havedata$$)\n\
\t\t@startline$$ <- 2\n\
\t}\n\
\t@nvars$$ <- length(@line1$$)\n\
\t\t\n\
\t@badvalue$$ <- -9.8696044731e301\n\
\t@data$$ <- vecread(string:@data$$,byfields:T,badvalue:@badvalue$$,\\\n\
\t\t\t\t\tstartline:@startline$$)\n\
\tif (isnull(@data$$)){\n\
\t\terror(\"No data in CHARACTER argument 1\")\n\
\t}\n\
\tif (length(@data$$) %% @nvars$$ != 0){\n\
\t\terror(\"data in CHARACTER argument 1 not a matrix\")\n\
\t}\n\
\tif (match(@badvalue$$,vector(@data$$),0) != 0){\n\
\t\tif (!@silent$$){\n\
\t\t\tprint(\"WARNING: unreadable items in CHARACTER argument set to MISSING\",\\\n\
\t\t\t\tmacroname:T)\n\
\t\t}\n\
\t\t@data$$[vector(@data$$) == @badvalue$$] <- ?\n\
\t}\n\
\t@data$$ <- matrix(@data$$,@nvars$$)'\n\
\n\
\tdelete(@badvalue$$, @line1$$, @nvars$$)\n\
}elseif (!isreal(@data$$)){\n\
\terror(\"argument 1 not REAL or CHARACTER matrix\")\n\
}elseif ($v == 1){\n\
\t@names$$ <- getlabels(@data$$,2,silent:T)\n\
\tif (isnull(@names$$)){\n\
\t\terror(\"no variable names provided and argument 1 has no labels\")\n\
\t}\n\
}\n\
\n\
@n$$ <- length(@names$$)\n\
if (!isnull(@data$$)){\n\
\t@n$$ <- min(@n$$, ncols(@data$$))\n\
}\n\
if (!isnull(@factors$$)){\n\
\tif (islogic(@factors$$)){\n\
\t\tif (length(@factors$$) != @n$$){\n\
\t\t\terror(\"Length of value of 'factors' != number of variables\")\n\
\t\t}\n\
\t} elseif (!isvector(@factors$$,integer:T,positive:T)){\n\
\t\terror(\"non-LOGICAL value for 'factors' not a vector of positive integers\")\n\
\t} elseif (max(@factors$$) > @n$$ ||\\\n\
\t\t\t  length(unique(@factors$$)) != length(@factors$$)){\n\
\t\terror(\"duplicate or out of range values for 'factors'\")\n\
\t} else {\n\
\t\t@factors$$ <- match(run(@n$$),@factors$$,0) != 0\n\
\t}\n\
} else {\n\
\t@factors$$ <- rep(F,@n$$)\n\
}\n\
\n\
for(@i$$,run(@n$$)){\n\
\t@NAME$$ <- @names$$[@i$$]\n\
\tif (match(\"\\\"*\\\"\",@NAME$$,0,exact:F) != 0){\n\
\t\t@names$$[@i$$] <- @NAME$$ <- <<@NAME$$>> # strip quotes\n\
\t}\n\
\tif (!isname(@NAME$$)){\n\
\t\terror(paste(\"'\",@NAME$$,\"' is not a valid variable name\",sep:\"\"))\n\
\t}\n\
\tif (isfunction(<<@NAME$$>>)){\n\
\t\terror(paste(@NAME$$, \"is a function name and can't be used for data\"))\n\
\t}\n\
}\n\
for(@i$$,run(@n$$)){\n\
\t@NAME$$ <- @names$$[@i$$]\n\
\tif (ismacro(<<@NAME$$>>) && !@silent$$){\n\
\t\tprint(paste(\"WARNING: macro\",@NAME$$,\\\n\
\t\t\t\"has been replaced by data vector\"), macroname:T)\n\
\t}\n\
\t<<@NAME$$>> <- vector(@data$$[,@i$$])\n\
\tif(@strip$$){\n\
\t\tif (anymissing(<<@NAME$$>>)){\n\
\t\t\t@I$$ <- !ismissing(<<@NAME$$>>)\n\
\t\t\t<<@NAME$$>> <- if (sum(@I$$) == 0){\n\
\t\t\t\tif (!@silent$$){\n\
\t\t\t\t\tprint(paste(\"WARNING:\",@NAME$$,\\\n\
\t\t\t\t\t\t\"is all MISSING; set to NULL\"))\n\
\t\t\t\t}\n\
\t\t\t\tNULL\n\
\t\t\t}else{\n\
\t\t\t\t<<@NAME$$>>[@I$$]\n\
\t\t\t}\n\
\t\t}\n\
\t}\n\
\t@what$$ <- if (@factors$$[@i$$]){\n\
\t\t<<@NAME$$>> <- makefactor(<<@NAME$$>>)\n\
\t\tpaste(\"factor\",@NAME$$,\" with\",max(<<@NAME$$>>,silent:T),\"levels\")\n\
\t} else {\n\
\t\tpaste(\"vector\",@NAME$$)\n\
\t}\n\
\tif (!@quiet$$){\n\
\t\tprint(paste(\"Column\",@i$$,\"saved as\",@what$$))\n\
\t}\n\
}\n\
delete(@data$$,@names$$,@i$$,@n$$,@strip$$,@factors$$,@what$$,@quiet$$,@silent$$)", /*MEND*/


/*MNAME*/ "anovapred",
/*MTEXT*/ "# $S(a,b,...) , where a, b, ..., are all the factors in the model\n\
if($v == 0){\n\
\terror(\"usage is $S(a,b,...) where a,b,... are all the factors in model\")\n\
}\n\
if(!isdefined(DEPVNAME) || !isdefined(RESIDUALS)||\\\n\
\t !isdefined(HII) || !isdefined(DF)){\n\
\terror(\"apparently no active linear model\")\n\
}\n\
@error$$ <- dim(SS)[1]\n\
if (DF[@error$$] == 0){\n\
\t@seest$$ <- @sepred$$ <- tabs(,$V,count:T)\n\
\t@seest$$[] <- @sepred$$[] <- ?\n\
}else{\n\
\t@mse$$ <- diag(SS[@error$$,,])'/DF[@error$$]\n\
\t@w$$ <- getoptions(warnings:T)\n\
\tsetoptions(warnings:F)\n\
\t@seest$$ <- sqrt(tabs(@mse$$*HII,$V,mean:T))\n\
\t@sepred$$ <- sqrt(tabs(@mse$$*(1+HII),$V,mean:T))\n\
\tsetoptions(warnings:delete(@w$$,return:T))\n\
\tdelete(@mse$$)\n\
}\n\
delete(@error$$)\n\
structure(estimate:tabs(<<DEPVNAME>> - RESIDUALS,$V,mean:T),\\\n\
SEest:delete(@seest$$,return:T), SEpred:delete(@sepred$$,return:T))", /*MEND*/


/*MNAME*/ "colplot",
/*MTEXT*/ "#$S(y [,title:\\\"Title of your choice\\\"][other graphics keywords])\n\
if($v < 1){\n\
\terror(\"$S() needs at least 1 non-keyword argument\",macroname:F)\n\
}\n\
@y$$ <- argvalue($1,\"argument 1\",\"real matrix\")\n\
@n$$ <- nrows(@y$$)\n\
chplot(1,@y$$,yaxis:F,lines:T,$K,xticks:run(1,@n$$,floor((@n$$-1)/15+1)),\\\n\
\txlab:\"Row Number\",title:\"Plot of columns vs row number\")\n\
delete(@y$$,@n$$)", /*MEND*/


/*MNAME*/ "rowplot",
/*MTEXT*/ "#$S(y [,title:\\\"Title of your choice\\\"][other graphics keywords])\n\
if($v < 1){\n\
\terror(\"$S() needs at least 1 non-keyword argument\",macroname:F)\n\
}\n\
@y$$ <- argvalue($1,\"argument 1\",\"real matrix\")'\n\
@n$$ <- nrows(@y$$)\n\
chplot(1,@y$$,yaxis:F,lines:T,$K,xticks:run(1,@n$$,floor((@n$$-1)/15+1)),\\\n\
\txlab:\"Column Number\",title:\"Plot of rows vs column number\")\n\
delete(@y$$,@n$$)", /*MEND*/


/*MNAME*/ "regs",
/*MTEXT*/ "#  $S(x,y [,T] [,GLM keyword phrases]),matrix or vector y, matrix x\n\
@xname$$ <- \"@X\"\n\
@yname$$ <- \"@Y\"\n\
@Xvars$$ <- matrix(argvalue($1,\"argument 1\",\"real matrix nonmissing\"))\n\
<<@yname$$>> <- matrix(argvalue($2,\"argument 2\",\"real matrix nonmissing\"))\n\
@nocon$$ <- if($v > 2){\n\
\targvalue($3,\"argument 3\",\"TF\")\n\
}else{\n\
\tF\n\
}\n\
if (nrows(<<@yname$$>>) != nrows(@Xvars$$)){\n\
\terror(\"number of rows of x and y are different\")\n\
}\n\
@p$$ <- ncols(@Xvars$$)\n\
\n\
<<paste(@xname$$,1,sep:\"\")>> <- @Xvars$$[,1]\n\
@model$$ <- paste(@yname$$,\"=\",@xname$$,1,sep:\"\")\n\
if(@p$$ > 1){\n\
\tfor(@i$$,run(2,@p$$)){\n\
\t\t<<paste(@xname$$,@i$$,sep:\"\")>> <- @Xvars$$[,@i$$]\n\
\t\t@model$$ <- paste(@model$$,\"+\",@xname$$,@i$$,sep:\"\")\n\
\t}\n\
\tdelete(@i$$)\n\
}\n\
if (delete(@nocon$$,return:T)){\n\
\t@model$$ <- paste(@model$$,\"-1\",sep:\"\")\n\
}\n\
if(isvector(<<@yname$$>>)){\n\
\tif ($k > 0){\n\
\t\tregress(@model$$,$K)\n\
\t}else{\n\
\t\tregress(@model$$)\n\
\t}\n\
}else{\n\
\tif ($k > 0){\n\
\t\tmanova(@model$$,$K)\n\
\t}else{\n\
\t\tmanova(@model$$)\n\
\t}\n\
\tif (getoptions(warnings:T)){\n\
\t\tprint(\"NOTE: use secoefs() to obtain coefficients and standard errors.\")\n\
\t}\n\
}\n\
delete(@xname$$,@yname$$,@p$$,@model$$,@Xvars$$)", /*MEND*/


/*LOCKED*/

/*MNAME*/ "&breakif",
/*MTEXT*/ "if($1){\n\
\t@depth$$ <- if($N==1){\n\
\t\t1\n\
\t}else{\n\
\t\t$02\n\
\t}\n\
\t@break$$ <- macro(paste(\"delete(@depth$$);break \",@depth$$,\";\"),inline:T)\n\
\t@break$$()\n\
}", /*MEND*/

/*NOTFORMACSTART*/ /*macros up to NOTFORMACEND are not defined on Macintosh*/
/*  970201 interact:T added to calls to shell() in edit and more*/
#ifdef DEFINEEDIT
#if defined(COMP_DJGPP) || (defined(SKEL_WXWIN) && defined(OS_WIN32))


/*MNAME*/ "edit",
/*MTEXT*/ "# $S(realVar), $S(macro [,stripdol:F]), $S(), or $S(0)\n\
@editor$$ <- \"edit\" #change for different default editor\n\
@tmpfile$$ <- \"\\\\tmp\\\\mv\" #change for different temp name start\n\
@delete$$ <- \"erase\"   #change for different delete file command\n\
if(isdefined(EDITOR) && ischar(EDITOR) && isscalar(EDITOR)){\n\
\t@editor$$ <- EDITOR\n\
}\n\
@strip$$ <- keyvalue($K,\"strip*\",\"TF\",default:T)\n\
@arg$$ <- if($v == 0){\n\
\tmacro(\"====> Replace this line with lines of your macro <====\")\n\
}else{\n\
\targvalue($01,\"argument 1\")\n\
}\n\
\n\
if($v > 2 || (!ismacro(@arg$$) && !isreal(@arg$$))){\n\
\terror(\"usage: $S(realVar), $S(macro [,stripdol:F]), $S(), or $S(0)\")\n\
}\n\
@save$$ <- if($v == 2){\n\
\targvalue($02,\"argument 2\",\"TF\")\n\
}else{\n\
\tF\n\
}\n\
@tmpfile$$ <- paste(@tmpfile$$,round(100000*runi(1)),sep:\"\")\n\
if(ismacro(@arg$$)){\n\
\tmacrowrite(@tmpfile$$,@arg$$,name:\"macro_to_edit\",stripdol:@strip$$,new:T)\n\
}else{\n\
\t@vector$$ <- isscalar(@arg$$) && @arg$$[1] == 0\n\
\tif(!@vector$$){\n\
\t\tmatwrite(@tmpfile$$,edited:@arg$$,new:T)\n\
\t}\n\
}\n\
shell(paste(@editor$$,@tmpfile$$),interact:T)\n\
if(ismacro(@arg$$)){\n\
\t@arg$$ <- macroread(@tmpfile$$,quiet:T)\n\
}else{\n\
\tif(!@vector$$){\n\
\t\t@arg$$ <- matread(@tmpfile$$,quiet:T)\n\
\t}else{\n\
\t\t@arg$$ <- vecread(@tmpfile$$,quiet:T)\n\
\t}\n\
\tdelete(@vector$$)\n\
}\n\
shell(paste(@delete$$,@tmpfile$$),interact:T)\n\
delete(@delete$$, @tmpfile$$, @editor$$)\n\
if(delete(@save$$,return:T)){\n\
\t$01 <- delete(@arg$$,return:T);;\n\
}else{\n\
\tdelete(@arg$$,return:T)\n\
}", /*MEND*/

#elif defined(BASE_UNIX) /* COMP_DJGPP || SKEL_WXWIN && OS_WIN32 */


/*MNAME*/ "edit",
/*MTEXT*/ "# $S(realVar), $S(macro [,stripdol:F]), $S(), or $S(0)\n\
@editor$$ <- \"vi\"\n\
@tmpfile$$ <- \"/tmp/macanova.\"\n\
@delete$$ <- \"rm\"\n\
if(isdefined(EDITOR) && ischar(EDITOR) && isscalar(EDITOR)){\n\
\t@editor$$ <- EDITOR\n\
}\n\
@strip$$ <- keyvalue($K,\"strip*\",\"TF\",default:T)\n\
@arg$$ <- if($v == 0){\n\
\tmacro(\"====> Replace this line with lines of your macro <====\")\n\
}else{\n\
\targvalue($01,\"argument 1\")\n\
}\n\
\n\
if($v > 2 || (!ismacro(@arg$$) && !isreal(@arg$$))){\n\
\terror(\"usage: $S(realVar), $S(macro [,stripdol:F]), $S(), or $S(0)\")\n\
}\n\
@save$$ <- if($v == 2){\n\
\targvalue($02,\"argument 2\",\"TF\")\n\
}else{\n\
\tF\n\
}\n\
@tmpfile$$ <- paste(@tmpfile$$,round(100000*runi(1)),sep:\"\")\n\
if(ismacro(@arg$$)){\n\
\tmacrowrite(@tmpfile$$,@arg$$,name:\"macro_to_edit\",stripdol:@strip$$,new:T)\n\
}else{\n\
\t@vector$$ <- isscalar(@arg$$) && @arg$$[1] == 0\n\
\tif(!@vector$$){\n\
\t\tmatwrite(@tmpfile$$,edited:@arg$$,new:T)\n\
\t}\n\
}\n\
shell(paste(@editor$$,@tmpfile$$),interact:T)\n\
if(ismacro(@arg$$)){\n\
\t@arg$$ <- macroread(@tmpfile$$,quiet:T)\n\
}else{\n\
\tif(!@vector$$){\n\
\t\t@arg$$ <- matread(@tmpfile$$,quiet:T)\n\
\t}else{\n\
\t\t@arg$$ <- vecread(@tmpfile$$,quiet:T)\n\
\t}\n\
\tdelete(@vector$$)\n\
}\n\
shell(paste(@delete$$,@tmpfile$$),interact:T)\n\
delete(@delete$$, @tmpfile$$, @editor$$)\n\
if(delete(@save$$,return:T)){\n\
\t$01 <- delete(@arg$$,return:T);;\n\
}else{\n\
\tdelete(@arg$$,return:T)\n\
}", /*MEND*/

#endif /*COMP_DJGPP || SKEL_WXWIN && OS_WIN32*/
#endif /*DEFINEEDIT*/

#ifdef DEFINEMORE
#ifdef BASE_UNIX

/*MNAME*/ "more",
/*MTEXT*/ "# $S(var) or $S(macro [,stripdol:F])\n\
if($v != 1){\n\
\terror(\"usage: $S(x [,keywords])\")\n\
}\n\
@x$$ <- argvalue($1, \"$1\")\n\
if(!ismacro(@x$$) && !isreal(@x$$) && !ischar(@x$$) && !islogic(@x$$)){\n\
\terror(\"$1 is not a macro or a REAL, LOGICAL, or CHARACTER variable\")\n\
}\n\
@strip$$ <- keyvalue($K,\"strip*\",\"TF\",default:T)\n\
@tmpfile$$ <- paste(\"/tmp/macanova.\",round(100000*runi(1)),sep:\"\")\n\
@pager$$ <- if(isdefined(PAGER) && ischar(PAGER) && isscalar(PAGER)){\n\
\tPAGER\n\
}else{\n\
\t\"more\"\n\
}\n\
if(ismacro(@x$$)){\n\
\tmacrowrite(@tmpfile$$,name:\"$1\",@x$$,stripdol:@strip$$,new:T)\n\
}elseif($k == 0){\n\
\tprint(file:@tmpfile$$,name:\"$1\",@x$$,new:T)\n\
}else{\n\
\tprint(file:@tmpfile$$,name:\"$1\",@x$$,$K,new:T)\n\
}\n\
shell(paste(@pager$$,@tmpfile$$, \"; rm\", @tmpfile$$),interact:T)\n\
delete(@x$$, @tmpfile$$, @pager$$, @strip$$)", /*MEND*/
#endif /*BASE_UNIX*/

#endif /*DEFINEMORE*/
/*NOTFORMACEND*/


/*MNAME*/ "console",
/*MTEXT*/ "# usage: y <- $S() reads from keyboard or following lines in batch file\n\
if($k==0){\n\
\tvecread(CONSOLE)\n\
}else{\n\
\tvecread(CONSOLE,$K)\n\
}", /*MEND*/


/*MNAME*/ "clipreaddata",
/*MTEXT*/ "# clipreaddata(name1,...,namek [,echo:T or F]), unquoted variable names\n\
# clipreaddata([,echo:T or F]), names from 1st line of CLIPBOARD\n\
if (!ismacro(readdata)){\n\
\tgetmacros(readdata,quiet:T,printname:F)\n\
}\n\
if ($N > 0) {\n\
\treaddata(string:CLIPBOARD,$0)\n\
} else {\n\
\treaddata(string:CLIPBOARD)\n\
}", /*MEND*/


/*MNAME*/ "clipwritedat",
/*MTEXT*/ "# $S(x1,x2,a,b,... [,missing:M] [, putnames:F] \\\n\
#        [,fieldwidth:w  or  format:fmt])\n\
if (!ismacro(writedata)) {\n\
\tgetmacros(writedata,silent:T)\n\
}\n\
CLIPBOARD <- writedata($0, keep:T);;", /*MEND*/


/*MNAME*/ "fromclip",
/*MTEXT*/ "# usage: x <- $S([bywords:T  or  bylines:T])\n\
#    or  x <- $S(ncols [, bywords:T  or  bylines:T])\n\
if($v==0){\n\
\tif ($k > 0){\n\
\t\tvecread(string:CLIPBOARD,$K)\n\
\t}else{\n\
\t\tvecread(string:CLIPBOARD)}\n\
}else{\n\
\t@n$$ <- argvalue($01,\"number of columns\",\"positive integer scalar\")\n\
\tif ($k > 0){\n\
\t\tmatrix(vecread(string:CLIPBOARD,$K),@n$$)'\n\
\t}else{\n\
\t\tmatrix(vecread(string:CLIPBOARD),@n$$)'\n\
\t}\n\
}", /*MEND*/


/*MNAME*/ "toclip",
/*MTEXT*/ "#usage: $S(x)  or (e.g.)  $S(x,format:\"6.3f\",missing:\"NA\",linesep:\";\")\n\
if($v != 1){\n\
\terror(\"usage is $S(x [,keyword phrases])\", macroname:F)\n\
}\n\
CLIPBOARD <- if($k == 0){\n\
\t$01\n\
}else{\n\
\t@format$$ <- keyvalue($K,\"format\",\"string\",default:\".17g\")\n\
\t@missing$$ <- keyvalue($K,\"missing\",\"string\",default:\"?\")\n\
\t@sep$$ <- keyvalue($K,\"sep*\",\"string\",default:\"\\t\")\n\
\t@linesep$$ <- keyvalue($K,\"linesep*\",\"string\",default:\"\\n\")\n\
\tpaste($01,multiline:T,sep:delete(@sep$$,return:T),\\\n\
\t\tformat:delete(@format$$,return:T),missing:delete(@missing$$,return:T),\\\n\
\t\tlinesep:delete(@linesep$$,return:T))\n\
};;", /*MEND*/


/*LOCKED*/

/*MNAME*/ "&enter",
/*MTEXT*/ "# usage: x <- $S(3.1 2.75 3.12 4.5) (no commas needed)\n\
vecread(string:\"$0\")", /*MEND*/


/*LOCKED*/

/*MNAME*/ "&enterchars",
/*MTEXT*/ "# usage: x <- $S(weight age) (use no quotes, commas)\n\
vecread(string:\"$0\",bywords:T)", /*MEND*/


/*LOCKED*/

/*MNAME*/ "&%haslabels",
/*MTEXT*/ "!isnull(getlabels($1,1,silent:T))", /*MEND*/


/*LOCKED*/

/*MNAME*/ "&%hasnotes",
/*MTEXT*/ "# notes <- if(hasnotes(x)){getnotes(x)}else{\"no notes available\"}\n\
!isnull(getnotes($1,silent:T))", /*MEND*/

/*
\t981121 added keyword phrase nofileok:T to calls to macroread()
\t990929 getmacros() now accepts quoted macro names as arguments
\t       and does limited checking that the names are legal names
*/

/*MNAME*/ "%getmacros",
/*MTEXT*/ "#$S(macro1,macro2,...[,quiet:T]) retrieves macros from MACROFILES\n\
if($v==0){\n\
\terror(\"must specify at least one macro name\")\n\
}\n\
if(!isdefined(MACROFILES) && (!isdefined(MACROFILE))){\n\
\terror(\"neither MACROFILES nor MACROFILE exists\")\n\
}\n\
@files$$ <- if(isdefined(MACROFILES)){\n\
\tMACROFILES\n\
}else{\n\
\tMACROFILE\n\
}\n\
@silent$$ <- keyvalue($K, \"silent\",\"TF\",default:F)\n\
@printname$$ <- keyvalue($K, \"printname\",\"TF\",default:!@silent$$)\n\
\n\
@nf$$ <- length(@files$$)\n\
@args$$ <- $A\n\
for(@i$$,run($v)){\n\
\t@macname$$ <- @args$$[@i$$]\n\
\tif (!isname(@macname$$)){\n\
\t\t# maybe the argument was \"macroname\"\n\
\t\tif(match(\"\\\"*\\\"\",@macname$$,0,exact:F) != 0){\n\
\t\t\t@macname$$ <- <<@macname$$>>\n\
\t\t}\n\
\t\tif (!isname(@macname$$)){\n\
\t\t\terror(paste(\"'\",@macname$$, \"' is not a legal macro name\",sep:\"\"))\n\
\t\t}\n\
\t}\n\
\tfor (@j$$,run(@nf$$)){\n\
\t\t<<@macname$$>> <- if($k==0){\n\
\t\t\tmacroread(@files$$[@j$$],@macname$$,\\\n\
\t\t\t\tsilent:F,printname:F,nofileok:T,notfoundok:T)\n\
\t\t}else{\n\
\t\t\tmacroread(@files$$[@j$$],@macname$$,\\\n\
\t\t\t\tprintname:F,badkeyok:T,nofileok:T,notfoundok:T,\\\n\
\t\t\t\t\tsilent:@silent$$,$K)\n\
\t\t}\n\
\t\tif (!isnull(<<@macname$$>>)){\n\
\t\t\tif (@printname$$){\n\
\t\t\t\tprint(paste(@macname$$,\" read from file \\\"\",\\\n\
\t\t\t\t\tgetfilename(last:T),\"\\\"\",sep:\"\"))\n\
\t\t\t}\n\
\t\t\tbreak\n\
\t\t}\n\
\t}\n\
\tif (isnull(<<@macname$$>>)){\n\
\t\tprint(paste(\"WARNING:\",@macname$$,\"not found\"))\n\
\t\tdelete(<<@macname$$>>)\n\
\t}\n\
}\n\
delete(@macname$$,@files$$,@i$$,@j$$,@nf$$,@args$$,@silent$$,@printname$$)", /*MEND*/


/*MNAME*/ "addmacrofile",
/*MTEXT*/ "# $S(fileName [,atEnd] [,quiet:T]), atEnd = T or F\n\
@newfiles$$ <- argvalue($1,\"macro file name(s)\",\"character vector\")\n\
@atend$$ <- if($v > 1){\n\
\targvalue($2,\"argument 2\",\"TF\")\n\
}else{\n\
\tF\n\
}\n\
@quiet$$ <- keyvalue($K,\"quiet\",\"TF\",default:F)\n\
\n\
for (@i$$,1,length(@newfiles$$)){\n\
\tif (@newfiles$$[@i$$] == \"\"){\n\
\t\tif (ncomps(GRAPHWINDOWS) <= 1){\n\
\t\t\terror(\"null file name \\\"\\\" illegal in this version\")\n\
\t\t}\n\
\t\t@newfiles$$[@i$$] <- getfilename(type:\"text\")\n\
\t}\n\
}\n\
if (!@quiet$$) {\n\
\tfor (@i$$,1,length(@newfiles$$)) {\n\
\t\tprint(paste(\"Added macro file \\\"\",@newfiles$$[@i$$],\"\\\"\",sep:\"\"))\n\
\t}\n\
}\n\
\n\
MACROFILES <- if (!isdefined(MACROFILES)){\n\
\t@newfiles$$\n\
}elseif(@atend$$){\n\
\tvector(MACROFILES,@newfiles$$)\n\
}else{\n\
\tvector(@newfiles$$,MACROFILES)\n\
}\n\
delete(@newfiles$$,@atend$$, @quiet$$)", /*MEND*/


/*MNAME*/ "%addhelpfile",
/*MTEXT*/ "# $S(fileName [,T]) or $S(vector(fileName,indexName) [,T])\n\
@names$$ <- argvalue($1,\"help file name(s)\",\"character vector\")\n\
@atend$$ <- if($v > 1){\n\
\targvalue($2,\"argument 2\",\"TF\")\n\
}else{\n\
\tF\n\
}\n\
if (length(@names$$) > 2){\n\
\terror(\"argument 1 not CHARACTER scalar or vector of length 2\")\n\
}\n\
@newfile$$ <- @names$$[1]\n\
@newindex$$ <- if (isscalar(@names$$)){\n\
\t\"index\"\n\
} else {\n\
\tif (@names$$[2] == \"\"){\n\
\t\terror(\"null index name \\\"\\\"\")\n\
\t} else {\n\
\t\t@names$$[2]\n\
\t}\n\
}\n\
delete(@names$$)\n\
if (@newfile$$ == \"\"){\n\
\tif (ncomps(GRAPHWINDOWS) <= 1){\n\
\t\terror(\"null file name \\\"\\\" not legal in this version\")\n\
\t}\n\
\t@newfile$$ <- getfilename(type:\"text\")\n\
}\n\
if (isdefined(HELPFILES) && !isdefined(HELPINDICES)) {\n\
\tHELPINDICES <- rep(\"index\", length(HELPFILES))\n\
}\n\
HELPFILES <- if (!isdefined(HELPFILES)){\n\
\t@newfile$$\n\
}elseif(@atend$$){\n\
\tvector(HELPFILES,@newfile$$)\n\
}else{\n\
\tvector(@newfile$$,HELPFILES)\n\
}\n\
HELPINDICES <- if (!isdefined(HELPINDICES)){\n\
\t@newindex$$\n\
}elseif(@atend$$) {\n\
\tvector(HELPINDICES,@newindex$$)\n\
}else{\n\
\tvector(@newindex$$,HELPINDICES)\n\
}\n\
delete(@newfile$$,@newindex$$,@atend$$)\n\
if (length(HELPINDICES) != length(HELPFILES)){\n\
\tprint(\"WARNING: HELPINDICES and HELPFILES have different lengths\",\\\n\
\t\tmacroname:T)\n\
}", /*MEND*/


/*MNAME*/ "adddatapath",
/*MTEXT*/ "# $S(dirName) or $S(dirName,atEnd), atEnd T or F\n\
@newpaths$$ <- argvalue($1,\"argument 1\",\"character vector\")\n\
@atend$$ <- if($v > 1){\n\
\targvalue($02,\"argument 2\",\"TF\")\n\
}else{\n\
\tF\n\
}\n\
for (@i$$,1,length(@newpaths$$)){\n\
\tif (@newpaths$$[@i$$] == \"\"){\n\
\t\tif (ncomps(GRAPHWINDOWS) <= 1){\n\
\t\t\terror(\"null path name \\\"\\\" illegal in this version\")\n\
\t\t}\n\
\t\t@newpaths$$[@i$$] <- getfilename(pathonly:T)\n\
\t}\n\
}\n\
DATAPATHS <- if (!isdefined(DATAPATHS)){\n\
\t@newpaths$$\n\
}elseif(@atend$$){\n\
\tvector(DATAPATHS,@newpaths$$)\n\
}else{\n\
\tvector(@newpaths$$,DATAPATHS)\n\
}\n\
delete(@newpaths$$,@atend$$)", /*MEND*/


/*MNAME*/ "%getdata",
/*MTEXT*/ "# y <- $S(datasetName [,quiet:T]) retrieves data set from DATAFILE\n\
if($v > 1){\n\
\terror(\"usage: y <- $S(datasetName, [,quiet:T])\",macroname:F)\n\
}\n\
if($v==1){\n\
\t@set$$ <- if(match(\"\\\"*\\\"\", \"$1\", 0, exact:F) != 0){\n\
\t\t$1\n\
\t}else{\n\
\t\t\"$1\"\n\
\t}\n\
\t@data$$ <- if($k==0){\n\
\t\tmatread(DATAFILE,@set$$)\n\
\t}else{\n\
\t\tmatread(DATAFILE,@set$$,$K)\n\
\t}\n\
\tdelete(@set$$)\n\
}else{\n\
\t@data$$ <- if($k==0){\n\
\t\tmatread(DATAFILE)\n\
\t}else{\n\
\t\tmatread(DATAFILE,$K)\n\
\t}\n\
}\n\
delete(@data$$,return:T)", /*MEND*/


/*MNAME*/ "%help",
/*MTEXT*/ "#$S(topic), $S(topic, subtopic:\"?\") or $S(topic, subtopic:\"subtopic_name\")\n\
if ($N == 0){\n\
\tgethelp()\n\
\treturn\n\
}\n\
if (!isdefined(HELPFILES)){\n\
\tprint(\"WARNING: No variable HELPFILES; using gethelp()\", macroname:T)\n\
\tgethelp($0)\n\
\treturn\n\
}\n\
@what$$ <- \"$S\"\n\
@news$$ <- match(\"news*\",$A,0,exact:F) != 0\n\
@ntopics$$ <- $N\n\
@reset$$ <- @allfiles$$ <- NULL\n\
if ($k > 0){\n\
\t@keys$$ <- structure($K)\n\
\t@index$$ <- keyvalue(@keys$$, \"index\", \"string\")\n\
\tif (!isnull(@index$$) && $N > 1) {\n\
\t\terror(\"No other arguments allowed with keyword 'index'\")\n\
\t}\n\
\t@file$$ <- keyvalue(@keys$$, \"file\",\"string\")\n\
\t@alt$$ <- keyvalue(@keys$$, \"alt\",\"TF\")\n\
\t@orig$$ <- keyvalue(@keys$$, \"orig\",\"TF\")\n\
\t@key$$ <- keyvalue(@keys$$, \"key\",\"character vector\")\n\
\t@k$$ <- sum(vector(!isnull(@file$$), !isnull(@alt$$), !isnull(@orig$$),\\\n\
\t\t!isnull(@key$$)))\n\
\t@nosearch$$ <- @k$$ > 0\n\
\t@ntopics$$ <-- delete(@k$$, return:T)\n\
\tif (!@nosearch$$) {\n\
\t\t@allfiles$$ <- keyvalue(@keys$$, \"allf*\", \"TF\")\n\
\t\tif (!isnull(@allfiles$$)){\n\
\t\t\t@ntopics$$ <-- 1\n\
\t\t}\n\
\t\t@reset$$ <- keyvalue(@keys$$, \"reset\", \"TF\")\n\
\t\tif (!isnull(@reset$$)){\n\
\t\t\t@ntopics$$ <-- 1\n\
\t\t}\n\
\t\t@usage$$ <- keyvalue(@keys$$, \"usage\", \"TF\")\n\
\t\tif (!isnull(@usage$$)){\n\
\t\t\t@ntopics$$ <-- 1\n\
\t\t\tif (@usage$$) {\n\
\t\t\t\t@what$$ <- \"usage\"\n\
\t\t\t}\n\
\t\t}\n\
\t\tif (!isnull(keyvalue(@keys$$, \"scroll*\",\"TF\"))){\n\
\t\t\t@ntopics$$ <-- 1\n\
\t\t}\n\
\t\t@silent$$ <- keyvalue(@keys$$, \"silent*\",\"TF\")\n\
\t\tif (!isnull(@silent$$)){\n\
\t\t\t@ntopics$$ <-- 1\n\
\t\t} else {\n\
\t\t\t@silent$$ <- F\n\
\t\t}\n\
\t\tif (!isnull(keyvalue(@keys$$, \"printname*\",\"TF\"))){\n\
\t\t\t@ntopics$$ <-- 1\n\
\t\t}\n\
\t\tif (!isnull(keyvalue(@keys$$, \"news\",\"positive integer vector\"))){\n\
\t\t\t@ntopics$$ <-- 1\n\
\t\t}\n\
\t\tif (!isnull(keyvalue(@keys$$, \"subtopic*\",\"character vector\"))){\n\
\t\t\t@ntopics$$ <-- 1\n\
\t\t}\n\
\t}\n\
\tdelete(@keys$$,@file$$,@alt$$,@orig$$,@key$$)\n\
} else {\n\
\t@index$$ <- NULL\n\
\t@silent$$ <- @nosearch$$ <- F\n\
}\n\
\t\t\t\n\
if (delete(@nosearch$$, return:T)) {\n\
\tdelete(@what$$,@news$$,@ntopics$$)\n\
\tgethelp($0)\n\
} elseif (!isvector(HELPFILES, char:T)) {\n\
\terror(\"HELPFILES not a CHARACTER vector\")\n\
} elseif (sum(HELPFILES != \"\") == 0) {\n\
\t\terror(\"All elements of HELPFILES are \\\"\\\"\")\n\
} else {\n\
\tif (isnull(@allfiles$$)){\n\
\t\t@allfiles$$ <- @ntopics$$ > 1\n\
\t}\n\
\t@orighelp$$ <- getfilename(help:T)\n\
\t@which$$ <- match(@orighelp$$, HELPFILES, 0)\n\
\tif (!isnull(@index$$)){\n\
\t\tif (!isvector(HELPINDICES, char:T)){\n\
\t\t\terror(\"$S(index:\\\"name\\\") when HELPINDICES does not exist\")\n\
\t\t}\n\
\t\tif (@index$$ == \"\"){\n\
\t\t\terror(\"\\\"\\\" is not a legal value for 'index'\")\n\
\t\t}\n\
\t\t@J$$ <- match(@index$$,HELPFILES, 0, ignorecase:T)\n\
\n\
\t\tif (@J$$ == 0) { # does it end in name?\n\
\t\t\t@J$$ <- match(paste(\"*\",@index$$,sep:\"\"),HELPFILES,0,\\\n\
\t\t\t\texact:F, ignorecase:T)\n\
\t\t}\n\
\t\tif (@J$$ == 0) { #does it end in name + extension?\n\
\t\t\t@J$$ <- match(paste(\"*\",@index$$,\".???\",sep:\"\"),HELPFILES,0,\\\n\
\t\t\t\texact:F, ignorecase:T)\n\
\t\t}\n\
\t\tif (@J$$ == 0) { #does it include name\n\
\t\t\t@J$$ <- match(paste(\"*\",@index$$,\"*\",sep:\"\"),HELPFILES,0,\\\n\
\t\t\t\texact:F, ignorecase:T)\n\
\t\t}\n\
\t\tif (@J$$ == 0){\n\
\t\t\terror(paste(\"\\\"\",@index$$,\\\n\
\t\t\t\t\"\\\" does not match any help file name\",sep:\"\"))\n\
\t\t}\n\
\t\t@index$$ <- HELPINDICES[@J$$]\n\
\t\tif (@J$$ == @which$$){\n\
\t\t\t@result$$ <- gethelp(HELPINDICES[@J$$], silent:T)\n\
\t\t} else {\n\
\t\t\t@result$$ <- gethelp(file:HELPFILES[@J$$], HELPINDICES[@J$$], silent:T)\n\
\t\t\tgethelp(file:@orighelp$$)\n\
\t\t}\n\
\t\tif (!@result$$ && !@silent$$){\n\
\t\t\tprint(paste(\"WARNING: topic\",HELPINDICES[@J$$], \"in file\",\\\n\
\t\t\t\tHELPFILES[@J$$],\"not found\"), macroname:T)\n\
\t\t}\n\
\t\tdelete(@index$$, @J$$)\n\
\t} else {\n\
\t\t@helpfiles$$ <- HELPFILES\n\
\t\tif (@which$$ > 0 && !delete(@news$$, return:T)){\n\
\t\t\t@helpfiles$$ <- rotate(@helpfiles$$, -@which$$ + 1)\n\
\t\t}\n\
\t\t@result$$ <- F\n\
\t\tif (delete(@which$$,return:T) == 0) {\n\
\t\t\t@result$$ <- gethelp(file:@orighelp$$,$0,silent:T,printname:@allfiles$$)\n\
\t\t}\n\
\t\tif (!@result$$ || @allfiles$$) {\n\
\t\t\tfor (@i$$, 1,length(@helpfiles$$)) {\n\
\t\t\t\t@result$$ <- @result$$ ||\\\n\
\t\t\t\t\tgethelp(file:@helpfiles$$[@i$$],$0,silent:T,printname:@allfiles$$)\n\
\t\t\t\tif (!@allfiles$$ && @result$$) {\n\
\t\t\t\t\tbreak\n\
\t\t\t\t}\n\
\t\t\t}\n\
\t\t\tdelete(@i$$)\n\
\t\t}\n\
\t\tif (!@result$$){\n\
\t\t\tif (!@silent$$) {\n\
\t\t\t\tprint(paste(\"WARNING: no\",@what$$,\"found\"), macroname:T)\n\
\t\t\t}\n\
\t\t\tif (isnull(@reset$$)) {\n\
\t\t\t\t@reset$$ <- T\n\
\t\t\t}\n\
\t\t} \n\
\t\tif (isnull(@reset$$)){\n\
\t\t\t@reset$$ <- @allfiles$$\n\
\t\t}\n\
\t\tif (delete(@reset$$, return:T)) {\n\
\t\t\tgethelp(file:@orighelp$$, silent:T)\n\
\t\t}\n\
\t\tdelete(@helpfiles$$,@what$$, @orighelp$$, @silent$$, @allfiles$$)\n\
\t}\n\
\tdelete(@result$$, return:T,invisible:T)\n\
}", /*MEND*/


/*MNAME*/ "%usage",
/*MTEXT*/ "if ($N > 0) {\n\
\tif (!ismacro(help)){\n\
\t\tgetmacros(help, silent:T)\n\
\t}\n\
\t@usagekey$$ <- if ($k > 0){\n\
\t\t!isnull(keyvalue($K,\"usage\",\"TF\"))\n\
\t} else {\n\
\t\tF\n\
\t}\n\
\tif (!delete(@usagekey$$, return:T)){\n\
\t\thelp($0, usage:T)\n\
\t} else {\n\
\t\thelp($0)\n\
\t}\n\
} else {\n\
\tgetusage()\n\
}", /*MEND*/


/*MNAME*/ "designhelp",
/*MTEXT*/ "# usage $S(topic1 [, topic2 ...] [help keywords])\n\
#       $S(topic1 [, topic2 ...], usage:T) with no other keywords\n\
#       $S(index:T)\n\
if(!ismacro(_gethelp)){\n\
\tgetmacros(_gethelp,quiet:T,printname:F)\n\
}\n\
___HELPFILE_ <- \"design.hlp\"\n\
___INDEXTOP_ <- \"design_index\"\n\
___MACRO_ <- \"$S\"\n\
_gethelp($0)", /*MEND*/


/*MNAME*/ "tserhelp",
/*MTEXT*/ "# usage $S(topic1 [, topic2 ...] [help keywords])\n\
#       $S(topic1 [, topic2 ...], usage:T) with no other keywords\n\
#       $S(index:T)\n\
if(!ismacro(_gethelp)){\n\
\tgetmacros(_gethelp,quiet:T,printname:F)\n\
}\n\
___HELPFILE_ <- \"tser.hlp\"\n\
___INDEXTOP_ <- \"tser_index\"\n\
___MACRO_ <- \"$S\"\n\
_gethelp($0)", /*MEND*/


/*MNAME*/ "arimahelp",
/*MTEXT*/ "# usage $S(topic1 [, topic2 ...] [help keywords])\n\
#       $S(topic1 [, topic2 ...], usage:T) with no other keywords\n\
#       $S(index:T)\n\
if(!ismacro(_gethelp)){\n\
\tgetmacros(_gethelp,quiet:T,printname:F)\n\
}\n\
___HELPFILE_ <- \"arima.mac\"\n\
___INDEXTOP_ <- \"arima_index\"\n\
___MACRO_ <- \"$S\"\n\
_gethelp($0)", /*MEND*/


/*MNAME*/ "mulvarhelp",
/*MTEXT*/ "# usage $S(topic1 [, topic2 ...] [help keywords])\n\
#       $S(topic1 [, topic2 ...], usage:T) with no other keywords\n\
#       $S(index:T)\n\
if(!ismacro(_gethelp)){\n\
\tgetmacros(_gethelp,quiet:T,printname:F)\n\
}\n\
___HELPFILE_ <- \"mulvar.mac\"\n\
___INDEXTOP_ <- \"mulvar_index\"\n\
___MACRO_ <- \"$S\"\n\
_gethelp($0)", /*MEND*/


/*MNAME*/ "mathhelp",
/*MTEXT*/ "# usage $S(topic1 [, topic2 ...] [help keywords])\n\
#       $S(topic1 [, topic2 ...], usage:T) with no other keywords\n\
#       $S(index:T)\n\
if(!ismacro(_gethelp)){\n\
\tgetmacros(_gethelp,quiet:T,printname:F)\n\
}\n\
___HELPFILE_ <- \"math.mac\"\n\
___INDEXTOP_ <- \"math_index\"\n\
___MACRO_ <- \"$S\"\n\
_gethelp($0)", /*MEND*/


/*MNAME*/ "graphicshelp",
/*MTEXT*/ "# usage $S(topic1 [, topic2 ...] [help keywords])\n\
#       $S(topic1 [, topic2 ...], usage:T) with no other keywords\n\
#       $S(index:T)\n\
if(!ismacro(_gethelp)){\n\
\tgetmacros(_gethelp,quiet:T,printname:F)\n\
}\n\
___HELPFILE_ <- \"graphics.mac\"\n\
___INDEXTOP_ <- \"graph_index\"\n\
___MACRO_ <- \"$S\"\n\
_gethelp($0)", /*MEND*/


/*MNAME*/ "regresshelp",
/*MTEXT*/ "# usage $S(topic1 [, topic2 ...] [help keywords])\n\
#       $S(topic1 [, topic2 ...], usage:T) with no other keywords\n\
#       $S(index:T)\n\
if(!ismacro(_gethelp)){\n\
\tgetmacros(_gethelp,quiet:T,printname:F)\n\
}\n\
___HELPFILE_ <- \"regress.mac\"\n\
___INDEXTOP_ <- \"regress_index\"\n\
___MACRO_ <- \"$S\"\n\
_gethelp($0)", /*MEND*/


/*MNAME*/ "userfunhelp",
/*MTEXT*/ "# usage $S(topic1 [, topic2 ...] [help keywords])\n\
#       $S(topic1 [, topic2 ...], usage:T) with no other keywords\n\
#       $S(index:T)\n\
if(!ismacro(_gethelp)){\n\
\tgetmacros(_gethelp,quiet:T,printname:F)\n\
}\n\
___HELPFILE_ <- \"userfun.hlp\"\n\
___INDEXTOP_ <- \"userfun_index\"\n\
___MACRO_ <- \"$S\"\n\
_gethelp($0)", /*MEND*/


/*MNAME*/ "_gethelp",
/*MTEXT*/ "@file$$ <- delete(___HELPFILE_,return:T)\n\
@indextop$$ <- delete(___INDEXTOP_,return:T)\n\
@macro$$ <- delete(___MACRO_,return:T)\n\
@keys$$ <- if ($k > 0){\n\
\tstructure($K)\n\
} else {\n\
\tstructure(notakey:NULL)\n\
}\n\
@usage$$ <- keyvalue(@keys$$,\"usage\",\"TF\")\n\
@index$$ <- keyvalue(@keys$$,\"index\",\"TF\")\n\
@scrollbk$$ <- keyvalue(@keys$$,\"scrollback\",\"TF\")\n\
@key$$ <- keyvalue(@keys$$,\"key\",\"string\")\n\
@subtopic$$ <- keyvalue(@keys$$,\"subtop*\",\"character vector\")\n\
@news$$ <- keyvalue(@keys$$,\"news\")\n\
@badkeys$$ <- vector(\"orig*\",\"file\",\"alt*\")\n\
\n\
@J$$ <- unique(match(@badkeys$$,compnames(@keys$$),0,exact:F))\n\
@J$$ <- @J$$[@J$$ > 0]\n\
if (!isnull(@J$$)){\n\
\terror(paste(\"keyword '\",compnames(@keys$$)[@J$$[1]],\"' illegal on \",\\\n\
\t\t@macro$$, sep:\"\"), macroname:F)\n\
}\n\
delete(@badkeys$$,@J$$, @keys$$)\n\
\n\
@speckeys$$ <- (!isnull(@usage$$)) + (!isnull(@index$$)) +\\\n\
\t(!isnull(@scrollbk$$)) + (!isnull(@key$$)) + (!isnull(@subtopic$$))\n\
\n\
if ($k == $N && ($k > 0 && @speckeys$$ == 0 ||\\\n\
\t$k > 1 && @speckeys$$ == 1 && !isnull(@scrollbk$$))){\n\
\tgethelp(file:@file$$,$K)\n\
} else {\t\n\
\tif (!isnull(@subtopic$$)) {\n\
\t\tif (!isnull(@usage$$) || !isnull(@index$$) || !isnull(@key$$) ||\\\n\
\t\t\t!isnull(@news$$)){\n\
\t\t\terror(\"'subtopic' illegal with 'usage', 'index', 'key' and 'news'\")\n\
\t\t}\n\
\t\tif ($v != 1){\n\
\t\t\terror(\"with keyword 'subtopic', exactly one topic must specified\")\n\
\t\t}\n\
\t}\n\
\tif (!isnull(@usage$$) && (!isnull(@index$$) || !isnull(@key$$) || !isnull(@news$$))){\n\
\t\terror(\"'usage' illegal with 'index', 'key' or 'news'\")\n\
\t}\n\
\tif (!isnull(@key$$) && (!isnull(@index$$) || !isnull(@news$$))){\n\
\t\terror(\"'key' illegal with 'index' or 'news'\")\n\
\t}\n\
\tif (!isnull(@index$$) && !isnull(@news$$)){\n\
\t\terror(\"'index' illegal with 'news'\")\n\
\t}\n\
\tif (isnull(@usage$$)){\n\
\t\t@usage$$ <- F\n\
\t}\n\
\tif (isnull(@index$$)){\n\
\t\t@index$$ <- $v == 0 && isnull(@key$$) && isnull(@subtopic$$)\n\
\t} elseif (!@index$$) {\n\
\t\tif ($v == 0 && $k == 1 && isnull(@key$$)) {\n\
\t\t\terror(paste(\"'index:F' but no topics specified for\",@macro$$),\\\n\
\t\t\t\tmacroname:F)\n\
\t\t}\n\
\t} else {\n\
\t\tif ($v > 0 || $k > 2 || $k == 2 && isnull(@scrollbk$$)){\n\
\t\t\terror(\"scrollback:T only other argument allowed with 'index:T'\")\n\
\t\t}\n\
\t}\n\
\tif (isnull(@scrollbk$$)){\n\
\t\t@scrollbk$$ <- F\n\
\t}\n\
\tif (isnull(@key$$)){\n\
\t\tif (!@index$$){\n\
\t\t\tgethelp(file:@file$$,$0)\n\
\t\t} else {\n\
\t\t\tif (@scrollbk$$){\n\
\t\t\t\tgethelp(file:@file$$,paste(@indextop$$),scrollback:@scrollbk$$)\n\
\t\t\t}else{\n\
\t\t\t\tgethelp(file:@file$$,paste(@indextop$$))\n\
\t\t\t}\n\
\t\t}\n\
\t} else {\n\
\t\t# ignore @scrollback$$ because of the trailing message which is needed\n\
\t\t# since gethelp() gives instructions on what to do\n\
\t\tgethelp(file:@file$$,key:@key$$)\n\
\t\t@what$$ <- if (@key$$ == \"?\"){\n\
\t\t\t\"these headings\"\n\
\t\t} else {\n\
\t\t\t\"these topics\"\n\
\t\t}\n\
\t\tprint(paste(\"Use '\",@macro$$,\"' instead of 'help' for \",@what$$, sep:\"\"))\n\
\t}\n\
}\n\
gethelp(orig:T) # reset help file to standard\n\
delete(@usage$$,@file$$,@indextop$$,@macro$$,@key$$, @index$$)", /*MEND*/

/*NOTFORMACSTART*/ /*macros up to NOTFORMACEND are not defined on Macintosh*/
#ifdef TEK
/* Intended to put terminal emulator in Tektronix 40xx mode */

/*MNAME*/ "tek",
/*MTEXT*/ "putascii(getascii(getoptions(tekset:T)[1]))", /*MEND*/

/* Puts xterm in Tek4014 mode  (obsoleted by new version of tek)*/

/*MNAME*/ "tekx",
/*MTEXT*/ "putascii(vector(27,91,63,51,56,104,27,56))", /*MEND*/

/* Intended to put terminal emulator in VT100 mode */

/*MNAME*/ "vt",
/*MTEXT*/ "putascii(getascii(getoptions(tekset:T)[2]))", /*MEND*/

/* puts xterm in vt100 mode (obsoleted by new version of vt)*/

/*MNAME*/ "vtx",
/*MTEXT*/ "putascii(vector(27,3))", /*MEND*/

#endif /*TEK*/
/*NOTFORMACEND*/
#endif /*BCPP*/
/*ENDMACROS*/
	(char *) 0, (char *) 0
