#! /bin/sh

# sh script to creat iniMacro.h from macro file iniMacro.mac

# usage: iniMacro.sh iniMacro.mac [out.h]  (default for out.h is iniMacro.h)

# So that iniMacro.h can be used as input to iniMacro.awk in creating
# macIniMacro.r, iniMacro.mac must include certain comments and
# C preprocessor directives.

# The macros themselves are in the standard form readable by macroread.

# Everything through a line containing Start_Of_Macros is copied
# unchanged.

# Comment lines (and any other lines starting with ')') are deleted.

# Each macro name is enclosed in quotes, preceded by /*MNAME*/ and
# followed by a comma.

# If a macro names is preceded by a comment line of the form /*LOCKED*/
# '&' is prepended to the name of the macro in the initialization
# code in iniMacro.h.  This results in the macro being a locked variable
# after startup.  Note: Nothing needed to be done to iniMacro.awk to allow
# macIniMacro.r to inherit this name.

# The text of each macro is preceded by /*MTEXT*/, enclosed in quotes,
# quotes and backslashes are escaped, tabs are changed to '\t', and
# lines are ended with '\n\'.  The text is followed by a comma and /*mend*/

# Inside macros, temporary names at the start of a line or not
# immediately preceded by '"' will have $$ added unless they already
# end in $$.  

# CAUTION about tabs
# In the macros in iniMacro.mac themselves, any tabs in quoted strings
# should be designated by '\t'

# CAUTION about lines starting with ')'
# There should be no lines starting with ')' except for comment lines
# between a name line and macro text

# CAUTION about temporary names in quoted strings
# Because of the handling of temporary variable names, care must
# be taken in iniMacro.mac to ensure that any temporary name
# without trailing $$ in a quoted string, should immediately
# follow the '"'.  If it doesn't, $$ will be erroneously appended
# Examples:
#   @xname <- "@X"  is changed to @xname$$ <- "@X"  [ok]
#   @mod <- "@y$$ = @x$$" is changed to @mod$$ <- "@y$$ = @x$$"  [ok]
#   @mod$$ <- "@y$$ = @x$$" is unchanged  [ok]
#   @mod <- "@y = @x" is changed to @mod$$ <- "@y = @x$$"  [not ok]

# Version 990308
#         990928 macros can now begin with '_'; fixed another small bug
#         000616 adds $$ to @[_a-zA-Z][_a-zA-Z0-9]*.  See cautionary
#                admonishment above
#         010729 checks for /*LOCKED*/ before a macro; if present, name
#                used has '&' as prefix
#         011017 when both locked and out-of-line, prefix is "%&"

OUTFILE=$2
if [ "$OUTFILE" = "" ]
then
	OUTFILE=iniMacro.h
fi

# sed copies everything up to the line containing Start_Of_Macros
# Then it changes all case of \ to \\, " to \", TABS to \t and
# deletes lines starting with ')'
# 000616 changes \(@[_a-zA-Z][_a-zA-Z0-9]*\) to \1$$
# 	-e 's/\(\("[^"]*"[^"@]*\)*\)\(@[_a-zA-Z][_a-zA-Z0-9]*\)/\1\3$$/g' 

sed -e '1,/Start_Of_Macros/b'\
	-e 's/\\/\\\\/g' \
	-e 's/"/\\"/g' \
	-e 's/	/\\t/g' \
	-e '/^)/d' \
	-e '/"@[_a-zA-Z][_a-zA-Z0-9]*/s/[^"]@[_a-zA-Z][_a-zA-Z0-9]*/&$$/g' \
	-e '/"@[_a-zA-Z][_a-zA-Z0-9]*/s/^@[_a-zA-Z][_a-zA-Z0-9]*/&$$/' \
	-e '/"@[_a-zA-Z][_a-zA-Z0-9]*/s/\(@[_a-zA-Z][_a-zA-Z0-9]*\$\$\)\$\$/\1/g' \
	-e '/"@[_a-zA-Z][_a-zA-Z0-9]*/b' \
	-e 's/@[_a-zA-Z][_a-zA-Z0-9]*/&$$/g' \
	-e 's/\(@[_a-zA-Z][_a-zA-Z0-9]*\$\$\)\$\$/\1/g' $1 |\
awk 'BEGIN	{
				outfile ="'"$OUTFILE"'"
				startMacro = "/*STARTMACROS*/"
				inMacro = 0
				prefix = lockedline = ""
			}
inMacro == 0 && /^[		]*\/\* *[Ll][Oo][Cc][Kk][Ee][Dd] *\*\// {
				prefix = "&"
				lockedline = $0
				next
			}
inMacro == 0 && /^[a-zA-Z_][a-zA-Z0-9_]* *[mM][aA][cC][rR][oO] */	{
			if (startMacro != ""){
				print startMacro > outfile
				startMacro = ""
			}
			inMacro = 1
			name = $1
			nLines = 0
			if (NF > 2 && $3 ~ /[oO][uU][tT][lL][iI][nN][eE]/){
				prefix = prefix "%"
			}
			if (lockedline != ""){
				printf ("\n%s\n", lockedline) > outfile
			}
			lockedline = ""
			printf("\n/*MNAME*/ \"%s%s\",\n/*MTEXT*/ \"", prefix, name) > outfile
			prefix = ""
			next
		}
inMacro == 0	{print > outfile; next}
/^%[a-zA-Z_][a-zA-Z0-9]*%/	{
								inMacro = 0
								printf("\", /*MEND*/\n") > outfile
								next
							}
inMacro != 0	{
					if (nLines > 0)printf("\\n\\\n") > outfile
					nLines++
					printf("%s",$0) > outfile
					next
				}
END	{
		print("/*ENDMACROS*/")  > outfile
		print("\t(char *) 0, (char *) 0") > outfile
		printf ("New version of %s created\n", outfile)
	}'

