/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.13 or later
*(C)*
*(C)* Copyright (c) 2003 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/


#ifdef SEGMENTED
#if defined(MPW) || defined(MW_CW)
#pragma segment Initialize
#endif /*MPW||MW_CW*/
#endif /*SEGMENTED*/

#include "version.h"

#include "globals.h"

#include "mvIface.h"

#include "mvGraphs.h"
#include "mvIo.h"

#ifdef READLINE
#undef RETURN
#if defined (READLINE_LIBRARY)
#  include "readline.h"
#  include "history.h"
#else /*READLINE_LIBRARY*/
#  include <readline/readline.h>
#  include <readline/history.h>
#endif /*READLINE_LIBRARY*/

/*
   By suitable defines in a Makefile you can change the default
   location of the readline setup file.

   On any platform, if INPUTRCFILE is defined, it is assumed to be
   a complete path name for the setup file.

   If DEFAULTINPUTRC is defined (but not INPUTRCFILE) it will be
   DATAPATH/DEFAULTINPDATAPATH where DATAPATH is taken from symbol
   "DATAPATH" created by initialize().  This may be set by including
   a -D'DATAPATHNAME="name"' in the compile.  Otherwise it is
   either the user's home directory (unix) or the directory where
   MacAnova is located.

   Neither INPUTRCFILE or DEFAULTINPUTRC defined:
     djgpp:  DATAPATH/INPUTRC
	 Unix:  ~/.inputrc (readline default).
*/
#ifndef DEFAULTINPUTRC
#ifdef DJGPP
#define DEFAULTINPUTRC "INPUTRC"
#endif /*DJGPP*/
#endif /*DEFAULTINPUTRC*/
#endif /*READLINE*/

/*
  970611 added User() and loadUser() to function list
  970627 added asLong() to function list
  971012 added invdunnett() and cumdunnett() to function list
  971109 added rpoi() to function list
  971210 added gethistory() and sethistory() to function list
  980106 initialize() now calls rl_initialize()
  980310 added rbin() to function list
  980317 added isarray() to function list
  980329 addit batchInit() to allocate and initialize globals used by
         batch() and putprompt()
  980401 Added new argument prompt to initialize and use it to initialize
         prompt.
  980723 Added getnotes() to function list
  980727 Added setlabels(), attachnotes() and addnotes() to function list
  980801 Added argvalue() to function list
  980903 Added getascii() to function list
  990304 Added qrdecode() to function list
  990325 Added Arima.mac to initial MACROFILES variable
         Added isname() to function list
  990501 Removed fplot(), fchplot(), flineplot() and fboxplot()
         to be replaced by macros in MacAnova.mac
  990522 Added isfunction() to function list
         Now includes mvGraphs.h
         Call to iniGraphWindows() to initialize special symbol GRAPHWINDOWS
  990524 Added stringplot() to function list
  990602 Added Mouse() to function list
  990729 Deleted obsolete function names hcopyc() and ccopyh()
         (same as ctoh() and h2oc())
  000214 Added odometer() to function list
         Added initilization of MaxInteger
  000430 Added memoryinfo() to function list
  000504 Added functions alltrue() and anytrue() and made changes to
         accommodate 'quoted' arguments for some functions, allowing
         delayed evaluation of the arguments.
  000523 Added function getfilename()
  000525 Added function log2()
  000707 Added allocation of LineBuffer used by fillFromDialog() in
         Macintosh version
  000708 Added syntaxStatusInit() to allocated space for WHILESTARTS,
         WHILECONDITIONS, WHILELIMITS, WHILEBRACKETLEV, FORVECTORS,
         IFCONDITIONS, and IFBRACKETLEV defined in globkb.h to save
		 space in globals segment
  001025 Allocates LineBuffer used by fillFromDialog() in WX version
         Moved declaration of LineBuffer to mvIo.h
  001027 Removed allocation of LineBuffer; now done in fillFromDialog
  001123 Added functions primefactors(), goodfactors()
  001213 Set help() and usage() for delayed evaluation of arguments
  010319 Allocate space in inputInit() for new global LastPathOpened
  010503 Add isnumber()
  010601 Includes mvIface.h instead of macIface.h or wxIface.h
         Removed defines of CR and LF for Macintosh
  010616 Added initialization of HELPFILES
  010619 Changed help() to gethelp() to use with new macro called help
  010703 Added initialization of HELPINDICES
  010727 Added islocked(), lockvars() and unlockvars()
  010809 Initialize ThisSqbracketlev and SQBRACKETLEV in inputInit
  011026 Added getkeywords()
  020319 Added pushmodel(), popmodel()
  020321 New argument maxModelStackDepth to initialize()
  020528 Added polygamma()
  020529 Added digamma()
  030207 Intialize Notes and Label handles to 0
  030305 Added hdivh(), hdivhj(), cdivc(), cdivcj()
  030514 Added typeof()
  030515 Added shapeof()
  030518 Added isEqual()
  030702 Added lowess()
  030703 initialize() allocates space for DBOUT using dbugInit()
*/

#if (0) /*externs not needed because prototypes are in matProto.h*/
extern Symbolhandle Transform();
extern Symbolhandle anovacoefs();
extern Symbolhandle array();
extern Symbolhandle batch();
extern Symbolhandle bin();
extern Symbolhandle boxplot();
extern Symbolhandle callC(); /*kb*/
extern Symbolhandle cellstats();
extern Symbolhandle changestr(); /*kb*/
extern Symbolhandle cholesky(); /*kb*/
extern Symbolhandle cluster(); /*kb*/
extern Symbolhandle columnops(); /*kb*/
extern Symbolhandle compnames();
extern Symbolhandle concat();
extern Symbolhandle contrast();
extern Symbolhandle cor();
extern Symbolhandle cumcdf();
extern Symbolhandle deleter();
extern Symbolhandle describe();
extern Symbolhandle diag(); /*kb*/
extern Symbolhandle dim(); /*kb*/
extern Symbolhandle dynload(); /*kb*/
extern Symbolhandle eigen();
extern Symbolhandle elapsedTime();
extern Symbolhandle evaluate(); /*kb*/
extern Symbolhandle findfactors(); /*kb*/
extern Symbolhandle fft();
extern Symbolhandle getHistory();
extern Symbolhandle getlabs();
extern Symbolhandle getseed();
extern Symbolhandle glm();
extern Symbolhandle hconcat(); /*kb*/
extern Symbolhandle help();
extern Symbolhandle isEqual(); /*kb*/
extern Symbolhandle iswhat(); /*kb*/
extern Symbolhandle inforead(); /*kb*/
extern Symbolhandle keyvalue(); /*kb*/
extern Symbolhandle kmeans(); /*kb*/
extern Symbolhandle listbrief();
extern Symbolhandle lockvars(); /*kb*/
extern Symbolhandle lowessmv(); /*kb*/
extern Symbolhandle macro();
extern Symbolhandle macrousage(); /*kb*/
extern Symbolhandle makefactor();
extern Symbolhandle makestr();
extern Symbolhandle makesymbols(); /*kb*/
extern Symbolhandle matrix();
extern Symbolhandle memoryInfo(); /*kb*/
extern Symbolhandle modelvar();
extern Symbolhandle movavg();
extern Symbolhandle myplot();
extern Symbolhandle nameOf(); /*kb*/
extern Symbolhandle ndims(); /*kb*/
extern Symbolhandle outer();
extern Symbolhandle partacf();
extern Symbolhandle paste();
extern Symbolhandle polyroot();
extern Symbolhandle power1();
extern Symbolhandle predtable();
extern Symbolhandle print();
extern Symbolhandle pushPopModel(); /*kb*/
extern Symbolhandle putascii();
extern Symbolhandle qr();
extern Symbolhandle rangen();
extern Symbolhandle rational();
extern Symbolhandle readdata();
extern Symbolhandle regpred();
extern Symbolhandle renamer();
extern Symbolhandle rep();
extern Symbolhandle readMouse();
extern Symbolhandle restore();
extern Symbolhandle xvarsrows(); /*kb*/
extern Symbolhandle run();
extern Symbolhandle samplesize();
extern Symbolhandle odometer();
extern Symbolhandle save();
extern Symbolhandle selecter(); /*kb*/
extern Symbolhandle setlabs(); /*kb*/
extern Symbolhandle setoptions(); /*kb*/
extern Symbolhandle setseed();
extern Symbolhandle sort();
extern Symbolhandle shell();
extern Symbolhandle solve();
extern Symbolhandle split();
extern Symbolhandle spool();
extern Symbolhandle stemleaf();
extern Symbolhandle svd(); /*kb*/
extern Symbolhandle swp(); /*kb*/
extern Symbolhandle t2val();
extern Symbolhandle tabs();
extern Symbolhandle testLogicals();
extern Symbolhandle toeplitz();
extern Symbolhandle transpose();
extern Symbolhandle trideigen();
extern Symbolhandle tserops();
extern Symbolhandle tval();
extern Symbolhandle varnames(); /*kb*/
extern Symbolhandle xvariables(); /*kb*/
extern Symbolhandle yates();
#endif /*0*/

enum    modePrefixes   /* special characters starting names; stripped off*/
{
	lockPrefix      = '&',	/* starts name of locked constant */
	quotedArgPrefix = '%'	/* starts name of function with quoted args */
};

static struct
{                               /* constants */
	char           *name;
	double          value;
}                   Constants[] =
{
	{"DELTAT",     1.0}, /* not to be locked */
	{"&PI",        3.14159265358979323846}, /* to be locked */
	{"&E",         2.71828182845904523536}, /* to be locked */
	{"&DEGPERRAD", 57.29577951308232087680}, /* to be locked */
	{"&NA",        MISSINGVALUE}, /* to be locked */
	{(char *) 0,   0.0} 
};

static struct
{
	char           *name;
	char           *string;
}               stringvars[] =

{
#ifdef CONSOLENAME
	{"&CONSOLE", CONSOLENAME}, /* "/dev/tty" or "Dev:Console" */
#endif /*CONSOLENAME*/
	{(char *) 0, (char *) 0}
};

/*
  000504  name Starting with '%' (quotedArgPrefix) means args should be 'quoted'
*/
static struct
{/* built in functions */
	char           *name;
    Symbolhandle  (*function) (Symbolhandle);
}               builtins[] =

{
	{"cat",          concat},
	{"vector",       concat}, /* identical to cat()*/
	{"macro",        macro},
	{"makestr",      makestr},
	{"structure",    makestr}, /*synonym for makestr()*/
	{"strconcat",    makestr},     /*kb*/
	{"compnames",    compnames},   /*kb*/
	{"varnames",     varnames}, /*kb*/
	{"getlabels",    getlabs},   /*kb*/
	{"getnotes",     getlabs},   /*kb*/
	{"setlabels",    setlabs},   /*kb*/
	{"appendnotes",  setlabs},   /*kb*/
	{"attachnotes",  setlabs},   /*kb*/
	{"modelvars",    modelvars}, /*kb*/
	{"modelinfo",    xvariables}, /*kb*/
	{"xvariables",   xvariables}, /*kb*/
	{"xrows",        xvarsrows}, /*kb*/
	{"anova",        glm},
	{"factor",       makefactor},  /*kb*/
	{"match",        makefactor},  /*kb*/
	{"unique",       makefactor},  /*kb*/
	{"matrix",       matrix},
	{"delete",       deleter},
	{"list",         listbrief},
	{"listbrief",    listbrief},
	{"hconcat",      hconcat},     /*kb*/
	{"vconcat",      hconcat},     /*kb*/
	{"select",       selecter},      /*kb*/
	{"paste",        paste},       /*kb*/
	{"shell",        shell},       /*kb*/
	{"User",         callC},       /*kb*/
	{"loadUser",     dynload},     /*kb*/
	{"asLong",       asLong},      /*kb*/
	{"array",        array},
	{"print",        print},
	{"fprint",       print},       /*kb*/
	{"matprint",     print},       /*kb*/
	{"write",        print},       /*kb*/
	{"fwrite",       print},       /*kb*/
	{"matwrite",     print},       /*kb*/
	{"macrowrite",   print},       /*kb*/
	{"error",        print},       /*kb*/
	{"putascii",     putascii},    /*kb*/
	{"getascii",     putascii},    /*kb*/
	{"contrast",     contrast},
	{"anovacoefs",   anovacoefs},
	{"log",          Transform},
	{"log10",        Transform},
	{"log2",         Transform},
	{"exp",          Transform},
	{"abs",          Transform},
	{"cellstats",    cellstats},
	{"describe",     describe},
	{"descriptive",  describe},    /*kb*/
	{"halfnorm",     sort},        /*kb*/
	{"rankits",      sort},
	{"regress",      glm},
	{"lowess",       lowessmv},    /*kb*/
	{"setoptions",   setoptions},
	{"getoptions",   setoptions},
	{"gettime",      elapsedTime}, /*kb*/
	{"memoryinfo",   memoryInfo},  /*kb*/
	{"sort",         sort},
	{"split",        split},
	{"tval",         tval},
	{"t2val",        t2val},
	{"tint",         tval},
	{"t2int",        t2val},
	{"bin",          bin},
	{"stemleaf",     stemleaf},
	{"regpred",      regpred},
	{"glmpred",      regpred},
	{"yates",        yates},
	{"rational",     rational},    /*kb*/
	{"partacf",      partacf},     /*kb*/
	{"yulewalker",   partacf},     /*kb*/
	{"movavg",       movavg},      /*kb*/
	{"autoreg",      movavg},      /*kb*/
	{"toeplitz",     toeplitz},      /*kb*/
	{"polyroot",     polyroot},      /*kb*/
	{"rft",          fft},         /*kb*/
	{"hft",          fft},         /*kb*/
	{"cft",          fft},         /*kb*/
	{"hconj",        tserops},     /*kb*/
	{"cconj",        tserops},     /*kb*/
	{"hreal",        tserops},     /*kb*/
	{"creal",        tserops},     /*kb*/
	{"himag",        tserops},     /*kb*/
	{"cimag",        tserops},     /*kb*/
	{"htoc",         tserops},     /*kb*/
	{"ctoh",         tserops},     /*kb*/
	{"hprdh",        tserops},     /*kb*/
	{"hprdhj",       tserops},     /*kb*/
	{"cprdc",        tserops},     /*kb*/
	{"cprdcj",       tserops},     /*kb*/
	{"hdivh",        tserops},     /*kb*/
	{"hdivhj",       tserops},     /*kb*/
	{"cdivc",        tserops},     /*kb*/
	{"cdivcj",       tserops},     /*kb*/
	{"cmplx",        tserops},     /*kb*/
	{"hpolar",       tserops},     /*kb*/
	{"cpolar",       tserops},     /*kb*/
	{"hrect",        tserops},     /*kb*/
	{"crect",        tserops},     /*kb*/
	{"unwind",       tserops},     /*kb*/
	{"convolve",     tserops},     /*kb*/
	{"rotate",       tserops},     /*kb*/
	{"rotation",     rotatefac},   /*kb*/
	{"reverse",      tserops},     /*kb*/
	{"padto",        tserops},     /*kb*/
	{"sin",          Transform},
	{"cos",          Transform},
	{"atan",         Transform},
	{"atan2",        Transform},   /*kb*/
	{"hypot",        Transform},   /*kb*/
	{"sqrt",         Transform},
	{"lgamma",       Transform},
	{"polygamma",    Transform},   /*kb*/
	{"digamma",      Transform},   /*kb*/
	{"asin",         Transform},
	{"acos",         Transform},
	{"tan",          Transform},
	{"sinh",         Transform},
	{"cosh",         Transform},
	{"tanh",         Transform},
	{"atanh",        Transform},
	{"floor",        Transform},
	{"ceiling",      Transform},
	{"round",        Transform},
	{"nbits",        Transform},
	{"primefactors", findfactors}, /*kb*/
	{"goodfactors",  findfactors}, /*kb*/
	{"predtable",    predtable},
	{"glmtable",     predtable}, /*kb*/
	{"rep",          rep},
	{"run",          run},
	{"fastanova",    glm},
	{"screen",       glm},
	{"setseeds",     setseed},
	{"getseeds",     getseed},
	{"rnorm",        rangen},
	{"rpoi",         rangen},
	{"rbin",         rangen},
	{"runi",         rangen},
	{"ismissing",    sort},
	{"anymissing",   sort},        /*kb*/
	{"isreal",       iswhat},      /*kb*/
	{"ischar",       iswhat},      /*kb*/
	{"islogic",      iswhat},      /*kb*/
	{"isvector",     iswhat},      /*kb*/
	{"ismatrix",     iswhat},      /*kb*/
	{"isarray",      iswhat},      /*kb*/
	{"isscalar",     iswhat},      /*kb*/
	{"isfactor",     iswhat},      /*kb*/
	{"isnull",       iswhat},      /*kb*/
	{"isdefined",    iswhat},      /*kb*/
	{"isstruc",      iswhat},      /*kb*/
	{"ismacro",      iswhat},      /*kb*/
	{"isgraph",      iswhat},      /*kb*/
	{"isfunction",   iswhat},      /*kb*/
	{"isname",       iswhat},      /*kb*/
	{"isnumber",     iswhat},      /*kb*/
	{"islocked",     iswhat},      /*kb*/
	{"equal",        isEqual},     /*kb*/
	{"%alltrue",     testLogicals},/*kb*/ /*args to be 'quoted'*/
	{"%anytrue",     testLogicals},/*kb*/ /*args to be 'quoted'*/
	{"argvalue",     keyvalue},    /*kb*/		
	{"keyvalue",     keyvalue},    /*kb*/		
	{"lockvars",     lockvars},    /*kb*/
	{"unlockvars",   lockvars},    /*kb*/
	{"rename",       renamer},     /*kb*/
	{"nameof",       nameOf},      /*kb*/
	{"getkeywords",  nameOf},      /*kb*/
	{"typeof",       nameOf},      /*kb*/
	{"shapeof",      nameOf},      /*kb*/
	{"wtregress",    glm},
	{"wtanova",      glm},
	{"manova",       glm},
	{"wtmanova",     glm},
	{"grade",        sort},        /*kb*/
	{"rank",         sort},
	{"sum",          columnops},   /*kb*/
	{"prod",         columnops},   /*kb*/
	{"max",          columnops},   /*kb*/
	{"min",          columnops},   /*kb*/
	{"dim",          dim},         /*kb*/
	{"nrows",        dim},         /*kb*/
	{"ncols",        dim},         /*kb*/
	{"ndims",        dim},         /*kb*/
	{"ncomps",       dim},         /*kb*/
	{"length",       dim},         /*kb*/
	{"diag",         diag},        /*kb*/
	{"dmat",         diag},        /*kb*/
	{"swp",          swp},         /*kb*/
	{"bcprd",        swp},         /*kb*/
	{"det",          solve},       /*kb*/
	{"solve",        solve},       /*kb*/
	{"rsolve",       solve},       /*kb*/
	{"cholesky",     cholesky},    /*kb*/
	{"trilower",     qr},          /*kb*/
	{"triupper",     qr},          /*kb*/
	{"triunpack",    qr},          /*kb*/
	{"qrdecode",     qr},          /*kb*/
	{"qr",           qr},          /*kb*/
	{"regsqr",       qr},          /*kb*/
	{"glmfit",       glm},         /*kb*/
	{"robust",       glm},
	{"poisson",      glm},
	{"ipf",          glm},
	{"tabs",         tabs},
	{"logistic",     glm},
	{"probit",       glm},         /*kb*/
	{"pushmodel",    pushPopModel},/*kb*/
	{"popmodel",     pushPopModel},/*kb*/
	{"secoefs",      anovacoefs},
	{"coefs",        anovacoefs},
	{"cor",          cor},
	{"trace",        eigen},
	{"plot",         myplot},
	{"addpoints",    myplot},
	{"chplot",       myplot},
	{"addchars",     myplot},
	{"lineplot",     myplot},
	{"addlines",     myplot},
	{"stringplot",   myplot},
	{"addstrings",   myplot},
	{"showplot",     myplot},
	{"Mouse",        readMouse},
	{"makesymbols",  makesymbols},
#if (0) /*990501 removed fplot,fchplot,flineplot, fboxplot*/
	{"fplot",        myplot},
	{"fchplot",      myplot},
	{"flineplot",    myplot},
	{"fboxplot",     boxplot},
#endif /*0*/
	{"eigen",        eigen},
	{"eigenvals",    eigen},
	{"releigen",     eigen},
	{"releigenvals", eigen},       /*kb*/
	{"trideigen",    trideigen},   /*kb*/
	{"svd",          svd},         /*kb*/
	{"cluster",      cluster},     /*kb*/
	{"kmeans",       kmeans},      /*kb*/
	{"changestr",    changestr},   /*kb*/
	{"t",            transpose},
	{"outer",        outer},
	{"boxplot",      boxplot},
	{"save",         save},
	{"asciisave",    save},
	{"restore",      restore},
	{"batch",        batch},
	{"cumnor",       cumcdf},
	{"invnor",       cumcdf},      /*kb*/
	{"cumchi",       cumcdf},
	{"invchi",       cumcdf},      /*kb*/
	{"cumstu",       cumcdf},
	{"invstu",       cumcdf},      /*kb*/
	{"cumF",         cumcdf},
	{"invF",         cumcdf},
	{"cumgamma",     cumcdf},      /*kb*/
	{"invgamma",     cumcdf},      /*kb*/
	{"cumbeta",      cumcdf},      /*kb*/
	{"invbeta",      cumcdf},      /*kb*/
	{"cumstudrng",   cumcdf},      /*go*/
	{"invstudrng",   cumcdf},      /*go*/
	{"cumdunnett",   cumcdf},      /*go*/
	{"invdunnett",   cumcdf},      /*go*/
	{"cumbin",       cumcdf},
	{"cumpoi",       cumcdf},
	{"vecread",      readdata},    /*kb*/
	{"matread",      readdata},    /*kb*/
	{"macroread",    readdata},    /*kb*/
	{"inforead",     readdata},    /*kb*/
	{"read",         readdata},    /*kb*/
	{"getfilename",  readdata},    /*kb*/
	{"getdirname",   readdata},    /*go*/
	{"findfile",     findfile},    /*go*/
	{"spool",        spool},
	{"power",        power1},
	{"power2",       power1},
	{"samplesize",   samplesize},
	{"setodometer",  odometer},
	{"%Help",        help},
	{"evaluate",     evaluate},
	{"%gethelp",     help},
	{"%getusage",    help},
	{"macrousage",   macrousage},
	{"gethistory",   getHistory},
	{"sethistory",   getHistory},
#ifdef SKEL_CARAPACE
	{"setmenubar",   setmenubar},
	{"getmenubar",   getmenubar},
	{"setmdialog",   setmdialog},
	{"setdialog",    setdialog},
	{"pushcommand",  pushcommand},
	{"doguihelp",    guihelp},
	{"alert",        alert},
	{"replacestr",   replacestr},
	{"setdyn",   setdyn},      // Appended on March 17,2005
	{"getdyn",   getdyn},      // Appended on March 17,2005
	{"rmvdyn",   rmvdyn},      // Appended on March 31,2005
	{"resetdyn",   resetdyn},      // Appended on July 21,2005	
	{"reset2dyn",   reset2dyn},      // Appended on August 5,2005		

/*	{"getmdialog",   getmdialog},*/
#endif /* SKEL_CARAPACE */
	{(char *) 0,     (Symbolhandle (*) (Symbolhandle)) 0}
};


/*
  970612 added inputInit() to allocate pointers to stack of input strings
  990705 Replaced initialization of MACRONAMES by initialization of MACROINFO
  010809 Added initializtion of all ThisSqbracketlev and SQBRACKETLEV
*/
#ifdef HASDYNLOAD
int dynloadInit(void);
#endif /*HASDYNLOAD*/

static int inputInit(void)
{
	INPUTSTACK = (mvMacroPtr) mygetpointer(NINPUTSTRINGS * sizeof(mvMacro));

	if (INPUTSTACK == (mvMacroPtr) 0)
	{
		return (0);
	}
	
	for (INPUTLEVEL = 0; INPUTLEVEL < NINPUTSTRINGS; INPUTLEVEL++)
	{
		ThisInputstring = (unsigned char **) 0;
		ThisInputstart = 0;
		ThisInputend = 0;
		ThisBracketlev = 0;
		ThisParenlev = 0;
		ThisSqbracketlev = 0;
		ThisWdepth = 0;
		ThisInputname[0] = '\0';
		ThisScratchHandle = (char **) 0;
	} /*for (INPUTLEVEL = 0; INPUTLEVEL < NINPUTSTRINGS; INPUTLEVEL++)*/
	INPUTLEVEL = 0;

	strcpy(ThisInputname, "prompt level");

	INPUTSTRING = ThisInputstring;
	ISTRCHAR = ThisIstrchar;
	BRACKETLEV = ThisBracketlev;
	PARENLEV = ThisParenlev;
	SQBRACKETLEV = ThisSqbracketlev;
	WDEPTH = ThisWdepth;
	
	MACROINFO = (mvMacroInfo *) mygetpointer(2*MAXMDEPTH*sizeof(mvMacroInfo));
	
	if (MACROINFO == (mvMacroInfo *) 0)
	{
		return (0);
	}

	for (ACTIVEMACROS = 1; ACTIVEMACROS <= 2*MAXMDEPTH; ACTIVEMACROS++)
	{
		ThisMacroName[0] = '\0';
	}
	ACTIVEMACROS = 0;

	LastPathOpened = (char *) mygetpointer(PATHSIZE + 1);
	
	if (LastPathOpened == (char *) 0)
	{
		return (0);
	}
	LastPathOpened[0] = '\0';

	return (1);
} /*inputInit()*/

/*
	Allocate WHILESTARTS, WHILECONDITIONS, WHILELIMITS, 
	WHILEBRACKETLEV, FORVECTORS, IFCONDITIONS, and IFBRACKETLEV, 
	all defined in globkb.h
	The original definitions were
	EXTERN long            WHILESTARTS[INITDIM(MAXWDEPTH)];
	EXTERN long            WHILECONDITIONS[INITDIM(MAXWDEPTH)];
	EXTERN long            WHILELIMITS[INITDIM(MAXWDEPTH)];
	EXTERN long            WHILEBRACKETLEV[INITDIM(MAXWDEPTH)];
	EXTERN double        **FORVECTORS[INITDIM(MAXWDEPTH)];
	EXTERN long            IFCONDITIONS[INITDIM(MAXIDEPTH)];
	EXTERN long            IFBRACKETLEV[INITDIM(MAXIDEPTH)];
*/

static int syntaxStatusInit(void)
{
	long      needed = (4*MAXWDEPTH + 2*MAXIDEPTH)*sizeof(long) +
				MAXWDEPTH*sizeof(double **);
	char     *place = mygetpointer(needed);
	
	if (place == (char *) 0)
	{
		return (0);
	}
	WHILESTARTS = (long *) place;
	place += MAXWDEPTH*sizeof(long);
	WHILECONDITIONS = (long *) place;
	place += MAXWDEPTH*sizeof(long);
	WHILELIMITS = (long *) place;
	place += MAXWDEPTH*sizeof(long);
	WHILEBRACKETLEV = (long *) place;
	place += MAXWDEPTH*sizeof(long);
	FORVECTORS = (double ***) place;
	place += MAXWDEPTH*sizeof(double **);
	IFCONDITIONS = (long *) place;
	place += MAXIDEPTH*sizeof(long);
	IFBRACKETLEV = (long *) place;
	return (1);
} /*syntaxStatusInit()*/

static int dbugInit(void)
{
	DBOUT = mygetpointer(DBOUTLENGTH);
	
	return (DBOUT != (char *) 0);
} /*dbugInit()*/

#ifndef STANDARDPROMPT
#define STANDARDPROMPT "Cmd> "
#endif /*STANDARDPROMPT*/

static int batchInit(void)
{
	int        i;
	
	BATCHECHO = (long *) mygetpointer(MAXBDEPTH * sizeof(long));

	if (BATCHECHO == (long *) 0)
	{
		return (0);
	}
	INPUTFILE = (FILE **) mygetpointer(MAXBDEPTH * sizeof(FILE *));

	if (INPUTFILE == (FILE **) 0)
	{
		return (0);
	}

	INPUTFILENAMES = (char ***) mygetpointer(MAXBDEPTH * sizeof(char **));

	if (INPUTFILENAMES == (char ***) 0)
	{
		return (0);
	}
	/* Flag for whether last input character read was CR*/
	LASTINPUTWASCR = (char *) mygetpointer(MAXBDEPTH * sizeof(char));

	if (LASTINPUTWASCR == (char *) 0)
	{
		return (0);
	}
	PROMPTS = (promptType *)
	  mygetpointer(2 * MAXBDEPTH * sizeof (promptType));

	if (PROMPTS == (promptType *) 0)
	{
		return (0);
	}
	
	DEFAULTPROMPTS = PROMPTS + MAXBDEPTH;

	for (i = 0; i < MAXBDEPTH; i++)
	{
		BATCHECHO[i] = (i == 0) ? 1 : 0;
		INPUTFILE[i] = (FILE *) 0;
		INPUTFILENAMES[i] = (char **) 0;
		LASTINPUTWASCR[i] = '\0';
		strcpy((char *) DEFAULTPROMPTS[i],
			   (i == 0) ? STANDARDPROMPT : NullString);
	}
	strcpy((char *) MOREPROMPT, "More> ");
	return (1);
} /*batchInit()*/

char     *StdinName = "standard input";

static void      cleanString(char *stringvar)
{
#ifdef BASE_MACINTOSH
	while (*stringvar != '\0')
	{
		if (*stringvar == LF)
		{
			*stringvar = '\n';
		}
		stringvar++;
	}
#endif /*MACINTOSH*/
} /*cleanString()*/
/*
  check function name to see if function symbol should have QUOTEARGS
  bit set
*/
#define quoteArgs(NAME) (NAME[0] == quotedArgPrefix)

#define MAXPARMLENGTH 50

int initFilesAndPaths(char **searchPaths, char **helpFiles, char **macroFiles,   
		char **dataFiles, char *helpFileName, char *homePath)
{
	Symbolhandle symh;
	int i,length,length2,nPaths,nFiles;
	char *tempNames[MAXPARMLENGTH],*installedpath,*thispath;

	i = 0;
	installedpath = (char *) 0;
	while (searchPaths[i])
	{
		installedpath = searchPaths[i];
		i++;
	}


	/* install CHARACTER scalar HOME */
	length = strlen(homePath);

	if(length > 0)
	{
		symh = CInstall("HOME",length + 2);
		if (symh == SymbolError)
		{
			return(0);
		}
		strcpy(STRINGPTR(symh),homePath);
#ifndef NOSEPARATOR
		/* make sure HOME terminates with DIRSEPARATOR[0]*/
		if(STRINGVALUE(symh, length-1) != DIRSEPARATOR[0])
		{
			STRINGVALUE(symh, length) = DIRSEPARATOR[0];
			STRINGVALUE(symh, length+1) = '\0';
		}
#endif /*NOSEPARATOR*/
#ifdef BASE_MICROSOFT
		{
			char       *pc;

			for (pc = STRINGPTR(symh); *pc != '\0'; pc++)
			{ /* ensure all separators are standard backslashes*/
				if (*pc == '/')
				{
					*pc = DIRSEPARATOR[0];
				}
			} /*for (pc = STRINGPTR(symh); *pc != '\0'; pc++)*/

		}
#endif /*BASE_MICROSOFT*/
#ifndef BASE_MACINTOSH
		setMvDirInfo(homeVol, STRINGPTR(symh));
#endif /*BASE_MACINTOSH*/
	} /*if(length > 0)*/

/*
   Install CHARACTER vector DATAPATHS containing searchPaths 
*/
	i = 0;
	nPaths = 0;
	length = 0;
	while (searchPaths[i])
	{
		thispath = searchPaths[i];
		nPaths++;
		i++;
		length2 = strlen(thispath);
#ifndef NOSEPARATOR
                if (thispath[length2-1] != DIRSEPARATOR[0])
                {
                        length2++;
                }
#endif /*NOSEPARATOR*/
                length++; /* account for trailing null */
                length += length2;
	}

        symh = CInstall("DATAPATHS",length + 3);
        if (symh == SymbolError)
        {
                return(0);
        }
	length = 0;
	for(i=0;i<nPaths;i++)
	{
                strcpy(skipStrings(STRINGPTR(symh), i), searchPaths[i]);
		length += strlen(searchPaths[i]);
        	STRINGVALUE(symh,length) = STRINGVALUE(symh,length+1) = '\0';
#ifndef NOSEPARATOR
                if (STRINGVALUE(symh,length-1) != DIRSEPARATOR[0])
                {
        		STRINGVALUE(symh,length) = DIRSEPARATOR[0];
			STRINGVALUE(symh,length+1) = NULL;
                        length++;
                }
#endif /*NOSEPARATOR*/
		length++;  /* for trailing null */
	}
	setDIM(symh, 1, nPaths);

/*   Install CHARACTER scalar DATAPATH containing first search path */
        length = strlen(searchPaths[0]);
#ifndef NOSEPARATOR
        if (searchPaths[0][length-1] != DIRSEPARATOR[0])
        {
                length++;
        }
#endif /*NOSEPARATOR*/
        symh = CInstall("DATAPATH",length + 1);
        if (symh == SymbolError)
        {
                return(0);
        }
        strcpy(STRINGPTR(symh),searchPaths[0]);
#ifndef NOSEPARATOR
        STRINGVALUE(symh, length-1) = DIRSEPARATOR[0];
        STRINGVALUE(symh, length) = '\0';
#endif /*NOSEPARATOR*/

	/* set up for DATAFILE and DATAFILES */
	nFiles = 0;
	length = 0;
	while (dataFiles[nFiles] && i < MAXPARMLENGTH)
	{
		tempNames[nFiles] = dataFiles[nFiles];
		length2 = strlen(tempNames[nFiles]);
		nFiles++;
                length++; /* account for trailing null */
                length += length2;
	}
	if(nFiles == 0) 
	{
		tempNames[0] = DATAFILENAME;
		nFiles++;
		length2 = strlen(tempNames[0]);
                length++; /* account for trailing null */
                length += length2;
	}
        symh = CInstall("DATAFILES",length + 3);
        if (symh == SymbolError)
        {
                return(0);
        }
	length = 0;
	for(i=0;i<nFiles;i++)
	{
                strcpy(skipStrings(STRINGPTR(symh), i), tempNames[i]);
		length += strlen(tempNames[i]);
        	STRINGVALUE(symh,length) = '\0';
		length++;  /* for trailing null */
	}
	setDIM(symh, 1, nFiles);
	

        /* install CHARACTER scalar DATAFILE, which is just DATAFILES[0] */
        length = strlen(tempNames[0]);

        symh = CInstall("DATAFILE", length + 1);
        if (symh == SymbolError)
        {
                return(0);
        }

        strcpy(STRINGPTR(symh), tempNames[0]);


	/* set up for MACROFILE and MACROFILES */
	nFiles = 0;
	length = 0;
	while (macroFiles[nFiles] && i < MAXPARMLENGTH)
	{
		tempNames[nFiles] = macroFiles[nFiles];
		length2 = strlen(tempNames[nFiles]);
		nFiles++;
                length++; /* account for trailing null */
                length += length2;
	}
#ifdef SKEL_CARAPACE
        tempNames[nFiles] = GUIMACROFILENAME;
	length += strlen(tempNames[nFiles++])+1;
#endif /* SKEL_CARAPACE */
        tempNames[nFiles] = GRAPHICSMACROFILENAME;
	length += strlen(tempNames[nFiles++])+1;
        tempNames[nFiles] = REGRESSMACROFILENAME;
	length += strlen(tempNames[nFiles++])+1;
        tempNames[nFiles] = DESIGNMACROFILENAME;
	length += strlen(tempNames[nFiles++])+1;
        tempNames[nFiles] = TSMACROFILENAME;
	length += strlen(tempNames[nFiles++])+1;
        tempNames[nFiles] = ARIMAMACROFILENAME;
	length += strlen(tempNames[nFiles++])+1;
        tempNames[nFiles] = MULVARMACROFILENAME;
	length += strlen(tempNames[nFiles++])+1;
        tempNames[nFiles] = MATHMACROFILENAME;
	length += strlen(tempNames[nFiles++])+1;
        tempNames[nFiles] = MACROFILENAME;
	length += strlen(tempNames[nFiles++])+1;

        symh = CInstall("MACROFILES",length + 3);
        if (symh == SymbolError)
        {
                return(0);
        }
	length = 0;
	for(i=0;i<nFiles;i++)
	{
                strcpy(skipStrings(STRINGPTR(symh), i), tempNames[i]);
		length += strlen(tempNames[i]);
        	STRINGVALUE(symh,length) = '\0';
		length++;  /* for trailing null */
	}
	setDIM(symh, 1, nFiles);
	

        /* install CHARACTER scalar MACROFILE, which is just MACROFILES[nFiles-1] */
        length = strlen(tempNames[nFiles-1]);

        symh = CInstall("MACROFILE", length + 1);
        if (symh == SymbolError)
        {
                return(0);
        }

        strcpy(STRINGPTR(symh), tempNames[nFiles-1]);


                
#if 0
        /* Create global HELPFILENAME*/
        if (defaultHelpFile == (char *) 0)
        {
                defaultHelpFile = HELPNAME;
                length = strlen(helpPathName);
        }
        else
        {
                length = 0;
        }
        length += strlen(defaultHelpFile);

        HELPFILENAME = mygethandle(length + 1);
        if(HELPFILENAME == (char **) 0)
        {
                return(0);
        }
        if (length > strlen(defaultHelpFile) + 1)
        {
                strcpy(*HELPFILENAME, helpPathName);
        }
        else
        {
                (*HELPFILENAME)[0] = '\0';
        }
        strcat(*HELPFILENAME, defaultHelpFile);
#endif

	/* Create global HELPFILENAME*/
	if (helpFileName == (char *) 0)
	{
		helpFileName = HELPNAME;
	}
	length = strlen(helpFileName);
	
	HELPFILENAME = mygethandle(length + 1);
	if(HELPFILENAME == (char **) 0)
	{
		return(0);
	}
	strcpy(*HELPFILENAME, helpFileName);

/* help files are a little different from macro files.  We can put the -addhelp stuff
 * before the standard help files, but when we get to standard files, a file specified
 * by -help replaces macanova.hlp.
 */

	/* set up for HELPFILES */
	nFiles = 0;
	length = 0;
	while (helpFiles[nFiles] && i < MAXPARMLENGTH)
	{
		tempNames[nFiles] = helpFiles[nFiles];
		length2 = strlen(tempNames[nFiles]);
		nFiles++;
                length++; /* account for trailing null */
                length += length2;
	}

        tempNames[nFiles] = *HELPFILENAME;
	length += strlen(tempNames[nFiles++])+1;

#ifdef GRAPHICSHELPFILENAME
        tempNames[nFiles] = GRAPHICSHELPFILENAME;
	length += strlen(tempNames[nFiles++])+1;
#endif /*GRAPHICSHELPFILENAME*/

#ifdef GUIHELPFILENAME
        tempNames[nFiles] = GUIHELPFILENAME;
	length += strlen(tempNames[nFiles++])+1;
#endif /*GUIHELPFILENAME*/

#ifdef REGRESSHELPFILENAME
        tempNames[nFiles] = REGRESSHELPFILENAME;
	length += strlen(tempNames[nFiles++])+1;
#endif /*REGRESSHELPFILENAME*/

#ifdef DESIGNHELPFILENAME
        tempNames[nFiles] = DESIGNHELPFILENAME;
	length += strlen(tempNames[nFiles++])+1;
#endif /*DESIGNHELPFILENAME*/

#ifdef TSHELPFILENAME
        tempNames[nFiles] = TSHELPFILENAME;
	length += strlen(tempNames[nFiles++])+1;
#endif /*TSHELPFILENAME*/

#ifdef ARIMAHELPFILENAME
        tempNames[nFiles] = ARIMAHELPFILENAME;
	length += strlen(tempNames[nFiles++])+1;
#endif /*ARIMAHELPFILENAME*/

#ifdef MULVARHELPFILENAME
        tempNames[nFiles] = MULVARHELPFILENAME;
	length += strlen(tempNames[nFiles++])+1;
#endif /*MULVARHELPFILENAME*/

#ifdef MATHHELPFILENAME
        tempNames[nFiles] = MATHHELPFILENAME;
	length += strlen(tempNames[nFiles++])+1;
#endif /*MATHHELPFILENAME*/

        symh = CInstall("HELPFILES",length + 3);
        if (symh == SymbolError)
        {
                return(0);
        }
	length = 0;
	for(i=0;i<nFiles;i++)
	{
                strcpy(skipStrings(STRINGPTR(symh), i), tempNames[i]);
		length += strlen(tempNames[i]);
        	STRINGVALUE(symh,length) = '\0';
		length++;  /* for trailing null */
	}
	setDIM(symh, 1, nFiles);

	/* set up for HELPINDICES */
	i = 0;
	nFiles = 0;
	length = 0;
        tempNames[nFiles] = HELPINDEXNAME;

#ifdef GRAPHICSHELPFILENAME
        tempNames[nFiles] = GRAPHICSHELPINDEXNAME;
	length += strlen(tempNames[nFiles++])+1;
#endif /*GRAPHICSHELPFILENAME*/

#ifdef GUIHELPFILENAME
        tempNames[nFiles] = GUIHELPINDEXNAME;
	length += strlen(tempNames[nFiles++])+1;
#endif /*GUIHELPFILENAME*/

#ifdef REGRESSHELPFILENAME
        tempNames[nFiles] = REGRESSHELPINDEXNAME;
	length += strlen(tempNames[nFiles++])+1;
#endif /*REGRESSHELPFILENAME*/

#ifdef DESIGNHELPFILENAME
        tempNames[nFiles] = DESIGNHELPINDEXNAME;
	length += strlen(tempNames[nFiles++])+1;
#endif /*DESIGNHELPFILENAME*/

#ifdef TSHELPFILENAME
        tempNames[nFiles] = TSHELPINDEXNAME;
	length += strlen(tempNames[nFiles++])+1;
#endif /*TSHELPFILENAME*/

#ifdef ARIMAHELPFILENAME
        tempNames[nFiles] = ARIMAHELPINDEXNAME;
	length += strlen(tempNames[nFiles++])+1;
#endif /*ARIMAHELPFILENAME*/

#ifdef MULVARHELPFILENAME
        tempNames[nFiles] = MULVARHELPINDEXNAME;
	length += strlen(tempNames[nFiles++])+1;
#endif /*MULVARHELPFILENAME*/

#ifdef MATHHELPFILENAME
        tempNames[nFiles] = MATHHELPINDEXNAME;
	length += strlen(tempNames[nFiles++])+1;
#endif /*MATHHELPFILENAME*/

        symh = CInstall("HELPINDICES",length + 3);
        if (symh == SymbolError)
        {
                return(0);
        }
	length = 0;
	for(i=0;i<nFiles;i++)
	{
                strcpy(skipStrings(STRINGPTR(symh), i), tempNames[i]);
		length += strlen(tempNames[i]);
        	STRINGVALUE(symh,length) = '\0';
		length++;  /* for trailing null */
	}
	setDIM(symh, 1, nFiles);
	

	return(1);
}


/* Use fake handles for builtin functions*/
 /* Array of pointers to builtins */
static Symbol             **FunctionPointers;
 /* array of Function Symbols */
static FunctionSymbol      *FunctionSymbols;


/*
   Initialize everything in sight.  Called once from main().
   950921 Moved initial reading of "~/.inputrc" ("INPUTRC" on DOS) here

   960426 Added initialization of CHARACTER vector MACROFILES

   960502 Added code to get path of MacAnova folder on MacintoLE
		  and MACROFILES no longer includes a complete path
   960124 Added initialization of SELECTION under Motif
   970324 Add function read() which combines matread() and macroread()
   970530 Added initialization of math constants MV_PI, MV_E, ...
   970612 Added initialization of INPUTSTRINGS, etc.
   990324 arima.mac now part of MACROFILESsh

   960503 Added initialization of CHARACTER variable DATAPATHS

   961003 Added arguments dataPath and macroPath
          DATAPATH is initialized to dataPath
		  DATAPATHS is initialized to vector(dataPath,macroPath),
		  except only 1 if they are identical.  If either is
		  (char *) 0, defines DATAPATHNAME and/or MACROPATHNAME
		  are used
		  Also, the default initialization of MACROFILE, DATAFILE
		  and MACROFILES no longer includes a complete path
   960124 Added initialization of SELECTION under Motif
   970324 Add function read() which combines matread() and macroread()
   970530 Added initialization of math constants MV_PI, MV_E, ...
   970612 Added initialization of INPUTSTRINGS, etc.
   990324 arima.mac now part of MACROFILES
   990423 Added initialization of special symbol GRAPHWINDOWS
   000430 Added function memoryinfo()
   000824 Added multivar.mac, math.mac and graphics.mac to standard MACROFILES
          changed macro NMACROFILES to MAXMACROFILES
   000830 Made sure MACROFILE initialized to "MacAnova.mac"
   000904 Added regress.mac to standard MACROFILES
   001025 Changed MAXLINE to LINECHUNK
*/

static void initMathConstants(void)
{
/*  initialized math constants */
	MV_E    = exp(1);
	MV_PI_4 = atan(1.0);
	MV_PI_2 = 2.0*MV_PI_4;
	MV_PI   = 4.0*MV_PI_4;

#ifdef M_E
	MV_LOG2E    =       M_LOG2E;
	MV_LOG10E   =      M_LOG10E;
	MV_LN2      =         M_LN2;
	MV_LN10     =        M_LN10;
	MV_1_PI     =        M_1_PI;
	MV_2_PI     =        M_2_PI;
	MV_2_SQRTPI =    M_2_SQRTPI;
	MV_SQRT2    =       M_SQRT2;
#ifdef M_SQRT1_2
	MV_SQRT1_2  =     M_SQRT1_2;
#else /*M_SQRT1_2*/
	MV_SQRT1_2  =      M_SQRT_2; /*Borland math.h name of constant */
#endif /*M_SQRT1_2*/
#else /*M_E*/
	MV_LOG2E    =  1.4426950408889634074;
	MV_LOG10E   = 0.43429448190325182765;
	MV_LN2      = 0.69314718055994530942;
	MV_LN10     = 2.30258509299404568402;
	MV_1_PI     = 0.31830988618379067154;
	MV_2_PI     = 0.63661977236758134308;
	MV_2_SQRTPI = 1.12837916709551257390;
	MV_SQRT2    = 1.41421356237309504880;
	MV_SQRT1_2  = 0.70710678118654752440;
#endif /*M_E*/

#ifdef DSIGNIF
	/* if necessary, initialize maximum integer represented by double */
	if (MaxInteger < 0)
	{
		int      n = DSIGNIF - 1;

		MaxInteger = 1.0;
		while (n-- > 0)
		{
			MaxInteger *= 2.0;
		}
	}
#endif /*DSIGNIF*/

	/*
	   set value for MISSING and infinities
	*/

#if LONGSPERDOUBLE == 2
	setBdouble(Missing, HIMISSING, LOWMISSING);
#ifdef HASINFINITY
	setBdouble(PlusInfinity, HIPLUSINFINITY, LOWPLUSINFINITY);
	setBdouble(MinusInfinity, HIMINUSINFINITY, LOWMINUSINFINITY);
#endif /*HASINFINITY*/
#else /*LONGSPERDOUBLE == 2*/
	MISSING = MISSINGVALUE; /* may have to do something fancier than this */
#ifdef HASINFINITY
	PLUSINFINITY = 1.0/0.0;
	MINUSINFINITY = -1.0/0.0;
#endif /*HASINFINITY*/
#endif /*LONGSPERDOUBLE == 2*/

/*
  TOOBIGVALUE should be HUGE_VAL which may be infinite
*/
#ifndef NOSTRTOD
	{
		char          tooBigValueString[30];
		
		strcpy(tooBigValueString, TOOBIGVALUESTRING);
		TOOBIGVALUE = mystrtod(tooBigValueString,(char **) 0);
	}
#else /*NOSTRTOD*/
#ifdef HUGE_VAL
	TOOBIGVALUE = HUGE_VAL;
#else /*HUGE_VAL*/
	TOOBIGVALUE = exp(1e200);
#endif /*HUGE_VAL*/
#endif /*NOSTRTOD*/

	OLDMISSING = OLDMISSINGVALUE;  /* -99999.9999 = original value for MISSING*/
}

#define MAXMACROFILES   50
#define MAXHELPFILES    50

#ifndef FULLPATHSIZE
#define FULLPATHSIZE  1024
#endif /*FULLPATHSIZE*/

#include <float.h>


#ifdef SKEL_CARAPACE

void cpcinitialize(long maxModelStackDepth,
	char ** searchPaths,  /* This will include any paths added on the command line
                               * via -path foo, and will end with the two standard paths,
                               * which are the path to the user's standard MacAnova support
                               * directory and the installed system MacAnova directory.
                               */
	char ** helpFiles,    /* This will include any help files added on the command line
                               * via -addhelp foo.
                               */
	char ** macroFiles,   /* This will include any macro files added on the command line
                               * via -macro foo.
                               */
	char ** dataFiles,    /* This will include any data files added on the command line
                               * via -data foo.
                               */
	char *  defaultHelpFile,  /* This will have the file set via a -help foo or NULL */
	char *  homePath,     /* This will have the home path, either the default
                               * determined from the environment or that set by
                               * -home foo on the command line.
                               */
	char * prompt)
{
	long            i;
	Symbolhandle    symh;
	long            nBuiltins = 0;
	double          value;
#ifdef READLINE
#if defined(INPUTRCFILE) || defined(DEFAULTINPUTRC)
	Keymap           keymap; /* current keymap */
	char            *backwardkillword = "\033\010"; /*Esc-Backspace*/
	FILE            *readlineBindings = (FILE *) 0;
	Symbolhandle     symhPath = SymbolZero;
#endif /*defined(INPUTRCFILE) || defined(DEFAULTINPUTRC)*/
#endif /*READLINE*/
	WHERE("initialize");

	getElapsedTime((double *) 0); /* save current time */
	getElapsedTime((double *) 0); /* do it twice for good measure */

	initMathConstants();
	
	if (!inputInit())
	{ /* allocate space for INPUTSTRINGS and ISTRCHARS*/
		goto fatalError;
	}
	LINECHUNK = DEFAULTLINECHUNK;

	/*
	  Allocate space for globals, BATCHECHO, INPUTFILE, INPUTFILENAMES,
	  LASTINPUTWASCR, PROMPTS
	  */
	if (!batchInit())
	{
		goto fatalError;
	}
	
	/*
		Allocate WHILESTARTS, WHILECONDITIONS, WHILELIMITS, 
		WHILEBRACKETLEV, FORVECTORS, IFCONDITIONS, IFBRACKETLEV
	*/
	if (!syntaxStatusInit())
	{
		goto fatalError;
	}
	/*
		create size TABLESIZE symbol table and zero it out
	*/
	if(!Symbolinit())
	{ 
		goto fatalError;
	}
	/*
		allocate global DBOUT
	*/
	if (!dbugInit())
	{
		goto fatalError;
	}

	/*
		allocate glm related globals
	*/
	if(!glmInit(maxModelStackDepth))
	{
		goto fatalError;
	}

#ifdef HASDYNLOAD
	/* Allocate table space for dynloads */
	if (!dynloadInit())
	{
		goto fatalError;
	}
#endif /*HASDYNLOAD*/

	/*
		Allocate and initialize space for special symbols
	*/

	SpecialSymbols =
	  (Symbolhandle *) mygetpointer(NSPECIALSYMBOLS*sizeof(Symbolhandle));
	SpecialSymbolNames =
	  (mvSymbolName *) mygetpointer(NSPECIALSYMBOLS*sizeof(mvSymbolName));
	if (SpecialSymbols == (Symbolhandle *) 0 ||
		SpecialSymbolNames == (mvSymbolName *) 0)
	{
		goto fatalError;
	}
	for (i = 0; i < NSPECIALSYMBOLS; i++)
	{
		SpecialSymbols[i] = SymbolZero;
	}

	/* 
		Create fake handles for function symbols
	*/
	for (nBuiltins = 0; builtins[nBuiltins].name; nBuiltins++)
	{ /* count builtins */
		;
	}
	FunctionPointers = (Symbol **) mygetpointer(nBuiltins * sizeof(Symbol *));
	FunctionSymbols = (FunctionSymbol *) mygetpointer(nBuiltins* sizeof(FunctionSymbol));
	if(FunctionPointers == (Symbol **) 0 ||
	   FunctionSymbols == (FunctionSymbol *) 0)
	{
		goto fatalError;
	}

	/*
		make fake Symbolhandle for each builtin function
	*/
	for (i = 0;i < nBuiltins; i++)
	{
		symh = FunctionPointers + i;
		*symh = (Symbol *) (FunctionSymbols+i);
		setTYPE(symh, BLTIN);
		markFakeSymbol(symh); /* indicates it's fake if we ever need to know */
		setNDIMS(symh, 1);
		setDIM(symh, 1, 1);
		setPREV(symh, SymbolZero);
		setNEXT(symh, SymbolZero);
		setFPTR(symh, builtins[i].function);
		setLABELS(symh, (LabelsType **) 0);
		setNOTES(symh, (NotesType **) 0);
		
		if (quoteArgs(builtins[i].name))
		{
			setTYPE(symh, COMPLETETYPE(symh) | QUOTEARGSMASK);
			setNAME(symh, builtins[i].name + 1);
		}
		else
		{
			setNAME(symh, builtins[i].name);
		}
		
		Addsymbol(symh);
	} /*for (i = 0;i < nBuiltins; i++)*/

	/*
	  now install pre-defined constants
	  010727 pre-defined numerical constants are locked variables
	*/

	for (i = 0; Constants[i].name; i++)
	{
		int     lockit = (Constants[i].name[0] == lockPrefix) ? 1 : 0;
		
		symh = RInstall(Constants[i].name+lockit, 1L);

		if(symh == SymbolError)
		{
			goto fatalError;
		}
		if (lockit)
		{
			setLocked(symh);
		}
		if(strcmp(NAME(symh),"PI") == 0)
		{
			value = MV_PI;
		}
		else if(strcmp(NAME(symh),"E") == 0)
		{
			value = MV_E;
		}
		else if(strcmp(NAME(symh),"DEGPERRAD") == 0)
		{
			value = 45.0/MV_PI_4;
		}
		else if (strcmp(NAME(symh), "NA") == 0)
		{
			value = MISSING;
		}
		else
		{
			value = Constants[i].value;
		}
		DATAVALUE(symh,0) = value;
	} /*for (i = 0; Constants[i].name; i++)*/

	
	/*
	  Create NULLSYMBOL (value returned by print(), etc.
	  Note that it is *not* installed in the symbol table
	*/
	NULLSYMBOL = Makesymbol(NULLSYM);
	if(NULLSYMBOL == SymbolError)
	{
		goto fatalError;
	}
	setNAME(NULLSYMBOL, "NULLSYMBOL");


	/* Initialize special symbols */

	/* Special symbol CLIPBOARD */

	strcpy(CLIPBOARDNAME, NAMEOFCLIPBOARD);
	if (!iniClipboard(CLIPBOARDNAME))
	{
		goto fatalError;
	}

#ifdef HASSELECTION
	/* Special symbol SELECTION */
	strcpy(SELECTIONNAME, NAMEOFSELECTION);
	if (!iniClipboard(SELECTIONNAME))
	{
		goto fatalError;
	}
#endif /*HASSELECTION*/

	/* Special symbol GRAPHWINDOWS */
	strcpy(GRAPHWINDOWSNAME, NAMEOFGRAPHWINDOWS);
	if (!iniGraphWindows(GRAPHWINDOWSNAME))
	{
		goto fatalError;
	}
	
	/* Install pre-defined macros */
	if (!iniMacros())
	{
		goto fatalError;
	}
	
	/* install pre-defined string variables (they must all be scalar) */

	for (i = 0; stringvars[i].name; i++)
	{
		int     lockit = (stringvars[i].name[0] == lockPrefix) ? 1 : 0;
		char   *name = stringvars[i].name+lockit;
		char   *contents = stringvars[i].string;
		
		symh = CInstall(name, strlen(contents) + 1);
		if(symh == SymbolError)
		{
			goto fatalError;
		}
		if (lockit)
		{
			setLocked(symh);
		}
		strcpy(STRINGPTR(symh), contents);
		cleanString(STRINGPTR(symh));
	} /*for (i = 0; stringvars[i].name; i++)*/

	setDefaultOptions(); /* set default options */
	if (prompt != (char *) 0)
	{
		strcpy((char *) DEFAULTPROMPTS[0], (char *) prompt);
	}
	strcpy((char *) PROMPTS[0], (char *) DEFAULTPROMPTS[0]);

#ifdef VERSION
	sprintf(OUTSTR,"MacAnova %s ",VERSION);
#ifdef TODAY
	strcat(OUTSTR,TODAY);
#endif /*TODAY*/
	symh = CInstall("VERSION", strlen(OUTSTR) + 1);
	if(symh == SymbolError)
	{
		goto fatalError;
	}
	strcpy(STRINGPTR(symh),OUTSTR);
	setLocked(symh);

	*OUTSTR = '\0';
	VERSION_ID = myduphandle(STRING(symh));
	if(VERSION_ID == (char **) 0)
	{
		goto fatalError;
	}
#endif /*VERSION*/

#ifdef READLINE
	/* Initialize line editing and history */
#if defined(INPUTRCFILE) || defined(DEFAULTINPUTRC)
	using_history(); /*added 980108*/
	rl_initialize(); /*added 980106 */
	rl_initialize_funmap();
	keymap = rl_get_keymap();
	if (keymap != (Keymap) 0)
	{ /* this binding is standard but not the default in readline 1.1 */
		rl_set_key (backwardkillword, rl_backward_kill_word, keymap);
	} /*if (keymap != (Keymap) 0)*/
#ifndef INPUTRCFILE
	pathName = OUTSTR;
	strcpy(pathName, dataPathName);
	strcat(pathName, DEFAULTINPUTRC);
#else /*INPUTRCFILE*/
	pathName = INPUTRCFILE;
#endif /*INPUTRCFILE*/
	if (pathName[0] != '\0')
	{
		readlineBindings = fmyopen(pathName, TEXTREADMODE);
		if (readlineBindings != (FILE *) 0)
		{
			fclose(readlineBindings);
			rl_read_init_file(pathName);
		} /*if (readlineBindings != (FILE *) 0)*/
	} /*if (pathName[0] != '\0')*/
#endif /*defined(INPUTRCFILE) || defined(DEFAULTINPUTRC)*/
#endif /*READLINE*/

/*
   initialize or clear virtually everything in sight
*/
	*OUTSTR = '\0';
	for (i = 0;i < MAXWDEPTH;i++)
	{
		WHILELIMITS[i] = MAXWHILE;
		FORVECTORS[i] = (double **) 0;
	}
	clearGlobals();

	INPUTFILE[0] = STDIN;
	INPUTFILENAMES[0] = &StdinName;

	for (i=1;i<MAXBDEPTH;i++)
	{
		INPUTFILE[i] = (FILE *) 0;
		INPUTFILENAMES[i] = (char **) 0;
	}

	TMPHANDLE = mygethandle(sizeof(double)); /*warehouseable length*/
	if(TMPHANDLE == (char **) 0)
	{
		goto fatalError;
	}
	INPUTSTRING = ThisInputstring = (unsigned char **) TMPHANDLE;
	(*INPUTSTRING)[0] = '\0';

	GUBED = 0; /* this is here to make it easy to change for debugging */

	if(!initFilesAndPaths(searchPaths, helpFiles, macroFiles,   
		dataFiles, defaultHelpFile, homePath))
	{
		goto fatalError;
	}


#ifdef DJGPP
	INTERRUPT = INTSET;
	(void) interrupted(0); /* to get DJGPP started */
	INTERRUPT = INTNOTSET;
#endif /*DJGPP*/
#if defined(BCPP) || defined (wx_msw)
	(void) _control87(EM_OVERFLOW,EM_OVERFLOW);
#endif /*BCPP||wxmsw*/

#ifdef wx_motif
#if (0) /* inititalization now done in base frame initializer */
	initializePSPrinter();
#endif
#endif /*wx_motif*/
	return ;

  fatalError:
	FatalError = 1;
	
} /*cpcinitialize()*/
#endif /*SKEL_CARAPACE*/


void initialize(long maxModelStackDepth,
				char * helpFileName, char * dataFileName,
				char * macroFileName,
				char * dataPath, char * macroPath, char * homePath,
				char * prompt, char *appDir)
{
	long            i;
	Symbolhandle    symh, symh1;
	long            length, nMacroFiles = 0, nHelpFiles = 0, nPaths = 0;
	long            length1,length2,length3, nBuiltins = 0;
	int             macAnovaMacIndex = 0;
	char           *dataPathName = (char *) 0, *macroPathName = (char *) 0;
	char           *macroFileNames[MAXMACROFILES];
	char           *helpFileNames[MAXHELPFILES], *helpIndexNames[MAXHELPFILES];
	char           *place;
	char            completePath[PATHSIZE+1];
	char            helpPathName[PATHSIZE+1];
	char			homePathName[FULLPATHSIZE+1];
	char		userPathName[FULLPATHSIZE+1];
	char		appPathName[FULLPATHSIZE+1];
	char           *pathName;
	double          value;
#ifdef READLINE
#if defined(INPUTRCFILE) || defined(DEFAULTINPUTRC)
	Keymap           keymap; /* current keymap */
	char            *backwardkillword = "\033\010"; /*Esc-Backspace*/
	FILE            *readlineBindings = (FILE *) 0;
	Symbolhandle     symhPath = SymbolZero;
#endif /*defined(INPUTRCFILE) || defined(DEFAULTINPUTRC)*/
#endif /*READLINE*/
	WHERE("initialize");

	getElapsedTime((double *) 0); /* save current time */
	getElapsedTime((double *) 0); /* do it twice for good measure */

	pathName = (char *) 0;

	if(appDir != (char *) 0 && strlen(appDir) + strlen("SharedSupport") + 3 < FULLPATHSIZE)
	{
		strcpy(appPathName,appDir);
		length = strlen(appDir);
		if(appPathName[length-1] != DIRSEPARATOR[0])
		{
			appPathName[length] = DIRSEPARATOR[0];
			appPathName[length+1] = '\0';
		}
		strcat(appPathName,"SharedSupport");
		strcat(appPathName,DIRSEPARATOR);
	}
	else
	{
		strcpy(appPathName,DATAPATHNAME);
#if defined(BASE_MICROSOFT)
		pathName = get_dataPath();
		for (i = 0; pathName[i] != '\0'; i++)
		{ /* make sure all path separators are '\\' */
			if (pathName[i] == '/')
			{
				pathName[i] = DIRSEPARATOR[0];
			}
		} /*for (i == 0; pathName[i] != '\0'; i++)*/
		if(strlen(pathName) + strlen("SharedSupport") + 3 < FULLPATHSIZE)
		{
			strcpy(appPathName,pathName);
		}
#elif defined(VMS)
		/* I don't really know what to do for VMS */
		strncpy(appPathName,getenv("ANOVA$HOME"),FULLPATHSIZE);
		appPathName[FULLPATHSIZE-1] = '\0';
#endif
		if(appPathName[strlen(appPathName)-1] != DIRSEPARATOR[0])
		{
			strcat(appPathName,DIRSEPARATOR);
		}
		strcat(appPathName,"SharedSupport");
		strcat(appPathName,DIRSEPARATOR);
	}

	if(homePath != (char *) 0)
	{
		if((i=strlen(homePath)) < FULLPATHSIZE - 3)
		{
			strcpy(homePathName,homePath);
		}
	}
	else
	{
#if defined(BASE_MICROSOFT)
		strcpy(homePathName,getenv("HOMEDRIVE"));
		if(strlen(homePathName)+strlen(getenv("HOMEPATH")) + 3 < FULLPATHSIZE)
		{
			strcat(homePathName,getenv("HOMEPATH"));
		}
#elif defined(BASE_UNIX)
		if(strlen(getenv("HOME")) + 3 < FULLPATHSIZE)
		{
			strcpy(homePathName,getenv("HOME"));
		}
#elif defined(BASE_MACINTOSH)
		/* macGetPath() returns address of completePath */
/* 
   FIX ME I don't think that this will still work, but on
   the other hand, I don't think that this code is ever 
   used on the Mac any more
*/
		{
			Integer     homeVolume = getMvVolID(homeVol);
			LongInt     homeDirectory = getMvDirID(homeVol);
			
			pathName = macGetPath(completePath, homeVolume, homeDirectory);
		}
#elif defined(VMS)
		if(strlen(getenv("ANOVA$HOME")) + 3 < FULLPATHSIZE)
		{
			strcpy(homePathName,getenv("ANOVA$HOME"));
		}
#endif
	}
	i = strlen(homePathName);
	if(homePathName[i-1] != DIRSEPARATOR[0])
	{
		homePathName[i] = DIRSEPARATOR[0];
		homePathName[i+1] = 0;
	}
	homePath = homePathName;
	strcpy(userPathName,homePathName);
	if(strlen(userPathName) + strlen("MyMacAnovaFiles") + 3 < FULLPATHSIZE)
	{
		strcat(userPathName,"MyMacAnovaFiles");
		strcat(userPathName,DIRSEPARATOR);
	}

	
/*  initialized math constants */
	MV_E    = exp(1);
	MV_PI_4 = atan(1.0);
	MV_PI_2 = 2.0*MV_PI_4;
	MV_PI   = 4.0*MV_PI_4;

#ifdef M_E
	MV_LOG2E    =       M_LOG2E;
	MV_LOG10E   =      M_LOG10E;
	MV_LN2      =         M_LN2;
	MV_LN10     =        M_LN10;
	MV_1_PI     =        M_1_PI;
	MV_2_PI     =        M_2_PI;
	MV_2_SQRTPI =    M_2_SQRTPI;
	MV_SQRT2    =       M_SQRT2;
#ifdef M_SQRT1_2
	MV_SQRT1_2  =     M_SQRT1_2;
#else /*M_SQRT1_2*/
	MV_SQRT1_2  =      M_SQRT_2; /*Borland math.h name of constant */
#endif /*M_SQRT1_2*/
#else /*M_E*/
	MV_LOG2E    =  1.4426950408889634074;
	MV_LOG10E   = 0.43429448190325182765;
	MV_LN2      = 0.69314718055994530942;
	MV_LN10     = 2.30258509299404568402;
	MV_1_PI     = 0.31830988618379067154;
	MV_2_PI     = 0.63661977236758134308;
	MV_2_SQRTPI = 1.12837916709551257390;
	MV_SQRT2    = 1.41421356237309504880;
	MV_SQRT1_2  = 0.70710678118654752440;
#endif /*M_E*/

#ifdef DSIGNIF
	/* if necessary, initialize maximum integer represented by double */
	if (MaxInteger < 0)
	{
		int      n = DSIGNIF - 1;

		MaxInteger = 1.0;
		while (n-- > 0)
		{
			MaxInteger *= 2.0;
		}
	}
#endif /*DSIGNIF*/

	/*
	   set value for MISSING and infinities
	*/

#if LONGSPERDOUBLE == 2
	setBdouble(Missing, HIMISSING, LOWMISSING);
#ifdef HASINFINITY
	setBdouble(PlusInfinity, HIPLUSINFINITY, LOWPLUSINFINITY);
	setBdouble(MinusInfinity, HIMINUSINFINITY, LOWMINUSINFINITY);
#endif /*HASINFINITY*/
#else /*LONGSPERDOUBLE == 2*/
	MISSING = MISSINGVALUE; /* may have to do something fancier than this */
#ifdef HASINFINITY
	PLUSINFINITY = 1.0/0.0;
	MINUSINFINITY = -1.0/0.0;
#endif /*HASINFINITY*/
#endif /*LONGSPERDOUBLE == 2*/

/*
  TOOBIGVALUE should be HUGE_VAL which may be infinite
*/
#ifndef NOSTRTOD
	{
		char          tooBigValueString[30];
		
		strcpy(tooBigValueString, TOOBIGVALUESTRING);
		TOOBIGVALUE = mystrtod(tooBigValueString,(char **) 0);
	}
#else /*NOSTRTOD*/
#ifdef HUGE_VAL
	TOOBIGVALUE = HUGE_VAL;
#else /*HUGE_VAL*/
	TOOBIGVALUE = exp(1e200);
#endif /*HUGE_VAL*/
#endif /*NOSTRTOD*/

	OLDMISSING = OLDMISSINGVALUE;  /* -99999.9999 = original value for MISSING*/
	
	if (!inputInit())
	{ /* allocate space for INPUTSTRINGS and ISTRCHARS*/
		goto fatalError;
	}
	LINECHUNK = DEFAULTLINECHUNK;

#if defined(MACINTOSH) || defined(WXWIN)
	/*
	  000707 allocation done to preserve precious global space
	  001025 changed size from MAXLINE+1 to MAXDIALOGBUF + 1
	  001027 changed how things are done; allocation now in fillFromDialog()
	*/
#if (0)
	LineBuffer = (unsigned char *) mygetpointer(MAXDIALOGBUF + 1);
	if (LineBuffer == (unsigned char *) 0)
	{
		goto fatalError;
	}
#endif /*0*/
#endif /*MACINTOSH || WXWIN*/

	/*
	  Allocate space for globals, BATCHECHO, INPUTFILE, INPUTFILENAMES,
	  LASTINPUTWASCR, PROMPTS
	  */
	if (!batchInit())
	{
		goto fatalError;
	}
	
	/*
		Allocate WHILESTARTS, WHILECONDITIONS, WHILELIMITS, 
		WHILEBRACKETLEV, FORVECTORS, IFCONDITIONS, IFBRACKETLEV
	*/
	if (!syntaxStatusInit())
	{
		goto fatalError;
	}
	/*
		create size TABLESIZE symbol table and zero it out
	*/
	if(!Symbolinit())
	{ 
		goto fatalError;
	}
	/*
		allocate global DBOUT
	*/
	if (!dbugInit())
	{
		goto fatalError;
	}

	/*
		allocate glm related globals
	*/
	if(!glmInit(maxModelStackDepth))
	{
		goto fatalError;
	}

#ifdef HASDYNLOAD
	/* Allocate table space for dynloads */
	if (!dynloadInit())
	{
		goto fatalError;
	}
#endif /*HASDYNLOAD*/

	/*
		Allocate and initialize space for special symbols
	*/

	SpecialSymbols =
	  (Symbolhandle *) mygetpointer(NSPECIALSYMBOLS*sizeof(Symbolhandle));
	SpecialSymbolNames =
	  (mvSymbolName *) mygetpointer(NSPECIALSYMBOLS*sizeof(mvSymbolName));
	if (SpecialSymbols == (Symbolhandle *) 0 ||
		SpecialSymbolNames == (mvSymbolName *) 0)
	{
		goto fatalError;
	}
	for (i = 0; i < NSPECIALSYMBOLS; i++)
	{
		SpecialSymbols[i] = SymbolZero;
	}

	/* 
		Create fake handles for function symbols
	*/
	for (nBuiltins = 0; builtins[nBuiltins].name; nBuiltins++)
	{ /* count builtins */
		;
	}
	FunctionPointers = (Symbol **) mygetpointer(nBuiltins * sizeof(Symbol *));
	FunctionSymbols = (FunctionSymbol *) mygetpointer(nBuiltins*
													  sizeof(FunctionSymbol));
	if(FunctionPointers == (Symbol **) 0 ||
	   FunctionSymbols == (FunctionSymbol *) 0)
	{
		goto fatalError;
	}

	/*
		make fake Symbolhandle for each builtin function
	*/
	for (i = 0;i < nBuiltins; i++)
	{
		symh = FunctionPointers + i;
		*symh = (Symbol *) (FunctionSymbols+i);
		setTYPE(symh, BLTIN);
		markFakeSymbol(symh); /* indicates it's fake if we ever need to know */
		setNDIMS(symh, 1);
		setDIM(symh, 1, 1);
		setPREV(symh, SymbolZero);
		setNEXT(symh, SymbolZero);
		setFPTR(symh, builtins[i].function);
		setLABELS(symh, (LabelsType **) 0);
		setNOTES(symh, (NotesType **) 0);
		
		if (quoteArgs(builtins[i].name))
		{
			setTYPE(symh, COMPLETETYPE(symh) | QUOTEARGSMASK);
			setNAME(symh, builtins[i].name + 1);
		}
		else
		{
			setNAME(symh, builtins[i].name);
		}
		
		Addsymbol(symh);
	} /*for (i = 0;i < nBuiltins; i++)*/

	/*
	  now install pre-defined constants
	  010727 pre-defined numerical constants are locked variables
	*/

	for (i = 0; Constants[i].name; i++)
	{
		int     lockit = (Constants[i].name[0] == lockPrefix) ? 1 : 0;
		
		symh = RInstall(Constants[i].name+lockit, 1L);

		if(symh == SymbolError)
		{
			goto fatalError;
		}
		if (lockit)
		{
			setLocked(symh);
		}
		if(strcmp(NAME(symh),"PI") == 0)
		{
			value = MV_PI;
		}
		else if(strcmp(NAME(symh),"E") == 0)
		{
			value = MV_E;
		}
		else if(strcmp(NAME(symh),"DEGPERRAD") == 0)
		{
			value = 45.0/MV_PI_4;
		}
		else if (strcmp(NAME(symh), "NA") == 0)
		{
			value = MISSING;
		}
		else
		{
			value = Constants[i].value;
		}
		DATAVALUE(symh,0) = value;
	} /*for (i = 0; Constants[i].name; i++)*/

	
	/*
	  Create NULLSYMBOL (value returned by print(), etc.
	  Note that it is *not* installed in the symbol table
	*/
	NULLSYMBOL = Makesymbol(NULLSYM);
	if(NULLSYMBOL == SymbolError)
	{
		goto fatalError;
	}
	setNAME(NULLSYMBOL, "NULLSYMBOL");


	/* Initialize special symbols */

	/* Special symbol CLIPBOARD */

	strcpy(CLIPBOARDNAME, NAMEOFCLIPBOARD);
	if (!iniClipboard(CLIPBOARDNAME))
	{
		goto fatalError;
	}

#ifdef HASSELECTION
	/* Special symbol SELECTION */
	strcpy(SELECTIONNAME, NAMEOFSELECTION);
	if (!iniClipboard(SELECTIONNAME))
	{
		goto fatalError;
	}
#endif /*HASSELECTION*/

	/* Special symbol GRAPHWINDOWS */
	strcpy(GRAPHWINDOWSNAME, NAMEOFGRAPHWINDOWS);
	if (!iniGraphWindows(GRAPHWINDOWSNAME))
	{
		goto fatalError;
	}
	
	/* Install pre-defined macros */
	if (!iniMacros())
	{
		goto fatalError;
	}
	
	/* install pre-defined string variables (they must all be scalar) */

	for (i = 0; stringvars[i].name; i++)
	{
		int     lockit = (stringvars[i].name[0] == lockPrefix) ? 1 : 0;
		char   *name = stringvars[i].name+lockit;
		char   *contents = stringvars[i].string;
		
		symh = CInstall(name, strlen(contents) + 1);
		if(symh == SymbolError)
		{
			goto fatalError;
		}
		if (lockit)
		{
			setLocked(symh);
		}
		strcpy(STRINGPTR(symh), contents);
		cleanString(STRINGPTR(symh));
	} /*for (i = 0; stringvars[i].name; i++)*/

	setDefaultOptions(); /* set default options */
	strcpy((char *) DEFAULTPROMPTS[0], (char *) STANDARDPROMPT);
	if (prompt != (char *) 0 && strlen(prompt) > 0)
	{
		strcpy((char *) DEFAULTPROMPTS[0], (char *) prompt);
	}
	strcpy((char *) PROMPTS[0], (char *) DEFAULTPROMPTS[0]);

#ifdef VERSION
	sprintf(OUTSTR,"MacAnova %s ",VERSION);
#ifdef TODAY
	strcat(OUTSTR,TODAY);
#endif /*TODAY*/
	symh = CInstall("VERSION", strlen(OUTSTR) + 1);
	if(symh == SymbolError)
	{
		goto fatalError;
	}
	strcpy(STRINGPTR(symh),OUTSTR);
	setLocked(symh);

	*OUTSTR = '\0';
	VERSION_ID = myduphandle(STRING(symh));
	if(VERSION_ID == (char **) 0)
	{
		goto fatalError;
	}
#endif /*VERSION*/


	/* Create global HELPFILENAME*/
	if (helpFileName == (char *) 0)
	{
		helpFileName = HELPNAME;
	}
	length = strlen(helpFileName);
	
	HELPFILENAME = mygethandle(length + 1);
	if(HELPFILENAME == (char **) 0)
	{
		goto fatalError;
	}
	strcpy(*HELPFILENAME, helpFileName);

/*   Install CHARACTER scalar DATAPATH containing dataPath if
     defined, otherwise userPathName */
	pathName = (dataPath != (char *) 0 ? dataPath : userPathName);
	
	length = strlen(pathName);
#ifndef NOSEPARATOR
	if (pathName[length-1] != DIRSEPARATOR[0])
	{
		length++;
	}
#endif /*NOSEPARATOR*/
	symh = CInstall("DATAPATH",length + 1);
	if (symh == SymbolError)
	{
		goto fatalError;
	}
	strcpy(STRINGPTR(symh),pathName);
#ifndef NOSEPARATOR
	STRINGVALUE(symh, length-1) = DIRSEPARATOR[0];
	STRINGVALUE(symh, length) = '\0';
#endif /*NOSEPARATOR*/
/* 
   Install CHARACTER vector DATAPATHS containing dataPath, userPathName,
   and appPathName. 
*/
	nPaths = 2;
	length = 0;
	length1 = 0;
	if(dataPath != (char *) 0)
	{
		nPaths = 3;
		length1 = strlen(dataPath);
		length = length + length1 + 1; /* for null */
#ifndef NOSEPARATOR
		if(dataPath[length1-1] != DIRSEPARATOR[0])
		{
			length1++;
			length++;
		}
#endif /*NOSEPARATOR*/
	}
	length2 = strlen(userPathName);
	length = length + length2 + 1; /* for null */
#ifndef NOSEPARATOR
	if(userPathName[length2-1] != DIRSEPARATOR[0])
	{
		length2++;
		length++;
	}
#endif /*NOSEPARATOR*/
	length3 = strlen(appPathName);
	length = length + length3 + 1;
#ifndef NOSEPARATOR
	if(appPathName[length3-1] != DIRSEPARATOR[0])
	{
		length3++;
		length++;
	}
#endif /*NOSEPARATOR*/
	
	symh1 = CInstall("DATAPATHS",length + 1);
	if (symh1 == SymbolError)
	{
		goto fatalError;
	}
	if(nPaths == 3)
	{
		strcpy(STRINGPTR(symh1),dataPath);
#ifndef NOSEPARATOR
		STRINGPTR(symh1)[length1-1] = DIRSEPARATOR[0];
		STRINGPTR(symh1)[length1] = '\0';
#endif /*NOSEPARATOR*/
	}
	strcpy(skipStrings(STRINGPTR(symh1), nPaths-2), userPathName);
#ifndef NOSEPARATOR
	STRINGVALUE(symh1,length1+length1*1+length2-1) = DIRSEPARATOR[0];
	STRINGVALUE(symh1,length1+length1+1+length2) = '\0';
#endif /*NOSEPARATOR*/
	strcpy(skipStrings(STRINGPTR(symh1), nPaths-1), appPathName);
#ifndef NOSEPARATOR
	STRINGVALUE(symh1,length1+length1*1+length2+1+length3-1) = DIRSEPARATOR[0];
	STRINGVALUE(symh1,length1+length1*1+length2+1+length3) = '\0';
#endif /*NOSEPARATOR*/
	setDIM(symh1, 1, nPaths);
	
	/* pre-install CHARACTER scalar HOME */
	length = strlen(homePath);

	if(length > 0)
	{
#ifndef NOSEPARATOR
		if (homePath[length-1] != DIRSEPARATOR[0])
		{ /* make room for terminating separator */
			length++;
		}
#endif /*NOSEPARATOR*/
		symh = CInstall("HOME",length + 1);
		if (symh == SymbolError)
		{
			goto fatalError;
		}
		strcpy(STRINGPTR(symh),homePath);
#ifndef NOSEPARATOR
		/* make sure HOME terminates with DIRSEPARATOR[0]*/
		STRINGVALUE(symh, length-1) = DIRSEPARATOR[0];
		STRINGVALUE(symh, length) = '\0';
#endif /*NOSEPARATOR*/
#ifdef BASE_MICROSOFT
		{
			char       *pc;
		
			for (pc = STRINGPTR(symh); *pc != '\0'; pc++)
			{ /* ensure all separators are standard backslashes*/
				if (*pc == '/')
				{
					*pc = DIRSEPARATOR[0];
				}
			} /*for (pc = STRINGPTR(symh); *pc != '\0'; pc++)*/
		}
#endif /*BASE_MICROSOFT*/
#ifndef BASE_MACINTOSH
		setMvDirInfo(homeVol, STRINGPTR(symh));
#endif /*BASE_MACINTOSH*/
	} /*if(length > 0)*/

	/* install CHARACTER scalar DATAFILE */
	if(dataFileName == (char *) 0)
	{
		dataFileName = DATAFILENAME;
	}
	
	length = strlen(dataFileName);

	symh = CInstall("DATAFILE", length + 1);
	if (symh == SymbolError)
	{
		goto fatalError;
	}

	strcpy(STRINGPTR(symh), dataFileName);

	/* Install CHARACTER scalar MACROFILE and CHARACTER vector MACROFILES*/
	for (i = 0; i < MAXMACROFILES; i++)
	{
		macroFileNames[i] = NullString;
	}
	if (macroFileName != (char *) 0)
	{
		macroFileNames[nMacroFiles++] = macroFileName;
	}

	/*
	  If and when new standard macro files are added, make sure
	  the total number of names, counting MacAnova.mac, does not
	  exceed MAXMACROFILES

	  000829 added new files and reordered them; MacAnova.mac is now last
	*/

#ifdef GRAPHICSMACROFILENAME
	macroFileNames[nMacroFiles++] = GRAPHICSMACROFILENAME;
#endif /*GRAPHICSMACROFILENAME*/

#ifdef REGRESSMACROFILENAME
	macroFileNames[nMacroFiles++] = REGRESSMACROFILENAME;
#endif /*REGRESSMACROFILENAME*/

#ifdef DESIGNMACROFILENAME
	macroFileNames[nMacroFiles++] = DESIGNMACROFILENAME;
#endif /*DESIGNMACROFILENAME*/

#ifdef TSMACROFILENAME
	macroFileNames[nMacroFiles++] = TSMACROFILENAME;
#endif /*TSMACROFILENAME*/

#ifdef ARIMAMACROFILENAME
	macroFileNames[nMacroFiles++] = ARIMAMACROFILENAME;
#endif /*ARIMAMACROFILENAME*/

#ifdef MULVARMACROFILENAME
	macroFileNames[nMacroFiles++] = MULVARMACROFILENAME;
#endif /*MULVARMACROFILENAME*/

#ifdef MATHMACROFILENAME
	macroFileNames[nMacroFiles++] = MATHMACROFILENAME;
#endif /*MATHMACROFILENAME*/

#ifdef MACROFILENAME
    macAnovaMacIndex = nMacroFiles;
	macroFileNames[nMacroFiles++] = MACROFILENAME;
#endif /*MACROFILENAME*/

	if (nMacroFiles > 0)
	{
	    /* 000830 MACROFILE now initialized from macAnovaMacIndex element of macroFileNames */
		length = strlen(macroFileNames[macAnovaMacIndex]) + 1;

		symh = CInstall("MACROFILE",length);
		if(symh == SymbolError)
		{
			goto fatalError;
		}
		strcpy(STRINGPTR(symh), macroFileNames[macAnovaMacIndex]);
	    length = 0;
		for (i = 0; i < nMacroFiles; i++)
		{
			length += strlen(macroFileNames[i]) + 1;
		}
		symh = CInstall("MACROFILES", length);
		if (symh == SymbolError)
		{
			goto fatalError;
		}
		setDIM(symh, 1, nMacroFiles);
		place = STRINGPTR(symh);
		for (i = 0; i < nMacroFiles; i++)
		{
			place = copyStrings(macroFileNames[i], place, 1);
		}
	} /*if (nMacroFiles > 0)*/
		
	/*
	  010616 Install CHARACTER vector HELPFILES

	  If and when new standard help files are added, make sure
	  the total number of names, counting MacAnova.hlp, does not
	  exceed MAXHELPFILES
	  010620 Now include path
	  010703 Also install CHARACTER vector HELPINDICES
	  010725 Modified installation code; file names installed are
             exactly as defined
	*/
	helpIndexNames[nHelpFiles] = HELPINDEXNAME;
	helpFileNames[nHelpFiles++] = *HELPFILENAME;

#ifdef GRAPHICSHELPFILENAME
	helpIndexNames[nHelpFiles] = GRAPHICSHELPINDEXNAME;
	helpFileNames[nHelpFiles++] = GRAPHICSHELPFILENAME;
#endif /*GRAPHICSHELPFILENAME*/

#ifdef REGRESSHELPFILENAME
	helpIndexNames[nHelpFiles] = REGRESSHELPINDEXNAME;
	helpFileNames[nHelpFiles++] = REGRESSHELPFILENAME;
#endif /*REGRESSHELPFILENAME*/

#ifdef DESIGNHELPFILENAME
	helpIndexNames[nHelpFiles] = DESIGNHELPINDEXNAME;
	helpFileNames[nHelpFiles++] = DESIGNHELPFILENAME;
#endif /*DESIGNHELPFILENAME*/

#ifdef TSHELPFILENAME
	helpIndexNames[nHelpFiles] = TSHELPINDEXNAME;
	helpFileNames[nHelpFiles++] = TSHELPFILENAME;
#endif /*TSHELPFILENAME*/

#ifdef ARIMAHELPFILENAME
	helpIndexNames[nHelpFiles] = ARIMAHELPINDEXNAME;
	helpFileNames[nHelpFiles++] = ARIMAHELPFILENAME;
#endif /*ARIMAHELPFILENAME*/

#ifdef MULVARHELPFILENAME
	helpIndexNames[nHelpFiles] = MULVARHELPINDEXNAME;
	helpFileNames[nHelpFiles++] = MULVARHELPFILENAME;
#endif /*MULVARHELPFILENAME*/

#ifdef MATHHELPFILENAME
	helpIndexNames[nHelpFiles] = MATHHELPINDEXNAME;
	helpFileNames[nHelpFiles++] = MATHHELPFILENAME;
#endif /*MATHHELPFILENAME*/

	if (nHelpFiles > 0)
	{
		/*
		  Install HELPFILES and HELPINDICES
		*/
		Symbolhandle  symhFileNames, symhIndexNames;
		char         *filesPlace, *indexPlace;
		long          fileNameLength = 0, indexNameLength = 0;

		/* find lengths */
		for (i = 0; i < nHelpFiles; i++)
		{
			fileNameLength += strlen(helpFileNames[i]) + 1;
			indexNameLength += strlen(helpIndexNames[i]) + 1;
		} /*for (i = 1; i < nHelpFiles; i++)*/

		symhFileNames = CInstall("HELPFILES", fileNameLength);
		symhIndexNames = CInstall("HELPINDICES", indexNameLength);
		if (symhFileNames == SymbolError || symhIndexNames == SymbolError)
		{
			goto fatalError;
		}
		setDIM(symhFileNames, 1, nHelpFiles);
		setDIM(symhIndexNames, 1, nHelpFiles);

		helpFileNames[0] = *HELPFILENAME; /*play it safe; dereference again*/
		filesPlace = STRINGPTR(symhFileNames);
		indexPlace = STRINGPTR(symhIndexNames);
		for (i = 0; i < nHelpFiles; i++)
		{
			filesPlace = copyStrings(helpFileNames[i], filesPlace, 1);
			indexPlace = copyStrings(helpIndexNames[i], indexPlace, 1);
		}
	} /*if (nMacroFiles > 0)*/

#ifdef READLINE
	/* Initialize line editing and history */
#if defined(INPUTRCFILE) || defined(DEFAULTINPUTRC)
	using_history(); /*added 980108*/
	rl_initialize(); /*added 980106 */
	rl_initialize_funmap();
	keymap = rl_get_keymap();
	if (keymap != (Keymap) 0)
	{ /* this binding is standard but not the default in readline 1.1 */
		rl_set_key (backwardkillword, rl_backward_kill_word, keymap);
	} /*if (keymap != (Keymap) 0)*/
#ifndef INPUTRCFILE
	pathName = OUTSTR;
	strcpy(pathName, dataPathName);
	strcat(pathName, DEFAULTINPUTRC);
#else /*INPUTRCFILE*/
	pathName = INPUTRCFILE;
#endif /*INPUTRCFILE*/
	if (pathName[0] != '\0')
	{
		readlineBindings = fmyopen(pathName, TEXTREADMODE);
		if (readlineBindings != (FILE *) 0)
		{
			fclose(readlineBindings);
			rl_read_init_file(pathName);
		} /*if (readlineBindings != (FILE *) 0)*/
	} /*if (pathName[0] != '\0')*/
#endif /*defined(INPUTRCFILE) || defined(DEFAULTINPUTRC)*/
#endif /*READLINE*/

/*
   initialize or clear virtually everything in sight
*/
	*OUTSTR = '\0';
	for (i = 0;i < MAXWDEPTH;i++)
	{
		WHILELIMITS[i] = MAXWHILE;
		FORVECTORS[i] = (double **) 0;
	}
	clearGlobals();

	INPUTFILE[0] = STDIN;
	INPUTFILENAMES[0] = &StdinName;

	for (i=1;i<MAXBDEPTH;i++)
	{
		INPUTFILE[i] = (FILE *) 0;
		INPUTFILENAMES[i] = (char **) 0;
	}

	TMPHANDLE = mygethandle(sizeof(double)); /*warehouseable length*/
	if(TMPHANDLE == (char **) 0)
	{
		goto fatalError;
	}
	INPUTSTRING = ThisInputstring = (unsigned char **) TMPHANDLE;
	(*INPUTSTRING)[0] = '\0';

	GUBED = 0; /* this is here to make it easy to change for debugging */

#ifdef DJGPP
	INTERRUPT = INTSET;
	(void) interrupted(0); /* to get DJGPP started */
	INTERRUPT = INTNOTSET;
#endif /*DJGPP*/
#if defined(BCPP) || defined (wx_msw)
	(void) _control87(EM_OVERFLOW,EM_OVERFLOW);
#endif /*BCPP||wxmsw*/

#ifdef wx_motif
#if (0) /* inititalization now done in base frame initializer */
	initializePSPrinter();
#endif
#endif /*wx_motif*/
	return ;

  fatalError:
	FatalError = 1;
	
} /*initialize()*/


