/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 5.00 or later
*(C)*
*(C)* Copyright (c) 2006 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/


#ifdef SEGMENTED
#if defined(MPW) || defined(MW_CW)
#pragma segment List
#endif /*MPW||MW_CW*/
#endif /*SEGMENTED*/

/*
  990215 changed myerrorout() to putOutMsg()
  990226 replaced putOUTSTR() by putErrorOUTSTR()
  990430 Cleaned up some of listbrief()
  010728 Added keyword phrase 'locked:T'
  050729 It is no longer an error if no matching variables are
         found with 'keep:T'; NULL is returned
  051102 Fixed minor bug so that list(nrows:m,factor:T) lists only
         factors with m rows; previsouls factor:T was ignored
  060921 list(macros:F) behaves like list(all:T,macros:F)
*/

#include "globals.h"

static long            NObjects; /*number of items to list*/

#define Trash  NAMEFORTRASH
static Symbolhandle    Trash; /* will contain handles of items to list */

enum listTrash
{
	GOBJECTS = 0,
	NTRASH
};

/*
  return 1 if name1 > name2, -1 if name1 < name2, 0 if name1 == name2
  when name1 is folded to lower case letters.  name2 is assumed already in
  lower case
*/
static short compareNames(char *name1, char *name2)
{
	register char           c1, c2, *nm1 = name1, *nm2 = name2;
	
	do
	{
		c1 = *nm1++;
		c2 = *nm2++;
		if (isupper(c1))
		{
			c1 = tolower(c1);
		}
		
		if (c1 != c2)
		{
			return( (c1 > c2) ? 1 : -1);
		}
	} while (c1 != '\0');

	return (0);
} /*compareNames()*/

static char     Target[NAMELENGTH+1];
static long     MatchType;

/*
	980521 moved definition of match codes to globdefs.h
	010728 modified listTypeCodes
*/

enum listTypeCodes
{
	IREALS   = 0x0001,
	ICHARS   = 0x0002,
	ILOGICS  = 0x0004,
	IMACROS  = 0x0008,
	ISTRUCS  = 0x0010,
	IGRAPHS  = 0x0020,
	INULLS   = 0x0040,
	IFACTORS = 0x0080,
	ILABELED = 0x0100,
	INOTES   = 0x0200,
	ILOCKED  = 0x0400,
	IDIMENSIONED = IREALS|ICHARS|ILOGICS,
	IALL     = IREALS|ICHARS|ILOGICS|IMACROS|ISTRUCS|IGRAPHS|INULLS
};

static char        *TypeKeys[] = 
{
	"real", "char", "logi", "macr", "stru", "grap",
	"null", "fact", "labe", "note", "lock", "all", (char *) 0
};
static short        KeyCodes[] = 
{
	IREALS, ICHARS, ILOGICS, IMACROS, ISTRUCS, IGRAPHS,
	INULLS, IFACTORS, ILABELED, INOTES, ILOCKED, IALL
};

/* 970513 made v a de-referenced pointer rather than a handle */
static void sortObjects(void)
{
	long            n = NObjects;
	Symbolhandle   *v = (Symbolhandle *) *GARBAGEVALUE(Trash, GOBJECTS);
	Symbolhandle    z;
	char            pivot[NAMELENGTH+1], *pc1, *pc2;
	long            i, j, p, lv[20], uv[20];

	lv[0] = 0;
	uv[0] = n - 1;
	p = 0;
	while (p > -1)
	{
		if (lv[p] >= uv[p])
		{
			p = p - 1;
		} /*if (lv[p] >= uv[p])*/
		else
		{
			i = lv[p] - 1;
			j = uv[p];
			pc1 = NAME(v[j]);
			pc2 = pivot;
			while (*pc1)
			{
				*pc2 = *pc1++;
				if (isupper(*pc2))
				{
					*pc2 = tolower(*pc2);
				}
				pc2++;
			}
			*pc2 = '\0';
			while (i < j)
			{
				i = i + 1;
				while (compareNames(NAME(v[i]),pivot) < 0)
				{
					i = i + 1;
				}
				j = j - 1;
				while (j > i)
				{
					if (compareNames(NAME(v[j]), pivot) <= 0)
					{
						break;
					}
					j = j - 1;
				}
				if (i < j)
				{
					z = v[i];
					v[i] = v[j];
					v[j] = z;
				}
			}
			j = uv[p];
			z = v[i];
			v[i] = v[j];
			v[j] = z;
			if (i - lv[p] < uv[p] - i)
			{
				lv[p + 1] = lv[p];
				uv[p + 1] = i - 1;
				lv[p] = i + 1;
			}
			else
			{
				lv[p + 1] = i + 1;
				uv[p + 1] = uv[p];
				uv[p] = i - 1;
			}
			p++;
		} /*if (lv[p] >= uv[p]){}else{}*/
	} /*while (p > -1)*/
} /*sortObjects()*/

static void addToList(Symbolhandle symh)
{
	Symbolhandle     **objects =
	  (Symbolhandle **) GARBAGEVALUE(Trash, GOBJECTS);
	
	if (matchName(NAME(symh),MatchType,Target))
	{
		(*objects)[NObjects++] = symh;
	}
} /*addToList()*/

/*
  970513 Added argument fmt.
         On listbrief() fmt is (char *) 0 when object is last in the
         output line
  970617 list() reportswhether a macro is inline or out of line
  970628 list() reports dimensions of LONG variables
  980512 returns size of object when brief < -1
  010728 list() reports if variable is locked
  030207 list() reports if variable has labels
  030208 list() reports if variable has notes
*/
static long listObject(Symbolhandle symh,long brief, char *fmt)
{
	char            *name;
	int              type;
	long             i, nclass, size = 0;
	WHERE("listObject");
	
	name = NAME(symh);
	type = TYPE(symh);
	
	if(brief < 0)
	{ /* list()*/
		if (symh != SymbolZero)
		{
			char        *outstr = OUTSTR, *inLine = (char *) 0;
			int          labels = 0, notes = 0, locked = 0;
			int          nExtra = 0;

			sprintf(outstr,"%-14s %s%-6s",name,
					isSpecial(symh) ? "*" : " ", typeName(type));
			outstr += strlen(outstr);
			if(brief < -1)
			{ /*size:T*/
				size = SizeofSymbol(symh);
				sprintf(outstr, " %7ld   ", size);
				outstr += strlen(outstr);
			}

			if (type == REAL || type == LOGIC || type == CHAR ||
				type == LONG || type == STRUC)
			{
				for (i = 1; i <= NDIMS(symh); i++)
				{
					sprintf(outstr, " %-5ld", DIMVAL(symh,i));
					outstr += strlen(outstr);
				}
				if ((nclass = NCLASS(symh)) > 0)
				{
					sprintf(outstr, " FACTOR with %ld levels",nclass);
					outstr += strlen(outstr);
				}
				labels = HASLABELS(symh);
				if (labels)
				{
					nExtra++;
				}
			} /*if (type == REAL||type == LOGIC||type == CHAR||type == STRUC)*/
			else if (type == MACRO)
			{
				inLine = (NCLASS(symh) < 0) ? "in-line" : "out-of-line";
				nExtra++;
			}
			notes = HASNOTES(symh);
			if (notes)
			{
				nExtra++;
			}
			locked = isLocked(symh);
			if (locked)
			{
				nExtra++;
			}
			
			if (labels || locked || inLine)
			{
				char    *comma = ", ";
				
				strcpy(outstr, " (");
				outstr += strlen(outstr);
				if (inLine)
				{
					sprintf(outstr, "%s%s", inLine,
							(--nExtra > 0) ? comma : NullString);
					outstr += strlen(outstr);
				}
				
				if (labels)
				{
					sprintf(outstr, "%s%s", "labels",
							(--nExtra > 0) ? comma : NullString);
					outstr+= strlen(outstr);
				}
				if (notes)
				{
					sprintf(outstr, "%s%s", "notes",
							(--nExtra > 0) ? comma : NullString);
					outstr+= strlen(outstr);
				}
				if (locked)
				{
					strcpy(outstr, "locked");
					outstr += strlen(outstr);
				}
				strcpy(outstr, ")");
			} /*if (labels || locked || inLine)*/
			putOUTSTR();
		} /*if (symh != SymbolZero)*/
	} /*if (brief < 0)*/
	else
	{ /* listbrief() */
		if (fmt != (char *) 0)
		{ /*not last in line */
			sprintf(OUTSTR, fmt, name);
			myprint(OUTSTR);
		}
		else
		{
			putOutMsg(name);
		}
	} /*if (brief < 0){}else{}*/
	*OUTSTR = '\0';

	return (size);
} /*listObject()*/

static char    *KeyNames[] =
{
	"keep", "size", "invis", "nrows", "ncols", "ndims"
};

enum listKeyCodes
{
	KKEEP,
	KSIZE,
	KINVIS,
	KNROWS,
	KNCOLS,
	KNDIMS,
	NKEYS
};

/*
  030207 modified to process ILABELED and changed handling of ILOCKED
         locked:T or labels:T limit listing to locked or labeled
  030208 now processes INOTES; notes:T limits listing to variables
         with notes
*/
static int matchTypeAndShape(Symbolhandle symh, long op,
							 long nCols, long nRows, long nDims)
{
	if (symh != SymbolZero)
	{
		long        type = TYPE(symh);
		int         typeMatched, shapeMatched, labelMatched, notesMatched;
		int         lockMatched, factorMatched;

		typeMatched = !(op & IALL) ||
		  (op & IREALS) && type == REAL ||
		  (op & ICHARS) && type == CHAR ||
		  (op & IMACROS) && type == MACRO ||
		  (op & ILOGICS) && type == LOGIC ||
		  (op & IGRAPHS) && type == PLOTINFO ||
		  (op & INULLS) && type == NULLSYM ||
		  (op & ISTRUCS) && type == STRUC;
		
		shapeMatched = 
		  (nCols < 0 ||
		   nCols == 1 && isVector(symh) ||
		   nCols > 1 && NDIMS(symh) >= 2 &&
		   DIMVAL(symh,2) == nCols) &&
		  (nRows < 0 || DIMVAL(symh,1) == nRows) &&
		  (nDims < 0 || NDIMS(symh) == nDims);

		labelMatched = !(op & ILABELED) || HASLABELS(symh);
		notesMatched = !(op & INOTES) || HASNOTES(symh);
		lockMatched = !(op & ILOCKED) || isLocked(symh);
		factorMatched = !(op & IFACTORS) || NCLASS(symh) > 0 && type == REAL;
		
		return (typeMatched && shapeMatched && labelMatched && notesMatched &&
				lockMatched && factorMatched);
	} /*if (symh != SymbolZero)*/
	return (0);
} /*matchTypeAndShape()*/

/*
  970513 modified slightly; removed globals Fmt and NperLine (now local
         to listbrief()) and added fmt as argument to listObject().
         The net effect is to eliminate some trailing spaces on listbrief()
         output.
  980521 Added arguments to call to scanPat() indicating attempting to
         match a MacAnova type name and providing a maximum length for
		 the pattern
  000128 Fixed minor bug which make recognition of some keywords dependent
         on their position in the argument list
  000202 Changed it so that keywords are operative in debugging mode
         (GUBED & 2 != 0)
  000430 Improved behavior in debugging mode. Add matchTypeAndShape()
  010728 Added keyword phrase 'locked:T' 
  030207 Added keyword phrase 'labeled:T' to restrict the list to variables
         with labels.
         Modified output of list() to include 'labeled' for a variable
         with dimension labels
         Only variables matching all non-type qualifications are listed
         This is a slight change in behaviour
  030208 Add keyword phrase 'notes:T' to restrict list to variables with
         notes
  030904 Fixed bug so that factors:T doesn't cause a crash
  060921 Changed behavior so that, for example, listbrief(macros:F) 
         does the same as listbrief(all:T,macros:F), listing everything
         except macros
*/
Symbolhandle    listbrief(Symbolhandle list)
{
	Symbolhandle    symh, result = SymbolZero;
	Symbolhandle  **objects;
	long            type, i, j, badarg = 0, startArgs = 0;
	long            ikey;
	long            brief = (strcmp(FUNCNAME,"listbrief") == 0) ? 1 : -1;
	long            foundUndef = 0;
	long            foundMissing = 0;
	char           *keyword, *string;
	char            foundKeyword = 0;
	long            nargs = NARGS(list);
	long            nargs1;
	long            keep = 0, invis = 0;
	long            nosort = 0;
	long            maxNobj = Nsymbols - Nfunctions;
	long            nRows = -1, nCols = -1, nDims = -1;
	long            needed;
	int             listEverything = GUBED & 2;
	WHERE("list");
	
	*OUTSTR = '\0';
	Trash = GarbInstall(NTRASH);
	if (Trash == SymbolError)
	{
		goto errorExit;
	}
	
	if (nargs == 1 && COMPVALUE(list, 0) == EmptyArgument)
	{
		nargs = 0;
	}
	
	/* 
	  Unadvertised feature:  Allow last argument 'nosort:T' to suppress sort
	*/

	if (nargs > 0 && (symh = COMPVALUE(list,nargs-1)) != EmptyArgument &&
	   (keyword = isKeyword(symh)) && strcmp(keyword,"nosort") == 0 &&
	   isTorF(symh))
	{
		nosort = (DATAVALUE(symh,0) != 0.0) ? 1 : 0;
		Removesymbol(symh);
		nargs--;
		COMPVALUE(list, nargs) = SymbolZero;
	}
	nargs1 = nargs; /* eventually number of non-keywords */
	
	if (listEverything)
	{ /* for debugging lists everything including scratch */
		maxNobj += Nscratch;
	}
	
	if (!getScratch(objects,0,maxNobj,Symbolhandle))
	{
		goto errorExit;
	}
	NObjects = 0;
	MatchType = ANYMATCH;
	

	if (nargs > 0)
	{
		/*
			check for keys other than symbol type
			keep, size, invis, nrows, ncols, ndims
		*/
		for (ikey = 0; ikey < NKEYS; ikey++)
		{			
			long        keyValue, keyindex;

			if ((keyindex = findKeyword(list, KeyNames[ikey], 0)) >= 0)
			{
				nargs1--;
				
				symh = COMPVALUE(list, keyindex);
				keyword = isKeyword(symh);
				if (!listEverything)
				{
					setNAME(symh, USEDKEYWORD);
				}
				
				if (ikey != KNROWS && ikey != KNCOLS && ikey != KNDIMS)
				{
					/* 'invisible', 'size', 'keep' have LOGICAL scalar values */
					if (!isTorF(symh))
					{
						notTorF(keyword);
						goto errorExit;
					}
					keyValue = (DATAVALUE(symh,0) != 0.0);
					if (symh == COMPVALUE(list,nargs-1) && !listEverything)
					{
						Removesymbol(symh);
						COMPVALUE(list,nargs-1) = SymbolZero;
						nargs--;
					}
				} /*if (ikey != KNROWS && ikey != KNCOLS && ikey != KNDIMS)*/
				else
				{
					/* integer keyword values */
					if (!isInteger(symh, POSITIVEVALUE))
					{
						notPositiveInteger(keyword);
						goto errorExit;
					}
					keyValue = (long) DATAVALUE(symh,0);
				} /*if (ikey != KNROWS && ikey != KNCOLS){}else{}*/

				switch (ikey)
				{
				  case KKEEP:
					keep = keyValue;
					break;
					
				  case KSIZE:
					if (brief > 0)
					{
						sprintf(OUTSTR,
								"WARNING: %s not legal keyword with %s(); ignored",
								keyword, FUNCNAME);
						putErrorOUTSTR();
					} /*if (brief > 0)*/
					else
					{
						brief = (keyValue) ? -2 : -1;
					}
					break;

				  case KINVIS:
					invis = (keyValue) ? 1 : 0;
					break;

				  case KNROWS:
					nRows = keyValue;
					foundKeyword = 1;
					break;

				  case KNCOLS:
					nCols = keyValue;
					foundKeyword = 1;
					break;
					
				  case KNDIMS:
					nDims = keyValue;
					foundKeyword = 1;
				} /*switch (ikey)*/
			} /*if ((keyindex = findKeyword(list, KeyNames[ikey], 0)) >= 0)*/
		} /*for (ikey = 0; ikey < NKEYS; ikey++)*/
	} /*if (!(listEverything) && nargs > 0)*/

	symh = COMPVALUE(list,0);
	if (nargs > 0 && isCharOrString(symh))
	{ /* first arg may be a pattern */
		if (isscratch(NAME(symh)) ||
		   (keyword = isKeyword(symh)) && strncmp(keyword,"pat",3) == 0)
		{
			string = STRINGPTR(symh);
			if (strlen(string) > NAMELENGTH)
			{
				sprintf(OUTSTR,"ERROR: '%s' is too long a pattern for %s()",
						string, FUNCNAME);
				goto errorExit;
			}
			if (!scanPat(string, (long *) &MatchType, Target,
						 variableNameCheck, NAMELENGTH))
			{
				sprintf(OUTSTR,"ERROR: '%s' is not valid pattern for %s()",
						string, FUNCNAME);
				goto errorExit;
			}
			
			startArgs = 1;
			symh = SymbolZero;
			if (nargs == 1 && !listEverything)
			{
				nargs--;
			}
			nargs1--;
		}
	} /*if (nargs > 0 && isCharOrString(symh))*/

	if (nargs1 == 0 && !foundKeyword)
	{  /* no arguments or just pattern, list everything that matches */
		for (symh = Firstsymbol(0); symh != SymbolZero ;
			 symh = Nextsymbol(symh,0))
		{ /* omit builtins, undefined, temporary, and scratch variables */
			type = TYPE(symh);
			if (type != BLTIN &&
				(listEverything ||
				 type != ASSIGNED &&
				 (invis || !istempname(NAME(symh)) && !invisname(NAME(symh)))))
			{
				addToList(symh);
			}
		}

		if (listEverything)
		{ /* list scratch table */
			for (symh = Firstsymbol(1); symh != SymbolZero; 
				symh = Nextsymbol(symh,1))
			{
				addToList(symh);
			}
		}
	} /*if (nargs1 == 0 && !foundKeyword)*/
	else   /* one or more arguments or unprocessed keywords*/
	{
		int       op = 0, notop = 0;

		for (i = startArgs;i < nargs;i++)
		{
			symh = COMPVALUE(list,i);
			if ((keyword = isKeyword(symh)))
			{
				if (strcmp(keyword,USEDKEYWORD+2) != 0)
				{
					/* a keyword, but one not processed */
					for (j = 0; (string = TypeKeys[j]) != (char *) 0;j++)
					{
						if (strncmp(keyword,string,4) == 0)
						{
							if (!isTorF(symh))
							{
								notTorF(keyword);
								goto errorExit;
							}
							if (DATAVALUE(symh,0) != 0.0)
							{
								op |= KeyCodes[j];
								notop &= ~KeyCodes[j];
							}
							else
							{
								op &= ~KeyCodes[j];
								notop |= KeyCodes[j];
							}
							break;
						} /*if (strncmp(keyword,string,4) == 0)*/
					} /*for (j=0; (string = TypeKeys[j]) != (char *) 0;j++)*/
					
					if (string != (char *) 0)
					{
						foundKeyword = 1;
					}
					else if (!listEverything)
					{
						badKeyword(FUNCNAME, keyword);
						goto errorExit;
					}
				} /*if (strcmp(keyword,USEDKEYWORD+2) != 0)*/
			} /*if (keyword = isKeyword(symh))*/
			else if (foundKeyword && !listEverything)
			{ /*
				keyword like 'nrows' or 'real previously found
				limiting variables listed
			   */
				sprintf(OUTSTR,
						"ERROR: illegal combination of keywords and variables in %s()",
						FUNCNAME);
				goto errorExit;
			}
		} /*for (i = startArgs;i < nargs;i++)*/

		if (foundKeyword)
		{ /* foundKeyword is 0 if only 'keep', 'size' or 'invis' found */
			if (!op && notop)
			{
				op = IALL & ~notop;
			}

			if (op == 0 && nRows < 0 && nCols < 0 && nDims < 0 &&
				!listEverything)
			{
				sprintf(OUTSTR,
						"WARNING: no object types specified for %s()",
						FUNCNAME);
				putErrorOUTSTR();
			} /*if (op == 0 && nRows < 0 && nCols < 0 && nDims < 0)*/
			else	
			{
				if (nRows > 0 || nCols > 0 || nDims > 0)
				{
					op = (op == 0) ? 
					  IDIMENSIONED : op & (IDIMENSIONED | IFACTORS);
				}

				for (symh = Firstsymbol(0); symh != SymbolZero;
						symh = Nextsymbol(symh,0))
				{/* omit scratch, builtins, undefined, temporary variables */
					if ((listEverything && TYPE(symh) != BLTIN ||
						 (isDefined(symh) &&
						  (invis ||
						   !istempname(NAME(symh)) && !invisname(NAME(symh)))))
						&& matchTypeAndShape(symh, op, nCols, nRows, nDims))
					{
						addToList(symh);
					}
				}

				if (listEverything)
				{
					/* Check the scratch table, too */
					for (symh = Firstsymbol(1); symh != SymbolZero;
							symh = Nextsymbol(symh,1))
					{
						if (matchTypeAndShape(symh, op, nCols, nRows, nDims))
						{
							addToList(symh);
						}
					}
				} /*if (listEverything)*/
			} /*if (op == 0 && nRows < 0 && nCols < 0 && nDims < 0){}else{}*/
		} /* if (foundKeyword) */
		else
		{
			for (i = startArgs;i < nargs; i++)
			{
				symh = COMPVALUE(list,i);
				if (symh == EmptyArgument)
				{
					foundMissing = 1;
				}
				else if (listEverything)
				{
					addToList(symh);
				}
				else if (isscratch(NAME(symh)))
				{
					badarg = 1;
				}
				else if (!isDefined(symh))
				{
					foundUndef = 1;
				}
				else if (!isKeyword(symh))
				{
					addToList(symh);
				}
			} /*for (i = startArgs;i < nargs; i++)*/
		} /* if (foundKeyword){}else{}*/
		
		if (badarg)
		{
			sprintf(OUTSTR,
					"WARNING: arguments to %s() must not be expressions or strings",
					FUNCNAME);
			putErrorOUTSTR();
		} /*if (badarg)*/

		if (foundMissing)
		{
			sprintf(OUTSTR,
					"WARNING: missing argument(s) to %s()", FUNCNAME);
			putErrorOUTSTR();
		} /*if (foundMissing)*/

		if (foundUndef)
		{
			for (i = 0;i < nargs; i++)
			{
				symh = COMPVALUE(list,i);
				if (symh != SymbolZero && !isDefined(symh))
				{
					sprintf(OUTSTR,"WARNING: %s is not a variable",
							NAME(symh));
					putErrorOUTSTR();
				}
			} /*for (i = 0;i < nargs; i++)*/
		} /*if (foundUndef)*/
	} /*if (nargs == 1 && symh == SymbolZero){}else{}*/

	if (NObjects != 0)
	{
		if (!nosort)
		{
			sortObjects();
		}
	
		if (!keep)
		{
			if (brief < 0)
			{
				/* list() */
				long     totalSize = 0;

				for (i=0;i < NObjects;i++)
				{
					totalSize += listObject((*objects)[i], brief, (char *) 0);
				}
				if (brief < -1)
				{
					/*list(...,size:T)*/
					sprintf(OUTSTR, "Total %-16s %7ld", "of listed",
							totalSize);
					putOUTSTR();
					sprintf(OUTSTR, "Total %-16s %7ld", "memory in use",
							CurrentMemory);
					putOUTSTR();
#if(0)
					sprintf(OUTSTR, "Total %-16s %7ld", "maximum memory used",
							MaxMemory);
					putOUTSTR();
#endif
				}
			} /*if (brief < 0)*/
			else
			{
				/* listbrief()*/
				long            nPerLine = SCREENWIDTH/(NAMELENGTH+1), nlines;
				char            fmt[10];

				if (nPerLine < 1)
				{
					nPerLine = 1;
				}
				nlines = (NObjects - 1)/nPerLine + 1;

				sprintf(fmt,"%%-%lds",(long) NAMELENGTH+1);
				for (j = 0;j < nlines; j++)
				{
					for (i = j;i < NObjects;i += nlines)
					{
						(void) listObject((*objects)[i], brief,
										  (i + nlines < NObjects) ?
										  fmt : (char *) 0);
					}
				} /*for (j = 0;j < nlines; j++)*/
			} /*if (brief < 0){}else{}*/
		} /*if (!keep)*/
		else
		{
			/* list(...,keep:T) or listbrief(...,keep:T) */
			needed = 0;
			for (i = 0;i < NObjects;i++)
			{
				needed += strlen(NAME((*objects)[i])) + 1;
			}
			if ((result = Install(SCRATCH,CHAR)) == SymbolError ||
			   (TMPHANDLE = mygethandle(needed)) == (char **) 0)
			{
				goto errorExit;
			}
			setSTRING(result,TMPHANDLE);
			setNDIMS(result,1);
			setDIM(result,1,NObjects);
			string = *TMPHANDLE;
			for (i = 0;i < NObjects;i++)
			{
				string = copyStrings(NAME((*objects)[i]), string, 1);
			}
		} /*if (!keep){}else{}*/
	} /*if (NObjects != 0)*/	

	if (!keep || NObjects == 0)
	{
		result = NULLSYMBOL;
	} /*if (!keep)*/

	emptyTrash();
	
	return (result);

  errorExit:
	putErrorOUTSTR();

	Removesymbol(result);

	emptyTrash();
	
	return (0);
} /*listbrief()*/


