/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.13 or later
*(C)*
*(C)* Copyright (c) 2003 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/


#ifdef SEGMENTED
#if defined(MPW) || defined(MW_CW)
#pragma segment Lowess
#endif /*MPW||MW_CW*/
#endif /*SEGMENTED*/


#include "globals.h"

/*
  030703 New file containing lowess(), lowest() and lowessmv()
*/

/*
 Functions lowess() and lowest() are adapted from a translation
 to C from ratfor code by 
   W. S. Cleveland
   ATT Bell Laboratories
   Murray Hill NJ 07974
 
 obtained from statlib.
 They may be freely used.

 The following is adapted from docmumentation in the original file
 
	   COMPUTER PROGRAMS FOR LOCALLY WEIGHTED REGRESSION

	  This package consists  of  two  FORTRAN  programs  for
 smoothing    scatterplots   by   robust   locally   weighted
 regression, or lowess.   The  principal  routine  is  LOWESS
 which   computes   the  smoothed  values  using  the  method
 described in The Elements of Graphing Data, by William S.
 Cleveland    (Wadsworth,    555 Morego   Street,   Monterey,
 California 93940).

	  LOWESS calls a support routine, LOWEST, the code for
 which is included. LOWESS also calls a routine  SORT,  which
 the user must provide.

	  To reduce the computations, LOWESS  requires  that  the
 arrays  X  and  Y,  which  are  the  horizontal and vertical
 coordinates, respectively, of the scatterplot, be such  that
 X  is  sorted  from  smallest  to  largest.   The  user must
 therefore use another sort routine which will sort X  and  Y
 according  to X.
	  To summarize the scatterplot, YS,  the  fitted  values,
 should  be  plotted  against X.   No  graphics  routines are
 available in the package and must be supplied by the user.

	  The following are data and output from LOWESS that  can
 be  used  to check your implementation of the routines.  The
 notation (10)v means 10 values of v.

 X values:
   1  2  3  4  5  (10)6  8  10  12  14  50

 Y values:
	18  2  15  6  10  4  16  11  7  3  14  17  20  12  9  13  1  8  5  19

 YS values with F = .25, NSTEPS = 0, DELTA = 0.0
  13.659  11.145  8.701  9.722  10.000  (10)11.300  13.000  6.440  5.596
	5.456  18.998

 YS values with F = .25, NSTEPS = 0 ,  DELTA = 3.0
   13.659  12.347  11.034  9.722  10.511  (10)11.300  13.000  6.440  5.596
	 5.456  18.998

 YS values with F = .25, NSTEPS = 2, DELTA = 0.0
   14.811  12.115  8.984  9.676  10.000  (10)11.346  13.000  6.734  5.744
	 5.415  18.998

							LOWESS

 Calling sequence

 lowess(x,y,n,f,nsteps,delta,ys,rw,res);

 Purpose

 lowess() computes the smooth of a scatterplot of y  against  x
 using  robust  locally  weighted regression.  Fitted values,
 ys, are computed at each of the  values  of  the  horizontal
 axis in x.

 Argument description

	   x = Input; abscissas of the points on the
		   scatterplot; the values in X must be ordered
		   from smallest to largest.
	   y = Input; ordinates of the points on the
		   scatterplot.
	   n = Input; dimension of x,y,ys,rw, and res.
	   f = Input; specifies the amount of smoothing; f is
		   the fraction of points used to compute each
		   fitted value; as f increases the smoothed values
		   become smoother; choosing f in the range .2 to
		   .8 usually results in a good fit; if you have no
		   idea which value to use, try f = .5.
  nsteps = Input; the number of iterations in the robust
		   fit; if nsteps = 0, the nonrobust fit is
		   returned; setting nsteps equal to 2 should serve
		   most purposes.
   delta = Input; nonnegative parameter which may be used
		   to save computations; if n is less than 100, set
		   delta equal to 0.0; if n is greater than 100 you
		   should find out how delta works by reading the
		   additional instructions section.
	  ys = Output; fitted values; ys(i) is the fitted value
		   at x(i); to summarize the scatterplot, ys(i)
		   should be plotted against x(i).
	  rw = Output; robustness weights; rw(i) is the weight
		   given to the point (x(i),y(i)); if rw is (double *) 0,
		   robustness weights are not computed.
	 res = Output; residuals; res(i) = y(i)-ys(i).

 Other programs called

		lowest()
		SSORT()

                   Additional instructions

   DELTA can be used to save computations.   Very  roughly  the
   algorithm  is  this:   on the initial fit and on each of the
   NSTEPS iterations locally weighted regression fitted  values
   are computed at points in X which are spaced, roughly, DELTA
   apart; then the fitted values at the  remaining  points  are
   computed  using  linear  interpolation.   The  first locally
   weighted regression (l.w.r.) computation is carried  out  at
   X(1)  and  the  last  is  carried  out at X(N).  Suppose the
   l.w.r. computation is carried out at  X(I).   If  X(I+1)  is
   greater  than  or  equal  to  X(I)+DELTA,  the  next  l.w.r.
   computation is carried out at X(I+1).   If  X(I+1)  is  less
   than X(I)+DELTA, the next l.w.r.  computation is carried out
   at the largest X(J) which is greater than or equal  to  X(I)
   but  is not greater than X(I)+DELTA.  Then the fitted values
   for X(K) between X(I)  and  X(J),  if  there  are  any,  are
   computed  by  linear  interpolation  of the fitted values at
   X(I) and X(J).  If N is less than 100 then DELTA can be  set
   to  0.0  since  the  computation time will not be too great.
   For larger N it is typically not necessary to carry out  the
   l.w.r.  computation for all points, so that much computation
   time can be saved by taking DELTA to be  greater  than  0.0.
   If  DELTA =  Range  (X)/k  then,  if  the  values  in X were
   uniformly  scattered  over  the  range,  the   full   l.w.r.
   computation  would be carried out at approximately k points.
   Taking k to be 50 often works well.

   Method

   The fitted values are computed by using the nearest neighbor
   routine  and  robust locally weighted regression of degree 1
   with the tricube weight function.  A few additional features
   have  been  added.  Suppose r is FN truncated to an integer.
   Let  h  be  the  distance  to  the  r-th  nearest   neighbor
   from X(I).   All  points within h of X(I) are used.  Thus if
   the r-th nearest neighbor is exactly the  same  distance  as
   other  points,  more  than r points can possibly be used for
   the smooth at  X(I).   There  are  two  cases  where  robust
   locally  weighted regression of degree 0 is actually used at
   X(I).  One case occurs when  h  is  0.0.   The  second  case
   occurs  when  the  weighted  standard error of the X(I) with
   respect to the weights w(j) is  less  than  .001  times  the
   range  of the X(I), where w(j) is the weight assigned to the
   j-th point of X (the tricube  weight  times  the  robustness
   weight)  divided by the sum of all of the weights.  Finally,
   if the w(j) are all zero for the smooth at X(I), the  fitted
   value is taken to be Y(I).

				   LOWEST

   Calling sequence

   ys = lowest(x, y, n, xs, nleft, nright, w, rw, &ok);

   Purpose

   lowest() is a support routine for lowess() and  ordinarily  will
   not  be  called  by  the  user.   The  fitted  value,  ys,  is
   computed  at  the  value,   xs,   of  the   horizontal   axis.
   Robustness  weights,   rw,   can  be employed in computing the
   fit.

   Argument description

	  x = Input;abscissas of the points on the
	  scatterplot;the values in x must be ordered
	  from smallest to largest.
	  y = Input;ordinates of the points on the
	  scatterplot.
	  n = Input;dimension of x, y,w,  and rw.
	 xs = Input;value of the horizontal axis at which the
	  smooth is computed.
	 ys = Output;fitted value at xs.
  nleft = Input;index of the first point which should be
	  considered in computing the fitted value.
 nright = Input;index of the last point which should be
	  considered in computing the fitted value.
	  w = Output;w(i) is the weight for y(i) used in the
	  expression for ys,  which is the sum from
	  i = nleft to nright of w(i)*y(i);w(i) is
	  defined only at locations nleft to nright.
	 rw = Input;robustness weights; no weights used when rw is
          (double *) 0
	 ok = Output;logical variable;if the weights for the
	  smooth are all 0.0,  the fitted value,  ys,  is not
	  computed and ok is 0;if the
	  fitted value is computed ok is 1

   Method

   The smooth at xs is computed using (robust) locally weighted
   regression of degree 1.  The tricube weight function is used
   with h equal to the maximum of xs-x(nleft) and x(nright)-xs.
   Two  cases  where  the  program  reverts to locally weighted
   regression of degree 0 are described  in  the  documentation
   for lowess().
*/

/*
   980410 In C version of lowest()
       ys now longer in argument list; lowest() returns ys as value
       userw no longer in argument list; presence of weights is
        signaled by non-NULL rw.
   Coding has been modified so that subscripts are 0 based.
   030707 new argument fitInfo contains info on fit
          fitInfo[0]  weighted mean of x's
          fitInfo[1]  weighted mean of y's
          fitInfo[2]  slope (0 if sum(((xfitinfo[0])^2)) is too small
		  returned value is fitInfo[1] + fitInfo[2]*(xs - fitInfo[0])
*/

#undef MAX
#define MAX(a,b) (((a) > (b)) ? (a) : (b))

#define infoLength 3
typedef double fitInfoType[infoLength];

static double lowest(double x [], double y [], long n, double xs,
					 long nleft, long nright, double w [], double * rw,
					 fitInfoType fitInfo, int * ok)
{
	/*	double x[n], y[n], w[n], rw[n];*/
	double          range, h, h1, h9, a, ys = 0.0;
	long            j;
	int             userw = (rw != (double *) 0);
	
	range = x[n-1] - x[0];

	h = MAX(xs - x[nleft], x[nright] - xs);
	h9 = .999*h;
	h1 = .001*h;
	 /* compute sum of weights */
	a = 0.0;

	for (j = nleft;j < n;j++)
	{
		double          r;
		
		/*  compute weights (pick up all ties on right) */
		r = fabs(x[j] - xs);
		if (r <= h9)
		{ /*  small enough for non - zero weight */
			if (r > h1)
			{
				double       temp = r/h;
				
				temp = 1.0 - temp*temp*temp;
				w[j] = temp*temp*temp;
			}
			else
			{
				w[j] = 1.0;
			}
			
			if (userw)
			{
				w[j] *= rw[j];
			}
			a += w[j];
		} /*if (r <= h9)*/
		else if (x[j] > xs)
		{
			break; /*  get out at first zero wt on right */
		}
		else
		{
			w[j] = 0.0;
		}
	} /*for (j=nleft;j < n;j++)*/

	*ok = a > 0.0;
	fitInfo[0] = fitInfo[1] = fitInfo[2] = 0.0;
	if (*ok)
	{ /*  weighted least squares */
		long         nrt = j - 1;
		/* nrt is rightmost pt (may be > nright because of ties) */

		for (j = nleft; j <= nrt ; j++)
		{
			w[j] /= a; /*  make sum of w[j] == 1 */
			fitInfo[1] += w[j]*y[j]; /*  weighted center of y values */
		}
		if (h > 0.0)
		{ /*  use linear fit */
			double           b = 0.0, c = 0.0;

			for (j = nleft; j <= nrt ; j++)
			{
				fitInfo[0] += w[j]*x[j]; /*  weighted center of x values */
			}

			for (j = nleft;j <= nrt ;j++)
			{
				double     temp = x[j] - fitInfo[0];
				
				b += w[j]*temp*y[j];
				c += w[j]*temp*temp;
			}
			
			if (sqrt(c) > .001*range) 
			{
				/*  points are spread out enough to compute slope */
				b /= c;
				fitInfo[2] = b;
			} /*if (sqrt(c) > .001*range) */
		} /*if (h > 0.0)*/
		else
		{
			fitInfo[0] = xs;
		}
		ys = fitInfo[1] + fitInfo[2]*(xs - fitInfo[0]);
	} /*if (*ok)*/
	return (ys);
} /*lowest()*/

/*
   980410 C version of lowess()
          Coding was modified so that subscripts are 0 based.
   030707 new arguments xpred, ypred, n1.
          When n1 > 0, ypred[i] is set to smoothed value for xpred[i],
          i = 0, 1, ..., n1-1 using the same type of weighted
          regression
          ypred[i] is MISSING when weights are all 0
*/

void   sortquick(double X[], long n);

#define SORT(X,N) sortquick(X,N)

static void lowess(double x [], double y [], long n, double f, long nsteps,
				   double delta, double ys [], double rw [], 
				   double * xpred, double * ypred, long n1, double res [])
{
	/*double x[n], y[n], ys[n], rw[n], res[n];*/

	fitInfoType    fitInfo, fitInfoLeft;
	long           ns;
	int            iter;
	WHERE("lowess");

	if (n < 2)
	{
		ys[0] = y[0];
		if (n1 > 0)
		{
			long      ip;
			
			for (ip = 0; ip < n1; ip++)
			{
				ypred[ip] = y[0];
			}
		} /*if (n1 > 0)*/		
		return ;
	} /*if (n < 2)*/

	ns = (long) (f*n + 1e-12 + .5);
	/*  at least two,  at most n points */
	if (ns < 2)
	{
		ns = 2;
	}
	else if (ns > n)
	{
		ns = n;
	}
	
	for (iter = 0; iter <= nsteps; iter++)
	{
		/*  robustness iterations */
		long          nleft = 0;
		long          nright = ns - 1;
		long          last = -1; /*  index of prev estimated point */
        long          i;
		int           ok, okLeft;
		
		i = 0; /*  index of current point */
		do /* while (last < n-1)*/
		{
			double        cut;

			while (nright < n-1)
			{
				/*  move nleft,  nright to right if radius decreases */
				double       d1 = x[i] - x[nleft];
				double       d2 = x[nright + 1] - x[i];

			/*  if d1 <= d2 with x[nright + 1] == x[nright],  lowest fixes */
				if (d1 <= d2)
				{
					break;
				}
				
				/*  radius will not decrease by move right */
				nleft++;
				nright++;
			} /*while (nright < n-1)*/

			ys[i] = lowest(x, y, n, x[i], nleft, nright, res,
						   (iter > 0) ? rw : (double *) 0, fitInfo, &ok);

			if (iter == nsteps && i == 0 && n1 > 0)
			{
				/*
				  Save info for possible extrapolation to left
				*/
				int       l;

				for (l = 0; l < 3; l++)
				{
					fitInfoLeft[l] = fitInfo[l];
				}
				okLeft = ok;
			} /*if (iter == nsteps && n1 > 0 && i == 0)*/
			
			/*  fitted value at x[i] */
			if (!ok)
			{
				/*  all weights zero - copy over value (all rw == 0) */
				ys[i] = y[i];
			}

			if (last < i - 1)
			{
				/*  
					points were skipped on last trip through loop --
					interpolate
					Can't happen when delta == 0
				*/
				double    denom = x[i] - x[last]; /*  non - zero - proof? */
				long      j;

				for (j=last + 1;j < i; j++)
				{
					double       alpha = (x[j] - x[last])/denom;

					ys[j] = alpha*ys[i] + (1.0 - alpha)*ys[last];
				}
			} /*if (last < i - 1)*/
			last = i; /*  last point actually estimated */

			cut = x[last] + delta; /*  x coord of close points */

			for (i = last + 1; i < n && x[i] <= cut; i++)
			{
				/*  find close points */
				if (x[i] == x[last])
				{
					/*  exact match in x */
					ys[i] = ys[last];
					last = i;
				} /*if (x[i] == x[last])*/
			} /*for (i=last + 1;i < n && x[i] <= cut; i++)*/
			if (i > last + 1)
			{
				/*  
					Can't happen when delta = 0
					Back 1 point so interpolation within delta,
					but always go forward]
				*/
				i--;
				/*  
					ys[j] not yet computed for j = last + 1,...,i-1
				*/
			}
		} while (last < n-1);

		if (n1 > 0)
		{
			/*
			  Find predicted values at xpred[0], ..., xpred[n1-1]

			  For xpred[j] < x[0] or xpred[j] > x[n-1], linear extrapolation
			  is used, using the line determined for x[0] or x[n-1]
			  If xpred[j] == x[k] for some k, ypred[j] = ys[k]
			  Otherwise, lowest() is used to ypred[j]
			*/
			long             nleft = 0, nright = ns - 1;
			long             ip, ipl, ipr;
			
			/* Extrapolate to left */
			for (ipl = 0; ipl < n1 && xpred[ipl] <= x[0]; ipl++)
			{
				if (okLeft)
				{
					ypred[ipl] = 
					  fitInfoLeft[1] + fitInfoLeft[2]*(xpred[ipl] - fitInfoLeft[0]);
				}
				else
				{
					setMissing(ypred[ip]);
				}
			} /*for (ipl = 0; ipl < n1 && xpred[ipl] <= x[0]; ipl++)*/

			/* Extrapolate to right */
			for (ipr = n1-1 ; ipr >= ipl && xpred[ipr] >= x[n-1]; ipr--)
			{
				if (ok)
				{
					ypred[ipr] = 
					  fitInfo[1] + fitInfo[2]*(xpred[ipr] - fitInfo[0]);
				}
				else
				{
					setMissing(ypred[ip]);
				}
			} /*for (ipr = n1-1 ; ipr > ipl && xpred[ipr] >= x[n-1]; ipr--)*/

			/* fit itermediate points */
			for (ip = ipl; ip <= ipr; ip++)
			{
				double       xp = xpred[ip];
				long         k;

				while (nright < n-1)
				{
					double       d1 = xp - x[nleft];
					double       d2 = x[nright + 1] - xp;

					if (d1 >= 0.0 && d1 <= d2)
					{
						break;
					}
					nleft++;
					nright++;
				} /*while (nright < n-1)*/
				
				/* see if xp was a value used in fitting */
				for (k = nleft; k <= nright; k++)
				{
					if (xp == x[k])
					{
						ypred[ip] = ys[k];
						break;
					}
				} /*for (k = nleft; k <= nright; k++)*/
				
				if (k > nright)
				{
					ypred[ip] = lowest(x, y, n, xp, nleft, nright, res,
									   rw, fitInfo, &ok);
					if (!ok)
					{
						setMissing(ypred[ip]);
					}
				} /*if (k > nright)*/

				/*
				  handle duplicate xpred values, if any
				*/
				while (ip < ipr && xpred[ip+1] == xp)
				{
					ypred[ip+1] = ypred[ip];
					ip++;
				}
			} /*for (ip = ipl; ip <= ipr; ip++)*/
		} /*if (n1 > 0)*/
		
		if (iter < nsteps)
		{
			/*  compute robustness weights except last time */
			double        cmad, c1, c9;

			for (i = 0;i < n ;i++)/*  residuals */
			{
				res[i] = y[i] - ys[i];
			}

			for (i = 0;i < n ;i++)
			{
				rw[i] = fabs(res[i]);
			}

			SORT(rw, n);

			cmad = 3.0*(rw[n/2] + rw[n - n/2 - 1]); /*  6 median abs resid */

			c9 = .999*cmad;
			c1 = .001*cmad;

			for (i = 0;i < n ;i++)
			{
				double           r = fabs(res[i]);
				if (r <= c1)
				{
					rw[i] = 1.0; /*  near 0,  avoid underflow */
				}
				else if (r > c9)
				{
					rw[i] = 0.0; /*  near 1,  avoid underflow */
				}
				else
				{
					double         temp = r/cmad;
				
					temp = 1.0 - temp*temp;
					rw[i] = temp*temp;
				}
			} /*for (i = 0;i < n ;i++)*/
		} /*if (iter < nsteps)*/		
	} /*for (iter = 0;iter <= nsteps;iter++)*/
} /*lowess()*/

/*
  Code to implement MacAnova function lowess()
  Usage:
   fit <- lowess(x, y [,xpred:xp] [, frac:f] [, iter:m] [, delta:del])
  or
   fit <- lowess(structure(x, y)  [,xpred:xp] [, frac:f] [, iter:m]\
                 [, delta:del])

    x, y      REAL vectors with the same length with no missing values
              and x[i] <= x[i+1]
    xp        REAL vector with no missing values and xp[i] <- xp[i+1]
    f         REAL scalar, 0 < f < 1, default = 2/3
    m         integer > 0, default = 3
    del       REAL scalar > 0, default = .01*(max(x) - min(x))

  Written by C. Bingham, 030702
  030707 check for monotonicity by function isMonotoneVector(), probably
         to be moved to utils.c
*/

enum monotoneCodes
{
	monotoneUp            = 0x01,
	monotoneDown          = 0x02,
	monotoneStrict        = 0x04,
	monotoneIncreasing    = monotoneUp | monotoneStrict,
	monotoneNonDecreasing = monotoneUp,
	monotoneDecreasing    = monotoneDown | monotoneStrict,
	monotoneNonIncreasing = monotoneDown
};

static int isMonotoneVector(Symbolhandle symh, int type)
{
	double           *x;
	long              i, n;
	int               strict = type & monotoneStrict;

	if (!isVector(symh) || TYPE(symh) != REAL)
	{
		goto errorExit;
	}
	
	x = DATAPTR(symh);
	n = symbolSize(symh);

	if (type & monotoneUp)
	{
		for (i = 1; i < n; i++)
		{
			if (x[i] < x[i-1] || strict && x[i] == x[i-1])
			{
				goto errorExit;
			}
		} /*for (i = 1; i < n; i++)*/
	}
	else
	{
		for (i = 1; i < n; i++)
		{
			if (x[i] > x[i-1] || strict && x[i] == x[i-1])
			{
				goto errorExit;
			}
		} /*for (i = 1; i < n; i++)*/
	}
	
	return (1);
	
  errorExit:
	return (0);
}

enum lowessScratch
{
	GRES  =   0,
	GRW,
	NTRASH
};

/* These defaults are the same as used in R */
#define FRACTDEFAULT .66666666666666
#define ITERDEFAULT  3

Symbolhandle     lowessmv(Symbolhandle list)
{
	Symbolhandle       symhX1 = SymbolZero;
	Symbolhandle       arg1, arg2;
	Symbolhandle       result = SymbolZero;
	Symbolhandle       symhTemp;
	double            *x, *y, *ys;
	double            *x1, *y1;
	double           **rw = (double **) 0, **res;
	double             fract = FRACTDEFAULT;
	double             delta = -1.0;
	long               iter = ITERDEFAULT;
	long               nargs = NARGS(list);
	long               n, n1;
	char              *names[4];
	TRASH(NTRASH, errorExit);
	
	arg1 = COMPVALUE(list, 0);
	if (nargs == 1 && arg1 == EmptyArgument)
	{
		badNargs(FUNCNAME, atLeast1Args);
		goto errorExit;
	}
	if (arg1 == EmptyArgument)
	{
		noData(FUNCNAME, 1);
		goto errorExit;
	}


	while (nargs > 0)
	{
		Symbolhandle        symhKey = COMPVALUE(list, nargs - 1);
		char               *keyword = isKeyword(symhKey);
		
		if (keyword == (char *) 0)
		{
			break;
		}
		if (strncmp(keyword, "f", 1) == 0)
		{
			if (!isNumber(symhKey, POSITIVEVALUE))
			{
				notPositiveReal(keyword);
				goto errorExit;
			}
			fract = DATAVALUE(symhKey, 0);
			if (fract > 1.0)
			{
				sprintf(OUTSTR,
						"ERROR: '%s' value > 1 in %s()", keyword, FUNCNAME);
				goto errorExit;
			}
		} /*if (strncmp(keyword, "f", 1) == 0)*/
		else if (strncmp(keyword, "xp", 2) == 0)
		{
			if (TYPE(symhKey) != REAL || !isVector(symhKey) ||
				anyMissing(symhKey) ||
				!isMonotoneVector(symhKey, monotoneNonDecreasing))
			{
				sprintf(OUTSTR,
						"ERROR: '%s' value not nondecreasing REAL vector",
						keyword);
				goto errorExit;
			}
			symhX1 = reuseArg(list, nargs - 1, 1, 1);
		}
		else if (strncmp(keyword, "iter", 4) == 0)
		{
			if (!isInteger(symhKey, NONNEGATIVEVALUE))
			{
				notNonNegativeInteger(keyword);
				goto errorExit;
			}
			iter = (long) DATAVALUE(symhKey, 0);
		}
		else if (strncmp(keyword, "del", 3) == 0)
		{
			if (!isNumber(symhKey, NONNEGATIVEVALUE))
			{
				notNonNegativeReal(keyword);
				goto errorExit;
			}
			delta = DATAVALUE(symhKey, 0);
		}
		else
		{
			badKeyword(FUNCNAME, keyword);
			goto errorExit;
		}
		nargs--;
	} /*while (nargs > 0)*/
	
	if (nargs == 1)
	{
		/* lowess(structure(x,y) ...) */
		if (TYPE(arg1) != STRUC || NCOMPS(arg1) < 2)
		{
			sprintf(OUTSTR,
					"ERROR: %s() argument not structure with >= 2 components",
					FUNCNAME);
			goto errorExit;
		}
		arg2 = COMPVALUE(arg1, 1);
		arg1 = COMPVALUE(arg1, 0);
	} /*if (nargs == 1)*/
	else if (nargs == 2)
	{
		/* lowess(x,y ...) */
		arg2 = COMPVALUE(list, 1);
	}
	else
	{
		sprintf(OUTSTR,
				"ERROR: %s() must have 1 or 2 non-keyword arguments",
				FUNCNAME);
		goto errorExit;
	}
	
	if (!checkArgType(arg1, "x", NONMISSINGREALVECTOR) ||
		!checkArgType(arg2, "y", NONMISSINGREALVECTOR))
	{
		goto errorExit;
	}

	n = symbolSize(arg1);
	if (symbolSize(arg2) != n)
	{
		sprintf(OUTSTR, "x and y not same length in %s()",
				FUNCNAME);
		goto errorExit;
	}
	
	if (!isMonotoneVector(arg1, monotoneNonDecreasing))
	{
		sprintf(OUTSTR,
				"ERROR: x not in increasing order in %s()", FUNCNAME);
		goto errorExit;
	}
	
	n1 = (symhX1) ? symbolSize(symhX1) : 0;

	names[0] = "x";
	names[1] = "y";
	names[2] = "xpred";
	names[3] = "ypred";
	
	result = StrucInstall(SCRATCH, (n1 > 0) ? 4 : 2);
	if (result == SymbolError)
	{
		goto errorExit;
	}
	COMPVALUE(result, 0) = symhTemp = Makereal(n);
	if (symhTemp == SymbolError)
	{
		goto errorExit;
	}
	Setname(symhTemp, names[0]);

	COMPVALUE(result, 1) = symhTemp = Makereal(n);
	if (symhTemp == SymbolError)
	{
		goto errorExit;
	}
	Setname(symhTemp, names[1]);
	
	if (n1 > 0)
	{
		Cutsymbol(symhX1);
		COMPVALUE(result, 2) = symhX1;
		Setname(symhX1, names[2]);
		  
		COMPVALUE(result, 3) = symhTemp = Makereal(n1);
		if (symhTemp == SymbolError)
		{
			goto errorExit;
		}
		Setname(symhTemp, names[3]);
	} /*if (n1 > 0)*/
	
	if (!getScratch(res, GRES, n, double) ||
		iter > 0 && !getScratch(rw, GRW, n, double))
	{
		goto errorExit;
	}
	
	x = DATAPTR(arg1);
	y = DATAPTR(arg2);
	doubleCopy(x, DATAPTR(COMPVALUE(result, 0)), n);
	
	ys = DATAPTR(COMPVALUE(result, 1));
	
	if (n1 > 0)
	{
		x1 = DATAPTR(symhX1);
		y1 = DATAPTR(COMPVALUE(result, 3));
	}
	else
	{
		x1 = y1 = (double *) 0;
	}

	if (delta < 0.0)
	{
		/* default  for delta same as in R */
		delta = .01*(x[n-1] - x[0]);
	}
	
	lowess(x, y, n, fract, iter, delta, ys, (rw) ? *rw : (double *) 0,
		   x1, y1, n1, *res);
	emptyTrash();
	
	return (result);
	
  errorExit:
	emptyTrash();
	
	if (*OUTSTR)
	{
		putErrorOUTSTR();
	}
	return (SymbolError);
} /*lowessmv()*/

