/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.07 or later
*(C)*
*(C)* Copyright (c) 1999 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

/*
  The functions in this file were first converted from Fortran to C
  using F2C from code used in Multreg, written by Sanford Weisberg.
  He had adapted it from the original code (Version 8/10/81) for
  Regressions by Leaps and Bounds by G. M. Furnival and R. W. Wilson.
  See their paper, Regression by Leaps and Bounds, Technometrics 16 (1974)
  499-511.  Code derived from the Furnival and Wilson code includes
  leapsscreen() in file screen(), and all the functions in this file
  and files lpsmisc.c and lpslandb.c.  Various changes and editing have
  been made by Oehlert and Bingham in adapting it for MacAnova. 
*/

/*
  990215 changed myerrorout() to putOutMsg() and putOUTSTR() to
         putErrorOUTSTR()
*/
#ifdef SEGMENTED
#if defined(MPW) || defined(MW_CW)
#pragma segment Screen
#endif /*MPW||MW_CW*/
#endif /*SEGMENTED*/


#include "globals.h"

#define ns (*NS)
#define nd (*ND)
#define kx (*KX)
#define no (*NO)
#define it (*IT)
#define nf (*NF)
#define ib (*IB)
#define mb (*MB)
#define s2 (*S2)
#define tl (*TL)
#define nv (*NV)
#define fd (*FD)
#define ne (*leapsNE)
#define iv (*IV)

#define rr(i,j) ( RR[(i-1) + nd*(j-1)] )
#define xi(i,j) ( XI[(i-1) + nd*(j-1)] )
#define xn(i,j) ( XN[(i-1) + nd*(j-1)] )
#define xm(i)   ( XM[(i-1)] )
#define dd(i)   ( DD[(i-1)] )
#define d(i)    ( D[(i-1)] )
#define dt(i)   ( DT[(i-1)] )
#define co(i)   ( CO[(i-1)] )
#define sc(i)   ( SC[(i-1)] )
#define sq(i)   ( SQ[(i-1)] )
#define cl(i)   ( CL[(i-1)] )
#define rm(i)   ( RM[(i-1)] )
#define inn(i)  ( INN[(i-1)] )
#define ind(i)  ( IND[(i-1)] )
#define myint(i)  ( INT[(i-1)] )
#define ipp(i)  ( IPP[(i-1)] )

/* converted fortran, send proper pointer for array element in function call */

/* 990406 Copied following from screen.c */
enum methodCodes 
{
	RSQMETHOD = 1,
	ADJRSQMETHOD,
	CPMETHOD
};

/*
  990316 converted declaration to standard C, using [] to indicate
         arrays rather than pointers
*/
/*
  Bounds from forward pivots

  longparms[0] = ns = number of best regressions to return
  longparms[1] = nd = 1st dimension of RR (= nvars + 1)
  longparms[2] = kx = nvars
  longparms[3] = no = number of observations
  longparms[4] = it = type of input matrix (0 => products about 0,
                 1 => products about means, 2 => correlation matrix)
                 Always 1 in MacAnova
  longparms[5] = nf = number of forced variables
  longparms[6] = ib = criterion (1 = R^2, 2 = adjR^2, 3 = cp)
  longparms[7] = mb = mbest
  longparms[8] = nv = number of rows and columns of RR (= nvars + 1)
  RR nv by nv cross product matrix or correlation matrix
  XI and XN nv by nv matrices of swept versions of RR
  XM vector of variable means (not used)
  DD, D, DT, CO, SC, SQ vectors of length nd
    CO (co) co(j) initialized to 2^(nvars-j), j = 1,...,nvars
    SC (sc) will be used for scaling factors (powers of 16)
    SQ (sq) will be used for scaled diagonals of RR
  CL (cl) vector of length ns, codes for best models models
  RM (rm) vector of length ns, vector of scaled RSS for best models
  INN (inn) vector of length nd (nvars + 1); first nf elements are forced
    variables; next nvars-kx are unforced variables; last is depv
  IPP (ipp) vector of length nd
  IND (ind) vector of length nd, indices in model being examined
  INT (myint) vector of length nd
  *S2 (s2) estimate of sigma squared; if < 0 still needs to be computed
  *TL (tl) tolerance value
  *FD (fd) penalty for Cp
  *leapsNE (ne) error code returned
  *IV (iv) Version of algorithm (always 0 in MacAnova)
*/
long leapsforwd(long longparms [], double RR [], double XI [], double XN [],
				double XM [], double DD [], double D [], double DT [],
				double CO [], double SC [], double SQ [], double CL [],
				double RM [], long INN [], long IPP [], long IND [],
				long INT [], double * S2, double * TL, double * FD,
				long * leapsNE,  long * IV)
{
	double          ci, cn, zip, tol, tal, tul, rs, sig, bit, df, pen,
	                wt, bnd, wst;
	long            nbit, l, mr, n, nn, ky, tmpi, tmpi0, tmpi1, tmpi2;
	long            kz, m, mn, ls, mh, mt, nb, j, lb;
	long           *NS, *ND, *KX, *NO, *IT, *NF, *IB, *MB, *NV;
	long            iq0 = 0, iq1 = 1; /* values for iq */
	long            ly0 = 0; /* value for ly, last argument to leapspivot()*/
	long            ml0 = 0; /* value for ml, 2nd argument to leapspivot()*/
	long            mt0 = 0, mt1 = 1;/* values for mt, arg 6 to leapscopy()*/
	double          tn, dx, sel, di;
	double          tmpd, tmpd2;
	WHERE("leapsforwd");
	
	NS = longparms;
	ND = longparms + 1;
	KX = longparms + 2;
	NO = longparms + 3;
	IT = longparms + 4;
	NF = longparms + 5;
	IB = longparms + 6;
	MB = longparms + 7;
	NV = longparms + 8;

	bit = 2.0;
	nbit = 1;
	while (bit != bit + 1.0)
	{
		nbit++;
		bit += bit;
	}

#if (0) /* test moved to screen() 990310 */
	if (nv > nbit)
	{
		ne = 1;
		putOutErrorMsg("ERROR:  too many variables for screen");
		return (1);
	}
#endif /*0*/

	for (l = 1; l <= nv; l++)
	{
		ind(l) = l;
		myint(l) = 0;
#ifdef USEPOW
		co(l) = pow((double) 2.0, (double) (nv - l));
#else /*USEPOW*/
		co(l) = intpow((double) 2.0, (double) (nv - l));
#endif /*USEPOW*/
	} /*for (l = 1; l <= nv; l++)*/

	zip = 0.0;
	leapscopy(RR, RR, IND, IND, NV, &mt0, NV, ND, SC, &cn, &zip, &iq0);
	/*
	  Check subscript list
	  Copy inn to ind
	  Initialize scaling factors sc and scaled total SS in sq
	  Initialize myint to 1's
	*/
	kz = kx + 1; /* position in inn of dependent variable */
	for (l = 1; l <= kz; l++)
	{
		m = inn(l);
		ind(l) = m;
		if (m < 1 || m > nv || myint(m) != 0)
		{
			ne = 2;
			return (1);
		}
		if (rr(m, m) <= zip)
		{
			ne = 3;
			return (1);
		}
		tn = rr(m, m);
		/*
		  scale by powers of 16
		*/
		tmpd = log(tn) / 2.0 / log(16.);
		tmpd = (tmpd >= 0 ? floor(tmpd) : -floor(fabs(tmpd)));
#ifdef USEPOW
		sc(m) = pow(16.0, tmpd);
#else /*USEPOW*/
		sc(m) = intpow(16.0, tmpd);
#endif /*USEPOW*/
		sq(m) = rr(m, m) / (sc(m) * sc(m));
		myint(m) = 1;
	} /*for (l = 1; l <= kz; l++)*/

	dx = zip;
	pen = fd;
	if (pen <= zip && ib == CPMETHOD)
	{
		pen = 2.0;
	}

	df = no - (it < 1 ? it : 1);
	sel = 1.0;
	ky = ind(kz);

	/*
	  Set tolerances
	*/
	tul = sq(ky) / pow(2.0, (double) nbit / 2.);
	tol = (double) (kx * kx) / pow(2.0, (double) nbit / 2.);
	tal = tl;
	if (tal < tol || tal >= 1.0)
	{
		tal = tol;
	}

	leapscopy(XN, RR, IND, IND, &kz, &mt1, &kz, ND, SC, &cn, &zip, &iq1);

	/*
	  Forced variables and mean square residual for c(p)
	*/
	for (l = 1; l <= kx; l++)
	{
#if (0) /* 990407 move computation of sig and s2 out of loop */
		sig = s2 / (sc(ky) * sc(ky));
#endif /*0*/
		if (l == nf + 1)
		{
			leapscopy(XI, XN, IND, IND, &kz, &mt1, &kz, ND, SC,
					  &ci, &cn, &iq0);
		}
		if (l > nf && (/*ib != CPMETHOD ||*/ s2 > zip))
		{ /*compute s2 for all methods if not specified (kb change) */
			break;
		}
		n = ind(l);
		ipp(l) = l + 1;
		tmpi = l - 1;
		leapstest(&wt, &n, XN, INT, D, DD, SQ, DT, &tmpi, &l, ND);
		if (wt < tal && l <= nf)
		{
			ne = 4;
			return (1);
		}
		if (wt < tol)
		{
			ne = 5;
			return (1);
		}
		tmpi = l - 1;
		tmpi0 = tmpi2 = l + 1;
		leapspivot(&n, &tmpi, &tmpi0, KX, XN, INT, IND, &cn, CO + n - 1,
				   DT, &iq1, ND, &nn, &ky, &tmpi2, KX);
#if (0)
		sig = xn(ky, ky) / (df - (double) kx);
#endif /*0*/
	} /*for (l = 1; l <= kx; l++)*/

	if (s2 <= 0.0)
	{
		sig = xn(ky, ky) / (df - (double) kx);
		s2 = sig * (sc(ky) * sc(ky));
	}
	else
	{
		sig = s2 / (sc(ky) * sc(ky));
	}
	
	/*
	  Load transform storage with worst value
	*/
	tmpd = sq(ky) + sq(ky); /* scaled 2*SStot */
	tmpd2 = sig + sig; 		/* scaled 2*(full model MSE) */
	leapstrans(&wst, &mr, &tmpd, &df, KX, &tmpd2, &pen, IB, MB);

	/*
	  initialize entire table with null model in cl and wst in rm
	*/
	for (l = 1; l <= ns; l++)
	{
		cl(l) = zip;
		rm(l) = wst;
	}
	mn = nf + 1;
	ipp(mn) = mn;

	for (;;)
	{
		/*
		  Forward algorithm
		*/
		ls = ipp(mn);
		mh = mt = mn;
		if (ls < kx - 2)
		{
			tmpi = mn - 1;
			tmpi2 = mn + kz - ls;
			leapscopy(XN, XI, INT, IND, &tmpi, &ls, &tmpi2, ND, SC,
					  &cn, &ci, &iq0);
		} /*if (ls < kx - 2)*/

		for (lb = ls; lb <= kx; lb++)
		{
			mn = mh + lb - ls;
			ipp(mn) = lb + 1;
			/*
			  Select pivot
			*/
			nb = lb;
			for (l = lb; l <= kx; l++)
			{
				j = ind(l);
				if (mn == 1)
				{
					dd(l) = -xi(j, ky) * xi(j, ky) / xi(j, j);
				}
				else if (mn > 1)
				{
					dd(l) = xi(j, j) / sq(j);
				}
				if (dd(l) < dd(nb))
				{
					nb = l;
				}
			} /*for (l = lb; l <= kx; l++)*/

			n = ind(nb);
			ind(nb) = ind(lb);
			ind(lb) = n;
			tmpi = mn - 1;

			leapstest(&wt, &n, XI, INT, D, DD, SQ, DT, &tmpi, &mn, ND);

			if (wt < tol)
			{
				break;
			}

			/*
			  Avoid pivot for last x
			*/
			if (lb < kx)
			{
				tmpi = mn - 1;
				tmpi2 = tmpi0 = lb + 1;
				leapspivot(&n, &tmpi, &tmpi0, KX, XI, INT, IND, &ci,
						   CO + n - 1, DT, &iq1, ND, &nn, &ky, &tmpi2, KX);
				rs = xi(ky, ky);
			}
			else
			{
				rs = xi(ky, ky) - xi(n, ky) * xi(n, ky) / xi(n, n);
				ci += co(n);
			}

			if (wt >= tal)
			{
				if (rs < tul)
				{
					ne = 6;
					return (1);
				}
				mt = mn + 1;
				leapsqstore(&rs, &ci, CL, RM, MB, &df, &mn, &sig, &pen,
							IB, NS);
			}

			if (lb == kx)
			{
				ci -= co(n);
			}
		} /*for (lb = ls; lb <= kx; lb++)*/

		if (mt <= mn)
		{
			sel = tal;
		}

		if (ipp(mn) > kx)
		{
			if (wt >= tal && ls < kx - 2)
			{
				leapspivot(&n, &ml0, &kz, KX, XI, INT, IND, &ci,
						   CO + n - 1, DT, &iq1, ND, &nn, &ky, &kz, &ly0);

				/*
				  Begin branch and bound
				  */

				mn = mh;	
				if (!leapslandb(XI, XN, D, DD, &ci, &cn, CO, CL, RM, IND, NS,
								ND, &df, IB, MB, &ls, &mn, &ky, &sig, &pen,
								&zip, KX, &kz, INT, IV))
				{
					goto errorExit;
				}
				
				di = (rs - xi(ky, ky)) / sq(ky);
				dx = (dx > fabs(di) ? dx : fabs(di));
				tmpi = mn - 1;
				tmpi1 = mn + kz - ls;
				leapscopy(XI, XN, INT, IND, &tmpi, &ls, &tmpi1, ND, SC,
						  &ci, &cn, &iq0);
			} /*if (wt >= tal && ls < kx - 2)*/

			for (;;)
			{
				/*
				  Backtrack
				*/
				if (mn == 1)
				{
					goto breakout;
				}
				mn--;
				n = myint(mn);
				tmpi = mn - 1;

				tmpd = -co(n);
				leapspivot(&n, &tmpi, IPP + mn - 1, KX, XI, INT, IND, &ci,
						   &tmpd, DT, &iq0, ND, &nn, &ky, IPP + mn - 1, KX);
				if (mn > nf && mn <= mt)
				{
					if (wt < tol || mn < mh)
					{
						break;
					}
					leapstrans(&bnd, &mr, &rs, &df, &mn, &sig, &pen, IB, MB);
					if (bnd < rm(mr))
					{
						break;
					}
				} /*if (mn > nf && mn <= mt)*/
			} /*for (;;)*/
		} /*if (ipp(mn) > kx)*/
	} /*for (;;)*/

  breakout:
	/*
	  Output
	*/
	
	for (lb = 1; lb <= ns; lb++)
	{
		if (cl(lb) != zip)
		{
			/*
			  Decode labels
			*/
			m = 0;

			for (l = 1; l <= nv; l++)
			{
				if (cl(lb) >= co(l))
				{
					m++;
					ind(m) = l;
					cl(lb) -= co(l);
				}
			} /*for (l = 1; l <= nv; l++)*/
			ind(m + 1) = ky;
			if(!leapsqprint(IND, &m, RM + lb - 1, SQ + ky - 1, &sig,
							SC + ky - 1, &df, IT, IB, &pen))
			{
				goto errorExit;
			}
		} /*if (cl(lb) != 0.0)*/
	} /*for (lb = 1; lb <= ns; lb++)*/
	
	if (sel < 1.0 && PRINTWARNINGS)
	{
		sprintf(OUTSTR,
				"WARNING: subsets with tolerances less than %11.4g have been",
				sel);
		putErrorOUTSTR();
		putOutMsg("encountered and omitted");
	}
	di = (sq(ky) - xi(ky, ky)) / sq(ky);
	dx = (dx > fabs(di) ? dx : fabs(di));

	if (dx > tol && PRINTWARNINGS)
	{
		sprintf(OUTSTR,
				"WARNING: largest discrepancy observed for r^2 is %g", dx);
		putErrorOUTSTR();
	}
	return (1);

  errorExit:
	return (0);
} /*leapsforwd()*/
