/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.12 or later
*(C)*
*(C)* Copyright (c) 2002 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

/*
  The functions in this file were first converted from Fortran to C
  using F2C from code used in Multreg, written by Sanford Weisberg.
  He had adapted it from the original code (Version 8/10/81) for
  Regressions by Leaps and Bounds by G. M. Furnival and R. W. Wilson.
  See their paper, Regression by Leaps and Bounds, Technometrics 16 (1974)
  499-511.  Code derived from the Furnival and Wilson code includes
  leapsscreen() in file screen(), and all the functions in this file
  and files lpsforwd.c and lpsmisc.c.  Various changes and editing have
  been made by Oehlert and Bingham in adapting it for MacAnova. 

  020321 undefined new macro DF whose name caused a conflict
*/

#ifdef SEGMENTED
#if defined(MPW) || defined(MW_CW)
#pragma segment Screen
#endif /*MPW||MW_CW*/
#endif /*SEGMENTED*/


#include "globals.h"
#undef DF /*macro not used here that conflicts with variable name*/
#include <math.h>

#ifndef LOOPCHUNK
#define LOOPCHUNK   50  /* check for interrupt every LOOPCHUNK times past test*/
#endif /*LOOPCHUNK*/


#define ci (*CI)
#define cn (*CN)
#define ns (*NS)
#define nd (*ND)
#define df (*DF)
#define ib (*IB)
#define mb (*MB)
#define iz (*IZ)
#define mn (*MN)
#define ky (*KY)
#define sig (*SIG)
#define pen (*pen)
#define zip (*ZIP)
#define kx (*KX)
#define kz (*KZ)
#define iv *IV
#define xi(i,j) ( XI[(i-1) + nd * (j-1)] )
#define xn(i,j) ( XN[(i-1) + nd * (j-1)] )
#define d(i) ( D[i-1] )
#define yi(i) ( YI[i-1] )
#define co(i) ( CO[i-1] )
#define cl(i) ( CL[i-1] )
#define rm(i) ( RM[i-1] )
#define ni(i) ( NI[i-1] )
#define ind(i) ( IND[i-1] )
/*
#                           leaps and bounds
*/
/*
  990316 Converted declaration to standard C form, using Name [] to
  indicate arrays and * Name to indicate pointers to scalars
*/

/*
  XI and XN nv by nv matrices of swept versions of RR
  D, YI vectors of length nvars + 1
  *CI (ci) codes current model
  CO (co) co(j) initialized to 2^(nvars-j), j = 1,...,nvars
  CL (ls) cl(j) codes for model j in current list of best models
  RM (rm) rm(j) criterion for model j in current list of best models
  IND (ind) list of variables in current model
  *NS total number of best regression to return
  *ND (nd) 1st dimension of XI and XN = nvars + 1
  *DF (df) n or n-1 (n-1 in MacAnova)
  *IB (ib) determines method used
  *MB (mb) mbest
  *IZ (iz)
  *MN (mn) number of swept in (modified)
  *KY (ky) index of dependent variable
  *SIG (sig) scaled full model MSE or user supplied estimate
  *PEN (pen) penalty used in computing Cp
  *ZIP (zip) always 0 in MacAnova, perhaps originally intended to be
           a fuzzy approximation to 0
  *KX (kx) number of independent variables, nvars
  *KZ (kz)
  NI (ni) integer vector
  *IV (iv) version of algorithm (0=partial reordering, 2=full reordering),
      always 0 in MacAnova
*/
long leapslandb(double XI [], double XN [], double D [], double YI [],
				double * CI, double * CN, double CO [], double CL [],
				double RM [], long IND [], long * NS, long * ND,
				double * DF, long * IB, long * MB, long * IZ, long * MN,
				long * KY, double * SIG, double * PEN, double * ZIP,
				long * KX, long * KZ, long NI [], long * IV)
{
	double          rs, bnd, tmpd;
	long            ip, l, n, ls, tmpi0, tmpi1, tmpi2, nn, lo,
	                mr, ld;
	long            lb, il, m, lc;
	long            iq0 = 0, iq3 = 3; /* values for iq */
	long            ly0 = 0; /* value for ly (last arg to leapspivot())*/
	long            ml0 = 0; /* value for ml (2nd arg to leapspivot()*/
#ifdef MACINTOSH
	long            loopcounter = 0;
#endif /*MACINTOSH*/
	WHERE("leapslandb");

	ni(iz) = kx;
	ip = iz;
	yi(iz) = xi(ky, ky);
	if (iv != 0)
	{
		/*
		  Initial p+1 variable regressions
		*/
		for (l = iz; l <= kx; l++)
		{
			n = ind(l);
			rs = xn(ky, ky) - xn(n, ky) * xn(n, ky) / xn(n, n);
			tmpd = cn + co(n);
			leapsqstore(&rs, &tmpd, CL, RM, MB, DF, MN, SIG, PEN, IB, NS);
		} /*for (l = iz; l <= kx; l++)*/
	} /*if (iv != 0)*/
	
	for (;;)
	{
		/*
		  First fathoming
		*/
		ls = ip + 1 + iv;
		for (lb = ls; lb <= kx; lb++)
		{
			long            mn1 = mn + kx - lb + iv;

			leapstrans(&bnd, &mr, YI + ip - 1, DF, &mn1, SIG, PEN, IB, MB);
			if (bnd < rm(mr))
			{
				break;
			}
		}
		if (bnd <= rm(mr))
		{
			ld = kx + ls - lb;
			if (ip <= iz)
			{
				/*
				  Finish inversion
				*/
				for (l = iz; l <= kx; l++)
				{
					n = ind(l);
					xi(n, n) = -d(n);
					tmpi2 = l - 1;
					leapspivot(&n, &ml0, IZ, &tmpi2, XI,
							   IND, IND, CI, ZIP, D, &iq3,
							   ND, &nn, KY, IZ, &ly0);
				}
				lo = kx;
			} /*if (ip <= iz)*/
			else
			{
				/*
				  Extend prior pivots
				*/
				if (iv != 1 && ld > ni(ip - 1))
				{
					il = ip - 2;
					for (l = iz; l <= il; l++)
					{
						if (ld > ni(l + 1))
						{
							m = ind(l);
							tmpi1 = l + 1;
							tmpi2 = ni(l + 1) + 1;
							if (xn(m, m) < zip)
							{
								leapspivot(&m, &ml0, &tmpi1, &ld, XN,
										   IND, IND, CN, ZIP, D, &iq3,
										   ND, NI + l, KY, &tmpi2, &ly0);
							}
							else if (xn(m, m) > zip)
							{
								leapspivot(&m, &ml0, &tmpi1, &ld, XI,
										   IND, IND, CI, ZIP, D, &iq3,
										   ND, NI + l, KY, &tmpi2, &ly0);
							} /*if (xn(m, m) > zip)*/
						} /*if (ld > ni(l + 1))*/
					} /*for (l = iz; l <= il; l++)*/
				} /*if (iv != 1 && ld > ni(ip - 1))*/
				lo = ld + iv * (kx - ld);

				tmpd = -co(n);
				leapspivot(&n, &ml0, &ip, &lo, XI, IND,
						   IND, CI, &tmpd, D, &iq0, ND,
						   NI + ip - 1, KY, &ip, &ly0);
			} /*if (ip <= iz){}else{}*/

			/*
			  Bound regressions
			*/
			for (lb = ip; lb <= lo; lb++)
			{
				n = ind(lb);
				rs = xi(ky, ky) - xi(n, ky) * xi(n, ky) / xi(n, n);
				if (ld == kx)
				{
					tmpd = ci - co(n);
					tmpi0 = kx - ip + mn - 1;
					leapsqstore(&rs, &tmpd, CL, RM, MB, DF, &tmpi0,
								SIG, PEN, IB, NS);
				}
				/*
				  Re-order variables
				*/
				m = lb;
				while (m != ip && rs > yi(m))
				{
					yi(m + 1) = yi(m);
					ind(m) = ind(m - 1);
					m--;
				}
				ind(m) = n;
				yi(m + 1) = rs;
			} /*for (lb = ip; lb <= lo; lb++)*/

			if (ls < kx)
			{
				ld = (ld < kx - 1 ? ld : kx - 1);
				/*
				  Second fathoming
				*/
				for (lb = ls; lb <= ld; lb++)
				{
					lc = ld + ls - lb;
					tmpi0 = mn + ld - lb + iv;
					leapstrans(&bnd, &mr, YI + lc, DF, &tmpi0,
							   SIG, PEN, IB, MB);
					if (bnd < rm(mr))
					{
						break;
					}
				}
				if (bnd < rm(mr))
				{
					/*
					  Fixed and p+1 variable regressions
					*/
					for (lb = ls; lb <= lc; lb++)
					{
						ip = lb - iv;
						n = ind(ip - 1);
						tmpi1 = lc + iv * (kz - lc) - 1;
						leapspivot(&n, &ml0, &ip, &tmpi1, XN, IND, IND,
								   CN, CO + n - 1, D, &iq0,
								   ND, NI + ip - 1, KY, &ip, &ly0);
						if (iv != 1)
						{
							leapsqstore(XN + ky - 1 + (ky - 1) * nd, CN, CL,
										RM, MB, DF, MN, SIG, PEN, IB, NS);
							mn++;
						} /*if (iv != 1)*/
						else
						{
							mn++;
							for (l = ip; l <= kx; l++)
							{
								m = ind(l);
								rs = xn(ky, ky) - xn(m, ky) * xn(m, ky) / xn(m, m);
								tmpd = cn + co(m);
								leapsqstore(&rs, &tmpd, CL, RM, MB, DF, MN,
											SIG, PEN, IB, NS);
							} /*for (l = ip; l <= kx; l++)*/
						} /*if (iv != 1){}else{}*/
					} /*for (lb = ls; lb <= lc; lb++)*/
				} /*if (bnd < rm(mr))*/
			} /*if (ls < kx)*/
		} /*if (bnd <= rm(mr))*/
		else
		{
			ip--;
		}
		for (;;)
		{
			/*
			  Backtrack
			*/
			if (ip <= iz)
			{
				return (1);
			}
#ifdef MACINTOSH
			loopcounter++;
			if(interrupted(DEFAULTTICKS) != INTNOTSET)
			{
				return (0);
			}
#endif /*MACINTOSH*/
			n = ind(ip - 1);
			if (xn(n, n) < zip)
			{
				break;
			}

			leapspivot(&n, &ml0, &ip, NI + ip - 1, XI, IND,
					   IND, CI, CO + n - 1, D, &iq0,
					   ND, &nn, KY, &ip, &ly0);
			ip = ip - 1;
		} /*for (;;)*/
		mn--;

		tmpd = -co(n);
		leapspivot(&n, &ml0, &ip, NI + ip - 1, XN, IND,
				   IND, CN, &tmpd, D, &iq0, ND, &nn, KY, &ip, &ly0);
	} /*for (;;)*/
} /*leapslandb()*/
