/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.12 or later
*(C)*
*(C)* Copyright (c) 2002 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

/*
  The functions in this file were first converted from Fortran to C
  using F2C from code used in Multreg, written by Sanford Weisberg.
  He had adapted it from the original code (Version 8/10/81) for
  Regressions by Leaps and Bounds by G. M. Furnival and R. W. Wilson.
  See their paper, Regression by Leaps and Bounds, Technometrics 16 (1974)
  499-511.  Code derived from the Furnival and Wilson code includes
  leapsscreen() in file screen(), and all the functions in this file
  and files lpsforwd.c and lpslandb.c.  Various changes and editing have
  been made by Oehlert and Bingham in adapting it for MacAnova. 

  020321 undefined new macro DF whose name caused a conflict

*/

#include <math.h>
#include "globals.h"

#undef DF /*macro not used here that conflicts with variable name*/

#define n (*N)
#define ml (*ML)
#define ls (*LS)
#define lx (*LX)
#define cn (*CN)
#define co (*CO)
#define iq (*IQ)
#define nd (*ND)
#define nn (*NN)
#define ky (*KY)
#define ll (*LL)
#define ly (*LY)
#define xn(i,j) ( XN[(i-1)+ *ND * (j-1)] )
#define dt(i) ( DT[i-1] )
#define ind(i) ( IND[i-1] )
#define myint(i) ( INT[i-1] )

/*
  990316 Converted declaration to standard C form, using Name [] to
  indicate arrays and * Name to indicate pointers to scalars
*/

/*
  Partial sweep

 *N (n) row  and column being pivoted
 *ML (ml), if non-zero maximum row and column being swept
 *LS (ls)
 *LX (lx), last column for Gaussian pivoting
 XN (xn) matrix being swept
 INT (myint) integer vector
 IND (ind) integer vector
 *CN (cn) codes model for current state of XN
 *CO (co) 0 or +- power of 2 used to modifiy CN when iq != 3
 DT (dt) double vector; used only when iq == 1
 *IQ (iq) code determining what is done, values 0, 1, 2 or 3
 *ND (nd) row dimension of XN
 *NN (nn) set to value of lx
 *KY (ky) index of dependent variable
 *LL (ll) lowest index to be pivoting
 *LY (ly) last index for Gauss-Jordan pivoting
*/
void leapspivot(long * N, long * ML, long * LS, long * LX, double XN [],
				long INT [], long IND [], double * CN, double * CO,
				double DT [], long * IQ, long * ND, long * NN, long * KY,
				long * LL, long * LY)
/*
dimension xn(nd,nd),dt(nd),ind(nd),int(nd)
*/
{
	double          b;
	long            j, l, k, m;

	nn = lx;
	if (iq != 3)
	{
		cn += co;
		xn(n, n) = -xn(n, n);
		xn(ky, ky) += xn(n, ky) * xn(n, ky) / xn(n, n);
	} /*if (iq != 3)*/

	if (ml != 0)
	{
		/*
		  Gauss-Jordan
		*/
		for (l = 1; l <= ml; l++)
		{
			j = myint(l);
			b = xn(n, j) / xn(n, n);
			if (iq == 0)
			{
				xn(j, j) += b * xn(n, j);
			}
			else if (iq == 1)
			{
				xn(j, j) = dt(l);
			}
			for (m = ll; m <= ly; m++)
			{
				k = ind(m);
				xn(j, k) += b * xn(n, k);
				xn(k, j) = xn(j, k);
			} /*for (m = ll; m <= ly; m++)*/
		} /*for (l = 1; l <= ml; l++)*/
	} /*if (ml != 0)*/

	if (ll <= lx)
	{
		/*
		  Gauss
		*/
		for (l = ll; l <= lx; l++)
		{
			j = ind(l);
			b = xn(n, j) / xn(n, n);
			for (m = ls; m <= l; m++)
			{
				k = ind(m);
				xn(j, k) += b * xn(n, k);
				xn(k, j) = xn(j, k);
			} /*for (m = ls; m <= l; m++)*/
			xn(j, ky) += b * xn(n, ky);
			xn(ky, j) = xn(j, ky);
		} /*for (l = ll; l <= lx; l++)*/
	} /*if (ll <= lx)*/
} /*leapspivot()*/


#define wt (*WT)
#define mn (*MN)
#define xi(i,j) ( XI[ (i-1) + nd * (j-1) ] )
#define d(i) ( D[i-1] )
#define dd(i) ( DD[i-1] )
#define sq(i) ( SQ[i-1] )

/*
  990316 Converted declaration to standard C form, using Name [] to
  indicate arrays and * Name to indicate pointers to scalars
*/

/*
  Check tolerances
  *WT (wt) returned value, minimum ratio
  *N (n)   row and column being checked
  XI (xi)  nd by nd matrix being checked
  INT (myint)
  D (d) d(n) is set to xi(n,n)
  DD (dd) 
  SQ (sq) scaled original values of sums of squares or corrected SS
  DT (dt) set to swept value of xi(myint(l),myint(l)), l = 1,...,ml
  *ML (ml)
  *MN (mn)
  *ND (nd) row dimension of XI
*/

void leapstest(double * WT, long * N, double XI [], long INT [], double D [],
			   double DD [], double SQ [], double DT [], long * ML,
			   long * MN, long * ND)
{
	long            j, l;

	d(n) = xi(n, n);
	wt = d(n) / sq(n);
	dd(mn) = wt * d(n); /* d(n)*d(n)/sq(n) = xi(n,n)*xi(n,n)/sq(n)*/
	myint(mn) = n;

	if (ml != 0)
	{
		for (l = 1; l <= ml; l++)
		{
			j = myint(l);
			/*
			*****the next two lines of code are modified for cdc (?)
			*/
			dt(l) = xi(j, j);
			if (xi(n, n) != 0.)
			{
				dt(l) -= xi(n, j) * xi(n, j) / xi(n, n);
			}
			if (-dd(l) / dt(l) < wt)
			{
				wt = -dd(l) / dt(l);
			}
		} /*for (l = 1; l <= ml; l++)*/
	} /*if (ml != 0)*/
} /*leapstest()*/


#define rs (*RS)
#define cl(i) ( CL[i-1] )

#ifdef SEGMENTED
#if defined(MPW) || defined(MW_CW)
#pragma segment Screen
#endif /*MPW||MW_CW*/
#endif /*SEGMENTED*/


#define rm(i) ( RM[i-1] )
#define mb (*MB)
#define df (*DF)
#define sig (*SIG)
#define pen (*PEN)
#define ib (*IB)
#define ns (*NS)
/*
#                   labels and saves best regressions
*/
/*
  990316 Converted declaration to standard C form, using Name [] to
  indicate arrays and * Name to indicate pointers to scalars
*/

/*
 *RS  scaled SSResid
 *CN (cn) encoded model
 *CL (cl) list of encoded models
 *RM (rm) list of scaled criteria
 *MB (mb) mbest
 *DF Total degrees of freedom (n or n-1)
 *MN Number of variables in model
 *SIG Scaled MSE from full model or scaled value provided by user
 *PEN Penalty used in computing Cp
 *IB Specifies method
 *NS eventual total size of criterion table (mb or mb*kx, not used)
*/
void leapsqstore(double * RS, double * CN, double CL [], double RM [],
				 long * MB, double * DF, long * MN, double * SIG, double * PEN,
				 long * IB, long * NS)
{
	double          tr;
	long            l, m, mr;

	leapstrans(&tr, &mr, RS, DF, MN, SIG, PEN, IB, MB);

	/*
	  mr is current length of table; mb for adjR2 & Cp, mn*mb for R2
	*/
	if (rm(mr) > tr)
	{
		/* first check if we already have model */
		for (m = 1; m <= mb; m++)
		{
			l = mr - mb + m;
			if (cl(l) == cn)
			{
				return;
			}
		} /*for (m = 1; m <= mb; m++)*/

		/* insert new model and criterion in list of best */
		if (mb != 1)
		{
			for (m = 2; m <= mb; m++)
			{
				if (tr >= rm(l - 1))
				{
					break;
				} /*if (tr >= rm(l - 1))*/
				rm(l) = rm(l - 1);
				cl(l) = cl(l - 1);
				l--;
			} /*for (m = 2; m <= mb; m++)*/
		} /*if (mb != 1)*/

		rm(l) = tr;
		cl(l) = cn;
	} /*if (rm(mr) > tr)*/
} /*leapsqstore()*/


#define mt (*MT)
#define mp (*MP)
#define ci (*CI)
#define sc(i) ( SC[i-1] )

/*
  990316 Converted declaration to standard C form, using Name [] to
  indicate arrays and * Name to indicate pointers to scalars
*/

/*
  Matrix copy
    Copy XI to XN, possible scaled
	XN (xn) destination matrix, 1st dimension *ND
	XI (xi) source matrix, 1st dimension *ND
	INT (myint) destination list of variables
	IND (ind) source list of variables
	*ML (ml) ml + 1 is first element of INT set
	*MT (mt) first element of IND copyied
	*ND (nd) first dimension of XN and XI
	SC (sc) vector of scaling factors
	*CN (cn) model description for destination (set to ci)
	*CI (ci) model description for source
	*IQ (iq) if 1, scale while copying; if 0 don't scale
*/
void leapscopy(double XN [], double XI [], long INT [], long IND [],
			   long * ML, long * MT, long * MP, long * ND, double SC [],
			   double * CN, double * CI, long * IQ)
{
	long            j, k, l, m;
	for (l = 1; l <= mp; l++)
	{
		if (l > ml)
		{
			m = mt + l - ml - 1;
			myint(l) = ind(m);
		}
		k = myint(l);
		for (m = 1; m <= l; m++)
		{
			j = myint(m);
			if (iq == 0)
			{
				xn(j, k) = xi(j, k);
			}
			else if (iq == 1)
			{
				xn(j, k) = xi(j, k) / (sc(j) * sc(k));
			}
			xn(k, j) = xn(j, k);
		} /*for (m = 1; m <= l; m++)*/
	} /*for (l = 1; l <= mp; l++)*/
	cn = ci;
} /*leapscopy()*/


#undef myint

#define tr (*TR)
#define mr (*MR)

/* 990406 Copied following from screen.c */
enum methodCodes 
{
	RSQMETHOD = 1,
	ADJRSQMETHOD,
	CPMETHOD
};

/*
#             transform of criterion-smaller must be better
*/
/*
  990316 Converted declaration to standard C form, using Name [] to
  indicate arrays and * Name to indicate pointers to scalars

  *TR (tr) is the criterion value returned
  *MR (mr) is the number of values to compute, returned
  *RS (rs) = scaled RSS for model
  *DF (df) Total degrees of freedom (n if no constant term, n-1 otherwise)
  *MN (mn) Number of variables in the model
  *SIG (sig) is either scaled estimated variance from the full model
             or scaled value supplied by the user
  *PEN (pen) is the CP penalty
  *IB (ib) specifies the method
  *MB (mb) is mbest
*/
void leapstrans(double * TR, long * MR, double * RS, double * DF, long * MN,
				double * SIG, double * PEN, long * IB, long * MB)
{
	mr = mb;

	switch ((int) ib)
	{
	  case RSQMETHOD:
		/* return scaled SSresid = scaled (1 - R^2)*SStot */
		tr = rs;
		mr = mn * mb;
		break;

	  case ADJRSQMETHOD:
		/* return scaled SSresid/(n - k) = (1-Radj^2)*SStot/(n-1) */
		tr = rs / (df - (double) mn);
		break;

	  case CPMETHOD:
		/* return scaled RSS + pen*p*mse = mse*(Cp+n) */
		tr = rs + pen * (mn) * sig;
		break;
	} /*switch ((int) ib)*/
} /*leapstrans()*/
