/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.12 or later
*(C)*
*(C)* Copyright (c) 2001 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/


#ifdef SEGMENTED
#if defined(MPW) || defined(MW_CW)
#pragma segment Utils
#endif /*MPW||MW_CW*/
#endif /*SEGMENTED*/

#include "globals.h"
#include "macIface.h" /* Note: macIface.h replaces MultiSkel.h */
#include "mvIo.h"

/*
   960502 added parameters fileName and vRefNum
          Moved check for fileName[0] == '\0' here
		  readWrite should be either READIT or WRITEIT

   960503 changed name of macOpen() to macFindFile(); changed name
          of file to macFindFile.c
   960530 If actual file name is supplied and volume != (Integer *) 0,
          and *volume is set to Id of current directory.
          
   960820 Fixed minor bug; with fileName[0] != '\0', *volume was ignored

   990325 On reading on Macintosh, when ntypes is 0, 'ttro' is an additional
          default type
   010521 Uses new Macros for accessing elements of Reply
   010523 Moved macGetPath() and getMacPathName() from macUtils.c
          Renamed macSetInfo() to macSetFInfo()
          Added mac{Set,Get}Vol()
	      Added mac{Set,Get}Cur{Dir,Vol}()
          Replaced calls to {Set,Get}Vol() by calls to mac{Set,get}Vol()
   010602 Added calls to macSetCurDir() and macSetCurVol() to try
          to ensure dialog box starts up in right place, that is, the
		  last directory where this type of file was most recent referenced
   010605 Save directory information in MvDirectories[lastVol] when directory
          is not null and fileName is ""
*/

static char       FileName[64]; /* macFindFile() returns address of FileName */

char *macFindFile(char *fileName, char * cMessage, char * cDefaultname,
				  unsigned long control,
				  mvDirInfoPtr directory) 
{
	SFTypeList      myTypes;
	Point           p;
	Integer         nTypes;
	Integer         vRefNum;
	LongInt         dirID;
	WHERE("macFindFile");

	if (directory != (mvDirInfoPtr) 0)
	{
		vRefNum = getMvDirVolume(directory);
		dirID = getMvDirDirectory(directory);
	}
	else
	{
		vRefNum = 0;
		dirID = 0;
	}
	if (fileName[0] == '\0')
	{
		Str255          pMessage, pDefaultname;

		c2pstrcpy(pMessage, cMessage);
		c2pstrcpy(pDefaultname, cDefaultname);

		/* Use standard file dialog box */
		SetPt(&p, 82, 90);
	
		DeactivateWindow();		/*unhighlight front window*/

#if (USE_NEWSF) 
		if (directory != (mvDirInfoPtr) 0)
		{
			/*
				Ensure dialog box opens in last folder used for this
				type of file
				Note: these calls reference a low memory global
			*/
			macSetCurDir(directory);
			macSetCurVol(directory);
		}
#endif /*USE_NEWSF*/
		if (control & mvWriteit)
		{						/* write */
			DialogEdit = true;
			NDialogButtons = 2;
			ButtonChars[0] = 's';
			ButtonChars[1] = 'n';
#if (USE_NEWSF)
			CustomPutFile(pMessage, pDefaultname, &Reply, 0, p,
						  NullDialogHookYDPtr, MyDialogFilterYDPtr, nil,
						  NullActivateYDPtr, nil);
#else /*USE_NEWSF*/
			SFPPutFile(p, pMessage, pDefaultname, NullDialogHookPtr, &Reply,
					   putDlgID, MyDialogFilterPtr);
#endif /*USE_NEWSF*/
		} /*if (readWrite == WRITEIT)*/
		else
		{/* read */
			nTypes = 0;
			if(!(control & anyTypes))
			{ /* default */
				control = mvTextFile | mvROTextFile;
			}

			if (control & mvAsciiSaveFile)
			{
				myTypes[nTypes++] = ASCIIKEEPFILETYPE;
			}
			if (control & mvSaveFile)
			{
				myTypes[nTypes++] = KEEPFILETYPE;
			}
			if (control & mvRsrcFile)
			{
				myTypes[nTypes++] = 'rsrc';
			}
			if (control & mvTextFile)
			{
				myTypes[nTypes++] = 'TEXT';
			}
			if (control & mvROTextFile)
			{
				myTypes[nTypes++] = 'ttro';
			}
			if (nTypes > 4)
			{
				nTypes = 4; /* play it safe */
			}

			DialogEdit = false;
			NDialogButtons = 2;
			ButtonChars[0] = 'o'; /* Open */
			ButtonChars[1] = 'n'; /* Cancel */

#if (USE_NEWSF) 
			CustomGetFile(NullFileFilterYDPtr, nTypes, myTypes,
						  &Reply, 0, p, NullDialogHookYDPtr,
						  MyDialogFilterYDPtr, nil,
						  NullActivateYDPtr, nil);
#else /*USE_NEWSF*/
			SFPGetFile(p, pMessage, NullFileFilterPtr,	
					   nTypes, myTypes, NullDialogHookPtr, &Reply,
					   getDlgID, MyDialogFilterPtr);
#endif /*USE_NEWSF*/
		} /*if (readWrite == WRITEIT){}else{}*/
		macUpdate((WindowPtr) 0);
	
		if (!ReplyGood)
		{ /* cancelled */
			return ((char *) 0);
		}
		vRefNum = ReplyVRefNum;
#if (USE_NEWSF)
		dirID = Reply.sfFile.parID;
#else /*USE_NEWSF*/
		dirID = 0;
#endif /*USE_NEWSF*/
		if (directory != (mvDirInfoPtr) 0)
		{
			setMvDirVolume(directory, vRefNum);
			setMvDirDirectory(directory, dirID);
			copyMvDirInfo(lastVol, directory);
		}
		ReplyGetFInfo(); /* Finder information to global*/

		p2cstrcpy(FileName, ReplyName);
		fileName = FileName;
	} /*if (fileName[0] == '\0')*/
	else if (directory != (mvDirInfoPtr) 0 && vRefNum == 0)
	{
		mvDirInfo     dirInfo;

		if (macGetVol(&dirInfo) == noErr)
		{
			setMvDirVolume(directory, getMvDirVolume(&dirInfo));
			setMvDirDirectory(directory, getMvDirDirectory(&dirInfo));
		}
	}
 
	if (vRefNum != 0)
	{
		mvDirInfo     dirInfo;

		dirInfo.vRefNum = vRefNum;
		dirInfo.dirID = dirID;
		macSetVol(&dirInfo);
	}
	return (fileName);

} /*macFindFile()*/

/*
	960502 Function to return full path name of a specified directory
	In case of error, it returns a null string ("")
	
	volume and directory for current default directory can be obtained by
		errorFlag = HGetVol((StringPtr) 0, &volume, &directory);

	Following code adapted from function GetFullPath on p. 2-45 of
	Inside Macintosh - Files
*/
char * macGetPath(char *fullPath, Integer volume, LongInt directory)
{
	CInfoPBRec      pb;
	Integer         errorFlag;
	char           *place1, *place2;
	Str255          dirName;
	WHERE("macGetPath");
	
	*fullPath = '\0';
	pb.hFileInfo.ioCompletion = 0;
	pb.hFileInfo.ioNamePtr = dirName;
	pb.hFileInfo.ioVRefNum = volume;
	pb.dirInfo.ioDrParID = directory;
	pb.dirInfo.ioFDirIndex = -1;

	do /* while (pb.dirInfo.ioDrDirID != fsRtDirID);*/
	{
		char       dirCName[256];
		
		pb.dirInfo.ioDrDirID = pb.dirInfo.ioDrParID;
		errorFlag = PBGetCatInfo(&pb, false);
		if (errorFlag != noErr)
		{
			goto errorExit;
		}
		p2cstrcpy(dirCName, dirName);

		place1 = fullPath + strlen(fullPath);
		place2 = place1 + strlen(dirCName) + 1;
		if (place2 - fullPath > PATHSIZE)
		{ /* path too long */
			goto errorExit;
		}
		while (place1 >= fullPath)
		{
			*place2-- = *place1--;
		}
		strcpy(fullPath, dirCName);
		*place2 = ':';
	} while (pb.dirInfo.ioDrDirID != fsRtDirID);

	return (fullPath);

  errorExit:
	*fullPath = '\0';
	return (fullPath);
} /*macGetPath()*/

/*
	Get the path name of the current directory
    010324 moved from readdata() where it was static
*/

int getMacPathName(char pathName [])
{
	LongInt             dirID;
	Integer             vRefNum, errorFlag;

	errorFlag = HGetVol((StringPtr) 0, &vRefNum, &dirID);
	if (errorFlag != noErr)
	{
		return ((int) errorFlag);
	}
	return((*macGetPath(pathName, vRefNum, dirID) == '\0') ? -1 : 0);
} /*getMacPathName()*/

OSErr macSetVol(mvDirInfoPtr dirInfo)
{
#if (USE_NEWSF)
	return (HSetVol((StringPtr) 0, getMvDirVolume(dirInfo),
					getMvDirDirectory(dirInfo)));
#else /*USE_NEWSF*/
	return (SetVol((StringPtr) 0, getMvDirVolume(dirInfo)));
#endif /*USE_NEWSF*/
} /*macSetVol()*/

OSErr macGetVol(mvDirInfoPtr dirInfo)
{
	Integer      vRefNum;
	OSErr        errorCode;
	
#if (USE_NEWSF)
	LongInt      dirID;

	errorCode = HGetVol((StringPtr) 0, &vRefNum, &dirID);
	setMvDirDirectory(dirInfo, dirID);
#else /*USE_NEWSF*/
	errorCode = GetVol((StringPtr) 0, &vRefNum);
#endif /*USE_NEWSF*/
	setMvDirVolume(dirInfo, vRefNum);

	return (errorCode);
} /*macGetVol()*/

enum
{
	mvCurDir = 0x0398,
	mvCurVol = 0x0214
};

void macSetCurDir(mvDirInfoPtr dirInfo)
{
	* ((LongInt *) mvCurDir) = getMvDirDirectory(dirInfo);
} /*macSetCurDir()*/

void macSetCurVol(mvDirInfoPtr dirInfo)
{
	* ((Integer *) mvCurVol) = -getMvDirVolume(dirInfo);
} /*macSetCurVol()*/

void macGetCurDir(mvDirInfoPtr dirInfo)
{
	setMvDirDirectory(dirInfo, *((LongInt *) mvCurDir));
} /*macGetCurDir()*/

void macGetCurVol(mvDirInfoPtr dirInfo)
{
	setMvDirVolume(dirInfo, -(*((Integer *) mvCurVol)));
} /*macGetCurVol()*/

void macSetFInfo(mvFileInfoPtr fileInfo, OSType creator)
{
	FInfo       fndrInfo;
	OSErr       errorCode;
	
#if (USE_NEWSF)
	FSSpec      fss;

	fss.vRefNum = getMvFileVolume(fileInfo);
	fss.parID = getMvFileDirectory(fileInfo);
	str63cpy(fss.name, getMvFileName(fileInfo));
	errorCode = FSpGetFInfo(&fss, &fndrInfo);
	if (errorCode != noErr)
	{
		goto errorExit;
	}
	fndrInfo.fdType = getMvFileType(fileInfo);
	fndrInfo.fdCreator = creator;
	errorCode = FSpSetFInfo(&fss, &fndrInfo);
#else /*USE_NEWSF*/
	errorCode = GetFInfo(getMvFileName(fileInfo), getMvFileVolume(fileInfo),
						 &fndrInfo);
	{
		char       name[256];
		p2cstrcpy(name, getMvFileName(fileInfo));
	}
	if (errorCode != noErr)
	{
		goto errorExit;
	}
	fndrInfo.fdType = getMvFileType(fileInfo);
	fndrInfo.fdCreator = creator;
	errorCode = SetFInfo(getMvFileName(fileInfo), getMvFileVolume(fileInfo),
						 &fndrInfo);
#endif /*USE_NEWSF*/
	if (errorCode == noErr)
	{
		return;
	}

  errorExit:
	{
		char      fileName[256];

		p2cstrcpy(fileName, getMvFileName(fileInfo));
		fileErrorMsg(errorCode, fileName);
	}
} /*macSetFInfo()*/
