/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.12 or later
*(C)*
*(C)* Copyright (c) 2001 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/


/*
  990603 Added readFromMouse() as general interface to platform specific code
         There should be an equivalent version in wxIo.cc
  990609 Changed include of matdat.h to include of mvIo.h
  000329 Made modifications to ensure that Newline is written as CR under CW Pro 5
  000707 Made LineBuffer a pointer allocated in initialize() so as to reduce the
         size of global objects
  001027 Renamed LineBuffer to DlogLineBuffer and made it a handle allocated
         in fillFromDialog() and cleaned up when necessary in cleanitup()
  010529 Added splitPathName() (moved from readdata.c)
  010604 Added buffer length argument to splitPathName()
  020815 Added resetHistory(); getCurrentCommand() longer static
*/

#include "globals.h"
#include "mvIo.h"
#include "macIface.h" /* Note: macIface.h replaces MultiSkel.h */

#define BEL '\007'

/* Macros for working with file and path names */
#ifdef MSDOS
/*  '/' equivalent to '\' on MSDOS*/
#define isSlash(C) ((C) == DIRSEPARATOR[0] || (C) == '/')
#define isPath(N) (strchr(N, '/') || strchr(N, '\\') || strchr(N, ':'))
#else /*MSDOS*/
#define isSlash(C) ((C) == DIRSEPARATOR[0])
#define isPath(N) strchr(N, DIRSEPARATOR[0])
#endif /*MSDOS*/

extern char       *TooManyErrors;


#ifdef SEGMENTED
#if defined(MPW) || defined(MW_CW)
#pragma segment Macio
#endif /*MPW||MW_CW*/
#endif /*SEGMENTED*/


#ifndef OUTLINESIZE
#define OUTLINESIZE   400  /*size of output line buffer*/
#endif /*OUTLINESIZE*/

/* OutLineBuf is allocated by mygetpointer in main() in macMain.c */

extern char      *OutLineBuf; /* buffer for building output lines */
static long       OutLinePlace = 0;

#ifndef MP_CW_5
#define fmyputs fputs
#define myputc  putc
#else /*MP_CW_5*/
/* 
	replacements for putc() and fputs() that are intended to ensure
	CR is written as newline character
*/
static int myputc(int c, FILE * fp)
{
	WHERE("myputc");
	c = (isNewline(c)) ? CR : c;
	return (putc(c, fp));
} /*myputc()*/

static int fmyputs(char * msg, FILE * fp)
{
	char      c;
	
	while ((c = *msg++))
	{
		if (myputc(c, fp) == EOF)
		{
			return(EOF);
		}
	}
	return (0);
} /*fmyputs()*/
#endif /*MP_CW_5*/

/*
  Write line of output to file fp which may specify command window
  If writing to console, interpret leading '\007's as BEL's
  If spooling is active, write line to spool file
  If line starts with "ERROR:", ring bell and count error
*/

static void lineToWindow(char * msg,  long msgLength)
{
	long            nLines, cutLength;
	long            windno;
	long            isOutLine = (msg == OutLineBuf);
	Str255          wTitle;
	char           *outstr = (char *) wTitle;
	unsigned char  *dashes = "\p\n----------------------------------\n";
	long            teLength;
	WHERE("lineToWindow");

	if (UseWindows > 0)
	{
		if (teCmd == (TEHandle) 0)
		{
			return;
		}
		teLength = TextLength(teCmd);
	}
	while(*msg == BEL)
	{						/* turn leading BEL's into beeps */
		if(UseWindows > 0)
		{
			mybeep(1);
		}
		msg++;
		msgLength--;
	}

	if (msgLength > 0)
	{
		if(UseWindows > 0)
		{
			if(UndoStatus != CANTUNDO && UndoStatus != OUTPUTUNDO)
			{
				clearUndoInfo();
			}

			if(msgLength > CMDWINDOWLIMIT)
			{
				sprintf(outstr,
						"WARNING: output line longer than %ld characters is truncated",
						(long) CMDWINDOWLIMIT);
				myAlert(outstr);
				msgLength = CMDWINDOWNEWSIZE;
			}

			if (teLength + msgLength > CMDWINDOWLIMIT)
			{
				myAlert("Output window contents are too long; saving window");

				DoFile(saveitas);

				for(windno=0;windno<MAXWINDOWS;windno++)
				{
					if(CmdWindows[windno].cmdWind == (WindowPtr) 0)
					{
						break;
					}
				}

				if(windno < MAXWINDOWS)
				{				/* we have room for a new window */
					TESetSelect(teLength, teLength, teCmd);
					TEInsert(dashes+1, dashes[0], teCmd);
					outstr = (char *) "\pContinued in new window\n";
					TEInsert(outstr+1, outstr[0], teCmd);
					CmdStrPos = TextLength(teCmd);
					ScrollToInsertPt();
					GetWTitle(CmdWind, wTitle);

					clearUndoInfo();

					DoWind(-newwind); /* create new window without prompt*/

					TESetSelect(0, 0, teCmd);
					outstr = (char *) "\pOutput continued from ";
					TEInsert(outstr+1, outstr[0], teCmd);
					TEInsert(wTitle+1, wTitle[0], teCmd);
					outstr = (char *) dashes;
					TEInsert(outstr+1, outstr[0], teCmd);
				} /*if(windno < MAXWINDOWS)*/
				else
				{				/* no room, trim off top */
					CmdInsertLine = 1;
					TESetSelect(0L, 0L, teCmd);
					ScrollToInsertPt();
				
					cutLength = msgLength + teLength - CMDWINDOWNEWSIZE;
					sprintf(outstr,
							"Cannot open a new window\n%ld characters will be trimmed off the top",
							cutLength);
					myAlert(outstr);

					TESetSelect(0L, cutLength, teCmd);
					TECut(teCmd);
					macUpdate(CmdWind);
					if(UndoStatus == OUTPUTUNDO)
					{
						LastLength -= cutLength;
						LastCmdStrPos -= cutLength;
						if(LastCmdStrPos < 0)
						{
							clearUndoInfo();
						}
					} /*if(UndoStatus == OUTPUTUNDO)*/
				} /*if(windno < MAXWINDOWS){}else{}*/
			} /*if (TextLength(teCmd) > CMDWINDOWLIMIT)*/

			teLength = TextLength(teCmd);
			TESetSelect(teLength, teLength, teCmd);
			TEInsert(msg, msgLength, teCmd);
			nLines = NLines(teCmd);
			CmdInsertLine = nLines;
			CmdStrPos = TextLength(teCmd);
			CmdScrollLines = nLines + 1 - ViewLines;
			SetCtlMax(CmdScroll, (CmdScrollLines > 0 ? CmdScrollLines : 0));
			CmdWindows[CurrentWindow].cmdDirty = CmdDirty = true;
		} /*if(UseWindows > 0)*/

		if(strncmp(ErrorStart,msg, (int) 6) == 0)
		{
			mybeep(1);
			if(!UseWindows && strcmp(msg, TooManyErrors) == 0)
			{
				if (!twoChoiceAlert("Go On", "Quit", "gq", msg))
				{
					if (isOutLine)
					{
						OutLinePlace = 0;
						OutLineBuf[0] = '\0';
					}
					if(!quitAlert('e'))
					{
						INTERRUPT = INTSET;
					}
				}
			} /*if(!UseWindows && strcmp(msg, TooManyErrors) == 0)*/
		} /*if(strncmp(ErrorStart,msg, (int) 6) == 0)*/
	} /*if (msgLength > 0)*/
	
	if (isOutLine)
	{
		OutLinePlace = 0;
		OutLineBuf[0] = '\0';
	}
} /*lineToWindow()*/

/* 970306 added */

void incrementNLINES(char *msg)
{
	while (*msg != '\0')
	{
		if (*msg++ == '\n')
		{
			NLINES++;
		}
	}
} /*incrementNLINES()*/

/*
   Open file in appropriate mode; may do things differently on different
   platforms;

   990713 Path names starting with '~' from DATAPATHS and DATAPATH
          are now expanded

   990723 Fixed bug arising when file name had already been expanded so
          it really pointed to the static buffer used by expandFilename()
          which was overwritten by expandFilename() here

   010324 After a successful open, the path of the file is copied to
          LastPathOpened.  LastPathOpened is unchanged after unsuccessful
          attempt.  Also the Macintosh and Unix/Dos versions of fmyopen()
          are now identical

   010605 tryFile created with code from fmyopen(); when fileName is ""
          if nothing else works, look in lastVol directory.  Now no two versions
		  of fmyopen() are the same.
*/

static FILE * tryFile(char lastPath [], char * fileName, char * mode)
{
	int             pathLength = strlen(lastPath);
	int             nameLength = strlen(fileName);
	FILE           *trialFile = (FILE *) 0;

	if (pathLength > 0 && !isSlash(lastPath[pathLength-1]))
	{
		lastPath[pathLength++] = DIRSEPARATOR[0];
		lastPath[pathLength] = '\0';
	}
	if (pathLength + nameLength <= PATHSIZE)
	{
		strcpy(lastPath + pathLength, fileName);
		if (okfilename(lastPath))
		{
			trialFile = fopen(lastPath,mode);
		}
	} /*if (okpathname(lastPath) && pathLength+nameLength <= PATHSIZE)*/
	return (trialFile);
} /*tryFile()*/

FILE * fmyopen(char * name, char * mode)
{
	Symbolhandle   dataPaths = (Symbolhandle) 0;
	FILE           *trialFile;
	long            ipath, npaths, pathLength, nameLength;
	char           *pathName;
	char            lastPath[PATHSIZE + 1];
	char            fileName[PATHSIZE + 1];

	strcpy(fileName, name);
	trialFile = fopen(fileName, mode);
	if (trialFile != (FILE *) 0)
	{
#ifdef MACINTOSH
		if (!isPath(fileName) && getMacPathName(lastPath) == noErr &&
			strlen(lastPath) + strlen(fileName) <= PATHSIZE)
		{
			strcat(lastPath, fileName);
		}
		else
#endif /*MACINTOSH*/
		{
			strcpy(lastPath, fileName);
		}
	}
	else if (!strchr(mode,'w') && !isPath(fileName))
	{	
		if (((dataPaths = Lookup("DATAPATHS")) != (Symbolhandle) 0 ||
			 (dataPaths = Lookup("DATAPATH")) != (Symbolhandle) 0) &&
			TYPE(dataPaths) == CHAR)
		{
			char        *pathName1;

			pathName = STRINGPTR(dataPaths);
			npaths = symbolSize(dataPaths);
			nameLength = strlen(fileName);

			for (ipath = 0; ipath < npaths;
				 ipath++, pathName = skipStrings(pathName, 1))
			{
				pathName1 = expandFilename(pathName);
			
				if (pathName1 != (char *) 0)
				{
					pathLength = strlen(pathName1);
					if (okpathname(pathName1) &&
						pathLength + nameLength <= PATHSIZE)
					{
						strcpy(lastPath, pathName1);
						/* Kludge: Fixup expandFilename buffer if needed */
						if (pathName1 == name)
						{
							strcpy(name, fileName);
						}
						trialFile = tryFile(lastPath, fileName, mode);
						if (trialFile != (FILE *) 0)
						{
							break;
						}
					} /*if (okpathname(pathName1) && pathLength+nameLength <= PATHSIZE)*/
					lastPath[0] = '\0';
				} /*if (pathName1 != (char *) 0)*/
			} /*for (ipath = 0; ipath < npaths; ...)*/
			if (trialFile == (FILE *) 0 && isMvDirInfoSet(lastVol))
			{
				macSetVol(getMvDirInfo(lastVol));
				
				trialFile = fopen(fileName, mode);
				if (trialFile != (FILE *) 0 && getMacPathName(lastPath) != noErr)
				{
					lastPath[0] = '\0';
				}
			} /*if (trialFile == (FILE *) 0 && isMvDirInfoSet(lastVol))*/
		}
	}
	
	if (trialFile != (FILE *) 0 && lastPath[0] != '\0')
	{
		strcpy(LastPathOpened, lastPath);
	}
	
	return (trialFile);
} /*fmyopen()*/

/*
	If fp != STDOUT, put msg to file fp; otherwise add msg to OutLineBuf and
	flush *OutLineBuf if last character is CR (Mac end of line)
	000520 added check for line starting with "WARNING:"
*/

void fmyprint(char *msg,FILE *fp)
{
	long            i, msgLength;
	char           *outline, c;
	WHERE("fmyprint");

	if(fp == STDOUT)
	{
		/* collect line in OutLineBuf */
		int         isError = strncmp(ErrorStart, msg, (int) 6) == 0;
		int         printwarnings = 
		  PRINTWARNINGS || isError || strncmp(WarningStart,msg, (int) 8) != 0;
		int         printit =  (!SWALLOWOUTPUT || isError) && printwarnings;
		int         spoolit =  (!SWALLOWSPOOL || isError) && printwarnings;

		if(isError && MAXERRORS1)
		{
			NERRORS++;
		}
		if (SPOOLFILE != (FILE *) 0 && spoolit)
		{
			fputs(msg, SPOOLFILE);
			if (SWALLOWSPOOL)
			{
				fputs("\n", SPOOLFILE);
			}
		} /*if (SPOOLFILE != (FILE *) 0 && spoolit)*/

		if (printit)
		{
			msgLength = strlen(msg);
			if (OutLinePlace + msgLength > OUTLINESIZE)
			{
				lineToWindow(OutLineBuf, OutLinePlace);
				if(msgLength > OUTLINESIZE)
				{
					lineToWindow(msg, msgLength);
					msgLength = 0;
				}
			}
			outline = OutLineBuf + OutLinePlace;
			for (i = 0; i < msgLength; i++)
			{
				c = *msg++;
				*outline++ = (c == NL) ? CR : c;
			}
			*outline = '\0';
			OutLinePlace += msgLength;
			if (OutLinePlace > 0 && OutLineBuf[OutLinePlace-1] == CR)
			{
				lineToWindow(OutLineBuf, OutLinePlace);
			}
			if (SWALLOWOUTPUT)
			{ /* must be error message; add EOL */
				fmyflush(STDOUT);
				TESetSelect(TextLength(teCmd), TextLength(teCmd), teCmd);
				TEKey(Return, teCmd);
				CmdWindows[CurrentWindow].cmdDirty = CmdDirty = true;
				CmdInsertLine = NLines(teCmd);
				CmdStrPos = TextLength(teCmd);
				CmdScrollLines = NLines(teCmd) + 1 - ViewLines;
				SetCtlMax(CmdScroll, (CmdScrollLines > 0 ? CmdScrollLines : 0));
				if (CmdInsertLine - CmdLine > ViewLines - 1)
				{
					DoScroll(CmdInsertLine - ViewLines - CmdLine + 5);
				}
			} /*if (SWALLOWOUTPUT)*/
		} /*if (printit)*/
	} /*if(fp == STDOUT)*/
	else
	{
		fmyputs(msg, fp);
	} /*if(fp == STDOUT){}else{}*/
} /*fmyprint()*/

/*
  terminate output line and pause if writing the console and screen is full
  (no pausing on Mac)
*/

void fmyeol(FILE * fp)
{
	if(fp == STDOUT)
	{
		if (OutLinePlace > 0)
		{ /* flush OutLineBuf */
			lineToWindow(OutLineBuf, OutLinePlace);
		}
		if(UseWindows > 0 && !SWALLOWOUTPUT)
		{
			if(UndoStatus != CANTUNDO && UndoStatus != OUTPUTUNDO)
			{
				clearUndoInfo();
			}
			TESetSelect(TextLength(teCmd), TextLength(teCmd), teCmd);
			TEKey(Return, teCmd);
			CmdWindows[CurrentWindow].cmdDirty = CmdDirty = true;
			CmdInsertLine = NLines(teCmd);
			CmdStrPos = TextLength(teCmd);
			CmdScrollLines = NLines(teCmd) + 1 - ViewLines;
			SetCtlMax(CmdScroll, (CmdScrollLines > 0 ? CmdScrollLines : 0));
			if (CmdInsertLine - CmdLine > ViewLines - 1)
			{
				DoScroll(CmdInsertLine - ViewLines - CmdLine + 5);
			}
		} /*if(UseWindows > 0 && !SWALLOWOUTPUT)*/
		if (SPOOLFILE != (FILE *) 0 && !SWALLOWSPOOL)
		{
			myputc(Newline, SPOOLFILE);
		}
		if(timetocheck(DEFAULTTICKS))
		{
			SkelChkOneMaskedEvent (everyEvent ^ keyUpMask, Sleep);
		}
		if(INTERRUPT != INTNOTSET)
		{
			INTERRUPT = PRINTABORT;
		}
	} /*if(fp == STDOUT)*/
	else
	{
		myputc(Newline, fp);
	}
} /*fmyeol()*/

void fmyflush(FILE * fp)
{
	if(fp != STDOUT)
	{
		fflush(fp);
	}
	else
	{
		lineToWindow(OutLineBuf, OutLinePlace);

		if(SPOOLFILE != (FILE *) 0)
		{
			fflush(SPOOLFILE);
		}
	}
} /*fmyflush()*/

/*
  sound n beeps, but only if not reading batch file and output is to console
*/

void mybeep(long n)
{
	if(BDEPTH == 0 && ISATTY)
	{
		while(n-- > 0)
		{
			SysBeep(30);
		}
	}
} /*mybeep()*/

/*
	Draw title in Console dialog box using system font
*/
pascal void consoleUserItem(WindowPtr theWindow, Integer item)
{
	Rect           r;
	Integer        oldFont = theWindow->txFont;
	Integer        oldSize = theWindow->txSize;
	Integer        itemType;
	Handle         itemHandle;
	unsigned char *msg = "\pEnter up to 500 characters.  Click Done to end";
	Integer        h, v;
	FontInfo       info;
	WHERE("editCommandUserItem");

	TextFont(systemFont);
	TextSize(12);
	GetFontInfo(&info);

	GetDItem((DialogPtr) theWindow, item, &itemType, &itemHandle, &r);

	h = (r.left + r.right - StringWidth( msg))/2;
	v = r.bottom - info.descent;
	MoveTo(h, v);
	DrawString( msg);

	TextFont(oldFont);
	TextSize(oldSize);
}

/*
  Fill global LINE with line from FILE *fn
  On Dos and Unix, fn should always be non-null.
  On Mac, STDIN is (FILE *) 0, and LINE is filled from dialog window when 
  fn == STDIN
*/

/*
  LineBuffer and Place are used only in fillFromDialog()
  However, fillFromDialog() may be called more than once for
  each actual use of a dialog box, since the dialog box can obtain
  several new-line separated lines

  001025 replace MAXLINE by MAXDIALOGBUF; LineBuffer now declared in mvIo.h
         and initialized in initialize() with length MAXDIALOGBUF +1 
  001027 Renamed LineBuffer to DlogLineBuffer and made it a handle allocated
         in fillFromDialog() and cleaned up when necessary in cleanitup()
         Place is now a long, with value < 0 signaling were starting
*/

static Boolean        HitDone = false;
static long           DlogLineBufferSize;
#define Place DlogLineBufferPlace

int fillFromDialog(void)
{
	GrafPtr               thePort;
	DialogPtr             theDialog;
	Integer               length;
	Rect                  r;
	Handle                itemHandle;
	TEHandle              textH;
	Integer               itemType, itemHit, nButtons = 2;
	Integer               doneItem = cancel;
	Integer               textItem = nButtons + 2, titleItem = textItem+1;
	unsigned char        *pc, *place;
	Integer               teLength;
	SignedByte            flags;
	WHERE("fillFromDialog");

	/*
	  Use dialog window or what was read previously
	  Always terminate (*LINE) with '\n'
	  Return only through first '\n'.
	  */

	if (Place < 0)
	{ /* first time in this trip */
		if (DlogLineBuffer != (unsigned char **) 0)
		{ /*Shouldn't happen but play it safe*/
			mydisphandle((char **) DlogLineBuffer);
		}
		DlogLineBuffer = (unsigned char **) mygethandle(LINELENGTH);
		Place = 0;
		HitDone = false;
		if (DlogLineBuffer != (unsigned char **) 0)
		{
			DlogLineBufferSize = LINELENGTH;
		}
		else
		{
			itemHit = doneItem;
			goto errorExit;
		}
	} /*if (Place < 0)*/

	(*LINE)[0] = '\0';

	if (Place == 0)
	{
		/* fill LineBuffer from dialog box */
		if (HitDone)
		{
			HitDone = false;
			itemHit = doneItem;
			goto normalExit;
		}
		/* need to read from Dialog box */
		GetPort(&thePort);
		SetDAFont(CmdFont);
		theDialog = getDlog(CONSOLEDLOG, true, nButtons, "nd");
		SetPort((GrafPtr) theDialog);
		TextFont(CmdFont);
		TextSize(CmdFontSize);
		GetDItem(theDialog, titleItem, &itemType, &itemHandle, &r);
		SetDItem(theDialog, titleItem, itemType,
				 (Handle) ConsoleUserItemPtr, &r);

		ShowWindow((WindowPtr) theDialog);

		do
		{
			ModalDialog(MyDialogFilterPtr,&itemHit);
		} while (itemHit > nButtons);

		HitDone = (itemHit == doneItem);

		/* directly access the text edit handle rather than use GetDItem() */
		textH = ((DialogPeek) theDialog)->textH;
		flags = HGetState((Handle) TextHandle(textH));
		HLock((Handle) TextHandle(textH));
		teLength = TextLength(textH);
		if(teLength >= DlogLineBufferSize)
		{
			SetDAFont(systemFont);
			sprintf(OUTSTR,
					"WARNING: more than %ld characters entered; last %d ignored",
					DlogLineBufferSize, teLength - DlogLineBufferSize);
			myAlert(OUTSTR);
			teLength = DlogLineBufferSize;
		}
		if(teLength > 0)
		{
			BlockMove((Ptr) *TextHandle(textH), (Ptr) *DlogLineBuffer,
					  teLength);
		}
		HSetState((Handle) TextHandle(textH), flags);

		(*DlogLineBuffer)[teLength] = '\0';
		strcpy(*LINE, (char *) *DlogLineBuffer);

		SetPort(thePort);
		SetCursor(&WATCH);
		DisposDialog(theDialog);
		SetDAFont(systemFont);
		if ((*LINE)[0] == '\0' && HitDone)
		{
			HitDone = false;
			itemHit = doneItem;
			goto normalExit;
		}
	} /*if(Place == 0)*/
	else
	{
		itemHit = (HitDone) ? doneItem : ok;
		strcpy(*LINE, (char *) *DlogLineBuffer + Place);
	}

	place = *DlogLineBuffer + Place;
	pc = (unsigned char *) strchr((char *) place, '\n');
	if(pc != (unsigned char *) 0 && pc[1] != '\0')
	{ /* still something not seen */
		pc++;
		length = pc - place;
		Place += length;
		itemHit = ok;
	} /*if(pc != (unsigned char *) 0)*/
	else
	{ /* this is the last of it */
		length = strlen((char *) place);
		if(pc == (unsigned char *) 0)
		{
			(*LINE)[length++] = '\n';
		}
		Place = 0;
		HitDone = false;
	} /*if(Place == LineBuffer){}else{}*/
	(*LINE)[length] = '\0';

  normalExit:
  errorExit:
	if (itemHit == doneItem)
	{
		mydisphandle((char **) DlogLineBuffer);
		DlogLineBuffer = (unsigned char **) 0;
	}
	return ((itemHit == doneItem) ? EOF : 1);
} /*fillFromDialog()*/

/*
  Local codes and static variables related to mouse input
  Other codes are in mvIo.h
  Installed 990603
*/

/*
  If OUTOFRECTRESTART is defined, moving cursor out of the port rectangle
  restarts the operation
*/	
#define OUTOFRECTRESTART

enum mouseKeyCodes
{
	abortKey          = QKEYCODE,
	restartKey        = RKEYCODE,
	interruptKey      = IKEYCODE,
	commandKey        = COMMANDKEYCODE,
	shifterKey        = SHIFTKEYCODE
};

/*
  The following is in mvIo.h
*/
#if (0)
typedef struct mouseInputInfo
{
	int       op;
	int       windno;
	long      n;
	double   *x;
	double   *y;
} mouseInputInfo;
#endif /*0*/

static mouseInputInfo       InputFromMouse;

enum linePatternCodes
{
	solidLine = 0,
	dashedLine
};

static Pattern LinePatterns [] =
{
	/* solid line */
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
	/* slant strip pattern */
	0xf0, 0x78, 0x3c, 0x1e, 0x0f, 0x87, 0xc3, 0xe1
};

/*
  Private routines related to "reading" the mouse
*/
static int mouseCheckKeys(void)
{
	scanKeyboard();
	if (isKeyPressed(commandKey) && isKeyPressed(interruptKey))
	{
		return (MouseInterrupt);
	}
	if (isKeyPressed(abortKey))
	{
		return (MouseAbort);
	}
	if (isKeyPressed(restartKey))
	{
		return (MouseRestart);
	}
	return (0);
} /*mouseCheckKeys()*/

static int isShifted(void)
{
	scanKeyboard();
	return (isKeyPressed(shifterKey));
} /*isShifted()*/

#define HEPS   2
#define VEPS   2

static void markPoint(Point aPoint)
{
	PenMode(patXor);

	MoveTo(aPoint.h - HEPS, aPoint.v);
	LineTo(aPoint.h + HEPS, aPoint.v);
	MoveTo(aPoint.h, aPoint.v - VEPS);
	LineTo(aPoint.h, aPoint.v + VEPS);
} /*markPoint()*/

static void drawCrossHairs(Point aPoint)
{
	GrafPtr         thePort;
	WindowPtr       graphWindow;
	Rect            r;

	GetPort(&thePort);
	graphWindow = (WindowPtr) thePort;
	r = graphWindow->portRect;
	PenMode(patXor);

	MoveTo(r.left, aPoint.v);
	LineTo(r.right - RIGHTOFFSET, aPoint.v);
	MoveTo(aPoint.h, r.top);
	LineTo(aPoint.h, r.bottom);
} /*drawCrossHairs()*/

static void drawBox(Point startPoint, Point otherPoint)
{
	PenMode(patXor);

#if defined(MPW3_2)
	PenPat((ConstPatternParam) (LinePatterns + dashedLine));
#elif defined(MW_CW) || defined(MPW_SC) || defined(MPW_MRC) /*MPW3_2*/
	PenPat((const Pattern *) LinePatterns + dashedLine);
#else /*MPW3_2*/
	PenPat((Pattern) LinePatterns[dashedLine]);
#endif /*MPW3_2*/

	MoveTo(startPoint.h, startPoint.v);
	LineTo(startPoint.h, otherPoint.v);
	LineTo(otherPoint.h, otherPoint.v);
	LineTo(otherPoint.h, startPoint.v);
	LineTo(startPoint.h, startPoint.v);
} /*drawBox()*/

static void drawLineSegment(Point startPoint, Point otherPoint)
{
	PenMode(patXor);

#if defined(MPW3_2)
	PenPat((ConstPatternParam) (LinePatterns + solidLine));
#elif defined(MW_CW) || defined(MPW_SC) || defined(MPW_MRC) /*MPW3_2*/
	PenPat((const Pattern *) LinePatterns + solidLine);
#else /*MPW3_2*/
	PenPat((Pattern) LinePatterns[solidLine]);
#endif /*MPW3_2*/

	MoveTo(startPoint.h, startPoint.v);
	LineTo(otherPoint.h, otherPoint.v);
} /*drawLineSegment()*/

static int readCrossHairs(long ix [], long iy [], int npoints)
{
	Point           lastPoint, thisPoint;
	PenState        pnState;
	int             dirty = 0;
	int             reply = MouseNormalReturn;
	int             ipoint = 0;
	GrafPtr         thePort;
	Rect            r;
	WHERE("readCrossHairs");

	GetPort(&thePort);
	r = ((WindowPtr) thePort)->portRect;

	GetPenState(&pnState);

#if defined(MPW3_2)
	PenPat((ConstPatternParam) (LinePatterns + solidLine));
#elif defined(MW_CW) ||  defined(MPW_SC) || defined(MPW_MRC) /*MPW3_2*/
	PenPat((const Pattern *) LinePatterns + solidLine);
#else /*MPW3_2*/
	PenPat((Pattern) LinePatterns[solidLine]);
#endif /*MPW3_2*/

	while (StillDown())
	{
		;
	}

	HideCursor();

	for (ipoint = 0; ipoint < npoints; ipoint++)
	{
		lastPoint.h = lastPoint.v = -32000;
		
		while(!Button())
		{
			reply = mouseCheckKeys();
			if (reply != MouseNormalReturn)
			{
				break;
			}
			GetMouse(&thisPoint);
			if (thisPoint.h != lastPoint.h ||thisPoint.v != lastPoint.v)
			{
				if (dirty)
				{
					drawCrossHairs(lastPoint);
					dirty = 0;
				}	
#ifdef OUTOFRECTRESTART
				if (!PtInRect(thisPoint, &r))
				{
					reply = MouseRestart;
					break;
				}
#endif /*OUTOFRECTRESTART*/
				drawCrossHairs(thisPoint);
				lastPoint = thisPoint;
				dirty = 1;
			}
		} /*while (!Button())*/
		while (Button())
		{
			;
		}
		if (dirty)
		{
			drawCrossHairs(thisPoint);
		}
		if (reply != MouseNormalReturn)
		{
			npoints = ipoint;
			break;
		}
		markPoint(thisPoint);
		ix[ipoint] = (long) thisPoint.h;
		iy[ipoint] = (long) thisPoint.v;
	} /*for (ipoint = 0; ipoint < npoints; ipoint++)*/

	/* unmark the marked points */
	for (ipoint = 0; ipoint < npoints; ipoint++)
	{
		lastPoint.h = (Integer) ix[ipoint];
		lastPoint.v = (Integer) iy[ipoint];
		markPoint(lastPoint);
		iy[ipoint] = r.bottom - iy[ipoint];
	}
	SetPenState(&pnState);
	ShowCursor();
	return (reply);
} /*readCrossHairs*/

static int readBox(long ix [], long iy [])
{
	Point           startPoint, lastPoint, aPoint;
	PenState        pnState;
	int             dirty = 0;
	int             reply = MouseNormalReturn;
	GrafPtr         thePort;
	Rect            r;

	GetPort(&thePort);
	r = ((WindowPtr) thePort)->portRect;

	GetPenState(&pnState);
	GetMouse(&startPoint);
	lastPoint.h = lastPoint.v = -32000;
	aPoint = lastPoint;

	while (StillDown())
	{
		/* wait for release */
		;
	}
	
	while (!Button())
	{
		Point        thisPoint;
		
		reply = mouseCheckKeys();
		if (reply != MouseNormalReturn)
		{
			break;
		}

		GetMouse(&thisPoint);
		if (thisPoint.h != lastPoint.h || thisPoint.v != lastPoint.v)
		{
#ifdef OUTOFRECTRESTART
			if (!PtInRect(thisPoint, &r))
			{
				reply = MouseRestart;
				break;
			}
#endif /*OUTOFRECTRESTART*/
			if (dirty)
			{
				/* Erase current box */
				drawBox(startPoint, aPoint);
				dirty = 0;
			}	
			aPoint = thisPoint;

			if (isShifted())
			{
				int      h = aPoint.h - startPoint.h;
				int      v = aPoint.v - startPoint.v;

				if (abs(h) > abs(v))
				{
					aPoint.h = startPoint.h + ((h > 0) ? abs(v) : -abs(v));
				}
				else
				{
					aPoint.v = startPoint.v + ((v > 0) ? abs(h) : -abs(h));
				}
			}
			drawBox(startPoint,aPoint);
			lastPoint = thisPoint;
			dirty = 1;
		} /*if (thisPoint.h != lastPoint.h || thisPoint.v != lastPoint.v)*/
	} /*while (!Button())*/
	if (dirty)
	{
		drawBox(startPoint, aPoint);
	}
	while (Button())
	{
		;
	}
	SetPenState(&pnState);

	iy[0] = (long) (r.bottom - startPoint.v);
	ix[0] = (long) startPoint.h;
	iy[1] = (long) (r.bottom - aPoint.v);
	ix[1] = (long) aPoint.h;

	return (reply);

} /*readBox()*/

static int readLine(long ix [], long iy [], int nlines)
{
	Point           startPoint, lastPoint, thisPoint, aPoint;
	PenState        pnState;
	int             dirty;
	int             reply = MouseNormalReturn;
	int             iline;
	GrafPtr         thePort;
	Rect            r;
	WHERE("readLine");
	
	GetPort(&thePort);
	r = ((WindowPtr) thePort)->portRect;

	GetPenState(&pnState);
	lastPoint.h = lastPoint.v = -32000;
	aPoint = lastPoint;

	while (StillDown())
	{
		/* wait for release */
		;
	}

	GetMouse(&startPoint);
	ix[0] = (long) startPoint.h;
	iy[0] = (long) startPoint.v;
	for (iline = 0; iline < nlines; iline++)
	{
		dirty = 0;

		/* Now track until Mouse again clicked */
		while (!Button())
		{
			reply = mouseCheckKeys();
			if (reply != MouseNormalReturn)
			{
				if (dirty)
				{
					iline++;
				}
				break;
			}
			GetMouse(&thisPoint);
			if (thisPoint.h != lastPoint.h ||thisPoint.v != lastPoint.v)
			{
				if (dirty)
				{
					drawLineSegment(startPoint,aPoint);
					dirty = 0;
				}	
				aPoint = thisPoint;
#ifdef OUTOFRECTRESTART
				if (!PtInRect(thisPoint, &r))
				{
					reply = MouseRestart;
					break;
				}
#endif /*OUTOFRECTRESTART*/
				if (isShifted())
				{
					int      h = aPoint.h - startPoint.h;
					int      v = aPoint.v - startPoint.v;
	
					if (abs(h) > abs(v))
					{
						aPoint.v = startPoint.v;
					}
					else
					{
						aPoint.h = startPoint.h;
					}
				} /*if (isShifted())*/
				drawLineSegment(startPoint, aPoint);
				ix[iline + 1] = (long) aPoint.h;
				iy[iline + 1] = (long) aPoint.v;
				lastPoint = thisPoint;
				dirty = 1;
			} /*if (thisPoint.h != lastPoint.h ||thisPoint.v != lastPoint.v)*/
		} /*while (!Button())*/
		while (Button())
		{
			;
		}
		if (reply != MouseNormalReturn)
		{
			break;
		}
		startPoint = aPoint;
	} /*for (iline = 0; iline < nlines; iline++)*/

	nlines = iline;

	/*
		Erase segments and reverse y coordinate
	*/
	thisPoint.h = ix[0];
	thisPoint.v = iy[0];
	iy[0] = r.bottom - iy[0];
	for (iline = 1; iline <= nlines; iline++)
	{
		startPoint = thisPoint;
		thisPoint.h = ix[iline];
		thisPoint.v = iy[iline];
		drawLineSegment(startPoint, thisPoint);
		iy[iline] = r.bottom - iy[iline];
	} /*for (iline = 1; iline <= nlines; i++)*/
	SetPenState(&pnState);

	return (reply);
} /*readLine()*/

/*
	Alternative event handler that
	(a)  Ignores mouse clicks in content of any text window
	(b)  Ignores keys except possibly menu commands
	(c)  Ignores mouse clicks in any close box
	(d)  In graph window does the appropriate thing for reading the
	     mouse
*/

static pascal Boolean ReadMouseEventHook(EventRecord *theEvent)
{
	Point           evtPt = theEvent->where;
	GrafPtr         evtPort;
	Integer         evtPart;
	Integer         evtMods = theEvent->modifiers;
	int             n = InputFromMouse.n;
	WHERE("ReadMouseEventHook");

	if (InputFromMouse.op == noMouseOp)
	{
		goto passBackEvent;
	}
	switch (theEvent->what)
	{
	  case mouseDown:
		evtPart = FindWindow(evtPt, &evtPort);

		switch (evtPart)
		{
		  case inSysWindow:
		  case inMenuBar:
		  case inGrow:
		  case inDrag:
		  case inZoomIn:
		  case inZoomOut:
			/*pass back to event handler*/
			goto passBackEvent;

		  case inGoAway:
			/* Ignore any attempt to close windows */
			goto ignoreEvent;

		  case inContent:
			{
				int          whichGraph, reply;
				long         ix[MaxMousePoints], iy[MaxMousePoints];
				long         xmax, ymax;
				Rect         r;

				if (evtPort != FrontWindow())
				{
					goto passBackEvent;
				}

				whichGraph = whichGraphWindow(evtPort);
				if (whichGraph < 0 || whichGraph >= NGRAPHS + NPANELS)
				{
					/* ignore if it's not a graphics window */
					goto ignoreEvent;
				}

				if (whichGraph >= NGRAPHS)
				{
					/* Must be panel */
					goto passBackEvent;
				}

				r = ((WindowPtr) evtPort)->portRect;
				/*
				  Set current limits of plotting area; may be different
				  from when graph was first drawn if it has been resized
				*/
				xmax = r.right - RIGHTOFFSET;
				ymax = r.bottom;

				InputFromMouse.windno = whichGraph;
				GraphMap = GraphMapArray + whichGraph;

				switch (InputFromMouse.op)
				{
				  case getPoint:
					reply = readCrossHairs(ix, iy, n);
					break;

				  case getLine:
					reply = readLine(ix, iy, n);
					break;

				  case getBox:
					reply = readBox(ix, iy);
					break;
				} /*switch (InputFromMouse.op)*/

				if (reply != MouseNormalReturn)
				{
					if (reply != MouseRestart)
					{
						InputFromMouse.windno = reply;
					}
					goto eventProcessed;					
				} /*if (reply != MouseNormalReturn)*/

				if (InputFromMouse.op != getBox)
				{
					int      i;
					
					if (InputFromMouse.op == getLine)
					{
						n++;
					}
					for (i = 0; i < n; i++)
					{
						InputFromMouse.x[i] = mapScreenToX(ix[i], xmax);
						InputFromMouse.y[i] = mapScreenToY(iy[i], ymax);
					}
					InputFromMouse.n = n;
				} /*if (InputFromMouse.op != getBox)*/
				else
				{
					/*
					  Return x, y coordinates of the four corners of
					  the box, with the first corner repeated at the end
					  to complete the rectangle
					*/
					InputFromMouse.n = 5;
					/* x on left side */
					InputFromMouse.x[0] = InputFromMouse.x[1] =
						InputFromMouse.x[4] = mapScreenToX(ix[0], xmax);
					/* x on right side */
					InputFromMouse.x[2] = InputFromMouse.x[3] =
						mapScreenToX(ix[1], xmax);
					/* y on top */
					InputFromMouse.y[0] = InputFromMouse.y[3] =
						InputFromMouse.y[4] = mapScreenToY(iy[0], ymax);
					/* y on bottom */
					InputFromMouse.y[1] = InputFromMouse.y[2] =
						mapScreenToY(iy[1], ymax);
				}
				InputFromMouse.op = noMouseOp;
			} /*case inContent*/
			goto eventProcessed;
		} /*switch (evtPart)*/

	  case keyDown:
	  case autoKey:
#ifndef	supportDialogs
		if (evtMods & cmdKey)	/* try menu equivalent */
		{
			goto passBackEvent;
		}
		goto ignoreEvent;
#endif

	  case keyUp:
		goto ignoreEvent;

	  default:
		/*pass back to event handler*/
		goto passBackEvent;
	} /*switch (theEvent->what)*/

  eventProcessed:
	/* fall through */
  ignoreEvent:
	return (true);

  passBackEvent:
	return (false);
} /*ReadMouseEventHook()*/

int readFromMouse(double x [], double y [], int op, long * n)
{
	SkelEventHookProcPtr   oldHook = SkelGetEventHook();
	WindowPtr              front = (WindowPtr) -1;
	int                    i;
	WHERE("readFromMouse");

	for (i = 0; i < NGRAPHS; i++)
	{
		if (GraphWind[i] != (WindowPtr) 0)
		{
			break;
		}
	} /*for (i = 0; i < NGRAPHS; i++)*/

	if (i == NGRAPHS)
	{
		return (MouseNoGraphs);
	}

	InputFromMouse.op = op;
	InputFromMouse.windno = 0;
	InputFromMouse.x = x;
	InputFromMouse.y = y;
	InputFromMouse.n = *n;

	while (InputFromMouse.op != noMouseOp && InputFromMouse.windno >= 0)
	{
		WindowPtr   thisWindow = FrontWindow();
	
		/*
		  Don't allow a window to be closed
		*/	
		DisableItem(WindowMenu, closeit);

		/* Prevent pasting to command window */
		DisableItem(EditMenu, pastit);
		if (thisWindow != front)
		{
			int         whichGraph;
			
			front = thisWindow;
			whichGraph = whichGraphWindow(thisWindow);
			if (whichGraph >= 0 && whichGraph < NGRAPHS + NPANELS)
			{
				SetCursor(&CROSS);
			}
		} /*if (thisWindow != front)*/
		SkelSetEventHook((SkelEventHookProcPtr) ReadMouseEventHook);
		/*
		  Indirect call to SkelEventLoop() which
		  routes all events to ReadMouseEventHook()
		*/
		SkelDoEvents(everyEvent ^ keyUpMask);
		SkelSetEventHook((SkelEventHookProcPtr) oldHook);
		if (interrupted(DEFAULTTICKS) != INTNOTSET)
		{
			InputFromMouse.windno = MouseInterrupt;
			break;
		}
	} /*while (InputFromMouse.op != noMouseOp && InputFromMouse.windno >= 0)*/
	InitCursor();
	EnableItem(WindowMenu, closeit);

	*n = InputFromMouse.n;

	return (InputFromMouse.windno);
	
} /*readFromMouse()*/

#ifdef TIMEANDMEMORY
#include <time.h>
#endif /*TIMEANDMEMORY*/

/*
   Return filename part of path
*/

static char * pathTail(char *filename)
{
	char     *tail;

	for (tail = filename + strlen(filename) - 1;tail >= filename;tail--)
	{
#ifndef MSDOS
		if (isSlash(*tail))
		{
			break;
		}
#else /*MSDOS*/
		if (isSlash(*tail) || *tail == ':')
		{
			break;
		}
#endif /*MSDOS*/
	} /*for (tail = filename + strlen(filename) - 1;tail >= filename;tail--)*/
	tail++;
	return (tail);
} /*pathTail()*/

/*
  If argument prompt is NULL, use global PROMPT (BDEPTH == 0)
  or batch file name (BDEPTH > 0), preceded by EOL.  This should only
  happen at first prompt for a command line.
  If prompt != NULL (probably "more> "), print it with no preceding EOL

  980415 modified to reflect changed prompt handling which allows a default
  prompt to be supplied instead of the file name when BDEPTH > 0.
*/

void putprompt(char * prompt)
{
	char           *prmpt;
	WHERE("putprompt");

	/* write out the prompt */
	prmpt = (prompt == (char *) 0) ? PROMPT : prompt;

	if(BDEPTH == 0)
	{
		if(prompt == (char *) 0)
		{
			myeol();
		}
		if (!SWALLOWOUTPUT)
		{
			myprint(prmpt);
		}
		else
		{
			lineToWindow(prmpt, strlen(prmpt));
			if (SPOOLFILE != (FILE *) 0)
			{
				fmyprint(prmpt, SPOOLFILE);
			}
		}
		fmyflush(STDOUT);
		if (UseWindows > 0)
		{
			TEKey(Return, teCmd);
			TEKey(Backspace, teCmd);
			CmdStrPos = TextLength(teCmd);
			
			if (CURRENTSCROLLBACK)
			{
				Integer      linesBack =
					GetCmdChL(CmdStrPos) - GetCmdChL(LastCmdStrPos) - ViewLines + 2;

				if (linesBack > 0)
				{
					DoScroll(-linesBack);
				}
			} /*if (CURRENTSCROLLBACK)*/
			CURRENTSCROLLBACK = SCROLLBACK; /* reset to default */
		} /*if (UseWindows > 0)*/
	} /*if(BDEPTH == 0)*/
	else if(BATCHECHO[BDEPTH])
	{
		if(prompt == (char *) 0)
		{
			myeol();
		}
		if (prmpt[0] != '\0')
		{
			myprint(prmpt);
		}
		else
		{
			myprint(pathTail(*INPUTFILENAMES[BDEPTH]));
			myprint("> ");
		}
	} /*if(BDEPTH == 0){}else{}*/
} /*putprompt()*/


#ifdef SAVEHISTORY
/* 970112 code to handle history*/

enum historyCodes
{
	NOHISTORY = 0,
	UPHISTORY,
	DOWNHISTORY,
	TOPHISTORY,
	BOTTOMHISTORY
};

enum historyErrors
{
	NOHISTORYERROR = 0,
	NOHISTORYTOGET,
	OFFHISTORYBOTTOM,
	OFFHISTORYTOP,
	HISTORYALLOCATIONERROR
};

/* handle for vector of char handles for history of previous commands*/
static char      ****SavedLines = (char ****) 0;
/* 
   Handle to contents of current command, created when accessing previous
   commands
*/
static char        **CurrentCommand = (char **) 0;
/* the current number of lines that are currently saved <= HISTORY */
static long          NSavedLines = 0;
/*
  index in history list of line currently being displayed if it came
  from the history list.  -1 means not a previous command
*/
static long          CurrentLine = -1;

/* 
   Shift all saved lines up 1 in preparation for adding new line to list
   Dispose of the earliest one if limit has been reached
*/
static void   shiftSavedLines(void)
{
	short       i;
	char      **temp;
	
	if (NSavedLines > 0)
	{
		temp = (*SavedLines)[NSavedLines-1];
		for (i = NSavedLines - 1; i > 0; i--)
		{
			(*SavedLines)[i] = (*SavedLines)[i - 1];
		}
		(*SavedLines)[0] = (char **) 0;
		if (NSavedLines < HISTORY)
		{
			(*SavedLines)[NSavedLines] = temp;
		} /*if (NSavedLines < HISTORY)*/
		else
		{
			mydisphandle(temp);
		}
		resetHistory();
	} /*if (NSavedLines > 0)*/
} /*shiftSavedLines()*/

/*
  Retrieve a line from history
  Values of op
    UPHISTORY     get next earlier command than currently displayed
    DOWNHISTORY   get next later command than currently displayed
    TOPHISTORY    get earliest command saved
    BOTTOMHISTORY redisplay command that had been typed before recalling
                  earlier commands
*/    

static char ** getHistoryLine(int op, int * historyError)
{
	char     **temp = (char **) 0;
	long       current = CurrentLine;
	int        incr = 0;
	WHERE("getHistoryLine");

	if (SavedLines != (char ****) 0)
	{
		if (op == UPHISTORY)
		{
			current++;
		}
		else if (op == DOWNHISTORY)
		{
			current--;
		}
		else if (op == TOPHISTORY)
		{
			current = NSavedLines - 1;
		}
		else if (op == BOTTOMHISTORY)
		{
			current = -1;
		}
		*historyError = NOHISTORYERROR;
		if (current >= 0 && current < NSavedLines)
		{
			temp = (*SavedLines)[current];
			CurrentLine = current;
		}
		else if (current < 0)
		{
			if (current < -1)
			{
				*historyError = OFFHISTORYBOTTOM;
			}
			temp = CurrentCommand;
			resetHistory();
		}
		else
		{
			*historyError = OFFHISTORYTOP;
			if (NSavedLines > 0)
			{
				temp = (*SavedLines)[NSavedLines-1];
				CurrentLine = NSavedLines - 1;
			}
			else
			{
				temp = CurrentCommand;
				resetHistory();
			}
		}
	} /*if (SavedLines != (char ****) 0)*/
	else
	{
		*historyError = NOHISTORYTOGET;
		temp = CurrentCommand;
		resetHistory();
	}
	return (temp);
} /*getHistoryLine()*/

/*
   Initialize or change the length of a vector of handles to hold recent
   history

   If there are memory allocation problems, mygetsafehandle() or
   mygrowsafehandle() will print an error message, and history will be turned
   off because SavedLines will be NULL

   971023 Fixed apparent memory leak when history length is reduced
   971210 Negative historyLength disposes of lines but not SavedLines
*/

void initHistory(long historyLength)
{
	long           i;
	WHERE("initHistory");

	if (SavedLines != (char ****) 0 && NSavedLines > historyLength)
	{
		long        start = (historyLength >= 0) ? historyLength : 0;
		
		/* dispose of extras, if any */
		for (i = start; i < NSavedLines; i++)
		{
			mydisphandle((*SavedLines)[i]);
			(*SavedLines)[i] = (char **) 0;
		}
		NSavedLines = start;
	} /*if (SavedLines != (char ****) 0)*/
	
	if (historyLength < 0)
	{
		historyLength = HISTORY;
	}
	else if (historyLength > 0)
	{
		if (SavedLines == (char ****) 0)
		{
			SavedLines = (char ****) mygetsafehandle((historyLength) * sizeof(char **));
			if (SavedLines == (char ****) 0)
			{
				historyLength = 0;
			}
			else
			{
				for (i = 0; i < historyLength; i++)
				{
					(*SavedLines)[i] = (char **) 0;
				}
			}
			NSavedLines = 0;
		}
		else if (historyLength != HISTORY)
		{
			char       ****savedLines =
			  (char ****) mygrowsafehandle((char **) SavedLines,
										   historyLength * sizeof(char **));

			if (savedLines == (char ****) 0)
			{
				/* don't changed HISTORY */
				historyLength = HISTORY;
			}
			else
			{
				SavedLines = savedLines;
				/* clear new slots, if any */
				for (i = NSavedLines; i < historyLength; i++)
				{
					(*SavedLines)[i] = (char **) 0;
				}
				if (historyLength < NSavedLines)
				{
					NSavedLines = historyLength;
				}
			}
		}
	} /*if (historyLength > 0)*/
	else if (SavedLines != (char ****) 0)
	{
		mydisphandle((char **) SavedLines);
		SavedLines = (char ****) 0;
		NSavedLines = 0;
	}

	resetHistory();
	mydisphandle(CurrentCommand);
	CurrentCommand = (char **) 0;
	HISTORY = historyLength;
} /*initHistory()*/

/*
  Reset CurrentLine pointer
  020815 new
*/

void resetHistory(void)
{
	CurrentLine = -1;
} /*resetHistory()*/

/*
   Save *lineH in history list

   All previous lines are shifted up 1, discarding the last if the
   limit has been reached.  The line being saved is put in slot 0
   
   990708 Added argument offset
*/

void saveInHistory(char ** lineH, int offset)
{
	long        i;
	WHERE("saveInHistory");
	
	mydisphandle(CurrentCommand);
	CurrentCommand = (char **) 0;
	resetHistory();
	if (SavedLines != (char ****) 0 && (*lineH)[0] != '\0')
	{
		long        nDoubles = strlen(*lineH + offset)/sizeof(double) + 1;
		char      **line = mygetsafehandle(nDoubles*sizeof(double));
		/*
			allocate a multiple of sizeof(double) to make it more likely
			to use a handle in one of the caches
		*/

		if (line == (char **) 0)
		{
			goto errorExit;
			
		}
		shiftSavedLines();
		(*SavedLines)[0] = line;
		strcpy(*line, *lineH + offset);
		if (NSavedLines < HISTORY)
		{
			NSavedLines++;
		}
	} /*if (SavedLines != (char ****) 0 && (*lineH)[0] != '\0')*/
	return;

  errorExit:
/* don't know anything better to do than clear history list */
	myAlert("History of past commands cleared");
	for (i = 0;i < NSavedLines;i++)
	{
		mydisphandle((*SavedLines)[i]);
		(*SavedLines)[i] = (char **) 0;
	} /*for(i=0;i<NSavedLines;i++)*/
	mydisphandle(CurrentCommand);
	CurrentCommand = (char **) 0;
	NSavedLines = 0;
	resetHistory();

} /*saveInHistory()*/

/*
  Create a handle containing everything from the start of the command
  line to the end of command window

  020815 made a public function (no longer static)
*/
char ** getCurrentCommand(void)
{
			
	Integer      length = TextLength(teCmd) - CmdStrPos;
	char       **command;
	
	command = mygetsafehandle((length/sizeof(double) + 1)*sizeof(double));
	if (command != (char **) 0)
	{
		if (length > 0)
		{
			HLock(TextHandle(teCmd));
			BlockMove((Ptr) *TextHandle(teCmd) + CmdStrPos,
					  (Ptr) *command, (Size) length);
			HUnlock(TextHandle(teCmd));
		}
		(*command)[length] = '\0';
	}
	return (command);
} /*getCurrentCommand()*/

/*
	970112 Handle Up History and Down History commands on Edit menu
*/

char * recallHistory(unsigned char ch, Integer mods)
{
	char        **lineH;
	char         *line = (char *) 0;
	int           historyOp, historyError;
	WHERE("recallHistory");

	if (!Running && BDEPTH <= 0)
	{
		if (CurrentLine < 0)
		{
			mydisphandle(CurrentCommand);
			CurrentCommand = getCurrentCommand();
		} /*if (CurrentLine < 0)*/

		if (ch == UpArrow)
		{
			historyOp = (mods & cmdKey) ? TOPHISTORY : UPHISTORY;
		}
		else
		{
			historyOp = (mods & cmdKey) ? BOTTOMHISTORY : DOWNHISTORY;
		}
		lineH = getHistoryLine(historyOp, &historyError);
		line = (lineH != (char **) 0) ? *lineH : NullString;

		if (historyError != NOHISTORYERROR)
		{
			mybeep(1);
		}
	} /*if (!Running && BDEPTH <= 0)*/
	return (line);
} /*recallHistory()*/

enum historyScratch
{
	GLINES = 0,
	NTRASH
};


long getSomeHistory(long nlines, char *** historyHandle)
{
	char         **lines;
	long           i, needed;
	char          *place;
	TRASH(NTRASH, errorExit);
	
	if (NSavedLines <= 0)
	{
		nlines = 0;
	}
	else if (nlines == 0 || nlines >= NSavedLines)
	{
		nlines = NSavedLines - 1;
	}
	
	if (nlines == 0)
	{
		needed = 1;
	}
	else
	{
		needed = 0;
		for (i = 1; i <= nlines; i++)
		{
			needed += strlen(*(*SavedLines)[i]) + 1;
		}
	}
	
	if (!getScratch(lines, GLINES, needed, char))
	{
		goto errorExit;
	}
	
	place = *lines;

	if (nlines == 0)
	{
		nlines = 1;
		place[0] = '\0';
	}
	else
	{
		for (i = nlines; i > 0; i--)
		{
			place = copyStrings(*(*SavedLines)[i], place, 1);
		}
	}
	unTrash(GLINES);
	*historyHandle = lines;
	emptyTrash();
	return (nlines);
		
  errorExit:
	return (0);
} /*getSomeHistory()*/

/*
	990707 Fixed bug
	       Also changed to reduce possibility that commands could move.
	990708 Changed call to saveInHistory and rewrote
*/
void setSomeHistory(long nlines, char ** commands)
{
	long         i, history = HISTORY;
	int          skippedlines = (nlines > HISTORY) ? nlines - HISTORY : 0;
	WHERE("setSomeHistory");

	initHistory(-1); /* get rid of what we've got */
	initHistory(history);

	for (i = 0; i < nlines; i++, skippedlines++)
	{
		saveInHistory(commands, skipStrings(*commands, skippedlines) - *commands);
	}
} /*setSomeHistory()*/
	
#endif /*SAVEHISTORY*/

/* 
  Expand leading '~' in a filename.  A name of the form ~/path (or ~\path
  under MSDOS) is expanded to home/path or home\path, where home is the value
  of MacAnova CHARACTER variable HOME.  Under Unix, HOME is predefined to
  have value taken from environmental variable $HOME.  Under MSDOS, HOME is
  predefined to contain the name of the directory containing MACANOVA.EXE.
  Under Unix, a name of form ~username/path is expanded to homedir/path where
  getpwname() is used to get the home directory homedir for user username.
  Such a file name is not expanded under MSDOS and results in an error.
  On Macintosh, HOME is predefined to be the full name of the MacIntosh folder.
  The usage ~username/path is not legal on a Mac
*/

static char    FullPathName[PATHSIZE];

char *expandFilename(char * fname)
{
	Symbolhandle    home;
	char           *pc;
	char           *userdir = (char *) 0;
	char           *sep1 = DIRSEPARATOR;
	char           *sep2 = "/";
	long            dirNameLength;

	*OUTSTR = '\0';
	if(fname[0] == EXPANSIONPREFIX)
	{ /* name starts with '~' */
		if(fname[1] == *sep1 || fname[1] == *sep2)
		{/* treat ~:... or ~/... specially */
			home = Lookup("HOME"); /* should be initialized at startup */
			if(home == (Symbolhandle) 0 || TYPE(home) != CHAR ||
			   !isScalar(home))
			{
				sprintf(OUTSTR,
						"ERROR: cannot expand %c%c unless CHARACTER variable HOME is defined",
						fname[0], fname[1]);
				goto errorExit;
			}
			userdir = STRINGPTR(home);
			pc = fname+2;
		}
		else
		{
/* can't expand ~name on a Macintosh */
			userdir = (char *) 0;
		}

		if(userdir == (char *) 0)
		{
			putPieces("ERROR: unable to expand ",fname,
					  (char *) 0, (char *) 0);
			goto errorExit;
		}

		dirNameLength = strlen(userdir);
/* allow for the possibility that the HOME ends in ":" */
		if(userdir[dirNameLength-1] == *sep1)
		{
			dirNameLength--;
		}
		if(dirNameLength + strlen(pc) + 1 >= PATHSIZE)
		{
			putPieces("ERROR: expanded file name ", userdir, pc,
					  " is too long");
			goto errorExit;
		}

		strcpy(FullPathName,userdir);
		FullPathName[dirNameLength++] = *sep1;
		FullPathName[dirNameLength] = '\0';
		strcat(FullPathName,pc);
	} /*if(fname[0] == EXPANSIONPREFIX)*/
	else
	{
		if(strlen(fname) > PATHSIZE)
		{
			putPieces("ERROR: filename ", fname, " is too long", (char *) 0);
			goto errorExit;
		}

		strcpy(FullPathName, fname);
	}

	return ((char *) FullPathName);

  errorExit:
	putOUTSTR();

	return ((char *) 0);

} /*expandFilename()*/

/*
  On Macintosh accept non-empty names of the form
       pathElement or pathElement:pathElement or pathElement:...:pathElement
  where a pathElement does not contain '/' but can be empty
  If path != 0, only restriction is starting with '.'
  If path == 0, it should be file name not ending in ':'
*/
static int filenameCharsOK(char * fname)
{
	return (1);
} /*filenameCharsOK()*/

static int filenameFormOK(char * fname, int path)
{
	int        ok = (fname[0] != '.' && strlen(fname) <= PATHSIZE &&
					 (path || !isSlash(fname[strlen(fname)-1])));
	char      *sepPlace;
	
	for (sepPlace = fname - 1; ok && sepPlace != (char *) 0;
		 sepPlace = strchr(sepPlace, DIRSEPARATOR[0]))
	{
		ok = *(++sepPlace) != '.';
	}
	return (ok);
} /* filenameFormOK()*/

/*
   silent check of appropriateness of file name
*/
long okfilename(char *fname)
{
	return (*fname != '\0' && filenameFormOK(fname, 0) &&
			filenameCharsOK(fname));
} /*okfilename()*/

/*
   silent check of appropriateness of path name
*/
long okpathname(char *fname)
{
	return (*fname != '\0' && filenameFormOK(fname, 1) &&
			filenameCharsOK(fname));
} /*okpathname()*/

/*
  Check file names for legality on platform
  On the Mac, essentially any name is legal as long as it is not too
  long and does not start with '.' (names starting with '.' may be interpreted
  as driver names by the OS)
*/

/* Check filename with error messages */
long isfilename(char *fname)
{
	if (okfilename(fname))
	{
		return (1);
	}

	*OUTSTR = '\0';
	if(*fname == '\0')
	{
		sprintf(OUTSTR,"ERROR: zero length file name");
	}
	else if(strlen(fname) >= PATHSIZE)
	{
		putPieces("ERROR: file name ", fname, " is too long", (char *) 0);
	}
	else if(*fname == '.')
	{
		sprintf(OUTSTR, "ERROR: file names must not start with '.'");
	}
	else
	{
		putPieces("ERROR: improper file name ", fname, (char *) 0, (char *) 0);
	}

	putOUTSTR();
	
	return (0);
} /*isfilename()*/

/*
  010529 new function
  function to split off either the file name (nameOnly != 0) or the
  directory name (nameOnly == 0) from full name in fullPathName, returning
  the piece wanted in pathPart which has length partLength
*/
void splitPathName(int nameOnly, char fullPathName [], char pathPart [],
				   int partLength)
{
	int        n = strlen(fullPathName);
	char       separator = DIRSEPARATOR[0];
	char      *tail, *pieceWanted;

	for (tail = fullPathName + n; *tail != separator && tail >= fullPathName;
		 n--, tail--)
	{
		;
	}
	tail++;
	n++;

	if (nameOnly)
	{
		pieceWanted = tail;
		n = strlen(tail);
	}
	else
	{
		pieceWanted = fullPathName;
	}
	if (n >= partLength)
	{
		n = partLength - 1;
	}
	
	strncpy(pathPart, pieceWanted, n);
	pathPart[n] = '\0';
} /*splitPathName()*/


