/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.13 or later
*(C)*
*(C)* Copyright (c) 2003 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

/*
  Header unifying some of the handling of graph windows
  990522 Created
  010524 Modified openPlotFile()
  030615 Added global PLOTFILEMODE for use with Carapace
*/
#ifndef MVGRAPHSH__
#define MVGRAPHSH__

#undef	EXTERN
#undef	INIT
#undef	INITDIM
#undef	INITARRAY
#undef  COMMA

#define COMMA ,

#ifdef MAIN__
#define EXTERN
#define INIT(VAL) = VAL
#define INITDIM(N) N
#define INITARRAY(VALS) = { VALS }
#else
#define EXTERN extern
#define INIT(VAL)
#define INITDIM(N)
#define INITARRAY(VALS)
#endif /*MAIN__*/

#ifndef PLATFORMH__
#include "platform.h"
#endif /* PLATFORMH__ */

#ifndef SYMBOLH__
#include "Symbol.h"
#endif /*SYMBOLH__*/

#ifdef MACINTOSH
#include <Types.h>
#endif /*MACINTOSH*/

/*
	Graph windows
*/
#ifndef NGRAPHS
/* NGRAPHS is the Maximum number of graph windows excluding panels, if any*/
#if defined(WXWIN) || defined(SKEL_WXWIN) || defined(MACINTOSH) || defined(SKEL_TRANSKEL)
#define NGRAPHS      8
#elif defined(SKEL_CARAPACE)
#define NGRAPHS     24
#else
#define NGRAPHS      1
#endif
#endif /*NGRAPHS*/

#ifndef LASTGRAPH
#define LASTGRAPH "LASTPLOT"
#endif /*LASTGRAPH*/

/*
  Globals related to plots
*/

EXTERN FILE           *PLOTFILE  INIT((FILE *) 0);
EXTERN char          **PLOTFILENAME INIT((char **) 0); /* handle */
EXTERN char           *PLOTFILEMODE;

/*
  DEF.*COLOR are defined in platform.h, possibly platform specific
*/
EXTERN unsigned char   DEFAULTPOINTCOLOR  INIT((unsigned char) DEFPOINTCOLOR);
EXTERN unsigned char   DEFAULTLINECOLOR   INIT((unsigned char) DEFLINECOLOR);
EXTERN unsigned char   DEFAULTSTRINGCOLOR INIT((unsigned char) DEFSTRINGCOLOR);
EXTERN unsigned char   DEFAULTFGCOLOR     INIT((unsigned char) DEFFGCOLOR);
EXTERN unsigned char   DEFAULTBGCOLOR     INIT((unsigned char) DEFBGCOLOR);
EXTERN unsigned char   DEFAULTFRAMECOLOR  INIT((unsigned char) DEFFRAMECOLOR);

EXTERN unsigned char   DEFAULTXTICKLENGTH INIT((unsigned char) TICKUNIT);
EXTERN unsigned char   DEFAULTYTICKLENGTH INIT((unsigned char) TICKUNIT);

/* values for global ThisWindow */
enum thisWindowCodes
{
	FINDEMPTYWINDOW  = -1, /*no window keyword on plotting command*/
	PREVIEWWINDOW    = -2, /*creating picture for preview part of EPSF file*/
	PICTWINDOW       = -3  /*creating picture for writing to file*/
};


EXTERN long    ThisWindow INIT(FINDEMPTYWINDOW); /*current graphics window*/

EXTERN char    LastGraphName[] INIT(LASTGRAPH);

/* Globals for naming components of GRAPHWINDOWS */
EXTERN char   GraphWindowName[] INIT("Graph_");
EXTERN char   EmptyWindowName[] INIT("Empty_");

#undef	EXTERN
#undef	INIT
#undef	INITDIM
#undef	INITARRAY
#undef  COMMA

#include "plot.h"

#ifndef NOPROTOTYPES
#include "keywords.h"

/*
  Graph related prototypes (not including MacAnova functions)
*/

/* plotutil.c*/
void set_point(struct curve_points ** /*plot*/, long /*ipoint*/,
			   double /*x*/, double /*y*/, char * /*string*/,
			   unsigned char /*pointColor*/, unsigned char /*linecolor*/);
void undef_point(struct curve_points ** /*plot*/,long /*ipoint*/);
long do_whole_graph(whole_graph ** /*graph*/, FILE * /*fp*/,
					long /*termType*/, long /*windno*/,
					plotKeyValuesPtr /*keyValues*/);
void deleteGraph(struct whole_graph ** /*graph*/);
struct whole_graph ** copyGraph(struct whole_graph ** /*graph*/);
void getGraphExtremes(whole_graph ** /*graph*/, double * /*extremes*/,
					  long /*pointsOnly*/);
long getPlotKeys(Symbolhandle/*list*/,long /*startKey*/,
				 plotKeyValuesPtr /*keyValues*/);
void unsetPlotKeys(plotKeyValuesPtr /*keyValues*/, long /*all*/);
long checkPlotKeys(long /*toFile*/, plotKeyValuesPtr /*keyValues*/,
				   long * /*termType*/);
int saveGraph(Symbolhandle /*symh*/, char * /*name*/);
int restoreGraph(Symbolhandle /*symh*/);
FILE *openPlotFile(plotKeyValuesPtr /*keyValues*/);
long sizeofPlot(whole_graph ** /*graph*/);
int screenDump(char * /*fname*/);
Symbolhandle makeNullGraph(int /*whichGraph*/, Symbolhandle /*nullsym*/);
void setGraphWindow(Symbolhandle /*symhGraph*/, int /*whichWindow*/);
int getNgraphs(void);
Symbolhandle getGraphWindow(int /*whichWindow*/);
int whichGraphWindowsComponent(Symbolhandle /*symhGraph*/);
void resetGraphWindowNames(void);
void nameGraphWindow(Symbolhandle /*symhGraph*/, int /*whichWindow*/);
void attachGraphWindow(Symbolhandle /*symhGraph*/, int /*whichWindow*/);
int detachGraphWindow(Symbolhandle /*symhGraph*/);
long iniGraphWindows(char * /*name*/);
int restoreGraphWindows(Symbolhandle /*symh*/);

#else /*NOPROTOTYPES*/
/* plotutil.c*/
void set_point(/*struct curve_points ** plot, long ipoint,
			   double x, double y, char * string,
			   unsigned char pointColor, unsigned char linecolor*/);
void undef_point(/*struct curve_points ** plot,long ipoint*/);
long do_whole_graph(/*whole_graph ** graph, FILE * fp,
					long termType, long windno,
					plotKeyValuesPtr keyValues*/);
void deleteGraph(/*struct whole_graph ** graph*/);
struct whole_graph ** copyGraph(/*struct whole_graph ** graph*/);
void getGraphExtremes(/*whole_graph ** graph, double * extremes,
					  long pointsOnly*/);
long getPlotKeys(/*Symbolhandlelist,long startKey,
				 plotKeyValuesPtr keyValues*/);
void unsetPlotKeys(/*plotKeyValuesPtr keyValues, long all*/);
long checkPlotKeys(/*long toFile, plotKeyValuesPtr keyValues,
				   long * termType*/);
int saveGraph(/*Symbolhandle symh, char * name*/);
int restoreGraph(/*Symbolhandle symh*/);
FILE *openPlotFile(/*plotKeyValuesPtr keyValues*/);
long sizeofPlot(/*whole_graph ** graph*/);
int screenDump(/*char * fname*/);
Symbolhandle makeNullGraph(/*int whichGraph, Symbolhandle nullsym*/);
void setGraphWindow(/*Symbolhandle symhGraph, int whichWindow*/);
int getNgraphs(/*void*/);
Symbolhandle getGraphWindow(/*int whichWindow*/);
int whichGraphWindowsComponent(/*Symbolhandle symhGraph*/);
void resetGraphWindowNames(/*void*/);
void nameGraphWindow(/*Symbolhandle symhGraph, int whichWindow*/);
void attachGraphWindow(/*Symbolhandle symhGraph, int whichWindow*/);
int detachGraphWindow(/*Symbolhandle symhGraph*/);
long iniGraphWindows(/*char * name*/);
#endif /*NOPROTOTYPES*/

#endif /*MVGRAPHSH__*/

