/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.13 or later
*(C)*
*(C)* Copyright (c) 2003 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

/*
  990609 Changed name of header file from matdat.h to mvIo.h
         Added constants related to Mouse input
  000414 Changed code name NOERROR to READNOERROR
  001026 Added globals and constants related to global LINE
  001102 Added globals MatchDelay and DefaultMatchDelay related to
         option 'matchdelay' in windowed versions
  010524 Removed #pragma segment stuff
        Changed defines to enum.
  010601 Replaced if defined(MACINTOSH) || defined (WXWIN) by
         ifdef HASCONSOLEDIALOG
         Added define BS, VT, FF, MSDOSEOF, DEL
  030716 Add enum getinputReturnCodes
*/

#ifndef MVIOH__
#define MVIOH__

#undef EXTERN
#undef INIT
#undef INITDIM
#undef INITARRAY
#undef COMMA

#ifdef MAIN__
#define EXTERN
#define INIT(VAL) = VAL
#define INITDIM(N) N
#define INITARRAY(VALS) = { VALS }
#else
#define EXTERN extern
#define INIT(VAL)
#define INITDIM(N)
#define INITARRAY(VALS)
#endif /*MAIN__*/

#define COMMA ,

/*
  Constants and variables related to input from files
*/

/*
  010319 added global LastPathOpened; storage allocated in initialize()
         value is set by
*/

EXTERN char          *LastPathOpened INIT((char *) 0); /* length PATHSIZE+1 */

/* 001025 added globals related to input line length */
#ifndef DEFAULTMAXLINE /*may be set in Makefile*/
#ifdef BCPP
#define DEFAULTMAXLINE  500 /* maximum length of input line */
#else /*BCPP*/
#define DEFAULTMAXLINE 2000 /* maximum length of input line */
#endif /*BCPP*/
#endif /*DEFAULTMAXLINE*/

#ifndef DEFAULTLINECHUNK
#define DEFAULTLINECHUNK 500L
#endif /*DEFAULTLINECHUNK*/


/*
  MAXLINE is the value of option 'linelength'
  Prior to 001025 MAXLINE was defined as DEFAULTMAXLINE now is
  MAXLINE is the absolute limit to the length of a data line that can be read
  LINELENGTH is the current length of *LINE
  LINECHUNK  is the size of the initial allocation of LINE
  (char **) LINE is defined in globkb.h
*/

EXTERN long            MAXLINE INIT(DEFAULTMAXLINE);
EXTERN long            LINELENGTH INIT(0);
EXTERN long            LINECHUNK INIT(DEFAULTLINECHUNK);

/* global handle for line input buffer */
EXTERN char          **LINE INIT((char **) 0);

#ifdef HASCONSOLEDIALOG
/*
  Used by fillFromDialog; initialized in initialize()
  001027 renamed LineBuffer to DlogLineBuffer and changed it
         to a handle to be allocated in fillFromDialog() and, when
         necessary, disposed of in cleanitup()
*/
EXTERN unsigned char  **DlogLineBuffer INIT((unsigned char **) 0);
EXTERN long             DlogLineBufferPlace INIT(-1);

#endif /*HASCONSOLEDIALOG*/

#ifdef SCROLLABLEWINDOW
#ifndef DEFAULTMATCHDELAY
#ifdef MACINTOSH
#define DEFAULTMATCHDELAY     20
#else /*MACINTOSH*/
#define DEFAULTMATCHDELAY    400
#endif /*MACINTOSH*/
#endif /*SCROLLABLEWINDOW*/

EXTERN long DefaultMatchDelay INIT(DEFAULTMATCHDELAY);
EXTERN long MatchDelay        INIT(DEFAULTMATCHDELAY);

#endif /*SCROLLABLEWINDOW*/

#undef	EXTERN
#undef	INIT
#undef	INITDIM
#undef	INITARRAY
#undef  COMMA

enum
{
	MAXFMT          = DEFAULTMAXLINE,       
	MAXN            =      50,            
	MAXNAME         =     200,
	ENDMACROLENGTH  = MAXNAME,       
	ENDMACROSTART   =     '%',           
	FORMATITEMSTART =     '%',           
	COMPNAMESTART   =     '$',
	COMMENTSTART    =     ')', /* initial character on comment line */
	FORMATSTART     =     '(' /* initial character on Fortran format line */
};

#undef SP
#undef TAB
#undef BS
#undef NL
#undef VT
#undef FF
#undef CR
#undef MSDOSEOF
#undef DEL

#define SP        ' '
#define TAB       '\t'
#define BS        0x08 /* 8*/
#define NL        0x0a /*10*/
#define LF        0x0a /*10*/
#define VT        0x0b /*11*/
#define FF        0x0c /*12*/
#define CR        0x0d /*13*/
#define MSDOSEOF  0x1a /*26*/
#define DEL       0x7f /*127*/


#ifdef READDATA__
/* following should have exactly MAXN "%lf"'s */

static char DefaultFormat[] = "%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf" ;
#endif /*READDATA__*/

enum getinputReturnCodes
{
	fatalOutOfMemory1  = -2, /*Out of memory fatal error (windowed)*/
	endOfInputFile     = -1, /*End of input file (Mac non-interactive mode)*/
	fatalOutOfMemory2  =  0, /*Out of memory fatal error (nonwindowed)*/
	inputLineReady     =  1, /*Normal return, input line is in INPUTSTRING*/
	inputLineReadyExtra = 2, /*Same, except there are extra characters (windowed)*/
	noInputString      =  3  /*Unable to allocate INPUTSTRING (windowed)*/
};

enum readErrorCodes
{
	READNOERROR = 0,
	OPENERROR,
	NOTFOUND,
	COMPONENTNOTFOUND,
	BADHEADER,
	NODATA,
	TWOFORMATS,
	SHORTDATA,
	BADFORMAT,
	BADCFORMAT,
	NOFIELDS,
	NTOOBIG,
	CANTALLOC,
	NOLINES,
	NOMACRO
};


enum charFmtCodes
{
	BYLINES = 1,
	BYFIELDS,
	BYQUOTEDFIELDS
};

/*
  Constants and variables related to input using the Mouse
*/

typedef struct mouseInputInfo
{
	int       op;
	int       windno;
	long      n;
	double   *x;
	double   *y;
} mouseInputInfo;

enum mouseOpCode
{
	noMouseOp = 0,
	getPoint,
	getLine,
	getBox,
	MaxMousePoints = 20
};

enum mouseReturnCodes
{
	MouseNormalReturn   =  0,
	MouseNoGraphs       = -1,
	MouseInterrupt      = -2,
	MouseAbort          = -3,
	MouseRestart        = -4
};

/*
  stuff related to mvFindFile
*/

enum findFileCodes
{
	mvReadit        = 0x00,
	mvWriteit       = 0x01,
	mvTextFile      = mvWriteit << 1,
	mvROTextFile    = mvTextFile << 1,
	mvAsciiSaveFile = mvROTextFile << 1,
	mvSaveFile      = mvAsciiSaveFile << 1,
	mvPSFile        = mvSaveFile << 1,
	mvEpsFile       = mvPSFile << 1,
	mvPictFile      = mvEpsFile << 1,
	mvTekFile       = mvPictFile << 1,
	mvRsrcFile      = mvTekFile << 1,
	anyTypes        = mvTextFile | mvAsciiSaveFile |
		mvSaveFile | mvPSFile | mvPictFile | mvTekFile | mvRsrcFile
} /*enum findFileCodes*/;

#endif /*MVIOH__*/
