/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.13 or later
*(C)*
*(C)* Copyright (c) 2003 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/


#ifdef SEGMENTED
#if defined(MPW) || defined(MW_CW)
#pragma segment Myplot
#endif /*MPW||MW_CW*/
#endif /*SEGMENTED*/

/*
   960312 Replaced all explicit usages of form (*xxx)->yyy by suitable
   macro YYY(xxx), adding new definitions to plot.h

   960612 Found and removed memory leak on showplot() (an unnecessary
   whole_graph was allocated by not deleted).
   Also changed code so that data being plotted is not duplicated by
   showplot().

   970105 added support for keyword height and width

   970513 allow structure argument to have more than 2 components, with
   all but the first 2 completely ignored.

   970515 moved code out of myplot() into setTicks()

   970520 Reorganized some of myplot(), fixing some minor bugs
          plot(graph,x,y), chplot(graph,x,y,c) and lineplot(graph,x,y)
          imply the use of add:T when graph is a GRAPH variable

   970522 moved code out of buildPlot() into setGraphStrings()

   971203 added check for undefined arguments (missing arguments may be
          OK because of the use of $K in macros)
   971215 changed code to use new structure plotKeyValues in calls to
          getPlotKeys(), etc.
   980618 lineplot() with lines:F is treated as plot()
   980716 Strings to be drawn by addstrings must be no longer than
          MAXCHARITEMSIZE (200) characters
   980724 Implemented notes:charVector
   990211 Replaced putOUTSTR() by putErrorOUTSTR() and substituted
          putOutErrorMsg() for myerrorout().
   990501 disabled fplot(), fchplot(), flineplot().  They will be replaced
          by macros in MacAnova.mac
   990507 Implemented keys:structure(keyword phrases) with the plotting commands
          in such a way that, say, plot(x,y,xmin:3,ymin:5,dumb:T) is
		  equivalent to plot(x,y,keys:structure(xmin:3,ymin:5,dumb:T))

          Implemented new keywords 'x', 'y', 'symbols' and 'strings'.  For
          example, plot(x:x,y:y,strings:charVec) and
          plot(keys:structure(x:x,y:y,strings:charVec) is equivalent to
		  addstrings(x,y,charVec) and plot(x,y,symbols:ch,lines:T) and
		  plot(x,y,keys:structure(symbols:ch,lines:T)) and
          plot(keys:structure(x:x,y:y,symbols:ch,lines:T)) are equivalent to
          chplot(x,y,ch,lines:T)

   990522 Implemented assignment to GRAPHWINDOWS
          Now includes new header mvGraphs.h
   990806 Implemented 'logx:T' and 'logy:T' which specify log scale
   010327 Replaced all instances of (Symbolhandle) 0 by SymbolZero,
          SymbolError or EmptyArgument
   010727 Changed ISLINKED to isLinked
   030715 Added Carapace define
*/

#include "globals.h" /* brings matProto.h which brings plot.h*/

#ifdef WXWIN
#include "wx/wxIface.h"
#endif /*WXWIN*/

#if defined(SKEL_CARAPACE)
#include "cpcglue/cpcIface.h"
#endif /*CARAPACE*/

#ifdef MACINTOSH
#include "macIface.h"
#endif /*MACINTOSH*/

#include "mvGraphs.h"

#ifdef MPW
#include <Memory.h>
#endif

enum plotScratch
{
	GGRAPH = 0,
	GPLOTC = GGRAPH,
	NTRASH
};

#define trash NAMEFORTRASH

enum myPlotCodes
{
	NEEDXY         = 0x0001,
	/* Characteristics of plot */
	PTPLOT         = 0x0002,
	CHARPLOT       = 0x0004,
	LINEPLOT       = 0x0008,
	STRINGPLOT     = 0x0010,
	PLOTTYPEMASK   = PTPLOT | CHARPLOT | LINEPLOT,
	/* characteristics of actions*/
	TOFILE         = 0x0020,
	SHOW           = 0x0040,
	ADDTOPLOT      = 0x0080,
	WINDOWASSIGN   = 0x0100,
	/* function codes */
	IPLOT          = (PTPLOT | NEEDXY),
	ICHPLOT        = (CHARPLOT | NEEDXY),
	ILINEPLOT      = (LINEPLOT | NEEDXY),
	ISTRINGPLOT    = (STRINGPLOT | NEEDXY),
	IADDPOINTS     = (IPLOT | ADDTOPLOT),
	IADDCHARS      = (ICHPLOT | ADDTOPLOT),
	IADDLINES      = (ILINEPLOT | ADDTOPLOT),
	IADDSTRINGS    = (ISTRINGPLOT | ADDTOPLOT),
	ISHOWPLOT      = (SHOW),
#if (0) /* Disabled 990501*/
	IFPLOT         = (IPLOT | TOFILE),
	IFCHPLOT       = (ICHPLOT | TOFILE),
	IFLINEPLOT     = (ILINEPLOT | TOFILE),
#endif /*0*/
	IWINDOWASSIGN  = WINDOWASSIGN
} /*enum myPlotCodes*/;

typedef struct opEntry
{
	char *name;
	short iop;
} opEntry;

static opEntry Ops[] =
{/* name        iop  */
	{"plot",         IPLOT},
	{"chplot",       ICHPLOT},
	{"lineplot",     ILINEPLOT},
	{"stringplot",   ISTRINGPLOT},
	{"addpoints",    IADDPOINTS},
	{"addchars",     IADDCHARS},
	{"addlines",     IADDLINES},
	{"addstrings",   IADDSTRINGS},
	{"showplot",     ISHOWPLOT},
#if (0) /*990501 removed following 3 functions */
	{"fplot",        IFPLOT},
	{"fchplot",      IFCHPLOT},
	{"flineplot",    IFLINEPLOT},
#endif /*0*/
	{"windowassign", IWINDOWASSIGN},
	{(char *) 0,  0}
} ;


#define generateChars (cmax < 0)

enum fineExtremeCodes
{
	FINDXMIN    = 0x01,
	FINDXMAX    = 0x02,
	FINDYMIN    = 0x04,
	FINDYMAX    = 0x08
};

/*
  970522 setGraphStrings() added with code taken from buildPlot()
         It implements addstrings()
*/
static int setGraphStrings(double **x, double **y, long n,
						   Symbolhandle stringsymbol, whole_graph **graph,
						   long justify)
{
	long             cplace = 0;
	long             i, length;
	long             nstrings;
	double           xi, yi;
	plot_string    **this_string = (plot_string**) 0;
	plot_string    **last_string = (plot_string**) 0;
	WHERE("setGraphStrings");

	nstrings = GRAPHNSTRINGS(graph);
	last_string = GRAPHFIRST_STRING(graph);
	for (i = 1;i < nstrings;i++)
	{/* find final struct plot_string */
		last_string = NEXTSTRING(last_string);
	}

	for (i = 0;i < n;i++)
	{
		xi = (*x)[i];
		yi = (*y)[i];
		length = strlen(STRINGPTR(stringsymbol) + cplace);
		if (length > 0 && !isMissing(xi) && !isMissing(yi))
		{
			TMPHANDLE = mygethandle(sizeof(plot_string));
			this_string = (plot_string **) TMPHANDLE;
			if (nstrings == 0)
			{
				GRAPHFIRST_STRING(graph) = this_string;
			}
			else
			{
				NEXTSTRING(last_string) = this_string;
			}
			if (TMPHANDLE == (char **) 0)
			{
				goto errorExit;
			}
			NEXTSTRING(this_string) = (plot_string **) 0;
			PLOTSTRING(this_string) = (char **) 0;

			GRAPHNSTRINGS(graph) = ++nstrings;
			STRINGX(this_string) = xi;
			STRINGY(this_string) = yi;
			last_string = this_string;

			TMPHANDLE = mygethandle(length + 1);
			PLOTSTRING(this_string) = TMPHANDLE;
			if (TMPHANDLE == (char **) 0)
			{
				goto errorExit;
			}
			strcpy(*TMPHANDLE,STRINGPTR(stringsymbol) + cplace);
			JUSTIFYSTR(this_string) = justify;
#if (1) /* 980714 added in anticipation of having color and styles */
			STRINGCOLOR(this_string) = DEFAULTSTRINGCOLOR;
			STRINGSTYLE(this_string) = PLAIN;
#else /*1*/
			STRINGCOLOR(this_string) = 0;
			STRINGSTYLE(this_string) = 0;
#endif /*1*/
		} /*if (length > 0 && !isMissing(xi) && !isMissing(yi))*/
		cplace += length + 1;
	} /*for (i = 0;i < n;i++)*/
	return (1);

  errorExit:
	return (0);
} /*setGraphStrings()*/

/*
  970522 added setGraphPoints() with code from buildPlot()
*/

static int setGraphPoints(int charplot,
						  double **x, double **y, Symbolhandle symhc,
						  double x0, double incx, long nrows, long ncols,
						  long cRows, long cCols, long nchars, long cmax,
						  whole_graph **graph, long style, long linetype,
						  long add)
{
	double           xi, yi;
	long             nplots;
	long             i, iplot, j, ij;
	long             cplace, cstart;
	int              xisIndex = !isMissing(x0);
	curve_points   **this_plot = (curve_points **) 0;
	curve_points   **last_plot = (curve_points **) 0;
	unsigned char    pointColor = DEFAULTPOINTCOLOR;
	unsigned char    lineColor = DEFAULTLINECOLOR;
	char           **plotcH = (char **) 0;
	WHERE("setGraphPoints");
	TRASH(NTRASH, errorExit);

	if (charplot)
	{
		if (generateChars || TYPE(symhc) == REAL)
		{
			long        nch = nDigits(labs(cmax)), cval;
			char       *plotcP;

			if (!getScratch(plotcH,GPLOTC,nchars*(nch+1),char))
			{
				goto errorExit;
			}
			plotcP = *plotcH;
			for (i = 0;i < nchars;i++)
			{
				cval = (generateChars) ?
					(i+1) % 1000 : (long) DATAVALUE(symhc,i);
				sprintf(OUTSTR,"%ld", cval);
				plotcP = copyStrings(OUTSTR, plotcP, 1);
			} /*for (i = 0;i < nchars;i++)*/
			*OUTSTR = '\0';
		}
		else
		{
			plotcH = STRING(symhc);
		}
	} /*if (charplot)*/

	if (add)
	{/* use existing struct whole_graph */
		nplots = GRAPHNPLOTS(graph);
		last_plot = GRAPHFIRST_PLOT(graph);
		for (i = 1;i < nplots;i++)
		{/* find final struct curve_points */
			last_plot = NEXTGRAPH(last_plot);
		} /*for (i = 1;i < nplots;i++)*/
	} /*if (add)*/
	else
	{
		nplots = 0;
	}

/* loop over columns of y, adding new struct curve_point for each col*/
	cstart = 0;
	for (iplot = 0;iplot < ncols; iplot++)
	{
		TMPHANDLE = mygethandle(sizeof(curve_points));
		this_plot = (curve_points **) TMPHANDLE;
		if (nplots == 0)
		{
			GRAPHFIRST_PLOT(graph) = this_plot;
		}
		else
		{
			NEXTGRAPH(last_plot) = this_plot;
		}
		if (TMPHANDLE == (char **) 0)
		{
			goto errorExit;
		}
		mylockhandle(TMPHANDLE);

		last_plot = this_plot;
		nplots++;
		GRAPHNPLOTS(graph) = nplots;
		NEXTGRAPH(this_plot) = (curve_points **) 0;
		GRAPHPOINTS(this_plot) = (coordinate **) 0;

		TMPHANDLE = mygethandle(nrows * sizeof(coordinate));
		GRAPHPOINTS(this_plot) = (coordinate **) TMPHANDLE;
		if (TMPHANDLE == (char **) 0)
		{
			goto errorExit;
		}
		mylockhandle(TMPHANDLE);

		PLOTSTYLE(this_plot) = style;

		POINTTYPE(this_plot) = (iplot == 0) ? 5 : iplot-1;
		LINETYPE(this_plot) = iplot + linetype;

		PCOUNT(this_plot) = nrows;

		if (charplot)
		{
			cstart = (cCols > 1 && iplot > 0) ?
				skipStrings(*plotcH+cstart,cRows) - *plotcH  : 0;
			cplace = cstart;
			j = 0;
		} /*if (charplot)*/

		ij = iplot*nrows;		/* iplot*nrows + i */
		for (i = 0; i < nrows; i++, ij++)
		{
			xi = (xisIndex) ? x0 + incx * (double) i : (*x)[i];
			yi = (*y)[ij];
			if (isMissing(xi) || isMissing(yi))
			{
				undef_point(this_plot, i);
			}
			else
			{
				set_point(this_plot,i,xi,yi,
						  (charplot) ? *plotcH + cplace : (char *) 0,
						  pointColor, lineColor);
			}

			if (charplot)
			{
				j++;
				/* characters are wrapped around cyclically (kb) */
				if (j >= cRows)
				{
					j = 0;
					cplace = cstart;
				}
				else
				{
					cplace = skipStrings(*plotcH + cplace,1) - *plotcH;
				}
			} /*if (charplot)*/
		} /*for (i = 0; i < nrows; i++, ij++)*/
	} /*for (iplot = 0;iplot < ncols; iplot++)*/
	return (1);

  errorExit:
	emptyTrash();
	return(0);

} /*setGraphPoints()*/

/*
  000707 initial values of extremes[] are now assumed to be set
         on entry
*/
static int compExtremes(Symbolhandle symhx, Symbolhandle  symhy, 
						whole_graph ** graph, int xisIndex,
						double extremes [], unsigned long findExtremes)
{
	double               *x, *y;
	double                value;
	double                hugedbl = HUGEDBL;
	long                  notMissing = 0;
	long                  i;
	long                  findXmin = (findExtremes & FINDXMIN);
	long                  findXmax = (findExtremes & FINDXMAX);
	long                  findYmin = (findExtremes & FINDYMIN);
	long                  findYmax = (findExtremes & FINDYMAX);
	int                   negX = 0, negY = 0;
	long                  dims[2], ny, ncols, nvals;
	WHERE("compExtremes");
	
	if (symhx == SymbolZero)
	{
		getGraphExtremes(graph, extremes, 0);
	} /*if (symhx == SymbolZero)*/
	else
	{
		(void) isMatrix(symhy, dims);
		ny = dims[0];
		ncols = dims[1];
		nvals = ny*ncols;
		x = DATAPTR(symhx);

#if (0) /* initial values are now set before call */
		extremes[IXMIN] = extremes[IYMIN] = hugedbl;
		extremes[IXMAX] = extremes[IYMAX] = -hugedbl;
#endif /*0*/
		if (xisIndex)
		{
			double           incx  = (DIMVAL(symhx, 1) == 1) ? 1.0 : x[1];
			
			if (findXmin)
			{
				double       xmin = (incx > 0) ? x[0] : x[0] + (ny - 1)*incx;
				
				if (xmin < extremes[IXMIN])
				{
					extremes[IXMIN] = xmin;
				}
			}
			
			if (findXmax)
			{
				double       xmax = (incx > 0) ? x[0] + (ny - 1)*incx : x[0];

				if (xmax > extremes[IXMAX])
				{
					extremes[IXMAX] = xmax;
				}
			}
		} /*if (xisIndex)*/
		else
		{
			for (i = 0;i < ny;i++)
			{
				value = x[i];
				if (!isMissing(value))
				{
					notMissing++;

					if (findXmin && value < extremes[IXMIN])
					{
						extremes[IXMIN] = value;
					}
					if (findXmax && value > extremes[IXMAX])
					{
						extremes[IXMAX] = value;
					}
				}
			} /*for (i = 0;i < ny;i++)*/

			if (notMissing == 0)
			{
				sprintf(OUTSTR,
						"ERROR: no non-missing x values for %s() to plot",
						FUNCNAME);
				goto errorExit;
			}
		} /*if (xisIndex){}else{}*/

		y = DATAPTR(symhy);
		notMissing = 0;

		for (i = 0;i < nvals;i++)
		{
			value = y[i];
			if (!isMissing(value))
			{
				notMissing++;
			
				if (findYmin && value < extremes[IYMIN])
				{
					extremes[IYMIN] = value;
				}
				if (findYmax && value > extremes[IYMAX])
				{
					extremes[IYMAX] = value;
				}
			} /*if (!isMissing(value))*/
		} /*for (i = 0;i < nvals;i++)*/

		if (notMissing == 0)
		{
			sprintf(OUTSTR,
					"ERROR: no non-missing y values for %s() to plot",
					FUNCNAME);
			goto errorExit;
		}
	} /*if (symhx == SymbolZero){}else{}*/
	

	return (1);

  errorExit:
	putErrorOUTSTR();

	return (0);
} /*compExtremes()*/

	
/*
   Build or add components to whole_graph structure graph
   970515 modified so that it handles showplot() case, too, setting
   only extremes
*/
static long buildPlot(short op, whole_graph **graph,
					  Symbolhandle symhx, Symbolhandle symhy,
					  Symbolhandle symhc, int xisIndex,
					  long style, long cmax, long nchars,
					  long cRows, long cCols,
					  plotKeyValuesPtr keyValues)
{
	int                   ok = !(op & NEEDXY);
	WHERE("buildPlot");

	while (!ok) /* loop traversed at most once */
	{ /* not showplot() */
		double              **x = DATA(symhx), **y = DATA(symhy);
		double                x0, incx;
		long                  ny, ncols, nvals;
		long                  notMissing = 0;
		long                  dims[2];
		
		(void) isMatrix(symhy, dims);
		ny = dims[0];
		ncols = dims[1];
		nvals = ny*ncols;
		if (xisIndex)
		{
			x0 = (*x)[0];
			incx = (isScalar(symhx)) ? 1.0 : (*x)[1];
		}
		else
		{
			setMissing(x0);
		}
		
		if (!(op & STRINGPLOT))
		{
			if (!setGraphPoints((op & CHARPLOT) != 0,
								x, y, symhc, x0, incx,
								ny, ncols, cRows, cCols,
								nchars, cmax, graph, style,
								keyValues->linetype, keyValues->add))
			{
				goto errorExit;
			}
		} /*if (!(op & STRINGPLOT))*/
		else
		{
			/* addstrings()*/
			if (!setGraphStrings(x, y, ny, symhc, graph, keyValues->justify))
			{
				goto errorExit;
			}
		} /*if (!(op & STRINGPLOT)){}else{}*/
		ok = 1;
		/* fall through */

	  errorExit:
		if (!ok)
		{
			putErrorOUTSTR();
			break;
		}
	} /*while (!ok)*/

	if (ok)
	{
		GRAPHXMIN(graph) = keyValues->xmin;
		GRAPHXMAX(graph) = keyValues->xmax;
		GRAPHYMIN(graph) = keyValues->ymin;
		GRAPHYMAX(graph) = keyValues->ymax;
	}

	return (ok);
} /* buildPlot()*/

#define TICKLENGTHNOTSET (4*TICKUNIT)
/*
  970515 moved (and appropriately modified) code from myplot() to here

  Add x- (isX != 0) or y- ticks (isX == 0) to graph
  Dispose of previous ticks if necessary (we're modifying and replacing
  LASTPLOT)

  Argument nticks is ignored if ticks != (Symbolhandle) 0

*/

static void setTicks(whole_graph ** graph, Symbolhandle ticks, long nticks,
					 Symbolhandle ticklabs, unsigned short ticklength,
					 int dispose, char whichAxis, char silent)
{
	int       attachTicks;
	int       isX = whichAxis == 'x';

	if (ticks != UNSETSYMH)
	{
		nticks = symbolSize(ticks);
		attachTicks =
		  nticks > 1 || !isNull(ticks) && !isMissing(DATAVALUE(ticks, 0));

		if (isX)
		{
			if (dispose)
			{ /* graph from LASTPLOT which will be updated */
				mydisphandle((char **) GRAPHXTICKS(graph));
				GRAPHXTICKS(graph) = (double **) 0;
			}
			if (attachTicks)
			{
				GRAPHXTICKS(graph) = DATA(ticks);
			}
		} /*if (isX)*/
		else
		{
			if (dispose)
			{ /* graph from LASTPLOT which will be updated */
				mydisphandle((char **) GRAPHYTICKS(graph));
				GRAPHYTICKS(graph) = (double **) 0;
			}
			if (attachTicks)
			{
				GRAPHYTICKS(graph) = DATA(ticks);
			}
		} /*if (dispose)*/
		setDATA(ticks, (double **) 0);
	} /*if (ticks != UNSETSYMH)*/

	if (ticklabs != UNSETSYMH)
	{
		double    **tickhandle = (isX) ?
			GRAPHXTICKS(graph) : GRAPHYTICKS(graph);

		if (tickhandle == (double **) 0)
		{
			sprintf(OUTSTR,
					"WARNING: '%cticklabs' ignored without '%c' tick locations",
					whichAxis, whichAxis);
		}
		else if (symbolSize(ticklabs) != nticks)
		{
			sprintf(OUTSTR,
					"WARNING: value of '%cticklabs' has wrong length; ignored",
					whichAxis);
		}
		if (*OUTSTR)
		{
			mydisphandle(STRING(ticklabs));
			if (!silent)
			{
				putErrorOUTSTR();
			}
			else
			{
				*OUTSTR = '\0';
			}
		}
		else
		{
			if (isX)
			{
				if (dispose)
				{
					mydisphandle(GRAPHXTICKLABS(graph));
				}
				GRAPHXTICKLABS(graph) = STRING(ticklabs);
			}
			else
			{
				if (dispose)
				{
					mydisphandle(GRAPHYTICKLABS(graph));
				}
				GRAPHYTICKLABS(graph) = STRING(ticklabs);
			}
		}
		setSTRING(ticklabs, (char **) 0);
	} /*if (ticklabs != UNSETSYMH)*/

	if (ticklength == TICKLENGTHNOTSET)
	{
		ticklength = DEFAULTTICKLENGTH;
	} /*if (ticklength == TICKLENGTHNOTSET)*/

	if (isX)
	{
		GRAPHNXTICKS(graph) = nticks;
		GRAPHXTICKLENGTH(graph) = ticklength;
	} /*if (isX)*/
	else
	{
		GRAPHNYTICKS(graph) = nticks;
		GRAPHYTICKLENGTH(graph) = ticklength;
	} /*if (is(X)){}else{}*/
} /*setTicks()*/

/*
  Install information, but not data, in whole_graph handle
  990523 created using code from myplot()
*/
static void	setGraphInfo(whole_graph ** graph, plotKeyValuesPtr keyValues,
						 long nxticks, long nyticks, int dispose)
{
	setTicks(graph, keyValues->xticks, nxticks, keyValues->xticklabs,
			 keyValues->xticklength, dispose, 'x', keyValues->silent);

	setTicks(graph, keyValues->yticks, nyticks, keyValues->yticklabs,
			 keyValues->yticklength, dispose, 'y', keyValues->silent);

	GRAPHFGCOLOR(graph) = DEFAULTFGCOLOR;
	GRAPHBGCOLOR(graph) = DEFAULTBGCOLOR;
	GRAPHFRAMECOLOR(graph) = DEFAULTFRAMECOLOR;

	GRAPHLOGX(graph) = keyValues->logx;
	GRAPHLOGY(graph) = keyValues->logy;

	GRAPHXAXIS(graph) = keyValues->xaxis;
	GRAPHYAXIS(graph) = keyValues->yaxis;

	GRAPHFRAME(graph) = (short) keyValues->frame;

	strcpy(GRAPHXLAB(graph), keyValues->xlab);
	strcpy(GRAPHYLAB(graph), keyValues->ylab);
	strcpy(GRAPHTITLE(graph), keyValues->title);
} /*setGraphInfo()*/

enum setKeyDefaultsCodes
{
	beforeGetPlotKeys,
	afterGetPlotKeys
};
/*
  990517 setKeyDefaults() sets the elements of *keyValues.  It is called
         at least twice with differnt values of when
  990825 window:0 and window:1 now legal on non-windowed versions
*/
static void setKeyDefaults(long op, int assignment, plotKeyValuesPtr keyValues,
						   int when)
{
	switch (when)
	{
	  case beforeGetPlotKeys:
		/* set which keywords are allowable */
		unsetPlotKeys(keyValues, 1);

		if (assignment)
		{
			keyValues->screendump = DONTSETPOINTER;
			keyValues->epsf = DONTSETCHAR;
			keyValues->ps = DONTSETCHAR;
			keyValues->landscape = DONTSETCHAR;
			keyValues->newFile = DONTSETCHAR;
			keyValues->file = DONTSETPOINTER;
#ifdef MULTIPLOTWINDOWS
			keyValues->dumb = DONTSETCHAR;
			keyValues->height = DONTSETLONG;
			keyValues->width = DONTSETLONG;
#endif /*MULTIPLOTWINDOWS*/
		} /*if (assignment)*/
		else
		{
#ifndef SCREENDUMP
			keyValues->screendump = DONTSETPOINTER;
#endif /*SCREENDUMP*/

#ifndef EPSFPLOTSOK
			keyValues->epsf = DONTSETCHAR;
#endif /*EPSFPLOTSOK*/
		} /*if (assignment){}else{}*/

		keyValues->xticklength = keyValues->yticklength = TICKLENGTHNOTSET;
		keyValues->pause = DEFAULTPLOTPAUSE;

		if (op & SHOW)
		{
			keyValues->x = keyValues->y = DONTSETSYMH;
			keyValues->symbols = keyValues->strings = DONTSETSYMH;
			keyValues->add = DONTSETCHAR;
			keyValues->lines = DONTSETCHAR;
			keyValues->linetype = DONTSETLONG;
			keyValues->thickness = DONTSETREAL;
			keyValues->impulses = DONTSETCHAR;
			keyValues->justify = DONTSETCHAR;
		} /*if (op == ISHOWPLOT)*/
		else if (op & STRINGPLOT)
		{
			keyValues->symbols = DONTSETSYMH;
			keyValues->impulses = DONTSETCHAR;
			keyValues->lines = DONTSETCHAR;
			keyValues->linetype = DONTSETLONG;
			keyValues->thickness = DONTSETREAL;
			keyValues->justify = CENTRE;
		}
		else
		{
			keyValues->lines = (op & LINEPLOT) != 0;
			keyValues->impulses = 0;
		}
		
		keyValues->frame = UNSETTICKS | UNSETFRAME;
		break;

	  case afterGetPlotKeys:
		unsetPlotKeys(keyValues, 0); /* change DONTSETs to UNSETs */

		if (keyValues->add == UNSETCHAR)
		{
			keyValues -> add = (assignment || (op & SHOW)) ?
				0 : ((op & ADDTOPLOT) != 0);
		}
		if (keyValues->keep == UNSETCHAR)
		{
			keyValues->keep = 1;
		}
		if (keyValues->show == UNSETCHAR)
		{
			keyValues->show = 1;
		}
		if (keyValues->landscape == UNSETCHAR)
		{
			keyValues->landscape = 0;
		}
		break;
	} /*switch (when)*/
} /*setKeyDefaults()*/

/*
  990518 moved code from myplot() to this new function to set the final
         values in structure keyValues
*/
static unsigned long setKeyFinal(whole_graph ** graph,
								 plotKeyValuesPtr keyValues, int useGraph)
{
	unsigned long    findExtremes = 0;

	if (useGraph)
	{
		keyValues->xmin =
		  (isMissing(keyValues->xmin) || keyValues->xmin != UNSETREAL) ?
		  keyValues->xmin : GRAPHXMIN(graph);
	
		keyValues->xmax =
		  (isMissing(keyValues->xmax) || keyValues->xmax != UNSETREAL) ?
		  keyValues->xmax : GRAPHXMAX(graph);
	
		keyValues->ymin =
		  (isMissing(keyValues->ymin) || keyValues->ymin != UNSETREAL) ?
		  keyValues->ymin : GRAPHYMIN(graph);
	
		keyValues->ymax =
		  (isMissing(keyValues->ymax) || keyValues->ymax != UNSETREAL) ?
		  keyValues->ymax : GRAPHYMAX(graph);
	
		if (isMissing(keyValues->xmin) ||
			!isMissing(keyValues->xmax) && keyValues->xmin >= keyValues->xmax)
		{
			findExtremes |= FINDXMIN;
		}
	
		if (isMissing(keyValues->xmax) ||
			!isMissing(keyValues->xmin) && keyValues->xmin >= keyValues->xmax)
		{
			findExtremes |= FINDXMAX;
		}
	
		if (isMissing(keyValues->ymin) ||
			!isMissing(keyValues->ymax) && keyValues->ymin >= keyValues->ymax)
		{
			findExtremes |= FINDYMIN;
		}
	
		if (isMissing(keyValues->ymax) ||
			!isMissing(keyValues->ymin) && keyValues->ymin >= keyValues->ymax)
		{
			findExtremes |= FINDYMAX;
		}
	
		if (findExtremes & (FINDXMIN | FINDXMAX | FINDYMIN | FINDYMAX))
		{
			double           extremes[4];

			getGraphExtremes(graph, extremes, 1);
	
			keyValues->xmin = (findExtremes & FINDXMIN) ?
			  extremes[IXMIN] : keyValues->xmin;
			keyValues->xmax = (findExtremes & FINDXMAX) ?
			  extremes[IXMAX] : keyValues->xmax;
			keyValues->ymin = (findExtremes & FINDYMIN) ?
			  extremes[IYMIN] : keyValues->ymin;
			keyValues->ymax = (findExtremes & FINDYMAX) ?
			  extremes[IYMAX] : keyValues->ymax;
		} /*if (findExtremes & (FINDXMIN | FINDXMAX | FINDYMIN | FINDYMIN))*/

		keyValues->xticklength =
		  (keyValues->xticklength != TICKLENGTHNOTSET) ?
		  keyValues->xticklength : GRAPHXTICKLENGTH(graph);
		keyValues->yticklength =
		  (keyValues->yticklength != TICKLENGTHNOTSET) ?
		  keyValues->yticklength : GRAPHYTICKLENGTH(graph);

		keyValues->logx = (keyValues->logx != UNSETCHAR) ?
		  keyValues->logx : GRAPHLOGX(graph);
		keyValues->logy = (keyValues->logy != UNSETCHAR) ?
		  keyValues->logy : GRAPHLOGY(graph);

		keyValues->xaxis = (keyValues->xaxis != UNSETCHAR) ?
		  keyValues->xaxis : GRAPHXAXIS(graph);
		keyValues->yaxis = (keyValues->yaxis != UNSETCHAR) ?
		  keyValues->yaxis : GRAPHYAXIS(graph);

		if ((keyValues->frame & UNSETFRAME) &&
			(keyValues->frame & UNSETTICKS))
		{
			/* neither border or ticks keywords; use values from graph*/
			keyValues->frame = (long) GRAPHFRAME(graph);
		}
		else if (keyValues->frame & UNSETFRAME)
		{
			/* ticks but not border; take border from graph */
			keyValues->frame = (keyValues->frame & TICKSALLSIDES) |
			  ((long) GRAPHFRAME(graph) & FRAMEALLSIDES);
		}
		else if (keyValues->frame & UNSETTICKS)
		{
			/* border but not ticks; set ticks like border*/
			keyValues->frame = (keyValues->frame & FRAMEALLSIDES) |
			  (TICKSFACTOR * (keyValues->frame & FRAMEALLSIDES));
		}

		if (keyValues->title[0] == '\0')
		{
			strcpy(keyValues->title, GRAPHTITLE(graph));
		}
		if (keyValues->xlab[0] == '\0')
		{
			strcpy(keyValues->xlab, GRAPHXLAB(graph));
		}
		if (keyValues->ylab[0] == '\0')
		{
			strcpy(keyValues->ylab, GRAPHYLAB(graph));
		}
	} /*if (useGraph)*/
	else
	{
		double                hugedbl = HUGEDBL;

		if (isMissing(keyValues->xmin) || keyValues->xmin == UNSETREAL)
		{
			findExtremes |= FINDXMIN;
		}

		if (isMissing(keyValues->xmax) || keyValues->xmax == UNSETREAL)
		{
			findExtremes |= FINDXMAX;
		}

		if (isMissing(keyValues->ymin) || keyValues->ymin == UNSETREAL)
		{
			findExtremes |= FINDYMIN;
		}

		if (isMissing(keyValues->ymax) || keyValues->ymax == UNSETREAL)
		{
			findExtremes |= FINDYMAX;
		}

		if (!(findExtremes & (FINDXMIN | FINDXMAX)) &&
			keyValues->xmin >= keyValues->xmax)
		{
			findExtremes |= FINDXMIN | FINDXMAX;
		}

		if (!(findExtremes & (FINDYMIN | FINDYMAX)) &&
			keyValues->ymin >= keyValues->ymax)
		{
			findExtremes |= FINDYMIN | FINDYMAX;
		}

		keyValues->xmin = (findExtremes & FINDXMIN) ? hugedbl : keyValues->xmin;
		keyValues->xmax = (findExtremes & FINDXMAX) ? -hugedbl : keyValues->xmax;
		keyValues->ymin = (findExtremes & FINDYMIN) ? hugedbl : keyValues->ymin;
		keyValues->ymax = (findExtremes & FINDYMAX) ? -hugedbl : keyValues->ymax;

		if ((keyValues->frame & UNSETFRAME) && (keyValues->frame & UNSETTICKS))
		{
			/* neither border or ticks keywords; use values from graph*/
			keyValues->frame = FRAMEALLSIDES | TICKSALLSIDES;
		}
		else if (keyValues->frame & UNSETFRAME)
		{
			/* ticks but not border; use complete border */
			keyValues->frame =
			  (keyValues->frame & TICKSALLSIDES) | FRAMEALLSIDES;
		}
		else if (keyValues->frame & UNSETTICKS)
		{
			/* border but not ticks; set ticks like border*/
			keyValues->frame = (keyValues->frame & FRAMEALLSIDES) |
			  (TICKSFACTOR * (keyValues->frame & FRAMEALLSIDES));
		}
	} /*if (useGraph){}else{}*/

	keyValues->logx = (keyValues->logx != UNSETCHAR) ? keyValues->logx : 0;
	keyValues->logy = (keyValues->logy != UNSETCHAR) ? keyValues->logy : 0;
	keyValues->xaxis = (keyValues->xaxis != UNSETCHAR) ? keyValues->xaxis : 1;
	keyValues->yaxis = (keyValues->yaxis != UNSETCHAR) ? keyValues->yaxis : 1;

	return (findExtremes);
} /*setKeyFinal()*/

/*
  Check symhx, symhy, retrieve NAME(symhx) and NAME(symhy) if appropriate
  and determine dimensioning information
*/
static int checkGraphData(Symbolhandle symhx, Symbolhandle symhy,
						short op, int keyX, int strucArg,
						plotKeyValuesPtr keyValues,
						long * nx, long * ny, long * ncols)
{
	long              dims[2];
	char              name[NAMELENGTH + 1], *keyword;
	char             *whatx = (strucArg) ? "component 1 of structure" : "x";
	char             *whaty = (strucArg) ? "component 2 of structure" : "y";

	if (TYPE(symhx) != REAL && TYPE(symhx) != NULLSYM ||
		TYPE(symhx) == REAL && !isVector(symhx))
	{
		sprintf(OUTSTR,
				"ERROR: %s must be NULL or a REAL vector for %s()",
				whatx, FUNCNAME);
	}
	else if (TYPE(symhy) != REAL && TYPE(symhy) != NULLSYM ||
			 TYPE(symhy) == REAL && (!isMatrix(symhy, dims) ||
									 (op & STRINGPLOT) && dims[1] > 1))
	{
		sprintf(OUTSTR,
				"ERROR: %s must be NULL or a REAL vector%s for %s()",
				whaty, (op & STRINGPLOT) ? NullString : " or matrix",
				FUNCNAME);
	}
	else if (!keyX && !keyValues->add)
	{
		/*
			When data is not supplied through keywords 'x' and 'y'
			in a structure, use names or keywords as axis labels only
			when not adding info and when they have not been set by
			keywords 'xlab' & 'ylab'
		*/
		if (keyValues->xlab[0] == '\0')
		{
			strcpy(name, NAME(symhx));
			keyword = isKeyword(symhx);
			if (keyword)
			{
				strncpy(keyValues->xlab, keyword, XLABELSIZE);
			}
			else if (!isscratch(name) && strcmp(name, NUMSCRATCH+2) != 0 &&
					 strcmp(name, VECSCRATCH+2) != 0 &&
					 strcmp(name, MATSCRATCH+2) != 0)
			{
				strncpy(keyValues->xlab, name + ((istempname(name)) ? 1 : 0),
						XLABELSIZE);
			}
			keyValues->xlab[XLABELSIZE] = '\0';
		} /*if (keyValues->xlab[0] == '\0')*/

		if (keyValues->ylab[0] == '\0')
		{
			strcpy(name, NAME(symhy));

			keyword = isKeyword(symhy);
			if (keyword)
			{
				strncpy(keyValues->ylab, keyword, YLABELSIZE);
			}
			else if (!isscratch(name) && strcmp(name, NUMSCRATCH+2) != 0 &&
					 strcmp(name, VECSCRATCH+2) != 0 &&
					 strcmp(name, MATSCRATCH+2) != 0)
			{
				strncpy(keyValues->ylab, name + ((istempname(name)) ? 1 : 0),
						YLABELSIZE);
			}
			keyValues->ylab[YLABELSIZE] = '\0';
		} /*if (keyValues->ylab[0] == '\0')*/
	} /*if (!keyValues->add)*/

	if (*OUTSTR == '\0' && TYPE(symhx) != NULLSYM && TYPE(symhy) != NULLSYM)
	{
		*nx = symbolSize(symhx);
		*ny = dims[0];
		*ncols = dims[1];

		if (*nx != *ny)
		{
			if (op & STRINGPLOT || *nx > 2)
			{
				sprintf(OUTSTR,
						"ERROR: %s and %s do not have same number of rows for %s()",
						whatx, whaty, FUNCNAME);
			}
			else if (anyMissing(symhx))
			{
				sprintf(OUTSTR,
						"ERROR: %s to be expanded for %s() has MISSING values",
						whatx, FUNCNAME);
			}
		} /*if (*nx != *ny)*/
	} /*if (*OUTSTR == '\0')*/

	return (*OUTSTR == '\0');
} /*checkGraphData()*/

#ifdef EPX
static void outwitGcc(int i)
{
	;
} /*outwitGcc()*/
#endif /*EPX*/

/* for debugging purposes*/
#ifdef DUMPPLOTKEYS
void dumpPlotKeys(char * /*where*/, char * /*msg*/,
				  plotKeyValuesPtr /*keyValues*/);
#endif /*DUMPPLOTKEYS*/
/*
   General entry for plot(), lineplot(), chplot(), fplot(), flineplot(),
   fchplot(), addpoints(), addlines(), addchars(), addstrings()

   960503 changed macOpen() to macFindFile()
   960522 added capability to save PICT and EPSF files on mac
   960524 Default value for window is 0 (use last window) with add:T or any of
          addlines(), addpoints(), addchars(), addstrings().
   990517 Moved various pieces of code to static functions setKeyDefaults()
          and setKeyFinal()
          Modified code to implement GRAPHWINDOW[i] <- var which results
          to a call to myplot() with "windowassign" as FUNCNAME 
   990522 Added mvGraph.h
   991028 Fixed bug; now elements of numerical vector of symbols is checked
          for MISSING
   000524 Changed keyValues to keyValuesData and added pointer
          keyValues = &keyValuesData
		  All WARNING messages are protected by if (!keyValues->silent)
          to implement 'silent:T' as a keyword phrase.
   000705 It is no longer an error for x or y to be NULL; 
   000707 Corrected bug; with xmin:?, etc., when adding to a plot, the
          computed extreme did not take into account the extreme in the
          existing GRAPH symbol; also fixed bug when x is vector(x0,inc)
          with inc < 0
   010327 Fixed problem when first argument was empty as happened in a macro
          with something like showplot($K,xlab:"x") when there were no keywords
   010516 Use atLeastNArgs constant on badNargs() call
   010522 Changed call to macSetInfo()
   010523 Replaced macSetInfo() by macSetFInfo()
   010524 Changed call to findPlotFile() abd openPlotFile()
          Moved setting of Macintosh creator and type to openPlotFile()
		  in plotutil.c
   010525 Changed references to PlotVolume and PlotDirectory to use
          accessor macros
   010709 Fixed bug so that, say, chplot(1,runi(10)) has points labeled 1 - 10
*/

#undef PLACE
#define PLACE(I) ALERT("Place %g", (double) I, 0,0,0)

Symbolhandle    myplot(Symbolhandle list)
{
	Symbolhandle          symhx = SymbolZero;
	Symbolhandle          symhy = SymbolZero;
	Symbolhandle          symhsymbols = SymbolZero;
	Symbolhandle          symhfile = SymbolZero;
	Symbolhandle          argi;
	Symbolhandle          graphsymbol = SymbolZero;
	Symbolhandle          lastplot = SymbolZero;
	Symbolhandle          oldplot;
	Symbolhandle          symhstrings = SymbolZero;
	Symbolhandle          trash = SymbolZero;
	Symbolhandle          symhKeys = SymbolZero;
	Symbolhandle          graphWindows = GRAPHWINDOWSSYMBOL;
	Symbolhandle          graphWindowsComp = SymbolZero;
	whole_graph         **graph = (whole_graph **) 0;
	plotKeyValues         keyValuesData;
	plotKeyValuesPtr      keyValues = &keyValuesData;
	char                  filePath[PATHSIZE+1];
	FILE                 *plotfile = STDOUT;
	long                  termType = STANDARDTERM;
	long                  nargs, startArg;
	long                  startKey = -1;
	long                  graphArg;
	long                  i, nchars = 0, margs;
	long                  dims[2], nx, ny, cRows, cCols;
	long                  ncols = 0;
	long                  style = POINTS;
	long                  nxticks, nyticks;
	long                  windno, graphWindowsIndex = -1;
	long                  ngraphs = getNgraphs();
	int                   usePreviousGraph, useLASTPLOT;
	int                   xisIndex = 0, strucArg = 0, toFile = 0, showPlot;
	int                   keyX = 0, keyY = 0, keyStrings = 0, keySymbols = 0;
	int                   haveData = 0, haveSymbols = 0, haveStrings = 0;
	int                   assignment;
	short                 op;
	unsigned long         findExtremes;
	double                cmax = 0.0;
	char                 *keyword;
	char                 *keyStructureName = "keys";
	char                 *keyXname = "x", *keyYname = "y";
	char                 *keySymbolsName = "symbols";
	char                 *keyStringsName = "strings";
	WHERE("myplot");

	OUTSTR[0] = '\0';

	/* Which function initiated the call? */
	for (i = 0; Ops[i].name != (char *) 0;i++)
	{
		if (strcmp(FUNCNAME,Ops[i].name) == 0)
		{
			op = Ops[i].iop;
			break;
		}
	} /*for (i = 0; Ops[i].name != (char *) 0;i++)*/

	if (Ops[i].name == (char *) 0)
	{ /* should never happen */
		sprintf(OUTSTR,"ERROR: (internal) %s not recognized in myplot()",
				FUNCNAME);
		goto errorExit;
	}

	showPlot = (op & SHOW) != 0;
	assignment = (op & WINDOWASSIGN) != 0;

	lastplot = Lookup(LastGraphName); /*find LASTPLOT */

	if (lastplot != SymbolZero && TYPE(lastplot) != PLOTINFO)
	{
		/* Somebody is fooling around */
		lastplot = SymbolZero;
	}

#ifdef EPX
/*
  The following was added to outflank a possible gcc compiler bug
  without it, showPlot turned up as 1 later on even when it was 0 here
*/
	outwitGcc(showPlot);
#endif /*EPX*/

	nargs = NARGS(list);

	if (COMPVALUE(list, 0) == EmptyArgument)
	{
		if (nargs == 1)
		{
			nargs = 0;
		}
		else
		{
			noData(FUNCNAME, 1);
			goto errorExit;
		}
	} /*if (COMPVALUE(list, 0) == EmptyArgument)*/
	
	if (assignment)
	{
		graphArg = 1;
		graphWindowsComp = COMPVALUE(list, 0);

		/* Which window; a match should be certain */
		graphWindowsIndex = whichGraphWindowsComponent(graphWindowsComp);

		/*
		  Note: graphWindowsComp is either a GRAPH variable or
		        a NULL variable
		*/
		startArg = 1; /* index of 1st unprocessed argument, if any */
		if (nargs == 1)
		{
			/*
				Must be GRAPHWINDOWS[i] <- GraphVar
				Essentially equivalent to showplot(GraphVar)
			*/
			op |= SHOW;
			showPlot = 1;
		} /*if (nargs == 1)*/
		oldplot = (TYPE(graphWindowsComp) == PLOTINFO) ?
			graphWindowsComp : SymbolZero;
	} /*if (assignment)*/
	else
	{
		/* An ordinary plotting command() */
		if (nargs == 0 && !showPlot)
		{
			badNargs(FUNCNAME, -1 + atLeastNArgs);
			goto errorExit;
		} /*if (nargs == 0 && !showPlot)*/

		if (nargs > 0)
		{
			/* Is argument 1 a GRAPH variable?*/
			oldplot = COMPVALUE(list, 0);
			if (TYPE(oldplot) == PLOTINFO)
			{
				graphArg = 1;
				startArg = 1;
			}
			else
			{
				oldplot = SymbolZero;
				graphArg = 0; /* no GRAPH argument */
				startArg = 0;
			}
		} /*if (nargs > 0)*/
		else
		{
			oldplot = SymbolZero;
			graphArg = 0; /* no GRAPH argument */
			startArg = 0;
		}

		if ((showPlot || op & ADDTOPLOT) && oldplot == SymbolZero)
		{
			if (lastplot == SymbolZero)
			{
				sprintf(OUTSTR,
						"ERROR: no GRAPH variable argument to %s() and no GRAPH variable %s",
						FUNCNAME, LastGraphName);
				goto errorExit;
			}
			oldplot = lastplot;
		} /*if ((showPlot || op & ADDTOPLOT) && oldplot == SymbolZero)*/ 
	} /*if (assignment){}else{}*/

	/*
	  For showplot(), addchars(), addpoints() and addstrings() 
	  oldplot is the graph variable they are displaying or modifying.
	  This may be from the first argument or LASTPLOT
	*/
	margs = nargs - startArg; /* number of unprocessed arguments */

	if (!assignment && margs > 0 && op != ISHOWPLOT)
	{
		/*
			Look at the next 1, 2 or 3 arguments for data and possibly
			symbols or strings not specified by symbols:c or strings:s.

			Data can be of the form x, y or structure(x,y,...)
		*/
		Symbolhandle           symhstruc = COMPVALUE(list, startArg);

		/*
		  Is next argument structure(x,y,...) and not part of a keyword
		  phrase?
		*/
		if (isStruc(symhstruc) && !isKeyword(symhstruc))
		{
			startArg++;
			strucArg = 1;
			symhx = COMPVALUE(symhstruc, 0);
			symhy = COMPVALUE(symhstruc, 1);

			if (NCOMPS(symhstruc) < 2 ||
				symhx == EmptyArgument || symhy == EmptyArgument ||
				(TYPE(symhx) != REAL && TYPE(symhx) != NULLSYM) ||
				(TYPE(symhy) != REAL && TYPE(symhy) != NULLSYM))
			{
				sprintf(OUTSTR,
						"ERROR: First 2 components of structure arg. to %s() must be REAL or NULL",
						FUNCNAME);
				goto errorExit;
			}
			haveData = 1;
		} /*if (isStruc(symhstruc) && !isKeyword(symhstruc))*/
		else if (margs > 1)
		{
			/*
			  At least two unprocessed arguments; treat as x, y
			  If they are keyword phrases, keywords are used as x or
			  y axis labels unless either keyword is "keys"
			*/
			symhx = COMPVALUE(list, startArg);
			symhy = COMPVALUE(list, startArg + 1);

			if ((keyword = isKeyword(symhx)) &&
				strncmp(keyword, keyStructureName, 3) == 0 ||
				(keyword = isKeyword(symhy)) &&
				strncmp(keyword, keyStructureName, 3) == 0)
			{
				/* one of 1st two args is keys:var */
				symhx = symhy = SymbolZero;
			}
			else
			{
				char    *badxory = "argument %ld to %s() must be REAL or NULL";
				
				if (!argOK(symhx, NULLSYM, startArg + 1) ||
					!argOK(symhy, NULLSYM, startArg + 2))
				{
					goto errorExit;
				}
				if (TYPE(symhx) != REAL && TYPE(symhx) != NULLSYM)
				{
					sprintf(OUTSTR, badxory, startArg + 1, FUNCNAME);
					goto errorExit;
				}
				if (TYPE(symhy) != REAL && TYPE(symhy) != NULLSYM)
				{
					sprintf(OUTSTR, badxory, startArg + 2, FUNCNAME);
					goto errorExit;
				}
				
				startArg += 2;
				haveData = 1;
			} /*if (isStruc(symhstruc) && !isKeyword(symhstruc)){}else if (margs > 1){}*/
		}

		/*
		  Are non-keyword symbols or strings supplied?
		*/
		if (startArg < nargs && !isKeyword(COMPVALUE(list, startArg)) &&
			COMPVALUE(list, startArg) != EmptyArgument)
		{
			if (op & STRINGPLOT)
			{
				symhstrings = COMPVALUE(list, startArg++);

				if (!argOK(symhstrings, CHAR, startArg))
				{
					goto errorExit;
				}
				margs--;
				haveStrings = 1;
			} /*if (op & STRINGPLOT)*/
			else if (op & CHARPLOT)
			{
				symhsymbols = COMPVALUE(list, startArg++);
				if (!argOK(symhsymbols, 0, startArg))
				{
					goto errorExit;
				}
				if (TYPE(symhsymbols) != REAL && TYPE(symhsymbols) != CHAR)
				{
					sprintf(OUTSTR,
							"ERROR: plotting symbols for %s() not CHARACTER or REAL",
							FUNCNAME);
				}
				else
				{
					margs--;
					haveSymbols = 1;
				}
			} /*if (op & STRINGPLOT){}else if (op & CHARPLOT){}*/
			else
			{
				sprintf(OUTSTR,
						"ERROR: %s() has a non-keyword argument that is not data",
						FUNCNAME);
			}
			if (*OUTSTR)
			{
				goto errorExit;
			}
		} /*if (startArg < nargs && !isKeyword(COMPVALUE(list, startArg)))*/
	} /*if (!assignment && margs > 0 && op != ISHOWPLOT)*/

	/*
	  At this point, if x and y have been supplied, symhx and symhy have
	  been set and are known to be REAL or NULL and haveData != 0

	  If plotting symbols have been provided for chplot() as a non keyword
	  argument, symhsymbols has been set and is known to be REAL or CHAR  and
	  haveSymbols != 0

	  If strings have been provided for addstrings() as a non-keyword argument,
	  symhstrings has been set and is known to be CHAR and haveStrings != 0

	  There should be no later non-keyword arguments except possibly for
	  (Symbolhandle) 0 arguments.

	  startArg is the index of the next argument to be processed

	  margs is the number of non-processed arguments

	  graphArg != 0 signals assignment or that the first argument of
	  plot(), lineplot(), ..., addxxx() is a graph variable.

	  oldplot != 0 means that a PLOTINFO variable was first argument
	  or that LASTPLOT exists and the command was showplot() or
	  addxxxx().  In the latter case, oldplot is LASTPLOT

	  In GRAPHWINDOWS[i] <- var, oldplot will be (Symbolhandle) 0
	  only when var is a structure and GRAPHWINDOWS[i] is NULL.
	*/


	if (!assignment)
	{
		/*
		  Scan remaining argument list for undefined arguments and keywords
		  It is an error for any argument to be undefined or for any argument
		  that is not missing ((Symbolhandle) 0) to not be a keyword phrase
	
		  If keyword "keys" is used, it must be the last argument and
		  no other keyword phrases can be used.
	
		  We specifically look for keywords 'x', 'y', 'symbols' and 'strings'
		  setting flags keyX, keyY, keySymbols and keyStrings if found
		*/
		for (i = startArg; i < nargs; i++)
		{
			argi = COMPVALUE(list, i);

			if (argi != EmptyArgument && !isDefined(argi))
			{
				undefArg(FUNCNAME, argi, i+1);
				goto errorExit;
			}

			keyword = isKeyword(argi);
			if (keyword)
			{
				if (strcmp(keyword, "x") == 0)
				{
					keyX = 1;
				}
				else if (strcmp(keyword, "y") == 0)
				{
					keyY = 1;
				}
				else if (strncmp(keyword, keySymbolsName, 6) == 0)
				{
					keySymbols = 1;
				}
				else if (strncmp(keyword, keyStringsName, 6) == 0)
				{
					keyStrings = 1;
				}
				else if (strncmp(keyword, keyStructureName, 3) == 0)
				{
					symhKeys = argi;

					nargs--;
					if (startKey >= 0)
					{
						sprintf(OUTSTR,
								"ERROR: no other keyword can be used with '%s' on %s()",
								keyword, FUNCNAME);
					}
					else if (i != nargs)
					{
						sprintf(OUTSTR,
								"ERROR: keyword '%s' not last argument to %s()",
								keyword, FUNCNAME);
					}
					else if (TYPE(symhKeys) != STRUC)
					{
						sprintf(OUTSTR,
								"ERROR: value of keyword '%s' not a structure on %s()",
								keyword, FUNCNAME);
					}
				} /*if (strncmp(keyword, keyStructureName, 3) == 0)*/

				if (startKey < 0)
				{
					startKey = i;
				}
			} /*if (keyword)*/
			else if (argi != EmptyArgument)
			{
				sprintf(OUTSTR,
						"ERROR: non keyword argument to %s()%s", FUNCNAME,
						(op == ISHOWPLOT) ?
							" after data and symbols, if any" : NullString);
			}
			if (*OUTSTR)
			{
				goto errorExit;
			}
		} /*for (i = startArg; i < nargs; i++)*/
	} /*if (!assignment)*/
	else if (nargs > 1)
	{
		/*
		  windowassign(graphvar, symhKeys)  with no keyword arguments
	
		  Treat argument 2 to windowassign() as if it were keys:structure
		*/
		startKey = 1;
		symhKeys = COMPVALUE(list, startKey);
	}

	if (startKey < 0)
	{
		startKey = nargs;
	}

	if (symhKeys != SymbolZero)
	{
		long        nkeys = NCOMPS(symhKeys);
		
		/*
		  Scan structure symhKeys for keywords 'x', 'y', 'symbols'
		  and 'strings'
		*/
		for (i = 0; i < nkeys; i++)
		{
			Symbolhandle       symhComp = COMPVALUE(symhKeys, i);
			
			keyword = (symhComp != SymbolZero) ?
			   NAME(symhComp) : NullString;

			if (keyword[0])
			{
				if (strcmp(keyword, keyXname) == 0)
				{
					keyX = 1;
				}
				else if (strcmp(keyword, keyYname) == 0)
				{
					keyY = 1;
				}
				else if (strncmp(keyword, keySymbolsName, 6) == 0)
				{
					keySymbols = 1;
				}
				else if (strncmp(keyword, keyStringsName, 6) == 0)
				{
					keyStrings = 1;
				}
			} /*if (keyword[0] != '\0')*/
		} /*for (i = 0; i < nkeys; i++)*/

		if (assignment && keyX)
		{
			op |= NEEDXY;
		}
	} /*if (symhKeys != SymbolZero)*/

	/*
	  At this point, we know where the data and possibly symbols or
	  strings are.  For example, when x and y have been specified as the
	  first two (one if in structure) arguments, haveData != 0 and
	  symhx and symhy are handles to the data.  When they have been specified
	  as part of structure keys, keyX and keyY are set, but we don't
	  have the handles
	*/
	if (keyStrings && keySymbols)
	{
		sprintf(OUTSTR,
				"ERROR: you can't use both 'symbols' and 'strings' with %s()",
				FUNCNAME);
	}
	else if (keyStrings || keySymbols || keyX || keyY)
	{
		char       *duplicated =
		  "ERROR: %s for %s() specified twice; don't know which to use";

		if (showPlot)
		{
			sprintf (OUTSTR,
					 "ERROR: you can't use any of keywords '%s', '%s' 'symbols' or 'strings' with %s()",
					 keyXname, keyYname, FUNCNAME);
		} /*if (showPlot)*/
		else if (keyX && !keyY || keyY && !keyX)
		{
			sprintf(OUTSTR,
					"ERROR: one of keywords '%s' or '%s' used without the other in %s()",
					keyXname, keyYname, FUNCNAME);
		}
		else if (keyX && haveData)
		{
			sprintf(OUTSTR, duplicated, "x and y", FUNCNAME);
		}
		else if (keyStrings)
		{
			if (op & PLOTTYPEMASK)
			{
				badKeyword(FUNCNAME, keyStringsName);
				goto errorExit;
			}
			if (haveStrings)
			{
				sprintf(OUTSTR, duplicated, keyStringsName, FUNCNAME);
			}
			else if (assignment && oldplot == SymbolZero)
			{
				sprintf(OUTSTR,
						"ERROR: %s[i]<-structure(x:x,y:y,strings:s) with empty %s[i]",
						NAME(graphWindows), NAME(graphWindows));
			}
			else
			{
				op |= STRINGPLOT;
			}
		}
		else if (keySymbols)
		{
			if (haveSymbols)
			{
				sprintf(OUTSTR, duplicated, "plotting symbols", FUNCNAME);
			}
			else
			{
				op |= CHARPLOT;
			}
		}
	}	
	else if ((op & STRINGPLOT) && !haveStrings)
	{
		sprintf(OUTSTR,	"ERROR: no strings for %s() to draw", FUNCNAME);
	}
	if (*OUTSTR)
	{
		goto errorExit;
	}

	/*
      990501 fplot(), flineplot(), fchplot() have been removed and
	  file:"filename" is no longer allowed as argument 1
	  Removed code for checking argument 1 as a file name
	*/

	if (!haveData && !keyX)
	{
		char    *noPlottingPositions =
		  "ERROR: %s provided to %s() but no plotting positions";

		/* No data has been provided */
		if (haveSymbols || keySymbols)
		{
			sprintf(OUTSTR, noPlottingPositions, keySymbolsName, FUNCNAME);
		} /*if (haveSymbols || keySymbols)*/
		else if (haveStrings || keyStrings)
		{
			sprintf(OUTSTR, noPlottingPositions, keyStringsName, FUNCNAME);
		}
		else if (op & PLOTTYPEMASK)
		{
			sprintf(OUTSTR,
					"ERROR: no plotting positions specified for %s()",
					FUNCNAME);
		}
		else if (assignment)
		{
			if (oldplot == SymbolZero)
			{
				sprintf(OUTSTR,
						"ERROR: %s[i] <- structure without data and %s[i] is empty",
						NAME(graphWindows), NAME(graphWindows));
			}
			else
			{
				showPlot = 1;
				op |= SHOW;
			}
		}

		if (*OUTSTR)
		{
			goto errorExit;
		}
	} /*if (!haveData && !keyX)*/

	/* Set which keywords are allowable and some default values */

	setKeyDefaults(op, assignment, keyValues, beforeGetPlotKeys);

	if (getPlotKeys((symhKeys == SymbolZero) ? list : symhKeys,
					startKey, keyValues) < 0)
	{
		goto errorExit;
	}

	/* File names are now (951113) expanded by getPlotKeys()*/

	/*
		Change DONTSETS to UNSETS and set other defaults for unset keys
        In particular this sets 'add' (default 0 for assignment and plot(),
		lineplot(), chplot(), 1 for addxxxx() functions), 'show' (default 1)
		and	'landscape' (default 0)
	*/
	setKeyDefaults(op, assignment, keyValues, afterGetPlotKeys);

	if (assignment)
	{
		if ((!keyValues->keep || !keyValues->show))
		{
			sprintf(OUTSTR,
					"ERROR: %s[I] <- structure(...,%s:F) is illegal",
					GRAPHWINDOWSNAME, (!keyValues->show) ? "show" : "keep");
		}
#ifdef MULTIPLOTWINDOWS
		else if (keyValues->window != UNSETLONG &&
				 keyValues->window != graphWindowsIndex + 1)
		{
			sprintf(OUTSTR,
					"ERROR: %s[I] <- structure(...,window:J) with J != I",
					GRAPHWINDOWSNAME);
		}
#endif /*MULTIPLOTWINDOWS*/
		if (*OUTSTR)
		{
			goto errorExit;
		}
		if (showPlot)
		{
			keyValues->add = 0;
		}
	} /*if (assignment)*/
	
	if (!haveData && !keyX && keyValues->add)
	{
		sprintf(OUTSTR,
				"ERROR: add:T illegal for %s() unless plotting positions given",
				FUNCNAME);
		goto errorExit;
	}
	toFile = (keyValues->file != UNSETPOINTER);

	/*
	  checkPlotKeys() checks keyword values for appropriateness and
	  may set or correct values for 'show' (default 1), 'ps',
	  'new' (default 0), 'dumb', 'epsf' (default 0), 'keep' (default 1),
	  'notes', 'window', 'height', 'width'
	*/
	if (!checkPlotKeys(toFile, keyValues, &termType))
	{
		goto errorExit;
	}

#ifdef MULTIPLOTWINDOWS
	if (assignment)
	{
		keyValues->dumb = 0;
		keyValues->window = graphWindowsIndex + 1;
	} /*if (assignment)*/
#endif /*MULTIPLOTWINDOWS*/

	if (showPlot)
	{
		/* No Data to add */
		keyValues->add = 0;
	}

	windno = keyValues->window;
	if (!assignment && windno != UNSETCHAR &&
		windno > 0 && windno <= ngraphs)
	{
		graphWindowsComp = getGraphWindow(windno - 1);
	}

#ifdef DUMPPLOTKEYS
	/* Code used in debugging */
	if (GUBED & 8)
	{
		dumpPlotKeys(NAMEFORWHERE, " after checkPlotKeys()",
					 keyValues);
	}
#endif /*DUMPPLOTKEYS*/

	if (graphArg && (!assignment || symhKeys != SymbolZero))
	{
		/*
          plot(graphVar,...), lineplot(graphVar,...), chplot(graphVar,...),
		  addlines(graphVar,...), addpoints(graphVar,...),
		  addchars(graphVar,...), addstrings((graphVar,...) or
		  GRAPHWINDOWS[i] <- structure(plot keys) 

		  This does not include GRAPHWINDOWS[i] <- graphVar for which
		  GRAPHWINDOWS[i] is already set and oldplot == GRAPHWINDOWS[i].

		  On non-assignments, if it is known which window is to be drawn
		  then graphWindowsComp is the appropriate element of GRAPHWINDOWS
		  
		  With plot(graphVar, ...), lineplot(graphVar, ...), etc., oldplot is
		  graphVar, but it is not yet in GRAPHWINDOWS unless it is LASTPLOT
		  
		  After GRAPHWINDOWS[i] <- var, the first argument is GRAPHWINDOWS[i]
		  (graphWindowsComp) which may be NULL or a graph variable.  If it is
		  NULL, then oldplot == (Symbolhandle) 0; otherwise oldplot is
		  GRAPHWINDOWS[i]
		*/
		if (!keyValues->add && !showPlot && !assignment)
		{
			sprintf(OUTSTR,
					"ERROR: add:F is illegal on %s() when GRAPH variable is argument",
					FUNCNAME);
		}
		else if (oldplot == SymbolZero && showPlot)
		{
			/* Must be assignment*/
			sprintf(OUTSTR,
					"ERROR: no data provided when %s[%ld] is not a GRAPH variable",
					NAME(graphWindows), windno);
		}
		if (*OUTSTR)
		{
			goto errorExit;
		}

		if (!assignment)
		{
			keyValues->add = !showPlot;
		}
	} /*if (graphArg && (!assignment || symhKeys != SymbolZero))*/
	else if ((keyValues->add || showPlot) && (!assignment || nargs > 1))
	{
		oldplot = lastplot;
		if (oldplot == SymbolZero)
		{
			sprintf(OUTSTR,
					"ERROR: no GRAPH variable %s for %s()%s",
					LastGraphName, FUNCNAME,
					(op & ADDTOPLOT) ? NullString : " with add:T");
			goto errorExit;
		}
	}

	/*
	  990527 addchars(x,y,symbols:c,...,add:F) is equivalent to
	         chplot(x,y,symbols:c) and similarly for addpoints(), addlines(),
	         and addstrings()
	*/
	if (keyValues->add)
	{
		op |= ADDTOPLOT;
	}
	else
	{
		op &= ~ADDTOPLOT;
	}

#if defined(EPSFPLOTSOK) && 0
	if (keyValues->epsf)
	{
		if (!keyValues->silent)
		{
			putOutErrorMsg("WARNING: encapsulated PostScript not yet implemented");
		}
		keyValues->epsf = 0;
	} /*if (keyValues->epsf)*/
#endif /*EPSFPLOTSOK*/

	if (op & LINEPLOT && !keyValues->lines)
	{
		/* pretend it's plot()*/
		op = PTPLOT | (op & ~LINEPLOT);
	}
	if (assignment && !(op & PLOTTYPEMASK))
	{
		op |= (keyValues->lines) ? LINEPLOT :  PTPLOT;
	} /*if (assignment && !(op & PLOTTYPEMASK))*/

	if (!showPlot && keyValues->linetype != UNSETLONG)
	{
		/*
		  linetype:k < 0 is the same as linetype:abs(k) but does not
		  generate warning message; linetype:0 is the same as linetype:-1
		*/
		if (keyValues->linetype > 0)
		{
			keyValues->linetype--; /* now between 0 and MAXLINETYPE-1 */
			if (!keyValues->lines)
			{
				if (!keyValues->silent)
				{
					putOutErrorMsg("WARNING: keyword 'linetype' ignored when not drawing lines");
				}
			}
		}
		else if(keyValues->linetype < 0)
		{
			keyValues->linetype = -keyValues->linetype - 1;
		}
	} /*if (!showPlot && keyValues->linetype != UNSETLONG)*/
	else
	{
		keyValues->linetype = 0;
	}

	if (!showPlot && keyValues->thickness != UNSETREAL)
	{
		if (!keyValues->lines)
		{
			if (!keyValues->silent)
			{
				putOutErrorMsg("WARNING: keyword 'thickness' ignored when not drawing lines");
			}
		}
		else
		{
			/*
			  Code thickness into linetype so actual type is
			  linetype % MAXLINETYPE

			  After this point, keyValues->thickness is irrelevant
			*/
			keyValues->linetype += MAXLINETYPE *
				(long) ((double) UNITWIDTH * keyValues->thickness / (double) MAXLINETYPE);
		}
	} /*if (!showPlot && thickness != UNSETREAL)*/

	if (toFile)
	{
		strcpy(filePath, keyValues->file);
		keyValues->file = filePath;
	} /*if (toFile)*/

#ifdef SCREENDUMP
	/* Expanding file name is now done by getPlotKeys() */
	if (keyValues->screendump != UNSETPOINTER)
	{
		if (termType != STANDARDTERM)
		{
			sprintf(OUTSTR,
					"ERROR: screendump:fileName can only be used with high resolution plot");
			goto errorExit;
		}

#ifdef MACINTOSH
		if(keyValues->screendump[0] != '\0' && !isfilename(keyValues->screendump))
#else  /*MACINTOSH*/
		if(!isfilename(keyValues->screendump))
#endif /*MACINTOSH*/
		{
			goto errorExit;
		}
	} /*if (keyValues->screendump != UNSETPOINTER)*/
#endif /*SCREENDUMP*/

#ifdef DUMPPLOTKEYS
	if (GUBED & 8)
	{
		dumpPlotKeys(NAMEFORWHERE, " before Lookup()",
					 keyValues);
	}
#endif /*DUMPPLOTKEYS*/

	if (keyX)
	{
		symhx = keyValues->x;
		symhy = keyValues->y;
		/* known to be REAL or NULL; checked in getPlotKeys() */
		haveData = 1;
	}
	if (haveData)
	{ /* check x and y for appropriateness */
		if (!checkGraphData(symhx, symhy, op, keyX, strucArg,
							keyValues, &nx, &ny, &ncols))
		{
			goto errorExit;
		}
		if (TYPE(symhx) == NULLSYM || TYPE(symhy) == NULLSYM)
		{
			/* do nothing if either x or y is NULL */
			if (!keyValues->silent)
			{
				sprintf(OUTSTR,
						"WARNING: x and/or y is NULL in %s()", FUNCNAME);
				putErrorOUTSTR();
			}
			return (NULLSYMBOL);
		} /*if (TYPE(symhx) == NULLSYM && TYPE(symhy) == NULLSYM)*/

		xisIndex = (nx != ny);
	} /*if (haveData)*/

	if (op & CHARPLOT)
	{/* find out what characters to plot */
		if (haveSymbols || keySymbols)
		{ /* characters or integers are supplied */
			char      *what = (keySymbols) ? "'symbols'" : "chars";

			if (keySymbols)
			{
				symhsymbols = keyValues->symbols;
			}

			if (TYPE(symhsymbols) != REAL && TYPE(symhsymbols) != CHAR)
			{
				sprintf(OUTSTR,
						"ERROR: %s must be REAL or CHARACTER for %s()",
						what, FUNCNAME);
			}
			else if (!isMatrix(symhsymbols, dims))
			{
				sprintf(OUTSTR,
						"ERROR: %s must be a vector%s for %s()",
						(ncols > 0) ? " or matrix" : "", what, FUNCNAME);
			}
			else if ((cCols = dims[1]) != 1 && cCols != ncols)
			{
				sprintf(OUTSTR,
						"ERROR: ncols(%s) must be 1 or match ncols(y)",	what);
			}
			else if (TYPE(symhsymbols) == REAL)
			{ /* check to see values are integers */
				cRows = dims[0];
				if (cCols == 1 && cRows == ncols)
				{
					cCols = ncols;
					cRows = 1;
				}

				nchars = cRows*cCols;

				for (i = 0;i < nchars;i++)
				{
					double     value = DATAVALUE(symhsymbols, i);
					
					if (isMissing(value) || value < 0.0 || value > 999.0 ||
						value != floor(value))
					{
						sprintf(OUTSTR,
								"ERROR: numerical %s for %s() not integers >= 0 and < 1000",
								what, FUNCNAME);
						break;
					}
					cmax = (value > cmax) ? value : cmax;
				} /*for (i = 0;i < nchars;i++)*/
			}
			else if (isScalar(symhsymbols) &&
					 strncmp(STRINGPTR(symhsymbols),"###", 3) == 0)
			{
				symhsymbols = SymbolZero;
				nchars = -1; /*signal to set cRows, cCols, nchars later*/
			}
			else
			{ /* characters */
				cRows = dims[0];
				if (cCols == 1 && cRows == ncols)
				{
					cCols = ncols;
					cRows = 1;
				}
			}
		} /*if (haveSymbols || keySymbols)*/
		else
		{ /* no labelling information supplied; will be generated */
			symhsymbols = SymbolZero;
			nchars = -1; /*signal to set cRows, cCols, nchars later*/
		} /*if (haveSymbols || keySymbols){}else{}*/
	} /* if (op & CHARPLOT) */
	else if (op & STRINGPLOT)
	{
		char     *pc;
		char      what[30];

		if (keyStrings)
		{
			sprintf(what, "values for '%s'", keyStringsName);
			symhstrings = keyValues->strings;
		}
		else
		{
			strcpy(what, keyStringsName);
		}

		if (!isVector(symhstrings) || TYPE(symhstrings) != CHAR ||
		   symbolSize(symhstrings) != ny)
		{
			sprintf(OUTSTR,
					"ERROR: %s not a CHARACTER vector of the same length as x for %s()",
					what, FUNCNAME);
		}
		else
		{
			pc = STRINGPTR(symhstrings);
	
			for (i = 0; i < ny; i++)
			{
				if (strlen(pc) > MAXCHARITEMSIZE)
				{
					sprintf(OUTSTR,
							"ERROR: %s for %s() contains a string longer than %ld characters",
							what, FUNCNAME, (long) MAXCHARITEMSIZE);
					break;
				}
				pc = skipStrings(pc, 1);
			} /*for (i = 0; i < ny; i++)*/
		}
	} /* if (op & CHARPLOT){}else if (op & STRINGPLOT){}*/

	if (*OUTSTR)
	{
		goto errorExit;
	}

	if (nchars < 0)
	{
		if (ncols > 1)
		{
			cRows = 1;
			cCols = ncols;
		}
		else
		{
			cRows = ny;
			cCols = 1;
		}
		nchars = cRows*cCols;
		/* negative cmax indicates labels will need to be built */
		cmax = -((nchars < 999) ? nchars : 999);
	} /*if (nchars < 0)*/
	
	usePreviousGraph = (keyValues->add || showPlot) && oldplot != SymbolZero;

	useLASTPLOT = usePreviousGraph && oldplot == lastplot;

	if (usePreviousGraph)
	{
		/*
		  Adding to or displaying data from earlier plot defined in oldplot
		*/

		/*
		  Detach LASTPLOT from GRAPHWINDOWS if we are creating a
		  new LASTPLOT or modifying the old but are not displaying it in
		  a window, either because it is being written to a file, is a
		  dumb plot, or because show:F was an argument
		*/
		
		if (lastplot != SymbolZero && isLinked(lastplot) &&
			(!keyValues->show ||
			 keyValues->keep && (toFile || keyValues->dumb)) &&
			!detachGraphWindow(lastplot))
		{
			goto errorExit;
		}
		
		/*
		  If oldplot is in GRAPHWINDOWS, detach it unless it is known
		  we will be modifying (not replacing) window with which it
		  is associated.
		*/

		if (keyValues->keep && isLinked(oldplot))
		{
			if ((windno == UNSETLONG || windno == 0 ||
				 !keyValues->keep && oldplot == graphWindowsComp ||
				 oldplot != getGraphWindow(windno - 1) ||
				 !showPlot && !keyValues->add) &&
				!detachGraphWindow(oldplot))
			{
				goto errorExit;
			}
		} /*if (keyValues->keep && isLinked(oldplot))*/

		if (useLASTPLOT && graphArg)
		{ /*LASTPLOT is argument; remove from arg list */
			COMPVALUE(list, 0) = SymbolZero;
		}

		/*
		   At this point oldplot is either the GRAPHWINDOWS component being
		   assigned to, a supplied graph symbol or LASTPLOT

		   Duplicate oldplot unless one of these conditions is true
		     (a) oldplot is scratch
			 (b) keep:T and oldplot is LASTPLOT
		*/

		if (!isscratch(NAME(oldplot)) &&
			(!keyValues->keep || oldplot != lastplot))
		{
			/* make copy of entire oldplot including data*/
			graphsymbol = Install(SCRATCH, PLOTINFO);
			if (graphsymbol == SymbolZero ||
				!Copy(oldplot, graphsymbol))
			{
				goto errorExit;
			}
			setNAME(graphsymbol, SCRATCH);
		} /*if (showPlot){}else{}*/
		else
		{
			graphsymbol = oldplot;
		}
		graph = GRAPH(graphsymbol);

		/* save info from graph only if not replaced by keywords */

		nxticks = GRAPHNXTICKS(graph);
		nyticks = GRAPHNYTICKS(graph);

	}/*if (usePreviousGraph)*/
	else
	{
		oldplot = graphsymbol = GraphInstall(SCRATCH);
		if (graphsymbol == SymbolError)
		{
			goto errorExit;
		}
		graph = GRAPH(graphsymbol);

		nxticks = nyticks = 1;
	} /*if (usePreviousGraph){}else{}*/

	/*
	  graphsymbol is either LASTPLOT, or a scratch variable
	  
	  At this point graph is always the same as GRAPH(graphsymbol)
	*/
	findExtremes = setKeyFinal(graph, keyValues, usePreviousGraph);

	/*
		990512 removed code that expanded short x vector to x0 + i*incx
               That is now done in setGraphPoints()
	*/

	/* set style of points to be plotted */

	if (op & CHARPLOT)
	{
		style = CHARS | ((keyValues->impulses) ? IMPULSES : 0) |
		  ((keyValues->lines) ? LINES : 0);
	}
	else if (op & PTPLOT)
	{
		style = ((keyValues->impulses) ? IMPULSES : POINTS) |
		  ((keyValues->lines) ? LINES : 0);
	}
	else if (op & LINEPLOT)
	{
		style = LINES | ((keyValues->impulses) ? IMPULSES : 0);
	}

	if (!assignment && keyValues->window == UNSETLONG)
	{
		windno = (keyValues->add) ? 0 : FINDEMPTYWINDOW;
	}

#if defined(EPSFPLOTSOK) && 0
	if (keyValues->epsf)
	{
		if (!keyValues->silent)
		{
			putOutErrorMsg("WARNING: encapsulated PostScript not yet implemented");
		}
		keyValues->epsf = 0;
	} /*if (keyValues->epsf)*/
#endif /*EPSFPLOTSOK*/

	if (findExtremes || keyValues->logx || keyValues->logy)
	{
		double                 extremes[4];
		unsigned long          control = findExtremes;
		char                  *nonPosValue =
		  "WARNING: cases with %c <= 0 with 'log%c:T' are ignored by %s()";
		char                  *nonPosKey =
		  "WARNING: value for '%cmin' <= 0 with 'log%c:T' ignored by %s()";

		control |= (keyValues->logx) ? FINDXMIN : 0;
		control |= (keyValues->logy) ? FINDYMIN : 0;

		if (usePreviousGraph)
		{
			getGraphExtremes(graph, extremes, 1);
		}
		else
		{
			double         hugedbl = HUGEDBL;
			
			extremes[IXMIN] = extremes[IYMIN] = hugedbl;
			extremes[IXMAX] = extremes[IYMAX] = -hugedbl;
		}

		if (!compExtremes(symhx, symhy, graph, xisIndex, extremes, control))
		{
			goto errorExit;
		}

		if (findExtremes & FINDXMIN)
		{
			keyValues->xmin = extremes[IXMIN];
		}
		if (findExtremes & FINDXMAX)
		{
			keyValues->xmax = extremes[IXMAX];
		}
		if (findExtremes & FINDYMIN)
		{
			keyValues->ymin = extremes[IYMIN];
		}
		if (findExtremes & FINDYMAX)
		{
			keyValues->ymax = extremes[IYMAX];
		}
		if (keyValues->logx)
		{
			if (extremes[IXMIN] <= 0)
			{
				if (!keyValues->silent)
				{
					sprintf(OUTSTR, nonPosValue, 'x', 'x', FUNCNAME);
					putErrorOUTSTR();
				}
			}
			else if (keyValues->xmin <= 0)
			{
				if (!keyValues->silent)
				{
					sprintf(OUTSTR, nonPosKey, 'x', 'x', FUNCNAME);
					putErrorOUTSTR();
				}
			}
		} /*if (keyValues->logx)*/
		
		if (keyValues->logy)
		{
			if (extremes[IYMIN] <= 0)
			{
				if (!keyValues->silent)
				{
					sprintf(OUTSTR, nonPosValue, 'y', 'y', FUNCNAME);
					putErrorOUTSTR();
				}
			}
			else if (keyValues->ymin <= 0)
			{
				if (!keyValues->silent)
				{
					sprintf(OUTSTR, nonPosKey, 'y', 'y', FUNCNAME);
					putErrorOUTSTR();
				}
			}
		} /*if (keyValues->logx)*/
	} /*if (findExtremes || keyValues->logx || keyValues->logy)*/
	
	if (toFile)
	{
		/* fileName must be pointing to filePath */
#ifdef HASFINDFILE
		char                 *tmpFileName;

		tmpFileName = findPlotFile(keyValues, "Graph");

		if (tmpFileName == (char *) 0)
		{ /* Canceled */
			goto errorExit;
		}
		if (tmpFileName != keyValues->file)
		{
			strncpy(keyValues->file, tmpFileName, PATHSIZE);
		}
#endif /*HASFINDFILE*/

		if (!isfilename(keyValues->file))
		{
			goto errorExit;
		}

		/* Should have filename */
		plotfile = openPlotFile(keyValues);
		if (plotfile == (FILE *) NULL)
		{
			goto errorExit;
		}
    } /*if (toFile)*/
	else
	{
		plotfile = STDOUT;
	}

#ifdef DUMPPLOTKEYS
	if (GUBED & 8)
	{
		dumpPlotKeys(NAMEFORWHERE, " before buildPlot()",
					 keyValues);
	}
#endif /*DUMPPLOTKEYS*/

	if (!buildPlot(op, graph, symhx, symhy,
				   (op & STRINGPLOT) ? symhstrings : symhsymbols, xisIndex,
				   style, (long) cmax, nchars, cRows, cCols, keyValues))
	{
		goto errorExit;
	}

	/*
	  Add axis labels, title, frame		and ticks stuff to graph
	*/
	setGraphInfo(graph, keyValues, nxticks, nyticks, oldplot == lastplot);

	if (keyValues->keep)
	{
		/*
			save plotting info in LASTPLOT
		*/
		if (graphsymbol != lastplot)
		{
			if (lastplot != SymbolZero)
			{
				Removesymbol(lastplot);
			}
			else
			{
				Remove(LastGraphName);
			}

			if (isscratch(NAME(graphsymbol)))
			{
				/* rename scratch graphsymbol to LASTPLOT */
				if (graphsymbol == COMPVALUE(list, 0))
				{
					COMPVALUE(list, 0) = SymbolZero;
				}
				Cutsymbol(graphsymbol);
				setNAME(graphsymbol, LastGraphName);
				Addsymbol(graphsymbol);
				lastplot = graphsymbol;
			} /*if (isscratch(NAME(graphsymbol)))*/
			else
			{
				Symbolhandle      symhTemp = Install(LastGraphName, PLOTINFO);
				/*
				   graphsymbol is a new symbol with GRAPH(graphsymbol) = graph.
				*/
				if (symhTemp == SymbolError || !Copy(graphsymbol, symhTemp))
				{
					Removesymbol(symhTemp);
					if (!keyValues->silent)
					{
						sprintf(OUTSTR,
								"WARNING: %s() cannot create %s",
								FUNCNAME, LastGraphName);
						putErrorOUTSTR();
					}
					lastplot = SymbolZero;
				}
				else
				{
					setNAME(symhTemp, LastGraphName);
					lastplot = graphsymbol = symhTemp;
				}
			} /*if (isscratch(NAME(graphsymbol))){}else{}*/
		} /*if (graphsymbol != lastplot)*/

		if (lastplot != SymbolZero && keyValues->notes != SymbolZero)
		{
			if (setNotes(lastplot, STRING(keyValues->notes)))
			{
				setSTRING(keyValues->notes, (char **) 0);
			}
			else
			{
				if (!keyValues->silent)
				{
					sprintf(OUTSTR, "WARNING: notes not set in %s()",
							FUNCNAME);
					putErrorOUTSTR();
				}
			}
		} /*if (lastplot != SymbolZero && keyValues->notes != SymbolZero)*/
	} /*if (keyValues->keep)*/

	if (keyValues->show && !do_whole_graph(GRAPH(graphsymbol), plotfile,
										  termType, windno, keyValues))
	{
		goto errorExit;
	}

	if (plotfile != STDOUT)
	{
		/*
			010524 Moved setting of Macintosh creator and type to openPlotFile()
			in plotutil.c
		*/
#ifdef MACINTOSH
		mvFileInfo      fileInfo;

		setMvFileVolume(&fileInfo, getMvVolID(plotVol));
		setMvFileDirectory(&fileInfo, getMvDirID(plotVol));
		setMvFileNameC(&fileInfo, keyValues->file);
		if (keyValues->epsf)
		{
			setMvFileType(&fileInfo, 'EPSF');
		}
		else if (keyValues->dumb || keyValues->ps)
		{
			setMvFileType(&fileInfo, 'TEXT');
		}
		else
		{
			setMvFileType(&fileInfo, 'PICT');
		}
		macSetFInfo(&fileInfo, CREATOR);
#endif /*MACINTOSH*/
		fclose(plotfile);
	} /*if (plotfile != STDOUT)*/

	if (!showPlot && !keyValues->keep)
	{
		/* cannot happen on assignment */
		Removesymbol(graphsymbol);
		graphsymbol = SymbolZero;
	} /*if (!showPlot && !keyValues->keep)*/

	if (!assignment && !toFile && keyValues->show && !keyValues->dumb &&
		ThisWindow >= 0 && ThisWindow < ngraphs)
	{
		graphWindowsComp = getGraphWindow(ThisWindow);

		if (graphWindowsComp != lastplot)
		{
			Delete(graphWindowsComp);
		}
		if (!keyValues->keep || lastplot == SymbolZero)
		{
			setGraphWindow(makeNullGraph(ThisWindow, NULLSYMBOL), ThisWindow);
		} /*if (!keyValues->keep || lastplot == SymbolZero)*/
		else
		{
			attachGraphWindow(lastplot, ThisWindow);
		} /*if (!keyValues->keep || lastplot == SymbolZero){}else{}*/
	} /*if (!assignment && ThisWindow < ngraphs)*/

	resetGraphWindowNames();
	*OUTSTR = '\0';

#ifdef MACINTOSH
	UNLOADSEG(do_whole_graph);
#endif /*MACINTOSH*/

	return ((assignment) ? lastplot : NULLSYMBOL);

  errorExit:
	putErrorOUTSTR();

	if (plotfile != STDOUT)
	{
		fclose(plotfile);
	}

	if (!showPlot)
	{
		Removesymbol(graphsymbol);
	}
	else
	{
		emptyTrash();
	}

#ifdef MACINTOSH
	UNLOADSEG(do_whole_graph);
#endif /*MACINTOSH*/
		return (0);
} /*myplot()*/

