/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.12 or later
*(C)*
*(C)* Copyright (c) 2001 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/


#ifdef SEGMENTED
#if defined(MPW) || defined(MW_CW)
#pragma segment Batch
#endif /*MPW||MW_CW*/
#endif /*SEGMENTED*/

#include "globals.h"

#include "mvIface.h"

#include "mvIo.h"

/*
  Routine to spool output to a file or suspend spooling
  Modified by kb to accept second logical argment:
  	spool(fileName [,new:T])
  If new:T is present, the file is opened with fopen(fileName,"w") instead of
  fopen(fileName,"a")
    spool()  toggles spooling on and off once started.
	spool(,new:T) restarts spooling on last specified file.

  SPOOLFILE should be non-zero if and only if spooling is in effect.
  SpoolFileName should be non-zero if and only if spooling is in effect
  or the last attempt to spool was successful.

  960502 Changed fopen() to fmyopen()
  960503 Changed macOpen() to macFindFile()
  960719 spool() now prints starting message after spool("")
         Changed a number of output places to use putPieces()
  970506 Use TEXTWRITEMODE and TEXTAPPENDMODE as fmyopen() mode
  990226 Replaced putOUTSTR() by putErrorOUTSTR() and myerrorout() by
         putOutErrorMsg()
  010521 Changed calling squence to macFindFile()
  010522 Changed call to macSetInfo()
  010523 Replaced macSetInfo() by macSetFInfo()
  010525 Changed macFindFile() to mvFindFile() and dropped wxFindFile()

  010601 Include mvIface.h instead of macIface.h or wxIface.h
  010802 Added keyword phrase everything:T to force spooling even when
         option 'quiet' is True.
  010803 spool(,everything:T) toggles forcing status when spooling
         and toggles forcing status and starts spooling when not
         spooling
         spool(,new:T [,everything:T or F]) restarts spooling
         and sets forcing status
*/

Symbolhandle    spool(Symbolhandle list)
{
	Symbolhandle    symhFile;
	FILE           *spoolFile;
	char          **spoolFileName;
	int             rewindit = -1;
	int             openSpool = 0, everything = -1;
	int             iarg;
	long            nargs = NARGS(list);
	char           *keyword;
	char           *fileName;
	WHERE("spool");
	
#ifdef MACINTOSH
	if (!UseWindows)
	{ /*  */
		sprintf(OUTSTR,"WARNING: %s() disabled in non-interactive mode",
				FUNCNAME);
		goto errorExit;
	}
#endif /*MACINTOSH*/
	
	*OUTSTR = '\0';

	if (nargs > 3)
	{
		badNargs(FUNCNAME, atMost3Args);
		goto errorExit;
	}
	symhFile = COMPVALUE(list,0);
	nargs = (nargs == 1 && symhFile == EmptyArgument) ? 0: nargs;

	if (symhFile != EmptyArgument)
	{
		if (!isCharOrString(symhFile))
		{
			notCharOrString("file name");
		}
	
		fileName = STRINGPTR(symhFile);
	} /*if (symhFile != (Symbolhandle) 0)*/		
	else
	{
		fileName = (char *) 0;
	}
	
	for (iarg = 1; iarg < nargs; iarg++)
	{
		Symbolhandle          symhKey = COMPVALUE(list, iarg);
		
		keyword = isKeyword(symhKey);
		
		if (keyword == (char *) 0)
		{
			if (!isTorF(symhKey))
			{
				sprintf(OUTSTR,
						"ERROR: non keyword argument %d to %s() not T or F",
						iarg + 1, FUNCNAME);
				goto errorExit;
			}
			keyword = "new";
		}
		else if (strcmp(keyword, "new") != 0 &&
				 strcmp(keyword, "everything") != 0)
		{
			badKeyword(FUNCNAME, keyword);
			goto errorExit;
		}
		if (!isTorF(symhKey))
		{
			notTorF(keyword);
			goto errorExit;
		}

		if (keyword[0] == 'n')
		{
			rewindit = (DATAVALUE(symhKey,0) != 0.0);
		}
		else
		{
			everything = (DATAVALUE(symhKey,0) != 0.0);
		}
	} /*for (iarg = 1; iarg < nargs; iarg++)*/
	
	if (rewindit < 0)
	{
		rewindit = 0;
	}
	
/* now see about file name */
	if (fileName == (char *) 0)
	{ /* none given */
		/*
		  new:T     close file, reopen at beginning
		  no argument, so stop or restart (new:T) spooling
		*/

		if (SPOOLFILENAME == (char **) 0)
		{
			putOutErrorMsg("WARNING: not spooling at present");
			everything = -1;
			openSpool = 0;
		}
		else if (SPOOLFILE != (FILE *) 0 && (rewindit || everything < 0))
		{
			/* Toggle status (!rewindit) or restart */
			if (rewindit)
			{
				putPieces("Restart spooling on ", *SPOOLFILENAME,
						  (char *) 0, (char *) 0);
				openSpool = 1;
			}
			else 
			{
				putPieces("Spooling on ", *SPOOLFILENAME," suspended", (char *) 0);
				openSpool = 0;
			}
			fclose(SPOOLFILE);
			SPOOLFILE = (FILE *) 0;
		} /*if (SPOOLFILE != (FILE *) 0 && (rewindit || everything < 0)*/
		else if (everything < 0)
		{
			/*
			  SPOOLFILE must be NULL so spooling is off
			  restart or resume
			*/
			spoolFileName = SPOOLFILENAME;
			SPOOLFILENAME = (char **) 0;
			putPieces((rewindit) ? "Restart" : "Resume", " spooling on ",
					  *spoolFileName, (char *) 0);
			openSpool = 1;
		}
	} /*if (fileName == (char *) 0)*/
	else if (SPOOLFILENAME == (char **) 0 || SPOOLFILE == (FILE *) 0 ||
			strcmp(*SPOOLFILENAME,fileName) != 0 || rewindit)
	{
		/* new filename provided */
		if (SPOOLFILENAME != (char **) 0)
		{
			mydisphandle(SPOOLFILENAME);
			SPOOLFILENAME = (char **) 0;
		}

#ifdef HASFINDFILE
		fileName = mvFindFile(fileName, "Save spooled output as:", NullString,
							  mvWriteit | mvTextFile,
							  getMvDirInfo(spoolVol));

		if (fileName == (char *) 0)
		{
			strcpy(OUTSTR,"WARNING: spooling request cancelled");
			goto errorExit;
		}
#endif /*HASFINDFILE*/

		fileName = expandFilename(fileName);
		if (fileName == (char *) 0 || !isfilename(fileName))
		{
			goto errorExit;
		}
		if ((spoolFileName = mygethandle(strlen(fileName) + 1)) ==
		   (char **) 0)
		{
			goto errorExit;
		}
		strcpy(*spoolFileName, fileName);
		openSpool = 1;
	}
	else
	{
		putPieces("WARNING: already spooling on file ", *SPOOLFILENAME,
				  (char *) 0, (char *) 0);
		openSpool = 0;
	}
	
	if (fileName != (char *) 0 && everything < 0)
	{
		everything = 0;
	}
	if (openSpool)
	{
		if (SPOOLFILE != (FILE *) 0)
		{
			fclose(SPOOLFILE);
			SPOOLFILE = (FILE *) 0;
		}
		spoolFile = fmyopen(*spoolFileName, (rewindit) ?
							TEXTWRITEMODE : TEXTAPPENDMODE);
		if (spoolFile == (FILE *) 0)
		{
			putPieces("ERROR: cannot open file ", *spoolFileName,
					  " for spooling", (char *) 0);
			mydisphandle(spoolFileName);
			goto errorExit;
		}

#ifdef HASFINDFILE
		if (symhFile != (Symbolhandle) 0 && STRINGVALUE(symhFile, 0) == '\0')
		{
			/* Because SPOOLFILE not yet set this should not be spooled */
			putPieces("Spooling started on file ", *spoolFileName,
					  (char *) 0, (char *) 0);
		}
#endif /*HASFINDFILE*/

		SPOOLFILE = spoolFile;
		SPOOLFILENAME = spoolFileName;

#ifdef MACINTOSH
		{
			mvFileInfo        fileInfo;
	
			setMvFileVolume(&fileInfo, SpoolVolume);
			setMvFileDirectory(&fileInfo, SpoolDirectory);
			setMvFileType(&fileInfo, 'TEXT');
			c2pstrcpy(getMvFileName(&fileInfo), *SPOOLFILENAME);
			macSetFInfo(&fileInfo, TEXTCREATOR);
		}
#endif /*MACINTOSH*/
	} /*if (openSpool)*/

	if (everything > 0)
	{
		if (ACTIVEMACROS > 0)
		{
			ThisMacroQuiet &= ~spoolQuiet;
		}
		else
		{
			QUIET &= ~spoolQuiet;
		}
	} /*if (everything > 0)*/
	else if (everything == 0)
	{
		if (ACTIVEMACROS > 0)
		{
			ThisMacroQuiet |= spoolQuiet;
		}
		else
		{
			QUIET |= spoolQuiet;
		}
	}
#if (0)
	PRINT("ERROR: everything = %d, QUIET = %d",everything, QUIET,0,0);
#endif /*0*/
#ifdef MACINTOSH
	setCmdFileItems(FrontWindow() == CmdWind);
#endif /*MACINTOSH*/

	return (NULLSYMBOL);

  errorExit:
	putErrorOUTSTR();
	return (0);
	
} /*spool()*/
