/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.13 or later
*(C)*
*(C)* Copyright (c) 2003 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/



#ifdef SEGMENTED
#if defined(MPW) || defined(MW_CW)
#pragma segment Startmsg
#endif /*MPW||MW_CW*/
#endif /*SEGMENTED*/

#include "version.h"

#include "globals.h"

#include "mvIface.h"

/*
   951113  Changed to do centering using centerBuffer().

   000309  Added type messageLine (really only cosmetic)

   010112  Added home page line and changed names of COPYRIGHTx to
           MVCOPYRIGHTx
   010601  Include mvIface.h instead of macIface.h
   010813  Changed startupMsg() to mvStartupMsg() to avoid conflict with Mac header
   030615  Added define for Carapace
*/
char       *Welcome[] = 
{
	(char *) 0, /* "M A C A N O V A   x.xx" */
	"An Interactive Program for Statistical Analysis and Matrix Algebra",
	"For information on major features, type 'help(macanova)'",
	"For information on linear models and GLM's, type 'help(glm)'",
	"For latest information on changes, type 'help(news)'",
#if defined (MACINTOSH)
	"For information on Macintosh version, type 'help(macintosh)'",
#elif defined (BCPP) || defined (COMP_DJGPP)
	"For information on DOS versions, type 'help(dos_windows)'",
#elif defined (WXWINMSW)
	"For information on Windows version, type 'help(wx)'",
#elif defined (WXWINMOTIF)
	"For information on Motif version, type 'help(wx)'",
#elif defined(BASE_UNIX)
	"For information on Unix version, type 'help(unix)'",
#elif defined(SKEL_CARAPACE)
	"For information on Carapace version, type 'help(carapace)'",
#endif

#ifdef TODAY
	(char *) 0, /* "Version of xx/xx/xx" */
#endif /*TODAY*/

#ifdef MVCOPYRIGHT1
	(char *) 0,
#endif /*MVCOPYRIGHT1*/

#ifdef MVCOPYRIGHT2
	(char *) 0,
#endif /*MVCOPYRIGHT2*/

#ifdef MVCOPYRIGHT3
	(char *) 0,
#endif /*MVCOPYRIGHT3*/

#ifdef MVCOPYRIGHT4
	(char *) 0,
#endif /*MVCOPYRIGHT4*/

#ifdef MVHOMEPAGE
	(char *) 0,
#endif /*MVHOMEPAGE*/

#ifdef MACANOVA2AVAIL
	(char *) 0,
#endif /*MACANOVA2AVAIL*/

#ifdef MACANOVA31AVAIL
	(char *) 0,
#endif /*MACANOVA31AVAIL*/

#ifdef BACKUPS
	(char *) 0,
	(char *) 0,
#endif /*BACKUPS*/
	(char *) 0
};

#define MAXLENGTHMSG  80

typedef char   messageLine[MAXLENGTHMSG + 1];

static messageLine  Version = "";

#ifdef TODAY
static messageLine  Today = ""; 
#endif /*TODAY*/

#ifdef MVCOPYRIGHT1
static messageLine  Copyright1;
#endif /*MVCOPYRIGHT1*/
#ifdef MVCOPYRIGHT2
static messageLine  Copyright2;
#endif /*MVCOPYRIGHT2*/
#ifdef MVCOPYRIGHT3
static messageLine  Copyright3;
#endif /*MVCOPYRIGHT3*/
#ifdef MVCOPYRIGHT4
static messageLine  Copyright4;
#endif /*MVCOPYRIGHT4*/

#ifdef MVHOMEPAGE
static messageLine  HomePage;
#endif /*MVHOMEPAGE*/

#ifdef MACANOVA2AVAIL
static messageLine  Macanova2Msg = "";
#endif /*MACANOVA2AVAIL*/

#ifdef MACANOVA31AVAIL
static messageLine  Macanova31Msg = "";
#endif /*MACANOVA31AVAIL*/

#ifdef BACKUPS
static messageLine  BackupMsg1 = "";
static messageLine  BackupMsg2 = "";
#endif /*BACKUPS*/

/*
   Concatenate head and tail in msgs and return msgs
*/
static char *makeMsgLine(char *head, char *tail, char *msgs)
{
	int       i;

	for (i = 0; i < MAXLENGTHMSG && *head != '\0';i++)
	{
		msgs[i] = *head++;
	} /*for (i = 0; i < MAXLENGTHMSG && *head != '\0';i++)*/

	while (i < MAXLENGTHMSG && *tail != '\0')
	{
		msgs[i++] = *tail++;
	}
	msgs[i] = '\0';
	return (msgs);
} /*makeMsgLine()*/

void mvStartupMsg(long quiet)
{
 	char		  **message;
	int             length = SCREENWIDTH;
	WHERE("mvStartupMsg");
	
	message = Welcome;
#ifdef VERSION
	*message++ = makeMsgLine("M A C A N O V A   ", VERSION, Version);
#else /*VERSION*/
	*message++ = makeMsgLine("M A C A N O V A",NullString, Version);
#endif /*VERSION*/

	while(*message != (char *) 0)
	{
		message++;
	}

#ifdef TODAY
	*message++ = makeMsgLine("Version ",TODAY, Today);
#endif /*TODAY*/

#ifdef MVCOPYRIGHT1
	*message++ = makeMsgLine(MVCOPYRIGHT1, NullString, Copyright1);
#endif /*MVCOPYRIGHT1*/
#ifdef MVCOPYRIGHT2
	*message++ = makeMsgLine(MVCOPYRIGHT2, NullString, Copyright2);
#endif /*MVCOPYRIGHT2*/
#ifdef MVCOPYRIGHT3
	*message++ = makeMsgLine(MVCOPYRIGHT3, NullString, Copyright3);
#endif /*MVCOPYRIGHT3*/
#ifdef MVCOPYRIGHT4
	*message++ = makeMsgLine(MVCOPYRIGHT4, NullString, Copyright4);
#endif /*MVCOPYRIGHT4*/

#ifdef MVHOMEPAGE
	*message++ = makeMsgLine("MacAnova home page: ", MVHOMEPAGE, HomePage);
#endif /*MVHOMEPAGE*/

#ifdef MACANOVA2AVAIL
	*message++ = makeMsgLine("MacAnova 2.4x may be accessed as ",
							MACANOVA2AVAIL, Macanova2Msg);
#endif /*MACANOVA2AVAIL*/

#ifdef MACANOVA31AVAIL
	*message++ = makeMsgLine("MacAnova 3.1x may be accessed as ",
							MACANOVA31AVAIL, Macanova31Msg);
#endif /*MACANOVA31AVAIL*/

#ifdef BACKUPS
	*message++ = makeMsgLine("Past versions of MacAnova 4.0x are ",NullString,
						  BackupMsg1);

	*message++ = makeMsgLine(BACKUPS,NullString,BackupMsg2);
#endif /*BACKUPS*/
	*message = (char *) 0;

	if (!quiet)
	{
#ifdef MACINTOSH
		DoApple(-aboutmacanova);
#endif /*MACINTOSH*/
		message = Welcome;
		length =  (length >= BUFFERLENGTH) ? BUFFERLENGTH-1 : length;

		while(*message != (char *) 0)
		{
			(void) centerBuffer(OUTSTR, *message, length);
			putOUTSTR();
			if(message++ == Welcome)
			{					/* extra line after first */
				myeol();
			}
		} /*while(*message != (char *) 0)*/
#ifdef SKEL_CARAPACE
#define MYCENTEROUT(A)  (void) centerBuffer(OUTSTR,A,length);putOUTSTR();
		myeol();
		MYCENTEROUT("This release of MacAnova has a completely rewritten user interface.");
		MYCENTEROUT("You won't see a lot of new capabilities in this first release, but new");
		MYCENTEROUT("capabilities are on the way. The computational core has not changed.");
		myeol();
		MYCENTEROUT("The changes that you will notice now are that you type commands");
		MYCENTEROUT("into the lower pane of the window, and graphics look somewhat different.");
		MYCENTEROUT("Also, you will find that you can do many things from menus.  We are still");
		MYCENTEROUT("working on the menus, and future versions will be more complete.");
		myeol();
		MYCENTEROUT("MacAnova now has some dynamic graphics. However, they're");
		MYCENTEROUT("still a little buggy (we're working on it!), so you may with to");
		MYCENTEROUT("make sure that you have saved before using them.");
		myeol();
		MYCENTEROUT("Please let us know if you find any problems.  Happy computing!");
		MYCENTEROUT("gary@stat.umn.edu and kb@stat.umn.edu");
#endif /* SKEL_CARAPACE */
	} /*if (!quiet)*/	
} /*mvStartupMsg()*/


