
# line 2 "mainpars.y"
/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 3.35 or later
*(C)*
*(C)* Copyright (c) 1994 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

#ifdef MPW
#ifdef MPW3
#pragma segment Mainpars
#else
#define __SEG__ Mainpars
#endif /*MPW3*/
#endif /*MPW*/

/*
  main parser for macanova by G. Oehlert, as modifed by C. Bingham for use
  with MacAnova 3.0
*/

/*
	Version of 920623 made many changes, here and in yylex.c and Symbol.c
	All names are returned by yylex as VAR unless they are BLTINs or MACROs
	followed by '('.  Thus the parser should never see STRUC, UNDEF, and should
	never see MACRO or BLTIN or BATCH unless followed by '('
	The number of shift/reduce conflicts has been reduced to 13

	930201 Added 'break n' functionality which also is associated with token
	BREAK, and modified yylex so that 'breakall' is equivalent to
	'break WDEPTH'.  When n is a literal positive integer, 'break n' results
	in macanova	breaking out of exactly n enclosing loops.

	930208 Added a[..][..] capability, now 17 shift/reduce errors

	930804 Added operators <-+ <-- <-*, <-/, <-^, <-%%; 23 shift/reduce errors

	931007 Added lines to make use of batch() as an expression illegal.

	931103 Added error checking for left side of <-+, <--, etc; 47 S/R errors

	931110 Communication with Ifsetup() now by return value rather than global

	931228 Found that mainpars.c as directly produced was too large for the
           Mac MPW compiler.  Accordingly, rearranged various cases and used
		   more gotos, to maximize the length of sequences of case statements
           with identical code that could be folded into one another by
           editing of mainpars.c for use on the Macintosh.  The first case
           of a set of, say 3, cases to be folded has a comment of the
		   form 'Mac: fold 3 cases'. Script mainpars.awk for awk uses this
		   to create mainpars.c.mac.new.  This also has the parser tables
		   edited out and put into file mainpars.r.mac.new in a form readable
		   by MPW resource compiler Rez.

		   Note: Some of the folded cases are not identical in mainpars.y, but
		   are translated into identical code by Yacc.  Conversely, some
		   cases that appear to be identical in mainpars.y are translated
		   by Yacc in different ways.
	
	940110 Fixed bug in byname BATCH parenlist

	940119 Found and fixed bug.  yyparse() was telling yylex() to send an
	unnecessary (and harmful) ';' after if(l){..}else{...} and after
	while(l){...} .

	940331 Added bitwise operations BITAND, BITXOR, BITOR, and BITNOT

	940411 Added line adding  component ASSIGN  to definition of 'assign',
	in order to generate useful error message.
	Now 48 shift/reduce conflicts

	940630 Implemented if{...}elseif{...}...elseif{...}else{...}
	Now 49 shift/reduce

	941108 Changed precedence of '!' to be lower than EQ, NE, etc

VERSION: 941115   This date is used by mainpars.awk in making macMainpars.r

*/

#include "globals.h"

#ifdef MACINTOSH

#define CHECKINTERRUPT	if(INTERRUPT != INTNOTSET) YYERROR

#else /*MACINTOSH*/

#define CHECKINTERRUPT

#endif /*MACINTOSH*/

/* these must be defined identically in yylex.c */
#define SKIPBRACKETBLOCK  3
#define SKIPPARENBLOCK    4

#define YYSTYPE Symbolhandle 	/* handle to symbol */
#define YYDEBUG 1

extern Symbolhandle Install(), Extract();
extern Symbolhandle Makelist(), Addlist(), Growlist();
extern Symbolhandle Assign(), Subassign(), Arith();
extern Symbolhandle Unary(), Logic(), Element();
extern Symbolhandle Byname();
extern void         Removelist();
extern void         Unscratch();
extern void         RemoveUndef();

void                cleanitup();

#ifdef MACINTOSH
extern short *Pyyexca;
extern short *Pyyact;
extern short *Pyypact;
extern short *Pyypgo;
extern short *Pyyr1;
extern short *Pyyr2;
extern short *Pyychk;
extern short *Pyydef;
#endif /*MACINTOSH*/

static void         setupfakefor();
static long         BynameType;
static long         ParseMacroArgs;
static int          Op, AssignOp;
static int          TargetLevel;

#ifdef UNDEFINED__
static WHERE("yyparse"); /*now set by make*/
#endif /*UNDEFINED__*/

long    PARSEMACROARGS = 0;  /* for communication with yylex */
long    YYLEXMESSAGE = 0;  /* for communication with yylex */
long    LASTCH = '\0';  /* for communication with yylex */
int     ALTINPUTTOKENS[5*MAXWDEPTH];  /* for communication with yylex */
int    *ALTTOKENPTR;  /* for communication with yylex */
# define CHAR 257
# define REAL 258
# define BLTIN 259
# define LIST 260
# define STRUC 261
# define MACRO 262
# define UNDEF 263
# define VAR 264
# define END 265
# define LOGIC 266
# define MACRO1 267
# define NUMBER 268
# define IF 269
# define ELSE 270
# define ELSEIF 271
# define WHILE 272
# define FOR 273
# define BREAK 274
# define BREAKALL 275
# define BATCH 276
# define ERROR 277
# define FATALERROR 278
# define PLOTINFO 279
# define ASSIGNED 280
# define GARB 281
# define LONG 282
# define BADTOKEN 283
# define LEFTANGLE 284
# define RIGHTANGLE 285
# define LEFTBRACK2 286
# define RIGHTBRACK2 287
# define MULT 288
# define ADD 289
# define DIV 290
# define MOD 291
# define SUB 292
# define EXP 293
# define GE 294
# define LE 295
# define GT 296
# define LT 297
# define EQ 298
# define NE 299
# define AND 300
# define OR 301
# define NOT 302
# define ENDOFL 303
# define POW 304
# define MATMULT 305
# define TRMATMULT 306
# define MATMULTTR 307
# define BITAND 308
# define BITXOR 309
# define BITOR 310
# define BITNOT 311
# define ASSIGN 312
# define ASSIGN1 313
# define ASSIGNMULT 314
# define ASSIGNADD 315
# define ASSIGNDIV 316
# define ASSIGNSUB 317
# define ASSIGNPOW 318
# define ASSIGNMOD 319
# define UNARYMINUS 320
#define yyclearin yychar = -1
#define yyerrok yyerrflag = 0
extern int yychar;
extern short yyerrflag;
#ifndef YYMAXDEPTH
#define YYMAXDEPTH 150
#endif
#ifndef YYSTYPE
#define YYSTYPE int
#endif
YYSTYPE yylval, yyval;
# define YYERRCODE 256

# line 1018 "mainpars.y"

static int          SeenError = 0;

void yyerror(s)
char           *s;
{
	/*  output error message */
	unsigned char   c = '\0', *inputString = *INPUTSTRING;
	int             start = 0;
	int             istrchar = ISTRCHAR;
	int             iswarning = (strncmp("WARNING:", s, 8) == 0);
	WHERE("yyerror");

	if (!SeenError && (PRINTWARNINGS || !iswarning))
	{
		while(istrchar >= 0)
		{
			if((c = inputString[istrchar]) == '#' || c == '"')
			{
				break;
			}
			istrchar--;
		}
		if(c == '#')
		{
			ISTRCHAR = istrchar;
		}
		
		myprint(s);

		myprint(" near ");
		if(strlen(s) + 6 + ISTRCHAR > SCREENWIDTH)
		{
			myeol();
		}
		
		if(ISTRCHAR > SCREENWIDTH && !(GUBED & 2))
		{
			start = ISTRCHAR - (SCREENWIDTH - 5);
			start = (start >= 0) ? start : 0;
			myprint(". . .");
		}

		c = inputString[ISTRCHAR];
		inputString[ISTRCHAR] = '\0';
		mymultiprint((char *) inputString + start);
		inputString[ISTRCHAR] = c;
		SeenError = !iswarning;
	} /*if (!SeenError && (PRINTWARNINGS || !iswarning))*/
	
	if(s == OUTSTR)
	{
		*OUTSTR = '\0';
	}
} /*yyerror()*/

/* tell yylex to report  ';for()' to the parser */
static int fakefortokens[] = {(int) ';', FOR, (int) '(', (int) ')', 0 };
static void setupfakefor(void)
{
	int   i;

	for(i=0;fakefortokens[i];i++)
	{
		ALTINPUTTOKENS[i] = fakefortokens[i];
	 }
	ALTINPUTTOKENS[i] = 0;
}

static void notfunction(Symbolhandle symh)
{
	if(isscratch(NAME(symh)))
	{
		yyerror("syntax error");
	}
	else
	{
		sprintf(OUTSTR,"ERROR: %s is not function or macro",NAME(symh));
		yyerror(OUTSTR);
		*OUTSTR = '\0';
	}
}

/*
   Clean up looping related globals
*/
static void cleanLoops(long level1, long level2)
{
	int      i;
	
	for(i=level1;i<level2;i++)
	{
		WHILECONDITIONS[i] = 0;
		WHILELIMITS[i] = MAXWHILE;
		mydisphandle((char **) FORVECTORS[i]);
		FORVECTORS[i] = (double **) 0;
	}
}

static Symbolhandle runCommand(Symbolhandle func, Symbolhandle list)
{
	Symbolhandle      result;
	
	strcpy(FUNCNAME,NAME(func));
	result = (*(FPTR(func)))(list);
#ifdef MACINTOSH
	UNLOADSEG(*(FPTR(func)));
#endif /*MACINTOSH*/
	Removelist(list);
	return (result);
}

/*
  Called from main before every invocation of getinput.
  It needs to be here because it resets PARSEMACROARGS, ALTTOKENPTR, and
  ALTINPUTTOKENS which are defined only here and in yylex.c
  It also resets static variable SeenError
*/

/* global for communication with Assign() in Symbol.c */
extern int     Nassigned;

void cleanitup(void)
{
	WHERE("cleanitup");

	PRINT1(4096,
		   "Nscratch = %ld, Maxscratch = %ld, Nsymbols = %ld, Maxsymbols = %ld\n",
		   Nscratch, Maxscratch, Nsymbols, Maxsymbols);
	PRINT1(4096,
		   "Nhandles = %ld, Maxhandles = %ld, Nassigned = %ld\n",
		   Nhandles, Maxhandles, Nassigned, 0);
	PRINT1(4096,
		   "CurrentMemory = %ld, MaxMemory = %ld\n",
		   CurrentMemory, MaxMemory, 0, 0);

	SeenError = 0;

	Unscratch(); /* clears scratch symbol table (SCRATCH, keywords, UNDEF's)*/
	RemoveUndef(); /* clears ASSIGNED && temp variables from symbol table */
	Nassigned = 0;

	INTERRUPT = INTNOTSET;
	*OUTSTR = '\0';

	if(BDEPTH == 0)
	{
		NLINES = 0;
	}

	if(ISATTY && BDEPTH == 0 && MAXERRORS == 0)
	{
		NERRORS = 0;
	}

	restoreFormat(); /* restore current default format */

	BRACKETLEV = 0;
	PARENLEV = 0;
	MDEPTH = 0;
	WDEPTH = 0;
	IDEPTH = 0;
	MLEVEL = -1;
	PARSEMACROARGS = 0;
	ALTTOKENPTR = (int *) 0;
	ALTINPUTTOKENS[0] = 0;
	cleanLoops(0, MAXWDEPTH);
}
short yyexca[] ={
-1, 1,
	0, -1,
	-2, 0,
-1, 175,
	294, 0,
	295, 0,
	298, 0,
	299, 0,
	61, 0,
	60, 0,
	62, 0,
	-2, 75,
-1, 176,
	294, 0,
	295, 0,
	298, 0,
	299, 0,
	61, 0,
	60, 0,
	62, 0,
	-2, 76,
-1, 178,
	294, 0,
	295, 0,
	298, 0,
	299, 0,
	61, 0,
	60, 0,
	62, 0,
	-2, 77,
-1, 180,
	294, 0,
	295, 0,
	298, 0,
	299, 0,
	61, 0,
	60, 0,
	62, 0,
	-2, 78,
-1, 182,
	294, 0,
	295, 0,
	298, 0,
	299, 0,
	61, 0,
	60, 0,
	62, 0,
	-2, 79,
-1, 184,
	294, 0,
	295, 0,
	298, 0,
	299, 0,
	61, 0,
	60, 0,
	62, 0,
	-2, 80,
-1, 186,
	294, 0,
	295, 0,
	298, 0,
	299, 0,
	61, 0,
	60, 0,
	62, 0,
	-2, 81,
-1, 213,
	313, 175,
	-2, 57,
-1, 277,
	313, 176,
	-2, 58,
	};
# define YYNPROD 207
# define YYLAST 1384
short yyact[]={

  10, 300, 110, 221, 219, 222, 220, 224, 223, 281,
 283, 221, 219, 222, 220, 224, 223, 276, 247, 113,
 221, 219, 222, 220, 224, 223, 136, 310, 285,  43,
 138, 140, 142, 295, 144, 263, 315, 148, 152,   1,
 217, 246, 221, 219, 222, 220, 224, 223, 101, 207,
 104, 102, 105, 103, 107, 106,  73,  98,  36,  98,
 322, 130, 131, 211, 268, 135, 126,  73, 132, 127,
  98, 277, 319, 318, 317, 316, 154, 156, 158, 160,
 162, 164, 166, 168, 170, 172, 173, 174, 175, 176,
 178, 180, 182, 184, 186, 188, 190, 192, 194, 314,
 258, 293, 292, 199, 200, 201, 202, 203, 204, 126,
 126, 210, 127, 127, 124, 124, 289, 288, 286, 110,
 113, 138, 140, 142, 196, 112, 125,  15, 197,  73,
 210,  94,  98, 297, 272,  77,  75, 214,  76, 129,
  81,   5,   4, 270,  41,  40, 115, 212, 305, 255,
 261,  26,  71,  90,  87,  92, 257, 256, 254,  73,
  22, 262,  98,  38,  34, 228, 226,  32, 227,  74,
 232, 303, 145,  20, 280,  20, 110,  19, 312, 253,
 278,  16, 198,  73, 100,  94,  98, 218,  11,  77,
  75,  20,  76,   3,  81,   2,   0, 264,  20, 267,
   0,   0,  20,  20,  20,   0,  71,  90,  87,  92,
   0, 151,   0, 273,   0,   0,   0,  96, 259,   0,
   0, 311,   0, 108,   0,   0,   0, 154, 156, 158,
 160, 162, 164, 166, 168, 170, 176, 178, 180, 182,
 184, 186, 188, 190, 192, 194, 153, 115,  20,  20,
  20,  20,  20,  20,  20,  20,  20,   0, 269,   0,
   0,  20,  20,  20,  20,  20,  20,  20,  20,  20,
  20,  96, 284,  42,   0,   0, 274, 298,   0, 299,
   0, 302,   0, 216,  67, 245,  61,   0,  65,   0,
  48,  46,   0,  47, 101,  52, 104, 102, 105, 103,
 107, 106, 216,   0, 313, 294,  99,  45,  57,   0,
  59, 248,   0, 320,   0,   0,  73,  99, 241,  98,
   0,   0, 228, 226, 296, 227, 304, 232, 306, 307,
   0,   0, 308, 309,   0, 234,   0,   0,   0,  66,
 237,  87, 239,  73,   0, 241,  98,   0,   0, 228,
 226, 271, 227,   0, 232, 321,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0, 237,  87, 239,
  72,   0,  63,   0,   0,   0,   0,   0,   0,  99,
   0,   0,   0,   0,  82, 216,   0,  93,  91,   0,
   0,  88,  89,  95,  97,   0,  70,  83,  78,  79,
  80,  86,  85,  84, 243,   0,   0,   0,   0,  99,
  73,   0, 241,  98, 233,   0, 228, 226,   0, 227,
   0, 232,   0,   0,  72,   0,   0, 234, 229, 230,
 231, 243,   0,  99, 237,  87, 239, 282,  82,   0,
   0,  93,  91,   0,   0,  88,  89,  95,  97,   0,
  70,  83,  78,  79,  80,  86,  85,  84,   0,  20,
   0,  20,  20,   0,   0,  20,  20, 301,  73, 111,
 241,  98,   0, 225, 228, 226,   0, 227,   0, 232,
   0,   0,   0,   0,   0,   0, 109,   0,  20, 123,
   0,   0, 237,  87, 239,   0, 137,   0, 243,   0,
   0,   0,   0,  73, 146, 241,  98,   0,   0, 228,
 226,   0, 227,   0, 232,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0, 237,  87, 239,
   0,   0,   0,   0,   0,  68,   0,   0,   0,  53,
   0,   0,  60,  58,   0,   0,  55,  56,  62,  64,
   0,  44,  54,  49,  50,  51, 243,  31,   0,   0,
  69,   0,   0,   0,  18,   0,  99,  30,   0,  29,
   0, 233,   0,   0, 240, 238,   0,   0, 235, 236,
 242, 244,   0,   7, 234, 229, 230, 231,  86,  85,
  84, 243,   0,  99, 249,   0,   0,   0, 233,   0,
   0, 240, 238,   0,   0, 235, 236, 242, 244,   0,
   0, 234, 229, 230, 231,  86,  85,  84,  73,   0,
 241,  98,   0,   0, 228, 226,   0, 227,   0, 232,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0, 237,  87, 239,   0,   0,  25,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0, 260,
  99,   0,   0,   0,   0, 233,   0,   0, 240, 238,
   0,   0, 235, 236, 242, 244,   0,   0, 234, 229,
 230, 231,  86,  85,  84,   0,   0,   0,   0,   0,
  73,   0, 241,  98,   0, 279, 228, 226,   0, 227,
   0, 232,   0,   0,   0,   0, 243,   0,   0,   0,
   0,   0,   0,   0, 237,  87, 239,   0,  99,   0,
   0, 128,   0, 233,   0,   0, 240, 238, 290, 291,
 235, 236, 242, 244,   0,   0, 234, 229, 230, 231,
  86,  85,  84,   0, 122, 133, 134, 249,   0,   0,
   0, 119,   0,  99, 121, 215, 120, 147, 233, 149,
 150, 240, 238,   0,   0, 235, 236, 242, 244,   0,
 208, 234, 229, 230, 231,  86,  85,  84, 243,   0,
  13,  73,   0,  35,  98,   0,  39, 228,  17,  27,
   0,   0, 232,  37,   0,   0,  23,  24,  14,   0,
  21,  12,  28,   0, 205,   0,   0,  73,  36, 241,
  98,   0,   0, 228, 226,   0, 227,   0, 232,   0,
   0,   0,   0,   0,   0,   0,   0,   6,   0,   0,
   0, 237,  87, 239,  25,  33,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0, 250, 251,   0,
   0,   0,   0, 122,   0,   0,   0,   0,   0,   0,
 119,   0,   0, 121,   0, 120,   0,   0,  99,   0,
   0,   0,   0, 233,   0,   0, 240, 238,   0, 116,
 235, 236, 242, 244,   0,   0, 234, 229, 230, 231,
  86,  85,   0,   0,   0, 243, 122,   0,   0,   0,
   0,   0,   0, 119, 118,   8, 121,   8, 120,  73,
   0, 241,  98,   0,   0, 228, 226,   0, 227,   0,
 232,   0, 265,   0,   0,   0,   0,   0,   0,   0,
   8,   0,   0, 237,  87, 239,   0,   0,   0,   0,
  99,   0,   0,  25,   0, 233,   0,   0, 240, 238,
   0,   0, 235, 236, 242, 244, 122,   0, 234, 229,
 230, 231,  86, 119, 252,   0, 121, 215, 120,   0,
  35,   0,   0,  39,   0, 213,  27, 287,   0,   0,
  37,   0,   0,  23,  24,   0,  25,   0,   0,  28,
 122,   0,   0,   0,   0,  36,   0, 119, 206,   0,
 121,   0, 120,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0, 122, 209,   0, 116,   0,   0,   0,
 119,   0,  33, 121, 275, 120,   0,   0,   0,   0,
 122,  99,   0,   0,   0,   0, 233, 119,   0,   0,
 121,   0, 120,   0,   0,   0,  25,   0, 122, 234,
 229, 230, 231,   0,   0, 119,   0,  99, 121,   0,
 120,   0, 233,   0,   0, 240, 238,   0,   0, 235,
 236, 242, 244,   0,   0, 234, 229, 230, 231,  35,
  25,   0,  39,   0,  17,  27,   0,   0,  73,  37,
   0,  98,  23,  24, 228, 226,  21, 227,  28, 232,
   0,   0,   0,  25,  36,   0,   0,   0,   0,   0,
   0,   0, 237,  87, 239,   0,   0,   0,   0,   0,
  25,   0,  35, 114,   0,  39,   0,  17,  27,   0,
   0,  33,  37,   0,   0,  23,  24,   0,  25,   0,
   0,  28,   0,   0,   0,   0,   0,  36,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,  99,
   0,   0,   0,   0, 233,   0, 266, 240, 238,   0,
   0, 235, 236, 242,  33,   0,   0, 234, 229, 230,
 231,   0,  35,   0,   0,  39,   0, 213,  27,   0,
   0,   8,  37,   8,   8,  23,  24,   8,   8,   0,
   0,  28,   0,   0,   0,   0,   0,  36,   0,   0,
   0,   0,   0,   0,   0,   0,  35,   0,   0,  39,
   8,  17,  27,   0,   0,   0,  37,   0,   0,  23,
  24,   0,   0,   0,  33,  28,   0,   0,   0,  35,
   0,  36,  39,   0, 213,  27,   0,   0,   0,  37,
   0,   0,  23,  24,   0,   0,  35,   0,  28,  39,
 114,  17,  27,   0,  36,   0,  37,   0,  33,  23,
  24,   0,   0,  21,  35,  28,   0,  39,   0,  17,
  27,  36,   0,   0,  37,   9,   0,  23,  24,   0,
   0,  33,   0,  28,   0,   0,   0,   0,   0,  36,
   0,   0,   0,   0, 117,   0,   0,   0,  33,   0,
   0,   0,   0,   0,   0, 139, 141, 143,   0,   0,
   0,   0,   0,   0,   0,   0,  33,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,  99,   0,
   0,   0,   0, 233,   0,   0, 240, 238,   0,   0,
 235, 236,   0,   0,   0,   0, 234, 229, 230, 231,
   0, 155, 157, 159, 161, 163, 165, 167, 169, 171,
   0,   0,   0,   0, 177, 179, 181, 183, 185, 187,
 189, 191, 193, 195 };
short yypact[]={

 524,-1000, 524,-1000,-1000,-1000,-1000,-1000, -30, 248,
 147,-1000,-1000,-1000,-1000,-264, -89,  85, 820,  28,
-150,  99,-209,  99,  99, 524,  28,-1000,-1000, 997,
 997, 997,-1000,1015,  28,  99,1015,  99,  99,-1000,
-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,
-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,
-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,
-1000,-1000,-1000,-226,-1000, 997, 997, 997, 997, 997,
 997, 997, 997, 997,1015,1015,1015,1015, 997, 997,
 997, 997, 997, 997, 997, 997, 997, 997,-1000,  33,
  69,-1000,1015,1015,1015,1015,1015,1015,-1000,-1000,
 711,-272,-1000, 432,-1000, -18,-1000,-1000,-1000, 957,
1015,1015,1015,-294,  28,  99,  99,-1000,-1000, 923,
-1000,-1000,-1000,-1000,-1000, -25,  93,-1000,  31,-1000,
  31,-1000,1052,-1000, 771,-149,-1000,-1000, 374,-1000,
-1000,-1000,-1000,-229, 745,-1000, 745,-1000,  31,-1000,
  31,-1000,  31,-1000,  31,-1000,  31,-1000,  31,-1000,
  20,-1000, 582, 654, 771, 123, 123,-1000, 123,-1000,
 123,-1000, 123,-1000, 123,-1000, 123,-1000,1052,-1000,
1052,-1000, 873,-1000, 873,-1000, 863,-1000,1015, 467,
 467, 467, 467, 467, 467,-1000,-1000, -29,-1000,-1000,
 307,  90, 980,  85,-296,-1000,-193,-1000,-1000,-1000,
-1000,-1000,-1000,-1000,-1000,  28,1015,1015,1015,1015,
1015,1015,1015,1015,1015,1015,1015,1015,1015,1015,
1015,1015,1015,1015,1015,-1000,-1000,-1000,-1000,-303,
-1000,-1000,-1000, -31,  -5,  99,  -6,  -7,  28,  28,
-1000, -21, -22,-1000,  18,-1000,-1000, 467,-1000,-1000,
-244,-1000,-1000, 280,  89,-1000,1015,  28,1015,-311,
1015,-1000,-1000,-1000,-1000,-1000, 524,-1000, 524, 524,
-1000,-1000, 524, 524, -66,-1000,-1000,-1000, 467, 467,
-1000,-1000, 467,1015, -26, -87, -50, -51, -52, -53,
-1000,-1000,1015, 467,-1000, 524,-1000,-1000,-1000,-1000,
 467, -65,-1000 };
short yypgo[]={

   0,  39, 195, 193, 142, 141, 904,1285,   0, 188,
 184, 127, 182, 181, 469, 180, 178, 177, 174, 172,
 171, 187, 721,  63, 137, 169, 167, 164, 163, 161,
 160, 158, 157, 156, 151, 150, 149, 148, 147,  49,
 143 };
short yyr1[]={

   0,   1,   1,   2,   2,   3,   3,   4,   4,   4,
   4,   4,   4,   4,   4,   4,   4,   4,   4,  10,
   5,  12,   6,   6,  15,   6,  16,   6,  18,   6,
  20,   6,   6,   6,   6,   6,   6,   6,   6,   6,
   6,   6,  21,  21,  21,  21,  21,  21,   7,   7,
  23,  17,  17,  13,  13,  11,  11,  11,  11,  11,
  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,
  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,
  11,  11,  11,  11,  11,  11,  11,  11,  11,  29,
  27,  26,  26,  26,  26,   8,   8,   8,   8,  31,
   8,  32,   8,  33,   8,   8,   8,   8,   8,   8,
   8,   9,   9,   9,   9,   9,   9,   9,   9,   9,
   9,   9,   9,   9,   9,   9,   9,   9,   9,   9,
   9,   9,   9,   9,   9,   9,   9,   9,   9,   9,
   9,   9,   9,   9,   9,   9,   9,   9,   9,   9,
   9,   9,   9,   9,   9,   9,   9,   9,  34,  34,
  34,  34,  34,  34,  34,  34,  34,  35,  30,  36,
  37,  30,  30,  28,  28,  24,  24,  19,  19,  38,
  38,  38,  38,  38,  38,  39,  39,  39,  39,  39,
  22,  22,  22,  22,  14,  14,  14,  14,  14,  14,
  40,  14,  25,  25,  25,  25,  25 };
short yyr2[]={

   0,   1,   1,   1,   2,   1,   2,   1,   1,   2,
   2,   2,   2,   2,   2,   1,   2,   1,   1,   0,
   3,   0,   4,   2,   0,   5,   0,   7,   0,   5,
   0,   6,   3,   3,   3,   3,   3,   3,   5,   3,
   4,   3,   1,   1,   1,   1,   1,   1,   2,   3,
   3,   2,   3,   3,   2,   1,   1,   1,   2,   3,
   3,   3,   3,   3,   3,   3,   3,   3,   3,   3,
   3,   3,   2,   2,   2,   3,   3,   3,   3,   3,
   3,   3,   3,   3,   3,   3,   2,   2,   1,   0,
   6,   2,   3,   2,   3,   1,   1,   1,   1,   0,
   6,   0,   6,   0,   6,   3,   3,   1,   2,   1,
   1,   3,   3,   3,   3,   3,   3,   3,   3,   3,
   2,   2,   2,   2,   2,   2,   2,   2,   2,   2,
   2,   3,   3,   3,   3,   3,   3,   3,   3,   3,
   3,   2,   2,   2,   2,   2,   2,   2,   2,   2,
   2,   2,   2,   2,   2,   2,   2,   2,   4,   4,
   4,   2,   2,   2,   2,   2,   3,   0,   6,   0,
   0,   8,   2,   1,   2,   1,   2,   3,   2,   1,
   2,   2,   3,   3,   2,   1,   1,   1,   2,   2,
   2,   3,   3,   3,   2,   2,   3,   3,   2,   2,
   0,   4,   5,   5,   3,   3,   3 };
short yychk[]={

-1000,  -1,  -2,  -3,  -4,  -5, 303,  59,  -6,  -7,
  -8,  -9, 277, 256, 274, -11, -13, 264,  40, -17,
 -19, 276, -30, 272, 273, 123, -34, 265, 278,  45,
  43,  33, -26, 311, -27, 259, 284, 269, -28, 262,
  -4,  -5, 303,  59, 303,  59,  43,  45,  42, 305,
 306, 307,  47, 291, 304, 298, 299,  60, 295,  62,
 294,  38, 300, 124, 301,  40,  91,  36, 287, 312,
 303,  59, 277,  36, -25,  43,  45,  42, 305, 306,
 307,  47, 291, 304, 310, 309, 308,  61, 298, 299,
  60, 295,  62, 294,  38, 300, 124, 301,  39, 286,
 -10, 312, 315, 317, 314, 316, 319, 318, 312, -14,
  91, -14,  40,  -8, 303, -11,  59,  -7,  -6,  40,
  45,  43,  33, -14, 264, 276, 259, 262, -22,  40,
 270, 271, 277, -22, -22,  -1,  -8, -14,  -8,  -7,
  -8,  -7,  -8,  -7,  -8, -19, -14, -22,  -8, -22,
 -22, -24, 264, -19,  -8,  -7,  -8,  -7,  -8,  -7,
  -8,  -7,  -8,  -7,  -8,  -7,  -8,  -7,  -8,  -7,
  -8,  -7,  -8,  -8,  -8,  -8,  -8,  -7,  -8,  -7,
  -8,  -7,  -8,  -7,  -8,  -7,  -8,  -7,  -8,  -7,
  -8,  -7,  -8,  -7,  -8,  -7,  91,  59, -12,  -8,
  -8,  -8,  -8,  -8,  -8,  93, 287, -39,  59, 303,
  -8, -23, -38, 264, -24,  44, -19, 312, -21, 315,
 317, 314, 316, 319, 318,  41,  43,  45,  42, 305,
 306, 307,  47, 291, 304, 298, 299,  60, 295,  62,
 294,  38, 300, 124, 301, 303,  59, 312, -21, -14,
 -22, -22,  41, -39, -31, -36, -32, -33, 125, 125,
 285, -35, -29, 264,  -8,  59, 303,  -8,  93, 287,
 -40,  44,  44,  -8, -23,  44, 313, 264, -15, -14,
 -18, 312, -21,  41, 303,  59, 123, -22, 123, 123,
 -14, -14, 123, 123, 287, 277,  44,  44,  -8,  -8,
 312, -21,  -8, -20,  -1, -37,  -1,  -1,  -1,  -1,
  93, 287, -16,  -8, 125, 123, 125, 125, 125, 125,
  -8,  -1, 125 };
short yydef[]={

   0,  -2,   1,   2,   3,   5,   7,   8,  97,   0,
   0,  15,  17,  18,  19,  95,  96,  57,   0,  55,
   0,   0,  98,   0,   0,   0, 107, 109, 110,   0,
   0,   0,  56,   0,  88,   0,   0,   0,   0, 173,
   4,   6,   9,  10,  11,  12, 120, 121, 122, 123,
 124, 125, 126, 127, 128, 141, 142, 143, 144, 145,
 146, 147, 148, 149, 150, 153, 154, 155, 156, 157,
  13,  14,  16,   0,  54,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,  87,   0,
   0,  21,   0,   0,   0,   0,   0,   0,  23, 162,
   0, 164, 108,   0,  91,  95,  93, 152,  97,   0,
   0,   0,   0, 161,  58,   0,   0, 174,  48,   0,
  99, 169, 172, 101, 103,   0,   0, 165,  72, 129,
  73, 130,  86, 151,  74,   0, 163,  51, 178, 167,
  89,  53, 175,   0,  59, 111,  60, 112,  61, 113,
  62, 114,  63, 115,  64, 116,  65, 117,  66, 118,
  67, 119,  68,  69,  70,  -2,  -2, 131,  -2, 132,
  -2, 133,  -2, 134,  -2, 135,  -2, 136,  82, 137,
  83, 138,  84, 139,  85, 140,   0,  20,   0,  32,
  33,  34,  35,  36,  37, 194, 195, 200, 198, 199,
 185, 186, 187,  -2,   0, 179,   0,  24,  41,  42,
  43,  44,  45,  46,  47,  71,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,  92,  94,  28,  39, 166,
  49,  52, 190,   0,   0,   0,   0,   0, 105, 106,
 177,   0,   0, 176, 206, 204, 205,  22, 196, 197,
   0, 180, 181, 188, 189, 184,   0,  -2,   0, 160,
   0,  30,  40, 191, 192, 193,   0, 170,   0,   0,
 159, 158,   0,   0,   0, 201, 182, 183,  50,  25,
  26,  38,  29,   0,   0,   0,   0,   0,   0,   0,
 202, 203,   0,  31, 100,   0, 102, 104, 168,  90,
  27,   0, 171 };
#ifndef lint
static char yaccpar_sccsid[] = "@(#)yaccpar	4.1	(Berkeley)	2/11/83";
#endif

#
# define YYFLAG -1000
# define YYERROR goto yyerrlab
# define YYACCEPT return(0)
# define YYABORT return(1)

/*	parser for yacc output	*/

#ifdef YYDEBUG
int yydebug = 0; /* 1 for debugging */
#endif
YYSTYPE yyv[YYMAXDEPTH]; /* where the values are stored */
int yychar = -1; /* current input token number */
int yynerrs = 0;  /* number of errors */
short yyerrflag = 0;  /* error recovery flag */

yyparse() {

	short yys[YYMAXDEPTH];
	short yyj, yym;
	register YYSTYPE *yypvt;
	register short yystate, *yyps, yyn;
	register YYSTYPE *yypv;
	register short *yyxi;

	yystate = 0;
	yychar = -1;
	yynerrs = 0;
	yyerrflag = 0;
	yyps= &yys[-1];
	yypv= &yyv[-1];

 yystack:    /* put a state and value onto the stack */

#ifdef YYDEBUG
	if( yydebug  ) printf( "state %d, char 0%o\n", yystate, yychar );
#endif
		if( ++yyps> &yys[YYMAXDEPTH] ) { yyerror( "yacc stack overflow" ); return(1); }
		*yyps = yystate;
		++yypv;
		*yypv = yyval;

 yynewstate:

	yyn = yypact[yystate];

	if( yyn<= YYFLAG ) goto yydefault; /* simple state */

	if( yychar<0 ) if( (yychar=yylex())<0 ) yychar=0;
	if( (yyn += yychar)<0 || yyn >= YYLAST ) goto yydefault;

	if( yychk[ yyn=yyact[ yyn ] ] == yychar ){ /* valid shift */
		yychar = -1;
		yyval = yylval;
		yystate = yyn;
		if( yyerrflag > 0 ) --yyerrflag;
		goto yystack;
		}

 yydefault:
	/* default state action */

	if( (yyn=yydef[yystate]) == -2 ) {
		if( yychar<0 ) if( (yychar=yylex())<0 ) yychar = 0;
		/* look through exception table */

		for( yyxi=yyexca; (*yyxi!= (-1)) || (yyxi[1]!=yystate) ; yyxi += 2 ) ; /* VOID */

		while( *(yyxi+=2) >= 0 ){
			if( *yyxi == yychar ) break;
			}
		if( (yyn = yyxi[1]) < 0 ) return(0);   /* accept */
		}

	if( yyn == 0 ){ /* error */
		/* error ... attempt to resume parsing */

		switch( yyerrflag ){

		case 0:   /* brand new error */

			yyerror( "syntax error" );
		yyerrlab:
			++yynerrs;

		case 1:
		case 2: /* incompletely recovered error ... try again */

			yyerrflag = 3;

			/* find a state where "error" is a legal shift action */

			while ( yyps >= yys ) {
			   yyn = yypact[*yyps] + YYERRCODE;
			   if( yyn>= 0 && yyn < YYLAST && yychk[yyact[yyn]] == YYERRCODE ){
			      yystate = yyact[yyn];  /* simulate a shift of "error" */
			      goto yystack;
			      }
			   yyn = yypact[*yyps];

			   /* the current yyps has no shift onn "error", pop stack */

#ifdef YYDEBUG
			   if( yydebug ) printf( "error recovery pops state %d, uncovers %d\n", *yyps, yyps[-1] );
#endif
			   --yyps;
			   --yypv;
			   }

			/* there is no state on the stack with an error shift ... abort */

	yyabort:
			return(1);


		case 3:  /* no shift yet; clobber input char */

#ifdef YYDEBUG
			if( yydebug ) printf( "error recovery discards char %d\n", yychar );
#endif

			if( yychar == 0 ) goto yyabort; /* don't discard EOF, quit */
			yychar = -1;
			goto yynewstate;   /* try again in the same state */

			}

		}

	/* reduction by production yyn */

#ifdef YYDEBUG
		if( yydebug ) printf("reduce %d\n",yyn);
#endif
		yyps -= yyr2[yyn];
		yypvt = yypv;
		yypv -= yyr2[yyn];
		yyval = yypv[1];
		yym=yyn;
			/* consult goto table to find next state */
		yyn = yyr1[yyn];
		yyj = yypgo[yyn] + *yyps + 1;
		if( yyj>=YYLAST || yychk[ yystate = yyact[yyj] ] != -yyn ) yystate = yyact[yypgo[yyn]];
		switch(yym){
			
case 1:
# line 194 "mainpars.y"
{yyval = yypvt[-0];/*Mac: fold 6 cases*/} break;
case 2:
# line 195 "mainpars.y"
{yyval = yypvt[-0];} break;
case 3:
# line 198 "mainpars.y"
{yyval = yypvt[-0];} break;
case 4:
# line 199 "mainpars.y"
{yyval = yypvt[-0];} break;
case 5:
# line 202 "mainpars.y"
{yyval = yypvt[-0];} break;
case 6:
# line 203 "mainpars.y"
{yyval = yypvt[-0];} break;
case 7:
# line 206 "mainpars.y"
{
						yyval = NULLSYMBOL;
					  lev0Return:
						if(BRACKETLEV == 0)
						{
							return(0);
						}
					} break;
case 8:
# line 214 "mainpars.y"
{
					yyval = NULLSYMBOL;
				  lev0Unscratch:
					
					if(BRACKETLEV == 0)
					{
						Unscratch();
					}
				} break;
case 9:
# line 223 "mainpars.y"
{
						yyval = yypvt[-1];
						goto lev0Return;
					} break;
case 10:
# line 227 "mainpars.y"
{
						yyval = yypvt[-1];
						goto lev0Unscratch;
					} break;
case 11:
# line 231 "mainpars.y"
{goto dobatch;/*Mac: fold 2 cases*/} break;
case 12:
# line 233 "mainpars.y"
{
					  dobatch:
						(void) runCommand(Lookup("batch"), yypvt[-1]);
		  				return (0); /* terminate parse */
					} break;
case 13:
# line 239 "mainpars.y"
{
						yyval = yypvt[-1];
						if(BRACKETLEV == 0)
						{
							prexpr(yypvt[-1]);
							return(0);
						}
					} break;
case 14:
# line 247 "mainpars.y"
{
						yyval = yypvt[-1];
						if(BRACKETLEV == 0)
						{
							prexpr(yypvt[-1]);
							Unscratch();
						}
					} break;
case 15:
# line 255 "mainpars.y"
{
						yyerror("ERROR: illegal use of batch()");
						Removelist(yypvt[-0]);
		  				return (0); /* terminate parse */
					} break;
case 16:
# line 260 "mainpars.y"
{yyval = (Symbolhandle) 0;YYERROR;/*Mac: fold 2 cases*/} break;
case 17:
# line 261 "mainpars.y"
{yyval = (Symbolhandle) 0;YYERROR;} break;
case 18:
# line 262 "mainpars.y"
{yyval = (Symbolhandle) 0;yyerrok; return(0);} break;
case 19:
# line 265 "mainpars.y"
{  /* break, break n, breakall */
					if(yypvt[-0] == (Symbolhandle) 0)
					{
						YYERROR;
					}
					TargetLevel = WDEPTH - (int) DATAVALUE(yypvt[-0],0);
					Removesymbol(yypvt[-0]);
					if(TargetLevel < 0)
					{
						YYERROR;
					}
					{
						YYLEXMESSAGE = MAXWDEPTH +
							WHILEBRACKETLEV[TargetLevel] + 1;
						cleanLoops(TargetLevel,WDEPTH);
					}
				} break;
case 20:
# line 283 "mainpars.y"
{
					yyval = NULLSYMBOL;
				} break;
case 21:
# line 293 "mainpars.y"
{
							yyval = Assign(yypvt[-1],(Symbolhandle) 0,-1);
							if(yyval == (Symbolhandle) 0)
							{
								YYERROR;
							}
						} break;
case 22:
# line 301 "mainpars.y"
{
							yyval = Assign(yypvt[-3],yypvt[-0],0);
						  checkResult:
							if(yyval == (Symbolhandle) 0)
							{
								YYERROR;
							}
						} break;
case 23:
# line 309 "mainpars.y"
{
							yyerror("ERROR: illegal assignment to structure component");
							YYERROR;
						} break;
case 24:
# line 314 "mainpars.y"
{
							  subassigncheck2:
								yyval = Subassign(yypvt[-2],yypvt[-1],(Symbolhandle) 0,1);
								if(yyval == (Symbolhandle) 0)
								{
									YYERROR;
								}
							} break;
case 25:
# line 323 "mainpars.y"
{
							  subassign5:
								yyval = Subassign(yypvt[-4],yypvt[-3],yypvt[-0],0);
								Removelist(yypvt[-3]);
								if(yyval == (Symbolhandle) 0)
								{
									YYERROR;
								}
							} break;
case 26:
# line 333 "mainpars.y"
{
								yyval = Subassign(yypvt[-3],yypvt[-1],(Symbolhandle) 0,1);
								if(yyval == (Symbolhandle) 0)
								{
									YYERROR;
								}
							} break;
case 27:
# line 341 "mainpars.y"
{
								yyval = Subassign(yypvt[-5],yypvt[-3],yypvt[-0],0);
								Removelist(yypvt[-3]);
								if(yyval == (Symbolhandle) 0)
								{
									YYERROR;
								}
							} break;
case 28:
# line 350 "mainpars.y"
{
								goto subassigncheck2;
							} break;
case 29:
# line 354 "mainpars.y"
{
								goto subassign5;
							} break;
case 30:
# line 358 "mainpars.y"
{
							  subassigncheck3:
								yyval = Subassign(yypvt[-3],yypvt[-1],(Symbolhandle) 0,1);
								if(yyval == (Symbolhandle) 0)
								{
									YYERROR;
								}
							} break;
case 31:
# line 367 "mainpars.y"
{
							  subassign6:
								yyval = Subassign(yypvt[-5],yypvt[-3],yypvt[-0],0);
								Removelist(yypvt[-3]);
								if(yyval == (Symbolhandle) 0)
								{
									YYERROR;
								}
							} break;
case 32:
# line 377 "mainpars.y"
{
								Op = ADD;
							  doassignop:
								yyval = Assign(yypvt[-2],(Symbolhandle) 0,-Op);
								if(yyval == (Symbolhandle) 0)
								{
									YYERROR;
								}
								yyval = Arith(yypvt[-2], yypvt[-0], Op);
								UNLOADSEG(Arith);
								if(yyval != (Symbolhandle) 0)
								{
									yyval = Assign(yypvt[-2],yyval,0);
								}
								goto checkResult;
							} break;
case 33:
# line 393 "mainpars.y"
{
								Op = SUB;
								goto doassignop;
							} break;
case 34:
# line 397 "mainpars.y"
{
								Op = MULT;
								goto doassignop;
							} break;
case 35:
# line 401 "mainpars.y"
{
								Op = DIV ;
								goto doassignop;
							} break;
case 36:
# line 405 "mainpars.y"
{
								Op = MOD;
								goto doassignop;
							} break;
case 37:
# line 409 "mainpars.y"
{
								Op = EXP;
								goto doassignop;
							} break;
case 38:
# line 416 "mainpars.y"
{ /* Mac: fold 4 cases */
									AssignOp = LONGDATAVALUE(yypvt[-0],1);
									goto assignoperror;
								} break;
case 39:
# line 421 "mainpars.y"
{
									AssignOp = LONGDATAVALUE(yypvt[-0],1);
									goto assignoperror;
								} break;
case 40:
# line 426 "mainpars.y"
{
									AssignOp = LONGDATAVALUE(yypvt[-0],1);
									goto assignoperror;
								} break;
case 41:
# line 430 "mainpars.y"
{
									AssignOp = LONGDATAVALUE(yypvt[-0],1);
								  assignoperror:
									sprintf(OUTSTR,
											"ERROR: illegal left side of %s",
											opName(AssignOp));
									yyerror(OUTSTR);
									YYERROR;
								} break;
case 42:
# line 442 "mainpars.y"
{
	 						Op = ADD;
							AssignOp = ASSIGNADD;
						  setassignOp:
							{
								Symbolhandle symhOp = LongInstall(SCRATCH, 2);
								if(symhOp == (Symbolhandle) 0)
								{
									YYERROR;
								}
								LONGDATAVALUE(symhOp,0) = (long) Op;
								LONGDATAVALUE(symhOp,1) = (long) AssignOp;
								yyval = symhOp;
							}
						} break;
case 43:
# line 457 "mainpars.y"
{Op = SUB;AssignOp = ASSIGNSUB;goto setassignOp;} break;
case 44:
# line 458 "mainpars.y"
{Op = MULT;AssignOp = ASSIGNMULT;goto setassignOp;} break;
case 45:
# line 459 "mainpars.y"
{Op = DIV;AssignOp = ASSIGNDIV;goto setassignOp;} break;
case 46:
# line 460 "mainpars.y"
{Op = MOD;AssignOp = ASSIGNMOD;goto setassignOp;} break;
case 47:
# line 461 "mainpars.y"
{Op = EXP;AssignOp = ASSIGNPOW;goto setassignOp;} break;
case 48:
# line 464 "mainpars.y"
{
							yyval = yypvt[-0];
						} break;
case 49:
# line 467 "mainpars.y"
{
									yyval = yypvt[-0];
				 				} break;
case 50:
# line 471 "mainpars.y"
{
								yyval = Assign(yypvt[-2],yypvt[-0], 1);

								goto checkResult;
							} break;
case 51:
# line 480 "mainpars.y"
{
									yyval = runCommand(yypvt[-1], yypvt[-0]);
								  setScrName1:
									if(yyval == (Symbolhandle) 0)
									{
										YYERROR;
									}
									setScratchName(yyval);
									CHECKINTERRUPT;
								} break;
case 52:
# line 491 "mainpars.y"
{
									yyval = runCommand(yypvt[-2],yypvt[-0]);
									goto setScrName1;
								} break;
case 53:
# line 497 "mainpars.y"
{
									yyval = Extract(yypvt[-2],yypvt[-0],1);
									
								  setScrName2:
									if(yyval == (Symbolhandle) 0)
									{
										YYERROR;
									}
									setScratchName(yyval);
								} break;
case 54:
# line 507 "mainpars.y"
{
							yyval = Extract(yypvt[-1],yypvt[-0],0);
							goto setScrName2;
						} break;
case 55:
# line 513 "mainpars.y"
{yyval = yypvt[-0];} break;
case 56:
# line 514 "mainpars.y"
{
						YYERROR;
					} break;
case 57:
# line 518 "mainpars.y"
{yyval = yypvt[-0];} break;
case 58:
# line 519 "mainpars.y"
{yyval = yypvt[-1];} break;
case 59:
# line 520 "mainpars.y"
{
							Op = ADD;
						  arith:
							yyval = Arith(yypvt[-2],yypvt[-0],Op);
							/* Arith() removes SCRATCH unconsumed args */
							UNLOADSEG(Arith);
							goto setScrName1;
						 } break;
case 60:
# line 528 "mainpars.y"
{ Op = SUB;goto arith; } break;
case 61:
# line 529 "mainpars.y"
{ Op = MULT;goto arith; } break;
case 62:
# line 530 "mainpars.y"
{ Op = MATMULT;goto arith; } break;
case 63:
# line 531 "mainpars.y"
{ Op = TRMATMULT;goto arith; } break;
case 64:
# line 532 "mainpars.y"
{ Op = MATMULTTR;goto arith; } break;
case 65:
# line 533 "mainpars.y"
{ Op = DIV;goto arith; } break;
case 66:
# line 534 "mainpars.y"
{ Op = MOD;goto arith; } break;
case 67:
# line 535 "mainpars.y"
{ Op = EXP;goto arith; } break;
case 68:
# line 537 "mainpars.y"
{ Op = BITOR;goto arith; } break;
case 69:
# line 538 "mainpars.y"
{ Op = BITXOR;goto arith; } break;
case 70:
# line 539 "mainpars.y"
{ Op = BITAND;goto arith; } break;
case 71:
# line 541 "mainpars.y"
{yyval = yypvt[-1];} break;
case 72:
# line 543 "mainpars.y"
{
									Op = '-';
								  unary:
									yyval = Unary(yypvt[-0],Op);
									/*Unary() reuses space if arg is SCRATCH */
									UNLOADSEG(Unary);
									goto setScrName1;
								} break;
case 73:
# line 551 "mainpars.y"
{
									Op = '+';
									goto unary;
								} break;
case 74:
# line 556 "mainpars.y"
{
									Op = BITNOT;
									goto unary;
								} break;
case 75:
# line 561 "mainpars.y"
{
						Op = EQ;
					  logic:
						yyval = Logic(yypvt[-2],yypvt[-0],Op);
						/*Logic() removes unconsumed scratch args */
						UNLOADSEG(Logic);
						goto setScrName1;
					} break;
case 76:
# line 569 "mainpars.y"
{ Op = EQ; goto logic; } break;
case 77:
# line 570 "mainpars.y"
{ Op = NE; goto logic; } break;
case 78:
# line 571 "mainpars.y"
{ Op = LT; goto logic; } break;
case 79:
# line 572 "mainpars.y"
{ Op = LE; goto logic; } break;
case 80:
# line 573 "mainpars.y"
{ Op = GT; goto logic; } break;
case 81:
# line 574 "mainpars.y"
{ Op = GE; goto logic; } break;
case 82:
# line 575 "mainpars.y"
{ Op = AND; goto logic; } break;
case 83:
# line 576 "mainpars.y"
{ Op = AND; goto logic; } break;
case 84:
# line 577 "mainpars.y"
{ Op = OR; goto logic; } break;
case 85:
# line 578 "mainpars.y"
{ Op = OR; goto logic; } break;
case 86:
# line 579 "mainpars.y"
{
							yyval = Logic(yypvt[-0],yypvt[-0],NOT);
							UNLOADSEG(Unary);
							goto setScrName1;
						} break;
case 87:
# line 584 "mainpars.y"
{
							Symbolhandle list = Addlist( Makelist(), yypvt[-1] );
							if(list == (Symbolhandle) 0)
							{
								YYERROR;
							}
							strcpy(FUNCNAME,"prime");
							yyval = transpose(list);
							Removelist(list);
							UNLOADSEG(transpose);
							goto setScrName1;
				  		} break;
case 88:
# line 597 "mainpars.y"
{yyval = yypvt[-0];} break;
case 89:
# line 601 "mainpars.y"
{
							long reply = Macrosetup(yypvt[-1],yypvt[-0]);

							Removelist(yypvt[-0]);
							if(!reply)
							{
								YYERROR;
							}
						} break;
case 90:
# line 611 "mainpars.y"
{
							yyval = yypvt[-1];
							MDEPTH--;
						} break;
case 91:
# line 617 "mainpars.y"
{goto missingRparen; /*Mac: fold 4 cases*/} break;
case 92:
# line 618 "mainpars.y"
{goto missingRparen;} break;
case 93:
# line 619 "mainpars.y"
{goto missingRparen;} break;
case 94:
# line 620 "mainpars.y"
{
								  missingRparen:
									yyval = 0;
									yyerror("ERROR: missing ')'");
									YYERROR;
								} break;
case 95:
# line 628 "mainpars.y"
{yyval = yypvt[-0];/*Mac: fold 3 cases*/} break;
case 96:
# line 629 "mainpars.y"
{yyval = yypvt[-0];} break;
case 97:
# line 630 "mainpars.y"
{yyval = yypvt[-0];} break;
case 98:
# line 632 "mainpars.y"
{
					IDEPTH--;
					yyval = yypvt[-0];
				} break;
case 99:
# line 637 "mainpars.y"
{
						if(IFCONDITIONS[IDEPTH-1] > 0)
						{
							 yyval = yypvt[-1];
							 YYLEXMESSAGE = SKIPBRACKETBLOCK;
						}
						CHECKINTERRUPT;
						if(!Ifsetup((Symbolhandle) 0, ELSE))
						{
							yyval = 0;
							YYERROR;
						}
					} break;
case 100:
# line 651 "mainpars.y"
{
						if(IFCONDITIONS[IDEPTH-1] == 0)
						{
							 yyval = yypvt[-1];
						}
						IDEPTH--;
					} break;
case 101:
# line 659 "mainpars.y"
{
							int    ifOK;

							CHECKINTERRUPT;
							ifOK = Ifsetup(yypvt[-0], WHILE);
							Removelist(yypvt[-0]);
							if(!ifOK)
							{
								yyval = 0;
								YYERROR;
							}
							if(WHILECONDITIONS[WDEPTH-1] == 0)
							{
								YYLEXMESSAGE = SKIPBRACKETBLOCK;
							}
							else if(WHILELIMITS[WDEPTH-1] <= 0)
							{ /* too many reps */
								yyerrok ;
								return(0) ;
							}
						} break;
case 102:
# line 681 "mainpars.y"
{
							if(WHILECONDITIONS[WDEPTH-1])
							{
								ISTRCHAR = WHILESTARTS[WDEPTH-1];
								if(yypvt[-1] != (Symbolhandle) 0 && BRACKETLEV > 0 &&
									 isscratch(NAME(yypvt[-1])))
								{
									Removesymbol(yypvt[-1]);
									yypvt[-1] = NULLSYMBOL;
								}
								YYLEXMESSAGE = -((long) ';');
							}
							else
							{
								WHILELIMITS[WDEPTH-1] = MAXWHILE;
							}
							WDEPTH--;
							yyval = yypvt[-1];
						} break;
case 103:
# line 701 "mainpars.y"
{
							int   ifOK;

							CHECKINTERRUPT;
							ifOK = Ifsetup(yypvt[-0], FOR);
							Removelist(yypvt[-0]);
							if(!ifOK)
							{
								yyval = 0;
								YYERROR;
							}
						 } break;
case 104:
# line 714 "mainpars.y"
{
							if(WHILECONDITIONS[WDEPTH-1] &&
								 --WHILELIMITS[WDEPTH-1] > 0)
							{
								ISTRCHAR = WHILESTARTS[WDEPTH-1];
								setupfakefor();
								ALTTOKENPTR = ALTINPUTTOKENS;
								if(yypvt[-1] != (Symbolhandle) 0 && BRACKETLEV > 0 &&
									 isscratch(NAME(yypvt[-1])))
								{
									Removesymbol(yypvt[-1]);
									yypvt[-1] = NULLSYMBOL;
								}
							}
							else
							{
								WHILELIMITS[WDEPTH-1] = MAXWHILE;
							}
							WDEPTH--;
							yyval = yypvt[-1];
						} break;
case 105:
# line 736 "mainpars.y"
{yyval = yypvt[-1];/*Mac: fold 2 cases*/} break;
case 106:
# line 737 "mainpars.y"
{yyval = yypvt[-1];} break;
case 107:
# line 738 "mainpars.y"
{yyval = yypvt[-0];} break;
case 108:
# line 739 "mainpars.y"
{
						notfunction(yypvt[-1]);
						yyval = 0;
						YYERROR;
					} break;
case 109:
# line 745 "mainpars.y"
{ return(END); } break;
case 110:
# line 747 "mainpars.y"
{
						return (FATALERROR);
					} break;
case 111:
# line 752 "mainpars.y"
{yyval = yypvt[-0];/*Mac: fold 9 cases*/} break;
case 112:
# line 753 "mainpars.y"
{yyval = yypvt[-0];} break;
case 113:
# line 754 "mainpars.y"
{yyval = yypvt[-0];} break;
case 114:
# line 755 "mainpars.y"
{yyval = yypvt[-0];} break;
case 115:
# line 756 "mainpars.y"
{yyval = yypvt[-0];} break;
case 116:
# line 757 "mainpars.y"
{yyval = yypvt[-0];} break;
case 117:
# line 758 "mainpars.y"
{yyval = yypvt[-0];} break;
case 118:
# line 759 "mainpars.y"
{yyval = yypvt[-0];} break;
case 119:
# line 760 "mainpars.y"
{yyval = yypvt[-0];} break;
case 120:
# line 761 "mainpars.y"
{yyval = yypvt[-1];/*Mac: fold 9 cases*/} break;
case 121:
# line 762 "mainpars.y"
{yyval = yypvt[-1];} break;
case 122:
# line 763 "mainpars.y"
{yyval = yypvt[-1];} break;
case 123:
# line 764 "mainpars.y"
{yyval = yypvt[-1];} break;
case 124:
# line 765 "mainpars.y"
{yyval = yypvt[-1];} break;
case 125:
# line 766 "mainpars.y"
{yyval = yypvt[-1];} break;
case 126:
# line 767 "mainpars.y"
{yyval = yypvt[-1];} break;
case 127:
# line 768 "mainpars.y"
{yyval = yypvt[-1];} break;
case 128:
# line 769 "mainpars.y"
{yyval = yypvt[-1];} break;
case 129:
# line 770 "mainpars.y"
{yyval = yypvt[-0];/*Mac: fold 12 cases*/} break;
case 130:
# line 771 "mainpars.y"
{yyval = yypvt[-0];} break;
case 131:
# line 772 "mainpars.y"
{yyval = yypvt[-0];} break;
case 132:
# line 773 "mainpars.y"
{yyval = yypvt[-0];} break;
case 133:
# line 774 "mainpars.y"
{yyval = yypvt[-0];} break;
case 134:
# line 775 "mainpars.y"
{yyval = yypvt[-0];} break;
case 135:
# line 776 "mainpars.y"
{yyval = yypvt[-0];} break;
case 136:
# line 777 "mainpars.y"
{yyval = yypvt[-0];} break;
case 137:
# line 778 "mainpars.y"
{yyval = yypvt[-0];} break;
case 138:
# line 779 "mainpars.y"
{yyval = yypvt[-0];} break;
case 139:
# line 780 "mainpars.y"
{yyval = yypvt[-0];} break;
case 140:
# line 781 "mainpars.y"
{yyval = yypvt[-0];} break;
case 141:
# line 782 "mainpars.y"
{yyval = yypvt[-1];/*Mac: fold 10 cases*/} break;
case 142:
# line 783 "mainpars.y"
{yyval = yypvt[-1];} break;
case 143:
# line 784 "mainpars.y"
{yyval = yypvt[-1];} break;
case 144:
# line 785 "mainpars.y"
{yyval = yypvt[-1];} break;
case 145:
# line 786 "mainpars.y"
{yyval = yypvt[-1];} break;
case 146:
# line 787 "mainpars.y"
{yyval = yypvt[-1];} break;
case 147:
# line 788 "mainpars.y"
{yyval = yypvt[-1];} break;
case 148:
# line 789 "mainpars.y"
{yyval = yypvt[-1];} break;
case 149:
# line 790 "mainpars.y"
{yyval = yypvt[-1];} break;
case 150:
# line 791 "mainpars.y"
{yyval = yypvt[-1];} break;
case 151:
# line 792 "mainpars.y"
{yyval = yypvt[-0];/*Mac: fold 2 cases*/} break;
case 152:
# line 793 "mainpars.y"
{yyval = yypvt[-0];} break;
case 153:
# line 794 "mainpars.y"
{yyval = yypvt[-1];/*Mac: fold 5 cases*/} break;
case 154:
# line 795 "mainpars.y"
{yyval = yypvt[-1];} break;
case 155:
# line 796 "mainpars.y"
{yyval = yypvt[-1];} break;
case 156:
# line 797 "mainpars.y"
{yyval = yypvt[-1];} break;
case 157:
# line 798 "mainpars.y"
{yyval = yypvt[-1];} break;
case 158:
# line 801 "mainpars.y"
{ goto bracklist4; /*Mac: fold 3 cases */} break;
case 159:
# line 802 "mainpars.y"
{ goto bracklist4; } break;
case 160:
# line 803 "mainpars.y"
{
								  bracklist4:
									yyval = Element(yypvt[-2],yypvt[-0]);
									Removelist(yypvt[-0]);
									goto setScrName2;
								} break;
case 161:
# line 809 "mainpars.y"
{ goto bracklist2; /*Mac: fold 5 cases */} break;
case 162:
# line 810 "mainpars.y"
{ goto bracklist2; } break;
case 163:
# line 811 "mainpars.y"
{ goto bracklist2; } break;
case 164:
# line 812 "mainpars.y"
{ goto bracklist2; } break;
case 165:
# line 813 "mainpars.y"
{
							  bracklist2:
								yyval = Element(yypvt[-1],yypvt[-0]);
								Removelist(yypvt[-0]);
								goto setScrName2;
							} break;
case 166:
# line 819 "mainpars.y"
{
								yyval = Element(yypvt[-2],yypvt[-0]);
								Removelist(yypvt[-0]);
								goto setScrName2;
							} break;
case 167:
# line 826 "mainpars.y"
{
							int    ifOK;

							CHECKINTERRUPT;
							ifOK = Ifsetup(yypvt[-0], IF);
							Removelist(yypvt[-0]);
							if(!ifOK)
							{
								yyval = 0;
								YYERROR;
							}
							if(IFCONDITIONS[IDEPTH-1] == 0)
							{
								YYLEXMESSAGE = SKIPBRACKETBLOCK;
							}
						} break;
case 168:
# line 843 "mainpars.y"
{
							yyval = (IFCONDITIONS[IDEPTH-1] ) ? yypvt[-1] : NULLSYMBOL;
						} break;
case 169:
# line 847 "mainpars.y"
{
							if(IFCONDITIONS[IDEPTH-1] > 0)
							{
								 YYLEXMESSAGE = SKIPPARENBLOCK;
							}
						} break;
case 170:
# line 854 "mainpars.y"
{
							int    ifOK;

							if(IFCONDITIONS[IDEPTH-1] > 0)
							{
								YYLEXMESSAGE = SKIPBRACKETBLOCK;
								Removelist(yypvt[-0]);
								yypvt[-0] = (Symbolhandle) 0;
							}
							CHECKINTERRUPT;
							ifOK = Ifsetup(yypvt[-0], ELSEIF);
							if(yypvt[-0] != (Symbolhandle) 0)
							{
								Removelist(yypvt[-0]);
							}
							if(!ifOK)
							{
								yyval = 0;
								YYERROR;
							}
						} break;
case 171:
# line 876 "mainpars.y"
{
							if(yypvt[-4] == (Symbolhandle) 0)
							{
								yyval = yypvt[-7];
							}
							else if(IFCONDITIONS[IDEPTH-1] > 0)
							{
								yyval = yypvt[-1];
							}
							else
							{
								yyval = NULLSYMBOL;
							}
						} break;
case 172:
# line 890 "mainpars.y"
{ yyval = 0 ; YYERROR;} break;
case 173:
# line 893 "mainpars.y"
{yyval = yypvt[-0];} break;
case 174:
# line 894 "mainpars.y"
{
							yyval = yypvt[-1];
							PARSEMACROARGS = ParseMacroArgs;
						} break;
case 175:
# line 902 "mainpars.y"
{yyval = yypvt[-0];} break;
case 176:
# line 903 "mainpars.y"
{yyval = yypvt[-1];} break;
case 177:
# line 907 "mainpars.y"
{
						yyval = Byname(yypvt[-1],&BynameType,&ParseMacroArgs);
						if(yyval == (Symbolhandle) 0)
						{
							YYERROR;
						}
						else
						{
							YYLEXMESSAGE = -BynameType;
						}
					} break;
case 178:
# line 918 "mainpars.y"
{
							yyval = 0;
							yyerror("ERROR: missing '>>'");
							YYERROR;
						} break;
case 179:
# line 926 "mainpars.y"
{
							  growMake:
								yyval = Growlist(Makelist());
								goto checkResult;
							} break;
case 180:
# line 931 "mainpars.y"
{goto growAddMake;/*Mac: fold 2 cases*/} break;
case 181:
# line 932 "mainpars.y"
{
							  growAddMake:
								yyval = Growlist(Addlist(Makelist(),yypvt[-1]));
								goto checkResult;
							} break;
case 182:
# line 937 "mainpars.y"
{goto growAdd; /*Mac: fold 2 cases*/} break;
case 183:
# line 938 "mainpars.y"
{
							  growAdd:
								yyval = Growlist(Addlist(yypvt[-2],yypvt[-1]));
								goto checkResult;
							} break;
case 184:
# line 943 "mainpars.y"
{
								yyval = Growlist(yypvt[-1]);
								goto checkResult;
							} break;
case 185:
# line 950 "mainpars.y"
{goto addMake; /*Mac: fold 2 cases*/} break;
case 186:
# line 951 "mainpars.y"
{
							  addMake:
								yyval = Addlist( Makelist(), yypvt[-0] );
								goto checkResult;
							} break;
case 187:
# line 956 "mainpars.y"
{
								yyval = yypvt[-0];
								goto checkResult;
							} break;
case 188:
# line 960 "mainpars.y"
{goto addlist;/*Mac: fold 2 cases*/} break;
case 189:
# line 961 "mainpars.y"
{
							  addlist:
								yyval = Addlist( yypvt[-1], yypvt[-0]);
								goto checkResult;
							} break;
case 190:
# line 968 "mainpars.y"
{
									  makelist:
										yyval = Makelist();
										goto checkResult;
									} break;
case 191:
# line 973 "mainpars.y"
{
									yyval = yypvt[-1];
								} break;
case 192:
# line 976 "mainpars.y"
{goto missingRparen; /*Mac: fold 2 cases*/} break;
case 193:
# line 977 "mainpars.y"
{goto missingRparen;} break;
case 194:
# line 981 "mainpars.y"
{goto makelist;/*Mac: fold 2 cases*/} break;
case 195:
# line 983 "mainpars.y"
{goto makelist;} break;
case 196:
# line 985 "mainpars.y"
{ yyval = yypvt[-1]; goto checkResult;/*Mac: fold 2 cases*/} break;
case 197:
# line 987 "mainpars.y"
{ yyval = yypvt[-1]; goto checkResult;} break;
case 198:
# line 988 "mainpars.y"
{goto missingRbracket;/*Mac: fold 3 cases*/} break;
case 199:
# line 989 "mainpars.y"
{goto missingRbracket;} break;
case 200:
# line 991 "mainpars.y"
{
					  missingRbracket:
						yyerror("ERROR: missing ']'");
						yyval = 0;
						YYERROR;
					} break;
case 202:
# line 1004 "mainpars.y"
{ yyval = yypvt[-2]; goto checkResult;/*Mac: fold 2 cases*/} break;
case 203:
# line 1006 "mainpars.y"
{ yyval = yypvt[-2]; goto checkResult;} break;
case 204:
# line 1007 "mainpars.y"
{goto missingRbrack2; /*Mac: fold 3 cases*/} break;
case 205:
# line 1008 "mainpars.y"
{goto missingRbrack2;} break;
case 206:
# line 1010 "mainpars.y"
{
					  missingRbrack2:
						yyerror("ERROR: missing ']]'");
						yyval = 0;
						YYERROR;
					} break; 
		}
		goto yystack;  /* stack new state and value */

	}
