/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 5.00 or later
*(C)*
*(C)* Copyright (c) 2004 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/


#ifdef SEGMENTED
#if defined(MPW) || defined(MW_CW)
#pragma segment Tabs
#endif /*MPW||MW_CW*/
#endif /*SEGMENTED*/


#include "globals.h"

#define coord(I,J) (classVars[J][I])

enum doitCodes
{
	JMEAN = 0,
	JVAR,
	JCOVAR,
	JCOUNT,
	JSTDDEV,
	JMIN,
	JMAX,
	JSUM,
	JPROD,
	NALL,
	JWORK = NALL,
	NDEFAULT = 3,
	WORKSIZE = 100
};

enum tabCodes
{
	IMEAN   = 1 << JMEAN,
	IVAR    = 1 << JVAR,
	ICOVAR  = 1 << JCOVAR,
	ICOUNT  = 1 << JCOUNT,
	ISTDDEV = 1 << JSTDDEV,
	IMIN    = 1 << JMIN,
	IMAX    = 1 << JMAX,
	ISUM    = 1 << JSUM,
	IPROD   = 1 << JPROD
};

enum tabScratch
{
	GMEAN = 0,
	GVAR,
	GCOVAR,
	GCOUNT,
	GSTDDEV,
	GMIN,
	GMAX,
	GSUM,
	GPROD,
	GWORK,
	NTRASH
};


/*
  Modified by kb to allow for multivariate data

  971212 changed argument list to celltabs() to avoid need for global static
         variables
         tabs(,a,b,...,count:T) is now ok

  990722 changed argument list to celltabs(), putting all the output handles
         in an array of handles; made changes implementing keywords
		 'stddev', 'min' and 'max' and modified other aspects of the code

  990723 Every thing but count MISSING when count = 0

  991208 Fixed bug; means were not MISSING when count = 0 unless variance
         or standard deviation was computed

  000428 Implemented keywords 'sum' and 'prod'

  000531 Implemented keyword 'covar'

  041005 fixed bug involving missing values with multivariate data
*/

static int      celltabs(Symbolhandle list, long nfactors, double ** out [],
						 unsigned long control)
{
	Symbolhandle    symh;
	long            i, j, ncells, icell;
	long            ivar;
	long            term[MAXDIMS];
	long            length;
	double          hugedbl = HUGEDBL;
	double         *classVars[MAXDIMS];
	long            nclass[MAXDIMS], types[MAXDIMS];
	long            nMissing = 0;
	double          yy;
	double        **tmeans = out[JMEAN];
	double        **tvars = out[JVAR];
	double        **tcounts = out[JCOUNT];
	double        **tstddev = out[JSTDDEV];
	double        **tmin = out[JMIN];
	double        **tmax = out[JMAX];
	double        **tsum = out[JSUM];
	double        **tprod = out[JPROD];
	double        **tcovar = out[JCOVAR];
	double        **twork = out[JWORK];
	double        **ty;
	double         *y;
	double         *counts = (double *) 0;
	double         *vars = (double *) 0, *means = (double *) 0;
	double         *stddevs = (double *) 0;
	double         *mins = (double *) 0, *maxs = (double *) 0;
	double         *sums = (double *) 0, *prods = (double *) 0;
	double         *covar = (double *) 0;
	long            ndata, ny;
	int             foundOverflow = 0;
	WHERE("celltabs");
	
	symh = COMPVALUE(list, 0);
	if (symh == (Symbolhandle) 0 || isNull(symh))
	{
		ty = (double **) 0;
		y = (double *) 0;
		ndata = symbolSize(COMPVALUE(list, 1));
		ny = 1;
	}
	else
	{
		long         dim[2];

		(void) isMatrix(symh, dim);
		ty = DATA(symh);
		y = *ty;
		ndata = dim[0];
		ny = dim[1];
	}

	ncells = 1;
	if (nfactors > 0)
	{
		for (i = 0; i < nfactors; i++)
		{
			symh = COMPVALUE(list, i+1);

			term[i] = ncells;
			nclass[i] = labs(isFactor(symh));
			classVars[i] = DATAPTR(symh);
			types[i] = TYPE(symh);
			ncells *= (long) nclass[i];
		} /*for (i = 0; i < nfactors; i++)*/
	} /*if (nfactors > 0)*/
	else
	{
		term[0] = nclass[0] = 1;
		classVars[0] = (double *) 0;
		types[0] = NULLSYM;
	}
		
	length = ncells * ny;

	/*
	  ncells is the number of means or other statistics to be computed
	  term contains the offsets for indexing into array of means
	*/

	/*
	  Initialize arrays for results
	*/
	if (control & (ICOUNT | IMEAN | IVAR | ICOVAR | ISTDDEV | ISUM | IPROD))
	{
		counts = *tcounts;
		doubleFill(counts, 0.0, length);
	}

	if (control & (IMEAN | IVAR | ICOVAR | ISTDDEV))
	{
		means = *tmeans;
		doubleFill(means, 0.0, length);
	}

	if (control & IVAR)
	{
		vars = *tvars;
	}

	if (control & ISTDDEV)
	{
		stddevs = *tstddev;
		if (vars == (double *) 0)
		{
			vars = stddevs;
		}
	} /*if (control & ISTDDEV)*/

	if (vars)
	{
		doubleFill(vars, 0.0, length);
	}
	
	if (control & IMIN)
	{
		mins = *tmin;
		doubleFill(mins, hugedbl, length);
	}
	
	if (control & IMAX)
	{
		maxs = *tmax;
		doubleFill(maxs, -hugedbl, length);
	}
	
	if (control & ISUM)
	{
		sums = *tsum;
		if (means == (double *) 0)
		{
			doubleFill(sums, 0.0, length);
		}
	}

	if (control & IPROD)
	{
		prods = *tprod;
		doubleFill(prods, 1.0, length);
	}
	
	if (control & ICOVAR)
	{
		covar = *tcovar;
		doubleFill(covar, MISSING, length*ny);
	}
	
	/*
	  compute counts, sums, products and extremes, whatever is needed
	*/
	for (ivar = 0;ivar < ny;ivar++)
	{
		/* mMissing = number of cases remaining with MISSING factor levels */
		long          mMissing;

		for (i = 0; i < ndata; i++)
		{
			yy = (y != (double *) 0) ? y[i] : 1.0;
			if (!isMissing(yy))
			{
				for (j = 0; j < nfactors; j++)
				{
					if (isMissing(coord(i,j)))
					{
						if (ivar == 0)
						{
							/* 
							   increment count of cases with MISSING
							   factor levels
							*/
							nMissing++;
						}
						break;
					} /*if (isMissing(coord(i,j)))*/
				} /*for (j = 0; j < nfactors; j++)*/

				if (j == nfactors)
				{
					/* all factor levels are non MISSING*/
					icell = 0;
					for (j = 0; j < nfactors; j++)
					{
						long        level;

						level = (types[j] == REAL) ? (long) coord(i,j) :
							((coord(i,j) != 0.0)) ? 2 : 1;
						icell += term[j] * (level-1);	/* find cell number */
					}
					if (counts)
					{
						counts[icell]++; /* increment count */
					}
					if (means)
					{
						means[icell] += yy; /* increment sum  */
					}
					if (mins && yy < mins[icell])
					{
						mins[icell] = yy; /*update minimum */
					}
					if (maxs && yy > maxs[icell])
					{
						maxs[icell] = yy; /* update maximum */
					}
					if (sums && means == (double *) 0)
					{
						sums[icell] += yy; /* increment sum */
					}
					if (prods)
					{
						prods[icell] *= yy; /* update product */
					}
				} /*if (j == nfactors)*/
			} /*if (!isMissing(yy))*/
		} /*for (i = 0; i < ndata; i++)*/
		mMissing = nMissing;

		/*
		  At this point, if needed, the cell counts, sums, extremes and
		  products have been found for variable ivar

		*/
		for (icell = 0; icell < ncells; icell++)
		{
			if (means || sums)
			{
				double         value = (means) ? means[icell] : sums[icell];

				if (counts[icell] > 0.0)
				{
#ifdef HASINFINITY
					if (isInfinite(value))
					{
						setMissing(value);
						foundOverflow = 1;
						if (means)
						{
							means[icell] = value;
						}
					}
					else
#endif /*HASINFINITY*/
					if (means)
					{
						means[icell] = value/counts[icell];
					}

					if (sums)
					{
						sums[icell] = value;
					}
				} /*if (counts[icell] > 0.0)*/
				else if (sums)
				{
					setMissing(sums[icell]);
				}
			} /*if (means || sums)*/

			if (mins && mins[icell] == hugedbl)
			{
				setMissing(mins[icell]);
			}

			if (maxs && maxs[icell] == -hugedbl)
			{
				setMissing(maxs[icell]);
			}

			if (prods)
			{
				double            value = prods[icell];
				
				if (counts[icell] == 0.0)
				{
					setMissing(value);
				}
				else
#ifdef HASINFINITY
				if (isInfinite(value))
				{
					setMissing(value);
					foundOverflow = 1;
				}
#endif /*HASINFINITY*/
				prods[icell] = value;
			}
		} /*for (icell = 0; icell < ncells; icell++)*/

		if (vars)
		{
			/* 
			   Compute SS for variances and stddevs

			   mMissing contains number of cases with one or more MISSING
			   factor level
			*/
			for (i = 0; i < ndata; i++)
			{
				yy = y[i];
				if (!isMissing(yy))
				{
					if (mMissing)
					{
						/* there are still cases with MISSING factor levels*/
						for (j = 0; j < nfactors; j++)
						{
							if (isMissing(coord(i,j)))
							{
								mMissing--;
								break;
							}
						} /*for (j = 0; j < nfactors; j++)*/
					} /*if (mMissing)*/
					else
					{
						j = nfactors;
					}
					
					if (j == nfactors)
					{
						/* All coordinates non MISSING */
						icell = 0;
						for (j = 0; j < nfactors; j++)
						{
							long        k;

							k = (types[j] == REAL) ? ((long) coord(i,j) - 1) :
							((coord(i,j) != 0.0)) ? 1 : 0;
							icell += term[j] * k; /* find cell number */
						} /*for (j = 0; j < nfactors; j++)*/
#ifdef HASINFINITY
						if (!isMissing(means[icell]))
#endif /*HASINFINITY*/
						{
							double       d = yy - means[icell];
							vars[icell] += d * d; /* get variance*/
						}
					} /*if (j == nfactors)*/
				} /*if (!isMissing(yy))*/
			} /*for (i = 0; i < ndata; i++)*/
		} /*if (vars)*/
		
		for (icell = 0; icell < ncells; icell++)
		{
#ifdef HASINFINITY
			if (means && isMissing(means[icell]) ||
				vars && isInfinite(vars[icell]))
			{
				if (vars)
				{
					setMissing(vars[icell]);
				}
				if (stddevs)
				{
					setMissing(stddevs[icell]);
				}
				foundOverflow = 1;
			}
			else
#endif /*HASINFINITY*/
			{
				if (means || vars || stddevs)
				{
					/* divide variances by d.f. & compute sd's*/
					long          n = (long) counts[icell];

					if (n > 1)
					{
						if (vars)
						{
							vars[icell] /= (n - 1.0);
						}
						if (stddevs)
						{
							stddevs[icell] = sqrt(vars[icell]);
						}
					}
					else if (n == 1)
					{
						if (vars)
						{
							vars[icell] = 0.0;
						}
						if (stddevs)
						{
							stddevs[icell] = 0.0;
						}
					}
					else
					{
/* Set means, vars, stddevs MISSING for n = 0*/
						if (means)
						{
							setMissing(means[icell]);
						}
						if (vars)
						{
							setMissing(vars[icell]);
						}
						if (stddevs)
						{
							setMissing(stddevs[icell]);
						}
					}
				} /*if (means || vars || stddev)*/
			}
		} /*for (icell = 0; icell < ncells; icell++)*/

		if (ivar < ny - 1)
		{
			/* shift pointers */
			if (counts)
			{
				counts += ncells;
			}
			if (means)
			{
				means += ncells;
			}
			if (vars)
			{
				vars += ncells;
			}
			if (stddevs)
			{
				stddevs += ncells;
			}
			if (mins)
			{
				mins += ncells;
			}
			if (maxs)
			{
				maxs += ncells;
			}
			if (sums)
			{
				sums += ncells;
			}
			if (prods)
			{
				prods += ncells;
			}
			y += ndata;
		} /*if (ivar < ny - 1)*/		
	} /*for (ivar = 0;ivar < ny;ivar++)*/

	if (covar)
	{
		/*
		  Note: If variances already computed this does it again

		  If ny <= WORKSIZE, twork = out[JWORK] is (double **) 0
		  and array work is used for scratch.
		  If ny > WORKSIZE, out[JWORK] should be a handle pointing
		  to scratch of length ny
		*/
		double       work[WORKSIZE];
		double      *d = (twork != (double **) 0) ? *twork : work;
		double       df;
		long         nyncells = ny*ncells;
		long         j1, j2;
		long         mMissing = nMissing;

		means = *tmeans;
		counts = *tcounts;
		y = *ty;
		
		for (i = 0; i < ndata; i++)
		{
			if (mMissing)
			{
				/* there are still cases with MISSING factor levels*/
				for (j = 0; j < nfactors; j++)
				{
					if (isMissing(coord(i,j)))
					{
						mMissing--;
						break;
					}
				} /*for (j = 0; j < nfactors; j++)*/
			} /*if (mMissing)*/
			else
			{
				j = nfactors;
			}

			if (j == nfactors)
			{
				/* 
				   All coordinates non-missing 
				   Compute cell number icell
				*/
				icell = 0;
				for (j = 0; j < nfactors; j++)
				{
					long        k;

					k = (types[j] == REAL) ? ((long) coord(i,j) - 1) :
					  ((coord(i,j) != 0.0)) ? 1 : 0;
					icell += term[j] * k; /* find cell number */
				} /*for (j = 0; j < nfactors; j++)*/

				df = counts[icell] - 1.0;

				if (df >= 0)
				{
					if (isMissing(covar[icell]))
					{
						/* initialize covar to 0 in this cell */
						long     nysqncells = ny*ny*ncells;

						for (j = icell; j < nysqncells; j += ncells)
						{
							covar[j] = 0.0;
						}
					} /*if (isMissing(covar[icell]))*/

					if (df > 0)
					{
						long         k, l;

						for (j = 0, k = i, l = icell; j < ny;
							 j++, k += ndata, l += ncells)
						{
							d[j] = y[k] - means[l];
						}

						for (j1 = 0; j1 < ny; j1++)
						{
							double        d1 = d[j1];
							long          place1 = icell + j1*ncells;

							for (j2 = 0; j2 <= j1; j2++, place1 += nyncells)
							{
								covar[place1] += d1*d[j2];
							}
						} /*for (j1 = 0; j1 < ny; j1++)*/
					} /*if (df > 0)*/
				} /*if (df >= 0)*/
			} /*if (j == nfactors)*/
		} /*for (i = 0; i < ndata; i++)*/
		
		/* fill out other halves of covariance matrices & divide by d.f.*/
		for (icell = 0; icell < ncells; icell++)
		{
			df = counts[icell] - 1.0;
			for (j1 = 0; j1 < ny; j1++)
			{
				long          place1 = icell + j1*ncells;
				long          place2 = icell + j1*nyncells;

				for (j2 = 0; j2 <= j1; j2++)
				{
					if (df > 0)
					{
						covar[place1] /= df;
					}
					covar[place2] = covar[place1];
					place1 += nyncells;
					place2 += ncells;
				}
			} /*for (j1 = 0; j1 < ny; j1++)*/
		} /*for (icell = 0; icell < ncells; icell++)*/
	} /*if (covar)*/
	
	return (!foundOverflow);
} /*celltabs()*/


/*
  Modified by kb to allow for the variable tabulated being a matrix

  961114 keyword 'n' made a synonym for keyword 'count'  Component
  of result is still named 'count'
  
  980818 Fixed bug (control not set properly on tabs(NULL,a,b,...) and
         set dimensions of result with Setdims()
  990226 Replaced putOUTSTR() by putErrorOUTSTR()
  990722 Added keywords 'stddev', 'min', 'max' for computing additional
         cell statistics
  990723 Add keyword 'all'
         In an empty cell, values of all statistics (except count) are MISSING
  991208 When no factors are provided, tabs() does computations with all
         data in a single cell
  000429 Add keywords 'sum' and 'prod' that compute cell sums and products,
         with MISSING for an empty cell
  000531 Add keyword 'covar' to compute covariance matrices
  011214 Added keyword 'silent' to suppress warning messages.  Fixed bug
         in which array doit was overrun.
*/


Symbolhandle    tabs(Symbolhandle list)
{
	Symbolhandle    result = (Symbolhandle) 0, symh, symhY;
	double        **y, **out[NALL + 1];
	long            dim[2], ny, ndata;
	long            ncells, needed, i, k;
	long            ncomps, nfactors;
	long            type;
	long            nargs = NARGS(list);
	long            nclass[MAXDIMS+1];
	int             all = 0;
	int             silent = 0;
	int             foundMissingInFactor = 0;
	int             doit[NALL];
	unsigned long   control;
	int             which1;
	int             defaultOnly = 1;
	char            *compname[NALL], *keyword;
	WHERE("tabs");
	TRASH(NTRASH, errorExit);
	
	symhY = COMPVALUE(list, 0);
	if (symhY == (Symbolhandle) 0 || isNull(symhY))
	{
		ncomps = 1;
		control = ICOUNT;
		ndata = -1;
		ny = 1;
		y = (double **) 0;
	} /*if (symhY == (Symbolhandle) 0 || isNull(symhY))*/
	else if (!argOK(symhY, REAL, (nargs > 1) ? 1 : 0))
	{
		goto errorExit;
	}
	else if (!isMatrix(symhY, dim))
	{
		sprintf(OUTSTR,
				"ERROR: %s() argument 1 must be REAL vector or matrix",
				FUNCNAME);
		goto errorExit;
	}
	else
	{
		ndata = dim[0];
		ny = dim[1];
		y = DATA(symhY);
	}
	
	compname[JMEAN] = "mean";
	compname[JVAR] = "var";
	compname[JCOVAR] = "covar";
	compname[JCOUNT] = "count";
	compname[JSTDDEV] = "stddev";
	compname[JMIN] = "min";
	compname[JMAX] = "max";
	compname[JSUM] = "sum";
	compname[JPROD] = "prod";

	for (i = 0; i < NALL;i++)
	{
		out[i] = (double **) 0;
		doit[i] = -1;
	}
	out[JWORK] = (double **) 0;
	control = 0;
	
	/*
	  scan backwards from end for keywords 
	*/
	for (; nargs > 0; nargs--) 
	{
		int         logValue;

		if ((symh = COMPVALUE(list,nargs-1)) == (Symbolhandle) 0 ||
			(keyword = isKeyword(symh)) == (char *) 0)
		{
			break;
		}
		if (!isTorF(symh))
		{
			notTorF(keyword);
			goto errorExit;
		}
		logValue = DATAVALUE(symh, 0) != 0.0;
		
		if (strcmp(keyword, "silent") == 0)
		{
			silent = logValue;
		}
		else
		{
			defaultOnly = 0;
			
			if (strcmp(keyword, "all") == 0)
			{
				all = logValue;
				if (!all)
				{
					sprintf(OUTSTR,
							"ERROR: %s:F illegal on %s()", keyword, FUNCNAME);
					goto errorExit;
				}
				for (i = 0; i < NALL; i++)
				{
					if (i != JCOVAR)
					{
						doit[i] = abs(doit[i]);
					}
				}
			} /*if (strcmp(keyword, "all") == 0)*/
			else
			{
				if (strncmp(keyword, compname[JMEAN], 4) == 0)
				{
					which1 = JMEAN;
				}
				else if (strncmp(keyword, compname[JVAR],3 )  == 0)
				{
					which1 = JVAR;
				}
				else if (strncmp(keyword, compname[JCOVAR], 5)  == 0)
				{
					which1 = JCOVAR;
				}
				else if (strncmp(keyword, compname[JCOUNT], 5) == 0 ||
						 strcmp(keyword, "n") == 0)
				{
					which1 = JCOUNT;
				}
				else if (strncmp(keyword, compname[JSTDDEV], 3)  == 0)
				{
					which1 = JSTDDEV;
				}
				else if (strncmp(keyword, compname[JMIN], 3)  == 0)
				{
					which1 = JMIN;
				}
				else if (strncmp(keyword, compname[JMAX], 3)  == 0)
				{
					which1 = JMAX;
				}
				else if (strncmp(keyword, compname[JSUM], 3)  == 0)
				{
					which1 = JSUM;
				}
				else if (strncmp(keyword, compname[JPROD], 4)  == 0)
				{
					which1 = JPROD;
				}
				else
				{
					badKeyword(FUNCNAME, keyword);
					goto errorExit;
				}

				doit[which1] = logValue;
			} /*if (strcmp(keyword, "all") == 0){}else{}*/
		}
	} /*for (; nargs > 0; nargs--) */

	if (defaultOnly && y != (double **) 0)
	{
		control = IMEAN | IVAR | ICOUNT;
		ncomps = NDEFAULT;
	}
	else
	{
		ncomps = 0;

		for (i = 0; i < NALL; i++)
		{
			if (doit[i] > 0)
			{
				control |= 1 << i;
				ncomps++;
			}
		} /*for (i = 0; i < NALL; i++)*/
		if (y == (double **) 0)
		{
			if (doit[JCOUNT] <= 0)
			{
				control |= ICOUNT;
				ncomps++;
			}
		}
	}
	if (ncomps == 0)
	{
		sprintf(OUTSTR, "ERROR: nothing specified for %s() to compute",
				FUNCNAME);
		goto errorExit;
	}

	nfactors = nargs - 1;

	if (y == (double **) 0)
	{
		/* no data vector; possibly computing cell counts*/
		if (nfactors < 1)
		{
			sprintf(OUTSTR,
					"ERROR: no data or factors provided to %s()", FUNCNAME);
		}
		else if (!defaultOnly && (ncomps > 1 || !(control & ICOUNT)))
		{
			sprintf(OUTSTR,
					"ERROR: %s() needs data when computing anything other than cell counts",
					FUNCNAME);
		} /*if (ncomps > 1 || !(control & ICOUNT))*/
		if (*OUTSTR)
		{
			goto errorExit;
		}
	} /*if (y == (double **) 0)*/
	else if (anyMissing(symhY))
	{
		if (control & ICOVAR)
		{
			sprintf(OUTSTR,
					"ERROR: MISSING values in argument 1 not allowed with '%s:T'",
					compname[JCOVAR]);
			goto errorExit;
		}
		if (!silent)
		{
			sprintf(OUTSTR,
					"WARNING: MISSING values in argument 1 to %s() omitted",
					FUNCNAME);
			putErrorOUTSTR();
		}
	}

	if (nfactors + (ny > 1) + ((control & ICOVAR) != 0) > MAXDIMS)
	{
		sprintf(OUTSTR,
				"ERROR: %sresult would have more than %d dimensions",
				(ncomps == 1) ? NullString : "component of ", MAXDIMS);
		goto errorExit;
	}

	/* Check factors and compute number of cells */
	ncells = 1;
	for (i = 0; i < nfactors; i++)
	{
		symh = COMPVALUE(list, (i + 1));
		if (!argOK(symh, 0, i + 2))
		{
			goto errorExit;

		}
		type = TYPE(symh);
		if (type != REAL && type != LOGIC || !isVector(symh))
		{
			sprintf(OUTSTR,
					"ERROR: argument %ld to %s() not REAL or LOGICAL vector",
					i + 2, FUNCNAME);
			goto errorExit;
		}

		if (anyMissing(symh))
		{
			foundMissingInFactor++;
		}

		if (ndata < 0)
		{
			ndata = symbolSize(symh);
		}

		nclass[i] = labs(isFactor(symh));
		if (nclass[i] > MAXFACTORLEVELS)
		{
			sprintf(OUTSTR,
					"ERROR: maximum level of factor argument to %s() > %d",
					FUNCNAME, MAXFACTORLEVELS);
		}
		else if (nclass[i] == 0)
		{
			sprintf(OUTSTR,
					"ERROR: elements of factor argument to %s() too big or not positive integers",
					FUNCNAME);
		}
		else if (symbolSize(symh) != ndata)
		{
			sprintf(OUTSTR,
					"ERROR: length of factor %ld doesn't match other arguments of %s()",
					i + 1, FUNCNAME);
		}
		else if (0 && nclass[i] > MAXCOORD/ncells)
		{
			sprintf(OUTSTR,
					"ERROR: table produced by %s() has more than %ld cells",
					FUNCNAME, (long) MAXCOORD);
		}
		
		if (*OUTSTR)
		{
			goto errorExit;
		}
		ncells *= nclass[i];
	} /*for (i = 0; i < nfactors; i++)*/

	if (foundMissingInFactor > 0 && !silent)
	{
		sprintf(OUTSTR,
				"WARNINGS: MISSING values found in %d factor%s",
				foundMissingInFactor,
				(foundMissingInFactor > 1) ? "s" : NullString);
		putErrorOUTSTR();
	}
	
	needed = ncells*ny;
	if ((control & (ICOUNT|IMEAN|IVAR|ISTDDEV|ISUM|IPROD|ICOVAR)) &&
		!getScratch(out[JCOUNT], GCOUNT, needed, double) ||
		(control & (IMEAN | IVAR | ISTDDEV | ICOVAR)) &&
		!getScratch(out[JMEAN], GMEAN, needed, double) ||
		(control & IVAR) &&
		!getScratch(out[JVAR], GVAR, needed, double) ||
		(control & ISTDDEV) &&
		!getScratch(out[JSTDDEV], GSTDDEV, needed, double) ||
		(control & IMIN) &&
		!getScratch(out[JMIN], GMIN, needed, double) ||
		(control & IMAX) &&
		!getScratch(out[JMAX], GMAX, needed, double) ||
		(control & ISUM) &&
		!getScratch(out[JSUM], GSUM, needed, double) ||
		(control & IPROD) &&
		!getScratch(out[JPROD], GPROD, needed, double) ||
		(control & ICOVAR) &&
		(!getScratch(out[JCOVAR], GCOVAR, needed*ny, double) ||
		 ny > WORKSIZE && !getScratch(out[JWORK], GWORK, ny, double)))
	{
		goto errorExit;
	}
	
	result = (ncomps == 1) ? RInstall(SCRATCH, 0) :
		RSInstall(SCRATCH, ncomps, (char **) 0, 0);

	if (result == (Symbolhandle) 0)
	{
		goto errorExit;
	}
	
	nclass[nfactors + 1] = nclass[nfactors] = ny;
	
	for (k = i = 0;i < NALL;i++)
	{
		if (control & (1 << i))
		{
			long        ndims = nfactors;
			
			symh = (ncomps > 1) ? COMPVALUE(result, k++) : result;
			unTrash(i);
			setDATA(symh, out[i]);

			if (i == JCOVAR)
			{
				ndims += 2;
			}
			else if (ny > 1)
			{
				ndims++;
			}
			else if (ndims == 0)
			{
				ndims = 1;
			}
			
			Setdims(symh, ndims, nclass);
			if (ncomps > 1)
			{
				setNAME(symh, compname[i]);
			}
			else
			{
				break;
			}
		} /*if (control & (1 << i))*/
	} /*for (i = 0;i < NALL;i++)*/

	if (!celltabs(list, nfactors, out, control) && !silent)
	{
		sprintf(OUTSTR,
				"WARNING: overflow in %s() computations; set to MISSING",
				FUNCNAME);
		putErrorOUTSTR();
	} /*if (!celltabs(list, nfactors, out, control))*/	
	
	emptyTrash();

	return (result);

  errorExit:
	putErrorOUTSTR();
	emptyTrash();
	Removesymbol(result);
	
	return(0);
	
} /*Symbolhandle tabs()*/


