# File to be included by Makefile for MacAnova 4.13, version of 030519
# It specifies the construction of tar files for distribution

# Copyright 2003 Christopher Bingham and Gary Oehlert
# See file COPYING for details
# This may be freely distributed as long as this notice is retained
# and file COPYING is included.

# Values of VERSIONNUMBER and RELEASENUMBER must be provided, either
# by setting environmental variables as in
#  VERSIONNUMBER=4.13;RELEASENUMBER=2;export VERSIONNUMBER RELEASENUMBER; make -f tarfiles.make tarfiles
# or (if make allows it) on the make command line
# make VERSIONNUMBER=4.13 RELEASENUMBER=2 -f tarfiles.make tarfiles

# 980718 added mvsave.h to list of source files
# 980722 added mvhelp.h to list of source files
# 980827 added tser.hlp to list of auxiliary files
# 980901 changed documentation tar file name to docs$VERSIONNUMBER.tar
# 981119 added sun and linux86wx files
# 981210 added wxwin/wxwin168/msw/wx_text.cpp
# 990305 added iniMacro.h, iniMacro.mac and iniMacro.sh
# 990324 added arima.mac to AUXFILES
# 990523 added mvGraphs.h to SRCFILES
# 990609 changed matdat.h to mvIo.h
# 990619 removed reference to typedefs.h (no longer needed)
# 990831 updated macro DOCS to reflect the new configuration
# 000421 added mac/ProjectDefinesPpc.pch to MACSRCFILES
# 000904 added graphics.mac, math.mac, mulvar.mac and regress.mac to
#        AUXFILES; added mac/macIniMacro.r to MACSRCFILES
# 001003 Changed Userfun.hlp to userfun.hlp
# 001005 Added graphics.mac math.mac mulvar.mac regress.mac to AUXFILES
#        Macro FTPMV replaces DOCS; removed changes.txt
# 001011 Makefile.dir/Makefile.develop and Makefile.dir/Makefile.distrib 
#        replace Makefile.
# 001123 Added findfact.c to SRCFILES
# 010529 Added unxIface.h, dosIface.h to SRCFILES
# 010601 Added mvIface.h to SRCFILES
# 010926 Added makesymb.c to SRCFILES
# 030519 Added equal.c to SRCFILES
# 030708 Added lowessmv.c and utils.h to SRCFILES
# 030913 Added mvParse.h to SRCFILES

# Constants for making tar files
TARFLAGS = chf#create, follow symbolic links
TARVERSION = $(VERSIONNUMBER).$(RELEASENUMBER)#version and release number

#directory for files for distribution
#DISTRIBUTION = /FACHOME/pigseye/kb/src/macanova/distribution$(VERSIONNUMBER)
DISTRIBUTION = $(HOME)/src/macanova/distribution$(VERSIONNUMBER)

#tar file names
SRCTARFILE = macanova$(TARVERSION).tar
AUXTARFILE = auxfiles$(TARVERSION).tar
OBJTARFILE = objfiles$(TARVERSION).tar
DOCSTARFILE = $(DISTRIBUTION)/docs$(VERSIONNUMBER).tar

TESTFILESDIR = testfiles$(VERSIONNUMBER)
INSTALLSCRIPT = $(DISTRIBUTION)/src/install.sh
#DISTRIBUTION = ../distribution$(VERSIONNUMBER)

#FTPMV = /usr/local/FTP/pub/macanova# ftp directory
FTPMV = /APPS/FTP/pub/macanova# ftp directory

# Lists of files for tar
AUXFILES =	COPYING copying.dj macanova.dat\
	macanova.hlp macanova.nws macanova.ini macanova.mac\
	tser.mac tser.hlp design.mac design.hlp arima.mac userfun.hlp\
	graphics.mac math.mac mulvar.mac regress.mac

# One of the *.env files, renamed make.env, should be in the same
# directory as Makefile, the controlling make file.
# Makefile.develop is identical to the Makefile used in developing
# MacAnova at the U of Minnesota
# Makefle.distrib is a modification the does not have files related
# to parsers (mainpars.c mainpars.h mreader.c) as target.  It is what
# should normally be used when porting MacAnova to a new environment
# and should be copied to the main source directory.  This is done
# by script install.sh

MAKEFILES = \
	Makefile.dir/Makefile.develop \
	Makefile.dir/Makefile.distrib \
	Makefile.dir/hpcc.env \
	Makefile.dir/hpgcc.env \
	Makefile.dir/hpgccwx.env \
	Makefile.dir/linux86.env \
	Makefile.dir/linux86wx.env \
	Makefile.dir/sgi.env \
	Makefile.dir/sgiwx.env \
	Makefile.dir/sun.env \
	Makefile.dir/sunwx.env

# Obsolete Make files or Makefiles for which no *.env file has been
# provided.

OLDMAKEFILES = \
	Makefile.dir/OLD/Makefile.dec Makefile.dir/OLD/Makefile.epx\
	Makefile.dir/OLD/Makefile.gcc.hp Makefile.dir/OLD/Makefile.hp\
	Makefile.dir/OLD/Makefile.sgi Makefile.dir/OLD/Makefile.wx.hp

# files specific to a platform/compiler combination that are
# included by platform.h at compile time.
PLATFORMFILES = \
	platfrms/alphavms.h platfrms/linux86.h   platfrms/sgiwx.h \
	platfrms/bcpp.h     platfrms/linux86wx.h platfrms/sun.h \
	platfrms/djgpp.h    platfrms/mpw.h       platfrms/sunwx.h \
	platfrms/genunix.h  platfrms/mpwmrc.h    platfrms/vaxvms.h \
	platfrms/hpcc.h     platfrms/mpwsc.h     platfrms/wxwinmsw.h \
	platfrms/hpgcc.h    platfrms/mwcw.h \
	platfrms/hpgccwx.h  platfrms/sgi.h

# mainpars.sed.xxx, mainpars.awk.xxx and mainpars.sed.xxx are scripts to
# post process output from the version of yacc being used
# byacc is appropriate for Berdeley yacc 1.9
# hp    is appropriate for HPUX yacc
# dec   is appropriate for DEC Ultrix yacc
# iniMacro.sh is a shell script to produce iniMacro.h from iniMacro.mac,
#   which contains the pre-defined macros in a form readable by macroread()
# iniMacro.awk is an awk or gawk program that produces the Macintosh rez
#   file macIniMacro.r from iniMacro.h
# fixmainp.hp.c is source for fixmainp which post processes yacc or byacc
#   output changing all occurrences of printf(fmt, ...) in mainpars.c to
#   (sprintf(OUTSTR, fmt, ...), myprint(OUTSTR)); 
# Use of these is all commented out in the distributed Makefile, since
# normally the parsers mreader.c and mainpars.c don't need to be recreated
# when porting MacAnova.

MAINTAINENCE =\
	mainpars.sed.byacc mainpars.awk.byacc mreader.sed.byacc\
	mainpars.sed.dec   mainpars.awk.dec   mreader.sed.dec\
	mainpars.sed.hp    mainpars.awk.hp    mreader.sed.hp\
	iniMacro.awk       iniMacro.sh        iniMacro.mac\
	dobinarytest       whichbinaries      swapbinaries\
	fixmainp.hp.c
 
SRCFILES = \
	Ifsetup.c    Lang.c       Macroset.c   Subassig.c   Symbol.c    \
	Symbol.h     anovacoe.c   array.c      batch.c      betabase.c  \
	bin.c        blas.c       blas.h       boxplot.c    cellstat.c  \
	cellstts.c   changest.c   cholesky.c   cluster.c    columnop.c  \
	comIface.c   comclipb.c   comhandl.c   commonio.c   concat.c    \
	contrast.c   cor.c        dbug.h       delete.c     describe.c  \
	dim.c        dunnett.c    dynload.c    dynload.h    eigen.c     \
	eigutils.c   equal.c      errorMsg.c   fft.c        fftProto.h  \
	fftsubs.c    findfact.c   funbalan.c   gammabas.c   getmodel.c  \
	glm.c        glm.h        glmutils.c   globals.h    globdefs.h  \
	globkb.h     gramschm.c   graphics.c   handlers.c   handlers.h  \
	hconcat.c    help.c       iniMacro.c   iniMacro.h   initiali.c  \
	ioProto.h    ipf.c        iswhat.c     iterfns.c    iterglm.c   \
	keywords.c   keywords.h   kmeans.c     labels.h     labutils.c  \
	linpacCH.c   linpacGE.c   linpacQR.c   linpack.h    list.c      \
	lowessmv.c   lpsforwd.c   lpslandb.c   lpsmisc.c    macro.c     \
	main.c       mainpars.c   mainpars.h   mainpars.y   makeFact.c  \
	makestr.c    makesymb.c   matProto.h   mathutil.c   matrix.c    \
	movavg.c     mreader.c    mreader.y    mvGraphs.h   mvIface.h   \
	mvIo.h       mvParse.h    mvhelp.h     mvsave.h     mvutils.h   \
	myplot.c     normbase.c   odometer.c   outer.c      partacf.c   \
	paste.c      platform.h   plot.h       plotutil.c   polyroot.c  \
	power.c      predtabl.c   print.c      printano.c   printreg.c  \
	pvals.c      pvalsub.c    qr.c         random.c     rankquic.c  \
	rational.c   readFile.c   readHead.c   readdata.c   regpred.c   \
	rep.c        restore.c    rotation.c   run.c        save.c      \
	screen.c     select.c     setOptio.c   shell.c      solve.c     \
	sort.c       sortquic.c   split.c      spool.c      startMsg.c  \
	stemleaf.c   stemsubs.c   studentb.c   studrang.c   svd.c       \
	swp.c        tabs.c       term.c       toeplitz.c   trans.c     \
	transpos.c   trideige.c   tser.h       tserProt.h   tserops.c   \
	tsersubs.c   ttests.c     unbalano.c   unxHandl.c   unxIface.h  \
	unxIo.c      unxProto.h   utils.c      utilstru.c   varnames.c  \
	version.h    yates.c      yylex.c 

# The following are header files for readline2.0
READLINE2HEADERFILES = \
    readline2/chardefs.h readline2/history.h readline2/keymaps.h\
	readline2/readline.h readline2/tilde.h

WXSRCFILES =\
	wx/wxIface.h     wx/wxProto.h     wx/wxgraph.h     wx/wxhandl.c\
	wx/wxmain.h      wx/wxIo.cc       wx/wxPrint.cc    wx/wxdialog.cc\
	wx/wxgraph.cc    wx/wxmain.cc     wx/wxtframe.cc   wx/wxtwind.cc\
	wx/aiai.xbm      wx/fload.xbm\
	wx/make.env # modified version of $(WXDIR)/src/make.env


# Modified wxwin 1.65 files which were used in prior to Version 4.04
WXWIN165FILES = wxwin/wxwin165/readme.txt wxwin/wxwin165/base/wb_print.cc\
		wxwin/wxwin165/base/wb_ps.cc wxwin/wxwin165/msw/wx_main.cc\
		wxwin/wxwin165/x/wx_ipc.cc wxwin/wxwin165/x/wx_main.cc \
		wxwin/wxwin165/x/wx_text.cc

WXWIN168FILES = wxwin/wxwin168/readme.txt wxwin/wxwin168/msw/wx_text.cpp

DOSSRCFILES = dos/dosProto.h dos/dosHandl.c dos/dosIo.c dos/readme.dos \
		dos/dosIface.h

DOSWXFILES = dos/wx/readmewx.txt dos/wx/macanova.rc dos/wx/macanova.ico \
		dos/wx/icon.dmp

DJGPPSRCFILES = dos/djgpp/readme.djg\
	dos/djgpp/readline/readline.c dos/djgpp/readline/chardefs.h\
	dos/djgpp/readline/history.h  dos/djgpp/readline/keymaps.h\
	dos/djgpp/readline/readline.h dos/djgpp/readline/sysdep.h\
	dos/djgpp/djgpp2/Makefile.dj2 dos/djgpp/djgpp2/doprnt.c\
	dos/djgpp/djgpp1/Makefile.djg dos/djgpp/djgpp1/control.c\
	dos/djgpp/djgpp1/doprnt.c     dos/djgpp/djgpp1/exphdlr.c

BCPPSRCFILES = dos/bcpp/maov4p0.mak dos/bcpp/readme.bc

MACSRCFILES = \
	mac/MacAnova.r            mac/TransSkel.c      mac/macMain.c     \
	mac/MacAnovaFonts.Hqx     mac/TransSkel.h      mac/macMainpars.c \
	mac/Makefile.mpw          mac/TransSkelProto.h mac/macMainpars.r \
	mac/ProjectDefinesPpc.pch mac/macBatchMode.c   mac/macPrint.c    \
	mac/SkelApple.c           mac/macFindFile.c    mac/macProto.h    \
	mac/SkelEvents.c          mac/macGraph.c       mac/macUtils.c    \
	mac/SkelGetRectDevice.c   mac/macHandle.c      mac/profile.h     \
	mac/SkelGetWindDevice.c   mac/macIface.h       mac/readme.mac    \
	mac/SkelGetWindRect.c     mac/macInput.c       mac/macIniMacro.r \
	mac/SkelWindTitleHt.c     mac/macIo.c

USRFUNSRCFILES = userfun/Userfun.h userfun/dynload.h userfun/goo.c\
	userfun/fooeval.c\
	userfun/dos/fooeval.ide userfun/dos/goo.ide\
	userfun/mac/UserFunction.pch userfun/mac/fooevalinfo.mu.Hqx\
	userfun/mac/goo.mu.Hqx userfun/mac/gooppc.mu.Hqx\
	userfun/mac/fooeval.mu.Hqx userfun/mac/fooevalppc.mu.Hqx\
	userfun/mac/gooarginfo.mu.Hqx

DOCUMENTATIONFILES =\
	Intro.pdf           README\
	referenc.pdf        manual/README       manual/manapdxa.pdf\
	manual/manapdxb.pdf manual/manapdxc.pdf manual/manapdxd.pdf\
	manual/manapdxe.pdf manual/manapdxf.pdf manual/manchp01.pdf\
	manual/manchp03.pdf manual/manchp04.pdf manual/manchp05.pdf\
	manual/manchp06.pdf manual/manchp07.pdf manual/manchp08.pdf\
	manual/manchp09.pdf manual/manchp10.pdf manual/manchp11.pdf\
	manual/manchp2a.pdf manual/manchp2b.pdf manual/mancntnt.pdf\
	manual/manpref.pdf  manual/manrefs.pdf

TESTFILES = \
	$(TESTFILESDIR)/TestChapter2a.in $(TESTFILESDIR)/TestChapter2a.out.unx\
	$(TESTFILESDIR)/TestChapter2b.in $(TESTFILESDIR)/TestChapter2b.out.unx\
	$(TESTFILESDIR)/TestChapter3.in  $(TESTFILESDIR)/TestChapter3.out.unx\
	$(TESTFILESDIR)/TestChapter4.in  $(TESTFILESDIR)/TestChapter4.out.unx\
	$(TESTFILESDIR)/TestChapter5.in  $(TESTFILESDIR)/TestChapter5.out.unx\
	$(TESTFILESDIR)/TestChapter6.in  $(TESTFILESDIR)/TestChapter6.out.unx\
	$(TESTFILESDIR)/TestChapter7.in  $(TESTFILESDIR)/TestChapter7.out.unx\
	$(TESTFILESDIR)/TestChapter8.in  $(TESTFILESDIR)/TestChapter8.out.unx\
	$(TESTFILESDIR)/TestChapter9.in  $(TESTFILESDIR)/TestChapter9.out.unx\
	$(TESTFILESDIR)/TestChapter10.in $(TESTFILESDIR)/TestChapter10.out.unx\
	$(TESTFILESDIR)/data/Hald        $(TESTFILESDIR)/MacAnova.dat\
	$(TESTFILESDIR)/MacAnova.mac     $(TESTFILESDIR)/Tser.mac\
	$(TESTFILESDIR)/mybatch.txt      $(TESTFILESDIR)/mybatch1.txt\
	$(TESTFILESDIR)/readme.tes

ALLSRCFILES = \
	$(MAKEFILES) $(OLDMAKEFILES) $(MAINTAINENCE) $(SRCFILES)\
	$(READLINE2HEADERFILES)\
	$(WXSRCFILES) $(DOSSRCFILES) $(DJGPPSRCFILES) $(BCPPSRCFILES)\
	$(DOSWXFILES) $(WXWIN168FILES)\
	$(MACSRCFILES) $(USRFUNFILES) $(PLATFORMFILES)

tarsrcfiles:
	rm -f $(SRCTARFILE)
	tar $(TARFLAGS) $(SRCTARFILE) $(ALLSRCFILES) $(TESTFILES)
	tar vtf $(SRCTARFILE) > $(DISTRIBUTION)/$(SRCTARFILE).dir
	gzip $(SRCTARFILE)
	mv $(SRCTARFILE).gz $(DISTRIBUTION)
	mv $(INSTALLSCRIPT) $(INSTALLSCRIPT).bak
	sed -e 's/^VERSION=.*/VERSION=$(VERSIONNUMBER)/'\
		 -e 's/^RELEASE=.*/RELEASE=$(RELEASENUMBER)/'\
		 $(INSTALLSCRIPT).bak > $(INSTALLSCRIPT)
	cp $(INSTALLSCRIPT) $(DISTRIBUTION)

tarauxfiles:
	cd $(DISTRIBUTION)/aux_files;\
	tar $(TARFLAGS) ../$(AUXTARFILE) $(AUXFILES); \
	tar vtf ../$(AUXTARFILE) > ../$(AUXTARFILE).dir;\
	rm -f ../$(AUXTARFILE).gz; \
	gzip ../$(AUXTARFILE)

tardocfiles:
	cd $(FTPMV)/docs ; tar $(TARFLAGS) $(DOCSTARFILE) $(DOCUMENTATIONFILES)
	tar vtf $(DOCSTARFILE) > $(DOCSTARFILE).dir
	rm -f $(DOCSTARFILE).gz
	gzip $(DOCSTARFILE)

tarobjfiles:
	tar $(TARFLAGS) $(OBJTARFILE) $(ALLOBJ)

tarfiles: tarsrcfiles tarauxfiles
