/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.11 or later
*(C)*
*(C)* Copyright (c) 2001 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/


#ifdef SEGMENTED
#if defined(MPW) || defined(MW_CW)
#pragma segment Transpose
#endif /*MPW||MW_CW*/
#endif /*SEGMENTED*/

#include "globals.h"

/*
  970801 minor change:  When NDIMS(arg) == 1 and arg has a label,
         the row label of arg' is now "@" (expandable numeric) instead
         of "" (no label).  The old behavior can be gotten by
         matrix(a',labels:structure(getlabels(a),""))

  971104 added check on value of myhandlelength()

  981015 minor changes.  dim[0] is used instead of ndims until actual
         copying is done.
  990212 Changed putOUTSTR() to putErrorOUTSTR()
  000305 Modified to handle generalized transpose (permutation of dimensions)
*/

Symbolhandle    doTranspose(Symbolhandle arg, double *params,
								   unsigned long control,
								   unsigned long *status)
{
	Symbolhandle result = (Symbolhandle) 0;
	long         type = (arg != (Symbolhandle) 0) ? TYPE(arg) : NULLSYM;
	WHERE("doTranspose");
	
	*status = 0;
	result = Install(SCRATCH,type);
	if (result == (Symbolhandle) 0)
	{
		goto errorExit;
	}
	
	
	if (type != NULLSYM)
	{
		long         size, j, rplace, aplace;
		long         kr;
		long         ndims;
		long         dim[MAXDIMS+1], newdim[MAXDIMS];
		long         tmpincs[MAXDIMS];
		long         ii[MAXDIMS], incs[MAXDIMS], place[MAXDIMS];
		int          jj[MAXDIMS];
		long         handleLength;
		short        charType = (type == CHAR);
		short        isChange = (params == (double *) 0);
		short        genTranspose = (params != (double *) 0);
		double     **axH, **rxH;
		double      *ax, *rx;
		char       **asH, **rsH;
		char        *as, *rs;

		size = symbolSize(arg);
		getDims(dim, arg);
		ndims = dim[0];

		if (ndims == 1 && !genTranspose)
		{
			/* note: original value of ndims is still in dim[0] */
			ndims = 2;
			dim[2] = 1;
		}

		/* permute dimensions*/
		for (j = 0; j < ndims; j++)
		{
			if (genTranspose)
			{
				jj[j] = (long) params[j] - 1;
				if (jj[j] != j)
				{
					isChange = 1;
				}
			}
			else
			{
				jj[j] = ndims - j - 1;
			}
			tmpincs[j] = (j == 0) ? 1 : dim[j]*tmpincs[j-1];
			newdim[j] = dim[jj[j] + 1];
		} /*for (j = 0; j < ndims; j++)*/

		if (!isChange)
		{
			/* permutation is identity; just copy symbol */
			mvSymbolName      symName;
			
			strcpy(symName, NAME(result));
			
			if (!Copy(arg, result))
			{
				goto errorExit;
			}
			setNAME(result, symName);
		} /*if (!isChange)*/
		else
		{
			Setdims(result, ndims, newdim);
		
			if (charType)
			{		
				asH = STRING(arg);
				handleLength = myhandlelength(asH);
				if (handleLength < 0)
				{
					goto errorExit;
				}
				TMPHANDLE = mygethandle(handleLength);
				setSTRING(result,TMPHANDLE);
				rsH = STRING(result);
			}
			else
			{
				axH = DATA(arg);
				TMPHANDLE = mygethandle(myhandlelength((char **) axH));
				setDATA(result,(double **) TMPHANDLE);
				rxH = DATA(result);
			}
			if (TMPHANDLE == (char **) 0)
			{
				goto errorExit;
			}


			if (!charType)
			{
				ax = *axH;
				rx = *rxH;
			}
			else
			{
				as = *asH;
				rs = *rsH;
			}
	
			if (HASLABELS(arg))
			{
				char          *labels[MAXDIMS];
				long           lengths[MAXDIMS];
				char           numericLabel[2];
			
				getAllLabels(arg, labels, lengths, (long *) 0);

				if (dim[0] == 1)
				{ /* make room for 1 exandable numeric row label */
					numericLabel[0] =  NUMERICLABEL;
					numericLabel[1] = '\0';
					lengths[1] = lengths[0];
					lengths[0] = 2;
				}
				else
				{
					/* reorder lengths */
					long        tmplengths[MAXDIMS];
				
					for (j = 0; j < ndims; j++)
					{
						tmplengths[j] = lengths[j];
					}
					for (j = 0; j < ndims; j++)
					{
						lengths[j] = tmplengths[jj[j]];
					}
				}
			
				TMPHANDLE = createLabels(ndims, lengths);
				if (TMPHANDLE == (char **) 0 || !setLabels(result, TMPHANDLE))
				{
					mydisphandle(TMPHANDLE);
					goto errorExit;
				}
				getAllLabels(arg, labels, (long *) 0, (long *) 0);
				if (dim[0] == 1 && ndims == 2)
				{
					/*
						arg was vector and operation is ordinary transpose
						new first dimension is being
						created and given a numerica row label
					*/
					labels[1] = numericLabel;
				}
				/* append dimension labels in permuted order */
				for (j = 0; j < ndims; j++)
				{
					appendLabels(result, labels[jj[j]], j, dontExpand);
				}
			} /*if (HASLABELS(arg))*/

			for (j = 0; j < ndims; j++)
			{
				ii[j] = place[j] = 0;
				incs[j] = tmpincs[jj[j]];
			} /*for (j = 0; j < ndims; j++)*/

			/*
				Step through result element by element, picking
				out elements from arg in permuted order

				aplace is element number (REAL, LOGICAL arg) or byte
				number (CHARACTER arg) in ax (from arg) and rplace
				is element or byte number in rx (from result)
			*/

			rplace = aplace = 0;
			for (kr = 0; kr < size; kr++)
			{
				if (!charType)
				{
					rx[rplace++] = ax[aplace]; 
				}
				else
				{/* string equivalent of rx[rplace++] = ax[aplace]*/
					rplace = copyStrings(as + aplace, rs + rplace, 1) - rs; 
				}
		
				/* 
				   The following is essentially equivalent to
				   ndims nested for loops over subscripts in result
				*/
				for (j = 0; j < ndims; j++)
				{
					if (ii[j] == 0)
					{
						place[j] = aplace; 
					}
					if (++ii[j] < newdim[j])
					{
						if (!charType)
						{
							aplace += incs[j]; 
						}
						else
						{/* string equivalent of aplace += incs[j] */
							aplace = skipStrings(as + aplace, incs[j]) - as; 
						}
						break; 
					} /*if (++ii[j] < newdim[j])*/
					ii[j] = 0; 
					aplace = place[j]; 
				} /*for (j = 0; j < ndims; j++)*/
			} /*for (kr = 0; kr < size; kr++)*/
		} /*if (!isChange){}else{}*/
		
#if (USENOMISSING)
		if (NOMISSING(arg))
		{
			setNOMISSING(result);
		}
#endif /*USENOMISSING*/
	} /*if (type != NULLSYM)*/ 

	return(result);
	
  errorExit:
	Removesymbol(result);
	return (0);
} /*doTranspose()*/

static long checkArgNdims(Symbolhandle arg, double *ndims,
					   unsigned long control, unsigned long *status)
{
	long     ndims1 = (long) *ndims;
	long     ndims2 = (isNull(arg)) ? 0 : NDIMS(arg);
	
	return (ndims2 == 0 || ndims1 == ndims2);
} /*checkArgNdims()*/

/*
   Returns non-zero if and only if all non-structure, non-null
   components have NDIMS(component) == ndims
*/

static long checkNdims(Symbolhandle arg, int ndims)
{
	unsigned long    status = 0;
	unsigned long    control = LEFTRECUR;
	double           dNdims = (double) ndims;

	return (doRecurCheck1(checkArgNdims, arg, &dNdims, control,
						   &status));
} /*checktypes()*/

static long argIsRealLogicOrChar(Symbolhandle arg, double *params,
								 unsigned long control, unsigned long *status)
{
	long     type = (arg != (Symbolhandle) 0) ? TYPE(arg) : NULLSYM;

	return (type == REAL || type == LOGIC || type == CHAR || type == NULLSYM);
}

/*
   Returns non-zero if and only if all non-structure components are non-null
   and have type REAL, LOGIC, or CHAR
*/

static long checktypes(Symbolhandle arg)
{
	unsigned long    status = 0;
	unsigned long    control = LEFTRECUR;
	
	return (doRecurCheck1(argIsRealLogicOrChar, arg, (double *) 0, control,
						   &status));
} /*checktypes()*/


/*
  Make a transpose of a matrix
  Reverse order of dimensions for an array
  usage:
    y <- x' or y <- t(x)
    When ndims(x) = 1, y is 1 by ndims(x) with y[1,j] = x[j]
	When ndims(x) > 1
      dim(y) = reverse(dim(x))
      y[i1,i2,...,ip] = x[ip,...,i2,i1]

  Permute dimensions of an array
    y <- t(x,J) # p = ndims(x), J = vector(j1,...,jp) permutation of run(p)
     with y <- t(x,j1,j2,...) equivalent to t(x,vector(j1,j2,...)).
	 Result:
	    dim(y) = dim(x)[J], and if I is a vector of length p
	    y[I[J[1]],I[J[2]],...,I[J[p]]] = x[I[1],I[2],...,I[p]]

     When p = ndims(x) > 1, t(x,run(p,1)) is equivalent to t(x)

  010516 Use atLeastNArgs constant on badNargs() call
*/

Symbolhandle    transpose(Symbolhandle list)
{

	/* make a transpose of a matrix */

	Symbolhandle    result, arg;
	Symbolhandle    symhJ;
	double          permutation[MAXDIMS];
	long            nargs = NARGS(list);
	unsigned long   control = 0, status;
	int             i, ndims;
	WHERE("transpose");

	*OUTSTR = '\0';
	
	arg = COMPVALUE(list, 0);

	if (nargs == 1 && arg == (Symbolhandle) 0)
	{
		badNargs(FUNCNAME, -1 + atLeastNArgs);
		goto errorExit;
	}
	
	if (!argOK(arg, (long) 0, (nargs > 1) ? 1 : 0))
	{
		goto errorExit;
	}

	if (!checktypes(arg))
	{
		if (isStruc(arg))
		{
			sprintf(OUTSTR,
					"ERROR: component of structure not REAL, LOGICAL, CHARACTER or NULL for %s()",
					FUNCNAME);
		}
		else
		{
			badType(FUNCNAME, -TYPE(arg), (nargs > 1) ? 1 : 0);
		}
		goto errorExit;
	} /*if (!checktypes(arg))*/

	if (isStruc(arg))
	{
		control |= LEFTRECUR;
	}
	if (nargs > 1)
	{
		int             jj[MAXDIMS], check[MAXDIMS], maxjval = 0;

		for (i = 0; i < MAXDIMS; i++)
		{
			check[i] = 0;
		}
		ndims = 0;
		for (i = 1; i < nargs; i++)
		{
			int           j, size;
			
			symhJ = COMPVALUE(list, i);
			if (!argOK(symhJ, REAL, i + 1))
			{
				goto errorExit;
			}
			size = symbolSize(symhJ);
			for (j = 0; j < size; j++)
			{
				double     value = DATAVALUE(symhJ, j);
				long       jvalue;
				
				if (isMissing(value) || value != floor(value) ||
					value < 1.0 || value > (double) MAXDIMS)
				{
					sprintf(OUTSTR,
							"ERROR: element of permutation for %s() not integer between 0 and %ld",
							FUNCNAME, (long) MAXDIMS);
					goto errorExit;
				}
				jvalue = (long) value;
				if (check[jvalue])
				{
					sprintf(OUTSTR,
							"ERROR: duplicate value in permutation for %s()",
							FUNCNAME);
					goto errorExit;
				}
				if (jvalue > maxjval)
				{
					maxjval = jvalue;
				}
				check[jvalue] = 1;
				jj[ndims++] = jvalue;
			} /*for (j = 0; j < size; j++)*/
		} /*for (i = 1; i < nargs; i++)*/

		if (!checkNdims(arg, ndims))
		{
			sprintf(OUTSTR,
					"ERROR: mismatch of permutation length for %s() argument dimension",
					FUNCNAME);
			goto errorExit;
		}
		if (maxjval > ndims)
		{
			sprintf(OUTSTR,
					"ERROR: max(permutation vector) > number of dimensions in %s()",
					FUNCNAME);
			goto errorExit;
		}
		for (i = 0; i < ndims; i++)
		{
			permutation[i] = (double) jj[i];
		}
	} /*if (nargs > 1)*/

	result = doRecur1(doTranspose, arg,
					  (nargs > 1) ? permutation : (double *) 0,
					  control, &status);

	if (result == 0)
	{
		goto errorExit;
	}
	
	return (result);

  errorExit:
	putErrorOUTSTR();

	return (0);
} /*transpose()*/
