/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.13 or later
*(C)*
*(C)* Copyright (c) 2003 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

#ifndef TSERH__
#define TSERH__

/* defines various constants for time series related functions */

#include "tserProt.h"

/*
  Codes for time series related functions

  000413 Dropped some unused codes; converted to enums where possible
  010108 Increased permissible number of factors to 21 from 19;
         added MAXNFACTOR to enum tserConstants.
         added new error code to report when new bug hit
  030305 Added codes for complex division
*/
#if defined(HPGCC) || defined(HPPA) || defined(MACINTOSH)
#define INTSARELONGS 1  /* figure out sometime how to do this */
#else
#define INTSARELONGS 0
#endif

#if (INTSARELONGS)
enum tserOpConstants
{
	IRFT      = 0x00000001,
	IHFT      = 0x00000002,
	ICFT      = 0x00000004,

	IHCONJ    = 0x00000008,
	ICCONJ    = 0x00000010,

	IHREAL    = 0x00000020,
	ICREAL    = 0x00000040,

	IHIMAG    = 0x00000080,
	ICIMAG    = 0x00000100,

	IHTOC     = 0x00000200,
	ICTOH     = 0x00000400,

	IHPRDH    = 0x00000800,
	IHPRDHJ   = 0x00001000,
	IHPRD     = (IHPRDH | IHPRDHJ),

	ICPRDC    = 0x00002000,
	ICPRDCJ   = 0x00004000,
	ICPRD     = (ICPRDC | ICPRDCJ),
	IPRD      = (IHPRD | ICPRD),

	IHDIVH    = 0x00008000,
	IHDIVHJ   = 0x00010000,
	IHDIV     = (IHDIVH | IHDIVHJ),

	ICDIVC    = 0x00020000,
	ICDIVCJ   = 0x00040000,
	ICDIV     = (ICDIVC | ICDIVCJ),
	IDIV      = (IHDIV | ICDIV),

	ICMPLX    = 0x00080000,

	IHPOLAR   = 0x00100000,
	ICPOLAR   = 0x00200000,
	IPOLAR    = (IHPOLAR | ICPOLAR),

	IHRECT    = 0x00400000,
	ICRECT    = 0x00800000,
	IRECT     = (IHRECT | ICRECT),

	IUNWIND   = 0x01000000,

	ICONVOLVE = 0x02000000,
	IROTATE   = 0x04000000,
	IPADTO    = 0x08000000,
	IREVERSE  = 0x10000000
};
#else /*INTSARELONGS*/
#define IRFT       0x00000001L
#define IHFT       0x00000002L
#define ICFT       0x00000004L

#define IHCONJ     0x00000008L
#define ICCONJ     0x00000010L

#define IHREAL     0x00000020L
#define ICREAL     0x00000040L

#define IHIMAG     0x00000080L
#define ICIMAG     0x00000100L

#define IHTOC      0x00000200L
#define ICTOH      0x00000400L

#define IHPRDH    0x00000800L
#define IHPRDHJ   0x00001000L
#define IHPRD     (IHPRDH | IHPRDHJ)

#define ICPRDC    0x00002000L
#define ICPRDCJ   0x00004000L
#define ICPRD     (ICPRDC | ICPRDCJ)
#define IPRD      (IHPRD | ICPRD)

#define IHDIVH    0x00008000L
#define IHDIVHJ   0x00010000L
#define IHDIV     (IHDIVH | IHDIVHJ)

#define ICDIVC    0x00020000L
#define ICDIVCJ   0x00040000L
#define ICDIV     (ICDIVC | ICDIVCJ)
#define IDIV      (IHDIV | ICDIV)

#define ICMPLX    0x00080000L

#define IHPOLAR   0x00100000L
#define ICPOLAR   0x00200000L
#define IPOLAR    (IHPOLAR | ICPOLAR)

#define IHRECT    0x00400000L
#define ICRECT    0x00800000L
#define IRECT     (IHRECT | ICRECT)

#define IUNWIND   0x01000000L

#define ICONVOLVE 0x02000000L
#define IROTATE   0x04000000L
#define IPADTO    0x08000000L
#define IREVERSE  0x10000000L
#endif /*INTSARELONGS*/


enum fftErrors
{
	FFTNOERROR   =     0,
	FFTTOOBIGPRIME,
	FFTTOOMANYFACTORS,
	FFTTOOLARGEUNPAIREDPRODUCT,
	FFTOUTOFMEMORY
};

enum tserConstants
{
	IHERMITIAN   =     0,
	IREAL,
	PMAX         =    29,  /* largest permissible prime factor */
	MAXNFACTOR   =    21,  /* maximum permitted number of factors */
	NEST                   /* maximum number of factors + 1 */
};



#endif /*TSERH__*/
