/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 5.03 or later
*(C)*
*(C)* Copyright (c) 2006 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/


#ifdef SEGMENTED
#if defined(MPW) || defined(MW_CW)
#pragma segment Tserops
#endif /*MPW||MW_CW*/
#endif /*SEGMENTED*/

/*
  990226 replaced putOUTSTR() by putErrorOUTSTR()
  990729 Deleted obsolete function names hcopyc() and ccopyh()
         (same as ctoh() and h2oc())
  001012 Fixed argument parsing bug in tserops()
  010515 rotate() and reverse() work with CHARACTER argument 1
  030314 added cdivc(), cdivcj(), hdivh(), hdivhj()
  060808 added keywords 'cycles", 'radians', 'degrees' to {c,h}{polar,rect}()
*/

#include "globals.h"
#include "keywords.h"
#include "tser.h"

typedef struct opEntry
{
	char *name;
	long iop;
	long nargs;
} opEntry;
/*
	0 < nargs < 10  means function expects exactly nargs nonkeyword arguments
	nargs < 0 means functions expects uptto abs(nargs) args, no keywords
	nargs = nkey*10 + nonkey means functions expects nonkeyword arguments
	   and up to nkey keyword phrases.
    On hpolar() and cpolar() unwind:F can be specified by simply F as arg 2
	On convolve(), reverse:T can be specified by simply T as arg 3
*/

static opEntry Ops[] =
{ /* name        iop      nargs */
	{"hconj",    IHCONJ,     1}, /* hconj(hx) */
	{"cconj",    ICCONJ,     1}, /* cconj(cx) */
	{"hreal",    IHREAL,     1}, /* hreal(hx) */
	{"creal",    ICREAL,     1}, /* creal(cx) */
	{"himag",    IHIMAG,     1}, /* himag(hx) */
	{"cimag",    ICIMAG,     1}, /* cimag(cx) */
	{"htoc",     IHTOC,      1}, /* htoc(hx) */
	{"ctoh",     ICTOH,      1}, /* ctoh(cx) */
	{"hprdh",    IHPRDH,    -2}, /* hprdh(hx1 [,hx2]) */
	{"hprdhj",   IHPRDHJ,   -2}, /* hprdhj(hx1 [,hx2]) */
	{"cprdc",    ICPRDC,    -2}, /* cprdc(cx1 [,cx2]) */
	{"cprdcj",   ICPRDCJ,   -2}, /* cprdcj(cx1 [,cx2]) */
	{"hdivh",    IHDIVH,    -2}, /* hdivh(hx1 [,hx2]) */
	{"hdivhj",   IHDIVHJ,   -2}, /* hdivhj(hx1 [,hx2]) */
	{"cdivc",    ICDIVC,    -2}, /* cdivc(cx1 [,cx2]) */
	{"cdivcj",   ICDIVCJ,   -2}, /* cdivcj(cx1 [,cx2]) */
	{"cmplx",    ICMPLX,    -2}, /* cmplx(rx1[, rx2]) */
	{"hpolar",   IHPOLAR,   31}, /* hpolar(hx [,unwind:F] [,crit:val]) */
	{"cpolar",   ICPOLAR,   31}, /* cpolar(cx [,unwind:F] [,crit:val]) */
	{"hrect",    IHRECT,    11}, /* hrect(hx) */
	{"crect",    ICRECT,    11}, /* crect(cx) */
	{"unwind",   IUNWIND,   11}, /* unwind(theta [,crit:val]) */
	{"convolve", ICONVOLVE, 22}, /* convolve(coefs,rx[,reverse:T][,decimate:T]) */
	{"rotate",   IROTATE,    2}, /* rotate(rx,lag) */
	{"reverse",  IREVERSE,   1}, /* reverse(rx) */
	{"padto",    IPADTO,     2}, /* padto(rx,nrows) */
	{(char *) 0, 0,          0}
};

#define CRITERION .75 /* default for keyword 'crit' */


enum Tser
{
	KUNWIND = 0,
	KCRITERION,
	KREVERSE,
	KDECIMATE,
	KRADIANS,
	KCYCLES,
	KDEGREES
};

static keywordList TserKeys[] = 
{
	InitKeyEntry("unwind", 0, IUNWIND | IPOLAR, LOGICSCALAR), 
	InitKeyEntry("criterion", 4, IUNWIND | IPOLAR, REALSCALAR), 
	InitKeyEntry("reverse", 0, ICONVOLVE, LOGICSCALAR), 
	InitKeyEntry("decimate", 3, ICONVOLVE, POSITIVEINT),
	InitKeyEntry("radians", 6, IPOLAR | IRECT | IUNWIND, LOGICSCALAR),
	InitKeyEntry("cycles", 6, IPOLAR | IRECT | IUNWIND, LOGICSCALAR),
	InitKeyEntry("degrees", 6, IPOLAR | IRECT | IUNWIND, LOGICSCALAR)
};

#define Unwind    KeyLogValue(TserKeys,KUNWIND)
#define Criterion KeyRealValue(TserKeys,KCRITERION)
#define Reverse   KeyLogValue(TserKeys,KREVERSE)
#define Decimate  KeyIntValue(TserKeys,KDECIMATE)
#define Radians   KeyLogValue(TserKeys,KRADIANS)
#define Cycles    KeyLogValue(TserKeys,KCYCLES)
#define Degrees   KeyLogValue(TserKeys,KDEGREES)

static void doCharOps(Symbolhandle *args, Symbolhandle result, long iop)
{
	char              *startIn, *endIn = STRINGPTR(args[0]);
	char              *placeOut = STRINGPTR(result);
	long               j;
	long               nrows = DIMVAL(result, 1);
	long               ncols = (NDIMS(result) == 1) ? 1 : DIMVAL(result, 2);
	long               krot;
	WHERE("doCharOps");

	if (iop == IROTATE)
	{
		krot = (long) DATAVALUE(args[1],0);
		if (krot != 0)
		{
			krot = krot % nrows;
		}
		if (krot < 0)
		{
			krot += nrows;
		}
	} /*if (iop == IROTATE)*/

	for (j = 0; j < ncols; j++)
	{
		startIn = endIn;
		endIn = skipStrings(startIn, nrows);
		if (iop == IROTATE)
		{
			if (krot > 0)
			{
				placeOut = copyStrings(skipStrings(startIn, nrows - krot),
									   placeOut, krot);
			} /*if (krot > 0)*/			
			placeOut = copyStrings(startIn, placeOut, nrows - krot);
		} /*if (iop == IROTATE)*/
		else
		{
			char      *placeIn = endIn;
			long       i;

			for (i = nrows; i > 0; i--)
			{
				if (i == 1)
				{
					placeIn = startIn;
				}
				else
				{
					for (placeIn -= 2; *placeIn; placeIn--)
					{
						;
					}
					placeIn++;
				}
				placeOut = copyStrings(placeIn, placeOut, 1);
			} /*for (i = nrows; i > 0; i++)*/
		} /*if (iop == IROTATE){}else{}*/
	} /*for (j = 0; j < ncols; j++)*/
} /*doCharOps()*/
/*
   960904 fixed bug in cmplx() that did not allow cmplx(re) as equivalent to
   cmplx(re,0*re)

   980730 added new argument to reuseArg() so that notes will not be kept.
   981213 dropped unused 5th argument (ncolsOut) to qonerg()
   010515 rotate() and reverse() now accept CHARACTER argument
   010516 Use atLeastNArgs constant on badNargs() call
   030305 Made changes to install hdivh(), hdivhj(), cdivc(), cdivcj()
*/

Symbolhandle tserops(Symbolhandle list)
{
	Symbolhandle      arg[2], result = SymbolError, symhKey;
	opEntry          *op;
	long              nargs = NARGS(list);
	long              margs = 1, nonKeyArgs, nkeys, startKeys;
	long              mkeys = NKeys(TserKeys);
	keywordListPtr    keys = TserKeys;
	int               logicOrCharOK, missingOK, integerArg2;
	int               arg1IsChar;
	long              iop = 0, i;
	long              dims1[2], dims2[2];
	long              ncoefs;
	long              nrowsIn1, nrowsIn2, ncolsIn1, ncolsIn2;
	long              nrowsOut, ncolsOut;
	long              krot, keyStatus;
	int               xprd, xdiv, rect, polar;
	double            value = 0.0;
	double           *presult, *parg0, *parg1;
	double            angles2cycles = 8.0*MV_PI_4;
	double            angles2degrees = MV_PI_4/45.0;
	double            angles2radians = 1.0;
	double            units = -1.0;
	long              reuseArg1, reuseArg2;
	char             *programProblem =
	  "ERROR:****Programming problem %d for %s()";

	WHERE("tserops");
	
	*OUTSTR = '\0';

	/* set defaults */
	Criterion = CRITERION;
	Decimate = 1;
	Reverse = 0;
	Unwind = 1;

	unsetLogical(Radians);
	unsetLogical(Cycles);
	unsetLogical(Degrees);
	
	for (op = Ops; op->iop != 0 && strcmp(op->name, FUNCNAME) != 0; op++)
	{
		;
	}
	
	iop = op->iop;

	if (iop == 0)
	{ /* should not occur unless programming error */

		sprintf(OUTSTR, "ERROR: (internal) unrecognized command %s()",
				FUNCNAME);
		goto errorExit;
	}

	xprd = (iop & IPRD) != 0;
	xdiv = (iop & IDIV) != 0;
	polar = (iop & (IPOLAR | IUNWIND)) != 0;
	rect = (iop & IRECT) != 0;

/* 
   logicOrCharOK != 0 means 1st argument can be LOGICAL or CHARACTER
   missingOK != 0 means 1st argument can contain MISSING values
   integerArg2 != 0 means argument 2 must be a scalar integer
*/
	logicOrCharOK = (iop & (IROTATE | IREVERSE)) != 0;
	missingOK = logicOrCharOK || (iop & IPADTO) != 0;
	integerArg2 = (iop & (IROTATE | IPADTO)) != 0;

	margs = op->nargs;
	nonKeyArgs = labs(margs);
	startKeys = nonKeyArgs;

/*
  nonKeyArgs is expected number of nonKeyword arguments
  nkeys is the maximum number of keyword arguments
  margs is the negative of the maximum number of arguments.
*/
	if (margs > 10)
	{
		nonKeyArgs = margs % 10;
		nkeys = margs / 10;
		margs = -(nonKeyArgs + nkeys);
	}
	else
	{
		nonKeyArgs = labs(margs);
		nkeys = 0;
	}

	startKeys = nonKeyArgs;

	if (nargs > labs(margs) || margs > 0 && nargs != margs)
	{
		badNargs(FUNCNAME,margs);
		goto errorExit;
	} /*if (nargs > labs(margs) || margs > 0 && nargs != margs)*/
	
	if (!xprd && !xdiv && (iop & ICMPLX) == 0 && nargs < nonKeyArgs)
	{
		badNargs(FUNCNAME, -nonKeyArgs + atLeastNArgs);
		goto errorExit;
	} /*if (!xprd && !xdiv && (iop & ICMPLX) == 0 && nargs < nonKeyArgs)*/
	
	/* Check non keyword arguments */
	arg[1] = EmptyArgument;
	for (i = 0;i < nonKeyArgs && i < nargs; i++)
	{
		arg[i] = COMPVALUE(list, i);
		if (!argOK(arg[i], 0, i+1))
		{
			goto errorExit;
		}
		if (isKeyword(arg[i]))
		{
			sprintf(OUTSTR,
					"ERROR: argument %ld to %s() is keyword phrase",
					i + 1, FUNCNAME);
			goto errorExit;
		} /*if (isKeyword(arg[i]))*/

		if (i == 1 && integerArg2)
		{
			if (!isInteger(arg[i], ANYVALUE) ||
				(iop & IPADTO) != 0 && DATAVALUE(arg[i], 0) < 1)
			{
				char       msg[40];
				
				sprintf(msg, "argument 2 to %s()", FUNCNAME);
				if ((iop & IPADTO) != 0)
				{
					notPositiveInteger(msg);
				}
				else
				{
					notInteger(msg);
				}
				goto errorExit;
			}
		} /*if (i == 1 && integerArg2)*/
		else
		{
			long        type = TYPE(arg[i]);

			if (i == 0)
			{
				arg1IsChar = (type == CHAR);
			}
			if (type != REAL &&
				(i > 0 || !logicOrCharOK || type != LOGIC && !arg1IsChar))
			{
				badType(FUNCNAME, -type, i+1);
				goto errorExit;
			}
		
			if ((i > 0 || !missingOK) && anyMissing(arg[i]))
			{
				sprintf(OUTSTR,
						"ERROR: argument %ld to %s() has MISSING values",
						i + 1, FUNCNAME);
			}
			if (*OUTSTR)
			{
				goto errorExit;
			}
		} /*if (i == 1 && integerArg2){}else{}*/
	} /*for (i = 0;i < nonKeyArgs && i < nargs; i++)*/

/*	All operations except convolve() may have matrix for arg 1 */
	if ((iop & ICONVOLVE) != 0 && !isVector(arg[0]))
	{
		sprintf(OUTSTR,"ERROR: first argument to %s() is not vector",
				FUNCNAME);
	} /*if ((iop & ICONVOLVE) != 0 && !isVector(arg[0]))*/
	else if (!isMatrix(arg[0],dims1))
	{
		sprintf(OUTSTR,
				"ERROR: first argument to %s() is not a vector or matrix",
				FUNCNAME);
	}
	if (*OUTSTR)
	{
		goto errorExit;
	} /*if (*OUTSTR)*/

/*
  hpolar(hx,F) is equivalent to hpolar(hx,unwind:F)
  cpolar(hx,F) is equivalent to cpolar(hx,unwind:F)
  convolve(a,x,T) is equivalent to convolve(a,x,reverse:T)
*/
	if (nargs > nonKeyArgs && !isKeyword(symhKey = COMPVALUE(list, i)))
	{
		/* hpolar(), cpolar() and convolve() allow you to not use keyword */
		if ((iop & (IPOLAR | ICONVOLVE)) == 0)
		{
			sprintf(OUTSTR,
					"ERROR: too many non-keyword phrase arguments to %s()",
					FUNCNAME);
			goto errorExit;
		}
		if (!isTorF(symhKey))
		{
			sprintf(OUTSTR,
					"ERROR: argument %ld to %s() not T or F", i + 1, FUNCNAME);
			goto errorExit;
		}
		value = DATAVALUE(symhKey,0);
		if ((iop & IPOLAR) != 0)
		{
			Unwind = (value != 0.0);
		}
		else
		{
			Reverse = (value != 0.0);
		}
		startKeys++;
	} /*if (nargs > i && !isKeyword(symhKey = COMPVALUE(list, i)))*/

	if ((xprd || xdiv) && nargs == 1)
	{
		/* 
		   [ch]prd[ch]j?(x) equivalent to [ch]prd[ch]j?(x,x)
		   [ch]div[ch]j?(x) equivalent to [ch]div[ch]j?(x,x)
		*/
		arg[1] = arg[0];
		dims2[0] = dims1[0];
		dims2[1] = dims1[1];
	} /*if ((xprd || xdiv) && nargs == 1)*/
	
	if (nonKeyArgs == 2 && nargs > 1 && !isKeyword(arg[1]))
	{
		if (integerArg2)
		{ /* these require scalar arg 2; already checked */
			krot = (long) DATAVALUE(arg[1], 0);
		}
		else if (!isMatrix(arg[1], dims2))
		{
			sprintf(OUTSTR,
					"ERROR: second argument to %s() is not vector or matrix",
					FUNCNAME);
			goto errorExit;
		}
	} /*if (nonKeyArgs == 2)*/
	
	/* Check keywords*/

	/* Pick off keyword values */
	keyStatus = getAllKeyValues(list, startKeys, iop, keys, mkeys);

	if (keyStatus < 0)
	{
		goto errorExit;
	}
	nargs -= keyStatus;
	
	if (Criterion < 0.5 || Criterion > 1.0)
	{
		sprintf(OUTSTR,
				"ERROR: value for %s() must be between 0.5 and 1.0",
				KeyName(keys,KCRITERION));
		goto errorExit;
	}
	
	if (logicalIsSet(Radians) && Radians)
	{
		units = angles2radians;
	}
	if (logicalIsSet(Degrees) && Degrees)
	{
		units = (units < 0.0) ? angles2degrees : 1000.0;
	}
	if (logicalIsSet(Cycles) && Cycles)
	{
		units = (units < 0.0) ? angles2cycles: 1000.0;
	}

	if (units > 900.0)
	{
		sprintf(OUTSTR, 
				"ERROR: more than 1 of 'degrees:T', 'cycles:T' and 'radians:T' for %s()",
				FUNCNAME);
		goto errorExit;
	}

	if (units < 0.0)
	{
		units = ANGLES2RADIANS;
	}
	
	/* Might it be possible to reuse scratch arguments; further checks later*/
	reuseArg1 = !arg1IsChar && !integerArg2 && iop != ICONVOLVE &&
		isscratch(NAME(arg[0]));
	reuseArg2 = (xprd || xdiv) && nargs > 1 && isscratch(NAME(arg[1]));

	*OUTSTR = '\0';

/* Determine size of nrowsOut by ncolsOut output matrix */

	nrowsOut = nrowsIn1 = dims1[0];
	ncolsIn1 = dims1[1];

	if (nonKeyArgs == 1 || polar)
	{/*
	   hconj(), cconj(), hreal(), creal(), himag(), cimag(), htoc(), ctoh(),
	   hrect(), crect(), hpolar(), cpolar(), unwind()
	 */
		if (iop & (ICCONJ | ICRECT | ICPOLAR))
		{
			ncolsOut = ncolsIn1 + (ncolsIn1 & 1);
		}
		else if (iop & (ICREAL | ICIMAG | ICTOH))
		{
			ncolsOut = (ncolsIn1 + 1)/2;
		}
		else if (iop & IHTOC)
		{
			ncolsOut = 2*ncolsIn1;
		}
		else
		{ /* hconj, hreal, himag, hrect, unwind, reverse */
			ncolsOut = ncolsIn1;
		}
	} /*if (margs == 1 || polar*/
	else if (xprd || xdiv)
	{
		/*
		  hprdh(), hprdhj(), cprdc(), cprdcj(),
		  hdivh(), hdivhj(), cdivc(), cdivcj()
		*/
		nrowsIn2 = dims2[0];
		ncolsIn2 = dims2[1];
		if (nrowsIn1 != nrowsIn2 &&
			((iop & (IHPRD | IHDIV)) || nrowsIn1 > 1 && nrowsIn2 > 1))
		{
			sprintf(OUTSTR,
					"ERROR: arguments to %s() have different numbers of rows",
					FUNCNAME);
		}
		else
		{
			if (iop & (ICPRD | ICDIV))
			{
				if (nrowsIn1 == 1)
				{
					nrowsOut = nrowsIn2;
				}

				if (ncolsIn1 > 2 && ncolsIn2 > 2 && labs(ncolsIn1-ncolsIn2) > 1)
				{
					sprintf(OUTSTR,
							"ERROR: number columns of arguments to %s() do not match",
							FUNCNAME);
				}
				else
				{
					ncolsOut = (ncolsIn1 > ncolsIn2) ? ncolsIn1 : ncolsIn2;
					ncolsOut += ncolsOut & 1; /* increase to even */
				}
			} /*if (iop & (ICPRD | ICDIV))*/
			else
			{/* hprdh(), hprdhj(), hdivh(), hdivhj() */
				if (ncolsIn1 > 1 && ncolsIn2 > 1 && ncolsIn1 != ncolsIn2)
				{
					sprintf(OUTSTR,
							"ERROR: arguments to %s() have different numbers of columns",
							FUNCNAME);
				}
				else
				{
					ncolsOut = (ncolsIn1 > ncolsIn2) ? ncolsIn1 : ncolsIn2;
				}
			}  /*if (iop & (ICPRDC | ICPRDCJ | ICDIVC | ICDIVCJ )){}else{}*/
		} /*if (dims2[0] != nrowsIn1)*/
	}
	/* margs == 2: cmplx(), convolve(), rotate(), padto() */
	else if (iop & ICMPLX)
	{
		if (nargs > 1 && (dims2[0] != nrowsIn1 || dims2[1] != ncolsIn1))
		{
			sprintf(OUTSTR,
					"ERROR: arguments to %s() not the same size and shape",
					FUNCNAME);
		}
		else
		{
			ncolsOut = 2*ncolsIn1;
		}
	}
	else if (iop & ICONVOLVE)
	{
		ncoefs = nrowsIn1;
		nrowsIn1 = dims2[0];
		nrowsOut = (nrowsIn1 - 1)/Decimate + 1;
		ncolsOut = ncolsIn1 = dims2[1];
	}
	else if (iop & (IROTATE | IPADTO))
	{ /* padto(), rotate() */
		ncolsOut = ncolsIn1;
		nrowsOut = (iop == IROTATE) ? nrowsIn1 : krot;
	}
	else
	{ /* should never happen */
		sprintf(OUTSTR, programProblem, 1, FUNCNAME);
	}
	if (*OUTSTR)
	{
		goto errorExit;
	}

	if (reuseArg1 && (nrowsOut != nrowsIn1 || ncolsOut != ncolsIn1))
	{
		reuseArg1 = 0;
	}
	if (reuseArg2 && (nrowsOut != nrowsIn2 || ncolsOut != ncolsIn2))
	{
		reuseArg2 = 0;
	}
	if (reuseArg1 && reuseArg2)
	{
		reuseArg2 = 0;
	}
	
	if (reuseArg1)
	{
		result = reuseArg(list, 0, 0, 0);
	}
	else if (reuseArg2)
	{
		result = reuseArg(list, 1, 0, 0);
	}
	else
	{
		if (isTooBig(nrowsOut, ncolsOut, sizeof(double)))
		{
			resultTooBig(FUNCNAME);
			goto errorExit;
		}
		
		if (!arg1IsChar)
		{
			result = RInstall(SCRATCH,nrowsOut*ncolsOut);
		}
		else
		{
			result = CInstall(SCRATCH, myhandlelength(STRING(arg[0])));
		}
		if (result == SymbolError)
		{
			goto errorExit;
		}
	}
	
	setNCLASS(result, -1);

	if (ncolsOut > 1)
	{
		setNDIMS(result,2);
		setDIM(result,2,ncolsOut);
	}
	setDIM(result,1,nrowsOut);

	if (logicOrCharOK)
	{
		setTYPE(result, TYPE(arg[0]));
	}

	if (arg1IsChar)
	{
		doCharOps(arg, result, iop);
	} /*if (arg1IsChar)*/
	else
	{
		presult = DATAPTR(result);
		parg0 = DATAPTR(arg[0]);
		if (arg[1] != EmptyArgument)
		{
			parg1 = DATAPTR(arg[1]);
		}
		
		if (margs == 1 && !rect)
		{
			/* 981213 eliminated unused 5th argument (ncolsOut) to qonerg */
			qonerg(parg0, presult, dims1, iop);
		}
		else if (xprd || xdiv || iop == ICMPLX)
		{
			qtworg(parg0, (iop == ICMPLX && nargs == 1) ? (double *) 0 : parg1,
				   presult, dims1, dims2, iop);
		}
		else if (polar || rect)
		{
			qrect(parg0, presult, nrowsIn1, ncolsIn1, Unwind, Criterion, 
				  units, iop);
		}
		else if (iop == IPADTO)
		{
			qpadto(parg0, presult, nrowsIn1, ncolsIn1, nrowsOut);
		}
		else if (iop == IROTATE)
		{
			qrota(parg0, presult, nrowsIn1, ncolsIn1, krot);
		}
		else if (iop == ICONVOLVE)
		{
			qconv(parg0, parg1, presult, 
				  ncoefs, nrowsIn1, ncolsIn1, Decimate, Reverse);
		}
		else	
		{ /* should not happen */
			sprintf(OUTSTR, programProblem, 2, FUNCNAME);
			goto errorExit;
		}
	} /*if (arg1IsChar){}else{}*/

	if (interrupted(DEFAULTTICKS) != INTNOTSET)
	{
		goto errorExit;
	}
	
	return (result);

  errorExit:
	putErrorOUTSTR();
	Removesymbol(result);
	return(0);
	
} /*tserops()*/
