/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.12 or later
*(C)*
*(C)* Copyright (c) 2002 Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/


#ifdef SEGMENTED
#if defined(MPW) || defined(MW_CW)
#pragma segment Ttests
#endif /*MPW||MW_CW*/
#endif /*SEGMENTED*/


#include "globals.h"

/*
   020806 modified code directly to use Qstu() with new argument 'upper' 
          instead of passing going through tquantile()
*/

#ifndef NOPROTOTYPES
static long tmeanss(double */*x*/, long /*n*/, double */*mean*/, double */*ss*/);
#else /*NOPROTOTYPES*/
static long tmeanss(/*double **x, long n, double *mean, double *ss*/);
#endif /*NOPROTOTYPES*/

static long tmeanss(double * x, long n, double * mean, double * ss)
{
	double          d, xi;
	long            i, nNonmissing = 0;

	*mean = *ss = 0.0;

	for (i = 0; i < n; i++)
	{
		xi = x[i];
		if (!isMissing(xi))
		{
			*mean += xi;
			nNonmissing++;
		}
	} /*for (i = 0; i < n; i++)*/

#ifdef HASINFINITY
	if (isInfinite(*mean))
	{
		return (-1);
	}
#endif /*HASINFINITY*/

	if (nNonmissing > 1)
	{
		*mean /= nNonmissing;
		for (i = 0; i < n; i++)
		{
			xi = x[i];
			if (!isMissing(xi))
			{
				d = xi - *mean;
				*ss += d*d;
			} /*if (!isMissing(xi))*/ 
		} /*for (i = 0; i < n; i++)*/
#ifdef HASINFINITY
		if (isInfinite(*ss))
		{
			return (-2);
		}
#endif /*HASINFINITY*/
	} /*if (nNonmissing > 1)*/
	
	return (nNonmissing);
} /*tmeanss()*/

enum   ttestCodes
{
	ITVAL,
	ITINT,
	IT2VAL,
	IT2INT
};

#define MINCOVERAGE .00000001
#define MAXCOVERAGE .99999999
/*
  990725 tval() and tint() now work with first argument that is a matrix
         Deleted obsolete code for tint() and t2int()
*/
Symbolhandle    tval(Symbolhandle list)
{
	/* one sample t test of zero mean and confidence interval for mean */

	Symbolhandle    result = (Symbolhandle) 0;
	Symbolhandle    symhX, arg2;
	double          level;
	int             iop, margs, nargs = NARGS(list);
	int             returnDF = 0;
	char           *keyword = (char *) 0, *compNames[2];
	long            j, n, ncols;
	long            nNonmissing;
	long            dims[2];
#ifdef HASINFINITY
	int             foundOverflow = 0;
#endif /*HASINFINITY*/
	int             foundTooShort = 0, foundZeroSD = 0, foundMissing = 0;
	char           *resultMissing = "; result set MISSING";
	WHERE("tint");
	
	*OUTSTR = '\0';
	compNames[0] = "t";
	compNames[1] = "df";
	
	iop = (strcmp(FUNCNAME,"tval") == 0) ? ITVAL : ITINT;
	margs = (iop == ITVAL) ? -2 : 2;

	if (margs > 0 && nargs != margs || margs < 0 && nargs > abs(margs))
	{
		badNargs(FUNCNAME,margs);
		goto errorExit;
	}
	
	symhX = COMPVALUE(list, 0);

	if (!isMatrix(symhX, dims) || (n = dims[0]) < 2 ||
		TYPE(symhX) != REAL)
	{
		sprintf(OUTSTR,
				"ERROR: %s() data not a REAL vector or matrix with at least 2 rows",
				FUNCNAME);
		goto errorExit;
	}
	ncols = dims[1];

	if (nargs > 1)
	{
		arg2 = COMPVALUE(list, 1);
	
		if (iop == ITINT)
		{
			if (isKeyword(arg2))
			{
				sprintf(OUTSTR,
						"ERROR: argument 2 (conf. level)  to %s() must not be keyword phrase",
						FUNCNAME);
				goto errorExit;
			}
			if (!isNumber(arg2, POSITIVEVALUE) ||
				(level = DATAVALUE(arg2, 0)) > MAXCOVERAGE ||
				level < MINCOVERAGE)
			{
				sprintf(OUTSTR,
						"ERROR: confidence level for %s() must be a number between 0 and 1",
						FUNCNAME);
				goto errorExit;
			}
		} /*if (iop == ITINT)*/
		else
		{
			if ((keyword = isKeyword(arg2)) && strcmp(keyword,"df") != 0)
			{
				badKeyword(FUNCNAME,keyword);
				goto errorExit;
			}					
			if (!isTorF(arg2))
			{
				char        msg[30];
					
				if (keyword)
				{
					strcpy(msg, keyword);
				}
				else
				{
					sprintf(msg,"degrees of freedom for %s()", FUNCNAME);
				}
				notTorF(msg);
				goto errorExit;
			} /*if (!isTorF(arg2))*/

			returnDF = (DATAVALUE(arg2, 0) != 0.0);
			keyword = "df";
		} /*if (iop == ITINT){}else{}*/
	} /*if (nargs > 1)*/
	
	
	if (returnDF)
	{
		result = RSInstall(SCRATCH, 2, compNames, ncols);
	}
	else
	{
		result = RInstall(SCRATCH, ncols*((iop == ITVAL) ? 1 : 2));
	}
	
	if (result == (Symbolhandle) 0)
	{
		goto errorExit;
	}
	if (ncols > 1 && iop == ITINT)
	{
		setNDIMS(result, 2);
		setDIM(result, 1, 2);
		setDIM(result, 2, ncols);
	}
	
	for (j = 0; j < ncols; j++)
	{
		double          mean, ss, se, t, df, lower, upper;

		nNonmissing = tmeanss(DATAPTR(symhX) + j*n, n, &mean, &ss);

#ifdef HASINFINITY
		if (nNonmissing < 0)
		{
			foundOverflow = (foundOverflow) ? foundOverflow : nNonmissing;
			if (iop == ITINT)
			{
				setMissing(lower);
				setMissing(upper);
			}
			else
			{
				setMissing(t);
				setMissing(df);
			}
		}
		else
#endif /*HASINFINITY*/
		{
			if (nNonmissing < 2)
			{
				foundTooShort = (foundTooShort) ?
				  foundTooShort : nNonmissing + 1;
			}
			else if (iop == ITVAL && ss == 0.0)
			{
				foundZeroSD = 1;
			}
			if (nNonmissing < n)
			{
				foundMissing = 1;
			} /*if (nNonmissing < n)*/
	
			if (nNonmissing > 0)
			{
				df = (double) (nNonmissing - 1);
			}
			else
			{
				setMissing(df);
			}
			
			if (nNonmissing <= 1)
			{
				if (iop == ITVAL)
				{
					setMissing(t);
				}
				else
				{
					setMissing(lower);
					setMissing(upper);
				}
			} /*if (nNonmissing <= 1)*/
			else
			{
				se = sqrt((ss / df) / (double) nNonmissing);
				if (iop == ITVAL)
				{
					if (ss > 0.0)
					{
						t = mean / se;
					}
					else
					{
						setMissing(t);
					}
				} /*if (iop == ITVAL)*/
				else
				{
					t = Qstu(0.5*(1.0 - level), df, 1);
					lower = mean - t*se;
					upper = mean + t*se;
				}
			} /*if (nNonmissing <= 1){}else{}*/
		}
		
		if (iop == ITVAL)
		{
			if (!returnDF)
			{
				DATAVALUE(result, j) = t;
			}
			else
			{
				DATAVALUE(COMPVALUE(result, 0), j) = t;
				DATAVALUE(COMPVALUE(result, 1), j) = df;
			}
		} /*if (iop == ITVAL)*/
		else
		{
			DATAVALUE(result, 2*j) = lower;
			DATAVALUE(result, 2*j + 1) = upper;
		}
	} /*for (j = 0; j < ncols; j++)*/
	
#ifdef HASINFINITY
	if (foundOverflow)
	{
		sprintf(OUTSTR,
				"WARNING: too large %s computed by %s(); result set MISSING", 
				(foundOverflow == -1) ? "mean" : "SD", FUNCNAME);
		putErrorOUTSTR();
	} /*if (foundOverflow)*/
#endif /*HASINFINITY*/

	if (foundTooShort)
	{
		if (ncols == 1)
		{
			sprintf(OUTSTR,
					"WARNING: %s non-MISSING value%s in data for %s()%s",
					(foundTooShort == 1) ? "no" : "only 1" ,
					(nNonmissing == 0) ? "s" : NullString,
					FUNCNAME, resultMissing);
		}
		else
		{
			sprintf(OUTSTR,
					"WARNING: < 2 non-MISSING values in data column for %s()%s",
					FUNCNAME, resultMissing);
		}
		putErrorOUTSTR();
		foundMissing = 0;
	} /*if (foundTooShort)*/

	if (foundZeroSD)
	{
		
		sprintf(OUTSTR,
				"WARNING: estimated SD of data%s for %s() is 0%s",
				(ncols == 1) ? NullString : " column",
				FUNCNAME, resultMissing);
		putErrorOUTSTR();
	} /*if (foundZeroSD)*/	

	if (foundMissing)
	{
		sprintf(OUTSTR,
				"WARNING: Missing values found in data%s for %s()",
				(ncols == 1) ? NullString : " column", FUNCNAME);
		putErrorOUTSTR();
	} /*if (foundMissing)*/
	
	return (result);

  errorExit:
	putErrorOUTSTR();

	Removesymbol(result);
	return (0);
} /*tval()*/

/* 
  t2val(x1,x2)  or t2val(x1,x2,df:T) or t2val(x1,x2,pooled:F)
  t2int(x1,x2,level) or t2int(x1,x2,level,pooled:F)

  Note: t2val(x1,x2,F) is equivalent to t2val(x1,x2,pooled:F) and
  t2int(x1,x2,level,F) is equivalent to t2int(x1,x2,level,pooled:F)

  960920 Fixed bug that caused crash if keywords were used
  980406 Fixed bug that prevented keyword 'df' from working right
  990727 Modified so that x1 and x2 can be matrices with the same number
         of columns
  010516 Use atLeastNArgs constant on badNargs() call
*/
enum keyIndex
{
	KPOOLED,
	KDF
};

Symbolhandle    t2val(Symbolhandle list)
{
	/* two sample t test of zero difference and C.I. for difference*/

	Symbolhandle    arg[2];
	Symbolhandle    result = (Symbolhandle) 0;
	double          level;
	long            i, j;
	long            dims[2][2], nvals[2], ncols;
	long            nargs = NARGS(list);
	long            mnNargs, mxNargs;
	int             returnDF = 0, pooled = 1, logvalue = 0;
	int             iop;
	int             foundEmpty = 0, foundMissing = 0, foundTooShort = 0;
	int             foundZeroSD = 0;
#ifdef HASINFINITY
	int             foundOverflow = 0;
#endif /*HASINFINITY*/
	char           *compNames[2], *keyword = (char *) 0;
	char           *keyNames[2];
	char           *resultMissing = "; result set MISSING";
	WHERE("t2val");
	
	*OUTSTR = '\0';
	compNames[0] = "t";
	compNames[1] = "df";
	keyNames[KPOOLED] = "pooled";
	keyNames[KDF] = compNames[1];
	
	iop = (strcmp(FUNCNAME,"t2val") == 0) ? IT2VAL : IT2INT;

	mnNargs = (iop == IT2VAL) ? 2 : 3;
	mxNargs = mnNargs + 1;

	if (nargs > mxNargs)
	{
		badNargs(FUNCNAME,-mxNargs);
		goto errorExit;
	}
	if (nargs < mnNargs)
	{
		badNargs(FUNCNAME, -mnNargs + atLeastNArgs);
		goto errorExit;
	}
	
	for (i = 0; i < 2;i++)
	{
		arg[i] = COMPVALUE(list, i);
		if (!isMatrix(arg[i], dims[i]) || TYPE(arg[i]) != REAL)
		{
			sprintf(OUTSTR,
					"ERROR: argument %ld to %s() not a REAL matrix",
					i + i, FUNCNAME);
			goto errorExit;
		}
		nvals[i] = dims[i][0];
	} /*for (i = 0; i < 2;i++)*/

	if (dims[0][1] != dims[1][1])
	{
		sprintf(OUTSTR,
				"ERROR: arguments 1 and 2 to %s() don't have the same number of columns",
				FUNCNAME);
		goto errorExit;
	}
	ncols = dims[0][1];

	if (iop == IT2INT)
	{
		Symbolhandle          arg3 = COMPVALUE(list, 2);
		
		if (isKeyword(arg3))
		{
			sprintf(OUTSTR,
					"ERROR: argument 3 (conf. level) to %s() must not be keyword phrase",
					FUNCNAME);
			goto errorExit;
		}
		if (!isNumber(arg3, POSITIVEVALUE) ||
			(level = DATAVALUE(arg3, 0)) < MINCOVERAGE ||
			level > MAXCOVERAGE)
		{
			sprintf(OUTSTR,
					"ERROR: confidence level for %s() must be a number between 0 and 1",
					FUNCNAME);
			goto errorExit;
		}
		i = 3;
	} /*if (iop == IT2INT)*/
	else
	{
		i = 2;
	}
	if (i < nargs)
	{
		Symbolhandle         symhKey = COMPVALUE(list, i);

		if ((keyword = isKeyword(symhKey)))
		{
			if (strncmp(keyword, keyNames[KPOOLED], 4) != 0 &&
			   (iop == IT2INT || strcmp(keyword, keyNames[KDF]) != 0))
			{
				badKeyword(FUNCNAME, keyword);
				goto errorExit;
			}
		} /*if ((keyword = isKeyword(symhKey)))*/		
		else
		{
			keyword = keyNames[KPOOLED];
		}
		if (!isTorF(symhKey))
		{
			sprintf(OUTSTR,
					"ERROR: argument %ld for %s() must be %s", i+1, FUNCNAME,
					(iop == IT2VAL) ? "df:T or pooled:F" : "pooled:F");
			goto errorExit;
		} /*if (!isTorF(symhKey))*/

		logvalue = (DATAVALUE(symhKey,0) != 0.0);
	} /*if (i < nargs)*/

	if (keyword)
	{
		if (strncmp(keyword, keyNames[KPOOLED], 4) == 0)
		{
			pooled = logvalue;
			returnDF = (iop == IT2VAL && !pooled);
		}
		else
		{ /* must be "df" */
			returnDF = logvalue;
			pooled = 1;
		}
	} /*if (keyword)*/

	if (returnDF)
	{
		result = RSInstall(SCRATCH, 2, compNames, ncols);
	}
	else
	{
		result = RInstall(SCRATCH, ((iop == IT2VAL) ? 1 : 2) * ncols);
	}

	if (result == (Symbolhandle) 0)
	{
		goto errorExit;
	}

	if (ncols > 1 && iop == IT2INT)
	{
		setNDIMS(result, 2);
		setDIM(result, 1, 2);
		setDIM(result, 2, ncols);
	}
	
	for (j = 0; j < ncols; j++)
	{
		long           nNonmissing[2], nn;
		double         se, mean[2], ss[2], var, t, df;
		double         lower, upper;
		double         fn1, fn2;

		nn = 0;
		t = var = 0.0;
		for (i = 0; i < 2;i++)
		{
			nNonmissing[i] =
			  tmeanss(DATAPTR(arg[i]) + j*nvals[i], nvals[i], mean+i, ss+i);

#ifdef HASINFINITY
			if (nNonmissing[i] < 0)
			{
				foundOverflow = nNonmissing[i];
				setMissing(t);
				break;
			}
#endif /*HASINFINITY*/

			if (nNonmissing[i] < 1)
			{
				foundEmpty |= i+1;
				setMissing(t);
				break;
			} /*if (nNonmissing[i] < 1)*/

			if (nNonmissing[i] < nvals[i])
			{
				foundMissing |= i+1;
			}
			nn += nNonmissing[i];
			var += ss[i]; /* pooled SS */
		} /*for (i = 0; i < 2;i++)*/

		if (!isMissing(t))
		{			
			int        thisIsTooShort = 0;

			if (nn < 3 ||
				!pooled && (nNonmissing[0] < 2 || nNonmissing[1] < 2))
			{
				thisIsTooShort = foundTooShort = 1;
			}
			else if (iop == IT2VAL && var == 0.0)
			{
				foundZeroSD = 1;
			}

			fn1 = (double) nNonmissing[0];
			fn2 = (double) nNonmissing[1];

			if (pooled)
			{
				df = (double) (nn - 2);
				if (df >= 1)
				{
					var /= df;
					se = sqrt(var * (1.0/fn1 + 1.0/fn2));
				}
				else
				{
					var = se = 0.0;
					setMissing(df);
				}
			} /*if (pooled)*/
			else
			{
				if (!thisIsTooShort)
				{
					ss[0] /= fn1*(fn1 - 1.0);
					ss[1] /= fn2*(fn2 - 1.0);
					var = ss[0] + ss[1];
					se = sqrt(var);
				}
				else
				{
					se = var = 0.0;
				}
				
				if (var > 0.0)
				{
					df = var*var/
					  (ss[0]*ss[0]/(fn1 - 1.0) + ss[1]*ss[1]/(fn2 - 1.0));
				}
				else
				{
					setMissing(df);
				}
			} /*if (pooled){}else{}*/
	
			if (iop == IT2VAL)
			{
				if (se > 0)
				{
					t = (mean[0] - mean[1]) / se;
				}
				else
				{
					setMissing(t);
				}
			}
			else
			{
				if (!isMissing(df))
				{
					t = Qstu(0.5*(1.0 - level), df, 1);
					lower = mean[0] - mean[1] - t*se;
					upper = mean[0] - mean[1] + t*se;
				}
				else
				{
					setMissing(lower);
					setMissing(upper);
				}
			}
		} /*if (!isMissing(t))*/
		else
		{
			if (iop == IT2INT)
			{
				setMissing(lower);
				setMissing(upper);
			}
			
			setMissing(df);
		}
		
		if (iop == IT2VAL)
		{
			if (!returnDF)
			{
				DATAVALUE(result, j) = t;
			}
			else
			{
				DATAVALUE(COMPVALUE(result, 0), j) = t;
				DATAVALUE(COMPVALUE(result, 1), j) = df;
			}
		} /*if (iop == IT2VAL)*/
		else
		{
			DATAVALUE(result, 2*j) = lower;
			DATAVALUE(result, 2*j + 1) = upper;
		}
	} /*for (j = 0; j < ncols; j++)*/
	
#ifdef HASINFINITY
	if (foundOverflow)
	{
		sprintf(OUTSTR,
				"WARNING: too large %s computed by %s(); result set MISSING", 
				(foundOverflow == -1) ? "mean" : "SD", FUNCNAME);
		putErrorOUTSTR();
	} /*if (foundOverflow)*/
#endif /*HASINFINITY*/

	if (foundEmpty)
	{
		if (ncols == 1)
		{
			sprintf(OUTSTR,
					"WARNING: no non-MISSING values in argument %d to %s()",
					foundEmpty, FUNCNAME);
		}
		else
		{
			sprintf(OUTSTR,
					"WARNING: no non-MISSING values in column of argument %d to %s()",
					(foundEmpty & 1) ? 1 : 2, FUNCNAME);
		}
		putErrorOUTSTR();
		foundMissing = 0;
	} /*if (foundEmpty)*/
	
	if (foundTooShort)
	{
		sprintf(OUTSTR,
				"WARNING: too few non-MISSING values in data%s for %s()%s",
				(ncols == 1) ? NullString : " column",
				FUNCNAME, resultMissing);
		putErrorOUTSTR();
		foundMissing = 0;
	} /*if (foundTooShort)*/

	if (foundZeroSD)
	{
		
		sprintf(OUTSTR,
				"WARNING: zero estimated SE of difference%s for %s()%s",
				(ncols == 1) ? NullString : " for data column",
				FUNCNAME, resultMissing);
		putErrorOUTSTR();
	} /*if (foundZeroSD)*/	

	if (foundMissing)
	{
		sprintf(OUTSTR,
				"WARNING: Missing values found in data%s for %s()",
				(ncols == 1) ? NullString : " column", FUNCNAME);
		putErrorOUTSTR();
	} /*if (foundMissing)*/
	
	return (result);

  errorExit:
	putErrorOUTSTR();

	Removesymbol(result);
	return (0);
} /*t2val()*/


