/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.11 or later
*(C)*
*(C)* Copyright (c) 2001 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

/*
 * File:	unxIface.h and dosIface.h
 */



/*
  Header unifying interface for Unix and DOS command line versions
  New 010529
  010605 added lastVol to enum volumeCodes

*/
#ifndef UNXIFACEH__
#define UNXIFACEH__
#undef EXTERN
#undef COMMA
#define COMMA ,

#ifdef MAIN__
#define EXTERN
#define INIT(VAL) = VAL
#define INITDIM(N) N
#define INITARRAY(VALS) = { VALS }
#else
#define EXTERN extern
#define INIT(VAL)
#define INITDIM(N)
#define INITARRAY(VALS)
#endif /*MAIN__*/

#ifndef PLATFORMH__
#include "../platform.h"
#endif /* PLATFORMH__ */

/*
	Graph windows
*/
#ifndef NGRAPHS
#define NGRAPHS          1
#endif /*NGRAPHS*/

#ifndef FULLPATHSIZE
#ifdef BCPP
#define FULLPATHSIZE 256
#else /*BCPP*/
#define FULLPATHSIZE 1024
#endif /*BCPP*/
#endif /*FULLPATHSIZE*/

#ifndef PATHSIZE
#define PATHSIZE 256
#endif /*PATHSIZE*/

typedef struct mvDirInfo
{
	char     dirName[FULLPATHSIZE + 1];
} mvDirInfo, *mvDirInfoPtr;

/*
  Access macros for mvDirInfo structure
  Prototypes:
	char *           getMvDirVolume(mvDirInfoPtr); **not defined**
	char *           getMvDirDirectory(mvDirInfoPtr); 
	void             setMvDirVolume(mvDirInfoPtr, char *); **not defined**
	void             setMvDirDirectory(mvDirInfoPtr, char *);
	void             setMvDir(mvDirInfoPtr, mvDirInfoPtr);
*/

#define getMvDirDirectory(PTR)   ((PTR)->dirName)
#define setMvDirDirectory(PTR, DIR) \
	(strncpy((PTR)->dirName, DIR, FULLPATHSIZE),\
	 (PTR)->dirName[FULLPATHSIZE] = '\0')
#define setMvDir(PTR1, PTR2) setMvDirDirectory(PTR1, getMvDirDirectory(PTR2))

typedef struct mvFileInfo
{
	mvDirInfo        dirInfo;
	char             fileName[PATHSIZE];
} mvFileInfo, *mvFileInfoPtr;

/*
  Access macros for mvFileInfo structure
  Prototypes:
	mvDirInfoPtr     getMvFileDirInfo(mvFileInfoPtr);
	char *           getMvFileVolume(mvFileInfoPtr); **not defined**
	char *           getMvFileDirectory(mvFileInfoPtr);
	char *           getMvFileName(mvFileInfoPtr);
	void             setMvFileDirInfo(mvFileInfoPtr, mvDirInfoPtr);
	void             setMvFileVolume(mvFileInfoPtr, char *); **not defined**
	void             setMvFileDirectory(mvFileInfoPtr, char *);
	void             setMvFileName(mvFileInfoPtr, char *);
	void             setMvFile(mvFileInfoPtr, mvFileInfoPtr);
*/
#define getMvFileDirInfo(PTR)       (&((PTR)->dirInfo))
#define getMvFileDirectory(PTR)     ((PTR)->dirInfo.dirName)
#define getMvFileName(PTR)          ((PTR)->fileName)

#define setMvFileDirInfo(PTR1, PTR2) (setMvDir(&(PTR1)->dirInfo, PTR2))	 
#define setMvFileDirectory(PTR, DIR) (setMvDirDirectory(PTR->dirInfo, DIR))
#define setMvFileName(PTR, NAME)   (strncpy((PTR)->fileName, NAME, PATHSIZE),\
		(PTR)->fileName[PATHSIZE] = '\0')
#define setMvFile(PTR1, PTR2)		(setMvDir(&PTR1->dirInfo, &PTR2->dirInfo),\
									 setMvFileName(PTR1, getMvFileName(PTR2))) 

enum volumeCodes
{
	keepVol,   /* goes with save(), asciisave() */
	restoreVol,
	spoolVol,
	batchVol,
	readVol,
	printVol,
#ifndef SCROLLABLEWINDOW
	windowVol,   /* goes with Open and Save Window */
#endif /*SCROLLABLEWINDOW*/
	helpVol,
	plotVol,
#ifdef HASDYNLOADS
	dynloadVol,
#endif /*HASDYNLOADS*/
	homeVol,
	currentVol,  /*Current directory*/
	lastVol,     /*most recently used by mvFindFile()*/
	nVolumeCodes
};

/*
	Following may be intialized as an array of length nVolumeCodes
	in initializeGlobals() in main.c
*/
EXTERN mvDirInfoPtr    MvDirectories INIT((mvDirInfoPtr) 0);

/*
  Access macros for MvDirectories (taken from macIface.h; will need to
  be modified for use with wx
  Prototypes
    void               setMvVolID(int, Integer);
	void               setMvDirID(int, LongInt);
	void               setMvDirInfo(int, char *);
	void               copyMvDirInfo(int, mvDirInfoPointer);
	mvDirInfoPointer   getMvDirInfo(int);
	Integer            getMvVolID(int);
	LongInt            getMvDirID(int);
	int                isMvDirInfoSet(int);
*/

#define getMvDirInfo(I)       (&MvDirectories[I])
#define setMvDirInfo(I, V)    strcpy(getMvDirInfo(I)->dirName, V)
#define copyMvDirInfo(I, PTR) strcpy(getMvDirInfo(I)->dirName, (PTR)->dirName)
#define isMvDirInfoSet(I)     (getMvDirInfo(I)->dirName[0] != '\0')

#undef	INIT
#undef	INITDIM
#undef	INITARRAY
#undef  COMMA
#undef  EXTERN

#ifdef BASE_MICROSOFT
#include "dosProto.h"
#else /*BASE_MICROSOFT*/
#include "unxProto.h"
#endif /*BASE_MICROSOFT*/

#endif /*UNXIFACEH__*/





