/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 5.00 or later
*(C)*
*(C)* Copyright (c) 2004 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

/*
   This file defines a number of macros any of which might otherwise
   defined in a Makefile, being specific to the compilation date and the
   installation configuration of MacAnova and its auxilliary files.  Either
   this file or Makefile can be customized for a particular installation.

    VERSION              Version number (e.g., 4.03)
    MAKE_DATE            Date of compilation or make, to be kept up to date
    PROCESSOR_COMPILER   Name with system type and compiler abbreviation
    TODAY                Combines two preceding for startup message
   
   Particularly on Unix the following paths should be customized
    DATAPATHNAME         Path to default data file
    MACROPATHNAME        Path to default macro file
    HELPPATHNAME         Path to standard help file

   For most installations, these defaults should be fine
    DATAFILENAME         Name of default data file
    MACROFILENAME        Name of default macro file
    HELPNAME             Name of standard help file (*not* HELPFILENAME)

  960314  Bumped version number to 4.00
  960505  Modified definitions of DATAFILENAME, MACROFILENAME, etc.
  960507  Bumped version number to 4.01
  960612  Bumped version number to 4.02
  961226  Bumped version number to 4.03
  970313  Bumped version number to 4.04
  970619  Bumped version number to 4.05
  971112  Bumped version number to 4.06
  980716  Bumped version number to 4.07
  990324  Added define ARIMAMACROFILENAME
  990711  Bumped version number to 4.10
  000224  Bumped version number to 4.11
          added define MULTIVARMACROFILENAME in case multivar.mac
          is released
  000824  Added define GRAPHICSMACROFILENAME
  010616  Added defines for standard help file names; used by initialize
          to create variable HELPFILES
  010619  Bumped version to 4.12
  010703  Added defines ARIMAHELPINDEXNAME, GRAPHICSHELPINDEXNAME, ....
  010813  Defines of PROCESSOR_COMPILER for SC
  020809  Bumped version to 4.13
  041005  Bumped version to 5.00
  050117  Changed macro/help/data names to .hlp.txt, .mac.txt, etc forms
          Bumped version to 5.01
          Moved define of MACANOVAINI here from globkb.h
*/

#ifndef _VERSIONH_

#define _VERSIONH_

/*
   If neither MAKE_DATE or TODAY are defined in Makefile, the following
   date should be changed whenever MacAnova is re-made
*/

#ifndef MAKE_DATE
#define MAKE_DATE    "09/18/05" /* Day of Make */
#endif /*MAKE_DATE*/

#ifndef VERSION
#define VERSION "5.05 "     		/* version of MacAnova */
#endif /*VERSION*/

#ifndef PROCESSOR_COMPILER

#ifdef MW_CW

#ifndef MW_CW_5
#define CWTAG "[CW]"
#else /*MW_CW_5*/
#define CWTAG "[CW5]"
#endif /*MW_CW_5*/



#ifndef powerc
#if (__option(code68881)) || (mc68881) /*#ifdef  __MC68881__ did not work, being defined when it shouldn't be*/
#define NEEDS68881
#define PROCESSOR_COMPILER "Macintosh with Co-Processor " CWTAG
#else /*__option(code68881)*/
#define PROCESSOR_COMPILER "Macintosh without Co-Processor " CWTAG
#endif /*__option(code68881)*/
#else /*powerc*/
#define PROCESSOR_COMPILER "Power Macintosh " CWTAG
#endif /*powerc*/
#endif /*MW_CW*/

#ifdef MPW

#ifdef MPW_MRC
#define PROCESSOR_COMPILER "Power Macintosh " "[MPW_MRC]"
#else /*MPW_MRC*/

#ifdef mc68881
#define NEEDS68881
#ifndef MPW_SC
#define PROCESSOR_COMPILER "Macintosh with Co-Processor [MPWC]"
#else /*MPW_SC*/
#define PROCESSOR_COMPILER "Macintosh with Co-Processor [MPW_SC]"
#endif /*MPW_SC*/
#else /*mc68881*/
#ifndef MPW_SC
#define PROCESSOR_COMPILER "Macintosh without Co-Processor [MPWC]"
#else /*MPW_SC*/
#define PROCESSOR_COMPILER "Macintosh without Co-Processor [MPW_SC]"
#endif /*MPW_SC*/
#endif /*mc68881*/

#endif /*MPW_MRC*/

#endif /*MPW*/


#ifdef BCPP
#define PROCESSOR_COMPILER "DOS [BCPP4]"
#endif /*BCPP*/

#ifdef COMP_DJGPP
#define PROCESSOR_COMPILER "DOS [DJGPP]"
#endif /*COMP_DJGPP*/

#ifdef WXWINMSW
#define PROCESSOR_COMPILER "Win32s [BCPP5.0]"
#endif /*WXWINMSW*/

#ifdef WXWINMOTIF
#define PROCESSOR_COMPILER "Motif [gcc]"
#endif /*WXWINMOTIF*/

#endif /*PROCESSOR_COMPILER*/

#ifndef TODAY /* Caution: This doesn't work with MPW C in Version 3.2.3 MPW */
#ifndef VMS	/* I. Pardowitz, TODAY-generation here not working under VMS */
#define	TODAY "of " MAKE_DATE " (" PROCESSOR_COMPILER ")"
#else /*VMS*/
#define TODAY MAKE_DATE
#endif /*VMS */
#endif /*TODAY*/

/*
   On Unix and perhaps other platforms, DATAPATHNAME should probably be
   defined in Makefile, or an additional entry can be put here
*/

/* 960503 changed value from ":" or "./" or ".\\" */
#ifndef DATAPATHNAME
#define DATAPATHNAME ""
#endif /*DATAPATHNAME*/

#ifndef DATAFILENAME
#if defined(SKEL_CARAPACE) || defined(BASE_UNIX) || defined (OS_DOS32)
#define DATAFILENAME "MacAnova.dat.txt"
#elif defined(MPW) || defined(MW_CW)
#define DATAFILENAME "MacAnova.dat"
#elif defined(BASE_MICROSOFT)
#define DATAFILENAME "MACANOVA.DAT"
#else /*MPW || MW_CW*/
#define DATAFILENAME "macanova.dat"
#endif /*MPW || MW_CW*/
#endif /*DATAFILENAME*/

#ifndef MACROPATHNAME
#define MACROPATHNAME DATAPATHNAME
#endif /*MACROPATHNAME*/

/* define initialization file name */
#ifndef MACANOVAINI
#if defined(SKEL_CARAPACE) || defined(BASE_UNIX) || defined (OS_DOS32)
#define MACANOVAINI "MacAnova.ini.txt"
#elif defined(MSDOS)
#define MACANOVAINI "MACANOVA.INI"
#elif defined(MACINTOSH)
#define MACANOVAINI "MacAnova.ini"
#else
#define MACANOVAINI ".macanova.ini"
#endif
#endif /*MACANOVAINI*/


#ifndef MACROFILENAME
#if defined(SKEL_CARAPACE) || defined(BASE_UNIX) || defined (OS_DOS32)
#define MACROFILENAME "MacAnova.mac.txt"
#elif defined(MPW) || defined(MW_CW)
#define MACROFILENAME "MacAnova.mac"
#elif defined(BASE_MICROSOFT)
#define MACROFILENAME "MACANOVA.MAC"
#else /*MPW || MW_CW*/
#define MACROFILENAME "macanova.mac"
#endif /*MPW || MW_CW*/
#endif /*MACROFILENAME*/

/*
  000224 added macros giving names of Mulvar.mac
  000824 added macros giving names of Graphics.mac, Math.mac
         These can be suppressed by defining NOTSMACRO,
         NODESIGNMACRO, NOARIMAMACRO, NOMULVARMACRO,
         NOGRAPHICSMACRO and/or NOMATHMACRO in the make file, or by defining
         TSMACROFILENAME, DESIGNMACROFILENAME, ARIMAMACROFILENAME,
         GRAPHICSMACROFILENAME and/or MATHMACROFILENAME there
  000904 added macro giving giving name of Regress.mac
*/
#if !defined(NOTSMACRO) && !defined(TSMACROFILENAME)
#if defined(SKEL_CARAPACE) || defined(BASE_UNIX) || defined (OS_DOS32)
#define TSMACROFILENAME "Tser.mac.txt"
#elif defined(MPW) || defined(MW_CW)
#define TSMACROFILENAME "Tser.mac"
#elif defined(BCPP) || defined(COMP_DJGPP)
#define TSMACROFILENAME "TSER.MAC"
#else /*MPW || MW_CW*/
#define TSMACROFILENAME "tser.mac"
#endif /*MPW || MW_CW*/
#endif /*!NOTSMACRO && !TSMACROFILENAME*/

#if !defined(NODESIGNMACRO) && !defined(DESIGNMACROFILENAME)
#if defined(SKEL_CARAPACE) || defined(BASE_UNIX) || defined (OS_DOS32)
#define DESIGNMACROFILENAME "Design.mac.txt"
#elif defined(MPW) || defined(MW_CW)
#define DESIGNMACROFILENAME "Design.mac"
#elif defined(BCPP) || defined(COMP_DJGPP)
#define DESIGNMACROFILENAME "DESIGN.MAC"
#else /*MPW || MW_CW*/
#define DESIGNMACROFILENAME "design.mac"
#endif /*MPW || MW_CW*/
#endif /*!NODESIGNMACRO && !DESIGNMACROFILENAME*/

#if !defined(NOARIMAMACRO) && !defined(ARIMAMACROFILENAME)
#if defined(SKEL_CARAPACE) || defined(BASE_UNIX) || defined (OS_DOS32)
#define ARIMAMACROFILENAME "Arima.mac.txt"
#elif defined(MPW) || defined(MW_CW)
#define ARIMAMACROFILENAME "Arima.mac"
#elif defined(BCPP) || defined(COMP_DJGPP)
#define ARIMAMACROFILENAME "ARIMA.MAC"
#else /*MPW || MW_CW*/
#define ARIMAMACROFILENAME "arima.mac"
#endif /*MPW || MW_CW*/
#endif /*!NOARIMAMACRO && !ARIMAMACROFILENAME*/

#if !defined(NOMULVARMACRO) && !defined(MULVARMACROFILENAME)
#if defined(SKEL_CARAPACE) || defined(BASE_UNIX) || defined (OS_DOS32)
#define MULVARMACROFILENAME "Mulvar.mac.txt"
#elif defined(MPW) || defined(MW_CW)
#define MULVARMACROFILENAME "Mulvar.mac"
#elif defined(BCPP) || defined(COMP_DJGPP)
#define MULVARMACROFILENAME "MULVAR.MAC"
#else /*MPW || MW_CW*/
#define MULVARMACROFILENAME "mulvar.mac"
#endif /*MPW || MW_CW*/
#endif /*!NOMULVARMACRO && !MULVARMACROFILENAME*/

#if !defined(NOGRAPHICSMACRO) && !defined(GRAPHICSMACROFILENAME)
#if defined(SKEL_CARAPACE) || defined(BASE_UNIX) || defined (OS_DOS32)
#define GRAPHICSMACROFILENAME "Graphics.mac.txt"
#elif defined(MPW) || defined(MW_CW)
#define GRAPHICSMACROFILENAME "Graphics.mac"
#elif defined(BCPP) || defined(COMP_DJGPP)
#define GRAPHICSMACROFILENAME "GRAPHICS.MAC"
#else /*MPW || MW_CW*/
#define GRAPHICSMACROFILENAME "graphics.mac"
#endif /*MPW || MW_CW*/
#endif /*!NOGRAPHICSMACRO && !GRAPHICSMACROFILENAME*/

#if !defined(NOMATHMACRO) && !defined(MATHMACROFILENAME)
#if defined(SKEL_CARAPACE) || defined(BASE_UNIX) || defined (OS_DOS32)
#define MATHMACROFILENAME "Math.mac.txt"
#elif defined(MPW) || defined(MW_CW)
#define MATHMACROFILENAME "Math.mac"
#elif defined(BCPP) || defined(COMP_DJGPP)
#define MATHMACROFILENAME "MATH.MAC"
#else /*MPW || MW_CW*/
#define MATHMACROFILENAME "math.mac"
#endif /*MPW || MW_CW*/
#endif /*!NOMATHMACRO && !MATHMACROFILENAME*/

#if !defined(NOREGRESSMACRO) && !defined(REGRESSMACROFILENAME)
#if defined(SKEL_CARAPACE) || defined(BASE_UNIX) || defined (OS_DOS32)
#define REGRESSMACROFILENAME "Regress.mac.txt"
#elif defined(MPW) || defined(MW_CW)
#define REGRESSMACROFILENAME "Regress.mac"
#elif defined(BCPP) || defined(COMP_DJGPP)
#define REGRESSMACROFILENAME "REGRESS.MAC"
#else /*MPW || MW_CW*/
#define REGRESSMACROFILENAME "regress.mac"
#endif /*MPW || MW_CW*/
#endif /*!NOREGRESSMACRO && !REGRESSMACROFILENAME*/

#if defined(SKEL_CARAPACE)
#define GUIMACROFILENAME "Gui.mac.txt"
#endif

/*
   On Unix and perhaps other platforms, HELPPATHNAME should probably be
   defined in Makefile, or an additional entry can be put here
*/

/* changed definition to "" instead of ":", "./" or ".\\" */
#ifndef HELPPATHNAME
#define HELPPATHNAME ""
#endif /*HELPPATHNAME*/

#ifndef HELPNAME
#if defined(SKEL_CARAPACE) || defined(BASE_UNIX) || defined (OS_DOS32)
#define HELPNAME "MacAnova.hlp.txt"
#elif defined(MPW) || defined(MW_CW)
#define HELPNAME "MacAnova.hlp"
#elif defined(BASE_MICROSOFT)
#define HELPNAME "MACANOVA.HLP"
#else /*MPW || MW_CW*/
#define HELPNAME "macanova.hlp"
#endif /*MPW || MW_CW*/
#endif /*HELPNAME*/
#define HELPINDEXNAME "macanova_index"

#if !defined(NOTSHELP) && !defined(TSHELPFILENAME)
#if defined(SKEL_CARAPACE) || defined(BASE_UNIX) || defined (OS_DOS32)
#define TSHELPFILENAME "Tser.hlp.txt"
#elif defined(MPW) || defined(MW_CW)
#define TSHELPFILENAME "Tser.hlp"
#elif defined(BCPP) || defined(COMP_DJGPP)
#define TSHELPFILENAME "TSER.HLP"
#else /*MPW || MW_CW*/
#define TSHELPFILENAME "tser.hlp"
#endif /*MPW || MW_CW*/
#endif /*!NOTSHELP && !TSHELPFILENAME*/
#define TSHELPINDEXNAME "tser_index"

#if !defined(NODESIGNHELP) && !defined(DESIGNHELPFILENAME)
#if defined(SKEL_CARAPACE) || defined(BASE_UNIX) || defined (OS_DOS32)
#define DESIGNHELPFILENAME "Design.hlp.txt"
#elif defined(MPW) || defined(MW_CW)
#define DESIGNHELPFILENAME "Design.hlp"
#elif defined(BCPP) || defined(COMP_DJGPP)
#define DESIGNHELPFILENAME "DESIGN.hlp"
#else /*MPW || MW_CW*/
#define DESIGNHELPFILENAME "design.hlp"
#endif /*MPW || MW_CW*/
#endif /*!NODESIGNHELP && !DESIGNHELPFILENAME*/
#define DESIGNHELPINDEXNAME "design_index"

#if !defined(NOARIMAHELP) && !defined(ARIMAHELPFILENAME)
#if defined(SKEL_CARAPACE) || defined(BASE_UNIX) || defined (OS_DOS32)
#define ARIMAHELPFILENAME "Arima.mac.txt"
#elif defined(MPW) || defined(MW_CW)
#define ARIMAHELPFILENAME "Arima.mac"
#elif defined(BCPP) || defined(COMP_DJGPP)
#define ARIMAHELPFILENAME "ARIMA.MAC"
#else /*MPW || MW_CW*/
#define ARIMAHELPFILENAME "arima.mac"
#endif /*MPW || MW_CW*/
#endif /*!NOARIMAHELP && !ARIMAHELPFILENAME*/
#define ARIMAHELPINDEXNAME "arima_index"

#if !defined(NOMULVARHELP) && !defined(MULVARHELPFILENAME)
#if defined(SKEL_CARAPACE) || defined(BASE_UNIX) || defined (OS_DOS32)
#define MULVARHELPFILENAME "Mulvar.mac.txt"
#elif defined(MPW) || defined(MW_CW)
#define MULVARHELPFILENAME "Mulvar.mac"
#elif defined(BCPP) || defined(COMP_DJGPP)
#define MULVARHELPFILENAME "MULVAR.MAC"
#else /*MPW || MW_CW*/
#define MULVARHELPFILENAME "mulvar.mac"
#endif /*MPW || MW_CW*/
#endif /*!NOMULVARHELP && !MULVARHELPFILENAME*/
#define MULVARHELPINDEXNAME "mulvar_index"

#if !defined(NOGRAPHICSHELP) && !defined(GRAPHICSHELPFILENAME)
#if defined(SKEL_CARAPACE) || defined(BASE_UNIX) || defined (OS_DOS32)
#define GRAPHICSHELPFILENAME "Graphics.mac.txt"
#elif defined(MPW) || defined(MW_CW)
#define GRAPHICSHELPFILENAME "Graphics.mac"
#elif defined(BCPP) || defined(COMP_DJGPP)
#define GRAPHICSHELPFILENAME "GRAPHICS.MAC"
#else /*MPW || MW_CW*/
#define GRAPHICSHELPFILENAME "graphics.mac"
#endif /*MPW || MW_CW*/
#endif /*!NOGRAPHICSHELP && !GRAPHICSHELPFILENAME*/
#define GRAPHICSHELPINDEXNAME "graph_index"

#if !defined(NOMATHHELP) && !defined(MATHHELPFILENAME)
#if defined(SKEL_CARAPACE) || defined(BASE_UNIX) || defined (OS_DOS32)
#define MATHHELPFILENAME "Math.mac.txt"
#elif defined(MPW) || defined(MW_CW)
#define MATHHELPFILENAME "Math.mac"
#elif defined(BCPP) || defined(COMP_DJGPP)
#define MATHHELPFILENAME "MATH.MAC"
#else /*MPW || MW_CW*/
#define MATHHELPFILENAME "math.mac"
#endif /*MPW || MW_CW*/
#endif /*!NOMATHHELP && !MATHHELPFILENAME*/
#define MATHHELPINDEXNAME "math_index"

#if !defined(NOREGRESSHELP) && !defined(REGRESSHELPFILENAME)
#if defined(SKEL_CARAPACE) || defined(BASE_UNIX) || defined (OS_DOS32)
#define REGRESSHELPFILENAME "Regress.mac.txt"
#elif defined(MPW) || defined(MW_CW)
#define REGRESSHELPFILENAME "Regress.mac"
#elif defined(BCPP) || defined(COMP_DJGPP)
#define REGRESSHELPFILENAME "REGRESS.MAC"
#else /*MPW || MW_CW*/
#define REGRESSHELPFILENAME "regress.mac"
#endif /*MPW || MW_CW*/
#endif /*!NOREGRESSHELP && !REGRESSHELPFILENAME*/
#define REGRESSHELPINDEXNAME "regress_index"

#if !defined(NOUSERFUNHELP) && !defined(USERFUNHELPFILENAME)
#if defined(SKEL_CARAPACE) || defined(BASE_UNIX) || defined (OS_DOS32)
#define USERFUNHELPFILENAME "Userfun.hlp.txt"
#elif defined(MPW) || defined(MW_CW)
#define USERFUNHELPFILENAME "Userfun.hlp"
#elif defined(BCPP) || defined(COMP_DJGPP)
#define USERFUNHELPFILENAME "USERFUN.HLP"
#else /*MPW || MW_CW*/
#define USERFUNHELPFILENAME "userfun.hlp"
#endif /*MPW || MW_CW*/
#endif /*!NOUSERFUNHELP && !USERFUNHELPFILENAME*/
#define USERFUNHELPINDEXNAME "userfun_index"

#if defined(SKEL_CARAPACE)
#define GUIHELPFILENAME "Gui.hlp.txt"
#define GUIHELPINDEXNAME "gui_index"
#endif

#endif /* _VERSIONH_ */
