/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 4.12 or later
*(C)*
*(C)* Copyright (c) 2002 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*                                                                 
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/

/*
  980119 added OverflowTextFrame()
  980328 added #include <math.h> before extern "C" {... #include "globals.h"}
         and removed undef and def of __cplusplus
  980401 moved #include <Xm/Text.h> to after #include "wx.h"
  980729 fixed wx_msw bug in fmyeol() which put two newlines on spool file
         for every one that was wanted.
  981002 fixed bug in get_dataPath that did not get the complete path
         when it containd a blank.  This affected only Windows version
  981028 Moved #include <pwd.h> to after #include "../globals.h"
  981110 Corrected Code for reading from console so it doesn't crash when
         cancel is clicked.
  990215 Replaced myerrorout() by putOutErrorMsg() and putOUTSTR() by
         putErrorOUTSTR()
  000308 If argument to myAlert is OUTSTR, OUTSTR is cleared
  000705 Added code to check for 'q' and 'r' during mouse input
  001027 Renamed LineBuffer to DlogLineBuffer and made it a handle allocated
         in fillFromDialog() and cleaned up when necessary in cleanitup()
  020407 Incorporated wxFileNameFromPath() and wxPathOnly() from
		 wb_utils.cpp in wxwin168/src/base for use in the Windows
		 version.  For reasons that remain unexplained, when these
		 are loaded from the compiled WxWin library, they crash

*/

// For compilers that support precompilation, includes "wx.h".
#include "wx_prec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx.h"
#endif

#ifdef wx_motif
#include <Xm/Text.h>
#endif  // wx_motif

#include <math.h>

/*
  980403 moved the following before include wxIface.h and wxmain.h
  990609 changed matdat.h to mvIo.h
  001025 changed MAXLINE to MAXDIALOGBUF
  010529 added splitPathName()
  010604 modified splitPathName()
*/
extern "C" {
#include "../globals.h"
#include "../mvIo.h"
/*
  981028 moved the include <pwd.h> from outside the extern to after globals.h
*/
#ifdef HASPWDH
#include <pwd.h>
#endif /*HASPWDH*/
} /*extern "C"*/

#include "wxIface.h"
#include "wxmain.h"

/*
  Functions in this file
  Various input/output related utilities
	void lineToWindow(char *msg, long msgLength)
	void fmyeol(FILE *fp)
	void putprompt(char *prompt)
	void myAlert(char *msgs)
	int mygetosversion(void)
	void fmyprint(char *msg,FILE *fp)

  Functions having to do with maintaining the command history
	static void   shiftSavedLines(void)
	static char ** getHistoryLine(int op, int * historyError)
	void initHistory(long historyLength)
	void saveInHistory(char ** lineH, int offset)
	static char **getCurrentCommand(void)
	char * recallHistory(int id)
	long getSomeHistory(long nlines, char *** historyHandle)
	void setSomeHistory(long nlines, char ** commands)

  Functions having to do with obtaining and checking file names
	char *expandFilename(char *fname)
	static int filenameFormOK(char *fname, int path)
	static int filenameCharsOK(char * fname)
	long okfilename(char *fname)
	long isfilename(char *fname)

  More output utilities
	void fmyflush(FILE *fp)
	void mybeep(long n)

  MacAnovaBaseFrame member functions and others
	void MacAnovaBaseFrame::OverflowTextFrame(int outOfMemory, long cutLength)
	void MacAnovaBaseFrame::lineToWindow(char * msg,  long msgLength)
	void MacAnovaBaseFrame::fmyeol(FILE * fp)
	int fillFromDialog(void)
	void MacAnovaBaseFrame::PutPrompt(char *prompt)
	void MacAnovaBaseFrame::PutPrompt(char *prompt, long frameIndex)

  More input/output utilities
	char *get_dataPath(void)
	void incrementNLINES(char *msg)
	FILE * fmyopen(char * fileName, char * mode)
	char * wxFindFile(char *fileName, char *message, char *defaultname,
		unsigned long control, mvDirInfoPtr dirInfo)
*/
enum mouseKeyCodes
{
	abortKey          = 'q',
	restartKey        = 'r',
	interruptKey      = 'i',
	commandKey        = WXK_CONTROL,
	shifterKey        = WXK_SHIFT
};

extern "C" {

void lineToWindow(char *msg, long msgLength)
{
	BaseFrame->lineToWindow(msg,msgLength);
} /*lineToWindow()*/

void fmyeol(FILE *fp)
{
	BaseFrame->fmyeol(fp);
} /*fmyeol()*/

void putprompt(char *prompt)
{
	BaseFrame->PutPrompt(prompt);
}

void myAlert(char *msgs)
{
	(void) wxMessageBox(msgs,"MacAnova Alert");
	if (msgs == OUTSTR)
	{
		*msgs = '\0';
	}
}

int mygetosversion(void)
{
	return (::wxGetOsVersion());
} /*mygetosversion*/

#define BEL '\007'
extern char       *TooManyErrors;

#ifndef OUTLINESIZE
#define OUTLINESIZE   400  /*size of output line buffer*/
#endif /*OUTLINESIZE*/

/* buffer for building output lines */
static char       wxOutLineBuf[OUTLINESIZE];
       char      *OutLineBuf = wxOutLineBuf;
static long       OutLinePlace = 0;


/*
	If fp != STDOUT, put msg to file fp; otherwise add msg to OutLineBuf and
	flush *OutLineBuf if last character is CR (Mac end of line)
	000520 added check for line starting with "WARNING:"
	010802 Now checks global accessed through macro SWALLOWOUTPUT to 
		   determine if stuff is printed on STDOUT.  Error messages
		   are always printed; also checks global access through
		   SWALLOWSPOOL to see if it should be spooled
*/


void fmyprint(char *msg,FILE *fp)
{
	long            i, msgLength;
	char           *outline, c;
	WHERE("fmyprint");

	if(fp == STDOUT)
	{		/* collect line in OutLineBuf */
		int         isError = strncmp(ErrorStart, msg, (int) 6) == 0;
		int         printwarnings = 
		  PRINTWARNINGS || isError || strncmp(WarningStart,msg, (int) 8) != 0;
		int         printit =  (!SWALLOWOUTPUT || isError) && printwarnings;
		int         spoolit =  (!SWALLOWSPOOL || isError) && printwarnings;

		if(isError && MAXERRORS1)
		{
			NERRORS++;
		}
		if (SPOOLFILE != (FILE *) 0 && spoolit)
		{
			fputs(msg, SPOOLFILE);
			if (SWALLOWSPOOL)
			{
				fputs("\n", SPOOLFILE);
			}
		} /*if (SPOOLFILE != (FILE *) 0 && spoolit)*/

		if (printit)
		{
			msgLength = strlen(msg);
			if (OutLinePlace + msgLength > OUTLINESIZE)
			{
				lineToWindow(OutLineBuf, OutLinePlace);
				if(msgLength > OUTLINESIZE)
				{
					lineToWindow(msg, msgLength);
					msgLength = 0;
				}
			}
			outline = OutLineBuf + OutLinePlace;
			for (i = 0; i < msgLength; i++)
			{
				c = *msg++;
#ifndef wx_msw
				*outline++ = (c == CR) ? NL : c;
#else /*wx_msw*/
				if (c == NL)
				{
					*outline++ = CR;
				}
				*outline++ = c;
#endif /*wx_msw*/
			} /*for (i = 0; i < msgLength; i++)*/
			*outline = '\0';
			OutLinePlace += msgLength;
			if (OutLinePlace > 0 && OutLineBuf[OutLinePlace-1] == NL)
			{
				lineToWindow(OutLineBuf, OutLinePlace);
			}
			if (SWALLOWOUTPUT)
			{ /* must be error message; add EOL */
				fmyflush(STDOUT);

				MacAnovaTextWindow  *runningWindow = BaseFrame->RunningWindow;
#ifdef wx_msw
				lineToWindow(EOL_STRING,strlen(EOL_STRING));
#else /*wx_msw*/
				long         length = runningWindow->GetLastPosition();

				Widget textWidget = (Widget) runningWindow->handle;
				XmTextInsert (textWidget, length, EOL_STRING);
				runningWindow->ShowPosition(length); //XmTextScroll(textWidget,1);
#endif /*wx_msw*/
				long                nLines = runningWindow->GetNumberOfLines();
				
				runningWindow->CmdInsertLine = nLines;
				runningWindow->CmdStrPos = runningWindow->GetLastPosition();
			} /*if (SWALLOWOUTPUT)*/
		} /*if (printit)*/
	} /*if(fp == STDOUT)*/
	else
	{
		fputs(msg, fp);
	} /*if(fp == STDOUT){}else{}*/
} /*fmyprint()*/

#ifdef SAVEHISTORY

/* 970112 code to handle history*/

extern "C" {
enum historyCodes
{
	NOHISTORY = 0,
	UPHISTORY,
	DOWNHISTORY,
	TOPHISTORY,
	BOTTOMHISTORY
};

enum historyErrors
{
	NOHISTORYERROR = 0,
	NOHISTORYTOGET,
	OFFHISTORYBOTTOM,
	OFFHISTORYTOP,
	HISTORYALLOCATIONERROR
};

#if (0) /* for debugging */
void displayLine(char * line)
{
	char        *outstr = OUTSTR;

	while (*line)
	{
		sprintf(outstr,"%02x", (int) *line++);
		outstr += strlen(outstr);
	}
	myAlert(OUTSTR);
	*OUTSTR = '\0';
} /*displayLine()*/
#endif /*0*/

/* handle for vector of char handles for history of previous commands*/
static char      ****SavedLines = (char ****) 0;
/*
   Handle to contents of current command, created when accessing previous
   commands
*/
static char        **CurrentCommand = (char **) 0;
/* the current number of lines that are currently saved <= HISTORY */
static long          NSavedLines = 0;
/*
  index in history list of line currently being displayed if it came
  from the history list.  -1 means not a previous command
*/
static long          CurrentLine = -1;

/*
   Shift all saved lines up 1 in preparation for adding new line to list
   Dispose of the earliest one if limit has been reached
*/
static void   shiftSavedLines(void)
{
	short       i;
	char      **temp;
	
	if (NSavedLines > 0)
	{
		temp = (*SavedLines)[NSavedLines-1];
		for (i = NSavedLines - 1; i > 0; i--)
		{
			(*SavedLines)[i] = (*SavedLines)[i - 1];
		}                                 
		(*SavedLines)[0] = (char **) 0;
		if (NSavedLines < HISTORY)
		{
			(*SavedLines)[NSavedLines] = temp;
		} /*if (NSavedLines < HISTORY)*/
		else
		{
			mydisphandle(temp);
		}
		CurrentLine = -1;
	} /*if (NSavedLines > 0)*/
} /*shiftSavedLines()*/

/*
  Retrieve a line from history
  Values of op
    UPHISTORY     get next earlier command than currently displayed
    DOWNHISTORY   get next later command than currently displayed
    TOPHISTORY    get earliest command saved
    BOTTOMHISTORY redisplay command that had been typed before recalling
                  earlier commands
*/
static char ** getHistoryLine(int op, int * historyError)
{
	char     **temp = (char **) 0;
	long       current = CurrentLine;
	int        incr = 0;
	WHERE("getHistoryLine");

	if (SavedLines != (char ****) 0)
	{
		if (op == UPHISTORY)
		{
			current++;
		}
		else if (op == DOWNHISTORY)
		{
			current--;
		}
		else if (op == TOPHISTORY)
		{
			current = NSavedLines - 1;
		}
		else if (op == BOTTOMHISTORY)
		{
			current = -1;
		}
		*historyError = NOHISTORYERROR;
		if (current >= 0 && current < NSavedLines)
		{
			temp = (*SavedLines)[current];
			CurrentLine = current;
		}
		else if (current < 0)
		{
			if (current < -1)
			{
				*historyError = OFFHISTORYBOTTOM;
			}
			temp = CurrentCommand;
			CurrentLine = -1;
		}
		else
		{
			*historyError = OFFHISTORYTOP;
			if (NSavedLines > 0)
			{
				temp = (*SavedLines)[NSavedLines-1];
				CurrentLine = NSavedLines - 1;
			}
			else
			{
				temp = CurrentCommand;
				CurrentLine = -1;
			}
		}
	} /*if (SavedLines != (char ****) 0)*/
	else
	{
		*historyError = NOHISTORYTOGET;
		temp = CurrentCommand;
		CurrentLine = -1;
	}
	return (temp);
} /*getHistoryLine()*/

/*
   Initialize or change the length of a vector of handles to hold recent
   history

   If there are memory allocation problems, mygethandle() or
   mygrowhandle() will print an error message, and history will be turned
   off because SavedLines will be NULL

   971023 Fixed apparent memory leak when history length is reduced
   971210 Negative historyLength disposes of lines but not SavedLines
*/

void initHistory(long historyLength)
{
	long           i;
	WHERE("initHistory");

	if (SavedLines != (char ****) 0 && NSavedLines > historyLength)
	{
		long        start = (historyLength >= 0) ? historyLength : 0;
		
		/* dispose of extras, if any */
		for (i = start; i < NSavedLines; i++)
		{
			mydisphandle((*SavedLines)[i]);
			(*SavedLines)[i] = (char **) 0;
		}
		NSavedLines = start;
	} /*if (SavedLines != (char ****) 0)*/
	
	if (historyLength < 0)
	{
		historyLength = HISTORY;
	}
	else if (historyLength > 0)
	{
		if (SavedLines == (char ****) 0)
		{
			SavedLines = (char ****) mygetsafehandle((historyLength) * sizeof(char **));
			if (SavedLines == (char ****) 0)
			{
				historyLength = 0;
			}
			else
			{
				for (i = 0; i < historyLength; i++)
				{
					(*SavedLines)[i] = (char **) 0;
				}
			}
			NSavedLines = 0;
		}
		else if (historyLength != HISTORY)
		{
			char       ****savedLines =
			  (char ****) mygrowsafehandle((char **) SavedLines,
										   historyLength * sizeof(char **));

			if (savedLines == (char ****) 0)
			{
				/* don't changed HISTORY */
				historyLength = HISTORY;
			}
			else
			{
				SavedLines = savedLines;
				/* clear new slots, if any */
				for (i = NSavedLines; i < historyLength; i++)
				{
					(*SavedLines)[i] = (char **) 0;
				}
				if (historyLength < NSavedLines)
				{
					NSavedLines = historyLength;
				}
			}
		}
	} /*if (historyLength > 0)*/
	else if (SavedLines != (char ****) 0)
	{
		mydisphandle((char **) SavedLines);
		SavedLines = (char ****) 0;
		NSavedLines = 0;
	}

	CurrentLine = -1;
	mydisphandle(CurrentCommand);
	CurrentCommand = (char **) 0;
	HISTORY = historyLength;
} /*initHistory()*/

/*
   Save *lineH in history list.

   All previous lines are shifted up 1, discarding the last if the
   limit has been reached.  The line being saved is put in slot 0
*/

void saveInHistory(char ** lineH, int offset)
{
	long        i;
	WHERE("saveInHistory");
	
	mydisphandle(CurrentCommand);
	CurrentCommand = (char **) 0;
	CurrentLine = -1;
	if (SavedLines != (char ****) 0 && (*lineH)[offset] != '\0')
	{
		long        nDoubles = strlen(*lineH + offset)/sizeof(double) + 1;
		char      **line = mygethandle(nDoubles*sizeof(double));
		/*
			allocate a multiple of sizeof(double) to make it more likely
			to use a handle in one of the caches
		*/

		if (line == (char **) 0)
		{
			goto errorExit;
		}
		shiftSavedLines();
		(*SavedLines)[0] = line;
		strcpy(*line, *lineH + offset);
		if (NSavedLines < HISTORY)
		{
			NSavedLines++;
		}
	} /*if (SavedLines != (char ****) 0 && (*lineH)[offset] != '\0')*/
	return;

  errorExit:
/* don't know anything better to do than clear history list */
	myAlert("History of past commands cleared");
	for (i = 0;i < NSavedLines;i++)
	{
		mydisphandle((*SavedLines)[i]);
		(*SavedLines)[i] = (char **) 0;
	} /*for(i=0;i<NSavedLines;i++)*/
	mydisphandle(CurrentCommand);
	CurrentCommand = (char **) 0;
	NSavedLines = 0;
	CurrentLine = -1;

} /*saveInHistory()*/


/*
  Create a handle containing everything from the start of the command
  line to the end of command window
*/
static char **getCurrentCommand(void)
{
	char               **lineH = (char **) 0, *line;
	MacAnovaTextWindow  *mtw = BaseFrame->RunningWindow;
	
	line = mtw->GetSelection(mtw->CmdStrPos, mtw->GetLastPosition());
	if (line != (char *) 0)
	{
		int     nDoubles;

#ifdef wx_msw
		stripCR(line);
#endif /*wx_msw*/
		nDoubles = strlen(line)/sizeof(double) + 1;

		lineH = mygethandle(nDoubles * sizeof(double));
		if (lineH != (char **) 0)
		{
			strcpy(*lineH, line);
		}
		delete [] line;
	} /*if (line != (char *) 0)*/
	return (lineH);
} /*getCurrentCommand()*/

/*
	970112 Handle Up History and Down History commands on Edit menu
	971210 Added getSomeHistory() and setSomeHistory()
*/

char * recallHistory(int id)
{
	char        **lineH;
	char         *line = (char *) 0;
	int           historyOp, historyError;
	WHERE("recallHistory");

	if (!Running && BDEPTH <= 0)
	{
		if (CurrentLine < 0)
		{
			mydisphandle(CurrentCommand);
			CurrentCommand = getCurrentCommand();
		} /*if (CurrentLine < 0)*/

		if (id == MACANOVA_EDIT_UPHISTORY)
		{
			historyOp = UPHISTORY;
		}
		else
		{
			historyOp = DOWNHISTORY;
		}
		lineH = getHistoryLine(historyOp, &historyError);
		line = (lineH != (char **) 0) ? *lineH : NullString;

		if (historyError != NOHISTORYERROR)
		{
			mybeep(1);
		}
	} /*if (!Running && BDEPTH <= 0)*/
	return (line);
} /*recallHistory()*/

enum historyScratch
{
	GLINES = 0,
	NTRASH
};

long getSomeHistory(long nlines, char *** historyHandle)
{
	char         **lines;
	long           i, needed;
	char          *place;
	TRASH(NTRASH, errorExit);
	
	if (NSavedLines <= 0)
	{
		nlines = 0;
	}
	else if (nlines == 0 || nlines >= NSavedLines)
	{
		nlines = NSavedLines - 1;
	}
	
	if (nlines == 0)
	{
		needed = 1;
	}
	else
	{
		needed = 0;
		for (i = 1; i <= nlines; i++)
		{
			needed += strlen(*(*SavedLines)[i]) + 1;
		}
	}
	
	if (!getScratch(lines, GLINES, needed, char))
	{
		goto errorExit;
	}
	
	place = *lines;

	if (nlines == 0)
	{
		nlines = 1;
		place[0] = '\0';
	}
	else
	{
		for (i = nlines; i > 0; i--)
		{
			place = copyStrings(*(*SavedLines)[i], place, 1);
		}
	}
	unTrash(GLINES);
	*historyHandle = lines;
	emptyTrash();
	return (nlines);
		
  errorExit:
	return (0);
} /*getSomeHistory()*/

void setSomeHistory(long nlines, char ** commands)
{
	long         i, history = HISTORY;
 	int          skippedlines = (nlines > HISTORY) ? nlines - HISTORY : 0;

	initHistory(-1); /* get rid of what we've got */
	initHistory(history);

	for (i = 0; i < nlines; i++, skippedlines++)
	{
		saveInHistory(commands,
					  skipStrings(*commands, skippedlines) - *commands);
	}
} /*setSomeHistory()*/
} /*extern "C" */

#endif /*SAVEHISTORY*/



#ifdef MSDOS
/*  '/' equivalent to '\' on MSDOS*/
#define isSlash(C) ((C) == DIRSEPARATOR[0] || (C) == '/')
#else /*MSDOS*/
#define isSlash(C) ((C) == DIRSEPARATOR[0])
#endif /*MSDOS*/

/*
  Expand leading '~' in a filename.  A name of the form ~/path (or ~\path
  under MSDOS) is expanded to home/path or home\path, where home is the value
  of MacAnova CHARACTER variable HOME.  Under Unix, HOME is predefined to
  have value taken from environmental variable $HOME.  Under MSDOS, HOME is
  predefined to contain the name of the directory containing MACANOVA.EXE.
  Under Unix, a name of form ~username/path is expanded to homedir/path where
  getpwname() is used to get the home directory homedir for user username.
  Such a file name is not expanded under MSDOS and results in an error.
  On Macintosh, HOME is not predefined, but may be set in MacAnova.ini.  The
  usage ~username/path is not legal on a Mac

  950829 Fixed bug in DJGPP version
         Under MSDOS, any embedded '/' are changed to DIRSEPARATOR[0]
         (backslash))
*/

static char    FullPathName[PATHSIZE];

char *expandFilename(char *fname)
{
	Symbolhandle    home;
	char           *pc;
	char           *userdir;
	long            dirNameLength;
#ifdef HASPWDH
	struct passwd  *passwd;
#endif /*HASPWDH*/
		
	if (fname[0] == EXPANSIONPREFIX)
	{
		if (isSlash(fname[1]))
		{						/* treat ~/... as $HOME/... */
			home = Lookup("HOME"); /* should be initialized at startup */
			if(home == (Symbolhandle) 0 || TYPE(home) != CHAR ||
			   !isScalar(home))
			{
				userdir = getenv("HOME");
				if (userdir == (char *) 0)
				{
					sprintf(OUTSTR,
							"ERROR: cannot expand %c%c;  no CHARACTER var. HOME or environment var. $HOME",
							fname[0], fname[1]);
					goto errorExit;
				}
			}
			else
			{
				userdir = STRINGPTR(home);
			}
			
			pc = fname+2;
		} /*if (isSlash(fname[1]))*/
		else
		{
#ifndef HASPWDH
			/* can't expand ~name on except on Unix */
			userdir = (char *) 0;
#else  /*HASPWDH*/
			char        c;

			for(pc = fname+1;*pc && !isSlash(*pc);pc++)
			{
				;
			}
			c = *pc;
			*pc = '\0';
			passwd = getpwnam(fname+1);
			*pc = c;
			if(c != '\0')
			{
				pc++;
			}
				
			userdir = (passwd != (struct passwd *) 0) ?
				passwd->pw_dir :  (char *) 0;
#endif /*HASPWDH*/
		} /*if (isSlash(fname[1])){}else{}*/
		
		if(userdir == (char *) 0)
		{
			putPieces("ERROR: unable to expand ", fname,
					  (char *) 0, (char *) 0);
			goto errorExit;
		}
			
		dirNameLength = strlen(userdir);
		/* allow for the possiblity that HOME ends in '/' */
		if (isSlash(userdir[dirNameLength-1]))
		{
			dirNameLength--;
		}
		
		if(dirNameLength + strlen(pc) + 1 >= PATHSIZE)
		{
			putPieces("ERROR: expanded file name ",userdir,pc," is too long");
			goto errorExit;
		}
		
		strcpy(FullPathName,userdir);
		FullPathName[dirNameLength++] = DIRSEPARATOR[0];
		FullPathName[dirNameLength] = '\0';
		strcat(FullPathName,pc);
	} /*if (fname[0] == EXPANSIONPREFIX)*/
	else
	{
		if(strlen(fname) >= PATHSIZE)
		{
			putPieces("ERROR: file name ", fname, " is too long", (char *) 0);
			goto errorExit;
		}
		
		strcpy(FullPathName, fname);
	} /*if (fname[0] == EXPANSIONPREFIX){}else{}*/
	
#ifdef MSDOS
	for (pc = FullPathName; *pc; pc++)
	{
		if (isSlash(*pc))
		{
			*pc = DIRSEPARATOR[0];
		}
	} /*for (pc = FullPathName; *pc; pc++)*/
#endif /*MSDOS*/

	return ((char *) FullPathName);
		
  errorExit:
	putErrorOUTSTR();
	
	return ((char *) 0);

} /*expandFilename()*/
	
/*
  Check file names for legality on platform
*/

#define isDot(C) ((C) == '.')

#ifdef MSDOS

/*
  Functions to check on file names
  static int filenameFormOK(char *fname, static int pathOK)
  static int filenameCharsOK(char * fname)
  long okfilename(char *fname) (calls filenameFormOK(), filenameCharsOK())
  long okpathname(char *fname) (calls filenameFormOK(), filenameCharsOK())
  long isfilename(char *fname) (calls okfilename(), prints error msg) 

*/
/*
  For MSDOS and Windows acceptable names are of the form
    pathElement/.../pathElement, or /pathElement/.../pathElement,
  possible prefixed by a drive designator of the form [a-zA-Z]:
  '/' may be replaced by '\'.

  For MSDOS pathElement is of form '.', '..', Name or Name.Ext, and
  Name has from 1 to 8 each characters and Ext has from 0 to three
  characters

  For Windows, when long file names are not permitted (Windows 3.1),
  acceptable names are the same as for MSDOS.
  When long file names are permitted (Windows 95 and Windows NT),
  a path element may contain up to 255 characters including any
  number of dots and the elements of a name (sequences of non-dots,
  non-slashes) preceding and/or following a dot can be of any length
  including 0.  Thus ..abc..d is a legal name.

  If path != 0, names ending in / or \ are acceptable.  Otherwise, they
  are not.

  970805 In Windows 95 and Windows NT long file names with multiple
  dots (a.b.c) are allowed

  020117 When MSDOS is defined, file names can include '~'
*/
#ifdef wx_msw
#define MAXPATHCOMPONENT  255 /*limit on component of Windows 95 path*/
#endif /*wx_msw*/

static int filenameFormOK(char *fname, int pathOK)
{
	int        maxComponent, maxNamePart, maxNDots;
	char      *start = fname, *pc;
	int        namePartLength = 0, componentLength = 0, nDots = 0;
#ifdef wx_msw
	int        longFileNames =
    	(ThisMachine == mvWindows95 || ThisMachine == mvWindowsNT);
#endif /*wx_msw*/

	if (strlen(start) >= PATHSIZE)
	{
		/* length too long for buffers */
		return (0);
	}
	if (isalpha(start[0]) && start[1] == ':')
	{ /*skip volume id and ':'*/
		start += 2;
	}

#ifdef wx_msw
	maxComponent = (!longFileNames) ? 12 : MAXPATHCOMPONENT;
	maxNamePart = (!longFileNames) ? 8 : MAXPATHCOMPONENT;
	maxNDots = (!longFileNames) ? 1 : MAXPATHCOMPONENT/2;
#else /*wx_msw*/
	maxComponent = 12;
	maxNamePart = 8;
	maxNDots = 1;
#endif /*wx_msw*/

	for(pc = start;*pc != '\0';pc++)
	{
		if(*pc == ':')
		{	/* have already skipped 'x:', ':' now illegal */
			return (0);
		}
		if(isDot(*pc))
		{
			if (nDots == 0 && namePartLength == 0)
			{
				if(isSlash(pc[1]))
				{/* pathElement is '.' */
					pc++;
				}
				else if(isDot(pc[1]) && isSlash(pc[2]))
				{/* pathElement is '..' */
					pc += 2;
				}
				else
				{
#ifdef wx_msw
					if (!longFileNames)
					{ 	/* no Name; error */
						return (0);
					}
					nDots++;
#else /*wx_msw*/
					/* no Name; error */
					return (0);
#endif /*wx_msw*/
				}
			} /*if (nDots == 0 && namePartLength == 0)*/
			else
			{/* Name completed */
				if (++nDots > maxNDots)
				{
					return (0);
				}
				namePartLength = 0;
				componentLength++;
#ifdef wx_msw
				maxNamePart = (!longFileNames) ? 3 : maxNamePart;
#else /*wx_msw*/
				maxNamePart = 3;
#endif /*wx_msw*/
			}
		} /*if(isDot(*pc))*/
 		else if(isSlash(*pc))
		{/* end of pathElement */
			componentLength = namePartLength = nDots = 0;
#ifdef wx_msw
			maxNamePart = (!longFileNames) ? 8 : maxNamePart;
#else /*wx_msw*/
			maxNamePart = 8;
#endif /*wx_msw*/
		}
		else
		{ /* name character */
			namePartLength++;
			componentLength++;
			if (namePartLength > maxNamePart ||
				componentLength > maxComponent)
			{
				return (0);
			}
		}
	} /*for(pc = start;*pc;pc++)*/

	return ( pathOK || (--pc , !isSlash(*pc)));
} /* filenameFormOK()*/

#else /*MSDOS*/

/*
  On Unix accept non-empty names of the form
       pathElement or pathElement/pathElement or pathElement/.../pathElement
  where a pathElement does not contain '/' but can be empty/  Note that
  this includes paths with repeated '/' (e.g., "foo//bar") or
  of the form "../bar"
*/
static int filenameFormOK(char * fname, int path)
{
	/*
	   only current restriction is that it can't end in '/';
	   or start with '-'
	   repeated '/'s allowed
	   */
	return (fname[0] != '-' && (path || !isSlash(fname[strlen(fname)-1])));

} /* filenameFormOK()*/

#endif /*MSDOS*/

/*
   Check for illegal characters in fname
   All alphanumerics are legal plus the characters in okNonAlnum.
   Since fname has been checked for form before being checked
   here, special characters like '/' are included in okNonAlnum
*/
static int filenameCharsOK(char * fname)
{
#if defined(wx_msw)
	/*
	  Only allow standard DOS filename characters under Windows
	  under Windows 95 and NT allow also '[', /]' and ' '
	*/
	char      *okNonAlnum = (ThisMachine == mvWin32s) ?
		".\\/:_-`!@#$%^&(){}~'" : ".\\/:_-`!@#$%^&(){}[]~' ";
#elif defined(MSDOS)
	char      *okNonAlnum = ".\\/:_-`!@#$%^&(){}~'";
#elif defined(VMS) /*MSDOS*/
	/* For VMS allow only : */
	char      *okNonAlnum = ".:[]<>_-+$";
#else  /*MSDOS*/
	/*don't allow (){}[]`'";|\~ even though that are technically legal */
	char      *okNonAlnum = "./_,-+!@#$%^&";
#endif /*MSDOS*/
	char       c;
	
	while((c = *fname++) != '\0')
	{
		if (!isalnum(c) && !strchr(okNonAlnum, c))
		{
			return (0);
		}
	} /*while((c = *fname++) != '\0')*/
	return (1);
} /*filenameCharsOK()*/

/*
   silent check of appropriateness of file name
*/
long okfilename(char *fname)
{
	return (*fname != '\0' && filenameFormOK(fname, 0) &&
			filenameCharsOK(fname));
} /*okfilename()*/

/*
   silent check of appropriateness of path name
*/
long okpathname(char *fname)
{
	return (*fname != '\0' && filenameFormOK(fname, 1) &&
			filenameCharsOK(fname));
} /*okpathname()*/

/* Check filename with error messages */
long isfilename(char *fname)
{
	if (okfilename(fname))
	{
		return (1);
	}

	if(*fname == '\0')
	{
		putOutErrorMsg("ERROR: zero length file name");
	}
	else
	{
		putPieces("ERROR: improper file name ", fname, (char *) 0, (char *) 0);
	}
	return (0);
} /*isfilename()*/

/*
  010529 new function
  function to split off either the file name (nameOnly != 0) or the
  directory name (nameOnly == 0) from full name in fullPathName, returning
  the piece wanted in pathPart which has length partLength
*/
void splitPathName(int nameOnly, char fullPathName [], char pathPart [],
				   int partLength)
{
	int        n = strlen(fullPathName);
	char       separator = DIRSEPARATOR[0];
	char      *tail, *pieceWanted;

	for (tail = fullPathName + n; *tail != separator && tail >= fullPathName;
		 n--, tail--)
	{
		;
	}
	tail++;
	n++;

	if (nameOnly)
	{
		pieceWanted = tail;
		n = strlen(tail);
	}
	else
	{
		pieceWanted = fullPathName;
	}
	if (n >= partLength)
	{
		n = partLength - 1;
	}
	
	strncpy(pathPart, pieceWanted, n);
	pathPart[n] = '\0';
} /*splitPathName()*/

void fmyflush(FILE *fp)
{
	if(fp != STDOUT)
	{
		fflush(fp);
	}
	else
	{
		lineToWindow(OutLineBuf, OutLinePlace);

		if(SPOOLFILE != (FILE *) 0)
		{
			fflush(SPOOLFILE);
		}
	}
} /*fmyflush()*/

void mybeep(long n)
{
	if(BDEPTH == 0 && ISATTY)
	{
		while(n-- > 0)
		{
			wxBell();
		}
	}
} /*mybeep()*/


} /* extern "C" */

void MacAnovaBaseFrame::OverflowTextFrame(int outOfMemory, long cutLength)
{
	char            wTitle[PATHSIZE+1];
	char            outstr[400];
	char           *dashes = "----------------------------------";

#if (0)
	ALERT("XX Length + msgLength = %d, CMDWINDOWLIMIT = %d",
					Length+msgLength,CMDWINDOWLIMIT,0,0);
#endif /*0*/
	myAlert("Output window contents are too long; saving window");

	TextFrames[RunningFrame]->SaveWindowAs();

	if(numTextFrames < MAXWINDOWS)
	{				/* we have room for a new window */
		if(!outOfMemory)
		{
			RunningWindow->SetInsertionPointEnd();
			RunningWindow->WriteText(EOL_STRING);
			RunningWindow->SetInsertionPointEnd();
			RunningWindow->WriteText(dashes);
			RunningWindow->SetInsertionPointEnd();
			RunningWindow->WriteText(EOL_STRING);
			RunningWindow->SetInsertionPointEnd();
			RunningWindow->WriteText("Continued in new window");
			RunningWindow->SetInsertionPointEnd();
			RunningWindow->WriteText(EOL_STRING);
		}
		RunningWindow->CmdStrPos = RunningWindow->GetLastPosition();
		strncpy(wTitle,TextFrames[RunningFrame]->GetTitle(),PATHSIZE);
		wTitle[PATHSIZE] = '\0';

		RunningWindow->clearUndoInfo();

		NewTextFrame(TRUE,FALSE); /* create new window without prompt*/

		RunningFrame = numTextFrames-1;
		RunningWindow = TextFrames[RunningFrame]->text_window;

		sprintf(outstr,"Output continued from %s",wTitle);

		RunningWindow->WriteText(outstr);
		RunningWindow->WriteText(EOL_STRING);
		RunningWindow->WriteText(dashes);
 		RunningWindow->WriteText(EOL_STRING);
	} /*if(numTextFrames < MAXWINDOWS)*/
	else
	{				/* no room, trim off top */
		RunningWindow->CmdInsertLine = 1;
		RunningWindow->SetSelection(0L, 0L);
		RunningWindow->ShowPosition(0);

		sprintf(outstr,
			"Cannot open a new window%s%ld characters will be trimmed off the top",
			EOL_STRING,cutLength);
		myAlert(outstr);

		RunningWindow->SetSelection(0L, cutLength);
		RunningWindow->Cut();
		if(RunningWindow->UndoStatus == OUTPUTUNDO)
		{
			RunningWindow->LastLength -= cutLength;
			RunningWindow->LastCmdStrPos -= cutLength;
			if(RunningWindow->LastCmdStrPos < 0)
			{
				RunningWindow->clearUndoInfo();
			}
		}			/*if(UndoStatus == OUTPUTUNDO)*/
	} /*if(numTextFrames < MAXWINDOWS){}else{}*/

} /* MacAnovaBaseFrame::OverflowTextFrame() */


/*
  Write line of output to file fp which may specify command window
  If writing to console, interpret leading '\007's as BEL's
  If spooling is active, write line to spool file
  If line starts with "ERROR:", ring bell and count error
	970305 NLINES is now incremented here
*/

void MacAnovaBaseFrame::lineToWindow(char * msg,  long msgLength)
{
	long            nLines, cutLength;
	long            isOutLine = (msg == OutLineBuf);
	char            outstr[400];
	long            length;
	WHERE("lineToWindow");

	length = RunningWindow->GetLastPosition();

#ifdef wx_msw //980218
	if (msgLength == 1 && msg[0] == '\n')
	{
		msgLength++;
	}
#endif /*wx_msw*/

	while(*msg == BEL)
	{							/* turn leading BEL's into beeps */
		mybeep(1);
		msg++;
		msgLength--;
	}

	if (msgLength > 0)
	{
		if(UseWindows > 0)
		{
			if (RunningWindow->UndoStatus != CANTUNDO &&
				RunningWindow->UndoStatus != OUTPUTUNDO)
			{
				RunningWindow->clearUndoInfo();
			}

			if (msgLength > CMDWINDOWLIMIT)
			{
				sprintf(outstr,
						"WARNING: output line longer than %ld characters is truncated",
						(long) CMDWINDOWLIMIT);
				myAlert(outstr);
				msgLength = CMDWINDOWNEWSIZE;
			} /*if (msgLength > CMDWINDOWLIMIT)*/
			length = RunningWindow->GetLastPosition();

			if (length + msgLength > CMDWINDOWLIMIT)
			{
				OverflowTextFrame(0,msgLength + length - CMDWINDOWNEWSIZE);
				length = RunningWindow->GetLastPosition();
			} /*if (length + msgLength > CMDWINDOWLIMIT)*/

			//RunningWindow->SetInsertionPoint(length);
#ifdef wx_msw
			RunningWindow->Synch();
#define KLUDGELENGTH 29900
			if (TextFrames[RunningFrame]->JustCreated)
			{
				/*
				  The code here is a kludge to try and get around a problem
				  of not being able to add text to a fairly long window
				  that was still far shorter than what one could reasonably
				  expect.

				  Essentially the window is created with a message of length
                  KLUDGELENGTH consisting of the line to be printed followed
                  by a lot of spaces.  Then immediately after inserting that
                  in the window, everything but the line to be printed is
				  removed.  The hope was that this would force Windows to
                  allocate at the start a big enough buffer; it seemed
				  to fix the problem.

				  The code taken from wx_text.cc was originally inserted
				  to try to pin down an apparent bug, but was kept because
				  it avoided allocating and deallocating an additional
				  2*KLUDGELENGTH bytes that are really not needed here.
				*/
				TextFrames[RunningFrame]->JustCreated = FALSE;
				// nothing yet in the window
				long        kludgeSize = KLUDGELENGTH - length;
				char       *kludgeBuffer = new char[kludgeSize+1];
				int         i;

#if (0)
				for (i = 0; i < msgLength; i++)
				{
					kludgeBuffer[i] = msg[i];
				}
				for(;i < kludgeSize;i++)
				{
					kludgeBuffer[i] = ' ';
				}
				kludgeBuffer[i] = '\0';
				RunningWindow->WriteText(kludgeBuffer);
#else /*0*/
				/*
				  Code adapted from wxTextWindow::WriteText()
				*/
				int         j;

				for (i = 0, j = 0; i < msgLength; i++, j++)
				{
     				if (msg[i] == '\n')
      				{
        				kludgeBuffer[j++] = '\r';
      				}
					kludgeBuffer[j] = msg[i];
				}
				while (j < kludgeSize)
				{
					kludgeBuffer[j++] = ' ';
				}
				kludgeBuffer[j] = '\0';
 				//myAlert("before SendMessage(...EM_REPLACESEL...)");
    			SendMessage((HWND) RunningWindow->ms_handle,
                			EM_REPLACESEL, 0, (LPARAM) kludgeBuffer);

#endif /*0*/
				delete [] kludgeBuffer;

				/*
				  Now remove everything but the line being printed
				*/
                //myAlert("Before RunningWindow->Remove()");
				RunningWindow->Remove(length + msgLength+1,
									  RunningWindow->GetLastPosition());
				//myAlert("After RunningWindow->Remove()");
#if (0)
				ALERT("length = %ld, msgLength = %ld, GetLastPosition() = %ld",
					  length, msgLength,RunningWindow->GetLastPosition(),0);
#endif /*0*/

			} /*if (TextFrames[RunningFrame]->JustCreated)*/
			else
			{
				RunningWindow->WriteText(msg);
			} /*if (TextFrames[RunningFrame]->JustCreated){}else{}*/

			long newLength = RunningWindow->GetLastPosition();
			if (newLength < length + msgLength)
			{
		/* Entire message did not go to window; overflow and write again*/
#if (0)
				RunningWindow->CheckForError();
#endif /*0*/
				if(CMDWINDOWNEWSIZE < length)
				{
					cutLength =  msgLength + length - CMDWINDOWNEWSIZE;
				}
				else
				{
					cutLength = ((2*msgLength < 1000) ? 1000 : 2*msgLength);
				}
				OverflowTextFrame(1, cutLength);
                RunningWindow->WriteText(msg);
			} /*if (newLength < length + msgLength)*/

#else //wx_msw
			//RunningWindow->Replace(length,length,msg);
			Widget textWidget = (Widget) RunningWindow->handle;
			XmTextInsert(textWidget,length,msg);

#endif // wx_msw
			nLines = RunningWindow->GetNumberOfLines();
			RunningWindow->CmdInsertLine = nLines;
			RunningWindow->CmdStrPos = RunningWindow->GetLastPosition();
			incrementNLINES(msg);
		} /*if(UseWindows > 0)*/

		if(strncmp(ErrorStart, msg, (int) 6) == 0)
		{
			mybeep(1);
		} /*if(strncmp(ErrorStart, msg, (int) 6) == 0)*/
	} /*if (msgLength > 0)*/

	if (isOutLine)
	{
		OutLinePlace = 0;
		OutLineBuf[0] = '\0';
	}
} /*MacAnovaBaseFrame::lineToWindow()*/

/*
   terminate output line and pause if writing the console and screen is full
   (no pausing on Mac)
   */

void MacAnovaBaseFrame::fmyeol(FILE * fp)
{
	long            length,nLines;
	char           *Newline = "\n";
    WHERE("MacAnovaBaseFrame::fmyeol");

	if(fp == STDOUT)
	{
		incrementNLINES(Newline);
		if (OutLinePlace > 0)
		{						/* flush OutLineBuf */
			lineToWindow(OutLineBuf, OutLinePlace);
		}
		if(UseWindows > 0 && !SWALLOWOUTPUT)
		{
			if(RunningWindow->UndoStatus != CANTUNDO &&
			   RunningWindow->UndoStatus != OUTPUTUNDO)
			{
				RunningWindow->clearUndoInfo();
			}

#ifdef wx_msw
 			lineToWindow(EOL_STRING,strlen(EOL_STRING));
#else //wx_msw
			length = RunningWindow->GetLastPosition();
			Widget textWidget = (Widget) RunningWindow->handle;
			XmTextInsert (textWidget, length, EOL_STRING);
			RunningWindow->ShowPosition(length); //XmTextScroll(textWidget,1);
#endif		// wx_msw
			nLines = RunningWindow->GetNumberOfLines();
			RunningWindow->CmdInsertLine = nLines;
			RunningWindow->CmdStrPos = RunningWindow->GetLastPosition();
		}						/*if(UseWindows > 0)*/

		if (SPOOLFILE != (FILE *) 0 && !SWALLOWSPOOL)
		{
			putc(Newline[0], SPOOLFILE);
		}

		wxYield();

		if (INTERRUPT != INTNOTSET)
		{
			INTERRUPT = PRINTABORT;
		}
	}						/*if(fp == STDOUT)*/
	else
	{
		putc(Newline[0], fp);
	}
} /*MacAnovaBaseFrame::fmyeol()*/

/*
  Fill line from dialog window (CONSOLE).  This code was formerly part of
  fillLINE() which was moved to commonio.c along with supporting routines
  datagetc and fileToLine()

  981110 Fixed code so that it recognizes when the uses clicks cancel
         This now signals the end of the read.  We never see anything
         the user had already put in the dialog box so we can't use it;
         this contrasts with the Done button in the Macintosh version
*/

/*
  LineBuffer and Place are used only in fillFromDialog()
  However, fillFromDialog() may be called more than once for
  each actual use of a dialog box, since the dialog box can obtain
  several new-line separated lines

  001025 replaced MAXLINE by MAXDIALOGBUF and made LineBuffer a pointer instead
         of a static array, initialized in initialize() to have length
         MAXDIALOGBUF
  001027 Renamed LineBuffer to DlogLineBuffer and made it a handle allocated
         in fillFromDialog() and cleaned up when necessary in cleanitup()
         Place is now a long, with value < 0 signaling were starting
*/

static Bool           HitDone = FALSE;
static long           DlogLineBufferSize;
#define Place DlogLineBufferPlace

extern "C" {
enum fillFromDialogConstants
{
	ok,
	doneItem
};

int fillFromDialog(void)
{
	long               length;
	unsigned char     *pc, *place;
	long               Length;
	int                itemHit = ok;
	char              *reply;
	WHERE("fillFromDialog");

	/*
	  Use dialog window or what was read previously
	  Always terminate (*LINE) with '\n'
	  Return only through first '\n'.
	  */
	if (Place < 0)
	{					/* first time in this trip */
		if (DlogLineBuffer != (unsigned char **) 0)
		{ /*Shouldn't happen but play it safe*/
			mydisphandle((char **) DlogLineBuffer);
		}
		DlogLineBuffer = (unsigned char **) mygethandle(LINELENGTH);
		Place = 0;
		HitDone = FALSE;
		if (DlogLineBuffer != (unsigned char **) 0)
		{
			DlogLineBufferSize = LINELENGTH;
		}
		else
		{
			itemHit = doneItem;
			goto errorExit;
		}
	} /*if (Place < 0)*/

	(*LINE)[0] = '\0';
	if (Place == 0)
	{
		/* fill DlogLineBuffer from dialog box */
		if(HitDone)
		{
			HitDone = FALSE;
			itemHit = doneItem;
			goto normalExit;
		}
		/* need to read from Dialog box */
		reply = wxGetTextFromUser("Enter Line","Console Input");

		HitDone = (reply == (char *) 0);

        if (HitDone)
        {
			itemHit = doneItem;
			HitDone = FALSE;
        	Length = 0;
			goto normalExit;
        }
		place = *DlogLineBuffer; /* dereference handle */
		Length = strlen(reply);
		if(Length > DlogLineBufferSize)
		{
			sprintf(OUTSTR,
					"WARNING: more than %ld characters entered; last %ld ignored",
					(long) DlogLineBufferSize, Length - DlogLineBufferSize);
			myAlert(OUTSTR);
			Length = DlogLineBufferSize;
		}
		strncpy((char *) place, reply, Length);

        place[Length] = '\0';
		strcpy(*LINE, (char *) place);

		if(place[0] == '\0' && HitDone)
		{
			HitDone = FALSE;
			itemHit = doneItem;
			goto normalExit;
		}
	} /* if (Place == 0)*/
	else
	{
		itemHit = (HitDone) ? doneItem : ok;
		place = *DlogLineBuffer + Place;
		strcpy(*LINE, (char *) place);
	}

	pc = (unsigned char *) strchr((char *) place, '\n');
	if (pc != (unsigned char *) 0 && pc[1] != '\0')
	{/* still something not seen */
		pc++;
		length = pc - place;
		Place += length;
		itemHit = ok;
	} /*if (pc != (unsigned char *) 0 && pc[1] != '\0')*/
	else
	{					/* this is the last of it */
		length = strlen((char *) place);
		if(pc == (unsigned char *) 0)
		{
			(*LINE)[length++] = '\n';
		}
		Place = 0;
		HitDone = FALSE;
	} /*if(pc != (unsigned char *) 0){}else{}*/
	(*LINE)[length] = '\0';

  normalExit:
  errorExit:
	if (itemHit == doneItem)
	{
		mydisphandle((char **) DlogLineBuffer);
		DlogLineBuffer = (unsigned char **) 0;
	}
	return ((itemHit == doneItem) ? EOF : 1);
} /*fillFromDialog()*/

} /* extern "C" */

void MacAnovaBaseFrame::PutPrompt(char *prompt)
{
	PutPrompt(prompt, RunningFrame);
}

void MacAnovaBaseFrame::PutPrompt(char *prompt, long frameIndex)
{
	TextFrames[frameIndex]->text_window->PutPrompt(prompt);
}

/*
  990607 new MacAnovaCanvas member functions
	void OnEvent(wxMouseEvent& event);
  private:
	void drawCrossHairs(wxIntPoint aPoint);
	void drawBox(wxIntPoint startPoint, wxIntPoint otherPoint);
	void drawLineSegment(wxIntPoint startPoint, wxIntPoint otherPoint);
	int readCrossHairs(long  ix [], long iy [], int npoints);
	int readBox(long ix [], long iy []);
	int readLine(long ix [], long iy [], int nlines);
*/

static int whichGraphWindow(MacAnovaCanvas *canvas)
{
	int              i;

	for (i = 0 ;i < NUMCANVASFRAMES;i++)
	{
    	MacAnovaCanvasFrame      *cf = BaseFrame->CanvasFrames[i];

		if (cf != (MacAnovaCanvasFrame *) 0 && cf->canvas == canvas)
		{
			break;
		}
	} /*for (i = 0 ;i < NUMCANVASFRAMES;i++)*/
    return ((i == NUMCANVASFRAMES) ? -1 : i);
} /*whichGraphWindow()*/

#if (0)
#undef mapScreenToX
#undef mapScreenToY
#define mapScreenToX(X,XMAX) ((double) X)
#define mapScreenToY(Y,YMAX) ((double) Y)
#endif /*0*/

#ifdef wx_msw
#define CATCHANDRELEASE
#else
#undef CATCHANDREALEASE
#endif /*wx_msw*/

int MacAnovaBaseFrame::ReadMouseEventHook(MacAnovaCanvas * thisCanvas)
{
    wxMouseEvent     *theEvent = thisCanvas->MouseEvent;
    WXTYPE            type = theEvent->eventType;
    mouseInputInfo   *inputFromMouse = &BaseFrame->InputFromMouse;
	WHERE("ReadMouseEventHook");

 	if (!ReadingFromMouse && theEvent->ButtonDown())
    {
		int               n = InputFromMouse.n;
    	int               reply;
    	int               xmax, ymax;
    	long              ix[MaxMousePoints+1], iy[MaxMousePoints+1];
    	int               thisWindow = whichGraphWindow(thisCanvas);

 		ReadingFromMouse = TRUE;
#ifdef CATCHANDRELEASE
        thisCanvas->CaptureMouse();
#endif /*CATCHANDRELEASE*/
        thisCanvas->Dirty = 0;
        switch (InputFromMouse.op)
        {
          case getPoint:
			reply = thisCanvas->readCrossHairs(ix, iy, n);
            break;

		  case getLine:
			reply = thisCanvas->readLine(ix, iy, n);
          	break;

          case getBox:
			reply = thisCanvas->readBox(ix, iy);
			break;
        } /*switch (InputFromMouse.op)*/

#ifdef CATCHANDRELEASE
        thisCanvas->ReleaseMouse();
#endif /*CATCHANDRELEASE*/
        if (reply != MouseNormalReturn)
   		{
        	if (reply != MouseRestart)
            {
            	InputFromMouse.windno = reply;
            }
            ReadingFromMouse = FALSE;
            goto eventProcessed;
        } /*if (reply != MouseNormalReturn)*/
		InputFromMouse.windno = thisWindow;
        GraphMap = GraphMapArray + thisWindow;
        xmax = GraphMap->screenxmax;
        ymax = GraphMap->screenymax;
#if (0)
		PRINT("thisWindow = %d, GraphMap->logx = %d, GraphMap->logy = %d", thisWindow,GraphMap->logx,GraphMap->logy,0);
#endif /*0*/
        if (InputFromMouse.op != getBox)
        {
		 	int      i;

			if (InputFromMouse.op == getLine)
            {
            	n++;
            }

            for (i = 0; i < n; i++)
            {
            	InputFromMouse.x[i] = mapScreenToX(ix[i], xmax);
                InputFromMouse.y[i] = mapScreenToY(iy[i], ymax);
            }
            InputFromMouse.n = n;
        } /*if (InputFromMouse.op != getBox)*/
        else
        {
          /*
        	Return x, y coordinates of the four corners of
        	the box, with the first corner repeated at the end
        	to complete the rectangle
          */
        	InputFromMouse.n = 5;
            /* x on left side */
            InputFromMouse.x[0] = InputFromMouse.x[1] =
            InputFromMouse.x[4] = mapScreenToX(ix[0], xmax);
            /* x on right side */
            InputFromMouse.x[2] = InputFromMouse.x[3] =
            	mapScreenToX(ix[1], xmax);
            /* y on top */
            InputFromMouse.y[0] = InputFromMouse.y[3] =
            InputFromMouse.y[4] = mapScreenToY(iy[0], ymax);
            /* y on bottom */
            InputFromMouse.y[1] = InputFromMouse.y[2] =
            	mapScreenToY(iy[1], ymax);
		}
        InputFromMouse.op = noMouseOp;
        ReadingFromMouse = FALSE;
    } /*if (!ReadingFromMouse)*/
    else
    {
    	goto passBackEvent;
    }

    /* Fall through */

  eventProcessed:
	/* fall through */
  ignoreEvent:
	return (1);

  passBackEvent:
	return (0);
} /*ReadMouseEventHook()*/


#if (0)
/* For Debugging */
void eventAlert(char * whatevent, wxMouseEvent  *event)
{
    int           ix, iy;
    WHERE("eventAlert");
    wxGetMousePosition(&ix, &iy);
	ALERT1(128, "case %s, ix = %d, iy = %d, op = %d",
    	whatevent, ix, iy, BaseFrame->InputFromMouse.op);
}
#else /*0*/
#define eventAlert(TYPE, EVENT)
#endif /*0*/

/*
  Function to receive Mouse events associated with Graph Window
  If no ReadMouse operation is in progress, it passes it
  off the ReadMouseEventHook()
*/
void MacAnovaCanvas::OnEvent(wxMouseEvent& event)
{
	mouseInputInfo      *inputFromMouse = &BaseFrame->InputFromMouse;

	float         x = event.x;
	float         y = event.y;
	WXTYPE        type = event.eventType;
    int           ix, iy;
    int           xmax, ymax;
	int           useAlert = 0;
	WHERE("MacAnovaCanvas::OnEvent");

    wxGetMousePosition(&ix, &iy);
	GetClientSize(&xmax, &ymax);
    MouseEvent->eventHandle = event.eventHandle;
    MouseEvent->eventObject = event.eventObject;
    MouseEvent->eventType = event.eventType;
    MouseEvent->controlDown = event.controlDown;
    MouseEvent->leftDown = event.leftDown;
    MouseEvent->middleDown = event.middleDown;
    MouseEvent->rightDown = event.rightDown;
    MouseEvent->shiftDown = event.shiftDown;
    MouseEvent->x = event.x;
    MouseEvent->y = event.y;

    if (inputFromMouse->op == noMouseOp ||
    	!BaseFrame->ReadMouseEventHook(this))
    {
		switch (type)
		{
		  case wxEVENT_TYPE_ENTER_WINDOW:
			eventAlert("wxEVENT_TYPE_ENTER_WINDOW", MouseEvent);
			break;

		  case wxEVENT_TYPE_LEAVE_WINDOW:
			eventAlert("wxEVENT_TYPE_LEAVE_WINDOW", MouseEvent);
			break;

		  case wxEVENT_TYPE_MOTION:
			break;

		  case wxEVENT_TYPE_LEFT_DOWN:
			eventAlert("wxEVENT_TYPE_LEFT_DOWN", MouseEvent);
			break;

		  case wxEVENT_TYPE_LEFT_UP:
			eventAlert("wxEVENT_TYPE_LEFT_UP", MouseEvent);
			break;

		  case wxEVENT_TYPE_LEFT_DCLICK:
			eventAlert("wxEVENT_TYPE_LEFT_DCLICK", MouseEvent);
			break;

		  case wxEVENT_TYPE_MIDDLE_DOWN:
			eventAlert("wxEVENT_TYPE_MIDDLE_DOWN", MouseEvent);
			break;

		  case wxEVENT_TYPE_MIDDLE_UP:
			eventAlert("wxEVENT_TYPE_MIDDLE_UP", MouseEvent);
			break;

		  case wxEVENT_TYPE_MIDDLE_DCLICK:
			eventAlert("wxEVENT_TYPE_MIDDLE_DCLICK", MouseEvent);
			break;

		  case wxEVENT_TYPE_RIGHT_DOWN:
			eventAlert("wxEVENT_TYPE_RIGHT_DOWN", MouseEvent);
			break;

		  case wxEVENT_TYPE_RIGHT_UP:
			eventAlert("wxEVENT_TYPE_RIGHT_UP", MouseEvent);
			break;

		  case wxEVENT_TYPE_RIGHT_DCLICK:
			eventAlert("wxEVENT_TYPE_RIGHT_DCLICK", MouseEvent);
			break;

		  default:
       		eventAlert("default", MouseEvent);
			break;
		} /*switch (type)*/
    } /*if (inputFromMouse->op == noMouseOp || !ReadMouseEventHook(MouseEvent))*/
} /*MacAnovaCanvas::OnEvent()*/

#define CHECKSHIFTED
#define isShifted() (MouseEvent->shiftDown != 0)

#define OUTOFRECTRESTART

static void getOrigDims(MacAnovaCanvas *canvas, int * xmax, int * ymax)
{
    int                      whichWindow = whichGraphWindow(canvas);

	*xmax = GraphMapArray[whichWindow].screenxmax;
    *ymax = GraphMapArray[whichWindow].screenymax;
} /*getOrigDims()*/

#define CHECKMOUSEKEYS
int MacAnovaCanvas::mouseCheckKeys(void)
{
	long          ch = KeyEvent->keyCode;
	int           shifted = isShifted();
	int           reply = MouseNormalReturn;
	
	if (KeyEvent->controlDown && ch == interruptKey)
	{
		reply = MouseInterrupt;
	}
	else if (!KeyEvent->controlDown && ch == abortKey)
	{
		reply = MouseAbort;
	}
	else if (!KeyEvent->controlDown && ch == restartKey)
	{
		reply = MouseRestart;
	}

	if (reply != MouseNormalReturn)
	{
		KeyEvent->keyCode = 0;
	}
	
	return (reply);
} /*mouseCheckKeys()*/

void MacAnovaCanvas::drawCrossHairs(wxIntPoint aPoint)
{
	int                xmax, ymax;

    getOrigDims(this, &xmax, &ymax);
	SetLogicalFunction(wxXOR);
	IntDrawLine(0, aPoint.y, xmax, aPoint.y);
	IntDrawLine(aPoint.x, 0, aPoint.x, ymax);
	SetLogicalFunction(wxCOPY);
} /*MacAnovaCanvas::drawCrossHairs()*/

void MacAnovaCanvas::drawLineSegment(wxIntPoint startPoint,
									 wxIntPoint otherPoint)
{
	SetLogicalFunction(wxXOR);
	IntDrawLine(startPoint.x, startPoint.y, otherPoint.x, otherPoint.y);
	SetLogicalFunction(wxCOPY);
} /*MacAnovaCanvas::drawLineSegment()*/

void MacAnovaCanvas::drawBox(wxIntPoint startPoint, wxIntPoint otherPoint)
{
	wxIntPoint        points[5];
	WHERE("drawBox");

	SetLogicalFunction(wxXOR);
	points[0].x = points[1].x = points[4].x = startPoint.x;
    points[2].x = points[3].x = otherPoint.x;
	points[0].y = points[3].y = points[4].y = startPoint.y;
    points[1].y = points[2].y = otherPoint.y;

	int        i;

	for (i = 0; i < 4; i++)
	{
		IntDrawLine(points[i].x, points[i].y, points[i+1].x, points[i+1].y);
	}
	SetLogicalFunction(wxCOPY);
} /*MacAnovaCanvas::drawBox()*/

#define HEPS   3
#define VEPS   3
void MacAnovaCanvas::markPoint(wxIntPoint here)
{
	SetLogicalFunction(wxXOR);
	IntDrawLine(here.x, here.y - VEPS, here.x, here.y + VEPS);
 	IntDrawLine(here.x-HEPS, here.y, here.x + HEPS, here.y);
	SetLogicalFunction(wxCOPY);
} /*MacAnovaCanvas::markPoint()*/

int MacAnovaCanvas::inWindow(void)
{
	int                xmax, ymax;

    getOrigDims(this, &xmax, &ymax);
    return (MouseEvent->x >= 0 && MouseEvent->x <= xmax &&
            MouseEvent->y >= 0 && MouseEvent->y <= ymax);
} /*MacAnovaCanvas::inWindow()*/

int MacAnovaCanvas::Button(void)
{
    wxYield();
	return (MouseEvent->ButtonDown());
} /*MacAnovaCanvas::Button()*/

int MacAnovaCanvas::readCrossHairs(long ix [], long iy [], int npoints)
{
	wxIntPoint      lastPoint, thisPoint;
	int             reply = MouseNormalReturn;
	int             ipoint;
	WHERE("readCrossHairs");

//	r = ((WindowPtr) thePort)->portRect;


	while (Button())
	{
		;
	}

	if (INTERRUPT != INTNOTSET)
	{
		return (MouseRestart);
	}
	
	for (ipoint = 0; ipoint < npoints; ipoint++)
	{
		lastPoint.x = lastPoint.y = -32000;
		
		while (!Button())
		{
			if (INTERRUPT != INTNOTSET)
			{
				reply = MouseRestart;
				break;
			}

#ifdef CHECKMOUSEKEYS
			reply = mouseCheckKeys();
			if (reply != MouseNormalReturn)
			{
				break;
			}
#endif /*CHECKMOUSEKEYS*/
			thisPoint.x = (int) MouseEvent->x;
			thisPoint.y = (int) MouseEvent->y;

			if (thisPoint.x != lastPoint.x ||thisPoint.y != lastPoint.y)
			{
				if (Dirty)
				{
					drawCrossHairs(lastPoint);
					Dirty = 0;
				}
#ifdef OUTOFRECTRESTART
	        	if (!inWindow() ||
	        		MouseEvent->eventType == wxEVENT_TYPE_LEAVE_WINDOW ||
	            	MouseEvent->eventType == wxEVENT_TYPE_ENTER_WINDOW)
	        	{
	        		reply = MouseRestart;
	            	break;
	        	}
#endif /*OUTOFRECTRESTART*/
				drawCrossHairs(thisPoint);
				lastPoint.x = thisPoint.x;
				lastPoint.y = thisPoint.y;
				Dirty = 1;
			}
		} /*while (!Button())*/
		if (Dirty)
		{
			drawCrossHairs(thisPoint);
            Dirty = 0;
		}
		while (Button())
		{
            ;
		}
		ix[ipoint] = (long) thisPoint.x;
		iy[ipoint] = (long) thisPoint.y;
		if (reply != MouseNormalReturn)
		{
			npoints = ipoint;
			break;
		}
		markPoint(thisPoint);
		ix[ipoint] = (long) thisPoint.x;
		iy[ipoint] = (long) thisPoint.y;
	} /*for (ipoint = 0; ipoint < npoints; ipoint++)*/

	/* unmark the marked points */

	int                xmax, ymax;

    getOrigDims(this, &xmax, &ymax);
	for (ipoint = 0; ipoint < npoints; ipoint++)
	{
		lastPoint.x = ix[ipoint];
		lastPoint.y = iy[ipoint];
		markPoint(lastPoint);
        iy[ipoint] = ymax - iy[ipoint];
	}
	return (reply);                             
} /*readCrossHairs*/

int MacAnovaCanvas::readBox(long ix [], long iy [])
{

#if (1)
	wxIntPoint      startPoint, lastPoint, aPoint;
	int             reply = MouseNormalReturn;


	while (Button())
	{
		;
	}

	if (INTERRUPT != INTNOTSET)
	{
		return (MouseRestart);
	}
	
	Dirty = 0;
    startPoint.x = (int) MouseEvent->x;
    startPoint.y = (int) MouseEvent->y;
	lastPoint.x = lastPoint.y = -32000;

	while (!Button())
	{
		wxIntPoint        thisPoint;

        if (INTERRUPT != INTNOTSET)
        {
        	reply = MouseRestart;
        	break;
        }

#ifdef CHECKMOUSEKEYS
		reply = mouseCheckKeys();
		if (reply != MouseNormalReturn)
        {
        	break;
        }
#endif /*CHECKMOUSEKEYS*/

		thisPoint.x = (int) MouseEvent->x;
        thisPoint.y = (int) MouseEvent->y;

        if (thisPoint.x != lastPoint.x || thisPoint.y != lastPoint.y)
		{
#ifdef OUTOFRECTRESTART
			if (!inWindow() ||
            	MouseEvent->eventType == wxEVENT_TYPE_LEAVE_WINDOW ||
                MouseEvent->eventType == wxEVENT_TYPE_ENTER_WINDOW)
            {
            	reply = MouseRestart;
                break;
            }
#endif /*OUTOFRECTRESTART*/
			if (Dirty)
			{
				/* Erase current box */
				drawBox(startPoint, aPoint);
				Dirty = 0;
			}	
			aPoint = thisPoint;
#ifdef CHECKSHIFTED
			if (isShifted())
			{
				int      x = aPoint.x - startPoint.x;
				int      y = aPoint.y - startPoint.y;

				if (abs(x) > abs(y))
				{
					aPoint.x = startPoint.x + ((x > 0) ? abs(y) : -abs(y));
				}
				else
				{
					aPoint.y = startPoint.y + ((y > 0) ? abs(x) : -abs(x));
				}
			}
#endif /*CHECKSHIFTED*/
			drawBox(startPoint,aPoint);
			lastPoint = thisPoint;
			Dirty = 1;
		} /*if (thisPoint.h != lastPoint.h || thisPoint.v != lastPoint.v)*/
	} /*while (!Button())*/
	if (Dirty)
	{
		drawBox(startPoint, aPoint);
	}
	while (Button())
	{
		;
	}
	if (INTERRUPT != INTNOTSET)
    {
		reply = MouseRestart;
	}

	int                xmax, ymax;

    getOrigDims(this, &xmax, &ymax);
	iy[0] = (long) (ymax - startPoint.y);
	ix[0] = (long) startPoint.x;
	iy[1] = (long) (ymax - aPoint.y);
	ix[1] = (long) aPoint.x;

	return (reply);
#else
	putOutErrorMsg("ERROR: readBox() not implemented");
	return (MouseAbort);
#endif
} /*MacAnovaCanvas::readBox()*/

int MacAnovaCanvas::readLine(long ix [], long iy [], int nlines)
{
#if (1)
	wxIntPoint      startPoint, lastPoint, thisPoint, aPoint;
	int             reply = MouseNormalReturn;
	int             iline;
	WHERE("readLine");

	while (Button())
	{
		;
	}

	if (INTERRUPT != INTNOTSET)
	{
		return (MouseRestart);
	}
	
    startPoint.x = (int) MouseEvent->x;
    startPoint.y = (int) MouseEvent->y;
	lastPoint.x = lastPoint.y = -32000;

	ix[0] = (long) startPoint.x;
	iy[0] = (long) startPoint.y;
	for (iline = 0; iline < nlines; iline++)
	{
		Dirty = 0;

		/* Now track until Mouse again clicked */
		while (!Button())
		{
			if (INTERRUPT != INTNOTSET)
			{
				if (Dirty)
				{
					iline++;
				}
				reply = MouseRestart;
                break;
			}

#ifdef CHECKMOUSEKEYS
			reply = mouseCheckKeys();
			if (reply != MouseNormalReturn)
			{
				if (Dirty)
				{
					iline++;
				}
				break;
			}
#endif /*CHECKMOUSEKEYS*/
    		thisPoint.x = (int) MouseEvent->x;
    		thisPoint.y = (int) MouseEvent->y;
			if (thisPoint.x != lastPoint.x ||thisPoint.y != lastPoint.y)
			{
				if (Dirty)
				{
					drawLineSegment(startPoint,aPoint);
					Dirty = 0;
				}
                aPoint.x = thisPoint.x;
                aPoint.y = thisPoint.y;
#ifdef CHECKSHIFTED
				if (isShifted())
				{
					int      x = aPoint.x - startPoint.x;
					int      y = aPoint.y - startPoint.y;
	
					if (abs(x) > abs(y))
					{
						aPoint.y = startPoint.y;
					}
					else
					{
						aPoint.x = startPoint.x;
					}
				} /*if (isShifted())*/
#endif /*CHECKSHIFTED*/
				drawLineSegment(startPoint, aPoint);
				ix[iline + 1] = (long) aPoint.x;
				iy[iline + 1] = (long) aPoint.y;
				lastPoint = thisPoint;
				Dirty = 1;
#ifdef OUTOFRECTRESTART
	        	if (!inWindow() ||
	        		MouseEvent->eventType == wxEVENT_TYPE_LEAVE_WINDOW ||
	            	MouseEvent->eventType == wxEVENT_TYPE_ENTER_WINDOW)
	        	{
	        		reply = MouseRestart;
	            	iline++;
	            	break;
	        	}
#endif /*OUTOFRECTRESTART*/
			} /*if (thisPoint.x != lastPoint.x ||thisPoint.y != lastPoint.y)*/
		} /*while (!Button())*/
		while (Button())
		{
			;
		}

		if (reply != MouseNormalReturn)
		{
			break;
		}
		startPoint.x = aPoint.x;
		startPoint.y = aPoint.y;
	} /*for (iline = 0; iline < nlines; iline++)*/

	nlines = iline;

	/*
		Erase segments and reverse y coordinate
	*/
	int                xmax, ymax;

    getOrigDims(this, &xmax, &ymax);
	thisPoint.x = ix[0];
	thisPoint.y = iy[0];
    iy[0] = ymax - iy[0];
	for (iline = 1; iline <= nlines; iline++)
	{
		startPoint.x = thisPoint.x;
		startPoint.y = thisPoint.y;
		thisPoint.x = ix[iline];
		thisPoint.y = iy[iline];
        iy[iline] = ymax - iy[iline];
		drawLineSegment(startPoint, thisPoint);
	} /*for (iline = 1; iline <= nlines; i++)*/
	return (reply);
#else
	putOutErrorMsg("ERROR: readLine() not implemented");
	return (MouseAbort);
#endif
} /*MacAnovaCanvas::readLine()*/

#if (0) /*Macintosh versions of readBox() and readLine() */
static int readBox(long ix [], long iy [])
{
	Point           startPoint, lastPoint, aPoint;
	PenState        pnState;
	int             dirty = 0;
	int             reply = MouseNormalReturn;
	GrafPtr         thePort;
	Rect            r;

	GetPort(&thePort);
	r = ((WindowPtr) thePort)->portRect;

	GetPenState(&pnState);
	GetMouse(&startPoint);
	lastPoint.h = lastPoint.v = -32000;

	while (StillDown())
	{
		/* wait for release */
		;
	}

	while (!Button())
	{
		Point        thisPoint;

		reply = mouseCheckKeys();
		if (reply != MouseNormalReturn)
		{
			break;
		}

		GetMouse(&thisPoint);
		if (thisPoint.h != lastPoint.h || thisPoint.v != lastPoint.v)
		{
#ifdef OUTOFRECTRESTART
			if (!PtInRect(thisPoint, &r))
			{
				reply = MouseRestart;
				break;
			}
#endif /*OUTOFRECTRESTART*/
			if (dirty)
			{
				/* Erase current box */
				drawBox(startPoint, aPoint);
				dirty = 0;
			}	
			aPoint = thisPoint;

			if (isShifted())
			{
				int      h = aPoint.h - startPoint.h;
				int      v = aPoint.v - startPoint.v;

				if (abs(h) > abs(v))
				{
					aPoint.h = startPoint.h + ((h > 0) ? abs(v) : -abs(v));
				}
				else
				{
					aPoint.v = startPoint.v + ((v > 0) ? abs(h) : -abs(h));
				}
			}
			drawBox(startPoint,aPoint);
			lastPoint = thisPoint;
			dirty = 1;
		} /*if (thisPoint.h != lastPoint.h || thisPoint.v != lastPoint.v)*/
	} /*while (!Button())*/
	if (dirty)
	{
		drawBox(startPoint, aPoint);
	}
	while (Button())
	{
		;
	}
	SetPenState(&pnState);

	iy[0] = (long) (r.bottom - startPoint.v);
	ix[0] = (long) startPoint.h;
	iy[1] = (long) (r.bottom - aPoint.v);
	ix[1] = (long) aPoint.h;

	return (reply);

} /*readBox()*/

static int readLine(long ix [], long iy [], int nlines)
{
	Point           startPoint, lastPoint, thisPoint, aPoint;
	PenState        pnState;
	int             dirty;
	int             reply = MouseNormalReturn;
	int             iline;
	GrafPtr         thePort;
	Rect            r;
	WHERE("readLine");

	GetPort(&thePort);
	r = ((WindowPtr) thePort)->portRect;

	GetPenState(&pnState);
	lastPoint.h = lastPoint.v = -32000;

	while (StillDown())
	{
		/* wait for release */
		;
	}

//	HideCursor();

	GetMouse(&startPoint);
	ix[0] = (long) startPoint.h;
	iy[0] = (long) startPoint.v;
	for (iline = 0; iline < nlines; iline++)
	{
		dirty = 0;

		/* Now track until Mouse again clicked */
		while (!Button())
		{
			reply = mouseCheckKeys();
			if (reply != MouseNormalReturn)
			{
				if (dirty)
				{
					iline++;
				}
				break;
			}
			GetMouse(&thisPoint);
			if (thisPoint.h != lastPoint.h ||thisPoint.v != lastPoint.v)
			{
				if (dirty)
				{
					drawLineSegment(startPoint,aPoint);
					dirty = 0;
				}	
				aPoint = thisPoint;
#ifdef OUTOFRECTRESTART
				if (!PtInRect(thisPoint, &r))
				{
					reply = MouseRestart;
					break;
				}
#endif /*OUTOFRECTRESTART*/
				if (isShifted())
				{
					int      h = aPoint.h - startPoint.h;
					int      v = aPoint.v - startPoint.v;
	
					if (abs(h) > abs(v))
					{
						aPoint.v = startPoint.v;
					}
					else
					{
						aPoint.h = startPoint.h;
					}
				} /*if (isShifted())*/
				drawLineSegment(startPoint, aPoint);
				ix[iline + 1] = (long) aPoint.h;
				iy[iline + 1] = (long) aPoint.v;
				lastPoint = thisPoint;
				dirty = 1;
			} /*if (thisPoint.h != lastPoint.h ||thisPoint.v != lastPoint.v)*/
		} /*while (!Button())*/
		while (Button())
		{
			;
		}
		if (reply != MouseNormalReturn)
		{
			break;
		}
		startPoint = aPoint;
	} /*for (iline = 0; iline < nlines; iline++)*/

	nlines = iline;

	/*
		Erase segments and reverse y coordinate
	*/
	thisPoint.h = ix[0];
	thisPoint.v = iy[0];
	iy[0] = r.bottom - iy[0];
	for (iline = 1; iline <= nlines; iline++)
	{
		startPoint = thisPoint;
		thisPoint.h = ix[iline];
		thisPoint.v = iy[iline];
		drawLineSegment(startPoint, thisPoint);
		iy[iline] = r.bottom - iy[iline];
	} /*for (iline = 1; iline <= nlines; i++)*/
	SetPenState(&pnState);

//	ShowCursor();

	return (reply);
} /*readLine()*/
#endif /*0*/

int MacAnovaBaseFrame::readFromMouse(double x [], double y [], int op,
									 long * n)
{
	int                    i;
    wxCanvasDC            *dc;
	WHERE("readFromMouse");

	for (i = 0 ;i < NUMCANVASFRAMES;i++)
	{
		if (CanvasFrames[i] != (MacAnovaCanvasFrame *) 0)
		{
			break;
		}
	} /*for (i = 0 ;i < NUMCANVASFRAMES;i++)*/
	if (i == NUMCANVASFRAMES)
	{
		sprintf(OUTSTR,	"ERROR: No graph windows are open");
		return (MouseNoGraphs);
	}
    dc = CanvasFrames[i]->canvasDC;

    int    xmax, ymax;
    CanvasFrames[i]->canvas->GetClientSize(&xmax,&ymax);

	InputFromMouse.op = op;
	InputFromMouse.windno = 0;
	InputFromMouse.x = x;
	InputFromMouse.y = y;
	InputFromMouse.n = *n;

	while (InputFromMouse.op != noMouseOp && InputFromMouse.windno >= 0)
	{
		wxYield();
		if (interrupted(DEFAULTTICKS) != INTNOTSET)
		{
			InputFromMouse.windno = MouseInterrupt;
			break;
		}
	} /*while (InputFromMouse.op != noMouseOp && InputFromMouse.windno >= 0)*/
	
	*n = InputFromMouse.n;
	InputFromMouse.op = noMouseOp;

	return (InputFromMouse.windno);
} /*MacAnovaBaseFrame::readFromMouse()*/

#if defined(wx_msw) || defined(wx_motif)
#define READMOUSE
#endif /*wx_msw*/

extern "C" {
int readFromMouse(double x [], double y [], int op, long * n)
{
#ifndef READMOUSE
	return (0);
#else /*READMOUSE*/
	return (BaseFrame->readFromMouse(x, y, op, n));
#endif /*READMOUSE*/
} /*readFromMouse()*/

#if (0) /*Macintosh Version*/
int readFromMouse(double x [], double y [], int op, long * n)
{
	SkelEventHookProcPtr   oldHook = SkelGetEventHook();
	WindowPtr              front = (WindowPtr) -1;
	int                    i;
	WHERE("readFromMouse");

	for (i = 0; i < NGRAPHS; i++)
	{
		if (GraphWind[i] != (WindowPtr) 0)
		{
			break;
		}
	} /*for (i = 0; i < NGRAPHS; i++)*/

	if (i == NGRAPHS)
	{
		return (MouseNoGraphs);
	}

	InputFromMouse.op = op;
	InputFromMouse.windno = 0;
	InputFromMouse.x = x;
	InputFromMouse.y = y;
	InputFromMouse.n = *n;

	while (InputFromMouse.op != noMouseOp && InputFromMouse.windno >= 0)
	{
		WindowPtr   thisWindow = FrontWindow();
	
		/*
		  Don't allow a window to be closed
		*/	
		DisableItem(WindowMenu, closeit);

		/* Prevent pasting to command window */
		DisableItem(EditMenu, pastit);
		if (thisWindow != front)
		{
			int         whichGraph;
			
			front = thisWindow;
			whichGraph = whichGraphWindow(thisWindow);
			if (whichGraph >= 0 && whichGraph < NGRAPHS + NPANELS)
			{
				SetCursor(&CROSS);
			}
		} /*if (thisWindow != front)*/
		SkelSetEventHook((SkelEventHookProcPtr) ReadMouseEventHook);
		/*
		  Indirect call to SkelEventLoop() which
		  routes all events to ReadMouseEventHook()
		*/
		SkelDoEvents(everyEvent ^ keyUpMask);
		SkelSetEventHook((SkelEventHookProcPtr) oldHook);
		if (interrupted(DEFAULTTICKS) != INTNOTSET)
		{
			InputFromMouse.windno = MouseInterrupt;
			break;
		}
	} /*while (InputFromMouse.op != noMouseOp && InputFromMouse.windno >= 0)*/
	InitCursor();
	EnableItem(WindowMenu, closeit);

	*n = InputFromMouse.n;

	return (InputFromMouse.windno);
	
} /*readFromMouse()*/
#endif /*0*/
} /* extern "C" */

#ifdef wx_msw
extern "C" {
extern char      **ArgV; /*defined and set equal to argv in main()*/
static char        Path[PATHSIZE+1];
/* instead of string.h */
/*
  981002 Fixed bug so that it correctly handles path names containing
         spaces
*/
char *get_dataPath(void)
{
	char    *pathp;
	int      quoted = 0, i;
	char     c;

	pathp = GetCommandLine();

	if (pathp[0] == '"')
	{
		pathp++;
		quoted = 1;
	}
	for (i = 0; (c = pathp[i]) != '\0' && i < PATHSIZE ; i++)
	{
		if (quoted && c == '"' || !quoted && c == ' ')
		{
			break;
		}
		Path[i] = c;
	} /*for (i = 0; (c = pathp[i]) != '\0' && i < PATHSIZE ; i++)*/
	Path[i] = '\0';
	if (i == PATHSIZE)
	{
		putOutErrorMsg("ERROR: Path to MacAnova.exe too long");
		Path[0] = '\0';
		return(Path);
	}

	if( (pathp = strrchr(Path,'\\')) == (char *) 0 )
	{
		pathp = strrchr(Path,'/');
	}
	if(pathp)
	{
		*(++pathp) = 0x00;
	}
	else
	{
		Path[0] = '\0';
	}

	return(Path);
} /*get_dataPath()*/

}						/* extern "C" */
#endif /* wx_msw */

static char FileNameFromFileSelector[FULLPATHSIZE+1];

extern "C" {

/*
  Increment global NLINES for every newline in msg
*/

void incrementNLINES(char *msg)
{
	WHERE("incrementNLINES");
	
	while (*msg != '\0')
	{
		if (*msg++ == '\n')
		{
			NLINES++;
		}
	}
} /*incrementNLINES()*/

#ifdef WXWINMSW
/*  '/' equivalent to '\' on MSDOS*/
#define isSlash(C) ((C) == DIRSEPARATOR[0] || (C) == '/')
#define isPath(N) (strchr(N, '/') || strchr(N, '\\') || strchr(N, ':'))
#else							/*WXWINMSW*/
#define isSlash(C) ((C) == DIRSEPARATOR[0])
#define isPath(N) strchr(N, DIRSEPARATOR[0])
#endif							/*WXWINMSW*/

/*
   Open file in appropriate mode; may do things differently on different
   platforms;

   990713 Path names starting with '~' from DATAPATHS and DATAPATH
          are now expanded

   990723 Fixed bug arising when file name had already been expanded so
          it really pointed to the static buffer used by expandFilename()
          which was overwritten by expandFilename() here

   010322 Somewhat reorganized things; global LastPathOpened is now
          set after a successful open

   010605 tryFile created with code from fmyopen(); when fileName is ""
          if nothing else works, look in lastVol directory.  Now the Macintosh
		  and WxWin versions of fmyopen() are identical, but differ from
          Unix/DOS

*/

static FILE * tryFile(char lastPath [], char * fileName, char * mode)
{
	int             pathLength = strlen(lastPath);
	int             nameLength = strlen(fileName);
	FILE           *trialFile = (FILE *) 0;

	if (pathLength > 0 && !isSlash(lastPath[pathLength-1]))
	{
		lastPath[pathLength++] = DIRSEPARATOR[0];
		lastPath[pathLength] = '\0';
	}
	if (pathLength + nameLength <= PATHSIZE)
	{
		strcpy(lastPath + pathLength, fileName);
		if (okfilename(lastPath))
		{
			trialFile = fopen(lastPath,mode);
		}
	} /*if (okpathname(lastPath) && pathLength+nameLength <= PATHSIZE)*/
	return (trialFile);
} /*tryFile()*/

FILE * fmyopen(char * name, char * mode)
{
	Symbolhandle   dataPaths = (Symbolhandle) 0;
	FILE           *trialFile;
	long            ipath, npaths, pathLength, nameLength;
	char           *pathName;
	char            lastPath[PATHSIZE + 1];
	char            fileName[PATHSIZE + 1];

	strcpy(fileName, name);
	trialFile = fopen(fileName, mode);
	if (trialFile != (FILE *) 0)
	{
#ifdef MACINTOSH
		if (!isPath(fileName) && getMacPathName(lastPath) == noErr &&
			strlen(lastPath) + strlen(fileName) <= PATHSIZE)
		{
			strcat(lastPath, fileName);
		}
		else
#endif /*MACINTOSH*/
		{
			strcpy(lastPath, fileName);
		}
	}
	else if (!strchr(mode,'w') && !isPath(fileName))
	{	
		if (((dataPaths = Lookup("DATAPATHS")) != (Symbolhandle) 0 ||
			 (dataPaths = Lookup("DATAPATH")) != (Symbolhandle) 0) &&
			TYPE(dataPaths) == CHAR)
		{
			char        *pathName1;

			pathName = STRINGPTR(dataPaths);
			npaths = symbolSize(dataPaths);
			nameLength = strlen(fileName);

			for (ipath = 0; ipath < npaths;
				 ipath++, pathName = skipStrings(pathName, 1))
			{
				pathName1 = expandFilename(pathName);
			
				if (pathName1 != (char *) 0)
				{
					pathLength = strlen(pathName1);
					if (okpathname(pathName1) &&
						pathLength + nameLength <= PATHSIZE)
					{
						strcpy(lastPath, pathName1);
						/* Kludge: Fixup expandFilename buffer if needed */
						if (pathName1 == name)
						{
							strcpy(name, fileName);
						}
						trialFile = tryFile(lastPath, fileName, mode);
						if (trialFile != (FILE *) 0)
						{
							break;
						}
					} /*if (okpathname(pathName1) && pathLength+nameLength <= PATHSIZE)*/
					lastPath[0] = '\0';
				} /*if (pathName1 != (char *) 0)*/
			} /*for (ipath = 0; ipath < npaths; ...)*/
			if (trialFile == (FILE *) 0 && isMvDirInfoSet(lastVol))
			{
				char         *lastDirectory =
				  getMvDirDirectory(getMvDirInfo(lastVol));
				
				pathLength = strlen(lastDirectory);
				if (pathLength + nameLength <= PATHSIZE)
				{
					strcpy(lastPath, lastDirectory);
					trialFile = tryFile(lastPath, fileName, mode);
				}
			} /*if (trialFile == (FILE *) 0 && isMvDirInfoSet(lastVol))*/
		}
	}
	
	if (trialFile != (FILE *) 0)
	{
		strcpy(LastPathOpened, lastPath);
	}
	
	return (trialFile);
} /*fmyopen()*/

#ifdef wx_msw
/*
  This is the same as wxFileNameFromPath() in src/base/wb_utils.cpp
  Things crashed when this was used, but not with this replacement
*/

#define wxFileNameFromPath  myWxFileNameFromPath
#define wxPathOnly          myWxPathOnly
#define _MAXPATHLEN         500 /*matches define in wb_utils.cpp*/

// Return just the filename, not the path
// (basename)

static char *
wxFileNameFromPath (char *path)
{
  if (path)
    {
      register char *tcp;

      tcp = path + strlen (path);
      while (--tcp >= path)
	{
	  if (*tcp == '/' || *tcp == '\\'
#ifdef VMS
     || *tcp == ':' || *tcp == ']')
#else
     )
#endif
	    return tcp + 1;
	}			/* while */
#ifdef wx_msw
      if (isalpha (*path) && *(path + 1) == ':')
	return path + 2;
#endif
    }
  return path;
}

/*
  The following duplicates wxPathOnly() in src/base/wb_utils.cpp
  For reasons that are not understood, referencing the compiled
  version in the Wx library caused a crash that doesn't happen with
  this version.
*/
// Return just the directory, or NULL if no directory
static char *
wxPathOnly (char *path)
{
  if (path && *path)
    {
      static char buf[_MAXPATHLEN];

      // Local copy
      strcpy (buf, path);

      int l = strlen(path);
      Bool done = FALSE;

      int i = l - 1;

      // Search backward for a backward or forward slash
      while (!done && i > -1)
      {
        // ] is for VMS
        if (path[i] == '/' || path[i] == '\\' || path[i] == ']')
        {
          done = TRUE;
#ifdef VMS
          buf[i+1] = 0;
#else
          buf[i] = 0;
#endif

          return buf;
        }
        else i --;
      }

/* there's a bug here somewhere, so replaced with my original code.
      char *tcp;
      // scan back
      for (tcp = &buf[strlen (buf) - 1]; tcp >= buf; tcp--)
	{
	  // Search for Unix or Dos path sep {'\\', '/'}
	  if (*tcp == '\\' || *tcp == '/')
	    {
	      *tcp = '\0';
	      return buf;
	    }
	}			// for()
*/
#ifdef wx_msw
      // Try Drive specifier
      if (isalpha (buf[0]) && buf[1] == ':')
	{
	  // A:junk --> A:. (since A:.\junk Not A:\junk)
	  buf[2] = '.';
	  buf[3] = '\0';
	  return buf;
	}
#endif
    }

  return NULL;
}
#endif /*wx_msw*/


/*
	981004 fixed bug when fileName[0] == '\0' and defaultName != 0
    and defaultName did not already exist so wxPathOnly returned NULL

	010525 changed calling sequence to match new macFindFile

    010605 modified so that the suggested path is taken from dirInfo if
           not part of the fileName
*/
char * wxFindFile(char *fileName, char * message, char * defaultname,
				  unsigned long control, mvDirInfoPtr dirInfo)
{
	WHERE("wxFindFile");

	if (fileName[0] == '\0')
	{
		char         tempPath[FULLPATHSIZE+1], tempName[FULLPATHSIZE+1];
		char        *argPath = (char *) 0, *argName = (char *) 0;
		char        *dirName = NullString;


		/* file:"" */
		if (defaultname != (char *) 0 && defaultname[0] != '\0')
		{
#if (1)
			splitPathName(1, defaultname, tempName, FULLPATHSIZE + 1);
#else /*1*/
			/* wxFileNameFromPath() returns pointer to position in its arg */
 			strncpy(tempName,wxFileNameFromPath(defaultname), FULLPATHSIZE);
#endif /*1*/
			if (tempName[0] != '\0')
			{
				argName = tempName;
			}

#if (1)
			splitPathName(0, defaultname, tempPath, FULLPATHSIZE + 1);
			dirName = tempPath;
#else
			/* wxPathOnly() returns pointer to a static buffer */

            dirName = wxPathOnly(defaultname);

            if (dirName == (char *) 0)
            {
				dirName = NullString;
            }
#endif
		} /*if (defaultname && defaultname[0] != '\0')*/

		if (dirName[0] == '\0')
		{
			dirName = getMvDirDirectory(dirInfo);
			strncpy(tempPath, dirName, FULLPATHSIZE);
			dirName = tempPath;
		}
		if (dirName[0] != '\0')
		{
			argPath = tempPath;
		}
		else
		{
			argPath = (char *) 0;
		}
		
		/*
		  Prototype in include/base/wbcmdlg.h
		  WXDLLEXPORT char *wxFileSelector(
		    Constdata char *message = wxFileSelectorPromptStr,
			Const char *default_path = NULL,
            Const char *default_filename = NULL,
			int *indexDefaultExtension = NULL,
			Constdata char *wildcard = wxFileSelectorDefaultWildcardStr,
			int flags = 0,
			wxWindow *parent = NULL,
			int x = -1,
			int y = -1);
		*/

		argPath = wxFileSelector(message, argPath, argName, NULL, "*");

 		if(argPath == (char *) 0)
		{
			return((char *) 0);
		}
		strncpy(FileNameFromFileSelector, argPath, FULLPATHSIZE);
		fileName = FileNameFromFileSelector;
		if (dirInfo != (mvDirInfoPtr) 0)
		{
			char         thisDir[FULLPATHSIZE + 1];
		
			splitPathName(0, FileNameFromFileSelector,
						  thisDir, FULLPATHSIZE + 1);
			if (thisDir[0] != '\0')
			{
				setMvDirDirectory(dirInfo, thisDir);
				copyMvDirInfo(lastVol, dirInfo);
			}
		} /*if (dirInfo != (mvDirInfoPtr) 0)*/
	} /*if (fileName[0] == '\0')*/
	return(fileName);
} /*wxFindFile()*/


} /*extern "C" */
