/*
*(C)* The file is part of the source distribution of MacAnova
*(C)* version 5.03 or later
*(C)*
*(C)* Copyright (c) 2006 by Gary Oehlert and Christopher Bingham
*(C)* unless indicated otherwise
*(C)*
*(C)* You may give out copies of this software;  for conditions see the
*(C)* file COPYING included with this distribution
*(C)*
*(C)* This file is distributed WITHOUT ANY WARANTEE; without even
*(C)* the implied warantee of MERCHANTABILITY or FITNESS FOR
*(C)* A PARTICULAR PURPOSE
*(C)*
*/


#ifdef SEGMENTED
#if defined(MPW) || defined(MW_CW)
#pragma segment Yylex
#endif /*MPW||MW_CW*/
#endif /*SEGMENTED*/

#define YYLEX_C

#include "globals.h"
#include "mainpars.h"

#include "mvIface.h"

#include "mvParse.h"

/*
  030913 Declarations of PARSEARGS, YYLEXMESSAGE, LASTCH, BATCHBLOCKCOUNT,
         ALTINPUTTOKENS[], and ALTTOKENPTR moved to mvParse.h

  031215 Changed debugging macros.  LOCATEBADTOKEN replaced by BADTOKENCODE
         and FATALERRORCODE added.
         When activated, these print a code unique to the place where they
         are invoked

  051228 Fixed bug which led to error when for() or while() was first thing
         in a macro
*/

#if (0) /*Change to 1 to locate where BADTOKEN is being returned*/
#define BADTOKENCODE(I) \
	(PRINT("BADTOKEN location %g%s%s%s",\
           (double) I,NullString,NullString,NullString), BADTOKEN)
#else
#define BADTOKENCODE(I) BADTOKEN
#endif

#if (0) /*Change to 1 to locate where FATALERROR is being returned*/
#define FATALERRORCODE(I) \
	(PRINT("FATALERROR location %g%s%s%s",\
           (double) I,NullString,NullString,NullString), FATALERROR)
#else
#define FATALERRORCODE(I) FATALERROR
#endif

/*
   Define any special platform dependent character codes here for use 
   by yylex()
*/

#if defined(MACINTOSH) && !defined(LEFTSPECIALQUOTE)
#define LESSTHANOREQUALTO  0xb2 /* Macintosh <= */
#define MORETHANOREQUALTO  0xb3 /* Macintosh >= */
#define NOTEQUALTO         0xad /* Macintosh != */
#define LEFTSPECIALQUOTE   0xc7 /* Macintosh << */
#define RIGHTSPECIALQUOTE  0xc8 /* Macintosh >> */
#endif /*MACINTOSH*/


/*
   Size of chunks to get storage for strings and macro arguments
   It is chosen so that the first piece (almost always the only one
   needed) will be available in the "warehouse" for list handle
*/

#define STRINGCHUNK  (DEFAULTLISTLENGTH * sizeof(Symbolhandle))

/*
  File contains routines related to lexical scanning.  The main entrance is
  yylex().  Most of the work is done in yylex1().

  When an error is found and an error message printed, yylex() returns token
  ERROR or FATALERROR.  If no error message is printed, BADTOKEN is returned.

  930506: Dereferenced handle INPUTSTRING to inputString = *INPUTSTRING.
  Note this must be redone after every call that might cause memory to move,
  notably any .*Install(), mygethandle(), etc.

  950803  Removed all references to very obsolete type MACRO1

  950804  Implemented new token SEMI1 to be returned when ';' or newline
  is hit, BRACKETLEV > 0, and the next non-white character is not '}'.  This
  allows parser to remove more unwanted scratch variables

  950822  Included a check for appropriate preceding character (LASTCH) on
  several elements (for, while, else, elseif, quit) so as to give more
  informative error messages.  Also LASTCH is now set when a character
  is taken from ALTINPUTTOKENS.

  950929  Modified parseQuotedArgs so that an escaped quote (\") never changes
  the inquote status.  This parallels a similar change in getinput()
  (in commonio.c).
  The general philosophy is that a macro receives as argument exactly what
  is typed in, after trimming off leading and trailing white space

  960226 Added check for tokens MATDIV (%/%) and DIVMAT (%\%)

  960726 Allow quit(T), quit(F) in addition to quit() and quit.

  960816  Fixed skipBrackets so that an escaped quote (\") does not change
          inQuote status.

  970612  MDEPTH incremented when expanded (in-line) macro header is seen.

  970623  Changes to implement (a) out-of-line macros and evaluated
          strings

  970820  Modified skipBrackets to use new utility findBracket

  980617  scanNumber() no sets NOMISSING bit

  980826  Implemented new syntax element 'next' using new token NEXTREP

  980902  Fixed minor bug: Wrong error message on encountering {stop}

  981118  Added automatic search of macro files when 'name *(' encountered,
          where name is not the name of a defined symbol.

  981231  Fixed minor bug which was seen when break n was at the end of
          a line.

  990204  Macro names are now saved on a stack by PushMacroName() as expanded
          macros are encountered          

  990212  Changed most uses of putOUTSTR() to putErrorOUTSTR()

  990705  Replaced PushMacroName(macroName) by
          PushMacroInfo(macroName,BRACKETLEV)

  990711  Added code to recognize 'return *(', 'return *[;}]' 'return *:'
          Other usages are illegal
          
  000307  Parsing of macro arguments now keeps track of '{' and '}' as well
          as '(', ')', '[', and ']'

  000504  Modifications to recognize quotearg bit in function symbols; this
          signals that arguments are to be turned into CHARACTER scalars
          before the function is invoked.  Changed the names of various
          variables, etc., to reflect the fact that this behavior is no longer
          limited to macro arguments.

  000609  Lexer recognizes \xH in strings, where H is a one or to digit hex-
          adecimal number.

  000609  Lexer ignores empty lines within { . . .}

  010308  Made searchForMacro publically accessible so it can be called
          by Byname() in Symbol.c
          a$b(... does not trigger a macro search
  010601  Include mvIface.h instead of macIface.h or wxIface.h

  010809  A count of square brackets is not kept in SQBRACKETLEV
          Square brackets are also counted in parseQuotedArgs()
          We're moving toward having an input line with unclosed
          '(' or '[' to be continued on a new line, just as happens with
          unclosed '{'.

  010810  Modified handling of quit, exit, bye, ... to give informative
          messages if enclosed in (...) or [...]

  020705  Slight restructuring, moving two switch statements out of
          yylex1() to findAssignOp() and findPercentOp()

  030910  Updated comments for skipComment() and skipWhiteSpace()
          Modified code when '#' is hit so that SEMI1 can be returned
          if appropriate, to fix memory leak

  030911  Changed name of skipComment() to skipToEOL()
  030913  Move declarations of various communication constants and
          variables to new header mvParse.h which is now included
*/
/*
  Globals INLINEMACROHEAD[], OUTLINEMACROHEAD[] and EVALUATIONHEAD[]
  declared in mvParse.h

  Codes notAHeaderCode, inlineMacroHeaderCode, outlineMacroHeaderCode,
  evaluationHeaderCode, headerChar2Code, headerChar4Code,
  macroHeaderLength defined in mvParse.h

  Changed made 030913
*/

/*
  Skip white space, allowing for line continuations

  950804 modified so as to skip comments when ignoreComments != 0
*/

enum skipCodes
{
	ONLYSPACEANDTAB     = 0x00,
	DONTSKIPCOMMENTS    = 0x00,
	DONTSKIPNEWLINES    = 0x00,
	SKIPCOMMENTS        = 0x01,
	SKIPNEWLINES        = 0x02
};

/*
  Skip to the end of line.

  When the line ends with an escaped newline ('\\\n'), '\\' is returned
  Otherwise, '\n' is returned

  In both cases, ISTRCHAR is set so that (*INPUTSTRING)[ISTRCHAR] is
  the character returned.

  If skipCommment runs off end of line (no new line), '\0' is returned
  and (*INPUTSTRING)[ISTRCHAR] will be trailing '\0'

  930911 changed name from skipComment() to skipToEOL() to better reflect
  what it does
*/
static unsigned int skipToEOL(void)
{
	register unsigned char  c, c1;
	register unsigned char *inputString = *INPUTSTRING;

	while((c = inputString[ISTRCHAR]) != '\0' && !isNewline(c))
	{
		if (c == '\\' &&
			(c1 = inputString[ISTRCHAR + 1], isNewline(c1)))
		{ /* leave positioned before backslash newline*/
			break;
		}
		ISTRCHAR++;
	}
	return (c);
} /*skipToEOL()*/

/*
   Skip white space (' ' or '\t' and optionally '\n' and/or comment),
   including unquoted newlines escaped by backslash.
   
   This skips "white space" and returns the character terminating
   white space with ISTRCHAR pointing at that character.

   ' ', '\t' and escaped new line '\\\n' are always "white"

   With skipControl & SKIPNEWLINES != 0, '\n' is also considered
   "white space"

   With skipControl & SKIPCOMMENTS != 0, a comment starting with '#'
   through the character before a terminating '\n' is "white"

   It should never be called while a quoted string is scanned

   970623 returns '\n' if run off end of line
   001206 changed name of argument from ignoreComments to skipControl
          added an option to treat newlines as white space
   030910 Updated comments
*/
static unsigned int skipWhiteSpace(int skipControl)
{
	register unsigned char  c, c1;
	register unsigned char *inputString = *INPUTSTRING;

	while((c = inputString[ISTRCHAR]) == ' ' || c == '\t' ||
		  (skipControl & SKIPNEWLINES) && isNewline(c) ||
		  (skipControl & SKIPCOMMENTS) && c == '#' ||
		  (c == '\\' &&
		   (c1  = inputString[ISTRCHAR+1], isNewline(c1))))
	{
		ISTRCHAR++;
		if (c == '\\')
		{ /* skip escaped new line */
			ISTRCHAR++;
		}
		else if (c == '#')
		{ /* skipControl must be SKIPCOMMENTS */
			(void) skipToEOL();
			/*
			  Should be positioned before '\n', before backslash '\n',
			  or before '\0' in case of missing '\n'
			*/
		}
	}
	return ((c != '\0') ? (unsigned int) c : '\n');
} /*skipWhiteSpace()*/

#if (0) /*no longer used*/
/*
  Scan *INPUTSTRING for match of s, i.e., match all characters in s and
  determine that next character in *INPUTSTRING is not alpha-numeric.
  If match is found, return non-zero, otherwise zero.
*/

static long matchKeyword(char *s)
{
	unsigned char   *inputString = *INPUTSTRING + ISTRCHAR - 1;
	
	while(*s)
	{
		if (*s++ != *inputString++)
		{
			return (0);
		}
	} /*while(*s)*/

	return (!isnamechar(*inputString));

} /*matchKeyword()*/
#endif /*0*/

/*
   960816 Fixed bug so that escaped quote in quotes does not change inQuotes
   970820 Rewrote so that it uses new utility findBracket()
   010809 Added ability to skip to ']' from '['.  I don't know it's needed,
          but I'm now keeping track of square brackets
*/
static char skipBrackets(long goalLevel, char leftbracket)
{
	char             c;
	long             level;
	WHERE("skipBrackets");

	if (leftbracket == '{')
	{
		level = BRACKETLEV;
	}
	else if (leftbracket == '(')
	{
		level = PARENLEV;
	}
	else if (leftbracket == '[')
	{
		level = SQBRACKETLEV;
	}
	else
	{
		/* should not happen */
		return (0);
	}
	
	c = findBracket(goalLevel, leftbracket, *INPUTSTRING, &level,
					&ISTRCHAR, &LASTCH);

	if (leftbracket == '{')
	{
		BRACKETLEV = level;
	}
	else if (leftbracket == '(')
	{
		PARENLEV = level;
	}
	else
	{
		SQBRACKETLEV = level;
	}
	
	/*
	  return the last character read;
	  if it was '\0', ISTRCHAR still points to it
	*/
	return (c);
} /*skipBrackets()*/


/*
  YYLEXMESSAGE == SKIPBRACKETBLOCK : skip compound statements on if, elseif,
  or while with condition F

  YYLEXMESSAGE == SKIPPARENBLOCK : skip '(...)' after elseif when condition
  from preceding if or elseif is T

  YYLEXMESSAGE > MAXWDEPTH : fakeout parser to get out of breaks or breakall
       or to skip to the end of a loop on next

*/

static unsigned int processBreak(void)
{
	unsigned char         *inputString = *INPUTSTRING;
	unsigned int           c;
	long                   currentBracketLev = BRACKETLEV;
	long                   wantedBracketLev, extraBrackets;
	long                   i;
	WHERE("processBreak");

	ALTTOKENPTR = ALTINPUTTOKENS;
	*ALTTOKENPTR = 0;

	if (YYLEXMESSAGE == SKIPBRACKETBLOCK)
	{
		/*
		  Just got F condition on if or while, skip to end.
		  Should be positioned immediately before '{' or erroneous
		  character.
		  */

		YYLEXMESSAGE = 0;
		if ((c = inputString[ISTRCHAR++]) != '{')
		{/* must be erroneous character; return it */
			ALTTOKENPTR = (int *) 0;
			return (c);
		}

 		BRACKETLEV++;
		/* move ISTRCHAR to point after real compound statement */
		c = skipBrackets(currentBracketLev, c);
		BRACKETLEV = currentBracketLev + 1;
		/* create empty compound statement */
		if (c == '}')
		{
			i = 0;
			ALTINPUTTOKENS[i++] = (int) ';';
			ALTINPUTTOKENS[i++] = (int) '}';
			ALTINPUTTOKENS[i] = 0;
		} /*if (c == '}')*/
		else
		{/* end of line before matching '}' */
			sprintf(OUTSTR,
					"ERROR: need matching '{' and '}' after 'if', 'while', and 'for'");
			putErrorOUTSTR();
			ALTTOKENPTR = (int *) 0;
		} /*if (c == '}'){}else{}*/
		return ((unsigned int) '{');
	} /*if (YYLEXMESSAGE == SKIPBRACKETBLOCK)*/

	if (YYLEXMESSAGE == SKIPPARENBLOCK)
	{
		/*
		  Previous T condition on elseif; skip '(...)'.
		  Should be positioned immediately before '('
		  */
		long                   currentParenLev = PARENLEV;

		YYLEXMESSAGE = 0;
		if ((c = inputString[ISTRCHAR++]) != '(')
		{/* must be erroneous character; return it (should not happen)*/
			ALTTOKENPTR = (int *) 0;
			return (c);
		}
 		PARENLEV++;
		/* move ISTRCHAR to point after matching ')' */
		c = skipBrackets(currentParenLev, c);
		PARENLEV = currentParenLev + 1;
		/* create empty paren list */
		if (c == ')')
		{
			i = 0;
			ALTINPUTTOKENS[i++] = (int) ')';
			ALTINPUTTOKENS[i] = 0;
		}
		else
		{			/* end of line before matching ')' */
			ALTTOKENPTR = (int *) 0;
		}
		return ((unsigned int) '(');
	} /*if (YYLEXMESSAGE == SKIPPARENBLOCK)*/

	/*
	  If YYLEXMESSAGE > MAXWDEPTH it should be of form (wanted
	  bracketlevel) + MAXWDEPTH + 1  and we are just past
	  "break", "breakall" or "next".  We need to set up ALTINPUTTOKENS
	  with enough ';}' to fake out the parser to think
	  that we are getting out of any enclosing if's, as well
	  as enough ';}' to convince it that we are getting out of
	  the enclosing while (break) or all the enclosing whiles
	  (breakall)
	  In every case the character we return now will be ';'
	  */

	if (YYLEXMESSAGE > MAXWDEPTH)
	{/* must be 'break', 'breakall', 'next' or 'return' */
		wantedBracketLev = YYLEXMESSAGE - MAXWDEPTH - 1;

		YYLEXMESSAGE = 0;
		*ALTTOKENPTR++ = (int) '}';
		extraBrackets = currentBracketLev - wantedBracketLev - 1;

		for(i = 0;i< extraBrackets;i++)
		{
			*ALTTOKENPTR++ = (int) ';';
			*ALTTOKENPTR++ = (int) '}';
		}
		c = skipBrackets(wantedBracketLev, '{');

		if (c != (int) '}') /* end of line before matching '}' */
		{
			sprintf(OUTSTR,"ERROR: missing '}'");
			putErrorOUTSTR();
			ALTTOKENPTR--; /* wipe out one '}' to force error in parser */
		}
		*ALTTOKENPTR = 0;
		ALTTOKENPTR = ALTINPUTTOKENS;
		BRACKETLEV = currentBracketLev;
		return ((unsigned int) ';');
	} /*if (YYLEXMESSAGE > MAXWDEPTH)*/
	return (BADTOKENCODE(1)); /* shouldn't ever happen */
} /*processBreak()*/

/*
  Definitions of normalParse, quoteArgPunct and quoteArg moved to
  mvParse.h 030913
*/

/*
  create CHAR symbols containing macro arguments.  Called by
  yylex when PARSEARGS != normalParse

  If PARSEARGS == quoteArgPunct, the next character is known to be
  '(', ',' , ')', or '\n'
  Otherwise, it encapsulates INPUTSTRING up to next one of these characters
  outside of quotes and at the same bracket level as an argument

  In initially collects characters in stringBuff.  If that fills up, then
  temporary storage is allocated for collection.  There should be no practical
  limit on the length of a macro argument.
  
  000307 '{' and '}' are now counted in addition to with '(', '[', ')' and ']'
         If the macroBracketLevel > 1, a new line does not terminate an argument
  000504 Changed name from parseMacro() to parseQuotedArgs() reflect that it
         now may be parsing the arguments of a function whose arguments are
         changed into CHARACTER scalars
*/

enum parenStackConstants
{
	parenStackOverflow = 0,
	parenStackUnderflow,
	parenStackLength = 200
};


#define pushParen(C) ((parenStackPlace < parenStackLength) ? \
	(parenStack[parenStackPlace++] = C) : \
	(parenStackPlace++))

#define popParen() ((--parenStackPlace < 0) ? parenStackUnderflow : \
	((parenStackPlace >= parenStackLength) ? parenStackOverflow : \
	(parenStack[parenStackPlace])))

enum parenCodes
{
	parenCode,
	bracketCode,
	sqbracketCode
};

static unsigned int parseQuotedArgs(void)
{
	char            stringBuff[STRINGCHUNK];
	char           *tempString;
	char          **tempStringH = (char **) 0;
	unsigned char  *inputString = *INPUTSTRING;
	unsigned int    c, c1;
	int             argParensLevel[3];
	int             empty = 1, escaped = 0, inQuotes;
	long            place, nleft;
	unsigned char   parenStack[parenStackLength];
	int             parenStackPlace;
	char           *leftParens = "({[", *rightParens = ")}]";
	WHERE("parseQuotedArgs");

	Yylval = SymbolZero;

/*
   PARSEARGS == quoteArgPunct means next character is known to be
   '(', ',', ')' or '\n'
*/
	if (PARSEARGS == quoteArgPunct)
	{
		c = inputString[ISTRCHAR++];
		if (c == '(' || c == ',')
		{
			/* expect macro or function argument next time in */
			PARSEARGS = quoteArg;
			if (c == '(')
			{
				PARENLEV++;
			}
		}
		else /* ')' or end of line */
		{
			PARSEARGS = normalParse;
			if (c == ')')
			{
				PARENLEV--;
			}
			else
			{
				c = ENDOFL;
			}
		}
		return (c);
	} /*if (PARSEARGS == quoteArgPunct)*/

	/*
	  PARSEARGS must be quoteArg.
	  Convert to CHAR everything up to ',' or ')' with 
	  argParenLevel <= 0 && argBracketLevel <= 0 && argSqbracketLevel <= 0
	*/

	/* skip leading white space, allowing for line continuations */
	c = skipWhiteSpace(ONLYSPACEANDTAB);
	if (isNewline(c) || c == '\0')
	{
		PARSEARGS = normalParse;
		return ((c != '\0') ? ENDOFL : c);
	}

	/* at this point c is non-white and should be the same as 
	   (*INPUTSTRING)[ISTRCHAR]
	*/
	Yylval = Install(SCRATCH, CHAR);
	if (Yylval == SymbolError)
	{
		return (FATALERRORCODE(1));
	}

	setNDIMS(Yylval,1);
	setDIM(Yylval,1,1);

	argParensLevel[0] = argParensLevel[1] = argParensLevel[2] = 0;
	inQuotes = 0;
	place = 0;
	tempString = stringBuff;
	nleft = STRINGCHUNK - 1;

	/* collect a macro or function argument in tempString*/
	parenStackPlace = 0;

	while(1)
	{
		if (nleft == 0)
		{
			/* no room; allocate overflow space */
			if (tempStringH == (char **) 0)
			{
				/* must have just filled up stringBuff */
				tempStringH = mygethandle(place + STRINGCHUNK);
				if (tempStringH != (char **) 0)
				{
					stringBuff[place] = '\0';
					strcpy(*tempStringH, stringBuff);
				}
			}
			else
			{
				tempStringH = mygrowhandle(tempStringH, place + STRINGCHUNK);
			}
			setSTRING(Yylval, tempStringH);
			if (tempStringH == (char **) 0)
			{
				c = FATALERRORCODE(2);
				goto clearYylval;
			}
			nleft = STRINGCHUNK - 1;
			tempString = *tempStringH;
			inputString = *INPUTSTRING;
		} /*if (nleft == 0)*/
		
		ISTRCHAR++;
		while(!inQuotes && c == '\\' &&
			  (c1 = inputString[ISTRCHAR] , isNewline(c1)))
		{/* allow for continuation lines */
			ISTRCHAR++;
			c = inputString[ISTRCHAR++];
		}
		if (c == '\0')
		{
			ISTRCHAR--;
			break;
		}

		tempString[place++] = c;
		nleft--;
		if (c == '"')
		{
			if (!escaped)
			{
				inQuotes = !inQuotes;
			}
		}
		else if (!inQuotes &&
				 ((parenStackPlace == 0 &&
				  (c == ',' || c == ')')) ||
				  argParensLevel[bracketCode] <= 0 && isNewline(c)))
		{	/*
				found end of argument put back character for next call
			*/
			ISTRCHAR--;
			place--;
			nleft++;
			PARSEARGS = (c == ',') ? quoteArgPunct : normalParse;
			break;
		}
		escaped = (c == '\\') ? !escaped : 0;		
		if (empty && !isspace(c))
		{
			empty = 0;
		}
		if (!inQuotes)
		{
			/* 000307 keep track of '{' and '}', too */
			char      *isParen = strchr(leftParens, c);
			
			if (isParen)
			{
				int      jParen = isParen - leftParens;
				
				pushParen(c);
				argParensLevel[jParen]++;
			} /*if (isParen)*/
			else
			{
				isParen = strchr(rightParens, c);
				if (isParen)
				{
					int             jParen = isParen - rightParens;
					unsigned char   previousParen = popParen();
					
					if (previousParen == parenStackUnderflow)
					{
						sprintf(OUTSTR,
								"ERROR: too many %c's in macro or function argument",
								c);
						goto errorExit;
					}
					if (previousParen != parenStackOverflow
						&& previousParen != leftParens[jParen])
					{
						sprintf(OUTSTR,
								"ERROR: use of ending '%c' doesn't match starting '%c'",
								c, previousParen);
						goto errorExit;
					}
					argParensLevel[jParen]--;
				} /*if (isParen)*/
			} /*if (isParen){}else{}*/
		} /*if (!inQuotes)*/
		c = inputString[ISTRCHAR];
	} /*while(1)*/
	tempString[place] = '\0';

	while(place > 0 && ((c1 = tempString[place-1]) == ' ' || c1 == '\t'))
	{ /* trim off trailing white space */
		place--;
	}

	if (c == '\0')
	{ /* run out of string before finished parsing macro function args*/
		PARSEARGS = normalParse;
		goto clearYylval;
	} /*if (c == '\0')*/

	if (place == 0 || empty)
	{
		/*
		  On empty argument, '()', '(,...)', '(...,,...)', or
		  '(...,)' , return null symbol, and token VAR.
		  White argument (e.g., ', ,') is also considered null
		  */
		c = VAR;
		goto clearYylval;
	} /*if (place == 0 || empty)*/
	else if (place > 0)
	{
		tempString[place++] = '\0';
		if (tempStringH == (char **) 0)
		{
			tempStringH = mygethandle(place);
			if (tempStringH != (char **) 0)
			{
				strcpy(*tempStringH, stringBuff);
			}
		}
		else
		{
			tempStringH = mygrowhandle(tempStringH, place);
		}
		setSTRING(Yylval, tempStringH);
		if (tempStringH == (char **) 0)
		{
			c = FATALERRORCODE(3);
			goto clearYylval;
		}
		/* macro or function argument has been transformed to CHAR var */
		c = VAR;
	}

	return (c);

  clearYylval:
	Removesymbol(Yylval);
	Yylval = SymbolZero;
	return (c);

  errorExit:
	yyerror(OUTSTR);
	return (0);
} /*parseQuotedArgs()*/

/*
  980617 NOMISSING bit is set on successful scan
*/
static unsigned int scanNumber(void)
{
	unsigned char      *inputString = *INPUTSTRING, *endptr;
	double              y, absy;
	WHERE("scanNumber");
	
	y = mystrtod((char *) inputString+ISTRCHAR, (char **) &endptr);

	if (y == 0 && endptr == inputString+ISTRCHAR)
	{ /* should not happen */
		ISTRCHAR++;
		yyerror("ERROR: nonnumeric where numeric expected");
		return (ERROR);
	}

	if ((absy = fabs(y), doubleEqualBdouble(absy, TooBigValue)))
	{
		ISTRCHAR += endptr - (inputString+ISTRCHAR);
		yyerror("ERROR: value too big to be represented in computer");
		return (ERROR);
	}

	ISTRCHAR = endptr - inputString;

	Yylval = RInstall(NUMSCRATCH, (long) 1);
	if (Yylval == SymbolError)
	{
		return (FATALERRORCODE(4));
	}
	DATAVALUE(Yylval,0) = y;
#if (USENOMISSING)
	setNOMISSING(Yylval);
#endif /*USENOMISSING*/

	return (NUMBER);
} /*scanNumber()*/

#define MAXTOKENSCAN 25
/*
   Check whether string in token is a syntax element (if, else, elseif, 
   break, breakall, for, while, end, quit, stop).

   If token is not a syntax element, 0 is returned;  otherwise the
   parsing token is returned (IF, ELSE, ELSEIF, BREAK, BREAKALL, FOR,
   WHILE, END)

   950912 replaced calls to strncmp() with character by character comparisons

   990703 fixed bug in recognition of 'next' (any token starting with 'next'
          was recognized)

   990711 recognized 'return'
*/
#define SyntaxStartChars "befinqrws" /*possible initial leters of syntax elts*/
static int isSyntaxElement(char token [])
{
	char           c0 = token[0];
	char           c1 = token[1], c2 = token[2], c3 = token[3];
	char           c4, c5;

	switch (c0)
	{
	  case 'i':
		/* if */
		return ((c1 == 'f' && c2 == '\0') ? IF : 0);

	  case 'e':
		if (c1 == 'l')
		{ /* else and elseif*/
			if (c2 != 's' || c3 != 'e')
			{
				return (0);
			}
			c4 = token[4];
			if (c4 == '\0')
			{
				return (ELSE);
			}
			return ((c4 == 'i' && token[5] == 'f' && token[6] == '\0') ?
					ELSEIF : 0);
		} /*if (c1 == 'l')*/
		else if (c1 == 'x')
		{/* exit*/
			return ((c2 == 'i' && c3 == 't' && token[4] == '\0') ?
					END : 0);
		}
		/* end */
		return ((c1 == 'n' && c2 == 'd' && c3 == '\0') ? END : 0);

	  case 'f':
		/* for */
		return ((c1 == 'o' && c2 == 'r' && c3 == '\0') ? FOR : 0);

	  case 'w':
		/* while */
		return ((c1 == 'h' && c2 == 'i' && c3 == 'l' && token[4] == 'e' &&
				 token[5] == '\0') ? WHILE : 0);

	  case 'n':
		/* next */
		return ((c1 == 'e' && c2 == 'x' && c3 == 't' && token[4] == '\0') ?
				NEXTREP : 0);

	  case 'b':
		if (c1 == 'r')
		{ /* break and breakall*/
			if (c2 != 'e' || c3 != 'a' || token[4] != 'k')
			{
				return (0);
			}
			c5 = token[5];
			if (c5 == '\0')
			{
				return (BREAK);
			}
			return ((c5 == 'a' && token[6] == 'l' && token[7] == 'l' &&
					 token[8] == '\0') ? BREAKALL : 0);
		}

		/* bye */
		return ((c1 == 'y' && c2 == 'e' && c3 == '\0') ? END : 0);

	  case 'q':
		/* quit */
		return ((c1 == 'u' && c2 == 'i' && c3 == 't' && token[4] == '\0') ?
				END : 0);

	  case 's':
		/* stop */
		return ((c1 == 't' && c2 == 'o' && c3 == 'p' && token[4] == '\0') ?
				END : 0);
	  case 'r':
		/* return */
		return ((c1 == 'e' && c2 == 't' && c3 == 'u' && token[4] == 'r' &&
				 token[5] == 'n' && token[6] == '\0') ?
				EXITMACRO : 0);

	  default:
		return (0);
	} /*switch (c)*/
} /*isSyntaxElement()*/

/*
  Function to decode opcodes of the form <-{+,-,*,*,^,%%}.
  c1 is the character immediately following '<-' and c2 is the
  character following that

  020705 created by moving some code from yylex1()
*/
static int findAssignedOp(int c1, int c2)
{
	int          c;

	switch (c1)
	{
	  case '+':
		c = ASSIGNADD;
		break;

	  case '-':
		c = ASSIGNSUB;
		break;

	  case '*':
		if (c2 == '*')
		{
			c = ASSIGNPOW;
			ISTRCHAR++;
		}
		else
		{
			c = ASSIGNMULT;
		}
		break;

	  case '/':
		c = ASSIGNDIV;
		break;

	  case '%':
		if (c2 == '%')
		{
			c = ASSIGNMOD;
			ISTRCHAR++;
		}
		else
		{
			ISTRCHAR--;
		}
		break;

	  case '^':
		c = ASSIGNPOW;
		break;
	} /*switch (c1)*/
	return (c);
} /*findAssignedOp()*/

/*
  Function to decode opcodes starting with '%'

  020705 created by moving some code from yylex1()
*/

static int findPercentOp(int c2)
{
	int          c;
	switch (c2)
	{
	  case '%':
		c = MOD; /* '%%' */
		break;
#ifdef BITOPS
	  case '!':
		c = BITNOT; /* '%!' */
		break;
	  case '&':
		c = BITAND; /* '%&' */
		break;
	  case '^':
		c = BITXOR; /* '%^' */
		break;
	  case '|':
		c = BITOR; /* '%|' */
		break;
#endif /*BITOPS*/

	  default:
		if ((*INPUTSTRING)[ISTRCHAR++] == '%')
		{
			switch (c2)
			{
			  case '*':
				c = MATMULT;   /* '%*%' */
				break;
			  case 'c':
				c = TRMATMULT;   /* '%c%' */
				break;
			  case 'C':
				c = MATMULTTR;   /* '%C%' */
				break;
			  case '/':
				c = MATDIV;   /* '%/%' */
				break;
			  case '\\':
				c = DIVMAT;   /* '%\\%' */
				break;
			  default:
				ISTRCHAR--;
				c = BADTOKENCODE(3);
			} /*switch (c2)*/
		} /*if ((*INPUTSTRING)[ISTRCHAR++] == '%')*/
		else 
		{
			ISTRCHAR--;
			c = BADTOKENCODE(4);
		}
		break;
	} /*switch (c2)*/
	return (c);
} /*findPercentOp()*/

/*
  Function to recognize and decode expanded macro or evaluated
  string header.  It returns a 0 if it is not such a header and
  a code indicating which type of header it is otherwise

  If it is such a header ISTRCHAR is advanced to beyond it
  030911 created with code from yylex1()
*/
static unsigned char isMacroHeader(void)
{
	unsigned char        headType = notAHeaderCode;
	unsigned char       *inputString = *INPUTSTRING;

	if (inputString[ISTRCHAR] == headerChar2Code &&
		inputString[ISTRCHAR+2] == headerChar4Code)
	{
		headType = inputString[ISTRCHAR+1];

		/*
		  Start of expanded macro signaled by "{#)#" (inline)
		  or "{#]#" (out-of-line); start of evaluated string
		  signalled by "{#}#"
		*/
		if (headType == inlineMacroHeaderCode ||
			headType == outlineMacroHeaderCode ||
			headType == evaluationHeaderCode)
		{
			/* advance to beyond header */
			ISTRCHAR += macroHeaderLength - 1;
		}
		else
		{
			headType = notAHeaderCode;
		}
	}
	return (headType);
} /*isMacroHeader()*/


#define ishexdigit(c) (isdigit(c) || c >= 'a' && c <= 'f' || c >= 'A' && c <= 'F')
/*
  yylex1() is where all the real work of identifying tokens is done
  981231 eliminated label 'start' and several goto's, replacing them by
         a while (1) loop and continue statements
  990204 Names of expanded macros are pushed onto stack as they are encountered
  990711 Added code to recognize and process new syntax element 'return'
  001206 Modified so that empty lines within {...} are ignored; this is
         intended to have the effect that, for example
		 {
		    1

         }
         has value 1 rather than NULL
  020705 moved some switch statements to new functions findAssignedOp()
         and findPercentOp()
  030911 Changed code processing macro and evaluated string heads to
         skip entire heads to the next line
         Changed code processing '#' when BRACKLEV > 0 to skip past
         white space, treating comments and newlines as white
  051228 changed code so that FOR and WHILE can immediately follow
         INLINEHEAD and OUTLINEHEAD
*/
#define UNCLOSEDOK  0 /*may be global option sometime */

static unsigned int yylex1(void)
{
	unsigned char       *inputString;
	char                 stringBuff[STRINGCHUNK];
	char                *tempString;
	char               **tempStringH = (char **) 0;
	int                  nleft;
	unsigned int         c, c1, c2;
	double               y = -1.0;
	long                 i, j, nameStart,tokenLength, savedPlace;
	long                 place;
	char                 token[MAXTOKENSCAN+1];
	unsigned char       *endptr;
	int                  errorToken;
	char                *noForWhile = "ERROR: '%s' without 'for' or 'while'";
	WHERE("yylex1");

	while(1)
	{
		*OUTSTR = '\0';

		inputString = *INPUTSTRING;

		/*
		  Don't replace this by call to skipWhiteSpace as that translates
		  '\0' to '\n'
		*/
		while ((c = (unsigned int) inputString[ISTRCHAR]) == ' ' || c == '\t')
		{
			ISTRCHAR++;				/* skip whitespace */
		}

		if (c == '\0')
		{
			int     outlinemacro = !isscratch(ThisInputname);
		
			if (INPUTLEVEL == 0)
			{
				return ((BRACKETLEV != 0) ? BADTOKENCODE(2) : 0);
			}
#if (0) /* cleanitup now cleans all of them */
			mydisphandle((char **) ThisInputstring);
			ThisInputstring = (unsigned char **) 0;
#endif /*0*/

			popInputlevel();

			if (outlinemacro)
			{
				continue;
			}

#if (0)
			/*
			  971206 disabled check for new line to avoid problem; not sure why it is
			  here so I hope no problem has been introduced.
			  */
			if (isNewline((*INPUTSTRING)[ISTRCHAR]))
			{
				ISTRCHAR++;
			}
#endif /*0*/
			return (c); /* c is still '\0' which tells parser to quit */

		} /*if (c == '\0')*/
	
	
		if (c == '}' && LASTCH != ';' && LASTCH != ENDOFL)
		{/* force ';' before '}' */
			c = ';';
			return(c);
		}

		ISTRCHAR++;

		switch (c)
		{
		  case '\n':
#ifndef MPW
		  case '\r':
#endif /*MPW*/
			if (BRACKETLEV > 0)
			{
				if (LASTCH == ';' || LASTCH == ENDOFL)
				{
					continue;
				}
				c = skipWhiteSpace(SKIPCOMMENTS | SKIPNEWLINES);
				if (c == '}')
				{
					return (ENDOFL);
				} 
#if (0)
				/* 
				   Part of an attempt to allow newlines inside (..) and
                   [...] but this way doesn't work;
				   maybe I'll come back to it
				*/
				else if ((PARENLEV > 0 || SQBRACKETLEV > 0) && UNCLOSEDOK)
				{
					continue;
				}
#endif /*0*/
				return (SEMI1);
			} /*if (BRACKETLEV > 0)*/
#if (0)
			else if ((PARENLEV > 0 || SQBRACKETLEV > 0) && UNCLOSEDOK)
			{
				continue;
			}
#endif /*0*/
			else if (c = skipWhiteSpace(SKIPCOMMENTS), !isNewline(c))
			{
				/*
				  Probably impossible to get here now that brackets and
				  parentheses are checked in expanding a macro.  Originally
				  extra '}' in a macro could reduce BRACKETLEV <= 0 but
				  in the structure of the unexpanded line, BRACKETLEV > 0
				  so additional lines could be entered.

				  If BRACKETLEV == 0, then the newline that brought us
				  to this switch case should be followed by '\0' which
				  causes skipWithSpace() to return a newline which means
				  this block is not executed
				*/

				yyerror("ERROR: probably too many }'s in macro prior to or");
				return (ERROR);
			}
			return(ENDOFL);

		  case ';':
			if (BRACKETLEV > 0 && 
				(c1 = skipWhiteSpace(SKIPCOMMENTS | SKIPNEWLINES)) != '}')
			{
				if (c1 == ';')
				{
					ISTRCHAR++;
				}
				c = SEMI1;
			} /*if (BRACKETLEV > 0 && (c1 = skipWhiteSpace(SKIPCOMMENTS | SKIPNEWLINES)) != '}')*/
		
			return (c);

			/* simply pass through certain math symbols */
		  case '+':
		  case '-':
		  case '/':
		  case '$':
		  case ',':
		  case '\'':
			return (c);

		  case '(':
			PARENLEV++;
			return (c);
		  case ')':
			PARENLEV--;
			if (PARENLEV < 0)
			{
				yyerror("ERROR: too many )'s");
				c = ERROR;
			}
			return (c);
		  case '[':
		  case ']':
			if (inputString[ISTRCHAR] == c)
			{
				/* start of '[[' or ']]'*/
				c = (c == '[') ? LEFTBRACK2 : RIGHTBRACK2;
			}
			else if (c == '[')
			{
				SQBRACKETLEV++;
			}
			else
			{
				SQBRACKETLEV--;
			}

			return (c);

		  case '*':
			/* "**" is equivalent to "^" */
			if ((unsigned int) inputString[ISTRCHAR] != '*')
			{
				return (c);
			}
			ISTRCHAR++;
			/* fall through */
		  case '^':
			return (POW);
		  case '<':
		  case '>':
			if (inputString[ISTRCHAR] == c)
			{
				if (c == '>')
				{
					ISTRCHAR++;
					return (RIGHTANGLE);
				}

				for(i=2;inputString[ISTRCHAR-1+i] == '<';i++)
				{
					;				/* count number of successive '<'s */
				}

				if ((i & 1) == 0)
				{					/* even number of '<'s */
					MACROSTART = ISTRCHAR-1;
					ISTRCHAR++;
					return (LEFTANGLE);
				}
				else
				{/* probably of form expr <<<name>> parsed as expr < <<name>> */
					return ('<');
				}

			}
			/* fall through */

		  case '=':
		  case '!':
			/* '<', '>', '='. '!' */
			if ((c2 = (unsigned int) inputString[ISTRCHAR++]) == '=')
			{
				if (c == '=')
				{
					c = EQ;
				}
				else if (c == '>')
				{
					c = GE;
				}
				else if (c == '<')
				{
					c = LE;
				}
				else
				{
					c = NE;
				}
			}
			else if (c == '<' && c2 == '-')
			{
				c = ASSIGN;
				c1 = (unsigned int) inputString[ISTRCHAR];
				if (strchr("+*-/^%",(int) c1))
				{
					c2 = (unsigned int) inputString[ISTRCHAR+1];
					if (c1 == '+' || c1 == '-')
					{ /* must have white space after <-+ and <-- */
						if (c2 == '\\')
						{
							savedPlace = ISTRCHAR++;
							(void) skipWhiteSpace(ONLYSPACEANDTAB);
							for(i=savedPlace;i<ISTRCHAR;i++)
							{
								c2 = inputString[i];
								if (c2 == ' ' || c2 == '\t')
								{
									break;
								}
							}
							ISTRCHAR = savedPlace;
						}
						if (c2 != ' ' && c2 != '\t')
						{
							c2 = '\0';
						}					
					} /*if (c1 == '+' || c1 == '-')*/

					if (c2 != '\0')
					{
						ISTRCHAR++;
						c = findAssignedOp(c1, c2);
					} /*if (c2 != '\0') */
				} /*if (strchr("+*-/^%",(int) c1))*/
			}/*else if (c == '<' && c2 == '-')*/
			else
			{
				ISTRCHAR--;
			}
			return (c);

#ifdef LESSTHANOREQUALTO
		  case LESSTHANOREQUALTO: /* special code for <= */
			return (LE);
#endif /*LESSTHANOREQUALTO*/

#ifdef MORETHANOREQUALTO
		  case MORETHANOREQUALTO: /* special code for >= */
			return (GE);
#endif /*MORETHANOREQUALTO*/

#ifdef NOTEQUALTO
		  case NOTEQUALTO: /* special code for >= */
			return (NE);
#endif /*NOTEQUALTO*/

#ifdef LEFTSPECIALQUOTE /* special code for << */
		  case LEFTSPECIALQUOTE:
			return (LEFTANGLE);
#endif /*LEFTSPECIALQUOTE*/

#ifdef RIGHTSPECIALQUOTE /* special code for >> */
		  case RIGHTSPECIALQUOTE:
			return (RIGHTANGLE);
#endif /*RIGHTSPECIALQUOTE*/

		  case ':':
			return (ASSIGN1);
		  case '&': /* '&&' equivalent to '&' and '||' equivalent to '|' */
		  case '|':
			if (inputString[ISTRCHAR] == c)
			{
				ISTRCHAR++;
				c = (c == '&') ? AND : OR;
			}
			return (c);

		  case '%':
			c2 = inputString[ISTRCHAR++];
			c = findPercentOp(c2);
			return (c);

		  case '?':					/* missing value */
			Yylval = RInstall(NUMSCRATCH, (long) 1);
			if (Yylval == SymbolError)
			{
				return (FATALERRORCODE(5));
			}

			setNDIMS(Yylval,1);
			setDIM(Yylval,1,1);
			setMissing(DATAVALUE(Yylval,0));

			return (NUMBER);		/* missing value */

		  case '"':					/* start of string */
			inputString = *INPUTSTRING;
			Yylval = Install(STRINGSCRATCH, CHAR);
			if (Yylval == SymbolError)
			{
				return (FATALERRORCODE(6));
			}
			setNDIMS(Yylval,1);
			setDIM(Yylval,1,1);

			place = 0;  /* position in string being collected */
			tempString = stringBuff;
			nleft = STRINGCHUNK - 1;
			/*
			  First use stringBuff as collecting place.  Only when that has
			  been filled allocate new space.
			  */
			while (1)
			{
				if (nleft == 0)
				{ /* need more room; get room for STRINGCHUNK more characters */
					if (tempStringH == (char **) 0)
					{ /* tempStringH not yet allocated */
						tempStringH = mygethandle(place + STRINGCHUNK);
						if (tempStringH != (char **) 0)
						{
							stringBuff[place] = '\0';
							strcpy(*tempStringH, stringBuff);
						}
					}
					else
					{ /* reallocate tempStringH */
						tempStringH = mygrowhandle(tempStringH,
												   place + STRINGCHUNK);
					}
					setSTRING(Yylval, tempStringH);
					if (tempStringH == (char **) 0)
					{
						errorToken = FATALERRORCODE(7);
						goto errorExit;
					}
					nleft = STRINGCHUNK - 1;
					tempString = *tempStringH;
					inputString = *INPUTSTRING;
				} /*if (nleft == 0)*/
			
				c = inputString[ISTRCHAR++];
				if (c == '\0')
				{
					ISTRCHAR--;
					c = '"';
				}
				if (c == '"')
				{
					break;
				}

				if (c == '\\')
				{/* inside a string, so always accept escaped characters */
					/*
					  diad			  result
					  '\n'				NL
					  '\r'				CR
					  '\t'				TAB
					  '\ijk', i < 4		decoded ascii character
					  '\EOL'			'\EOL', where EOL is '\0'
					  '\somethingElse'	'somethingElse'
					  */
					c2 = inputString[ISTRCHAR++];
					if (c2 == '\0')
					{ /* run off end of *INPUTSTRING; fake closing quote */
						c = '"';
						ISTRCHAR--;
					}
					else if (isNewline(c2))
					{ /* '\' at end of line within string */
						ISTRCHAR--;
					}
					else if (c2 == 'n')
					{ 
						c = '\n';
					}
					else if (c2 == 'r')
					{
						c = '\r';
					}
					else if (c2 == 't')
					{
						c = '\t';
					}
					else if (isdigit(c2) && c2 < '8')
					{/* octal characters, e.g., \012 */
						savedPlace = ISTRCHAR;
						c = c2 - '0';
						for (j = 1;j < 3 ;j++)
						{
							c2 = inputString[ISTRCHAR++];
							if (!isdigit(c2) || c2 > '7')
							{
								break;
							}
							c = 8*c + c2 - '0';
						} /*for (j = 1;j < 3 ;j++)*/
						if (j < 3)
						{
							ISTRCHAR--;
						}
						if (c == '\0')
						{
							yyerror("ERROR: illegal octal null character");
							errorToken = ERROR;
							goto errorExit;
						}
						else if (c > 0377)
						{ /* use only first two digits */
							c /= 8;
							ISTRCHAR--;
						}
					}
					else if (c2 == 'x')
					{/* Hex characters, e.g., \x */
						int         c3;

						c = 0;
						for (j = 0; j < 2; j++)
						{
							c3 = inputString[ISTRCHAR];
							if (!ishexdigit(c3))
							{
								break;
							}
							ISTRCHAR++;
							c = 16*c + ((isdigit(c3)) ? c3 - '0' : 10 + tolower(c3) - 'a');
						}
						if (j == 0)
						{
							c = c2;
						}
						else if (c == '\0')
						{
							yyerror("ERROR: illegal hex null character");
							errorToken = ERROR;
							goto errorExit;
						}
					}
					else
					{
						c = c2;
					}
				} /*if (c == '\\')*/
				tempString[place++] = c;
				nleft--;
			} /*while (1)*/

			tempString[place++] = '\0';
			if (tempStringH == (char **) 0)
			{
				tempStringH = mygethandle(place);
				if (tempStringH != (char **) 0)
				{
					strcpy(*tempStringH, stringBuff);
				}
			}
			else
			{
				tempStringH = mygrowhandle(tempStringH, place);
			}
			setSTRING(Yylval,tempStringH);
			if (tempStringH == (char **) 0)
			{
				errorToken = FATALERRORCODE(8);
				goto errorExit;
			}

			return (VAR); /* quoted string*/

		  case '{': /* start of compound statement, expanded macro or evaluation*/
			BRACKETLEV++;

			{
				/* check to see if '{' is start of macro body or
				   evaluated string (header of form "{#?#" where
				   ? is one of ')', ']' or '}')
				*/
				unsigned char     headType = isMacroHeader();

				if (headType != notAHeaderCode)
				{
					/*
					  It's a macro header or evaluation string header
					  ISTRCHAR has been advanced to point beyond it
					*/
					int         inlineMacro, outlineMacro, evaluation;
					char        macroName[NAMELENGTH + 1];
					int         headToken, iToken;

					/*
					  Start of expanded macro signaled by "{#)#" (inline)
					  or "{#]#" (out-of-line); start of evaluated string
					  signalled by "{#}#"
					*/

					if (ALTTOKENPTR != (int *) 0)
					{
						for (iToken = 0; ALTTOKENPTR[iToken] != 0; iToken++)
						{ /* find end of ALTTOKENPTR list */
							;
						} /*for (iToken = 0; ALTTOKENPTR[iToken] != 0; iToken++)*/
					} /*if (ALTTOKENPTR != (int *) 0)*/
					else
					{
						ALTTOKENPTR = ALTINPUTTOKENS;
						iToken = 0;
					}

					/*
					  Just one of the following will be non-zero
					*/
					inlineMacro = (headType == inlineMacroHeaderCode);
					outlineMacro = (headType == outlineMacroHeaderCode);
					evaluation = (headType == evaluationHeaderCode);

					if (inlineMacro || outlineMacro)
					{
						int             place = 0;

						if (isnamestart(inputString[ISTRCHAR]))
						{
							do
							{
								macroName[place++] = inputString[ISTRCHAR++];
							} while (isnamechar(inputString[ISTRCHAR]) &&
									 place < NAMELENGTH);
						} /*if (isnamestart(inputString[ISTRCHAR]))*/
						macroName[place] = '\0';
					} /*if (inlineMacro || outlineMacro)*/

					if (isNewline(inputString[ISTRCHAR]))
					{
						ISTRCHAR++;
					}
					  
					if (inlineMacro)
					{
						headToken = INLINEHEAD;
						/* 
						   970614 MLEVEL incremented ; it should be
						   decremented by parser
						*/
						if (++MLEVEL > MAXLEVEL)
						{
							c = ERROR;
						} /*if (MLEVEL > MAXLEVEL)*/
					}
					else if (outlineMacro)
					{
						headToken = OUTLINEHEAD;
						/* 
						   970615 MDEPTH incremented ; it should be
						   decremented by parser
						*/
						if (++MDEPTH > MAXMDEPTH)
						{
							c = ERROR;
						} /*if (MDEPTH > MAXMDEPTH)*/
					}
					else
					{
						headToken = EVALHEAD;
					}

					if (c == ERROR)
					{
						sprintf(OUTSTR,
								"ERROR: macros nested too deeply or too many active");
						yyerror(OUTSTR);
			
					}
					else
					{
						if (!evaluation)
						{
							PushMacroInfo(macroName, BRACKETLEV);
						}
						  
						/* set next token to be returned after '{' */
						ALTTOKENPTR[iToken++] = headToken;
						ALTTOKENPTR[iToken] = 0;
					}
				} /*if (headType != notAHeaderCode)*/
			}
			
			return (c);

		  case '}':					/* end of one level of compound statement */
			BRACKETLEV--;
			if (BRACKETLEV < 0)
			{
				yyerror("ERROR: too many }'s");
				c = ERROR;
			}
			return (c);

		  case '#': /* start of comment */
			/*030910 modified code to fix bug*/
			if (BRACKETLEV > 0)
			{
				/* skip whitespace, comments and newlines */
				ISTRCHAR--; /*back over '#'*/
				c = skipWhiteSpace(SKIPCOMMENTS | SKIPNEWLINES);

				return ((c == '}') ? ENDOFL : SEMI1);
			}
			else
			{
				/* just skip comment */
				c = skipToEOL();

				if (c != '\0')
				{ /* hit '\n' or '\r' */
					if (c == '\\')
					{ /* must have been "\\\n" */
						continue;
					}
					ISTRCHAR++;
				}

				return (ENDOFL);
			}
			
		  case '\\':				/* may be continuation line */
			if (c2 = inputString[ISTRCHAR], isNewline(c2))
			{
				ISTRCHAR++;
				continue;
			}
			/* else fall through */

		  default:					/* see if number or variable name*/
			if (!isnamestart(c) && !isdigit(c) && c != '.' && c != TEMPPREFIX)
			{/*illegal character*/
				yyerror("ERROR: illegal character");
				return (ERROR);
			}

			c2 = inputString[ISTRCHAR]; /* next character*/
			if ((c == 'T' || c == 'F') && !isnamechar(c2))
			{ /* treat T and F special so they don't need to be in symbol table */
				Yylval = RInstall(LOGICSCRATCH,(long) 1);
				if (Yylval == SymbolError)
				{
					return (FATALERRORCODE(9));
				}
				setTYPE(Yylval,LOGIC);
				DATAVALUE(Yylval,0) = (c == 'T') ? 1.0 : 0.0;
				return (VAR);
			}

			/* c is alpha or '@' or '.' or digit */
			nameStart = ISTRCHAR-1;	/* save loc of 1st char of name */
			if (isnamestart(c) || c == TEMPPREFIX)
			{/* may be keyword, name, or temporary name */
				tokenLength = 0;
				token[0] = c;
				while (isnamechar(c = inputString[ISTRCHAR]))
				{
					ISTRCHAR++;
					if (tokenLength < MAXTOKENSCAN-1)
					{
						token[++tokenLength] = c;
					}
					else
					{
						break;
					}
				} /*while (isnamechar(c = inputString[ISTRCHAR]))*/

				token[++tokenLength] = '\0';

				if (c == '.')
				{
					ISTRCHAR++;
					yyerror("ERROR:  do not use . in variable names");
					return (ERROR);
				}
				
				if (tokenLength > NAMELENGTH)
				{
					/* variable name too long */
					putPieces("ERROR: variable name too long: ", token,
							  (char *) 0, (char *) 0);
					ISTRCHAR -= tokenLength - NAMELENGTH - 1;
					return (ERROR);
				} /*if (tokenLength > NAMELENGTH)*/

				/*
				  See if it is a syntax element; SyntaxStartChars are possible
				  1st characters; TEMPPREFIX starts temp name.  If it is
				  followed by ':' it will be treated as a keyword; if it
				  is preceded by '$' it will be treated as a structure
				  component.
				  */
				c1 = skipWhiteSpace(SKIPCOMMENTS);
				/*
				  c1 is first non-white character after syntax elements,
				  treating comments as white
				  */
				if (token[0] != TEMPPREFIX && c1 != ':' && LASTCH != '$' &&
					strchr(SyntaxStartChars, (int) token[0]) &&
					(c = isSyntaxElement(token)) != 0)
				{/* check for key words;  */
					char      lparen = '(';
					char     *dontEnclose =
					  "ERROR: '%s' must not be enclosed in %c...%c";

					switch (c)
					{
					  case ELSE:
						lparen = '{';
						/* fall through */
					  case ELSEIF:
						if (LASTCH != '}')
						{
							sprintf(OUTSTR,
									"ERROR: %s does not follow '}'", token);
							c = ERROR;
							break;
						}
						/* fall through */
					  case IF:
						IFBRACKETLEV[IDEPTH] = BRACKETLEV;
						if (!findLParen(c, lparen))
						{
							c = ERROR;
						}
						break;
						
					  case WHILE:
						WHILESTARTS[WDEPTH] = ISTRCHAR+1;
						/* fall through*/
					  case FOR:
						WHILEBRACKETLEV[WDEPTH] = BRACKETLEV;
						if (LASTCH != ';' && LASTCH != '{' &&
							LASTCH != ENDOFL && LASTCH != '\0' &&
							LASTCH != INLINEHEAD && LASTCH != OUTLINEHEAD)
						{
							sprintf(OUTSTR,
									"ERROR: illegal use of '%s'", token);
							c = ERROR;
						}
						else if (!findLParen(c, lparen))
						{
							c = ERROR;
						}
						break;
					
					  case BREAK:
					  case NEXTREP:
						/* Yylval == 0 causes parser error */
						Yylval = SymbolError;

						if (WDEPTH <= ThisWdepth)
						{
							sprintf(OUTSTR, noForWhile, token);
							c = BREAK;
							break;
						}
						savedPlace = ISTRCHAR;
						if (isdigit(c1) || c1 == '-' || c1 == '+')
						{ /* break n */
							y = mystrtod((char *) inputString+ISTRCHAR,
										 (char **) &endptr);
							ISTRCHAR = endptr - inputString;
							c2 = skipWhiteSpace(SKIPCOMMENTS);

							/* 
							   981231 added '\n' to list and removed ' ' and '\t'
							   from it
							   */
							if (c2 != '\n' && c2 != ';' && c2 != '}')
							{
								ISTRCHAR++;
								c = BADTOKENCODE(8);
							}
							else
							{
								ISTRCHAR = endptr - inputString;
								if (y <= 0 || y != floor(y))
								{
									sprintf(OUTSTR,
											"ERROR: %s n, with n not integer > 0",
											token);
									yyerror(OUTSTR);
									y = -1;
								}
								else if ((int) y > WDEPTH - ThisWdepth)
								{
									sprintf(OUTSTR,
											"ERROR: %s n where n > loop depth",
											token);
									yyerror(OUTSTR);
									y = -1;
								}
							}
						} /*if (isdigit(c1) || c1 == '-' || c1 == '+')*/
						else
						{ /* break or next */
							if (c1 != ' ' && c1 != '\t' && !isNewline(c1) &&
								c1 != ';' && c1 != '}')
							{
								ISTRCHAR++;
								c = BADTOKENCODE(9);
							}
							else
							{
								y = 1.0;
							}
						}

						if (y > 0.0)
						{
							Yylval = RInstall(NUMSCRATCH, (long) 1);
							if (Yylval == SymbolZero)
							{
								c = FATALERRORCODE(10);
							}
							else
							{
								DATAVALUE(Yylval,0) = y;
							}
						} /*if (y > 0.0)*/
						break;

					  case BREAKALL:
						if (WDEPTH <= ThisWdepth)
						{
							sprintf(OUTSTR, noForWhile, token);
							c = ERROR;
						} /*if (WDEPTH <= 0)*/
						else
						{
							if ((c1 = inputString[ISTRCHAR]) != ';' &&
								c1 != '}')
							{
								ISTRCHAR++;
								c = BADTOKENCODE(10);
							}
							else
							{
								y = (double) (WDEPTH - ThisWdepth) ;
								Yylval = RInstall(NUMSCRATCH, (long) 1);
								if (Yylval == SymbolZero)
								{
									c = FATALERRORCODE(11);
								}
								else
								{
									DATAVALUE(Yylval,0) = y;
									c = BREAK;
								}
							}
						} /*if (WDEPTH <= 0){}else{}*/
						break;

					  case EXITMACRO:
						{
							if (MDEPTH == 0 && MLEVEL == MAXMDEPTH + 1)
							{
								sprintf(OUTSTR,
										"ERROR: '%s' used outside a macro",
										token);
							}
							else if (c1 != lparen && c1 != '}' && c1 != ';' &&
									 c1 != '\n')
							{
								sprintf(OUTSTR,
										"ERROR: '%s' must be followed by '(', ';' or '}'",
										token);
							}
							if (*OUTSTR)
							{
								c = ERROR;
							}
							else if (c1 != '(')
							{
								ALTTOKENPTR = ALTINPUTTOKENS;
								ALTTOKENPTR[0] = '(';
								ALTTOKENPTR[1] = ')';
								ALTTOKENPTR[2] = '\0';
							}
						}
						break;

					  case END:
						if (BRACKETLEV > 0)
						{
							c = ERROR;

							if (INPUTLEVEL > 0 || MDEPTH > 0 ||
								MLEVEL > MAXMDEPTH + 1)
							{
								sprintf(OUTSTR,
										"ERROR: '%s' is illegal in a macro or evaluated string",
										token);
							}
							else
							{
								sprintf(OUTSTR,
										dontEnclose, token, '{','}');
							}
						} /*if (BRACKETLEV > 0)*/
						else if (PARENLEV > 0)
						{
							sprintf(OUTSTR, dontEnclose, token, '(',')');
							c = ERROR;
						}
						else if (SQBRACKETLEV > 0)
						{
							sprintf(OUTSTR, dontEnclose, token, '[',']');
							c = ERROR;
						}
						else if (LASTCH != ';' && LASTCH != '\0')
						{
							sprintf(OUTSTR,
									"ERROR: '%s' is not at start of line and does not follow ';'", token);
							c = ERROR;
						}
						else
						{
							/*
							  960726 allow quit(T), quit(F) as will as quit()
							  T (default) and F only relevant with scrollable window
							  If F, quit is unconditional with no opportunity to save windows;
							  otherwise, a dialog box queries the user about each window
							  before quitting
							  */
#ifdef SCROLLABLEWINDOW
							SaveOnQuit = 1;
#endif /*SCROLLABLEWINDOW*/

							if (c1 == '(')
							{
								ISTRCHAR++;
								c2 = skipWhiteSpace(SKIPCOMMENTS);
								if (c2 == 'T' || c2 == 'F')
								{
#ifdef SCROLLABLEWINDOW
									SaveOnQuit = (c2 == 'T');
#endif /*SCROLLABLEWINDOW*/
									ISTRCHAR++;
									c2 = skipWhiteSpace(SKIPCOMMENTS);
								} /*if (c2 == 'T' || c2 == 'F')*/
								ISTRCHAR++;
								if (c2 != ')')
								{
									sprintf(OUTSTR,
											"ERROR: must be %s(), %s(T) or %s(F)",
											token, token, token);
									c = ERROR;
								} /*if (c2 != ')')*/
							} /*if (c1 == '(')*/ 

							if (c != ERROR)
							{
								c2 = skipWhiteSpace(SKIPCOMMENTS);
								if (!isNewline(c2))
								{
									sprintf(OUTSTR,
											"ERROR: '%s' not last command in line",
											token);
									c = ERROR;
								} /*if (!isNewline(c2))*/
							} /*if (c != ERROR)*/						
						} /*if (BRACKETLEV > 0){}else{}else{}*/ 
						break;
					} /*switch (c)*/
					
					if (c == ERROR && *OUTSTR != '\0')
					{
						yyerror(OUTSTR);
					}
					return (c);
				} /*if token represents a syntax element*/

				/* must be name or temporary name but not a syntax element */

				if (token[0] == TEMPPREFIX && !isnamestart(token[1]))
				{
					ISTRCHAR++;
					if (isnamechar(token[1]))
					{
						sprintf(OUTSTR, "ERROR: illegal temporary name");
					}
					else
					{
						sprintf(OUTSTR,
								"ERROR: '%c' not followed by alphabetic character",
								TEMPPREFIX);
					}
					yyerror(OUTSTR);
					return(ERROR);
				}/*if (token[0] == TEMPPREFIX && !isnamestart(token[1]))*/

/*
  If token is "NULL", create NULL symbol.  Otherwise try and find it.
  If token is the name of an ordinary symbol, Lookup() returns that symbol.
  If token is the name of a special symbol such as CLIPBOARD, Lookup()
  sets its current value and returns the special symbol.
  If token is not the name of a special or ordinary symbol, Lookup() returns
  SymbolZero.
  It is a fatal error if token is the name of a special symbol and
  Lookup() returns SymbolZero.  This cannot happen with CLIPBOARD
  or SELECTION (on wx_motif)
  Special symbols may not be in the Symbol table.
*/
				if (token[0] == NULLNAME[0] && strcmp(token, NULLNAME) == 0)
				{
					Yylval = Install(NULLSCRATCH, NULLSYM);
				}
				else
				{
					Yylval = Lookup(token);
				}
			
				if (Yylval == SymbolZero)
				{
					if (Findspecial(token) != SymbolZero ||
						token[0] == NULLNAME[0] &&
						strcmp(token, NULLNAME) == 0)
					{
						return (FATALERRORCODE(12));
					}
					Yylval = Install(token, UNDEF);
					if (Yylval == SymbolError)
					{
						return (FATALERRORCODE(13));
					}
					setNDIMS(Yylval,1);
				} /*if (Yylval == SymbolZero)*/

				/*
				  920623 Return VAR for all names except BLTINs or MACROs
				         followed by '('
				  950803 Replaced switch by simpler if statement.
				         Unfortunately this eliminates any check for an
					     illegal Symbol type.  This should never occur :-),
				         but if it does it should be caught elsewhere.

				  010308 Don't look for '(' if last token was '$'
				  */
				c = TYPE(Yylval);
				if ((c == BLTIN || c == MACRO || c == UNDEF) &&
					LASTCH != '$' && findLParen(0, '('))
				{
					if (c != UNDEF)
					{
						c = (c == BLTIN) ? BLTINTOKEN : MACROTOKEN;
						if (c == MACROTOKEN)
						{				/* macro invocation */
							MACROSTART = nameStart;
							PARSEARGS = quoteArgPunct;
						}
						else if (tokenLength == 5 &&
								 token[0] == 'b' && strcmp(token, "batch") == 0)
						{
							if (BATCHBLOCKCOUNT <= 0)
							{			/* not on r.h.s of binarary operator*/
								return (BATCH);
							} /*if (BATCHBLOCKCOUNT <= 0)*/
							yyerror("ERROR: illegal use of batch()");
							return(ERROR);
						}
						else if (QUOTEARGS(Yylval))
						{
							PARSEARGS = quoteArgPunct;
						}
						
						return (c);
					} /*if (c != UNDEF)*/
					else
					{
						/*
						  981118 code to search files automatically for
						  macros
						*/
						Symbolhandle symh = searchForMacro(token);
					
						if (!isNull(symh))
						{
							/*
							  Found and read in macro; substitute it for Yylval
							*/
							Removesymbol(Yylval);
							Yylval = symh;
							Cutsymbol(Yylval);
							setNAME(Yylval, token);
							Addsymbol(Yylval);
							MACROSTART = nameStart;
							PARSEARGS = quoteArgPunct;
							return (MACROTOKEN);
						} /*if (!isNull(symh))*/
					} /*if (c != UNDEF){}else{}*/
				}
				return (VAR);	/* any legal type */
			} /*if (isnamestart(c) || c == TEMPPREFIX)*/

			/* if not alpha, ought to be number unless it is of form \.[a-zA-z] */
			if (c == '.' && !isdigit(c1 = inputString[ISTRCHAR]))
			{
				ISTRCHAR++;
				if (c1 == TEMPPREFIX || isnamestart(c1) || isnamechar(c1))
				{
					yyerror("ERROR:  do not use . in variable names");
					return (ERROR);
				}
				return (BADTOKENCODE(13));
			} /*if (c == '.' && !isdigit(c1 = inputString[ISTRCHAR]))*/
			ISTRCHAR--;
			return (scanNumber());
		} /*switch (c)*/
	} /*while(1)*/
	
  errorExit:
	Removesymbol(Yylval);
	Yylval = SymbolZero;
	return (errorToken);
		
} /*yylex1()*/

/* Start of publically accessible functions */

/*
  981117 function to search standard macro files for
         a macro named macroName.  If it finds it, its Symbolhandle
         is returned.  Otherwise (and this includes the case when an
         object that is not a macro is found), it returns 0.

		 Its behavior depends on the value of global MACROSEARCH.  If its value
         is 0, no search is done; if the value is 1 a search with a warning
         message is done; if the value is 2, a search is done but no
         warning message is given
  981121 Removed test opening of files, since readdata() was modified to
         not treat an unopenable file as an error.
  010308 No longer static so as to be accessible from Byname() in Symbol.c
*/

Symbolhandle searchForMacro(char macroName [])
{
	Symbolhandle      macroFiles = Lookup("MACROFILES");
	Symbolhandle      macroSymbol = SymbolZero;
	long              iFile, nFiles;
	WHERE("searchForMacro");
	
	if (MACROSEARCH != noMacroSearch)
	{
		if (!isVector(macroFiles) || TYPE(macroFiles) != CHAR)
		{
			macroFiles = Lookup("MACROFILE");
			if (!isCharOrString(macroFiles))
			{
				return (SymbolError);
			}
		}

		if (MACROSEARCH == verboseMacroSearch)
		{
			sprintf(OUTSTR,
					"WARNING: searching for unrecognized macro %s()", 
					macroName);
			ISTRCHAR++; /* include '(' in warning message */
			yyerror(OUTSTR);
			ISTRCHAR--;
		}
		
		/*
		  Successively try to open each file
		  */
		nFiles = symbolSize(macroFiles);
	
		for (iFile = 0; iFile < nFiles; iFile++)
		{
			char       *fileName = skipStrings(STRINGPTR(macroFiles),iFile);

			if (fileName[0] != '\0')
			{
				Symbolhandle list;
				long         typeList[2];
				char        *valueList[2];
				char        *keyList[2];
				char         thisFileName[PATHSIZE+1];
				char         thisMacroName[NAMELENGTH + 1];

				typeList[0] = typeList[1] = CHAR;

				strncpy(thisFileName, fileName, PATHSIZE);
				thisFileName[PATHSIZE] = '\0';
				valueList[0] = thisFileName;

				strncpy(thisMacroName, macroName, NAMELENGTH);
				thisMacroName[NAMELENGTH] = '\0';
				valueList[1] = thisMacroName;

				keyList[0] = keyList[1] = NullString;

				list = Buildlist(2, typeList, valueList, keyList);

				if (list != SymbolError)
				{
					char    funcname[NAMELENGTH + 1];
					
					strncpy(funcname, FUNCNAME, NAMELENGTH);
					funcname[NAMELENGTH] = '\0';
					strcpy(FUNCNAME, "macrosearch");
					OUTSTR[0] = '\0';
					macroSymbol = readdata(list);
					Removelist(list);
					strcpy(FUNCNAME, funcname);
					
					if (macroSymbol != SymbolError &&
						TYPE(macroSymbol) == MACRO)
					{
						break;
					}
					Removesymbol(macroSymbol);
					macroSymbol = SymbolZero;
				} /*if (list != SymbolError)*/
			} /*if (fileName[0] != '\0')*/
		} /*for (iFile = 0; iFile < nFiles; iFile++)*/
	} /*if (MACROSEARCH != noMacroSearch)*/
	if (macroSymbol == SymbolZero && MACROSEARCH == verboseMacroSearch)
	{
		sprintf(OUTSTR,
				"WARNING: macro %s() not found on external macro files",
				macroName);
		putErrorOUTSTR();
	}
	
	return (macroSymbol);
} /*searchForMacro()*/

/*
  Called after token matching BLTIN or MACRO name or after IF, ELSEIF, WHILE,
  and FOR.
  If next non-white character is '(' it resets ISTRCHAR & returns 1
  Otherwise, it returns 0 and does not reset ISTRCHAR
  If op != 0 it prints an error message if '(' is not found

  also called from Symbol.c and Ifsetup.c
*/

long findLParen(int op, char paren)
{
	long           savePlace = ISTRCHAR;
	unsigned int   c;
	WHERE("findLParen");

	/* skip white space including \-protected newlines */
	c = skipWhiteSpace(ONLYSPACEANDTAB);
	if (c != paren)
	{
		if (op != 0)
		{
			ISTRCHAR++;
			sprintf(OUTSTR,
					"ERROR: need '%c' after '%s%s'", paren, opName(op),
					(paren == '{' && op != ELSE) ? "(...)" : "");
			yyerror(OUTSTR);
		}
		ISTRCHAR = savePlace;
		return (0);
	} /*if (c != paren)*/
	return (1);
} /*findLParen(*/

int yylex(void)
{				/* lexical analyzer */

	unsigned int    c = 0;
	int             inputlevel = INPUTLEVEL; /* value at entrance */
	WHERE("yylex");

	Yylval = SymbolZero;

	if (ALTTOKENPTR)
	{
		/*
		  first see if we have stashed stuff away that should be returned
		  before we examine *INPUTSTRING again
		  */
		c = (unsigned int) *ALTTOKENPTR++;
		if (c != 0)
		{
			LASTCH  = c;
			if (c == ';')
			{ /*do nothing*/
				;
			}
			else if (c == '(')
			{
				PARENLEV++;
			}
			else if (c == ')')
			{
				PARENLEV--;
			}
			else if (c == '}')
			{
				BRACKETLEV--;
			}
			else if (c == '{')
			{
				BRACKETLEV++;
			}
			else if (c == ']')
			{
				SQBRACKETLEV--;
			}
			else if (c == '[')
			{
				SQBRACKETLEV++;
			}
		} /*if (c != 0)*/
		else
		{
			ALTTOKENPTR = (int *) 0;
		} /*if (c != 0){}else{}*/
	} /*if (ALTTOKENPTR)*/

	if (c == 0)
	{
		/* nothing was foundin ALTTOKENPTR*/
		if (PARSEARGS != normalParse)
		{
			c = parseQuotedArgs();
			if (c == 0)
			{
				return (ERROR);
			}
			LASTCH  = '\0';
		}
		else if (YYLEXMESSAGE > 0)
		{
			c = processBreak();
			LASTCH  = '\0';
		}
		else if (-YYLEXMESSAGE > ' ')
		{
			/*
			  -YYLEXMESSAGE is token to return next
			  After while(){} and if(){}, -YYLEXMESSAGE should be ';'
			  After <<name>> -YYLEXMESSAGE should be appropriate token
			  */
			c = -YYLEXMESSAGE;
			YYLEXMESSAGE = 0;
			LASTCH  = c;
		}
		else
		{
			/* normal scan of input stream */
			c = yylex1();
			LASTCH = (c == SEMI1) ? ';' : c;
		}
	} /*if (c == 0)*/

	/* for debugging purposes */
	if (GUBED & 1)
	{
		char        *outstr = OUTSTR;
		
		sprintf(outstr,"yylex[%d]: token = %s", inputlevel, tokenName(c));
		
		outstr += strlen(outstr);
		if (c == VAR || c == BLTINTOKEN || c == MACROTOKEN || c == UNDEF ||
			c == ASSIGNED || c == NUMBER)
		{
			if (Yylval != SymbolZero)
			{
				char    *name = NAME(Yylval);
				
				if (c == VAR)
				{
					sprintf(outstr, ", type = %s", typeName(TYPE(Yylval)));
					if (strcmp(name,LOGICSCRATCH) == 0)
					{
						outstr += strlen(outstr);
						sprintf(outstr, ", value = %s",
								(DATAVALUE(Yylval,0) != 0.0) ? "T" : "F");
					}
				}
				else if (c == NUMBER)
				{
					if (isMissing(DATAVALUE(Yylval,0)))
					{
						sprintf(outstr,", value = MISSING");
					}
					else
					{
						sprintf(outstr,
								", value = %-12.5g",DATAVALUE(Yylval,0));
					}
				}
				if (!isscratch(NAME(Yylval)))
				{
					outstr += strlen(outstr);
					sprintf(outstr,", name = '%s'", name);
				}
			} /*if (Yylval != SymbolZero)*/
		}
		else if (c == BREAK)
		{
			if (Yylval != SymbolZero)
			{
				sprintf(outstr," %g", DATAVALUE(Yylval,0));
			}
			else
			{
				sprintf(outstr," ERROR");
			}
		}
		putOUTSTR();
	} /*if (GUBED & 1)*/

#ifdef YYLEXDB

	printf("token = %s\n",tokenName(c));

#endif /*YYLEXDB*/
	if (c == NUMBER)
	{
		c = VAR;
	}
	else if (c == FATALERROR)
	{
		FatalError = 1;
	}
	
	return ((c) ? c : ENDOFL);

} /*yylex()*/

